Package: plotthis
Title: High-Level Plotting Built Upon 'ggplot2' and Other Plotting
        Packages
Version: 0.12.1
Authors@R: 
    person("Panwen", "Wang", , "pwwang@pwwang.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-4614-8970"))
Description: Provides high-level API and a wide range of options to create stunning, publication-quality plots effortlessly.
    It is built upon 'ggplot2' and other plotting packages, and is designed to be easy to use and to work seamlessly with 'ggplot2' objects.
    It is particularly useful for creating complex plots with multiple layers, facets, and annotations.
    It also provides a set of functions to create plots for specific types of data, such as Venn diagrams, alluvial diagrams, and phylogenetic trees.
    The package is designed to be flexible and customizable, and to work well with the 'ggplot2' ecosystem.
    The API can be found at <https://pwwang.github.io/plotthis/reference/index.html>.
License: GPL (>= 3)
Encoding: UTF-8
URL: https://github.com/pwwang/plotthis
        https://pwwang.github.io/plotthis/
BugReports: https://github.com/pwwang/plotthis/issues
RoxygenNote: 7.3.3
Depends: R (>= 4.2.0)
Imports: circlize, ggplot2, rlang, dplyr, tidyr, glue, forcats, gtable,
        reshape2, stringr, scales, gridtext, methods, patchwork,
        ggrepel, ggnewscale, cowplot, zoo
Suggests: plotly, testthat, Matrix, alluvial, datasets, ComplexHeatmap,
        cluster, clustree, gglogger, ggwordcloud, ggalluvial,
        ggVennDiagram (>= 1.5.0), ggupset, ggpubr, ggbeeswarm, ggforce,
        ggraph, ggridges, ggmanh, qqplotr, hexbin, igraph, iNEXT,
        scattermore, sf, terra, concaveman, plotROC, OptimalCutpoints,
        proxyC, metR
LazyData: true
Additional_repositories: https://bioconductor.org/packages/release/bioc
Config/Needs/website: rmarkdown
NeedsCompilation: no
Packaged: 2026-06-09 07:01:57 UTC; pwwang
Author: Panwen Wang [aut, cre] (ORCID: <https://orcid.org/0000-0002-4614-8970>)
Maintainer: Panwen Wang <pwwang@pwwang.com>
Repository: CRAN
Date/Publication: 2026-06-10 05:20:19 UTC
Built: R 4.6.0; ; 2026-06-10 06:34:31 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(SpatShapesPlot,SpatVector)
S3method(SpatShapesPlot,data.frame)
S3method(element_grob,element_textbox)
export(AlluvialPlot)
export(AreaPlot)
export(BarPlot)
export(BeeswarmPlot)
export(BoxPlot)
export(ChordPlot)
export(CircosPlot)
export(ClustreePlot)
export(CorPairsPlot)
export(CorPlot)
export(DensityPlot)
export(DimPlot)
export(DotPlot)
export(EnrichMap)
export(EnrichNetwork)
export(FeatureDimPlot)
export(GSEAPlot)
export(GSEASummaryPlot)
export(Heatmap)
export(Histogram)
export(JitterPlot)
export(LinePlot)
export(LollipopPlot)
export(ManhattanPlot)
export(Network)
export(PieChart)
export(QQPlot)
export(ROCCurve)
export(RadarPlot)
export(RarefactionPlot)
export(RidgePlot)
export(RingPlot)
export(SankeyPlot)
export(ScatterPlot)
export(SpatImagePlot)
export(SpatMasksPlot)
export(SpatPointsPlot)
export(SpatShapesPlot)
export(SpiderPlot)
export(SplitBarPlot)
export(TrendPlot)
export(UpsetPlot)
export(VelocityPlot)
export(VennDiagram)
export(ViolinPlot)
export(VolcanoPlot)
export(WaterfallPlot)
export(WordCloudPlot)
export(element_textbox)
export(palette_this)
export(show_palettes)
export(theme_blank)
export(theme_box)
export(theme_this)
importFrom(circlize,colorRamp2)
importFrom(cowplot,get_plot_component)
importFrom(dplyr,"%>%")
importFrom(dplyr,.data)
importFrom(dplyr,across)
importFrom(dplyr,add_count)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,case_when)
importFrom(dplyr,cur_group_id)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,first)
importFrom(dplyr,group_by)
importFrom(dplyr,group_map)
importFrom(dplyr,group_modify)
importFrom(dplyr,if_else)
importFrom(dplyr,last)
importFrom(dplyr,lead)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,reframe)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,slice_head)
importFrom(dplyr,slice_max)
importFrom(dplyr,slice_min)
importFrom(dplyr,slice_sample)
importFrom(dplyr,starts_with)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(forcats,fct_relabel)
importFrom(ggnewscale,new_scale_color)
importFrom(ggnewscale,new_scale_fill)
importFrom(ggplot2,.pt)
importFrom(ggplot2,CoordPolar)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,annotate)
importFrom(ggplot2,annotation_custom)
importFrom(ggplot2,as_labeller)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_equal)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,coord_polar)
importFrom(ggplot2,coord_sf)
importFrom(ggplot2,draw_key_point)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_grob)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expansion)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,fortify)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_area)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_curve)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_density_2d)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,geom_hex)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_label)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_linerange)
importFrom(ggplot2,geom_path)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_sf)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplotGrob)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggproto)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_colorbar)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guide_none)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer_data)
importFrom(ggplot2,layer_scales)
importFrom(ggplot2,margin)
importFrom(ggplot2,position_dodge)
importFrom(ggplot2,position_dodge2)
importFrom(ggplot2,position_jitter)
importFrom(ggplot2,position_jitterdodge)
importFrom(ggplot2,position_nudge)
importFrom(ggplot2,position_stack)
importFrom(ggplot2,scale_alpha_continuous)
importFrom(ggplot2,scale_alpha_manual)
importFrom(ggplot2,scale_color_gradientn)
importFrom(ggplot2,scale_color_identity)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,scale_fill_identity)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_linetype_discrete)
importFrom(ggplot2,scale_linetype_manual)
importFrom(ggplot2,scale_linewidth)
importFrom(ggplot2,scale_linewidth_continuous)
importFrom(ggplot2,scale_shape_discrete)
importFrom(ggplot2,scale_shape_identity)
importFrom(ggplot2,scale_shape_manual)
importFrom(ggplot2,scale_size)
importFrom(ggplot2,scale_size_area)
importFrom(ggplot2,scale_size_continuous)
importFrom(ggplot2,scale_size_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_x_reverse)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,scale_y_reverse)
importFrom(ggplot2,stat_bin)
importFrom(ggplot2,stat_density_2d)
importFrom(ggplot2,stat_summary)
importFrom(ggplot2,stat_summary_hex)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,unit)
importFrom(ggplot2,vars)
importFrom(ggplot2,waiver)
importFrom(ggplot2,wrap_dims)
importFrom(ggplot2,ylim)
importFrom(ggplot2,zeroGrob)
importFrom(ggrepel,geom_label_repel)
importFrom(ggrepel,geom_text_repel)
importFrom(glue,glue)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,rgb)
importFrom(grid,arrow)
importFrom(grid,convertUnit)
importFrom(grid,gList)
importFrom(grid,gpar)
importFrom(grid,grid.draw)
importFrom(grid,grid.grabExpr)
importFrom(grid,grid.lines)
importFrom(grid,grid.points)
importFrom(grid,grid.rect)
importFrom(grid,grid.text)
importFrom(grid,grobHeight)
importFrom(grid,grobTree)
importFrom(grid,grobWidth)
importFrom(grid,is.grob)
importFrom(grid,is.unit)
importFrom(grid,linesGrob)
importFrom(grid,textGrob)
importFrom(grid,unit)
importFrom(grid,viewport)
importFrom(gridtext,textbox_grob)
importFrom(gtable,gtable_add_cols)
importFrom(gtable,gtable_add_grob)
importFrom(gtable,gtable_add_rows)
importFrom(gtable,is.gtable)
importFrom(methods,formalArgs)
importFrom(patchwork,plot_annotation)
importFrom(patchwork,plot_layout)
importFrom(patchwork,plot_spacer)
importFrom(patchwork,wrap_elements)
importFrom(patchwork,wrap_plots)
importFrom(rlang,"%||%")
importFrom(rlang,":=")
importFrom(rlang,as_name)
importFrom(rlang,dots_n)
importFrom(rlang,eval_tidy)
importFrom(rlang,parse_expr)
importFrom(rlang,parse_exprs)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(scales,alpha)
importFrom(scales,breaks_extended)
importFrom(scales,pretty_breaks)
importFrom(scales,rescale)
importFrom(scales,scientific)
importFrom(stats,aggregate)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,dist)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stringr,str_wrap)
importFrom(tidyr,complete)
importFrom(tidyr,expand_grid)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,replace_na)
importFrom(tidyr,separate)
importFrom(tidyr,uncount)
importFrom(tidyr,unite)
importFrom(tidyr,unnest)
importFrom(utils,combn)
importFrom(utils,getFromNamespace)
importFrom(utils,modifyList)
importFrom(zoo,na.approx)

# plotthis <a href="https://pwwang.github.io/plotthis/"><img src="man/figures/logo.png" align="right" height="180" alt="plotthis website" /></a>

`plotthis` is an R package that is built upon `ggplot2` and other plotting packages. It provides high-level APIs and a wide range of options to create stunning, publication-quality plots effortlessly.

## Installation

```r
install.packages("plotthis")
# or to install the latest version:
remotes::install_github("pwwang/plotthis")
devtools::install_github("pwwang/plotthis")
```

You can also install the package using `conda`:

```
$ conda install pwwang::r-plotthis
```

## Gallery
[`AreaPlot`][2] / [`TrendPlot`][3] / [`ROCCurve`][28]

![](man/figures/area-trend.png)

[`ClustreePlot`][4] / [`LinePlot`][5] / [`Network`][27]

![](man/figures/clustree-line.png)

[`Heatmap`][6]

![](man/figures/heatmap.png)

[CorPlot][24] / [CorPairsPlot][25]

![](man/figures/cor-corpairs.png)

[`PieChart`][7] / [`RingPlot`][8] / [`QQPlot`][29]

![](man/figures/pie-ring.png)

[`VolcanoPlot`][9] / [`WordCloudPlot`][10]

![](man/figures/volcano-wordcloud.png)

[`BarPlot`][11] / [`SplitBarPlot`][11]

![](man/figures/bar-splitbar.png)

[`BoxPlot`][12] / [`ViolinPlot`][12] / [`BeeswarmPlot`][12]

![](man/figures/box-violin.png)

[`ChordPlot`][13] / [`UpsetPlot`][14] / [`VennDiagram`][15]

![](man/figures/chord-upset-venn.png)

[`DensityPlot`][16] / [`Histogram`][16] / [`RidgePlot`][23]

![](man/figures/density-histogram.png)

[`DimPlot`][17] / [`FeatureDimPlot`][17] / [`VelocityPlot`][32]

![](man/figures/dimplot.png)

[`DotPlot`][18] / [`ScatterPlot`][18] / [`LollipopPlot`][18]

![](man/figures/dot-scatter-lollipop.png)

[`EnrichMap`][19] / [`EnrichNetwork`][19]

![](man/figures/enrich.png)

[`GSEASummaryPlot`][20] / [`GSEAPlot`][20]

![](man/figures/gsea.png)

[`RadarPlot`][21] / [`SpiderPlot`][21] / [`RarefactionPlot`][26]

![](man/figures/radar-spider.png)

[`SankeyPlot`][22] / [`AlluvialPlot`][22]

![](man/figures/sankey-alluvial.png)

[`ManhattanPlot`][30]

![](man/figures/manh.png)

[`SpatImagePlot`][31] / [`SpatMasksPlot`][31] / [`SpatShapesPlot`][31] / [`SpatPointsPlot`][31]

![](man/figures/spatial.png)

## Credits

`plotthis` is greatly inspired by the [`SCP`][1] package, but with the plotting functions detached from the Seurat object or single-cell data analysis. It is designed to be more flexible and general-purpose, and can be used for a wide range of data types and analysis scenarios.


[1]: https://zhanghao-njmu.github.io/SCP/index.html
[2]: https://pwwang.github.io/plotthis/reference/AreaPlot.html
[3]: https://pwwang.github.io/plotthis/reference/TrendPlot.html
[4]: https://pwwang.github.io/plotthis/reference/ClustreePlot.html
[5]: https://pwwang.github.io/plotthis/reference/LinePlot.html
[6]: https://pwwang.github.io/plotthis/reference/Heatmap.html
[7]: https://pwwang.github.io/plotthis/reference/PieChart.html
[8]: https://pwwang.github.io/plotthis/reference/RingPlot.html
[9]: https://pwwang.github.io/plotthis/reference/VolcanoPlot.html
[10]: https://pwwang.github.io/plotthis/reference/WordCloudPlot.html
[11]: https://pwwang.github.io/plotthis/reference/barplot.html
[12]: https://pwwang.github.io/plotthis/reference/boxviolinplot.html
[13]: https://pwwang.github.io/plotthis/reference/chordplot.html
[14]: https://pwwang.github.io/plotthis/reference/upsetplot1.html
[15]: https://pwwang.github.io/plotthis/reference/venndiagram1.html
[16]: https://pwwang.github.io/plotthis/reference/densityhistoplot.html
[17]: https://pwwang.github.io/plotthis/reference/dimplot.html
[18]: https://pwwang.github.io/plotthis/reference/dotplot.html
[19]: https://pwwang.github.io/plotthis/reference/enrichmap1.html
[20]: https://pwwang.github.io/plotthis/reference/gsea.html
[21]: https://pwwang.github.io/plotthis/reference/radarplot.html
[22]: https://pwwang.github.io/plotthis/reference/sankeyplot.html
[23]: https://pwwang.github.io/plotthis/reference/RidgePlot.html
[24]: https://pwwang.github.io/plotthis/reference/CorPlot.html
[25]: https://pwwang.github.io/plotthis/reference/CorPairsPlot.html
[26]: https://pwwang.github.io/plotthis/reference/RarefactionPlot.html
[27]: https://pwwang.github.io/plotthis/reference/Network.html
[28]: https://pwwang.github.io/plotthis/reference/ROCCurve.html
[29]: https://pwwang.github.io/plotthis/reference/QQPlot.html
[30]: https://pwwang.github.io/plotthis/reference/ManhattanPlot.html
[31]: https://pwwang.github.io/plotthis/reference/spatialplots.html
[32]: https://pwwang.github.io/plotthis/reference/VelocityPlot.html

#' Atomic QQ/PP plot
#'
#' @description
#' Core implementation for drawing a single quantile-quantile (QQ) or
#' probability-probability (PP) plot. This is the internal workhorse
#' dispatched by the exported \code{\link{QQPlot}} function -- it takes a
#' **single** data frame (no \code{split_by} support) and returns a
#' \code{ggplot} object. The function compares the empirical distribution
#' of a numeric variable against a theoretical distribution (default:
#' standard normal) via the \pkg{qqplotr} package.
#'
#' Two plot types are supported via the \code{type} parameter:
#' \itemize{
#'   \item \strong{QQ plot} (\code{type = "qq"}, the default) -- plots
#'   sample quantiles against theoretical quantiles. Deviations from the
#'   reference line indicate departures from the assumed distribution
#'   (skewness, heavy tails, outliers).
#'   \item \strong{PP plot} (\code{type = "pp"}) -- plots empirical
#'   cumulative probability against theoretical cumulative probability.
#'   PP plots are more sensitive to deviations in the centre of the
#'   distribution, while QQ plots are more sensitive at the tails.
#' }
#'
#' The function can overlay \strong{confidence bands} (\code{band}) around
#' the reference line using several methods (pointwise confidence intervals,
#' Kolmogorov-Smirnov, Tukey's simultaneous intervals, or bootstrap).
#' Multiple bands can be combined and each receives a separate fill colour
#' from the \code{palette}.
#'
#' @section Architecture:
#'
#' \strong{QQPlotAtomic} executes the following steps:
#' \enumerate{
#'   \item \strong{ggplot dispatch} -- selects \code{gglogger::ggplot} or
#'   \code{ggplot2::ggplot} based on
#'   \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{Type validation} -- \code{match.arg()} resolves
#'   \code{type} to \code{"qq"} or \code{"pp"}.
#'   \item \strong{Input validation} -- \code{stopifnot()} checks that
#'   \code{band} is \code{TRUE}, a list, or \code{NULL}; \code{line} and
#'   \code{point} are lists or \code{NULL}; and \code{xlim}/\code{ylim}
#'   are numeric vectors of length 2 or \code{NULL}.
#'   \item \strong{Column resolution} -- \code{\link{check_columns}()}
#'   validates the \code{val} column.
#'   \item \strong{Value transformation} -- when \code{val_trans} is
#'   provided, it is applied to the \code{val} column (e.g. log-transform).
#'   \item \strong{Base ggplot} -- initialises
#'   \code{ggplot(data, aes(sample = !!sym(val)))}. The \code{sample}
#'   aesthetic is the standard interface for \pkg{qqplotr}.
#'   \item \strong{Band rendering} -- when \code{band} is not \code{NULL}:
#'   \enumerate{
#'     \item Selects the band stat function:
#'     \code{qqplotr::stat_qq_band} for QQ plots or
#'     \code{qqplotr::stat_pp_band} for PP plots.
#'     \item Converts \code{band = TRUE} to an empty list (default
#'     arguments).
#'     \item Normalises a single band (non-list or named list) into a
#'     list-of-lists format.
#'     \item Iterates over bands, assigning each a default fill aesthetic
#'     (\code{"Band_1"}, \code{"Band_2"}, ..., up to a maximum of 10).
#'     The user can override the fill mapping via \code{mapping} inside
#'     each band's argument list.
#'     \item Sets \code{alpha} per band, falling back to the
#'     \code{band_alpha} parameter.
#'     \item Adds each band to the plot via \code{do_call(band_fn, bnd)}.
#'   }
#'   \item \strong{Legend position resolution} -- if no bands were rendered
#'   or all bands use default \code{"Band_"} names, the legend defaults to
#'   \code{"none"} (when \code{legend.position} is a \code{waiver});
#'   otherwise defaults to \code{"right"}.
#'   \item \strong{Reference line} -- when \code{line} is not \code{NULL},
#'   adds \code{qqplotr::stat_qq_line} (QQ) or
#'   \code{qqplotr::stat_pp_line} (PP) via \code{do_call()}.
#'   \item \strong{Points} -- when \code{point} is not \code{NULL}, adds
#'   \code{qqplotr::stat_qq_point} (QQ) or
#'   \code{qqplotr::stat_pp_point} (PP) via \code{do_call()}.
#'   \item \strong{Fill colour scale} -- when bands are present,
#'   \code{scale_fill_manual()} is added with colours resolved via
#'   \code{\link{palette_this}()} using the band names, \code{palette},
#'   \code{palcolor}, and \code{palreverse}. The legend title is set to
#'   \code{fill_name}.
#'   \item \strong{Axis limits} -- \code{ggplot2::xlim()} and
#'   \code{ggplot2::ylim()} are applied if \code{xlim} or \code{ylim} are
#'   set.
#'   \item \strong{Labels and theme} -- \code{labs(title, subtitle, x, y)}
#'   with fallback to \code{val} for axis labels; \code{do_call(theme,
#'   theme_args)}; \code{ggplot2::theme()} with \code{aspect.ratio},
#'   \code{panel.grid.major} (grey80, dashed), \code{legend.position},
#'   and \code{legend.direction}.
#'   \item \strong{Dimension calculation} --
#'   \code{\link{calculate_plot_dimensions}()} with \code{base_height =
#'   4.5}, \code{aspect.ratio}, and legend metrics (number of bands, band
#'   name character width).
#'   \item \strong{Faceting} -- \code{\link{facet_plot}()} applies
#'   \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is provided.
#' }
#'
#' @inheritParams common_args
#' @param val A character string naming the numeric column whose distribution
#'   is compared against the theoretical distribution.
#' @param val_trans A transformation function applied to the \code{val}
#'   column before plotting. For example, \code{log} or \code{sqrt}.
#'   Default: \code{NULL} (no transformation).
#' @param type A character string specifying the plot type. Either
#'   \code{"qq"} (quantile-quantile, the default) or \code{"pp"}
#'   (probability-probability). Partial matching is supported.
#' @param band A list of arguments passed to \code{\link[qqplotr]{stat_qq_band}}
#'   or \code{\link[qqplotr]{stat_pp_band}}, depending on \code{type}.
#'   Set to \code{TRUE} or an empty list to use default arguments.
#'   Set to \code{NULL} (the default) to suppress bands entirely.
#'   To add multiple bands, provide a list of lists, each containing
#'   arguments for one band (e.g. different \code{bandType} or
#'   \code{distribution}). Each band can also include a custom
#'   \code{mapping} aesthetic to control its fill colour legend entry.
#' @param line A list of arguments passed to \code{\link[qqplotr]{stat_qq_line}}
#'   or \code{\link[qqplotr]{stat_pp_line}}, depending on \code{type}.
#'   Default: \code{list()} (adds a reference line with default arguments).
#'   Set to \code{NULL} to omit the line entirely.
#' @param point A list of arguments passed to \code{\link[qqplotr]{stat_qq_point}}
#'   or \code{\link[qqplotr]{stat_pp_point}}, depending on \code{type}.
#'   Default: \code{list()} (adds points with default arguments).
#'   Set to \code{NULL} to omit points (not recommended).
#' @param fill_name A character string for the fill legend title used
#'   when bands are present. Default: \code{"Bands"}.
#' @param band_alpha A numeric value in \code{[0, 1]} setting the
#'   transparency of all bands. Individual bands can override this via
#'   \code{alpha} inside the \code{band} argument list.
#'   Default: \code{0.5}.
#' @param seed A numeric value for the random seed used internally.
#'   Default: \code{8525}. Passed to \code{\link{set.seed}}.
#' @param xlim A numeric vector of length 2 specifying the x-axis limits.
#'   Default: \code{NULL} (use data range).
#' @param ylim A numeric vector of length 2 specifying the y-axis limits.
#'   Default: \code{NULL} (use data range).
#' @keywords internal
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'   attributes (in inches).
#' @importFrom rlang eval_tidy as_name
#' @importFrom utils modifyList
#' @importFrom ggplot2 aes waiver scale_fill_manual labs
QQPlotAtomic <- function(
    data,
    val,
    val_trans = NULL,
    type = c("qq", "pp"),
    band = NULL,
    line = list(),
    point = list(),
    fill_name = "Bands",
    band_alpha = 0.5,
    theme = "theme_this",
    theme_args = list(),
    palette = "Spectral",
    palcolor = NULL,
    palreverse = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    legend.position = waiver(),
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    seed = 8525,
    xlim = NULL,
    ylim = NULL,
    xlab = ifelse(type == "qq", "Theoretical Quantiles", "Probability Points"),
    ylab = ifelse(type == "qq", "Sample Quantiles", "Cumulative Probability"),
    ...
) {
    set.seed(seed)
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    type <- match.arg(type)
    stopifnot(
        "[QQPlot] 'band' must be TRUE, a list or NULL" = isTRUE(band) ||
            is.list(band) ||
            is.null(band)
    )
    stopifnot(
        "[QQPlot] 'line' must be a list or NULL" = is.list(line) ||
            is.null(line)
    )
    stopifnot(
        "[QQPlot] 'point' must be a list or NULL" = is.list(point) ||
            is.null(point)
    )
    stopifnot(
        "[QQPlot] 'xlim' must be a numeric vector of length 2 or NULL" = is.null(
            xlim
        ) ||
            (is.numeric(xlim) && length(xlim) == 2)
    )
    stopifnot(
        "[QQPlot] 'ylim' must be a numeric vector of length 2 or NULL" = is.null(
            ylim
        ) ||
            (is.numeric(ylim) && length(ylim) == 2)
    )

    val <- check_columns(data, val)

    if (!is.null(val_trans)) {
        data[[val]] <- val_trans(data[[val]])
    }

    p <- ggplot(data, aes(sample = !!sym(val)))

    bands <- c()
    if (!is.null(band)) {
        band_fn <- if (type == "qq") {
            qqplotr::stat_qq_band
        } else {
            qqplotr::stat_pp_band
        }
        if (isTRUE(band)) {
            band <- list()
        }
        if (length(band) == 0 || !is.null(names(band))) {
            # single band
            band <- list(band)
        }
        for (i in seq_along(band)) {
            bnd <- band[[i]]
            if (i == 1) {
                # Using paste, aes does not evaluate eagerly, is there a better way?
                default_bnd_fill <- aes(fill = "Band_1")
            } else if (i == 2) {
                default_bnd_fill <- aes(fill = "Band_2")
            } else if (i == 3) {
                default_bnd_fill <- aes(fill = "Band_3")
            } else if (i == 4) {
                default_bnd_fill <- aes(fill = "Band_4")
            } else if (i == 5) {
                default_bnd_fill <- aes(fill = "Band_5")
            } else if (i == 6) {
                default_bnd_fill <- aes(fill = "Band_6")
            } else if (i == 7) {
                default_bnd_fill <- aes(fill = "Band_7")
            } else if (i == 8) {
                default_bnd_fill <- aes(fill = "Band_8")
            } else if (i == 9) {
                default_bnd_fill <- aes(fill = "Band_9")
            } else if (i == 10) {
                default_bnd_fill <- aes(fill = "Band_10")
            } else {
                stop(
                    "[QQPlot] Too many bands! Please specify the fill aesthetic manually."
                )
            }
            bnd$mapping <- bnd$mapping %||% default_bnd_fill
            if (is.null(bnd$mapping$fill)) {
                bnd$mapping <- modifyList(bnd$mapping, default_bnd_fill)
            }
            bnd$alpha <- bnd$alpha %||% band_alpha
            bands <- c(bands, as_name(eval_tidy(bnd$mapping)$fill))

            p <- p + do_call(band_fn, bnd)
        }
    }

    if (length(bands) == 0 || all(startsWith(bands, "Band_"))) {
        legend.position <- ifelse(
            inherits(legend.position, "waiver"),
            "none",
            legend.position
        )
    } else {
        legend.position <- ifelse(
            inherits(legend.position, "waiver"),
            "right",
            legend.position
        )
    }

    if (!is.null(line)) {
        line_fn <- if (type == "qq") {
            qqplotr::stat_qq_line
        } else {
            qqplotr::stat_pp_line
        }
        p <- p + do_call(line_fn, line)
    }

    if (!is.null(point)) {
        point_fn <- if (type == "qq") {
            qqplotr::stat_qq_point
        } else {
            qqplotr::stat_pp_point
        }
        p <- p + do_call(point_fn, point)
    }

    if (length(bands) > 0) {
        p <- p +
            scale_fill_manual(
                name = fill_name,
                values = palette_this(
                    bands,
                    palette = palette,
                    palcolor = palcolor,
                    reverse = palreverse
                )
            )
    }
    if (!is.null(xlim)) {
        p <- p + ggplot2::xlim(xlim[1], xlim[2])
    }
    if (!is.null(ylim)) {
        p <- p + ggplot2::ylim(ylim[1], ylim[2])
    }
    p <- p +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% val,
            y = ylab %||% val
        ) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            panel.grid.major = element_line(colour = "grey80", linetype = 2),
            legend.position = legend.position,
            legend.direction = legend.direction
        )

    dims <- calculate_plot_dimensions(
        base_height = 4.5,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = length(bands),
        legend_nchar = if (length(bands) > 0) max(nchar(bands)) else 5
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction
    )
}


#' QQ/PP plot
#'
#' @description
#' Produces a quantile-quantile (QQ) plot or probability-probability (PP)
#' plot to compare the empirical distribution of a numeric variable against
#' a theoretical distribution (default: standard normal). The function
#' delegates to the \pkg{qqplotr} package for the underlying statistics
#' and rendering.
#'
#' Key features:
#' \itemize{
#'   \item \strong{QQ and PP modes} -- switch between quantile-quantile
#'   and probability-probability displays via \code{type}.
#'   \item \strong{Confidence bands} -- overlay one or more confidence bands
#'   (pointwise, KS, Tukey simultaneous, or bootstrap) with custom fill
#'   colours and alpha.
#'   \item \strong{Reference line} -- a diagonal reference line (QQ) or
#'   diagonal probability line (PP) for comparison.
#'   \item \strong{Distribution fitting} -- compare against any
#'   distribution supported by \pkg{qqplotr} (normal, exponential, uniform,
#'   etc.) by passing \code{distribution} and \code{dparams} inside the
#'   \code{band}, \code{line}, and \code{point} lists.
#'   \item \strong{Detrending} -- enable \code{detrend = TRUE} inside the
#'   argument lists to remove the reference line and visualise only
#'   deviations (flat PP plot centred at zero).
#'   \item \strong{Splitting} -- use \code{split_by} to produce separate
#'   QQ/PP plots for different groups, combined into a single layout.
#' }
#'
#' @section split_by Workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \strong{Common arg validation} --
#'   \code{\link{validate_common_args}()} checks the \code{seed} and
#'   \code{facet_by} constraints.
#'   \item \strong{Theme processing} -- \code{\link{process_theme}()}
#'   resolves the \code{theme} string or function.
#'   \item \strong{split_by column resolution} --
#'   \code{\link{check_columns}()} validates the \code{split_by} column(s)
#'   with \code{force_factor = TRUE}. Multiple columns are concatenated
#'   with \code{split_by_sep}.
#'   \item \strong{Data splitting} -- the data frame is split by
#'   \code{split_by} levels (droplevels applied, level order preserved).
#'   If \code{split_by} is \code{NULL}, the data is wrapped in a
#'   single-element list with name \code{"..."}.
#'   \item \strong{Per-split parameter resolution} --
#'   \code{\link{check_palette}()}, \code{\link{check_palcolor}()}, and
#'   \code{\link{check_legend}()} resolve per-split \code{palette},
#'   \code{palcolor}, \code{legend.position}, and
#'   \code{legend.direction}.
#'   \item \strong{Dispatch per split} -- \code{\link{QQPlotAtomic}()} is
#'   called for each split level. If \code{title} is a function, it
#'   receives the split level name and generates a dynamic title;
#'   otherwise the level name is used as the default title.
#'   \item \strong{Combination} -- results are combined via
#'   \code{\link{combine_plots}()} (when \code{combine = TRUE}) or
#'   returned as a named list (when \code{combine = FALSE}).
#' }
#'
#' @inheritParams common_args
#' @inheritParams QQPlotAtomic
#' @return A \code{ggplot} object (single plot), a \code{patchwork} object
#'   (combined split plots), or a named list of \code{ggplot} objects
#'   (when \code{combine = FALSE}), each with \code{height} and
#'   \code{width} attributes in inches.
#' @importFrom ggplot2 waiver
#' @export
#' @examples
#' set.seed(8525)
#' data <- data.frame(norm = rnorm(100))
#'
#' # Basic QQ plot with default confidence band
#' QQPlot(data, val = "norm", band = TRUE)
#'
#' # Multiple confidence bands with custom fill labels
#' QQPlot(data, val = "norm", band = list(
#'     list(bandType = "ks", mapping = ggplot2::aes(fill = "KS"), alpha = 0.3),
#'     list(bandType = "ts", mapping = ggplot2::aes(fill = "TS")),
#'     list(bandType = "pointwise", mapping = ggplot2::aes(fill = "Normal")),
#'     list(bandType = "boot", mapping = ggplot2::aes(fill = "Bootstrap"))
#' ), band_alpha = 0.6)
#'
#' # Compare against exponential distribution
#' data(airquality, package = "datasets")
#' di <- "exp"
#' dp <- list(rate = 2)
#' QQPlot(airquality, val = "Ozone",
#'     band = list(distribution = di, dparams = dp),
#'     line = list(distribution = di, dparams = dp),
#'     point = list(distribution = di, dparams = dp)
#' )
#'
#' # Detrended QQ plot: deviations from the reference line
#' de <- TRUE
#' QQPlot(airquality, val = "Ozone",
#'     band = list(distribution = di, dparams = dp, detrend = de),
#'     line = list(distribution = di, dparams = dp, detrend = de),
#'     point = list(distribution = di, dparams = dp, detrend = de)
#' )
#'
#' # PP plot (probability-probability)
#' QQPlot(data, val = "norm", type = "pp", band = TRUE)
#'
#' # PP plot with shifted/scaled normal distribution
#' dp <- list(mean = 2, sd = 2)
#' QQPlot(data, val = "norm", type = "pp",
#'     band = list(dparams = dp),
#'     point = list(dparams = dp))
#'
#' # PP plot with custom intercept/slope line
#' QQPlot(data, val = "norm", type = "pp", band = TRUE,
#'     line = list(ab = c(.2, .5)))
#'
#' # Detrended PP plot with axis limits
#' di <- "exp"
#' dp <- list(rate = .022)
#' de <- TRUE
#' QQPlot(airquality, val = "Ozone", type = "pp",
#'     band = list(distribution = di, detrend = de, dparams = dp),
#'     line = list(detrend = de),
#'     point = list(distribution = di, detrend = de, dparams = dp),
#'     ylim = c(-.5, .5)
#' )
QQPlot <- function(
    data,
    val,
    val_trans = NULL,
    type = c("qq", "pp"),
    split_by = NULL,
    split_by_sep = "_",
    band = NULL,
    line = list(),
    point = list(),
    fill_name = "Bands",
    band_alpha = 0.5,
    theme = "theme_this",
    theme_args = list(),
    palette = "Spectral",
    palcolor = NULL,
    palreverse = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    legend.position = waiver(),
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlim = NULL,
    ylim = NULL,
    xlab = ifelse(type == "qq", "Theoretical Quantiles", "Probability Points"),
    ylab = ifelse(type == "qq", "Sample Quantiles", "Cumulative Probability"),
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    seed = 8525,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    theme <- process_theme(theme)
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            QQPlotAtomic(
                datas[[nm]],
                val = val,
                val_trans = val_trans,
                type = type,
                band = band,
                line = line,
                point = point,
                fill_name = fill_name,
                band_alpha = band_alpha,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                seed = seed,
                xlim = xlim,
                ylim = ylim,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
qq_data <- data.frame(
    val = c(rnorm(80), rnorm(80, mean = 0.5)),
    group = factor(rep(c("A", "B"), each = 80))
)

test_that("QQPlot basic usage works", {
    p <- QQPlot(qq_data, val = "val")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("QQPlot with type = 'pp' works", {
    p <- QQPlot(qq_data, val = "val", type = "pp")
    expect_s3_class(p, "ggplot")
})

test_that("QQPlot with title and subtitle works", {
    p <- QQPlot(qq_data, val = "val", title = "QQ Title", subtitle = "QQ Subtitle")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "QQ Title")
    expect_equal(p$labels$subtitle, "QQ Subtitle")
})

test_that("QQPlot with split_by returns patchwork", {
    p <- QQPlot(qq_data, val = "val", split_by = "group", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("QQPlot with split_by and combine = FALSE returns list", {
    plots <- QQPlot(qq_data, val = "val", split_by = "group", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("QQPlot with multiple split_by columns returns a list with one plot per combination", {
    qq_data2 <- qq_data
    qq_data2$split2 <- factor(rep(c("s1", "s2"), 80))
    plots <- suppressMessages(QQPlot(qq_data2, val = "val",
                                     split_by = c("group", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("QQPlot with multiple split_by columns returns combined plot", {
    qq_data2 <- qq_data
    qq_data2$split2 <- factor(rep(c("s1", "s2"), 80))
    p <- suppressMessages(QQPlot(qq_data2, val = "val",
                                 split_by = c("group", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("QQPlot with xlab and ylab works", {
    p <- QQPlot(qq_data, val = "val", xlab = "Theory", ylab = "Sample")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$x, "Theory")
    expect_equal(p$labels$y, "Sample")
})

test_that("QQPlot with band (qqplotr) works", {
    skip_if_not_installed("qqplotr")
    p <- QQPlot(qq_data, val = "val", band = list())
    expect_s3_class(p, "ggplot")
})

#' Detect the input data format for Venn diagram processing
#'
#' Examines the structure of \code{data} and determines which of the four
#' supported formats it conforms to:
#' \itemize{
#'   \item \code{"long"} — a data frame with one row per element-set pair.
#'   \item \code{"wide"} — a data frame with logical/0-1 columns per set.
#'   \item \code{"list"} — a named list of element vectors per set.
#'   \item \code{"venn"} — a pre-computed \code{VennPlotData} object.
#' }
#'
#' @keywords internal
#' @param data A data frame, a named list, or a \code{VennPlotData} object.
#'  Data frames are classified as \code{"long"} when a single non-\code{NULL}
#'  \code{group_by} is provided, or \code{"wide"} when \code{group_by} has two
#'  or more columns or is \code{NULL}.
#' @param group_by A character string specifying the column name(s) identifying
#'  the set membership.  A single non-\code{NULL} column indicates long format.
#'  Two or more columns, or \code{NULL}, indicate wide format.
#' @param id_by A character string specifying the column name that identifies
#'  individual elements.  Required when \code{group_by} is a single column and
#'  \code{data} is a data frame.  Ignored for other input types.
#' @return A character string indicating the detected data format:
#'  \code{"long"}, \code{"wide"}, \code{"list"}, or \code{"venn"}.  Stops with
#'  an error if the data does not match any recognised type.
detect_venn_datatype <- function(data, group_by = NULL, id_by = NULL) {
    if (inherits(data, "data.frame")) {
        if (length(group_by) < 2 && !is.null(group_by)) {
            return("long")
        } else {
            # length(group_by) > 1
            return("wide")
        }
    } else if (inherits(data, "VennPlotData")) {
        return("venn")
    } else if (is.list(data)) {
        return("list")
    }

    stop(
        "Invalid data type. Please provide a data frame, a list or a VennPlotData object generated by prepare_venn_data()."
    )
}

#' Prepare input data for Venn diagram rendering
#'
#' Converts data in any supported input format into a \code{VennPlotData}
#' object suitable for rendering by \pkg{ggVennDiagram}.  When
#' \code{in_form = "auto"} (the default), the format is detected via
#' \code{\link{detect_venn_datatype}()}.
#'
#' @section Input formats:
#' \enumerate{
#'   \item \strong{Long format} — a data frame with a grouping column
#'         (\code{group_by}) identifying the set and an ID column
#'         (\code{id_by}) identifying each element.  Multiple \code{group_by}
#'         columns are concatenated with \code{group_by_sep}.
#' \preformatted{
#' group_by id_by
#' A        a1
#' A        a2
#' B        a1
#' B        a3
#' }
#'   \item \strong{Wide format} — a data frame where each column represents a
#'         set and each row an element.  Values must be logical or
#'         \code{0}/\code{1}.  Columns specified in \code{group_by} define the
#'         sets; if \code{group_by} is \code{NULL}, all columns are used.
#' \preformatted{
#' A    B
#' TRUE TRUE
#' TRUE FALSE
#' FALSE TRUE
#' }
#'   \item \strong{List format} — a named list where each element is a vector
#'         of identifiers belonging to that set.
#' \preformatted{
#' list(A = c("a1", "a2"), B = c("a1", "a3"))
#' }
#'   \item \strong{VennPlotData} — a pre-computed object returned by a
#'         previous call to \code{prepare_venn_data()}.  Returned as-is.
#' }
#'
#' @keywords internal
#' @param data A data frame, a named list, or a \code{VennPlotData} object
#'  to be prepared for Venn diagram rendering.  See the \strong{Input formats}
#'  section for the expected structure of each format.
#' @param in_form A character string specifying the input format.  One of
#'  \code{"auto"} (default; detect automatically via
#'  \code{\link{detect_venn_datatype}()}), \code{"long"}, \code{"wide"},
#'  \code{"list"}, or \code{"venn"}.
#' @param group_by A character string (or vector) specifying the column name(s)
#'  identifying set membership.  For long-format data, a single column defines
#'  the set; multiple columns are concatenated with \code{group_by_sep}.  For
#'  wide-format data, these are the set columns (must be logical or 0/1); when
#'  \code{NULL}, all columns are used as sets.  Ignored for list and
#'  \code{VennPlotData} input.
#' @param group_by_sep A character string used to concatenate multiple
#'  \code{group_by} columns when \code{in_form = "long"}.  Default \code{"_"}.
#' @param id_by A character string specifying the column name that identifies
#'  individual elements.  Required for long-format data; ignored otherwise.
#' @return A \code{VennPlotData} object suitable for rendering by
#'  \pkg{ggVennDiagram}.
prepare_venn_data <- function(
    data,
    in_form = "auto",
    group_by = NULL,
    group_by_sep = "_",
    id_by = NULL
) {
    # if (!requireNamespace("ggVennDiagram", quietly = TRUE)) {
    #     stop("ggVennDiagram is required for Venn diagram and its data processing.")
    # }
    if (in_form == "auto") {
        in_form <- detect_venn_datatype(data, group_by, id_by)
    }
    if (in_form == "venn") {
        if (!is.null(group_by)) {
            warning(
                "The 'group_by' argument is ignored when the input data is already a 'VennPlotData' object.",
                immediate. = TRUE
            )
        }
        return(data)
    }

    if (in_form == "list") {
        if (!is.list(data)) {
            stop("The input data must be a list when the in_form is 'list'.")
        }
        listdata <- data
    } else if (in_form == "long") {
        group_by <- check_columns(
            data,
            group_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = group_by_sep
        )
        listdata <- split(data[[id_by]], data[[group_by]])
    } else {
        # in_form == "wide"
        group_by <- check_columns(data, group_by, allow_multi = TRUE)
        if (is.null(group_by)) {
            group_by <- colnames(data)
        }
        for (g in group_by) {
            # columns must be logical or 0/1
            if (!is.logical(data[[g]]) && !all(data[[g]] %in% c(0, 1))) {
                stop(
                    "The columns in group_by must be logical or 0/1 when the in_form is 'wide'."
                )
            }
        }
        data$.id <- paste0("id", seq_len(nrow(data)))
        listdata <- lapply(group_by, function(g) {
            data[as.logical(data[[g]]), ".id", drop = TRUE]
        })
        names(listdata) <- group_by
        for (nm in names(listdata)) {
            if (length(listdata[[nm]]) == 0) {
                warning(
                    "The group '",
                    nm,
                    "' has no elements, ignored.",
                    immediate. = TRUE
                )
                listdata[[nm]] <- NULL
            }
        }
    }
    ggVennDiagram::process_data(ggVennDiagram::Venn(listdata))
}

#' Atomic Venn / Euler diagram (internal)
#'
#' @description
#' Core implementation for drawing a Venn or Euler diagram that visualises
#' the overlap relationships among multiple sets.  Supports four input formats
#' (long, wide, list, and pre-computed \code{VennPlotData}) which are
#' normalised by \code{\link{prepare_venn_data}()} into the internal
#' \code{VennPlotData} representation used by \pkg{ggVennDiagram}.
#'
#' Intersection regions can be filled either by a continuous colour gradient
#' based on the element count (\code{fill_mode = "count"} or \code{"count_rev"})
#' or by blended set colours (\code{fill_mode = "set"}).  Region labels can
#' display the raw count, the percentage of total elements, both, nothing, or
#' a custom function result.  Set labels always show the set name and its
#' total element count.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{Data type detection} — \code{\link{detect_venn_datatype}()}
#'         identifies whether \code{data} is in long, wide, list, or
#'         \code{VennPlotData} format.
#'   \item \strong{Data preparation} — \code{\link{prepare_venn_data}()}
#'         converts the input into a \code{VennPlotData} object suitable for
#'         rendering by \pkg{ggVennDiagram}.
#'   \item \strong{Geometry extraction} — \pkg{ggVennDiagram} utilities
#'         (\code{venn_regionedge()}, \code{venn_setedge()},
#'         \code{venn_regionlabel()}, \code{venn_setlabel()}) compute polygon
#'         vertices and label positions for all sets and their intersections.
#'   \item \strong{Fill resolution} — When \code{fill_mode = "set"},
#'         \code{\link{palette_this}()} assigns colours to each set; the
#'         \code{blend_colors()} helper blends the colours of overlapping sets
#'         for each intersection region.  When \code{fill_mode = "count"} or
#'         \code{"count_rev"}, a continuous gradient is applied across regions
#'         via \code{scale_fill_gradientn()} with a colour bar legend.
#'   \item \strong{Label computation} — Region labels are formatted per the
#'         \code{label} parameter: raw count, percentage, both, none, or a
#'         custom function that receives a data frame with columns
#'         \code{"id"}, \code{"X"}, \code{"Y"}, \code{"name"}, \code{"item"},
#'         and \code{"count"}.  Set labels are formatted as
#'         \code{"setName\\n(count)"}.
#'   \item \strong{Plot assembly} — \code{geom_polygon()} draws filled
#'         regions; \code{geom_path()} draws set outlines; two
#'         \code{\link[ggrepel]{geom_text_repel}()} layers add region labels
#'         and set labels.  For \code{fill_mode = "set"}, colours are mapped
#'         directly (no aesthetic mapping, no legend); for count modes, a
#'         \code{scale_fill_gradientn()} continuous scale is used.
#'   \item \strong{Theme and coordinate system} — \code{coord_equal()}
#'         enforces a square aspect ratio.  The theme removes all axis text,
#'         ticks, titles, grid lines, and panel borders.  The x-axis
#'         expansion is widened to accommodate set label text width.
#'   \item \strong{Dimension calculation} —
#'         \code{\link{calculate_plot_dimensions}()} computes \code{height}
#'         and \code{width} attributes (in inches) from \code{base_height},
#'         \code{aspect.ratio}, and legend metrics.  When
#'         \code{fill_mode = "set"}, the legend position is forced to
#'         \code{"none"}.
#' }
#'
#' @inheritParams common_args
#' @inheritParams prepare_venn_data
#' @param group_by A character string (or vector) specifying the column name(s)
#'  that define the set membership.  For \code{in_form = "long"}, this is the
#'  grouping column; for \code{in_form = "wide"}, these are the set columns
#'  (must be logical or 0/1); when \code{NULL} and \code{data} is a data
#'  frame, all columns are treated as sets (wide format).
#' @param label A character string or function controlling the text shown in
#'  each intersection region.  One of:
#'  \itemize{
#'    \item \code{"count"} (default) — the raw count of elements in that region.
#'    \item \code{"percent"} — the percentage of the total element count.
#'    \item \code{"both"} — count and percentage on separate lines.
#'    \item \code{"none"} — no region labels are drawn.
#'    \item A \strong{function} — receives a data frame with columns
#'          \code{"id"}, \code{"X"}, \code{"Y"}, \code{"name"}, \code{"item"},
#'          and \code{"count"}, and must return a character vector of labels.
#'  }
#' @param label_fg A character string specifying the colour of the label text.
#' @param label_size A numeric value specifying the font size of the label
#'  text.  When \code{NULL} (the default), auto-sized at 3.5 for region labels
#'  and 4 for set labels, scaled by \code{base_size / 12}.
#' @param label_bg A character string specifying the background colour of the
#'  label text (passed to \code{\link[ggrepel]{geom_text_repel}()} as
#'  \code{bg.color}).  Default \code{"white"}.
#' @param label_bg_r A numeric value specifying the corner radius of the label
#'  background rectangle (passed as \code{bg.r}).  Default \code{0.1}.
#' @param fill_mode A character string specifying how intersection regions are
#'  coloured.  One of:
#'  \itemize{
#'    \item \code{"count"} — continuous gradient based on element count
#'          (default palette: \code{"Spectral"}).
#'    \item \code{"count_rev"} — continuous gradient with reversed count order.
#'    \item \code{"set"} — discrete blended colours per set combination
#'          (default palette: \code{"Paired"}).  No legend is drawn.
#'  }
#' @param fill_name A character string for the colour bar legend title when
#'  \code{fill_mode} is \code{"count"} or \code{"count_rev"}.  Ignored when
#'  \code{fill_mode = "set"}.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @keywords internal
#' @importFrom rlang %||%
#' @importFrom ggplot2 geom_polygon geom_path coord_equal aes scale_x_continuous labs
#' @importFrom ggrepel geom_text_repel
VennDiagramAtomic <- function(
    data,
    in_form = "auto",
    group_by = NULL,
    group_by_sep = "_",
    id_by = NULL,
    label = "count",
    label_fg = "black",
    label_size = NULL,
    label_bg = "white",
    label_bg_r = 0.1,
    fill_mode = "count",
    palreverse = FALSE,
    fill_name = NULL,
    aspect.ratio = 1,
    palette = ifelse(fill_mode == "set", "Paired", "Spectral"),
    palcolor = NULL,
    alpha = 1,
    theme = "theme_this",
    theme_args = list(),
    title = NULL,
    subtitle = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    # if (!requireNamespace("ggVennDiagram", quietly = TRUE)) {
    #     stop("ggVennDiagram (v1.5+) is required for Venn diagram.")
    # }
    base_size <- theme_args$base_size %||% 12
    text_size_scale <- base_size / 12

    s <- data$.split
    data$.split <- NULL
    fill_mode <- match.arg(fill_mode, c("count", "set", "count_rev"))
    data <- prepare_venn_data(data, in_form, group_by, group_by_sep, id_by)
    data_regionedge <- ggVennDiagram::venn_regionedge(data)
    data_setedge <- ggVennDiagram::venn_setedge(data)
    data_regionlabel <- ggVennDiagram::venn_regionlabel(data)
    data_regionlabel$.split <- if (is.null(s)) NULL else s[1]
    data_setlabel <- ggVennDiagram::venn_setlabel(data)
    # Calculate the fill colors for the regions when fill_mode is set
    if (fill_mode == "set") {
        # 1: red 2: blue 3: green 4: purple
        colors <- palette_this(
            data$setLabel$id,
            palette = palette,
            palcolor = palcolor,
            reverse = palreverse
        )
        ids <- unique(data_regionedge$id)
        # 1: red 2: blue 3: green 4: purple
        # 1/2: blend(red, blue) 1/3: blend(red, green) 1/4: blend(red, purple)
        # ...
        blended_colors <- sapply(ids, function(id) {
            parts <- strsplit(id, "/")[[1]]
            blend_colors(colors[parts])
        })
        # "1" being transformed to "1.1" after sapply
        names(blended_colors) <- ids
        data_regionedge$fill <- blended_colors[data_regionedge$id]
        data_setedge$color <- colors[data_setedge$id]
    }

    data_setlabel$label <- paste0(
        data_setlabel$name,
        "\n(",
        data_setlabel$count,
        ")"
    )
    if (identical(label, "percent")) {
        data_regionlabel$label <- scales::percent(
            data_regionlabel$count / sum(data_regionlabel$count)
        )
    } else if (identical(label, "both")) {
        data_regionlabel$label <- paste(
            data_regionlabel$count,
            "\n",
            scales::percent(
                data_regionlabel$count / sum(data_regionlabel$count)
            )
        )
    } else if (identical(label, "count")) {
        data_regionlabel$label <- data_regionlabel$count
    } else if (is.function(label)) {
        data_regionlabel$label <- label(data_regionlabel)
    } else if (!identical(label, "none")) {
        stop(
            "Invalid label argument. Possible values are 'count', 'percent', 'both', 'none' or a function."
        )
    }

    p <- ggplot()

    if (fill_mode == "set") {
        p <- p +
            geom_polygon(
                data = data_regionedge,
                aes(!!sym("X"), !!sym("Y"), group = !!sym("id")),
                fill = data_regionedge$fill,
                alpha = alpha
            ) +
            geom_path(
                data = data_setedge,
                aes(!!sym("X"), !!sym("Y"), group = !!sym("id")),
                color = data_setedge$color,
                show.legend = FALSE
            )
    } else {
        p <- p +
            geom_polygon(
                data = data_regionedge,
                aes(
                    !!sym("X"),
                    !!sym("Y"),
                    fill = !!sym("count"),
                    group = !!sym("id")
                ),
                alpha = alpha
            ) +
            scale_fill_gradientn(
                n.breaks = 3,
                colors = palette_this(
                    palette = palette,
                    palcolor = palcolor,
                    reverse = palreverse
                ),
                na.value = "grey80",
                guide = guide_colorbar(
                    title = fill_name %||% "",
                    frame.colour = "black",
                    ticks.colour = "black",
                    title.hjust = 0
                )
            ) +
            geom_path(
                data = data_setedge,
                aes(
                    !!sym("X"),
                    !!sym("Y"),
                    color = !!sym("id"),
                    group = !!sym("id")
                ),
                color = "grey20",
                show.legend = FALSE
            )
    }

    if (!identical(label, "none")) {
        p <- p +
            geom_text_repel(
                data = data_regionlabel,
                aes(!!sym("X"), !!sym("Y"), label = !!sym("label")),
                color = label_fg,
                bg.color = label_bg,
                bg.r = label_bg_r,
                size = label_size %||% text_size_scale * 3.5,
                point.size = NA,
                max.overlaps = 100,
                force = 0,
                min.segment.length = 0,
                segment.colour = "black"
            )
    }

    p <- p +
        geom_text_repel(
            data = data_setlabel,
            aes(!!sym("X"), !!sym("Y"), label = !!sym("label")),
            color = label_fg,
            bg.color = label_bg,
            bg.r = label_bg_r,
            size = label_size %||% text_size_scale * 4,
            fontface = "bold",
            point.size = NA,
            max.overlaps = 100,
            force = 0,
            min.segment.length = 0,
            segment.colour = "black"
        ) +
        labs(title = title, subtitle = subtitle) +
        coord_equal() +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            axis.text = element_blank(),
            axis.ticks = element_blank(),
            axis.title = element_blank(),
            panel.border = element_blank()
        )

    maxchars <- max(sapply(data_setlabel$name, nchar))
    p <- p +
        scale_x_continuous(
            expand = expansion(
                add = 0.001 * maxchars * text_size_scale,
                mult = 0.1
            ),
        )

    dims <- calculate_plot_dimensions(
        base_height = 5.5,
        aspect.ratio = aspect.ratio,
        legend.position = if (fill_mode == "set") "none" else legend.position,
        legend.direction = legend.direction
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    return(p)
}


#' Venn / Euler diagram
#'
#' @description
#' Draws Venn or Euler diagrams that visualise the overlap relationships among
#' multiple sets.  Supports four input formats: long (one row per element-set
#' pair), wide (logical/0-1 columns per set), a named list (element vectors
#' per set), and a pre-computed \code{VennPlotData} object.
#'
#' Intersection regions can be filled by a continuous colour gradient encoding
#' the element count (\code{fill_mode = "count"} / \code{"count_rev"}) or by
#' blended set colours (\code{fill_mode = "set"}).  Region labels can display
#' counts, percentages, both, or a custom function.  Set labels always show
#' the set name and its total element count.
#'
#' Use \code{split_by} to produce separate Venn diagrams for each level of a
#' grouping variable.  Note that \code{split_by} is only supported when
#' \code{data} is a data frame (list and \code{VennPlotData} inputs cannot be
#' split).
#'
#' @section split_by Workflow:
#' When a non-\code{NULL} \code{split_by} is provided and the input is a
#' data frame:
#' \enumerate{
#'   \item \strong{Validation} — an error is raised if \code{data} is not a
#'         data frame (list and \code{VennPlotData} input cannot be split).
#'   \item \strong{Column resolution} — \code{split_by} is validated and
#'         optionally concatenated via \code{\link{check_columns}()} with
#'         \code{force_factor = TRUE} and \code{allow_multi = TRUE}.
#'   \item \strong{Data splitting} — the data frame is split by the unique
#'         levels of the \code{split_by} column, preserving factor level
#'         order.  Empty levels are dropped via \code{droplevels()}.
#'   \item \strong{Per-split colour and legend resolution} —
#'         \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
#'         and \code{\link{check_legend}()} resolve per-split palettes,
#'         custom colours, legend positions, and legend directions.
#'   \item \strong{Atomic dispatch} — \code{\link{VennDiagramAtomic}()} is
#'         called for each subset.  When \code{title} is a function, it
#'         receives the split level name for dynamic title generation.
#'   \item \strong{Combination} — results are passed to
#'         \code{\link{combine_plots}()} which returns a combined
#'         \code{patchwork} object (when \code{combine = TRUE}) or a named
#'         list of individual ggplot objects (when \code{combine = FALSE}).
#' }
#'
#' @inheritParams common_args
#' @inheritParams VennDiagramAtomic
#' @param split_by The column(s) to split the data by and produce separate
#'  Venn diagrams per subgroup.  Only supported when \code{data} is a data
#'  frame.  Multiple columns are concatenated with \code{split_by_sep}.
#' @param split_by_sep A character string to separate concatenated
#'  \code{split_by} columns.  Default \code{"_"}.
#' @param seed A numeric seed for reproducibility.  Default \code{8525}.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'  \code{patchwork} object.  When \code{FALSE}, returns a named list of
#'  individual \code{ggplot} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined layout
#'  when \code{combine = TRUE}.
#' @param byrow Logical; fill the combined layout by row (default \code{TRUE}).
#' @param axes,axis_titles Character strings specifying how axes and axis
#'  titles are handled across the combined layout.
#' @param guides A character string specifying how legends are collected
#'  across panels in the combined layout.
#' @param design A custom layout specification for the combined plot.
#'  Passed to \code{\link[patchwork]{wrap_plots}()}.
#' @return A \code{ggplot} object (single split), a \code{patchwork} object
#'  (combined sub-plots), or a named list of \code{ggplot} objects (when
#'  \code{combine = FALSE}), each with \code{height} and \code{width}
#'  attributes in inches.
#' @export
#' @examples
#' \donttest{
#' set.seed(8525)
#' data <- list(
#'     A = sort(sample(letters, 8)),
#'     B = sort(sample(letters, 8)),
#'     C = sort(sample(letters, 8)),
#'     D = sort(sample(letters, 8))
#' )
#'
#' # Basic Venn diagram with count labels
#' VennDiagram(data)
#'
#' # Fill by set membership (blended colours)
#' VennDiagram(data, fill_mode = "set")
#'
#' # Show both count and percentage
#' VennDiagram(data, label = "both")
#'
#' # Custom label function using set names
#' VennDiagram(data, label = function(df) df$name)
#'
#' # Custom palette and transparency
#' VennDiagram(data, palette = "material-indigo", alpha = 0.6)
#' }
VennDiagram <- function(
    data,
    in_form = c("auto", "long", "wide", "list", "venn"),
    split_by = NULL,
    split_by_sep = "_",
    group_by = NULL,
    group_by_sep = "_",
    id_by = NULL,
    label = "count",
    label_fg = "black",
    label_size = NULL,
    label_bg = "white",
    label_bg_r = 0.1,
    fill_mode = "count",
    palreverse = FALSE,
    fill_name = NULL,
    palette = ifelse(fill_mode == "set", "Paired", "Blues"),
    palcolor = NULL,
    alpha = 1,
    theme = "theme_this",
    theme_args = list(),
    title = NULL,
    subtitle = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    aspect.ratio = 1,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    seed = 8525,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed)
    in_form <- match.arg(in_form)
    theme <- process_theme(theme)
    if (!is.null(split_by) && !inherits(data, "data.frame")) {
        stop("'split_by' is only available for data frame.")
    }
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            if (length(datas) > 1 || !identical(nm, "...")) {
                datas[[nm]]$.split <- nm
            }
            VennDiagramAtomic(
                datas[[nm]],
                in_form = in_form,
                group_by = group_by,
                group_by_sep = group_by_sep,
                id_by = id_by,
                label = label,
                label_fg = label_fg,
                label_size = label_size,
                label_bg = label_bg,
                label_bg_r = label_bg_r,
                fill_mode = fill_mode,
                palreverse = palreverse,
                fill_name = fill_name,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                alpha = alpha,
                theme = theme,
                theme_args = theme_args,
                title = title,
                subtitle = subtitle,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
# VennDiagram accepts list input or data.frame (long/wide)
venn_list <- list(
    SetA = c("gene1", "gene2", "gene3", "gene4", "gene5"),
    SetB = c("gene3", "gene4", "gene5", "gene6", "gene7"),
    SetC = c("gene2", "gene4", "gene6", "gene8", "gene9")
)

# Long-format data frame
venn_long <- data.frame(
    id    = c("gene1", "gene2", "gene3", "gene4", "gene5",
               "gene3", "gene4", "gene5", "gene6", "gene7"),
    group = c(rep("SetA", 5), rep("SetB", 5))
)

test_that("VennDiagram with list input works", {
    skip_if_not_installed("ggVennDiagram")
    p <- VennDiagram(venn_list)
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("VennDiagram with title works", {
    skip_if_not_installed("ggVennDiagram")
    p <- VennDiagram(venn_list, title = "Venn Title")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "Venn Title")
})

test_that("VennDiagram with long data frame input works", {
    skip_if_not_installed("ggVennDiagram")
    p <- VennDiagram(venn_long, in_form = "long", id_by = "id", group_by = "group")
    expect_s3_class(p, "ggplot")
})

test_that("VennDiagram with fill_mode = 'count' works", {
    skip_if_not_installed("ggVennDiagram")
    p <- VennDiagram(venn_list, fill_mode = "count")
    expect_s3_class(p, "ggplot")
})

test_that("VennDiagram with fill_mode = 'set' works", {
    skip_if_not_installed("ggVennDiagram")
    p <- VennDiagram(venn_list, fill_mode = "set")
    expect_s3_class(p, "ggplot")
})

test_that("VennDiagram with label works", {
    skip_if_not_installed("ggVennDiagram")
    p <- VennDiagram(venn_list, label = "count")
    expect_s3_class(p, "ggplot")
})

test_that("VennDiagram with split_by returns patchwork", {
    skip_if_not_installed("ggVennDiagram")
    # Two separate Venn diagrams using split_by
    venn_split_data <- data.frame(
        id    = c("g1", "g2", "g3", "g4", "g2", "g3", "g5", "g6",
                  "g1", "g2", "g3", "g7", "g2", "g3", "g8", "g9"),
        group = c(rep("SetA", 4), rep("SetB", 4), rep("SetA", 4), rep("SetB", 4)),
        split = factor(rep(c("S1", "S2"), each = 8))
    )
    p <- VennDiagram(venn_split_data, in_form = "long", id_by = "id",
                     group_by = "group", split_by = "split", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("VennDiagram with multiple split_by columns returns combined plot", {
    skip_if_not_installed("ggVennDiagram")
    data2 <- data.frame(
        id    = c("g1", "g2", "g3", "g4", "g2", "g3", "g5", "g6",
                  "g1", "g2", "g3", "g7", "g2", "g3", "g8", "g9"),
        group = c(rep("SetA", 4), rep("SetB", 4), rep("SetA", 4), rep("SetB", 4)),
        split = factor(rep(c("S1", "S2"), each = 8))
    )
    data2$split2 <- factor(rep(c("s1", "s2"), 8))
    p <- suppressMessages(VennDiagram(data2, in_form = "long", id_by = "id",
                                      group_by = "group",
                                      split_by = c("split", "split2"), combine = TRUE))
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("VennDiagram with multiple split_by columns returns a list with one plot per combination", {
    skip_if_not_installed("ggVennDiagram")
    data2 <- data.frame(
        id    = c("g1", "g2", "g3", "g4", "g2", "g3", "g5", "g6",
                  "g1", "g2", "g3", "g7", "g2", "g3", "g8", "g9"),
        group = c(rep("SetA", 4), rep("SetB", 4), rep("SetA", 4), rep("SetB", 4)),
        split = factor(rep(c("S1", "S2"), each = 8))
    )
    data2$split2 <- factor(rep(c("s1", "s2"), 8))
    plots <- suppressMessages(VennDiagram(data2, in_form = "long", id_by = "id",
                                          group_by = "group",
                                          split_by = c("split", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

#' Atomic dot/lollipop plot
#'
#' Core implementation for dot plots and lollipop plots. This is the internal
#' workhorse dispatched by both \code{DotPlot()} and \code{LollipopPlot()}. It
#' renders a matrix of points where dot size encodes one variable and dot fill
#' colour encodes another, with optional background stripes and a lollipop
#' (bar + dot) display mode.
#'
#' The function supports two display modes:
#' \itemize{
#'   \item \strong{Dot plot} (\code{lollipop = FALSE}, the default) — renders
#'   \code{ggplot2::geom_point()} with \code{shape = 21} (filled circle). Either
#'   or both axes can be numeric (producing a scatter plot) or factor (producing
#'   a dot matrix). Dot size scales by \code{size_by} (or the per-combination
#'   observation count when \code{size_by} is \code{NULL}) via
#'   \code{scale_size(range = c(size_min, size_max))}. Dot fill follows the
#'   \code{fill_by} column via \code{scale_fill_gradientn()}.
#'   \item \strong{Lollipop plot} (\code{lollipop = TRUE}) — expects a numeric
#'   \code{x} and a factor/character \code{y}. Renders a two-layer bar:
#'   an outer shadow segment followed by an inner coloured segment, each capped
#'   by a filled dot. The outer shadow is black (or a custom colour when
#'   \code{border_color} is a string), and the inner segment + dot fill follow
#'   \code{fill_by}.
#' }
#'
#' When \code{add_bg = TRUE}, alternating background stripes are drawn via
#' \code{\link{bg_layer}()} along the discrete axis (vertical stripes for
#' factor \code{x}, horizontal stripes for factor \code{y}). The fill colour
#' scale is prepared via \code{\link{prepare_continuous_color_scale}()} and
#' can be trimmed with \code{lower_quantile}/\code{upper_quantile} or
#' \code{lower_cutoff}/\code{upper_cutoff}.
#'
#' When \code{fill_cutoff} is set (e.g. \code{"< 18"}), values of
#' \code{fill_by} matching the condition are set to \code{NA} and rendered in
#' grey ("grey80"), with a separate legend entry documenting the cutoff.
#'
#' \strong{Border colour modes:}
#' \itemize{
#'   \item \code{border_color = TRUE} — dot borders and lollipop inner bar
#'   colour track the \code{fill_by} gradient via
#'   \code{scale_color_gradientn()}; lollipop outer shadow is black.
#'   \item \code{border_color = "black"} (default) — constant black borders on
#'   dots and a black outer shadow on lollipop bars.
#'   \item \code{border_color = FALSE} — no dot borders, no lollipop outer
#'   shadow (the inner coloured segment remains).
#' }
#'
#' @section Architecture:
#'
#' \strong{DotPlotAtomic} executes the following steps:
#' \enumerate{
#'   \item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
#'   \code{ggplot2::ggplot} based on
#'   \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{bg_direction normalisation} — \code{match.arg()} resolves
#'   \code{"v"}/\code{"h"} abbreviations to \code{"vertical"}/\code{"horizontal"}.
#'   \item \strong{Axis type detection} — determines whether \code{x} (resp.
#'   \code{y}) is numeric by checking that the column is a single name and the
#'   data column is neither character nor factor.
#'   \item \strong{Column resolution} — non-numeric x and y are processed via
#'   \code{check_columns()} with \code{force_factor = TRUE, allow_multi = TRUE,
#'   concat_multi = TRUE} using \code{x_sep}/\code{y_sep}. \code{fill_by} and
#'   \code{facet_by} are also validated.
#'   \item \strong{NA / empty handling} — \code{process_keep_na_empty()} filters
#'   data and extracts \code{keep_empty} settings for x, y, and facet_by.
#'   \item \strong{Multi-facet keep_empty guard} — when \code{facet_by} has more
#'   than one column, the \code{keep_empty} values must be identical for all
#'   facet columns (consistent drop behaviour).
#'   \item \strong{fill_cutoff guard} — errors if \code{fill_cutoff} is set but
#'   \code{fill_by} is \code{NULL}.
#'   \item \strong{Size-by resolution} — when \code{size_by = NULL}:
#'     \itemize{
#'       \item Groups data by the unique combination of x, y, and facet_by
#'       columns and counts rows per combination into \code{.size}.
#'       \item If \code{fill_by} is present, \code{summarise()} also takes the
#'       first value of \code{fill_by} per group (with a warning that only the
#'       first value is used).
#'       \item Factor levels of x, y, and facet_by are preserved post-summary.
#'       \item Sets \code{size_by <- ".size"}.
#'     }
#'   \item \strong{fill_cutoff parsing} — if \code{fill_cutoff} is set:
#'     \itemize{
#'       \item Numeric shorthand (e.g. \code{18}) is converted to \code{"< 18"}.
#'       \item The string is parsed with regex \code{^(<=?|>=?)\\\\s*(-?[0-9.]+)$}
#'       to extract operator and numeric threshold.
#'       \item A \code{switch()} on the operator NAs-out matching values in
#'       \code{fill_by} (e.g. \code{"<"} sets values \emph{below} the threshold
#'       to \code{NA}).
#'       \item A label \code{"<fill_by> <fill_cutoff>"} is generated for the
#'       legend.
#'     }
#'   \item \strong{Default fill_by} — when \code{fill_by = NULL}, a synthetic
#'   \code{.fill_by} column (constant \code{1}) is created and the fill legend
#'   is suppressed.
#'   \item \strong{Continuous colour scale preparation} — when \code{fill_by}
#'   is numeric, \code{\link{prepare_continuous_color_scale}()} computes
#'   \code{feat_colors_value} (the range endpoints after optional quantile
#'   trimming or cutoff clamping).
#'   \item \strong{Base ggplot} — initialises \code{ggplot(data, aes(x, y))}.
#'   \item \strong{Background layer} — if \code{add_bg = TRUE}:
#'     \itemize{
#'       \item Vertical stripes (\code{bg_direction = "vertical"}) require a
#'       non-numeric x-axis; horizontal stripes require a non-numeric y-axis.
#'       \item Calls \code{\link{bg_layer}()} with the relevant discrete column
#'       and stripe styling.
#'     }
#'   \item \strong{Discrete axis scales} — \code{scale_x_discrete()} /
#'   \code{scale_y_discrete()} with \code{drop = !isTRUE(keep_empty_*)} for
#'   non-numeric axes.
#'   \item \strong{Lollipop branch} (\code{lollipop = TRUE}):
#'     \itemize{
#'       \item Sets x-axis expansion to \code{c(0, 0, 0.05, 0)} (bars start at
#'       x = 0 with a 5\% right pad).
#'       \item \strong{Outer shadow} — \code{geom_segment()} from x = 0 to the
#'       data value, coloured black (or \code{border_color} when a string),
#'       with linewidth \code{border_size * 4}. Skipped when
#'       \code{border_color = FALSE}.
#'       \item \strong{Inner coloured bar} — \code{geom_segment()} mapped to
#'       the \code{fill_by} gradient via \code{scale_color_gradientn()}, with
#'       linewidth \code{border_size * 2}.
#'       \item \code{ggnewscale::new_scale_color()} resets the colour scale for
#'       the subsequent point layer.
#'     }
#'   \item \strong{Point layer} — \code{geom_point(shape = 21)} (filled circle
#'   with border) with four dispatch paths:
#'     \itemize{
#'       \item \emph{Numeric size + gradient border}: \code{aes(fill, color)}
#'       both mapped to \code{fill_by}; \code{size = size_by} constant.
#'       \item \emph{Numeric size + constant/no border}: \code{aes(fill,
#'       color = "")} with a constant colour string.
#'       \item \emph{Column size + gradient border}: \code{aes(size, fill,
#'       color)} with \code{scale_size(range = c(size_min, size_max))} and a
#'       size legend (title = \code{size_name}, order = 1).
#'       \item \emph{Column size + constant/no border}: \code{aes(size, fill,
#'       color = "")} with the same size scale.
#'     }
#'   \item \strong{Fill scale} — \code{scale_fill_gradientn()} with
#'   \code{palette_this()}, \code{feat_colors_value} for rescaled colour
#'   mapping, \code{na.value = "grey80"}, and a colour-bar legend (title =
#'   \code{fill_name}, order = 2) or \code{guide_none()} when no
#'   \code{fill_by} was provided.
#'   \item \strong{Labels} — \code{labs(title, subtitle, x, y)} with fallback
#'   to column names via \code{\%||\%}.
#'   \item \strong{Theme} — \code{do_call(theme, theme_args)} plus
#'   \code{panel.grid.major} (grey80 dashed) and rotated x-axis text via
#'   \code{calc_just(x_text_angle)}.
#'   \item \strong{Colour (border) scale}:
#'     \itemize{
#'       \item \code{border_color = TRUE} — \code{scale_color_gradientn()}
#'       following the \code{fill_by} gradient (with border_alpha), guide
#'       suppressed (the fill colour-bar serves both).
#'       \item \code{border_color = FALSE} — \code{scale_color_manual()} with
#'       \code{"transparent"}, guide suppressed.
#'       \item Constant string — \code{scale_color_manual()} with the
#'       alpha-adjusted colour, guide suppressed.
#'     }
#'   \item \strong{fill_cutoff legend} — when \code{fill_cutoff} is active and
#'   there are NA values in \code{fill_by}, a \code{guide_legend()} (order = 3)
#'   is added showing the cutoff label with a grey fill and the border colour.
#'   \item \strong{Flip} — optional \code{coord_flip()} swaps x and y axes.
#'   \item \strong{Dimension calculation} — \code{calculate_plot_dimensions()}
#'   with \code{base_height = 4.5}, \code{aspect.ratio = NULL}, and per-axis
#'   scale factors (0.9 for the categorical axis driving width, 0.6 for the
#'   categorical axis driving height). Legend width, y-axis label length, and
#'   minimum dimensions (width \eqn{\ge} 5, height \eqn{\ge} 4) are factored in. A fallback
#'   manual calculation is used when \code{calculate_plot_dimensions()} returns
#'   \code{NULL}.
#'   \item \strong{Faceting} — \code{facet_plot()} applies
#'   \code{facet_grid}/\code{facet_wrap} with \code{drop = !isTRUE(keep_empty_facet)}.
#' }
#'
#' @inheritParams common_args
#' @param x A character vector specifying the column(s) to use for the x-axis.
#'   Can be numeric (for scatter/lollipop mode) or factor/character (for dot
#'   matrix mode). When multiple columns are provided for a non-numeric axis,
#'   they are concatenated with \code{x_sep} as the separator.
#' @param y A character vector specifying the column(s) to use for the y-axis.
#'   Can be numeric (for scatter mode) or factor/character (for dot matrix or
#'   lollipop mode). When multiple columns are provided for a non-numeric axis,
#'   they are concatenated with \code{y_sep} as the separator.
#' @param x_sep A character string used to join multiple \code{x} column values
#'   into a single factor level. Only used when x is non-numeric and multiple
#'   columns are provided. Default: \code{"_"}.
#' @param y_sep A character string used to join multiple \code{y} column values
#'   into a single factor level. Only used when y is non-numeric and multiple
#'   columns are provided. Default: \code{"_"}.
#' @param size_by A character string naming a numeric column whose values
#'   control dot size. When \code{NULL} (the default), the per-combination
#'   observation count is computed automatically (via \code{dplyr::summarise(n =
#'   n())}) and used as the size variable. If \code{fill_by} is also present,
#'   the first value of \code{fill_by} per combination is retained with a
#'   warning. A single numeric value is also accepted and sets a constant dot
#'   size (used by \code{ScatterPlot}).
#' @param size_min A numeric value for the smallest dot size in the
#'   \code{scale_size(range = c(size_min, size_max))} range.
#'   Default: \code{1}.
#' @param size_max A numeric value for the largest dot size in the
#'   \code{scale_size(range = c(size_min, size_max))} range.
#'   Default: \code{10}.
#' @param fill_by A character string naming a numeric column whose values
#'   control the fill colour of the dots (and lollipop inner bars). A
#'   continuous gradient from \code{palette} is applied via
#'   \code{scale_fill_gradientn()}. When \code{NULL} (the default), all dots
#'   are filled with a single constant colour from the middle of the palette.
#' @param fill_cutoff A string expression specifying which values of
#'   \code{fill_by} to grey out. Format: an operator followed by a number,
#'   e.g. \code{"< 18"}, \code{"<= 18"}, \code{"> 18"}, or \code{">= 18"}.
#'   Values matching the condition are set to \code{NA} and rendered in grey
#'   (\code{"grey80"}), while the rest are coloured by the fill gradient. The
#'   operator determines which side of the threshold is greyed out,
#'   independent of \code{palreverse}. A numeric value is also accepted as
#'   shorthand for \code{"<"} (e.g. \code{18} is equivalent to
#'   \code{"< 18"}). Requires \code{fill_by} to be set.
#' @param size_name A character string for the size legend title. When
#'   \code{NULL} (the default), the \code{size_by} column name is used.
#' @param fill_name A character string for the fill colour-bar legend title.
#'   When \code{NULL} (the default), the \code{fill_by} column name is used.
#' @param fill_cutoff_name A character string for the fill cutoff legend title
#'   (shown when \code{fill_cutoff} is active). Defaults to
#'   \code{"<fill_by> <fill_cutoff>"}, e.g. \code{"mpg < 18"}.
#' @param flip A logical value. If \code{TRUE}, the x and y axes are swapped via
#'   \code{coord_flip()}. Dimension calculation accounts for the flip.
#'   Default: \code{FALSE}.
#' @param lollipop A logical value. If \code{TRUE}, renders a lollipop plot
#'   with bars extending from x = 0 to each data point, capped by filled dots.
#'   Requires \code{x} to be numeric and \code{y} to be factor/character.
#'   Default: \code{FALSE}.
#' @param border_color Controls the dot border colour and lollipop outer-shadow
#'   appearance:
#'   \itemize{
#'     \item \code{TRUE} — dot borders and lollipop inner bars follow the
#'     \code{fill_by} gradient via \code{scale_color_gradientn()}; lollipop
#'     outer shadow is black.
#'     \item \code{"black"} (default) — constant black borders on dots and
#'     black outer shadow on lollipop bars.
#'     \item A colour string (e.g. \code{"red"}, \code{"#FF0000"}) — constant
#'     colour for both dot borders and lollipop outer shadows.
#'     \item \code{FALSE} — no dot borders and no lollipop outer shadow (the
#'     inner coloured bars remain visible in lollipop mode).
#'   }
#' @param border_size A numeric value for the stroke width of dot borders and
#'   the base linewidth of lollipop bars. In lollipop mode, the outer shadow
#'   uses \code{border_size * 4} and the inner bar uses \code{border_size * 2}.
#'   Default: \code{0.5}.
#' @param border_alpha A numeric value in \code{[0, 1]} controlling the
#'   transparency of dot borders and lollipop bar segments.
#'   Default: \code{1}.
#' @param add_bg A logical value. If \code{TRUE}, alternating background
#'   stripes are drawn behind the points via \code{\link{bg_layer}()}. The
#'   striped axis is determined by \code{bg_direction}. Requires the striped
#'   axis to be non-numeric. Default: \code{FALSE}.
#' @param bg_palette A character string specifying the palette for the
#'   background stripe colours. Passed to \code{\link{bg_layer}()}.
#'   Default: \code{"stripe"}.
#' @param bg_palcolor A character vector of colours for the background stripes.
#'   Passed to \code{\link{bg_layer}()}. When \code{NULL} (default), colours
#'   are derived from \code{bg_palette}.
#' @param bg_alpha A numeric value in \code{[0, 1]} for the transparency of
#'   the background stripes. Default: \code{0.2}.
#' @param bg_direction A character string specifying which axis receives the
#'   alternating background stripes. \code{"vertical"} (default) stripes by x
#'   levels; \code{"horizontal"} stripes by y levels. Abbreviations \code{"v"}
#'   and \code{"h"} are also accepted.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'   attributes.
#' @keywords internal
#' @importFrom dplyr %>% group_by summarise n first
#' @importFrom ggplot2 geom_point scale_y_discrete scale_size_area scale_fill_gradientn scale_color_gradientn labs
#' @importFrom ggplot2 coord_flip guide_colorbar guide_legend guides guide_none scale_size geom_segment
#' @importFrom ggnewscale new_scale_color
#' @importFrom scales alpha rescale
#' @importFrom ggplot2 waiver
DotPlotAtomic <- function(
    data,
    x,
    y,
    x_sep = "_",
    y_sep = "_",
    flip = FALSE,
    lollipop = FALSE,
    size_by = NULL,
    fill_by = NULL,
    fill_cutoff = NULL,
    palreverse = FALSE,
    size_name = NULL,
    fill_name = NULL,
    fill_cutoff_name = NULL,
    size_min = 1,
    size_max = 10,
    theme = "theme_this",
    theme_args = list(),
    palette = "Spectral",
    palcolor = NULL,
    alpha = 1,
    border_color = "black",
    border_size = 0.5,
    border_alpha = 1,
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    x_text_angle = 0,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    bg_direction = c("vertical", "horizontal", "v", "h"),
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_na = FALSE,
    keep_empty = FALSE,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    bg_direction <- match.arg(bg_direction)
    if (bg_direction %in% c("h", "horizontal")) {
        bg_direction <- "horizontal"
    } else {
        bg_direction <- "vertical"
    }
    x_is_numeric <- length(x) == 1 &&
        !is.character(data[[x]]) &&
        !is.factor(data[[x]])
    y_is_numeric <- length(y) == 1 &&
        !is.character(data[[y]]) &&
        !is.factor(data[[y]])
    if (!x_is_numeric) {
        x <- check_columns(
            data,
            x,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = x_sep
        )
    }
    if (!y_is_numeric) {
        y <- check_columns(
            data,
            y,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = y_sep
        )
    }
    fill_by <- check_columns(data, fill_by)
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )

    data <- process_keep_na_empty(data, keep_na, keep_empty)
    keep_empty_x <- keep_empty[[x]]
    keep_empty_y <- keep_empty[[y]]
    keep_empty_facet <- if (!is.null(facet_by)) {
        keep_empty[[facet_by[1]]]
    } else {
        NULL
    }
    if (length(facet_by) > 1) {
        stopifnot(
            "[DotPlot/LillipopPlot] `keep_empty` for `facet_by` variables must be identical." = identical(
                keep_empty_facet,
                keep_empty[[facet_by[2]]]
            )
        )
    }

    if (!is.null(fill_cutoff) && is.null(fill_by)) {
        stop(
            "[DotPlot/LollipopPlot]'fill_by' must be provided when 'fill_cutoff' is specified."
        )
    }

    if (!is.numeric(size_by)) {
        size_by <- check_columns(data, size_by)
    }
    if (is.null(size_by)) {
        if (is.null(fill_by)) {
            data <- data %>%
                group_by(!!!syms(unique(c(x, y, facet_by)))) %>%
                summarise(.size = n(), .groups = "drop")
        } else {
            warning(
                "[DotPlot] Using the first value of fill_by.",
                immediate. = TRUE
            )
            data <- data %>%
                group_by(!!!syms(unique(c(x, y, facet_by)))) %>%
                summarise(
                    !!sym(fill_by) := first(!!sym(fill_by)),
                    .size = n(),
                    .groups = "drop"
                )
        }
        # keep the levels of x, y, and facet_by
        for (col in unique(c(x, y, facet_by))) {
            if (is.factor(data[[col]])) {
                data[[col]] <- factor(data[[col]], levels = levels(data[[col]]))
            }
        }
        size_by <- ".size"
    }

    if (!is.null(fill_by) && !is.null(fill_cutoff)) {
        # Numeric shorthand: 18 → "< 18"
        if (is.numeric(fill_cutoff)) {
            fill_cutoff <- paste("<", fill_cutoff)
        }
        if (
            !is.character(fill_cutoff) ||
                length(fill_cutoff) != 1 ||
                is.na(fill_cutoff)
        ) {
            stop(
                "[DotPlot/LollipopPlot] 'fill_cutoff' must be a string like '< 18' or '> 18'."
            )
        }
        parsed <- regmatches(
            fill_cutoff,
            regexec("^(<=?|>=?)\\s*(-?[0-9.]+)$", fill_cutoff)
        )[[1]]
        if (length(parsed) != 3) {
            stop(
                "[DotPlot/LollipopPlot] 'fill_cutoff' must be a string like '< 18', '<= 18', '> 18', or '>= 18'."
            )
        }
        cutoff_op <- parsed[2]
        cutoff_val <- as.numeric(parsed[3])
        if (is.na(cutoff_val)) {
            stop(
                "[DotPlot/LollipopPlot] Invalid numeric value in 'fill_cutoff': ",
                fill_cutoff
            )
        }
        fill_cutoff_label <- paste0(fill_by, " ", fill_cutoff)
        switch(
            cutoff_op,
            "<" = {
                data[[fill_by]][data[[fill_by]] < cutoff_val] <- NA
            },
            "<=" = {
                data[[fill_by]][data[[fill_by]] <= cutoff_val] <- NA
            },
            ">" = {
                data[[fill_by]][data[[fill_by]] > cutoff_val] <- NA
            },
            ">=" = {
                data[[fill_by]][data[[fill_by]] >= cutoff_val] <- NA
            }
        )
    }
    if (is.null(fill_by)) {
        data$.fill_by <- 1
        fill_by <- ".fill_by"
        fill_legend <- FALSE
    } else {
        fill_legend <- TRUE
    }

    feat_colors_value <- NULL
    if (!is.null(fill_by) && is.numeric(data[[fill_by]])) {
        result <- prepare_continuous_color_scale(
            data,
            fill_by,
            lower_quantile = lower_quantile,
            upper_quantile = upper_quantile,
            lower_cutoff = lower_cutoff,
            upper_cutoff = upper_cutoff
        )
        data <- result$data
        feat_colors_value <- result$feat_colors_value
    }

    just <- calc_just(x_text_angle)
    p <- ggplot(data, aes(x = !!sym(x), y = !!sym(y)))
    if (add_bg) {
        if (bg_direction == "vertical") {
            if (x_is_numeric) {
                stop(
                    "Vertical 'bg_direction' is not supported when 'x' is numeric."
                )
            }
            p <- p +
                bg_layer(
                    data,
                    x,
                    isTRUE(keep_empty_x),
                    bg_palette,
                    bg_palcolor,
                    bg_alpha,
                    facet_by,
                    bg_direction
                )
        } else {
            if (y_is_numeric) {
                stop(
                    "Horizontal 'bg_direction' is not supported when 'y' is numeric."
                )
            }
            p <- p +
                bg_layer(
                    data,
                    y,
                    isTRUE(keep_empty_y),
                    bg_palette,
                    bg_palcolor,
                    bg_alpha,
                    facet_by,
                    bg_direction
                )
        }
    }
    if (!x_is_numeric) {
        p <- p + scale_x_discrete(drop = !isTRUE(keep_empty_x))
    }
    if (!y_is_numeric) {
        p <- p + scale_y_discrete(drop = !isTRUE(keep_empty_y))
    }

    if (isTRUE(lollipop)) {
        lw_outer <- border_size * 4
        lw_inner <- border_size * 2
        p <- p +
            scale_x_continuous(expand = c(0, 0, 0.05, 0))
        # outer shadow / border (hidden when border_color is FALSE)
        if (!isFALSE(border_color)) {
            outer_color <- if (isTRUE(border_color)) {
                scales::alpha("black", border_alpha)
            } else {
                scales::alpha(border_color, border_alpha)
            }
            p <- p +
                geom_segment(
                    aes(x = 0, xend = !!sym(x), yend = !!sym(y)),
                    color = outer_color,
                    linewidth = lw_outer
                )
        }
        # inner colored bar (always drawn)
        p <- p +
            geom_segment(
                aes(
                    x = 0,
                    xend = !!sym(x),
                    yend = !!sym(y),
                    color = !!sym(fill_by)
                ),
                linewidth = lw_inner
            ) +
            scale_color_gradientn(
                n.breaks = 5,
                colors = palette_this(
                    palette = palette,
                    palcolor = palcolor,
                    reverse = palreverse,
                    alpha = border_alpha
                ),
                values = if (!is.null(feat_colors_value)) {
                    scales::rescale(feat_colors_value)
                } else {
                    waiver()
                },
                limits = if (!is.null(feat_colors_value)) {
                    range(feat_colors_value)
                } else {
                    NULL
                },
                na.value = "grey80",
                guide = "none"
            ) +
            new_scale_color()
    }
    if (is.numeric(size_by)) {
        if (isTRUE(border_color)) {
            p <- p +
                geom_point(
                    aes(fill = !!sym(fill_by), color = !!sym(fill_by)),
                    size = size_by,
                    shape = 21,
                    stroke = border_size,
                    alpha = alpha
                )
        } else {
            p <- p +
                geom_point(
                    aes(fill = !!sym(fill_by), color = ""),
                    size = size_by,
                    shape = 21,
                    stroke = border_size,
                    alpha = alpha
                )
        }
    } else {
        if (isTRUE(border_color)) {
            p <- p +
                geom_point(
                    aes(
                        size = !!sym(size_by),
                        fill = !!sym(fill_by),
                        color = !!sym(fill_by)
                    ),
                    shape = 21,
                    stroke = border_size,
                    alpha = alpha
                ) +
                # scale_size_area(max_size = 6, n.breaks = 4) +
                scale_size(range = c(size_min, size_max)) +
                guides(
                    size = guide_legend(
                        title = size_name %||% size_by,
                        override.aes = list(fill = "transparent", shape = 21),
                        order = 1
                    )
                )
        } else {
            p <- p +
                geom_point(
                    aes(
                        size = !!sym(size_by),
                        fill = !!sym(fill_by),
                        color = ""
                    ),
                    shape = 21,
                    stroke = border_size,
                    alpha = alpha
                ) +
                # scale_size_area(max_size = 6, n.breaks = 4) +
                scale_size(range = c(size_min, size_max)) +
                guides(
                    size = guide_legend(
                        title = size_name %||% size_by,
                        override.aes = list(fill = "transparent", shape = 21),
                        order = 1
                    )
                )
        }
    }

    p <- p +
        scale_fill_gradientn(
            n.breaks = 5,
            colors = palette_this(
                palette = palette,
                palcolor = palcolor,
                reverse = palreverse
            ),
            values = if (!is.null(feat_colors_value)) {
                scales::rescale(feat_colors_value)
            } else {
                waiver()
            },
            limits = if (!is.null(feat_colors_value)) {
                range(feat_colors_value)
            } else {
                NULL
            },
            na.value = "grey80",
            guide = if (isTRUE(fill_legend)) {
                guide_colorbar(
                    title = fill_name %||% fill_by,
                    frame.colour = "black",
                    ticks.colour = "black",
                    title.hjust = 0,
                    order = 2
                )
            } else {
                guide_none()
            }
        ) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% x,
            y = ylab %||% y
        ) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major = element_line(colour = "grey80", linetype = 2),
            axis.text.x = element_text(
                angle = x_text_angle,
                hjust = just$h,
                vjust = just$v
            )
        )

    if (isTRUE(border_color)) {
        p <- p +
            scale_color_gradientn(
                n.breaks = 5,
                colors = palette_this(
                    palette = palette,
                    palcolor = palcolor,
                    reverse = palreverse,
                    alpha = border_alpha
                ),
                values = if (!is.null(feat_colors_value)) {
                    scales::rescale(feat_colors_value)
                } else {
                    waiver()
                },
                limits = if (!is.null(feat_colors_value)) {
                    range(feat_colors_value)
                } else {
                    NULL
                },
                na.value = "grey80",
                guide = "none"
            )
    } else {
        border_color_value <- if (isFALSE(border_color)) {
            "transparent"
        } else {
            scales::alpha(border_color, border_alpha)
        }
        p <- p +
            scale_color_manual(
                values = border_color_value,
                na.value = border_color_value,
                guide = "none"
            )
    }
    if (!is.null(fill_by) && !is.null(fill_cutoff) && anyNA(data[[fill_by]])) {
        fill_cutoff_colour <- if (isTRUE(border_color)) {
            scales::alpha("black", border_alpha)
        } else if (isFALSE(border_color)) {
            "transparent"
        } else {
            scales::alpha(border_color, border_alpha)
        }
        p <- p +
            guides(
                color = guide_legend(
                    title = fill_cutoff_name %||% fill_cutoff_label,
                    override.aes = list(
                        colour = fill_cutoff_colour,
                        fill = "grey80",
                        size = 3
                    ),
                    order = 3
                )
            )
    }

    if (isTRUE(flip)) {
        p <- p + coord_flip()
    }

    if (x_is_numeric) {
        nx <- 5
    } else if (isTRUE(keep_empty_x)) {
        nx <- nlevels(data[[x]])
    } else {
        nx <- nlevels(droplevels(data[[x]]))
    }
    if (y_is_numeric) {
        ny <- 5
    } else if (isTRUE(keep_empty_y)) {
        ny <- nlevels(data[[y]])
    } else {
        ny <- nlevels(droplevels(data[[y]]))
    }

    y_label_len <- if (!y_is_numeric) {
        max(sapply(strsplit(levels(data[[y]]), "\n"), function(x) {
            max(nchar(x))
        }))
    } else {
        0
    }
    legend_nchar <- max(
        nchar(c(
            fill_name %||% fill_by,
            if (!is.numeric(size_by)) size_name %||% size_by else character(0)
        )),
        na.rm = TRUE,
        5
    )

    # Use aspect.ratio = NULL so that each axis is sized independently from its content:
    #   - visual x-axis (groups when not flipped, terms when flipped) drives width via n_x
    #   - visual y-axis (terms when not flipped, groups when flipped) drives height via n_y
    # Scale factors: 0.9 per group (fewer categories, need more space),
    #                0.6 per term (often many, needs a compact but readable size).
    # min_width=5 ensures the plot is wide enough for a horizontal legend colorbar.
    # min_height=4 prevents the flipped case from being too short for the group rows.
    dims <- calculate_plot_dimensions(
        base_height = 4.5,
        aspect.ratio = NULL,
        n_x = if (isTRUE(flip)) ny else nx,
        x_scale_factor = if (isTRUE(flip)) 0.6 else 0.9,
        n_y = if (isTRUE(flip)) nx else ny,
        y_scale_factor = if (isTRUE(flip)) 0.9 else 0.6,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = 2,
        legend_nchar = legend_nchar,
        min_width = 5,
        min_height = 4
    )

    if (is.null(dims)) {
        if (ny / nx > 10) {
            height <- ny * 0.2
            width <- nx * 2
        } else if (ny / nx < 0.1) {
            height <- ny * 2
            width <- nx * 0.4
        } else {
            height <- ny * 0.5
            width <- nx * 0.8
        }
        width <- width + y_label_len * 0.1
        width <- max(width, 3)
        height <- max(height, 3)
        ar <- if (isTRUE(flip)) 1 / aspect.ratio else aspect.ratio
        coupled_height <- width * ar
        if (abs(coupled_height - height) / max(height, 1) < 0.5) {
            height <- coupled_height
            height <- max(height, 3)
        }
        if (!identical(legend.position, "none")) {
            if (legend.position %in% c("right", "left")) {
                width <- width + 1
            } else if (legend.direction == "horizontal") {
                height <- height + 1
            } else {
                width <- width + 2
            }
        }
        height <- max(height, 3)
        if (isTRUE(flip)) {
            attr(p, "height") <- width
            attr(p, "width") <- height
        } else {
            attr(p, "height") <- height
            attr(p, "width") <- width
        }
    } else {
        h <- dims$height
        w <- dims$width + y_label_len * 0.1
        # For dot-plot matrices rendered with a square panel (default aspect.ratio = 1),
        # prevent extreme H/W ratios that leave large blank areas inside the figure.
        # Neither dimension should be less than 50 % of the other.
        h <- max(h, w * 0.5)
        w <- max(w, h * 0.5)
        attr(p, "height") <- min(h, 12)
        attr(p, "width") <- min(w, 12)
    }

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction,
        drop = !isTRUE(keep_empty_facet)
    )
}

#' Dot Plot, Scatter Plot, and Lollipop Plot
#'
#' @rdname dotplot
#' @description
#' \code{DotPlot()} renders a matrix of filled circles (dot plot) where dot
#' size encodes one numeric variable and fill colour encodes another. Either
#' axis can be numeric or factor, enabling four layout combinations:
#'
#' \itemize{
#'   \item \strong{Both axes factor} — a classic dot matrix (e.g. genes × cell
#'   types), where each cell is a dot whose size reflects expression magnitude
#'   and whose colour reflects a summary statistic.
#'   \item \strong{Both axes numeric} — a scatter plot, with dots positioned by
#'   x/y coordinates, sized by a third variable, and coloured by a fourth.
#'   \item \strong{One numeric, one factor} — a strip plot or (with
#'   \code{lollipop = TRUE}) a lollipop chart.
#' }
#'
#' \code{LollipopPlot()} is a convenience wrapper that sets
#' \code{lollipop = TRUE}, producing horizontal bars from the y-axis to each
#' data point, capped by filled dots. It expects a numeric \code{x} and a
#' factor/character \code{y}.
#'
#' Key features:
#' \itemize{
#'   \item \strong{Auto-count:} when \code{size_by = NULL}, the per-combination
#'   observation count is computed automatically.
#'   \item \strong{fill_cutoff:} values in \code{fill_by} matching a threshold
#'   expression (e.g. \code{"< 18"}) are greyed out with a dedicated legend
#'   entry.
#'   \item \strong{Background stripes:} \code{add_bg = TRUE} draws alternating
#'   background bands along the discrete axis for visual grouping.
#'   \item \strong{Border modes:} \code{border_color} can track the fill
#'   gradient (\code{TRUE}), use a constant colour (\code{"black"}), or be
#'   suppressed (\code{FALSE}).
#'   \item \strong{Colour scale trimming:} \code{lower_quantile} /
#'   \code{upper_quantile} (or explicit \code{lower_cutoff} /
#'   \code{upper_cutoff}) trim the continuous fill scale extremes.
#' }
#'
#' @section split_by Workflow (DotPlot):
#'
#' When \code{split_by} is provided, the following pipeline executes:
#' \enumerate{
#'   \item \strong{Column validation} — \code{check_columns()} resolves
#'   \code{split_by} (force_factor, allow_multi, concat_multi).
#'   \item \strong{NA / empty pre-processing} — \code{process_keep_na_empty()}
#'   handles \code{keep_na} / \code{keep_empty} for the split column before
#'   splitting, then removes the split column from the per-split
#'   \code{keep_na}/\code{keep_empty} lists.
#'   \item \strong{Data splitting} — splits \code{data} by \code{split_by}
#'   levels (preserving factor level order).
#'   \item \strong{Per-split palette / colour} — \code{check_palette()} and
#'   \code{check_palcolor()} resolve per-split palette and colour overrides.
#'   \item \strong{Per-split legend} — \code{check_legend()} resolves
#'   \code{legend.position} and \code{legend.direction} per split.
#'   \item \strong{Per-split title} — when \code{title} is a function, it
#'   receives the default title (the split level name) and can return a
#'   custom string; otherwise \code{title \%||\% split_level} is used.
#'   \item \strong{Dispatch} — each split subset is passed to
#'   \code{\link{DotPlotAtomic}} (with \code{lollipop = FALSE}).
#'   \item \strong{Combination} — \code{\link{combine_plots}()} assembles the
#'   list of plots via \code{patchwork::wrap_plots}, honouring
#'   \code{nrow}/\code{ncol}/\code{byrow}/\code{design}.
#' }
#'
#' @section split_by Workflow (LollipopPlot):
#'
#' Same pipeline as \code{DotPlot} above, but dispatches to
#' \code{\link{DotPlotAtomic}} with \code{lollipop = TRUE}.
#'
#' @inheritParams DotPlotAtomic
#' @inheritParams common_args
#' @param split_by The column(s) to split data by and generate separate plots
#'   for each level. The split column is processed for \code{keep_na} /
#'   \code{keep_empty} before splitting.
#' @param split_by_sep A character string used to concatenate multiple
#'   \code{split_by} column values. Default: \code{"_"}.
#' @param seed The random seed for reproducibility. Passed to
#'   \code{validate_common_args()}. Default: \code{8525}.
#' @param combine A logical value. If \code{TRUE} (the default), the list of
#'   per-split plots is combined into a single \code{patchwork} object. If
#'   \code{FALSE}, returns the raw list.
#' @param nrow,ncol,byrow Integers controlling the layout of combined plots via
#'   \code{patchwork::wrap_plots()}. \code{byrow = TRUE} (default) fills the
#'   layout row-wise.
#' @param axes,axis_titles Strings controlling how axes and axis titles are
#'   handled across combined plots. Passed to \code{\link{combine_plots}()}.
#'   See \code{?patchwork::wrap_plots} for options (\code{"keep"},
#'   \code{"collect"}, \code{"collect_x"}, \code{"collect_y"}).
#' @param guides A string controlling guide collection across combined plots.
#'   Passed to \code{\link{combine_plots}()}.
#' @param design A custom layout specification for combined plots. Passed to
#'   \code{\link{combine_plots}()}. When specified, \code{nrow}, \code{ncol},
#'   and \code{byrow} are ignored.
#' @return A \code{ggplot} object (single plot), a \code{patchwork} object
#'   (when \code{combine = TRUE} with \code{split_by}), or a list of
#'   \code{ggplot} objects (when \code{combine = FALSE}).
#' @export
#' @examples
#' \donttest{
#' mtcars <- datasets::mtcars
#' mtcars$carb <- factor(mtcars$carb)
#' mtcars$gear <- factor(mtcars$gear)
#'
#' # --- Basic dot plot (factor × factor, size + fill) ---
#' DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
#'         fill_by = "mpg", fill_cutoff = "< 18")
#' DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
#'         fill_by = "mpg", fill_cutoff = "> 18")
#'
#' # --- Background stripes ---
#' DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
#'         fill_by = "mpg", fill_cutoff = "< 18", add_bg = TRUE)
#' DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
#'         fill_by = "mpg", fill_cutoff = "< 18", add_bg = TRUE,
#'         bg_direction = "h")
#'
#' # --- Faceting ---
#' DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
#'         fill_by = "mpg", fill_cutoff = "< 18", facet_by = "cyl")
#' DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
#'         fill_by = "mpg", fill_cutoff = "< 18", facet_by = "cyl",
#'         facet_scales = "free_x")
#'
#' # --- split_by ---
#' DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
#'         fill_by = "mpg", fill_cutoff = "< 18", split_by = "cyl")
#' DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
#'         fill_by = "mpg", fill_cutoff = "< 18", split_by = "cyl",
#'         palette = list("4" = "Set1", "6" = "Paired", "8" = "Reds"))
#'
#' # --- Scatter plot (both axes numeric) ---
#' DotPlot(mtcars, x = "qsec", y = "drat", size_by = "wt",
#'         fill_by = "mpg", fill_cutoff = "< 18",
#'         fill_cutoff_name = "Small mpgs")
#'
#' # --- keep_na and keep_empty ---
#' mtcars$carb[mtcars$carb == "1"] <- NA
#' mtcars$gear[mtcars$gear == "3"] <- NA
#' DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
#'         fill_by = "mpg", fill_cutoff = "< 18",
#'         keep_na = TRUE, keep_empty = TRUE)
#'
#' # --- Border customization ---
#' DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
#'         fill_by = "mpg", border_color = "red", border_size = 2)
#' DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
#'         fill_by = "mpg", border_color = TRUE, border_size = 1.5,
#'         border_alpha = 0.5)
#' DotPlot(mtcars, x = "carb", y = "gear",
#'         fill_by = "mpg", border_color = FALSE)
#'
#' # --- Colour scale trimming ---
#' DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
#'         fill_by = "mpg", lower_quantile = 0.05, upper_quantile = 0.95)
#' DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
#'         fill_by = "mpg", lower_cutoff = 15, upper_cutoff = 25)
#' }
DotPlot <- function(
    data,
    x,
    y,
    x_sep = "_",
    y_sep = "_",
    flip = FALSE,
    split_by = NULL,
    split_by_sep = "_",
    size_name = NULL,
    fill_name = NULL,
    fill_cutoff_name = NULL,
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    bg_direction = c("vertical", "horizontal", "v", "h"),
    size_by = NULL,
    fill_by = NULL,
    fill_cutoff = NULL,
    palreverse = FALSE,
    size_min = 1,
    size_max = 10,
    theme = "theme_this",
    theme_args = list(),
    palette = "Spectral",
    palcolor = NULL,
    alpha = 1,
    border_color = "black",
    border_size = 0.5,
    border_alpha = 1,
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    x_text_angle = 0,
    seed = 8525,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_na = FALSE,
    keep_empty = FALSE,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    keep_na <- check_keep_na(keep_na, c(x, y, split_by, fill_by, facet_by))
    keep_empty <- check_keep_empty(
        keep_empty,
        c(x, y, split_by, fill_by, facet_by)
    )
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            DotPlotAtomic(
                datas[[nm]],
                x = x,
                y = y,
                x_sep = x_sep,
                y_sep = y_sep,
                flip = flip,
                bg_direction = bg_direction,
                size_min = size_min,
                size_max = size_max,
                lollipop = FALSE,
                size_by = size_by,
                fill_by = fill_by,
                fill_cutoff = fill_cutoff,
                palreverse = palreverse,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                alpha = alpha,
                border_color = border_color,
                border_size = border_size,
                border_alpha = border_alpha,
                lower_quantile = lower_quantile,
                upper_quantile = upper_quantile,
                lower_cutoff = lower_cutoff,
                upper_cutoff = upper_cutoff,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                x_text_angle = x_text_angle,
                size_name = size_name,
                fill_name = fill_name,
                fill_cutoff_name = fill_cutoff_name,
                add_bg = add_bg,
                bg_palette = bg_palette,
                bg_palcolor = bg_palcolor,
                bg_alpha = bg_alpha,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                keep_na = keep_na,
                keep_empty = keep_empty,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

#' @rdname dotplot
#' @description
#' \code{LollipopPlot()} is a convenience wrapper around \code{DotPlot()}
#' that sets \code{lollipop = TRUE}. It renders a horizontal bar extending
#' from the y-axis (\code{x = 0}) to each data point, capped by a filled dot.
#' The bar has a two-layer construction: an outer shadow (black or custom
#' colour) and an inner coloured segment that follows the \code{fill_by}
#' gradient. Dot size scales by \code{size_by} (or the per-combination
#' observation count when \code{size_by = NULL}).
#'
#' Expects \code{x} to be a numeric column and \code{y} to be a factor or
#' character column.
#'
#' @inheritParams DotPlot
#' @inheritParams common_args
#' @param x A character string naming the column for the x-axis. Must be a
#'   numeric column (bars extend from 0 to the data value).
#' @param y A character string naming the column for the y-axis. Must be a
#'   factor or character column (each level gets a lollipop bar).
#' @return A \code{ggplot} object (single plot), a \code{patchwork} object
#'   (when \code{combine = TRUE} with \code{split_by}), or a list of
#'   \code{ggplot} objects (when \code{combine = FALSE}).
#' @export
#' @examples
#' \donttest{
#' mtcars <- datasets::mtcars
#'
#' # --- Basic lollipop ---
#' LollipopPlot(mtcars, x = "qsec", y = "drat", size_by = "wt",
#'              fill_by = "mpg")
#'
#' # --- Faceting ---
#' LollipopPlot(mtcars, x = "qsec", y = "drat", size_by = "wt",
#'              fill_by = "mpg", fill_cutoff = "< 18", facet_by = "cyl",
#'              facet_scales = "free_y")
#'
#' # --- split_by ---
#' LollipopPlot(mtcars, x = "qsec", y = "drat", size_by = "wt",
#'              split_by = "vs", palette = list("0" = "Reds", "1" = "Blues"))
#'
#' # --- Border customization ---
#' LollipopPlot(mtcars, x = "qsec", y = "drat", size_by = "wt",
#'              fill_by = "mpg", border_color = "red", border_size = 2)
#' LollipopPlot(mtcars, x = "qsec", y = "drat", size_by = "wt",
#'              fill_by = "mpg", border_color = TRUE, border_size = 1.5,
#'              border_alpha = 0.5)
#' LollipopPlot(mtcars, x = "qsec", y = "drat",
#'              fill_by = "mpg", border_color = FALSE)
#' }
LollipopPlot <- function(
    data,
    x,
    y,
    y_sep = NULL,
    flip = FALSE,
    split_by = NULL,
    split_by_sep = "_",
    size_name = NULL,
    fill_name = NULL,
    fill_cutoff_name = NULL,
    size_by = NULL,
    fill_by = NULL,
    fill_cutoff = NULL,
    palreverse = FALSE,
    size_min = 1,
    size_max = 10,
    theme = "theme_this",
    theme_args = list(),
    palette = "Spectral",
    palcolor = NULL,
    alpha = 1,
    border_color = "black",
    border_size = 0.5,
    border_alpha = 1,
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    x_text_angle = 0,
    seed = 8525,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_na = FALSE,
    keep_empty = FALSE,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    keep_na <- check_keep_na(keep_na, c(y, split_by, fill_by, facet_by))
    keep_empty <- check_keep_empty(
        keep_empty,
        c(y, split_by, fill_by, facet_by)
    )
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            DotPlotAtomic(
                datas[[nm]],
                lollipop = TRUE,
                x = x,
                y = y,
                x_sep = NULL,
                y_sep = y_sep,
                flip = flip,
                size_min = size_min,
                size_max = size_max,
                size_by = size_by,
                fill_by = fill_by,
                fill_cutoff = fill_cutoff,
                palreverse = palreverse,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                alpha = alpha,
                border_color = border_color,
                border_size = border_size,
                border_alpha = border_alpha,
                lower_quantile = lower_quantile,
                upper_quantile = upper_quantile,
                lower_cutoff = lower_cutoff,
                upper_cutoff = upper_cutoff,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                x_text_angle = x_text_angle,
                size_name = size_name,
                fill_name = fill_name,
                fill_cutoff_name = fill_cutoff_name,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                keep_na = keep_na,
                keep_empty = keep_empty,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
# DotPlot data: x = genes (factor), y = cell types (factor), fill_by = numeric, size_by = numeric
data <- data.frame(
    gene = factor(rep(paste0("Gene", 1:5), each = 4)),
    celltype = factor(rep(paste0("CellType", 1:4), 5)),
    expression = runif(20, 0, 10),
    pct = runif(20, 0, 1),
    group = factor(rep(c("g1", "g2"), 10))
)

test_that("DotPlot returns a ggplot for basic usage", {
    p <- DotPlot(data, x = "gene", y = "celltype", fill_by = "expression", size_by = "pct")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("DotPlot works without size_by", {
    p <- suppressWarnings(DotPlot(data, x = "gene", y = "celltype", fill_by = "expression"))
    expect_s3_class(p, "ggplot")
})

test_that("DotPlot works with flip = TRUE", {
    p <- suppressWarnings(DotPlot(data, x = "gene", y = "celltype", fill_by = "expression", flip = TRUE))
    expect_s3_class(p, "ggplot")
})

test_that("DotPlot sets title and labels", {
    p <- DotPlot(data, x = "gene", y = "celltype",
                 title = "Dot Title", xlab = "Genes", ylab = "Cell Types")
    expect_equal(p$labels$title, "Dot Title")
    expect_equal(p$labels$x, "Genes")
    expect_equal(p$labels$y, "Cell Types")
})

test_that("DotPlot with split_by returns patchwork", {
    p <- suppressWarnings(DotPlot(data, x = "gene", y = "celltype", fill_by = "expression",
                 split_by = "group", combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("DotPlot with split_by and combine = FALSE returns list", {
    plots <- suppressWarnings(DotPlot(data, x = "gene", y = "celltype", fill_by = "expression",
                     split_by = "group", combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 2)
})

test_that("DotPlot with multiple split_by columns returns a list with one plot per combination", {
    data2 <- data
    data2$split2 <- factor(rep(c("s1", "s2"), each = 10))
    plots <- suppressMessages(suppressWarnings(DotPlot(data2, x = "gene", y = "celltype",
                                                       fill_by = "expression",
                                                       split_by = c("group", "split2"), combine = FALSE)))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("DotPlot with multiple split_by columns returns combined plot", {
    data2 <- data
    data2$split2 <- factor(rep(c("s1", "s2"), each = 10))
    p <- suppressMessages(suppressWarnings(DotPlot(data2, x = "gene", y = "celltype",
                                                   fill_by = "expression",
                                                   split_by = c("group", "split2"), combine = TRUE)))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("LollipopPlot with multiple split_by columns returns a list with one plot per combination", {
    data2 <- data
    data2$split2 <- factor(rep(c("s1", "s2"), each = 10))
    plots <- suppressMessages(LollipopPlot(data2, x = "gene", y = "expression",
                                           split_by = c("group", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("LollipopPlot with multiple split_by columns returns combined plot", {
    data2 <- data
    data2$split2 <- factor(rep(c("s1", "s2"), each = 10))
    p <- suppressMessages(LollipopPlot(data2, x = "gene", y = "expression",
                                       split_by = c("group", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("DotPlot with facet_by works", {
    p <- suppressWarnings(DotPlot(data, x = "gene", y = "celltype", fill_by = "expression",
                 facet_by = "group"))
    expect_s3_class(p, "ggplot")
})

test_that("DotPlot with fill_cutoff works", {
    p <- suppressWarnings(DotPlot(data, x = "gene", y = "celltype", fill_by = "expression",
                 fill_cutoff = "< 0.05"))
    expect_s3_class(p, "ggplot")
})

test_that("DotPlot width/height attributes are numeric", {
    p <- suppressWarnings(DotPlot(data, x = "gene", y = "celltype", fill_by = "expression"))
    expect_true(is.numeric(attr(p, "width")))
    expect_true(is.numeric(attr(p, "height")))
})

test_that("DotPlot with lower/upper quantile works", {
    p <- suppressWarnings(DotPlot(data, x = "gene", y = "celltype", fill_by = "expression",
                 lower_quantile = 0.1, upper_quantile = 0.9))
    expect_s3_class(p, "ggplot")
})

test_that("DotPlot with explicit cutoffs works", {
    p <- suppressWarnings(DotPlot(data, x = "gene", y = "celltype", fill_by = "expression",
                 lower_cutoff = 0, upper_cutoff = 5))
    expect_s3_class(p, "ggplot")
})

test_that("DotPlot quantile/cutoff works with lollipop", {
    data_lolli <- data.frame(
        expression = c(0.1, 0.5, 0.8, 1.2, 2.0),
        celltype = paste0("CellType", 1:5)
    )
    p <- suppressWarnings(LollipopPlot(data_lolli, x = "expression", y = "celltype",
        fill_by = "expression",
        lower_quantile = 0, upper_quantile = 0.95))
    expect_s3_class(p, "ggplot")
})

#' Atomic Sankey / alluvial diagram (internal)
#'
#' @description
#' Core implementation for drawing a Sankey (alluvial) diagram using
#' \code{\link[ggalluvial]{geom_alluvium}} (or \code{\link[ggalluvial]{geom_flow}})
#' and \code{\link[ggalluvial]{geom_stratum}}.  This function takes a **single**
#' data frame (no \code{split_by} support) and returns a \code{ggplot} object
#' with faceting applied.
#'
#' The function supports five input formats (\code{in_form}) which control how
#' the data columns are interpreted.  Nodes (strata) are rendered as vertical
#' blocks whose fill colour and alpha can be customised independently of the
#' links (alluvia / flows) connecting them.  Link colours can match node
#' colours or use a separate palette; link borders can be set to a fixed colour
#' or to follow the fill colour (\code{links_color = ".fill"}).
#'
#' Automatic legend resolution determines whether nodes on different x-axis
#' positions receive a merged legend or separate legends, based on overlaps
#' between stratum values across positions.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{Format resolution} — \code{in_form} is matched and aliased
#'         (\code{"long"} → \code{"lodes"}, \code{"wide"} → \code{"alluvia"}).
#'   \item \strong{Data parsing by format} — one of five code paths executes:
#'   \itemize{
#'     \item \strong{Lodes / long} — \code{x}, \code{stratum}, \code{alluvium},
#'           and \code{links_fill_by} are validated via
#'           \code{\link{check_columns}()}.  Multi-column inputs are
#'           concatenated with their respective separators.  When \code{y} is
#'           \code{NULL}, counts are computed per (\code{x}, \code{stratum},
#'           \code{alluvium}, \code{links_fill_by}, \code{facet_by}).
#'     \item \strong{Counts} (x without \code{"."} prefix) — numeric \code{x}
#'           columns are pivoted to long form via
#'           \code{\link[tidyr]{pivot_longer}()}, creating the y-axis from the
#'           cell values.  \code{alluvium} and \code{stratum} default to
#'           \code{links_fill_by}.
#'     \item \strong{Counts with source node} (x with \code{"."} as first
#'           element, \code{is_flowcounts}) — same as counts but also injects
#'           \code{links_fill_by} values as an additional first column of
#'           nodes, visualising how flows originate from source groups.
#'     \item \strong{Alluvia / wide} — validated via
#'           \code{\link[ggalluvial]{is_alluvia_form}()} and converted to lodes
#'           form via \code{\link[ggalluvial]{to_lodes_form}()}.
#'           \code{stratum} and \code{alluvium} are ignored.
#'     \item \strong{Auto / fallback} — when no other branch matches, the lodes
#'           path is attempted and validated via
#'           \code{\link[ggalluvial]{is_lodes_form}()}.
#'   }
#'   \item \strong{Palette assignment} — \code{\link{palette_this}()} resolves
#'         colours for both nodes (\code{nodes_palette} / \code{nodes_palcolor})
#'         and links (\code{links_palette} / \code{links_palcolor}).
#'   \item \strong{Flow-counts guide logic} — when \code{is_flowcounts} is
#'         \code{TRUE}, the links guide is suppressed if the first-column node
#'         colours match the link colours, avoiding redundant legends.  When
#'         the palettes are identical but colours differ (too few colours in
#'         the palette), the first-column node colours are reused.
#'   \item \strong{Legend auto-detection} — when \code{nodes_legend = "auto"}:
#'         if \code{nodes_label = TRUE} or if \code{stratum} and
#'         \code{links_fill_by} share identical values and colours, the nodes
#'         legend is hidden.  Otherwise, overlapping stratum values across
#'         x-axis positions are checked: any overlap triggers a merged legend;
#'         no overlap produces separate legends per position.
#'   \item \strong{Base ggplot} — constructed with \code{aes(x = x,
#'         stratum = stratum, alluvium = alluvium, y = y)}.
#'   \item \strong{Separate node fill layers} — when \code{stratum} differs
#'         from \code{links_fill_by} and \code{nodes_legend = "separate"}, a
#'         \code{\link[ggplot2]{geom_col}()} +
#'         \code{\link[ggplot2]{scale_fill_manual}()} layer pair is added per
#'         x-axis position, each followed by
#'         \code{\link[ggnewscale]{new_scale_fill}()} to produce independent
#'         legends.
#'   \item \strong{Link rendering} — \code{\link[ggalluvial]{geom_alluvium}()}
#'         (default) or \code{\link[ggalluvial]{geom_flow}()} when
#'         \code{flow = TRUE}.  When \code{links_color = ".fill"}, the colour
#'         aesthetic is mapped to \code{links_fill_by} and the colour scale
#'         guide is suppressed.  For \code{geom_flow} with a distinct
#'         \code{stratum} and \code{links_fill_by}, \code{stat = "alluvium"}
#'         is forced to preserve the fill variable through the flow stat
#'         transformation.
#'   \item \strong{Link fill scale} — \code{\link[ggplot2]{scale_fill_manual}()}
#'         with \code{links_colors} and the resolved \code{links_guide}.
#'   \item \strong{Node rendering} — \code{\link[ggalluvial]{geom_stratum}()}
#'         with \code{nodes_color} (border) and \code{nodes_alpha}.  When
#'         \code{nodes_color = ".fill"}, the colour aesthetic maps to
#'         \code{stratum}.
#'   \item \strong{Node fill scale} — \code{\link[ggplot2]{scale_fill_manual}()}
#'         with \code{nodes_colors}.  The guide is \code{"none"} when
#'         \code{nodes_legend} is \code{"none"} or \code{"separate"}
#'         (already handled by per-position layers); otherwise \code{"legend"}.
#'   \item \strong{Labels} — when \code{nodes_label = TRUE},
#'         \code{\link[ggplot2]{geom_label}()} with
#'         \code{stat = \link[ggalluvial]{StatStratum}} and
#'         \code{min.y = nodes_label_miny}.
#'   \item \strong{Axes, theme, labels} — \code{\link[ggplot2]{scale_x_discrete}()},
#'         \code{\link[ggplot2]{scale_y_continuous}()}, custom theme, plot
#'         title / subtitle, axis labels, and aesthetic adjustments (aspect
#'         ratio, legend position / direction, grid removal, x-text angle).
#'   \item \strong{Coordinate flip} — when \code{flip = TRUE},
#'         \code{\link[ggplot2]{coord_flip}()} swaps the axes.
#'   \item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
#'         computes \code{height} and \code{width} attributes from the number
#'         of x-axis positions, legend metrics, and flip state, scaled by
#'         \code{aspect.ratio}.
#'   \item \strong{Faceting} — \code{\link{facet_plot}()} wraps the result
#'         when \code{facet_by} is provided.
#' }
#'
#' @inheritParams common_args
#' @param in_form A character string specifying the input data format.
#'  One of \code{"auto"} (default), \code{"long"}, \code{"lodes"},
#'  \code{"wide"}, \code{"alluvia"}, or \code{"counts"}.
#'  \code{"long"} is an alias for \code{"lodes"}; \code{"wide"} is an alias for
#'  \code{"alluvia"}.  See the \code{data} parameter of \code{\link{SankeyPlot}}
#'  for format descriptions.
#' @param x A character vector of column name(s) for the x-axis categories.
#'  Each unique value or concatenated pair represents a time point, state, or
#'  position along the horizontal axis.  Behaviour depends on \code{in_form}:
#'  for \code{"lodes"} at least one column is expected; for \code{"alluvia"}
#'  and \code{"counts"} at least two columns are required.  In the latter two
#'  cases \code{x_sep} is not used.
#' @param x_sep A character string to join multiple \code{x} columns when
#'  \code{in_form} is \code{"lodes"} or auto-determined as lodes.
#'  Default \code{"_"}.
#' @param y A character string specifying the numeric column for the y-axis
#'  (frequency / value).  When \code{NULL} (default), the count of observations
#'  per combination of \code{x}, \code{stratum}, \code{alluvium},
#'  \code{links_fill_by}, and \code{facet_by} is computed automatically.
#'  Ignored when \code{in_form} is \code{"counts"}.
#' @param stratum A character string specifying the column that defines the
#'  node categories at each x-axis position.  Each unique value becomes a
#'  stratum (node block) at each x position.  When \code{NULL}, defaults to
#'  \code{links_fill_by}.  Multiple columns are concatenated with
#'  \code{stratum_sep}.  Ignored in \code{"alluvia"} format.
#' @param stratum_sep A character string to join multiple \code{stratum}
#'  columns.  Default \code{"_"}.
#' @param alluvium A character string specifying the column that identifies
#'  individual flows (alluvia) across x-axis positions.  Each unique value
#'  represents a single observational unit tracked across positions.  When
#'  \code{NULL} in \code{"counts"} format, an auto-generated identifier is
#'  created.  Multiple columns are concatenated with \code{alluvium_sep}.
#'  Ignored in \code{"alluvia"} format.
#' @param alluvium_sep A character string to join multiple \code{alluvium}
#'  columns.  Default \code{"_"}.
#' @param flow A logical value.  When \code{FALSE} (default),
#'  \code{\link[ggalluvial]{geom_alluvium}()} is used for the links.  When
#'  \code{TRUE}, \code{\link[ggalluvial]{geom_flow}()} is used instead, which
#'  draws the flows with a directional gradient between x positions.
#' @param nodes_color A character string specifying the border colour of the
#'  node (stratum) rectangles.  Use the special value \code{".fill"} to match
#'  the border colour to the node fill colour.  Default \code{"grey30"}.
#' @param links_fill_by A character string specifying the column that
#'  determines the fill colour of the links (alluvia / flows).  When
#'  \code{NULL} in \code{"lodes"} format, defaults to \code{alluvium}.  In
#'  \code{"counts"} format with the \code{"."} prefix, this parameter is
#'  required.  Multiple columns are concatenated with
#'  \code{links_fill_by_sep}.
#' @param links_fill_by_sep A character string to join multiple
#'  \code{links_fill_by} columns.  Default \code{"_"}.
#' @param links_name A character string for the legend title of the link fill
#'  scale.  When \code{NULL} (default), the \code{links_fill_by} column name
#'  is used.
#' @param links_color A character string specifying the border colour of the
#'  links (alluvia / flows).  Use the special value \code{".fill"} to match
#'  the link border colour to the link fill colour.  Default \code{"gray80"}.
#' @param nodes_palette A character string specifying the colour palette for
#'  the node (stratum) fill.  Passed to \code{\link{palette_this}()}.
#'  Default \code{"Paired"}.
#' @param nodes_palcolor A character vector of custom colours for the node
#'  fill, used as \code{palcolor} in \code{\link{palette_this}()}.  When
#'  \code{NULL} (default), the palette colours are used directly.
#' @param nodes_alpha A numeric value in \eqn{[0, 1]} controlling the
#'  transparency of the node (stratum) fill.  Default \code{1}.
#' @param nodes_label A logical value.  When \code{TRUE}, stratum labels are
#'  drawn inside each node using \code{\link[ggplot2]{geom_label}()} with
#'  \code{\link[ggalluvial]{StatStratum}}.  Default \code{FALSE}.
#' @param nodes_label_miny A numeric value specifying the minimum y
#'  (frequency) threshold for displaying node labels.  Nodes with y-values
#'  below this threshold are not labelled.  Default \code{0}.
#' @param nodes_width A numeric value (typically 0–1) specifying the width of
#'  the node (stratum) rectangles as a fraction of the x-axis spacing.
#'  Default \code{0.25}.
#' @param nodes_legend Controls how the node legend is displayed.  One of:
#'  \describe{
#'    \item{\code{"auto"} (default)}{Automatically determined:
#'      if \code{nodes_label = TRUE}, or if \code{stratum} is identical to
#'      \code{links_fill_by} with matching colours, the legend is hidden.
#'      Otherwise, overlapping stratum values across x positions are checked:
#'      any overlap produces a merged legend; no overlap produces separate
#'      legends per x position.}
#'    \item{\code{"merge"}}{A single merged legend for all nodes.}
#'    \item{\code{"separate"}}{One legend per x-axis position, generated via
#'      separate \code{\link[ggplot2]{scale_fill_manual}()} layers.}
#'    \item{\code{"none"}}{No node legend is shown.}
#'  }
#' @param links_palette A character string specifying the colour palette for
#'  the link fill.  Passed to \code{\link{palette_this}()}.
#'  Default \code{"Paired"}.
#' @param links_palcolor A character vector of custom colours for the link
#'  fill, used as \code{palcolor} in \code{\link{palette_this}()}.  When
#'  \code{NULL} (default), the palette colours are used directly.
#' @param links_alpha A numeric value in \eqn{[0, 1]} controlling the
#'  transparency of the link fill.  Default \code{0.6}.
#' @param legend.box A character string specifying the arrangement of legend
#'  boxes, either \code{"vertical"} (default) or \code{"horizontal"}.
#' @param flip A logical value.  When \code{TRUE},
#'  \code{\link[ggplot2]{coord_flip}()} is applied to swap the x and y axes.
#'  Default \code{FALSE}.
#' @param ... Additional arguments passed to
#'  \code{\link[ggalluvial]{geom_alluvium}()} or
#'  \code{\link[ggalluvial]{geom_flow}()}, depending on the \code{flow}
#'  setting.  For \code{geom_flow} with a distinct \code{links_fill_by}
#'  column, passing \code{stat = "alluvium"} preserves the fill variable.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @keywords internal
#' @importFrom utils combn
#' @importFrom rlang sym syms %||% dots_n
#' @importFrom dplyr %>% group_by summarise n ungroup mutate add_count
#' @importFrom tidyr pivot_wider pivot_longer
#' @importFrom ggnewscale new_scale_fill
#' @importFrom ggplot2 geom_col scale_fill_manual geom_label after_stat scale_x_discrete scale_y_continuous labs coord_flip
SankeyPlotAtomic <- function(
    data,
    in_form = c("auto", "long", "lodes", "wide", "alluvia", "counts"),
    x,
    x_sep = "_",
    y = NULL,
    stratum = NULL,
    stratum_sep = "_",
    alluvium = NULL,
    alluvium_sep = "_",
    flow = FALSE,
    nodes_color = "grey30",
    links_fill_by = NULL,
    links_fill_by_sep = "_",
    links_name = NULL,
    links_color = "gray80",
    nodes_palette = "Paired",
    nodes_palcolor = NULL,
    palreverse = FALSE,
    nodes_alpha = 1,
    nodes_label = FALSE,
    nodes_width = 0.25,
    nodes_label_miny = 0,
    nodes_legend = c("auto", "separate", "merge", "none"),
    expand = c(0, 0, 0, 0),
    links_palette = "Paired",
    links_palcolor = NULL,
    links_alpha = 0.6,
    legend.box = "vertical",
    keep_empty = TRUE,
    x_text_angle = 0,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    flip = FALSE,
    theme = "theme_this",
    theme_args = list(),
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }

    is_flowcounts <- FALSE
    nodes_legend <- match.arg(nodes_legend)
    in_form <- match.arg(in_form)
    if (in_form == "long") {
        in_form <- "lodes"
    }
    if (in_form == "wide") {
        in_form <- "alluvia"
    }
    if (in_form == "lodes") {
        x <- check_columns(
            data,
            x,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = x_sep
        )
        alluvium <- check_columns(
            data,
            alluvium,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = alluvium_sep
        )
        links_fill_by <- check_columns(
            data,
            links_fill_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = links_fill_by_sep
        ) %||%
            alluvium
        stratum <- check_columns(
            data,
            stratum,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = stratum_sep
        ) %||%
            links_fill_by
        if (is.null(y)) {
            data <- add_count(
                data,
                !!!syms(unique(c(
                    x,
                    stratum,
                    alluvium,
                    links_fill_by,
                    facet_by
                ))),
                name = ".y"
            )
            y <- ".y"
        }
    } else if (
        !identical(x[1], ".") &&
            (in_form == "counts" ||
                (in_form == "auto" &&
                    length(x) > 1 &&
                    all(sapply(data[, x, drop = FALSE], is.numeric))))
    ) {
        x <- check_columns(data, x, allow_multi = TRUE)
        # if (!is.null(stratum)) warning("[SankeyPlot] 'stratum' is ignored in 'counts' format.")
        if (!is.null(y)) {
            warning("[SankeyPlot] 'y' is ignored in 'counts' format.")
        }
        if (is.null(alluvium)) {
            alluvium <- "alluvium"
            data[[alluvium]] <- as.character(1:nrow(data))
        } else {
            alluvium <- check_columns(
                data,
                alluvium,
                force_factor = TRUE,
                allow_multi = TRUE,
                concat_multi = TRUE,
                concat_sep = alluvium_sep
            )
        }
        links_fill_by <- check_columns(
            data,
            links_fill_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = links_fill_by_sep
        ) %||%
            alluvium
        data <- pivot_longer(
            data,
            cols = x,
            names_to = "x",
            values_to = "Frequency"
        )
        data$x <- factor(data$x, levels = x)
        stratum <- check_columns(
            data,
            stratum,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = stratum_sep
        ) %||%
            links_fill_by
        x <- "x"
        y <- "Frequency"
    } else if (
        identical(x[1], ".") &&
            (in_form == "counts" ||
                (in_form == "auto" &&
                    length(x) > 1 &&
                    all(sapply(data[, x[-1], drop = FALSE], is.numeric))))
    ) {
        is_flowcounts <- TRUE
        x <- check_columns(data, x[-1], allow_multi = TRUE)
        stratum <- check_columns(
            data,
            stratum,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = stratum_sep
        )
        stratum_levels <- if (is.null(stratum)) {
            NULL
        } else {
            levels(data[[stratum]])
        }
        if (!is.null(y)) {
            warning("[SankeyPlot] 'y' is ignored in 'counts' format.")
        }
        if (is.null(alluvium)) {
            alluvium <- "alluvium"
            data[[alluvium]] <- as.character(1:nrow(data))
        } else {
            alluvium <- check_columns(
                data,
                alluvium,
                force_factor = TRUE,
                allow_multi = TRUE,
                concat_multi = TRUE,
                concat_sep = alluvium_sep
            )
        }
        links_fill_by <- check_columns(
            data,
            links_fill_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = links_fill_by_sep
        )
        stopifnot(
            "[SankeyPlot] The 'links_fill_by' must be provided in 'counts' format." = !is.null(
                links_fill_by
            )
        )
        data <- pivot_longer(
            data,
            cols = x,
            names_to = "x",
            values_to = "Frequency"
        )
        if (is.null(stratum)) {
            stratum <- links_fill_by
        } else {
            data[[stratum]] <- factor(data[[stratum]], levels = stratum_levels)
        }
        dotdata <- data[data$x == x[1], , drop = FALSE]
        dotdata$x <- links_fill_by
        dotdata[[stratum]] <- dotdata[[links_fill_by]]
        data <- rbind(data, dotdata)
        data$x <- factor(data$x, levels = unique(c(links_fill_by, x)))
        data[[stratum]] <- factor(
            data[[stratum]],
            levels = unique(c(levels(data[[links_fill_by]]), stratum_levels))
        )
        x <- "x"
        y <- "Frequency"
    } else if (
        in_form == "alluvia" ||
            (in_form == "auto" &&
                length(x) > 1 &&
                ggalluvial::is_alluvia_form(data, axes = x, weight = y))
    ) {
        x <- check_columns(data, x, force_factor = TRUE, allow_multi = TRUE)
        stopifnot(
            "[SankeyPlot] 'x' must be at least 2 columns in 'alluvia' format." = length(
                x
            ) >=
                2
        )
        if (!is.null(stratum)) {
            warning("[SankeyPlot] 'stratum' is ignored in 'alluvia' format.")
        }
        if (!is.null(alluvium)) {
            warning("[SankeyPlot] 'alluvium' is ignored in 'alluvia' format.")
        }
        links_fill_by <- check_columns(
            data,
            links_fill_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = links_fill_by_sep
        )
        if (is.null(y)) {
            data <- add_count(
                data,
                !!!syms(unique(c(x, links_fill_by, facet_by))),
                name = ".y"
            )
            y <- ".y"
        }
        # make a copy of links_fill_by in case it's one of x or alluvium that gets transformed later
        if (!is.null(links_fill_by) && links_fill_by %in% x) {
            is_flowcounts <- identical(links_fill_by, x[1])
            data <- ggalluvial::to_lodes_form(
                data,
                axes = x,
                diffuse = !!sym(links_fill_by)
            )
        } else {
            data <- ggalluvial::to_lodes_form(data, axes = x)
        }
        x <- "x"
        alluvium <- "alluvium"
        stratum <- "stratum"
    } else {
        # maybe lodes
        x <- check_columns(
            data,
            x,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = x_sep
        )
        alluvium <- check_columns(
            data,
            alluvium,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = alluvium_sep
        )
        links_fill_by <- check_columns(
            data,
            links_fill_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = links_fill_by_sep
        ) %||%
            alluvium
        stratum <- check_columns(
            data,
            stratum,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = stratum_sep
        ) %||%
            links_fill_by
        if (
            !ggalluvial::is_lodes_form(
                data,
                key = x,
                value = stratum,
                id = alluvium
            )
        ) {
            stop(
                "[SankeyPlot] 'data' must be in 'lodes/long' or 'alluvia/wide' format."
            )
        }
        if (is.null(y)) {
            data <- add_count(
                data,
                !!!syms(unique(c(
                    x,
                    stratum,
                    alluvium,
                    links_fill_by,
                    facet_by
                ))),
                name = ".y"
            )
            y <- ".y"
        }
    }

    nodes_colors <- palette_this(
        levels(data[[stratum]]),
        palette = nodes_palette,
        palcolor = nodes_palcolor,
        reverse = palreverse
    )
    links_colors <- palette_this(
        levels(data[[links_fill_by]]),
        palette = links_palette,
        palcolor = links_palcolor,
        reverse = palreverse
    )
    links_guide <- guide_legend(
        order = 1,
        override.aes = list(
            alpha = min(links_alpha + 0.2, 1),
            color = "transparent"
        )
    )
    if (is_flowcounts) {
        if (identical(nodes_colors[names(links_colors)], links_colors)) {
            links_guide <- "none"
        } else if (
            identical(links_palette, nodes_palette) &&
                identical(links_palcolor, nodes_palcolor)
        ) {
            links_guide <- "none"
            # Plotting the flow of the first column of nodes. Links guide is still showing because
            # the first column of nodes have different colors as the links. It is probably because nodes_palette
            # does not have enough colors. Please use a palette with more colors for both
            # nodes_palette and links_palette.
            nodes_colors1 <- palette_this(
                levels(data[[links_fill_by]]),
                palette = nodes_palette,
                palcolor = nodes_palcolor,
                reverse = palreverse
            )
            nodes_colors <- c(
                nodes_colors1,
                nodes_colors[setdiff(names(nodes_colors), names(nodes_colors1))]
            )
        }
    }

    just <- calc_just(x_text_angle)
    base_size <- theme_args$base_size %||% 12
    text_size_scale <- base_size / 12
    expand <- norm_expansion(expand, x_type = "discrete", y_type = "continuous")
    if (nodes_legend == "auto") {
        if (
            isTRUE(nodes_label) ||
                (identical(stratum, links_fill_by) &&
                    identical(nodes_colors, links_colors))
        ) {
            nodes_legend <- "none"
        } else {
            stratum_values <- lapply(levels(data[[x]]), function(xval) {
                as.character(unique(data[
                    data[[x]] == xval,
                    stratum,
                    drop = TRUE
                ]))
            })
            idxes <- combn(seq_along(stratum_values), 2)
            nodes_legend <- ifelse(
                !is_flowcounts &&
                    any(sapply(
                        as.data.frame(idxes),
                        function(idx) {
                            length(intersect(
                                stratum_values[[idx[1]]],
                                stratum_values[[idx[2]]]
                            )) >
                                0
                        }
                    )),
                "merge",
                "separate"
            )
        }
    }

    p <- ggplot(
        data = data,
        aes(
            x = !!sym(x),
            stratum = !!sym(stratum),
            alluvium = !!sym(alluvium),
            y = !!sym(y)
        )
    )

    # fill nodes on each x so they can have different legends/guides
    # but when stratum and alluvium are the same, they should be filled with the same palette
    if (!identical(stratum, links_fill_by) && nodes_legend == "separate") {
        xs <- levels(data[[x]])
        for (i in seq_along(xs)) {
            xdf <- filter(data, !!sym(x) == xs[i])
            xdf <- xdf[order(xdf[[stratum]]), , drop = FALSE]
            p <- p +
                geom_col(
                    data = xdf,
                    inherit.aes = FALSE,
                    aes(x = !!sym(x), fill = !!sym(stratum), y = 0),
                    width = 0
                ) +
                scale_fill_manual(
                    name = xs[i],
                    values = nodes_colors,
                    breaks = unique(xdf[[stratum]]),
                    guide = guide_legend(
                        order = i + 1,
                        override.aes = list(alpha = min(nodes_alpha + 0.2, 1))
                    )
                ) +
                new_scale_fill()
        }
    }

    if (!isTRUE(flow)) {
        if (identical(links_color, ".fill")) {
            p <- p +
                ggalluvial::geom_alluvium(
                    aes(
                        fill = !!sym(links_fill_by),
                        color = !!sym(links_fill_by)
                    ),
                    width = nodes_width,
                    alpha = links_alpha,
                    na.rm = !keep_empty,
                    ...
                ) +
                scale_color_manual(guide = "none", values = links_colors)
        } else {
            p <- p +
                ggalluvial::geom_alluvium(
                    aes(fill = !!sym(links_fill_by)),
                    width = nodes_width,
                    alpha = links_alpha,
                    color = links_color,
                    na.rm = !keep_empty,
                    ...
                )
        }
    } else {
        if (identical(links_color, ".fill")) {
            # stratum changed to "stratum" after flow stat
            if (identical(stratum, links_fill_by)) {
                p <- p +
                    ggalluvial::geom_flow(
                        aes(
                            fill = after_stat(!!sym("stratum")),
                            color = after_stat(!!sym("stratum"))
                        ),
                        width = nodes_width,
                        alpha = links_alpha,
                        na.rm = !keep_empty,
                        ...
                    ) +
                    scale_color_manual(guide = "none", values = links_colors)
            } else {
                p <- p +
                    ggalluvial::geom_flow(
                        aes(
                            fill = !!sym(links_fill_by),
                            color = !!sym(links_fill_by)
                        ),
                        width = nodes_width,
                        alpha = links_alpha,
                        na.rm = !keep_empty,
                        ...
                    ) +
                    scale_color_manual(guide = "none", values = links_colors)
            }
        } else {
            if (identical(stratum, links_fill_by)) {
                p <- p +
                    ggalluvial::geom_flow(
                        aes(fill = after_stat(!!sym("stratum"))),
                        width = nodes_width,
                        alpha = links_alpha,
                        color = links_color,
                        na.rm = !keep_empty,
                        ...
                    ) +
                    scale_color_manual(guide = "none", values = links_colors)
            } else if (dots_n(...) == 0 || !"stat" %in% names(list(...))) {
                p <- p +
                    ggalluvial::geom_flow(
                        aes(fill = !!sym(links_fill_by)),
                        width = nodes_width,
                        alpha = links_alpha,
                        color = links_color,
                        na.rm = !keep_empty,
                        stat = "alluvium",
                        ...
                    )
            } else {
                warning(
                    "[SankeyPlot] You probably see no color filling for the links. ",
                    paste0(
                        "This is because 'flow' stat of ggalluvial::geom_flow loses '",
                        links_fill_by,
                        "' "
                    ),
                    "while building the plot. Please use 'stat = 'alluvium' instead."
                )
                p <- p +
                    ggalluvial::geom_flow(
                        aes(fill = !!sym(links_fill_by)),
                        width = nodes_width,
                        alpha = links_alpha,
                        color = links_color,
                        na.rm = !keep_empty,
                        ...
                    )
            }
        }
    }

    p <- p +
        scale_fill_manual(
            name = links_name %||% links_fill_by,
            values = links_colors,
            breaks = levels(data[[links_fill_by]]),
            guide = links_guide
        ) +
        new_scale_fill()

    if (identical(nodes_color, ".fill")) {
        p <- p +
            ggalluvial::geom_stratum(
                aes(fill = !!sym(stratum), color = !!sym(stratum)),
                alpha = nodes_alpha,
                width = nodes_width,
                na.rm = !keep_empty
            ) +
            scale_color_manual(guide = "none", values = nodes_colors)
    } else {
        p <- p +
            ggalluvial::geom_stratum(
                aes(fill = !!sym(stratum)),
                alpha = nodes_alpha,
                width = nodes_width,
                color = nodes_color,
                na.rm = !keep_empty
            )
    }

    p <- p +
        scale_fill_manual(
            values = nodes_colors,
            breaks = levels(data[[stratum]]),
            guide = ifelse(
                nodes_legend %in% c("none", "separate"),
                "none",
                "legend"
            )
        )

    if (isTRUE(nodes_label)) {
        p <- p +
            geom_label(
                aes(label = !!sym(stratum)),
                stat = ggalluvial::StatStratum,
                min.y = nodes_label_miny,
                size = text_size_scale * 3
            )
    }

    p <- p +
        scale_x_discrete(expand = expand$x) +
        scale_y_continuous(expand = expand$y) +
        do_call(theme, theme_args) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% x,
            y = ylab %||% ifelse(identical(y, ".y"), links_fill_by, "Frequency")
        ) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            legend.box = legend.box,
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border = element_blank(),
            axis.ticks.x = element_blank(),
            axis.text.x = element_text(
                angle = x_text_angle,
                hjust = just$h,
                vjust = just$v
            )
        )

    if (isTRUE(flip)) {
        p <- p + coord_flip()
        dims <- calculate_plot_dimensions(
            base_height = 6,
            aspect.ratio = aspect.ratio,
            n_y = nlevels(data[[x]]),
            y_scale_factor = 1.5,
            legend.position = legend.position,
            legend.direction = legend.direction,
            legend_n = length(unique(c(
                levels(data[[stratum]]),
                levels(data[[links_fill_by]])
            ))),
            legend_nchar = max(nchar(unique(c(
                levels(data[[stratum]]),
                levels(data[[links_fill_by]])
            )))),
            flip = TRUE
        )
        attr(p, "height") <- dims$height
        attr(p, "width") <- dims$width
    } else {
        dims <- calculate_plot_dimensions(
            base_height = 6,
            aspect.ratio = aspect.ratio,
            n_x = nlevels(data[[x]]),
            x_scale_factor = 1.5,
            legend.position = legend.position,
            legend.direction = legend.direction,
            legend_n = length(unique(c(
                levels(data[[stratum]]),
                levels(data[[links_fill_by]])
            ))),
            legend_nchar = max(nchar(unique(c(
                levels(data[[stratum]]),
                levels(data[[links_fill_by]])
            ))))
        )
        attr(p, "height") <- dims$height
        attr(p, "width") <- dims$width
    }

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction
    )
}


#' Sankey / Alluvial Plot
#'
#' @description
#' Draws Sankey (alluvial) diagrams to visualise flow, movement, or change
#' from one categorical state to another across discrete positions (time
#' points, stages, or groups).  The plot consists of \strong{nodes} (vertical
#' blocks, or strata) representing categories at each position, and
#' \strong{links} (alluvia / flows) representing the observation units that
#' move between categories across positions.
#'
#' The function accepts data in several formats, controlled by \code{in_form}:
#' \describe{
#'   \item{\code{"lodes"} / \code{"long"}}{Each row is an observation at one
#'     x-position, with columns for \code{x}, \code{stratum}, \code{alluvium},
#'     and optionally \code{y}.}
#'   \item{\code{"alluvia"} / \code{"wide"}}{Each row is an observation unit
#'     tracked across all positions; \code{x} columns represent the categories
#'     at each position.  Converted internally via
#'     \code{\link[ggalluvial]{to_lodes_form}()}.}
#'   \item{\code{"counts"}}{Numeric columns under each \code{x} represent
#'     frequencies.  When the first element of \code{x} is \code{"."}, the
#'     \code{links_fill_by} values are injected as an additional first column
#'     of nodes, visualising the source distribution of flows.}
#'   \item{\code{"auto"} (default)}{Automatically detects the format:
#'     numeric multi-column \code{x} → \code{"counts"};
#'     multi-column \code{x} passing \code{is_alluvia_form} → \code{"alluvia"};
#'     otherwise → \code{"lodes"}.}
#' }
#'
#' Supports \strong{split_by} to produce separate sub-plots for different
#' subsets of the data, \strong{facet_by} for within-plot faceting, and
#' independent styling of nodes and links (colours, alpha, borders, labels,
#' and legend behaviour).
#'
#' \code{AlluvialPlot} is an alias of \code{SankeyPlot}.
#'
#' @section split_by workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item The \code{split_by} column(s) are validated and coerced to factors
#'         via \code{\link{check_columns}()}.  Multi-column \code{split_by}
#'         is concatenated with \code{split_by_sep}.
#'   \item Empty factor levels are dropped from \code{split_by}.
#'   \item The data is split by \code{split_by} level (preserving level order).
#'         If \code{split_by} is \code{NULL}, the data is wrapped in a
#'         single-element list with name \code{"..."}.
#'   \item \code{\link{SankeyPlotAtomic}()} is called for each split, with
#'         \code{title} resolved per level (supports function-valued titles).
#'   \item Results are combined via \code{\link{combine_plots}()} (when
#'         \code{combine = TRUE}) or returned as a named list.
#' }
#'
#' @inheritParams common_args
#' @inheritParams SankeyPlotAtomic
#' @return A \code{ggplot} object (single panel, no \code{split_by}), a
#'  \code{patchwork} object (when \code{combine = TRUE} and
#'  \code{split_by} is used), or a named list of \code{ggplot} objects
#'  (when \code{combine = FALSE}).  Each plot carries \code{height} and
#'  \code{width} attributes in inches.
#' @export
#' @rdname sankeyplot
#' @examples
#' \donttest{
#' # Examples from ggalluvial datasets
#' set.seed(8525)
#'
#' data(UCBAdmissions, package = "datasets")
#' UCBAdmissions <- as.data.frame(UCBAdmissions)
#' SankeyPlot(as.data.frame(UCBAdmissions), x = c("Gender", "Dept"),
#'     y = "Freq", nodes_width = 1/12, links_fill_by = "Admit", nodes_label = TRUE,
#'     nodes_palette = "simspec", links_palette = "Set1", links_alpha = 0.5,
#'     nodes_palcolor = "black", links_color = "transparent")
#'
#' data(HairEyeColor, package = "datasets")
#' SankeyPlot(as.data.frame(HairEyeColor), x = c("Hair", "Eye", "Sex"),
#'     y = "Freq", links_fill_by = "Eye", nodes_width = 1/8, nodes_alpha = 0.4,
#'     flip = TRUE, reverse = FALSE, knot.pos = 0, links_color = "transparent",
#'     ylab = "Freq", links_alpha = 0.5, links_name = "Eye (links)", links_palcolor = c(
#'         Brown = "#70493D", Hazel = "#E2AC76", Green = "#3F752B", Blue = "#81B0E4"))
#'
#' data(Refugees, package = "alluvial")
#' country_regions <- c(
#'     Afghanistan = "Middle East",
#'     Burundi = "Central Africa",
#'     `Congo DRC` = "Central Africa",
#'     Iraq = "Middle East",
#'     Myanmar = "Southeast Asia",
#'     Palestine = "Middle East",
#'     Somalia = "Horn of Africa",
#'     Sudan = "Central Africa",
#'     Syria = "Middle East",
#'     Vietnam = "Southeast Asia"
#' )
#' Refugees$region <- country_regions[Refugees$country]
#' SankeyPlot(Refugees, x = "year", y = "refugees", alluvium = "country",
#'     links_fill_by = "country", links_color = ".fill", links_alpha = 0.75,
#'     links_palette = "Set3", facet_by = "region", x_text_angle = -45, nodes_legend = "none",
#'     theme_args = list(strip.background = ggplot2::element_rect(fill="grey80")),
#'     decreasing = FALSE, nodes_width = 0, nodes_color = "transparent", ylab = "refugees",
#'     title = "Refugee volume by country and region of origin")
#'
#' data(majors, package = "ggalluvial")
#' majors$curriculum <- as.factor(majors$curriculum)
#' SankeyPlot(majors, x = "semester", stratum = "curriculum", alluvium = "student",
#'     links_fill_by = "curriculum", flow = TRUE, stat = "alluvium", nodes_palette = "Set2",
#'     links_palette = "Set2")
#'
#' data(vaccinations, package = "ggalluvial")
#' vaccinations <- transform(vaccinations,
#'     response = factor(response, rev(levels(response))))
#' SankeyPlot(vaccinations, x = "survey", stratum = "response", alluvium = "subject",
#'     y = "freq", links_fill_by = "response", nodes_label = TRUE, nodes_alpha = 0.5,
#'     nodes_palette = "seurat", links_palette = "seurat", links_alpha = 0.5,
#'     legend.position = "none", flow = TRUE, expand = c(0, 0, 0, .15), stat = "alluvium",
#'     title = "vaccination survey responses at three points in time")
#'
#' data(Titanic, package = "datasets")
#' SankeyPlot(as.data.frame(Titanic), x = c("Class", "Sex"), y = "Freq",
#'     links_fill_by = "Survived", flow = TRUE, facet_by = "Age", facet_scales = "free_y",
#'     nodes_label = TRUE, expand = c(0.05, 0), xlab = "", links_palette = "Set1",
#'     nodes_palcolor = "white", nodes_label_miny = 10)
#'
#' # Simulated examples
#' df <- data.frame(
#'     Clone = paste0("clone", 1:10),
#'     Timepoint1 = sample(c(rep(0, 30), 1:100), 10),
#'     Timepoint2 = sample(c(rep(0, 30), 1:100), 10)
#' )
#' SankeyPlot(df, x = c("Timepoint1", "Timepoint2"), alluvium = "Clone",
#'     links_color = ".fill")
#'
#' df <- data.frame(
#'     Clone = rep(paste0("clone", 1:6), each = 2),
#'     Timepoint1 = sample(c(rep(0, 30), 1:100), 6),
#'     Timepoint2 = sample(c(rep(0, 30), 1:100), 6),
#'     Group = rep(c("A", "B"), 6)
#' )
#' SankeyPlot(df, x = c(".", "Timepoint1", "Timepoint2"),
#'     stratum = "Group", links_fill_by = "Clone", links_color = ".fill")
#' }
SankeyPlot <- function(
    data,
    in_form = c("auto", "long", "lodes", "wide", "alluvia", "counts"),
    x,
    x_sep = "_",
    y = NULL,
    stratum = NULL,
    stratum_sep = "_",
    alluvium = NULL,
    alluvium_sep = "_",
    split_by = NULL,
    split_by_sep = "_",
    keep_empty = TRUE,
    flow = FALSE,
    expand = c(0, 0, 0, 0),
    nodes_legend = c("auto", "separate", "merge", "none"),
    nodes_color = "grey30",
    links_fill_by = NULL,
    links_fill_by_sep = "_",
    links_name = NULL,
    links_color = "gray80",
    nodes_palette = "Paired",
    nodes_palcolor = NULL,
    nodes_alpha = 1,
    nodes_label = FALSE,
    nodes_label_miny = 0,
    nodes_width = 0.25,
    links_palette = "Paired",
    links_palcolor = NULL,
    palreverse = FALSE,
    links_alpha = 0.6,
    legend.box = "vertical",
    x_text_angle = 0,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    flip = FALSE,
    theme = "theme_this",
    theme_args = list(),
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    theme <- process_theme(theme)
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            SankeyPlotAtomic(
                datas[[nm]],
                in_form = in_form,
                x = x,
                x_sep = x_sep,
                y = y,
                stratum = stratum,
                stratum_sep = stratum_sep,
                alluvium = alluvium,
                alluvium_sep = alluvium_sep,
                keep_empty = keep_empty,
                nodes_legend = nodes_legend,
                nodes_color = nodes_color,
                links_fill_by = links_fill_by,
                links_fill_by_sep = links_fill_by_sep,
                links_name = links_name,
                links_color = links_color,
                expand = expand,
                nodes_label_miny = nodes_label_miny,
                nodes_palette = nodes_palette,
                nodes_palcolor = nodes_palcolor,
                nodes_alpha = nodes_alpha,
                nodes_label = nodes_label,
                nodes_width = nodes_width,
                flow = flow,
                links_palette = links_palette,
                links_palcolor = links_palcolor,
                palreverse = palreverse,
                links_alpha = links_alpha,
                legend.box = legend.box,
                x_text_angle = x_text_angle,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position,
                legend.direction = legend.direction,
                flip = flip,
                theme = theme,
                theme_args = theme_args,
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}


#' @rdname sankeyplot
#' @export
AlluvialPlot <- SankeyPlot

set.seed(8525)
# SankeyPlot / AlluvialPlot test data
# Wide format: each row is an entity, columns are levels in the flow
sankey_wide <- data.frame(
    Year1 = factor(sample(c("A", "B", "C"), 60, replace = TRUE)),
    Year2 = factor(sample(c("X", "Y", "Z"), 60, replace = TRUE)),
    Year3 = factor(sample(c("P", "Q"), 60, replace = TRUE)),
    group = factor(rep(c("G1", "G2"), each = 30))
)

# Long/alluvia format: stratum column defines nodes, alluvium defines flow paths
n <- 120
sankey_long <- data.frame(
    time   = factor(rep(c("T1", "T2", "T3"), each = n / 3)),
    status = factor(c(
        sample(c("A", "B"), n / 3, replace = TRUE),
        sample(c("A", "B", "C"), n / 3, replace = TRUE),
        sample(c("B", "C"), n / 3, replace = TRUE)
    )),
    id = rep(seq_len(n / 3), 3)
)

test_that("SankeyPlot with wide-format data works", {
    skip_if_not_installed("ggalluvial")
    p <- SankeyPlot(sankey_wide, x = c("Year1", "Year2", "Year3"),
                    in_form = "wide", links_fill_by = "Year1")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("SankeyPlot with title and subtitle works", {
    skip_if_not_installed("ggalluvial")
    p <- SankeyPlot(sankey_wide, x = c("Year1", "Year2", "Year3"),
                    in_form = "wide", links_fill_by = "Year1",
                    title = "Sankey Title", subtitle = "Subtitle")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "Sankey Title")
    expect_equal(p$labels$subtitle, "Subtitle")
})

test_that("SankeyPlot with split_by returns patchwork", {
    skip_if_not_installed("ggalluvial")
    p <- SankeyPlot(sankey_wide, x = c("Year1", "Year2", "Year3"),
                    in_form = "wide", links_fill_by = "Year1",
                    split_by = "group", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("SankeyPlot with split_by combine = FALSE returns list", {
    skip_if_not_installed("ggalluvial")
    plots <- SankeyPlot(sankey_wide, x = c("Year1", "Year2", "Year3"),
                        in_form = "wide", links_fill_by = "Year1",
                        split_by = "group", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("SankeyPlot with multiple split_by columns returns a list with one plot per combination", {
    skip_if_not_installed("ggalluvial")
    sankey_wide2 <- sankey_wide
    sankey_wide2$split2 <- factor(rep(c("s1", "s2"), 30))
    plots <- suppressMessages(SankeyPlot(sankey_wide2, x = c("Year1", "Year2", "Year3"),
                                         in_form = "wide", links_fill_by = "Year1",
                                         split_by = c("group", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("SankeyPlot with multiple split_by columns returns combined plot", {
    skip_if_not_installed("ggalluvial")
    sankey_wide2 <- sankey_wide
    sankey_wide2$split2 <- factor(rep(c("s1", "s2"), 30))
    p <- suppressMessages(SankeyPlot(sankey_wide2, x = c("Year1", "Year2", "Year3"),
                                     in_form = "wide", links_fill_by = "Year1",
                                     split_by = c("group", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("SankeyPlot with flip = TRUE works", {
    skip_if_not_installed("ggalluvial")
    p <- SankeyPlot(sankey_wide, x = c("Year1", "Year2", "Year3"),
                    in_form = "wide", links_fill_by = "Year1", flip = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("SankeyPlot in long/lodes format works", {
    skip_if_not_installed("ggalluvial")
    p <- SankeyPlot(sankey_long, in_form = "long",
                    x = "time", stratum = "status", alluvium = "id")
    expect_s3_class(p, "ggplot")
})

test_that("AlluvialPlot is an alias for SankeyPlot", {
    skip_if_not_installed("ggalluvial")
    p <- AlluvialPlot(sankey_wide, x = c("Year1", "Year2", "Year3"),
                      in_form = "wide", links_fill_by = "Year1")
    expect_s3_class(p, "ggplot")
})

test_that("SankeyPlot with links_fill_by works", {
    skip_if_not_installed("ggalluvial")
    p <- SankeyPlot(sankey_wide, x = c("Year1", "Year2", "Year3"),
                    in_form = "wide", links_fill_by = "Year1")
    expect_s3_class(p, "ggplot")
})

#' Get the running enrichment score of a gene set
#'
#' @param genes A vector of genes
#' @param gene_ranks A numeric vector of gene ranks with names
#' @param exponent A numeric value to raise the gene ranks to
#' @param hits_only A logical value to return only the running enrichment score of the hits
#' @return A numeric vector of the running enrichment score
#' @keywords internal
gsea_running_score <- function(
    genes,
    gene_ranks,
    exponent = 1,
    hits_only = TRUE
) {
    genes <- intersect(genes, names(gene_ranks))
    N <- length(gene_ranks)
    Nh <- length(genes)
    Phit <- Pmiss <- numeric(N)
    hits <- names(gene_ranks) %in% genes
    Phit[hits] <- abs(gene_ranks[hits])^exponent
    NR <- sum(Phit)
    if (NR == 0) {
        NR <- 1e-3
    }
    Phit <- cumsum(Phit) / NR
    Pmiss[!hits] <- 1 / (N - Nh)
    Pmiss <- cumsum(Pmiss)
    runningScore <- Phit - Pmiss
    if (hits_only) {
        runningScore <- runningScore[hits]
        names(runningScore) <- names(gene_ranks)[hits]
        runningScore[genes]
    } else {
        names(runningScore) <- names(gene_ranks)
        runningScore
    }
}

#' Prepare fgsea result for plotting
#'
#' @param data A data frame of fgsea results
#' @return A data frame with the desired columns for plotting and the gene ranks and gene sets as attributes
#' @keywords internal
prepare_fgsea_result <- function(data) {
    data$ID <- data$pathway
    data$Description <- data$pathway
    data$pathway <- NULL
    data$pvalue <- data$pval
    data$pval <- NULL
    data$p.adjust <- data$padj
    data$p.adjust[is.na(data$p.adjust)] <- 1
    data$padj <- NULL
    if (is.character(data$leadingEdge)) {
        data$leadingEdge <- strsplit(data$leadingEdge, ",")
    }
    data$core_enrichment <- sapply(data$leadingEdge, paste0, collapse = "/")
    data$leadingEdge <- NULL
    data$NES[is.na(data$NES)] <- 0

    data
}

#' GSEA summary dot plot
#'
#' @description
#' Produces a summary dot plot of GSEA (Gene Set Enrichment Analysis) results.
#' Each row represents a gene set (term), positioned along the x-axis by its
#' Normalized Enrichment Score (NES). Dot colour encodes the significance level
#' (typically \code{-log10(p.adjust)}) on a continuous gradient, and each row
#' includes a miniature line plot showing the gene ranks or running enrichment
#' score for that term's gene set.
#'
#' The function supports both \code{DOSE} and \code{fgsea} package output
#' formats via the \code{in_form} parameter. Terms can be ranked and selected
#' by a significance metric (\code{top_term}, \code{metric}), with
#' non-significant terms rendered in grey. The per-term line plots can show
#' either the raw preranked gene statistics (\code{line_by = "prerank"}) or
#' the running enrichment score (\code{line_by = "running_score"}).
#'
#' @rdname gsea
#' @inheritParams common_args
#' @param in_form The format of the input data. Options:
#'  \itemize{
#'    \item{\code{"auto"} (default): Automatically detect the format. If the
#'      data contains a column named \code{"leadingEdge"}, it is treated as
#'      \code{"fgsea"}; if it contains \code{"core_enrichment"}, it is treated
#'      as \code{"dose"}.}
#'    \item{\code{"fgsea"}: The input data is from the \code{fgsea} package.
#'      The \code{prepare_fgsea_result()} helper renames columns internally.}
#'    \item{\code{"dose"}: The input data is from the \code{DOSE} package.}
#'  }
#' @param gene_ranks A named numeric vector of gene-level rank statistics, with
#'  gene identifiers as names. Used to construct the per-term line plots.
#'  If a character string starting with \code{"@"}, the attribute of \code{data}
#'  with that name (minus the \code{"@"}) is used as the gene ranks vector.
#' @param gene_sets A named list of gene sets. Each name must correspond to an
#'  \code{ID} in \code{data}, and each element is a character vector of gene
#'  identifiers. Typically read from a GMT file. If a character string starting
#'  with \code{"@"}, the attribute of \code{data} with that name (minus the
#'  \code{"@"}) is used.
#' @param top_term Integer specifying the number of top terms to display, ranked
#'  by \code{metric}. If \code{NULL}, all terms are shown.
#' @param metric Character string specifying the column name used to rank terms
#'  and assess significance. Typically \code{"p.adjust"} or \code{"pvalue"}.
#'  Terms are ranked by this column (ascending, lower is better) when
#'  \code{top_term} is set. The same column is transformed to
#'  \code{-log10(metric)} for the colour gradient.
#' @param cutoff Numeric threshold for the \code{metric} column. Terms with
#'  values below this cutoff are coloured on a gradient; terms above are drawn
#'  in grey (\code{"grey80"}) and labelled as insignificant via
#'  \code{nonsig_name}. Default is \code{0.05}. If \code{NULL}, all terms are
#'  treated as significant.
#' @param character_width Integer specifying the maximum character width for
#'  wrapping term descriptions on the y-axis. Default is \code{50}.
#' @param line_plot_size Numeric controlling the size of the per-term miniature
#'  enrichment plots embedded in each row. Expressed as a fraction of the plot
#'  panel dimensions. Default is \code{0.25}.
#' @param metric_name Character string for the colour bar legend title.
#'  Defaults to the value of \code{metric}.
#' @param nonsig_name Character string for the legend entry label used for
#'  non-significant terms. Default is \code{"Insignificant"}.
#' @param linewidth Numeric specifying the line width within the per-term
#'  miniature enrichment plots. Default is \code{0.2}.
#' @param line_by The method used to compute the per-term line plots:
#'  \itemize{
#'    \item{\code{"prerank"} (default): Use the gene ranks as the bar heights
#'      (raw ranking metric).}
#'    \item{\code{"running_score"}: Use the running enrichment score computed
#'      by \code{gsea_running_score()}.}
#'  }
#' @importFrom scales pretty_breaks scientific
#' @importFrom ggplot2 geom_linerange layer_scales theme_void ylim
#' @export
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @examples
#' \donttest{
#' data(gsea_example)
#'
#' # Default summary dot plot with preranked gene statistics
#' GSEASummaryPlot(gsea_example)
#'
#' # Use running enrichment score for per-term line plots
#' GSEASummaryPlot(gsea_example, line_by = "running_score")
#'
#' # Raise the significance cutoff (all terms are coloured)
#' GSEASummaryPlot(gsea_example, cutoff = 0.01)
#' }
GSEASummaryPlot <- function(
    data,
    in_form = c("auto", "dose", "fgsea"),
    gene_ranks = "@gene_ranks",
    gene_sets = "@gene_sets",
    top_term = 10,
    metric = "p.adjust",
    cutoff = 0.05,
    character_width = 50,
    line_plot_size = 0.25,
    metric_name = metric,
    nonsig_name = "Insignificant",
    linewidth = 0.2,
    line_by = c("prerank", "running_score"),
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    alpha = 0.6,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    theme = "theme_this",
    theme_args = list(),
    palette = "Spectral",
    palcolor = NULL,
    palreverse = FALSE,
    seed = 8525,
    ...
) {
    set.seed(seed)
    in_form <- match.arg(in_form)
    theme <- process_theme(theme)
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    line_by <- match.arg(line_by)
    if (inherits(data, "gseaResult")) {
        data <- as.data.frame(data)
    }

    if (
        is.character(gene_ranks) &&
            length(gene_ranks) == 1 &&
            startsWith(gene_ranks, "@")
    ) {
        gene_ranks <- attr(data, substring(gene_ranks, 2))
    }
    if (is.null(gene_ranks)) {
        stop("'gene_ranks' must be provided")
    }
    if (is.null(names(gene_ranks))) {
        stop("'gene_ranks' must have names")
    }
    if (!is.numeric(gene_ranks)) {
        stop("'gene_ranks' must be numeric")
    }
    gene_ranks <- gene_ranks[order(-gene_ranks)]

    if (
        is.character(gene_sets) &&
            length(gene_sets) == 1 &&
            startsWith(gene_sets, "@")
    ) {
        gene_sets <- attr(data, substring(gene_sets, 2))
    }
    if (is.null(gene_sets)) {
        stop("'gene_sets' must be provided")
    }
    if (!is.list(gene_sets)) {
        stop("'gene_sets' must be a list")
    }

    if (in_form == "auto") {
        if ("leadingEdge" %in% colnames(data)) {
            in_form <- "fgsea"
        } else if ("core_enrichment" %in% colnames(data)) {
            in_form <- "dose"
        } else {
            stop(
                "Cannot detect the input format. Please set 'in_form' to 'fgsea' or 'dose'."
            )
        }
    }
    if (in_form == "fgsea") {
        data <- prepare_fgsea_result(data)
    }
    if (!is.null(top_term)) {
        data <- slice_min(data, !!sym(metric), n = top_term, with_ties = FALSE)
    }
    data$ID <- factor(data$ID, levels = rev(unique(data$ID)))
    data <- data[order(data$ID), , drop = FALSE]

    if (!is.null(cutoff)) {
        # data <- data[data[[metric]] < cutoff, , drop = FALSE]
        data$.signif <- data[[metric]] < cutoff
    } else {
        data$.signif <- TRUE
    }
    data$metric <- -log10(data[[metric]])
    check_columns(data, "Description", force_factor = TRUE)
    data$Description <- droplevels(data$Description)
    # data <- data[order(data$Description), , drop = FALSE]
    data$Description <- str_wrap(data$Description, width = character_width)
    data$Description <- factor(
        data$Description,
        levels = unique(data$Description)
    )
    data$y <- as.integer(data$Description)
    sig_metrics <- data$metric[data$.signif]

    if (all(data$.signif)) {
        p <- ggplot(data, aes(x = !!sym("NES"), y = !!sym("y")))
    } else {
        p <- ggplot(data, aes(x = !!sym("NES"), y = !!sym("y"), fill = "")) +
            guides(
                fill = guide_legend(
                    title = nonsig_name %||% "Insignificant",
                    override.aes = list(color = "grey80", shape = 15, size = 4),
                    order = 2
                )
            )
    }
    # need a layer to get the scales of the plot
    p <- p + geom_point(aes(color = !!sym("metric")), size = 0)
    x_range <- diff(layer_scales(p)$x$range$range)
    y_range <- diff(layer_scales(p)$y$range$range)
    colors <- palette_this(
        sig_metrics,
        n = sum(data$.signif),
        palette = palette,
        palcolor = palcolor,
        alpha = alpha,
        type = "continuous",
        transparent = FALSE,
        reverse = palreverse
    )
    line_plot_list <- list()

    for (i in seq_len(nrow(data))) {
        if (isTRUE(data$.signif[i])) {
            # since the values are continuous, number of colors is not equal to number of points
            color <- colors[ceiling(
                data$metric[i] * length(colors) / max(sig_metrics)
            )]
        } else {
            color <- "grey80"
        }
        hits <- intersect(
            gene_sets[[as.character(data$ID[i])]],
            names(gene_ranks)
        )
        if (line_by == "running_score") {
            scores <- gsea_running_score(hits, gene_ranks)
        } else {
            scores <- gene_ranks[match(hits, names(gene_ranks))]
        }

        df <- data.frame(
            x = factor(hits, levels = names(gene_ranks)),
            y = 0,
            ymin = pmin(scores, 0),
            ymax = pmax(scores, 0)
        )
        yr <- max(max(df$ymax), abs(min(df$ymin))) * 1.05
        x_min <- -0.05 * length(gene_ranks)
        x_max <- 1.05 * length(gene_ranks)

        lp <- ggplot(df, aes(x = !!sym("x"), y = !!sym("y"))) +
            geom_rect(
                xmin = x_min,
                xmax = x_max,
                ymin = -yr,
                ymax = yr,
                fill = color
            ) +
            geom_linerange(
                aes(ymin = !!sym("ymin"), ymax = !!sym("ymax")),
                linewidth = linewidth
            ) +
            ylim(-yr, yr) +
            theme_void() +
            ggplot2::theme(
                axis.text = element_blank(),
                axis.title = element_blank(),
                axis.ticks = element_blank(),
                legend.position = "none"
            )

        line_plot_list[[i]] <- annotation_custom(
            ggplotGrob(lp),
            xmin = data$NES[i] - x_range * line_plot_size / 2,
            xmax = data$NES[i] + x_range * line_plot_size / 2,
            ymin = i - y_range * line_plot_size / 8,
            ymax = i + y_range * line_plot_size / 8
        )
    }

    if (any(data$NES > 0) && any(data$NES < 0)) {
        p <- p + geom_vline(xintercept = 0, linetype = 2, color = "grey80")
    }

    p <- p + line_plot_list
    if (length(colors) == 0) {
        # in case all terms are not significant
        p <- p + scale_color_gradientn(colors = "grey80", guide = "none")
    } else {
        p <- p +
            scale_color_gradientn(
                name = metric_name,
                colors = colors,
                breaks = pretty_breaks(n = 4),
                labels = function(x) scientific(10^(-x), digits = 2),
                guide = guide_colorbar(
                    frame.colour = "black",
                    ticks.colour = "black",
                    title.hjust = 0,
                    order = 1
                )
            )
    }
    p <- p +
        scale_y_continuous(
            breaks = seq_len(nrow(data)),
            labels = data$Description
        ) +
        scale_x_continuous(expand = c(0.05, x_range * line_plot_size / 2)) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% "NES",
            y = ylab %||% ""
        ) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major.y = element_line(colour = "grey80", linetype = 2),
        )

    max_nchar_y <- min(max(nchar(levels(data$Description))), character_width)
    dims <- calculate_plot_dimensions(
        base_height = nrow(data) * 0.65,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction
    )
    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width + max_nchar_y * 0.1

    p
}

#' Atomic GSEA plot (internal)
#'
#' @description
#' Core implementation for drawing a single GSEA (Gene Set Enrichment Analysis)
#' ridge plot. This is the workhorse behind the exported \code{\link{GSEAPlot}}
#' function — it takes a single gene set and produces a three-panel figure
#' showing the running enrichment score, hit positions, and the ranked list
#' metric.
#'
#' The plot consists of three vertically stacked panels:
#' \enumerate{
#'   \item \strong{Running score panel} — traces the running enrichment score
#'   across the ranked gene list, with a peak annotation and optional gene
#'   labels for core enrichment genes.
#'   \item \strong{Hit indicator panel} — marks the positions of gene set
#'   members with vertical lines, colour-coded by the rank metric value
#'   (red for positive, blue for negative).
#'   \item \strong{Gene ranking panel} — shows every gene's rank as a vertical
#'   segment, with annotations indicating positively and negatively correlated
#'   tails and the zero-crossing point.
#' }
#'
#' The running enrichment score is computed via \code{\link{gsea_running_score}()}
#' using the \code{gene_ranks} and \code{genes} provided. Core enrichment genes
#' (from the \code{core_enrichment} column) can be labelled on the score panel
#' using \code{\link[ggrepel]{geom_text_repel}()}.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
#'   \code{ggplot2::ggplot} based on
#'   \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{Data subsetting} — if \code{data} contains multiple gene
#'   sets (\code{nrow > 1}), it is filtered to the single row matching
#'   \code{gs}. An error is raised if no rows match.
#'   \item \strong{Gene rank resolution} — resolves \code{gene_ranks} from its
#'   \code{"@"}-prefixed attribute reference, validates that it is a named
#'   numeric vector, and sorts it in descending order.
#'   \item \strong{Title and subtitle construction} — \code{title} defaults to
#'   \code{data$Description}. \code{subtitle} is built from the NES value and
#'   the \code{metric} column with significance stars (\code{ns/*/**/***/****}).
#'   \item \strong{Plot data preparation} — builds a data frame with position
#'   (1:N), gene names, running enrichment score (via
#'   \code{gsea_running_score()} with \code{hits_only = FALSE}), raw ranks,
#'   and a hit indicator column.
#'   \item \strong{Running score panel} — renders the top panel with a red/blue
#'   background (positive/negative score regions), a horizontal baseline at
#'   \code{y = 0}, the running score line, and annotations for the peak
#'   (dashed reference lines and an upward/downward triangle). Title and
#'   subtitle are added via \code{\link[ggplot2]{ggtitle}()}.
#'   \item \strong{Gene labeling} — when \code{n_coregenes > 1} or
#'   \code{genes_label} is provided, core enrichment genes are extracted from
#'   \code{data$core_enrichment} and labelled on the running score panel using
#'   \code{geom_text_repel()}. Missing genes trigger a warning.
#'   \item \strong{Hit indicator panel} — renders the middle panel: vertical
#'   lines (\code{geom_linerange}) at each hit position, with coloured
#'   rectangles beneath showing the rank metric gradient (red for positive,
#'   blue for negative).
#'   \item \strong{Gene ranking panel} — renders the bottom panel: vertical
#'   segments (\code{geom_segment}) for every gene's rank, with annotations
#'   for positively/negatively correlated tails. If both signs exist, the
#'   zero-crossing point is marked with a dashed vertical line.
#'   \item \strong{Panel assembly} — the three panels are stacked vertically
#'   via \code{\link[patchwork]{wrap_plots}()} with spacer panels and
#'   proportional heights (\code{3.5 / 1 / 1.5}). When \code{ylab} is provided,
#'   a rotated \code{\link[grid]{textGrob}()} is added to the right side and
#'   the plot width increases from 7.5 to 8 inches.
#'   \item \strong{Dimensions} — height is fixed at 6.5 inches. Width is 7.5
#'   inches (without \code{ylab}) or 8 inches (with \code{ylab}). Attributes
#'   \code{height} and \code{width} are stored on the returned object.
#' }
#'
#' @inheritParams common_args
#' @param gene_ranks A named numeric vector of gene-level rank statistics, with
#'  gene identifiers as names. Used to calculate the running enrichment score
#'  and the ranked list metric. If a character string starting with
#'  \code{"@"}, the attribute of \code{data} with that name (minus the
#'  \code{"@"}) is used.
#' @param gs Character string specifying the \code{ID} of the gene set to plot.
#'  Must match a value in \code{data$ID}.
#' @param genes Character vector of gene identifiers belonging to the gene set.
#'  These are intersected with \code{names(gene_ranks)} for the running score
#'  calculation.
#' @param metric Character string specifying the column name in \code{data}
#'  used to compute significance stars in the subtitle. Default is
#'  \code{"p.adjust"}.
#' @param sample_coregenes Logical; if \code{TRUE}, core enrichment genes are
#'  sampled randomly for labelling. If \code{FALSE} (default), the first
#'  \code{n_coregenes} core enrichment genes are used.
#' @param line_width Numeric specifying the line width for the running
#'  enrichment score curve. Default is \code{1.5}.
#' @param line_alpha Numeric alpha transparency for the running score line and
#'  hit indicator bars. Default is \code{1}.
#' @param line_color Character string specifying the colour of the running
#'  enrichment score line. Default is \code{"#6BB82D"}.
#' @param n_coregenes Integer specifying the number of core enrichment genes to
#'  label on the running score plot. Default is \code{10}. Ignored when
#'  \code{genes_label} is provided.
#' @param genes_label Character vector of specific gene names to label on the
#'  running score plot. When provided, \code{n_coregenes} is ignored.
#' @param label_fg Character string specifying the text colour of gene labels.
#'  Default is \code{"black"}.
#' @param label_bg Character string specifying the background colour of gene
#'  labels. Default is \code{"white"}.
#' @param label_bg_r Numeric specifying the corner radius of the label
#'  background. Default is \code{0.1}.
#' @param label_size Numeric specifying the font size of the label text.
#'  Default is \code{4}.
#' @param ylab Character string for a right-side y-axis label (rotated 90
#'  degrees). When provided, an extra text grob is added to the right of the
#'  assembled plot, and the plot width increases from 7.5 to 8 inches.
#' @keywords internal
#' @return A \code{patchwork} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @importFrom scales alpha
#' @importFrom ggplot2 ggtitle theme_classic annotate
#' @importFrom patchwork plot_layout wrap_plots plot_spacer
GSEAPlotAtomic <- function(
    data,
    gene_ranks = "@gene_ranks",
    gs,
    genes,
    metric = "p.adjust",
    sample_coregenes = FALSE,
    line_width = 1.5,
    line_alpha = 1,
    line_color = "#6BB82D",
    n_coregenes = 10,
    genes_label = NULL,
    label_fg = "black",
    label_bg = "white",
    label_bg_r = 0.1,
    label_size = 4,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    if (nrow(data) > 1) {
        data <- data[data$ID == gs, , drop = FALSE]
    }
    if (nrow(data) == 0) {
        stop("Gene set ", gs, " is not in the data")
    }
    if (
        is.character(gene_ranks) &&
            length(gene_ranks) == 1 &&
            startsWith(gene_ranks, "@")
    ) {
        gene_ranks <- attr(data, substring(gene_ranks, 2))
    }
    if (is.null(gene_ranks)) {
        stop("'gene_ranks' must be provided")
    }
    if (is.null(names(gene_ranks))) {
        stop("'gene_ranks' must have names")
    }
    if (!is.numeric(gene_ranks)) {
        stop("'gene_ranks' must be numeric")
    }
    gene_ranks <- gene_ranks[order(-gene_ranks)]

    title <- title %||% data$Description
    sig <- case_when(
        data[[metric]] > 0.05 ~ "ns",
        data[[metric]] <= 0.05 & data[[metric]] > 0.01 ~ "*",
        data[[metric]] <= 0.01 & data[[metric]] > 0.001 ~ "**",
        data[[metric]] <= 0.001 & data[[metric]] > 0.0001 ~ "***",
        data[[metric]] <= 0.0001 ~ "****"
    )
    subtitle <- subtitle %||%
        paste0(
            "(NES=",
            round(data$NES, 3),
            ", ",
            metric,
            "=",
            format(data[[metric]], digits = 3, scientific = TRUE),
            ", ",
            sig,
            ")"
        )

    df <- data.frame(
        x = 1:length(gene_ranks),
        genes = names(gene_ranks),
        runningScore = gsea_running_score(genes, gene_ranks, hits_only = FALSE),
        ranks = gene_ranks,
        position = as.integer(names(gene_ranks) %in% genes)
    )

    index_max <- which.max(abs(df$runningScore))
    p <- ggplot(df, aes(x = !!sym("x"))) +
        ggplot2::xlab(NULL) +
        theme_classic(base_size = 12) +
        ggplot2::theme(
            panel.grid.major = element_line(colour = "grey90", linetype = 2),
            panel.grid.minor = element_line(colour = "grey90", linetype = 2)
        ) +
        scale_x_continuous(expand = c(0.01, 0))

    ############# The Running Score Panel #############
    p1 <- p +
        # background
        geom_rect(
            data = data.frame(
                xmin = -Inf,
                xmax = Inf,
                ymin = c(0, -Inf),
                ymax = c(Inf, 0),
                fill = c(alpha("#C40003", 0.2), alpha("#1D008F", 0.2))
            ),
            mapping = aes(
                xmin = !!sym("xmin"),
                xmax = !!sym("xmax"),
                ymin = !!sym("ymin"),
                ymax = !!sym("ymax"),
                fill = I(!!sym("fill"))
            ),
            inherit.aes = FALSE
        ) +
        geom_hline(yintercept = 0, linetype = 1, color = "grey40") +
        # running score
        geom_line(
            aes(y = !!sym("runningScore")),
            color = line_color,
            linewidth = line_width,
            alpha = line_alpha
        ) +
        annotate(
            geom = "segment",
            x = 0,
            xend = df$x[index_max],
            y = df$runningScore[index_max],
            yend = df$runningScore[index_max],
            linetype = 2
        ) +
        annotate(
            geom = "segment",
            x = df$x[index_max],
            xend = df$x[index_max],
            y = 0,
            yend = df$runningScore[index_max],
            linetype = 2
        ) +
        annotate(
            geom = "point",
            x = df$x[index_max],
            y = df$runningScore[index_max],
            fill = ifelse(data$NES < 0, "#5E34F5", "#F52323"),
            color = "black",
            size = 2.5,
            shape = ifelse(data$NES < 0, 25, 24)
        ) +
        ggplot2::ylab("Enrichment Score") +
        ggplot2::theme(
            axis.text.x = element_blank(),
            axis.ticks.x = element_blank(),
            axis.line = element_blank(),
            panel.border = element_rect(
                color = "black",
                fill = "transparent",
                linewidth = 1
            ),
            plot.margin = margin(t = 0.2, r = 0.2, b = 0, l = 0.2, unit = "cm"),
            legend.position = "none",
            plot.subtitle = element_text(face = "italic")
        ) +
        ggtitle(title, subtitle)

    # label the genes
    if (
        (is.numeric(n_coregenes) && n_coregenes > 1) || length(genes_label) > 0
    ) {
        if (length(genes_label) == 0) {
            genes_label_tmp <- unlist(strsplit(data$core_enrichment, "/"))
            n_coregenes <- min(n_coregenes, length(genes_label_tmp))
            if (isTRUE(sample_coregenes)) {
                genes_label_tmp <- sample(
                    genes_label_tmp,
                    n_coregenes,
                    replace = FALSE
                )
            } else {
                genes_label_tmp <- df$genes[df$genes %in% genes_label_tmp][
                    1:n_coregenes
                ]
            }
        } else {
            genes_label_tmp <- genes_label
        }
        df_gene <- df[
            df$position == 1 & df$genes %in% genes_label_tmp,
            ,
            drop = FALSE
        ]
        gene_drop <- genes_label_tmp[!genes_label_tmp %in% df_gene$genes]
        if (length(gene_drop) > 0) {
            if (identical(data$ID, data$Description)) {
                warning(
                    "Gene ",
                    paste(gene_drop, collapse = ","),
                    " is not in the geneset ",
                    data$ID,
                    immediate. = TRUE
                )
            } else {
                warning(
                    "Gene ",
                    paste(gene_drop, collapse = ","),
                    " is not in the geneset ",
                    data$ID,
                    ": ",
                    data$Description,
                    immediate. = TRUE
                )
            }
        }
        x_nudge <- diff(range(df$x)) * 0.05
        y_nudge <- diff(range(df$runningScore)) * 0.05
        p1 <- p1 +
            geom_point(
                data = df_gene,
                mapping = aes(y = !!sym("runningScore")),
                color = "black"
            ) +
            geom_text_repel(
                data = df_gene,
                mapping = aes(
                    y = !!sym("runningScore"),
                    label = !!sym("genes")
                ),
                min.segment.length = 0,
                max.overlaps = 100,
                segment.colour = "grey40",
                color = label_fg,
                bg.color = label_bg,
                bg.r = label_bg_r,
                size = label_size,
                nudge_x = ifelse(df_gene$runningScore >= 0, x_nudge, -x_nudge),
                nudge_y = ifelse(df_gene$runningScore > 0, -y_nudge, y_nudge)
            )
    }

    ############# The Line Plot Panel #############
    p2 <- ggplot(df, aes(x = !!sym("x"))) +
        geom_linerange(
            aes(ymax = !!sym("position")),
            ymin = 0,
            alpha = line_alpha
        ) +
        ggplot2::xlab(NULL) +
        ggplot2::ylab(NULL) +
        theme_classic(base_size = 12) +
        ggplot2::theme(
            legend.position = "none",
            plot.margin = margin(
                t = -0.1,
                b = 0,
                r = 0.2,
                l = 0.2,
                unit = "cm"
            ),
            panel.border = element_rect(
                color = "black",
                fill = "transparent",
                linewidth = 1
            ),
            axis.line.y = element_blank(),
            axis.line.x = element_blank(),
            axis.ticks = element_blank(),
            axis.text = element_blank()
        ) +
        scale_x_continuous(expand = c(0.01, 0)) +
        scale_y_continuous(expand = c(0, 0))

    x <- df$x
    y <- y_raw <- df$ranks
    y[y > quantile(y_raw, 0.98)] <- quantile(y_raw, 0.98)
    y[y < quantile(y_raw, 0.02)] <- quantile(y_raw, 0.02)
    col <- rep("white", length(y))
    y_pos <- which(y > 0)
    if (length(y_pos) > 0) {
        y_pos_i <- cut(
            y[y_pos],
            breaks = seq(
                min(y[y_pos], na.rm = TRUE),
                max(y[y_pos], na.rm = TRUE),
                len = 100
            ),
            include.lowest = TRUE
        )
        col[y_pos] <- colorRampPalette(c("#F5DCDC", "#C40003"))(100)[y_pos_i]
    }

    y_neg <- which(y < 0)
    if (length(y_neg) > 0) {
        y_neg_i <- cut(
            y[y_neg],
            breaks = seq(
                min(y[y_neg], na.rm = TRUE),
                max(y[y_neg], na.rm = TRUE),
                len = 100
            ),
            include.lowest = TRUE
        )
        col[y_neg] <- colorRampPalette(c("#1D008F", "#DDDCF5"))(100)[y_neg_i]
    }
    xmin <- which(!duplicated(col))
    xmax <- xmin + as.numeric(table(col)[as.character(unique(col))])
    d <- data.frame(
        ymin = 0,
        ymax = 0.3,
        xmin = xmin,
        xmax = xmax,
        col = unique(col)
    )
    p2 <- p2 +
        geom_rect(
            aes(
                xmin = !!sym("xmin"),
                xmax = !!sym("xmax"),
                ymin = !!sym("ymin"),
                ymax = !!sym("ymax"),
                fill = I(!!sym("col"))
            ),
            data = d,
            alpha = 0.95,
            inherit.aes = FALSE
        )

    ############# The gene ranking panel #############
    p3 <- p +
        geom_segment(
            aes(
                x = !!sym("x"),
                xend = !!sym("x"),
                y = !!sym("ranks"),
                yend = 0
            ),
            color = "grey30"
        )

    cross_x <- median(df$x[which.min(abs(df$ranks))])
    if (max(df$ranks) > 0) {
        p3 <- p3 +
            annotate(
                geom = "text",
                x = 0,
                y = Inf,
                vjust = 1.4,
                hjust = 0,
                color = "#C81A1F",
                size = 4,
                label = " Positively correlated"
            )
    }
    if (min(df$ranks) < 0) {
        p3 <- p3 +
            annotate(
                geom = "text",
                x = Inf,
                y = -Inf,
                vjust = -0.5,
                hjust = 1.02,
                color = "#3C298C",
                size = 4,
                label = "Negtively correlated "
            )
    }
    if (max(df$ranks) > 0 && min(df$ranks) < 0) {
        p3 <- p3 +
            geom_vline(xintercept = cross_x, linetype = 2, color = "black") +
            annotate(
                geom = "text",
                y = 0,
                x = cross_x,
                vjust = ifelse(diff(abs(range(df$ranks))) > 0, -0.3, 1.3),
                size = 4,
                label = paste0("Zero cross at ", cross_x)
            )
    }
    p3 <- p3 +
        ggplot2::ylab("Ranked List Metric") +
        ggplot2::xlab(xlab %||% "Rank in Ordered Dataset") +
        ggplot2::theme(
            plot.margin = margin(
                t = -0.1,
                r = 0.2,
                b = 0.2,
                l = 0.2,
                unit = "cm"
            ),
            axis.line = element_blank(),
            axis.line.x = element_blank(),
            panel.border = element_rect(
                color = "black",
                fill = "transparent",
                linewidth = 1
            )
        )

    if (!is.null(ylab)) {
        p4 <- textGrob(label = ylab, rot = -90, hjust = 0.5)

        p <- wrap_plots(
            p1,
            plot_spacer(),
            p2,
            plot_spacer(),
            p3,
            p4,
            heights = c(3.5, -0.19, 1, -0.24, 1.5),
            widths = c(12, .5)
        ) +
            plot_layout(axes = "collect", design = "AF\nBF\nCF\nDF\nEF")
        dims <- calculate_plot_dimensions(
            base_height = 6.5,
            aspect.ratio = NULL,
            legend.position = "none"
        )
        attr(p, "height") <- dims$height
        attr(p, "width") <- 8
    } else {
        p <- wrap_plots(
            p1,
            plot_spacer(),
            p2,
            plot_spacer(),
            p3,
            ncol = 1,
            heights = c(3.5, -0.19, 1, -0.24, 1.5)
        ) +
            plot_layout(axes = "collect")
        dims <- calculate_plot_dimensions(
            base_height = 6.5,
            aspect.ratio = NULL,
            legend.position = "none"
        )
        attr(p, "height") <- dims$height
        attr(p, "width") <- 7.5
    }

    p
}

#' @rdname gsea
#' @inheritParams common_args
#' @inheritParams GSEAPlotAtomic
#' @param in_form The format of the input data. See \code{\link{GSEASummaryPlot}}
#'  for details.
#' @param gene_sets A named list of gene sets. Each name must correspond to an
#'  \code{ID} in \code{data}, and each element is a character vector of gene
#'  identifiers. A GSEA ridge plot is generated for each gene set in the list.
#'  If you only want to plot a subset of gene sets, subset the list before
#'  passing it to this function. If a character string starting with
#'  \code{"@"}, the attribute of \code{data} with that name (minus the
#'  \code{"@"}) is used.
#' @param gs Character vector of gene set \code{ID}s to plot. If \code{NULL}
#'  (default), all gene sets in \code{gene_sets} that appear in \code{data$ID}
#'  are plotted.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'  \code{patchwork} object. When \code{FALSE}, returns a named list of
#'  individual \code{patchwork} objects (one per gene set).
#' @param ncol,nrow Integer number of columns / rows for the combined layout
#'  (passed to \code{\link{combine_plots}()}).
#' @param byrow Logical; fill the combined layout by row. Default \code{TRUE}
#'  (passed to \code{\link{combine_plots}()}).
#' @param seed A numeric seed for reproducibility. Passed to
#'  \code{\link{validate_common_args}()}.
#' @param axes A character string specifying how axes should be treated across
#'  the combined layout (passed to \code{\link{combine_plots}()}).
#' @param axis_titles A character string specifying how axis titles should be
#'  treated across the combined layout. Defaults to \code{axes}.
#' @param guides A character string specifying how guides (legends) should be
#'  collected across panels (passed to \code{\link{combine_plots}()}).
#' @param design A custom layout design for the combined plot (passed to
#'  \code{\link{combine_plots}()}).
#' @export
#' @return A \code{patchwork} object when \code{combine = TRUE}, or a named
#'  list of \code{patchwork} objects when \code{combine = FALSE}. Each
#'  individual plot has \code{height} and \code{width} attributes in inches.
#' @examples
#' \donttest{
#' data(gsea_example)
#'
#' # Single gene set
#' GSEAPlot(gsea_example, gene_sets = attr(gsea_example, "gene_sets")[1])
#'
#' # Multiple gene sets arranged in a grid
#' GSEAPlot(gsea_example, gene_sets = attr(gsea_example, "gene_sets")[1:4])
#' }
GSEAPlot <- function(
    data,
    in_form = c("auto", "dose", "fgsea"),
    gene_ranks = "@gene_ranks",
    gene_sets = "@gene_sets",
    gs = NULL,
    sample_coregenes = FALSE,
    line_width = 1.5,
    line_alpha = 1,
    line_color = "#6BB82D",
    n_coregenes = 10,
    genes_label = NULL,
    label_fg = "black",
    label_bg = "white",
    label_bg_r = 0.1,
    label_size = 4,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    seed = 8525,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    set.seed(seed)
    in_form <- match.arg(in_form)
    if (inherits(data, "gseaResult")) {
        data <- as.data.frame(data)
    }
    if (
        is.character(gene_ranks) &&
            length(gene_ranks) == 1 &&
            startsWith(gene_ranks, "@")
    ) {
        gene_ranks <- attr(data, substring(gene_ranks, 2))
    }
    if (is.null(gene_ranks)) {
        stop("'gene_ranks' must be provided")
    }
    if (is.null(names(gene_ranks))) {
        stop("'gene_ranks' must have names")
    }
    if (!is.numeric(gene_ranks)) {
        stop("'gene_ranks' must be numeric")
    }
    gene_ranks <- gene_ranks[order(-gene_ranks)]

    if (
        is.character(gene_sets) &&
            length(gene_sets) == 1 &&
            startsWith(gene_sets, "@")
    ) {
        gene_sets <- attr(data, substring(gene_sets, 2))
    }
    if (is.null(gene_sets)) {
        stop("'gene_sets' must be provided")
    }
    if (!is.list(gene_sets)) {
        stop("'gene_sets' must be a list")
    }

    if (in_form == "auto") {
        if ("leadingEdge" %in% colnames(data)) {
            in_form <- "fgsea"
        } else if ("core_enrichment" %in% colnames(data)) {
            in_form <- "dose"
        } else {
            stop(
                "Cannot detect the input format. Please set 'in_form' to 'fgsea' or 'dose'."
            )
        }
    }
    if (in_form == "fgsea") {
        data <- prepare_fgsea_result(data)
    }
    gsnames <- intersect(as.character(data$ID), names(gene_sets))
    gene_sets <- gene_sets[gsnames]
    data <- data[as.character(data$ID) %in% gsnames, , drop = FALSE]
    theme <- process_theme(theme)
    gs <- gs %||% names(gene_sets)

    plots <- lapply(gs, function(g) {
        GSEAPlotAtomic(
            data,
            gene_ranks = gene_ranks,
            gs = g,
            genes = gene_sets[[g]],
            sample_coregenes = sample_coregenes,
            line_width = line_width,
            line_alpha = line_alpha,
            line_color = line_color,
            n_coregenes = n_coregenes,
            genes_label = genes_label,
            label_fg = label_fg,
            label_bg = label_bg,
            label_bg_r = label_bg_r,
            label_size = label_size,
            title = title,
            subtitle = subtitle,
            xlab = xlab,
            ylab = ylab,
            ...
        )
    })
    names(plots) <- gs

    combine_plots(
        plots,
        combine = combine,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

data("gsea_example", package = "plotthis")

test_that("GSEAPlot basic usage works", {
    p <- GSEAPlot(gsea_example)
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("GSEAPlot with title works", {
    gs <- gsea_example$ID[1]
    p <- GSEAPlot(gsea_example, gs = gs, title = "GSEA Title")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
})

test_that("GSEAPlot with specific gene sets works", {
    gs_ids <- gsea_example$ID[1:2]
    p <- GSEAPlot(gsea_example, gs = gs_ids, combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("GSEAPlot with combine = FALSE returns list", {
    gs_ids <- gsea_example$ID[1:3]
    plots <- GSEAPlot(gsea_example, gs = gs_ids, combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 3)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("GSEAPlot with n_coregenes works", {
    p <- GSEAPlot(gsea_example, gs = gsea_example$ID[1], n_coregenes = 5)
    expect_s3_class(p, "ggplot")
})

test_that("GSEASummaryPlot basic usage works", {
    p <- GSEASummaryPlot(gsea_example)
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("GSEASummaryPlot with title works", {
    p <- GSEASummaryPlot(gsea_example, title = "GSEA Summary Title")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "GSEA Summary Title")
})

test_that("GSEASummaryPlot with top_term works", {
    p <- GSEASummaryPlot(gsea_example, top_term = 5)
    expect_s3_class(p, "ggplot")
})

test_that("GSEASummaryPlot with line_by = 'running_score' works", {
    p <- GSEASummaryPlot(gsea_example, line_by = "running_score")
    expect_s3_class(p, "ggplot")
})

test_that("GSEASummaryPlot with in_form = 'dose' works", {
    p <- GSEASummaryPlot(gsea_example, in_form = "dose")
    expect_s3_class(p, "ggplot")
})

#' Detect the type of the input data for an UpSet plot
#'
#' Inspects the structure of \code{data} and returns a classifier string that
#' downstream functions use to select the parsing strategy.  When \code{data}
#' is an \code{UpsetPlotData} object (from \code{\link{prepare_upset_data}()})
#' it short-circuits to \code{"upset"} immediately.
#'
#' @keywords internal
#' @param data A data frame or a named list of element vectors.
#' @param group_by A character string specifying the column name for the set-definition groups.
#'   Only meaningful when \code{data} is a data frame.
#' @param id_by A character string specifying the column name for instance identifiers.
#'   Required when \code{group_by} is a single column and \code{data} is a data frame.
#' @return A character string, one of \code{"long"}, \code{"wide"}, \code{"list"},
#'   or \code{"upset"}.
#'   \describe{
#'     \item{\code{"long"}}{Long-format data --- each row records one (set, element) pair.}
#'     \item{\code{"wide"}}{Wide-format data --- each row is an element and each set has its own logical/0-1 column.}
#'     \item{\code{"list"}}{A named list of element vectors.}
#'     \item{\code{"upset"}}{Already an \code{UpsetPlotData} object.}
#'   }
detect_upset_datatype <- function(data, group_by = NULL, id_by = NULL) {
    if (inherits(data, "UpsetPlotData")) {
        return("upset")
    }

    if (inherits(data, "data.frame")) {
        if (length(group_by) < 2 && !is.null(group_by)) {
            return("long")
        } else {
            # length(group_by) > 1
            return("wide")
        }
    }

    if (inherits(data, "list")) {
        return("list")
    }

    stop(
        "Invalid data type. Please provide a data frame, a list or an UpsetPlotData object generated by prepare_upset_data()."
    )
}


#' Prepare data for an UpSet plot
#'
#' Converts raw input (long data frame, wide data frame, named list, or an
#' existing \code{UpsetPlotData} object) into the internal format expected by
#' \code{\link{UpsetPlotAtomic}()}.  This is the data-preparation workhorse
#' that handles all input format parsing.
#'
#' @section Input formats:
#'
#' \strong{Long format} --- one row per (set, element) pair:
#' \preformatted{
#' group_by id_by
#' A        a1
#' A        a2
#' B        a1
#' B        a3
#' }
#' Requires both \code{group_by} and \code{id_by}.
#'
#' \strong{Wide format} --- each row is an element, each set has a logical or
#' 0/1 membership column:
#' \preformatted{
#' A      B
#' TRUE   TRUE
#' TRUE   FALSE
#' FALSE  TRUE
#' }
#' The set columns are identified by \code{group_by} (or all columns except
#' \code{id_by} when \code{group_by = NULL}).
#'
#' \strong{List format} --- a named list of element vectors:
#' \preformatted{
#' list(A = c("a1", "a2"), B = c("a1", "a3"))
#' }
#'
#' \strong{UpsetPlotData} --- a pre-processed object (returned by this function)
#' is returned unchanged (with a warning if \code{group_by} is also provided).
#'
#' @param data A data frame, a named list of element vectors, or an
#'   \code{UpsetPlotData} object.
#' @param in_form A character string specifying the input format.  One of
#'   \code{"auto"} (default; detect from \code{data} structure),
#'   \code{"long"}, \code{"wide"}, \code{"list"}, or \code{"upset"}.
#' @param group_by A character vector of column name(s) defining the sets.
#'   In long format, this is the column with set labels.  In wide format,
#'   these are the set-membership columns.  When \code{NULL} in wide format,
#'   all columns except \code{id_by} are used.  Ignored for list and upset
#'   inputs.
#' @param group_by_sep A character string to concatenate multiple
#'   \code{group_by} columns when \code{in_form = "long"}.  Default: \code{"_"}.
#' @param id_by A character string specifying the column name for instance
#'   identifiers.  Required for long format; optional for wide format (a
#'   synthetic \code{.id} column is created if omitted).
#' @param specific A logical value.  When \code{TRUE} (default), only specific
#'   intersections are returned (elements belonging exclusively to the shown
#'   set combination).  When \code{FALSE}, all overlapping items are included.
#'   See \url{https://github.com/gaospecial/ggVennDiagram/issues/64}.
#' @return An \code{UpsetPlotData} object --- a data frame with an
#'   \code{Intersection} list-column of set labels and one row per element.
#'   A \code{"group_order"} attribute preserves the original set order.
#' @keywords internal
#' @importFrom rlang sym
#' @importFrom dplyr distinct %>%
#' @importFrom utils getFromNamespace
#' @importFrom tidyr uncount
prepare_upset_data <- function(
    data,
    in_form = "auto",
    group_by = NULL,
    group_by_sep = "_",
    id_by = NULL,
    specific = TRUE
) {
    if (in_form == "auto") {
        in_form <- detect_upset_datatype(data, group_by, id_by)
    }

    if (in_form == "upset") {
        if (!is.null(group_by)) {
            warning(
                "The group_by argument is ignored when the input data is an UpsetPlotData object.",
                immediate. = TRUE
            )
        }
        return(data)
    }

    process_upset_data <- getFromNamespace(
        "process_upset_data",
        "ggVennDiagram"
    )
    if (in_form == "list") {
        listdata <- data
        group_order <- names(listdata)
    } else if (in_form == "long") {
        group_by <- check_columns(
            data,
            group_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = group_by_sep
        )
        group_order <- levels(data[[group_by]])
        listdata <- split(data[[id_by]], data[[group_by]])
    } else {
        # in_form == "wide"
        group_by <- check_columns(data, group_by, allow_multi = TRUE)
        if (is.null(group_by)) {
            group_by <- setdiff(colnames(data), id_by)
        }
        group_order <- group_by
        for (g in group_by) {
            # columns must be logical or 0/1
            if (!is.logical(data[[g]]) && !all(data[[g]] %in% c(0, 1))) {
                stop(
                    "[UpSetPlot] The columns in group_by must be logical or 0/1 when the in_form is 'wide', but column '",
                    g,
                    "' is not."
                )
            }
        }
        id_by <- check_columns(data, id_by)
        if (is.null(id_by)) {
            id_by <- ".id"
            data$.id <- paste0("id", seq_len(nrow(data)))
        } else {
            # check if id_by column values are unique
            if (any(duplicated(data[[id_by]]))) {
                stop(
                    "[UpSetPlot] The values in id_by column must be unique when the in_form is 'wide', but there are duplicated values in column '",
                    id_by,
                    "'."
                )
            }
        }
        listdata <- lapply(group_by, function(g) {
            data[as.logical(data[[g]]), id_by, drop = TRUE]
        })
        names(listdata) <- group_by
        for (nm in names(listdata)) {
            if (length(listdata[[nm]]) == 0) {
                warning(
                    "The group '",
                    nm,
                    "' has no elements, ignored.",
                    immediate. = TRUE
                )
                listdata[[nm]] <- NULL
            }
        }
    }

    data <- process_upset_data(
        ggVennDiagram::Venn(listdata),
        specific = specific
    )
    idnames <- paste0(
        as.character(data$left_data$set),
        " (",
        as.character(data$left_data$size),
        ")"
    )
    names(idnames) <- as.character(seq_along(idnames))

    sep <- ifelse(specific, "/", "~")
    data <- distinct(data$main_data, !!sym("id"), !!sym("size"))

    data$Intersection <- lapply(as.character(data$id), function(x) {
        idnames[strsplit(x, sep, fixed = TRUE)[[1]]]
    })

    out <- uncount(data, !!sym("size"))
    attr(out, "group_order") <- group_order
    out
}

#' Atomic UpSet plot (internal)
#'
#' @description
#' Core implementation for drawing a single UpSet plot.  This is the internal
#' workhorse behind the exported \code{\link{UpsetPlot}} function --- it takes a
#' **single** data partition (no \code{split_by} support) and returns a
#' \code{ggplot} object.  The plot comprises three visual components:
#' \itemize{
#'   \item A \strong{horizontal bar chart} (top) showing the size of each
#'         intersection, with bars filled by a gradient of intersection count.
#'   \item A \strong{set-size bar chart} (left) showing the total number of
#'         elements in each set (added automatically by \code{ggupset}).
#'   \item A \strong{combination matrix} (bottom) where rows are sets, columns
#'         are intersections, and dots indicate set membership.  Connected dots
#'         trace the intersection pattern.
#' }
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{ggplot dispatch} --- selects \code{gglogger::ggplot} or
#'         \code{ggplot2::ggplot} based on
#'         \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{Data preparation} --- \code{\link{prepare_upset_data}()}
#'         converts the input into the internal \code{UpsetPlotData} format
#'         with an \code{Intersection} list-column (set-membership labels per
#'         element) and a \code{"group_order"} attribute for set ordering.
#'   \item \strong{Text scale} --- \code{base_size} from \code{theme_args}
#'         (default 12) drives a \code{text_size_scale} that proportionally
#'         scales the combination-matrix row gap, label sizes, and point sizes.
#'   \item \strong{Bar geom} --- \code{geom_bar()} counts rows per intersection.
#'         Bars are filled by count via \code{scale_fill_gradientn()} with
#'         colours from \code{\link{palette_this}()}.
#'   \item \strong{Bar labels} --- when \code{label = TRUE},
#'         \code{\link[ggrepel]{geom_text_repel}()} places count labels above
#'         each bar, with configurable foreground, background, and size.
#'   \item \strong{Scale x upset} --- \code{ggupset::scale_x_upset(...)} is
#'         called, returning two components: a \code{ScaleUpset} (index \code{[[1]]})
#'         and a \code{CoordCombMatrix} (index \code{[[2]]}).  Only the scale is
#'         applied; the bundled coord is discarded to avoid a "Coordinate
#'         system already present" conflict.
#'   \item \strong{Labels and theme} --- \code{labs()} sets title, subtitle,
#'         and axis text.  \code{do_call(theme, theme_args)} applies the chosen
#'         theme.  Aspect ratio, legend position, legend direction, and grid
#'         lines are configured.
#'   \item \strong{Combination matrix} --- \code{ggupset::axis_combmatrix()}
#'         builds the lower panel via an \code{override_plotting_function}:
#'         \itemize{
#'           \item Reorders rows to match \code{group_order} (reversed for
#'                 top-to-bottom set listing).
#'           \item Alternating row backgrounds via \code{geom_rect()} with
#'                 white / \code{#F7F7F7} fills.
#'           \item Set-membership dots (\code{geom_point()}) in black for
#'                 present and \code{#E0E0E0} for absent.
#'           \item Connecting lines (\code{geom_line()}) through observed
#'                 dots to trace the intersection pattern.
#'         }
#'   \item \strong{Matrix theme} --- \code{ggupset::theme_combmatrix()}
#'         applies the combination-matrix appearance with
#'         \code{combmatrix_gap} between rows (scaled by
#'         \code{text_size_scale}).
#'   \item \strong{Dimension calculation} ---
#'         \code{\link{calculate_plot_dimensions}()} computes height from
#'         \code{n_sets} and width from \code{n_intersections}, both passed
#'         via \code{...} (default 99).  \code{aspect.ratio} is used as a
#'         standard H/W coupling with \code{x_scale_factor = 0.6}.  An
#'         additional width adjustment accounts for set-label character width.
#' }
#'
#' @inheritParams common_args
#' @inheritParams prepare_upset_data
#' @param label A logical value.  When \code{TRUE} (default), count labels are
#'   displayed above each intersection bar via \code{\link[ggrepel]{geom_text_repel}()}.
#' @param label_fg A character string specifying the colour of the label text.
#'   Default: \code{"black"}.
#' @param label_size A numeric value specifying the size of the label text.
#'   Default: \code{NULL} (computed from \code{base_size / 12 * 3.5}).
#' @param label_bg A character string specifying the background fill colour of
#'   the label.  Default: \code{"white"}.
#' @param label_bg_r A numeric value specifying the corner radius of the label
#'   background, passed to \code{geom_text_repel(bg.r)}.  Default: \code{0.1}.
#' @param combmatrix_gap A numeric value specifying the gap between rows of the
#'   combination matrix, measured at \code{base_size = 12}.  The actual gap is
#'   scaled by \code{text_size_scale = base_size / 12}.  Default: \code{6}.
#' @param levels A character vector of set levels to include, in display order.
#'   When \code{NULL} (default), all detected sets are shown.  Accepted for
#'   compatibility but not currently used by the internal implementation.
#' @param ... Additional arguments passed to \code{\link[ggupset]{scale_x_upset}}.
#'   Useful options include \code{n_sets} and \code{n_intersections} to limit
#'   the number of shown sets / intersections.
#' @return A \code{ggplot} object with \code{height} and \code{width} attributes
#'   (in inches).
#' @keywords internal
#' @importFrom rlang %||% sym
#' @importFrom ggplot2 geom_bar labs guide_colorbar scale_fill_gradientn
#' @importFrom ggplot2 aes geom_rect geom_point geom_line scale_x_continuous expansion
#' @importFrom ggplot2 scale_fill_manual scale_color_manual guides element_blank unit
#' @importFrom ggrepel geom_text_repel
UpsetPlotAtomic <- function(
    data,
    in_form = "auto",
    group_by = NULL,
    group_by_sep = "_",
    id_by = NULL,
    label = TRUE,
    label_fg = "black",
    label_size = NULL,
    label_bg = "white",
    label_bg_r = 0.1,
    palette = "material-indigo",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    specific = TRUE,
    combmatrix_gap = 6,
    theme = "theme_this",
    theme_args = list(),
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    aspect.ratio = 0.6,
    legend.position = "right",
    legend.direction = "vertical",
    levels = NULL,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }

    data <- prepare_upset_data(
        data,
        in_form,
        group_by,
        group_by_sep,
        id_by,
        specific
    )
    group_order <- attr(data, "group_order")

    base_size <- theme_args$base_size %||% 12
    text_size_scale <- base_size / 12

    p <- ggplot(data, aes(x = !!sym("Intersection"))) +
        geom_bar(
            aes(fill = after_stat(!!sym("count"))),
            alpha = alpha,
            color = "black",
            width = 0.5
        ) +
        scale_fill_gradientn(
            n.breaks = 3,
            colors = palette_this(
                palette = palette,
                palcolor = palcolor,
                reverse = palreverse
            ),
            na.value = "grey80",
            guide = guide_colorbar(
                title = "",
                alpha = alpha,
                frame.colour = "black",
                ticks.colour = "black",
                title.hjust = 0
            )
        )
    if (isTRUE(label)) {
        p <- p +
            geom_text_repel(
                aes(label = after_stat(!!sym("count"))),
                stat = "count",
                colour = label_fg,
                size = label_size %||% text_size_scale * 3.5,
                bg.color = label_bg,
                bg.r = label_bg_r,
                nudge_y = 0.08 * text_size_scale,
                point.size = NA,
                max.overlaps = 100,
                force = 0,
                min.segment.length = 0,
                segment.colour = NA
            )
    }
    upset_args <- list(...)
    ytrans <- upset_args$ytrans %||% "identity"

    # scale_x_upset() internally calls axis_combmatrix(), bundling a CoordCombMatrix into its
    # return list. We extract only the ScaleUpset (index [[1]]) and discard the bundled
    # CoordCombMatrix (index [[2]]), then add our own axis_combmatrix() once — avoiding
    # a second coord system replacement and its "Coordinate system already present" message.
    scale_x_upset_res <- ggupset::scale_x_upset(...)
    p <- p +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% "",
            y = ylab %||% "Intersection size"
        ) +
        scale_x_upset_res[[1]] +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major = element_line(colour = "grey80", linetype = 2)
        ) +
        # try to keep the group order in combmatrix
        ggupset::axis_combmatrix(
            sep = " // ",
            ytrans = ytrans,
            override_plotting_function = function(df) {
                # df$single_label <- factor(df$single_label, levels = group_order)
                # single_label is something like: 'C (5)''B (5)''D (5)''A (5)'
                # but group_order is something like: 'A', 'B', 'C', 'D'
                # so we need to extract the group name from single_label and match it with group_order
                df$group_name <- sapply(
                    as.character(df$single_label),
                    function(x) {
                        # extract the group name from single_label, which is the part before the first " ("
                        strsplit(x, " (", fixed = TRUE)[[1]][1]
                    }
                )
                df$group_name <- factor(
                    df$group_name,
                    levels = rev(group_order)
                )
                df <- df[order(df$group_name), , drop = FALSE]
                df$single_label <- factor(
                    df$single_label,
                    levels = unique(df$single_label)
                )
                ggplot(df, aes(x = !!sym("at"), y = !!sym("single_label"))) +
                    geom_rect(
                        aes(fill = !!sym("index") %% 2 == 0),
                        ymin = df$index - 0.5,
                        ymax = df$index + 0.5,
                        xmin = 0,
                        xmax = 1
                    ) +
                    geom_point(
                        aes(color = !!sym("observed")),
                        size = text_size_scale * 3
                    ) +
                    geom_line(
                        data = function(dat) dat[dat$observed, , drop = FALSE],
                        aes(group = !!sym("labels")),
                        linewidth = 1.2 * text_size_scale
                    ) +
                    ggplot2::ylab("") +
                    ggplot2::xlab("") +
                    scale_x_continuous(limits = c(0, 1), expand = c(0, 0)) +
                    scale_y_discrete(expand = c(0, 0)) +
                    scale_fill_manual(
                        values = c(`TRUE` = "white", `FALSE` = "#F7F7F7")
                    ) +
                    scale_color_manual(
                        values = c(`TRUE` = "black", `FALSE` = "#E0E0E0")
                    ) +
                    guides(color = "none", fill = "none") +
                    ggplot2::theme(
                        panel.background = element_blank(),
                        axis.text.x = element_blank(),
                        axis.text.y = element_text(
                            size = 12 * text_size_scale^0.7,
                            color = "black"
                        ),
                        axis.ticks.y = element_blank(),
                        axis.ticks.length = unit(0, "pt"),
                        axis.title.y = element_blank(),
                        axis.title.x = element_blank(),
                        axis.line = element_blank(),
                        panel.border = element_blank()
                    )
            }
        ) +
        ggupset::theme_combmatrix(
            combmatrix.label.extra_spacing = combmatrix_gap * text_size_scale
        )

    n_sets <- upset_args$n_sets %||% 99
    n_sets <- min(n_sets, length(unique(unlist(data$Intersection))))
    n_intersections <- upset_args$n_intersections %||% 99
    n_intersections <- min(n_intersections, length(unique(data$Intersection)))
    maxchars <- max(sapply(unique(unlist(data$Intersection)), nchar))

    # Height driven by number of sets; width driven by number of intersections.
    # x_scale_factor = 0.6 preserves the original per-intersection width (the original
    # default aspect.ratio = 0.6 was used as a width-per-intersection scale, not H/W ratio).
    # aspect.ratio is now used as the standard H/W ratio coupling.
    dims <- calculate_plot_dimensions(
        base_height = 4.5 + n_sets * 0.5,
        aspect.ratio = aspect.ratio,
        n_x = n_intersections,
        x_scale_factor = 0.6,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = n_sets,
        legend_nchar = maxchars
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width + maxchars * 0.05

    p
}

#' UpSet Plot
#'
#' @description
#' Draws an UpSet plot visualising set intersections and set sizes.  The plot
#' comprises:
#' \itemize{
#'   \item A \strong{horizontal bar chart} showing the size of each
#'         intersection, filled by the intersection count.
#'   \item A \strong{combination matrix} (rows = sets, columns = intersections)
#'         with membership dots and connecting lines.
#'   \item A \strong{set-size bar chart} on the left of the matrix (added
#'         automatically by \code{ggupset}).
#' }
#'
#' The function accepts data in four formats:
#' \itemize{
#'   \item \strong{List} --- a named list of element vectors (one per set).
#'   \item \strong{Long} --- a data frame with one row per (set, element) pair.
#'   \item \strong{Wide} --- a data frame where each row is an element and each
#'         set has its own logical or 0/1 membership column.
#'   \item \strong{UpsetPlotData} --- a pre-processed object from
#'         \code{\link{prepare_upset_data}()}.
#' }
#'
#' Supports splitting into sub-plots via \code{split_by}, per-split colour
#' palettes and legend control, and combining sub-plots via \code{patchwork}.
#'
#' @section split_by Workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \strong{Guard} --- \code{split_by} is only supported for
#'         \code{data.frame} input.  If \code{data} is a list (or other
#'         non-data.frame type) and \code{split_by} is non-\code{NULL}, an
#'         error is raised.
#'   \item \strong{Column validation} --- \code{\link{check_columns}()} resolves
#'         the \code{split_by} column(s) with \code{force_factor = TRUE} and
#'         \code{allow_multi = TRUE}.  Multiple columns are concatenated with
#'         \code{split_by_sep}.
#'   \item \strong{Data splitting} --- empty factor levels in \code{split_by}
#'         are dropped via \code{\link[base]{droplevels}()}, then the data
#'         frame is split by \code{split_by} (level order is preserved).  If
#'         \code{split_by} is \code{NULL}, the data is wrapped in a
#'         single-element list with name \code{"..."}.
#'   \item \strong{Per-split resolution} --- \code{\link{check_palette}()},
#'         \code{\link{check_palcolor}()}, and \code{\link{check_legend}()}
#'         resolve per-split \code{palette}, \code{palcolor},
#'         \code{legend.position}, and \code{legend.direction}.
#'   \item \strong{Atomic dispatch} --- \code{UpsetPlotAtomic} is called for
#'         each split.  If \code{title} is a function, it receives the split
#'         level name and can generate dynamic titles.  When in wide mode
#'         (\code{in_form} is \code{"auto"} or \code{"wide"}) and
#'         \code{group_by} is \code{NULL}, the set columns are auto-detected
#'         as all columns except \code{id_by} and \code{split_by}.
#'   \item \strong{Combination} --- Results are combined via
#'         \code{\link{combine_plots}()} (when \code{combine = TRUE}) or
#'         returned as a named list.
#' }
#'
#' @inheritParams common_args
#' @inheritParams UpsetPlotAtomic
#' @param split_by The column(s) to split the data by and produce separate
#'   sub-plots.  Only supported for \code{data.frame} input (list input raises
#'   an error).  Multiple columns concatenated with \code{split_by_sep}.
#' @param split_by_sep A character string to separate concatenated
#'   \code{split_by} columns.  Default: \code{"_"}.
#' @param seed A numeric seed for reproducibility.  Passed to
#'   \code{\link{validate_common_args}()}.  Default: \code{8525}.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'   \code{patchwork} object.  When \code{FALSE}, returns a named list of
#'   individual \code{ggplot} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined layout
#'   (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param byrow Logical; fill the combined layout by row.  Default \code{TRUE}
#'   (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axes A character string specifying how axes should be treated across
#'   the combined layout (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axis_titles A character string specifying how axis titles should be
#'   treated across the combined layout.  Defaults to \code{axes}.
#' @param guides A character string specifying how legends should be collected
#'   across panels (passed to \code{\link{combine_plots}()}).
#' @param design A custom layout design for the combined plot (passed to
#'   \code{\link{combine_plots}()}).
#' @return A \code{ggplot} object, a \code{patchwork} object, or a named list
#'   of \code{ggplot} objects (when \code{combine = FALSE}), each with
#'   \code{height} and \code{width} attributes in inches.
#' @export
#' @examples
#' \donttest{
#' # ---- list input -------------------------------------------------------
#' data <- list(
#'     A = 1:5,
#'     B = 2:6,
#'     C = 3:7,
#'     D = 4:8
#' )
#' UpsetPlot(data)
#' UpsetPlot(data, label = FALSE)
#' UpsetPlot(data, palette = "Reds", specific = FALSE)
#'
#' # ---- long-format data frame ------------------------------------------
#' data_long <- data.frame(
#'     group_by = factor(
#'          c(rep("A", 5), rep("B", 5), rep("C", 5), rep("D", 5)),
#'          levels = c("A", "B", "C", "D")
#'     ),
#'     id_by = c(1:5, 2:6, 3:7, 4:8)
#' )
#' UpsetPlot(data_long, in_form = "long", group_by = "group_by", id_by = "id_by")
#'
#' # ---- wide-format data frame ------------------------------------------
#' data <- data.frame(
#'     id = LETTERS[1:10],
#'     B = c(1, 0, 1, 1, 0, 0, 1, 0, 1, 0),
#'     A = c(1, 1, 1, 0, 0, 1, 0, 0, 1, 0),
#'     D = c(1, 0, 0, 1, 1, 0, 0, 1, 0, 1),
#'     C = c(0, 1, 1, 0, 1, 0, 1, 0, 1, 0)
#' )
#' UpsetPlot(data, in_form = "wide", id_by = "id", n_intersections = 4)
#' }
UpsetPlot <- function(
    data,
    in_form = c("auto", "long", "wide", "list", "upset"),
    split_by = NULL,
    split_by_sep = "_",
    group_by = NULL,
    group_by_sep = "_",
    id_by = NULL,
    label = TRUE,
    label_fg = "black",
    label_size = NULL,
    label_bg = "white",
    label_bg_r = 0.1,
    palette = "Blues",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    specific = TRUE,
    theme = "theme_this",
    theme_args = list(),
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    aspect.ratio = 0.6,
    legend.position = "right",
    legend.direction = "vertical",
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    seed = 8525,
    combmatrix_gap = 6,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed)
    in_form <- match.arg(in_form)
    theme <- process_theme(theme)
    if (!is.null(split_by) && !inherits(data, "data.frame")) {
        stop("'split_by' is only available for data frame.")
    }
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            if (is.null(group_by) && (in_form %in% c("auto", "wide"))) {
                group_by <- setdiff(colnames(datas[[nm]]), c(id_by, split_by))
            }
            UpsetPlotAtomic(
                datas[[nm]],
                in_form = in_form,
                group_by = group_by,
                group_by_sep = group_by_sep,
                id_by = id_by,
                combmatrix_gap = combmatrix_gap,
                label = label,
                label_fg = label_fg,
                label_size = label_size,
                label_bg = label_bg,
                label_bg_r = label_bg_r,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                alpha = alpha,
                specific = specific,
                theme = theme,
                theme_args = theme_args,
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
# UpsetPlot accepts list input or data.frame (long/wide)
upset_list <- list(
    SetA = c("gene1", "gene2", "gene3", "gene4", "gene5", "gene6"),
    SetB = c("gene3", "gene4", "gene5", "gene7", "gene8", "gene9"),
    SetC = c("gene2", "gene4", "gene6", "gene8", "gene10", "gene11"),
    SetD = c("gene1", "gene5", "gene7", "gene9", "gene11", "gene12")
)

# Long-format for upset
upset_long <- data.frame(
    id    = c("g1", "g2", "g3", "g4", "g5", "g3", "g4", "g5", "g6", "g7"),
    group = c(rep("SetA", 5), rep("SetB", 5))
)

test_that("UpsetPlot with list input works", {
    skip_if_not_installed("ggupset")
    p <- UpsetPlot(upset_list)
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("UpsetPlot with title works", {
    skip_if_not_installed("ggupset")
    p <- UpsetPlot(upset_list, title = "Upset Title")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "Upset Title")
})

test_that("UpsetPlot with long data frame input works", {
    skip_if_not_installed("ggupset")
    p <- UpsetPlot(upset_long, in_form = "long", id_by = "id", group_by = "group")
    expect_s3_class(p, "ggplot")
})

test_that("UpsetPlot with label works", {
    skip_if_not_installed("ggupset")
    p <- UpsetPlot(upset_list, label = "count")
    expect_s3_class(p, "ggplot")
})

test_that("UpsetPlot with split_by returns patchwork", {
    skip_if_not_installed("ggupset")
    upset_split <- data.frame(
        id    = c("g1","g2","g3","g4","g5","g3","g4","g5","g6","g7",
                  "g1","g2","g8","g9","g3","g8","g9","g3","g10","g11"),
        group = c(rep("SetA", 5), rep("SetB", 5), rep("SetA", 5), rep("SetB", 5)),
        split = factor(rep(c("S1", "S2"), each = 10))
    )
    p <- UpsetPlot(upset_split, in_form = "long", id_by = "id",
                   group_by = "group", split_by = "split", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("UpsetPlot with combine = FALSE returns list", {
    skip_if_not_installed("ggupset")
    upset_split <- data.frame(
        id    = c("g1","g2","g3","g4","g5","g3","g4","g5","g6","g7",
                  "g1","g2","g8","g9","g3","g8","g9","g3","g10","g11"),
        group = c(rep("SetA", 5), rep("SetB", 5), rep("SetA", 5), rep("SetB", 5)),
        split = factor(rep(c("S1", "S2"), each = 10))
    )
    plots <- UpsetPlot(upset_split, in_form = "long", id_by = "id",
                       group_by = "group", split_by = "split", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("UpsetPlot with multiple split_by columns returns a list with one plot per combination", {
    skip_if_not_installed("ggupset")
    data2 <- data.frame(
        id    = c("g1","g2","g3","g4","g5","g3","g4","g5","g6","g7",
                  "g1","g2","g8","g9","g3","g8","g9","g3","g10","g11"),
        group = c(rep("SetA", 5), rep("SetB", 5), rep("SetA", 5), rep("SetB", 5)),
        split = factor(rep(c("S1", "S2"), each = 10))
    )
    data2$split2 <- factor(rep(c("s1", "s2"), 10))
    plots <- suppressMessages(UpsetPlot(data2, in_form = "long", id_by = "id",
                                        group_by = "group",
                                        split_by = c("split", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("UpsetPlot with multiple split_by columns returns combined plot", {
    skip_if_not_installed("ggupset")
    data2 <- data.frame(
        id    = c("g1","g2","g3","g4","g5","g3","g4","g5","g6","g7",
                  "g1","g2","g8","g9","g3","g8","g9","g3","g10","g11"),
        group = c(rep("SetA", 5), rep("SetB", 5), rep("SetA", 5), rep("SetB", 5)),
        split = factor(rep(c("S1", "S2"), each = 10))
    )
    data2$split2 <- factor(rep(c("s1", "s2"), 10))
    p <- suppressMessages(UpsetPlot(data2, in_form = "long", id_by = "id",
                                    group_by = "group",
                                    split_by = c("split", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

set.seed(8525)
# TrendPlot expects: x = factor column for x-axis steps, group_by = fill group
trend_data <- data.frame(
    timepoint = factor(rep(c("T1", "T2", "T3", "T4"), times = 60),
                       levels = c("T1", "T2", "T3", "T4")),
    cell_type = factor(rep(rep(c("TypeA", "TypeB", "TypeC"), each = 4), 20)),
    sample = factor(rep(c("S1", "S2"), each = 120))
)

test_that("TrendPlot basic usage works", {
    p <- TrendPlot(trend_data, x = "timepoint")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("TrendPlot with group_by works", {
    p <- TrendPlot(trend_data, x = "timepoint", group_by = "cell_type")
    expect_s3_class(p, "ggplot")
})

test_that("TrendPlot with title and subtitle works", {
    p <- TrendPlot(trend_data, x = "timepoint", group_by = "cell_type",
                   title = "Trend Title", subtitle = "Trend Subtitle")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "Trend Title")
    expect_equal(p$labels$subtitle, "Trend Subtitle")
})

test_that("TrendPlot with split_by returns patchwork", {
    p <- TrendPlot(trend_data, x = "timepoint", group_by = "cell_type",
                   split_by = "sample", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("TrendPlot with combine = FALSE returns list", {
    plots <- TrendPlot(trend_data, x = "timepoint", group_by = "cell_type",
                       split_by = "sample", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("TrendPlot with multiple split_by columns returns a list with one plot per combination", {
    plots <- suppressMessages(TrendPlot(trend_data, x = "timepoint", group_by = "cell_type",
                                        split_by = c("sample", "cell_type"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 6)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("TrendPlot with multiple split_by columns returns combined plot", {
    p <- suppressMessages(TrendPlot(trend_data, x = "timepoint", group_by = "cell_type",
                                    split_by = c("sample", "cell_type"), combine = TRUE))
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("TrendPlot with multiple split_by columns and keep_na does not re-process split columns", {
    w <- capture_warnings(
        p <- suppressMessages(TrendPlot(trend_data, x = "timepoint", group_by = "cell_type",
                                        split_by = c("sample", "cell_type"),
                                        keep_na = TRUE, combine = TRUE))
    )
    # The original split columns are removed from keep_na before the atomic
    # summarizes the data (which drops 'sample'), so it must not warn.
    expect_false(any(grepl("not found", w)))
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("TrendPlot with scale_y = TRUE works", {
    p <- TrendPlot(trend_data, x = "timepoint", group_by = "cell_type", scale_y = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("TrendPlot with facet_by works", {
    p <- TrendPlot(trend_data, x = "timepoint", group_by = "cell_type",
                   facet_by = "sample")
    expect_s3_class(p, "ggplot")
})

test_that("TrendPlot with x_text_angle works", {
    p <- TrendPlot(trend_data, x = "timepoint", group_by = "cell_type",
                   x_text_angle = 45)
    expect_s3_class(p, "ggplot")
})

#' Atomic Manhattan plot (internal)
#'
#' @description
#' Core implementation for drawing a GWAS-style Manhattan plot.  This is the
#' internal workhorse dispatched by the exported \code{\link{ManhattanPlot}}
#' function --- it takes a **single** data frame (no \code{split_by} support)
#' and returns a \code{ggplot} object.  The plot displays genetic association
#' p-values across chromosomes, with \eqn{-\log_{10}(p)} on the y-axis and
#' genomic position on the x-axis.  Each chromosome is rendered in alternating
#' colours, and configurable horizontal dashed lines mark genome-wide
#' significance thresholds.
#'
#' The function is adapted from \code{ggmanh::manhattan_plot()} with the
#' following enhancements:
#' \itemize{
#'   \item Dot-separated argument names are converted to underscores
#'   (e.g. \code{chr.colname} \eqn{\rightarrow} \code{chr_by}).
#'   \item \code{chromosomes} merges the original \code{chromosome} and
#'   \code{chr.order} arguments into a single parameter for subsetting
#'   and reordering.
#'   \item \code{highlight} accepts index vectors or R expressions (via a
#'   character string) instead of a column name.
#'   \item Dedicated \code{pt_*}, \code{label_*}, and \code{highlight_*}
#'   parameter families give granular control over point appearance,
#'   label styling, and highlight styling.
#'   \item \code{pval_transform} accepts any function (or a character
#'   string parsed as a function) rather than a fixed log-transform toggle.
#' }
#'
#' Key features include per-chromosome alternating colours, configurable
#' significance threshold lines with labels, optional data thinning for
#' dense SNP sets, automatic y-axis rescaling (broken-axis) when a small
#' number of highly significant points would otherwise compress the
#' majority of the data, and support for highlighting and labeling
#' specific variants.
#'
#' @section Architecture:
#'
#' \strong{ManhattanPlotAtomic} executes the following steps:
#' \enumerate{
#'   \item \strong{ggplot dispatch} --- selects \code{gglogger::ggplot} or
#'   \code{ggplot2::ggplot} based on
#'   \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{signif_label_pos normalisation} --- \code{match.arg()}
#'   resolves \code{signif_label_pos} to \code{"left"} or \code{"right"}.
#'   \item \strong{Data preprocessing} ---
#'   \code{ggmanh::manhattan_data_preprocess()} performs chromosome
#'   ordering, optional thinning (\code{thin_n} points per
#'   \code{thin_bins} horizontal partitions), chromosome gap scaling
#'   (\code{chr_gap_scaling}), position preservation
#'   (\code{preserve_position}), and significance threshold colour
#'   assignment.  When \code{chromosomes} is a single value, it is
#'   passed as the \code{chromosome} filter; otherwise it is used for
#'   ordering via \code{chr.order}.
#'   \item \strong{label_by validation} --- \code{\link{check_columns}()}
#'   validates and factors the \code{label_by} column if provided.
#'   \item \strong{Chromosome colour assignment} --- if \code{highlight}
#'   and \code{highlight_color} are both set, or if \code{pt_color} is
#'   given, \code{pt_color} (defaulting to \code{"grey80"}) is used as
#'   the base colour for all chromosomes; otherwise
#'   \code{\link{palette_this}()} generates distinct colours per
#'   chromosome from \code{palette} / \code{palcolor}.
#'   \item \strong{pval_transform resolution} --- when a character string
#'   starting with \code{"-"} (e.g. \code{"-log10"}), the minus sign is
#'   stripped, the remainder is evaluated as a function, and the result
#'   is negated.  Plain character strings are evaluated directly.  The
#'   result is applied to the p-value column to produce \code{log10pval}.
#'   \item \strong{Y-axis rescaling check} --- when \code{rescale = TRUE},
#'   the ratio of the ceiling-scaled maximum to the significance-threshold
#'   jump is checked against \code{rescale_ratio_threshold}.  If the
#'   ratio exceeds the threshold, a broken y-axis is constructed via
#'   \code{ggmanh::get_transform()} that compresses the empty space
#'   between the main data and the extreme points.
#'   \item \strong{Single-chromosome branch} --- when only one chromosome
#'   is present, the original position column is used directly, x-axis
#'   breaks and labels use \code{waiver()}, and the x-axis label is set
#'   to the chromosome name (or \code{"Chromosome <name>"}).
#'   \item \strong{Multi-chromosome branch} --- position coordinates are
#'   recalculated across chromosome boundaries via
#'   \code{ggmanh::calc_new_pos_()}.  Chromosome centre positions serve
#'   as x-axis breaks, chromosome labels are displayed, and limits span
#'   the full genomic range.
#'   \item \strong{Base plot assembly} --- creates a \code{ggplot} object
#'   with \code{geom_point()} (chromosome-coloured dots),
#'   \code{scale_color_manual()}, \code{scale_y_continuous()} (with
#'   optional broken-axis transform), \code{scale_x_continuous()},
#'   \code{geom_hline()} for significance thresholds, the resolved theme
#'   (with hidden grid lines and suppressed legend), and \code{labs()}.
#'   \item \strong{Significance labels} --- when \code{signif_label = TRUE},
#'   \code{geom_text()} annotates each significance threshold at the
#'   left or right edge of the plot (controlled by
#'   \code{signif_label_pos}), coloured by \code{signif_color} (smallest
#'   threshold in black, others in grey by default).
#'   \item \strong{Variant labels} --- when \code{label_by} is provided,
#'   \code{ggrepel::geom_label_repel()} adds labels for variants that
#'   have non-empty values in the \code{label_by} column.  If \code{label_fg}
#'   is set, all labels receive that colour; otherwise each label inherits
#'   the chromosome colour of its point.
#'   \item \strong{Rescale tick marks} --- when y-axis rescaling is
#'   active, the axis tick style is changed (via \code{axis.ticks.y})
#'   and a double-equals annotation is placed at the jump point to
#'   indicate the axis break, with \code{coord_cartesian(clip = "off")}.
#'   \item \strong{Highlight overlay} --- when \code{highlight} is
#'   numeric (row indices) or a character string (R expression), the
#'   matching points are overlaid with a separate \code{geom_point()}
#'   layer styled by \code{highlight_*} parameters, optionally in a
#'   distinct colour when \code{highlight_color} is specified.
#'   \item \strong{Dimension calculation} ---
#'   \code{\link{calculate_plot_dimensions}()} computes height and width
#'   from the number of chromosomes (\code{base_height = 4.5},
#'   \code{x_scale_factor = 0.4}), and stores them as \code{height} /
#'   \code{width} attributes on the plot.
#' }
#'
#' @inheritParams common_args
#' @param chr_by A character string specifying the column name for chromosome
#'   identifiers.  Default: \code{"chr"}.
#' @param pos_by A character string specifying the column name for genomic
#'   positions (integer or numeric).  Default: \code{"pos"}.
#' @param pval_by A character string specifying the column name for p-values
#'   (numeric).  Default: \code{"pval"}.
#' @param label_by A character string specifying the column name for variant
#'   labels.  Only variants with non-empty values in this column will be
#'   labelled.  Default: \code{NULL} (no labels).
#' @param chromosomes A character or numeric vector specifying which
#'   chromosomes to include and/or their display order.  When \code{NULL}
#'   (the default), all chromosomes present in the data are plotted in
#'   their natural factor order.  A single value filters to that
#'   chromosome; a vector reorders and subsets.
#' @param pt_size A numeric value specifying the size of the points.
#'   Default: \code{0.75}.
#' @param pt_color A character string specifying a single colour for all
#'   background (non-highlighted) points.  When \code{NULL} (the default),
#'   alternating chromosome colours from \code{palette} / \code{palcolor}
#'   are used.  Typically set to \code{"grey80"} when \code{highlight} is
#'   used with a distinct \code{highlight_color}.
#' @param pt_alpha A numeric value in \code{[0, 1]} specifying the
#'   transparency of the points.  Default: \code{alpha} (aliased
#'   parameter).
#' @param pt_shape A numeric value specifying the shape of the points.
#'   Default: \code{19} (filled circle).
#' @param label_size A numeric value specifying the font size of the
#'   variant labels.  Default: \code{3}.
#' @param label_fg A character string specifying the colour of the variant
#'   labels.  When \code{NULL} (the default), each label inherits the
#'   colour of its corresponding point.
#' @param highlight Either a numeric vector of row indices or a character
#'   string containing an R expression (parsed via
#'   \code{rlang::parse_expr()}) to select variants to highlight.
#'   Default: \code{NULL} (no highlighting).
#' @param highlight_color A character string specifying the colour of
#'   highlighted points.  When \code{NULL} (the default), highlighted
#'   points inherit the chromosome colour from the underlying
#'   \code{geom_point()} layer.
#' @param highlight_size A numeric value specifying the size of
#'   highlighted points.  Default: \code{1.5}.
#' @param highlight_alpha A numeric value in \code{[0, 1]} specifying the
#'   transparency of highlighted points.  Default: \code{1}.
#' @param highlight_shape A numeric value specifying the shape of
#'   highlighted points.  Default: \code{19} (filled circle).
#' @param preserve_position A logical value.  When \code{TRUE} (the
#'   default), the width of each chromosome segment reflects its number
#'   of variants and variant positions are correctly scaled.  When
#'   \code{FALSE}, all chromosomes have equal width and variants are
#'   equally spaced.
#' @param chr_gap_scaling A numeric scaling factor for the gap between
#'   chromosomes.  Larger values increase the gap.  Default: \code{1}.
#' @param pval_transform A function or character string that can be
#'   evaluated to a function for transforming p-values.  Default:
#'   \code{"-log10"}, which computes \eqn{-\log_{10}(p)}.  Other
#'   examples: \code{"-log2"} or a custom
#'   \code{function(x) -log10(x)}.
#' @param signif A numeric vector of significance thresholds to draw as
#'   horizontal dashed lines.  Default: \code{c(5e-8, 1e-5)}.
#' @param signif_color A character vector of colours for the significance
#'   threshold lines, of equal length as \code{signif}.  When \code{NULL}
#'   (the default), the smallest threshold is coloured black and the rest
#'   grey.
#' @param signif_rel_pos A numeric value between \code{0.1} and \code{0.9}
#'   specifying the relative position of the y-axis jump when rescaling
#'   is active.  Default: \code{0.2}.
#' @param signif_label A logical value.  When \code{TRUE} (the default),
#'   significance threshold values are annotated on the plot.
#' @param signif_label_size A numeric value for the font size of the
#'   significance threshold labels.  Default: \code{3.5}.
#' @param signif_label_pos A character string specifying where to place
#'   the significance threshold labels: \code{"left"} (default) or
#'   \code{"right"}.
#' @param thin A logical value indicating whether to thin dense data by
#'   sampling points per horizontal partition.  Defaults to \code{TRUE}
#'   when \code{chromosomes} selects fewer chromosomes than in the data,
#'   and \code{FALSE} otherwise.
#' @param thin_n An integer specifying the maximum number of points per
#'   horizontal partition after thinning.  Default: \code{1000}.
#' @param thin_bins An integer specifying the number of horizontal bins
#'   for thinning.  Default: \code{200}.
#' @param rescale A logical value.  When \code{TRUE} (the default), the
#'   y-axis is automatically rescaled (broken axis) if extreme
#'   significance values would otherwise compress the main data cloud.
#' @param rescale_ratio_threshold A numeric threshold for triggering
#'   y-axis rescaling.  The ratio is computed as
#'   \code{ceiling(max(log10pval) / 5) * 5 / signif_jump}.
#'   Default: \code{5}.
#' @keywords internal
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'   attributes (in inches) attached.
#' @importFrom utils getFromNamespace
#' @importFrom rlang sym %||%
#' @importFrom ggplot2 waiver aes geom_point scale_y_continuous scale_color_manual annotate
#' @importFrom ggplot2 scale_x_continuous geom_hline element_line coord_cartesian geom_text
#' @importFrom ggrepel geom_label_repel
ManhattanPlotAtomic <- function(
    data,
    chr_by,
    pos_by,
    pval_by,
    label_by = NULL,
    chromosomes = NULL,
    pt_size = 0.75,
    pt_color = NULL,
    pt_alpha = alpha,
    pt_shape = 19,
    label_size = 3,
    label_fg = NULL,
    highlight = NULL,
    highlight_color = NULL,
    highlight_size = 1.5,
    highlight_alpha = 1,
    highlight_shape = 19,
    preserve_position = TRUE,
    chr_gap_scaling = 1,
    pval_transform = "-log10",
    signif = c(5e-08, 1e-05),
    signif_color = NULL,
    signif_rel_pos = 0.2,
    signif_label = TRUE,
    signif_label_size = 3.5,
    signif_label_pos = c("left", "right"),
    thin = NULL,
    thin_n = 1000,
    thin_bins = 200,
    rescale = TRUE,
    rescale_ratio_threshold = 5,
    palette = "Dark2",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    theme = "theme_this",
    theme_args = list(),
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = expression("\u002d" * log[10](p)),
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    signif_label_pos <- match.arg(signif_label_pos)

    mpdata <- suppressWarnings({
        # Some chromosomes are not specified in chr.order
        ggmanh::manhattan_data_preprocess(
            x = data,
            chromosome = if (length(chromosomes) == 1) chromosomes else NULL,
            signif = signif,
            pval.colname = pval_by,
            chr.colname = chr_by,
            pos.colname = pos_by,
            chr.order = chromosomes,
            signif.col = signif_color,
            preserve.position = preserve_position,
            thin = thin,
            thin.n = thin_n,
            thin.bins = thin_bins,
            pval.log.transform = TRUE,
            chr.gap.scaling = chr_gap_scaling,
        )
    })
    label_by <- check_columns(mpdata$data, label_by, force_factor = TRUE)

    if (
        (!is.null(highlight) && !is.null(highlight_color)) || !is.null(pt_color)
    ) {
        pt_color <- pt_color %||% "grey80"
        mpdata$chr.col <- rep(pt_color, length(levels(mpdata$data[[chr_by]])))
        names(mpdata$chr.col) <- levels(mpdata$data[[chr_by]])
    } else {
        mpdata$chr.col <- palette_this(
            levels(mpdata$data[[chr_by]]),
            palette = palette,
            palcolor = palcolor,
            reverse = palreverse,
            alpha = pt_alpha
        )
    }

    if (is.character(pval_transform) && startsWith(pval_transform, "-")) {
        pval_transform <- sub("^-", "", pval_transform)
        pval_transform_fn <- eval(parse(text = pval_transform))
        pval_transform <- function(x) -pval_transform_fn(x)
    }
    if (is.character(pval_transform)) {
        pval_transform <- eval(parse(text = pval_transform))
    }
    stopifnot(
        "[ManhattanPlot] 'pval_transform' must be a function" = is.function(
            pval_transform
        )
    )
    mpdata$data$log10pval <- pval_transform(mpdata$data[[pval_by]])
    mpdata$pval.colname <- "log10pval"

    trans <- list(trans = "identity", breaks = waiver())
    if (isTRUE(rescale)) {
        get_transform_jump <- getFromNamespace("get_transform_jump", "ggmanh")
        get_transform <- getFromNamespace("get_transform", "ggmanh")
        jump <- get_transform_jump(pval_transform(mpdata$signif))
        if (
            (ceiling(max(mpdata$data[[mpdata$pval.colname]]) / 5) * 5) / jump >
                rescale_ratio_threshold
        ) {
            trans <- get_transform(
                mpdata$data,
                jump,
                mpdata$pval.colname,
                jump.rel.pos = signif_rel_pos
            )
        }
    }

    ylimit <- c(
        0,
        ifelse(identical(trans$trans, "identity"), NA, max(trans$breaks))
    )

    if (length(unique(mpdata$data[[chr_by]])) == 1) {
        pos <- mpdata$true.pos.colname
        x_break <- waiver()
        x_break_label <- waiver()
        x_limits <- NULL
        chrname <- as.character(unique(mpdata$data[[chr_by]]))
        xlab <- xlab %||%
            ifelse(
                startsWith(chrname, "chr") || startsWith(chrname, "Chr"),
                chrname,
                paste0("Chromosome ", chrname)
            )
    } else {
        calc_new_pos_ <- getFromNamespace("calc_new_pos_", "ggmanh")
        if (mpdata$pos.colname == "new_pos_unscaled") {
            mpdata$data$new_pos_unscaled <- calc_new_pos_(
                mpdata$data$new_pos_unscaled,
                mpdata$data[[mpdata$chr.colname]],
                mpdata$chr.pos.info
            )
        }
        pos <- mpdata$pos.colname
        x_break <- mpdata$chr.pos.info$center_pos
        x_break_label <- mpdata$chr.labels
        x_limits <- c(
            min(mpdata$chr.pos.info$start_pos),
            max(mpdata$chr.pos.info$end_pos)
        )
    }
    xlab <- xlab %||% "Chromosome"

    expand <- c(0.02, 0.01, 0.025, 0.01)
    if (isTRUE(signif_label)) {
        expand[1] <- 0.1
    }
    expand <- norm_expansion(expand, "continuous", "continuous")

    p <- ggplot(
        mpdata$data,
        aes(
            x = !!sym(pos),
            y = !!sym(mpdata$pval.colname),
            color = !!sym(chr_by)
        )
    ) +
        geom_point(size = pt_size) +
        scale_color_manual(values = mpdata$chr.col, guide = "none") +
        scale_y_continuous(
            trans = trans$trans,
            breaks = trans$breaks,
            expand = expand$y,
            limits = ylimit
        ) +
        scale_x_continuous(
            name = xlab,
            breaks = x_break,
            labels = x_break_label,
            expand = expand$x,
            limits = x_limits
        ) +
        geom_hline(
            yintercept = pval_transform(mpdata$signif),
            linetype = 'dashed',
            color = mpdata$signif.col
        ) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            panel.grid.major = ggplot2::element_blank(),
            panel.grid.minor = ggplot2::element_blank(),
            legend.position = "none"
        ) +
        labs(x = xlab, y = ylab) +
        ggtitle(label = title, subtitle = subtitle)

    if (isTRUE(signif_label)) {
        p <- p +
            geom_text(
                data.frame(
                    x = if (signif_label_pos == "left") -Inf else Inf,
                    y = pval_transform(mpdata$signif),
                    label = mpdata$signif,
                    color = signif_color %||%
                        c("black", rep("grey80", length(mpdata$signif) - 1))
                ),
                mapping = aes(
                    x = !!sym("x"),
                    y = !!sym("y"),
                    label = !!sym("label"),
                    color = I(!!sym("color"))
                ),
                hjust = if (signif_label_pos == "left") -0.5 else 1.5,
                vjust = -0.5,
                size = signif_label_size
            )
    }

    if (!is.null(label_by)) {
        if (is.null(label_fg)) {
            p <- p +
                geom_label_repel(
                    aes(label = !!sym(label_by), color = !!sym(chr_by)),
                    size = label_size,
                    min.segment.length = 0,
                    max.overlaps = 100
                )
        } else {
            p <- p +
                geom_label_repel(
                    aes(label = !!sym(label_by)),
                    color = label_fg,
                    size = label_size,
                    min.segment.length = 0,
                    max.overlaps = 100
                )
        }
    }

    if (rescale & !identical(trans$trans, "identity")) {
        # if the plot is rescaled, change the tick at the "jump" to double line
        jump_tick_size <- 3.5

        p <- p +
            ggplot2::theme(
                axis.ticks.y = element_line(linetype = trans$y_axis_linetype)
            ) +
            annotate(
                geom = "point",
                shape = "=",
                x = -Inf,
                y = trans$jump,
                size = jump_tick_size
            ) +
            coord_cartesian(clip = "off")
    }

    if (is.numeric(highlight)) {
        hidata <- mpdata$data[highlight, , drop = FALSE]
    } else if (is.character(highlight)) {
        hidata <- mpdata$data %>% filter(!!rlang::parse_expr(highlight))
    } else {
        hidata <- NULL
    }
    if (!is.null(hidata)) {
        if (is.null(highlight_color)) {
            p <- p +
                geom_point(
                    data = hidata,
                    aes(
                        x = !!sym(pos),
                        y = !!sym(mpdata$pval.colname),
                        color = !!sym(chr_by)
                    ),
                    size = highlight_size,
                    shape = highlight_shape,
                    alpha = highlight_alpha
                )
        } else {
            p <- p +
                geom_point(
                    data = hidata,
                    aes(x = !!sym(pos), y = !!sym(mpdata$pval.colname)),
                    size = highlight_size,
                    shape = highlight_shape,
                    color = highlight_color,
                    alpha = highlight_alpha
                )
        }
    }

    n_chrom <- length(levels(mpdata$data[[chr_by]]))
    dims <- calculate_plot_dimensions(
        base_height = 4.5,
        aspect.ratio = NULL,
        n_x = n_chrom,
        x_scale_factor = 0.4,
        legend.position = "none"
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    p
}

#' Manhattan plot
#'
#' @description
#' Renders a publication-quality Manhattan plot for genetic association
#' results.  The y-axis displays \eqn{-\log_{10}(p)} (or a user-specified
#' transformation) of p-values, and the x-axis shows genomic positions
#' organised by chromosome.  Each chromosome is rendered in alternating
#' colours, and configurable horizontal dashed lines mark genome-wide
#' significance thresholds.
#'
#' The function is adapted from \code{ggmanh::manhattan_plot()} with
#' extended control over point appearance, variant labels, highlighting,
#' data thinning, y-axis rescaling, and \code{split_by} support for
#' creating multi-panel layouts (e.g. faceted by cohort or phenotype).
#'
#' @section split_by Workflow:
#'
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \strong{Column validation} --- \code{\link{check_columns}()}
#'   resolves \code{split_by} with \code{force_factor = TRUE},
#'   \code{allow_multi = TRUE}, and \code{concat_multi = TRUE}.  For
#'   \code{GRanges} inputs, validation is performed on the
#'   \code{@elementMetadata} slot.
#'   \item \strong{GRanges support} --- \code{data} can be a
#'   \code{data.frame} or a \code{GenomicRanges::GRanges} object.
#'   When \code{GRanges} is used, \code{split_by} is read from the
#'   metadata columns.
#'   \item \strong{Data splitting} --- drops unused \code{split_by}
#'   levels, splits \code{data} by \code{split_by} (preserving factor
#'   level order), and wraps into a named list.  When \code{split_by}
#'   is \code{NULL}, the data is wrapped as a single-element list with
#'   name \code{"..."}.
#'   \item \strong{Per-split palette / colour} ---
#'   \code{\link{check_palette}()} and \code{\link{check_palcolor}()}
#'   resolve per-split palette and colour overrides.
#'   \item \strong{Per-split title} --- when \code{title} is a function,
#'   it receives the default title (the split level name) and can return
#'   a custom string; otherwise \code{title \%||\% split_level} is used.
#'   \item \strong{Dispatch} --- each split subset is passed to
#'   \code{\link{ManhattanPlotAtomic}}.
#'   \item \strong{Combination} --- \code{\link{combine_plots}()}
#'   assembles the list of plots via \code{patchwork::wrap_plots},
#'   honouring \code{nrow} / \code{ncol} / \code{byrow} / \code{design}.
#' }
#'
#' @note \code{facet_by} is not supported by this plot type and triggers
#'   a warning if provided.  Use \code{split_by} instead to produce
#'   comparable multi-panel layouts.
#'
#' @inheritParams common_args
#' @inheritParams ManhattanPlotAtomic
#' @param split_by The column(s) to split data by and produce separate
#'   sub-plots.  Multiple columns are concatenated with
#'   \code{split_by_sep}.
#' @param split_by_sep A character string used to concatenate multiple
#'   \code{split_by} column values.  Default: \code{"_"}.
#' @param seed A numeric seed for reproducibility.  Passed to
#'   \code{\link{validate_common_args}()}.  Default: \code{8525}.
#' @param combine A logical value.  When \code{TRUE} (the default), the
#'   list of per-split plots is combined into a single \code{patchwork}
#'   object.  When \code{FALSE}, returns the raw list.
#' @param nrow,ncol,byrow Integers controlling the layout of combined
#'   plots via \code{patchwork::wrap_plots()}.  \code{byrow = TRUE}
#'   fills the layout row-wise.
#' @param axes,axis_titles Strings controlling how axes and axis titles
#'   are handled across combined plots.  Passed to
#'   \code{\link{combine_plots}()}.  See \code{?patchwork::wrap_plots}
#'   for options (\code{"keep"}, \code{"collect"}, \code{"collect_x"},
#'   \code{"collect_y"}).
#' @param guides A string controlling guide collection across combined
#'   plots.  Passed to \code{\link{combine_plots}()}.
#' @param design A custom layout specification for combined plots.
#'   Passed to \code{\link{combine_plots}()}.  When specified,
#'   \code{nrow}, \code{ncol}, and \code{byrow} are ignored.
#' @return A \code{ggplot} object (single plot, no \code{split_by}), a
#'   \code{patchwork} object (when \code{combine = TRUE} with
#'   \code{split_by}), or a named list of \code{ggplot} objects (when
#'   \code{combine = FALSE}).  Each individual plot carries
#'   \code{height} and \code{width} attributes.
#' @export
#' @examples
#' \donttest{
#' set.seed(1000)
#'
#' nsim <- 50000
#'
#' # --- Data simulation ---
#' simdata <- data.frame(
#'   "chromosome" = sample(c(1:22,"X"), size = nsim, replace = TRUE),
#'   "position" = sample(1:100000000, size = nsim),
#'   "P.value" = rbeta(nsim, shape1 = 5, shape2 = 1)^7,
#'   "cohort" = sample(c("A", "B"), size = nsim, replace = TRUE)
#' )
#' simdata$chromosome <- factor(simdata$chromosome, c(1:22, "X"))
#' options(repr.plot.width=10, repr.plot.height=5)
#'
#' # --- Basic Manhattan plot ---
#' if (requireNamespace("ggmanh", quietly = TRUE)) {
#' ManhattanPlot(
#'    simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
#'    title = "Simulated P.Values", ylab = "P")
#' }
#'
#' # --- split_by ---
#' if (requireNamespace("ggmanh", quietly = TRUE)) {
#' ManhattanPlot(
#'    simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
#'    title = "Simulated P.Values", ylab = "P", split_by = "cohort", ncol = 1)
#' }
#'
#' # --- Customized p-value transformation and significance threshold line colors ---
#' if (requireNamespace("ggmanh", quietly = TRUE)) {
#' ManhattanPlot(
#'    simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
#'    title = "Simulated -Log2 P.Values", ylab = "-log2(P)", pval_transform = "-log2",
#'    signif_color = c("red", "blue"))
#' }
#'
#' # --- Different palette and no significance threshold labels ---
#' if (requireNamespace("ggmanh", quietly = TRUE)) {
#' ManhattanPlot(
#'    simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
#'    palette = "Set1", signif_label = FALSE)
#' }
#'
#' # --- Reverse palette and label position on the right ---
#' if (requireNamespace("ggmanh", quietly = TRUE)) {
#' ManhattanPlot(
#'    simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
#'    palette = "Set1", palreverse = TRUE, signif_label_pos = "right")
#' }
#'
#' # --- Single chromosome ---
#' if (requireNamespace("ggmanh", quietly = TRUE)) {
#' ManhattanPlot(
#'    simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
#'    title = "Simulated P.Values", chromosomes = 5)
#' }
#'
#' # --- Chromosome subset and reorder ---
#' if (requireNamespace("ggmanh", quietly = TRUE)) {
#' ManhattanPlot(
#'    simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
#'    title = "Simulated P.Values", chromosomes = c(20, 4, 6))
#' }
#'
#' tmpdata <- data.frame(
#'   "chromosome" = c(rep(5, 10), rep(21, 5)),
#'   "position" = c(sample(250000:250100, 10, replace = FALSE),
#'     sample(590000:600000, 5, replace = FALSE)),
#'   "P.value" = c(10^-(rnorm(10, 100, 3)), 10^-rnorm(5, 9, 1)),
#'   "cohort" = c(rep("A", 10), rep("B", 5))
#' )
#'
#' simdata <- rbind(simdata, tmpdata)
#' simdata$chromosome <- factor(simdata$chromosome, c(1:22, "X"))
#'
#' # --- Disable y-axis rescaling ---
#' if (requireNamespace("ggmanh", quietly = TRUE)) {
#' ManhattanPlot(
#'     simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
#'     title = "Simulated P.Values - Significant", rescale = FALSE)
#' }
#'
#' # --- Y-axis rescaling with custom break position ---
#' if (requireNamespace("ggmanh", quietly = TRUE)) {
#' ManhattanPlot(
#'     simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
#'     title = "Simulated P.Values - Significant", rescale = TRUE, signif_rel_pos = 0.5)
#' }
#'
#' sig <- simdata$P.value < 5e-07
#'
#' simdata$label <- ""
#' simdata$label[sig] <- sprintf("Label: %i", 1:sum(sig))
#' simdata$label2 <- ""
#' i <- (simdata$chromosome == 5) & (simdata$P.value < 5e-8)
#' simdata$label2[i] <- paste("Chromosome 5 label", 1:sum(i))
#'
#' # --- Variant labels ---
#' if (requireNamespace("ggmanh", quietly = TRUE)) {
#' ManhattanPlot(simdata, label_by = "label", pval_by = "P.value", chr_by = "chromosome",
#'     pos_by = "position", title = "Simulated P.Values with labels", label_size = 4)
#' }
#'
#' # --- Variant labels with custom color ---
#' if (requireNamespace("ggmanh", quietly = TRUE)) {
#' ManhattanPlot(simdata, label_by = "label2", pval_by = "P.value", chr_by = "chromosome",
#'     pos_by = "position", title = "Simulated P.Values with labels",
#'     label_size = 3, label_fg = "black")
#' }
#'
#' simdata$color <- "Not Significant"
#' simdata$color[simdata$P.value <= 5e-8] <- "Significant"
#'
#' # --- Highlight points with custom shape ---
#' if (requireNamespace("ggmanh", quietly = TRUE)) {
#' ManhattanPlot(simdata, title = "Highlight Points with shapes",
#'     pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
#'     highlight = "color == 'Significant'", highlight_color = NULL, highlight_shape = 6,
#'     highlight_size = 5, pt_alpha = 0.2, pt_size = 1)
#' }
#'
#' # --- Highlight points with custom color ---
#' if (requireNamespace("ggmanh", quietly = TRUE)) {
#' ManhattanPlot(simdata, title = "Highlight Points",
#'     pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
#'     highlight = "color == 'Significant'", highlight_color = "black",
#'     pt_color = "lightblue", pt_alpha = 0.2, pt_size = 0.1)
#' }
#' }
ManhattanPlot <- function(
    data,
    chr_by,
    pos_by,
    pval_by,
    split_by = NULL,
    split_by_sep = "_",
    label_by = NULL,
    chromosomes = NULL,
    pt_size = 0.75,
    pt_color = NULL,
    pt_alpha = alpha,
    pt_shape = 19,
    label_size = 3,
    label_fg = NULL,
    highlight = NULL,
    highlight_color = NULL,
    highlight_size = 1.5,
    highlight_alpha = 1,
    highlight_shape = 19,
    preserve_position = TRUE,
    chr_gap_scaling = 1,
    pval_transform = "-log10",
    signif = c(5e-08, 1e-05),
    signif_color = NULL,
    signif_rel_pos = 0.2,
    signif_label = TRUE,
    signif_label_size = 3.5,
    signif_label_pos = c("left", "right"),
    thin = NULL,
    thin_n = 1000,
    thin_bins = 200,
    rescale = TRUE,
    rescale_ratio_threshold = 5,
    palette = "Dark2",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    theme = "theme_this",
    theme_args = list(),
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = expression("\u002d" * log[10](p)),
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    facet_by = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed = seed)
    if (!is.null(facet_by)) {
        warning(
            "[ManhattanPlot] 'facet_by' is not supported, using 'split_by' instead."
        )
    }

    theme <- process_theme(theme)
    if (is.data.frame(data)) {
        split_by <- check_columns(
            data,
            split_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = split_by_sep
        )
    } else if (inherits(data, "GRanges")) {
        metadata <- data@elementMetadata
        split_by <- check_columns(
            metadata,
            split_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = split_by_sep
        )
        data@elementMetadata <- metadata
    } else {
        stop("[ManhattanPlot] 'data' must be a data frame or a GRanges object.")
    }

    if (!is.null(split_by)) {
        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }

    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            ManhattanPlotAtomic(
                data = datas[[nm]],
                chr_by = chr_by,
                pos_by = pos_by,
                pval_by = pval_by,
                label_by = label_by,
                chromosomes = chromosomes,
                pt_size = pt_size,
                pt_color = pt_color,
                pt_alpha = pt_alpha,
                pt_shape = pt_shape,
                label_size = label_size,
                label_fg = label_fg,
                highlight = highlight,
                highlight_color = highlight_color,
                highlight_size = highlight_size,
                highlight_alpha = highlight_alpha,
                highlight_shape = highlight_shape,
                preserve_position = preserve_position,
                chr_gap_scaling = chr_gap_scaling,
                pval_transform = pval_transform,
                signif = signif,
                signif_color = signif_color,
                signif_rel_pos = signif_rel_pos,
                signif_label = signif_label,
                signif_label_size = signif_label_size,
                signif_label_pos = signif_label_pos,
                thin = thin,
                thin_n = thin_n,
                thin_bins = thin_bins,
                rescale = rescale,
                rescale_ratio_threshold = rescale_ratio_threshold,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                alpha = alpha,
                theme_args = theme_args,
                theme = theme,
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
# Manhattan plot needs chr, pos, pval columns
# Using a small but realistic dataset with multiple chromosomes
n_snps <- 200
mhtn_data <- data.frame(
    chr = sample(paste0("chr", 1:5), n_snps, replace = TRUE),
    pos = sample(1:1e6, n_snps),
    pval = runif(n_snps, min = 1e-10, max = 1),
    label = paste0("SNP", seq_len(n_snps)),
    group = factor(sample(c("A", "B"), n_snps, replace = TRUE))
)

test_that("ManhattanPlot basic usage works", {
    p <- ManhattanPlot(mhtn_data, chr_by = "chr", pos_by = "pos", pval_by = "pval")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("ManhattanPlot with title and subtitle works", {
    p <- ManhattanPlot(mhtn_data, chr_by = "chr", pos_by = "pos", pval_by = "pval",
                       title = "Manhattan Title", subtitle = "Subtitle")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "Manhattan Title")
    expect_equal(p$labels$subtitle, "Subtitle")
})

test_that("ManhattanPlot with split_by returns patchwork", {
    p <- ManhattanPlot(mhtn_data, chr_by = "chr", pos_by = "pos", pval_by = "pval",
                       split_by = "group", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("ManhattanPlot with split_by and combine = FALSE returns list", {
    plots <- ManhattanPlot(mhtn_data, chr_by = "chr", pos_by = "pos", pval_by = "pval",
                           split_by = "group", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("ManhattanPlot with multiple split_by columns returns a list with one plot per combination", {
    mhtn_data2 <- mhtn_data
    mhtn_data2$split2 <- factor(rep(c("s1", "s2"), each = 100))
    plots <- suppressMessages(ManhattanPlot(mhtn_data2, chr_by = "chr", pos_by = "pos", pval_by = "pval",
                                            split_by = c("group", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("ManhattanPlot with multiple split_by columns returns combined plot", {
    mhtn_data2 <- mhtn_data
    mhtn_data2$split2 <- factor(rep(c("s1", "s2"), each = 100))
    p <- suppressMessages(ManhattanPlot(mhtn_data2, chr_by = "chr", pos_by = "pos", pval_by = "pval",
                                        split_by = c("group", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("ManhattanPlot with specific chromosomes works", {
    p <- ManhattanPlot(mhtn_data, chr_by = "chr", pos_by = "pos", pval_by = "pval",
                       chromosomes = c("chr1", "chr2", "chr3"))
    expect_s3_class(p, "ggplot")
})

test_that("ManhattanPlot with label_by works", {
    p <- ManhattanPlot(mhtn_data, chr_by = "chr", pos_by = "pos", pval_by = "pval",
                       label_by = "label")
    expect_s3_class(p, "ggplot")
})

test_that("ManhattanPlot with highlight works", {
    highlights <- mhtn_data$pval < 1e-3
    if (any(highlights)) {
        p <- ManhattanPlot(mhtn_data, chr_by = "chr", pos_by = "pos", pval_by = "pval",
                           highlight = which(highlights))
        expect_s3_class(p, "ggplot")
    }
})

test_that("ManhattanPlot with preserve_position = FALSE works", {
    p <- ManhattanPlot(mhtn_data, chr_by = "chr", pos_by = "pos", pval_by = "pval",
                       preserve_position = FALSE)
    expect_s3_class(p, "ggplot")
})

#' Atomic Correlation Plot
#'
#' @description
#' Core implementation for drawing a scatter plot of two variables with a
#' linear regression line, optional correlation statistics, and point
#' highlighting.  This is the workhorse behind the exported
#' \code{\link{CorPlot}} — it takes a \strong{single} data frame (no
#' \code{split_by} support) and returns a \code{ggplot} object with
#' faceting applied.
#'
#' The function supports \strong{group-based colouring} (\code{group_by}),
#' point highlighting by expression or rowname, multiple annotation items
#' (regression equation, R-squared, p-value, Spearman/Pearson/Kendall
#' correlation, N), raster rendering for large datasets, configurable
#' regression line style, and faceting.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
#'         \code{ggplot2::ggplot} based on
#'         \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{Parameter normalisation} — \code{match.arg()} resolves
#'         \code{anno_position} abbreviations (\code{"tl"}, \code{"tr"},
#'         \code{"bl"}, \code{"br"}) to their full forms.
#'         \code{raster_dpi} is expanded to length 2 when given as a scalar.
#'   \item \strong{Column resolution} — \code{x}, \code{y}, and
#'         \code{group_by} are validated via \code{\link{check_columns}}.
#'         Multi-column \code{group_by} is concatenated with
#'         \code{group_by_sep}.
#'   \item \strong{Grouping fallback} — when \code{group_by = NULL}, a dummy
#'         \code{.group} column is created and the legend is suppressed
#'         (\code{legend.position = "none"}).
#'   \item \strong{Annotation data calculation} — the data is grouped by
#'         \code{facet_by} and a linear model (\code{lm(y ~ x)}) is fitted
#'         per group.  Each requested \code{anno_items} is computed:
#'         \itemize{
#'           \item \code{"eq"} — regression equation \code{y = a + bx}.
#'           \item \code{"r2"} — R-squared of the model.
#'           \item \code{"p"} — p-value of the x coefficient.
#'           \item \code{"spearman"} — Spearman's rho.
#'           \item \code{"pearson"} — Pearson's r.
#'           \item \code{"kendall"} — Kendall's tau.
#'           \item \code{"n"} — number of observations.
#'         }
#'         The results are stored in an \code{annodata} data frame for
#'         \code{geom_text_repel}.
#'   \item \strong{Highlight parsing} — the \code{highlight} argument is
#'         resolved into a \code{.highlight} logical column:
#'         \itemize{
#'           \item \code{TRUE} — highlights all points.
#'           \item A character expression — evaluated via
#'                 \code{dplyr::filter} to select rows.
#'           \item A character vector — matched against rownames of the data.
#'           \item A numeric vector — treated as row indices.
#'         }
#'   \item \strong{Point rendering} — two branches:
#'         \itemize{
#'           \item \strong{Raster mode} (\code{raster = TRUE}) — uses
#'                 \code{scattermore::geom_scattermore()} for efficient
#'                 rendering of large datasets.  Highlighted points are drawn
#'                 in two layers (stroke + fill).
#'           \item \strong{Vector mode} (\code{raster = FALSE}, default) —
#'                 uses \code{ggplot2::geom_point()} with configurable size,
#'                 shape, and alpha.  Highlighted points get an outer stroke
#'                 via a second point layer.
#'         }
#'   \item \strong{Regression line} — \code{geom_smooth(method = "lm")}
#'         draws the linear regression line with optional standard error band
#'         (\code{smooth_se}).
#'   \item \strong{Annotation text} — \code{ggrepel::geom_text_repel()}
#'         places the computed annotations at the specified
#'         \code{anno_position} corner, with background styling.
#'   \item \strong{Colour scale} — \code{scale_color_manual()} maps group
#'         levels to colours via \code{\link{palette_this}()}.
#'   \item \strong{Labels and theme} — \code{labs()} sets titles and axis
#'         labels.  The theme is applied via \code{do_call()}, with
#'         \code{aspect.ratio}, \code{legend.position}, and
#'         \code{legend.direction} enforced.
#'   \item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
#'         computes \code{height} and \code{width} attributes from
#'         \code{base_height = 4.5}, \code{aspect.ratio}, legend metrics, and
#'         the number of group levels.
#'   \item \strong{Faceting} — \code{\link{facet_plot}()} applies
#'         \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is
#'         provided.
#' }
#'
#' @inheritParams common_args
#' @param x A character string specifying the column name for the x-axis.
#'  Must be numeric.
#' @param y A character string specifying the column name for the y-axis.
#'  Must be numeric.
#' @param group_by A character vector of column names to colour the points
#'  by.  Each unique combination becomes a separate group in the legend.
#'  Multiple columns are concatenated with \code{group_by_sep}.  When
#'  \code{NULL}, all points are a single colour and the legend is hidden.
#' @param group_by_sep A character string to separate concatenated
#'  \code{group_by} columns.  Default \code{"_"}.
#' @param group_name A character string used as the colour legend title.
#'  When \code{NULL}, the \code{group_by} column name is used.
#' @param pt_size A numeric value specifying the size of the points.
#'  Default: \code{2}.
#' @param pt_shape A numeric value specifying the shape of the points
#'  (see \code{\link[ggplot2]{geom_point}}).  Default: \code{16} (filled
#'  circle).
#' @param raster A logical value.  When \code{TRUE}, uses
#'  \code{scattermore::geom_scattermore()} for efficient rendering of
#'  large datasets.  Default: \code{FALSE}.
#' @param raster_dpi An integer vector of length 1 or 2 specifying the
#'  raster resolution in (width, height) pixels.  When a single value is
#'  provided, it is recycled.  Default: \code{c(512, 512)}.
#' @param highlight Specifies which points to emphasise.  Can be:
#'  \itemize{
#'    \item \code{TRUE} — highlight all points.
#'    \item A character expression (e.g. \code{'Species == "setosa"'}) —
#'          evaluated via \code{dplyr::filter}.
#'    \item A character vector — matched against rownames of the data.
#'    \item A numeric vector — treated as row indices.
#'  }
#'  Default: \code{NULL} (no highlighting).
#' @param highlight_color A character string specifying the colour of the
#'  highlighted point borders.  Default: \code{"black"}.
#' @param highlight_size A numeric value specifying the size of the
#'  highlighted points (the inner fill).  Default: \code{1}.
#' @param highlight_alpha A numeric value specifying the alpha transparency
#'  of the highlighted points.  Default: \code{1}.
#' @param highlight_stroke A numeric value specifying the stroke width of
#'  the highlighted point borders.  The outer layer size is
#'  \code{highlight_size + highlight_stroke}.  Default: \code{0.8}.
#' @param anno_items A character vector specifying which statistics to
#'  display as text annotation.  Available items: \code{"eq"} (regression
#'  equation), \code{"r2"} (R-squared), \code{"p"} (p-value),
#'  \code{"spearman"}, \code{"pearson"}, \code{"kendall"},
#'  \code{"n"} (observation count).  Default: \code{c("eq", "r2", "p")}.
#' @param anno_size A numeric value specifying the font size of the
#'  annotation text (scaled by \code{base_size / 12}).  Default: \code{3}.
#' @param anno_fg A character string specifying the colour of the
#'  annotation text.  Default: \code{"black"}.
#' @param anno_bg A character string specifying the background colour of
#'  the annotation text boxes.  Default: \code{"white"}.
#' @param anno_bg_r A numeric value specifying the corner radius of the
#'  annotation text background boxes.  Default: \code{0.1}.
#' @param anno_position A character string specifying the corner position
#'  of the annotation text.  One of \code{"topleft"} (alias \code{"tl"}),
#'  \code{"topright"} (\code{"tr"}), \code{"bottomleft"} (\code{"bl"}),
#'  \code{"bottomright"} (\code{"br"}).
#' @param add_smooth A logical value.  When \code{TRUE} (default), a linear
#'  regression line (\code{geom_smooth(method = "lm")}) is added.
#' @param smooth_color A character string specifying the colour of the
#'  regression line.  Default: \code{"red2"}.
#' @param smooth_width A numeric value specifying the linewidth of the
#'  regression line.  Default: \code{1.5}.
#' @param smooth_se A logical value.  When \code{TRUE}, a standard error
#'  band is drawn around the regression line.  Default: \code{FALSE}.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @keywords internal
#' @importFrom stats cor lm coef
#' @importFrom rlang syms sym
#' @importFrom dplyr group_modify filter
#' @importFrom ggplot2 geom_point geom_smooth geom_text scale_color_manual labs
CorPlotAtomic <- function(
    data,
    x,
    y,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    pt_size = 2,
    pt_shape = 16,
    alpha = 1,
    raster = FALSE,
    raster_dpi = c(512, 512),
    highlight = NULL,
    highlight_color = "black",
    highlight_size = 1,
    highlight_alpha = 1,
    highlight_stroke = 0.8,
    anno_items = c("eq", "r2", "p"),
    anno_size = 3,
    anno_fg = "black",
    anno_bg = "white",
    anno_bg_r = 0.1,
    anno_position = c(
        "topleft",
        "topright",
        "bottomleft",
        "bottomright",
        "tl",
        "tr",
        "bl",
        "br"
    ),
    add_smooth = TRUE,
    smooth_color = "red2",
    smooth_width = 1.5,
    smooth_se = FALSE,
    theme = "theme_this",
    theme_args = list(),
    palette = ifelse(is.null(group_by), "Spectral", "Paired"),
    palcolor = NULL,
    palreverse = FALSE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    legend.position = waiver(),
    legend.direction = "vertical",
    seed = 8525,
    ...
) {
    set.seed(seed)
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    anno_position <- match.arg(anno_position)
    anno_position <- switch(
        anno_position,
        tl = "topleft",
        tr = "topright",
        bl = "bottomleft",
        br = "bottomright",
        anno_position
    )
    if (length(raster_dpi) == 1) {
        raster_dpi <- rep(raster_dpi, 2)
    }

    x <- check_columns(data, x)
    y <- check_columns(data, y)
    group_by <- check_columns(
        data,
        group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = group_by_sep
    )

    if (is.null(group_by)) {
        group_by <- ".group"
        data[[group_by]] <- factor("")
        legend.position <- ifelse(
            inherits(legend.position, "waiver"),
            "none",
            "right"
        )
    } else {
        legend.position <- ifelse(
            inherits(legend.position, "waiver"),
            "right",
            legend.position
        )
    }

    base_size <- theme_args$base_size %||% 12
    text_size_scale <- base_size / 12

    # Calcuate the equation of the line, r-squared, and rho
    annodata <- data %>%
        dplyr::group_by(!!!syms(facet_by)) %>%
        group_modify(function(dat, g) {
            m <- lm(data[[y]] ~ data[[x]])
            anno <- c()
            for (item in anno_items) {
                if (item == "eq") {
                    a <- format(as.numeric(coef(m)[1]), digits = 2)
                    b <- format(as.numeric(abs(coef(m)[2])), digits = 2)
                    if (is.na(coef(m)[2])) {
                        anno_eq <- substitute(
                            italic(y) == "NaN"
                        )
                    } else if (coef(m)[2] >= 0) {
                        anno_eq <- substitute(
                            italic(y) == a + b %.% italic(x),
                            list(a = a, b = b)
                        )
                    } else {
                        anno_eq <- substitute(
                            italic(y) == a - b %.% italic(x),
                            list(a = a, b = b)
                        )
                    }
                    anno <- c(anno, as.character(as.expression(anno_eq)))
                } else if (item == "r2") {
                    anno_r2 <- substitute(
                        italic(r)^2 ~ "=" ~ r2,
                        list(r2 = format(summary(m)$r.squared, digits = 2))
                    )
                    anno <- c(anno, as.character(as.expression(anno_r2)))
                } else if (item == "p") {
                    coefs <- summary(m)$coefficients
                    if (nrow(coefs) < 2 || all(is.na(coefs[2, ]))) {
                        # Case: x is constant → no slope, no p-value
                        anno_p <- substitute(
                            italic("coeff.") ~ italic(p) ~ "=" ~ "NA"
                        )
                    } else {
                        anno_p <- substitute(
                            italic("coeff.") ~ italic(p) ~ "=" ~ pvalue,
                            list(
                                pvalue = format(
                                    coefs[2, 4],
                                    digits = 2
                                )
                            )
                        )
                    }
                    anno <- c(anno, as.character(as.expression(anno_p)))
                } else if (item == "spearman") {
                    rho <- cor(dat[[x]], dat[[y]], method = "spearman")
                    anno_rho <- substitute(
                        italic("spearman's") ~ italic(rho) ~ "=" ~ value,
                        list(value = format(rho, digits = 2))
                    )
                    anno <- c(anno, as.character(as.expression(anno_rho)))
                } else if (item == "pearson") {
                    r <- cor(dat[[x]], dat[[y]], method = "pearson")
                    anno_r <- substitute(
                        italic("pearson's") ~ italic(r) ~ "=" ~ value,
                        list(value = format(r, digits = 2))
                    )
                    anno <- c(anno, as.character(as.expression(anno_r)))
                } else if (item == "kendall") {
                    tau <- cor(dat[[x]], dat[[y]], method = "kendall")
                    anno_tau <- substitute(
                        italic("kendall's") ~ italic(tau) ~ "=" ~ value,
                        list(value = format(tau, digits = 2))
                    )
                    anno <- c(anno, as.character(as.expression(anno_tau)))
                } else if (item == "n") {
                    n <- nrow(dat)
                    anno_n <- substitute(
                        italic(N) ~ "=" ~ value,
                        list(value = n)
                    )
                    anno <- c(anno, as.character(as.expression(anno_n)))
                } else {
                    stop(
                        "Unknown annotation item: ",
                        item,
                        ". Expect: eq, r2, p, spearman, pearson, kendall, n"
                    )
                }
            }
            data.frame(anno = anno)
        })

    p <- ggplot(data, aes(x = !!sym(x), y = !!sym(y)))

    data$.highlight <- FALSE
    if (!is.null(highlight)) {
        if (isTRUE(highlight)) {
            data$.highlight <- TRUE
        } else if (length(highlight == 1 && is.character(highlight))) {
            data <- data %>% mutate(.highlight = eval(parse(text = highlight)))
        } else {
            all_inst <- rownames(data) %||% 1:nrow(data)
            if (!any(highlight %in% all_inst)) {
                stop("No highlight items found in the data (rownames).")
            }
            if (!all(highlight %in% all_inst)) {
                warning(
                    "Some highlight items not found in the data (rownames)."
                )
            }
            data$.highlight <- all_inst %in% highlight
            rm(all_inst)
        }
    }

    if (isTRUE(raster)) {
        # Normal points
        if (sum(data$.highlight == FALSE) > 0) {
            p <- p +
                scattermore::geom_scattermore(
                    data = data[data$.highlight == FALSE, , drop = FALSE],
                    aes(color = !!sym(group_by)),
                    pointsize = ceiling(pt_size),
                    alpha = alpha,
                    pixels = raster_dpi
                )
        }
        # Highlight points
        if (sum(data$.highlight) > 0) {
            hi_df <- data[data$.highlight, , drop = FALSE]
            p <- p +
                scattermore::geom_scattermore(
                    data = hi_df,
                    aes(x = !!sym(x), y = !!sym(y)),
                    color = highlight_color,
                    pointsize = floor(highlight_size) + highlight_stroke,
                    alpha = highlight_alpha,
                    pixels = raster_dpi,
                    inherit.aes = FALSE
                ) +
                scattermore::geom_scattermore(
                    data = hi_df,
                    aes(color = !!sym(group_by)),
                    pointsize = floor(highlight_size),
                    alpha = highlight_alpha,
                    pixels = raster_dpi
                )
            rm(hi_df)
        }
    } else {
        # Normal points
        p <- p +
            geom_point(
                aes(color = !!sym(group_by)),
                size = pt_size,
                shape = pt_shape,
                alpha = alpha
            )
        # Highlight points
        if (sum(data$.highlight) > 0) {
            hi_df <- data[data$.highlight, , drop = FALSE]
            p <- p +
                geom_point(
                    data = hi_df,
                    aes(x = !!sym(x), y = !!sym(y)),
                    color = highlight_color,
                    size = highlight_size + highlight_stroke,
                    shape = pt_shape,
                    alpha = highlight_alpha,
                    inherit.aes = FALSE
                ) +
                geom_point(
                    data = hi_df,
                    aes(color = !!sym(group_by)),
                    size = highlight_size,
                    shape = pt_shape,
                    alpha = highlight_alpha
                )
            rm(hi_df)
        }
    }

    anno_x <- ifelse(grepl("left", anno_position), -Inf, Inf)
    anno_y <- ifelse(grepl("top", anno_position), Inf, -Inf)
    p <- p +
        geom_smooth(
            method = "lm",
            formula = y ~ x,
            se = smooth_se,
            color = smooth_color,
            linewidth = smooth_width,
            alpha = 0.5
        ) +
        geom_text_repel(
            data = annodata,
            parse = TRUE,
            hjust = 0,
            direction = "y",
            aes(label = !!sym("anno")),
            x = anno_x,
            y = anno_y,
            seed = seed,
            size = text_size_scale * anno_size,
            bg.color = anno_bg,
            bg.r = anno_bg_r,
            color = anno_fg,
            force = 0.5,
            max.overlaps = 100,
            segment.color = "transparent"
        ) +
        scale_color_manual(
            name = group_name %||% group_by,
            values = palette_this(
                levels(data[[group_by]]),
                palette = palette,
                palcolor = palcolor,
                reverse = palreverse
            )
        ) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% x,
            y = ylab %||% y
        ) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction
        )

    dims <- calculate_plot_dimensions(
        base_height = 4.5,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = nlevels(data[[group_by]]),
        legend_nchar = max(nchar(levels(data[[group_by]])))
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction
    )
}

#' Correlation scatter plot
#'
#' @description
#' Draws a scatter plot of two numeric variables with a linear regression
#' line, optional correlation statistics, and point highlighting.  This is
#' the public entry point that wraps \code{\link{CorPlotAtomic}} with
#' \code{split_by} support.
#'
#' Key features include \strong{group-based colouring} (\code{group_by}),
#' \strong{point highlighting} by expression, rowname, or index,
#' \strong{annotation items} (regression equation, R-squared, p-value,
#' Spearman/Pearson/Kendall rho, N), \strong{raster rendering} for large
#' datasets, \strong{faceting} (\code{facet_by}), and \strong{splitting}
#' into separate sub-plots via \code{split_by}.
#'
#' @section split_by workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item The \code{split_by} column is validated via
#'         \code{\link{check_columns}()} with \code{force_factor = TRUE}.
#'         Empty levels are dropped (\code{droplevels()}).
#'   \item The data frame is split by \code{split_by} (preserving level
#'         order).  If \code{split_by} is \code{NULL}, the data is wrapped
#'         in a single-element list with name \code{"..."}.
#'   \item Per-split \code{palette}, \code{palcolor},
#'         \code{legend.position}, and \code{legend.direction} are resolved
#'         via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
#'         and \code{\link{check_legend}()}.
#'   \item \code{\link{CorPlotAtomic}()} is called for each split.  When
#'         \code{title} is a function, it receives the split level name and
#'         can generate dynamic titles.
#'   \item Results are combined via \code{\link{combine_plots}()} (when
#'         \code{combine = TRUE}) or returned as a named list.
#' }
#'
#' @inheritParams common_args
#' @inheritParams CorPlotAtomic
#' @param split_by The column(s) to split the data by and produce separate
#'  sub-plots.  Multiple columns are concatenated with \code{split_by_sep}.
#' @param split_by_sep A character string to separate concatenated
#'  \code{split_by} columns.  Default \code{"_"}.
#' @param seed A numeric seed for reproducibility.  Passed to
#'  \code{\link{validate_common_args}()}.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'  \code{patchwork} object.  When \code{FALSE}, returns a named list of
#'  individual \code{ggplot} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined
#'  layout (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param byrow Logical; fill the combined layout by row.  Default
#'  \code{TRUE}.
#' @param axes A character string specifying how axes should be treated
#'  across the combined layout (passed to
#'  \code{\link[patchwork]{wrap_plots}}).
#' @param axis_titles A character string specifying how axis titles should
#'  be treated across the combined layout.  Defaults to \code{axes}.
#' @param guides A character string specifying how guides (legends) should
#'  be collected across panels (passed to \code{\link{combine_plots}()}).
#' @param design A custom layout design for the combined plot (passed to
#'  \code{\link{combine_plots}()}).
#' @return A \code{ggplot} object (when \code{split_by} is \code{NULL}),
#'  a \code{patchwork} object (when \code{combine = TRUE}), or a named
#'  list of \code{ggplot} objects (when \code{combine = FALSE}), each
#'  with \code{height} and \code{width} attributes in inches.
#' @export
#' @examples
#' \donttest{
#' data(iris)
#'
#' # Basic scatter with group colours
#' CorPlot(iris, "Sepal.Length", "Sepal.Width", group_by = "Species")
#'
#' # Highlight a specific group with custom stroke
#' CorPlot(iris, "Sepal.Length", "Sepal.Width", group_by = "Species",
#'     highlight = 'Species == "setosa"', highlight_stroke = 1.5,
#'     anno_items = c("eq", "pearson"), anno_position = "bottomright")
#'
#' # Faceted by species
#' CorPlot(iris, "Sepal.Length", "Sepal.Width", facet_by = "Species",
#'     facet_scales = "free")
#'
#' # Per-split palettes
#' CorPlot(iris, "Sepal.Length", "Sepal.Width", split_by = "Species",
#'     palette = c(setosa = "Set1", versicolor = "Dark2", virginica = "Paired"))
#' }
CorPlot <- function(
    data,
    x,
    y,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    split_by = NULL,
    split_by_sep = "_",
    pt_size = 2,
    pt_shape = 16,
    raster = FALSE,
    alpha = 1,
    raster_dpi = c(512, 512),
    highlight = NULL,
    highlight_color = "black",
    highlight_size = 1,
    highlight_alpha = 1,
    highlight_stroke = 0.8,
    anno_items = c("eq", "r2", "p"),
    anno_size = 3,
    anno_fg = "black",
    anno_bg = "white",
    anno_bg_r = 0.1,
    anno_position = c(
        "topleft",
        "topright",
        "bottomleft",
        "bottomright",
        "tl",
        "tr",
        "bl",
        "br"
    ),
    add_smooth = TRUE,
    smooth_color = "red2",
    smooth_width = 1.5,
    smooth_se = FALSE,
    theme = "theme_this",
    theme_args = list(),
    palette = ifelse(is.null(group_by), "Spectral", "Paired"),
    palcolor = NULL,
    palreverse = FALSE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    legend.position = waiver(),
    legend.direction = "vertical",
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    theme <- process_theme(theme)
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }

    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            CorPlotAtomic(
                data = datas[[nm]],
                x = x,
                y = y,
                group_by = group_by,
                group_by_sep = group_by_sep,
                group_name = group_name,
                pt_size = pt_size,
                pt_shape = pt_shape,
                raster = raster,
                raster_dpi = raster_dpi,
                highlight = highlight,
                highlight_color = highlight_color,
                highlight_size = highlight_size,
                highlight_alpha = highlight_alpha,
                highlight_stroke = highlight_stroke,
                anno_items = anno_items,
                anno_size = anno_size,
                anno_fg = anno_fg,
                anno_bg = anno_bg,
                anno_bg_r = anno_bg_r,
                anno_position = anno_position,
                add_smooth = add_smooth,
                smooth_color = smooth_color,
                smooth_width = smooth_width,
                smooth_se = smooth_se,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                alpha = alpha,
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                seed = seed,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

#' Atomic Correlation Pairs Plot
#'
#' @description
#' Core implementation for drawing a correlation pairs (scatterplot matrix)
#' grid.  This is the workhorse behind the exported
#' \code{\link{CorPairsPlot}} — it takes a \strong{single} data frame (no
#' \code{split_by} support) and returns a \code{patchwork} object.
#'
#' The grid arranges all pairwise scatter plots of selected columns.  The
#' upper or lower triangle displays correlation tiles (fill) while the
#' opposite triangle displays scatter plots with regression lines.  The
#' diagonal can show density plots, violin plots, histograms, box plots,
#' or a simple diagonal line.
#'
#' The function supports \strong{four layout orientations} (\code{layout}),
#' \strong{three correlation methods} (\code{cor_method}), configurable
#' diagonal plots using other plotthis functions, and custom correlation
#' tile formatting.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
#'         \code{ggplot2::ggplot} based on
#'         \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{facet_by guard} — raises an error if \code{facet_by} is
#'         provided (not supported in pairs plots; use \code{split_by}
#'         instead).
#'   \item \strong{Column resolution} — when \code{columns} is \code{NULL},
#'         all columns (except \code{group_by}) are used.  Otherwise,
#'         \code{columns} are validated via \code{\link{check_columns}()}.
#'         At least two columns are required.
#'   \item \strong{Default diag_type} — when \code{diag_type} is \code{NULL},
#'         it defaults to \code{"density"} (no \code{group_by}) or
#'         \code{"violin"} (with \code{group_by}).
#'   \item \strong{Layout determination} — \code{get_plot_info()} is called
#'         for each cell \code{(i, j)} to determine:
#'         \itemize{
#'           \item \strong{Type}: \code{layout} (diagonal line),
#'                 \code{"cor"} (scatter plot with regression),
#'                 \code{"fill"} (correlation tile).
#'           \item \strong{Axis/label positions}: which sides of the cell
#'                 should show x-axis, y-axis, x-label, and y-label, based
#'                 on the cell's location in the matrix and the chosen
#'                 layout.
#'         }
#'   \item \strong{Cell rendering} — \code{get_plot()} draws each cell
#'         based on its \code{info$type}:
#'         \itemize{
#'           \item \strong{Diagonal, \code{diag_type = "none"}} — draws an
#'                 X or backslash diagonal line via \code{geom_line()}.
#'           \item \strong{Diagonal, \code{"density"}} — delegates to
#'                 \code{\link{DensityPlot}()}.
#'           \item \strong{Diagonal, \code{"violin"}} — delegates to
#'                 \code{\link{ViolinPlot}()}; requires \code{group_by}.
#'           \item \strong{Diagonal, \code{"histogram"}} — delegates to
#'                 \code{\link{Histogram}()}.
#'           \item \strong{Diagonal, \code{"box"}} — delegates to
#'                 \code{\link{BoxPlot}()}; requires \code{group_by}.
#'           \item \strong{Off-diagonal, \code{"cor"}} — delegates to
#'                 \code{\link{CorPlot}()} for the scatter plot with
#'                 regression line.
#'           \item \strong{Off-diagonal, \code{"fill"}} — draws a
#'                 \code{geom_tile()} filled by the correlation value
#'                 with \code{scale_fill_gradientn()} and displays the
#'                 formatted correlation via
#'                 \code{ggrepel::geom_text_repel()}.
#'         }
#'   \item \strong{Axis/label positioning} — axes, axis titles, and labels
#'         are positioned at the margins of the full grid based on
#'         \code{info$xaxis}, \code{info$yaxis}, \code{info$xlab},
#'         \code{info$ylab} using \code{scale_x_*}/\code{scale_y_*} with
#'         appropriate \code{position} arguments.
#'   \item \strong{Layout assembly} — all cell plots are arranged into a
#'         matrix via \code{patchwork::wrap_plots()} with
#'         \code{ncol = length(columns)}.  Guides are collected, and the
#'         title/subtitle are added via \code{plot_annotation()}.  The
#'         result is wrapped in \code{patchwork::wrap_elements()} so the
#'         title displays correctly.
#'   \item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
#'         computes \code{height} and \code{width} attributes from
#'         \code{base_height = sqrt(length(columns)) * 4}, a fixed aspect
#'         ratio of 1, and legend metrics.
#' }
#'
#' @inheritParams common_args
#' @param columns A character vector of column names to include in the
#'  pairs plot.  When \code{NULL} (default), all columns except
#'  \code{group_by} are used.  At least two columns are required.
#' @param group_by A character vector of column names to colour the
#'  scatter points by.  Each unique combination becomes a separate group.
#'  Required for \code{diag_type = "violin"} and \code{diag_type = "box"}.
#'  Multiple columns are concatenated with \code{group_by_sep}.
#' @param group_by_sep A character string to separate concatenated
#'  \code{group_by} columns.  Default \code{"_"}.
#' @param group_name A character string used as the colour legend title
#'  in the scatter plots.  When \code{NULL}, the \code{group_by} column
#'  name is used.
#' @param diag_type A character string specifying the plot type for
#'  diagonal cells.  One of \code{"density"}, \code{"violin"},
#'  \code{"histogram"}, \code{"box"}, or \code{"none"} (diagonal line).
#'  Default: \code{"density"} (no \code{group_by}) or \code{"violin"}
#'  (with \code{group_by}).
#' @param diag_args A named list of additional arguments passed to the
#'  diagonal plot function (\code{\link{DensityPlot}},
#'  \code{\link{ViolinPlot}}, \code{\link{Histogram}}, or
#'  \code{\link{BoxPlot}}).  Default: \code{list()}.
#' @param layout A character string specifying the layout orientation.
#'  One of the following codes (dot = scatter, backslash/slash =
#'  diagonal): \code{.\\}, \code{\\\.}, \code{/.}, \code{./}.
#'  Default: \code{.\\}.
#' @param cor_method A character string specifying the correlation method
#'  for the fill tiles.  One of \code{"pearson"}, \code{"spearman"},
#'  \code{"kendall"}.  Default: \code{"pearson"}.
#' @param cor_palette A character string specifying the colour palette for
#'  the correlation fill tiles.  Default: \code{"RdBu"}.
#' @param cor_palcolor A character vector of custom colours used to create
#'  the correlation tile palette.  When \code{NULL}, the palette's default
#'  colours are used.
#' @param cor_size A numeric value specifying the font size of the
#'  correlation text in the fill tiles.  Default: \code{3}.
#' @param cor_format A character string specifying a glue template for
#'  formatting the correlation text.  The template is evaluated by
#'  \code{glue::glue()} with access to \code{corr} (the correlation value),
#'  \code{x}, and \code{y} (the column names).  Default:
#'  \code{"corr: \\{round(corr, 2)\\}"}.
#' @param cor_fg A character string specifying the colour of the
#'  correlation text.  Default: \code{"black"}.
#' @param cor_bg A character string specifying the background colour of
#'  the correlation text boxes.  Default: \code{"white"}.
#' @param cor_bg_r A numeric value specifying the corner radius of the
#'  correlation text background boxes.  Default: \code{0.1}.
#' @param ... Additional arguments passed to the underlying
#'  \code{\link{CorPlot}} for the scatter plot cells.
#' @details \code{theme} and \code{theme_args} are supported and passed to
#'   each individual cell plot.
#' @return A \code{patchwork} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @importFrom ggplot2 waiver geom_tile element_blank scale_fill_gradientn guide_colorbar scale_x_continuous
#' @importFrom ggplot2 geom_line scale_y_continuous
#' @importFrom ggrepel geom_text_repel
#' @importFrom patchwork wrap_plots plot_layout plot_annotation
#' @keywords internal
CorPairsPlotAtomic <- function(
    data,
    columns = NULL,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    diag_type = NULL,
    diag_args = list(),
    layout = c(".\\", "\\.", "/.", "./"),
    cor_method = c("pearson", "spearman", "kendall"),
    cor_palette = "RdBu",
    cor_palcolor = NULL,
    cor_size = 3,
    cor_format = "corr: {round(corr, 2)}",
    cor_fg = "black",
    cor_bg = "white",
    cor_bg_r = 0.1,
    theme = "theme_this",
    theme_args = list(),
    palette = ifelse(is.null(group_by), "Spectral", "Paired"),
    palcolor = NULL,
    palreverse = FALSE,
    title = NULL,
    subtitle = NULL,
    facet_by = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    seed = 8525,
    ...
) {
    set.seed(seed)
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    if (!is.null(facet_by)) {
        stop(
            "'facet_by' is not supported in CorPairsPlot. Consider using 'split_by'."
        )
    }
    if (is.null(columns)) {
        if (!is.null(group_by)) {
            columns <- setdiff(colnames(data), group_by)
        } else {
            columns <- colnames(data)
        }
    } else {
        columns <- check_columns(data, columns, allow_multi = TRUE)
    }
    if (length(columns) < 2) {
        stop("At least two columns are required.")
    }
    if (is.null(diag_type)) {
        diag_type <- ifelse(is.null(group_by), "density", "violin")
    }
    diag_type <- match.arg(
        diag_type,
        c("density", "violin", "histogram", "box", "none")
    )
    layout <- match.arg(layout)
    cor_method <- match.arg(cor_method)
    base_size <- theme_args$base_size %||% 12
    text_size_scale <- base_size / 12

    # get the plot info (plotype, axis position, etc.) based on the layout
    get_plot_info <- function(i, j, j1) {
        diag_no_x_axis <- diag_type %in% c("box", "violin", "none")
        xaxis <- yaxis <- xlab <- ylab <- "none"
        if (i == j1) {
            # diagonal
            if (layout %in% c(".\\", "/.")) {
                if (i == 1) {
                    # top corner
                    yaxis <- ifelse(
                        diag_type == "none",
                        "none",
                        ifelse(layout == ".\\", "left", "right")
                    )
                    xlab <- "top"
                } else if (i == length(columns)) {
                    # bottom corner
                    xaxis <- ifelse(diag_no_x_axis, "none", "bottom")
                    ylab <- ifelse(layout == ".\\", "right", "left")
                }
            } else {
                # layout %in% c("\\.", "./")
                if (i == 1) {
                    # top corner
                    xaxis <- ifelse(diag_no_x_axis, "none", "top")
                    xlab <- "none"
                    ylab <- ifelse(layout == "\\.", "left", "right")
                } else if (i == length(columns)) {
                    # bottom corner
                    yaxis <- ifelse(
                        diag_type == "none",
                        "none",
                        ifelse(layout == "\\.", "right", "left")
                    )
                    xlab <- "bottom"
                }
            }
            return(list(
                type = layout,
                xaxis = xaxis,
                yaxis = yaxis,
                xlab = xlab,
                ylab = ylab
            ))
        } else if (i < j1) {
            # upper triangle
            if (layout %in% c(".\\", "/.")) {
                xlab <- ifelse(i > 1, "none", "top")
                ylab <- ifelse(
                    j1 < length(columns),
                    "none",
                    ifelse(layout == "/.", "left", "right")
                )
                return(list(
                    type = "fill",
                    xaxis = xaxis,
                    yaxis = yaxis,
                    xlab = xlab,
                    ylab = ylab
                ))
            } else {
                # layout %in% c("\\.", "./")
                xaxis <- ifelse(i > 1, "none", "top")
                yaxis <- ifelse(
                    j1 < length(columns),
                    "none",
                    ifelse(layout == "\\.", "right", "left")
                )
                return(list(
                    type = "cor",
                    xaxis = xaxis,
                    yaxis = yaxis,
                    xlab = xlab,
                    ylab = ylab
                ))
            }
        } else {
            # i > j1, lower triangle
            if (layout %in% c(".\\", "/.")) {
                xaxis <- ifelse(i < length(columns), "none", "bottom")
                yaxis <- ifelse(
                    j1 > 1,
                    "none",
                    ifelse(layout == "/.", "right", "left")
                )
                return(list(
                    type = "cor",
                    xaxis = xaxis,
                    yaxis = yaxis,
                    xlab = xlab,
                    ylab = ylab
                ))
            } else {
                # layout %in% c("\\.", "./")
                xlab <- ifelse(i < length(columns), "none", "bottom")
                ylab <- ifelse(
                    j1 > 1,
                    "none",
                    ifelse(layout == "\\.", "left", "right")
                )
                return(list(
                    type = "fill",
                    xaxis = xaxis,
                    yaxis = yaxis,
                    xlab = xlab,
                    ylab = ylab
                ))
            }
        }
    }

    get_plot <- function(x, y, info) {
        # Use facet to add label
        expand <- c(0, 0)
        if (diag_type == "none") {
            if (info$type %in% c("./", "/.")) {
                p <- ggplot(
                    data.frame(x = c(0, 1), y = c(0, 1)),
                    aes(x = x, y = y)
                ) +
                    geom_line(color = "darkred", linewidth = 1.5) +
                    scale_x_continuous(expand = expand) +
                    scale_y_continuous(expand = expand)
            } else if (info$type %in% c(".\\", "\\.")) {
                p <- ggplot(
                    data.frame(x = c(0, 1), y = c(1, 0)),
                    aes(x = x, y = y)
                ) +
                    geom_line(color = "darkred", linewidth = 1.5) +
                    scale_x_continuous(expand = expand) +
                    scale_y_continuous(expand = expand)
            }
        } else if (info$type %in% c(".\\", "\\.", "/.", "./")) {
            info$type <- diag_type
        }
        if (info$type == "density") {
            # Use guides instead of legend.position to remove the legend
            # This way & theme(legend.position = ...) can work as expected
            args <- diag_args
            args$data <- data
            args$x <- x
            args$group_by <- group_by
            args$add_bars <- args$add_bars %||% FALSE
            args$palette <- args$palette %||% palette
            args$palcolor <- args$palcolor %||% palcolor
            p <- do_call(DensityPlot, args)
            if (
                is.null(group_by) ||
                    (identical(args$palette, palette) &&
                        identical(args$palcolor, palcolor))
            ) {
                p <- p + guides(fill = "none", color = "none")
            }
        } else if (info$type == "violin") {
            if (is.null(group_by)) {
                stop(
                    "No 'group_by' is specified for 'violin' plot on the diagnal cells. Consider using 'density' instead."
                )
            }
            args <- diag_args
            args$data <- data
            args$x <- group_by
            args$y <- x
            args$palette <- args$palette %||% palette
            args$palcolor <- args$palcolor %||% palcolor
            p <- do_call(ViolinPlot, args)
            if (
                identical(args$palette, palette) &&
                    identical(args$palcolor, palcolor)
            ) {
                p <- p + guides(fill = "none")
            }
        } else if (info$type == "histogram") {
            args <- diag_args
            args$data <- data
            args$x <- x
            args$group_by <- group_by
            args$palette <- args$palette %||% palette
            args$palcolor <- args$palcolor %||% palcolor
            p <- do_call(Histogram, args)
            if (
                is.null(group_by) ||
                    (identical(args$palette, palette) &&
                        identical(args$palcolor, palcolor))
            ) {
                p <- p + guides(fill = "none", color = "none")
            }
        } else if (info$type == "box") {
            if (is.null(group_by)) {
                stop(
                    "No 'group_by' is specified for 'box' plot on the diagnal cells. Consider using 'histogram' instead."
                )
            }
            args <- diag_args
            args$data <- data
            args$x <- group_by
            args$y <- x
            args$palette <- args$palette %||% palette
            args$palcolor <- args$palcolor %||% palcolor
            p <- do_call(BoxPlot, args)
            if (
                identical(args$palette, palette) &&
                    identical(args$palcolor, palcolor)
            ) {
                p <- p + guides(fill = "none")
            }
        } else if (info$type == "cor") {
            expand <- waiver()
            p <- CorPlot(
                data,
                x = x,
                y = y,
                group_by = group_by,
                group_name = group_name,
                palette = palette,
                palcolor = palcolor,
                legend.position = legend.position,
                legend.direction = legend.direction,
                seed = seed,
                ...
            )
            if (is.null(group_by)) {
                p <- p + guides(fill = "none", color = "none")
            }
        } else if (info$type == "fill") {
            corr <- cor(data[[x]], data[[y]], method = cor_method)
            # x, y for cor_format
            p <- ggplot(
                data.frame(i = 0.5, j = 0.5, x = y, y = x, corr = corr),
                aes(x = i, y = j, fill = corr)
            ) +
                geom_tile(width = 1, height = 1) +
                geom_text_repel(
                    mapping = aes(label = glue(cor_format)),
                    segment.color = "transparent",
                    force = 0,
                    color = cor_fg,
                    bg.color = cor_bg,
                    bg.r = cor_bg_r,
                    size = text_size_scale * cor_size,
                    seed = seed
                ) +
                scale_fill_gradientn(
                    colors = palette_this(
                        c(-1, 1),
                        palette = cor_palette,
                        palcolor = cor_palcolor,
                        reverse = palreverse
                    ),
                    limits = c(-1, 1),
                    breaks = c(-1, 0, 1),
                    labels = scales::number_format(accuracy = 0.1),
                    guide = guide_colorbar(
                        frame.colour = "black",
                        ticks.colour = "black",
                        title.hjust = 0
                    )
                ) +
                scale_x_continuous(expand = expand) +
                scale_y_continuous(expand = expand)
        }
        p <- p + do_call(theme, theme_args)

        suppressMessages({
            # place the axis and labels
            if (info$type %in% c("violin", "box")) {
                scale_x <- scale_x_discrete
                expand <- waiver()
            } else {
                scale_x <- scale_x_continuous
            }
            if (info$xlab == "top") {
                p <- p +
                    ggplot2::xlab(x) +
                    scale_x(position = "top", expand = expand) +
                    ggplot2::theme(
                        axis.title.x = element_textbox(face = "bold")
                    )
            } else if (info$xlab == "bottom") {
                p <- p +
                    ggplot2::xlab(x) +
                    scale_x(expand = expand) +
                    ggplot2::theme(
                        axis.title.x = element_textbox(face = "bold")
                    )
            } else {
                p <- p + ggplot2::theme(axis.title.x = element_blank())
            }
            if (info$ylab == "left") {
                p <- p +
                    ggplot2::ylab(y) +
                    scale_y_continuous(expand = expand) +
                    ggplot2::theme(
                        axis.title.y = element_textbox(
                            orientation = "left-rotated"
                        )
                    )
            } else if (info$ylab == "right") {
                p <- p +
                    ggplot2::ylab(y) +
                    scale_y_continuous(position = "right", expand = expand) +
                    ggplot2::theme(
                        axis.title.y = element_textbox(
                            orientation = "right-rotated"
                        )
                    )
            } else {
                p <- p + ggplot2::theme(axis.title.y = ggplot2::element_blank())
            }
            if (info$xaxis == "bottom") {
                p <- p + scale_x(expand = expand)
            } else if (info$xaxis == "top") {
                p <- p + scale_x(position = "top", expand = expand)
            } else {
                p <- p +
                    ggplot2::theme(
                        axis.text.x = ggplot2::element_blank(),
                        axis.ticks.x = ggplot2::element_blank()
                    )
            }
            if (info$yaxis == "left") {
                p <- p + scale_y_continuous(expand = expand)
            } else if (info$yaxis == "right") {
                p <- p + scale_y_continuous(position = "right", expand = expand)
            } else {
                p <- p +
                    ggplot2::theme(
                        axis.text.y = ggplot2::element_blank(),
                        axis.ticks.y = ggplot2::element_blank()
                    )
            }
        })
        p + ggplot2::theme(plot.margin = margin(1, 1, 1, 1), aspect.ratio = 1)
    }

    plots <- list()
    for (i in seq_along(columns)) {
        for (j in seq_along(columns)) {
            j1 <- ifelse(layout %in% c("./", "/."), length(columns) - j + 1, j)
            info <- get_plot_info(i, j, j1)
            # For debugging
            # info$i <- i
            # info$j <- j
            # info$j1 <- j1
            # str(info)
            p <- get_plot(columns[j1], columns[i], info)
            plots <- c(plots, list(p))
        }
    }

    p <- wrap_plots(plots, ncol = length(columns), byrow = TRUE) +
        plot_layout(guides = "collect") +
        plot_annotation(title = title, subtitle = subtitle) &
        ggplot2::theme(
            legend.position = legend.position,
            legend.direction = legend.direction
        )
    # so that the plot title can be displayed
    p <- patchwork::wrap_elements(p)

    dims <- calculate_plot_dimensions(
        base_height = sqrt(length(columns)) * 4,
        aspect.ratio = 1, # pairs plot panels are always square
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = if (!is.null(group_by)) nlevels(data[[group_by]]) else 1,
        legend_nchar = if (!is.null(group_by)) {
            max(nchar(levels(data[[group_by]])))
        } else {
            5
        }
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    p$data <- data
    p
}

#' Correlation pairs (scatterplot matrix)
#'
#' @description
#' Draws a grid of pairwise scatter plots for selected numeric columns,
#' arranged in a scatterplot matrix layout.  The upper or lower triangle
#' displays correlation tiles while the opposite triangle shows scatter
#' plots with regression lines.  Diagonal cells can show density plots,
#' violin plots, histograms, box plots, or a simple diagonal line.
#'
#' \strong{NOTE:} The \code{facet_by} parameter is \strong{not supported}
#' in CorPairsPlot (an error is raised if provided).  Use
#' \code{split_by} instead to create separate correlation pair matrices
#' per group.
#'
#' The function supports \strong{four layout orientations} (\code{layout}),
#' \strong{three correlation methods}, configurable diagonal plots via
#' other plotthis functions, custom correlation tile formatting, and
#' splitting into separate sub-plots via \code{split_by}.
#'
#' @section split_by workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item The \code{split_by} column is validated via
#'         \code{\link{check_columns}()} with \code{force_factor = TRUE}.
#'         Empty levels are dropped.
#'   \item The data frame is split by \code{split_by} (preserving level
#'         order).  If \code{split_by} is \code{NULL}, the data is wrapped
#'         in a single-element list with name \code{"..."}.  The
#'         \code{split_by} column is removed from each split's data before
#'         plotting.
#'   \item Per-split \code{palette}, \code{palcolor},
#'         \code{legend.position}, and \code{legend.direction} are resolved
#'         via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
#'         and \code{\link{check_legend}()}.
#'   \item \code{\link{CorPairsPlotAtomic}()} is called for each split.
#'         When \code{title} is a function, it receives the split level
#'         name and can generate dynamic titles.
#'   \item Results are combined via \code{\link{combine_plots}()} (when
#'         \code{combine = TRUE}) or returned as a named list.
#' }
#'
#' @inheritParams common_args
#' @inheritParams CorPairsPlotAtomic
#' @param split_by The column(s) to split the data by and produce separate
#'  sub-plots.  Multiple columns are concatenated with \code{split_by_sep}.
#' @param split_by_sep A character string to separate concatenated
#'  \code{split_by} columns.  Default \code{"_"}.
#' @param seed A numeric seed for reproducibility.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'  \code{patchwork} object.  When \code{FALSE}, returns a named list of
#'  individual \code{patchwork} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined
#'  layout.
#' @param byrow Logical; fill the combined layout by row.  Default
#'  \code{TRUE}.
#' @param axes A character string specifying how axes should be treated
#'  across the combined layout.
#' @param axis_titles A character string specifying how axis titles should
#'  be treated across the combined layout.  Defaults to \code{axes}.
#' @param guides A character string specifying how guides (legends) should
#'  be collected across panels.
#' @param design A custom layout design for the combined plot.
#' @return A \code{patchwork} object (when \code{combine = TRUE}) or a
#'  named list of \code{patchwork} objects (when \code{combine = FALSE}),
#'  each with \code{height} and \code{width} attributes in inches.
#' @export
#' @importFrom glue glue
#' @examples
#' \donttest{
#' set.seed(8525)
#' data <- data.frame(x = rnorm(100))
#' data$y <- rnorm(100, 10, sd = 0.5)
#' data$z <- -data$x + data$y + rnorm(100, 20, 1)
#' data$g <- sample(1:4, 100, replace = TRUE)
#'
#' # Histogram diagonal, slash layout
#' CorPairsPlot(data, diag_type = "histogram",
#'     diag_args = list(bins = 30, palette = "Paired"),
#'     layout = "/.")
#'
#' # No diagonal with axis title styling
#' CorPairsPlot(data, group_by = "g", diag_type = "none", layout = "./",
#'     theme_args = list(axis.title = element_textbox(
#'         color = "black", box.color = "grey20", size = 16, halign = 0.5,
#'         fill = "grey90", linetype = 1,
#'         width = grid::unit(1, "npc"),
#'         padding = ggplot2::margin(5, 5, 5, 5))))
#'
#' # Violin diagonal with custom format
#' CorPairsPlot(data, group_by = "g", diag_type = "violin", layout = "\\.",
#'     cor_format = "{x}\n{y}\ncorr: {round(corr, 2)}")
#'
#' # Per-split with bottom legend
#' CorPairsPlot(data, split_by = "g", diag_type = "none", layout = ".\\",
#'     legend.position = "bottom", legend.direction = "horizontal",
#'     group_name = "group")
#'
#' # Per-split with custom palette colours
#' CorPairsPlot(data, split_by = "g",
#'     palcolor = list("1" = "red", "2" = "blue", "3" = "green",
#'                     "4" = "yellow"))
#' }
CorPairsPlot <- function(
    data,
    columns = NULL,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    split_by = NULL,
    split_by_sep = "_",
    diag_type = NULL,
    diag_args = list(),
    layout = c(".\\", "\\.", "/.", "./"),
    cor_method = c("pearson", "spearman", "kendall"),
    cor_palette = "RdBu",
    cor_palcolor = NULL,
    cor_size = 3,
    cor_format = "corr: {round(corr, 2)}",
    cor_fg = "black",
    cor_bg = "white",
    cor_bg_r = 0.1,
    theme = "theme_this",
    theme_args = list(),
    palette = ifelse(is.null(group_by), "Spectral", "Paired"),
    palcolor = NULL,
    palreverse = FALSE,
    title = NULL,
    subtitle = NULL,
    facet_by = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed)
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }

    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            if (!is.null(split_by)) {
                datas[[nm]][[split_by]] <- NULL
            }

            CorPairsPlotAtomic(
                data = datas[[nm]],
                columns = columns,
                group_by = group_by,
                group_by_sep = group_by_sep,
                group_name = group_name,
                diag_type = diag_type,
                diag_args = diag_args,
                layout = layout,
                cor_method = cor_method,
                cor_palette = cor_palette,
                cor_palcolor = cor_palcolor,
                cor_size = cor_size,
                cor_format = cor_format,
                cor_fg = cor_fg,
                cor_bg = cor_bg,
                cor_bg_r = cor_bg_r,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                title = title,
                subtitle = subtitle,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                seed = seed,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
cor_data <- data.frame(
    v1 = rnorm(80),
    v2 = rnorm(80),
    v3 = rnorm(80),
    group = factor(rep(c("A", "B"), 40)),
    facet = factor(rep(c("f1", "f2"), each = 40))
)
cor_data$v2 <- cor_data$v1 * 0.8 + rnorm(80, sd = 0.3)
cor_data$v3 <- cor_data$v1 * 0.4 + rnorm(80)

test_that("CorPlot returns a ggplot for basic usage", {
    p <- CorPlot(cor_data, x = "v1", y = "v2")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("CorPlot with group_by works", {
    p <- CorPlot(cor_data, x = "v1", y = "v2", group_by = "group")
    expect_s3_class(p, "ggplot")
})

test_that("CorPlot sets title and labels", {
    p <- CorPlot(cor_data, x = "v1", y = "v2", title = "Correlation", xlab = "V1", ylab = "V2")
    expect_equal(p$labels$title, "Correlation")
    expect_equal(p$labels$x, "V1")
    expect_equal(p$labels$y, "V2")
})

test_that("CorPlot with split_by and combine = FALSE returns list", {
    plots <- CorPlot(cor_data, x = "v1", y = "v2", split_by = "facet", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("CorPlot with multiple split_by columns returns a list with one plot per combination", {
    plots <- suppressMessages(CorPlot(cor_data, x = "v1", y = "v2", split_by = c("group", "facet"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("CorPlot with facet_by works", {
    p <- CorPlot(cor_data, x = "v1", y = "v2", facet_by = "facet")
    expect_s3_class(p, "ggplot")
})

test_that("CorPlot with add_smooth = FALSE works", {
    p <- CorPlot(cor_data, x = "v1", y = "v2", add_smooth = FALSE)
    expect_s3_class(p, "ggplot")
})

test_that("CorPlot with different anno_items works", {
    p <- CorPlot(cor_data, x = "v1", y = "v2", anno_items = c("r2", "p"))
    expect_s3_class(p, "ggplot")
})

test_that("CorPlot width/height attributes are numeric", {
    p <- CorPlot(cor_data, x = "v1", y = "v2")
    expect_true(is.numeric(attr(p, "width")))
    expect_true(is.numeric(attr(p, "height")))
})

# CorPairsPlot tests
test_that("CorPairsPlot returns a ggplot for basic usage", {
    p <- CorPairsPlot(cor_data, columns = c("v1", "v2", "v3"))
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("CorPairsPlot with group_by works", {
    p <- CorPairsPlot(cor_data, columns = c("v1", "v2", "v3"), group_by = "group")
    expect_s3_class(p, "ggplot")
})

test_that("CorPairsPlot with split_by and combine = FALSE returns list", {
    plots <- CorPairsPlot(cor_data, columns = c("v1", "v2", "v3"),
                          split_by = "facet", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
})

test_that("CorPairsPlot with multiple split_by columns returns a list with one plot per combination", {
    plots <- suppressMessages(CorPairsPlot(cor_data, columns = c("v1", "v2", "v3"),
                                           split_by = c("group", "facet"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
})

test_that("CorPairsPlot with title does not error", {
    # CorPairsPlot returns a patchwork::wrap_elements object, not a plain ggplot
    p <- CorPairsPlot(cor_data, columns = c("v1", "v2", "v3"), title = "Pairs")
    # Should be some plot-like object with height/width attributes set
    expect_true(!is.null(p))
    expect_true(!is.null(attr(p, "height")))
})

#' Single-series line plot (internal)
#'
#' @description
#' Core implementation for drawing a single-series line plot.  This is the
#' workhorse behind \code{\link{LinePlotAtomic}} for ungrouped data -- it takes a
#' **single** data frame (no \code{split_by} support) and returns a
#' \code{ggplot} object.  Each x-axis category receives its own point and
#' connecting line, with optional per-x coloring, error bars, highlighted
#' points, and a horizontal reference line.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{ggplot dispatch} -- selects \code{gglogger::ggplot} or
#'         \code{ggplot2::ggplot} based on
#'         \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{Column resolution} -- \code{x} (forced to factor),
#'         \code{y}, and \code{facet_by} are validated via
#'         \code{\link{check_columns}}.
#'   \item \strong{Count aggregation} -- when \code{y = NULL}, the count of
#'         observations per x level (and facet) is computed and used as
#'         \code{y}.
#'   \item \strong{Error bar validation} -- if \code{add_errorbars = TRUE},
#'         checks that either \code{errorbar_min}/\code{errorbar_max} or
#'         \code{errorbar_sd} is provided.
#'   \item \strong{NA / empty-level handling} --
#'         \code{\link{process_keep_na_empty}()} applies \code{keep_na} and
#'         \code{keep_empty} policies.
#'   \item \strong{Highlight parsing} -- \code{highlight} (indices, row
#'         names, expression string, or \code{TRUE}) is resolved to a subset
#'         of data rows.
#'   \item \strong{Background layer} -- when \code{add_bg = TRUE},
#'         alternating background stripes are drawn via
#'         \code{\link{bg_layer}()}.
#'   \item \strong{Horizontal reference line} -- when \code{add_hline} is
#'         numeric, a \code{geom_hline()} is added at that intercept.
#'   \item \strong{Colour assignment} -- \code{\link{palette_this}()}
#'         assigns colours to all x-axis levels (including \code{NA},
#'         defaulting to \code{"grey80"}).
#'   \item \strong{Line rendering} -- when \code{color_line_by_x = TRUE},
#'         lines are coloured per x level via
#'         \code{\link[ggplot2]{geom_line}()} with \code{aes(color = x)} and
#'         \code{scale_color_manual()}.  Otherwise a single colour (the
#'         first palette entry) is used.
#'   \item \strong{Error bars} -- when \code{add_errorbars = TRUE}, three
#'         colour modes are supported: follow the line colour
#'         (\code{"line"}), track x levels, or use a constant colour.
#'         Error bars are added via
#'         \code{\link[ggplot2]{geom_errorbar}()}.
#'   \item \strong{Point rendering} -- when \code{fill_point_by_x = TRUE},
#'         points are filled per x level via
#'         \code{\link[ggplot2]{geom_point}()} with \code{aes(fill = x)} and
#'         \code{scale_fill_manual()}.  Otherwise a single fill colour
#'         (first palette entry) is used.
#'   \item \strong{Highlight overlay} -- an additional \code{geom_point()}
#'         layer draws highlighted rows in the specified colour and size.
#'   \item \strong{Plot assembly} -- \code{scale_x_discrete()},
#'         \code{labs()}, theme application, axis styling (angle, grid
#'         lines), and legend positioning.
#'   \item \strong{Dimension calculation} --
#'         \code{\link{calculate_plot_dimensions}()} computes \code{height}
#'         and \code{width} attributes (in inches) from x-axis category
#'         count, aspect ratio, and legend metrics.
#' }
#'
#' @inheritParams common_args
#' @param x A character string specifying the column name for the x-axis.
#'  Must be character or factor.
#' @param y A character string specifying the numeric column for the y-axis.
#'  When \code{NULL}, the count of observations per x level is used.
#' @param fill_point_by_x A logical value. When TRUE (default), points are
#'  filled by the x-axis categories using the palette. When FALSE, all
#'  points use the first palette colour.
#' @param color_line_by_x A logical value. When TRUE (default), lines are
#'  coloured by the x-axis categories using the palette. When FALSE, all
#'  lines use the first palette colour.
#' @param add_bg A logical value. When TRUE, alternating background stripes
#'  are drawn via \code{\link{bg_layer}()}. Default FALSE.
#' @param bg_palette A character string specifying the palette for the
#'  background stripe colours. Default \code{"stripe"}.
#' @param bg_palcolor A character vector of colours for the background
#'  stripes. When NULL (default), colours are derived from
#'  \code{bg_palette}.
#' @param bg_alpha A numeric value in \code{[0, 1]} for the transparency
#'  of background stripes. Default 0.2.
#' @param add_errorbars A logical value. When TRUE, error bars are added via
#'  \code{\link[ggplot2]{geom_errorbar}()}. Requires \code{errorbar_sd} or
#'  \code{errorbar_min}/\code{errorbar_max}. Default FALSE.
#' @param errorbar_width A numeric value for the width of the error bar
#'  caps. Default 0.1.
#' @param errorbar_alpha A numeric value in \code{[0, 1]} for the transparency of
#'  error bars. Default 1.
#' @param errorbar_color A character string for the colour of the error
#'  bars. When \code{"line"}, error bars are coloured the same as the lines
#'  (by x when \code{color_line_by_x = TRUE}, or single colour otherwise).
#'  Default \code{"grey30"}.
#' @param errorbar_linewidth A numeric value for the line width of error
#'  bars. Default 0.75.
#' @param errorbar_min A character string naming the column with the lower
#'  error bar bound. Ignored when \code{errorbar_sd} is provided.
#' @param errorbar_max A character string naming the column with the upper
#'  error bar bound. Ignored when \code{errorbar_sd} is provided.
#' @param errorbar_sd A character string naming the column with the standard
#'  deviation. When \code{errorbar_min} and \code{errorbar_max} are not
#'  provided, error bars are computed as y +/- \code{errorbar_sd}.
#' @param highlight A vector of row indices, row names, a single string
#'  expression (e.g. \code{"y > 10"}) filtering rows to highlight, or TRUE
#'  to highlight all points. When NULL (default), no highlighting is
#'  applied.
#' @param highlight_size A numeric value for the size of highlighted points.
#'  Defaults to \code{pt_size - 0.75}.
#' @param highlight_color A character string for the colour of highlighted
#'  points. Default \code{"red2"}.
#' @param highlight_alpha A numeric value in \code{[0, 1]} for the transparency of
#'  highlighted points. Default 0.8.
#' @param pt_alpha A numeric value in \code{[0, 1]} for the transparency of points.
#'  Default 1.
#' @param pt_size A numeric value for the point size. Default 5.
#' @param add_hline A numeric value specifying the y-intercept of a
#'  horizontal reference line. When FALSE (default), no line is drawn.
#' @param hline_type A character string specifying the line type of the
#'  horizontal reference line. Default \code{"solid"}.
#' @param hline_width A numeric value for the width of the horizontal
#'  reference line. Default 0.5.
#' @param hline_color A character string for the colour of the horizontal
#'  reference line. Default \code{"black"}.
#' @param hline_alpha A numeric value in \code{[0, 1]} for the transparency of the
#'  horizontal reference line. Default 1.
#' @param line_type A character string specifying the line type. Default
#'  \code{"solid"}.
#' @param line_width A numeric value for the line width (in mm). Default 1.
#' @param line_alpha A numeric value in \code{[0, 1]} for the transparency of the
#'  line. Default 0.8.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches).
#' @keywords internal
#' @importFrom rlang sym parse_expr
#' @importFrom ggplot2 geom_line scale_color_manual labs geom_rect geom_errorbar geom_point
LinePlotSingle <- function(
    data,
    x,
    y = NULL,
    fill_point_by_x = TRUE,
    color_line_by_x = TRUE,
    facet_by = NULL,
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    add_errorbars = FALSE,
    errorbar_width = 0.1,
    errorbar_alpha = 1,
    errorbar_color = "grey30",
    errorbar_linewidth = .75,
    errorbar_min = NULL,
    errorbar_max = NULL,
    errorbar_sd = NULL,
    highlight = NULL,
    highlight_size = pt_size - 0.75,
    highlight_color = "red2",
    highlight_alpha = 0.8,
    pt_alpha = 1,
    pt_size = 5,
    add_hline = FALSE,
    hline_type = "solid",
    hline_width = 0.5,
    hline_color = "black",
    hline_alpha = 1,
    line_type = "solid",
    line_width = 1,
    line_alpha = .8,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    x_text_angle = 0,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_empty = FALSE,
    keep_na = FALSE,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    x <- check_columns(data, x, force_factor = TRUE)
    y <- check_columns(data, y)
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )

    if (is.null(y)) {
        orig_data <- data
        data <- data %>%
            group_by(!!!syms(unique(x, facet_by))) %>%
            summarise(.y = n(), .groups = "drop")
        for (col in unique(c(x, facet_by))) {
            data[[col]] <- factor(
                data[[col]],
                levels = levels(orig_data[[col]])
            )
        }
        rm(orig_data)
        y <- ".y"
    }

    if (isTRUE(add_errorbars)) {
        if (
            is.null(errorbar_sd) &&
                (is.null(errorbar_min) || is.null(errorbar_max))
        ) {
            stop(
                "If 'errorbar_min' and 'errorbar_max' are not provided, 'errorbar_sd' must be provided."
            )
        }
        if (is.null(errorbar_min) || is.null(errorbar_max)) {
            data$errorbar_min <- data[[y]] - data[[errorbar_sd]]
            data$errorbar_max <- data[[y]] + data[[errorbar_sd]]
            errorbar_min <- "errorbar_min"
            errorbar_max <- "errorbar_max"
        }
    }

    data <- process_keep_na_empty(data, keep_na, keep_empty)
    keep_empty_x <- keep_empty[[x]]

    hidata <- NULL
    if (!is.null(highlight)) {
        if (isTRUE(highlight)) {
            hidata <- data
        } else if (is.numeric(highlight)) {
            hidata <- data[highlight, , drop = FALSE]
        } else if (is.character(highlight) && length(highlight) == 1) {
            hidata <- filter(data, !!parse_expr(highlight))
        } else if (is.null(rownames(data))) {
            stop(
                "No row names in the data, please provide a vector of indexes to highlight."
            )
        } else {
            hidata <- data[highlight, , drop = FALSE]
        }
    }

    p <- ggplot(data, aes(x = !!sym(x), y = !!sym(y)))
    if (isTRUE(add_bg)) {
        p <- p +
            bg_layer(
                data,
                x,
                isTRUE(keep_empty_x),
                bg_palette,
                bg_palcolor,
                bg_alpha,
                facet_by
            )
    }

    if (!is.null(add_hline) && !isFALSE(add_hline)) {
        p <- p +
            geom_hline(
                yintercept = add_hline,
                linetype = hline_type,
                linewidth = hline_width,
                color = hline_color,
                alpha = hline_alpha
            )
    }

    x_vals <- levels(data[[x]])
    if (anyNA(data[[x]])) {
        x_vals <- c(x_vals, NA)
    }
    colors <- palette_this(
        x_vals,
        palette = palette,
        palcolor = palcolor,
        NA_keep = TRUE,
        reverse = palreverse
    )

    if (isTRUE(color_line_by_x)) {
        p <- p +
            geom_line(
                aes(color = !!sym(x), group = 1),
                alpha = line_alpha,
                linetype = line_type,
                linewidth = line_width,
                show.legend = TRUE
            )

        if (isTRUE(keep_empty_x)) {
            p <- p +
                scale_color_manual(
                    name = x,
                    values = colors,
                    guide = "legend",
                    breaks = x_vals,
                    limits = x_vals,
                    drop = FALSE,
                    na.value = colors["NA"] %||% "grey80"
                )
        } else {
            p <- p +
                scale_color_manual(
                    name = x,
                    values = colors,
                    guide = "legend",
                    na.value = colors["NA"] %||% "grey80"
                )
        }
    } else {
        p <- p +
            geom_line(
                aes(group = 1),
                color = colors[[1]],
                alpha = line_alpha,
                linetype = line_type,
                linewidth = line_width
            )
    }

    if (isTRUE(add_errorbars)) {
        if (errorbar_color == "line" && isTRUE(color_line_by_x)) {
            p <- p +
                geom_errorbar(
                    aes(
                        ymin = !!sym(errorbar_min),
                        ymax = !!sym(errorbar_max),
                        color = !!sym(x)
                    ),
                    alpha = errorbar_alpha,
                    width = errorbar_width,
                    linewidth = errorbar_linewidth
                )
        } else if (errorbar_color == "line") {
            p <- p +
                geom_errorbar(
                    aes(ymin = !!sym(errorbar_min), ymax = !!sym(errorbar_max)),
                    color = colors[[1]],
                    alpha = errorbar_alpha,
                    width = errorbar_width,
                    linewidth = errorbar_linewidth
                )
        } else {
            p <- p +
                geom_errorbar(
                    aes(ymin = !!sym(errorbar_min), ymax = !!sym(errorbar_max)),
                    color = errorbar_color,
                    alpha = errorbar_alpha,
                    width = errorbar_width,
                    linewidth = errorbar_linewidth
                )
        }
    }

    if (isTRUE(fill_point_by_x)) {
        p <- p +
            geom_point(
                aes(fill = !!sym(x)),
                color = "grey20",
                alpha = pt_alpha,
                size = pt_size,
                shape = 21,
                show.legend = TRUE
            )
        if (isTRUE(keep_empty_x)) {
            p <- p +
                scale_fill_manual(
                    name = x,
                    values = colors,
                    guide = "legend",
                    breaks = x_vals,
                    limits = x_vals,
                    drop = FALSE,
                    na.value = colors["NA"] %||% "grey80"
                )
        } else {
            p <- p +
                scale_fill_manual(
                    name = x,
                    values = colors,
                    guide = "legend",
                    ,
                    na.value = colors["NA"] %||% "grey80"
                )
        }
    } else {
        p <- p +
            geom_point(
                fill = colors[[1]],
                color = "grey20",
                alpha = pt_alpha,
                size = pt_size,
                shape = 21
            )
    }

    if (!is.null(hidata)) {
        p <- p +
            geom_point(
                data = hidata,
                fill = highlight_color,
                color = "transparent",
                size = highlight_size,
                shape = 21
            )
    }

    just <- calc_just(x_text_angle)
    p <- p +
        scale_x_discrete(drop = !isTRUE(keep_empty_x)) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% x,
            y = ylab %||% y
        ) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major = element_line(colour = "grey80", linetype = 2),
            axis.text.x = element_text(
                angle = x_text_angle,
                hjust = just$h,
                vjust = just$v
            )
        )

    dims <- calculate_plot_dimensions(
        base_height = 4.5,
        aspect.ratio = aspect.ratio,
        n_x = nlevels(data[[x]]),
        x_scale_factor = 0.8,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = length(x_vals),
        legend_nchar = max(nchar(as.character(x_vals)), na.rm = TRUE)
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    p
}

#' Multi-series line plot (internal)
#'
#' @description
#' Core implementation for drawing a multi-series line plot.  This is the
#' workhorse behind \code{\link{LinePlotAtomic}} for grouped data -- it takes a
#' **single** data frame with a \code{group_by} column and returns a
#' \code{ggplot} object.  Each group receives its own line rendered in a
#' distinct colour from the palette, with optional error bars, highlighted
#' points, per-group horizontal reference lines, and background stripes.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{ggplot dispatch} -- selects \code{gglogger::ggplot} or
#'         \code{ggplot2::ggplot} based on
#'         \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{Column resolution} -- \code{x} (forced to factor),
#'         \code{y}, \code{group_by} (multi-column concatenation supported),
#'         and \code{facet_by} are validated via
#'         \code{\link{check_columns}}.
#'   \item \strong{Count aggregation} -- when \code{y = NULL}, the count of
#'         observations per (\code{x}, \code{group_by}, \code{facet_by})
#'         combination is computed and used as \code{y}.
#'   \item \strong{Error bar validation} -- if \code{add_errorbars = TRUE},
#'         checks that either \code{errorbar_min}/\code{errorbar_max} or
#'         \code{errorbar_sd} is provided.
#'   \item \strong{NA / empty-level handling} --
#'         \code{\link{process_keep_na_empty}()} applies \code{keep_na} and
#'         \code{keep_empty} policies.
#'   \item \strong{Highlight parsing} -- \code{highlight} (indices, row
#'         names, expression string, or \code{TRUE}) is resolved to a subset
#'         of data rows.
#'   \item \strong{Background layer} -- when \code{add_bg = TRUE},
#'         alternating background stripes are drawn via
#'         \code{\link{bg_layer}()}.
#'   \item \strong{Colour assignment} -- \code{\link{palette_this}()}
#'         assigns colours to all \code{group_by} levels (including
#'         \code{NA}, defaulting to \code{"grey80"}).
#'   \item \strong{Horizontal reference line} -- when \code{add_hline} is
#'         set, one or more \code{geom_hline()} are added.  If
#'         \code{hline_color = TRUE}, each horizontal line is coloured to
#'         match the corresponding group colour, and \code{add_hline} can
#'         be a named vector/ list mapping groups to intercept values.
#'   \item \strong{Line rendering} -- lines are coloured and grouped by
#'         \code{group_by} via \code{\link[ggplot2]{geom_line}()} with
#'         \code{aes(color = group_by, group = group_by)} and
#'         \code{scale_color_manual()}.
#'   \item \strong{Error bars} -- when \code{add_errorbars = TRUE}, three
#'         colour modes are supported: follow the line colour
#'         (\code{"line"}), track \code{group_by} colours, or use a
#'         constant colour.  Error bars are added via
#'         \code{\link[ggplot2]{geom_errorbar}()}.
#'   \item \strong{Point rendering} -- points are filled per group via
#'         \code{\link[ggplot2]{geom_point}()} with
#'         \code{aes(fill = group_by)} and \code{scale_fill_manual()}.
#'   \item \strong{Highlight overlay} -- an additional \code{geom_point()}
#'         layer draws highlighted rows in the specified colour and size.
#'   \item \strong{Plot assembly} -- \code{scale_x_discrete()},
#'         \code{labs()}, theme application, axis styling (angle, grid
#'         lines), and legend positioning.
#'   \item \strong{Dimension calculation} --
#'         \code{\link{calculate_plot_dimensions}()} computes \code{height}
#'         and \code{width} attributes (in inches) from x-axis category
#'         count, aspect ratio, and legend metrics (legend entries reflect
#'         group levels rather than x levels).
#' }
#'
#' @inheritParams common_args
#' @inheritParams LinePlotSingle
#' @param group_by A character vector of column names to group the data by.
#'  Each unique combination becomes a separate line. Multiple columns are
#'  concatenated with \code{group_by_sep}. When \code{NULL}, the
#'  \code{LinePlotSingle} path is used instead.
#' @param group_by_sep A character string used to join multiple
#'  \code{group_by} column values into a single group identifier. Default
#'  \code{"_"}.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches).
#' @keywords internal
#' @importFrom rlang syms
#' @importFrom dplyr summarise %>% mutate n
#' @importFrom ggplot2 geom_line scale_color_manual labs geom_errorbar geom_point
LinePlotGrouped <- function(
    data,
    x,
    y = NULL,
    group_by,
    group_by_sep = "_",
    facet_by = NULL,
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    add_errorbars = FALSE,
    errorbar_width = 0.1,
    errorbar_alpha = 1,
    errorbar_color = "grey30",
    errorbar_linewidth = .75,
    errorbar_min = NULL,
    errorbar_max = NULL,
    errorbar_sd = NULL,
    highlight = NULL,
    highlight_size = pt_size - 0.75,
    highlight_color = "red2",
    highlight_alpha = 0.8,
    pt_alpha = 1,
    pt_size = 5,
    add_hline = FALSE,
    hline_type = "solid",
    hline_width = 0.5,
    hline_color = "black",
    hline_alpha = 1,
    line_type = "solid",
    line_width = 1,
    line_alpha = .8,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    x_text_angle = 0,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_empty = FALSE,
    keep_na = FALSE,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    x <- check_columns(data, x, force_factor = TRUE)
    y <- check_columns(data, y)
    group_by <- check_columns(
        data,
        group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = group_by_sep
    )
    if (is.null(y)) {
        orig_data <- data
        data <- data %>%
            dplyr::group_by(!!!syms(unique(c(x, group_by, facet_by)))) %>%
            summarise(.y = n(), .groups = "drop")
        # keep the factor levels
        for (col in unique(c(x, group_by, facet_by))) {
            data[[col]] <- factor(
                data[[col]],
                levels = levels(orig_data[[col]])
            )
        }
        rm(orig_data)
        y <- ".y"
    }

    if (isTRUE(add_errorbars)) {
        if (
            is.null(errorbar_sd) &&
                (is.null(errorbar_min) || is.null(errorbar_max))
        ) {
            stop(
                "If 'errorbar_min' and 'errorbar_max' are not provided, 'errorbar_sd' must be provided."
            )
        }
        if (is.null(errorbar_min) || is.null(errorbar_max)) {
            data$errorbar_min <- data[[y]] - data[[errorbar_sd]]
            data$errorbar_max <- data[[y]] + data[[errorbar_sd]]
            errorbar_min <- "errorbar_min"
            errorbar_max <- "errorbar_max"
        }
    }

    data <- process_keep_na_empty(data, keep_na, keep_empty)

    keep_empty_x <- keep_empty[[x]]
    keep_empty_group <- if (!is.null(group_by)) keep_empty[[group_by]] else NULL

    hidata <- NULL
    if (!is.null(highlight)) {
        if (isTRUE(highlight)) {
            hidata <- data
        } else if (is.numeric(highlight)) {
            hidata <- data[highlight, , drop = FALSE]
        } else if (is.character(highlight) && length(highlight) == 1) {
            hidata <- filter(data, !!parse_expr(highlight))
        } else if (is.null(rownames(data))) {
            stop(
                "No row names in the data, please provide a vector of indexes to highlight."
            )
        } else {
            hidata <- data[highlight, , drop = FALSE]
        }
    }

    p <- ggplot(data, aes(x = !!sym(x), y = !!sym(y)))
    if (isTRUE(add_bg)) {
        p <- p +
            bg_layer(
                data,
                x,
                isTRUE(keep_empty_x),
                bg_palette,
                bg_palcolor,
                bg_alpha,
                facet_by
            )
    }
    group_vals <- levels(data[[group_by]])
    if (anyNA(data[[group_by]])) {
        group_vals <- c(group_vals, NA)
    }

    colors <- palette_this(
        group_vals,
        palette = palette,
        palcolor = palcolor,
        NA_keep = TRUE,
        reverse = palreverse
    )

    if (!is.null(add_hline) && !isFALSE(add_hline)) {
        if (isTRUE(hline_color)) {
            if (!is.list(add_hline)) {
                add_hline <- stats::setNames(
                    as.list(add_hline),
                    levels(data[[group_by]])[1:length(add_hline)]
                )
            }
            add_hline <- add_hline[intersect(
                levels(data[[group_by]]),
                names(add_hline)
            )]
            hline_color <- colors[names(add_hline)]

            add_hline <- unlist(add_hline, use.names = FALSE)
        }
        p <- p +
            geom_hline(
                yintercept = add_hline,
                linetype = hline_type,
                linewidth = hline_width,
                color = hline_color,
                alpha = hline_alpha
            )
    }

    p <- p +
        geom_line(
            aes(color = !!sym(group_by), group = !!sym(group_by)),
            alpha = line_alpha,
            linetype = line_type,
            linewidth = line_width,
            show.legend = TRUE
        )

    if (isTRUE(keep_empty_group)) {
        p <- p +
            scale_color_manual(
                name = group_by,
                values = colors,
                guide = "legend",
                na.value = colors["NA"] %||% "grey80",
                breaks = group_vals,
                limits = group_vals,
                drop = FALSE
            )
    } else {
        p <- p +
            scale_color_manual(
                name = group_by,
                values = colors,
                guide = "legend",
                na.value = colors["NA"] %||% "grey80"
            )
    }

    if (isTRUE(add_errorbars)) {
        if (errorbar_color == "line") {
            p <- p +
                geom_errorbar(
                    aes(
                        ymin = !!sym(errorbar_min),
                        ymax = !!sym(errorbar_max),
                        color = !!sym(group_by)
                    ),
                    alpha = errorbar_alpha,
                    width = errorbar_width,
                    linewidth = errorbar_linewidth
                )
        } else {
            p <- p +
                geom_errorbar(
                    aes(ymin = !!sym(errorbar_min), ymax = !!sym(errorbar_max)),
                    color = errorbar_color,
                    alpha = errorbar_alpha,
                    width = errorbar_width,
                    linewidth = errorbar_linewidth
                )
        }
    }

    p <- p +
        geom_point(
            aes(fill = !!sym(group_by)),
            color = "grey20",
            alpha = pt_alpha,
            size = pt_size,
            shape = 21,
            show.legend = TRUE
        )
    if (isTRUE(keep_empty_group)) {
        p <- p +
            scale_fill_manual(
                name = group_by,
                values = colors,
                guide = "legend",
                breaks = group_vals,
                limits = group_vals,
                drop = FALSE,
                na.value = colors["NA"] %||% "grey80"
            )
    } else {
        p <- p +
            scale_fill_manual(
                name = group_by,
                values = colors,
                guide = "legend",
                na.value = colors["NA"] %||% "grey80"
            )
    }

    if (!is.null(hidata)) {
        p <- p +
            geom_point(
                data = hidata,
                fill = highlight_color,
                color = "transparent",
                size = highlight_size,
                shape = 21
            )
    }

    just <- calc_just(x_text_angle)
    p <- p +
        scale_x_discrete(drop = !isTRUE(keep_empty_x)) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% x,
            y = ylab %||% y
        ) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major = element_line(colour = "grey80", linetype = 2),
            axis.text.x = element_text(
                angle = x_text_angle,
                hjust = just$h,
                vjust = just$v
            )
        )

    dims <- calculate_plot_dimensions(
        base_height = 4.5,
        aspect.ratio = aspect.ratio,
        n_x = nlevels(data[[x]]),
        x_scale_factor = 0.8,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = length(group_vals),
        legend_nchar = max(nchar(as.character(group_vals)), na.rm = TRUE)
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    p
}

#' Atomic line-plot dispatcher (internal)
#'
#' @description
#' Dispatcher that routes to \code{\link{LinePlotSingle}} or
#' \code{\link{LinePlotGrouped}} depending on whether \code{group_by} is
#' provided.  Handles faceting via \code{\link{facet_plot}()} after the base
#' plot is built.
#'
#' @section Dispatch Logic:
#' \enumerate{
#'   \item \strong{Column resolution} -- \code{facet_by} is validated via
#'         \code{\link{check_columns}}.
#'   \item \strong{Routing} -- when \code{group_by = NULL}, delegates to
#'         \code{\link{LinePlotSingle}} (ungrouped, single-series line plot).
#'         When \code{group_by} is provided, delegates to
#'         \code{\link{LinePlotGrouped}} (multi-series line plot).
#'   \item \strong{Facet keep_empty consistency} -- when \code{facet_by}
#'         contains multiple columns, their \code{keep_empty} values must be
#'         identical.
#'   \item \strong{Faceting} -- \code{\link{facet_plot}()} applies
#'         \code{facet_wrap} / \code{facet_grid} with the resolved
#'         \code{drop} argument (derived from \code{keep_empty}).
#' }
#'
#' @inheritParams common_args
#' @inheritParams LinePlotGrouped
#' @param x A character string specifying the column name for the x-axis.
#'  Must be character or factor.
#' @param y A character string specifying the numeric column for the y-axis.
#' @param group_by A character vector of column names to group the data by.
#'  When NULL, a single-series line plot is drawn via
#'  \code{\link{LinePlotSingle}}. When provided, a multi-series line plot
#'  is drawn via \code{\link{LinePlotGrouped}}.
#' @param fill_point_by_x_if_no_group A logical value. When TRUE (default),
#'  points are filled by the x-axis categories via the palette when
#'  \code{group_by = NULL}. Passed to \code{LinePlotSingle} as
#'  \code{fill_point_by_x}. Has no effect when \code{group_by} is set.
#' @param color_line_by_x_if_no_group A logical value. When TRUE (default),
#'  lines are coloured by the x-axis categories via the palette when
#'  \code{group_by = NULL}. Passed to \code{LinePlotSingle} as
#'  \code{color_line_by_x}. Has no effect when \code{group_by} is set.
#' @param facet_args A list of additional arguments passed to
#'  \code{\link{facet_plot}()} for fine-grained control over faceting
#'  (e.g. \code{scales}, \code{space}, \code{labeller}).
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches).
#' @keywords internal
LinePlotAtomic <- function(
    data,
    x,
    y = NULL,
    group_by = NULL,
    fill_point_by_x_if_no_group = TRUE,
    color_line_by_x_if_no_group = TRUE,
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    add_errorbars = FALSE,
    errorbar_width = 0.1,
    errorbar_alpha = 1,
    errorbar_color = "grey30",
    errorbar_linewidth = .75,
    errorbar_min = NULL,
    errorbar_max = NULL,
    errorbar_sd = NULL,
    highlight = NULL,
    highlight_size = pt_size - 0.75,
    highlight_color = "red2",
    highlight_alpha = 0.8,
    pt_alpha = 1,
    pt_size = 5,
    add_hline = FALSE,
    hline_type = "solid",
    hline_width = 0.5,
    hline_color = "black",
    hline_alpha = 1,
    line_type = "solid",
    line_width = 1,
    line_alpha = .8,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    x_text_angle = 0,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_empty = FALSE,
    keep_na = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_args = list(),
    facet_nrow = NULL,
    facet_ncol = NULL,
    facet_byrow = TRUE,
    ...
) {
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )
    if (is.null(group_by)) {
        p <- LinePlotSingle(
            data = data,
            x = x,
            y = y,
            fill_point_by_x = fill_point_by_x_if_no_group,
            color_line_by_x = color_line_by_x_if_no_group,
            facet_by = facet_by,
            add_bg = add_bg,
            bg_palette = bg_palette,
            bg_palcolor = bg_palcolor,
            bg_alpha = bg_alpha,
            add_errorbars = add_errorbars,
            errorbar_width = errorbar_width,
            errorbar_alpha = errorbar_alpha,
            errorbar_color = errorbar_color,
            errorbar_linewidth = errorbar_linewidth,
            errorbar_min = errorbar_min,
            errorbar_max = errorbar_max,
            errorbar_sd = errorbar_sd,
            highlight = highlight,
            highlight_size = highlight_size,
            highlight_color = highlight_color,
            highlight_alpha = highlight_alpha,
            pt_alpha = pt_alpha,
            pt_size = pt_size,
            add_hline = add_hline,
            hline_type = hline_type,
            hline_width = hline_width,
            hline_color = hline_color,
            hline_alpha = hline_alpha,
            line_type = line_type,
            line_width = line_width,
            line_alpha = line_alpha,
            theme = theme,
            theme_args = theme_args,
            palette = palette,
            palcolor = palcolor,
            palreverse = palreverse,
            x_text_angle = x_text_angle,
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            title = title,
            subtitle = subtitle,
            xlab = xlab,
            ylab = ylab,
            keep_empty = keep_empty,
            keep_na = keep_na,
            ...
        )
    } else {
        p <- LinePlotGrouped(
            data = data,
            x = x,
            y = y,
            group_by = group_by,
            facet_by = facet_by,
            add_bg = add_bg,
            bg_palette = bg_palette,
            bg_palcolor = bg_palcolor,
            bg_alpha = bg_alpha,
            add_errorbars = add_errorbars,
            errorbar_width = errorbar_width,
            errorbar_alpha = errorbar_alpha,
            errorbar_color = errorbar_color,
            errorbar_linewidth = errorbar_linewidth,
            errorbar_min = errorbar_min,
            errorbar_max = errorbar_max,
            errorbar_sd = errorbar_sd,
            highlight = highlight,
            highlight_size = highlight_size,
            highlight_color = highlight_color,
            highlight_alpha = highlight_alpha,
            pt_alpha = pt_alpha,
            pt_size = pt_size,
            add_hline = add_hline,
            hline_type = hline_type,
            hline_width = hline_width,
            hline_color = hline_color,
            hline_alpha = hline_alpha,
            line_type = line_type,
            line_width = line_width,
            line_alpha = line_alpha,
            theme = theme,
            theme_args = theme_args,
            palette = palette,
            palcolor = palcolor,
            palreverse = palreverse,
            x_text_angle = x_text_angle,
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            title = title,
            subtitle = subtitle,
            xlab = xlab,
            ylab = ylab,
            keep_empty = keep_empty,
            keep_na = keep_na,
            ...
        )
    }

    keep_empty_facet <- if (!is.null(facet_by)) {
        keep_empty[[facet_by[1]]]
    } else {
        NULL
    }
    if (length(facet_by) > 1) {
        stopifnot(
            "[LinePlot] `keep_empty` for `facet_by` variables must be identical." = identical(
                keep_empty_facet,
                keep_empty[[facet_by[2]]]
            )
        )
    }
    facet_args$plot <- p
    facet_args["facet_by"] <- list(facet_by)
    facet_args["facet_scales"] <- list(facet_scales)
    facet_args["drop"] <- list(!isTRUE(keep_empty_facet))
    facet_args["nrow"] <- list(facet_nrow)
    facet_args["ncol"] <- list(facet_ncol)
    facet_args["byrow"] <- list(facet_byrow)
    facet_args["legend.position"] <- list(legend.position)
    facet_args["legend.direction"] <- list(legend.direction)

    do_call(facet_plot, facet_args)
}

#' Line plot
#'
#' @description
#' Draws a line plot showing the change of a numeric value across the
#' progression of a categorical x-axis variable.  Each x-axis category is
#' rendered as a point connected by a line, with support for multiple
#' grouped series, error bars, highlighted points, background stripes,
#' and a horizontal reference line.
#'
#' Key features:
#' \itemize{
#'   \item \strong{Colour modes:} lines and points can be coloured by x
#'   category (single-series) or by a \code{group_by} variable
#'   (multi-series), or use a single uniform colour.
#'   \item \strong{Error bars:} additive error bars via
#'   \code{errorbar_sd}, \code{errorbar_min}, or \code{errorbar_max}.
#'   \item \strong{Highlighting:} specific points can be emphasised with a
#'   different colour and size via indices, row names, or a filter
#'   expression.
#'   \item \strong{Background stripes:} \code{add_bg = TRUE} draws
#'   alternating bands for visual grouping.
#'   \item \strong{Count aggregation:} omit \code{y} to plot observation
#'   counts per x category.
#' }
#'
#' @section split_by Workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \strong{Column validation} -- \code{\link{check_columns}()}
#'         resolves \code{split_by} with multi-column concatenation.
#'   \item \strong{NA / empty pre-processing} --
#'         \code{\link{process_keep_na_empty}()} handles \code{keep_na} /
#'         \code{keep_empty} for the split column before splitting, then
#'         removes the split column from the per-split lists.
#'   \item \strong{Data splitting} -- splits \code{data} by
#'         \code{split_by} levels, preserving factor level order.  When
#'         \code{split_by = NULL}, the data is wrapped in a single-element
#'         list with name \code{"..."}.
#'   \item \strong{Per-split palette / colour} --
#'         \code{\link{check_palette}()} and \code{\link{check_palcolor}()}
#'         resolve per-split palette and colour overrides.
#'   \item \strong{Per-split legend} -- \code{\link{check_legend}()}
#'         resolves \code{legend.position} and \code{legend.direction} per
#'         split level.
#'   \item \strong{Per-split title} -- when \code{title} is a function, it
#'         receives the default title (the split level name) and can return
#'         a custom string; otherwise \code{title \%||\% split_level} is
#'         used.
#'   \item \strong{Dispatch} -- each split subset is passed to
#'         \code{\link{LinePlotAtomic}}.
#'   \item \strong{Combination} -- \code{\link{combine_plots}()} assembles
#'         the list of plots via \code{patchwork::wrap_plots}, honouring
#'         \code{nrow}/\code{ncol}/\code{byrow}/\code{design}.
#' }
#'
#' @inheritParams common_args
#' @inheritParams LinePlotAtomic
#' @param split_by A character vector of column names to split the data by.
#'  Each split level produces a separate sub-plot. Multiple columns are
#'  concatenated with \code{split_by_sep}.
#' @param split_by_sep A character string used to join multiple
#'  \code{split_by} column values. Default \code{"_"}.
#' @param seed A numeric seed for reproducibility. Passed to
#'  \code{\link{validate_common_args}()}. Default 8525.
#' @param combine Logical; when TRUE (default), per-split plots are combined
#'  into a single \code{patchwork} object. When FALSE, a named list of
#'  \code{ggplot} objects is returned.
#' @param nrow,ncol Integer number of rows / columns for the combined
#'  layout (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param byrow Logical; fill the combined layout by row. Default TRUE
#'  (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axes A character string specifying how axes should be treated
#'  across the combined layout (passed to
#'  \code{\link[patchwork]{wrap_plots}}).
#' @param axis_titles A character string specifying how axis titles should
#'  be treated across the combined layout. Defaults to \code{axes}.
#' @param guides A character string specifying how guides should be
#'  collected across panels. Passed to \code{\link{combine_plots}()}.
#' @param design A custom layout specification for combined plots (passed
#'  to \code{\link{combine_plots}()}). Overrides \code{nrow}/\code{ncol}
#'  when specified.
#' @return A \code{ggplot} object, a \code{patchwork} object (when
#'  \code{combine = TRUE} with \code{split_by}), or a named list of
#'  \code{ggplot} objects (when \code{combine = FALSE}), each with
#'  \code{height} and \code{width} attributes in inches.
#' @export
#' @examples
#' \donttest{
#' data <- data.frame(
#'    x = factor(c("A", "B", "C", "D", "A", "B", "C", "D"), levels = LETTERS[1:6]),
#'    y = c(10, 8, 16, 4, 6, 12, 14, 2),
#'    group = c("G1", "G1", "G1", "G1", "G2", "G2", "G2", "G2"),
#'    facet = c("F1", "F1", "F2", "F2", "F3", "F3", "F4", "F4")
#' )
#'
#' # --- Basic usage ---
#' LinePlot(data, x = "x", y = "y")
#' LinePlot(data, x = "x", y = "y", highlight = "group == 'G1'",
#'    fill_point_by_x_if_no_group = FALSE, color_line_by_x_if_no_group = FALSE)
#'
#' # --- Grouped lines ---
#' LinePlot(data, x = "x", y = "y", group_by = "group")
#' LinePlot(data, x = "x", y = "y", group_by = "group",
#'    add_hline = 10, hline_color = "red")
#' LinePlot(data, x = "x", y = "y", group_by = "group", add_bg = TRUE,
#'    highlight = "y > 10")
#' LinePlot(data, x = "x", y = "y", group_by = "group", facet_by = "facet")
#' LinePlot(data, x = "x", y = "y", group_by = "group", split_by = "facet")
#'
#' # --- Per-split styling ---
#' LinePlot(data, x = "x", y = "y", split_by = "group",
#'          palcolor = list(G1 = c("red", "blue"), G2 = c("green", "black")))
#'
#' # --- keep_na and keep_empty ---
#' data <- data.frame(
#'    x = factor(c("A", "B", NA, "D", "A", "B", NA, "D"), levels = LETTERS[1:4]),
#'    y = c(10, 8, 16, 4, 6, 12, 14, 2),
#'    group = factor(c("G1", "G1", "G1", NA, NA, "G3", "G3", "G3"),
#'      levels = c("G1", "G2", "G3")),
#'    facet = c("F1", "F1", "F2", "F2", "F3", "F3", "F4", "F4")
#' )
#'
#' LinePlot(data, x = "x", y = "y", keep_na = TRUE)
#' LinePlot(data, x = "x", y = "y", keep_empty = TRUE)
#' LinePlot(data, x = "x", y = "y", keep_empty = 'level')
#' LinePlot(data, x = "x", y = "y", group_by = "group", keep_na = TRUE)
#' LinePlot(data, x = "x", y = "y", group_by = "group", keep_empty = TRUE)
#' LinePlot(data, x = "x", y = "y", group_by = "group",
#'    keep_empty = list(x = TRUE, group = 'level'))
#' }
LinePlot <- function(
    data,
    x,
    y = NULL,
    group_by = NULL,
    group_by_sep = "_",
    split_by = NULL,
    split_by_sep = "_",
    fill_point_by_x_if_no_group = TRUE,
    color_line_by_x_if_no_group = TRUE,
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    add_errorbars = FALSE,
    errorbar_width = 0.1,
    errorbar_alpha = 1,
    errorbar_color = "grey30",
    errorbar_linewidth = .75,
    errorbar_min = NULL,
    errorbar_max = NULL,
    errorbar_sd = NULL,
    highlight = NULL,
    highlight_size = pt_size - 0.75,
    highlight_color = "red2",
    highlight_alpha = 0.8,
    pt_alpha = 1,
    pt_size = 5,
    keep_na = FALSE,
    keep_empty = FALSE,
    line_type = "solid",
    line_width = 1,
    line_alpha = .8,
    add_hline = FALSE,
    hline_type = "solid",
    hline_width = 0.5,
    hline_color = "black",
    hline_alpha = 1,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    x_text_angle = 0,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    facet_by = NULL,
    facet_scales = "fixed",
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    facet_nrow = NULL,
    facet_ncol = NULL,
    facet_byrow = TRUE,
    facet_args = list(),
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    keep_na <- check_keep_na(keep_na, c(x, group_by, split_by, facet_by))
    keep_empty <- check_keep_empty(
        keep_empty,
        c(x, group_by, split_by, facet_by)
    )
    theme <- process_theme(theme)

    x <- check_columns(data, x, force_factor = TRUE)
    y <- check_columns(data, y)
    group_by <- check_columns(
        data,
        group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = group_by_sep
    )
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            LinePlotAtomic(
                datas[[nm]],
                x = x,
                y = y,
                group_by = group_by,
                fill_point_by_x_if_no_group = fill_point_by_x_if_no_group,
                color_line_by_x_if_no_group = color_line_by_x_if_no_group,
                add_bg = add_bg,
                bg_palette = bg_palette,
                bg_palcolor = bg_palcolor,
                bg_alpha = bg_alpha,
                add_errorbars = add_errorbars,
                errorbar_width = errorbar_width,
                errorbar_alpha = errorbar_alpha,
                errorbar_color = errorbar_color,
                errorbar_linewidth = errorbar_linewidth,
                errorbar_min = errorbar_min,
                errorbar_max = errorbar_max,
                errorbar_sd = errorbar_sd,
                highlight = highlight,
                highlight_size = highlight_size,
                highlight_color = highlight_color,
                highlight_alpha = highlight_alpha,
                pt_alpha = pt_alpha,
                pt_size = pt_size,
                line_type = line_type,
                line_width = line_width,
                line_alpha = line_alpha,
                add_hline = add_hline,
                hline_type = hline_type,
                hline_width = hline_width,
                hline_color = hline_color,
                hline_alpha = hline_alpha,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                x_text_angle = x_text_angle,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                facet_args = facet_args,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_nrow = facet_nrow,
                facet_ncol = facet_ncol,
                facet_byrow = facet_byrow,
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                keep_na = keep_na,
                keep_empty = keep_empty,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
data <- data.frame(
    x = factor(rep(c("A", "B", "C", "D", "E"), each = 20), levels = c("A", "B", "C", "D", "E")),
    y = rnorm(100),
    group1 = factor(rep(c("g1", "g2"), 50)),
    group2 = factor(rep(c("f1", "f2"), each = 50))
)

test_that("LinePlot returns a ggplot for basic usage", {
    p <- LinePlot(data, x = "x", y = "y")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("LinePlot works without explicit y (aggregates count)", {
    p <- LinePlot(data, x = "x")
    expect_s3_class(p, "ggplot")
})

test_that("LinePlot with group_by works", {
    p <- LinePlot(data, x = "x", y = "y", group_by = "group1")
    expect_s3_class(p, "ggplot")
})

test_that("LinePlot sets title and labels", {
    p <- LinePlot(data, x = "x", y = "y", title = "Line", xlab = "X Axis", ylab = "Y Axis")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "Line")
    expect_equal(p$labels$x, "X Axis")
    expect_equal(p$labels$y, "Y Axis")
})

test_that("LinePlot with split_by returns patchwork", {
    p <- LinePlot(data, x = "x", y = "y", split_by = "group2", combine = TRUE)
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("LinePlot with split_by and combine = FALSE returns list", {
    plots <- LinePlot(data, x = "x", y = "y", split_by = "group2", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("LinePlot with multiple split_by columns returns a list with one plot per combination", {
    plots <- suppressMessages(LinePlot(data, x = "x", y = "y", split_by = c("group1", "group2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("LinePlot with multiple split_by columns returns combined plot", {
    p <- suppressMessages(LinePlot(data, x = "x", y = "y", split_by = c("group1", "group2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("LinePlot with facet_by works", {
    p <- LinePlot(data, x = "x", y = "y", facet_by = "group2")
    expect_s3_class(p, "ggplot")
})

test_that("LinePlot with add_errorbars works when columns provided", {
    data2 <- data
    data2$ymin <- data2$y - 0.5
    data2$ymax <- data2$y + 0.5
    p <- LinePlot(data2, x = "x", y = "y", group_by = "group1",
                  add_errorbars = TRUE, errorbar_min = "ymin", errorbar_max = "ymax")
    expect_s3_class(p, "ggplot")
})

test_that("LinePlot with add_hline works", {
    p <- LinePlot(data, x = "x", y = "y", add_hline = 0)
    expect_s3_class(p, "ggplot")
})

test_that("LinePlot with flip = TRUE works", {
    p <- LinePlot(data, x = "x", y = "y", flip = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("LinePlot width/height attributes are numeric", {
    p <- LinePlot(data, x = "x", y = "y")
    expect_true(is.numeric(attr(p, "width")))
    expect_true(is.numeric(attr(p, "height")))
})

#' Internal helper to create a 3D dimension reduction plot using plotly
#'
#' @description
#' Renders an interactive 3D scatter plot via **plotly** for dimension reduction
#' data with three ordination axes. Called automatically by `DimPlotAtomic()`
#' when `length(dims) == 3`.
#'
#' The function supports two visualisation modes:
#' - **group_by** — discrete group colouring with one trace per group, matching
#'   the 2D legend behaviour via `label_use`.
#' - **features** — continuous colour scale with a `plotly` colorscale bar.
#'
#' Graphs / networks are rendered as 3D line segments (with NA separators for
#' line breaks). Lineage curves are fitted via `stats::loess()` per lineage
#' column and drawn as smoothed paths. For datasets exceeding 100 000 points,
#' per-point hover text is disabled to reduce the JSON payload.
#'
#' @section Architecture:
#' **DimPlotAtomic3D** executes the following steps:
#'
#' 1. **plotly availability check** — errors early if the `plotly` package is
#'    not installed.
#' 2. **Axis labels** — defaults `xlab` / `ylab` / `zlab` from `dims`.
#' 3. **Marker size scaling** — multiplies `pt_size` by 2 to convert from
#'    ggplot2 mm units to plotly pixel units.
#' 4. **Large-data detection** — disables per-point hover text when
#'    `nrow(data) > 1e5`.
#' 5. **Base plotly object** — `plotly::plot_ly()` initialisation.
#' 6. **Graph / network edges** (if `graph` is provided) — resolves the graph
#'    source (`@attribute`, Graph object, matrix, data.frame, column indices,
#'    or column names), aligns row/column names with `data`, sets upper-triangle
#'    and zero entries to `NA`, melts into an edge data frame, builds x/y/z
#'    coordinate vectors (interleaved with NA for line breaks), and adds
#'    `scatter3d` line traces.
#' 7. **Main scatter traces**:
#'    - *group_by mode*: iterates over `labels_tb` entries, creating one trace
#'      per group with the resolved colour. NA groups use `bg_color`. Legend
#'      labels use `label_use` (with count annotations if `show_stat = TRUE`).
#'    - *features mode*: builds a plotly-style colorscale from the continuous
#'      palette. NA points are rendered first in `bg_color` without a colour
#'      bar; non-NA points follow with a `showscale = TRUE` colour bar.
#' 8. **Highlight** — if `highlight` is specified, highlighted points are
#'    plotted as open circles with increased marker size (to account for
#'    stroke). Supports `TRUE` (all points), a filter expression string, or
#'    a vector of row names / numeric indices.
#' 9. **Lineages** — each lineage column is validated, trimmed to
#'    `[lineages_trim[1], lineages_trim[2]]` quantiles, ordered by value,
#'    and a `loess(span = lineages_span, degree = 2)` smooth is fitted per
#'    dimension. The smoothed curve is rendered as a 3D line trace.
#' 10. **Labels** — when `label = TRUE` and `group_by` is set, group median
#'     coordinates are computed via `aggregate()`. In 3D, plotly does not
#'     support text background/outline, so `label_bg` is used directly as
#'     the text colour. Text size is scaled by 3× for plotly.
#' 11. **Layout** — subtitle is composited with an optional "Showing N sampled
#'     points" note. The full title is rendered as HTML (`<br><sup>...</sup>`).
#'     The scene sets `aspectratio = list(x = 1, y = 1, z = 1)` and axis
#'     titles.
#'
#' @inheritParams DimPlotAtomic
#' @param colorby A character string naming the column used for colour mapping
#'   (either the `group_by` factor or the `features` numeric column).
#' @param colors A named character vector mapping group levels to hex colours.
#'   Only used in group_by mode.
#' @param feat_colors_value A numeric vector of the colour-scale anchor values
#'   for feature mode, as returned by `prepare_continuous_color_scale()`.
#' @param label_use A character vector of formatted legend labels (with
#'   optional count annotations) used in group_by mode.
#' @param labels_tb A table of group counts for trace iteration in group_by
#'   mode.
#' @param keep_empty_group A logical value. If `TRUE`, empty factor levels are
#'   preserved; traces with zero rows are still added to the legend.
#' @param n_sampled An integer indicating how many points were retained after
#'   down-sampling (displayed in the subtitle). `NULL` if no sampling occurred.
#' @keywords internal
DimPlotAtomic3D <- function(
    data,
    dims,
    group_by = NULL,
    features = NULL,
    colorby,
    colors = NULL,
    feat_colors_value = NULL,
    label_use = NULL,
    labels_tb = NULL,
    keep_empty_group = FALSE,
    bg_color = "grey80",
    color_name = "",
    pt_size = NULL,
    pt_alpha = 1,
    show_stat = TRUE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    label = FALSE,
    label_insitu = FALSE,
    label_size = 4,
    label_fg = "white",
    label_bg = "black",
    highlight = NULL,
    highlight_color = "black",
    highlight_size = 1,
    highlight_stroke = 0.8,
    highlight_alpha = 1,
    graph = NULL,
    edge_size = c(0.05, 0.5),
    edge_alpha = 0.1,
    edge_color = "grey40",
    lineages = NULL,
    lineages_trim = c(0.01, 0.99),
    lineages_span = 0.75,
    lineages_palette = "Dark2",
    lineages_palcolor = NULL,
    palette = "Spectral",
    palcolor = NULL,
    palreverse = FALSE,
    n_sampled = NULL
) {
    if (!requireNamespace("plotly", quietly = TRUE)) {
        stop(
            "Package 'plotly' is required for 3D plots. Install it with: install.packages('plotly')"
        )
    }

    xlab <- xlab %||% dims[1]
    ylab <- ylab %||% dims[2]
    zlab <- dims[3]

    # Scale pt_size for plotly (plotly marker sizes are in pixels, ggplot sizes are in mm)
    stopifnot(
        "[DimPlot] pt_size must be a numeric value for 3D plots." = is.numeric(
            pt_size
        )
    )
    marker_size <- max(pt_size * 2, 1)

    # Disable per-point hover text for large datasets to reduce JSON payload
    large_data <- nrow(data) > 1e5
    hover_mode <- if (large_data) "none" else "text"

    p <- plotly::plot_ly()

    ## Graph / network edges (rendered first so they appear behind points)
    if (!is.null(graph)) {
        if (
            is.character(graph) && length(graph) == 1 && startsWith(graph, "@")
        ) {
            graph_name <- substring(graph, 2)
            net_mat <- attr(data, graph_name)
            if (is.null(net_mat)) {
                stop(paste0(
                    "[DimPlot] The graph '",
                    graph_name,
                    "' is not found in the data attributes."
                ))
            }
        } else if (inherits(graph, "Graph")) {
            net_mat <- as.matrix(graph)
        } else if (
            is.matrix(graph) ||
                is.data.frame(graph) ||
                inherits(graph, "dgCMatrix")
        ) {
            net_mat <- graph
        } else if (is.numeric(graph)) {
            graph_cols <- colnames(data)[graph]
            net_mat <- data[graph_cols]
        } else if (is.character(graph)) {
            net_mat <- data[graph]
        } else {
            stop(
                "[DimPlot] The 'graph' should be a matrix, data.frame, Graph object, indexes, or column names."
            )
        }

        if (!is.matrix(net_mat)) {
            net_mat <- as.matrix(net_mat)
        }

        if (!is.null(rownames(net_mat)) && !is.null(colnames(net_mat))) {
            net_mat <- net_mat[rownames(data), rownames(data)]
        } else if (is.null(rownames(net_mat))) {
            stop("The graph matrix should have rownames and colnames.")
        } else if (is.null(rownames(data))) {
            stop("'graph' requires the input data to have rownames.")
        }
        net_mat[net_mat == 0] <- NA
        net_mat[upper.tri(net_mat)] <- NA

        net_df <- reshape2::melt(
            net_mat,
            na.rm = TRUE,
            stringsAsFactors = FALSE
        )
        net_df$value <- as.numeric(net_df$value)
        net_df$Var1 <- as.character(net_df$Var1)
        net_df$Var2 <- as.character(net_df$Var2)

        # Build edge coordinates with NA separators for line breaks
        edge_x <- as.vector(rbind(
            data[net_df$Var1, dims[1]],
            data[net_df$Var2, dims[1]],
            NA
        ))
        edge_y <- as.vector(rbind(
            data[net_df$Var1, dims[2]],
            data[net_df$Var2, dims[2]],
            NA
        ))
        edge_z <- as.vector(rbind(
            data[net_df$Var1, dims[3]],
            data[net_df$Var2, dims[3]],
            NA
        ))

        # Scale edge width: use mean of edge_size range, scaled for plotly
        edge_width <- mean(edge_size) * 10

        p <- plotly::add_trace(
            p = p,
            x = edge_x,
            y = edge_y,
            z = edge_z,
            type = "scatter3d",
            mode = "lines",
            line = list(color = edge_color, width = edge_width),
            opacity = edge_alpha,
            name = "edges",
            showlegend = FALSE,
            hoverinfo = "none"
        )
    }

    ## Main scatter traces
    if (!is.null(group_by)) {
        # Group-by mode: one trace per group for consistent legend with 2D (label_use)
        for (i in seq_along(labels_tb)) {
            grp <- names(labels_tb)[i]
            is_na_grp <- grp == "NA"

            if (is_na_grp) {
                grp_data <- data[is.na(data[[group_by]]), , drop = FALSE]
                grp_color <- bg_color
            } else {
                grp_data <- data[
                    !is.na(data[[group_by]]) &
                        as.character(data[[group_by]]) == grp,
                    ,
                    drop = FALSE
                ]
                grp_color <- colors[grp]
            }

            if (nrow(grp_data) == 0 && !isTRUE(keep_empty_group)) {
                next
            }

            legend_label <- if (!is.null(label_use)) label_use[i] else grp

            trace_args <- list(
                p = p,
                x = grp_data[[dims[1]]],
                y = grp_data[[dims[2]]],
                z = grp_data[[dims[3]]],
                type = "scatter3d",
                mode = "markers",
                marker = list(
                    size = marker_size,
                    color = grp_color,
                    opacity = pt_alpha
                ),
                name = legend_label,
                legendgroup = grp,
                showlegend = TRUE,
                hoverinfo = hover_mode
            )
            if (!large_data) {
                trace_args$text <- if (is_na_grp) {
                    paste0("Cell: ", rownames(grp_data))
                } else {
                    paste0("Cell: ", rownames(grp_data), "\nGroup: ", grp)
                }
            }
            p <- do_call(plotly::add_trace, trace_args)
        }
    } else {
        # Feature mode: continuous coloring
        feat_colors <- palette_this(
            palette = palette,
            palcolor = palcolor,
            type = "continuous",
            reverse = palreverse
        )
        n_colors <- length(feat_colors)
        plotly_colorscale <- lapply(seq_len(n_colors), function(i) {
            list((i - 1) / (n_colors - 1), feat_colors[i])
        })

        na_mask <- is.na(data[[features]])
        # Plot NA/background points
        if (any(na_mask)) {
            data_na <- data[na_mask, , drop = FALSE]
            na_trace_args <- list(
                p = p,
                x = data_na[[dims[1]]],
                y = data_na[[dims[2]]],
                z = data_na[[dims[3]]],
                type = "scatter3d",
                mode = "markers",
                marker = list(
                    size = marker_size,
                    color = bg_color,
                    opacity = pt_alpha
                ),
                name = "NA",
                showlegend = FALSE,
                hoverinfo = hover_mode
            )
            if (!large_data) {
                na_trace_args$text <- paste0("Cell: ", rownames(data_na))
            }
            p <- do_call(plotly::add_trace, na_trace_args)
        }
        # Plot non-NA points with colorscale
        data_valid <- data[!na_mask, , drop = FALSE]
        if (nrow(data_valid) > 0) {
            colorbar_title <- if (nchar(color_name) > 0) color_name else colorby
            feat_trace_args <- list(
                p = p,
                x = data_valid[[dims[1]]],
                y = data_valid[[dims[2]]],
                z = data_valid[[dims[3]]],
                type = "scatter3d",
                mode = "markers",
                marker = list(
                    size = marker_size,
                    color = data_valid[[features]],
                    colorscale = plotly_colorscale,
                    cmin = min(feat_colors_value, na.rm = TRUE),
                    cmax = max(feat_colors_value, na.rm = TRUE),
                    opacity = pt_alpha,
                    showscale = TRUE,
                    colorbar = list(
                        title = list(text = colorbar_title)
                    )
                ),
                showlegend = FALSE,
                hoverinfo = hover_mode
            )
            if (!large_data) {
                feat_trace_args$text <- paste0(
                    "Cell: ",
                    rownames(data_valid),
                    "\nValue: ",
                    round(data_valid[[features]], 3)
                )
            }
            p <- do_call(plotly::add_trace, feat_trace_args)
        }
    }

    ## Highlight
    if (!is.null(highlight)) {
        if (isTRUE(highlight)) {
            hi_df <- data
        } else if (length(highlight) == 1 && is.character(highlight)) {
            hi_df <- eval(parse(
                text = paste0("dplyr::filter(data, ", highlight, ")")
            ))
        } else {
            all_inst <- if (is.numeric(highlight)) {
                seq_len(nrow(data))
            } else {
                rownames(data)
            }
            if (!any(highlight %in% all_inst)) {
                stop("No highlight items found in the data (rownames).")
            }
            if (!all(highlight %in% all_inst)) {
                warning(
                    "Not all highlight items found in the data (rownames).",
                    immediate. = TRUE
                )
            }
            hi_df <- data[intersect(highlight, all_inst), , drop = FALSE]
        }
        if (nrow(hi_df) > 0) {
            hi_trace_args <- list(
                p = p,
                x = hi_df[[dims[1]]],
                y = hi_df[[dims[2]]],
                z = hi_df[[dims[3]]],
                type = "scatter3d",
                mode = "markers",
                marker = list(
                    size = highlight_size * 2 + highlight_stroke,
                    color = highlight_color,
                    opacity = highlight_alpha,
                    symbol = "circle-open"
                ),
                name = "highlight",
                showlegend = FALSE,
                hoverinfo = hover_mode
            )
            if (!large_data) {
                hi_trace_args$text <- paste0("Cell: ", rownames(hi_df))
            }
            p <- do_call(plotly::add_trace, hi_trace_args)
        }
    }

    ## Lineages
    if (!is.null(lineages)) {
        lineages <- unique(check_columns(
            data,
            lineages,
            force_factor = FALSE,
            allow_multi = TRUE
        ))
        lineage_colors <- palette_this(
            lineages,
            palette = lineages_palette,
            palcolor = lineages_palcolor,
            reverse = palreverse
        )

        for (l in lineages) {
            trim_pass <- (data[[l]] >
                quantile(data[[l]], lineages_trim[1], na.rm = TRUE) &
                data[[l]] < quantile(data[[l]], lineages_trim[2], na.rm = TRUE))
            na_pass <- !is.na(data[[l]])
            index <- which(trim_pass & na_pass)
            index <- index[order(data[index, l])]
            dat_sub <- data[index, , drop = FALSE]
            weights_used <- rep(1, nrow(dat_sub))

            fitted_vals <- lapply(dims, function(x) {
                loess(
                    formula(paste(x, l, sep = "~")),
                    weights = weights_used,
                    data = dat_sub,
                    span = lineages_span,
                    degree = 2
                )$fitted
            })
            names(fitted_vals) <- dims
            dat_smooth <- as.data.frame(fitted_vals)
            dat_smooth <- unique(na.omit(dat_smooth))

            if (nrow(dat_smooth) > 0) {
                p <- plotly::add_trace(
                    p = p,
                    x = dat_smooth[[dims[1]]],
                    y = dat_smooth[[dims[2]]],
                    z = dat_smooth[[dims[3]]],
                    text = paste0("Lineage: ", l),
                    type = "scatter3d",
                    mode = "lines",
                    line = list(width = 6, color = lineage_colors[l]),
                    name = l,
                    showlegend = TRUE,
                    hoverinfo = "text"
                )
            }
        }
    }

    ## Labels
    if (isTRUE(label) && !is.null(group_by)) {
        # Compute label positions at group median coordinates (same as 2D)
        label_data <- data[!is.na(data[[group_by]]), , drop = FALSE]
        label_df <- aggregate(
            label_data[, dims, drop = FALSE],
            by = list(label_data[[group_by]]),
            FUN = median
        )
        colnames(label_df)[1] <- ".group"
        if (!isTRUE(label_insitu)) {
            label_df[, ".label"] <- seq_len(nrow(label_df))
        } else {
            label_df[, ".label"] <- label_df[, ".group"]
        }
        # In 3D, plotly doesn't support text background/outline like geom_text_repel,
        # so we use label_bg as the text color for visibility
        p <- plotly::add_trace(
            p = p,
            x = label_df[[dims[1]]],
            y = label_df[[dims[2]]],
            z = label_df[[dims[3]]],
            text = as.character(label_df[[".label"]]),
            type = "scatter3d",
            mode = "text",
            textfont = list(
                size = label_size * 3,
                color = label_bg
            ),
            textposition = "middle center",
            hoverinfo = "text",
            showlegend = FALSE
        )
    }

    ## Layout
    # Use subtitle from parent (already includes show_stat "N = ..." if applicable)
    subtitle_text <- subtitle %||% ""
    if (!is.null(n_sampled)) {
        sample_note <- paste0("Showing ", n_sampled, " sampled points")
        if (nchar(subtitle_text) > 0) {
            subtitle_text <- paste0(subtitle_text, "; ", sample_note)
        } else {
            subtitle_text <- sample_note
        }
    }
    title_text <- title %||% ""
    if (nchar(title_text) > 0 && nchar(subtitle_text) > 0) {
        full_title <- paste0(title_text, "<br><sup>", subtitle_text, "</sup>")
    } else if (nchar(subtitle_text) > 0) {
        full_title <- paste0("<sup>", subtitle_text, "</sup>")
    } else {
        full_title <- title_text
    }

    p <- plotly::layout(
        p = p,
        title = list(
            text = full_title,
            font = list(size = 16, color = "black"),
            y = 0.95
        ),
        showlegend = TRUE,
        legend = list(
            itemsizing = "constant",
            y = 0.5,
            x = 1,
            xanchor = "left"
        ),
        scene = list(
            xaxis = list(title = xlab),
            yaxis = list(title = ylab),
            zaxis = list(title = zlab),
            aspectratio = list(x = 1, y = 1, z = 1)
        )
    )

    return(p)
}

#' Atomic Dimension Reduction Plot without splitting the data
#'
#' @description
#' Core implementation for dimension reduction visualisation. This is the
#' internal workhorse dispatched by both `DimPlot()` (group-based) and
#' `FeatureDimPlot()` (continuous feature expression). It renders a 2D
#' scatter plot of ordination axes with extensive annotation capabilities,
#' and automatically delegates to `DimPlotAtomic3D()` for interactive 3D
#' plots when three `dims` are provided.
#'
#' The function supports two primary colouring modes:
#' - **group_by** — discrete factor colouring with a legend of group levels,
#'   optional density/statistical overlays, and group mark / label annotations.
#' - **features** — continuous numeric colouring with a gradient colour bar,
#'   multi-feature faceting via `tidyr::pivot_longer()`, and optional cutoff
#'   / quantile trimming via `prepare_continuous_color_scale()`.
#'
#' Additional annotation layers include: graph / network edges drawn as
#' segments between connected nodes, 2D density contours (filled or outline),
#' group marks (hull, ellipse, rect, circle via `ggforce`), lineage curves
#' (LOESS-smoothed paths with optional whiskers), velocity / RNA-velocity
#' arrows (raw, grid, or stream via `VelocityPlot()`), statistical summary
#' mini-plots (pie, ring, bar, line) embedded at group centroids, background
#' points from other facets (faded context), and group labels with repulsion
#' (via `ggrepel::geom_text_repel()`).
#'
#' Rendering scales automatically: scatter points for small datasets,
#' `scattermore::geom_scattermore()` raster for `n > 1e5`, or hex-binned
#' aggregation (`stat_summary_hex()` / `geom_hex()`). Legend assembly uses
#' `cowplot::get_plot_component()` with independent guide-boxes for base
#' groups, lineages, velocity, and stat-by annotations — combined via
#' `rbind` / `cbind` and re-inserted with `add_grob()`.
#'
#' @section Architecture:
#' **DimPlotAtomic** executes the following steps:
#'
#' 1. **ggplot dispatch** — selects `gglogger::ggplot` or `ggplot2::ggplot`.
#' 2. **Order validation** — `match.arg(order)`.
#' 3. **Dimension resolution** — converts numeric indices to column names,
#'    validates exactly 2 or 3 dims via `check_columns()`.
#' 4. **Column resolution** — validates `group_by` (force_factor, concat_multi),
#'    `features`, and `facet_by`. Requires at least one of `group_by` or
#'    `features`. Blocks `facet_by` when multiple features are present (features
#'    themselves become facets).
#' 5. **Auto defaults** — `pt_size` defaults to `min(3000 / nrow(data), 0.6)`
#'    when `NULL`. If a character string is given, it is treated as a column name
#'    for variable point sizing (2D only, ignored in raster mode);
#'    `raster` auto-enables when `nrow(data) > 1e5`; `theme_blank` suppresses
#'    axis labels.
#' 6. **Label force-enable** — if `label_repel` or `label_insitu` is `TRUE`,
#'    `label` is forced to `TRUE` with a message.
#' 7. **NA / empty handling** — `process_keep_na_empty()` filters the data;
#'    `keep_empty` values extracted for group_by and facet_by.
#' 8. **group_by preprocessing** — group values / colours resolved via
#'    `palette_this()`. NA levels mapped to literal `"NA"` string.
#'    `label_use` is constructed based on `label`, `label_insitu`, and
#'    `show_stat` combinations. Facet labeller is set to `as_labeller()` with
#'    per-facet N annotations when `show_stat = TRUE`.
#' 9. **Multi-feature pivot** — when `length(features) > 1`, data is pivoted
#'    to long format via `tidyr::pivot_longer()` with `.feature` as the facet
#'    variable and `.value` as the numeric column.
#' 10. **Continuous colour scale** — when `features` is set,
#'     `prepare_continuous_color_scale()` applies quantile/cutoff trimming
#'     and returns anchor values for the gradient.
#' 11. **Point ordering** — applies the selected `order` strategy (as-is,
#'     reverse, high-top, low-top, random) by sorting or shuffling rows.
#' 12. **3D branch** — if `length(dims) == 3`, emits a warning for unsupported
#'     features, optionally down-samples with stratified group sampling (when
#'     `raster = TRUE`), and delegates to `DimPlotAtomic3D()`.
#' 13. **Group marks** (2D only, when `add_mark = TRUE`) — dispatches to
#'     `ggforce::geom_mark_hull/ellipse/rect/circle`, with `new_scale_fill()`
#'     / `new_scale_color()` to isolate mark scales.
#' 14. **Graph / network** (2D only) — resolves the graph source (same logic
#'     as 3D), melts the matrix, handles faceted data by per-facet edge
#'     splitting, and adds `geom_segment(aes(linewidth = value))` with
#'     `scale_linewidth_continuous()`.
#' 15. **Density overlay** (2D only) — filled (`stat_density_2d(geom = "raster")`)
#'     or outline (`geom_density_2d()`).
#' 16. **Base scales and theme** — x / y limits from data range,
#'     `do_call(theme, theme_args)`, aspect ratio and legend position.
#' 17. **Background points for facet context** — for faceted plots (excluding
#'     multi-feature auto-faceting), points from other facets are added as
#'     faded background (raster / hex / point depending on settings).
#' 18. **Main point layer** — dispatches by rendering mode:
#'     - *raster*: `scattermore::geom_scattermore()` with separate NA / non-NA
#'       group layers.
#'     - *hex*: `geom_hex()` (group_by with optional count alpha, or
#'       `stat_summary_hex()` for features). Raises `has_fill = TRUE`.
#'     - *standard*: `geom_point()`.
#' 19. **Highlight** (2D only) — resolves highlight specification (TRUE, filter
#'     expression, row names, indices), errors on hex + highlight combo, renders
#'     highlight points with stroke (outer ring + inner colour).
#' 20. **Colour scales** — `scale_color_manual()` for group_by (with
#'     `keep_empty`-aware `breaks`/`limits`/`drop`), or
#'     `scale_color_gradientn()` for features. `scale_fill_manual()` /
#'     `scale_fill_gradientn()` added when `has_fill` is TRUE.
#' 21. **Base legend** — extracted via `cowplot::get_plot_component("guide-box-bottom")`.
#' 22. **Lineages** (2D only, no facet_by) — per-lineage LOESS fitting
#'     (`span = lineages_span, degree = 2`) with optional whiskers connecting
#'     smoothed to raw coordinates. Rendered as `geom_path()` with background
#'     stroke + foreground colour, plus `scale_color_manual()`. Legend extracted
#'     as a separate guide-box.
#' 23. **Velocity** (2D only, no facet_by) — delegates to `VelocityPlot()` with
#'     `return_layer = TRUE`. Adds `new_scale_color()` when the velocity layer
#'     has its own colour scale. Legend extracted separately.
#' 24. **Stat-by mini-plots** (2D only, no facet_by, no features) — dispatches
#'     to `PieChart()` / `RingPlot()` / `BarPlot()` / `LinePlot()` via
#'     `do_call()`. Each mini-plot is rendered as `annotation_custom()` at the
#'     group's median coordinates, scaled by `stat_plot_size * range`.
#'     Legend extracted from the stat plot.
#' 25. **Group labels** (2D only, no features) — `geom_text_repel()` with
#'     optional repulsion (`force = label_repulsion`) or fixed segments
#'     (`force = 0`, bold, `min.segment.length = 0`). Labels positioned at
#'     group median coordinates (computed per facet if applicable).
#' 26. **Legend assembly** — when additional legends exist (lineages, velocity,
#'     stat_by), they are combined with the base legend via `cbind` (vertical
#'     direction) or `rbind` (horizontal). The combined legend is re-inserted
#'     via `add_grob(gtable, legend, legend.position)`.
#' 27. **Dimension calculation** — `calculate_plot_dimensions(base_height = 5.5,
#'     aspect.ratio, legend_n, legend_nchar)` sets `height` / `width` attributes.
#' 28. **Faceting** — `facet_plot()` is called only when no additional legends
#'     were assembled (otherwise the combined-legend grob is returned directly).
#'
#' @inheritParams common_args
#' @inheritParams VelocityPlot
#' @param dims A character vector of the column names to plot on the x, y (and optionally z) axes or a numeric vector
#'  of the column indices. When 3 dimensions are provided, a 3D interactive plot is created using plotly.
#'  Supported in 3D: group_by, features, labels, highlight, lineages, graph/network, show_stat, order.
#'  Not supported in 3D: add_mark, stat_by, add_density, velocity, hex, facet_by, raster.
#' @param features A character vector of the column names to plot as features (continuous colouring).
#'  When multiple features are provided and `facet_by` is not set, the data is pivoted to long format and
#'  faceted by feature name.
#' @param group_by A character string of the column name to group the data by for discrete colouring.
#'  A character/factor column is expected. If multiple columns are provided, the columns will be concatenated with `group_by_sep`.
#' @param group_by_sep A character string to concatenate the columns in `group_by`, if multiple columns are provided.
#' @param pt_size A numeric value for point size, or a character string naming a numeric column
#'  in \code{data} to map point sizes to. If \code{NULL} (default), auto-calculated as
#'  \code{min(3000 / nrow(data), 0.6)}. Column-name mapping only applies to 2D plots and
#'  is ignored with a message when \code{raster = TRUE}.
#' @param pt_alpha A numeric value in `[0, 1]` for the point transparency. Default is `1`.
#' @param bg_color A character string specifying the colour used for NA-valued points and
#'  background context points drawn from other facets. Default is `"grey80"`.
#' @param label_insitu A logical value. If `TRUE`, the raw group names are placed at the group median
#'  coordinates instead of numeric indices. Forces `label = TRUE`. Default is `FALSE`.
#' @param show_stat A logical value. If `TRUE` (default), the number of points per group is shown
#'  in the legend labels and subtitle. Ignored when `theme = "theme_blank"`.
#' @param order A character string controlling the draw order of points:
#'  * `"as-is"` (default) — the row order in the data is preserved.
#'  * `"reverse"` — rows are reversed.
#'  * `"high-top"` — points with high values (last factor levels for group_by) are drawn last (on top).
#'  * `"low-top"` — points with low values (first factor levels) are drawn last.
#'  * `"random"` — rows are randomly shuffled.
#'
#'  For `high-top` and `low-top`, NA values are always plotted at the bottom.
#'  When applied to `group_by`, only the draw order changes — legend colours and order
#'  are unaffected. Within the same level, point order is preserved. For precise
#'  control, set factor levels before plotting.
#'  See <https://github.com/pwwang/scplotter/issues/29#issuecomment-3009694130> for examples.
#' @param label A logical value. If `TRUE`, group labels (numeric indices by default, or group
#'  names when `label_insitu = TRUE`) are placed at the median coordinates of each group.
#'  Forced to `TRUE` when `label_repel` or `label_insitu` is set.
#' @param label_size A numeric value for the label text size. Passed to `ggrepel::geom_text_repel()`.
#'  Default is `4`.
#' @param label_fg A character string for the label text (foreground) colour. Default is `"white"`.
#' @param label_bg A character string for the label background / outline colour. Default is `"black"`.
#' @param label_bg_r A numeric value for the background fill ratio of the label bounding box.
#'  Passed to `ggrepel::geom_text_repel(bg.r = ...)`. Default is `0.1`.
#' @param label_repel A logical value. If `TRUE`, labels are repelled from each other with force
#'  `label_repulsion`. A visible point anchor is drawn. Forces `label = TRUE`.
#' @param label_repulsion A numeric value for the repulsion force when `label_repel = TRUE`.
#'  Passed to `ggrepel::geom_text_repel(force = ...)`. Default is `20`.
#' @param label_pt_size A numeric value for the size of the anchor point drawn when
#'  `label_repel = TRUE`. Default is `1`.
#' @param label_pt_color A character string for the colour of the label anchor point. Default is `"black"`.
#' @param label_segment_color A character string for the colour of the line segment connecting
#'  the label to the anchor. Used in non-repel mode (`label_repel = FALSE`) where
#'  `min.segment.length = 0`. Default is `"black"`.
#' @param highlight A specification for highlighted points:
#'  - `NULL` (default): no highlighting.
#'  - `TRUE`: highlight all points (adds a dark outline around every point).
#'  - A character string: a dplyr filter expression (e.g., `"clusters == 'Ductal'"`).
#'  - A character vector: row names to highlight.
#'  - A numeric vector: row indices to highlight.
#' @param highlight_alpha A numeric value in `[0, 1]` for the transparency of highlighted points.
#'  Default is `1`.
#' @param highlight_size A numeric value for the size of the inner (coloured) highlight point.
#'  Default is `1`.
#' @param highlight_color A character string for the colour of the outer highlight ring.
#'  Default is `"black"`.
#' @param highlight_stroke A numeric value for the thickness of the outer highlight ring (the
#'  difference between the outer ring size and `highlight_size`). Default is `0.8`.
#' @param add_mark A logical value. If `TRUE`, group boundaries are drawn around points using
#'  `ggforce` marks. Requires `group_by`. Only supported in 2D.
#' @param mark_type A character string specifying the mark shape. Options:
#'  `"hull"` (convex hull, default), `"ellipse"`, `"rect"`, or `"circle"`.
#' @param mark_expand A unit value for the outward expansion of the mark boundary.
#'  Passed to `ggforce::geom_mark_*(expand = ...)`. Default is `unit(3, "mm")`.
#' @param mark_alpha A numeric value in `[0, 1]` for the transparency of the mark fill.
#'  Default is `0.1`.
#' @param mark_linetype A numeric value for the line type of the mark boundary. Default is `1` (solid).
#' @param stat_by A character string naming a column used to compute per-group statistical summary
#'  mini-plots embedded at group centroid positions. Only supported with `group_by` (not `features`).
#'  Only supported in 2D without `facet_by`.
#' @param stat_plot_type A character string specifying the mini-plot type. Options:
#'  `"pie"` (default), `"ring"`, `"bar"`, or `"line"`.
#' @param stat_plot_size A numeric value for the size of the stat mini-plot, expressed as a
#'  fraction of the axis range. Default is `0.1`.
#' @param stat_args A list of additional arguments passed to the stat plot function
#'  (e.g., `list(palette = "Set1")`). Default is `list(palette = "Set1")`.
#' @param graph A specification for network / graph edges to overlay. Sources:
#'  - A character string starting with `"@"` (e.g., `"@graph"`): extracts the attribute
#'    named `"graph"` from `attributes(data)`.
#'  - A `Graph` object (e.g., Seurat): coerced to dense matrix via `as.matrix()`.
#'  - A `matrix`, `data.frame`, or `dgCMatrix`: used directly as the adjacency matrix.
#'  - Numeric indices or character column names: extracts columns from `data`.
#'  Edges are drawn for non-zero, lower-triangle entries. Requires `data` to have row names
#'  matching the matrix dimnames.
#' @param edge_size A numeric vector of length 2 specifying the range `[min, max]` for
#'  `scale_linewidth_continuous(range = ...)` applied to edge widths. Default is `c(0.05, 0.5)`.
#' @param edge_alpha A numeric value in `[0, 1]` for the transparency of graph edges.
#'  Default is `0.1`.
#' @param edge_color A character string for the colour of graph edges. Default is `"grey40"`.
#' @param add_density A logical value. If `TRUE`, a 2D density layer is overlaid. Only supported in 2D.
#' @param density_color A character string for the colour of the density contour lines.
#'  Used when `density_filled = FALSE`. Default is `"grey80"`.
#' @param density_filled A logical value. If `TRUE`, the density is rendered as a filled raster
#'  (`stat_density_2d(geom = "raster")`) instead of contour lines. A separate fill scale is used.
#' @param density_filled_palette A character string naming the palette for the filled density layer.
#'  Default is `"Greys"`.
#' @param density_filled_palcolor A character vector of specific colours for the filled density
#'  palette. Default is `NULL` (auto-resolved from `density_filled_palette`).
#' @param lineages A character vector of column names representing pseudotime / trajectory lineages.
#'  Each column is fitted with a LOESS smooth (`span = lineages_span, degree = 2`) across the
#'  2D embedding, after trimming the top and bottom `lineages_trim` quantiles.
#'  Only supported in 2D without `facet_by`.
#' @param lineages_trim A numeric vector of length 2 specifying the lower and upper quantile
#'  thresholds `[0, 1]` for trimming lineage values before LOESS fitting.
#'  Default is `c(0.01, 0.99)`.
#' @param lineages_span A numeric value passed as `span` to `stats::loess()` controlling the
#'  smoothness of the lineage curve. Smaller values follow the data more closely.
#'  Default is `0.75`.
#' @param lineages_palette A character string naming the palette for lineage colours.
#'  Default is `"Dark2"`.
#' @param lineages_palcolor A character vector of specific colours for lineage curves.
#'  Default is `NULL` (auto-resolved from `lineages_palette`).
#' @param lineages_arrow A ggplot2 `arrow` specification applied to the end of lineage paths.
#'  Default is `arrow(length = unit(0.1, "inches"))`.
#' @param lineages_linewidth A numeric value for the width of the lineage curve lines.
#'  Default is `1`.
#' @param lineages_line_bg A character string for the colour of the background (wider) stroke
#'  drawn behind each lineage curve for improved visibility. Default is `"white"`.
#' @param lineages_line_bg_stroke A numeric value for the additional width of the background
#'  stroke relative to `lineages_linewidth`. The background line has total width
#'  `lineages_linewidth + lineages_line_bg_stroke`. Default is `0.5`.
#' @param lineages_whiskers A logical value. If `TRUE`, short line segments connect the
#'  smoothed lineage curve to the original data coordinates of the fitted points.
#'  Default is `FALSE`.
#' @param lineages_whiskers_linewidth A numeric value for the width of the whisker lines.
#'  Default is `0.5`.
#' @param lineages_whiskers_alpha A numeric value in `[0, 1]` for the transparency of the
#'  whisker lines. Default is `0.5`.
#' @param velocity A specification for RNA-velocity arrows. Can be:
#'  - `NULL` (default): no velocity overlay.
#'  - A character / integer vector: column names or indices in `data` for the velocity embedding.
#'  - A data frame or matrix: the velocity embedding itself (must align with `data` rows).
#'  Only supported in 2D without `facet_by`.
#' @param velocity_plot_type A character string specifying the velocity rendering style.
#'  Options: `"raw"` (arrows from embedding), `"grid"` (grid-based arrows), or `"stream"`
#'  (streamlines). Default is `"raw"`.
#' @param velocity_n_neighbors A numeric value for the number of neighbours used in the
#'  velocity grid computation. Default is `NULL` (auto).
#' @param velocity_density A numeric value for the velocity kernel density bandwidth.
#'  Default is `1`.
#' @param velocity_smooth A numeric value for the velocity smoothing parameter.
#'  Default is `0.5`.
#' @param velocity_scale A numeric value for scaling the velocity arrows. Default is `1`.
#' @param velocity_min_mass A numeric value for the minimum cell mass threshold in velocity
#'  grid computation. Default is `1`.
#' @param velocity_cutoff_perc A numeric value for the velocity cutoff percentage.
#'  Default is `5`.
#' @param velocity_group_palette A character string naming the palette for velocity group
#'  colours (used in `"raw"` plot type). Default is `"Set2"`.
#' @param velocity_group_palcolor A character vector of specific colours for velocity groups.
#'  Default is `NULL` (auto-resolved from `velocity_group_palette`).
#' @param raster A logical value. If `TRUE`, points are rendered via
#'  `scattermore::geom_scattermore()` for efficient rasterised plotting.
#'  Default is `NULL`, which auto-enables when `nrow(data) > 1e5`.
#' @param raster_dpi A numeric vector of length 2 `[x_dpi, y_dpi]` specifying the raster
#'  resolution in pixels. Passed to `scattermore::geom_scattermore(pixels = ...)`.
#'  Default is `c(512, 512)`. If a single value is provided it is recycled to both dimensions.
#' @param hex A logical value. If `TRUE`, points are rendered as hexagonal bins via
#'  `geom_hex()` / `stat_summary_hex()`. Not supported with highlight.
#'  Default is `FALSE`. Only supported in 2D.
#' @param hex_linewidth A numeric value for the width of the hexagon boundary lines.
#'  Default is `0.5`.
#' @param hex_count A logical value. If `TRUE` and `group_by` is set, hex fill alpha
#'  is mapped to `after_stat(count)` so denser bins are more opaque.
#'  For `features` mode hex_count is ignored. Default is `!is.null(group_by)`.
#' @param hex_bins A numeric value for the number of hex bins along each axis.
#'  Passed to `geom_hex(bins = ...)`. Default is `50`.
#' @param hex_binwidth A numeric value for the width of individual hex bins.
#'  Passed to `geom_hex(binwidth = ...)`. Takes precedence over `hex_bins` when set.
#' @param bg_cutoff A numeric threshold. Feature values with absolute value below this
#'  cutoff are set to `NA` (and therefore rendered in `bg_color`). Default is `NULL`.
#' @param color_name A character string used as the title for the continuous colour bar
#'  in feature mode. Default is `""`.
#' @return A ggplot object or a plotly object (when 3 dimensions are provided)
#' @keywords internal
#' @importFrom scales rescale
#' @importFrom stats loess formula na.omit aggregate
#' @importFrom dplyr %>% group_by group_map pull filter
#' @importFrom tidyr pivot_longer
#' @importFrom ggplot2 scale_linewidth_continuous geom_density_2d stat_density_2d geom_hex
#' @importFrom ggplot2 ggplotGrob theme_void as_labeller stat_summary_hex
#' @importFrom cowplot get_plot_component
DimPlotAtomic <- function(
    data,
    dims = 1:2,
    group_by = NULL,
    group_by_sep = "_",
    features = NULL,
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    pt_size = NULL,
    pt_alpha = 1,
    bg_color = "grey80",
    bg_cutoff = NULL,
    color_name = "",
    label_insitu = FALSE,
    show_stat = !identical(theme, "theme_blank"),
    label = FALSE,
    label_size = 4,
    label_fg = "white",
    label_bg = "black",
    label_bg_r = 0.1,
    label_repel = FALSE,
    label_repulsion = 20,
    label_pt_size = 1,
    label_pt_color = "black",
    label_segment_color = "black",
    order = c("as-is", "reverse", "high-top", "low-top", "random"),
    highlight = NULL,
    highlight_alpha = 1,
    highlight_size = 1,
    highlight_color = "black",
    highlight_stroke = 0.8,
    add_mark = FALSE,
    mark_type = c("hull", "ellipse", "rect", "circle"),
    mark_expand = unit(3, "mm"),
    mark_alpha = 0.1,
    mark_linetype = 1,
    stat_by = NULL,
    stat_plot_type = c("pie", "ring", "bar", "line"),
    stat_plot_size = 0.1,
    stat_palette = "Set1",
    stat_args = list(),
    graph = NULL,
    edge_size = c(0.05, 0.5),
    edge_alpha = 0.1,
    edge_color = "grey40",
    add_density = FALSE,
    density_color = "grey80",
    density_filled = FALSE,
    density_filled_palette = "Greys",
    density_filled_palcolor = NULL,
    lineages = NULL,
    lineages_trim = c(0.01, 0.99),
    lineages_span = 0.75,
    lineages_palette = "Dark2",
    lineages_palcolor = NULL,
    lineages_arrow = ggplot2::arrow(length = unit(0.1, "inches")),
    lineages_linewidth = 1,
    lineages_line_bg = "white",
    lineages_line_bg_stroke = 0.5,
    lineages_whiskers = FALSE,
    lineages_whiskers_linewidth = 0.5,
    lineages_whiskers_alpha = 0.5,
    velocity = NULL,
    velocity_plot_type = c("raw", "grid", "stream"),
    velocity_n_neighbors = NULL,
    velocity_density = 1,
    velocity_smooth = 0.5,
    velocity_scale = 1,
    velocity_min_mass = 1,
    velocity_cutoff_perc = 5,
    velocity_group_palette = "Set2",
    velocity_group_palcolor = NULL,
    arrow_angle = 20,
    arrow_color = "black",
    streamline_l = 5,
    streamline_minl = 1,
    streamline_res = 1,
    streamline_n = 15,
    arrow_alpha = 1,
    streamline_width = c(0, 0.8),
    streamline_alpha = 1,
    streamline_color = NULL,
    streamline_palette = "RdYlBu",
    streamline_palcolor = NULL,
    streamline_bg_color = "white",
    streamline_bg_stroke = 0.5,
    keep_na = FALSE,
    keep_empty = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_nrow = NULL,
    facet_ncol = NULL,
    facet_byrow = TRUE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    theme = "theme_this",
    theme_args = list(),
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    raster = NULL,
    raster_dpi = c(512, 512),
    hex = FALSE,
    hex_linewidth = 0.5,
    hex_count = !is.null(group_by),
    hex_bins = 50,
    hex_binwidth = NULL,
    palette = ifelse(is.null(features), "Paired", "Spectral"),
    palcolor = NULL,
    palreverse = FALSE,
    seed = 8525,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    order <- match.arg(order)
    ## Setting up the parameters
    if (is.numeric(dims)) {
        dims <- colnames(data)[dims]
    }
    if (!length(dims) %in% c(2, 3)) {
        stop("Only 2 or 3 dimensions are allowed for dimension reduction plot.")
    }
    dims <- check_columns(data, dims, allow_multi = TRUE)
    if (length(raster_dpi) == 1) {
        raster_dpi <- rep(raster_dpi, 2)
    }

    group_by <- check_columns(
        data,
        group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = group_by_sep
    )
    features <- check_columns(data, features, allow_multi = TRUE)
    if (is.null(group_by) && is.null(features)) {
        stop("Either 'group_by' or 'features' should be specified.")
    }
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )
    if (length(features) > 1 && !is.null(facet_by)) {
        stop(
            "Cannot specify 'facet_by' with multiple features. The plot will be faceted by features."
        )
    }
    if (!is.numeric(pt_size) && !is.null(pt_size)) {
        pt_size <- check_columns(data, pt_size)
    }
    pt_size <- pt_size %||% min(3000 / nrow(data), 0.6)
    pt_size_num <- if (is.numeric(pt_size)) {
        pt_size
    } else {
        min(3000 / nrow(data), 0.6)
    }
    raster <- raster %||% (nrow(data) > 1e5)
    if (!is.numeric(pt_size) && isTRUE(raster)) {
        warning(
            "[DimPlot] 'raster' is enabled. Column-based point size ('pt_size') ",
            "is ignored, using numeric fallback for raster mode.",
            immediate. = TRUE
        )
        pt_size <- pt_size_num
    }
    xlab <- xlab %||% dims[1]
    ylab <- ylab %||% dims[2]
    if (identical(theme, "theme_blank")) {
        theme_args[["xlab"]] <- xlab
        theme_args[["ylab"]] <- ylab
    }
    if ((isTRUE(label_repel) || isTRUE(label_insitu)) && !isTRUE(label)) {
        warning(
            "[DimPlot] Forcing label to be TRUE when label_repel or label_insitu is TRUE.",
            immediate. = TRUE
        )
        label <- TRUE
    }

    data <- process_keep_na_empty(data, keep_na, keep_empty)
    keep_empty_group <- if (!is.null(group_by)) keep_empty[[group_by]] else NULL
    keep_empty_facet <- if (!is.null(facet_by)) {
        keep_empty[[facet_by[1]]]
    } else {
        NULL
    }
    if (length(facet_by) > 1) {
        keep_empty_facet2 <- if (!is.null(facet_by[2])) {
            keep_empty[[facet_by[2]]]
        } else {
            NULL
        }
        stopifnot(
            "[(Feature)DimPlot] `keep_empty` for `facet_by` variables must be identical." = identical(
                keep_empty_facet,
                keep_empty_facet2
            )
        )
    }

    facet_labeller <- "label_value"
    if (!is.null(group_by)) {
        group_vals <- levels(data[[group_by]])
        if (anyNA(data[[group_by]])) {
            group_vals <- c(group_vals, NA)
        }

        colors <- palette_this(
            group_vals,
            palette = palette,
            palcolor = palcolor,
            NA_keep = TRUE,
            reverse = palreverse
        )
        names(colors)[is.na(names(colors))] <- "NA"
        group_vals[is.na(group_vals)] <- "NA"
        labels_tb <- table(data[[group_by]], useNA = "ifany")
        # labels_tb <- labels_tb[labels_tb != 0]
        # convert NA names to "NA"
        names(labels_tb)[is.na(names(labels_tb))] <- "NA"
        labels_tb <- labels_tb[group_vals]

        if (isTRUE(label_insitu)) {
            if (isTRUE(show_stat)) {
                label_use <- paste0(names(labels_tb), "(", labels_tb, ")")
            } else {
                label_use <- paste0(names(labels_tb))
            }
        } else {
            if (isTRUE(label)) {
                if (isTRUE(show_stat)) {
                    label_use <- paste0(
                        seq_along(labels_tb),
                        ": ",
                        names(labels_tb),
                        "(",
                        labels_tb,
                        ")"
                    )
                } else {
                    label_use <- paste0(
                        seq_along(labels_tb),
                        ": ",
                        names(labels_tb)
                    )
                }
            } else {
                if (isTRUE(show_stat)) {
                    label_use <- paste0(names(labels_tb), "(", labels_tb, ")")
                } else {
                    label_use <- paste0(names(labels_tb))
                }
            }
        }
        if (isTRUE(show_stat)) {
            if (is.null(facet_by)) {
                subtitle <- subtitle %||%
                    paste0("N = ", sum(!is.na(data[[group_by]])))
            } else if (length(facet_by) == 1) {
                facet_labeller <- as_labeller(function(values) {
                    sapply(values, function(v) {
                        data_sub <- data[data[[facet_by]] == v, , drop = FALSE]
                        paste0(
                            v,
                            " (N = ",
                            sum(!is.na(data_sub[[group_by]])),
                            ")"
                        )
                    })
                })
            }
        }
    }

    ## Making long data for features when length(features) > 1
    if (length(dims) == 3 && length(features) > 1) {
        stop(
            "Multiple features are not supported for 3D plots. ",
            "Use `split_by = TRUE` in `FeatureDimPlot()` to plot features separately with `combine = FALSE`."
        )
    }
    multifeats <- length(features) > 1
    if (multifeats) {
        data <- data %>%
            pivot_longer(
                cols = features,
                names_to = ".feature",
                values_to = ".value"
            )
        data$.feature <- factor(data$.feature, levels = features)
        facet_by <- ".feature"
        features <- ".value"
    }
    if (!is.null(features)) {
        result <- prepare_continuous_color_scale(
            data,
            features,
            lower_quantile = lower_quantile,
            upper_quantile = upper_quantile,
            lower_cutoff = lower_cutoff,
            upper_cutoff = upper_cutoff,
            bg_cutoff = bg_cutoff
        )
        data <- result$data
        feat_colors_value <- result$feat_colors_value
    }
    colorby <- ifelse(is.null(features), group_by, features)
    if (order == "reverse") {
        data <- data[nrow(data):1, , drop = FALSE]
    } else if (order == "high-top") {
        data <- dplyr::arrange(data, !is.na(!!sym(colorby)), !!sym(colorby))
    } else if (order == "low-top") {
        data <- dplyr::arrange(
            data,
            !is.na(!!sym(colorby)),
            dplyr::desc(!!sym(colorby))
        )
    } else if (order == "random") {
        data <- data[sample(nrow(data)), , drop = FALSE]
    }

    ## 3D plot using plotly
    if (length(dims) == 3) {
        unsupported_args <- c(
            if (isTRUE(add_mark)) "add_mark",
            if (!is.null(stat_by)) "stat_by",
            if (isTRUE(add_density)) "add_density",
            if (!is.null(velocity)) "velocity",
            if (isTRUE(hex)) "hex",
            if (!is.null(facet_by)) "facet_by"
        )
        if (length(unsupported_args) > 0) {
            warning(
                "The following features are not supported for 3D plots and will be ignored: ",
                paste(unsupported_args, collapse = ", "),
                immediate. = TRUE
            )
        }

        # Downsample for large datasets (analogous to raster in 2D)
        n_sampled <- NULL
        if (isTRUE(raster) && nrow(data) > prod(raster_dpi)) {
            set.seed(seed)
            max_pts <- prod(raster_dpi)
            if (!is.null(group_by)) {
                # Stratified sampling: proportional to group size, at least 1 per non-empty group
                grp_vals_all <- as.character(data[[group_by]])
                grp_vals_all[is.na(grp_vals_all)] <- "NA"
                grp_counts <- table(grp_vals_all)
                grp_n <- pmax(
                    1L,
                    as.integer(round(grp_counts / sum(grp_counts) * max_pts))
                )
                names(grp_n) <- names(grp_counts)
                sampled_idx <- unlist(lapply(names(grp_counts), function(g) {
                    idx <- which(grp_vals_all == g)
                    sample(idx, min(length(idx), grp_n[g]))
                }))
                data <- data[sort(sampled_idx), , drop = FALSE]
            } else {
                data <- data[sort(sample(nrow(data), max_pts)), , drop = FALSE]
            }
            n_sampled <- nrow(data)
        }

        return(DimPlotAtomic3D(
            data = data,
            dims = dims,
            group_by = group_by,
            features = features,
            colorby = colorby,
            colors = if (!is.null(group_by)) colors else NULL,
            feat_colors_value = if (!is.null(features)) {
                feat_colors_value
            } else {
                NULL
            },
            label_use = if (!is.null(group_by)) label_use else NULL,
            labels_tb = if (!is.null(group_by)) labels_tb else NULL,
            keep_empty_group = isTRUE(keep_empty_group),
            bg_color = bg_color,
            color_name = color_name,
            pt_size = pt_size,
            pt_alpha = pt_alpha,
            show_stat = show_stat,
            title = title,
            subtitle = subtitle,
            xlab = xlab,
            ylab = ylab,
            label = label,
            label_insitu = label_insitu,
            label_size = label_size,
            label_fg = label_fg,
            label_bg = label_bg,
            highlight = highlight,
            highlight_color = highlight_color,
            highlight_size = highlight_size,
            highlight_stroke = highlight_stroke,
            highlight_alpha = highlight_alpha,
            graph = graph,
            edge_size = edge_size,
            edge_alpha = edge_alpha,
            edge_color = edge_color,
            lineages = lineages,
            lineages_trim = lineages_trim,
            lineages_span = lineages_span,
            lineages_palette = lineages_palette,
            lineages_palcolor = lineages_palcolor,
            palette = palette,
            palcolor = palcolor,
            palreverse = palreverse,
            n_sampled = n_sampled
        ))
    }

    # Do we have fill scale?
    has_fill <- FALSE
    p <- ggplot(data)

    ## Adding the mark
    if (isTRUE(add_mark)) {
        if (is.null(group_by)) {
            stop("Cannot add mark without 'group_by'.")
        }
        # if (!requireNamespace("ggforce", quietly = TRUE)) {
        #     stop("'ggforce' package is required for adding mark to the plot.")
        # }
        mark_type <- match.arg(mark_type)
        mark_fun <- switch(
            mark_type,
            hull = ggforce::geom_mark_hull,
            ellipse = ggforce::geom_mark_ellipse,
            rect = ggforce::geom_mark_rect,
            circle = ggforce::geom_mark_circle
        )
        p <- p +
            mark_fun(
                data = data[!is.na(data[[group_by]]), , drop = FALSE],
                mapping = aes(
                    x = !!sym(dims[1]),
                    y = !!sym(dims[2]),
                    color = !!sym(group_by),
                    fill = !!sym(group_by)
                ),
                expand = mark_expand,
                alpha = mark_alpha,
                linetype = mark_linetype,
                show.legend = FALSE
            ) +
            scale_fill_manual(
                values = colors[names(labels_tb)],
                guide = "none"
            ) +
            scale_color_manual(
                values = colors[names(labels_tb)],
                guide = "none"
            ) +
            new_scale_fill() +
            new_scale_color()
    }

    ## Adding the graph/network
    if (!is.null(graph)) {
        if (
            is.character(graph) && length(graph) == 1 && startsWith(graph, "@")
        ) {
            graph <- substring(graph, 2)
            net_mat <- attr(data, graph)
            if (is.null(net_mat)) {
                stop(paste0(
                    "[DimPlot] The graph '",
                    graph,
                    "' is not found in the data attributes."
                ))
            }
        } else if (inherits(graph, "Graph")) {
            # SeuratObject Graph
            net_mat <- as.matrix(graph)
        } else if (
            is.matrix(graph) ||
                is.data.frame(graph) ||
                inherits(graph, "dgCMatrix")
        ) {
            net_mat <- graph
        } else if (is.numeric(graph)) {
            graph <- colnames(data)[graph]
            net_mat <- data[graph]
        } else if (is.character(graph)) {
            net_mat <- data[graph]
        } else {
            stop(
                "[DimPlot] The 'graph' should be a matrix, data.frame, Graph object, indexes, or column names."
            )
        }

        if (!is.matrix(net_mat)) {
            net_mat <- as.matrix(net_mat)
        }

        if (!is.null(rownames(net_mat)) && !is.null(colnames(net_mat))) {
            net_mat <- net_mat[rownames(data), rownames(data)]
        } else if (is.null(rownames(net_mat))) {
            stop("The graph matrix should have rownames and colnames.")
        } else if (is.null(rownames(data))) {
            stop("'graph' requires the input data to have rownames.")
        }
        net_mat[net_mat == 0] <- NA
        net_mat[upper.tri(net_mat)] <- NA

        handle_single_facet_value <- function(mat) {
            net_df <- reshape2::melt(
                mat,
                na.rm = TRUE,
                stringsAsFactors = FALSE
            )
            net_df$value <- as.numeric(net_df$value)
            net_df$Var1 <- as.character(net_df$Var1)
            net_df$Var2 <- as.character(net_df$Var2)
            net_df$x <- data[net_df$Var1, dims[1]]
            net_df$y <- data[net_df$Var1, dims[2]]
            net_df$xend <- data[net_df$Var2, dims[1]]
            net_df$yend <- data[net_df$Var2, dims[2]]
            return(net_df)
        }

        if (!is.null(facet_by)) {
            net_df <- do_call(
                rbind,
                lapply(split(data, data[, facet_by]), function(d) {
                    d <- handle_single_facet_value(net_mat[
                        rownames(d),
                        rownames(d)
                    ])
                    d[, facet_by] <- d[1, facet_by]
                    d
                })
            )
        } else {
            net_df <- handle_single_facet_value(net_mat)
        }

        p <- p +
            geom_segment(
                data = net_df,
                mapping = aes(
                    x = !!sym("x"),
                    y = !!sym("y"),
                    xend = !!sym("xend"),
                    yend = !!sym("yend"),
                    linewidth = !!sym("value")
                ),
                color = edge_color,
                alpha = edge_alpha,
                show.legend = FALSE
            ) +
            scale_linewidth_continuous(range = edge_size, guide = "none")
    }

    ## Adding the density plot
    if (isTRUE(add_density)) {
        if (isTRUE(density_filled)) {
            filled_color <- palette_this(
                palette = density_filled_palette,
                palcolor = density_filled_palcolor,
                reverse = palreverse
            )
            p <- p +
                stat_density_2d(
                    geom = "raster",
                    aes(
                        x = !!sym(dims[1]),
                        y = !!sym(dims[2]),
                        fill = after_stat(!!sym("density"))
                    ),
                    contour = FALSE,
                    inherit.aes = FALSE,
                    show.legend = FALSE
                ) +
                scale_fill_gradientn(name = "Density", colours = filled_color) +
                new_scale_fill()
        } else {
            p <- p +
                geom_density_2d(
                    aes(x = !!sym(dims[1]), y = !!sym(dims[2])),
                    color = density_color,
                    inherit.aes = FALSE,
                    show.legend = FALSE
                )
        }
    }

    ## Setting up the theme
    x_min <- min(data[[dims[1]]], na.rm = TRUE)
    x_max <- max(data[[dims[1]]], na.rm = TRUE)
    y_min <- min(data[[dims[2]]], na.rm = TRUE)
    y_max <- max(data[[dims[2]]], na.rm = TRUE)
    p <- p +
        labs(title = title, subtitle = subtitle, x = xlab, y = ylab) +
        scale_x_continuous(limits = c(x_min, x_max)) +
        scale_y_continuous(limits = c(y_min, y_max)) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction
        )

    ## Adding the background points for each facet
    if (!is.null(facet_by) && isFALSE(multifeats)) {
        # Add the points from other subplots as background
        all_fc_values <- distinct(data, !!!syms(facet_by))
        colnames(all_fc_values) <- paste0(".facet_", colnames(all_fc_values))
        fc_data <- expand_grid(data, all_fc_values)
        # Remove the points inside the facet
        fc_indicator <- TRUE
        for (fc in facet_by) {
            fc_indicator <- fc_indicator &
                (fc_data[[fc]] == fc_data[[paste0(".facet_", fc)]])
        }
        fc_data <- fc_data[!fc_indicator, , drop = FALSE]
        fc_data[, facet_by, drop = FALSE] <- fc_data[,
            paste0(".facet_", facet_by),
            drop = FALSE
        ]

        if (isTRUE(raster)) {
            p <- p +
                scattermore::geom_scattermore(
                    data = fc_data,
                    mapping = aes(x = !!sym(dims[1]), y = !!sym(dims[2])),
                    pointsize = ceiling(pt_size_num),
                    alpha = pt_alpha / 2,
                    color = bg_color,
                    pixels = raster_dpi
                )
        } else if (isTRUE(hex)) {
            p <- p +
                geom_hex(
                    data = fc_data,
                    mapping = aes(x = !!sym(dims[1]), y = !!sym(dims[2])),
                    linewidth = hex_linewidth,
                    bins = hex_bins,
                    binwidth = hex_binwidth,
                    fill = bg_color,
                    alpha = pt_alpha / 2
                )
        } else {
            if (is.numeric(pt_size)) {
                p <- p +
                    geom_point(
                        data = fc_data,
                        mapping = aes(x = !!sym(dims[1]), y = !!sym(dims[2])),
                        size = pt_size,
                        alpha = pt_alpha / 2,
                        color = bg_color
                    )
            } else {
                p <- p +
                    geom_point(
                        data = fc_data,
                        mapping = aes(
                            x = !!sym(dims[1]),
                            y = !!sym(dims[2]),
                            size = !!sym(pt_size)
                        ),
                        alpha = pt_alpha / 2,
                        color = bg_color
                    )
            }
        }
        rm(fc_data)
    }

    ## Raserting the plot/Adding the points
    if (isTRUE(raster)) {
        # if (!requireNamespace("scattermore", quietly = TRUE)) {
        #     stop("'scattermore' package is required to raster the plot.")
        # }
        if (!is.null(group_by)) {
            p <- p +
                scattermore::geom_scattermore(
                    data = data[is.na(data[[group_by]]), , drop = FALSE],
                    mapping = aes(x = !!sym(dims[1]), y = !!sym(dims[2])),
                    color = bg_color,
                    pointsize = ceiling(pt_size_num),
                    alpha = pt_alpha,
                    pixels = raster_dpi,
                    show.legend = TRUE
                ) +
                scattermore::geom_scattermore(
                    data = data[!is.na(data[[group_by]]), , drop = FALSE],
                    mapping = aes(
                        x = !!sym(dims[1]),
                        y = !!sym(dims[2]),
                        color = !!sym(group_by)
                    ),
                    pointsize = ceiling(pt_size_num),
                    alpha = pt_alpha,
                    pixels = raster_dpi,
                    show.legend = TRUE
                )
        } else {
            # features
            p <- p +
                scattermore::geom_scattermore(
                    mapping = aes(x = !!sym(dims[1]), y = !!sym(dims[2])),
                    color = bg_color,
                    pointsize = ceiling(pt_size_num),
                    alpha = pt_alpha,
                    pixels = raster_dpi
                ) +
                scattermore::geom_scattermore(
                    mapping = aes(
                        x = !!sym(dims[1]),
                        y = !!sym(dims[2]),
                        color = !!sym(colorby)
                    ),
                    pointsize = ceiling(pt_size_num),
                    alpha = pt_alpha,
                    pixels = raster_dpi
                )
        }
    } else if (isTRUE(hex)) {
        # if (!requireNamespace("hexbin", quietly = TRUE)) {
        #     stop("'hexbin' package is required to add hexgons the plot.")
        # }
        has_fill <- TRUE
        if (isTRUE(hex_count)) {
            if (!is.null(features)) {
                stop(
                    "Don't know how to count for the hex when 'group_by' is not provided."
                )
            }
            p <- p +
                geom_hex(
                    mapping = aes(
                        x = !!sym(dims[1]),
                        y = !!sym(dims[2]),
                        color = !!sym(group_by),
                        fill = !!sym(group_by),
                        alpha = after_stat(!!sym("count"))
                    ),
                    linewidth = hex_linewidth,
                    bins = hex_bins,
                    binwidth = hex_binwidth
                )
        } else if (!is.null(group_by)) {
            p <- p +
                geom_hex(
                    mapping = aes(
                        x = !!sym(dims[1]),
                        y = !!sym(dims[2]),
                        color = !!sym(group_by),
                        fill = !!sym(group_by)
                    ),
                    linewidth = hex_linewidth,
                    bins = hex_bins,
                    binwidth = hex_binwidth,
                    show.legend = TRUE
                )
        } else {
            # features
            data_na <- data[is.na(data[[features]]), , drop = FALSE]
            if (nrow(data_na) > 0) {
                p <- p +
                    geom_hex(
                        data = data_na,
                        mapping = aes(x = !!sym(dims[1]), y = !!sym(dims[2])),
                        fill = bg_color,
                        linewidth = hex_linewidth,
                        bins = hex_bins,
                        binwidth = hex_binwidth,
                        alpha = pt_alpha / 2
                    )
            }
            p <- p +
                stat_summary_hex(
                    data = data[!is.na(data[[features]]), , drop = FALSE],
                    mapping = aes(
                        x = !!sym(dims[1]),
                        y = !!sym(dims[2]),
                        z = !!sym(colorby)
                    ),
                    linewidth = hex_linewidth,
                    bins = hex_bins,
                    binwidth = hex_binwidth,
                    alpha = pt_alpha
                )
        }
    } else {
        if (is.numeric(pt_size)) {
            p <- p +
                geom_point(
                    mapping = aes(
                        x = !!sym(dims[1]),
                        y = !!sym(dims[2]),
                        color = !!sym(colorby)
                    ),
                    size = pt_size,
                    alpha = pt_alpha,
                    show.legend = TRUE
                )
        } else {
            p <- p +
                geom_point(
                    mapping = aes(
                        x = !!sym(dims[1]),
                        y = !!sym(dims[2]),
                        color = !!sym(colorby),
                        size = !!sym(pt_size)
                    ),
                    alpha = pt_alpha,
                    show.legend = TRUE
                )
        }
    }

    if (!is.numeric(pt_size)) {
        p <- p +
            scale_size_area(max_size = 6, n.breaks = 4) +
            guides(
                size = guide_legend(
                    override.aes = list(fill = "grey30", shape = 21),
                    order = 1
                )
            )
    }

    ## Adding the highlight
    if (!is.null(highlight)) {
        if (isTRUE(hex)) {
            stop("Highlight is not supported for hex plot.")
        }
        if (isTRUE(highlight)) {
            hi_df <- data
        } else if (length(highlight) == 1 && is.character(highlight)) {
            hi_df <- eval(parse(text = paste0('filter(data, ', highlight, ')')))
        } else {
            all_inst <- if (is.numeric(highlight)) {
                1:nrow(data)
            } else {
                rownames(data)
            }
            if (!any(highlight %in% all_inst)) {
                stop("No highlight items found in the data (rownames).")
            }
            if (!all(highlight %in% all_inst)) {
                warning(
                    "Not all highlight items found in the data (rownames).",
                    immediate. = TRUE
                )
            }
            hi_df <- data[intersect(highlight, all_inst), , drop = FALSE]
            rm(all_inst)
        }
        if (nrow(hi_df) > 0) {
            if (isTRUE(raster)) {
                p <- p +
                    scattermore::geom_scattermore(
                        data = hi_df,
                        aes(x = !!sym(dims[1]), y = !!sym(dims[2])),
                        color = highlight_color,
                        pointsize = floor(highlight_size) + highlight_stroke,
                        alpha = highlight_alpha,
                        pixels = raster_dpi
                    ) +
                    scattermore::geom_scattermore(
                        data = hi_df,
                        aes(
                            x = !!sym(dims[1]),
                            y = !!sym(dims[2]),
                            color = !!sym(colorby)
                        ),
                        pointsize = floor(highlight_size),
                        alpha = highlight_alpha,
                        pixels = raster_dpi
                    )
            } else {
                p <- p +
                    geom_point(
                        data = hi_df,
                        aes(x = !!sym(dims[1]), y = !!sym(dims[2])),
                        color = highlight_color,
                        size = highlight_size + highlight_stroke,
                        alpha = highlight_alpha
                    ) +
                    geom_point(
                        data = hi_df,
                        aes(
                            x = !!sym(dims[1]),
                            y = !!sym(dims[2]),
                            color = !!sym(colorby)
                        ),
                        size = highlight_size,
                        alpha = highlight_alpha
                    )
            }
        }
    }

    if (!is.null(group_by)) {
        if (!isFALSE(keep_empty_group)) {
            p <- p +
                scale_color_manual(
                    values = colors[names(labels_tb)],
                    labels = label_use,
                    na.value = bg_color,
                    breaks = group_vals,
                    limits = group_vals,
                    drop = FALSE,
                    guide = guide_legend(
                        title.hjust = 0,
                        order = 1,
                        override.aes = list(size = 3, alpha = 1)
                    )
                )
            if (has_fill) {
                p <- p +
                    scale_fill_manual(
                        values = colors[names(labels_tb)],
                        labels = label_use,
                        na.value = bg_color,
                        breaks = group_vals,
                        limits = group_vals,
                        drop = FALSE,
                        guide = guide_legend(
                            title.hjust = 0,
                            order = 1
                        )
                    )
            }
        } else {
            p <- p +
                scale_color_manual(
                    values = colors[names(labels_tb)],
                    labels = label_use,
                    na.value = bg_color,
                    guide = guide_legend(
                        title.hjust = 0,
                        order = 1,
                        override.aes = list(size = 3, alpha = 1)
                    )
                )
            if (has_fill) {
                p <- p +
                    scale_fill_manual(
                        values = colors[names(labels_tb)],
                        labels = label_use,
                        na.value = bg_color,
                        guide = guide_legend(
                            title.hjust = 0,
                            order = 1
                        )
                    )
            }
        }
    } else {
        # features
        p <- p +
            scale_color_gradientn(
                name = color_name,
                colors = palette_this(
                    palette = palette,
                    palcolor = palcolor,
                    type = "continuous",
                    reverse = palreverse
                ),
                values = rescale(feat_colors_value),
                limits = range(feat_colors_value),
                na.value = bg_color,
                guide = guide_colorbar(
                    frame.colour = "black",
                    ticks.colour = "black",
                    title.hjust = 0
                )
            )
        if (has_fill) {
            p <- p +
                scale_fill_gradientn(
                    name = color_name,
                    colors = palette_this(
                        palette = palette,
                        palcolor = palcolor,
                        type = "continuous",
                        reverse = palreverse
                    ),
                    values = rescale(feat_colors_value),
                    limits = range(feat_colors_value),
                    na.value = bg_color,
                    guide = guide_colorbar(
                        frame.colour = "black",
                        ticks.colour = "black",
                        title.hjust = 0
                    )
                )
        }
    }
    # There may be warnings about fonts, but we don't care here
    legend_base <- suppressWarnings(get_plot_component(
        p +
            theme_this(
                legend.position = "bottom",
                legend.direction = legend.direction
            ),
        "guide-box-bottom"
    ))

    legend_list <- list()
    ## Adding the lineages
    if (!is.null(lineages)) {
        if (!is.null(facet_by)) {
            stop("'lineages' is not supported when 'facet_by' is not NULL.")
        }
        lineages <- unique(check_columns(
            data,
            lineages,
            force_factor = FALSE,
            allow_multi = TRUE
        ))
        lineage_colors <- palette_this(
            lineages,
            palette = lineages_palette,
            palcolor = lineages_palcolor,
            reverse = palreverse
        )
        lineage_layers <- lapply(lineages, function(l) {
            trim_pass <- (data[[l]] >
                quantile(data[[l]], lineages_trim[1], na.rm = TRUE) &
                data[[l]] < quantile(data[[l]], lineages_trim[2], na.rm = TRUE))
            na_pass <- !is.na(data[[l]])
            index <- which(trim_pass & na_pass)
            index <- index[order(data[index, l])]
            dat_sub <- data[index, , drop = FALSE]
            # if (is.null(weights)) {
            weights_used <- rep(1, nrow(dat_sub))
            # } else {
            # weights_used <- dat_sub[[weights]]
            # }
            fitted <- lapply(dims, function(x) {
                loess(
                    formula(paste(x, l, sep = "~")),
                    weights = weights_used,
                    data = dat_sub,
                    span = lineages_span,
                    degree = 2
                )$fitted
            })
            names(fitted) <- dims
            fitted[["index"]] <- index

            dat_smooth <- as.data.frame(fitted)
            colnames(dat_smooth) <- c(
                paste0("Axis_", 1:(ncol(dat_smooth) - 1)),
                "index"
            )
            dat_smooth[, "Lineages"] <- factor(l, levels = lineages)
            dat_smooth <- unique(na.omit(dat_smooth))
            curve <- list()
            if (isTRUE(lineages_whiskers)) {
                dat_smooth[, "raw_Axis_1"] <- data[
                    dat_smooth[, "index"],
                    dims[1]
                ]
                dat_smooth[, "raw_Axis_2"] <- data[
                    dat_smooth[, "index"],
                    dims[2]
                ]
                curve <- c(
                    curve,
                    geom_segment(
                        data = dat_smooth,
                        mapping = aes(
                            x = !!sym("Axis_1"),
                            y = !!sym("Axis_2"),
                            xend = !!sym("raw_Axis_1"),
                            yend = !!sym("raw_Axis_2"),
                            color = !!sym("Lineages")
                        ),
                        linewidth = lineages_whiskers_linewidth,
                        alpha = lineages_whiskers_alpha,
                        show.legend = TRUE,
                        inherit.aes = FALSE
                    )
                )
            }
            curve <- c(
                curve,
                geom_path(
                    data = dat_smooth,
                    mapping = aes(x = !!sym("Axis_1"), y = !!sym("Axis_2")),
                    color = lineages_line_bg,
                    linewidth = lineages_linewidth + lineages_line_bg_stroke,
                    arrow = lineages_arrow,
                    show.legend = TRUE,
                    inherit.aes = FALSE
                ),
                geom_path(
                    data = dat_smooth,
                    mapping = aes(
                        x = !!sym("Axis_1"),
                        y = !!sym("Axis_2"),
                        color = !!sym("Lineages")
                    ),
                    linewidth = lineages_linewidth,
                    arrow = lineages_arrow,
                    show.legend = TRUE,
                    inherit.aes = FALSE
                )
            )
            return(curve)
        })
        lineage_layers[[length(lineage_layers) + 1]] <- scale_color_manual(
            values = lineage_colors
        )

        suppressMessages({
            legend_list$lineages <- suppressWarnings(get_plot_component(
                ggplot() +
                    lineage_layers +
                    theme_this(
                        legend.position = "bottom",
                        legend.direction = legend.direction
                    ),
                "guide-box-bottom"
            ))
        })

        p <- suppressWarnings({
            p +
                new_scale_color() +
                lineage_layers +
                ggplot2::theme(legend.position = "none")
        })
        if (is.null(legend_list$lineages)) {
            legend_list["lineages"] <- list(NULL)
        }
    }

    if (!is.null(velocity)) {
        if (!is.null(facet_by)) {
            stop("'velocity' is not supported when 'facet_by' is not NULL.")
        }
        velocity_plot_type <- match.arg(velocity_plot_type)

        if (is.data.frame(velocity) || is.matrix(velocity)) {
            v_embedding <- velocity
        } else {
            v_embedding <- data[, velocity, drop = FALSE]
        }

        velocity_layers <- VelocityPlot(
            embedding = data[, dims, drop = FALSE],
            v_embedding = v_embedding,
            plot_type = velocity_plot_type,
            group_by = if (velocity_plot_type == "raw") {
                data[[group_by]]
            } else {
                NULL
            },
            group_name = group_by,
            group_palette = velocity_group_palette,
            group_palcolor = velocity_group_palcolor,
            n_neighbors = velocity_n_neighbors,
            density = velocity_density,
            keep_na = keep_na[[group_by]],
            keep_empty = keep_empty[[group_by]],
            smooth = velocity_smooth,
            scale = velocity_scale,
            min_mass = velocity_min_mass,
            cutoff_perc = velocity_cutoff_perc,
            arrow_angle = arrow_angle,
            arrow_color = arrow_color,
            streamline_l = streamline_l,
            streamline_minl = streamline_minl,
            arrow_alpha = arrow_alpha,
            streamline_res = streamline_res,
            streamline_n = streamline_n,
            streamline_width = streamline_width,
            streamline_alpha = streamline_alpha,
            streamline_color = streamline_color,
            streamline_palette = streamline_palette,
            streamline_palcolor = streamline_palcolor,
            streamline_bg_color = streamline_bg_color,
            streamline_bg_stroke = streamline_bg_stroke,
            return_layer = TRUE
        )
        velocity_scales <- attr(velocity_layers, "scales")
        if (!is.null(velocity_scales) && "color" %in% velocity_scales) {
            p <- p + new_scale_color() + velocity_layers
            legend_list$velocity <- suppressWarnings(get_plot_component(
                ggplot() +
                    velocity_layers +
                    theme_this(
                        legend.position = "bottom",
                        legend.direction = legend.direction
                    ),
                "guide-box-bottom"
            ))
        } else {
            p <- p + velocity_layers
            legend_list["velocity"] <- list(NULL)
        }
    }

    ## Adding the statistic plots
    if (!is.null(stat_by)) {
        if (!is.null(facet_by)) {
            stop("'stat_by' is not supported when 'facet_by' is not NULL.")
        }
        if (!is.null(features)) {
            stop("'stat_by' is not supported without 'group_by'.")
        }
        stat_by <- check_columns(data, stat_by, force_factor = TRUE)
        stat_plot_type <- match.arg(stat_plot_type)
        stat_args$data <- data
        stat_args$split_by <- group_by
        stat_args$legend.position <- "bottom"
        stat_args$legend.direction <- legend.direction
        stat_args$combine <- FALSE
        stat_args$theme <- theme
        stat_args$theme_args <- theme_args
        stat_args$title <- character(0)
        # keep all levels so they appear in the legend, even if some levels have no data
        # (in case some stat plots don't show all levels due to filtering)
        stat_args$keep_empty <- TRUE
        stat_args$palette <- stat_palette
        if (stat_plot_type == "pie") {
            stat_args$x <- stat_by
            stat_plots <- do_call(PieChart, stat_args)
        } else if (stat_plot_type == "ring") {
            stat_args$group_by <- stat_by
            stat_plots <- do_call(RingPlot, stat_args)
        } else if (stat_plot_type == "bar") {
            stat_args$x <- stat_by
            stat_plots <- do_call(BarPlot, stat_args)
        } else if (stat_plot_type == "line") {
            stat_args$x <- stat_by
            stat_plots <- do_call(LinePlot, stat_args)
        }

        coord_df <- aggregate(
            p$data[, dims],
            by = list(p$data[[group_by]]),
            FUN = median
        )
        colnames(coord_df)[1] <- group_by
        x_range <- diff(layer_scales(p)$x$range$range)
        y_range <- diff(layer_scales(p)$y$range$range)

        stat_plot_list <- list()
        for (i in seq_len(nrow(coord_df))) {
            stat_plot_list[[i]] <- annotation_custom(
                ggplotGrob(
                    stat_plots[[coord_df[i, group_by]]] +
                        theme_void() +
                        ggplot2::theme(legend.position = "none")
                ),
                xmin = coord_df[i, dims[1]] - x_range * stat_plot_size / 2,
                ymin = coord_df[i, dims[2]] - y_range * stat_plot_size / 2,
                xmax = coord_df[i, dims[1]] + x_range * stat_plot_size / 2,
                ymax = coord_df[i, dims[2]] + y_range * stat_plot_size / 2
            )
        }
        p <- p + stat_plot_list
        legend_list$stat_by <- suppressWarnings(get_plot_component(
            stat_plots[[coord_df[i, group_by]]] +
                ggplot2::theme(legend.position = "bottom"),
            "guide-box-bottom"
        ))
    }

    ## Adding the labels
    if (isTRUE(label)) {
        if (!is.null(features)) {
            stop("Adding labels is not supported when 'features' is specified.")
        }
        if (!is.null(facet_by)) {
            label_df <- aggregate(
                data[, dims],
                by = list(data[[group_by]], data[[facet_by]]),
                FUN = median
            )
            colnames(label_df)[2:length(facet_by) + 1] <- facet_by
        } else {
            label_df <- aggregate(
                data[, dims],
                by = list(data[[group_by]]),
                FUN = median
            )
        }
        colnames(label_df)[1] <- ".label"
        label_df <- label_df[!is.na(label_df[, ".label"]), , drop = FALSE]
        if (!isTRUE(label_insitu)) {
            label_df[, ".label"] <- seq_len(nrow(label_df))
        }
        if (isTRUE(label_repel)) {
            p <- p +
                geom_point(
                    data = label_df,
                    mapping = aes(x = !!sym(dims[1]), y = !!sym(dims[2])),
                    color = label_pt_color,
                    size = label_pt_size
                ) +
                geom_text_repel(
                    data = label_df,
                    aes(
                        x = !!sym(dims[1]),
                        y = !!sym(dims[2]),
                        label = !!sym(".label")
                    ),
                    point.size = label_pt_size,
                    max.overlaps = 100,
                    force = label_repulsion,
                    color = label_fg,
                    bg.color = label_bg,
                    bg.r = label_bg_r,
                    size = label_size,
                    inherit.aes = FALSE
                )
        } else {
            p <- p +
                geom_text_repel(
                    data = label_df,
                    aes(
                        x = !!sym(dims[1]),
                        y = !!sym(dims[2]),
                        label = !!sym(".label")
                    ),
                    fontface = "bold",
                    min.segment.length = 0,
                    segment.color = label_segment_color,
                    point.size = NA,
                    max.overlaps = 100,
                    force = 0,
                    color = label_fg,
                    bg.color = label_bg,
                    bg.r = label_bg_r,
                    size = label_size,
                    inherit.aes = FALSE
                )
        }
    }

    ## Posing all legends
    if (length(legend_list) > 0) {
        legend_list <- legend_list[!sapply(legend_list, is.null)]
        if (legend.direction == "vertical") {
            legend <- do_call(cbind, c(list(base = legend_base), legend_list))
        } else {
            legend <- do_call(rbind, c(list(base = legend_base), legend_list))
        }
        orig_data <- p$data
        gtable <- ggplotGrob(p + ggplot2::theme(legend.position = "none"))
        gtable <- add_grob(gtable, legend, legend.position)
        p <- wrap_plots(gtable)
        p$data <- orig_data
    }

    ## Putting reasonable height and width
    dims <- calculate_plot_dimensions(
        base_height = 5.5,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = if (!is.null(group_by)) length(group_vals) else 1,
        legend_nchar = if (!is.null(group_by)) max(nchar(label_use)) else 5
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    if (length(legend_list) == 0) {
        p <- facet_plot(
            p,
            facet_by,
            facet_scales,
            facet_nrow,
            facet_ncol,
            facet_byrow,
            labeller = facet_labeller,
            legend.position = legend.position,
            legend.direction = legend.direction,
            drop = !isTRUE(keep_empty_facet)
        )
    }
    p
}

#' DimPlot / FeatureDimPlot
#'
#' @description
#' `DimPlot` visualizes dimension reduction data (PCA, t-SNE, UMAP, etc.) as a 2D or 3D
#' scatter plot. `DimPlot()` colours points by a discrete grouping variable
#' (e.g., clusters), while `FeatureDimPlot()` colours points by a continuous
#' numeric feature (e.g., gene expression, lineage scores).
#'
#' Both functions share the same internal engine (`DimPlotAtomic`) and support
#' an extensive set of annotation layers: group boundary marks, network/graph
#' edges, 2D density contours, lineage/trajectory curves, RNA-velocity arrows
#' (raw, grid, or stream), statistical summary mini-plots at group centroids,
#' point highlighting, background context points from other facets, and
#' flexible label positioning.
#'
#' When `dims` has 3 elements, both functions automatically return an
#' interactive **plotly** 3D scatter plot (via `DimPlotAtomic3D`). Certain
#' 2D-only features are silently ignored in 3D mode (see `@param dims` for
#' the full list).
#'
#' Rendering scales with dataset size: standard `geom_point()` for small data,
#' automatic rasterisation via `scattermore::geom_scattermore()` when
#' `nrow(data) > 1e5`, or hex-bin aggregation (`geom_hex()` /
#' `stat_summary_hex()`).
#'
#' @section split_by Workflow (DimPlot):
#'
#' When `split_by` is specified, `DimPlot()` executes the following pipeline:
#'
#' 1. **Argument validation** — `validate_common_args()` checks the seed and
#'    blocks `split_by` + `velocity` combinations.
#' 2. **NA / empty normalisation** — `check_keep_na()` / `check_keep_empty()`
#'    convert `keep_na` / `keep_empty` to per-column lists.
#' 3. **Theme resolution** — `process_theme()` resolves the theme string to a
#'    theme function.
#' 4. **Split column resolution** — `check_columns()` validates `split_by`
#'    (force_factor, concat_multi).
#' 5. **Pre-filtering** — `process_keep_na_empty()` removes NA / empty levels
#'    from the split column, then `data` is split by `split_by` levels (order
#'    preserved). When `graph` references an attribute (`@graph`), the graph
#'    matrix is also subset per split.
#' 6. **Per-split parameter resolution** — `check_palette()`,
#'    `check_palcolor()`, `check_legend()` resolve palette, palcolor,
#'    legend.position, and legend.direction for each split.
#' 7. **Per-split dispatch** — each split is passed to `DimPlotAtomic()` with
#'    its resolved parameters. Title defaults to the split level name unless
#'    `title` is a function.
#' 8. **Combination** — `combine_plots()` assembles the list of plots via
#'    `patchwork::wrap_plots()`, applying `nrow`, `ncol`, `byrow`, `axes`,
#'    `axis_titles`, `guides`, and `design`.
#'
#' @section split_by Workflow (FeatureDimPlot):
#'
#' `FeatureDimPlot()` supports two forms of splitting:
#'
#' **A. `split_by = TRUE` (split by features)**
#'
#' 1. Each feature in `features` is dispatched individually to
#'    `DimPlotAtomic()`, producing one plot per feature. The plot title
#'    defaults to the feature name.
#' 2. Plots are combined via `combine_plots()` with `split_by = ".features"`.
#'
#' **B. `split_by` as a column name (split by data column)**
#'
#' 1. Data is split by the named column's levels (same pipeline as DimPlot
#'    steps 1–8 above). Graph attribute splitting is supported.
#'
#' @rdname dimplot
#' @inheritParams common_args
#' @inheritParams DimPlotAtomic
#' @return A `ggplot` object (single plot), a `patchwork` / `wrap_plots` object
#'   (when `split_by` is provided and `combine = TRUE`), or a list of `ggplot`
#'   objects (when `split_by` is provided and `combine = FALSE`).
#'   When `dims` has 3 elements, a **plotly** object is returned instead.
#' @export
#' @seealso \code{\link{VelocityPlot}}
#' @examples
#' \donttest{
#' data(dim_example)
#'
#' # basic dim plot
#' DimPlot(dim_example, group_by = "clusters")
#' DimPlot(dim_example, group_by = "clusters", theme = "theme_blank")
#' DimPlot(dim_example, group_by = "clusters", theme = ggplot2::theme_classic,
#'     theme_args = list(base_size = 16), palette = "seurat")
#'
#' # raster and highlighting
#' DimPlot(dim_example, group_by = "clusters", raster = TRUE, raster_dpi = 50)
#' DimPlot(dim_example, group_by = "clusters", highlight = 1:20,
#'     highlight_color = "black", highlight_stroke = 2)
#' DimPlot(dim_example, group_by = "clusters", highlight = TRUE, facet_by = "group",
#'     theme = "theme_blank")
#'
#' # labels
#' DimPlot(dim_example, group_by = "clusters", label = TRUE,
#'     label_size = 5, label_bg_r = 0.2)
#' DimPlot(dim_example, group_by = "clusters", label = TRUE, label_fg = "red",
#'     label_bg = "yellow", label_size = 5)
#' DimPlot(dim_example, group_by = "clusters", label = TRUE, label_insitu = TRUE)
#'
#' # group marks
#' DimPlot(dim_example, group_by = "clusters", add_mark = TRUE)
#' DimPlot(dim_example, group_by = "clusters", add_mark = TRUE, mark_linetype = 2)
#' DimPlot(dim_example, group_by = "clusters", add_mark = TRUE, mark_type = "ellipse")
#'
#' # density overlays
#' DimPlot(dim_example, group_by = "clusters", add_density = TRUE)
#' DimPlot(dim_example, group_by = "clusters", add_density = TRUE, density_filled = TRUE)
#' DimPlot(dim_example, group_by = "clusters", add_density = TRUE, density_filled = TRUE,
#'     density_filled_palette = "Blues", highlight = TRUE)
#'
#' # statistics at group centroids
#' DimPlot(dim_example, group_by = "clusters", stat_by = "group")
#' DimPlot(dim_example, group_by = "clusters", stat_by = "group",
#'     stat_plot_type = "bar", stat_plot_size = 0.06)
#'
#' # hex bins
#' DimPlot(dim_example, group_by = "clusters", hex = TRUE)
#' DimPlot(dim_example, group_by = "clusters", hex = TRUE, hex_bins = 20)
#' DimPlot(dim_example, group_by = "clusters", hex = TRUE, hex_count = FALSE)
#'
#' # graph / network edges
#' DimPlot(dim_example, group_by = "clusters", graph = "@graph", edge_color = "grey80")
#'
#' # lineages / trajectories
#' DimPlot(dim_example, group_by = "clusters", lineages = c("stochasticbasis_1", "stochasticbasis_2"))
#' DimPlot(dim_example, group_by = "clusters", lineages = c("stochasticbasis_1", "stochasticbasis_2"),
#'     lineages_whiskers = TRUE, lineages_whiskers_linewidth = 0.1)
#' DimPlot(dim_example, group_by = "clusters", lineages = c("stochasticbasis_1", "stochasticbasis_2"),
#'     lineages_span = 0.4)
#'
#' # split_by
#' DimPlot(dim_example, group_by = "clusters",  split_by = "group",
#'     palette = list(A = "Paired", B = "Set1"))
#'
#' # velocity
#' DimPlot(dim_example, group_by = "clusters", velocity = c("stochasticbasis_1", "stochasticbasis_2"),
#'     pt_alpha = 0)
#' DimPlot(dim_example, group_by = "clusters", velocity = 3:4,
#'     velocity_plot_type = "grid", arrow_alpha = 0.6)
#' DimPlot(dim_example, group_by = "clusters", velocity = 3:4,
#'     velocity_plot_type = "stream")
#'
#' # 3D plots (returns a plotly object)
#' DimPlot(dim_example, dims = 1:3, group_by = "clusters")
#' DimPlot(dim_example, dims = 1:3, group_by = "clusters", label = TRUE,
#'     label_insitu = TRUE)
#' DimPlot(dim_example, dims = c("basis_1", "basis_2", "stochasticbasis_1"),
#'     group_by = "clusters", graph = "@graph", edge_color = "grey80")
#'
#' # keep_na and keep_empty
#' dim_example$clusters[dim_example$clusters == "Ductal"] <- NA
#'
#' DimPlot(dim_example, group_by = "clusters", keep_na = FALSE, keep_empty = TRUE)
#' DimPlot(dim_example, group_by = "clusters", keep_na = TRUE, keep_empty = TRUE)
#' DimPlot(dim_example, group_by = "clusters", keep_na = TRUE, keep_empty = FALSE)
#' }
DimPlot <- function(
    data,
    dims = 1:2,
    group_by,
    group_by_sep = "_",
    split_by = NULL,
    split_by_sep = "_",
    pt_size = NULL,
    pt_alpha = 1,
    bg_color = "grey80",
    label_insitu = FALSE,
    show_stat = !identical(theme, "theme_blank"),
    label = FALSE,
    label_size = 4,
    label_fg = "white",
    label_bg = "black",
    label_bg_r = 0.1,
    label_repel = FALSE,
    label_repulsion = 20,
    label_pt_size = 1,
    label_pt_color = "black",
    label_segment_color = "black",
    order = c("as-is", "reverse", "high-top", "low-top", "random"),
    highlight = NULL,
    highlight_alpha = 1,
    highlight_size = 1,
    highlight_color = "black",
    highlight_stroke = 0.8,
    add_mark = FALSE,
    mark_type = c("hull", "ellipse", "rect", "circle"),
    mark_expand = unit(3, "mm"),
    mark_alpha = 0.1,
    mark_linetype = 1,
    stat_by = NULL,
    stat_plot_type = c("pie", "ring", "bar", "line"),
    stat_plot_size = 0.1,
    stat_args = list(palette = "Set1"),
    graph = NULL,
    edge_size = c(0.05, 0.5),
    edge_alpha = 0.1,
    edge_color = "grey40",
    add_density = FALSE,
    density_color = "grey80",
    density_filled = FALSE,
    density_filled_palette = "Greys",
    density_filled_palcolor = NULL,
    lineages = NULL,
    lineages_trim = c(0.01, 0.99),
    lineages_span = 0.75,
    lineages_palette = "Dark2",
    lineages_palcolor = NULL,
    lineages_arrow = arrow(length = unit(0.1, "inches")),
    lineages_linewidth = 1,
    lineages_line_bg = "white",
    lineages_line_bg_stroke = 0.5,
    lineages_whiskers = FALSE,
    lineages_whiskers_linewidth = 0.5,
    lineages_whiskers_alpha = 0.5,
    velocity = NULL,
    velocity_plot_type = c("raw", "grid", "stream"),
    velocity_n_neighbors = NULL,
    velocity_density = 1,
    velocity_smooth = 0.5,
    velocity_scale = 1,
    velocity_min_mass = 1,
    velocity_cutoff_perc = 5,
    velocity_group_palette = "Set2",
    velocity_group_palcolor = NULL,
    arrow_angle = 20,
    arrow_color = "black",
    arrow_alpha = 1,
    streamline_l = 5,
    streamline_minl = 1,
    streamline_res = 1,
    streamline_n = 15,
    streamline_width = c(0, 0.8),
    streamline_alpha = 1,
    streamline_color = NULL,
    streamline_palette = "RdYlBu",
    streamline_palcolor = NULL,
    streamline_bg_color = "white",
    streamline_bg_stroke = 0.5,
    keep_na = FALSE,
    keep_empty = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_nrow = NULL,
    facet_ncol = NULL,
    facet_byrow = TRUE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    theme = "theme_this",
    theme_args = list(),
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    raster = NULL,
    raster_dpi = c(512, 512),
    hex = FALSE,
    hex_linewidth = 0.5,
    hex_count = TRUE,
    hex_bins = 50,
    hex_binwidth = NULL,
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    keep_na <- check_keep_na(keep_na, c(facet_by, group_by, split_by, stat_by))
    keep_empty <- check_keep_empty(
        keep_empty,
        c(facet_by, group_by, split_by, stat_by)
    )
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    stopifnot(
        "[DimPlot] 'split_by' is not supported for velocity plot." = is.null(
            velocity
        ) ||
            is.null(split_by)
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
        if (
            is.character(graph) && length(graph) == 1 && startsWith(graph, "@")
        ) {
            # split the graph as well
            datas <- lapply(datas, function(d) {
                gh <- attr(data, substring(graph, 2))[rownames(d), rownames(d)]
                attr(d, substring(graph, 2)) <- gh
                d
            })
        }
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            DimPlotAtomic(
                datas[[nm]],
                dims = dims,
                group_by = group_by,
                group_by_sep = group_by_sep,
                pt_size = pt_size,
                pt_alpha = pt_alpha,
                bg_color = bg_color,
                label_insitu = label_insitu,
                show_stat = show_stat,
                label = label,
                label_size = label_size,
                label_fg = label_fg,
                label_bg = label_bg,
                label_bg_r = label_bg_r,
                label_repel = label_repel,
                label_repulsion = label_repulsion,
                label_pt_size = label_pt_size,
                label_pt_color = label_pt_color,
                label_segment_color = label_segment_color,
                order = order,
                highlight = highlight,
                highlight_alpha = highlight_alpha,
                highlight_size = highlight_size,
                highlight_color = highlight_color,
                highlight_stroke = highlight_stroke,
                add_mark = add_mark,
                mark_type = mark_type,
                mark_expand = mark_expand,
                mark_alpha = mark_alpha,
                mark_linetype = mark_linetype,
                stat_by = stat_by,
                stat_plot_type = stat_plot_type,
                stat_plot_size = stat_plot_size,
                stat_args = stat_args,
                graph = graph,
                edge_size = edge_size,
                edge_alpha = edge_alpha,
                edge_color = edge_color,
                add_density = add_density,
                density_color = density_color,
                density_filled = density_filled,
                density_filled_palette = density_filled_palette,
                density_filled_palcolor = density_filled_palcolor,
                lineages = lineages,
                lineages_trim = lineages_trim,
                lineages_span = lineages_span,
                lineages_palette = lineages_palette,
                lineages_palcolor = lineages_palcolor,
                lineages_arrow = lineages_arrow,
                lineages_linewidth = lineages_linewidth,
                lineages_line_bg = lineages_line_bg,
                lineages_line_bg_stroke = lineages_line_bg_stroke,
                lineages_whiskers = lineages_whiskers,
                lineages_whiskers_linewidth = lineages_whiskers_linewidth,
                lineages_whiskers_alpha = lineages_whiskers_alpha,
                velocity = velocity,
                velocity_plot_type = velocity_plot_type,
                velocity_n_neighbors = velocity_n_neighbors,
                velocity_density = velocity_density,
                velocity_smooth = velocity_smooth,
                velocity_scale = velocity_scale,
                velocity_min_mass = velocity_min_mass,
                velocity_cutoff_perc = velocity_cutoff_perc,
                arrow_alpha = arrow_alpha,
                velocity_group_palette = velocity_group_palette,
                velocity_group_palcolor = velocity_group_palcolor,
                arrow_angle = arrow_angle,
                arrow_color = arrow_color,
                streamline_l = streamline_l,
                streamline_minl = streamline_minl,
                streamline_res = streamline_res,
                streamline_n = streamline_n,
                streamline_width = streamline_width,
                streamline_alpha = streamline_alpha,
                streamline_color = streamline_color,
                streamline_palette = streamline_palette,
                streamline_palcolor = streamline_palcolor,
                streamline_bg_color = streamline_bg_color,
                streamline_bg_stroke = streamline_bg_stroke,
                keep_na = keep_na,
                keep_empty = keep_empty,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_nrow = facet_nrow,
                facet_ncol = facet_ncol,
                facet_byrow = facet_byrow,
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                theme = theme,
                theme_args = theme_args,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                raster = raster,
                raster_dpi = raster_dpi,
                hex = hex,
                hex_linewidth = hex_linewidth,
                hex_count = hex_count,
                hex_bins = hex_bins,
                hex_binwidth = hex_binwidth,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                seed = seed,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

#' @description
#' `FeatureDimPlot` to visualize feature expression on dimension reduction plots.
#' Colours points by a
#' continuous numeric variable (e.g., gene expression, module score, lineage
#' pseudotime) using a gradient colour scale, with optional quantile trimming
#' and background cutoff.
#'
#' When multiple `features` are provided and `facet_by` is not set, the data
#' is automatically pivoted to long format and faceted by feature name.
#' `split_by = TRUE` dispatches each feature to a separate plot for
#' independent layout control. `split_by` as a column name splits by that
#' column's levels, producing one plot per level with per-split palette
#' support.
#'
#' For detailed split_by workflows, see the main `DimPlot / FeatureDimPlot`
#' documentation (`@section split_by Workflow (FeatureDimPlot)`).
#'
#' @export
#' @rdname dimplot
#' @inheritParams common_args
#' @inheritParams DimPlotAtomic
#' @param split_by A character vector of column names to split the data by and
#'   plot separately. If `TRUE`, the data is split by `features` — each feature
#'   is plotted in its own panel. Use this instead of `facet_by` when you need
#'   independent layout control (`nrow`, `ncol`) or per-feature palettes via
#'   `split_by = TRUE` combined with `palette` / `palcolor`.
#' @return A `ggplot` object (single plot), a `patchwork` / `wrap_plots` object
#'   (when `split_by` is provided and `combine = TRUE`), or a list of `ggplot`
#'   objects (when `split_by` is provided and `combine = FALSE`).
#'   When `dims` has 3 elements, a **plotly** object is returned instead.
#' @export
#' @examples
#' \donttest{
#' data(dim_example)
#'
#' # single feature
#' FeatureDimPlot(dim_example, features = "stochasticbasis_1", pt_size = 2)
#' FeatureDimPlot(dim_example, features = "stochasticbasis_1", pt_size = 2, bg_cutoff = 0)
#' FeatureDimPlot(dim_example, features = "stochasticbasis_1", raster = TRUE, raster_dpi = 30)
#'
#' # multiple features (auto-pivoted to long, faceted by feature)
#' FeatureDimPlot(dim_example, features = c("stochasticbasis_1", "stochasticbasis_2"),
#'  pt_size = 2)
#'
#' # single feature with facet_by (facet_by works when only 1 feature)
#' FeatureDimPlot(dim_example, features = c("stochasticbasis_1"), pt_size = 2,
#'  facet_by = "group")
#'
#' # multiple features with split_by for independent layout
#' FeatureDimPlot(dim_example, features = c("stochasticbasis_1", "stochasticbasis_2"),
#'  split_by = "group", nrow = 2)
#'
#' # highlight and hex
#' FeatureDimPlot(dim_example, features = c("stochasticbasis_1", "stochasticbasis_2"),
#'  highlight = TRUE)
#' FeatureDimPlot(dim_example, features = c("stochasticbasis_1", "stochasticbasis_2"),
#'  hex = TRUE, hex_bins = 15)
#' FeatureDimPlot(dim_example, features = c("stochasticbasis_1", "stochasticbasis_2"),
#'  hex = TRUE, hex_bins = 15, split_by = "group", palette = list(A = "Reds", B = "Blues"))
#'
#' # 3D plots (returns a plotly object)
#' FeatureDimPlot(dim_example, dims = 1:3, features = "stochasticbasis_2", pt_size = 2)
#' FeatureDimPlot(dim_example, dims = c("basis_1", "basis_2", "stochasticbasis_1"),
#'  features = "stochasticbasis_2")
#' }
FeatureDimPlot <- function(
    data,
    dims = 1:2,
    features,
    split_by = NULL,
    split_by_sep = "_",
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    pt_size = NULL,
    pt_alpha = 1,
    bg_color = "grey80",
    bg_cutoff = NULL,
    label_insitu = FALSE,
    show_stat = !identical(theme, "theme_blank"),
    color_name = "",
    label = FALSE,
    label_size = 4,
    label_fg = "white",
    label_bg = "black",
    label_bg_r = 0.1,
    label_repel = FALSE,
    label_repulsion = 20,
    label_pt_size = 1,
    label_pt_color = "black",
    label_segment_color = "black",
    order = c("as-is", "reverse", "high-top", "low-top", "random"),
    highlight = NULL,
    highlight_alpha = 1,
    highlight_size = 1,
    highlight_color = "black",
    highlight_stroke = 0.8,
    add_mark = FALSE,
    mark_type = c("hull", "ellipse", "rect", "circle"),
    mark_expand = unit(3, "mm"),
    mark_alpha = 0.1,
    mark_linetype = 1,
    keep_na = FALSE,
    keep_empty = FALSE,
    stat_by = NULL,
    stat_plot_type = c("pie", "ring", "bar", "line"),
    stat_plot_size = 0.1,
    stat_args = list(palette = "Set1"),
    graph = NULL,
    edge_size = c(0.05, 0.5),
    edge_alpha = 0.1,
    edge_color = "grey40",
    add_density = FALSE,
    density_color = "grey80",
    density_filled = FALSE,
    density_filled_palette = "Greys",
    density_filled_palcolor = NULL,
    lineages = NULL,
    lineages_trim = c(0.01, 0.99),
    lineages_span = 0.75,
    lineages_palette = "Dark2",
    lineages_palcolor = NULL,
    lineages_arrow = arrow(length = unit(0.1, "inches")),
    lineages_linewidth = 1,
    lineages_line_bg = "white",
    lineages_line_bg_stroke = 0.5,
    lineages_whiskers = FALSE,
    lineages_whiskers_linewidth = 0.5,
    lineages_whiskers_alpha = 0.5,
    velocity = NULL,
    velocity_plot_type = c("raw", "grid", "stream"),
    velocity_n_neighbors = NULL,
    velocity_density = 1,
    velocity_smooth = 0.5,
    velocity_scale = 1,
    velocity_min_mass = 1,
    velocity_cutoff_perc = 5,
    velocity_group_palette = "Set2",
    velocity_group_palcolor = NULL,
    arrow_angle = 20,
    arrow_color = "black",
    arrow_alpha = 1,
    streamline_l = 5,
    streamline_minl = 1,
    streamline_res = 1,
    streamline_n = 15,
    streamline_width = c(0, 0.8),
    streamline_alpha = 1,
    streamline_color = NULL,
    streamline_palette = "RdYlBu",
    streamline_palcolor = NULL,
    streamline_bg_color = "white",
    streamline_bg_stroke = 0.5,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_nrow = NULL,
    facet_ncol = NULL,
    facet_byrow = TRUE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    theme = "theme_this",
    theme_args = list(),
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    raster = NULL,
    raster_dpi = c(512, 512),
    hex = FALSE,
    hex_linewidth = 0.5,
    hex_count = FALSE,
    hex_bins = 50,
    hex_binwidth = NULL,
    palette = "Spectral",
    palcolor = NULL,
    palreverse = FALSE,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    theme <- process_theme(theme)

    if (isTRUE(split_by)) {
        keep_na <- check_keep_na(keep_na, c(facet_by, stat_by))
        keep_empty <- check_keep_empty(keep_empty, c(facet_by, stat_by))
        plots <- lapply(
            features,
            function(feature) {
                DimPlotAtomic(
                    data,
                    dims = dims,
                    lower_quantile = lower_quantile,
                    upper_quantile = upper_quantile,
                    lower_cutoff = lower_cutoff,
                    upper_cutoff = upper_cutoff,
                    pt_size = pt_size,
                    pt_alpha = pt_alpha,
                    bg_color = bg_color,
                    color_name = color_name,
                    label_insitu = label_insitu,
                    show_stat = show_stat,
                    features = feature,
                    bg_cutoff = bg_cutoff,
                    label = label,
                    label_size = label_size,
                    label_fg = label_fg,
                    label_bg = label_bg,
                    label_bg_r = label_bg_r,
                    label_repel = label_repel,
                    label_repulsion = label_repulsion,
                    label_pt_size = label_pt_size,
                    label_pt_color = label_pt_color,
                    label_segment_color = label_segment_color,
                    order = order,
                    keep_na = keep_na,
                    keep_empty = keep_empty,
                    highlight = highlight,
                    highlight_alpha = highlight_alpha,
                    highlight_size = highlight_size,
                    highlight_color = highlight_color,
                    highlight_stroke = highlight_stroke,
                    add_mark = add_mark,
                    mark_type = mark_type,
                    mark_expand = mark_expand,
                    mark_alpha = mark_alpha,
                    mark_linetype = mark_linetype,
                    stat_by = stat_by,
                    stat_plot_type = stat_plot_type,
                    stat_plot_size = stat_plot_size,
                    stat_args = stat_args,
                    graph = graph,
                    edge_size = edge_size,
                    edge_alpha = edge_alpha,
                    edge_color = edge_color,
                    add_density = add_density,
                    density_color = density_color,
                    density_filled = density_filled,
                    density_filled_palette = density_filled_palette,
                    density_filled_palcolor = density_filled_palcolor,
                    lineages = lineages,
                    lineages_trim = lineages_trim,
                    lineages_span = lineages_span,
                    lineages_palette = lineages_palette,
                    lineages_palcolor = lineages_palcolor,
                    lineages_arrow = lineages_arrow,
                    lineages_linewidth = lineages_linewidth,
                    lineages_line_bg = lineages_line_bg,
                    lineages_line_bg_stroke = lineages_line_bg_stroke,
                    lineages_whiskers = lineages_whiskers,
                    lineages_whiskers_linewidth = lineages_whiskers_linewidth,
                    lineages_whiskers_alpha = lineages_whiskers_alpha,
                    velocity = velocity,
                    velocity_plot_type = velocity_plot_type,
                    velocity_n_neighbors = velocity_n_neighbors,
                    velocity_density = velocity_density,
                    velocity_smooth = velocity_smooth,
                    velocity_scale = velocity_scale,
                    velocity_min_mass = velocity_min_mass,
                    velocity_cutoff_perc = velocity_cutoff_perc,
                    arrow_alpha = arrow_alpha,
                    velocity_group_palette = velocity_group_palette,
                    velocity_group_palcolor = velocity_group_palcolor,
                    arrow_angle = arrow_angle,
                    arrow_color = arrow_color,
                    streamline_l = streamline_l,
                    streamline_minl = streamline_minl,
                    streamline_res = streamline_res,
                    streamline_n = streamline_n,
                    streamline_width = streamline_width,
                    streamline_alpha = streamline_alpha,
                    streamline_color = streamline_color,
                    streamline_palette = streamline_palette,
                    streamline_palcolor = streamline_palcolor,
                    streamline_bg_color = streamline_bg_color,
                    streamline_bg_stroke = streamline_bg_stroke,
                    facet_by = facet_by,
                    facet_scales = facet_scales,
                    facet_nrow = facet_nrow,
                    facet_ncol = facet_ncol,
                    facet_byrow = facet_byrow,
                    title = title %||% feature,
                    subtitle = subtitle,
                    xlab = xlab,
                    ylab = ylab,
                    theme = theme,
                    theme_args = theme_args,
                    aspect.ratio = aspect.ratio,
                    legend.position = legend.position,
                    legend.direction = legend.direction,
                    raster = raster,
                    raster_dpi = raster_dpi,
                    hex = hex,
                    hex_linewidth = hex_linewidth,
                    hex_count = hex_count,
                    hex_bins = hex_bins,
                    hex_binwidth = hex_binwidth,
                    palette = palette,
                    palcolor = palcolor,
                    palreverse = palreverse,
                    seed = seed,
                    ...
                )
            }
        )
        names(plots) <- features
        split_by <- ".features"
    } else {
        keep_na <- check_keep_na(keep_na, c(facet_by, stat_by, split_by))
        keep_empty <- check_keep_empty(
            keep_empty,
            c(facet_by, stat_by, split_by)
        )

        split_by_orig <- split_by
        split_by <- check_columns(
            data,
            split_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = split_by_sep
        )

        if (!is.null(split_by)) {
            data <- process_keep_na_empty(
                data,
                keep_na,
                keep_empty,
                col = split_by
            )
            keep_na[split_by_orig] <- NULL
            keep_empty[split_by_orig] <- NULL
            datas <- split(data, data[[split_by]])
            # keep the order of levels
            datas <- datas[levels(data[[split_by]])]
            if (
                is.character(graph) &&
                    length(graph) == 1 &&
                    startsWith(graph, "@")
            ) {
                # split the graph as well
                datas <- lapply(datas, function(d) {
                    gh <- attr(data, substring(graph, 2))[
                        rownames(d),
                        rownames(d)
                    ]
                    attr(d, substring(graph, 2)) <- gh
                    d
                })
            }
        } else {
            datas <- list(data)
            split_by <- names(datas) <- "..."
        }
        palette <- check_palette(palette, names(datas))
        palcolor <- check_palcolor(palcolor, names(datas))
        legend.direction <- check_legend(
            legend.direction,
            names(datas),
            "legend.direction"
        )
        legend.position <- check_legend(
            legend.position,
            names(datas),
            "legend.position"
        )

        plots <- lapply(
            names(datas),
            function(nm) {
                default_title <- if (
                    length(datas) == 1 && identical(nm, "...")
                ) {
                    NULL
                } else {
                    nm
                }
                if (is.function(title)) {
                    title <- title(default_title)
                } else {
                    title <- title %||% default_title
                }
                DimPlotAtomic(
                    datas[[nm]],
                    dims = dims,
                    features = features,
                    lower_quantile = lower_quantile,
                    upper_quantile = upper_quantile,
                    lower_cutoff = lower_cutoff,
                    upper_cutoff = upper_cutoff,
                    pt_size = pt_size,
                    pt_alpha = pt_alpha,
                    bg_color = bg_color,
                    color_name = color_name,
                    label_insitu = label_insitu,
                    show_stat = show_stat,
                    bg_cutoff = bg_cutoff,
                    label = label,
                    label_size = label_size,
                    label_fg = label_fg,
                    label_bg = label_bg,
                    label_bg_r = label_bg_r,
                    label_repel = label_repel,
                    label_repulsion = label_repulsion,
                    label_pt_size = label_pt_size,
                    label_pt_color = label_pt_color,
                    label_segment_color = label_segment_color,
                    order = order,
                    keep_na = keep_na,
                    keep_empty = keep_empty,
                    highlight = highlight,
                    highlight_alpha = highlight_alpha,
                    highlight_size = highlight_size,
                    highlight_color = highlight_color,
                    highlight_stroke = highlight_stroke,
                    add_mark = add_mark,
                    mark_type = mark_type,
                    mark_expand = mark_expand,
                    mark_alpha = mark_alpha,
                    mark_linetype = mark_linetype,
                    stat_by = stat_by,
                    stat_plot_type = stat_plot_type,
                    stat_plot_size = stat_plot_size,
                    stat_args = stat_args,
                    graph = graph,
                    edge_size = edge_size,
                    edge_alpha = edge_alpha,
                    edge_color = edge_color,
                    add_density = add_density,
                    density_color = density_color,
                    density_filled = density_filled,
                    density_filled_palette = density_filled_palette,
                    density_filled_palcolor = density_filled_palcolor,
                    lineages = lineages,
                    lineages_trim = lineages_trim,
                    lineages_span = lineages_span,
                    lineages_palette = lineages_palette,
                    lineages_palcolor = lineages_palcolor,
                    lineages_arrow = lineages_arrow,
                    lineages_linewidth = lineages_linewidth,
                    lineages_line_bg = lineages_line_bg,
                    lineages_line_bg_stroke = lineages_line_bg_stroke,
                    lineages_whiskers = lineages_whiskers,
                    lineages_whiskers_linewidth = lineages_whiskers_linewidth,
                    lineages_whiskers_alpha = lineages_whiskers_alpha,
                    velocity = velocity,
                    velocity_plot_type = velocity_plot_type,
                    velocity_n_neighbors = velocity_n_neighbors,
                    velocity_density = velocity_density,
                    velocity_smooth = velocity_smooth,
                    velocity_scale = velocity_scale,
                    velocity_min_mass = velocity_min_mass,
                    velocity_cutoff_perc = velocity_cutoff_perc,
                    arrow_alpha = arrow_alpha,
                    velocity_group_palette = velocity_group_palette,
                    velocity_group_palcolor = velocity_group_palcolor,
                    arrow_angle = arrow_angle,
                    arrow_color = arrow_color,
                    streamline_l = streamline_l,
                    streamline_minl = streamline_minl,
                    streamline_res = streamline_res,
                    streamline_n = streamline_n,
                    streamline_width = streamline_width,
                    streamline_alpha = streamline_alpha,
                    streamline_color = streamline_color,
                    streamline_palette = streamline_palette,
                    streamline_palcolor = streamline_palcolor,
                    streamline_bg_color = streamline_bg_color,
                    streamline_bg_stroke = streamline_bg_stroke,
                    facet_by = facet_by,
                    facet_scales = facet_scales,
                    facet_nrow = facet_nrow,
                    facet_ncol = facet_ncol,
                    facet_byrow = facet_byrow,
                    title = title,
                    subtitle = subtitle,
                    xlab = xlab,
                    ylab = ylab,
                    theme = theme,
                    theme_args = theme_args,
                    aspect.ratio = aspect.ratio,
                    legend.position = legend.position[[nm]],
                    legend.direction = legend.direction[[nm]],
                    raster = raster,
                    raster_dpi = raster_dpi,
                    hex = hex,
                    hex_linewidth = hex_linewidth,
                    hex_count = hex_count,
                    hex_bins = hex_bins,
                    hex_binwidth = hex_binwidth,
                    palette = palette[[nm]],
                    palcolor = palcolor[[nm]],
                    palreverse = palreverse,
                    seed = seed,
                    ...
                )
            }
        )
        names(plots) <- names(datas)
    }

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

data("dim_example", package = "plotthis")

test_that("DimPlot basic usage works", {
    p <- DimPlot(dim_example, group_by = "clusters")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("DimPlot with title and subtitle works", {
    p <- DimPlot(dim_example, group_by = "clusters", title = "DimPlot Title", subtitle = "Subtitle")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "DimPlot Title")
    expect_equal(p$labels$subtitle, "Subtitle")
})

test_that("DimPlot with label = TRUE works", {
    p <- DimPlot(dim_example, group_by = "clusters", label = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("DimPlot with split_by returns patchwork when combine = TRUE", {
    p <- DimPlot(dim_example, group_by = "clusters", split_by = "group", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("DimPlot with split_by and combine = FALSE returns a list", {
    plots <- DimPlot(dim_example, group_by = "clusters", split_by = "group", combine = FALSE)
    expect_true(is.list(plots))
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("DimPlot with multiple split_by columns returns a list with one plot per combination", {
    dim2 <- dim_example
    dim2$split2 <- factor(rep(c("s1", "s2"), length.out = nrow(dim2)))
    plots <- suppressMessages(DimPlot(dim2, group_by = "clusters",
                                      split_by = c("group", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("DimPlot with multiple split_by columns returns combined plot", {
    dim2 <- dim_example
    dim2$split2 <- factor(rep(c("s1", "s2"), length.out = nrow(dim2)))
    p <- suppressMessages(DimPlot(dim2, group_by = "clusters",
                                  split_by = c("group", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("DimPlot with specific dims works", {
    p <- DimPlot(dim_example, dims = c(1, 2), group_by = "clusters")
    expect_s3_class(p, "ggplot")
})

test_that("DimPlot with facet_by works", {
    p <- DimPlot(dim_example, group_by = "clusters", facet_by = "group")
    expect_s3_class(p, "ggplot")
})

test_that("DimPlot with add_density = TRUE works", {
    p <- DimPlot(dim_example, group_by = "clusters", add_density = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("DimPlot with add_mark = TRUE works", {
    p <- DimPlot(dim_example, group_by = "clusters", add_mark = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("FeatureDimPlot basic usage works", {
    p <- FeatureDimPlot(dim_example, features = "stochasticbasis_1")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("FeatureDimPlot with split_by = TRUE returns patchwork per feature", {
    p <- FeatureDimPlot(dim_example, features = c("stochasticbasis_1", "stochasticbasis_2"),
                        split_by = TRUE, combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("FeatureDimPlot with split_by = TRUE and combine = FALSE returns list per feature", {
    plots <- FeatureDimPlot(dim_example, features = c("stochasticbasis_1", "stochasticbasis_2"),
                            split_by = TRUE, combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("FeatureDimPlot with multiple split_by columns returns a list with one plot per combination", {
    dim2 <- dim_example
    dim2$split2 <- factor(rep(c("s1", "s2"), length.out = nrow(dim2)))
    plots <- suppressMessages(FeatureDimPlot(dim2, features = "stochasticbasis_1",
                                             split_by = c("group", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("FeatureDimPlot with multiple split_by columns returns combined plot", {
    dim2 <- dim_example
    dim2$split2 <- factor(rep(c("s1", "s2"), length.out = nrow(dim2)))
    p <- suppressMessages(FeatureDimPlot(dim2, features = "stochasticbasis_1",
                                         split_by = c("group", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("FeatureDimPlot with title works", {
    p <- FeatureDimPlot(dim_example, features = "stochasticbasis_1", title = "Feature Plot")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "Feature Plot")
})

test_that("FeatureDimPlot with upper/lower quantile works", {
    p <- FeatureDimPlot(dim_example, features = "stochasticbasis_1",
                        lower_quantile = 0.05, upper_quantile = 0.95)
    expect_s3_class(p, "ggplot")
})

#' Prepare cutoff data for ROC curve annotation
#'
#' Internal helper that computes the sensitivity and specificity values for a
#' set of user-supplied cutoff points (or computed via OptimalCutpoints methods)
#' across categories defined by group_by and/or facet_by columns. Used by
#' \code{ROCCurveAtomic()} to annotate the ROC curve with cutoff markers and labels.
#'
#' Cutoffs can be specified either as numeric values (raw score thresholds) or as
#' method names from the \code{OptimalCutpoints} package for automatic optimal
#' cutoff identification. When \code{n_cuts > 0}, \code{n_cuts} evenly-spaced
#' quantile values of the score distribution are used as cutoffs.
#'
#' @param data A data frame with the truth and score columns.
#' @param truth_by A character string of the column name that contains the true
#'   class labels (binary, 0/1 or TRUE/FALSE).
#' @param score_by A character string of the column name that contains the predicted scores.
#' @param cat_by A character string of the column name to categorise/group the data.
#'   When specified, cutoffs are calculated separately for each category level,
#'   enabling per-group or per-facet cutoff annotation.
#' @param cutoffs_at A vector of user-supplied cutoff values to plot as points.
#'   When non-NULL, overrides \code{n_cuts}. Supports both raw numeric values
#'   and method names from \code{\link[OptimalCutpoints]{optimal.cutpoints}}.
#' @param cutoffs_labels A character vector of user-supplied labels for the
#'   cutoffs. Must be the same length as \code{cutoffs_at}. When NULL, labels
#'   are generated automatically.
#' @param cutoffs_accuracy A numeric value specifying the rounding precision
#'   for automatically generated cutoff labels. Default: \code{0.001}.
#' @param n_cuts An integer specifying the number of evenly-spaced quantile-based
#'   cutoff points. Ignored when \code{cutoffs_at} is non-NULL. Default: \code{0}
#'   (no quantile cutoffs).
#' @param increasing A logical value. If TRUE (default), higher scores indicate
#'   the positive class; if FALSE, lower scores indicate the positive class.
#' @return A data frame with columns \code{cutoff}, \code{x} (1 - specificity),
#'   \code{y} (sensitivity), \code{label}, and \code{cat_by} (the category name).
#' @keywords internal
get_cutoffs_data <- function(
    data,
    truth_by,
    score_by,
    cat_by,
    cutoffs_at = NULL,
    cutoffs_labels = NULL,
    cutoffs_accuracy = 0.001,
    n_cuts = 0,
    increasing = TRUE
) {
    if (is.null(cutoffs_at) && n_cuts == 0) {
        return(NULL)
    }

    if (is.null(cutoffs_at) && n_cuts > 0) {
        cutoffs_at <- quantile(
            data[[score_by]],
            probs = seq(0, 1, length.out = n_cuts + 2)
        )
        cutoffs_at <- unname(cutoffs_at[-c(1, length(cutoffs_at))])
    }

    stopifnot(
        "cutoffs_labels should be NULL or a character vector of the same length as cutoffs_at." = is.null(
            cutoffs_labels
        ) ||
            length(cutoffs_labels) == length(cutoffs_at)
    )

    cutoff_methods <- c(
        "CB",
        "MCT",
        "MinValueSp",
        "MinValueSe",
        "ValueSe",
        "MinValueSpSe",
        "MaxSp",
        "MaxSe",
        "MaxSpSe",
        "MaxProdSpSe",
        "ROC01",
        "SpEqualSe",
        "Youden",
        "MaxEfficiency",
        "Minimax",
        "MaxDOR",
        "MaxKappa",
        "MinValueNPV",
        "MinValuePPV",
        "ValueNPV",
        "ValuePPV",
        "MinValueNPVPPV",
        "PROC01",
        "NPVEqualPPV",
        "MaxNPVPPV",
        "MaxSumNPVPPV",
        "MaxProdNPVPPV",
        "ValueDLR.Negative",
        "ValueDLR.Positive",
        "MinPvalue",
        "ObservedPrev",
        "MeanPrev",
        "PrevalenceMatching"
    )

    # Get sensitivity and specificity for given cutoff
    get_se_sp <- function(df, cutoff, cutoff_in = NULL) {
        cutoff_in <- cutoff_in %||% cutoff
        if (is.na(cutoff)) {
            stop(
                "[ROCCurve] Invalid cutoffs_at '",
                cutoff_in,
                "', which numerized as NA."
            )
        }
        min_score <- min(df[[score_by]], na.rm = TRUE)
        max_score <- max(df[[score_by]], na.rm = TRUE)
        if (cutoff > max_score || cutoff < min_score) {
            stop(
                "[ROCCurve] Invalid cutoffs_at '",
                cutoff_in,
                "', which is out of range of the score [",
                min_score,
                ", ",
                max_score,
                "]."
            )
        }

        if (increasing) {
            tp <- sum(df[[truth_by]] == 1 & df[[score_by]] >= cutoff)
            fp <- sum(df[[truth_by]] == 0 & df[[score_by]] >= cutoff)
            tn <- sum(df[[truth_by]] == 0 & df[[score_by]] < cutoff)
            fn <- sum(df[[truth_by]] == 1 & df[[score_by]] < cutoff)
        } else {
            tp <- sum(df[[truth_by]] == 1 & df[[score_by]] <= cutoff)
            fp <- sum(df[[truth_by]] == 0 & df[[score_by]] <= cutoff)
            tn <- sum(df[[truth_by]] == 0 & df[[score_by]] > cutoff)
            fn <- sum(df[[truth_by]] == 1 & df[[score_by]] > cutoff)
        }

        se <- tp / (tp + fn)
        sp <- tn / (tn + fp)
        c(cutoff = cutoff, x = 1 - sp, y = se)
    }

    # get the cutoff value and the corresponding sensitivity and specificity
    # for a given method
    get_cutoff <- function(df, cutoff, label) {
        if (!cutoff %in% cutoff_methods) {
            cutoff_num <- as.numeric(cutoff)
            se_sp <- get_se_sp(df, cutoff_num, cutoff)
            label <- label %||%
                scales::number(cutoff_num, accuracy = cutoffs_accuracy)
            return(list(c(se_sp, label = label)))
        } else {
            cutoff_info <- OptimalCutpoints::optimal.cutpoints(
                X = score_by,
                status = truth_by,
                methods = cutoff,
                data = as.data.frame(df),
                direction = ifelse(increasing, ">", "<"),
                tag.healthy = 0
            )
            n_cutoff_num <- tryCatch(
                {
                    length(cutoff_info[[cutoff]][[1]]$optimal.cutoff$cutoff)
                },
                error = function(e) {
                    warning(
                        "No optimal cutoff found for ",
                        cutoff,
                        immediate. = TRUE
                    )
                    0
                }
            )
            if (n_cutoff_num == 0) {
                se_sp <- get_se_sp(df, NA, cutoff)
                label <- label %||%
                    paste0("No optimal cutoff found for ", cutoff)
                return(list(c(se_sp, label = label)))
            } else {
                out <- lapply(seq_len(n_cutoff_num), function(i) {
                    cutoff_num <- cutoff_info[[cutoff]][[
                        1
                    ]]$optimal.cutoff$cutoff[i]
                    se_sp <- get_se_sp(df, cutoff_num, cutoff)
                    label <- cutoffs_labels[i] %||%
                        paste0(
                            scales::number(cutoff_num, cutoffs_accuracy),
                            " (by ",
                            cutoff,
                            ")"
                        )
                    c(se_sp, label = label)
                })
                return(out)
            }
        }
    }
    splits <- split(data, data[[cat_by]])
    do_call(
        rbind,
        lapply(names(splits), function(ns) {
            df <- do_call(
                rbind,
                lapply(seq_along(cutoffs_at), function(i) {
                    co <- get_cutoff(
                        splits[[ifelse(identical(ns, ""), 1, ns)]],
                        cutoffs_at[i],
                        if (is.null(cutoffs_labels)) NULL else cutoffs_labels[i]
                    )
                    co <- t(as.data.frame(co))
                    rownames(co) <- NULL
                    co
                })
            )
            df <- as.data.frame(df)
            df[[cat_by]] <- ns
            df$x <- as.numeric(df$x)
            df$y <- as.numeric(df$y)
            df
        })
    )
}


#' Atomic ROC curve (internal)
#'
#' Core implementation for drawing a single Receiver Operating Characteristic
#' (ROC) curve. This is the internal workhorse behind the exported
#' \code{\link{ROCCurve}} function. It takes a **single** data frame (no
#' \code{split_by} support) and returns a \code{ggplot} object.
#'
#' The function produces an ROC curve using \code{plotROC::geom_roc()}, with
#' the following capabilities:
#' \itemize{
#'   \item \strong{Multiple classifiers} — \code{score_by} accepts multiple
#'   column names, automatically pivoting them into a grouped format so
#'   several prediction scores can be compared on a single plot.
#'   \item \strong{AUC calculation} — area under the curve is computed via
#'   \code{plotROC::calc_auc()} and displayed either on the plot or in the
#'   legend, controlled by \code{show_auc}.
#'   \item \strong{Cutoff annotation} — user-specified cutoffs (numeric score
#'   thresholds or named optimal-cutoff methods from the
#'   \code{OptimalCutpoints} package) are rendered as markers with labels,
#'   using \code{ggrepel::geom_text_repel()} for label placement.
#'   \item \strong{Confidence intervals} — optional ROC confidence bands via
#'   \code{plotROC::geom_rocci()}.
#'   \item \strong{Axis flexibility} — supports reversed x-axis (displaying
#'   specificity) and percent-scaled axes.
#' }
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{show_auc resolution} — \code{match.arg()} resolves
#'   \code{show_auc} to one of \code{"auto"}, \code{"none"}, \code{"legend"},
#'   or \code{"plot"}.
#'   \item \strong{Column validation} — \code{\link{check_columns}()} validates
#'   \code{truth_by} (single column), \code{score_by} (multiple allowed), and
#'   \code{group_by} (factor, multi-column concatenated). An error is raised if
#'   \code{group_by} is provided alongside multiple \code{score_by} columns.
#'   \item \strong{Positive label encoding} — Converts \code{truth_by} to binary
#'   numeric (0/1) with three paths:
#'   \itemize{
#'     \item \code{pos_label} provided: re-factor with \code{pos_label} as the
#'     last level, then convert.
#'     \item \code{truth_by} is a factor: warn that the last level is treated
#'     as positive, then convert.
#'     \item Non-numeric, non-factor: coerce to factor, warn, then convert.
#'   }
#'   \item \strong{Multi-score_by expansion} — When \code{score_by} contains
#'   multiple columns, \code{tidyr::pivot_longer()} reshapes into a single
#'   \code{.score} column with a \code{.group} identifier, which becomes the
#'   \code{group_by} variable.
#'   \item \strong{ggplot dispatch} — Selects \code{gglogger::ggplot} or
#'   \code{ggplot2::ggplot} based on
#'   \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{Dummy group insertion} — When \code{group_by = NULL}, creates
#'   a synthetic \code{..group} column (constant \code{""}) so the curve still
#'   renders. The legend is suppressed (\code{"none"}).
#'   \item \strong{Auto AUC placement} — When \code{show_auc = "auto"},
#'   single-group or faceted plots place AUC on the plot; multi-group plots
#'   place it in the legend.
#'   \item \strong{Base ROC geometry} — \code{plotROC::geom_roc()} with
#'   \code{aes(d = truth, m = score, color = group)}, controlling direction
#'   via the \code{increasing} parameter.
#'   \item \strong{AUC calculation} — Temporarily facets via
#'   \code{\link{facet_plot}()}, then computes per-group (and per-facet) AUC
#'   values via \code{plotROC::calc_auc()}.
#'   \item \strong{Cutoff computation} — \code{get_cutoffs_data()} combines
#'   \code{group_by} and \code{facet_by} columns into a \code{.cat} identifier
#'   and computes per-category cutoff data, supporting both numeric thresholds
#'   and named \code{OptimalCutpoints} methods.
#'   \item \strong{Cutoff rendering} — When cutoff data is non-NULL, splits the
#'   \code{.cat} column back into group/facet columns and adds
#'   \code{geom_point()} (markers) and \code{geom_text_repel()} (labels) with
#'   configurable size, shape, stroke, colour, and background styling.
#'   \item \strong{Confidence intervals} — When \code{ci} is non-NULL,
#'   \code{plotROC::geom_rocci()} is added with the provided arguments.
#'   \item \strong{AUC display} — Three modes:
#'   \itemize{
#'     \item \code{"plot"}: \code{geom_text()} places AUC labels at a corner
#'     position determined by \code{increasing} and \code{x_axis_reverse}.
#'     \item \code{"legend"}: AUC values are appended to the
#'     \code{scale_color_manual()} labels (with per-facet prefixes when
#'     faceting is active).
#'     \item \code{"none"}: group level names are used as-is.
#'   }
#'   \item \strong{Diagonal reference} — \code{geom_abline()} draws the
#'   no-discrimination line (y = x, or y = -x when x-axis is reversed) as a
#'   dashed grey line.
#'   \item \strong{Color scale} — \code{scale_color_manual()} assigns
#'   palette-derived colours via \code{\link{palette_this}()}, with
#'   AUC-augmented labels when \code{show_auc = "legend"}.
#'   \item \strong{Axis formatting} — Percent labels on y-axis (and x-axis)
#'   when \code{percent = TRUE}. X-axis reversed (1 to 0) when
#'   \code{x_axis_reverse = TRUE}, changing the axis label to
#'   \code{"Specificity"}.
#'   \item \strong{Labels and theme} — \code{labs()} sets title, subtitle, x,
#'   and y labels. The theme, aspect ratio, legend position/direction, and
#'   dashed grid lines are applied.
#'   \item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
#'   computes \code{height} and \code{width} attributes from
#'   \code{base_height = 4.5}, aspect ratio, and legend metrics.
#'   \item \strong{Attribute storage} — \code{auc} and \code{cutoffs} data
#'   frames are stored as \code{attr(p, "auc")} and
#'   \code{attr(p, "cutoffs")} for retrieval by the exported wrapper.
#'   \item \strong{Faceting} — \code{\link{facet_plot}()} wraps the plot with
#'   \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is provided.
#' }
#'
#' @inheritParams common_args
#' @param data A data frame with the truth and score columns. See
#'   \url{https://CRAN.R-project.org/package=plotROC}
#'   for the expected format.
#' @param truth_by A character string naming the column that contains the true
#'   class labels (binary outcome, 0/1 or TRUE/FALSE).
#' @param score_by A character vector of column names containing the predicted
#'   scores (classifier output values). When multiple columns are provided, each
#'   column becomes a separate ROC curve grouped by a \code{.group} identifier.
#'   When multiple columns are used, \code{group_by} must be NULL.
#' @param pos_label A character string specifying the positive class label in
#'   \code{truth_by}. When NULL (default), the labels are handled by the
#'   \code{plotROC} package: if \code{truth_by} is a factor, the last level is
#'   used; otherwise it is coerced to a factor with a warning.
#' @param group_by A character vector of column names to group the ROC curve by.
#'   Each unique combination of group values renders a separate ROC curve.
#'   When \code{score_by} contains multiple columns, \code{group_by} must be
#'   NULL because the score columns themselves define the groups. Multiple
#'   \code{group_by} columns are concatenated with \code{group_by_sep}.
#' @param group_by_sep A character string used to separate concatenated
#'   \code{group_by} columns. Default: \code{"_"}.
#' @param group_name A character string to use as the legend title for the
#'   ROC curve groups. When NULL (default), the \code{group_by} column name
#'   is used.
#' @param x_axis_reverse A logical value. If TRUE, the x-axis is reversed (from
#'   1 to 0), displaying specificity instead of 1 - specificity. The x-axis
#'   label automatically changes to \code{"Specificity"}. Default: FALSE.
#' @param percent A logical value. If TRUE, the x and y axes are displayed as
#'   percentages (0 to 100). Default: FALSE.
#' @param ci A list of arguments passed to \code{plotROC::geom_rocci()} to add
#'   confidence intervals to the ROC curve. When NULL (default), no confidence
#'   intervals are drawn. Example: \code{ci = list(sig.level = 0.05)}.
#' @param n_cuts An integer specifying the number of evenly-spaced quantile-based
#'   cutoff points to annotate on the ROC curve. Quantiles are computed from the
#'   \code{score_by} distribution. Default: \code{0} (no quantile cutoffs).
#'   Ignored when \code{cutoffs_at} is non-NULL.
#' @param cutoffs_at A vector of user-supplied cutoff values to annotate as
#'   points on the ROC curve. When non-NULL, overrides \code{n_cuts}. Accepts
#'   raw numeric score thresholds and/or named method strings from the
#'   \code{\link[OptimalCutpoints]{optimal.cutpoints}} package for automatic
#'   optimal cutoff identification. Both \code{cutoffs_at} and
#'   \code{cutoffs.labels} are passed to \code{plotROC::geom_roc()}.
#'   Supported method values are:
#' \itemize{
#'   \item \code{"CB"} (cost-benefit method);
#'   \item \code{"MCT"} (minimises Misclassification Cost Term);
#'   \item \code{"MinValueSp"} (a minimum value set for Specificity);
#'   \item \code{"MinValueSe"} (a minimum value set for Sensitivity);
#'   \item \code{"ValueSe"} (a value set for Sensitivity);
#'   \item \code{"MinValueSpSe"} (a minimum value set for Specificity and Sensitivity);
#'   \item \code{"MaxSp"} (maximises Specificity);
#'   \item \code{"MaxSe"} (maximises Sensitivity);
#'   \item \code{"MaxSpSe"} (maximises Sensitivity and Specificity simultaneously);
#'   \item \code{"MaxProdSpSe"} (maximises the product of Sensitivity and Specificity);
#'   \item \code{"ROC01"} (minimises distance between ROC plot and point (0,1));
#'   \item \code{"SpEqualSe"} (Sensitivity = Specificity);
#'   \item \code{"Youden"} (Youden Index);
#'   \item \code{"MaxEfficiency"} (maximises Efficiency/Accuracy);
#'   \item \code{"Minimax"} (minimises the most frequent error);
#'   \item \code{"MaxDOR"} (maximises Diagnostic Odds Ratio);
#'   \item \code{"MaxKappa"} (maximises Kappa Index);
#'   \item \code{"MinValueNPV"} (a minimum value set for Negative Predictive Value);
#'   \item \code{"MinValuePPV"} (a minimum value set for Positive Predictive Value);
#'   \item \code{"ValueNPV"} (a value set for Negative Predictive Value);
#'   \item \code{"ValuePPV"} (a value set for Positive Predictive Value);
#'   \item \code{"MinValueNPVPPV"} (a minimum value set for Predictive Values);
#'   \item \code{"PROC01"} (minimises distance between PROC plot and point (0,1));
#'   \item \code{"NPVEqualPPV"} (Negative Predictive Value = Positive Predictive Value);
#'   \item \code{"MaxNPVPPV"} (maximises Positive and Negative Predictive Values simultaneously);
#'   \item \code{"MaxSumNPVPPV"} (maximises the sum of the Predictive Values);
#'   \item \code{"MaxProdNPVPPV"} (maximises the product of Predictive Values);
#'   \item \code{"ValueDLR.Negative"} (a value set for Negative Diagnostic Likelihood Ratio);
#'   \item \code{"ValueDLR.Positive"} (a value set for Positive Diagnostic Likelihood Ratio);
#'   \item \code{"MinPvalue"} (minimises p-value of the Chi-squared test);
#'   \item \code{"ObservedPrev"} (closest value to observed prevalence);
#'   \item \code{"MeanPrev"} (closest value to the mean of the test values);
#'   \item \code{"PrevalenceMatching"} (predicted prevalence equals observed prevalence).
#' }
#' @param cutoffs_labels A character vector of user-supplied labels for the
#'   cutoff points. Must be the same length as \code{cutoffs_at}. When NULL,
#'   labels are generated automatically (score value or method name).
#' @param cutoffs_accuracy A numeric value controlling the rounding precision of
#'   automatically generated cutoff labels. Default: \code{0.01}.
#' @param cutoffs_pt_size A numeric value specifying the size of the cutoff
#'   point markers. Default: \code{5}.
#' @param cutoffs_pt_shape A numeric value specifying the shape of the cutoff
#'   point markers. Default: \code{4} (cross).
#' @param cutoffs_pt_stroke A numeric value specifying the stroke width of the
#'   cutoff point markers. Default: \code{1}.
#' @param cutoffs_labal_fg A character string specifying the text colour of
#'   the cutoff labels. Default: \code{"black"}.
#' @param cutoffs_label_size A numeric value specifying the font size of the
#'   cutoff labels. Default: \code{4}.
#' @param cutoffs_label_bg A character string specifying the background colour
#'   of the cutoff labels. Default: \code{"white"}.
#' @param cutoffs_label_bg_r A numeric value specifying the background radius
#'   of the cutoff labels (passed to \code{ggrepel::geom_text_repel()}).
#'   Default: \code{0.1}.
#' @param show_auc A character string specifying the display mode for AUC values:
#' \itemize{
#'   \item \code{"auto"} (default): Automatically determine the position. When
#'   there is a single group or \code{facet_by} is provided, AUC is placed on
#'   the plot; otherwise AUC is placed in the legend.
#'   \item \code{"none"}: Do not display AUC values.
#'   \item \code{"legend"}: Display AUC values in the legend labels.
#'   \item \code{"plot"}: Display AUC values as text on the plot.
#' }
#' @param auc_accuracy A numeric value controlling the rounding precision of
#'   AUC values in labels. Default: \code{0.01}.
#' @param auc_size A numeric value specifying the font size of AUC labels when
#'   displayed on the plot. Default: \code{4}.
#' @param increasing A logical value. If TRUE (default), higher scores indicate
#'   the positive class; if FALSE, lower scores indicate the positive class.
#'   Controls the direction of comparison in the ROC analysis.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'   attributes (in inches) attached, plus \code{attr(p, "auc")} and
#'   \code{attr(p, "cutoffs")} data frames.
#' @keywords internal
#' @importFrom dplyr mutate summarise pull
#' @importFrom tidyr unite pivot_longer separate
#' @importFrom ggrepel geom_text_repel
#' @importFrom ggplot2 geom_abline scale_color_manual scale_x_reverse scale_x_continuous scale_y_continuous waiver
#' @importFrom ggplot2 geom_text labs
ROCCurveAtomic <- function(
    data,
    truth_by,
    score_by,
    pos_label = NULL,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    x_axis_reverse = FALSE,
    percent = FALSE,
    ci = NULL,
    n_cuts = 0,
    cutoffs_at = NULL,
    cutoffs_labels = NULL,
    cutoffs_accuracy = 0.01,
    cutoffs_pt_size = 5,
    cutoffs_pt_shape = 4,
    cutoffs_pt_stroke = 1,
    cutoffs_labal_fg = "black",
    cutoffs_label_size = 4,
    cutoffs_label_bg = "white",
    cutoffs_label_bg_r = 0.1,
    show_auc = c("auto", "none", "legend", "plot"),
    auc_accuracy = 0.01,
    auc_size = 4,
    increasing = TRUE,
    theme = "theme_this",
    theme_args = list(),
    palette = "Spectral",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    legend.position = waiver(),
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = ifelse(x_axis_reverse, "Specificity", "1 - Specificity"),
    ylab = "Sensitivity",
    ...
) {
    show_auc <- match.arg(show_auc)
    truth_by <- check_columns(data, truth_by, allow_multi = FALSE)
    score_by <- check_columns(
        data,
        score_by,
        force_factor = FALSE,
        allow_multi = TRUE
    )
    group_by <- check_columns(
        data,
        group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = group_by_sep
    )
    stopifnot(
        "'group_by' should be NULL when mulitple 'score_by' columns are provided." = length(
            score_by
        ) ==
            1 ||
            is.null(group_by)
    )

    if (!is.null(pos_label)) {
        data[[truth_by]] <- factor(
            data[[truth_by]],
            levels = c(setdiff(unique(data[[truth_by]]), pos_label), pos_label)
        )
        data[[truth_by]] <- as.numeric(data[[truth_by]]) - 1
    } else if (is.factor(data[[truth_by]])) {
        warning(
            "'pos_label' is NULL, the last level of '",
            truth_by,
            "' (",
            levels(data[[truth_by]])[length(levels(data[[truth_by]]))],
            ") will be used as the positive label."
        )
        data[[truth_by]] <- as.numeric(data[[truth_by]]) - 1
    } else if (!is.numeric(data[[truth_by]])) {
        data[[truth_by]] <- factor(data[[truth_by]])
        warning(
            "'pos_label' is NULL, value '",
            levels(data[[truth_by]])[length(levels(data[[truth_by]]))],
            "' from '",
            truth_by,
            "' will be used as the positive label."
        )
        data[[truth_by]] <- as.numeric(data[[truth_by]]) - 1
    }

    if (length(score_by) > 1) {
        data <- pivot_longer(
            data,
            cols = all_of(score_by),
            names_to = ".group",
            values_to = ".score"
        )
        group_by <- ".group"
        data[[group_by]] <- factor(data[[group_by]], levels = score_by)
        score_by <- ".score"
    }

    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    base_size <- theme_args$base_size %||% 12
    text_size_scale <- base_size / 12

    if (is.null(group_by)) {
        data$..group <- factor("")
        group_by <- "..group"
        legend.position <- ifelse(
            inherits(legend.position, "waiver"),
            "none",
            "right"
        )
    } else {
        legend.position <- ifelse(
            inherits(legend.position, "waiver"),
            "right",
            legend.position
        )
    }

    # determine where to place the AUC
    if (identical(show_auc, "auto")) {
        if (identical(group_by, "..group") || !is.null(facet_by)) {
            show_auc <- "plot"
        } else {
            show_auc <- "legend"
        }
    }

    p <- ggplot(
        data,
        aes(d = !!sym(truth_by), m = !!sym(score_by), color = !!sym(group_by))
    ) +
        # plotROC::geom_roc() doesn't support different cutoffs for different groups/facets
        plotROC::geom_roc(
            n.cuts = 0,
            cutoffs.at = NULL,
            cutoff.labels = NULL,
            increasing = increasing,
            ...
        )

    # facet the plot to calculate the correct AUCs for each facet
    attr(p, "height") <- attr(p, "width") <- 0
    auc <- plotROC::calc_auc(
        facet_plot(
            p,
            facet_by,
            facet_scales,
            facet_nrow,
            facet_ncol,
            facet_byrow
        )
    )

    cutoffs_df <- get_cutoffs_data(
        data %>%
            unite(".cat", !!!syms(unique(c(group_by, facet_by))), sep = " // "),
        truth_by,
        score_by,
        ".cat",
        cutoffs_at,
        cutoffs_labels,
        cutoffs_accuracy,
        n_cuts,
        increasing
    )

    if (!is.null(cutoffs_df)) {
        cutoffs_df <- cutoffs_df %>%
            separate(".cat", into = unique(c(group_by, facet_by)), sep = " // ")

        p <- p +
            geom_point(
                data = cutoffs_df,
                aes(x = !!sym("x"), y = !!sym("y"), color = !!sym(group_by)),
                inherit.aes = FALSE,
                shape = cutoffs_pt_shape,
                size = cutoffs_pt_size,
                stroke = cutoffs_pt_stroke
            ) +
            geom_text_repel(
                data = cutoffs_df,
                aes(label = !!sym("label"), x = !!sym("x"), y = !!sym("y")),
                inherit.aes = FALSE,
                size = cutoffs_label_size * text_size_scale,
                color = cutoffs_labal_fg,
                nudge_x = 0.01,
                bg.color = cutoffs_label_bg,
                bg.r = cutoffs_label_bg_r,
                min.segment.length = 10
            )
    }
    cutoffs_df$specificity <- 1 - cutoffs_df$x
    cutoffs_df$sensitivity <- cutoffs_df$y

    if (!is.null(ci)) {
        p <- p + do_call(plotROC::geom_rocci, ci)
    }

    if (show_auc == "plot") {
        if (identical(group_by, "..group")) {
            auc_df <- auc
            auc_df$AUC <- paste0(
                "AUC = ",
                scales::number(auc$AUC, accuracy = auc_accuracy)
            )
        } else {
            auc_df <- if (is.null(facet_by)) {
                auc
            } else {
                auc %>% dplyr::group_by(!!sym(facet_by))
            }
            auc_df <- auc_df %>%
                summarise(
                    AUC = paste0(
                        "AUC (",
                        !!sym(group_by),
                        ") = ",
                        scales::number(!!sym("AUC"), accuracy = auc_accuracy),
                        collapse = "\n"
                    ),
                    .groups = "drop"
                )
        }

        auc_pos_x <- ifelse(increasing, ifelse(x_axis_reverse, -1, 1), 0)
        auc_pos_y <- ifelse(increasing, 0, 1)
        auc_pos_hjust <- ifelse(increasing == x_axis_reverse, -0.05, 1.05)
        auc_pos_vjust <- ifelse(
            increasing,
            -0.5 / nlevels(data[[group_by]]),
            1 + .5 / nlevels(data[[group_by]])
        )
        p <- p +
            geom_text(
                data = auc_df,
                x = auc_pos_x,
                y = auc_pos_y,
                size = auc_size * text_size_scale,
                hjust = auc_pos_hjust,
                vjust = auc_pos_vjust,
                aes(label = !!sym("AUC")),
                inherit.aes = FALSE
            )
        labels <- levels(data[[group_by]])
    } else if (show_auc == "legend") {
        if (!is.null(facet_by)) {
            auc_df <- auc[
                order(auc[[facet_by]], auc[[group_by]]),
                ,
                drop = FALSE
            ]
            auc_df$AUC <- paste0(
                "AUC (",
                auc_df[[facet_by]],
                ") = ",
                scales::number(auc$AUC, accuracy = auc_accuracy)
            )
            labels <- auc_df %>%
                dplyr::group_by(!!sym(group_by)) %>%
                summarise(
                    AUC = paste(!!sym("AUC"), collapse = "\n"),
                    .groups = "drop"
                ) %>%
                mutate(AUC = paste0(!!sym(group_by), "\n", !!sym("AUC"))) %>%
                pull("AUC")
        } else {
            labels <- paste0(
                auc[[group_by]],
                " (AUC = ",
                scales::number(auc$AUC, accuracy = auc_accuracy),
                ")"
            )
        }
    } else {
        labels <- levels(data[[group_by]])
    }

    p <- p +
        # diagnal line
        geom_abline(
            intercept = 0,
            slope = ifelse(x_axis_reverse, -1, 1),
            linetype = "dashed",
            color = "grey50"
        ) +
        scale_color_manual(
            name = group_name %||% group_by,
            values = palette_this(
                levels(data[[group_by]]),
                palette = palette,
                palcolor = palcolor,
                alpha = alpha,
                reverse = palreverse
            ),
            labels = labels
        )

    if (isTRUE(percent)) {
        p <- p + scale_y_continuous(labels = scales::percent)
    }

    if (x_axis_reverse) {
        p <- p +
            scale_x_reverse(
                labels = if (isTRUE(percent)) scales::percent else waiver()
            )
    } else if (isTRUE(percent)) {
        p <- p + scale_x_continuous(labels = scales::percent)
    }

    p <- p +
        labs(title = title, subtitle = subtitle, x = xlab, y = ylab) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major = element_line(colour = "grey80", linetype = 2)
        )

    dims <- calculate_plot_dimensions(
        base_height = 4.5,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = length(labels),
        legend_nchar = max(nchar(labels))
    )

    attr(p, "auc") <- auc
    attr(p, "cutoffs") <- cutoffs_df
    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction
    )
}


#' ROC curve
#'
#' Draws one or more Receiver Operating Characteristic (ROC) curves for
#' evaluating binary classifier performance. The function wraps
#' \code{\link{ROCCurveAtomic}} with \code{split_by} handling, providing the
#' ability to generate separate ROC curves per split level and combine them
#' via \code{\link[patchwork]{wrap_plots}}.
#'
#' Key features:
#' \itemize{
#'   \item \strong{Multiple classifiers} — compare several prediction scores
#'   side-by-side by providing multiple \code{score_by} columns.
#'   \item \strong{AUC display} — area under the curve values shown on the
#'   plot or in the legend, with configurable precision.
#'   \item \strong{Optimal cutoffs} — identify and annotate optimal cutoff
#'   points using any of the 30+ methods from the \code{OptimalCutpoints}
#'   package, or supply custom numeric thresholds.
#'   \item \strong{Confidence intervals} — add ROC confidence bands via
#'   \code{plotROC::geom_rocci()}.
#'   \item \strong{Axis orientation} — reverse x-axis to show specificity or
#'   display axes as percentages.
#'   \item \strong{Splitting and faceting} — split data into sub-plots via
#'   \code{split_by} or facet within a single plot via \code{facet_by}.
#' }
#'
#' @section split_by Workflow:
#' When \code{split_by} is provided, the following pipeline executes:
#' \enumerate{
#'   \item \strong{Validation} — \code{\link{validate_common_args}()} checks
#'   the random seed and \code{facet_by} configuration.
#'   \item \strong{Column resolution} — \code{\link{check_columns}()} resolves
#'   \code{split_by} (force_factor, allow_multi, concat_multi).
#'   \item \strong{Data splitting} — Unused factor levels in \code{split_by}
#'   are dropped via \code{droplevels()}, and the data is split by
#'   \code{split_by} levels (preserving factor level order). If
#'   \code{split_by} is NULL, the data is wrapped in a single-element list
#'   named \code{"..."}.
#'   \item \strong{Per-split resolution} — \code{\link{check_palette}()},
#'   \code{\link{check_palcolor}()}, and \code{\link{check_legend}()} resolve
#'   per-split palette, colour, legend.position, and legend.direction
#'   overrides.
#'   \item \strong{Per-split dispatch} — For each split:
#'   \itemize{
#'     \item Title resolution: if \code{title} is a function, it receives
#'     the split level name; otherwise \code{title \%||\% split_level} is
#'     used.
#'     \item The \code{split_by} column is removed from the per-split data
#'     frame to avoid conflicts with the ROC analysis.
#'     \item \code{\link{ROCCurveAtomic}()} is called with the per-split
#'     palette, palcolor, legend.position, and legend.direction.
#'   }
#'   \item \strong{Combination} — \code{\link{combine_plots}()} assembles the
#'   list of plots via \code{patchwork::wrap_plots}, honouring
#'   \code{nrow}/\code{ncol}/\code{byrow}/\code{design}.
#'   \item \strong{AUC / cutoff collection} — When \code{combine = TRUE}, the
#'   per-split \code{auc} and \code{cutoffs} attributes are collected into
#'   combined data frames with a \code{split_by} column identifying the source
#'   split, and stored as \code{attr(p, "auc")} and
#'   \code{attr(p, "cutoffs")}.
#' }
#'
#' @inheritParams common_args
#' @inheritParams ROCCurveAtomic
#' @param split_by The column(s) to split the data by and produce separate
#'   ROC curve plots for each level. The \code{split_by} column is removed from
#'   the per-split data to avoid interfering with ROC analysis. Multiple columns
#'   are concatenated with \code{split_by_sep}.
#' @param split_by_sep A character string used to separate concatenated
#'   \code{split_by} columns. Default: \code{"_"}.
#' @param seed A numeric seed for reproducibility. Default: \code{8525}.
#'   Passed to \code{\link{validate_common_args}()}.
#' @param combine A logical value. When TRUE (default), the list of per-split
#'   plots is combined into a single \code{patchwork} object with
#'   \code{attr(p, "auc")} and \code{attr(p, "cutoffs")} containing the
#'   aggregated results. When FALSE, returns a named list of individual
#'   \code{ggplot} objects.
#' @param nrow,ncol Integer values specifying the number of rows and columns
#'   in the combined plot layout. Passed to \code{\link[patchwork]{wrap_plots}}.
#' @param byrow A logical value. If TRUE (default), the combined layout is
#'   filled row-wise. Passed to \code{\link[patchwork]{wrap_plots}}.
#' @param axes A character string specifying how axes are treated across the
#'   combined layout. Passed to \code{\link{combine_plots}()}. Options:
#'   \code{"keep"}, \code{"collect"}, \code{"collect_x"}, \code{"collect_y"}.
#' @param axis_titles A character string specifying how axis titles are treated
#'   across the combined layout. Defaults to \code{axes}. Passed to
#'   \code{\link{combine_plots}()}.
#' @param guides A character string specifying how legends are collected across
#'   panels in the combined layout. Passed to \code{\link{combine_plots}()}.
#' @param design A custom layout specification for the combined plot. Passed
#'   to \code{\link{combine_plots}()}. When specified, \code{nrow},
#'   \code{ncol}, and \code{byrow} are ignored.
#' @return A \code{patchwork} object (when \code{combine = TRUE}) with
#'   \code{attr(p, "auc")} and \code{attr(p, "cutoffs")} data frames
#'   containing aggregated AUC values and cutoff information across all
#'   splits. When \code{combine = FALSE}, returns a named list of
#'   \code{ggplot} objects, each with their own \code{attr(p[[i]], "auc")}
#'   and \code{attr(p[[i]], "cutoffs")}.
#' @export
#' @examples
#' set.seed(8525)
#'
#' D.ex <- rbinom(200, size = 1, prob = .5)
#' M1 <- rnorm(200, mean = D.ex, sd = .65)
#' M2 <- rnorm(200, mean = D.ex, sd = 1.5)
#' gender <- c("Male", "Female")[rbinom(200, 1, .49) + 1]
#'
#' data <- data.frame(D = D.ex, D.str = c("Healthy", "Ill")[D.ex + 1],
#'   gender = gender, M1 = M1, M2 = M2)
#'
#' # --- Basic ROC curve ---
#' ROCCurve(data, truth_by = "D", score_by = "M1")
#'
#' # --- Will warn about the positive label ---
#' ROCCurve(data, truth_by = "D.str", score_by = "M1")
#'
#' # --- Decreasing direction ---
#' ROCCurve(data, truth_by = "D", score_by = "M1", increasing = FALSE)
#'
#' # --- Multiple ROC curves (multiple classifiers) ---
#' ROCCurve(data, truth_by = "D", score_by = c("M1", "M2"), group_name = "Method")
#'
#' # --- Grouping by a column ---
#' ROCCurve(data, truth_by = "D", score_by = "M1", group_by = "gender", show_auc = "plot")
#'
#' # --- Reverse x-axis and display as percentages ---
#' ROCCurve(data, truth_by = "D", score_by = "M1", x_axis_reverse = TRUE, percent = TRUE)
#'
#' # --- Custom n_cuts and single colour ---
#' ROCCurve(data, truth_by = "D", score_by = "M1", n_cuts = 10, palcolor = "black")
#'
#' # --- Add confidence intervals ---
#' ROCCurve(data, truth_by = "D", score_by = "M1", ci = list(sig.level = .01))
#'
#' # --- Facet by a column ---
#' ROCCurve(data, truth_by = "D", score_by = "M1", facet_by = "gender")
#'
#' # --- Show cutoffs ---
#' ROCCurve(data, truth_by = "D", score_by = "M1", cutoffs_at = c(0, "ROC01", "SpEqualSe"))
#'
#' # --- Split by a column ---
#' p <- ROCCurve(data, truth_by = "D", score_by = "M1", split_by = "gender",
#'    cutoffs_at = c(0.2, "MaxSpSe"))
#' p
#' # Retrieve the AUC values
#' attr(p, "auc")
#' # Retrieve the cutoffs
#' attr(p, "cutoffs")
ROCCurve <- function(
    data,
    truth_by,
    score_by,
    pos_label = NULL,
    split_by = NULL,
    split_by_sep = "_",
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    x_axis_reverse = FALSE,
    percent = FALSE,
    ci = NULL,
    n_cuts = 0,
    cutoffs_at = NULL,
    cutoffs_labels = NULL,
    cutoffs_accuracy = 0.001,
    cutoffs_pt_size = 5,
    cutoffs_pt_shape = 4,
    cutoffs_pt_stroke = 1,
    cutoffs_labal_fg = "black",
    cutoffs_label_size = 4,
    cutoffs_label_bg = "white",
    cutoffs_label_bg_r = 0.1,
    show_auc = c("auto", "none", "legend", "plot"),
    auc_accuracy = 0.01,
    auc_size = 4,
    theme = "theme_this",
    theme_args = list(),
    palette = "Spectral",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    legend.position = waiver(),
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = ifelse(x_axis_reverse, "Specificity", "1 - Specificity"),
    ylab = "Sensitivity",
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    seed = 8525,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by)
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }

    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            if (!is.null(split_by)) {
                datas[[nm]][[split_by]] <- NULL
            }

            ROCCurveAtomic(
                datas[[nm]],
                truth_by,
                score_by,
                pos_label = pos_label,
                group_by = group_by,
                group_by_sep = group_by_sep,
                group_name = group_name,
                x_axis_reverse = x_axis_reverse,
                percent = percent,
                ci = ci,
                n_cuts = n_cuts,
                cutoffs_at = cutoffs_at,
                cutoffs_labels = cutoffs_labels,
                cutoffs_accuracy = cutoffs_accuracy,
                cutoffs_pt_size = cutoffs_pt_size,
                cutoffs_pt_shape = cutoffs_pt_shape,
                cutoffs_pt_stroke = cutoffs_pt_stroke,
                cutoffs_labal_fg = cutoffs_labal_fg,
                cutoffs_label_size = cutoffs_label_size,
                cutoffs_label_bg = cutoffs_label_bg,
                cutoffs_label_bg_r = cutoffs_label_bg_r,
                show_auc = show_auc,
                auc_accuracy = auc_accuracy,
                auc_size = auc_size,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                alpha = alpha,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    p <- combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )

    if (!combine) {
        return(p)
    } else {
        attr(p, "auc") <- do_call(
            rbind,
            lapply(seq_along(plots), function(i) {
                df <- attr(plots[[i]], "auc")
                if (!is.null(split_by)) {
                    df[[split_by]] <- names(datas)[i]
                }
                df
            })
        )
        attr(p, "cutoffs") <- do_call(
            rbind,
            lapply(seq_along(plots), function(i) {
                df <- attr(plots[[i]], "cutoffs")
                if (!is.null(split_by)) {
                    df[[split_by]] <- names(datas)[i]
                }
                df
            })
        )
        return(p)
    }
}

set.seed(8525)
roc_data <- data.frame(
    truth = factor(c(rep("pos", 60), rep("neg", 60))),
    score = c(rnorm(60, mean = 0.7, sd = 0.2), rnorm(60, mean = 0.3, sd = 0.2)),
    score2 = c(rnorm(60, mean = 0.6, sd = 0.25), rnorm(60, mean = 0.4, sd = 0.25)),
    group = factor(rep(c("G1", "G2"), 60))
)

test_that("ROCCurve basic usage works", {
    skip_if_not_installed("pROC")
    p <- ROCCurve(roc_data, truth_by = "truth", score_by = "score", pos_label = "pos")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("ROCCurve with title and subtitle works", {
    skip_if_not_installed("pROC")
    p <- ROCCurve(roc_data, truth_by = "truth", score_by = "score", pos_label = "pos",
                  title = "ROC Title", subtitle = "ROC Subtitle")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "ROC Title")
    expect_equal(p$labels$subtitle, "ROC Subtitle")
})

test_that("ROCCurve with group_by works", {
    skip_if_not_installed("pROC")
    p <- ROCCurve(roc_data, truth_by = "truth", score_by = c("score", "score2"), pos_label = "pos")
    expect_s3_class(p, "ggplot")
})

test_that("ROCCurve with split_by returns patchwork", {
    skip_if_not_installed("pROC")
    p <- ROCCurve(roc_data, truth_by = "truth", score_by = "score", pos_label = "pos",
                  split_by = "group", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("ROCCurve with combine = FALSE returns list", {
    skip_if_not_installed("pROC")
    plots <- ROCCurve(roc_data, truth_by = "truth", score_by = "score", pos_label = "pos",
                      split_by = "group", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("ROCCurve with multiple split_by columns returns a list with one plot per combination", {
    skip_if_not_installed("pROC")
    roc_data2 <- roc_data
    roc_data2$split2 <- factor(rep(c("s1", "s2"), each = 30))
    plots <- suppressMessages(ROCCurve(roc_data2, truth_by = "truth", score_by = "score", pos_label = "pos",
                                       split_by = c("group", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("ROCCurve with multiple split_by columns returns combined plot", {
    skip_if_not_installed("pROC")
    roc_data2 <- roc_data
    roc_data2$split2 <- factor(rep(c("s1", "s2"), each = 30))
    p <- suppressMessages(ROCCurve(roc_data2, truth_by = "truth", score_by = "score", pos_label = "pos",
                                   split_by = c("group", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("ROCCurve with show_auc = 'plot' works", {
    skip_if_not_installed("pROC")
    p <- ROCCurve(roc_data, truth_by = "truth", score_by = "score", pos_label = "pos",
                  show_auc = "plot")
    expect_s3_class(p, "ggplot")
})

test_that("ROCCurve with x_axis_reverse = TRUE works", {
    skip_if_not_installed("pROC")
    p <- ROCCurve(roc_data, truth_by = "truth", score_by = "score", pos_label = "pos",
                  x_axis_reverse = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("ROCCurve with facet_by works", {
    skip_if_not_installed("pROC")
    p <- ROCCurve(roc_data, truth_by = "truth", score_by = "score", pos_label = "pos",
                  facet_by = "group")
    expect_s3_class(p, "ggplot")
})

#' Atomic scatter plot (internal)
#'
#' @description
#' Core implementation for drawing a single scatter plot. This is the workhorse
#' behind the exported \code{\link{ScatterPlot}} function -- it takes a
#' **single** data frame (no \code{split_by} support) and returns a
#' \code{ggplot} object. The plot positions points by \code{x} and \code{y}
#' coordinates (both numeric), with optional size encoding via a third numeric
#' variable (\code{size_by}), and colour encoding (\code{color_by}) that can be
#' numeric (continuous gradient) or categorical (discrete palette).
#'
#' Key features:
#' \itemize{
#'   \item \strong{Variable point size} -- \code{size_by} accepts either a
#'   numeric constant (uniform dot size) or a column name (size scales with
#'   value via \code{scale_size_area()}).
#'   \item \strong{Colour modes} -- \code{color_by} can be numeric (rendered as
#'   a continuous gradient via \code{scale_fill_gradientn()} or
#'   \code{scale_color_gradientn()}) or factor/character (discrete palette
#'   via \code{scale_fill_manual()}/\code{scale_color_manual()}).
#'   \item \strong{Shape support} -- shapes 21--25 support separate fill and
#'   border (\code{color}) aesthetics; all other shapes use only the colour
#'   aesthetic. The \code{border_color} parameter can be a constant colour,
#'   \code{TRUE} (track the fill gradient), or omitted.
#'   \item \strong{Colour scale trimming} -- \code{lower_quantile} /
#'   \code{upper_quantile} (or explicit \code{lower_cutoff} /
#'   \code{upper_cutoff}) trim/clamp the continuous colour scale extremes.
#'   \item \strong{Axis transformation} -- \code{xtrans} and \code{ytrans}
#'   apply arbitrary scale transformations (e.g. \code{"log10"}, \code{"sqrt"})
#'   to the x and y axes via \code{scale_x_continuous()} /
#'   \code{scale_y_continuous()}.
#'   \item \strong{Point highlighting} -- \code{highlight} accepts indices,
#'   rownames, logical \code{TRUE}, or a string expression to select points
#'   that are overlaid with a distinct shape, size, colour, and alpha.
#' }
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{ggplot dispatch} -- selects \code{gglogger::ggplot} or
#'   \code{ggplot2::ggplot} based on
#'   \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{Column resolution} -- \code{facet_by} is validated and
#'   forced to factor via \code{\link{check_columns}()}. \code{size_by} is
#'   validated when it is a column name; numeric constants are kept as-is.
#'   \code{color_by} is validated.
#'   \item \strong{Dummy colour guard} -- when \code{color_by = NULL}, a
#'   synthetic \code{.color_by} column (constant \code{""}) is created and the
#'   colour legend is suppressed.
#'   \item \strong{Categorical colour coercion} -- when \code{color_by} is a
#'   non-numeric column, it is forced to factor via
#'   \code{\link{check_columns}()} with \code{force_factor = TRUE}.
#'   \item \strong{Continuous colour scale preparation} -- when
#'   \code{color_by} is numeric,
#'   \code{\link{prepare_continuous_color_scale}()} computes quantile-trimmed or
#'   cutoff-clamped colour range endpoints (\code{feat_colors_value}) and
#'   winsorizes out-of-range data.
#'   \item \strong{Highlight resolution} -- \code{highlight} is processed into
#'   a subset data frame (\code{hidata}) via one of four paths:
#'   \code{TRUE} (all points), numeric index vector, a single string expression
#'   (parsed with \code{rlang::parse_expr()} and applied via
#'   \code{\link[dplyr]{filter}()}), or a character vector of rownames.
#'   An error is thrown if rownames are used on data without row names.
#'   \item \strong{Shape fill detection} -- determines whether the point shape
#'   (\code{shape}) supports a separate fill aesthetic (shapes 21--25).
#'   \item \strong{Base ggplot} -- initialises \code{ggplot(data, aes(x, y))}.
#'   \item \strong{Point layer configuration} -- the aesthetic mapping and
#'   constant aesthetics for \code{geom_point()} are assembled:
#'   \itemize{
#'     \item \emph{Fill vs colour}: shapes 21--25 map \code{fill} (and
#'     optionally \code{color} when \code{border_color = TRUE}); other shapes
#'     map \code{color}.
#'     \item \emph{Size}: a numeric \code{size_by} is passed as a constant
#'     aesthetic; a column name is mapped to the \code{size} aesthetic.
#'   }
#'   \item \strong{Size scale} -- when \code{size_by} is a column name,
#'   \code{scale_size_area(max_size = 6)} is added with a size legend
#'   (order = 1, title = \code{size_name \%||\% size_by}).
#'   \item \strong{Fill / colour scale (4 branches)}:
#'   \itemize{
#'     \item \emph{Shape with fill + numeric colour}: \code{scale_fill_gradientn()}
#'     with \code{feat_colors_value} rescaling and either a colour-bar legend
#'     or suppressed guide. When \code{border_color = TRUE}, a matching
#'     \code{scale_color_gradientn()} is added.
#'     \item \emph{Shape with fill + factor colour}: \code{scale_fill_manual()}
#'     with \code{\link{palette_this}()} colours and a discrete legend. When
#'     \code{border_color = TRUE}, a matching \code{scale_color_manual()} is
#'     added.
#'     \item \emph{Shape without fill + numeric colour}: \code{scale_color_gradientn()}
#'     with continuous colour-bar legend.
#'     \item \emph{Shape without fill + factor colour}: \code{scale_color_manual()}
#'     with discrete legend.
#'   }
#'   \item \strong{Highlight overlay} -- when \code{hidata} is non-\code{NULL},
#'   a second \code{geom_point()} layer is added using \code{highlight_shape},
#'   \code{highlight_color}, \code{highlight_size}, and \code{highlight_alpha}.
#'   The fill or colour aesthetic is chosen based on whether
#'   \code{highlight_shape} is 21--25.
#'   \item \strong{Axis scales} -- \code{scale_x_continuous(trans = xtrans)}
#'   and \code{scale_y_continuous(trans = ytrans)} apply the requested
#'   transformations.
#'   \item \strong{Labels and theme} -- \code{labs(title, subtitle, x, y)} sets
#'   plot annotations. \code{do_call(theme, theme_args)} applies the theme.
#'   \code{panel.grid.major} is set to grey80 dashed lines.
#'   \item \strong{Dimension calculation} -- \code{\link{calculate_plot_dimensions}()}
#'   computes plot \code{height} and \code{width} from \code{base_height = 5},
#'   \code{aspect.ratio}, and legend metrics (number of legend items and
#'   maximum label character width).
#'   \item \strong{Faceting} -- \code{\link{facet_plot}()} wraps the plot with
#'   \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is provided.
#' }
#'
#' @inheritParams common_args
#' @param x A character string specifying the numeric column to use for the
#'  x-axis. A numeric column is expected.
#' @param y A character string specifying the numeric column to use for the
#'  y-axis. A numeric column is expected.
#' @param size_by Either a numeric constant (uniform dot size) or a character
#'  string naming a numeric column whose values control dot size via
#'  \code{scale_size_area(max_size = 6)}. Default: \code{2}.
#' @param size_name A character string for the size legend title. When
#'  \code{NULL} (default), the \code{size_by} column name is used. Ignored when
#'  \code{size_by} is a numeric constant.
#' @param color_by A character string naming a column whose values control
#'  dot colour. Can be numeric (continuous gradient via
#'  \code{scale_fill_gradientn()} / \code{scale_color_gradientn()}) or
#'  factor/character (discrete palette via
#'  \code{scale_fill_manual()} / \code{scale_color_manual()}). For shapes
#'  21--25, the colour is applied to the fill aesthetic. When \code{NULL}
#'  (default), all dots are rendered in a single colour derived from the
#'  palette.
#' @param color_name A character string for the colour legend title. When
#'  \code{NULL} (default), the \code{color_by} column name is used.
#' @param border_color Controls the point border colour. For shapes 21--25:
#'  \itemize{
#'    \item \code{"black"} (default) -- constant black border.
#'    \item A colour string (e.g. \code{"red"}, \code{"#FF0000"}) -- constant
#'    colour border.
#'    \item \code{TRUE} -- border colour tracks the \code{color_by} gradient
#'    / palette via \code{scale_color_gradientn()} /
#'    \code{scale_color_manual()}.
#'  }
#'  For shapes without a fill aesthetic (not 21--25), this parameter has no
#'  effect.
#' @param border_size A numeric value specifying the point border size. Default: \code{0.5}.
#' @param highlight Specifies which points to highlight with an overlaid
#'  \code{geom_point()} layer. Accepted values:
#'  \itemize{
#'    \item \code{NULL} (default) -- no highlighting.
#'    \item \code{TRUE} -- all points are highlighted.
#'    \item A numeric vector -- row indices of points to highlight.
#'    \item A single character string -- an R expression (e.g.
#'    \code{"x > 0"}) that is parsed with \code{rlang::parse_expr()} and
#'    evaluated via \code{\link[dplyr]{filter}()} to select rows.
#'    \item A character vector -- rownames of points to highlight. An error
#'    is thrown if the data has no rownames.
#'  }
#' @param highlight_shape A numeric value specifying the point shape for
#'  highlighted points. Default: \code{16} (filled circle). Shapes 21--25
#'  use the \code{fill} aesthetic; other shapes use \code{color}.
#' @param highlight_size A numeric value specifying the size of highlighted
#'  points. Default: \code{3}.
#' @param highlight_color A character string specifying the colour of
#'  highlighted points. Default: \code{"red"}.
#' @param highlight_alpha A numeric value in \code{[0, 1]} specifying the
#'  transparency of highlighted points. Default: \code{1}.
#' @param shape A numeric value specifying the point shape. Default: \code{21}
#'  (filled circle with border). Shapes 21--25 support separate fill and border
#'  colour aesthetics; all other shapes use a single colour aesthetic.
#' @param xtrans A character string specifying the transformation for the
#'  x-axis, passed to \code{scale_x_continuous(trans = ...)}. Common options:
#'  \code{"identity"} (default), \code{"log10"}, \code{"log2"},
#'  \code{"sqrt"}, \code{"reverse"}. See \code{ggplot2::continuous_scale()}
#'  for a full list.
#' @param ytrans A character string specifying the transformation for the
#'  y-axis, passed to \code{scale_y_continuous(trans = ...)}. Common options:
#'  \code{"identity"} (default), \code{"log10"}, \code{"log2"},
#'  \code{"sqrt"}, \code{"reverse"}.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @keywords internal
#' @importFrom utils getFromNamespace
#' @importFrom dplyr filter
#' @importFrom ggplot2 aes geom_point scale_size_area scale_fill_gradientn scale_color_gradientn labs
#' @importFrom ggplot2 guide_colorbar guide_legend guides guide_none scale_size waiver
#' @importFrom scales rescale
ScatterPlotAtomic <- function(
    data,
    x,
    y,
    size_by = 2,
    size_name = NULL,
    color_by = NULL,
    color_name = NULL,
    palreverse = FALSE,
    theme = "theme_this",
    theme_args = list(),
    alpha = ifelse(shape %in% 21:25, 0.65, 1),
    shape = 21,
    border_color = "black",
    border_size = 0.5,
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    xtrans = "identity",
    ytrans = "identity",
    highlight = NULL,
    highlight_shape = 16,
    highlight_size = 3,
    highlight_color = "red",
    highlight_alpha = 1,
    palette = ifelse(
        !is.null(color_by) && !is.numeric(data[[color_by]]),
        "Paired",
        "Spectral"
    ),
    palcolor = NULL,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )
    if (!is.numeric(size_by)) {
        size_by <- check_columns(data, size_by)
    }
    color_by <- check_columns(data, color_by)
    if (is.null(color_by)) {
        data$.color_by <- ""
        color_by <- ".color_by"
        color_legend <- FALSE
    } else {
        color_legend <- TRUE
    }
    if (!is.numeric(data[[color_by]])) {
        color_by <- check_columns(data, color_by, force_factor = TRUE)
    }

    feat_colors_value <- NULL
    if (!is.null(color_by) && is.numeric(data[[color_by]])) {
        result <- prepare_continuous_color_scale(
            data,
            color_by,
            lower_quantile = lower_quantile,
            upper_quantile = upper_quantile,
            lower_cutoff = lower_cutoff,
            upper_cutoff = upper_cutoff
        )
        data <- result$data
        feat_colors_value <- result$feat_colors_value
    }

    hidata <- NULL
    if (!is.null(highlight)) {
        if (isTRUE(highlight)) {
            hidata <- data
        } else if (is.numeric(highlight)) {
            hidata <- data[highlight, , drop = FALSE]
        } else if (is.character(highlight) && length(highlight) == 1) {
            hidata <- filter(data, !!parse_expr(highlight))
        } else if (is.null(rownames(data))) {
            stop(
                "No row names in the data, please provide a vector of indexes to highlight."
            )
        } else {
            hidata <- data[highlight, , drop = FALSE]
        }
    }

    shape_has_fill <- shape %in% 21:25
    hishape_has_fill <- highlight_shape %in% 21:25

    p <- ggplot(data, aes(x = !!sym(x), y = !!sym(y)))
    mapping <- list()
    point_layer_args <- list(shape = shape)
    if (shape_has_fill) {
        mapping[[length(mapping) + 1]] <- aes(fill = !!sym(color_by))
        if (isTRUE(border_color)) {
            mapping[[length(mapping) + 1]] <- aes(color = !!sym(color_by))
        } else {
            point_layer_args$color <- border_color
        }
        point_layer_args$stroke <- border_size
    } else {
        mapping[[length(mapping) + 1]] <- aes(color = !!sym(color_by))
    }
    if (is.numeric(size_by)) {
        point_layer_args$size <- size_by
    } else {
        mapping[[length(mapping) + 1]] <- aes(size = !!sym(size_by))
    }
    modify_list <- getFromNamespace("modify_list", "ggplot2")
    point_layer_args$mapping <- Reduce(modify_list, mapping)
    point_layer <- do_call(geom_point, point_layer_args)

    p <- p + point_layer
    if (!is.numeric(size_by)) {
        p <- p +
            scale_size_area(max_size = 6, n.breaks = 4) +
            guides(
                size = guide_legend(
                    title = size_name %||% size_by,
                    override.aes = list(fill = "grey30", shape = 21),
                    order = 1
                )
            )
    }
    if (shape_has_fill) {
        if (is.numeric(data[[color_by]])) {
            p <- p +
                scale_fill_gradientn(
                    n.breaks = 5,
                    colors = palette_this(
                        data[[color_by]],
                        palette = palette,
                        palcolor = palcolor,
                        reverse = palreverse,
                        alpha = alpha
                    ),
                    values = if (!is.null(feat_colors_value)) {
                        scales::rescale(feat_colors_value)
                    } else {
                        waiver()
                    },
                    limits = if (!is.null(feat_colors_value)) {
                        range(feat_colors_value)
                    } else {
                        NULL
                    },
                    na.value = "grey80",
                    guide = if (isTRUE(border_color) || isFALSE(color_legend)) {
                        # legend for border color will be added later
                        # so we don't need to show the legend for fill color
                        guide_none()
                    } else {
                        guide_colorbar(
                            title = color_name %||% color_by,
                            frame.colour = "black",
                            ticks.colour = "black",
                            title.hjust = 0,
                            order = 2
                        )
                    }
                )
            if (isTRUE(border_color)) {
                p <- p +
                    scale_color_gradientn(
                        n.breaks = 5,
                        colors = palette_this(
                            palette = palette,
                            palcolor = palcolor,
                            reverse = palreverse
                        ),
                        values = if (!is.null(feat_colors_value)) {
                            scales::rescale(feat_colors_value)
                        } else {
                            waiver()
                        },
                        limits = if (!is.null(feat_colors_value)) {
                            range(feat_colors_value)
                        } else {
                            NULL
                        },
                        na.value = "grey80",
                        guide = if (
                            isTRUE(border_color) && isTRUE(color_legend)
                        ) {
                            guide_colorbar(
                                title = color_name %||% color_by,
                                frame.colour = "black",
                                ticks.colour = "black",
                                title.hjust = 0,
                                order = 2
                            )
                        } else {
                            guide_none()
                        }
                    )
            }
        } else {
            # factor/character
            p <- p +
                scale_fill_manual(
                    values = palette_this(
                        levels(data[[color_by]]),
                        palette = palette,
                        palcolor = palcolor,
                        reverse = palreverse,
                        alpha = alpha
                    ),
                    na.value = "grey80",
                    guide = if (isTRUE(border_color) || isFALSE(color_legend)) {
                        guide_none()
                    } else {
                        guide_legend(
                            title = color_name %||% color_by,
                            override.aes = list(size = 4, alpha = 1),
                            order = 3
                        )
                    }
                )
            if (isTRUE(border_color)) {
                p <- p +
                    scale_color_manual(
                        values = palette_this(
                            levels(data[[color_by]]),
                            palette = palette,
                            palcolor = palcolor,
                            reverse = palreverse
                        ),
                        na.value = "black",
                        guide = if (
                            isTRUE(border_color) && isTRUE(color_legend)
                        ) {
                            guide_legend(
                                title = color_name %||% color_by,
                                order = 3
                            )
                        } else {
                            guide_none()
                        }
                    )
            }
        }
    } else {
        # shape has no fill
        if (is.numeric(data[[color_by]])) {
            p <- p +
                scale_color_gradientn(
                    n.breaks = 5,
                    colors = palette_this(
                        data[[color_by]],
                        palette = palette,
                        palcolor = palcolor,
                        reverse = palreverse,
                        alpha = alpha
                    ),
                    values = if (!is.null(feat_colors_value)) {
                        scales::rescale(feat_colors_value)
                    } else {
                        waiver()
                    },
                    limits = if (!is.null(feat_colors_value)) {
                        range(feat_colors_value)
                    } else {
                        NULL
                    },
                    na.value = "grey80",
                    guide = if (isTRUE(color_legend)) {
                        guide_colorbar(
                            title = color_name %||% color_by,
                            frame.colour = "black",
                            ticks.colour = "black",
                            title.hjust = 0,
                            order = 2
                        )
                    } else {
                        guide_none()
                    }
                )
        } else {
            # factor/character
            p <- p +
                scale_color_manual(
                    values = palette_this(
                        levels(data[[color_by]]),
                        palette = palette,
                        palcolor = palcolor,
                        reverse = palreverse,
                        alpha = alpha
                    ),
                    na.value = "grey80",
                    guide = if (isTRUE(color_legend)) {
                        guide_legend(
                            title = color_name %||% color_by,
                            override.aes = list(size = 4, alpha = 1),
                            order = 3
                        )
                    } else {
                        guide_none()
                    }
                )
        }
    }

    if (!is.null(hidata)) {
        if (hishape_has_fill) {
            p <- p +
                geom_point(
                    data = hidata,
                    shape = highlight_shape,
                    fill = highlight_color,
                    color = "transparent",
                    size = highlight_size,
                    alpha = highlight_alpha
                )
        } else {
            p <- p +
                geom_point(
                    data = hidata,
                    shape = highlight_shape,
                    color = highlight_color,
                    size = highlight_size,
                    alpha = highlight_alpha
                )
        }
    }

    p <- p +
        scale_x_continuous(trans = xtrans) +
        scale_y_continuous(trans = ytrans) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% x,
            y = ylab %||% y
        ) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major = element_line(colour = "grey80", linetype = 2)
        )

    dims <- calculate_plot_dimensions(
        base_height = 5,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = if (!is.null(color_by) && is.factor(data[[color_by]])) {
            nlevels(data[[color_by]])
        } else {
            1
        },
        legend_nchar = if (!is.null(color_by) && is.factor(data[[color_by]])) {
            max(nchar(levels(data[[color_by]])))
        } else {
            5
        }
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction
    )
}


#' Scatter Plot
#'
#' @description
#' Draws a scatter plot with optional size encoding, colour encoding
#' (continuous gradient or discrete palette), point highlighting, and axis
#' transformations. This is the user-facing wrapper around
#' \code{\link{ScatterPlotAtomic}} that adds \code{split_by} support
#' (generating separate sub-plots per group) and combines them via
#' \code{patchwork}.
#'
#' Key features:
#' \itemize{
#'   \item \strong{Variable point size} -- \code{size_by} accepts either a
#'   numeric constant or a column name.
#'   \item \strong{Colour modes} -- numeric \code{color_by} produces a
#'   continuous gradient; factor/character \code{color_by} produces a discrete
#'   palette.
#'   \item \strong{Colour scale trimming} -- \code{lower_quantile} /
#'   \code{upper_quantile} (or explicit \code{lower_cutoff} /
#'   \code{upper_cutoff}) trim/clamp continuous colour scale extremes.
#'   \item \strong{Border modes} -- \code{border_color} can be a constant
#'   colour, \code{TRUE} (track the fill gradient), or omitted.
#'   \item \strong{Point highlighting} -- \code{highlight} accepts indices,
#'   rownames, logical \code{TRUE}, or a string expression.
#'   \item \strong{Axis transformation} -- \code{xtrans} / \code{ytrans}
#'   support log, sqrt, and other scale transformations.
#'   \item \strong{Split sub-plots} -- \code{split_by} produces one scatter
#'   plot per group level, combined into a single \code{patchwork} layout.
#' }
#'
#' @section split_by Workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \strong{Seed validation} -- \code{\link{validate_common_args}()}
#'   sets the random seed for reproducibility.
#'   \item \strong{Theme resolution} -- \code{\link{process_theme}()} resolves
#'   the \code{theme} string or function.
#'   \item \strong{Split column resolution} -- \code{\link{check_columns}()}
#'   validates \code{split_by} (force_factor, allow_multi, concat_multi).
#'   \item \strong{Data splitting} -- unused factor levels are dropped and the
#'   data is split into a named list (preserving factor level order). When
#'   \code{split_by = NULL}, a single-element list named \code{"..."} is used.
#'   \item \strong{Per-split palette / colour} -- \code{\link{check_palette}()}
#'   and \code{\link{check_palcolor}()} resolve per-split palette and colour
#'   overrides.
#'   \item \strong{Per-split legend} -- \code{\link{check_legend}()} resolves
#'   \code{legend.position} and \code{legend.direction} per split.
#'   \item \strong{Per-split title} -- when \code{title} is a function, it
#'   receives the default title (the split level name) and can return a custom
#'   string; otherwise \code{title \%||\% split_level} is used.
#'   \item \strong{Dispatch} -- each split subset is passed to
#'   \code{\link{ScatterPlotAtomic}()}.
#'   \item \strong{Combination} -- \code{\link{combine_plots}()} assembles the
#'   list of plots via \code{patchwork::wrap_plots}, honouring
#'   \code{nrow}/\code{ncol}/\code{byrow}/\code{design}.
#' }
#'
#' @inheritParams common_args
#' @inheritParams ScatterPlotAtomic
#' @param split_by The column(s) to split data by and generate separate scatter
#'   plots for each level. The split column is processed before splitting;
#'   multiple columns are concatenated with \code{split_by_sep}.
#' @param split_by_sep A character string used to concatenate multiple
#'   \code{split_by} column values. Default: \code{"_"}.
#' @param seed The random seed for reproducibility. Passed to
#'   \code{\link{validate_common_args}()}. Default: \code{8525}.
#' @param combine A logical value. If \code{TRUE} (the default), the list of
#'   per-split plots is combined into a single \code{patchwork} object via
#'   \code{\link{combine_plots}()}. If \code{FALSE}, returns the raw list of
#'   \code{ggplot} objects.
#' @param nrow,ncol,byrow Integers controlling the layout of combined plots via
#'   \code{\link{combine_plots}()}. \code{byrow = TRUE} (default) fills the
#'   layout row-wise.
#' @param axes,axis_titles Strings controlling how axes and axis titles are
#'   handled across combined plots. Passed to \code{\link{combine_plots}()}.
#'   See \code{?patchwork::wrap_plots} for options (\code{"keep"},
#'   \code{"collect"}, \code{"collect_x"}, \code{"collect_y"}).
#' @param guides A string controlling guide collection across combined plots.
#'   Passed to \code{\link{combine_plots}()}.
#' @param design A custom layout specification for combined plots. Passed to
#'   \code{\link{combine_plots}()}. When specified, \code{nrow}, \code{ncol},
#'   and \code{byrow} are ignored.
#' @return A \code{ggplot} object (single plot), a \code{patchwork} object
#'   (when \code{combine = TRUE} with \code{split_by}), or a named list of
#'   \code{ggplot} objects (when \code{combine = FALSE}), each with
#'   \code{height} and \code{width} attributes in inches.
#' @export
#' @examples
#' set.seed(8525)
#'
#' data <- data.frame(
#'    x = rnorm(20),
#'    y = rnorm(20),
#'    w = abs(rnorm(20)),
#'    t = sample(c("A", "B"), 20, replace = TRUE)
#' )
#'
#' # --- Basic scatter plot ---
#' ScatterPlot(data, x = "x", y = "y")
#'
#' # --- Highlight points ---
#' ScatterPlot(data, x = "x", y = "y", highlight = 'x > 0')
#'
#' # --- Size encoding (column name) ---
#' ScatterPlot(data, x = "x", y = "y", size_by = "w")
#'
#' # --- Colour encoding (numeric gradient) ---
#' ScatterPlot(data, x = "x", y = "y", color_by = "w")
#'
#' # --- Colour encoding (categorical) with border ---
#' ScatterPlot(data, x = "x", y = "y", size_by = "w", color_by = "t",
#'  border_color = "red")
#'
#' # --- Border colour tracks fill gradient ---
#' ScatterPlot(data, x = "x", y = "y", size_by = "w", color_by = "t",
#'  border_color = TRUE)
#'
#' # --- Shape without fill (single colour aesthetic) ---
#' ScatterPlot(data, x = "x", y = "y", size_by = "w", color_by = "t",
#'  shape = 1, palette = "Set1")
#'
#' # --- split_by with per-split palcolor ---
#' ScatterPlot(data, x = "x", y = "y", split_by = "t",
#'             palcolor = list(A = "blue", B = "red"))
#'
#' # --- Colour scale limits (quantile-based) ---
#' ScatterPlot(data, x = "x", y = "y", color_by = "w",
#'             lower_quantile = 0.1, upper_quantile = 0.9)
#'
#' # --- Colour scale limits (explicit cutoffs) ---
#' ScatterPlot(data, x = "x", y = "y", color_by = "w",
#'             lower_cutoff = 0, upper_cutoff = 1)
ScatterPlot <- function(
    data,
    x,
    y,
    size_by = 2,
    size_name = NULL,
    color_by = NULL,
    color_name = NULL,
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    palreverse = FALSE,
    split_by = NULL,
    split_by_sep = "_",
    shape = 21,
    alpha = ifelse(shape %in% 21:25, 0.65, 1),
    border_color = "black",
    border_size = 0.5,
    highlight = NULL,
    highlight_shape = 16,
    highlight_size = 3,
    highlight_color = "red",
    highlight_alpha = 1,
    theme = "theme_this",
    theme_args = list(),
    palette = ifelse(
        !is.null(color_by) && !is.numeric(data[[color_by]]),
        "Paired",
        "Spectral"
    ),
    palcolor = NULL,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    seed = 8525,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    theme <- process_theme(theme)
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            ScatterPlotAtomic(
                datas[[nm]],
                x = x,
                y = y,
                size_by = size_by,
                size_name = size_name,
                color_by = color_by,
                highlight = highlight,
                highlight_shape = highlight_shape,
                highlight_size = highlight_size,
                highlight_color = highlight_color,
                highlight_alpha = highlight_alpha,
                color_name = color_name,
                lower_quantile = lower_quantile,
                upper_quantile = upper_quantile,
                lower_cutoff = lower_cutoff,
                upper_cutoff = upper_cutoff,
                palreverse = palreverse,
                theme = theme,
                theme_args = theme_args,
                alpha = alpha,
                shape = shape,
                border_color = border_color,
                border_size = border_size,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
data <- data.frame(
    x = rnorm(100),
    y = rnorm(100),
    size_col = runif(100, 1, 10),
    color_col = factor(rep(c("A", "B", "C", "D"), 25)),
    color_num = rnorm(100),
    group = factor(rep(c("g1", "g2"), 50))
)

test_that("ScatterPlot returns a ggplot for basic usage", {
    p <- ScatterPlot(data, x = "x", y = "y")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("ScatterPlot works with color_by (discrete)", {
    p <- ScatterPlot(data, x = "x", y = "y", color_by = "color_col")
    expect_s3_class(p, "ggplot")
})

test_that("ScatterPlot works with color_by (continuous)", {
    p <- ScatterPlot(data, x = "x", y = "y", color_by = "color_num")
    expect_s3_class(p, "ggplot")
})

test_that("ScatterPlot works with size_by column", {
    p <- ScatterPlot(data, x = "x", y = "y", size_by = "size_col")
    expect_s3_class(p, "ggplot")
})

test_that("ScatterPlot works with numeric size_by value", {
    p <- ScatterPlot(data, x = "x", y = "y", size_by = 3)
    expect_s3_class(p, "ggplot")
})

test_that("ScatterPlot sets title and labels", {
    p <- ScatterPlot(data, x = "x", y = "y", title = "Scatter", xlab = "X", ylab = "Y")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "Scatter")
    expect_equal(p$labels$x, "X")
    expect_equal(p$labels$y, "Y")
})

test_that("ScatterPlot with split_by returns combined or list", {
    p_combined <- ScatterPlot(data, x = "x", y = "y", split_by = "group", combine = TRUE)
    expect_true(inherits(p_combined, "gg") || inherits(p_combined, "patchwork"))

    p_list <- ScatterPlot(data, x = "x", y = "y", split_by = "group", combine = FALSE)
    expect_true(is.list(p_list))
    expect_length(p_list, 2)
})

test_that("ScatterPlot with multiple split_by columns returns a list with one plot per combination", {
    data2 <- data
    data2$split2 <- factor(rep(c("s1", "s2"), each = 50))
    plots <- suppressMessages(ScatterPlot(data2, x = "x", y = "y",
                                          split_by = c("group", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("ScatterPlot with multiple split_by columns returns combined plot", {
    data2 <- data
    data2$split2 <- factor(rep(c("s1", "s2"), each = 50))
    p <- suppressMessages(ScatterPlot(data2, x = "x", y = "y",
                                      split_by = c("group", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("ScatterPlot works with highlight as index vector", {
    p <- ScatterPlot(data, x = "x", y = "y", highlight = 1:10)
    expect_s3_class(p, "ggplot")
})

test_that("ScatterPlot works with highlight as expression string", {
    p <- ScatterPlot(data, x = "x", y = "y", highlight = "x > 0")
    expect_s3_class(p, "ggplot")
})

test_that("ScatterPlot works with facet_by", {
    p <- ScatterPlot(data, x = "x", y = "y", facet_by = "group")
    expect_s3_class(p, "ggplot")
})

test_that("ScatterPlot works with different shapes", {
    p <- ScatterPlot(data, x = "x", y = "y", shape = 16)
    expect_s3_class(p, "ggplot")
    p2 <- ScatterPlot(data, x = "x", y = "y", shape = 21)
    expect_s3_class(p2, "ggplot")
})

test_that("ScatterPlot width attribute is numeric", {
    p <- ScatterPlot(data, x = "x", y = "y")
    expect_true(is.numeric(attr(p, "width")))
    expect_true(is.numeric(attr(p, "height")))
})

test_that("ScatterPlot works with alpha", {
    p <- ScatterPlot(data, x = "x", y = "y", alpha = 0.5)
    expect_s3_class(p, "ggplot")
})

test_that("ScatterPlot with lower/upper quantile works", {
    p <- ScatterPlot(data, x = "x", y = "y", color_by = "color_num",
        lower_quantile = 0.1, upper_quantile = 0.9)
    expect_s3_class(p, "ggplot")
})

test_that("ScatterPlot with explicit cutoffs works", {
    p <- ScatterPlot(data, x = "x", y = "y", color_by = "color_num",
        lower_cutoff = -1, upper_cutoff = 1)
    expect_s3_class(p, "ggplot")
})

test_that("ScatterPlot quantile/cutoff works with border_color = TRUE", {
    p <- ScatterPlot(data, x = "x", y = "y", color_by = "color_num",
        border_color = TRUE, lower_quantile = 0.05, upper_quantile = 0.95)
    expect_s3_class(p, "ggplot")
})

#' Atomic Box / Violin / Bar / Beeswarm plot (internal)
#'
#' @description
#' Core implementation for drawing box plots, violin plots, bar plots (mean
#' ± error bars), or beeswarm plots.  This is the workhorse behind
#' \code{\link{BoxPlot}}, \code{\link{ViolinPlot}}, and
#' \code{\link{BeeswarmPlot}} — it takes a **single** data frame (no
#' \code{split_by} support) and returns a \code{ggplot} object.
#'
#' The \code{base} parameter selects the primary geometry:
#' \itemize{
#'   \item \code{"box"} — \code{geom_boxplot()}
#'   \item \code{"violin"} — \code{geom_violin()}
#'   \item \code{"bar"} — \code{stat_summary(fun = mean, geom = "col")}
#'         with optional error bars (SEM, SD, or CI).
#'   \item \code{"none"} — no primary geometry (used by
#'         \code{\link{BeeswarmPlot}} to draw beeswarm points alone).
#' }
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{Wide-to-long conversion} — when \code{in_form = "wide"},
#'         data is pivoted via \code{tidyr::pivot_longer()}.  The
#'         \code{keep_na} / \code{keep_empty} lists are filtered to the new
#'         column names.
#'   \item \strong{Column resolution} — \code{x}, \code{y}, \code{group_by},
#'         \code{facet_by}, and \code{paired_by} are validated via
#'         \code{\link{check_columns}}.
#'   \item \strong{NA / empty-level handling} — per-column
#'         \code{keep_empty} settings are extracted for \code{x},
#'         \code{group_by}, and \code{facet_by} independently.
#'   \item \strong{Beeswarm validation} — if \code{add_beeswarm = TRUE},
#'         the \code{ggbeeswarm} package is required.  Beeswarm is disabled
#'         (with a warning) when \code{paired_by} is provided.
#'   \item \strong{Paired data validation} — structural checks ensure each
#'         (\code{x}, \code{paired_by}) combination has exactly 2
#'         observations (one per group when \code{group_by} is present).
#'         Paired observations force \code{add_point = TRUE}.
#'   \item \strong{Summary statistics} — \code{.y_mean} and
#'         \code{.y_median} are pre-computed per (\code{x}, \code{group_by},
#'         \code{facet_by}) for use in trend lines and fill modes.
#'   \item \strong{y-axis limits} — \code{y_max} / \code{y_min} accept
#'         numeric values or quantile notation (\code{"q95"},
#'         \code{"q5"}).  For bar plots, the limit is extended upward by
#'         the error bar extent.
#'   \item \strong{Highlight} — \code{highlight} can be \code{TRUE} (all
#'         points), a numeric index vector, a logical expression string, or
#'         a character vector of row names.
#'   \item \strong{sort_x} — an R expression string (e.g.,
#'         \code{"mean(y)"}) evaluated per x-level to reorder categories.
#'   \item \strong{Flip transformation} — when \code{flip = TRUE}, factor
#'         levels are reversed and \code{aspect.ratio} is inverted.
#'   \item \strong{Base geometry} — the primary geom is added:
#'         \code{geom_boxplot()}, \code{geom_violin()}, or
#'         \code{stat_summary(fun = mean, geom = "col")}.  Error bars
#'         (SEM / SD / CI) are layered on bar plots via a custom
#'         \code{stat_summary(fun.data = ...)}.
#'   \item \strong{Fill mode} — \code{fill_mode} controls colour mapping:
#'         \itemize{
#'           \item \code{"dodge"} — fill by \code{group_by} (discrete).
#'           \item \code{"x"} — fill by x-axis categories (discrete).
#'           \item \code{"mean"} / \code{"median"} — fill by pre-computed
#'                 mean/median (continuous gradient).
#'         }
#'   \item \strong{Box overlay} — when \code{add_box = TRUE} on a non-box
#'         base, a box plot is overlaid via \code{ggnewscale::new_scale_fill()}
#'         with a white fill/black outline.
#'   \item \strong{Statistical comparisons} — two pathways:
#'         \itemize{
#'           \item \strong{Pairwise} (\code{comparisons}) — uses
#'                 \code{ggpubr::geom_pwc()} with automatic or explicit
#'                 comparison pairs.  Data is preprocessed to avoid test
#'                 failures from zero-variance or all-NA groups.
#'           \item \strong{Multiple-group} (\code{multiplegroup_comparisons})
#'                 — uses \code{ggpubr::stat_compare_means()} for omnibus
#'                 tests (e.g., Kruskal-Wallis).
#'         }
#'         After comparison layers are added, \code{y_max_use} is expanded to
#'         accommodate significance brackets.
#'   \item \strong{Points} — jittered points (\code{geom_point()} with
#'         \code{position_jitterdodge}) or beeswarm points
#'         (\code{ggbeeswarm::geom_beeswarm()}).  Paired observations
#'         add connecting lines (\code{geom_line()}) between matched
#'         subjects.
#'   \item \strong{Trend lines} — \code{stat_summary(fun = first)} draws
#'         lines connecting group medians.  When \code{trend_color} is
#'         \code{NULL} and \code{group_by} is present, lines are coloured
#'         per group.
#'   \item \strong{Reference lines} — \code{geom_hline()} at the specified
#'         y-intercept.
#'   \item \strong{Stat summary points} — a custom \code{stat_summary()}
#'         point layer displaying a user-specified summary statistic
#'         (e.g., mean) with a shape legend entry.
#'   \item \strong{Stack layout} — when \code{stack = TRUE}, facets are
#'         arranged with shared strip labels and negative panel spacing for
#'         a compact stacked appearance.
#'   \item \strong{Dimension calculation} — \code{calculate_plot_dimensions()}
#'         accounts for the number of x-levels × dodge groups, flip state,
#'         and stack layout adjustments.  Minimum dimensions are enforced
#'         from label character widths.
#'   \item \strong{Faceting} — \code{\link{facet_plot}()} wraps the result
#'         with the appropriate strip position based on flip/stack state.
#' }
#'
#' @inheritParams common_args
#' @param base A character string specifying the primary plot type:
#'  \code{"box"} (box plot), \code{"violin"} (violin plot), \code{"bar"}
#'  (mean bars with optional error bars), or \code{"none"} (no primary
#'  geometry, used by beeswarm plots).
#' @param x A character string of column name(s) for the x-axis.
#'  Character/factor columns are expected.  Multiple columns are
#'  concatenated with \code{x_sep}.
#' @param x_sep A character string to join multiple \code{x} columns.
#'  Default \code{"_"}.
#' @param y A character string of the numeric column for the y-axis.
#'  Not required when \code{in_form = "wide"} (data values are taken from
#'  the \code{x} columns).
#' @param in_form A character string: \code{"long"} (default) or
#'  \code{"wide"}.  In wide form, \code{x} columns are pivoted to long
#'  format.
#' @param sort_x An R expression string (e.g., \code{"mean(y)"}) to order
#'  x-axis categories.  Default \code{NULL} keeps the original order.
#'  When \code{keep_empty_x} is \code{TRUE}, empty levels are placed last.
#' @param flip Logical; if \code{TRUE}, swap the x and y axes.
#' @param group_by A character vector of column name(s) to dodge the
#'  boxes/violins by.  Multiple columns are concatenated with
#'  \code{group_by_sep}.
#' @param group_by_sep A character string to separate concatenated
#'  \code{group_by} columns.  Default \code{"_"}.
#' @param group_name A character string for the dodge legend title.
#' @param paired_by A character string naming a column that identifies
#'  paired observations.  Forces \code{add_point = TRUE} and connects
#'  paired observations with lines.
#' @param fill_mode A character string controlling fill colour mapping:
#'  \code{"dodge"} (fill by \code{group_by}, discrete),
#'  \code{"x"} (fill by x-axis categories, discrete),
#'  \code{"mean"} or \code{"median"} (fill by pre-computed statistic,
#'  continuous gradient).
#' @param position_dodge_preserve Passed to
#'  \code{\link[ggplot2]{position_dodge}()}: \code{"total"} preserves the
#'  overall group width; \code{"single"} preserves individual element width.
#' @param add_point Logical; add jittered or beeswarm points to the plot.
#' @param pt_color Colour of the points.  When \code{add_beeswarm = TRUE}
#'  and \code{pt_color} is \code{NULL}, points are coloured by the fill
#'  variable.
#' @param pt_size Numeric size of the points.  Default computed from
#'  data size: \code{min(3000 / nrow(data), 0.6)}.
#' @param pt_alpha Numeric transparency of the points.
#' @param jitter_width Numeric width of the jitter.  Defaults to
#'  \code{0.5}, but set to \code{0} when \code{paired_by} is provided.
#' @param jitter_height Numeric height of the jitter.  Default \code{0}.
#' @param add_beeswarm Logical; use \code{ggbeeswarm::geom_beeswarm()} for
#'  non-overlapping point layout instead of jitter.  Requires the
#'  \code{ggbeeswarm} package.
#' @param beeswarm_method Beeswarm layout method: \code{"swarm"},
#'  \code{"compactswarm"}, \code{"hex"}, \code{"square"}, or
#'  \code{"center"}.
#' @param beeswarm_cex Numeric scaling for point spacing.  Larger values
#'  spread points more.
#' @param beeswarm_priority Point layout priority: \code{"ascending"},
#'  \code{"descending"}, \code{"density"}, or \code{"random"}.
#' @param beeswarm_dodge Numeric dodge width for beeswarm points when
#'  \code{group_by} is provided.  Default \code{0.9}.
#' @param add_box Logical; overlay a box plot on the primary geometry.
#'  Mutually exclusive with \code{base = "box"} and \code{base = "bar"}.
#' @param box_color Colour of the overlaid box plot outline and fill.
#' @param box_width Width of the overlaid box plot.
#' @param box_ptsize Size of the median point in the overlaid box plot.
#' @param add_errorbar Type of error bars for bar plots (\code{base = "bar"}):
#'  \code{"SEM"} (standard error of the mean, default), \code{"SD"}
#'  (standard deviation), \code{"CI"} or \code{"CI95"} (95\% confidence
#'  interval), or \code{"none"}.  Silently ignored for non-bar bases.
#' @param errorbar_color Colour of the error bar lines and caps.
#' @param errorbar_width Width of the error bar caps.
#' @param errorbar_linewidth Line width of the error bars.
#' @param add_trend Logical; add trend lines connecting group medians.
#' @param trend_color Colour of the trend line.  When \code{NULL} and
#'  \code{group_by} is present, lines are coloured per group.
#' @param trend_linewidth Width of the trend line.
#' @param trend_ptsize Size of the trend line points.
#' @param add_stat A summary function (e.g., \code{mean}, \code{median}) to
#'  display as a point with a shape legend entry.
#' @param stat_name Legend title for the stat summary shape.
#' @param stat_color Colour of the stat summary point.
#' @param stat_size Size of the stat summary point.
#' @param stat_stroke Stroke width of the stat summary point.
#' @param stat_shape Shape (an integer) for the stat summary point.  Uses
#'  \code{scale_shape_identity()} so the shape is rendered directly.
#' @param add_bg Logical; add alternating background stripes.
#' @param bg_palette Palette for the background stripes.
#' @param bg_palcolor Custom colours for the background stripes.
#' @param bg_alpha Alpha transparency for the background stripes.
#' @param add_line A numeric y-intercept for a horizontal reference line.
#' @param line_color Colour of the reference line.
#' @param line_width Width of the reference line.
#' @param line_type Linetype of the reference line.
#' @param highlight A specification of points to highlight: \code{TRUE}
#'  (all), a numeric index vector, a logical expression string, or a
#'  character vector of row names.
#' @param highlight_color Colour of highlighted points.
#' @param highlight_size Size of highlighted points.
#' @param highlight_alpha Alpha of highlighted points.
#' @param comparisons A logical value (\code{TRUE} for all pairs) or a list
#'  of two-element vectors specifying pairwise comparisons.  Only available
#'  when \code{fill_mode = "dodge"} (i.e., \code{group_by} is present).
#' @param ref_group A character string specifying the reference group for
#'  comparisons.
#' @param pairwise_method Method for pairwise tests.  Default
#'  \code{"wilcox.test"}.
#' @param multiplegroup_comparisons Logical; perform an omnibus test
#'  (e.g., Kruskal-Wallis) across all groups.
#' @param multiple_method Method for the omnibus test.  Default
#'  \code{"kruskal.test"}.
#' @param sig_label Label format for significance annotations.  For
#'  pairwise comparisons: \code{"p.format"}, \code{"p.signif"}, or a
#'  \pkg{glue} template (e.g., \code{"p = {p}"}).  For multiple-group
#'  tests: \code{"p.format"} or \code{"p.signif"}.
#' @param sig_labelsize Size of the significance label text.
#' @param hide_ns Logical; hide non-significant comparison labels.
#' @param symnum_args A list of arguments passed to
#'  \code{\link[stats]{symnum}} for symbolic p-value coding.
#' @param step_increase Fractional step increase for stacking significance
#'  brackets when multiple comparisons exist.
#' @param stack Logical; stack facetted panels in a compact layout with
#'  shared strip labels.
#' @param y_max,y_min Numeric y-axis limits, or quantile notation strings
#'  (e.g., \code{"q95"} for the 95th percentile, \code{"q5"} for the
#'  5th percentile).
#' @param y_trans A character string for y-axis transformation
#'  (e.g., \code{"log10"}).
#' @param y_nbreaks Integer number of y-axis breaks.
#' @return A \code{ggplot} object, possibly faceted, with \code{height}
#'  and \code{width} attributes (in inches) attached.
#' @param y_brackets Numeric y-axis position for significance brackets
#'  (or p-value labels for multiple comparisons).  If NULL, the brackets are placed above the maximum y-value.
#' @keywords internal
#' @importFrom utils combn
#' @importFrom stats median quantile sd qt
#' @importFrom rlang sym syms parse_expr
#' @importFrom dplyr mutate ungroup first
#' @importFrom ggplot2 geom_boxplot geom_violin geom_point geom_line geom_hline geom_vline layer_data geom_col geom_errorbar
#' @importFrom ggplot2 scale_fill_manual scale_color_manual scale_shape_manual scale_linetype_manual stat_summary
#' @importFrom ggplot2 labs theme element_line element_text position_dodge position_jitter coord_flip layer_scales
#' @importFrom ggplot2 position_jitterdodge scale_shape_identity scale_size_manual scale_alpha_manual scale_y_continuous
BoxViolinPlotAtomic <- function(
    data,
    x,
    x_sep = "_",
    y = NULL,
    base = c("box", "violin", "bar", "none"),
    in_form = c("long", "wide"),
    sort_x = NULL,
    flip = FALSE,
    keep_empty = FALSE,
    keep_na = FALSE,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    paired_by = NULL,
    x_text_angle = ifelse(isTRUE(flip), 0, 45),
    step_increase = 0.1,
    position_dodge_preserve = "total",
    fill_mode = ifelse(!is.null(group_by), "dodge", "x"),
    palreverse = FALSE,
    symnum_args = NULL,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    alpha = 1,
    aspect.ratio = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    add_point = FALSE,
    pt_color = if (isTRUE(add_beeswarm)) NULL else "grey30",
    pt_size = NULL,
    pt_alpha = 1,
    y_nbreaks = 4,
    jitter_width = NULL,
    jitter_height = 0,
    stack = FALSE,
    y_max = NULL,
    y_min = NULL,
    y_trans = "identity",
    y_brackets = NULL,
    add_beeswarm = FALSE,
    beeswarm_method = "swarm",
    beeswarm_cex = 1,
    beeswarm_priority = "ascending",
    beeswarm_dodge = 0.9,
    add_box = FALSE,
    box_color = "black",
    box_width = 0.1,
    box_ptsize = 2.5,
    add_errorbar = "SEM",
    errorbar_color = "grey20",
    errorbar_width = 0.4,
    errorbar_linewidth = 0.6,
    add_trend = FALSE,
    trend_color = NULL,
    trend_linewidth = 1,
    trend_ptsize = 2,
    add_stat = NULL,
    stat_name = NULL,
    stat_color = "black",
    stat_size = 1,
    stat_stroke = 1,
    stat_shape = 25,
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    add_line = NULL,
    line_color = "red2",
    line_width = .6,
    line_type = 2,
    highlight = NULL,
    highlight_color = "red2",
    highlight_size = 1,
    highlight_alpha = 1,
    comparisons = NULL,
    ref_group = NULL,
    pairwise_method = "wilcox.test",
    multiplegroup_comparisons = FALSE,
    multiple_method = "kruskal.test",
    sig_label = "p.format",
    sig_labelsize = 3.5,
    hide_ns = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525,
    ...
) {
    set.seed(seed)
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    in_form <- match.arg(in_form)
    if (in_form == "wide") {
        data <- data %>%
            pivot_longer(cols = x, names_to = ".x", values_to = ".y")
        x <- ".x"
        y <- ".y"
        # if all values in keep_na are FALSE that means it is default
        if (is.list(keep_na) && all(sapply(keep_na, isFALSE))) {
            keep_na <- NULL
        } else if (
            is.list(keep_na) && length(setdiff(names(keep_na), c(x, y))) > 0
        ) {
            warning(
                "[Box/Violin/BeeswarmPlot] Ignoring `keep_na` for columns other than `.x` and `.y` when `in_form` is 'wide'."
            )
            keep_na <- keep_na[names(keep_na) %in% c(x, y)]
        }
        if (is.list(keep_empty) && all(sapply(keep_empty, isFALSE))) {
            keep_empty <- NULL
        } else if (
            is.list(keep_empty) && length(setdiff(names(keep_empty), x)) > 0
        ) {
            warning(
                "[Box/Violin/BeeswarmPlot] Ignoring `keep_empty` for columns other than `.x` when `in_form` is 'wide'."
            )
            keep_empty <- keep_empty[names(keep_empty) %in% x]
        }
    }
    x <- check_columns(
        data,
        x,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = x_sep
    )
    y <- check_columns(data, y)

    group_by <- check_columns(
        data,
        group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = group_by_sep
    )
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )
    paired_by <- check_columns(data, paired_by, force_factor = TRUE)
    base_size <- theme_args$base_size %||% 12
    sig_labelsize <- sig_labelsize * base_size / 12

    data <- process_keep_na_empty(data, keep_na, keep_empty)
    keep_empty_x <- keep_empty[[x]]
    keep_empty_group <- if (!is.null(group_by)) keep_empty[[group_by]] else NULL
    keep_empty_facet <- if (!is.null(facet_by)) {
        keep_empty[[facet_by[1]]]
    } else {
        NULL
    }
    if (length(facet_by) > 1) {
        stopifnot(
            "[Box/Violin/BeeswarmPlot] `keep_empty` for `facet_by` variables must be identical." = identical(
                keep_empty_facet,
                keep_empty[[facet_by[2]]]
            )
        )
    }

    # Validate beeswarm parameters
    if (isTRUE(add_beeswarm)) {
        if (!requireNamespace("ggbeeswarm", quietly = TRUE)) {
            stop(
                "Package 'ggbeeswarm' is required for beeswarm plots. Please install it with: install.packages('ggbeeswarm')"
            )
        }
        add_point <- TRUE
        if (!is.null(paired_by)) {
            warning(
                "'add_beeswarm' is not fully compatible with 'paired_by'. Using jittered points instead for paired data."
            )
            add_beeswarm <- FALSE
        }
    }

    if (!is.null(paired_by)) {
        if (!isTRUE(add_point)) {
            warning("Forcing 'add_point' = TRUE when 'paired_by' is provided.")
            add_point <- TRUE
        }

        if (any(is.na(data[[paired_by]]))) {
            warning(
                "'paired_by' contains missing values, removing corresponding rows."
            )
            data <- data[!is.na(data[[paired_by]]), , drop = FALSE]
        }
        n_total_col <- paste0(".n_total_", paired_by)
        sym_ntc <- sym(n_total_col)
        if (!is.null(group_by)) {
            # We should have exactly two groups for each x value
            # and for a pair, the two observations must belong to different groups
            # and the same paired_by value
            problem_groups <- data %>%
                dplyr::group_by(!!!syms(c(x, paired_by, group_by))) %>%
                dplyr::summarise(.n = dplyr::n(), .groups = "drop") %>%
                dplyr::add_count(
                    !!!syms(c(x, paired_by)),
                    name = n_total_col
                ) %>%
                dplyr::filter(!!sym(".n") != 1 | !!sym_ntc != 2) %>%
                dplyr::mutate(
                    .n = ifelse(
                        !!sym(".n") == 1,
                        !!sym(".n"),
                        paste0(!!sym(".n"), " (expecting 1)")
                    ),
                    !!sym_ntc := ifelse(
                        !!sym_ntc == 2,
                        !!sym_ntc,
                        paste0(!!sym_ntc, " (expecting 2)")
                    )
                )
            # If not, indicate which group (x, paired_by) has the problem
            if (nrow(problem_groups) > 0) {
                stop(
                    "When 'paired_by' and 'group_by' are both provided, each combination of 'x' and 'paired_by' must have exactly two observations, one for each group in 'group_by'. The following combinations do not satisfy this requirement:\n",
                    paste0(
                        apply(
                            problem_groups[, c(
                                x,
                                paired_by,
                                group_by,
                                ".n",
                                n_total_col
                            )],
                            1,
                            function(row) {
                                paste(
                                    paste(names(row), row, sep = "="),
                                    collapse = ", "
                                )
                            }
                        ),
                        collapse = "\n"
                    )
                )
            }
        } else if (dplyr::n_distinct(data[[x]], na.rm = TRUE) != 2) {
            stop(
                "Exactly two unique values of 'x' are required when 'paired_by' is provided without 'group_by'."
            )
        } else {
            problem_groups <- data %>%
                dplyr::group_by(!!!syms(c(x, paired_by))) %>%
                dplyr::summarise(.n = dplyr::n(), .groups = "drop") %>%
                dplyr::add_count(!!!syms(paired_by), name = n_total_col) %>%
                dplyr::filter(!!sym(".n") != 1 | !!sym_ntc != 2) %>%
                dplyr::mutate(
                    .n = ifelse(
                        !!sym(".n") == 1,
                        !!sym(".n"),
                        paste0(!!sym(".n"), " (expecting 1)")
                    ),
                    !!sym_ntc := ifelse(
                        !!sym_ntc == 2,
                        !!sym_ntc,
                        paste0(!!sym_ntc, " (expecting 2)")
                    )
                )
            if (nrow(problem_groups) > 0) {
                stop(
                    "When 'paired_by' is provided without 'group_by', each combination of 'x' and 'paired_by' must have exactly two observations, one for each value of 'x'. The following combinations do not satisfy this requirement:\n",
                    paste0(
                        apply(
                            problem_groups[, c(
                                x,
                                paired_by,
                                ".n",
                                n_total_col
                            )],
                            1,
                            function(row) {
                                paste(
                                    paste(names(row), row, sep = "="),
                                    collapse = ", "
                                )
                            }
                        ),
                        collapse = "\n"
                    )
                )
            }
        }

        # For paired tests, ensure data is sorted by paired_by so that
        # corresponding observations across groups are in the same order
        data <- data %>%
            dplyr::arrange(!!!syms(unique(c(paired_by, x, group_by))))
    }
    if (isTRUE(comparisons) && is.null(group_by)) {
        # stop("'group_by' must be provided to when 'comparisons' is TRUE.")
        if (length(levels(data[[x]])) < 2) {
            comparisons <- c()
        } else {
            comparisons <- combn(levels(data[[x]]), 2, simplify = FALSE)
        }
    }
    if (length(comparisons) > 0) {
        if (!is.list(comparisons) && !isTRUE(comparisons)) {
            comparisons <- list(comparisons)
        }
        ncomp <- sapply(comparisons, length)
        if (any(ncomp) > 2) {
            stop(
                "'comparisons' must be a list in which all elements must be vectors of length 2"
            )
        }
    }
    if (!isFALSE(multiplegroup_comparisons)) {
        stopifnot(
            "'sig_label' must be 'p.format' or 'p.signif' when 'multiplegroup_comparisons' is TRUE." = sig_label %in%
                c("p.format", "p.signif")
        )
    }

    orig_data <- data
    data <- data %>%
        dplyr::group_by(!!!syms(unique(c(x, group_by, facet_by)))) %>%
        mutate(.y_mean = mean(!!sym(y)), .y_median = median(!!sym(y))) %>%
        ungroup()

    # keep the factor levels
    for (col in unique(c(x, group_by, facet_by))) {
        data[[col]] <- factor(data[[col]], levels = levels(orig_data[[col]]))
    }
    rm(orig_data)

    values <- data[[y]][is.finite(data[[y]])]
    if (is.character(y_max)) {
        q_max <- as.numeric(sub("(^q)(\\d+)", "\\2", y_max)) / 100
        y_max_use <- quantile(values, q_max, na.rm = TRUE)
    } else {
        y_max_use <- max(values, na.rm = TRUE)
    }
    if (is.null(y_min)) {
        y_min_use <- min(values, na.rm = TRUE)
    } else if (is.character(y_min)) {
        q_min <- as.numeric(sub("(^q)(\\d+)", "\\2", y_min)) / 100
        y_min_use <- quantile(values, q_min, na.rm = TRUE)
    } else {
        y_min_use <- y_min
    }
    rm(values)

    if (!is.null(highlight)) {
        if (isTRUE(highlight)) {
            data$.highlight <- TRUE
        } else if (is.numeric(highlight)) {
            data$.highlight <- 1:nrow(data) %in% highlight
        } else if (is.character(highlight) && length(highlight) == 1) {
            data <- mutate(data, .highlight = !!parse_expr(highlight))
        } else if (is.null(rownames(data))) {
            stop(
                "No row names in the data, please provide a vector of indexes to highlight."
            )
        } else {
            data$.highlight <- rownames(data) %in% highlight
        }
        if (isFALSE(add_point)) {
            warning("Forcing add_point = TRUE when highlight is provided.")
            add_point <- TRUE
        }
    } else {
        data$.highlight <- FALSE
    }
    data$.highlight <- factor(
        as.character(data$.highlight),
        levels = c("TRUE", "FALSE")
    )

    if (!is.null(sort_x)) {
        x_levels <- data %>%
            dplyr::group_by(!!sym(x)) %>%
            dplyr::summarise(
                .sort_x = !!rlang::parse_expr(sort_x),
                .groups = "drop"
            ) %>%
            dplyr::arrange(!!sym(".sort_x")) %>%
            dplyr::pull(!!sym(x)) %>%
            as.character()

        if (!isFALSE(keep_empty_x)) {
            x_levels <- c(x_levels, setdiff(levels(data[[x]]), x_levels))
        }
        data[[x]] <- factor(data[[x]], levels = x_levels)
    }

    if (isTRUE(flip)) {
        data[[x]] <- factor(data[[x]], levels = rev(levels(data[[x]])))
        aspect.ratio <- 1 / aspect.ratio
        if (length(aspect.ratio) == 0 || is.na(aspect.ratio)) {
            aspect.ratio <- NULL
        }
    }

    base <- match.arg(base)
    if (isTRUE(add_box) && base == "box") {
        stop("Cannot add box plot to box plot.")
    }
    if (isTRUE(add_box) && base == "bar") {
        stop("Cannot add box plot to bar plot.")
    }

    # Validate and parse add_errorbar
    add_errorbar <- toupper(as.character(add_errorbar))
    if (add_errorbar != "NONE" && base != "bar") {
        # Silently ignore add_errorbar for non-bar bases
        add_errorbar <- "NONE"
    }
    if (base == "bar" && add_errorbar != "NONE") {
        if (add_errorbar == "CI") {
            add_errorbar <- "CI95"
        }
        if (
            !add_errorbar %in% c("SEM", "SD") &&
                !grepl("^CI\\d+$", add_errorbar)
        ) {
            stop(
                "'add_errorbar' must be one of 'SEM', 'SD', 'CI', 'CIXX' (e.g., 'CI95'), or 'none'. Got: '",
                add_errorbar,
                "'."
            )
        }
    }

    # For bar plots, adjust y_max_use/y_min_use based on mean + errorbar extent
    if (base == "bar" && !isTRUE(add_point)) {
        grp_cols <- unique(c(x, group_by, facet_by))
        grp_stats <- data %>%
            dplyr::group_by(!!!syms(grp_cols)) %>%
            dplyr::summarise(
                .mean = mean(!!sym(y), na.rm = TRUE),
                .sd = sd(!!sym(y), na.rm = TRUE),
                .n = dplyr::n(),
                .groups = "drop"
            )
        if (add_errorbar == "NONE") {
            bar_max <- max(grp_stats$.mean, na.rm = TRUE)
        } else if (add_errorbar == "SEM") {
            bar_max <- max(
                grp_stats$.mean + grp_stats$.sd / sqrt(grp_stats$.n),
                na.rm = TRUE
            )
        } else if (add_errorbar == "SD") {
            bar_max <- max(grp_stats$.mean + grp_stats$.sd, na.rm = TRUE)
        } else {
            ci_level <- as.numeric(sub("^CI", "", add_errorbar)) / 100
            bar_max <- max(
                grp_stats$.mean +
                    qt((1 + ci_level) / 2, df = pmax(grp_stats$.n - 1, 1)) *
                        grp_stats$.sd /
                        sqrt(grp_stats$.n),
                na.rm = TRUE
            )
        }
        if (is.null(y_max) || is.character(y_max)) {
            y_max_use <- bar_max
        }
        if (is.null(y_min)) {
            y_min_use <- 0
        }
    }

    fill_mode <- match.arg(fill_mode, c("dodge", "x", "mean", "median"))
    if (fill_mode == "dodge") {
        fill_by <- group_by
    } else if (fill_mode == "x") {
        fill_by <- x
    } else if (fill_mode == "mean") {
        fill_by <- ".y_mean"
    } else {
        fill_by <- ".y_median"
    }
    p <- ggplot(data, aes(x = !!sym(x), y = !!sym(y), fill = !!sym(fill_by)))
    if (isTRUE(add_bg)) {
        p <- p +
            bg_layer(
                data,
                x,
                isTRUE(keep_empty_x),
                bg_palette,
                bg_palcolor,
                bg_alpha,
                facet_by
            )
    }

    if (base == "box" || (base == "none" && isTRUE(add_box))) {
        p <- p +
            geom_boxplot(
                position = position_dodge(
                    width = 0.9,
                    preserve = position_dodge_preserve
                ),
                color = "black",
                width = 0.8,
                outlier.shape = NA,
                show.legend = TRUE
            )
    } else if (base == "violin") {
        p <- p +
            geom_violin(
                # There is a bug in ggplot2 with preserve = "single" for violin plots
                # See https://github.com/tidyverse/ggplot2/issues/2801
                # There is a fix but not yet released
                position = position_dodge(
                    width = 0.9,
                    preserve = position_dodge_preserve
                ),
                scale = "width",
                trim = TRUE,
                alpha = alpha,
                width = 0.8,
                show.legend = TRUE
            )
    } else if (base == "bar") {
        p <- p +
            stat_summary(
                fun = mean,
                geom = "col",
                position = position_dodge(
                    width = 0.9,
                    preserve = position_dodge_preserve
                ),
                width = 0.8,
                alpha = alpha,
                color = "black",
                show.legend = TRUE
            )
        if (add_errorbar != "NONE") {
            errorbar_fun <- function(y) {
                y <- y[!is.na(y)]
                m <- mean(y)
                n <- length(y)
                s <- sd(y)
                if (n < 2 || is.na(s)) {
                    return(data.frame(y = m, ymin = m, ymax = m))
                }
                if (add_errorbar == "SEM") {
                    se <- s / sqrt(n)
                    data.frame(y = m, ymin = m - se, ymax = m + se)
                } else if (add_errorbar == "SD") {
                    data.frame(y = m, ymin = m - s, ymax = m + s)
                } else {
                    # CI: extract level from e.g. "CI95"
                    ci_level <- as.numeric(sub("^CI", "", add_errorbar)) / 100
                    t_crit <- qt((1 + ci_level) / 2, df = n - 1)
                    me <- t_crit * s / sqrt(n)
                    data.frame(y = m, ymin = m - me, ymax = m + me)
                }
            }
            p <- p +
                stat_summary(
                    fun.data = errorbar_fun,
                    geom = "errorbar",
                    position = position_dodge(
                        width = 0.9,
                        preserve = position_dodge_preserve
                    ),
                    width = errorbar_width,
                    color = errorbar_color,
                    linewidth = errorbar_linewidth,
                    show.legend = FALSE
                )
        }
    }
    if (fill_mode == "dodge") {
        group_vals <- levels(data[[group_by]])
        if (anyNA(data[[group_by]])) {
            group_vals <- c(group_vals, NA)
        }
        group_colors <- palette_this(
            group_vals,
            palette = palette,
            palcolor = palcolor,
            reverse = palreverse,
            NA_keep = TRUE
        )

        if (isTRUE(keep_empty_group)) {
            p <- p +
                scale_fill_manual(
                    name = group_name %||% group_by,
                    values = group_colors,
                    na.value = group_colors['NA'] %||% "grey80",
                    breaks = group_vals,
                    limits = group_vals,
                    drop = FALSE
                )
        } else {
            p <- p +
                scale_fill_manual(
                    name = group_name %||% group_by,
                    values = group_colors,
                    na.value = group_colors['NA'] %||% "grey80"
                )
        }
    } else if (fill_mode == "x") {
        x_vals <- levels(data[[x]])
        if (anyNA(data[[x]])) {
            x_vals <- c(x_vals, NA)
        }
        x_colors <- palette_this(
            x_vals,
            palette = palette,
            palcolor = palcolor,
            reverse = palreverse,
            NA_keep = TRUE
        )

        if (isTRUE(keep_empty_x)) {
            p <- p +
                scale_fill_manual(
                    name = x,
                    values = x_colors,
                    na.value = x_colors['NA'] %||% "grey80",
                    breaks = x_vals,
                    limits = x_vals,
                    drop = FALSE
                )
        } else {
            p <- p +
                scale_fill_manual(
                    name = x,
                    values = x_colors,
                    na.value = x_colors['NA'] %||% "grey80"
                )
        }
    } else {
        p <- p +
            scale_fill_gradientn(
                name = paste0(y, " (", fill_mode, ")"),
                n.breaks = 3,
                colors = palette_this(
                    palette = palette,
                    palcolor = palcolor,
                    reverse = palreverse
                ),
                na.value = "grey80",
                guide = guide_colorbar(
                    frame.colour = "black",
                    ticks.colour = "black",
                    title.hjust = 0
                )
            )
    }

    # when base is none, boxes are added as base
    if (isTRUE(add_box) && !base %in% c("none", "bar")) {
        p <- p +
            new_scale_fill() +
            geom_boxplot(
                position = position_dodge(
                    width = 0.9,
                    preserve = position_dodge_preserve
                ),
                fill = box_color,
                color = box_color,
                width = box_width,
                show.legend = FALSE,
                outlier.shape = NA
            ) +
            stat_summary(
                fun = first,
                geom = "point",
                mapping = aes(y = !!sym(".y_median")),
                position = position_dodge(
                    width = 0.9,
                    preserve = position_dodge_preserve
                ),
                color = "black",
                fill = "white",
                size = box_ptsize,
                shape = 21
            )
    }

    if (length(comparisons) > 0) {
        if (isTRUE(comparisons)) {
            # group_use <- names(which(rowSums(table(data[[x]], data[[group_by]]) >= 2) >= 2))
            # print(group_use)
            if (any(rowSums(table(data[[x]], data[[group_by]]) >= 2) >= 3)) {
                message(
                    "Detected more than 2 groups. Use multiple_method for comparison"
                )
                # method <- multiple_method
                multiplegroup_comparisons <- TRUE
            } else {
                method <- pairwise_method

                if (!identical(fill_mode, "dodge")) {
                    stop(
                        "`comparisons` can only be used with `fill_mode = 'dodge'`."
                    )
                }

                # Preprocess data to avoid test failures
                # Check each x/facet combination for problematic data
                split_cols <- c(x, y, group_by)
                grouping_vars <- x
                if (!is.null(facet_by)) {
                    split_cols <- c(split_cols, facet_by)
                    grouping_vars <- c(grouping_vars, facet_by)
                }

                # Create grouping key for x and facet combinations
                if (length(grouping_vars) > 1) {
                    split_key <- interaction(
                        data[grouping_vars],
                        drop = TRUE,
                        sep = " // "
                    )
                } else {
                    split_key <- data[[grouping_vars]]
                }

                data_groups <- split(
                    data[, split_cols, drop = FALSE],
                    split_key
                )
                needs_fix <- FALSE

                # Check if any group will cause test failures
                for (group_data in data_groups) {
                    gs <- unique(as.character(group_data[[group_by]]))
                    if (length(gs) >= 2) {
                        yval1 <- group_data[[y]][
                            group_data[[group_by]] == gs[1]
                        ]
                        yval2 <- group_data[[y]][
                            group_data[[group_by]] == gs[2]
                        ]
                        # Check for zero variance or all NA

                        if (
                            all(is.na(yval1)) ||
                                all(is.na(yval2)) ||
                                (length(unique(yval1[!is.na(yval1)])) <= 3 ||
                                    length(unique(yval2[!is.na(yval2)])) <= 3)
                        ) {
                            needs_fix <- TRUE
                            break
                        }
                    }
                }

                pwc_data <- data
                if (needs_fix) {
                    warning(
                        "[Box/Violin/BeeswarmPlot] Some pairwise comparisons may fail due to insufficient data points or variability. Adjusting data to ensure valid comparisons."
                    )

                    # Split by facet if present
                    if (!is.null(facet_by)) {
                        facet_key <- interaction(
                            data[facet_by],
                            drop = TRUE,
                            sep = " // "
                        )
                        facet_splits <- split(
                            data[, split_cols, drop = FALSE],
                            facet_key
                        )
                    } else {
                        facet_splits <- list(data[, split_cols, drop = FALSE])
                    }

                    fixed_data_list <- lapply(
                        facet_splits,
                        function(facet_data) {
                            xdata <- split(facet_data, facet_data[[x]])
                            all_gs <- unique(as.character(facet_data[[
                                group_by
                            ]]))[1:2]

                            for (xval in names(xdata)) {
                                df <- xdata[[xval]]
                                gs <- unique(as.character(df[[group_by]]))

                                if (length(gs) < 2) {
                                    # Create minimal data for both groups
                                    df <- data.frame(
                                        x = xval,
                                        y = c(0, 1),
                                        group_by = all_gs
                                    )
                                    colnames(df) <- c(x, y, group_by)
                                    if (!is.null(facet_by)) {
                                        df[facet_by] <- unique(facet_data[
                                            facet_by
                                        ])
                                    }
                                } else {
                                    yval1 <- df[[y]][df[[group_by]] == gs[1]]
                                    yval2 <- df[[y]][df[[group_by]] == gs[2]]

                                    # Handle all NA cases
                                    if (all(is.na(yval1))) {
                                        yval1 <- c(
                                            0,
                                            rep(NA, length(yval1) - 1)
                                        )
                                    }
                                    if (all(is.na(yval2))) {
                                        yval2 <- c(
                                            1,
                                            rep(NA, length(yval2) - 1)
                                        )
                                    }

                                    # Handle zero variance cases
                                    unique_y1 <- unique(yval1[!is.na(yval1)])
                                    unique_y2 <- unique(yval2[!is.na(yval2)])

                                    if (
                                        length(unique_y1) == 1 &&
                                            length(unique_y2) == 1
                                    ) {
                                        # Both groups have same single value - add minimal relative variance
                                        # Calculate a small epsilon relative to the data scale
                                        all_y <- c(yval1, yval2)
                                        all_y_finite <- all_y[is.finite(all_y)]

                                        if (length(all_y_finite) > 0) {
                                            y_abs <- abs(all_y_finite)
                                            if (max(y_abs) > 0) {
                                                epsilon <- max(y_abs) * 1e-10
                                            } else {
                                                epsilon <- 1e-10
                                            }
                                        } else {
                                            epsilon <- 1e-10
                                        }

                                        # Add variance within each group while maintaining the same mean
                                        # This ensures the test will return p ≈ 1 (no significant difference)
                                        non_na_idx_1 <- which(!is.na(yval1))
                                        non_na_idx_2 <- which(!is.na(yval2))

                                        if (length(non_na_idx_1) >= 2) {
                                            yval1[non_na_idx_1[1]] <- unique_y1[
                                                1
                                            ] -
                                                epsilon
                                            yval1[non_na_idx_1[2]] <- unique_y1[
                                                1
                                            ] +
                                                epsilon
                                        } else if (length(non_na_idx_1) == 1) {
                                            yval1[non_na_idx_1[1]] <- unique_y1[
                                                1
                                            ]
                                        }

                                        if (length(non_na_idx_2) >= 2) {
                                            yval2[non_na_idx_2[1]] <- unique_y2[
                                                1
                                            ] -
                                                epsilon
                                            yval2[non_na_idx_2[2]] <- unique_y2[
                                                1
                                            ] +
                                                epsilon
                                        } else if (length(non_na_idx_2) == 1) {
                                            yval2[non_na_idx_2[1]] <- unique_y2[
                                                1
                                            ]
                                        }
                                    }

                                    df[[y]][df[[group_by]] == gs[1]] <- yval1
                                    df[[y]][df[[group_by]] == gs[2]] <- yval2
                                }
                                xdata[[xval]] <- df
                            }
                            do_call(rbind, xdata)
                        }
                    )
                    pwc_data <- do_call(rbind, fixed_data_list)
                }

                # Now call geom_pwc once with the preprocessed data
                # Add paired test support when paired_by is provided
                pwc_call <- list(
                    data = pwc_data,
                    label = sig_label,
                    label.size = sig_labelsize,
                    y.position = y_brackets %||% y_max_use,
                    step.increase = step_increase,
                    symnum.args = symnum_args,
                    tip.length = 0.03,
                    vjust = 0,
                    ref.group = ref_group,
                    method = method,
                    hide.ns = hide_ns
                )

                # Add paired test parameters if paired_by is provided
                if (!is.null(paired_by)) {
                    pwc_call$method.args <- c(
                        pwc_call$method.args,
                        list(paired = TRUE)
                    )
                }

                p <- p + do_call(ggpubr::geom_pwc, pwc_call)

                y_max_use <- layer_scales(p)$y$range$range[2]
            }
        } else if (!isTRUE(multiplegroup_comparisons)) {
            # if (!is.null(group_by)) {
            #     stop("`comparisons` can only be used when `group_by` is NULL is TRUE.")
            # }
            # Convert comparisons to indices
            comparisons <- lapply(
                comparisons,
                function(el) {
                    if (!is.numeric(el)) {
                        which(levels(data[[x]]) %in% el)
                    } else {
                        el
                    }
                }
            )

            # Preprocess data to avoid test failures (same as above for group_by case)
            split_cols <- if (!is.null(group_by)) c(x, y, group_by) else c(x, y)
            grouping_vars <- x
            if (!is.null(facet_by)) {
                split_cols <- c(split_cols, facet_by)
                grouping_vars <- c(grouping_vars, facet_by)
            }

            # Create grouping key for x and facet combinations
            if (length(grouping_vars) > 1) {
                split_key <- interaction(
                    data[grouping_vars],
                    drop = TRUE,
                    sep = " // "
                )
            } else {
                split_key <- data[[grouping_vars]]
            }

            data_groups <- split(data[, split_cols, drop = FALSE], split_key)
            needs_fix <- FALSE

            # For exact comparisons, we need to check x groups, not group_by groups
            # Check if any x group has zero variance
            for (group_data in data_groups) {
                yval <- group_data[[y]]
                # Check for zero variance or all NA
                if (
                    all(is.na(yval)) || length(unique(yval[!is.na(yval)])) <= 1
                ) {
                    needs_fix <- TRUE
                    break
                }
            }

            pwc_data <- data
            if (needs_fix) {
                warning(
                    "Some pairwise comparisons may fail due to insufficient variability. Adjusting data to ensure valid comparisons."
                )

                # Split by facet if present
                if (!is.null(facet_by)) {
                    facet_key <- interaction(
                        data[facet_by],
                        drop = TRUE,
                        sep = " // "
                    )
                    facet_splits <- split(
                        data[, split_cols, drop = FALSE],
                        facet_key
                    )
                } else {
                    facet_splits <- list(data[, split_cols, drop = FALSE])
                }

                fixed_data_list <- lapply(facet_splits, function(facet_data) {
                    xdata <- split(facet_data, facet_data[[x]])

                    for (xval in names(xdata)) {
                        df <- xdata[[xval]]
                        if (nrow(df) < 2) {
                            xdata[[xval]] <- df
                            next
                        }
                        yval <- df[[y]]

                        # Handle all NA cases
                        if (all(is.na(yval))) {
                            yval <- c(0, 1, rep(NA, length(yval) - 2))
                        }

                        # Handle zero variance cases
                        unique_y <- unique(yval[!is.na(yval)])

                        if (length(unique_y) == 1) {
                            # Single value - add minimal relative variance
                            # Calculate a small epsilon relative to the data scale
                            all_y_finite <- yval[is.finite(yval)]

                            if (length(all_y_finite) > 0) {
                                y_abs <- abs(all_y_finite)
                                if (max(y_abs) > 0) {
                                    epsilon <- max(y_abs) * 1e-10
                                } else {
                                    epsilon <- 1e-10
                                }
                            } else {
                                epsilon <- 1e-10
                            }

                            # Add symmetric variance around the mean
                            non_na_idx <- which(!is.na(yval))
                            if (length(non_na_idx) >= 2) {
                                yval[non_na_idx[1]] <- unique_y[1] - epsilon
                                yval[non_na_idx[2]] <- unique_y[1] + epsilon
                            }
                        }

                        df[[y]] <- yval
                        xdata[[xval]] <- df
                    }
                    do_call(rbind, xdata)
                })
                pwc_data <- do_call(rbind, fixed_data_list)
            }

            # Add paired test support when paired_by is provided
            method_args <- list(comparisons = comparisons)
            if (!is.null(paired_by)) {
                method_args$paired <- TRUE
            }

            p <- p +
                ggpubr::geom_pwc(
                    data = pwc_data,
                    label = sig_label,
                    label.size = sig_labelsize,
                    y.position = y_brackets %||% y_max_use,
                    step.increase = step_increase,
                    symnum.args = symnum_args,
                    tip.length = 0.03,
                    vjust = 0,
                    # comparisons = comparisons,
                    ref.group = ref_group,
                    method = pairwise_method,
                    method.args = method_args,
                    hide.ns = hide_ns
                )
            y_max_use <- layer_scales(p)$y$range$range[1] +
                (layer_scales(p)$y$range$range[2] -
                    layer_scales(p)$y$range$range[1]) *
                    1.15
        }
    }

    if (isTRUE(multiplegroup_comparisons)) {
        p <- p +
            ggpubr::stat_compare_means(
                mapping = if (!is.null(group_by)) {
                    aes(x = !!sym(x), y = !!sym(y), group = !!sym(group_by))
                } else {
                    aes(x = !!sym(x), y = !!sym(y))
                },
                inherit.aes = FALSE,
                method = multiple_method,
                symnum.args = symnum_args,
                label.y = y_brackets %||% y_max_use,
                size = sig_labelsize,
                label = sig_label,
                vjust = -0.5,
                hjust = ifelse(is.null(group_by), 0, 0.5)
            )
        y_max_use <- layer_scales(p)$y$range$range[1] +
            (layer_scales(p)$y$range$range[2] -
                layer_scales(p)$y$range$range[1]) *
                1.15
    }
    if (!is.null(y_max) && is.numeric(y_max)) {
        y_max_use <- max(y_max_use, y_max)
    }

    if (isTRUE(add_point)) {
        if (!is.null(paired_by)) {
            if (is.null(group_by)) {
                p <- p +
                    geom_line(
                        data = data,
                        mapping = aes(
                            x = !!sym(x),
                            y = !!sym(y),
                            group = !!sym(paired_by)
                        ),
                        color = pt_color,
                        alpha = pt_alpha,
                        linewidth = 0.3,
                        inherit.aes = FALSE
                    )
            } else {
                line_data <- data
                # re-calculate x
                # for the first group, x = integer(x) - n
                # for the second group, x = integer(x) + n
                line_data$.xint <- as.numeric(line_data[[x]])
                groups <- levels(line_data[[group_by]])
                line_data$.x <- ifelse(
                    line_data[[group_by]] == groups[1],
                    line_data$.xint - .225, # n = 0.225 = 0.9 / 2 / 2
                    line_data$.xint + .225
                )
                line_data$.line_group <- paste(
                    line_data[[paired_by]],
                    line_data[[x]],
                    sep = " // "
                )
                p <- p +
                    geom_line(
                        data = line_data,
                        mapping = aes(
                            x = !!sym(".x"),
                            y = !!sym(y),
                            group = !!sym(".line_group")
                        ),
                        color = pt_color,
                        alpha = pt_alpha,
                        linewidth = 0.3,
                        inherit.aes = FALSE
                    )
            }
        }

        # Use beeswarm or jittered points
        if (isTRUE(add_beeswarm)) {
            # Use ggbeeswarm for non-overlapping point layout
            if (!is.null(pt_color)) {
                p <- p +
                    ggbeeswarm::geom_beeswarm(
                        color = pt_color,
                        method = beeswarm_method,
                        cex = beeswarm_cex,
                        priority = beeswarm_priority,
                        dodge.width = beeswarm_dodge,
                        show.legend = FALSE
                    )
            } else {
                colors <- palette_this(
                    levels(data[[fill_by]]),
                    palette = palette,
                    palcolor = palcolor,
                    reverse = palreverse
                )
                p <- p +
                    ggbeeswarm::geom_beeswarm(
                        aes(color = !!sym(fill_by)),
                        method = beeswarm_method,
                        cex = beeswarm_cex,
                        priority = beeswarm_priority,
                        dodge.width = beeswarm_dodge
                    ) +
                    scale_color_manual(values = colors, guide = "legend")
            }
            if (any(data$.highlight == "TRUE")) {
                p <- p +
                    ggbeeswarm::geom_beeswarm(
                        data = data[data$.highlight == "TRUE", , drop = FALSE],
                        color = highlight_color,
                        method = beeswarm_method,
                        cex = beeswarm_cex,
                        priority = beeswarm_priority,
                        dodge.width = beeswarm_dodge,
                        show.legend = FALSE
                    ) +
                    scale_size_manual(
                        values = c(
                            "TRUE" = highlight_size,
                            "FALSE" = pt_size %||% min(3000 / nrow(data), 0.6)
                        ),
                        guide = "none"
                    ) +
                    scale_alpha_manual(
                        values = c(
                            "TRUE" = highlight_alpha,
                            "FALSE" = pt_alpha
                        ),
                        guide = "none"
                    )
            }
        } else {
            # Use regular jittered points
            p <- p +
                geom_point(
                    aes(
                        fill = !!sym(fill_by),
                        color = !!sym(".highlight"),
                        size = !!sym(".highlight"),
                        alpha = !!sym(".highlight")
                    ),
                    position = position_jitterdodge(
                        jitter.width = jitter_width %||%
                            ifelse(!is.null(paired_by), 0, 0.5),
                        jitter.height = jitter_height,
                        dodge.width = 0.9,
                        seed = seed
                    ),
                    show.legend = FALSE
                ) +
                scale_color_manual(
                    values = c("TRUE" = highlight_color, "FALSE" = pt_color),
                    guide = "none"
                ) +
                scale_size_manual(
                    values = c(
                        "TRUE" = highlight_size,
                        "FALSE" = pt_size %||% min(3000 / nrow(data), 0.6)
                    ),
                    guide = "none"
                ) +
                scale_alpha_manual(
                    values = c("TRUE" = highlight_alpha, "FALSE" = pt_alpha),
                    guide = "none"
                )
        }
    }

    if (isTRUE(add_trend)) {
        if (is.null(trend_color)) {
            p <- p +
                stat_summary(
                    fun = first,
                    geom = "line",
                    mapping = if (!is.null(group_by)) {
                        aes(
                            y = !!sym(".y_median"),
                            group = !!sym(group_by),
                            color = !!sym(group_by)
                        )
                    } else {
                        aes(y = !!sym(".y_median"), group = 1)
                    },
                    position = position_dodge(
                        width = 0.9,
                        preserve = position_dodge_preserve
                    ),
                    linewidth = trend_linewidth
                )
            if (!is.null(group_by)) {
                group_vals <- levels(data[[group_by]])
                if (anyNA(data[[group_by]])) {
                    group_vals <- c(group_vals, NA)
                }
                group_colors <- palette_this(
                    group_vals,
                    palette = palette,
                    palcolor = palcolor,
                    reverse = palreverse,
                    NA_keep = TRUE
                )

                if (isTRUE(keep_empty_group)) {
                    p <- p +
                        scale_color_manual(
                            values = group_colors,
                            na.value = group_colors['NA'] %||% "grey80",
                            breaks = group_vals,
                            limits = group_vals,
                            drop = FALSE
                        )
                } else {
                    p <- p +
                        scale_color_manual(
                            values = group_colors,
                            na.value = group_colors['NA'] %||% "grey80"
                        )
                }
            }
        } else {
            p <- p +
                stat_summary(
                    fun = first,
                    geom = "line",
                    mapping = if (!is.null(group_by)) {
                        aes(y = !!sym(".y_median"), group = !!sym(group_by))
                    } else {
                        aes(y = !!sym(".y_median"), group = 1)
                    },
                    position = position_dodge(
                        width = 0.9,
                        preserve = position_dodge_preserve
                    ),
                    color = trend_color,
                    linewidth = trend_linewidth
                )
        }

        p <- p +
            stat_summary(
                fun = first,
                geom = "point",
                mapping = if (!is.null(group_by)) {
                    aes(y = !!sym(".y_median"), group = !!sym(group_by))
                } else {
                    aes(y = !!sym(".y_median"), group = 1)
                },
                position = position_dodge(
                    width = 0.9,
                    preserve = position_dodge_preserve
                ),
                color = "black",
                fill = "white",
                size = trend_ptsize,
                shape = 21
            )
    }

    if (!is.null(add_line)) {
        p <- p +
            geom_hline(
                yintercept = add_line,
                color = line_color,
                linetype = line_type,
                linewidth = line_width
            )
    }

    if (!is.null(add_stat)) {
        p <- p +
            stat_summary(
                fun = add_stat,
                geom = "point",
                mapping = if (!is.null(group_by)) {
                    aes(shape = !!sym("stat_shape"), group = !!sym(group_by))
                } else {
                    aes(shape = !!sym("stat_shape"), group = 1)
                },
                position = position_dodge(
                    width = 0.9,
                    preserve = position_dodge_preserve
                ),
                color = stat_color,
                fill = stat_color,
                size = stat_size,
                stroke = stat_stroke,
            ) +
            scale_shape_identity(
                labels = stat_name %||%
                    paste0(y, " (", deparse(substitute(add_stat)), ")"),
                guide = guide_legend(title = "", order = 2)
            )
    }

    just <- calc_just(x_text_angle)
    p <- p +
        scale_x_discrete(drop = !isTRUE(keep_empty_x)) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% x,
            y = ylab %||% y
        )

    if (base == "bar") {
        p <- p +
            scale_y_continuous(
                trans = y_trans,
                n.breaks = y_nbreaks,
                expand = expansion(mult = c(0, 0.05))
            )
    } else {
        p <- p + scale_y_continuous(trans = y_trans, n.breaks = y_nbreaks)
    }

    x_maxchars <- max(nchar(levels(data[[x]])))
    nx <- nlevels(data[[x]])
    nd <- ifelse(is.null(group_by), 1, nlevels(data[[group_by]]))
    facet_free <- !is.null(facet_by) &&
        (identical(facet_scales, "free") ||
            (!flip && identical(facet_scales, "free_y")) ||
            (flip && identical(facet_scales, "free_x")))
    if (isTRUE(flip) && isTRUE(stack)) {
        facet_nrow <- facet_nrow %||% 1
        strip_position <- "top"
        p <- p +
            ggplot2::theme(
                # strip.text.x = element_text(angle = 90),
                panel.grid.major.x = element_line(color = "grey", linetype = 2),
                panel.spacing.x = unit(-1, "pt")
            )
        if (facet_free) {
            p <- p + coord_flip()
        } else {
            p <- p + coord_flip(ylim = c(y_min_use, y_max_use))
        }
    } else if (isTRUE(flip) && isFALSE(stack)) {
        strip_position <- "top"
        p <- p +
            ggplot2::theme(
                strip.text.y = element_text(angle = 0),
                panel.grid.major.x = element_line(color = "grey", linetype = 2),
            )
        if (facet_free) {
            p <- p + coord_flip()
        } else {
            p <- p + coord_flip(ylim = c(y_min_use, y_max_use))
        }
    } else if (isTRUE(stack)) {
        facet_ncol <- facet_ncol %||% 1
        strip_position <- "right"
        p <- p +
            ggplot2::theme(
                panel.spacing.y = unit(-1, "pt"),
                strip.text.y = element_text(angle = 0, hjust = 0),
                panel.grid.major.y = element_line(color = "grey", linetype = 2),
            )
        if (!facet_free) {
            p <- p + coord_cartesian(ylim = c(y_min_use, y_max_use))
        }
    } else {
        strip_position <- "top"
        p <- p +
            ggplot2::theme(
                strip.text.x = element_text(angle = 0),
                panel.grid.major.x = element_line(color = "grey", linetype = 2),
            )
        if (!facet_free) {
            p <- p + coord_cartesian(ylim = c(y_min_use, y_max_use))
        }
    }

    p <- p +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            axis.text.x = element_text(
                angle = x_text_angle,
                hjust = just$h,
                vjust = just$v
            ),
            legend.position = legend.position,
            legend.direction = legend.direction,
        )

    # Dimension calculation
    # When flipped: nx*nd categories land on the visual y-axis (height-driven);
    #               x-axis label chars add to width.
    # When not flipped: nx*nd categories land on the visual x-axis (width-driven);
    #                   x-axis label chars add to height.
    if (isTRUE(flip)) {
        label_min_width <- if (isTRUE(stack)) {
            max(3, 2 + x_maxchars * 0.05)
        } else {
            max(3, 2.2 + x_maxchars * 0.05)
        }
        dims <- calculate_plot_dimensions(
            base_height = label_min_width,
            aspect.ratio = aspect.ratio,
            n_y = nx * nd,
            y_scale_factor = 0.5,
            legend.position = legend.position,
            legend.direction = legend.direction,
            legend_n = nd,
            legend_nchar = if (is.null(group_by)) {
                5
            } else {
                max(nchar(levels(data[[group_by]])))
            },
            flip = TRUE
        )
        height <- dims$height
        width <- max(dims$width, label_min_width)
    } else {
        label_min_height <- if (isTRUE(stack)) {
            4 + x_maxchars * 0.05
        } else {
            2 + x_maxchars * 0.05
        }
        dims <- calculate_plot_dimensions(
            base_height = label_min_height,
            aspect.ratio = aspect.ratio,
            n_x = nx * nd,
            x_scale_factor = 0.5,
            legend.position = legend.position,
            legend.direction = legend.direction,
            legend_n = nd,
            legend_nchar = if (is.null(group_by)) {
                5
            } else {
                max(nchar(levels(data[[group_by]])))
            },
            flip = FALSE
        )
        height <- max(dims$height, label_min_height)
        width <- dims$width
    }

    attr(p, "height") <- height
    attr(p, "width") <- max(width, height)

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        strip.position = strip_position,
        legend.position = legend.position,
        legend.direction = legend.direction,
        drop = !isTRUE(keep_empty_facet)
    )
}

#' Box / Violin / Bar / Beeswarm plot (internal)
#'
#' @description
#' Internal wrapper that handles \code{split_by} processing and dispatches
#' to \code{\link{BoxViolinPlotAtomic}} for each split.  This is the
#' intermediate layer between the three public APIs
#' (\code{\link{BoxPlot}}, \code{\link{ViolinPlot}}, and
#' \code{\link{BeeswarmPlot}}) and the core implementation.
#'
#' @section split_by workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
#'         normalise the \code{keep_na} / \code{keep_empty} arguments.
#'   \item The \code{split_by} column is validated and its NA / empty levels
#'         are processed.  It is then removed from the per-column lists.
#'   \item The data is split by \code{split_by} (preserving level order).
#'         If \code{split_by} is \code{NULL}, the data is wrapped in a
#'         single-element list with name \code{"..."}.
#'   \item Per-split \code{palette}, \code{palcolor},
#'         \code{legend.position}, and \code{legend.direction} are resolved.
#'   \item \code{\link{BoxViolinPlotAtomic}()} is called for each split.
#'         When \code{title} is a function, it receives the split level name
#'         for dynamic titles.
#'   \item Results are combined via \code{\link{combine_plots}()}.
#' }
#'
#' @rdname BoxViolinPlot-internal
#' @inheritParams common_args
#' @inheritParams BoxViolinPlotAtomic
#' @param split_by The column(s) to split the data by for separate sub-plots.
#' @param split_by_sep Separator for concatenated \code{split_by} columns.
#' @param seed A numeric seed for reproducibility.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'  \code{patchwork} object.  When \code{FALSE}, returns a named list of
#'  \code{ggplot} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined layout.
#' @param byrow Logical; fill the combined layout by row (default \code{TRUE}).
#' @param axes,axis_titles Character strings for axis handling in the
#'  combined layout.
#' @param guides Character string for legend collection across panels.
#' @param design A custom layout design for the combined plot.
#' @return A \code{ggplot} object, a \code{patchwork} object, or a named list
#'  of \code{ggplot} objects (when \code{combine = FALSE}), each with
#'  \code{height} and \code{width} attributes in inches.
#' @keywords internal
#' @importFrom rlang %||%
BoxViolinPlot <- function(
    data,
    x,
    x_sep = "_",
    y = NULL,
    base = c("box", "violin", "bar", "none"),
    in_form = c("long", "wide"),
    split_by = NULL,
    split_by_sep = "_",
    symnum_args = NULL,
    sort_x = NULL,
    flip = FALSE,
    keep_empty = FALSE,
    keep_na = FALSE,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    paired_by = NULL,
    x_text_angle = ifelse(isTRUE(flip), 0, 45),
    step_increase = 0.1,
    fill_mode = ifelse(!is.null(group_by), "dodge", "x"),
    palreverse = FALSE,
    position_dodge_preserve = "total",
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    alpha = 1,
    aspect.ratio = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    add_point = FALSE,
    pt_color = if (isTRUE(add_beeswarm)) NULL else "grey30",
    pt_size = NULL,
    pt_alpha = 1,
    jitter_width = NULL,
    jitter_height = 0,
    stack = FALSE,
    y_max = NULL,
    y_min = NULL,
    y_brackets = NULL,
    add_beeswarm = FALSE,
    beeswarm_method = "swarm",
    beeswarm_cex = 1,
    beeswarm_priority = "ascending",
    beeswarm_dodge = 0.9,
    add_box = FALSE,
    box_color = "black",
    box_width = 0.1,
    box_ptsize = 2.5,
    add_errorbar = "SEM",
    errorbar_color = "grey20",
    errorbar_width = 0.4,
    errorbar_linewidth = 0.6,
    add_trend = FALSE,
    trend_color = NULL,
    trend_linewidth = 1,
    trend_ptsize = 2,
    add_stat = NULL,
    stat_name = NULL,
    stat_color = "black",
    stat_size = 1,
    stat_stroke = 1,
    stat_shape = 25,
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    add_line = NULL,
    line_color = "red2",
    line_width = .6,
    line_type = 2,
    highlight = NULL,
    highlight_color = "red2",
    highlight_size = 1,
    highlight_alpha = 1,
    comparisons = NULL,
    ref_group = NULL,
    pairwise_method = "wilcox.test",
    multiplegroup_comparisons = FALSE,
    multiple_method = "kruskal.test",
    sig_label = "p.format",
    sig_labelsize = 3.5,
    hide_ns = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed)
    keep_na <- check_keep_na(
        keep_na,
        c(x, split_by, group_by, paired_by, facet_by)
    )
    keep_empty <- check_keep_empty(
        keep_empty,
        c(x, split_by, group_by, paired_by, facet_by)
    )
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL

        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }

    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    stat_name <- stat_name %||%
        paste0(y, " (", deparse(substitute(add_stat)), ")")
    base <- match.arg(base)

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            BoxViolinPlotAtomic(
                datas[[nm]],
                x = x,
                x_sep = x_sep,
                y = y,
                base = base,
                in_form = in_form,
                sort_x = sort_x,
                flip = flip,
                keep_empty = keep_empty,
                keep_na = keep_na,
                group_by = group_by,
                group_by_sep = group_by_sep,
                group_name = group_name,
                paired_by = paired_by,
                x_text_angle = x_text_angle,
                fill_mode = fill_mode,
                palreverse = palreverse,
                step_increase = step_increase,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                alpha = alpha,
                position_dodge_preserve = position_dodge_preserve,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                add_point = add_point,
                pt_color = pt_color,
                pt_size = pt_size,
                pt_alpha = pt_alpha,
                symnum_args = symnum_args,
                jitter_width = jitter_width,
                jitter_height = jitter_height,
                stack = stack,
                y_max = y_max,
                y_min = y_min,
                y_brackets = y_brackets,
                add_beeswarm = add_beeswarm,
                beeswarm_method = beeswarm_method,
                beeswarm_cex = beeswarm_cex,
                beeswarm_priority = beeswarm_priority,
                beeswarm_dodge = beeswarm_dodge,
                add_box = add_box,
                box_color = box_color,
                box_width = box_width,
                box_ptsize = box_ptsize,
                add_errorbar = add_errorbar,
                errorbar_color = errorbar_color,
                errorbar_width = errorbar_width,
                errorbar_linewidth = errorbar_linewidth,
                add_trend = add_trend,
                trend_color = trend_color,
                trend_linewidth = trend_linewidth,
                trend_ptsize = trend_ptsize,
                add_stat = add_stat,
                stat_name = stat_name,
                stat_color = stat_color,
                stat_size = stat_size,
                stat_stroke = stat_stroke,
                stat_shape = stat_shape,
                add_bg = add_bg,
                bg_palette = bg_palette,
                bg_palcolor = bg_palcolor,
                bg_alpha = bg_alpha,
                add_line = add_line,
                line_color = line_color,
                line_width = line_width,
                line_type = line_type,
                highlight = highlight,
                highlight_color = highlight_color,
                highlight_size = highlight_size,
                highlight_alpha = highlight_alpha,
                comparisons = comparisons,
                ref_group = ref_group,
                pairwise_method = pairwise_method,
                multiplegroup_comparisons = multiplegroup_comparisons,
                multiple_method = multiple_method,
                sig_label = sig_label,
                sig_labelsize = sig_labelsize,
                hide_ns = hide_ns,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                seed = seed,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

#' Box / Violin / Bar / Beeswarm plot
#'
#' @description
#' `BoxPlot` draws box plots or bar plots (mean ± error bars) with extensive
#' customisation options.  Supports jittered or beeswarm points, paired
#' observations with connecting lines, trend lines, statistical test
#' annotations (pairwise or omnibus), background stripes, reference lines,
#' point highlighting, and custom summary statistic overlays.
#'
#' This is the public API — it delegates to \code{\link{BoxViolinPlot}}
#' with \code{base = "box"} or \code{base = "bar"}, which in turn dispatches
#' to \code{\link{BoxViolinPlotAtomic}} for each \code{split_by} level.
#'
#' @section Bar plots (\code{base = "bar"}):
#' When \code{base = "bar"}, bars display group means with optional error
#' bars.  \code{add_errorbar} controls the error bar type:
#' \itemize{
#'   \item \code{"SEM"} (default) — standard error of the mean.
#'   \item \code{"SD"} — standard deviation.
#'   \item \code{"CI"} or \code{"CI95"} — 95\% confidence interval.
#'   \item \code{"none"} — no error bars.
#' }
#' Error bars are computed via a custom \code{stat_summary(fun.data = ...)}
#' that handles per-group mean, SD, and sample size.
#'
#' @rdname boxviolinplot
#' @inheritParams BoxViolinPlot
#' @param base A character string: \code{"box"} (default) or \code{"bar"}.
#'  Bar plots show group means with optional error bars.
#' @param add_errorbar Type of error bars for bar plots.  See Details.
#' @param errorbar_color,errorbar_width,errorbar_linewidth Error bar
#'  appearance controls.
#' @return A \code{ggplot} object, a \code{patchwork} object, or a named list
#'  of \code{ggplot} objects (when \code{combine = FALSE}), each with
#'  \code{height} and \code{width} attributes in inches.
#' @export
#' @examples
#' \donttest{
#' set.seed(8525)
#' data <- data.frame(
#'     x = rep(LETTERS[1:8], each = 40),
#'     y = c(rnorm(160), rnorm(160, mean = 1)),
#'     group1 = sample(c("g1", "g2"), 320, replace = TRUE),
#'     group2 = sample(c("h1", "h2", "h3", "h4"), 320, replace = TRUE)
#' )
#'
#' # Basic box plot
#' BoxPlot(data, x = "x", y = "y")
#'
#' # With beeswarm points
#' BoxPlot(data, x = "x", y = "y", add_beeswarm = TRUE, pt_color = "grey30")
#'
#' # Stacked + flipped + faceted
#' BoxPlot(data,
#'     x = "x", y = "y",
#'     stack = TRUE, flip = TRUE, facet_by = "group1",
#'     add_bg = TRUE, bg_palette = "Paired")
#'
#' # Stacked + flipped + split_by with per-split colours
#' BoxPlot(data,
#'     x = "x", y = "y",
#'     stack = TRUE, flip = TRUE, split_by = "group1",
#'     add_bg = TRUE, bg_palette = "Paired",
#'     palcolor = list(g1 = c("red", "blue"), g2 = c("blue", "red")))
#'
#' # sort_x — order by mean(y)
#' data <- data.frame(
#'   x = factor(rep(LETTERS[1:5], each = 40),
#'      levels = c(LETTERS[1:2], "unused", LETTERS[3:5])),
#'   y = c(rnorm(40, mean = 5), rnorm(40, mean = 4), rnorm(40, mean = 3),
#'      rnorm(40, mean = 2), rnorm(40, mean = 1))
#' )
#' BoxPlot(data, x = "x", y = "y", sort_x = "mean(y)", keep_empty = TRUE)
#' BoxPlot(data, x = "x", y = "y", sort_x = "mean(-y)", keep_empty = TRUE)
#'
#' # Wide-form data
#' data_wide <- data.frame(A = rnorm(100), B = rnorm(100), C = rnorm(100))
#' BoxPlot(data_wide, x = c("A", "B", "C"), in_form = "wide")
#'
#' # Paired observations with connecting lines and paired test
#' paired_data <- data.frame(
#'     subject = rep(paste0("s", 1:10), each = 2),
#'     visit = rep(c("pre", "post"), times = 10),
#'     value = rnorm(20))
#' BoxPlot(paired_data,
#'     x = "visit", y = "value", comparisons = TRUE,
#'     paired_by = "subject", add_point = TRUE)
#'
#' # Paired + grouped
#' paired_group_data <- data.frame(
#'     subject = rep(paste0("s", 1:6), each = 2),
#'     x = rep(c("A", "B"), each = 6),
#'     group = rep(c("before", "after"), times = 6),
#'     value = rnorm(12))
#' BoxPlot(paired_group_data,
#'     x = "x", y = "value",
#'     paired_by = "subject", group_by = "group",
#'     comparisons = TRUE, pt_size = 3, pt_color = "red")
#'
#' # keep_na and keep_empty examples
#' data <- data.frame(
#'     x = factor(rep(c(LETTERS[1:3], NA, LETTERS[5:8]), each = 40),
#'        levels = c(LETTERS[1:8])),
#'     y = c(rnorm(160), rnorm(160, mean = 1)),
#'     group1 = sample(c("g1", "g2"), 320, replace = TRUE),
#'     group2 = factor(sample(c("h1", NA, "h3", "h4"), 320, replace = TRUE),
#'        levels = c("h1", "h2", "h3", "h4")))
#'
#' BoxPlot(data, x = "x", y = "y")
#' BoxPlot(data, x = "x", y = "y", keep_na = TRUE, keep_empty = TRUE)
#' BoxPlot(data, x = "x", y = "y", keep_na = TRUE, keep_empty = TRUE,
#'         facet_by = "group2")
#' BoxPlot(data, x = "x", y = "y", keep_na = TRUE, keep_empty = 'level')
#' BoxPlot(data, x = "x", y = "y", group_by = "group2")
#' BoxPlot(data, x = "x", y = "y", group_by = "group2",
#'         keep_na = TRUE, keep_empty = TRUE)
#' BoxPlot(data, x = "x", y = "y", group_by = "group2",
#'         keep_na = TRUE, keep_empty = 'level')
#'
#' # Per-column keep_na / keep_empty
#' BoxPlot(data, x = "x", y = "y", group_by = "group2",
#'         keep_na = list(x = TRUE, group2 = FALSE),
#'         keep_empty = list(x = FALSE, group2 = TRUE))
#'
#' # Bar plot (base = "bar")
#' data$y <- abs(data$y)
#' BoxPlot(data, x = "x", y = "y", base = "bar")
#' BoxPlot(data, x = "x", y = "y", base = "bar", add_errorbar = "SD")
#' BoxPlot(data, x = "x", y = "y", base = "bar", add_errorbar = "CI95")
#' BoxPlot(data, x = "x", y = "y", base = "bar", add_errorbar = "none")
#' BoxPlot(data, x = "x", y = "y", base = "bar", group_by = "group1")
#' BoxPlot(data, x = "x", y = "y", base = "bar", add_point = TRUE)
#' BoxPlot(data, x = "x", y = "y", base = "bar",
#'         fill_mode = "mean", palette = "Blues")
#' }
BoxPlot <- function(
    data,
    x,
    x_sep = "_",
    y = NULL,
    base = c("box", "bar"),
    in_form = c("long", "wide"),
    split_by = NULL,
    split_by_sep = "_",
    symnum_args = NULL,
    sort_x = NULL,
    flip = FALSE,
    keep_empty = FALSE,
    keep_na = FALSE,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    paired_by = NULL,
    x_text_angle = ifelse(isTRUE(flip), 0, 45),
    step_increase = 0.1,
    fill_mode = ifelse(!is.null(group_by), "dodge", "x"),
    palreverse = FALSE,
    position_dodge_preserve = "total",
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    alpha = 1,
    aspect.ratio = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    add_point = FALSE,
    pt_color = if (isTRUE(add_beeswarm)) NULL else "grey30",
    pt_size = NULL,
    pt_alpha = 1,
    jitter_width = NULL,
    jitter_height = 0,
    stack = FALSE,
    y_max = NULL,
    y_min = NULL,
    y_brackets = NULL,
    add_beeswarm = FALSE,
    beeswarm_method = "swarm",
    beeswarm_cex = 1,
    beeswarm_priority = "ascending",
    beeswarm_dodge = 0.9,
    add_trend = FALSE,
    trend_color = NULL,
    trend_linewidth = 1,
    trend_ptsize = 2,
    add_stat = NULL,
    stat_name = NULL,
    stat_color = "black",
    stat_size = 1,
    stat_stroke = 1,
    stat_shape = 25,
    add_errorbar = "SEM",
    errorbar_color = "grey20",
    errorbar_width = 0.4,
    errorbar_linewidth = 0.6,
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    add_line = NULL,
    line_color = "red2",
    line_width = .6,
    line_type = 2,
    highlight = NULL,
    highlight_color = "red2",
    highlight_size = 1,
    highlight_alpha = 1,
    comparisons = NULL,
    ref_group = NULL,
    pairwise_method = "wilcox.test",
    multiplegroup_comparisons = FALSE,
    multiple_method = "kruskal.test",
    sig_label = "p.format",
    sig_labelsize = 3.5,
    hide_ns = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    ...
) {
    base <- match.arg(base)
    stat_name <- stat_name %||%
        paste0(y, " (", deparse(substitute(add_stat)), ")")
    BoxViolinPlot(
        data = data,
        x = x,
        x_sep = x_sep,
        y = y,
        base = base,
        in_form = in_form,
        split_by = split_by,
        split_by_sep = split_by_sep,
        sort_x = sort_x,
        flip = flip,
        keep_empty = keep_empty,
        keep_na = keep_na,
        group_by = group_by,
        group_by_sep = group_by_sep,
        group_name = group_name,
        paired_by = paired_by,
        x_text_angle = x_text_angle,
        fill_mode = fill_mode,
        palreverse = palreverse,
        step_increase = step_increase,
        theme = theme,
        theme_args = theme_args,
        palette = palette,
        palcolor = palcolor,
        alpha = alpha,
        position_dodge_preserve = position_dodge_preserve,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction,
        add_point = add_point,
        pt_color = pt_color,
        pt_size = pt_size,
        pt_alpha = pt_alpha,
        symnum_args = symnum_args,
        jitter_width = jitter_width,
        jitter_height = jitter_height,
        stack = stack,
        y_max = y_max,
        y_min = y_min,
        y_brackets = y_brackets,
        add_beeswarm = add_beeswarm,
        beeswarm_method = beeswarm_method,
        beeswarm_cex = beeswarm_cex,
        beeswarm_priority = beeswarm_priority,
        beeswarm_dodge = beeswarm_dodge,
        add_trend = add_trend,
        trend_color = trend_color,
        trend_linewidth = trend_linewidth,
        trend_ptsize = trend_ptsize,
        add_stat = add_stat,
        stat_name = stat_name,
        stat_color = stat_color,
        stat_size = stat_size,
        stat_stroke = stat_stroke,
        stat_shape = stat_shape,
        add_errorbar = add_errorbar,
        errorbar_color = errorbar_color,
        errorbar_width = errorbar_width,
        errorbar_linewidth = errorbar_linewidth,
        add_bg = add_bg,
        bg_palette = bg_palette,
        bg_palcolor = bg_palcolor,
        bg_alpha = bg_alpha,
        add_line = add_line,
        line_color = line_color,
        line_width = line_width,
        line_type = line_type,
        highlight = highlight,
        highlight_color = highlight_color,
        highlight_size = highlight_size,
        highlight_alpha = highlight_alpha,
        comparisons = comparisons,
        ref_group = ref_group,
        pairwise_method = pairwise_method,
        multiplegroup_comparisons = multiplegroup_comparisons,
        multiple_method = multiple_method,
        sig_label = sig_label,
        sig_labelsize = sig_labelsize,
        hide_ns = hide_ns,
        facet_by = facet_by,
        facet_scales = facet_scales,
        facet_ncol = facet_ncol,
        facet_nrow = facet_nrow,
        facet_byrow = facet_byrow,
        title = title,
        subtitle = subtitle,
        xlab = xlab,
        ylab = ylab,
        seed = seed,
        combine = combine,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        ...
    )
}

#' @description
#' `ViolinPlot` draws violin plots with extensive customisation options.  Supports jittered
#' or beeswarm points, box plot overlays, trend lines, statistical test
#' annotations, background stripes, reference lines, point highlighting,
#' and custom summary statistic overlays.
#'
#' This is the public API — it delegates to \code{\link{BoxViolinPlot}}
#' with \code{base = "violin"}, which dispatches to
#' \code{\link{BoxViolinPlotAtomic}} for each \code{split_by} level.
#'
#' @rdname boxviolinplot
#' @inheritParams BoxViolinPlot
#' @export
#' @examples
#' \donttest{
#' ViolinPlot(data, x = "x", y = "y")
#' ViolinPlot(data, x = "x", y = "y", add_beeswarm = TRUE, pt_color = "grey30")
#' ViolinPlot(data, x = "x", y = "y", add_box = TRUE)
#' ViolinPlot(data, x = "x", y = "y", add_point = TRUE)
#' ViolinPlot(data, x = "x", y = "y", add_trend = TRUE)
#' ViolinPlot(data, x = "x", y = "y", add_stat = mean)
#' ViolinPlot(data, x = "x", y = "y", add_bg = TRUE)
#' ViolinPlot(data, x = "x", y = "y", add_line = 0)
#'
#' # Grouped
#' ViolinPlot(data, x = "x", y = "y", group_by = "group1")
#'
#' # Grouped + faceted + box overlay
#' ViolinPlot(data,
#'     x = "x", y = "y", group_by = "group1",
#'     facet_by = "group2", add_box = TRUE)
#'
#' # Highlight
#' ViolinPlot(data,
#'     x = "x", y = "y", add_point = TRUE,
#'     highlight = 'group1 == "g1"', alpha = 0.8,
#'     highlight_size = 1.5, pt_size = 1, add_box = TRUE)
#'
#' # Pairwise comparisons with formatted labels
#' if (requireNamespace("ggpubr", quietly = TRUE)) {
#'   # https://github.com/kassambara/ggpubr/issues/751
#'   library(ggpubr)
#'   ViolinPlot(data,
#'     x = "x", y = "y", group_by = "group1",
#'     comparisons = TRUE, sig_label = "p = {p}")
#' }
#'
#' # Explicit comparison list + hide non-significant
#' ViolinPlot(data,
#'     x = "x", y = "y", sig_label = "p.format", hide_ns = TRUE,
#'     facet_by = "group2", comparisons = list(c("D", "E")))
#'
#' # Continuous fill (mean) + omnibus test
#' ViolinPlot(data,
#'     x = "x", y = "y", fill_mode = "mean",
#'     facet_by = "group2", palette = "Blues",
#'     multiplegroup_comparisons = TRUE)
#'
#' # Per-split palettes
#' ViolinPlot(data,
#'     x = "x", y = "y", fill_mode = "mean",
#'     split_by = "group1", palette = c(g1 = "Blues", g2 = "Reds"))
#'
#' # Stacked faceting
#' ViolinPlot(data,
#'     x = "x", y = "y", stack = TRUE,
#'     facet_by = "group2", add_box = TRUE, add_bg = TRUE,
#'     bg_palette = "Paired")
#' }
ViolinPlot <- function(
    data,
    x,
    x_sep = "_",
    y = NULL,
    in_form = c("long", "wide"),
    split_by = NULL,
    split_by_sep = "_",
    symnum_args = NULL,
    sort_x = NULL,
    flip = FALSE,
    keep_empty = FALSE,
    keep_na = FALSE,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    paired_by = NULL,
    x_text_angle = ifelse(isTRUE(flip), 0, 45),
    step_increase = 0.1,
    fill_mode = ifelse(!is.null(group_by), "dodge", "x"),
    palreverse = FALSE,
    position_dodge_preserve = "total",
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    alpha = 1,
    aspect.ratio = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    add_point = FALSE,
    pt_color = if (isTRUE(add_beeswarm)) NULL else "grey30",
    pt_size = NULL,
    pt_alpha = 1,
    jitter_width = NULL,
    jitter_height = 0,
    stack = FALSE,
    y_max = NULL,
    y_min = NULL,
    y_brackets = NULL,
    add_beeswarm = FALSE,
    beeswarm_method = "swarm",
    beeswarm_cex = 1,
    beeswarm_priority = "ascending",
    beeswarm_dodge = 0.9,
    add_box = FALSE,
    box_color = "black",
    box_width = 0.1,
    box_ptsize = 2.5,
    add_trend = FALSE,
    trend_color = NULL,
    trend_linewidth = 1,
    trend_ptsize = 2,
    add_stat = NULL,
    stat_name = NULL,
    stat_color = "black",
    stat_size = 1,
    stat_stroke = 1,
    stat_shape = 25,
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    add_line = NULL,
    line_color = "red2",
    line_width = .6,
    line_type = 2,
    highlight = NULL,
    highlight_color = "red2",
    highlight_size = 1,
    highlight_alpha = 1,
    comparisons = NULL,
    ref_group = NULL,
    pairwise_method = "wilcox.test",
    multiplegroup_comparisons = FALSE,
    multiple_method = "kruskal.test",
    sig_label = "p.format",
    sig_labelsize = 3.5,
    hide_ns = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    ...
) {
    stat_name <- stat_name %||%
        paste0(y, " (", deparse(substitute(add_stat)), ")")
    BoxViolinPlot(
        data = data,
        x = x,
        x_sep = x_sep,
        y = y,
        base = "violin",
        in_form = in_form,
        split_by = split_by,
        split_by_sep = split_by_sep,
        sort_x = sort_x,
        flip = flip,
        keep_empty = keep_empty,
        keep_na = keep_na,
        group_by = group_by,
        group_by_sep = group_by_sep,
        group_name = group_name,
        paired_by = paired_by,
        x_text_angle = x_text_angle,
        fill_mode = fill_mode,
        palreverse = palreverse,
        step_increase = step_increase,
        theme = theme,
        theme_args = theme_args,
        palette = palette,
        palcolor = palcolor,
        alpha = alpha,
        position_dodge_preserve = position_dodge_preserve,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction,
        add_point = add_point,
        pt_color = pt_color,
        pt_size = pt_size,
        pt_alpha = pt_alpha,
        symnum_args = symnum_args,
        jitter_width = jitter_width,
        jitter_height = jitter_height,
        stack = stack,
        y_max = y_max,
        y_min = y_min,
        y_brackets = y_brackets,
        add_beeswarm = add_beeswarm,
        beeswarm_method = beeswarm_method,
        beeswarm_cex = beeswarm_cex,
        beeswarm_priority = beeswarm_priority,
        beeswarm_dodge = beeswarm_dodge,
        add_box = add_box,
        box_color = box_color,
        box_width = box_width,
        box_ptsize = box_ptsize,
        add_trend = add_trend,
        trend_color = trend_color,
        trend_linewidth = trend_linewidth,
        trend_ptsize = trend_ptsize,
        add_stat = add_stat,
        stat_name = stat_name,
        stat_color = stat_color,
        stat_size = stat_size,
        stat_stroke = stat_stroke,
        stat_shape = stat_shape,
        add_bg = add_bg,
        bg_palette = bg_palette,
        bg_palcolor = bg_palcolor,
        bg_alpha = bg_alpha,
        add_line = add_line,
        line_color = line_color,
        line_width = line_width,
        line_type = line_type,
        highlight = highlight,
        highlight_color = highlight_color,
        highlight_size = highlight_size,
        highlight_alpha = highlight_alpha,
        comparisons = comparisons,
        ref_group = ref_group,
        pairwise_method = pairwise_method,
        multiplegroup_comparisons = multiplegroup_comparisons,
        multiple_method = multiple_method,
        sig_label = sig_label,
        sig_labelsize = sig_labelsize,
        hide_ns = hide_ns,
        facet_by = facet_by,
        facet_scales = facet_scales,
        facet_ncol = facet_ncol,
        facet_nrow = facet_nrow,
        facet_byrow = facet_byrow,
        title = title,
        subtitle = subtitle,
        xlab = xlab,
        ylab = ylab,
        seed = seed,
        combine = combine,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        ...
    )
}

#' @description
#' `BeeswarmPlot` draws beeswarm plots — points arranged by the beeswarm algorithm to
#' avoid overlap while displaying the distribution density.  This is a
#' convenience wrapper that delegates to \code{\link{BoxViolinPlot}} with
#' \code{base = "none"} and \code{add_beeswarm = TRUE}.
#'
#' Requires the \pkg{ggbeeswarm} package.  To get a beeswarm plot WITH a
#' box plot, use \code{BeeswarmPlot(..., add_box = TRUE)}.  To get a violin
#' plot with beeswarm points, use \code{ViolinPlot(..., add_beeswarm = TRUE)}.
#'
#' @rdname boxviolinplot
#' @inheritParams BoxViolinPlot
#' @param add_violin Logical; whether to add a violin plot behind the
#'  beeswarm points.  **Not supported** — the function will stop with an
#'  error directing you to use \code{ViolinPlot(..., add_beeswarm = TRUE)}
#'  instead.
#' @export
#' @examples
#' \donttest{
#' # Basic beeswarm
#' BeeswarmPlot(data, x = "x", y = "y")
#'
#' # Control point size
#' BeeswarmPlot(data, x = "x", y = "y", pt_size = 1)
#'
#' # Beeswarm with box overlay
#' BeeswarmPlot(data, x = "x", y = "y", add_box = TRUE, pt_color = "grey30")
#'
#' # Grouped
#' BeeswarmPlot(data, x = "x", y = "y", group_by = "group1")
#'
#' # Grouped without dodging
#' BeeswarmPlot(data, x = "x", y = "y", group_by = "group1",
#'              beeswarm_dodge = NULL)
#'
#' # Hex layout with wider spacing
#' BeeswarmPlot(data,
#'     x = "x", y = "y", beeswarm_method = "hex",
#'     beeswarm_cex = 2)
#' }
BeeswarmPlot <- function(
    data,
    x,
    x_sep = "_",
    y = NULL,
    in_form = c("long", "wide"),
    split_by = NULL,
    split_by_sep = "_",
    symnum_args = NULL,
    sort_x = NULL,
    flip = FALSE,
    keep_empty = FALSE,
    keep_na = FALSE,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    paired_by = NULL,
    x_text_angle = ifelse(isTRUE(flip), 0, 45),
    step_increase = 0.1,
    fill_mode = ifelse(!is.null(group_by), "dodge", "x"),
    palreverse = FALSE,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    alpha = 1,
    aspect.ratio = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    pt_color = NULL,
    pt_size = NULL,
    pt_alpha = 1,
    position_dodge_preserve = "total",
    jitter_width = NULL,
    jitter_height = 0,
    stack = FALSE,
    y_max = NULL,
    y_min = NULL,
    y_brackets = NULL,
    add_violin = FALSE,
    beeswarm_method = "swarm",
    beeswarm_cex = 1,
    beeswarm_priority = "ascending",
    beeswarm_dodge = 0.9,
    add_box = FALSE,
    box_color = "black",
    box_width = 0.1,
    box_ptsize = 2.5,
    add_trend = FALSE,
    trend_color = NULL,
    trend_linewidth = 1,
    trend_ptsize = 2,
    add_stat = NULL,
    stat_name = NULL,
    stat_color = "black",
    stat_size = 1,
    stat_stroke = 1,
    stat_shape = 25,
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    add_line = NULL,
    line_color = "red2",
    line_width = .6,
    line_type = 2,
    highlight = NULL,
    highlight_color = "red2",
    highlight_size = 1,
    highlight_alpha = 1,
    comparisons = NULL,
    ref_group = NULL,
    pairwise_method = "wilcox.test",
    multiplegroup_comparisons = FALSE,
    multiple_method = "kruskal.test",
    sig_label = "p.format",
    sig_labelsize = 3.5,
    hide_ns = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    ...
) {
    if (isTRUE(add_violin)) {
        stop(
            "Adding violin to a beeswarm plot is not supported. Please use ViolinPlot(..., add_beeswarm = TRUE) instead."
        )
    }
    stat_name <- stat_name %||%
        paste0(y, " (", deparse(substitute(add_stat)), ")")

    BoxViolinPlot(
        data = data,
        x = x,
        x_sep = x_sep,
        y = y,
        base = "none",
        in_form = in_form,
        split_by = split_by,
        split_by_sep = split_by_sep,
        sort_x = sort_x,
        flip = flip,
        keep_empty = keep_empty,
        keep_na = keep_na,
        group_by = group_by,
        group_by_sep = group_by_sep,
        group_name = group_name,
        paired_by = paired_by,
        x_text_angle = x_text_angle,
        fill_mode = fill_mode,
        palreverse = palreverse,
        step_increase = step_increase,
        theme = theme,
        theme_args = theme_args,
        palette = palette,
        palcolor = palcolor,
        alpha = alpha,
        position_dodge_preserve = position_dodge_preserve,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction,
        add_point = TRUE,
        pt_color = pt_color,
        pt_size = pt_size,
        pt_alpha = pt_alpha,
        symnum_args = symnum_args,
        jitter_width = jitter_width,
        jitter_height = jitter_height,
        stack = stack,
        y_max = y_max,
        y_min = y_min,
        y_brackets = y_brackets,
        add_beeswarm = TRUE,
        beeswarm_method = beeswarm_method,
        beeswarm_cex = beeswarm_cex,
        beeswarm_priority = beeswarm_priority,
        beeswarm_dodge = beeswarm_dodge,
        add_box = add_box,
        box_color = box_color,
        box_width = box_width,
        box_ptsize = box_ptsize,
        add_trend = add_trend,
        trend_color = trend_color,
        trend_linewidth = trend_linewidth,
        trend_ptsize = trend_ptsize,
        add_stat = add_stat,
        stat_name = stat_name,
        stat_color = stat_color,
        stat_size = stat_size,
        stat_stroke = stat_stroke,
        stat_shape = stat_shape,
        add_bg = add_bg,
        bg_palette = bg_palette,
        bg_palcolor = bg_palcolor,
        bg_alpha = bg_alpha,
        add_line = add_line,
        line_color = line_color,
        line_width = line_width,
        line_type = line_type,
        highlight = highlight,
        highlight_color = highlight_color,
        highlight_size = highlight_size,
        highlight_alpha = highlight_alpha,
        comparisons = comparisons,
        ref_group = ref_group,
        pairwise_method = pairwise_method,
        multiplegroup_comparisons = multiplegroup_comparisons,
        multiple_method = multiple_method,
        sig_label = sig_label,
        sig_labelsize = sig_labelsize,
        hide_ns = hide_ns,
        facet_by = facet_by,
        facet_scales = facet_scales,
        facet_ncol = facet_ncol,
        facet_nrow = facet_nrow,
        facet_byrow = facet_byrow,
        title = title,
        subtitle = subtitle,
        xlab = xlab,
        ylab = ylab,
        seed = seed,
        combine = combine,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        ...
    )
}

set.seed(8525)
data <- data.frame(
    x = factor(rep(c("A", "B", "C", "D"), each = 25), levels = c("A", "B", "C", "D")),
    y = rnorm(100),
    group1 = factor(rep(c("g1", "g2"), 50)),
    group2 = factor(rep(c("f1", "f2"), each = 50))
)

# BoxPlot tests
test_that("BoxPlot returns a ggplot for basic usage", {
    p <- BoxPlot(data, x = "x", y = "y")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("BoxPlot sets title and labels", {
    p <- BoxPlot(data, x = "x", y = "y", title = "Box", xlab = "Groups", ylab = "Value")
    expect_equal(p$labels$title, "Box")
    expect_equal(p$labels$x, "Groups")
    expect_equal(p$labels$y, "Value")
})

test_that("BoxPlot works with group_by", {
    p <- BoxPlot(data, x = "x", y = "y", group_by = "group1")
    expect_s3_class(p, "ggplot")
})

test_that("BoxPlot with split_by returns combined plot", {
    p <- BoxPlot(data, x = "x", y = "y", split_by = "group2", combine = TRUE)
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("BoxPlot with split_by and combine = FALSE returns list", {
    plots <- BoxPlot(data, x = "x", y = "y", split_by = "group2", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("BoxPlot with multiple split_by columns returns a list with one plot per combination", {
    plots <- suppressMessages(BoxPlot(data, x = "x", y = "y", split_by = c("group1", "group2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("BoxPlot with multiple split_by columns returns combined plot", {
    p <- suppressMessages(BoxPlot(data, x = "x", y = "y", split_by = c("group1", "group2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("BoxPlot with flip = TRUE works", {
    p <- BoxPlot(data, x = "x", y = "y", flip = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("BoxPlot with facet_by works", {
    p <- BoxPlot(data, x = "x", y = "y", facet_by = "group2")
    expect_s3_class(p, "ggplot")
})

test_that("BoxPlot with add_point = TRUE works", {
    p <- BoxPlot(data, x = "x", y = "y", add_point = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("BoxPlot with fill_mode = 'x' works", {
    p <- BoxPlot(data, x = "x", y = "y", fill_mode = "x")
    expect_s3_class(p, "ggplot")
})

test_that("BoxPlot with fill_mode = 'mean' works", {
    p <- BoxPlot(data, x = "x", y = "y", fill_mode = "mean")
    expect_s3_class(p, "ggplot")
})

test_that("BoxPlot with add_trend = TRUE works", {
    p <- BoxPlot(data, x = "x", y = "y", add_trend = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("BoxPlot with sort_x works", {
    p <- BoxPlot(data, x = "x", y = "y", sort_x = "mean(y)")
    expect_s3_class(p, "ggplot")
})

test_that("BoxPlot base = 'bar' works", {
    p <- BoxPlot(data, x = "x", y = "y", base = "bar")
    expect_s3_class(p, "ggplot")
})

# ViolinPlot tests
test_that("ViolinPlot returns a ggplot for basic usage", {
    p <- ViolinPlot(data, x = "x", y = "y")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("ViolinPlot with add_box = TRUE works", {
    p <- ViolinPlot(data, x = "x", y = "y", add_box = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("ViolinPlot with add_point = TRUE works", {
    p <- ViolinPlot(data, x = "x", y = "y", add_point = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("ViolinPlot with group_by works", {
    p <- ViolinPlot(data, x = "x", y = "y", group_by = "group1")
    expect_s3_class(p, "ggplot")
})

test_that("ViolinPlot with split_by and combine = FALSE returns list", {
    plots <- ViolinPlot(data, x = "x", y = "y", split_by = "group2", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
})

test_that("ViolinPlot with multiple split_by columns returns a list with one plot per combination", {
    plots <- suppressMessages(ViolinPlot(data, x = "x", y = "y", split_by = c("group1", "group2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("ViolinPlot with facet_by works", {
    p <- ViolinPlot(data, x = "x", y = "y", facet_by = "group2")
    expect_s3_class(p, "ggplot")
})

test_that("ViolinPlot with fill_mode = 'mean' works", {
    p <- ViolinPlot(data, x = "x", y = "y", fill_mode = "mean")
    expect_s3_class(p, "ggplot")
})

test_that("ViolinPlot sets title and labels", {
    p <- ViolinPlot(data, x = "x", y = "y", title = "Violin", xlab = "X", ylab = "Y")
    expect_equal(p$labels$title, "Violin")
})

# BeeswarmPlot tests
test_that("BeeswarmPlot errors when add_violin = TRUE", {
    expect_error(
        BeeswarmPlot(data, x = "x", y = "y", add_violin = TRUE),
        "Adding violin"
    )
})

test_that("BeeswarmPlot returns a ggplot for basic usage", {
    skip_if_not_installed("ggbeeswarm")
    p <- BeeswarmPlot(data, x = "x", y = "y")
    expect_s3_class(p, "ggplot")
})

test_that("BeeswarmPlot with multiple split_by columns returns a list with one plot per combination", {
    skip_if_not_installed("ggbeeswarm")
    plots <- suppressMessages(BeeswarmPlot(data, x = "x", y = "y", split_by = c("group1", "group2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("BeeswarmPlot with multiple split_by columns returns combined plot", {
    skip_if_not_installed("ggbeeswarm")
    p <- suppressMessages(BeeswarmPlot(data, x = "x", y = "y", split_by = c("group1", "group2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("BoxViolinPlot with multiple split_by columns returns a list with one plot per combination", {
    plots <- suppressMessages(BoxViolinPlot(data, x = "x", y = "y", split_by = c("group1", "group2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("BoxViolinPlot with multiple split_by columns returns combined plot", {
    p <- suppressMessages(BoxViolinPlot(data, x = "x", y = "y", split_by = c("group1", "group2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

#' Check the columns if columns found in the data
#' @keywords internal
#' @param df A data frame
#' @param columns A character vector of column names
#' @param force_factor Whether to force the columns to be factors
#' @param allow_multi Whether to allow multiple columns
#' @param concat_multi Whether to concatenate multiple columns
#' @param concat_sep The separator to use for concatenation
#' @return A character string of the valid column
#' @importFrom tidyr unite
#' @importFrom rlang syms
check_columns <- function(
    df,
    columns,
    force_factor = FALSE,
    allow_multi = FALSE,
    concat_multi = FALSE,
    concat_sep = "_"
) {
    if (is.null(df)) {
        stop("The data is NULL.")
    }
    if (is.null(columns)) {
        return(NULL)
    }
    df_name <- deparse(substitute(df))
    param_name <- deparse(substitute(columns))
    if (isFALSE(allow_multi)) {
        if (length(columns) > 1) {
            stop(paste0("Only one column is allowed in '", param_name, "'"))
        }
        if (!columns %in% colnames(df)) {
            stop(paste0("'", columns, "' is/are not in the data."))
        }
    } else {
        notfound <- setdiff(columns, colnames(df))
        if (length(notfound) > 0) {
            stop(paste0(
                "'",
                paste0(notfound, collapse = ", "),
                "' is/are not in the data."
            ))
        }
        if (isTRUE(concat_multi) && length(columns) > 1) {
            message(
                "Multiple columns are provided in '",
                param_name,
                "'. They will be concatenated into one column."
            )
            new_col <- paste(columns, collapse = concat_sep)
            df <- unite(
                df,
                !!sym(new_col),
                !!!syms(columns),
                sep = concat_sep,
                remove = FALSE
            )
            # Try keep the order of levels
            if (isTRUE(force_factor)) {
                all_levels <- lapply(columns, function(col) {
                    if (is.factor(df[[col]])) {
                        levels(df[[col]])
                    } else {
                        unique(df[[col]])
                    }
                })
                all_levels <- do_call(expand_grid, all_levels)
                all_levels <- apply(all_levels, 1, paste, collapse = concat_sep)
                df[[new_col]] <- droplevels(factor(
                    df[[new_col]],
                    levels = unique(all_levels)
                ))
            }
            columns <- new_col
        }
    }
    if (isTRUE(force_factor)) {
        p <- parent.frame()
        for (col in columns) {
            if (!is.factor(df[[col]])) {
                p[[df_name]][[col]] <- factor(
                    df[[col]],
                    levels = unique(df[[col]])
                )
            } else if (!col %in% colnames(p[[df_name]])) {
                p[[df_name]][[col]] <- df[[col]]
            }
        }
    }
    return(columns)
}


#' Expand the plot area with CSS-like padding
#' @keywords internal
#' @param expand A numeric vector of length 1, 2, 3, or 4
#'   The values to expand the x and y axes. It is like CSS padding.
#'   When a single value is provided, it is used for both axes on both sides.
#'   When two values are provided, the first value is used for the top/bottom side and the second value is used for the left/right side.
#'   When three values are provided, the first value is used for the top side, the second value is used for the left/right side, and the third value is used for the bottom side.
#'   When four values are provided, the values are used for the top, right, bottom, and left sides, respectively.
#'   You can also use a named vector to specify the values for each side.
#'   When the axis is discrete, the values will be applied as 'add' to the 'expansion' function.
#'   When the axis is continuous, the values will be applied as 'mult' to the 'expansion' function.
#'   See also \url{https://ggplot2.tidyverse.org/reference/expansion.html}
#' @param x_type The type of x-axis, either "continuous" or "discrete"
#' @param y_type The type of y-axis, either "continuous" or "discrete"
#' @return A list with x and y values for expand
#' @importFrom ggplot2 expansion
norm_expansion <- function(
    expand,
    x_type,
    y_type,
    continuous_default = c(0.05, 0),
    discrete_default = c(0, 0.6)
) {
    .expand_by_type <- function(ex, type, both = FALSE) {
        if (type == "continuous" && !is.null(ex)) {
            ret <- c(ex, 0)
        } else if (type == "continuous" && is.null(ex)) {
            ret <- continuous_default
        } else if (type == "discrete" && !is.null(ex)) {
            ret <- c(0, ex)
        } else {
            # type == "discrete" && is.null(ex)
            ret <- discrete_default
        }
        if (both) {
            return(c(ret, ret))
        }
        return(ret)
    }

    if (is.null(expand)) {
        return(list(
            x = .expand_by_type(NULL, x_type, both = TRUE),
            y = .expand_by_type(NULL, y_type, both = TRUE)
        ))
    }
    if (is.null(names(expand))) {
        if (length(expand) == 1) {
            expand <- c(
                top = expand,
                right = expand,
                bottom = expand,
                left = expand
            )
        } else if (length(expand) == 2) {
            expand <- c(
                top = expand[1],
                right = expand[2],
                bottom = expand[1],
                left = expand[2]
            )
        } else if (length(expand) == 3) {
            expand <- c(
                top = expand[1],
                right = expand[2],
                bottom = expand[3],
                left = expand[2]
            )
        } else if (length(expand) == 4) {
            expand <- c(
                top = expand[1],
                right = expand[2],
                bottom = expand[3],
                left = expand[4]
            )
        } else {
            stop("Invalid length (", length(expand), ") of 'expand'")
        }
    }

    expand <- as.list(expand)
    if (
        "x" %in%
            names(expand) &&
            ("left" %in% names(expand) || "right" %in% names(expand))
    ) {
        stop("Cannot have both 'x' and 'left'/'right' in 'expand'")
    }
    if (
        "y" %in%
            names(expand) &&
            ("top" %in% names(expand) || "bottom" %in% names(expand))
    ) {
        stop("Cannot have both 'y' and 'top'/'bottom' in 'expand'")
    }
    if ("x" %in% names(expand)) {
        expand$left <- expand$right <- expand$x
        expand$x <- NULL
    }
    if ("y" %in% names(expand)) {
        expand$bottom <- expand$top <- expand$y
        expand$y <- NULL
    }
    return(list(
        x = c(
            .expand_by_type(expand$left, x_type),
            .expand_by_type(expand$right, x_type)
        ),
        y = c(
            .expand_by_type(expand$bottom, y_type),
            .expand_by_type(expand$top, y_type)
        )
    ))
}

#' Calculate hjust and vjust based on angle
#' @param angle A numeric value of the angle
#' @return A list with h and v values
#' @keywords internal
calc_just <- function(angle) {
    angle <- angle %% 360
    if (angle < 0) {
        angle <- angle + 360
    }
    if (angle < 10) {
        h <- 0.5
        v <- 1
    } else if (angle < 90) {
        h <- 1
        v <- 1
    } else if (angle < 135) {
        h <- 1
        v <- 0.5
    } else if (angle < 180) {
        h <- 1
        v <- 0.5
    } else if (angle < 225) {
        h <- 0
        v <- 0
    } else if (angle < 270) {
        h <- 0
        v <- 0
    } else if (angle < 315) {
        h <- 0
        v <- 0.5
    } else if (angle < 360) {
        h <- 0
        v <- 1
    } else {
        h <- 0.5
        v <- 1
    }
    list(h = h, v = v)
}


#' Calculate plot dimensions with aspect ratio consideration
#'
#' This function calculates plot height and width taking into account:
#' - Content-based scaling (number of items on axes)
#' - Aspect ratio constraints
#' - Legend position and direction
#' - Minimum and maximum dimension bounds
#'
#' @param base_height Base height for the plot (before legend adjustments). Default is 4.5.
#' @param aspect.ratio Aspect ratio (height/width). If NULL, width is calculated independently.
#' @param n_x Number of categories on x-axis (for width scaling)
#' @param n_y Number of categories on y-axis (for height scaling)
#' @param x_scale_factor Scaling factor per x-axis category. Default is 0.5.
#' @param y_scale_factor Scaling factor per y-axis category. Default is 0.5.
#' @param legend.position Position of legend ("none", "right", "left", "top", "bottom")
#' @param legend.direction Direction of legend ("vertical" or "horizontal")
#' @param flip Whether the plot is flipped (inverts aspect ratio)
#' @param min_width Minimum width in inches. Default is 3.
#' @param min_height Minimum height in inches. Default is 3.
#' @param max_width Maximum width in inches. Default is 12.
#' @param max_height Maximum height in inches. Default is 12.
#' @return A list with height and width components
#' @keywords internal
calculate_plot_dimensions <- function(
    base_height = 4.5,
    aspect.ratio = 1,
    n_x = NULL,
    n_y = NULL,
    x_scale_factor = 0.5,
    y_scale_factor = 0.5,
    legend.position = "right",
    legend.direction = "vertical",
    legend_n = 1,
    legend_nchar = 5,
    flip = FALSE,
    min_width = 3,
    min_height = 3,
    max_width = 12,
    max_height = 12
) {
    # Handle flip by inverting aspect ratio
    if (isTRUE(flip) && !is.null(aspect.ratio)) {
        aspect.ratio <- 1 / aspect.ratio
    }

    # Calculate content-based dimensions
    if (!is.null(aspect.ratio)) {
        # Strategy: Calculate width from content, then derive height from aspect ratio
        if (!is.null(n_x)) {
            # Calculate width based on x-axis content
            content_width <- 0.5 + n_x * x_scale_factor
            if (content_width > max_width) {
                # Content already overflows max width: cap width and anchor height to base_height
                width <- max_width
                height <- base_height * aspect.ratio
            } else {
                # Calculate height from aspect ratio: aspect.ratio = height / width
                height <- content_width * aspect.ratio
                width <- content_width

                # Check if this creates extreme dimensions
                if (height < min_height || height > max_height) {
                    # Adjust based on height constraints
                    height <- max(min_height, min(height, max_height))
                    width <- height / aspect.ratio

                    if (width < min_width || width > max_width) {
                        # Content requirements conflict with aspect ratio
                        warning(
                            "Content-based width (",
                            round(content_width, 2),
                            ") conflicts with aspect.ratio (",
                            round(aspect.ratio, 2),
                            "). Using content width; plot panel aspect ratio will differ from specified.",
                            call. = FALSE
                        )
                        width <- max(min_width, min(content_width, max_width))
                        height <- base_height
                    }
                }
            }
        } else if (!is.null(n_y)) {
            # Calculate height based on y-axis content
            content_height <- 0.5 + n_y * y_scale_factor
            if (content_height > max_height) {
                # Content already overflows max height: cap height and anchor width to base_height
                height <- max_height
                width <- base_height / aspect.ratio
            } else {
                # Calculate width from aspect ratio: width = height / aspect.ratio
                height <- content_height
                width <- height / aspect.ratio

                # Check if this creates extreme dimensions
                if (width < min_width || width > max_width) {
                    # Adjust based on width constraints
                    width <- max(min_width, min(width, max_width))
                    height <- width * aspect.ratio

                    if (height < min_height || height > max_height) {
                        # Content requirements conflict with aspect ratio
                        warning(
                            "Content-based height (",
                            round(content_height, 2),
                            ") conflicts with aspect.ratio (",
                            round(aspect.ratio, 2),
                            "). Using content height; plot panel aspect ratio will differ from specified.",
                            call. = FALSE
                        )
                        height <- max(
                            min_height,
                            min(content_height, max_height)
                        )
                        width <- base_height / aspect.ratio
                    }
                }
            }
        } else {
            # No content scaling, use base height and derive width from aspect ratio
            height <- base_height
            width <- height / aspect.ratio
        }
    } else {
        # No aspect ratio constraint - calculate dimensions independently
        height <- if (!is.null(n_y)) {
            0.5 + n_y * y_scale_factor
        } else {
            base_height
        }
        width <- if (!is.null(n_x)) {
            0.5 + n_x * x_scale_factor
        } else {
            base_height
        }
    }

    # Ensure minimum dimensions
    height <- max(height, min_height)
    width <- max(width, min_width)

    # Add legend space additively (after aspect ratio calculations)
    # Legend metrics (ggplot2 defaults at 11pt text):
    #   key:  ~0.25 in (unit(1.2, "lines") at 11pt) + internal padding
    #   text: ~0.07 in per character  (11pt, ~0.6 aspect ratio, 1/72 in/pt)
    #   row height (key + spacing): ~0.30 in
    #   outer margin / title:       ~0.35 in
    legend_key_w <- 0.30 # key width + internal margins
    legend_char_w <- 0.07 # inches per character of label text
    legend_row_h <- 0.30 # height of one stacked legend row
    legend_pad <- 0.35 # outer margin + optional title

    if (!identical(legend.position, "none")) {
        if (legend.position %in% c("right", "left")) {
            # Vertical legend: width = key + text, at least 1 in
            legend_width <- max(
                1.0,
                legend_key_w + legend_nchar * legend_char_w
            )
            width <- width + legend_width
        } else if (legend.direction == "horizontal") {
            # Horizontal legend (bottom / top): items sit side-by-side, may wrap
            # Estimate how many items fit per row given current (pre-legend) width
            item_w <- max(
                0.5,
                legend_key_w + legend_nchar * legend_char_w + 0.1
            )
            items_per_row <- max(1L, floor(width / item_w))
            n_rows <- ceiling(max(1L, legend_n) / items_per_row)
            legend_height <- max(1.0, legend_pad + n_rows * legend_row_h)
            height <- height + legend_height
        } else {
            # Vertical legend at bottom / top or floating: treat like right/left
            legend_width <- max(
                1.0,
                legend_key_w + legend_nchar * legend_char_w
            )
            width <- width + legend_width
        }
    }

    # Apply maximum bounds
    height <- min(height, max_height)
    width <- min(width, max_width)

    list(height = height, width = width)
}


#' Facetting a plot
#'
#' @param plot The plot to facet or a list list(plot, height, width) if guess_size is TRUE
#' @param facet_by The column(s) to split data by and plot separately or facet by
#'   If NULL, no faceting will be done
#' @param facet_scales Whether to scale the axes of facets.
#' @param nrow The number of rows in facet_wrap
#' @param ncol The number of columns in facet_wrap
#' @param byrow Whether to fill the plots by row
#' @param legend.position The position of the legend
#' @param legend.direction The direction of the legend
#' @param recalc_size Whether to re-calculate the size of the plot
#' @param ... Additional arguments to pass to facet_wrap or facet_grid
#' @return The faceted plot. If guess_size is TRUE, attr(p, "height") and attr(p, "width") will be set
#' @importFrom rlang sym
#' @importFrom ggplot2 facet_wrap facet_grid ggplot_build vars
#' @keywords internal
facet_plot <- function(
    plot,
    facet_by,
    facet_scales,
    nrow,
    ncol,
    byrow,
    legend.position = "right",
    legend.direction = "vertical",
    recalc_size = TRUE,
    ...
) {
    if (is.null(facet_by)) {
        return(plot)
    }

    if (recalc_size) {
        p <- facet_plot(
            plot,
            facet_by,
            facet_scales,
            nrow,
            ncol,
            byrow,
            recalc_size = FALSE,
            ...
        )
        d <- wrap_dims(length(unique(ggplot_build(p)$data[[1]]$PANEL)))
        height <- d[1] * attr(plot, "height")
        width <- d[2] * attr(plot, "width")
        if (!identical(legend.position, "none")) {
            # The legends are merged
            if (legend.position %in% c("left", "right")) {
                portion <- ifelse(legend.direction == "vertical", 0.2, 0.3)
                width <- width - (d[2] - 1) * (width / d[2] * portion)
            } else {
                portion <- ifelse(legend.direction == "vertical", 0.3, 0.2)
                height <- height - (d[1] - 1) * (height / d[1] * portion)
            }
        }
        attr(p, "height") <- height
        attr(p, "width") <- width
        return(p)
    }

    if (length(facet_by) == 1) {
        plot <- plot +
            ggplot2::facet_wrap(
                facets = facet_by,
                scales = facet_scales,
                nrow = nrow,
                ncol = ncol,
                dir = if (byrow) "h" else "v",
                ...
            )
    } else {
        args <- rlang::dots_list(...)
        args$strip.position <- NULL
        args$rows <- vars(!!sym(facet_by[[1]]))
        args$cols <- vars(!!sym(facet_by[[2]]))
        args$scales <- facet_scales
        plot <- plot + do_call(ggplot2::facet_grid, args)
    }

    return(plot)
}


#' Combine plots into one
#'
#' @keywords internal
#' @param plots A list of plots
#' @param combine Whether to combine the plots into one
#' @param split_by The column name to split the plots by. When provided,
#' the combined data from all sub-plots is available via \code{p$data}.
#' @param nrow The number of rows in the combined plot
#' @param ncol The number of columns in the combined plot
#' @param byrow Whether to fill the plots by row
#' @param recalc_size Whether to re-calculate the size of the combined plot
#' @return The faceted plot. If guess_size is TRUE, attr(p, "height") and attr(p, "width") will be set
#' @importFrom patchwork wrap_plots
#' @importFrom rlang %||%
#' @importFrom ggplot2 wrap_dims
combine_plots <- function(
    plots,
    combine = TRUE,
    split_by = NULL,
    nrow = NULL,
    ncol = NULL,
    byrow = NULL,
    axes = NULL,
    axis_titles = NULL,
    guides = NULL,
    design = NULL,
    recalc_size = TRUE
) {
    if (isFALSE(combine)) {
        return(plots)
    }

    if (recalc_size) {
        d <- wrap_dims(length(plots), nrow, ncol)
        nrow <- d[1]
        ncol <- d[2]
        p <- combine_plots(
            plots,
            combine = TRUE,
            split_by = split_by,
            nrow = nrow,
            ncol = ncol,
            byrow = byrow,
            axes = axes,
            axis_titles = axis_titles,
            guides = guides,
            design = design,
            recalc_size = FALSE
        )
        # Allow to work with external plots
        try(
            {
                attr(p, "height") <- nrow *
                    max(sapply(plots, function(x) attr(x, "height")))
                attr(p, "width") <- ncol *
                    max(sapply(plots, function(x) attr(x, "width")))
            },
            silent = TRUE
        )
        return(p)
    }
    # When it's gTree, also run wrap_plots to convert it to a patchwork object
    # CorPairsPlot
    if (length(plots) == 1 && !inherits(plots[[1]], "gTree")) {
        return(plots[[1]])
    }

    if (!is.null(split_by)) {
        # Build the row-bound combined data from all sub-plots.
        # Some plot types (e.g. Heatmap) may have sub-plots with different
        # column names, so we align columns before rbinding.
        split_dfs <- lapply(names(plots), function(nm) {
            d <- plots[[nm]]$data
            d[[split_by]] <- nm
            return(d)
        })
        if (all(vapply(split_dfs, is.data.frame, logical(1)))) {
            all_cols <- unique(unlist(lapply(split_dfs, names)))
            split_dfs <- lapply(split_dfs, function(d) {
                missing <- setdiff(all_cols, names(d))
                for (col in missing) {
                    d[[col]] <- NA
                }
                d[, all_cols, drop = FALSE]
            })
            combined_data <- do_call(rbind, split_dfs)
        } else {
            combined_data <- do_call(rbind, split_dfs)
        }

        # For standard ggplot plots, wrap_plots uses the last sub-plot as
        # the rendering base. We set the combined data on it and give each
        # layer that inherits the plot data explicit per-split data so
        # rendering is unaffected. Layers with their own explicit data
        # (e.g. geom_text with a computed AUC column) are left unchanged.
        # ggraph-based plots (e.g. Network, ClustreePlot) carry the graph
        # layout in the plot data, and their edge layers re-resolve
        # coordinates from it at build time (data = get_edges(), called
        # with the plot data as the layout). To keep the combined layout
        # inspectable without re-resolving edges against it, every layer
        # is instead frozen to the last sub-plot's own layout: waiver data
        # is pointed at it and function data (the edge layers) is
        # evaluated against it once, producing explicit coordinates.
        last_idx <- length(plots)
        last_orig_data <- plots[[last_idx]]$data
        if (inherits(last_orig_data, "layout_ggraph")) {
            for (i in seq_along(plots[[last_idx]]$layers)) {
                layer_data <- plots[[last_idx]]$layers[[i]]$data
                if (inherits(layer_data, "waiver")) {
                    plots[[last_idx]]$layers[[i]]$data <- last_orig_data
                } else if (is.function(layer_data)) {
                    plots[[last_idx]]$layers[[i]]$data <- layer_data(last_orig_data)
                }
            }
            # The rbind above keeps the first sub-plot's graph attribute,
            # which only holds that split's vertices and edges. Rebuild a
            # combined graph carrying every split's nodes and links, each
            # marked with the split column, so the exposed data stays a
            # complete layout. Vertex names repeat across splits, which
            # igraph::graph_from_data_frame rejects, hence tbl_graph().
            edge_dfs <- lapply(names(plots), function(nm) {
                e <- igraph::as_data_frame(
                    attr(plots[[nm]]$data, "graph"),
                    what = "edges"
                )
                e[[split_by]] <- nm
                return(e)
            })
            graph <- tidygraph::tbl_graph(
                nodes = combined_data,
                edges = do_call(rbind, edge_dfs)
            )
            attr(combined_data, "graph") <- graph
            # Expose the combined links table as well, each row tagged
            # with the split column, so the original link-level data
            # stays directly inspectable.
            attr(combined_data, "edges") <- do_call(rbind, edge_dfs)
        } else {
            plots[[last_idx]]$data <- combined_data
            for (i in seq_along(plots[[last_idx]]$layers)) {
                layer_data <- plots[[last_idx]]$layers[[i]]$data
                if (inherits(layer_data, "waiver")) {
                    plots[[last_idx]]$layers[[i]]$data <- last_orig_data
                }
            }
        }
    }

    p <- wrap_plots(
        plots,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )

    # For non-ggplot sub-plots (e.g. wrap_elements / grid grobs),
    # wrap_plots creates a waiver() $data instead of inheriting from
    # the last sub-plot. Set it explicitly so p$data always works.
    # For ggraph-based plots (e.g. Network) the layers were frozen to
    # each split's own layout above, so exposing the combined layout
    # (with the split column) here does not affect rendering.
    if (!is.null(split_by)) {
        p$data <- combined_data
    }

    p
}


#' Get a ggplot layer for background
#' @keywords internal
#' @param data A data frame
#' @param x A character string specifying the column name of the data frame to plot for the x-axis
#' @param keep_empty A character string specifying whether to keep empty levels
#' @param palette A character string specifying the palette to use
#' @param palcolor A character string specifying the color to use in the palette
#' @param alpha A numeric value specifying the transparency of the plot
#' @param facet_by A character string specifying the column name(s) of the data frame to facet the plot
#' @param direction A character string specifying the direction for the background
#' @return A ggplot layer for background
#' @importFrom ggplot2 geom_rect
#' @importFrom dplyr distinct
#' @importFrom tidyr expand_grid
bg_layer <- function(
    data,
    x,
    keep_empty,
    palette,
    palcolor,
    alpha,
    facet_by,
    direction = "vertical"
) {
    if (anyNA(data[[x]])) {
        randint <- sample.int(1e6, 1)
        levels(data[[x]]) <- c(levels(data[[x]]), paste0("__NA__", randint))
        data[[x]][is.na(data[[x]])] <- paste0("__NA__", randint)
    }
    lvls <- if (keep_empty) levels(data[[x]]) else levels(droplevels(data[[x]]))
    bg_color <- palette_this(lvls, palette = palette, palcolor = palcolor)
    bg_data <- data.frame(x = factor(lvls, levels = lvls))
    bg_data$x <- as.numeric(bg_data$x)
    bg_data$xmin <- ifelse(bg_data$x == min(bg_data$x), -Inf, bg_data$x - 0.5)
    bg_data$xmax <- ifelse(bg_data$x == max(bg_data$x), Inf, bg_data$x + 0.5)
    bg_data$ymin <- -Inf
    bg_data$ymax <- Inf
    bg_data$fill <- bg_color[lvls]

    if (!is.null(facet_by)) {
        unique_facet_values <- distinct(data, !!!syms(facet_by))
        bg_data <- expand_grid(bg_data, unique_facet_values)
        for (fb in facet_by) {
            bg_data[[fb]] <- factor(bg_data[[fb]], levels = levels(data[[fb]]))
        }
    }

    if (direction == "vertical") {
        geom_rect(
            data = bg_data,
            aes(
                xmin = !!sym("xmin"),
                xmax = !!sym("xmax"),
                ymin = !!sym("ymin"),
                ymax = !!sym("ymax")
            ),
            fill = bg_data$fill,
            alpha = alpha,
            inherit.aes = FALSE
        )
    } else {
        geom_rect(
            data = bg_data,
            aes(
                xmin = !!sym("ymin"),
                xmax = !!sym("ymax"),
                ymin = !!sym("xmin"),
                ymax = !!sym("xmax")
            ),
            fill = bg_data$fill,
            alpha = alpha,
            inherit.aes = FALSE
        )
    }
}

#' Convert RGBA to RGB
#' @keywords internal
rgba_to_rgb <- function(RGBA, BackGround = c(1, 1, 1)) {
    A <- RGBA[[length(RGBA)]]
    RGB <- RGBA[[-length(RGBA)]] * A + BackGround * (1 - A)
    return(RGB)
}

#' Blend two colors
#' @keywords internal
blend_to_color <- function(C1, C2, mode = "blend") {
    c1 <- C1[[1]]
    c1a <- C1[[2]]
    c2 <- C2[[1]]
    c2a <- C2[[2]]
    A <- 1 - (1 - c1a) * (1 - c2a)
    if (A < 1.0e-6) {
        return(list(c(0, 0, 0), 1))
    }
    if (mode == "blend") {
        out <- (c1 * c1a + c2 * c2a * (1 - c1a)) / A
        A <- 1
    }
    if (mode == "average") {
        out <- (c1 + c2) / 2
        out[out > 1] <- 1
    }
    if (mode == "screen") {
        out <- 1 - (1 - c1) * (1 - c2)
    }
    if (mode == "multiply") {
        out <- c1 * c2
    }
    return(list(out, A))
}

#' Blend a list of colors
#' @keywords internal
blend_rgblist <- function(Clist, mode = "blend", RGB_BackGround = c(1, 1, 1)) {
    N <- length(Clist)
    ClistUse <- Clist
    while (N != 1) {
        temp <- ClistUse
        ClistUse <- list()
        for (C in temp[1:(length(temp) - 1)]) {
            c1 <- C[[1]]
            a1 <- C[[2]]
            c2 <- temp[[length(temp)]][[1]]
            a2 <- temp[[length(temp)]][[2]]
            ClistUse <- append(
                ClistUse,
                list(blend_to_color(
                    C1 = list(c1, a1 * (1 - 1 / N)),
                    C2 = list(c2, a2 * 1 / N),
                    mode = mode
                ))
            )
        }
        N <- length(ClistUse)
    }
    Result <- list(ClistUse[[1]][[1]], ClistUse[[1]][[2]])
    Result <- rgba_to_rgb(Result, BackGround = RGB_BackGround)
    return(Result)
}

#' Blend colors
#'
#' This function blends a list of colors using the specified blend mode.
#'
#' @param colors Color vectors.
#' @param mode Blend mode. One of "blend", "average", "screen", or "multiply".
#'
#' @keywords internal
#' @return The blended color.
#' @importFrom grDevices col2rgb
blend_colors <- function(
    colors,
    mode = c("blend", "average", "screen", "multiply")
) {
    mode <- match.arg(mode)
    colors <- colors[!is.na(colors)]
    if (length(colors) == 0) {
        return(NA)
    }
    if (length(colors) == 1) {
        return(colors)
    }
    rgb <- as.list(as.data.frame(col2rgb(colors) / 255))
    Clist <- lapply(rgb, function(x) {
        list(x, 1)
    })
    blend_color <- blend_rgblist(Clist, mode = mode)
    blend_color <- grDevices::rgb(
        blend_color[1],
        blend_color[2],
        blend_color[3]
    )
    return(blend_color)
}

#' @importFrom grid is.grob grobWidth grobHeight
#' @importFrom gtable is.gtable gtable_add_rows gtable_add_cols gtable_add_grob
add_grob <- function(
    gtable,
    grob,
    position = c("top", "bottom", "left", "right", "none"),
    space = NULL,
    clip = "on"
) {
    position <- match.arg(position)
    if (position == "none" || is.null(grob)) {
        return(gtable)
    }

    if (is.null(space)) {
        if (is.gtable(grob)) {
            if (position %in% c("top", "bottom")) {
                space <- sum(grob$heights)
            } else {
                space <- sum(grob$widths)
            }
        } else if (is.grob(grob)) {
            if (position %in% c("top", "bottom")) {
                space <- grobHeight(grob)
            } else {
                space <- grobWidth(grob)
            }
        }
    }

    if (position == "top") {
        gtable <- gtable_add_rows(gtable, space, 0)
        gtable <- gtable_add_grob(
            gtable,
            grob,
            t = 1,
            l = mean(gtable$layout[
                grepl(pattern = "panel", x = gtable$layout$name),
                "l"
            ]),
            clip = clip
        )
    }
    if (position == "bottom") {
        gtable <- gtable_add_rows(gtable, space, -1)
        gtable <- gtable_add_grob(
            gtable,
            grob,
            t = dim(gtable)[1],
            l = mean(gtable$layout[
                grepl(pattern = "panel", x = gtable$layout$name),
                "l"
            ]),
            clip = clip
        )
    }
    if (position == "left") {
        gtable <- gtable_add_cols(gtable, space, 0)
        gtable <- gtable_add_grob(
            gtable,
            grob,
            t = mean(gtable$layout[grep("panel", gtable$layout$name), "t"]),
            l = 1,
            clip = clip
        )
    }
    if (position == "right") {
        gtable <- gtable_add_cols(gtable, space, -1)
        gtable <- gtable_add_grob(
            gtable,
            grob,
            t = mean(gtable$layout[grep("panel", gtable$layout$name), "t"]),
            l = dim(gtable)[2],
            clip = clip
        )
    }
    return(gtable)
}

#' Convert a color with arbitrary transparency to a fixed color
#'
#' This function takes a vector of colors and an alpha level and converts the colors
#' to fixed colors with the specified alpha level.
#'
#' @param colors Color vectors.
#' @param alpha Alpha level ranging from 0 to 1.
#' @return The colors with the specified alpha level.
#' @keywords internal
#' @importFrom grDevices col2rgb rgb
adjcolors <- function(colors, alpha) {
    has_names <- !is.null(names(colors))
    color_df <- as.data.frame(col2rgb(colors) / 255)
    colors_out <- sapply(color_df, function(color) {
        color_rgb <- rgba_to_rgb(list(color, alpha))
        return(rgb(color_rgb[1], color_rgb[2], color_rgb[3]))
    })
    if (has_names) {
        names(colors_out) <- names(colors)
    } else {
        names(colors_out) <- NULL
    }
    return(colors_out)
}

#' Process theme to allow 'ggplot2::theme_minimal' to work
#'
#' @param theme The theme to process
#' @return The processed theme
#' @keywords internal
#' @importFrom utils getFromNamespace
process_theme <- function(theme) {
    if (is.function(theme)) {
        return(theme)
    }

    if (!is.character(theme)) {
        stop("The theme must be a character string or a theme function.")
    }

    if (!grepl("::", theme)) {
        return(theme)
    }

    parts <- unlist(strsplit(theme, "::"))
    if (length(parts) != 2) {
        stop("Invalid theme '", theme, "'")
    }

    return(getFromNamespace(parts[2], parts[1]))
}


#' check_palette
#' Check if the palette can be properly used
#' @param palette palette
#' @param datas_name names of the split data
#' @keywords internal
#' @return named list containing palette names
check_palette <- function(palette, datas_name) {
    palette <- as.list(palette)
    stopifnot("'palette' must be specified" = length(palette) > 0)
    if (length(palette) == 1 && length(datas_name) > 1) {
        palette <- rep(palette, length(datas_name))
    }
    if (length(palette) < length(datas_name)) {
        stop(
            "The length of 'palette' (",
            length(palette),
            ") is less than the number ",
            "(",
            length(datas_name),
            ") of unique values in 'split_by'"
        )
    }
    if (is.null(names(palette))) {
        names(palette)[1:length(datas_name)] <- datas_name
    } else if (length(setdiff(datas_name, names(palette))) > 0) {
        stop(
            "Values in 'split_by' (",
            paste(setdiff(datas_name, names(palette)), collapse = ", "),
            ") ",
            "have no corresponding palette assigned in 'palette'"
        )
    }
    return(palette)
}


#' check_palcolor
#' Check if the palcolor can be properly used
#' @param palcolor palcolor
#' @param datas_name names of the split data
#' @keywords internal
#' @return named list containing color names
check_palcolor <- function(palcolor, datas_name) {
    if (is.null(palcolor)) {
        return(NULL)
    }
    # as.list() will turn c("red", "blue") into list("red", "blue")
    # but we need list(c("red", "blue"))
    if (!is.list(palcolor)) {
        palcolor <- list(palcolor)
    }
    if (identical(datas_name, "...") && !identical(names(palcolor), "...")) {
        palcolor <- list(palcolor)
        names(palcolor) <- datas_name
    }
    if (!identical(datas_name, "...") && all(sapply(palcolor, is.character))) {
        palcolor <- list(palcolor)
    }
    if (length(palcolor) == 1 && length(datas_name) > 1) {
        palcolor <- rep(palcolor, length(datas_name))
    }
    if (is.null(names(palcolor))) {
        names(palcolor)[1:length(datas_name)] <- datas_name
    }
    # It's okay that some split_by values have no corresponding palcolor
    # because they can be NULL
    return(palcolor)
}


#' check_legend
#' Check if the legend.position and legend.direction are valid
#' @param legend The value legend.position or legend.direction
#' @param datas_name names of the split data
#' @keywords internal
#' @importFrom ggplot2 waiver
#' @return named list containing legend names
check_legend <- function(
    legend,
    datas_name,
    which = c("legend.position", "legend.direction")
) {
    which <- match.arg(which)
    legend <- as.list(legend)

    if (inherits(legend, "waiver")) {
        legend <- list(legend)
    }

    if (length(legend) == 1 && length(datas_name) > 1) {
        legend <- rep(legend, length(datas_name))
    }
    if (length(legend) < length(datas_name)) {
        stop(
            "The length of ",
            which,
            " (",
            length(legend),
            ") is less than the number ",
            "(",
            length(datas_name),
            ") of unique values in 'split_by'"
        )
    }
    if (is.null(names(legend))) {
        names(legend)[1:length(datas_name)] <- datas_name
    } else if (length(setdiff(datas_name, names(legend))) > 0) {
        stop(
            "Values in 'split_by' (",
            paste(setdiff(datas_name, names(legend)), collapse = ", "),
            ") ",
            "have no corresponding ",
            which,
            "."
        )
    }

    return(legend)
}


#' check_keep_na
#' Check and normalize keep_na parameter
#' @param keep_na keep_na
#' @param cols column names if keep_na is a single value
#' @keywords internal
#' @return normalized keep_na
check_keep_na <- function(keep_na, cols = NA) {
    if (is.character(keep_na) || is.logical(keep_na)) {
        if (isTRUE(keep_na) || is.na(keep_na)) {
            keep_na <- NA
        }
        if (is.null(cols) || length(cols) == 0) {
            # no columns selected
            return(list())
        } else if (length(cols) == 1 && is.na(cols)) {
            return(keep_na)
        } else {
            cols <- unique(cols)
            return(stats::setNames(rep(list(keep_na), length(cols)), cols))
        }
    }
    if (is.null(names(keep_na)) || !is.list(keep_na)) {
        stop("'keep_na' must be a logical/character or a named list.")
    }
    out <- list()
    if (length(cols) > 1 || (length(cols) == 1 && !is.na(cols))) {
        cols <- unique(cols)
        out <- stats::setNames(rep(list(FALSE), length(cols)), cols)
    }
    for (name in names(keep_na)) {
        out[[name]] <- check_keep_na(keep_na[[name]])
    }

    return(out)
}

#' check_keep_empty
#' Check and normalize keep_empty parameter
#' @param keep_empty keep_empty
#' @param cols column names if keep_empty is a single value
#' @keywords internal
#' @return normalized keep_empty
check_keep_empty <- function(keep_empty, cols = NA) {
    if (identical(keep_empty, "levels")) {
        keep_empty <- "level"
    }
    if (is.character(keep_empty) && !identical(keep_empty, "level")) {
        stop("'keep_empty' must be one of TRUE/FALSE, 'level', or 'levels'.")
    }
    if (!is.list(keep_empty)) {
        if (is.null(cols) || length(cols) == 0) {
            # no columns selected
            return(list())
        } else if (length(cols) == 1 && is.na(cols)) {
            return(keep_empty)
        } else {
            cols <- unique(cols)
            return(stats::setNames(rep(list(keep_empty), length(cols)), cols))
        }
    }
    if (is.null(names(keep_empty))) {
        stop("'keep_empty' must have names when provided as a list.")
    }
    out <- list()
    if (length(cols) > 1 || (length(cols) == 1 && !is.na(cols))) {
        cols <- unique(cols)
        out <- stats::setNames(rep(list(FALSE), length(cols)), cols)
    }
    for (name in names(keep_empty)) {
        out[[name]] <- check_keep_empty(keep_empty[[name]])
    }
    return(out)
}

#' process_keep_na_empty
#' Process keep_na and keep_empty to data
#' @param data data frame
#' @param keep_na List of keep_na
#' @param keep_empty List of keep_empty
#' @keywords internal
#' @return processed data frame
process_keep_na_empty <- function(
    data,
    keep_na = NULL,
    keep_empty = NULL,
    col = NULL
) {
    if (!is.null(keep_na)) {
        if (!is.null(col) && col %in% names(keep_na)) {
            keep_na <- keep_na[col]
        }
        for (cl in names(keep_na)) {
            if (!cl %in% colnames(data)) {
                warning(
                    "Column '",
                    cl,
                    "' not found in data. Skipping 'keep_na' processing for this column."
                )
                next
            }
            if (!anyNA(data[[cl]])) {
                next
            }
            if (isFALSE(keep_na[[cl]])) {
                data <- data[!is.na(data[[cl]]), , drop = FALSE]
                next
            }
            if (is.factor(data[[cl]]) && !is.na(keep_na[[cl]])) {
                levels(data[[cl]]) <- c(levels(data[[cl]]), keep_na[[cl]])
            }
            data[[cl]][is.na(data[[cl]])] <- keep_na[[cl]]
        }
    }

    if (!is.null(keep_empty)) {
        if (!is.null(col) && col %in% names(keep_empty)) {
            keep_empty <- keep_empty[col]
        }

        for (cl in names(keep_empty)) {
            if (!cl %in% colnames(data)) {
                warning(
                    "Column '",
                    cl,
                    "' not found in data. Skipping 'keep_empty' processing for this column."
                )
                next
            }
            if (!is.factor(data[[cl]])) {
                next
            }
            if (isFALSE(keep_empty[[cl]])) {
                data[[cl]] <- droplevels(data[[cl]])
            }
        }
    }
    return(data)
}

#' Call a function with a list of arguments
#'
#' A faster alternative to \code{\link[base]{do.call}}, especially when there are
#' large objects in the argument list. Named arguments are looked up by symbol
#' in the evaluation environment rather than being copied into the call, which
#' avoids the copying overhead of \code{base::do.call}. Unnamed arguments are
#' embedded in the call directly and do not benefit from this optimization.
#' Borrowed from \code{Gmisc::fastDoCall}.
#'
#' @inheritParams base::do.call
#' @return The result of the function call
#' @keywords internal
do_call <- function(what, args, quote = FALSE, envir = parent.frame()) {
    # source: Gmisc
    # author: Max Gordon <max@gforge.se>

    if (quote) {
        args <- lapply(args, enquote) # nocov
    }

    if (
        is.null(names(args)) ||
            is.data.frame(args)
    ) {
        argn <- args
        args <- list()
    } else {
        # Add all the named arguments
        argn <- lapply(names(args)[names(args) != ""], as.name)
        names(argn) <- names(args)[names(args) != ""]
        # Add the unnamed arguments
        argn <- c(argn, args[names(args) == ""])
        args <- args[names(args) != ""]
    }

    if (is.character(what)) {
        fn <- strsplit(what, "[:]{2,3}")[[1]]
        what <- if (length(fn) == 1) {
            get(fn[[1]], envir = envir, mode = "function")
        } else {
            get(fn[[2]], envir = asNamespace(fn[[1]]), mode = "function")
        }
        call <- as.call(c(list(what), argn))
    } else if (is.function(what)) {
        f_name <- deparse1(substitute(what))
        call <- as.call(c(list(as.name(f_name)), argn))
        args[[f_name]] <- what
    } else if (is.name(what)) {
        call <- as.call(c(list(what), argn))
    } else {
        stop(
            "'what' must be a function, a character string, or a name, not ",
            class(what)[[1]]
        )
    }

    eval(call, envir = args, enclos = envir)
}

#' Prepare continuous color scale limits with quantile/cutoff controls
#'
#' Computes the lower and upper cutoffs for a continuous color/fill scale,
#' applies data winsorization (clamping), and returns the results needed
#' by \code{\link[ggplot2]{scale_fill_gradientn}} or
#' \code{\link[ggplot2]{scale_color_gradientn}}.
#'
#' @inheritParams common_args
#' @param data A data frame.
#' @param column The column name in \code{data} to use for the color scale.
#' @param bg_cutoff Optional numeric cutoff — values \code{<= bg_cutoff} are
#'   set to \code{NA} before computing cutoffs. Default is \code{NULL}.
#' @return A list with components:
#'   \item{data}{The modified data frame (with winsorized \code{column}).}
#'   \item{feat_colors_value}{Numeric vector of 100 evenly-spaced values from
#'     \code{lower_cutoff} to \code{upper_cutoff}.}
#'   \item{limits}{Numeric vector of length 2 giving the range
#'     \code{c(lower_cutoff, upper_cutoff)}.}
#' @keywords internal
#' @importFrom stats quantile
prepare_continuous_color_scale <- function(
    data,
    column,
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    bg_cutoff = NULL
) {
    if (!is.null(bg_cutoff)) {
        data[[column]][data[[column]] <= bg_cutoff] <- NA
    }

    if (all(is.na(data[[column]]))) {
        feat_colors_value <- rep(0, 100)
        feat_colors_value[1] <- 0
        feat_colors_value[100] <- 1e-3
    } else {
        lower_cutoff <- lower_cutoff %||%
            quantile(
                data[[column]][is.finite(data[[column]])],
                lower_quantile,
                na.rm = TRUE
            )
        upper_cutoff <- upper_cutoff %||%
            quantile(
                data[[column]][is.finite(data[[column]])],
                upper_quantile,
                na.rm = TRUE
            )
        if (upper_cutoff == lower_cutoff) {
            if (upper_cutoff == 0) {
                upper_cutoff <- 1e-3
            } else {
                upper_cutoff <- upper_cutoff + upper_cutoff * 1e-3
            }
        }
        if (upper_cutoff - lower_cutoff < 1e-6) {
            upper_cutoff <- lower_cutoff + 1e-3
        }
        feat_colors_value <- seq(
            lower_cutoff,
            upper_cutoff,
            length.out = 100
        )
    }

    data[[column]][
        data[[column]] > max(feat_colors_value, na.rm = TRUE)
    ] <- max(feat_colors_value, na.rm = TRUE)
    data[[column]][
        data[[column]] < min(feat_colors_value, na.rm = TRUE)
    ] <- min(feat_colors_value, na.rm = TRUE)

    list(
        data = data,
        feat_colors_value = feat_colors_value,
        limits = range(feat_colors_value)
    )
}

set.seed(8525)

# check_columns tests
test_that("check_columns returns column name when it exists", {
    df <- data.frame(a = 1:3, b = 4:6)
    result <- check_columns(df, "a")
    expect_equal(result, "a")
})

test_that("check_columns returns NULL when columns is NULL", {
    df <- data.frame(a = 1:3)
    result <- check_columns(df, NULL)
    expect_null(result)
})

test_that("check_columns errors when column not found", {
    df <- data.frame(a = 1:3)
    expect_error(check_columns(df, "b"), "not in the data")
})

test_that("check_columns errors when multiple columns and allow_multi = FALSE", {
    df <- data.frame(a = 1:3, b = 4:6)
    expect_error(check_columns(df, c("a", "b")), "Only one column")
})

test_that("check_columns allows multiple columns when allow_multi = TRUE", {
    df <- data.frame(a = 1:3, b = 4:6)
    result <- check_columns(df, c("a", "b"), allow_multi = TRUE)
    expect_equal(result, c("a", "b"))
})

test_that("check_columns concatenates when concat_multi = TRUE", {
    df <- data.frame(a = c("x", "y"), b = c("1", "2"))
    result <- check_columns(df, c("a", "b"), allow_multi = TRUE, concat_multi = TRUE)
    expect_equal(result, "a_b")
})

test_that("check_columns force_factor converts column to factor", {
    df <- data.frame(x = c("a", "b", "c"), y = 1:3)
    check_columns(df, "x", force_factor = TRUE)
    # After check_columns, the column should be factor in parent frame
    expect_true(is.character(df$x) || is.factor(df$x))
})

# calc_just tests
test_that("calc_just returns list with h and v for angle 0", {
    result <- calc_just(0)
    expect_true(is.list(result))
    expect_true("h" %in% names(result))
    expect_true("v" %in% names(result))
    expect_equal(result$h, 0.5)
    expect_equal(result$v, 1)
})

test_that("calc_just returns correct values for angle 45", {
    result <- calc_just(45)
    expect_equal(result$h, 1)
    expect_equal(result$v, 1)
})

test_that("calc_just returns correct values for angle 90", {
    result <- calc_just(90)
    expect_equal(result$h, 1)
    expect_equal(result$v, 0.5)
})

test_that("calc_just handles negative angle (mod 360)", {
    result_neg <- calc_just(-90)
    result_pos <- calc_just(270)
    expect_equal(result_neg, result_pos)
})

test_that("calc_just handles angle > 360", {
    result_over <- calc_just(405)  # 405 = 360 + 45
    result_norm <- calc_just(45)
    expect_equal(result_over, result_norm)
})

# norm_expansion tests
test_that("norm_expansion returns list with x and y keys", {
    result <- norm_expansion(NULL, "discrete", "continuous")
    expect_true(is.list(result))
    expect_true("x" %in% names(result))
    expect_true("y" %in% names(result))
})

test_that("norm_expansion with single value sets all four sides", {
    result <- norm_expansion(c(0.1), "continuous", "continuous")
    expect_true(is.list(result))
    expect_length(result$x, 4)
})

test_that("norm_expansion with two values works", {
    result <- norm_expansion(c(0.1, 0.2), "continuous", "continuous")
    expect_true(is.list(result))
})

test_that("norm_expansion errors for conflicting x and left/right", {
    expect_error(
        norm_expansion(c(x = 0.1, left = 0.2), "continuous", "continuous"),
        "Cannot have both"
    )
})

# # process_trans tests
# test_that("process_trans returns identity for NULL trans", {
#     res <- process_trans(NULL, null_as_identity = TRUE)
#     expect_identical(res$name, "identity")
#     expect_equal(res$trans(5), 5)

#     res <- process_trans(NULL, null_as_identity = FALSE)
#     expect_null(res$trans)
#     expect_null(res$name)
# })

# test_that("process_trans resolves string transformations", {
#     res <- process_trans("log2")
#     expect_equal(res$trans(8), 3)
#     expect_equal(res$name, "log2")
# })

# test_that("process_trans handles negated string transformations", {
#     res <- process_trans("-log2")
#     expect_equal(res$trans(8), -3)
#     expect_equal(res$name, "-log2")
# })

# test_that("process_trans accepts function transformations", {
#     res <- process_trans(function(x) x + 1)
#     expect_equal(res$trans(1), 2)
#     expect_equal(res$name, "transformed")
# })

# test_that("process_trans errors on invalid trans", {
#     expect_error(process_trans(1), "must be a string, a function, or NULL")
# })

# test_that("process_trans uses provided trans_name", {
#     res <- process_trans("log2", trans_name = "custom")
#     expect_equal(res$name, "custom")
#     res <- process_trans(function(x) x, trans_name = "custom")
#     expect_equal(res$name, "custom")
# })

# calculate_plot_dimensions tests
test_that("calculate_plot_dimensions returns list with height and width", {
    result <- calculate_plot_dimensions(base_height = 4.5, aspect.ratio = 1)
    expect_true(is.list(result))
    expect_true("height" %in% names(result))
    expect_true("width" %in% names(result))
})

test_that("calculate_plot_dimensions with n_x returns numeric dimensions", {
    result <- calculate_plot_dimensions(n_x = 5)
    expect_true(is.numeric(result$height))
    expect_true(is.numeric(result$width))
    expect_true(result$width > 0)
    expect_true(result$height > 0)
})

test_that("calculate_plot_dimensions with n_y returns numeric dimensions", {
    result <- calculate_plot_dimensions(n_y = 8)
    expect_true(is.numeric(result$height))
    expect_true(is.numeric(result$width))
})

test_that("calculate_plot_dimensions respects min bounds", {
    result <- calculate_plot_dimensions(n_x = 1, min_width = 3, min_height = 3)
    expect_true(result$width >= 3 || result$height >= 3)
})

# blend_colors tests
test_that("blend_colors blends two colors using blend mode", {
    result <- blend_colors(c("red", "blue"), mode = "blend")
    expect_true(grepl("^#[0-9A-Fa-f]{6}", result))
    expect_true(is.character(result))
    expect_length(result, 1)
})

test_that("blend_colors works with average mode", {
    result <- blend_colors(c("red", "blue"), mode = "average")
    expect_true(grepl("^#[0-9A-Fa-f]{6}", result))
})

test_that("blend_colors works with screen mode", {
    result <- blend_colors(c("red", "blue"), mode = "screen")
    expect_true(grepl("^#[0-9A-Fa-f]{6}", result))
})

test_that("blend_colors with single color returns same color", {
    result <- blend_colors("red")
    expect_equal(result, "red")
})

test_that("blend_colors returns NA for empty input", {
    result <- blend_colors(c(NA, NA))
    expect_true(is.na(result))
})

test_that("check_keep_na works for atomic values", {
    expect_equal(check_keep_na(TRUE), NA)
    expect_equal(check_keep_na(FALSE), FALSE)
    expect_equal(check_keep_na("missing"), "missing")
})

test_that("check_keep_na works for named list", {
    res <- check_keep_na(list(col1 = TRUE, col2 = FALSE, col3 = "missing"))
    expect_equal(res$col1, NA)
    expect_equal(res$col2, FALSE)
    expect_equal(res$col3, "missing")
})

test_that("check_keep_na works with default column", {
    res <- check_keep_na(list(col1 = TRUE, col2 = FALSE), cols = "missing")
    expect_equal(res$col1, NA)
    expect_equal(res$col2, FALSE)
    expect_equal(res$missing, FALSE)
    expect_equal(length(res), 3)

    res <- check_keep_na(TRUE, cols = "missing")
    expect_equal(res, list(missing = NA))
})

test_that("check_keep_empty works for atomic values", {
    expect_equal(check_keep_empty(TRUE), TRUE)
    expect_equal(check_keep_empty(TRUE), TRUE)
    expect_equal(check_keep_empty(FALSE), FALSE)
    expect_equal(check_keep_empty(FALSE), FALSE)
    expect_equal(check_keep_empty("level"), "level")
    expect_equal(check_keep_empty("levels"), "level")
})

test_that("check_keep_empty works for named list", {
    res <- check_keep_empty(list(col1 = TRUE, col2 = FALSE, col3 = "level"))
    expect_equal(res$col1, TRUE)
    expect_equal(res$col2, FALSE)
    expect_equal(res$col3, "level")
})

test_that("check_keep_empty works with default column", {
    res <- check_keep_empty(list(col1 = TRUE, col2 = FALSE), cols = "level")
    expect_equal(res$col1, TRUE)
    expect_equal(res$col2, FALSE)
    expect_equal(res$level, FALSE)
    expect_equal(length(res), 3)

    res <- check_keep_empty(TRUE, cols = "level")
    expect_equal(res, list(level = TRUE))
})

test_that("process_keep_na_empty keep_na works", {
    data <- data.frame(
        col1 = factor(c("A", "B", NA, "A", "C", NA)),
        col2 = c(1, 2, NA, 4, 5, NA)
    )
    keep_na <- list(col1 = "<NA>", col2 = "missing")
    processed_data <- process_keep_na_empty(data, keep_na)
    expect_equal(as.character(processed_data$col1), c("A", "B", "<NA>", "A", "C", "<NA>"))
    expect_equal(processed_data$col2, c(1, 2, "missing", 4, 5, "missing"))

    data <- data.frame(
        col1 = factor(c("A", "B", NA, "A", "C", NA)),
        col2 = c(1, 2, NA, 4, 5, NA)
    )
    keep_na <- list(col1 = FALSE, col2 = FALSE)
    processed_data <- process_keep_na_empty(data, keep_na)
    expect_equal(as.character(processed_data$col1), c("A", "B", "A", "C"))
    expect_equal(processed_data$col2, c(1, 2, 4, 5))
})

test_that("process_keep_na_empty keep_na works with given column", {
    data <- data.frame(
        col1 = factor(c("A", "B", NA, "A", "C", NA)),
        col2 = c(1, 2, NA, 4, 5, NA)
    )
    keep_na <- list(col1 = "<NA>", col2 = "missing")
    processed_data <- process_keep_na_empty(data, keep_na, col = "col1")
    expect_equal(as.character(processed_data$col1), c("A", "B", "<NA>", "A", "C", "<NA>"))
    expect_equal(processed_data$col2, c(1, 2, NA, 4, 5, NA))
})

test_that("process_keep_na_empty keep_empty works", {
    data <- data.frame(
        col1 = factor(c("A", "B"), levels = c("A", "B", "C", "D", "E")),
        col2 = factor(c("X", "Y"), levels = c("X", "Y", "Z"))
    )
    keep_empty <- list(col1 = "level", col2 = TRUE)
    processed_data <- process_keep_na_empty(data, keep_empty = keep_empty)
    expect_equal(levels(processed_data$col1), c("A", "B", "C", "D", "E"))
    expect_equal(levels(processed_data$col2), c("X", "Y", "Z"))

    keep_empty <- list(col1 = FALSE, col2 = FALSE)
    processed_data <- process_keep_na_empty(data, keep_empty = keep_empty)
    expect_equal(levels(processed_data$col1), c("A", "B"))
    expect_equal(levels(processed_data$col2), c("X", "Y"))
})

test_that("process_keep_na_empty keep_empty works with given column", {
    data <- data.frame(
        col1 = factor(c("A", "B"), levels = c("A", "B", "C", "D", "E")),
        col2 = factor(c("X", "Y"), levels = c("X", "Y", "Z"))
    )
    keep_empty <- list(col1 = "level", col2 = TRUE)
    processed_data <- process_keep_na_empty(data, keep_empty = keep_empty, col = "col1")
    expect_equal(levels(processed_data$col1), c("A", "B", "C", "D", "E"))
    expect_equal(levels(processed_data$col2), c("X", "Y", "Z"))

    keep_empty <- list(col1 = FALSE, col2 = FALSE)
    processed_data <- process_keep_na_empty(data, keep_empty = keep_empty, col = "col2")
    expect_equal(levels(processed_data$col1), c("A", "B", "C", "D", "E"))
    expect_equal(levels(processed_data$col2), c("X", "Y"))
})

#' Join the meta data to the main data frame for heatmap
#'
#' @param data A data frame containing the main data for the heatmap.
#' @param meta_data A data frame containing the meta data to be joined.
#' @param by A character string specifying the column name in `meta_data` to join on.
#' Either `rows_by` or `columns_by` should be specified in `data`.
#' @param cr_split_by A character string specifying the column name in `data` to join on.
#' Either `rows_split_by` or `columns_split_by` should be specified in `data`.
#' @param split_by A character string specifying the column name in `data` to join on.
#' Used to split the data into multiple heatmaps.
#' @param which A character string specifying whether to join on rows or columns.
#' Can be either `"row"` or `"column"`.
#' @importFrom dplyr left_join
#' @return A data frame with the meta data joined to the main data.
#' @keywords internal
join_heatmap_meta <- function(
    data,
    meta_data,
    by,
    cr_split_by,
    split_by,
    which
) {
    if (!by %in% colnames(meta_data)) {
        stop(sprintf(
            "[Heatmap] '%ss_by' (%s) must be a column in '%ss_data'.",
            which,
            by,
            which
        ))
    }

    join_by <- by
    if (
        !is.null(cr_split_by) &&
            cr_split_by %in% colnames(data) &&
            cr_split_by %in% colnames(meta_data)
    ) {
        # if split_by is in both data and meta_data, we join by both
        join_by <- c(by, cr_split_by)
    }
    if (
        !is.null(split_by) &&
            split_by %in% colnames(data) &&
            split_by %in% colnames(meta_data)
    ) {
        # if split_by is in both data and meta_data, we join by both
        join_by <- c(join_by, split_by)
    }

    out <- dplyr::left_join(
        data,
        meta_data,
        by = join_by,
        suffix = c("", paste0(".", which))
    )
    # factor lost when joining, so we need to restore the levels
    out[[by]] <- factor(out[[by]], levels = levels(data[[by]]))
    if (!is.null(cr_split_by) && cr_split_by %in% join_by) {
        if (is.factor(data[[cr_split_by]])) {
            out[[cr_split_by]] <- factor(
                out[[cr_split_by]],
                levels = levels(data[[cr_split_by]])
            )
        } else if (is.factor(meta_data[[cr_split_by]])) {
            # if the split_by is a factor in meta_data, we need to restore the levels
            out[[cr_split_by]] <- factor(
                out[[cr_split_by]],
                levels = levels(meta_data[[cr_split_by]])
            )
        } else {
            # if the split_by is not a factor, we convert it to a factor
            out[[cr_split_by]] <- factor(
                out[[cr_split_by]],
                levels = unique(out[[cr_split_by]])
            )
        }
    }
    if (!is.null(split_by) && split_by %in% join_by) {
        if (is.factor(data[[split_by]])) {
            out[[split_by]] <- factor(
                out[[split_by]],
                levels = levels(data[[split_by]])
            )
        } else if (is.factor(meta_data[[split_by]])) {
            # if the split_by is a factor in meta_data, we need to restore the levels
            out[[split_by]] <- factor(
                out[[split_by]],
                levels = levels(meta_data[[split_by]])
            )
        } else {
            # if the split_by is not a factor, we convert it to a factor
            out[[split_by]] <- factor(
                out[[split_by]],
                levels = unique(out[[split_by]])
            )
        }
    }

    out
}

#' Apply ordering to a data frame based on a specified expression
#' @param df A data frame to be ordered.
#' @param orderby A character string representing the expression to order by.
#' @param by A character string representing the column name to group by.
#' @param sby A character string representing the column name to group by for secondary ordering.
#' @param sby_levels A character vector representing the levels of the secondary grouping column.
#' @return A data frame ordered based on the specified expression.
#' @keywords internal
.apply_orderby <- function(df, orderby, by, sby, sby_levels) {
    if (!is.null(orderby)) {
        by_levels <- df %>%
            group_by(!!sym(by)) %>%
            summarise(.orderby = !!parse_expr(orderby)) %>%
            arrange(!!sym(".orderby")) %>%
            pull(!!sym(by)) %>%
            as.character() %>%
            unique()
        df[[by]] <- factor(df[[by]], levels = by_levels)

        if (!is.null(sby) && is.null(sby_levels)) {
            sby_levels <- df %>%
                group_by(!!sym(sby)) %>%
                summarise(.orderby = !!parse_expr(orderby)) %>%
                arrange(!!sym(".orderby")) %>%
                pull(!!sym(sby)) %>%
                as.character() %>%
                unique()
            df[[sby]] <- factor(df[[sby]], levels = sby_levels)
        }
    }
    df
}

#' Process/normalize data passed to [Heatmap()]
#'
#' This function is used to process the data passed to [Heatmap()].
#' @param data A data frame or matrix containing the data to be plotted.
#' Based on the `in_form`, the data can have the following formats:
#' * `matrix`: A matrix with rows and columns directly representing the heatmap.
#' * `long`: A data frame in long format with columns for values, rows, and columns.
#' * `wide-rows`: A data frame in wide format with columns for heatmap rows and values,
#'    and a single column for heatmap columns.
#' * `wide-columns`: A data frame in wide format with columns for heatmap columns and values,
#'    and a single column for heatmap rows.
#' * `auto`: Automatically inferred from the data format.
#'    When `data` is a matrix, `in_form` is set to `"matrix"`. When `columns_by` has more than one column,
#'    `in_form` is set to `"wide-columns"`. When `rows_by` has more than one column,
#'    `in_form` is set to `"wide-rows"`. Otherwise, it is set to `"long"`.
#' @param in_form The format of the data. Can be one of `"matrix"`, `"long"`, `"wide-rows"`, `"wide-columns"`, or `"auto"`.
#' Defaults to `"auto"`.
#' @param values_by A character of column name in `data` that contains the values to be plotted.
#' This is required when `in_form` is `"long"`. For other formats, the values are pivoted into a column named by `values_by`.
#' @param name A character string to name the heatmap (will be used to rename `values_by`).
#' @param split_by A character of column name in `data` that contains the split information to split into multiple heatmaps.
#' This is used to create a list of heatmaps, one for each level of the split.
#' Defaults to `NULL`, meaning no split.
#' @param split_by_sep A character string to concat multiple columns in `split_by`.
#' @param rows_by A vector of column names in `data` that contains the row information.
#' This is used to create the rows of the heatmap.
#' When `in_form` is `"long"` or `"wide-columns"`, this is requied, and multiple columns can be specified,
#' which will be concatenated by `rows_by_sep` into a single column.
#' @param rows_by_sep A character string to concat multiple columns in `rows_by`.
#' @param rows_name A character string to rename the column created by `rows_by`, which will be reflected in the name of the annotation or legend.
#' @param rows_split_by A character of column name in `data` that contains the split information for rows.
#' @param rows_split_by_sep A character string to concat multiple columns in `rows_split_by`.
#' @param rows_split_name A character string to rename the column created by `rows_split_by`, which will be reflected in the name of the annotation or legend.
#' @param columns_by A vector of column names in `data` that contains the column information.
#' This is used to create the columns of the heatmap.
#' When `in_form` is `"long"` or `"wide-rows"`, this is required, and multiple columns can be specified,
#' which will be concatenated by `columns_by_sep` into a single column.
#' @param columns_by_sep A character string to concat multiple columns in `columns_by`.
#' @param columns_name A character string to rename the column created by `columns_by`, which will be reflected in the name of the annotation or legend.
#' @param columns_split_by A character of column name in `data` that contains the split information for columns.
#' @param columns_split_by_sep A character string to concat multiple columns in `columns_split_by`.
#' @param columns_split_name A character string to rename the column created by `columns_split_by`, which will be reflected in the name of the annotation or legend.
#' @param rows_orderby A expression (in character) to specify how to order rows. It will be evaluated in the context of the data frame used for rows (after grouping by rows_split_by and rows_by). The expression should return a vector of the same length as the number of rows in the data frame. The default is NULL, which means no specific ordering.
#' Can't be used with cluster_rows = TRUE.
#' This is applied before renaming rows_by to rows_name.
#' @param columns_orderby A expression (in character) to specify how to order columns. It will be evaluated in the context of the data frame used for columns (after grouping by columns
#' split_by and columns_by). The expression should return a vector of the same length as the number of rows in the data frame. The default is NULL, which means no specific ordering.
#' Can't be used with cluster_columns = TRUE.
#' This is applied before renaming columns_by to columns_name.
#' @param pie_group_by A character of column name in `data` that contains the group information for pie charts.
#' This is used to create pie charts in the heatmap when `cell_type` is `"pie"`.
#' @param pie_group_by_sep A character string to concat multiple columns in `pie_group_by`.
#' @param pie_name A character string to rename the column created by `pie_group_by`, which will be reflected in the name of the annotation or legend.
#' @param rows_data A data frame containing additional data for rows, which can be used to add annotations to the heatmap.
#' It will be joined to the main data by `rows_by` and `split_by` if `split_by` exists in `rows_data`.
#' This is useful for adding additional information to the rows of the heatmap.
#' @param columns_data A data frame containing additional data for columns, which can be used to add annotations to the heatmap.
#' It will be joined to the main data by `columns_by` and `split_by` if `split_by` exists in `columns_data`.
#' This is useful for adding additional information to the columns of the heatmap.
#' @param keep_na Whether we should keep NA groups in rows, columns and split_by variables. Default is FALSE.
#' FALSE to remove NA groups; TRUE to keep NA groups.
#' A vector of column names can also be provided to specify which columns to keep NA groups.
#' Note that the record will be removed if any of the grouping columns has NA and is not specified to keep NA.
#' @return A list containing the processed data and metadata:
#' * `data`: A list of data frames, one for each level of `split_by`. If no `split_by` is provided, the name will be `"..."`.
#'    Each data frame is in the long format.
#' * `values_by`: The name of the column containing the values to be plotted.
#' * `rows_by`: The name of the column containing the row information.
#' * `rows_split_by`: The name of the column containing the row split information.
#' * `columns_by`: The name of the column containing the column information.
#' * `columns_split_by`: The name of the column containing the column split information.
#' * `pie_group_by`: The name of the column containing the pie group information.
#' @importFrom rlang sym syms %||% parse_expr
#' @importFrom dplyr arrange group_by summarise pull all_of
#' @keywords internal
process_heatmap_data <- function(
    data,
    in_form,
    values_by,
    name,
    split_by,
    split_by_sep,
    rows_orderby,
    columns_orderby,
    rows_by,
    rows_by_sep,
    rows_name,
    rows_split_by,
    rows_split_by_sep,
    rows_split_name,
    columns_by,
    columns_by_sep,
    columns_name,
    columns_split_by,
    columns_split_by_sep,
    columns_split_name,
    pie_group_by,
    pie_group_by_sep,
    pie_name,
    rows_data,
    columns_data,
    keep_na,
    keep_empty
) {
    if (identical(rows_by, columns_by) && !is.null(rows_by)) {
        stop("[Heatmap] 'rows_by' and 'columns_by' can not be the same.")
    }
    stopifnot("[Heatmap] no data is presented (nrow == 0)." = nrow(data) > 0)
    # Infer in_form
    if (in_form == "auto") {
        if (is.matrix(data)) {
            in_form <- "matrix"
        } else if (length(rows_by) > 1) {
            in_form <- "wide-rows"
        } else if (length(columns_by) > 1) {
            in_form <- "wide-columns"
        } else {
            in_form <- "long"
        }
    }

    if (in_form == "matrix") {
        stopifnot(
            "[Heatmap] 'split_by' is not supported when 'in_form = \"matrix\"'." = is.null(
                split_by
            )
        )
        stopifnot(
            "[Heatmap] 'rows_by' is not supported when 'in_form = \"matrix\"'." = is.null(
                rows_by
            )
        )
        stopifnot(
            "[Heatmap] 'columns_by' is not supported when 'in_form = \"matrix\"'." = is.null(
                columns_by
            )
        )
        stopifnot(
            "[Heatmap] 'pie_group_by' is not supported when 'in_form = \"matrix\"'." = is.null(
                pie_group_by
            )
        )
        stopifnot(
            "[Heatmap] 'keep_na' is not supported when 'in_form = \"matrix\"'." = isFALSE(
                keep_na
            )
        )

        rows_name <- rows_name %||% "rows"
        data <- as.data.frame(data)
        columns_by <- colnames(data)
        data[rows_name] <- rownames(data)
        rows_by <- rows_name

        in_form <- "wide-columns"
    }

    if (identical(rows_name %||% rows_by, columns_name %||% columns_by)) {
        if (!is.null(columns_name)) {
            # consider flip and names_side?
            columns_name <- paste0(columns_name, " ")
        } else {
            rows_name <- paste0(" ", rows_name)
        }
    }

    # pie_group_by should be always in the main data
    pie_group_by <- check_columns(
        data,
        pie_group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = pie_group_by_sep
    )

    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (in_form == "long") {
        # values_by
        values_by <- check_columns(data, values_by)
        stopifnot(
            "[Heatmap] 'values_by' must be specified when 'in_form = \"long\"'." = !is.null(
                values_by
            )
        )

        # rows_by/rows_split_by
        rows_by <- check_columns(
            data,
            rows_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = rows_by_sep
        )
        stopifnot(
            "[Heatmap] 'rows_by' must be specified when 'in_form = \"long\"'." = !is.null(
                rows_by
            )
        )

        # columns_by/columns_split_by
        columns_by <- check_columns(
            data,
            columns_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = columns_by_sep
        )
        stopifnot(
            "[Heatmap] 'columns_by' must be specified when 'in_form = \"long\"'." = !is.null(
                columns_by
            )
        )

        # join rows_data/columns_data
        if (!is.null(rows_data)) {
            data <- join_heatmap_meta(
                data,
                rows_data,
                by = rows_by,
                cr_split_by = rows_split_by,
                split_by = split_by,
                which = "row"
            )
        }
        rows_split_levels <- if (
            length(rows_split_by) == 1 && is.factor(data[[rows_split_by]])
        ) {
            levels(data[[rows_split_by]])
        } else {
            NULL
        }
        rows_split_by <- check_columns(
            data,
            rows_split_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = rows_split_by_sep
        )

        if (!is.null(columns_data)) {
            data <- join_heatmap_meta(
                data,
                columns_data,
                by = columns_by,
                cr_split_by = columns_split_by,
                split_by = split_by,
                which = "column"
            )
        }
        columns_split_leves <- if (
            length(columns_split_by) == 1 && is.factor(data[[columns_split_by]])
        ) {
            levels(data[[columns_split_by]])
        } else {
            NULL
        }
        columns_split_by <- check_columns(
            data,
            columns_split_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = columns_split_by_sep
        )

        data <- .apply_orderby(
            data,
            rows_orderby,
            rows_by,
            rows_split_by,
            rows_split_levels
        )
        data <- .apply_orderby(
            data,
            columns_orderby,
            columns_by,
            columns_split_by,
            columns_split_leves
        )

        # rename
        if (!is.null(rows_name)) {
            if (identical(rows_name, "")) {
                rows_name <- " "
            }
            data <- dplyr::rename(data, !!sym(rows_name) := all_of(rows_by))
            rows_by <- rows_name
        }
        if (!is.null(columns_name)) {
            if (identical(columns_name, "")) {
                columns_name <- " "
            }
            data <- dplyr::rename(data, !!sym(columns_name) := all_of(columns_by))
            columns_by <- columns_name
        }
    } else if (in_form == "wide-rows") {
        # columns_split_by columns_by pie_group_by rows1 rows2 ...
        # csb1             cb1        pgb1         0.1   0.2   ...
        # csb2             cb2        pgb2         0.3   0.4   ...
        #                    ... ...
        # rows_by
        rows_by <- rows_by %||%
            setdiff(
                colnames(data),
                c(columns_by, columns_split_by, pie_group_by)
            )
        rows_name <- rows_name %||%
            ifelse("rows" %in% colnames(data), "rows.1", "rows")
        values_by <- values_by %||%
            ifelse("value" %in% colnames(data), "value.1", "value")
        data <- tidyr::pivot_longer(
            data,
            cols = all_of(rows_by),
            names_to = rows_name,
            values_to = values_by
        )
        data[[rows_name]] <- factor(data[[rows_name]], levels = unique(rows_by))
        data <- data[order(data[[rows_name]]), , drop = FALSE]
        rows_by <- rows_name

        # columns_by/columns_split_by
        columns_by <- check_columns(
            data,
            columns_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = columns_by_sep
        )
        stopifnot(
            "[Heatmap] 'columns_by' must be specified when 'in_form = \"wide-rows\"'." = !is.null(
                columns_by
            )
        )

        # rows_data/columns_data
        if (!is.null(rows_data)) {
            data <- join_heatmap_meta(
                data,
                rows_data,
                by = rows_by,
                cr_split_by = rows_split_by,
                split_by = split_by,
                which = "row"
            )
        }
        rows_split_levels <- if (
            length(rows_split_by) == 1 && is.factor(data[[rows_split_by]])
        ) {
            levels(data[[rows_split_by]])
        } else {
            NULL
        }
        rows_split_by <- check_columns(
            data,
            rows_split_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = rows_split_by_sep
        )

        if (!is.null(columns_data)) {
            data <- join_heatmap_meta(
                data,
                columns_data,
                by = columns_by,
                cr_split_by = columns_split_by,
                split_by = split_by,
                which = "column"
            )
        }
        columns_split_levels <- if (
            length(columns_split_by) == 1 && is.factor(data[[columns_split_by]])
        ) {
            levels(data[[columns_split_by]])
        } else {
            NULL
        }
        columns_split_by <- check_columns(
            data,
            columns_split_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = columns_split_by_sep
        )

        data <- .apply_orderby(
            data,
            rows_orderby,
            rows_by,
            rows_split_by,
            rows_split_levels
        )
        data <- .apply_orderby(
            data,
            columns_orderby,
            columns_by,
            columns_split_by,
            columns_split_levels
        )

        if (!is.null(columns_name)) {
            if (identical(columns_name, "")) {
                columns_name <- " "
            }
            data <- dplyr::rename(data, !!sym(columns_name) := all_of(columns_by))
            columns_by <- columns_name
        }
    } else {
        # wide-columns
        # rows_split_by rows_by pie_group_by columns1 columns2 ...
        # rsb1          cb1        pgb1         0.1   0.2   ...
        # rsb2          cb2        pgb2         0.3   0.4   ...
        #                    ... ...
        # rows_by/rows_split_by
        rows_by <- check_columns(
            data,
            rows_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = rows_by_sep
        )
        stopifnot(
            "[Heatmap] 'rows_by' must be specified when 'in_form = \"wide-columns\"'." = !is.null(
                rows_by
            )
        )

        # columns_by
        columns_by <- columns_by %||%
            setdiff(colnames(data), c(rows_by, rows_split_by, pie_group_by))
        columns_name <- columns_name %||%
            ifelse("columns" %in% colnames(data), "columns.1", "columns")
        values_by <- values_by %||%
            ifelse("value" %in% colnames(data), "value.1", "value")
        data <- tidyr::pivot_longer(
            data,
            cols = all_of(columns_by),
            names_to = columns_name,
            values_to = values_by
        )
        data[[columns_name]] <- factor(
            data[[columns_name]],
            levels = columns_by
        )
        data <- data[order(data[[columns_name]]), , drop = FALSE]
        columns_by <- columns_name

        # rows_data/columns_data
        if (!is.null(rows_data)) {
            data <- join_heatmap_meta(
                data,
                rows_data,
                by = rows_by,
                cr_split_by = rows_split_by,
                split_by = split_by,
                which = "row"
            )
        }
        rows_split_levels <- if (
            length(rows_split_by) == 1 && is.factor(data[[rows_split_by]])
        ) {
            levels(data[[rows_split_by]])
        } else {
            NULL
        }
        rows_split_by <- check_columns(
            data,
            rows_split_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = rows_split_by_sep
        )

        if (!is.null(columns_data)) {
            data <- join_heatmap_meta(
                data,
                columns_data,
                by = columns_by,
                cr_split_by = columns_split_by,
                split_by = split_by,
                which = "column"
            )
        }
        columns_split_leves <- if (
            length(columns_split_by) == 1 && is.factor(data[[columns_split_by]])
        ) {
            levels(data[[columns_split_by]])
        } else {
            NULL
        }
        columns_split_by <- check_columns(
            data,
            columns_split_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = columns_split_by_sep
        )

        data <- .apply_orderby(
            data,
            rows_orderby,
            rows_by,
            rows_split_by,
            rows_split_levels
        )
        data <- .apply_orderby(
            data,
            columns_orderby,
            columns_by,
            columns_split_by,
            columns_split_leves
        )

        if (!is.null(rows_name)) {
            if (identical(rows_name, "")) {
                rows_name <- " "
            }
            data <- dplyr::rename(data, !!sym(rows_name) := all_of(rows_by))
            rows_by <- rows_name
        }
    }

    if (!is.null(rows_split_name) && !is.null(rows_split_by)) {
        if (identical(rows_split_name, "")) {
            rows_split_name <- " "
        }
        data <- dplyr::rename(data, !!sym(rows_split_name) := all_of(rows_split_by))
        rows_split_by <- rows_split_name
    }
    if (!is.null(columns_split_name) && !is.null(columns_split_by)) {
        if (identical(columns_split_name, "")) {
            columns_split_name <- " "
        }
        data <- dplyr::rename(
            data,
            !!sym(columns_split_name) := all_of(columns_split_by)
        )
        columns_split_by <- columns_split_name
    }
    if (!is.null(pie_name) && !is.null(pie_group_by)) {
        if (identical(pie_name, "")) {
            pie_name <- " "
        }
        data <- dplyr::rename(data, !!sym(pie_name) := all_of(pie_group_by))
        pie_group_by <- pie_name
    }
    if (!is.null(name)) {
        if (identical(name, "")) {
            name <- " "
        }
        data <- dplyr::rename(data, !!sym(name) := !!sym(values_by))
        values_by <- name
    }
    keep_na <- check_keep_na(
        keep_na,
        c(split_by, rows_by, rows_split_by, columns_by, columns_split_by)
    )
    if (!isFALSE(keep_empty)) {
        warning(
            "[Heatmap] 'keep_empty != FALSE' is not supported yet, and unused levels will be always dropped."
        )
    }
    data <- process_keep_na_empty(data, keep_na, list())

    list(
        data = if (is.null(split_by)) {
            stats::setNames(list(data), "...")
        } else {
            split(select(data, -!!sym(split_by)), data[[split_by]])
        },
        values_by = values_by,
        rows_by = rows_by,
        rows_split_by = rows_split_by,
        columns_by = columns_by,
        columns_split_by = columns_split_by,
        pie_group_by = pie_group_by,
        keep_na = keep_na,
        keep_empty = keep_empty
    )
}

#' Process data for LinkedHeatmap
#' @keywords internal
process_linkedheatmap_data <- function(
    data,
    split_by,
    split_by_sep,
    rows_split_by,
    rows_split_by_sep,
    rows_split_name,
    left_values_by,
    left_name,
    left_rows_by,
    left_rows_by_sep,
    left_rows_name,
    left_rows_orderby,
    left_columns_orderby,
    left_columns_by,
    left_columns_by_sep,
    left_columns_name,
    left_columns_split_by,
    left_columns_split_by_sep,
    left_columns_split_name,
    left_pie_group_by,
    left_pie_group_by_sep,
    left_pie_name,
    left_rows_data,
    left_columns_data,
    right_values_by,
    right_name,
    right_rows_by,
    right_rows_by_sep,
    right_rows_name,
    right_rows_orderby,
    right_columns_orderby,
    right_columns_by,
    right_columns_by_sep,
    right_columns_name,
    right_columns_split_by,
    right_columns_split_by_sep,
    right_columns_split_name,
    right_pie_group_by,
    right_pie_group_by_sep,
    right_pie_name,
    right_rows_data,
    right_columns_data,
    keep_na,
    keep_empty
) {
    if (!is.null(left_rows_by) && identical(left_rows_by, left_columns_by)) {
        stop(
            "[LinkedHeatmap] 'left_rows_by' and 'left_columns_by' can not be the same."
        )
    }
    if (!is.null(right_rows_by) && identical(right_rows_by, right_columns_by)) {
        stop(
            "[LinkedHeatmap] 'right_rows_by' and 'right_columns_by' can not be the same."
        )
    }
    stopifnot(
        "[LinkedHeatmap] no data is presented (nrow == 0)." = nrow(data) > 0
    )

    if (
        identical(
            left_rows_name %||% left_rows_by,
            left_columns_name %||% left_columns_by
        )
    ) {
        if (!is.null(left_columns_name)) {
            # consider flip and names_side?
            left_columns_name <- paste0(left_columns_name, " ")
        } else {
            left_rows_name <- paste0(" ", left_rows_name)
        }
    }
    if (
        identical(
            right_rows_name %||% right_rows_by,
            right_columns_name %||% right_columns_by
        )
    ) {
        if (!is.null(right_columns_name)) {
            # consider flip and names_side?
            right_columns_name <- paste0(right_columns_name, " ")
        } else {
            right_rows_name <- paste0(" ", right_rows_name)
        }
    }

    left_pie_group_by <- check_columns(
        data,
        left_pie_group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = left_pie_group_by_sep
    )

    right_pie_group_by <- check_columns(
        data,
        right_pie_group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = right_pie_group_by_sep
    )

    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    left_values_by <- check_columns(data, left_values_by)
    right_values_by <- check_columns(data, right_values_by)

    stopifnot(
        "[LinkedHeatmap] 'left_values_by/values_by' must be specified." = !is.null(
            left_values_by
        )
    )
    stopifnot(
        "[LinkedHeatmap] 'right_values_by/values_by' must be specified." = !is.null(
            right_values_by
        )
    )

    left_rows_by <- check_columns(
        data,
        left_rows_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = left_rows_by_sep
    )
    stopifnot(
        "[LinkedHeatmap] 'left_rows_by' must be specified." = !is.null(
            left_rows_by
        )
    )

    right_rows_by <- check_columns(
        data,
        right_rows_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = right_rows_by_sep
    )
    stopifnot(
        "[LinkedHeatmap] 'right_rows_by' must be specified." = !is.null(
            right_rows_by
        )
    )

    left_columns_by <- check_columns(
        data,
        left_columns_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = left_columns_by_sep
    )
    stopifnot(
        "[LinkedHeatmap] 'left_columns_by' must be specified." = !is.null(
            left_columns_by
        )
    )

    right_columns_by <- check_columns(
        data,
        right_columns_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = right_columns_by_sep
    )
    stopifnot(
        "[LinkedHeatmap] 'right_columns_by' must be specified." = !is.null(
            right_columns_by
        )
    )

    if (!is.null(left_rows_data)) {
        data <- join_heatmap_meta(
            data,
            left_rows_data,
            by = left_rows_by,
            cr_split_by = rows_split_by,
            split_by = split_by,
            which = "row"
        )
    }
    if (!is.null(right_rows_data)) {
        data <- join_heatmap_meta(
            data,
            right_rows_data,
            by = right_rows_by,
            cr_split_by = rows_split_by,
            split_by = split_by,
            which = "row"
        )
    }
    if (!is.null(left_columns_data)) {
        data <- join_heatmap_meta(
            data,
            left_columns_data,
            by = left_columns_by,
            cr_split_by = left_columns_split_by,
            split_by = split_by,
            which = "column"
        )
    }
    if (!is.null(right_columns_data)) {
        data <- join_heatmap_meta(
            data,
            right_columns_data,
            by = right_columns_by,
            cr_split_by = right_columns_split_by,
            split_by = split_by,
            which = "column"
        )
    }
    rows_split_levels <- if (
        length(rows_split_by) == 1 && is.factor(data[[rows_split_by]])
    ) {
        levels(data[[rows_split_by]])
    } else {
        NULL
    }
    rows_split_by <- check_columns(
        data,
        rows_split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = rows_split_by_sep
    )

    left_columns_split_levels <- if (
        length(left_columns_split_by) == 1 &&
            is.factor(data[[left_columns_split_by]])
    ) {
        levels(data[[left_columns_split_by]])
    } else {
        NULL
    }
    left_columns_split_by <- check_columns(
        data,
        left_columns_split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = left_columns_split_by_sep
    )

    right_columns_split_levels <- if (
        length(right_columns_split_by) == 1 &&
            is.factor(data[[right_columns_split_by]])
    ) {
        levels(data[[right_columns_split_by]])
    } else {
        NULL
    }
    right_columns_split_by <- check_columns(
        data,
        right_columns_split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = right_columns_split_by_sep
    )

    data <- .apply_orderby(
        data,
        left_rows_orderby,
        left_rows_by,
        rows_split_by,
        rows_split_levels
    )
    data <- .apply_orderby(
        data,
        left_columns_orderby,
        left_columns_by,
        left_columns_split_by,
        left_columns_split_levels
    )
    data <- .apply_orderby(
        data,
        right_rows_orderby,
        right_rows_by,
        rows_split_by,
        rows_split_levels
    )
    data <- .apply_orderby(
        data,
        right_columns_orderby,
        right_columns_by,
        right_columns_split_by,
        right_columns_split_levels
    )

    # rename
    if (!is.null(left_rows_name)) {
        if (identical(left_rows_name, "")) {
            left_rows_name <- " "
        }
        data <- dplyr::rename(data, !!sym(left_rows_name) := all_of(left_rows_by))
        left_rows_by <- left_rows_name
    }
    if (!is.null(left_columns_name)) {
        if (identical(left_columns_name, "")) {
            left_columns_name <- " "
        }
        data <- dplyr::rename(data, !!sym(left_columns_name) := all_of(left_columns_by))
        left_columns_by <- left_columns_name
    }
    if (!is.null(right_rows_name)) {
        if (identical(right_rows_name, "")) {
            right_rows_name <- " "
        }
        data <- dplyr::rename(data, !!sym(right_rows_name) := all_of(right_rows_by))
        right_rows_by <- right_rows_name
    }
    if (!is.null(right_columns_name)) {
        if (identical(right_columns_name, "")) {
            right_columns_name <- " "
        }
        data <- dplyr::rename(
            data,
            !!sym(right_columns_name) := all_of(right_columns_by)
        )
        right_columns_by <- right_columns_name
    }
    if (!is.null(rows_split_name) && !is.null(rows_split_by)) {
        if (identical(rows_split_name, "")) {
            rows_split_name <- " "
        }
        data <- dplyr::rename(data, !!sym(rows_split_name) := all_of(rows_split_by))
        rows_split_by <- rows_split_name
    }
    if (!is.null(left_columns_split_name) && !is.null(left_columns_split_by)) {
        if (identical(left_columns_split_name, "")) {
            left_columns_split_name <- " "
        }
        data <- dplyr::rename(
            data,
            !!sym(left_columns_split_name) := all_of(left_columns_split_by)
        )
        left_columns_split_by <- left_columns_split_name
    }
    if (
        !is.null(right_columns_split_name) && !is.null(right_columns_split_by)
    ) {
        if (identical(right_columns_split_name, "")) {
            right_columns_split_name <- " "
        }
        data <- dplyr::rename(
            data,
            !!sym(right_columns_split_name) := all_of(right_columns_split_by)
        )
        right_columns_split_by <- right_columns_split_name
    }
    if (!is.null(left_pie_name) && !is.null(left_pie_group_by)) {
        if (identical(left_pie_name, "")) {
            left_pie_name <- " "
        }
        data <- dplyr::rename(data, !!sym(left_pie_name) := all_of(left_pie_group_by))
        left_pie_group_by <- left_pie_name
    }
    if (!is.null(right_pie_name) && !is.null(right_pie_group_by)) {
        if (identical(right_pie_name, "")) {
            right_pie_name <- " "
        }
        data <- dplyr::rename(data, !!sym(right_pie_name) := all_of(right_pie_group_by))
        right_pie_group_by <- right_pie_name
    }
    if (!is.null(left_name)) {
        if (identical(left_name, "")) {
            left_name <- " "
        }
        data <- dplyr::rename(data, !!sym(left_name) := !!sym(left_values_by))
        left_values_by <- left_name
    }
    if (!is.null(right_name)) {
        if (identical(right_name, "")) {
            right_name <- " "
        }
        data <- dplyr::rename(data, !!sym(right_name) := !!sym(right_values_by))
        right_values_by <- right_name
    }

    keep_na <- check_keep_na(
        keep_na,
        unique(c(
            split_by,
            left_rows_by,
            rows_split_by,
            left_columns_by,
            left_columns_split_by,
            right_rows_by,
            right_columns_by,
            right_columns_split_by
        ))
    )
    if (!isFALSE(keep_empty)) {
        warning(
            "[LinkedHeatmap] 'keep_empty != FALSE' is not supported yet, and unused levels will be always dropped."
        )
    }
    data <- process_keep_na_empty(data, keep_na, list())

    list(
        data = if (is.null(split_by)) {
            stats::setNames(list(data), "...")
        } else {
            split(select(data, -!!sym(split_by)), data[[split_by]])
        },
        left_values_by = left_values_by,
        right_values_by = right_values_by,
        left_rows_by = left_rows_by,
        right_rows_by = right_rows_by,
        rows_split_by = rows_split_by,
        left_columns_by = left_columns_by,
        right_columns_by = right_columns_by,
        left_columns_split_by = left_columns_split_by,
        right_columns_split_by = right_columns_split_by,
        left_pie_group_by = left_pie_group_by,
        right_pie_group_by = right_pie_group_by,
        keep_na = keep_na,
        keep_empty = keep_empty
    )
}

#' Check the annotation specification
#' The annotations should not be any of by or split_by, and the columns must exist in the data
#'
#' @param data The data for Heatmap plot
#' @param annotation The annotation mappings
#' @param by The column for rows or columns
#' @param split_by The column for row or column splitting
#' @param which If the annotation is for row or column
#' @keywords internal
.check_annotation <- function(data, annotation, by, split_by, which) {
    forbidden_keys <- c(
        ".row",
        ".rows",
        ".rows.split",
        ".row.split",
        ".col",
        ".cols",
        ".column",
        ".columns",
        ".col.split",
        ".cols.split",
        ".column.split",
        ".columns.split",
        by,
        split_by
    )
    for (fkey in forbidden_keys) {
        if (fkey %in% names(annotation)) {
            stop(
                "[Heatmap] annotation key '",
                fkey,
                "' is taken by name/split annotation."
            )
        }
    }
    for (key in names(annotation)) {
        col <- annotation[[key]]
        if (!isFALSE(col)) {
            check_columns(data, col)
        }
    }
}

#' Resolve annotation aliases:
#'   .row         -> rows_by
#'   .rows.split  -> rows_split_by
#'   .col/.column -> columns_by
#'   .col.split/.column.split -> columns_split_by
#' @param lst A list of annotations, which may contain aliases for rows_by, rows_split_by, columns_by, and columns_split_by.
#' @param row_key The actual key for rows_by in the list.
#' @param rsplit_key The actual key for rows_split_by in the list.
#' @param col_key The actual key for columns_by in the list.
#' @param csplit_key The actual key for columns_split_by in the list.
#' @return A list of annotations with aliases resolved to the actual keys.
#' @keywords internal
.resolve_anno_aliases <- function(
    lst,
    row_key,
    rsplit_key,
    col_key,
    csplit_key
) {
    if (is.null(lst) || !is.list(lst)) {
        return(lst)
    }
    alias_map <- list(
        ".row" = row_key,
        ".rows" = row_key,
        ".rows.split" = rsplit_key,
        ".row.split" = rsplit_key,
        ".col" = col_key,
        ".cols" = col_key,
        ".column" = col_key,
        ".columns" = col_key,
        ".col.split" = csplit_key,
        ".cols.split" = csplit_key,
        ".column.split" = csplit_key,
        ".columns.split" = csplit_key
    )
    for (alias in names(alias_map)) {
        real <- alias_map[[alias]]
        if (!is.null(real) && alias %in% names(lst) && !real %in% names(lst)) {
            lst[[real]] <- lst[[alias]]
            lst[[alias]] <- NULL
        } else if (!is.null(real) && alias %in% names(lst)) {
            lst[[alias]] <- NULL # real key wins; drop alias
        } else if (is.null(real) && alias %in% names(lst)) {
            lst[[alias]] <- NULL # alias maps to NULL; drop
        }
    }
    lst
}

#' Get the grid.draw-able ggplot grob
#' The output from ggplotGrob can not be directly used in grid.draw, the position can not be set.
#' This function extracts the gTree from the ggplot grob.
#' @param p A ggplot object
#' @param void If TRUE, the theme_void will be added to the ggplot object
#' @param nolegend If TRUE, the legend will be removed from the ggplot object
#' @return A gTree object
#' @importFrom ggplot2 ggplotGrob theme_void theme
#' @keywords internal
.gggrob <- function(p, void = TRUE, nolegend = TRUE) {
    if (isTRUE(void)) {
        p <- p + theme_void()
    }
    if (isTRUE(nolegend)) {
        p <- p + theme(legend.position = "none")
    }
    for (g in ggplotGrob(p)$grobs) {
        if (
            inherits(g, "gTree") &&
                !inherits(g, "zeroGrob") &&
                !inherits(g, "absoluteGrob")
        ) {
            return(g)
        }
    }
    stop("No gTree found in the ggplot grob.")
}

#' Ensure to convert numeric to grid::unit with mm
#' @noRd
#' @keywords internal
.ensure_unit <- function(param, keys = c("width", "height")) {
    for (key in keys) {
        if (
            !is.null(param[[key]]) &&
                is.numeric(param[[key]]) &&
                !is.unit(param[[key]])
        ) {
            param[[key]] <- unit(param[[key]], "mm")
        }
    }
    return(param)
}

#' Reorder annotation list so split annotation (split_by) is farthest from heatmap body
#' and name annotation (by) is closest. User annotations stay in between.
#' For all annotation sides, ComplexHeatmap renders first-element = farthest,
#' last-element = closest to the heatmap body.
#' @param x A list of annotations
#' @param by The name of the annotation used for row/column names
#' @param split_by The name of the annotation used for splitting rows/columns
#' @param side The annotation side ("top", "bottom", "left", "right")
#' @return A reordered list of annotations
#' @keywords internal
.reorder_anno_side <- function(x, by, split_by, side) {
    keys <- setdiff(
        names(x),
        c("annotation_name_side", "show_annotation_name", "annotation_label")
    )
    name_key <- if (!is.null(by) && by %in% keys) by else character(0)
    split_key <- if (!is.null(split_by) && split_by %in% keys) {
        split_by
    } else {
        character(0)
    }
    if (side %in% c("top", "left")) {
        new_order <- c(
            split_key,
            setdiff(keys, c(name_key, split_key)),
            name_key
        )
    } else {
        new_order <- c(
            name_key,
            setdiff(keys, c(name_key, split_key)),
            split_key
        )
    }
    if (!length(new_order)) {
        return(x)
    }
    result <- x[c(
        "annotation_name_side", "show_annotation_name", "annotation_label",
        new_order
    )]
    if (!is.null(x$show_annotation_name)) {
        result$show_annotation_name <- x$show_annotation_name[
            intersect(new_order, names(x$show_annotation_name))
        ]
    }
    if (!is.null(x$annotation_label)) {
        result$annotation_label <- x$annotation_label[
            intersect(new_order, names(x$annotation_label))
        ]
    }
    result
}

#' Prepare and normalize annotation arguments
#'
#' Consolidates the new structured annotation list format with backward-compatible
#' deprecation support for old flat arguments. Handles \code{TRUE}/\code{FALSE}
#' shortcuts, \code{.default} inheritance (with recursive \code{params} merge),
#' alias resolution, and shortcut expansion.
#'
#' @param which \code{"row"} or \code{"column"}.
#' @param annotation The new structured annotation list.
#' @param annotation_side Deprecated. Old side argument.
#' @param annotation_palette Deprecated. Old palette argument.
#' @param annotation_palcolor Deprecated. Old palcolor argument.
#' @param annotation_type Deprecated. Old type argument.
#' @param annotation_params Deprecated. Old params argument.
#' @param annotation_agg Deprecated. Old agg argument.
#' @param row_key The actual column name for \code{rows_by}.
#' @param rsplit_key The actual column name for \code{rows_split_by}.
#' @param col_key The actual column name for \code{columns_by}.
#' @param csplit_key The actual column name for \code{columns_split_by}.
#' @param data The data frame used for column validation.
#' @return A list with components \code{annotation}, \code{annotation_type},
#'   \code{annotation_side}, \code{annotation_palette}, \code{annotation_palcolor},
#'   \code{annotation_agg}, \code{annotation_params}, \code{annotation_name},
#'   and \code{enabled}.
#' @keywords internal
.prep_annotations <- function(
    which = c("row", "column"),
    annotation = NULL,
    annotation_side = NULL,
    annotation_palette = NULL,
    annotation_palcolor = NULL,
    annotation_type = NULL,
    annotation_params = NULL,
    annotation_agg = NULL,
    row_key = NULL,
    rsplit_key = NULL,
    col_key = NULL,
    csplit_key = NULL,
    data = NULL
) {
    which <- match.arg(which)

    # Step 1: TRUE/FALSE/NULL shortcuts
    if (isTRUE(annotation)) {
        annotation <- list(.default = list())
    } else if (isFALSE(annotation)) {
        return(list(enabled = FALSE))
    } else if (is.null(annotation)) {
        annotation <- list()
    }

    # Normalize annotation to a list if it's a character vector
    if (!is.list(annotation)) {
        annotation <- as.list(annotation)
        if (is.null(names(annotation))) {
            names(annotation) <- unlist(annotation)
        }
    }

    # Step 2: Merge deprecated args with warnings
    deprec_map <- list(
        side = annotation_side,
        palette = annotation_palette,
        palcolor = annotation_palcolor,
        type = annotation_type,
        params = annotation_params,
        agg = annotation_agg
    )
    for (subkey in names(deprec_map)) {
        dep_val <- deprec_map[[subkey]]
        if (is.null(dep_val)) {
            next
        }
        warning(
            "`",
            which,
            "_annotation_",
            subkey,
            "` is deprecated. Use `",
            which,
            "_annotation = list(<key> = list(",
            subkey,
            " = ...))` instead.",
            call. = FALSE
        )
        if (is.list(dep_val) && !is.null(names(dep_val))) {
            # Named list: inject subkey into each named annotation entry
            for (nm in names(dep_val)) {
                if (is.null(annotation[[nm]])) {
                    annotation[[nm]] <- list()
                }
                if (!is.list(annotation[[nm]])) {
                    annotation[[nm]] <- list(col = annotation[[nm]])
                }
                annotation[[nm]][[subkey]] <- dep_val[[nm]]
            }
        } else {
            # Scalar: inject as .default
            if (is.null(annotation[[".default"]])) {
                annotation[[".default"]] <- list()
            }
            annotation[[".default"]][[subkey]] <- dep_val
        }
    }

    # Step 3: Normalize to full list format (shortcut expansion)
    for (nm in names(annotation)) {
        if (nm == ".default") {
            next
        }
        if (!is.list(annotation[[nm]])) {
            annotation[[nm]] <- list(col = annotation[[nm]])
        }
    }

    # Ensure .default exists
    if (is.null(annotation[[".default"]])) {
        annotation[[".default"]] <- list()
    }

    # Step 4: Apply .default inheritance
    defaults <- annotation[[".default"]]
    for (nm in names(annotation)) {
        if (nm == ".default") {
            next
        }
        for (dkey in names(defaults)) {
            if (dkey == "params") {
                # Recursive merge for params
                if (isFALSE(annotation[[nm]][["params"]])) {
                    # FALSE means explicitly disabled - keep it
                    next
                }
                annotation[[nm]][["params"]] <- utils::modifyList(
                    defaults[["params"]] %||% list(),
                    annotation[[nm]][["params"]] %||% list()
                )
            } else if (is.null(annotation[[nm]][[dkey]])) {
                annotation[[nm]][[dkey]] <- defaults[[dkey]]
            }
        }
    }

    # Step 5: Resolve aliases on top-level keys
    annotation <- .resolve_anno_aliases(
        annotation,
        row_key,
        rsplit_key,
        col_key,
        csplit_key
    )

    # Step 6: Extract individual components
    result <- list(
        annotation = list(),
        annotation_type = list(),
        annotation_side = list(),
        annotation_palette = list(),
        annotation_palcolor = list(),
        annotation_agg = list(),
        annotation_params = list(),
        annotation_name = list(),
        enabled = TRUE
    )
    for (nm in names(annotation)) {
        if (nm == ".default") {
            next
        }
        entry <- annotation[[nm]]

        result$annotation[[nm]] <- entry[["col"]] %||% nm

        if (!is.null(entry[["type"]])) {
            result$annotation_type[[nm]] <- entry[["type"]]
        }
        if (!is.null(entry[["side"]])) {
            result$annotation_side[[nm]] <- entry[["side"]]
        }
        if (!is.null(entry[["palette"]])) {
            result$annotation_palette[[nm]] <- entry[["palette"]]
        }
        if (!is.null(entry[["palcolor"]])) {
            result$annotation_palcolor[[nm]] <- entry[["palcolor"]]
        }
        if (!is.null(entry[["agg"]])) {
            result$annotation_agg[[nm]] <- entry[["agg"]]
        }
        if (isFALSE(result$annotation[[nm]])) {
            result$annotation_params[[nm]] <- FALSE
        } else if (!is.null(entry[["params"]])) {
            result$annotation_params[[nm]] <- entry[["params"]]
        }
        if (!is.null(entry[["name"]])) {
            result$annotation_name[[nm]] <- entry[["name"]]
        }
    }
    # Propagate .default values so .setup_annos() can use them as
    # fallbacks for built-in annotations (e.g. split/name annotations
    # that are added by .setup_annos(), not by the user).
    defaults <- annotation[[".default"]]
    if (!is.null(defaults[["type"]])) {
        result$annotation_type[[".default"]] <- defaults[["type"]]
    }
    if (!is.null(defaults[["side"]])) {
        result$annotation_side[[".default"]] <- defaults[["side"]]
    }
    if (!is.null(defaults[["palette"]])) {
        result$annotation_palette[[".default"]] <- defaults[["palette"]]
    }
    if (!is.null(defaults[["palcolor"]])) {
        result$annotation_palcolor[[".default"]] <- defaults[["palcolor"]]
    }
    if (!is.null(defaults[["agg"]])) {
        result$annotation_agg[[".default"]] <- defaults[["agg"]]
    }
    if (!is.null(defaults[["name"]])) {
        result$annotation_name[[".default"]] <- defaults[["name"]]
    }
    if (!is.null(defaults[["params"]])) {
        result$annotation_params[[".default"]] <- defaults[["params"]]
    }

    # Step 7: Validate annotation column references exist in the data.
    # NOTE: we do NOT forbid by/split_by keys here because after alias
    # resolution they represent built-in annotation configuration
    # (e.g. .row -> rows_by value).  .setup_annos() handles the merge.
    .check_annotation(data, result$annotation, NULL, NULL, which)

    result
}

#' Resolve display modes for row/column names and split titles
#'
#' `show_row_names`/`show_column_names` (kind = "names") and
#' `row_title`/`column_title` (kind = "titles") accept a character vector
#' combining the modes "inplace", "legend", "simple", "anno" (alias
#' "annotation"), and "none".  Modes are lower-priority defaults: they only
#' fill annotation type/params the user has not configured (per-key or via
#' `.default`).
#' @param show The value of `show_row_names`/`show_column_names` or
#'  `row_title`/`column_title`
#' @param kind Which arguments are being resolved: "names" or "titles"
#' @param which The direction ("row" or "column")
#' @param by The name-annotation column (`rows_by`/`columns_by`)
#' @param split_by The split-annotation column (`rows_split_by`/`columns_split_by`)
#' @param annotation,annotation_type,annotation_params The prepared annotation
#'  components returned by \code{\link{.prep_annotations}}
#' @param legend.position The legend position
#' @return A list with the possibly-updated annotation components plus the
#'  resolved \code{show} flag, \code{title}, \code{explicit}, \code{by_eff},
#'  and \code{enabled}
#' @keywords internal
.resolve_show_modes <- function(
    show,
    kind = c("names", "titles"),
    which = c("row", "column"),
    by = NULL,
    split_by = NULL,
    annotation = list(),
    annotation_type = list(),
    annotation_params = list(),
    annotation_name = list(),
    legend.position = "right"
) {
    kind <- match.arg(kind)
    which <- match.arg(which)
    valid <- c("inplace", "legend", "simple", "anno", "none")
    modes <- character(0)
    explicit <- FALSE
    title <- NULL
    show_flag <- NULL
    enabled <- NULL
    by_eff <- by
    dparams <- annotation_params[[".default"]]
    dshow_legend <- if (is.list(dparams)) dparams$show_legend else NULL

    if (isTRUE(show)) {
        modes <- "inplace"
    } else if (isFALSE(show)) {
        # FALSE only hides the in-place names; annotation logic is untouched
        if (kind == "names") {
            show_flag <- FALSE
        }
    } else if (is.character(show) && length(show) > 0) {
        if (any(!show %in% c(valid, "annotation"))) {
            if (kind == "titles") {
                # Mixed vector: literal titles, untouched
                return(list(
                    annotation = annotation,
                    annotation_type = annotation_type,
                    annotation_params = annotation_params,
                    show = NULL, title = show, explicit = FALSE,
                    by_eff = by, enabled = NULL
                ))
            }
            stop(
                "[Heatmap] Unknown display mode(s) for '",
                if (kind == "names") {
                    "show_row_names/show_column_names"
                } else {
                    "row_title/column_title"
                },
                "': ",
                paste(setdiff(show, c(valid, "annotation")), collapse = ", "),
                ". Valid modes: inplace, legend, simple, anno (alias: annotation), none.",
                call. = FALSE
            )
        }
        modes <- unique(ifelse(show == "annotation", "anno", show))
        if ("none" %in% modes && length(modes) > 1) {
            stop(
                "[Heatmap] 'none' cannot be combined with other display modes.",
                call. = FALSE
            )
        }
        if ("legend" %in% modes && "simple" %in% modes) {
            stop(
                "[Heatmap] 'legend' and 'simple' cannot be combined.",
                call. = FALSE
            )
        }
    }

    if (length(modes)) {
        explicit <- TRUE
        if (kind == "names") {
            show_flag <- "inplace" %in% modes
            if ("anno" %in% modes) {
                # Label annotation showing the row/column names themselves.
                # Without a grouping column, use a synthetic key so the
                # built-in name annotation is still injected.
                key <- by
                if (is.null(key)) {
                    key <- if (which == "row") "rownames" else "colnames"
                    if (key %in% names(annotation)) {
                        warning(
                            "[Heatmap] '", key,
                            "' is already used as an annotation key; ",
                            "skipping the 'anno' mode annotation.",
                            call. = FALSE
                        )
                        key <- NULL
                    }
                }
                if (!is.null(key)) {
                    if (is.null(annotation_type[[key]]) &&
                        is.null(annotation_type[[".default"]])) {
                        annotation_type[[key]] <- if (which == "row") {
                            "rownames"
                        } else {
                            "colnames"
                        }
                    }
                    by_eff <- key
                }
            }
            if ("legend" %in% modes && !"anno" %in% modes) {
                p <- annotation_params[[by_eff]]
                if (!is.null(by_eff) && !isFALSE(p) && is.null(p$show_legend) &&
                    is.null(dshow_legend) &&
                    !identical(legend.position, "none")) {
                    annotation_params[[by_eff]] <- utils::modifyList(
                        p %||% list(), list(show_legend = TRUE)
                    )
                }
            }
            if ("simple" %in% modes && !"anno" %in% modes) {
                p <- annotation_params[[by_eff]]
                if (!is.null(by_eff) && !isFALSE(p) && is.null(p$show_legend) &&
                    is.null(dshow_legend)) {
                    annotation_params[[by_eff]] <- utils::modifyList(
                        p %||% list(), list(show_legend = FALSE)
                    )
                }
            }
            if ("none" %in% modes) {
                if (!is.null(by_eff) && is.null(annotation[[by_eff]]) &&
                    is.null(annotation_type[[by_eff]]) &&
                    is.null(annotation_params[[by_eff]]) &&
                    is.null(annotation_name[[by_eff]]) &&
                    is.null(annotation_name[[".default"]]) &&
                    is.null(dparams)) {
                    # nothing configured: disable the name annotation entirely
                    annotation_params[[by_eff]] <- FALSE
                } else if (!is.null(by_eff) &&
                    !isFALSE(annotation_params[[by_eff]] %||% TRUE)) {
                    # user-configured: keep the annotation, but "none" still
                    # hides the displayed name and the legend
                    annotation_name[[by_eff]] <- FALSE
                    annotation_params[[by_eff]] <- utils::modifyList(
                        annotation_params[[by_eff]] %||% list(),
                        list(show_legend = FALSE)
                    )
                }
            }
            enabled <- !is.null(by_eff) &&
                ((!isFALSE(annotation[[by_eff]] %||% TRUE) &&
                    !isFALSE(annotation_params[[by_eff]] %||% TRUE)) ||
                    identical(annotation_type[[by_eff]] %||% "simple", "label"))
        } else {
            # kind == "titles"
            title <- if ("inplace" %in% modes) character(0) else NULL
            if ("anno" %in% modes) {
                if (!is.null(split_by) &&
                    is.null(annotation_type[[split_by]]) &&
                    is.null(annotation_type[[".default"]])) {
                    annotation_type[[split_by]] <- "label"
                }
            }
            if ("legend" %in% modes && !"anno" %in% modes) {
                p <- annotation_params[[split_by]]
                if (!is.null(split_by) && !isFALSE(p) &&
                    is.null(p$show_legend) && is.null(dshow_legend) &&
                    !identical(legend.position, "none")) {
                    annotation_params[[split_by]] <- utils::modifyList(
                        p %||% list(), list(show_legend = TRUE)
                    )
                }
            }
            if ("simple" %in% modes && !"anno" %in% modes) {
                p <- annotation_params[[split_by]]
                if (!is.null(split_by) && !isFALSE(p) &&
                    is.null(p$show_legend) && is.null(dshow_legend)) {
                    annotation_params[[split_by]] <- utils::modifyList(
                        p %||% list(), list(show_legend = FALSE)
                    )
                }
            }
            if ("none" %in% modes) {
                if (!is.null(split_by) && is.null(annotation[[split_by]]) &&
                    is.null(annotation_type[[split_by]]) &&
                    is.null(annotation_params[[split_by]]) &&
                    is.null(annotation_name[[split_by]]) &&
                    is.null(annotation_name[[".default"]]) &&
                    is.null(dparams)) {
                    annotation_params[[split_by]] <- FALSE
                }
            }
        }
    }

    list(
        annotation = annotation,
        annotation_type = annotation_type,
        annotation_params = annotation_params,
        annotation_name = annotation_name,
        show = show_flag,
        title = title,
        explicit = explicit,
        by_eff = by_eff,
        enabled = enabled
    )
}

#' Unified annotation builder for built-in (split/name) and user-defined annotations
#' @param which The annotation direction ("row" or "column")
#' @param names_side The side to place the row/column name annotation ("top", "bottom", "left", "right")
#' @param anno_title The title of the annotation (for split)
#' @param show_names A logical value indicating whether to show row/column names
#' @param annotation A list of user-defined annotations, where names are annotation names and values are annotation objects or parameters to build annotation objects
#' @param annotation_type A list of annotation types, where names are annotation names and values are annotation types ("simple", "label", "rownames", "colnames", "block", "ggcat", "ggseries", or "auto")
#' @param annotation_side A list of annotation sides, where names are annotation names and values are annotation sides ("top", "bottom", "left", "right")
#' @param annotation_palette A list of annotation palettes, where names are annotation names and values are palette names or color vectors
#' @param annotation_palcolor A list of annotation palette colors, where names are annotation names and values are color vectors to override the palette
#' @param annotation_agg A list of functions to aggregate the original data for each annotation, where names are annotation names and values are functions that take a vector of values in the cell and return an aggregated value
#' @param annotation_params A list of additional parameters for each annotation, where names are annotation names and values are lists of parameters to pass to the annotation constructor
#' @param annotation_name A list of display names for each annotation, where names are annotation names and values are character strings (aliasing the displayed name and legend title) or \code{FALSE} (hiding the displayed name)
#' @param split_by The name of the column used for split annotation
#' @param splits A factor vector of splits for the split annotation
#' @param by The name of the column used for name annotation
#' @param by_labels A factor vector of labels for the name annotation
#' @param flip A logical value indicating whether to flip the annotation (for ggseries annotations)
#' @param legend.direction The direction of the legend ("vertical" or "horizontal")
#' @param legend.position The position of the legend ("right", "left", "top", "bottom")
#' @param data A data frame used for ggcat and ggseries annotations, where each row corresponds to a cell in the heatmap and contains the original values before aggregation
#' @return A list of annotations and legends to be passed to ComplexHeatmap.
#' @keywords internal
.setup_annos <- function(
    which,
    names_side,
    anno_title,
    show_names,
    annotation,
    annotation_type,
    annotation_side,
    annotation_palette,
    annotation_palcolor,
    annotation_agg,
    annotation_params,
    annotation_name,
    split_by,
    splits,
    by,
    by_labels,
    flip,
    legend.direction,
    legend.position,
    data
) {
    .legends <- list()
    # --- 1. Normalize inputs ---
    if (is.null(annotation)) {
        annotation <- list()
    }
    if (!is.list(annotation)) {
        annotation <- as.list(annotation)
        names(annotation) <- unlist(annotation)
    }
    if (!is.list(annotation_palette)) {
        annotation_palette <- as.list(rep(
            if (is.character(annotation_palette)) {
                annotation_palette
            } else {
                "Paired"
            },
            length(annotation)
        ))
        names(annotation_palette) <- names(annotation)
    }
    if (!is.list(annotation_palcolor)) {
        annotation_palcolor <- rep(
            list(annotation_palcolor),
            length(annotation)
        )
        names(annotation_palcolor) <- names(annotation)
    }
    if (is.character(annotation_type) && !identical(annotation_type, "auto")) {
        annotation_type <- as.list(rep(annotation_type, length(annotation)))
        names(annotation_type) <- names(annotation)
    } else if (identical(annotation_type, "auto")) {
        annotation_type <- as.list(rep("auto", length(annotation)))
        names(annotation_type) <- names(annotation)
    } else if (!is.list(annotation_type)) {
        annotation_type <- as.list(rep("simple", length(annotation)))
        names(annotation_type) <- names(annotation)
    }

    # Add built-in split annotation
    if (!is.null(split_by)) {
        annotation[[split_by]] <- split_by
        annotation_type[[split_by]] <- annotation_type[[split_by]] %||% "simple"
        annotation_palette[[split_by]] <- annotation_palette[[split_by]] %||%
            "simspec"
        annotation_palcolor[[split_by]] <- annotation_palcolor[[split_by]]
    }
    # Add built-in name annotation
    by_name_annotation <- !is.null(by) &&
        (!isFALSE(annotation_params[[by]] %||% TRUE) ||
            annotation_type[[by]] %||% "simple" == "label")
    if (by_name_annotation) {
        annotation[[by]] <- by
        annotation_type[[by]] <- annotation_type[[by]] %||% "simple"
        annotation_palette[[by]] <- annotation_palette[[by]] %||% "Paired"
        annotation_palcolor[[by]] <- annotation_palcolor[[by]]
    }
    annotation_agg <- annotation_agg %||% list()
    annotation_params <- annotation_params %||% list()

    annos <- list(
        annotation_name_side = names_side,
        show_annotation_name = list(),
        annotation_label = list()
    )

    # --- 2. Process each annotation ---
    for (aname in names(annotation)) {
        if (aname %in% formalArgs(ComplexHeatmap::HeatmapAnnotation)) {
            annos[[aname]] <- annotation[[aname]]
            next
        }
        if (isFALSE(annotation_params[[aname]])) {
            annos[[aname]] <- NULL
            next
        }
        side <- annotation_side[[aname]] %||%
            annotation_side[[".default"]] %||%
            ifelse(which == "row", "left", "top")
        is_split <- identical(aname, split_by)
        is_builtin <- is_split || identical(aname, by)
        annotype <- annotation_type[[aname]] %||% "auto"
        if (is_split && annotype %in% c("names", "dimnames")) {
            # "names"/"dimnames" are aliases for the direction-specific
            # split annotation type showing concatenated names
            annotype <- if (which == "row") "rownames" else "colnames"
        }
        param <- annotation_params[[aname]] %||% list()
        param <- .ensure_unit(param)

        # `name` sub-key: display-name label and legend-title override.
        # FALSE hides the displayed name; the legend title stays the key.
        aname_disp <- annotation_name[[aname]] %||%
            annotation_name[[".default"]]
        if (isFALSE(aname_disp)) {
            annos$show_annotation_name[[aname]] <- FALSE
            aname_disp <- NULL
        } else if (is.character(aname_disp)) {
            annos$annotation_label[[aname]] <- aname_disp
        }

        if (is_builtin) {
            is_label <- annotype %in% c("label", "rownames", "colnames")
            # Built-in: use pre-computed splits/by_labels
            if (is.factor(splits)) {
                splits <- droplevels(splits)
            }
            if (is.factor(by_labels)) {
                by_labels <- droplevels(by_labels)
            }
            param$x <- if (identical(aname, split_by)) splits else by_labels
            param$title <- aname_disp %||% aname
            param$which <- ifelse(
                flip,
                setdiff(c("column", "row"), which),
                which
            )
            worh <- ifelse(param$which == "row", "width", "height")
            param$palette <- annotation_palette[[aname]]
            param$palcolor <- annotation_palcolor[[aname]]

            if (is_label && is_split) {
                # Start from user params so extra arguments (label_rot, etc.) pass through
                block_param <- param
                if (annotype %in% c("rownames", "colnames")) {
                    # Label each split block with the concatenated row/column names
                    sep <- param$sep %||% " "
                    wrap_by <- param$wrap_by %||% NULL
                    block_param$x <- vapply(
                        split(as.character(by_labels), splits),
                        function(labels) {
                            if (is.null(wrap_by)) {
                                return(paste(labels, collapse = sep))
                            }
                            idxs <- seq(1, length(labels), by = wrap_by)
                            paste(vapply(
                                idxs,
                                function(i) paste(
                                    labels[i:min(
                                        i + wrap_by - 1,
                                        length(labels)
                                    )],
                                    collapse = sep
                                ),
                                character(1)
                            ), collapse = "\n")
                        },
                        character(1)
                    )
                    block_param$sep <- NULL
                    block_param$wrap_by <- NULL
                } else {
                    block_param$x <- levels(param$x)
                }
                block_param$which <- param$which
                block_param$side <- side
                block_param$split_by <- split_by
                anno_legend <- do_call(anno_block, block_param)
                # annos$show_annotation_name[[aname]] <- FALSE
                param$show_legend <- FALSE
            } else {
                param$border <- param$border %||% TRUE
                param$legend.direction <- legend.direction
                show_legend <- !identical(legend.position, "none") &&
                    !annotype %in% c("label", "rownames", "colnames")
                if (is_split) {
                    show_legend <- show_legend && isFALSE(anno_title)
                } else {
                    show_legend <- show_legend && !show_names
                }
                param$show_legend <- param$show_legend %||% show_legend
                param$side <- side
                if (annotype %in% c("label", "rownames", "colnames")) {
                    anno_legend <- do_call(anno_text, param)
                } else {
                    param[[worh]] <- param[[worh]] %||% unit(2.5, "mm")
                    anno_legend <- do_call(anno_simple, param)
                }
            }
            annos[[aname]] <- anno_legend$anno
            if (isTRUE(param$show_legend)) {
                .legends[[aname]] <- anno_legend$legend
            }
        } else {
            # User-defined: extract and aggregate data
            annocol <- annotation[[aname]]
            annoagg <- annotation_agg[[aname]]
            annodata <- param$x %||% data
            annocol <- check_columns(annodata, annocol)
            if (annotype == "auto") {
                all_ones <- annodata %>%
                    group_by(!!!syms(unique(c(split_by, by)))) %>%
                    summarise(n = n(), .groups = "drop") %>%
                    pull("n")
                all_ones <- all(all_ones == 1)
                if (
                    is.character(annodata[[annocol]]) ||
                        is.factor(annodata[[annocol]]) ||
                        is.logical(annodata[[annocol]])
                ) {
                    annotype <- ifelse(all_ones, "pie", "simple")
                } else if (is.numeric(annodata[[annocol]])) {
                    annotype <- ifelse(all_ones, "points", "violin")
                } else {
                    stop(
                        "[Heatmap] Don't know how to handle ",
                        which,
                        " annotation type for column: ",
                        annocol
                    )
                }
            }
            if (
                annotype %in% c("simple", "points", "lines") && is.null(annoagg)
            ) {
                warning(
                    "[Heatmap] Assuming '",
                    which,
                    "_annotation_agg[\"",
                    aname,
                    "\"] = dplyr::first' for the simple annotation"
                )
                annoagg <- dplyr::first
            }
            if (annotype == "block" && is.null(annoagg)) {
                warning(
                    "[Heatmap] Assuming '",
                    which,
                    "_annotation_agg[\"",
                    aname,
                    "\"] = function(x) paste(unique(x), collapse = \",\")' for the block annotation"
                )
                annoagg <- function(x) paste(unique(x), collapse = ", ")
            }
            if (is.null(annoagg)) {
                annodata <- annodata %>%
                    select(!!!syms(unique(c(split_by, by, annocol))))
            } else if (annotype == "block") {
                annodata <- annodata %>%
                    group_by(!!sym(split_by)) %>%
                    summarise(
                        !!sym(annocol) := annoagg(!!sym(annocol)),
                        .groups = "drop"
                    )
            } else {
                annodata <- annodata %>%
                    group_by(!!!syms(unique(c(split_by, by)))) %>%
                    summarise(
                        !!sym(annocol) := annoagg(!!sym(annocol)),
                        .groups = "drop"
                    )
            }
            param$x <- annodata
            param$split_by <- split_by
            param$group_by <- by
            param$column <- annocol
            param$title <- aname_disp %||% aname
            param$side <- side
            param$which <- ifelse(
                flip,
                setdiff(c("column", "row"), which),
                which
            )
            param$palette <- annotation_palette[[aname]] %||% "Paired"
            param$palcolor <- annotation_palcolor[[aname]]
            param$legend.direction <- legend.direction
            if (annotype == "simple") {
                worh <- ifelse(param$which == "row", "width", "height")
                param[[worh]] <- param[[worh]] %||% unit(2.5, "mm")
            }
            if (flip) {
                pheight <- param$height
                param$height <- param$width
                param$width <- pheight
            }
            if (legend.position == "none") {
                param$show_legend <- FALSE
            }
            if (annotype == "label") {
                annotype <- "text"
            }
            if (!exists(paste0("anno_", annotype))) {
                stop("[Heatmap] Unsupported annotation type: ", annotype)
            }
            anno <- do_call(paste0("anno_", annotype), param)
            annos[[aname]] <- anno$anno
            .legends[[paste0(which, ".", aname)]] <- anno$legend
        }
    }

    list(annos = annos, legends = .legends)
}


#' Heatmap annotation function for categorical data
#' @param x A data frame
#' @param split_by A character string of the column name to split the data
#' @param group_by A character string of the column name to group the data
#' @param column A character string of the column name to plot
#' @param title A character string to name the legend
#' @param side A character string showing where the annotation is.
#' @param which A character string specifying the direction of the annotation. Default is "row".
#'  Other options are "column".
#' @param palette A character string specifying the palette of the annotation
#' @param palcolor A character vector of colors to override the palette
#' @param border A logical value indicating whether to draw the border of the annotation
#' @param legend.direction A character string specifying the direction of the legend. Default is "vertical".
#'  Other options are "horizontal".
#' @param show_legend A logical value indicating whether to show the legend
#' @param .plotting A function to create the plot for each split and each group
#' @param ... Other arguments passed to `ComplexHeatmap::AnnotationFunction`
#' @keywords internal
#' @importFrom grid grid.draw grid.lines viewport gpar
#' @importFrom tidyr unite
.anno_ggcat <- function(
    x,
    split_by = NULL,
    group_by,
    column,
    title,
    side = "left",
    which = "row",
    palette,
    palcolor = NULL,
    border = TRUE,
    legend.direction,
    show_legend = TRUE,
    .plotting,
    ...
) {
    column <- check_columns(x, column, force_factor = TRUE)
    clevels <- levels(x[[column]])

    if (!is.null(split_by)) {
        x <- x %>% dplyr::arrange(!!!syms(c(split_by, group_by)))
        x <- x %>% unite("..split", split_by, group_by, remove = FALSE)
        plots <- .plotting(
            data = x,
            column = column,
            group_by = "..split",
            palette = palette,
            palcolor = palcolor
        )
        plots <- lapply(plots, .gggrob)
    } else {
        plots <- .plotting(
            data = x,
            column = column,
            group_by = group_by,
            palette = palette,
            palcolor = palcolor
        )
        plots <- lapply(plots, .gggrob)
    }
    # add legend
    if (isTRUE(show_legend)) {
        lgd <- ComplexHeatmap::Legend(
            title = title,
            labels = clevels,
            legend_gp = gpar(
                fill = palette_this(
                    clevels,
                    palette = palette,
                    palcolor = palcolor
                )
            ),
            border = TRUE,
            nrow = if (legend.direction == "horizontal") 1 else NULL
        )
    } else {
        lgd <- NULL
    }
    anno <- ComplexHeatmap::AnnotationFunction(
        fun = function(index, k, n) {
            if (which == "row") {
                index <- rev(index)
            }
            grobs <- grobs[index]
            total <- length(index)
            # draw border
            grid.lines(
                x = c(0, 0),
                y = c(0, 1),
                gp = gpar(col = "black", lwd = 1)
            )
            grid.lines(
                x = c(1, 1),
                y = c(0, 1),
                gp = gpar(col = "black", lwd = 1)
            )
            grid.lines(
                x = c(0, 1),
                y = c(0, 0),
                gp = gpar(col = "black", lwd = 1)
            )
            grid.lines(
                x = c(0, 1),
                y = c(1, 1),
                gp = gpar(col = "black", lwd = 1)
            )
            for (i in seq_along(grobs)) {
                if (which == "row") {
                    if (side == "left") {
                        grobs[[i]]$vp <- viewport(
                            x = 0.5,
                            y = (i - 1) * 1 / total + 1 / (2 * total),
                            width = 0.95,
                            height = 1 / total,
                            angle = 180
                        )
                    } else {
                        grobs[[i]]$vp <- viewport(
                            x = 0.5,
                            y = (i - 1) * 1 / total + 1 / (2 * total),
                            width = 0.95,
                            height = 1 / total
                        )
                    }
                } else {
                    grobs[[i]]$vp <- viewport(
                        x = (i - 1) * 1 / total + 1 / (2 * total),
                        y = 0.5,
                        width = 1 / total,
                        height = 1
                    )
                }
                grid.draw(grobs[[i]])
            }
        },
        var_import = list(grobs = plots, which = which, side = side),
        n = length(plots),
        which = which,
        subsettable = TRUE,
        ...
    )
    list(anno = anno, legend = lgd)
}

#' Heatmap annotation functions
#'
#' @rdname heatmap-anno
#' @param x A data frame
#' @param split_by A character string of the column name to split the data
#' @param group_by A character string of the column name to group the data
#' @param column A character string of the column name to plot
#' @param title A character string to name the legend
#' @param which A character string specifying the direction of the annotation. Default is "row".
#'  Other options are "column".
#' @param palette A character string specifying the palette of the annotation
#' @param palcolor A character vector of colors to override the palette
#' @param border A logical value indicating whether to draw the border of the annotation
#' @param legend.direction A character string specifying the direction of the legend. Default is "vertical".
#'  Other options are "horizontal".
#' @param show_legend A logical value indicating whether to show the legend
#' @param .plotting A function to create the plot for each split and each group
#' @param ... Other arguments passed to `ComplexHeatmap::AnnotationFunction`
#' @keywords internal
#' @importFrom grid grid.draw grid.lines viewport gpar
#' @importFrom tidyr unite
.anno_ggseries <- function(
    x,
    split_by = NULL,
    group_by,
    column,
    title,
    side = "left",
    which = "row",
    palette,
    palcolor = NULL,
    border = TRUE,
    legend.direction,
    show_legend = TRUE,
    .plotting,
    ...
) {
    x$.x <- x[[group_by]]
    glevels <- levels(x[[group_by]])
    colors <- palette_this(glevels, palette = palette, palcolor = palcolor)
    if (!is.null(split_by)) {
        x <- x %>% dplyr::arrange(!!!syms(c(split_by, group_by)))
        x <- x %>% unite("..split", split_by, group_by, remove = FALSE)
        plots <- .plotting(
            data = x,
            column = column,
            group_by = "..split",
            palette = palette,
            palcolor = as.list(colors)
        )
        plots <- lapply(plots, .gggrob)
    } else {
        plots <- .plotting(
            data = x,
            column = column,
            group_by = group_by,
            palette = palette,
            palcolor = as.list(colors)
        )
        plots <- lapply(plots, .gggrob)
    }

    # add legend
    if (isTRUE(show_legend)) {
        lgd <- ComplexHeatmap::Legend(
            title = title,
            labels = glevels,
            legend_gp = gpar(
                fill = palette_this(
                    glevels,
                    palette = palette,
                    palcolor = colors
                )
            ),
            border = TRUE,
            nrow = if (legend.direction == "horizontal") 1 else NULL
        )
    } else {
        lgd <- NULL
    }

    anno <- ComplexHeatmap::AnnotationFunction(
        fun = function(index, k, n) {
            if (identical(which, "row")) {
                index <- rev(index)
            }
            grobs <- grobs[index]
            total <- length(index)
            # draw border
            grid.lines(
                x = c(0, 0),
                y = c(0, 1),
                gp = gpar(col = "black", lwd = 1)
            )
            grid.lines(
                x = c(1, 1),
                y = c(0, 1),
                gp = gpar(col = "black", lwd = 1)
            )
            grid.lines(
                x = c(0, 1),
                y = c(0, 0),
                gp = gpar(col = "black", lwd = 1)
            )
            grid.lines(
                x = c(0, 1),
                y = c(1, 1),
                gp = gpar(col = "black", lwd = 1)
            )
            for (i in seq_along(grobs)) {
                if (which == "row") {
                    if (side == "left") {
                        grobs[[i]]$vp <- viewport(
                            x = 0.5,
                            y = (i - 1) * 1 / total + 1 / (2 * total),
                            width = 0.95,
                            height = 1 / total,
                            angle = 180
                        )
                    } else {
                        grobs[[i]]$vp <- viewport(
                            x = 0.5,
                            y = (i - 1) * 1 / total + 1 / (2 * total),
                            width = 0.95,
                            height = 1 / total
                        )
                    }
                } else {
                    grobs[[i]]$vp <- viewport(
                        x = (i - 1) * 1 / total + 1 / (2 * total),
                        y = 0.5,
                        width = 1 / total,
                        height = 0.95
                    )
                }
                grid.draw(grobs[[i]])
            }
        },
        var_import = list(grobs = plots, which = which, side = side),
        n = length(plots),
        which = which,
        subsettable = TRUE,
        ...
    )

    list(anno = anno, legend = lgd)
}

#' @rdname heatmap-anno
#' @param x A data frame
#' @param split_by A character string of the column name to split the data (heatmap)
#' @param group_by A character string of the column name to group the data (rows or columns of the heatmap)
#' @param column A character string of the column name of the data `x` to plot
#' @param title A character string to name the legend
#' @param side A character string specifying the side of the annotation. Default is "left".
#' @param which A character string specifying the direction of the annotation. Default is "row".
#'  Other options are "column".
#' @param palette A character string specifying the palette of the annotation
#' @param palcolor A character vector of colors to override the palette
#' @param border A logical value indicating whether to draw the border of the annotation
#' @param legend.direction A character string specifying the direction of the legend. Default is "vertical".
#'  Other options are "horizontal".
#' @param show_legend A logical value indicating whether to show the legend
#' @param ... Other arguments passed to `ComplexHeatmap::AnnotationFunction`
#'  The parameters passed to `row_annotation_params` and `column_annotation_params` will be passed here.
anno_pie <- function(
    x,
    split_by = NULL,
    group_by,
    column,
    title,
    which = "row",
    palette,
    side = "left",
    palcolor = NULL,
    border = TRUE,
    legend.direction,
    show_legend = TRUE,
    ...
) {
    .anno_ggcat(
        x = x,
        split_by = split_by,
        group_by = group_by,
        column = column,
        title = title,
        which = which,
        side = side,
        palette = palette,
        palcolor = palcolor,
        border = border,
        legend.direction = legend.direction,
        show_legend = show_legend,
        .plotting = function(data, column, group_by, palette, palcolor) {
            PieChart(
                data,
                x = column,
                split_by = group_by,
                palette = palette,
                palcolor = palcolor,
                combine = FALSE
            )
        },
        ...
    )
}

#' @rdname heatmap-anno
anno_ring <- function(
    x,
    split_by = NULL,
    group_by,
    column,
    title,
    which = "row",
    palette,
    side = "left",
    palcolor = NULL,
    border = TRUE,
    legend.direction,
    show_legend = TRUE,
    ...
) {
    .anno_ggcat(
        x = x,
        split_by = split_by,
        group_by = group_by,
        column = column,
        which = which,
        title = title,
        side = side,
        palette = palette,
        palcolor = palcolor,
        border = border,
        legend.direction = legend.direction,
        show_legend = show_legend,
        .plotting = function(data, column, group_by, palette, palcolor) {
            RingPlot(
                data,
                group_by = column,
                split_by = group_by,
                palette = palette,
                palcolor = palcolor,
                combine = FALSE
            )
        },
        ...
    )
}

#' @rdname heatmap-anno
anno_bar <- function(
    x,
    split_by = NULL,
    group_by,
    column,
    title,
    which = "row",
    palette,
    side = "left",
    palcolor = NULL,
    border = TRUE,
    legend.direction,
    show_legend = TRUE,
    ...
) {
    .anno_ggcat(
        x = x,
        split_by = split_by,
        group_by = group_by,
        column = column,
        which = which,
        title = title,
        side = side,
        palette = palette,
        palcolor = palcolor,
        border = border,
        legend.direction = legend.direction,
        show_legend = show_legend,
        .plotting = function(data, column, group_by, palette, palcolor) {
            BarPlot(
                data,
                x = column,
                split_by = group_by,
                expand = c(0.05, 1),
                palette = palette,
                palcolor = palcolor,
                combine = FALSE
            )
        },
        ...
    )
}

#' @rdname heatmap-anno
anno_violin <- function(
    x,
    split_by = NULL,
    group_by,
    column,
    title,
    which = "row",
    palette,
    side = "left",
    palcolor = NULL,
    border = TRUE,
    legend.direction,
    show_legend = TRUE,
    ...
) {
    .anno_ggseries(
        x = x,
        split_by = split_by,
        group_by = group_by,
        column = column,
        which = which,
        title = title,
        side = side,
        palette = palette,
        palcolor = palcolor,
        border = border,
        legend.direction = legend.direction,
        show_legend = show_legend,
        .plotting = function(data, column, group_by, palette, palcolor) {
            ViolinPlot(
                data,
                x = ".x",
                y = column,
                split_by = group_by,
                combine = FALSE,
                palette = palette,
                palcolor = palcolor,
                flip = which == "row"
            )
        },
        ...
    )
}

#' @rdname heatmap-anno
anno_boxplot <- function(
    x,
    split_by = NULL,
    group_by,
    column,
    title,
    which = "row",
    palette,
    side = "left",
    palcolor = NULL,
    border = TRUE,
    legend.direction,
    show_legend = TRUE,
    ...
) {
    .anno_ggseries(
        x = x,
        split_by = split_by,
        group_by = group_by,
        column = column,
        which = which,
        title = title,
        side = side,
        palette = palette,
        palcolor = palcolor,
        border = border,
        legend.direction = legend.direction,
        show_legend = show_legend,
        .plotting = function(data, column, group_by, palette, palcolor) {
            BoxPlot(
                data,
                x = ".x",
                y = column,
                split_by = group_by,
                combine = FALSE,
                palette = palette,
                palcolor = palcolor,
                flip = which == "row"
            )
        },
        ...
    )
}

#' @rdname heatmap-anno
anno_density <- function(
    x,
    split_by = NULL,
    group_by,
    column,
    title,
    which = "row",
    palette,
    side = "left",
    palcolor = NULL,
    border = TRUE,
    legend.direction,
    show_legend = TRUE,
    ...
) {
    .anno_ggseries(
        x = x,
        split_by = split_by,
        group_by = group_by,
        column = column,
        title = title,
        which = which,
        side = side,
        palette = palette,
        palcolor = palcolor,
        border = border,
        legend.direction = legend.direction,
        show_legend = show_legend,
        .plotting = function(data, column, group_by, palette, palcolor) {
            DensityPlot(
                data,
                x = column,
                split_by = group_by,
                combine = FALSE,
                palette = palette,
                palcolor = palcolor,
                flip = which == "row"
            )
        },
        ...
    )
}

#' @rdname heatmap-anno
#' @param alpha A numeric value between 0 and 1 specifying the transparency of the annotation
anno_simple <- function(
    x,
    split_by = NULL,
    group_by = NULL,
    column = NULL,
    title,
    which = "row",
    side = "left",
    palette,
    palcolor = NULL,
    border = TRUE,
    legend.direction,
    show_legend = TRUE,
    alpha = 1,
    ...
) {
    if (!is.null(split_by)) {
        x <- do_call(rbind, split(x, x[[split_by]]))
    }
    if (!is.null(column)) {
        x <- x[[column]]
    }
    is_cont <- is.numeric(x)
    if (isFALSE(is_cont) && !is.factor(x)) {
        x <- factor(x, levels = unique(x))
    }
    # add legend
    if (is_cont) {
        col_fun <- colorRamp2(
            seq(min(x), max(x), length = 100),
            palette_this(palette = palette, palcolor = palcolor, alpha = alpha)
        )
        lgd <- if (isTRUE(show_legend)) {
            ComplexHeatmap::Legend(
                title = title,
                col_fun = col_fun,
                border = TRUE,
                direction = legend.direction
            )
        }
        anno <- ComplexHeatmap::anno_simple(
            x,
            col = col_fun,
            which = which,
            border = border,
            ...
        )
    } else {
        x_levels <- levels(x)
        colors <- palette_this(
            x_levels,
            palette = palette,
            palcolor = palcolor,
            alpha = alpha,
            NA_keep = TRUE
        )
        lgd <- if (isTRUE(show_legend)) {
            ComplexHeatmap::Legend(
                title = title,
                labels = x_levels,
                legend_gp = gpar(fill = colors),
                border = TRUE,
                nrow = if (legend.direction == "horizontal") 1 else NULL
            )
        }
        anno <- ComplexHeatmap::anno_simple(
            as.character(x),
            col = colors,
            which = which,
            border = border,
            ...
        )
    }

    list(anno = anno, legend = lgd)
}

#' @rdname heatmap-anno
anno_points <- function(
    x,
    split_by = NULL,
    group_by,
    column,
    title,
    which = "row",
    palette,
    side = "left",
    palcolor = NULL,
    border = TRUE,
    legend.direction,
    show_legend = TRUE,
    alpha = 1,
    ...
) {
    if (!is.null(split_by)) {
        x <- do_call(rbind, split(x, x[[split_by]]))
    }

    anno <- ComplexHeatmap::anno_points(
        x[[column]],
        which = which,
        border = border,
        ...
    )
    list(anno = anno, legend = NULL)
}

#' @rdname heatmap-anno
#' @param add_points A logical value indicating whether to add points to the annotation
anno_lines <- function(
    x,
    split_by = NULL,
    group_by,
    column,
    title,
    which = "row",
    palette,
    side = "left",
    palcolor = NULL,
    border = TRUE,
    legend.direction,
    show_legend = TRUE,
    alpha = 1,
    add_points = TRUE,
    ...
) {
    anno <- ComplexHeatmap::anno_lines(
        x[[column]],
        which = which,
        border = border,
        add_points = add_points,
        ...
    )
    list(anno = anno, legend = NULL)
}

#' @rdname heatmap-anno
#' @keywords internal
anno_block <- function(
    x,
    split_by = NULL,
    group_by = NULL,
    column = NULL,
    title,
    which = "row",
    side = "left",
    palette,
    palcolor = NULL,
    border = TRUE,
    legend.direction,
    show_legend = FALSE,
    alpha = 1,
    ...
) {
    if (is.null(split_by)) {
        stop(
            "[Heatmap] ",
            which,
            " annotation type 'block' for ",
            title,
            " must work with ",
            which,
            "s_split_by."
        )
    }
    if (!is.null(column)) {
        x <- x[[column]]
    }
    args <- list(
        labels = x,
        which = which,
        ...
    )
    args$gp <- args$gp %||% gpar()
    if (isFALSE(border)) {
        args$gp$col <- "transparent"
    }
    if (is.null(args$gp$fill)) {
        args$gp$fill <- palette_this(
            x,
            palette = palette,
            palcolor = palcolor,
            alpha = alpha,
            NA_keep = TRUE
        )
    }
    args$labels_rot <- args$labels_rot %||%
        (if (side == "left") {
            90
        } else if (side == "right") {
            -90
        } else if (side == "top") {
            0
        } else if (side == "bottom") {
            0
        } else {
            0
        })
    max_height <- ComplexHeatmap::max_text_height(x, gp = args$gp)
    max_width <- ComplexHeatmap::max_text_width(x, gp = args$gp)
    if (which == "row" && is.null(args$width)) {
        max_height <- max_height +
            max(
                max_height * 0.6,
                max_width * abs(sin((args$labels_rot + 90) / 180 * pi))
            )
        args$width <- args$width %||% max_height * 1.2
    } else if (which == "column" && is.null(args$height)) {
        max_height <- max_height +
            max(
                max_height * 0.6,
                max_width * abs(sin(args$labels_rot / 180 * pi))
            )
        args$height <- args$height %||% max_height * 1.2
    }
    anno <- do_call(ComplexHeatmap::anno_block, args)
    list(anno = anno, legend = NULL)
}

#' @rdname heatmap-anno
#' @keywords internal
anno_text <- function(
    x,
    split_by = NULL,
    group_by,
    column = NULL,
    title,
    which = "row",
    palette,
    side = "left",
    palcolor = NULL,
    border = TRUE,
    legend.direction,
    show_legend = FALSE,
    alpha = 1,
    ...
) {
    if (!is.null(split_by)) {
        x <- do_call(rbind, split(x, x[[split_by]]))
    }
    if (!is.null(column)) {
        x <- x[[column]]
    }
    if (!is.factor(x)) {
        x <- factor(x, levels = unique(x))
    }
    x_levels <- levels(x)
    colors <- palette_this(
        x_levels,
        palette = palette,
        palcolor = palcolor,
        alpha = alpha,
        NA_keep = TRUE
    )
    args <- list(...)
    args$x <- x_levels
    args$which <- which
    args$gp <- args$gp %||% args$labels_gp %||% gpar()
    args$labels_gp <- NULL
    args$rot <- args$rot %||%
        args$labels_rot %||%
        (if (side == "left") {
            90
        } else if (side == "right") {
            -90
        } else if (side == "top") {
            0
        } else if (side == "bottom") {
            0
        } else {
            0
        })
    args$labels_rot <- NULL
    if (is.null(args$gp$fill)) {
        args$gp$fill <- colors
    }
    if (is.null(args$gp$border)) {
        args$gp$border <- if (border) "black" else "transparent"
    }
    max_height <- ComplexHeatmap::max_text_height(x_levels, gp = args$gp)
    max_width <- ComplexHeatmap::max_text_width(x_levels, gp = args$gp)
    if (which == "row" && is.null(args$width)) {
        max_height <- max_height +
            max(
                max_height * 0.6,
                max_width * abs(sin((args$rot + 90) / 180 * pi))
            )
        args$width <- args$width %||% max_height * 1.2
    } else if (which == "column" && is.null(args$height)) {
        max_height <- max_height +
            max(max_height * 0.6, max_width * abs(sin(args$rot / 180 * pi)))
        args$height <- args$height %||% max_height * 1.2
    }
    args$location <- args$location %||% max_height * 0.55
    args$just <- args$just %||% c("center", "center")
    anno <- do_call(ComplexHeatmap::anno_text, args)
    list(anno = anno, legend = NULL)
}

#' Heatmap layer functions used to draw on the heatmap cells
#'
#' @rdname heatmap-layer
#' @param j An integer specifying the column index
#' @param i An integer specifying the row index
#' @param x A numeric vector specifying the x position
#' @param y A numeric vector specifying the y position
#' @param w A numeric vector specifying the width
#' @param h A numeric vector specifying the height
#' @param fill A character vector specifying the fill color
#' @keywords internal
layer_white_bg <- function(j, i, x, y, w, h, fill) {
    grid.rect(
        x = x,
        y = y,
        width = w,
        height = h,
        gp = gpar(fill = "white", col = "white", lwd = 1)
    )
}

#' @rdname heatmap-layer
#' @param alpha A numeric value between 0 and 1 specifying the transparency of the fill color
#' @keywords internal
layer_bg <- function(j, i, x, y, w, h, fill, alpha) {
    grid.rect(
        x = x,
        y = y,
        width = w,
        height = h,
        gp = gpar(col = fill, lwd = 1, fill = adjcolors(fill, alpha))
    )
}

#' @rdname heatmap-layer
#' @param color A character vector specifying the color of the reticle
#' @keywords internal
layer_reticle <- function(j, i, x, y, w, h, fill, color) {
    ind_mat <- ComplexHeatmap::restore_matrix(j, i, x, y)
    ind_top <- ind_mat[1, ]
    ind_left <- ind_mat[, 1]
    for (col in seq_len(ncol(ind_mat))) {
        grid.lines(
            x = unit(rep(x[ind_top[col]], each = 2), "npc"),
            y = unit(c(0, 1), "npc"),
            gp = gpar(col = color, lwd = 1.5)
        )
    }
    for (row in seq_len(nrow(ind_mat))) {
        grid.lines(
            x = unit(c(0, 1), "npc"),
            y = unit(rep(y[ind_left[row]], each = 2), "npc"),
            gp = gpar(col = color, lwd = 1.5)
        )
    }
}

#' @rdname heatmap-layer
#' @param data A dataframe used to create the annotation. Different from the data used to
#'  create the heatmap itself, which is aggregated data. This dataframe is the original data,
#'  where each cell could have multiple values.
#' @param dot_size A numeric value specifying the size of the dot or a function to calculate the size
#'  from the values in the cell. The function can take 1, 3, or 5 arguments: the first argument is
#'  the values in the cell before aggregation; the 2nd and 3rd arguments are the row and column
#'  indices; the 4th and 5th arguments are the row and column names.
#' @param row_names Row names from the heatmap matrix.
#' @param col_names Column names from the heatmap matrix.
#' @keywords internal
layer_dot <- function(
    j,
    i,
    x,
    y,
    w,
    h,
    fill,
    data,
    dot_size,
    alpha,
    row_names = NULL,
    col_names = NULL
) {
    if (is.numeric(dot_size) && length(dot_size) == 1) {
        # Simple numeric size
        grid.points(
            x,
            y,
            pch = 21,
            size = unit(dot_size, "mm"),
            gp = gpar(col = "black", lwd = 1, fill = adjcolors(fill, alpha))
        )
    } else {
        # dot_size is a pre-computed vector/list or will be computed from function
        grid.points(
            x,
            y,
            pch = 21,
            size = unit(
                scales::rescale(unlist(dot_size), to = c(.5, 12)),
                "mm"
            ),
            gp = gpar(col = "black", lwd = 1, fill = adjcolors(fill, alpha))
        )
    }
}

#' @rdname heatmap-layer
#' @param col_fun A function to calculate the color of the bars
#' @keywords internal
layer_bars <- function(j, i, x, y, w, h, fill, flip, col_fun, data, alpha) {
    # colors be like [1,1.9]: '#A6CEE3' (1.9,2.8]: '#1F78B4' (2.8,3.7]: '#B2DF8A'
    indices <- paste(i, j, sep = "-")
    data <- data[indices]
    ns <- lengths(data)
    if (flip) {
        # rep(w / ns, ns) can't keep the unit
        bw <- rep(w, lengths(data))
        bh <- rep(sapply(seq_along(h), function(m) h[m] / ns[m]), ns)
        by <- unlist(lapply(seq_along(y), function(m) {
            y[m] - h[m] / 2 + seq_along(data[[m]]) * h[m] / length(data[[m]])
        })) -
            bh / 2
        bx <- rep(x, lengths(data))
    } else {
        # rep(w / ns, ns) can't keep the unit
        bw <- rep(sapply(seq_along(w), function(m) w[m] / ns[m]), ns)
        bh <- rep(h, lengths(data))
        bx <- unlist(lapply(seq_along(x), function(m) {
            x[m] - w[m] / 2 + seq_along(data[[m]]) * w[m] / length(data[[m]])
        })) -
            bw / 2
        by <- rep(y, lengths(data))
    }
    bf <- unlist(lapply(data, col_fun))
    grid.rect(
        x = bx,
        y = by,
        width = bw,
        height = bh,
        gp = gpar(fill = bf, col = "transparent")
    )
}

#' @rdname heatmap-layer
#' @keywords internal
layer_pie <- function(
    j,
    i,
    x,
    y,
    w,
    h,
    fill,
    palette,
    palcolor,
    data,
    pie_size
) {
    indices <- paste(i, j, sep = "-")
    data <- data[indices]
    pies <- lapply(indices, function(idx) {
        p <- PieChart(
            data[[idx]],
            x = "Var",
            y = "Freq",
            label = NULL,
            palette = palette,
            palcolor = palcolor
        )
        ggplotGrob(p + theme_void() + theme(legend.position = "none"))
    })

    if (!is.function(pie_size)) {
        pie_sizes <- rep(pie_size %||% 1, length(pies))
    } else {
        pie_sizes <- sapply(data, function(d) {
            pie_size(sum(d$Freq, na.rm = TRUE))
        })
        pie_sizes <- scales::rescale(pie_sizes, to = c(0.2, 1))
    }
    idx <- which(sapply(pies[[1]]$grobs, function(g) {
        inherits(g, "gTree") &&
            !inherits(g, "zeroGrob") &&
            !inherits(g, "absoluteGrob")
    }))[1]
    for (m in seq_along(pies)) {
        pies[[m]]$grobs[[idx]]$vp <- viewport(
            x = x[m],
            y = y[m],
            width = pie_sizes[m] * w[m],
            height = pie_sizes[m] * h[m]
        )
        grid.draw(pies[[m]]$grobs[[idx]])
    }
}

#' @rdname heatmap-layer
#' @param colors A character vector specifying the fill color of the violin plot.
#'  If not provided, the fill color of row/column annotation will be used
#' @keywords internal
layer_boxviolin <- function(j, i, x, y, w, h, fill, flip, data, colors, fn) {
    vlndata <- data[paste(i, j, sep = "-")]
    vlnplots <- lapply(seq_along(vlndata), function(m) {
        p <- fn(
            data.frame(x = 1, y = vlndata[[m]]),
            x = "x",
            y = "y",
            palcolor = colors %||% fill[m],
            flip = flip
        )
        ggplotGrob(p + theme_void() + theme(legend.position = "none"))
    })
    idx <- which(sapply(vlnplots[[1]]$grobs, function(g) {
        inherits(g, "gTree") &&
            !inherits(g, "zeroGrob") &&
            !inherits(g, "absoluteGrob")
    }))[1]
    for (m in seq_along(vlnplots)) {
        wm <- if (flip) w[m] * 0.95 else w[m]
        hm <- if (flip) h[m] * 0.95 else h[m]
        vlnplots[[m]]$grobs[[idx]]$vp <- viewport(
            x = x[m],
            y = y[m],
            width = wm,
            height = hm
        )
        grid.draw(vlnplots[[m]]$grobs[[idx]])
    }
}

#' Atomic density/histogram plot
#'
#' @description
#' Core implementation for density and histogram plots. This is the internal workhorse
#' dispatched by both the `DensityPlot()` and `Histogram()` public wrappers. It
#' renders a grouped density curve or histogram, with optional data-distribution
#' bars along the y=0 axis, trend-line interpolation (histogram only), and full
#' faceting support.
#'
#' The function supports two plotting modes selected by `type`:
#' - **density** — renders `ggplot2::geom_density()` for each group.
#' - **histogram** — renders `ggplot2::geom_histogram()` with optional trend
#'   overlays (`use_trend`, `add_trend`), including zero-skip interpolation
#'   (`trend_skip_zero`) that uses `zoo::na.approx()` to bridge gaps where a
#'   bin has zero observations under a transformed y-axis.
#'
#' When `add_bars = TRUE`, a data rug is drawn along the bottom of the plot using
#' `ggplot2::geom_linerange()`. Each group's bars are offset vertically so they
#' stack without overlapping.
#'
#' @section Architecture:
#' **DensityHistoPlotAtomic** executes the following steps:
#'
#' 1. **ggplot dispatch** — selects `gglogger::ggplot` or `ggplot2::ggplot`
#'    based on `getOption("plotthis.gglogger.enabled")`.
#' 2. **Type check** — `match.arg(type, c("density", "histogram"))`.
#' 3. **Expansion normalization** — `norm_expansion()` converts the `expand`
#'    vector into CSS-padding-style x/y components.
#' 4. **Column resolution** — `check_columns()` validates `x`, `group_by`
#'    (force_factor, allow_multi, concat_multi), and `facet_by`.
#' 5. **Default group** — when `group_by` is `NULL`, a synthetic `.group`
#'    factor with a single empty-string level is created so the colour-mapping
#'    pipeline runs uniformly.
#' 6. **Histogram bin default** — if `type = "histogram"` and neither `bins`
#'    nor `binwidth` is set, `bins = 30` with a message.
#' 7. **Add-bars pre-calculation** — when `add_bars = TRUE`:
#'    - For density: max y = `max(density(x)$y) * 1.5`.
#'    - For histogram: max y = max bin count from `cut(x, s)`.
#'    - Computes `.ymin` and `.ymax` per row, offset by `bar_height * max_y`
#'      for each group so rugs stack without colliding.
#' 8. **NA / empty handling** — `process_keep_na_empty()` filters data and
#'    `keep_empty` values are extracted for group and facet dimensions.
#' 9. **Palette resolution** — `palette_this()` maps group levels to colours.
#' 10. **Base ggplot + scales** — initialises `ggplot(data, aes(x, fill, color))`,
#'     then adds `scale_fill_manual()` / `scale_color_manual()`. When
#'     `keep_empty_group` is `TRUE`, `drop = FALSE`, `breaks`, and `limits`
#'     are set to preserve empty factor levels.
#' 11. **Geometry layer**:
#'     - *Histogram (no trend)*: `geom_histogram(alpha, bins, binwidth, position)`.
#'     - *Histogram (use_trend / add_trend)*: adds `stat_bin(geom = "point")`
#'       for trend points + `stat_bin(geom = "line")` for the trend curve.
#'       When `trend_skip_zero = TRUE`, an `after_stat()` expression sets zero
#'       counts to `NA`, transforms y, applies `zoo::na.approx()` per
#'       `..group..`, and inverts — producing a continuous trend that
#'       interpolates over empty bins.
#'     - *Density*: `geom_density(alpha, position)`.
#' 12. **Add-bars geometry** — if `add_bars = TRUE`, `geom_linerange()` draws
#'     vertical ticks at the pre-computed `.ymin` / `.ymax` positions.
#' 13. **Scales, theme, labels** — x/y continuous scales with transforms,
#'     theme applied via `do_call(theme, theme_args)`, and axis / title
#'     labels (default y-lab: "Count" for histogram, "Density" for density).
#' 14. **Flip** — optional `coord_flip()`.
#' 15. **Dimension calculation** — `calculate_plot_dimensions(base_height = 3.5,
#'     aspect.ratio, legend, flip)` sets `height` / `width` attributes.
#' 16. **Faceting** — `facet_plot()` applies `facet_grid` / `facet_wrap`.
#'
#' @inheritParams common_args
#' @param x A character string specifying the column name for the x-axis values.
#'   A numeric column is expected.
#' @param group_by A character string specifying the column(s) to group the data
#'   by. Multiple columns are concatenated with `group_by_sep`. Each group
#'   receives a distinct fill and outline colour.
#' @param group_by_sep A character string used to join multiple `group_by`
#'   column values into a single factor level. Default: `"_"`.
#' @param group_name A character string used as the legend title for the
#'   `group_by` aesthetic. When `NULL` (default), the (possibly concatenated)
#'   `group_by` column name is used.
#' @param xtrans A character string specifying the transformation applied to
#'   the x-axis. Passed to `ggplot2::scale_x_continuous(transform = ...)`.
#'   Supported values include `"identity"` (default), `"log10"`, `"log2"`,
#'   `"sqrt"`, `"reverse"`, etc.
#' @param ytrans A character string specifying the transformation applied to
#'   the y-axis. Passed to `ggplot2::scale_y_continuous(transform = ...)`.
#'   Used by `trend_skip_zero` to correctly interpolate across zero bins on
#'   a transformed scale. Default: `"identity"`.
#' @param type A character string specifying the plot type. `"density"` (default)
#'   renders `geom_density()`; `"histogram"` renders `geom_histogram()` with
#'   optional trend overlays.
#' @param bins A numeric value specifying the number of bins for the histogram.
#'   Ignored when `type = "density"`. Defaults to `30` when neither `bins` nor
#'   `binwidth` is provided.
#' @param binwidth A numeric value specifying the width of individual bins for
#'   the histogram. Ignored when `type = "density"`. Takes precedence over
#'   `bins` when both are set.
#' @param flip A logical value. If `TRUE`, the x and y axes are swapped via
#'   `coord_flip()`. Dimension calculation accounts for the flip.
#' @param add_bars A logical value. If `TRUE`, a data-distribution rug is
#'   drawn along the y = 0 axis using `geom_linerange()`. Each group's bars
#'   are vertically offset to avoid overlap.
#' @param bar_height A numeric value specifying the height (in data units,
#'   relative to the maximum y) of the rug bars added by `add_bars`.
#'   The actual pixel height scales with `max_y`. Default: `0.025`.
#' @param bar_alpha A numeric value in `[0, 1]` for the transparency of the
#'   rug bars. Default: `1`.
#' @param bar_width A numeric value passed as the `linewidth` aesthetic of
#'   `geom_linerange()`. Controls the thickness of each rug tick.
#'   Default: `0.1`.
#' @param position A character string specifying the position adjustment for
#'   the bars or density curves. Default: `"identity"`, which shows the
#'   actual count / density per group (unlike `ggplot2`'s default `"stack"`).
#'   Other options: `"stack"`, `"dodge"`, `"fill"`.
#' @param use_trend A logical value. If `TRUE`, the histogram bars are replaced
#'   entirely by a trend line (points + connecting line). Only applies when
#'   `type = "histogram"`.
#' @param add_trend A logical value. If `TRUE`, a trend line is overlaid on
#'   top of the histogram bars. Only applies when `type = "histogram"`.
#' @param trend_alpha A numeric value in `[0, 1]` controlling the transparency
#'   of the trend points and line. Default: `1`.
#' @param trend_linewidth A numeric value for the thickness of the trend line.
#'   Default: `0.8`.
#' @param trend_pt_size A numeric value for the size of the trend points.
#'   Default: `1.5`.
#' @param trend_skip_zero A logical value. If `TRUE`, bins with zero count are
#'   set to `NA` before the trend line is computed, and `zoo::na.approx()` is
#'   used to interpolate across the gaps — producing a continuous curve even
#'   when some bins are empty. Requires `ytrans` to be correctly specified.
#'   Only applies when `type = "histogram"` and `use_trend` or `add_trend` is
#'   active.
#' @importFrom utils getFromNamespace
#' @importFrom zoo na.approx
#' @importFrom ggplot2 geom_density scale_fill_manual labs theme geom_histogram coord_flip waiver
#' @importFrom ggplot2 scale_color_manual scale_x_continuous scale_y_continuous stat_bin
#' @keywords internal
DensityHistoPlotAtomic <- function(
    data,
    x,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    xtrans = "identity",
    ytrans = "identity",
    type = c("density", "histogram"),
    bins = NULL,
    binwidth = NULL,
    flip = FALSE,
    keep_na = FALSE,
    keep_empty = FALSE,
    add_bars = FALSE,
    bar_height = 0.025,
    bar_alpha = 1,
    bar_width = .1,
    position = "identity",
    use_trend = FALSE,
    add_trend = FALSE,
    trend_alpha = 1,
    trend_linewidth = 0.8,
    trend_pt_size = 1.5,
    trend_skip_zero = FALSE,
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = .5,
    theme = "theme_this",
    theme_args = list(),
    aspect.ratio = 1,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    expand = c(bottom = 0, left = 0, right = 0),
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    legend.position = ifelse(is.null(group_by), "none", "right"),
    legend.direction = "vertical",
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    type <- match.arg(type)
    expand <- norm_expansion(
        expand,
        x_type = "continuous",
        y_type = "continuous"
    )
    x <- check_columns(data, x)
    group_by <- check_columns(
        data,
        group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = group_by_sep
    )
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )
    if (is.null(group_by)) {
        group_by <- ".group"
        data[[group_by]] <- factor("")
    }
    if (is.null(bins) && is.null(binwidth) && type == "histogram") {
        bins <- 30
        message("Using `bins = 30`. Pick better value with `binwidth`.")
    }
    if (isTRUE(add_bars)) {
        if (type == "density") {
            # calculate the max density for the y-axis
            max_y <- max(stats::density(data[[x]])$y) * 1.5
        } else {
            # calculate the max count for the y-axis by bins
            if (is.null(bins) && is.null(binwidth)) {
                s <- seq(min(data[[x]]), max(data[[x]]), length.out = 30)
            } else if (!is.null(bins)) {
                s <- seq(min(data[[x]]), max(data[[x]]), length.out = bins)
            } else {
                s <- seq(min(data[[x]]), max(data[[x]]), by = binwidth)
            }
            max_y <- max(table(cut(data[[x]], s)))
        }
        lnheight <- bar_height * max_y
        # calculate the ymin ymax for each group to plot the data lines
        data$.ymin <- lnheight * (1 - as.integer(data[[group_by]]))
        data$.ymax <- data$.ymin - lnheight
    }
    data <- process_keep_na_empty(data, keep_na, keep_empty)
    keep_empty_group <- if (!is.null(group_by)) keep_empty[[group_by]] else NULL
    keep_empty_facet <- if (!is.null(facet_by)) keep_empty[[facet_by]] else NULL
    if (length(facet_by) > 1) {
        stopifnot(
            "[Density/HistoPlot] `keep_empty` for `facet_by` variables must be identical." = identical(
                keep_empty_facet,
                keep_empty[[facet_by[2]]]
            )
        )
    }
    group_vals <- levels(data[[group_by]])
    if (anyNA(data[[group_by]])) {
        group_vals <- c(group_vals, NA)
    }
    group_colors <- palette_this(
        group_vals,
        palette = palette,
        palcolor = palcolor,
        NA_keep = TRUE,
        reverse = palreverse
    )

    p <- ggplot(
        data,
        aes(x = !!sym(x), fill = !!sym(group_by), color = !!sym(group_by))
    )
    if (isTRUE(keep_empty_group)) {
        p <- p +
            scale_fill_manual(
                name = group_name %||% group_by,
                values = group_colors,
                na.value = group_colors['NA'] %||% "grey80",
                breaks = group_vals,
                limits = group_vals,
                drop = FALSE
            ) +
            scale_color_manual(
                name = group_name %||% group_by,
                values = group_colors,
                na.value = group_colors['NA'] %||% "grey80",
                breaks = group_vals,
                limits = group_vals,
                drop = FALSE
            )
    } else {
        p <- p +
            scale_fill_manual(
                name = group_name %||% group_by,
                values = group_colors,
                na.value = group_colors['NA'] %||% "grey80"
            ) +
            scale_color_manual(
                name = group_name %||% group_by,
                values = group_colors,
                na.value = group_colors['NA'] %||% "grey80"
            )
    }

    if (type == "histogram") {
        if (!use_trend) {
            p <- p +
                geom_histogram(
                    alpha = alpha,
                    bins = bins,
                    binwidth = binwidth,
                    position = position,
                    show.legend = TRUE,
                    ...
                )
        }
        if (use_trend || add_trend) {
            p <- p +
                stat_bin(
                    geom = "point",
                    bins = bins,
                    binwidth = binwidth,
                    alpha = trend_alpha,
                    size = trend_pt_size,
                    position = position,
                    ...
                )
            if (trend_skip_zero) {
                if (inherits(ytrans, "transform")) {
                    ytrans_obj <- ytrans
                } else if (is.character(ytrans)) {
                    ytrans_obj <- getFromNamespace(
                        paste0("transform_", ytrans),
                        "scales"
                    )()
                } else if (is.function(ytrans)) {
                    ytrans_obj <- ytrans()
                } else {
                    stop(
                        "ytrans should be a character, a transform object, or a function returning a transform object."
                    )
                }
                p <- p +
                    stat_bin(
                        aes(
                            y = after_stat({
                                y <- ifelse(
                                    !!sym("count") > 0,
                                    !!sym("count"),
                                    NA
                                )
                                y <- ytrans_obj$transform(y)
                                y <- split(y, !!sym("..group.."))
                                y <- unlist(lapply(y, na.approx, na.rm = FALSE))
                                ytrans_obj$inverse(y)
                            })
                        ),
                        bins = bins,
                        binwidth = binwidth,
                        geom = "line",
                        position = position,
                        linewidth = trend_linewidth,
                        ...
                    )
            } else {
                p <- p +
                    stat_bin(
                        aes(y = after_stat(!!sym("count"))),
                        bins = bins,
                        binwidth = binwidth,
                        geom = "line",
                        position = position,
                        linewidth = trend_linewidth,
                        ...
                    )
            }
        }
    } else {
        p <- p +
            geom_density(
                alpha = alpha,
                position = position,
                show.legend = TRUE,
                ...
            )
    }
    if (isTRUE(add_bars)) {
        p <- p +
            geom_linerange(
                aes(ymin = !!sym(".ymin"), ymax = !!sym(".ymax")),
                alpha = bar_alpha,
                linewidth = bar_width
            )
    }
    p <- p +
        scale_x_continuous(expand = expand$x, transform = xtrans) +
        scale_y_continuous(expand = expand$y, transform = ytrans) +
        do_call(theme, theme_args) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% x,
            y = ylab %||% ifelse(type == "histogram", "Count", "Density")
        ) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction
        )

    if (flip) {
        p <- p + coord_flip()
    }

    dims <- calculate_plot_dimensions(
        base_height = 3.5,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = length(group_vals),
        legend_nchar = max(nchar(as.character(group_vals)), na.rm = TRUE),
        flip = flip
    )
    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    facet_plot(
        p,
        facet_by = facet_by,
        facet_scales = facet_scales,
        ncol = facet_ncol,
        nrow = facet_nrow,
        byrow = facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction
    )
}

#' Atomic ridge plot
#'
#' @description
#' Core implementation for ridge (joy) plots. Renders overlapping density curves
#' for each group on the y-axis using `ggridges::geom_density_ridges()`, with
#' optional vertical reference lines and wide-to-long data conversion.
#'
#' The function accepts data in two forms:
#' - **long form** (default) — a numeric `x` column plus a `group_by` factor
#'   column whose levels become the y-axis ridges.
#' - **wide form** — multiple numeric columns named in `group_by` are gathered
#'   via `tidyr::pivot_longer()` into `.x` / `.group` columns, then processed
#'   identically to the long form.
#'
#' Vertical reference lines (`add_vline`) can be specified as a numeric vector
#' (same lines for all groups), a named list (per-group values), or `TRUE`
#' (group means). When `vline_color = TRUE`, each line is coloured with a
#' darkened blend of the corresponding ridge fill.
#'
#' @section Architecture:
#' **RidgePlotAtomic** executes the following steps:
#'
#' 1. **ggplot dispatch** — selects `gglogger::ggplot` or `ggplot2::ggplot`.
#' 2. **Wide-to-long conversion** — when `in_form = "wide"`, calls
#'    `tidyr::pivot_longer()` on the `group_by` columns, producing `.group`
#'    (factor) and `.x` (values). `x` and `group_by` are redirected to these
#'    synthetic columns.
#' 3. **Column resolution** — `check_columns()` validates `x`, `group_by`
#'    (force_factor, allow_multi, concat_multi), and `facet_by`.
#' 4. **Default group** — when `group_by` is `NULL`, a synthetic `.group`
#'    factor with a single space character level is created so the fill
#'    pipeline runs uniformly.
#' 5. **Reverse ordering** — if `reverse = TRUE`, factor levels of `group_by`
#'    are reversed, flipping the y-axis ridge order.
#' 6. **NA / empty handling** — `process_keep_na_empty()` filters data. When
#'    `reverse = TRUE` and any group value is `NA`, the NA level is renamed
#'    to the literal string `"NA"` and moved to the end of the factor so
#'    colour mapping and display remain consistent.
#' 7. **Palette resolution** — `palette_this()` maps group levels to fill
#'    colours.
#' 8. **Base ggplot** — initialises `ggplot(data, aes(x, y, fill))` with
#'    `group_by` on the y-axis.
#' 9. **Ridge geometry** — `ggridges::geom_density_ridges(alpha, scale)`.
#'    When `scale` is `NULL`, ggridges auto-computes the overlap factor.
#' 10. **Vertical reference lines** — if `add_vline` is not `NULL` / `FALSE`:
#'     - `add_vline = TRUE` → computes `tapply(x, group_by, mean)`.
#'     - `vline_color = TRUE` → resolves per-group line colours by
#'       darkening each fill colour via `blend_colors(mode = "multiply")`.
#'       Named list elements are matched to factor levels.
#'     - Adds `geom_vline(xintercept, linetype, linewidth, color, alpha)`.
#' 11. **Scales and labels** — `scale_y_discrete(drop = !keep_empty_group)`,
#'     `scale_x_continuous()`, and `labs()`.
#' 12. **Fill scale** — `scale_fill_manual()`. When `keep_empty_group = TRUE`,
#'     `drop = FALSE`, `breaks`, and `limits` are set to preserve empty
#'     factor levels.
#' 13. **Flip-aware theme** — when `flip = TRUE`:
#'     - `coord_flip()` is applied.
#'     - x-axis text angle is set from `x_text_angle` with computed
#'       `hjust` / `vjust` via `calc_just()`.
#'     - Major grid lines are drawn on the x-axis.
#'     - When `flip = FALSE`, y-axis text is right-aligned and grid lines
#'       appear on the y-axis.
#' 14. **Theme application** — `do_call(theme, theme_args)` applies the
#'     resolved theme function, then `aspect.ratio` and legend position are
#'     set.
#' 15. **Dimension calculation** — `calculate_plot_dimensions(base_height = 1,
#'     n_y = nlevels(group_by), y_scale_factor = 1, aspect.ratio, legend,
#'     flip)` sets `height` / `width` attributes. The base height of 1 unit
#'     per ridge keeps individual ridges compact.
#' 16. **Faceting** — `facet_plot()` applies `facet_grid` / `facet_wrap`, with
#'     `drop = !keep_empty_facet`.
#'
#' @inheritParams common_args
#' @param data A data frame. Accepted in two forms:
#'   - **long** (`in_form = "long"`): a numeric column (named by `x`) and a
#'     factor column (named by `group_by`) whose levels become y-axis ridges.
#'   - **wide** (`in_form = "wide"`): multiple numeric columns listed in
#'     `group_by` are gathered into `.x` / `.group` via `tidyr::pivot_longer()`.
#' @param x A character string specifying the column name for the numeric
#'   values plotted on the x-axis. When `in_form = "wide"`, `x` should be
#'   `NULL`; the gathered values are stored in a synthetic `.x` column.
#' @param in_form A character string specifying whether `data` is in
#'   `"long"` (default) or `"wide"` format.
#' @param group_by A character string specifying the column(s) whose levels
#'   define the individual ridges on the y-axis. Multiple columns are
#'   concatenated with `group_by_sep`. In wide mode, these are the column
#'   names to gather.
#' @param group_by_sep A character string used to join multiple `group_by`
#'   column values into a single factor level. In wide form the columns are
#'   not concatenated (each becomes its own ridge). Default: `"_"`.
#' @param group_name A character string used as the legend title for the
#'   `group_by` fill aesthetic. Defaults to the (concatenated) `group_by`
#'   column name.
#' @param flip A logical value. If `TRUE`, the axes are swapped via
#'   `coord_flip()`. X-axis text angle and grid-line placement are adjusted
#'   accordingly.
#' @param alpha A numeric value in `[0, 1]` for the transparency of the
#'   ridge fill. Default: `0.8`.
#' @param reverse A logical value. If `TRUE`, the y-axis group order is
#'   reversed. NA groups are renamed to the literal string `"NA"` and
#'   placed at the end.
#' @param scale A numeric value controlling the vertical overlap of ridges.
#'   Passed to `ggridges::geom_density_ridges(scale = ...)`. Smaller values
#'   increase overlap. When `NULL`, ggridges auto-computes the scale.
#' @param add_vline A specification for vertical reference lines:
#'   - `NULL` or `FALSE`: no lines.
#'   - `TRUE`: draw a line at the mean of each group.
#'   - A numeric vector: draw the same lines for all groups.
#'   - A named list of numeric vectors: per-group lines, where names should
#'     match `group_by` levels.
#' @param vline_type A character string specifying the line type for the
#'   vertical reference lines. Passed as `linetype` to `geom_vline()`.
#'   Default: `"solid"`.
#' @param vline_color The colour of the vertical reference lines:
#'   - A literal colour value or vector (recycled): applied directly.
#'   - `TRUE` (default): each line is coloured with a darkened blend of
#'     the corresponding ridge fill colour, computed via
#'     `blend_colors(mode = "multiply")`.
#' @param vline_width A numeric value for the thickness of the vertical
#'   reference lines. Passed as `linewidth` to `geom_vline()`.
#'   Default: `0.5`.
#' @param vline_alpha A numeric value in `[0, 1]` for the transparency of
#'   the vertical reference lines. Default: `1`.
#' @param x_text_angle A numeric value specifying the angle (in degrees) for
#'   x-axis text when `flip = TRUE`. Used with `calc_just()` to compute
#'   optimal `hjust` / `vjust`. Default: `90`.
#' @param x_min,x_max Numeric limits for the x-axis. When `NULL` (default),
#'   limits are determined from the data range. Passed to `scale_x_continuous(limits = ...)`.
#' @param ... Additional arguments passed to `ggridges::geom_density_ridges()`
#'   (bandwidth, jittered_points, quantile_lines, etc.).
#' @importFrom tidyr pivot_longer
#' @keywords internal
RidgePlotAtomic <- function(
    data,
    x = NULL,
    in_form = c("long", "wide"),
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    add_vline = NULL,
    vline_type = "solid",
    vline_color = TRUE,
    vline_width = 0.5,
    vline_alpha = 1,
    flip = FALSE,
    alpha = 0.8,
    scale = NULL,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    x_text_angle = 90,
    x_min = NULL,
    x_max = NULL,
    keep_na = FALSE,
    keep_empty = FALSE,
    reverse = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    legend.position = "none",
    legend.direction = "vertical",
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    in_form <- match.arg(in_form)
    if (in_form == "wide") {
        data <- data %>%
            pivot_longer(cols = group_by, names_to = ".group", values_to = ".x")
        x <- ".x"
        group_by <- ".group"
    }
    x <- check_columns(data, x)
    group_by <- check_columns(
        data,
        group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = group_by_sep
    )
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )
    if (is.null(group_by)) {
        group_by <- ".group"
        data[[group_by]] <- factor(" ")
    }
    if (isTRUE(reverse)) {
        data[[group_by]] <- factor(
            data[[group_by]],
            levels = rev(levels(data[[group_by]]))
        )
    }

    data <- process_keep_na_empty(data, keep_na, keep_empty)
    keep_empty_group <- if (!is.null(group_by)) keep_empty[[group_by]] else NULL
    keep_empty_facet <- if (!is.null(facet_by)) keep_empty[[facet_by]] else NULL
    if (length(facet_by) > 1) {
        stopifnot(
            "[RidgePlot] `keep_empty` for `facet_by` variables must be identical." = identical(
                keep_empty_facet,
                keep_empty[[facet_by[2]]]
            )
        )
    }
    group_vals <- levels(data[[group_by]])
    if (anyNA(data[[group_by]])) {
        group_vals <- c(group_vals, NA)
    }

    x_min_user <- !is.null(x_min)
    x_max_user <- !is.null(x_max)
    x_min <- x_min %||% min(data[[x]], na.rm = TRUE)
    x_max <- x_max %||% max(data[[x]], na.rm = TRUE)
    if (x_min == x_max) {
        stop("[RidgePlot] x_min and x_max are equal. Please provide a valid range for the x-axis.")
    }
    colors <- palette_this(
        group_vals,
        palette = palette,
        palcolor = palcolor,
        NA_keep = TRUE,
        reverse = palreverse
    )
    if (anyNA(group_vals) && reverse) {
        names(colors)[is.na(names(colors))] <- "NA"
        group_vals[is.na(group_vals)] <- "NA"
        levels(data[[group_by]]) <- c("NA", setdiff(group_vals, "NA"))
        data[[group_by]][is.na(data[[group_by]])] <- "NA"
    }

    p <- ggplot(
        data,
        aes(x = !!sym(x), y = !!sym(group_by), fill = !!sym(group_by))
    )

    if (!is.null(scale)) {
        p <- p +
            ggridges::geom_density_ridges(
                alpha = alpha,
                scale = scale,
                show.legend = TRUE
            )
    } else {
        # Let the geom_density_ridges function to calculate the scale
        p <- p +
            ggridges::geom_density_ridges(alpha = alpha, show.legend = TRUE)
    }
    if (!is.null(add_vline) && !isFALSE(add_vline)) {
        if (isTRUE(add_vline)) {
            # calculate the mean of each group
            add_vline <- tapply(data[[x]], data[[group_by]], mean, na.rm = TRUE)
        }
        if (isTRUE(vline_color)) {
            if (!is.list(add_vline)) {
                add_vline <- as.list(add_vline)
                if (reverse) {
                    names(add_vline) <- rev(levels(data[[group_by]]))[
                        1:length(add_vline)
                    ]
                } else {
                    names(add_vline) <- levels(data[[group_by]])[
                        1:length(add_vline)
                    ]
                }
            }
            add_vline <- add_vline[intersect(
                levels(data[[group_by]]),
                names(add_vline)
            )]
            vline_color <- sapply(
                colors[names(add_vline)],
                function(cl) blend_colors(c(cl, cl, cl), mode = "multiply")
            )
            add_vline <- unlist(add_vline, use.names = FALSE)
        }
        p <- p +
            geom_vline(
                xintercept = add_vline,
                linetype = vline_type,
                linewidth = vline_width,
                color = vline_color,
                alpha = vline_alpha
            )
    }
    p <- p +
        scale_y_discrete(drop = !isTRUE(keep_empty_group), expand = c(0, 0))
    if (x_min_user || x_max_user) {
        expand_x <- c(
            if (x_min_user) 0 else 0.05,  # mult left
            0,                              # add left
            if (x_max_user) 0 else 0.05,   # mult right
            0                               # add right
        )
        p <- p + scale_x_continuous(
            limits = c(
                if (x_min_user) x_min else NA,
                if (x_max_user) x_max else NA
            ),
            expand = expand_x
        )
    } else {
        p <- p + scale_x_continuous()
    }
    p <- p +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% x,
            y = ylab %||% group_by
        )

    if (isTRUE(keep_empty_group)) {
        p <- p +
            scale_fill_manual(
                name = group_name %||% group_by,
                na.value = colors[length(colors)],
                values = colors,
                breaks = group_vals,
                limits = group_vals,
                drop = FALSE
            )
    } else {
        p <- p +
            scale_fill_manual(
                name = group_name %||% group_by,
                na.value = colors[length(colors)],
                values = colors
            )
    }

    if (flip) {
        just <- calc_just(x_text_angle)
        p <- p +
            ggplot2::theme(
                axis.text.x = element_text(
                    angle = x_text_angle,
                    hjust = just$h,
                    vjust = just$v
                ),
                axis.ticks.x = element_line(),
                panel.grid.major.x = element_line(color = "grey", linetype = 2)
            ) +
            coord_flip()
    } else {
        p <- p +
            ggplot2::theme(
                axis.text.x = element_text(),
                axis.text.y = element_text(hjust = 1),
                axis.ticks.y = element_line(),
                panel.grid.major.y = element_line(color = "grey", linetype = 2)
            )
    }

    p <- p +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction
        )

    dims <- calculate_plot_dimensions(
        base_height = 1,
        aspect.ratio = aspect.ratio,
        n_y = nlevels(data[[group_by]]),
        y_scale_factor = 1,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = length(group_vals),
        legend_nchar = max(nchar(as.character(group_vals)), na.rm = TRUE),
        flip = flip
    )
    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    facet_plot(
        p,
        facet_by = facet_by,
        facet_scales = facet_scales,
        ncol = facet_ncol,
        nrow = facet_nrow,
        byrow = facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction,
        drop = !isTRUE(keep_empty_facet)
    )
}

#' Ridge Plot
#'
#' @description
#' Ridge (joy) plot for visualising the distribution of a numeric variable across
#' multiple groups. Each group is rendered as a partially overlapping density
#' curve along the y-axis, making it easy to compare distribution shapes, central
#' tendency, and spread across categories.
#'
#' The function supports both **long** and **wide** data formats:
#' - **Long form** (`in_form = "long"`, default) — a numeric column (`x`) plus a
#'   factor column (`group_by`) whose levels become the y-axis ridges.
#' - **Wide form** (`in_form = "wide"`) — multiple numeric columns listed in
#'   `group_by` are gathered internally into long form.
#'
#' Optional vertical reference lines (`add_vline`) can mark group means,
#' specific values, or per-group thresholds. Supports faceting, split-by
#' splitting, and full palette customisation.
#'
#' @section split_by Workflow:
#'
#' When `split_by` is specified, `RidgePlot()` executes the following pipeline:
#'
#' 1. **Argument validation** — `validate_common_args()` checks the seed and
#'    facet-by consistency.
#' 2. **NA / empty normalisation** — `check_keep_na()` / `check_keep_empty()`
#'    convert `keep_na` / `keep_empty` to per-column lists.
#' 3. **Theme resolution** — `process_theme()` resolves the theme string to a
#'    theme function.
#' 4. **Split column resolution** — `check_columns()` validates `split_by`
#'    (force_factor, concat_multi).
#' 5. **Pre-filtering** — `process_keep_na_empty()` removes NA / empty levels
#'    from the split column, then `data` is split by `split_by` levels (order
#'    preserved).
#' 6. **Per-split parameter resolution** — `check_palette()`,
#'    `check_palcolor()`, `check_legend()` resolve palette, palcolor,
#'    legend.position, and legend.direction for each split.
#' 7. **Per-split dispatch** — each split is passed to `RidgePlotAtomic()` with
#'    its resolved parameters. Title defaults to the split level name unless
#'    `title` is a function (in which case it is called with the default).
#' 8. **Combination** — `combine_plots()` assembles the list of plots via
#'    `patchwork::wrap_plots()`, applying `nrow`, `ncol`, `byrow`, `axes`,
#'    `axis_titles`, `guides`, and `design`.
#'
#' @inheritParams common_args
#' @inheritParams RidgePlotAtomic
#' @return A `ggplot` object (single plot), a `patchwork` / `wrap_plots` object
#'   (when `split_by` is provided and `combine = TRUE`), or a list of `ggplot`
#'   objects (when `split_by` is provided and `combine = FALSE`).
#' @export
#' @examples
#' \donttest{
#' set.seed(8525)
#' data <- data.frame(
#'    x = c(rnorm(250, -1), rnorm(250, 1)),
#'    group = factor(rep(c("A", NA, LETTERS[3:5]), each = 100), levels = LETTERS[1:6])
#' )
#'
#' # basic usage
#' RidgePlot(data, x = "x")  # single ridge (no group_by)
#' RidgePlot(data, x = "x", add_vline = 0, vline_color = "black")
#'
#' # grouped ridges
#' RidgePlot(data, x = "x", group_by = "group")
#' RidgePlot(data, x = "x", group_by = "group",
#'    keep_na = TRUE, keep_empty = TRUE)
#' RidgePlot(data, x = "x", group_by = "group", reverse = TRUE)
#' RidgePlot(data, x = "x", group_by = "group",
#'    add_vline = TRUE, vline_color = TRUE, alpha = 0.7)
#'
#' # faceting
#' RidgePlot(data, x = "x", facet_by = "group",
#'    keep_na = TRUE, keep_empty = TRUE)
#'
#' # wide form
#' data_wide <- data.frame(
#'    A = rnorm(100),
#'    B = rnorm(100),
#'    C = rnorm(100),
#'    D = rnorm(100),
#'    E = rnorm(100),
#'    group = sample(letters[1:4], 100, replace = TRUE)
#' )
#' RidgePlot(data_wide, group_by = LETTERS[1:5], in_form = "wide")
#' RidgePlot(data_wide, group_by = LETTERS[1:5], in_form = "wide", facet_by = "group")
#'
#' # split_by with per-split palettes
#' RidgePlot(data_wide, group_by = LETTERS[1:5], in_form = "wide", split_by = "group",
#'    palette = list(a = "Reds", b = "Blues", c = "Greens", d = "Purples"))
#' }
RidgePlot <- function(
    data,
    x = NULL,
    in_form = c("long", "wide"),
    split_by = NULL,
    split_by_sep = "_",
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    scale = NULL,
    keep_na = FALSE,
    keep_empty = FALSE,
    add_vline = NULL,
    vline_type = "solid",
    vline_color = TRUE,
    vline_width = 0.5,
    vline_alpha = 1,
    flip = FALSE,
    alpha = 0.8,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    x_text_angle = 90,
    x_min = NULL,
    x_max = NULL,
    reverse = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    legend.position = "none",
    legend.direction = "vertical",
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    seed = 8525,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    keep_na <- check_keep_na(keep_na, c(group_by, split_by, facet_by))
    keep_empty <- check_keep_empty(keep_empty, c(group_by, split_by, facet_by))
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }

    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            RidgePlotAtomic(
                datas[[nm]],
                x = x,
                in_form = in_form,
                group_by = group_by,
                group_by_sep = group_by_sep,
                group_name = group_name,
                scale = scale,
                add_vline = add_vline,
                vline_type = vline_type,
                vline_color = vline_color,
                vline_width = vline_width,
                vline_alpha = vline_alpha,
                flip = flip,
                alpha = alpha,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                x_text_angle = x_text_angle,
                x_min = x_min,
                x_max = x_max,
                keep_na = keep_na,
                keep_empty = keep_empty,
                reverse = reverse,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

#' Density Plot / Histogram
#'
#' @description
#' Density plot for visualising the distribution of a numeric variable. Uses
#' `ggplot2::geom_density()` to render smooth kernel density estimates, with
#' optional grouping, faceting, split-by splitting, and data-distribution rug
#' bars along the baseline.
#'
#' This is the public entry point for density plots; the companion
#' `Histogram()` function provides binned-histogram rendering.
#' Both dispatch to the same internal engine (`DensityHistoPlotAtomic`)
#' with `type = "density"` or `type = "histogram"` respectively.
#'
#' @section split_by Workflow:
#'
#' When `split_by` is specified, `DensityPlot()` executes the following pipeline:
#'
#' 1. **Argument validation** — `validate_common_args()` checks the seed and
#'    facet-by consistency.
#' 2. **NA / empty normalisation** — `check_keep_na()` / `check_keep_empty()`
#'    convert `keep_na` / `keep_empty` to per-column lists.
#' 3. **Theme resolution** — `process_theme()` resolves the theme string to a
#'    theme function.
#' 4. **Split column resolution** — `check_columns()` validates `split_by`
#'    (force_factor, concat_multi).
#' 5. **Pre-filtering** — `process_keep_na_empty()` removes NA / empty levels
#'    from the split column, then `data` is split by `split_by` levels (order
#'    preserved).
#' 6. **Per-split parameter resolution** — `check_palette()`,
#'    `check_palcolor()`, `check_legend()` resolve palette, palcolor,
#'    legend.position, and legend.direction for each split.
#' 7. **Per-split dispatch** — each split is passed to
#'    `DensityHistoPlotAtomic(type = "density", ...)` with its resolved
#'    parameters. Title defaults to the split level name unless `title` is
#'    a function.
#' 8. **Combination** — `combine_plots()` assembles the list of plots via
#'    `patchwork::wrap_plots()`, applying `nrow`, `ncol`, `byrow`, `axes`,
#'    `axis_titles`, `guides`, and `design`.
#'
#' @rdname densityhistoplot
#' @inheritParams common_args
#' @inheritParams DensityHistoPlotAtomic
#' @return A `ggplot` object (single plot), a `patchwork` / `wrap_plots` object
#'   (when `split_by` is provided and `combine = TRUE`), or a list of `ggplot`
#'   objects (when `split_by` is provided and `combine = FALSE`).
#' @export
#' @examples
#' \donttest{
#' set.seed(8525)
#' data <- data.frame(
#'     x = c(rnorm(500, -1), rnorm(500, 1)),
#'     group = factor(rep(c("A", NA, "C", "D"), each = 250), levels = LETTERS[1:4]),
#'     facet = sample(c("F1", "F2"), 1000, replace = TRUE)
#' )
#'
#' # basic density
#' DensityPlot(data, x = "x")
#' DensityPlot(data, x = "x", group_by = "group")
#'
#' # NA / empty level handling
#' DensityPlot(data, x = "x", group_by = "group",
#'     keep_na = TRUE, keep_empty = TRUE)
#' DensityPlot(data, x = "x", group_by = "group",
#'     keep_na = TRUE, keep_empty = 'level')
#'
#' # faceting and splitting
#' DensityPlot(data, x = "x", group_by = "group", facet_by = "facet")
#' DensityPlot(data, x = "x", split_by = "facet", add_bars = TRUE)
#' DensityPlot(data, x = "x", split_by = "facet", add_bars = TRUE,
#'     palette = c(F1 = "Set1", F2 = "Set2"))
#' }
DensityPlot <- function(
    data,
    x,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    xtrans = "identity",
    ytrans = "identity",
    split_by = NULL,
    split_by_sep = "_",
    flip = FALSE,
    position = "identity",
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = .5,
    theme = "theme_this",
    theme_args = list(),
    add_bars = FALSE,
    bar_height = 0.025,
    bar_alpha = 1,
    bar_width = .1,
    keep_na = FALSE,
    keep_empty = FALSE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    expand = c(bottom = 0, left = 0, right = 0),
    facet_by = NULL,
    facet_scales = "free_y",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    legend.position = ifelse(is.null(group_by), "none", "right"),
    legend.direction = "vertical",
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    keep_na <- check_keep_na(keep_na, c(group_by, split_by, facet_by))
    keep_empty <- check_keep_empty(keep_empty, c(group_by, split_by, facet_by))
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )
    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }

    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            DensityHistoPlotAtomic(
                datas[[nm]],
                x = x,
                group_by = group_by,
                group_by_sep = group_by_sep,
                group_name = group_name,
                type = "density",
                flip = flip,
                xtrans = xtrans,
                ytrans = ytrans,
                position = position,
                aspect.ratio = aspect.ratio,
                add_bars = add_bars,
                bar_height = bar_height,
                bar_alpha = bar_alpha,
                bar_width = bar_width,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                alpha = alpha,
                theme = theme,
                theme_args = theme_args,
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                expand = expand,
                keep_na = keep_na,
                keep_empty = keep_empty,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

#' @description
#' Histogram for visualising the distribution of a numeric variable via binned
#' counts. Uses `ggplot2::geom_histogram()`, with optional trend-line overlays,
#' zero-skip interpolation, grouping, faceting, and split-by splitting.
#'
#' This is the histogram companion to `DensityPlot()`. Both dispatch to the
#' same internal engine (`DensityHistoPlotAtomic`) with `type = "histogram"`
#' or `type = "density"` respectively.
#'
#' When `use_trend = TRUE`, the histogram bars are replaced entirely by a
#' point-and-line trend; when `add_trend = TRUE`, the trend is overlaid on top
#' of the bars. The `trend_skip_zero` option uses `zoo::na.approx()` to
#' interpolate across empty bins for a continuous trend curve — particularly
#' useful with transformed y-axes.
#'
#' @section split_by Workflow:
#'
#' When `split_by` is specified, `Histogram()` executes the following pipeline:
#'
#' 1. **Argument validation** — `validate_common_args()` checks the seed and
#'    facet-by consistency.
#' 2. **NA / empty normalisation** — `check_keep_na()` / `check_keep_empty()`
#'    convert `keep_na` / `keep_empty` to per-column lists.
#' 3. **Theme resolution** — `process_theme()` resolves the theme string to a
#'    theme function.
#' 4. **Split column resolution** — `check_columns()` validates `split_by`
#'    (force_factor, concat_multi).
#' 5. **Pre-filtering** — `process_keep_na_empty()` removes NA / empty levels
#'    from the split column, then `data` is split by `split_by` levels (order
#'    preserved).
#' 6. **Per-split parameter resolution** — `check_palette()`,
#'    `check_palcolor()`, `check_legend()` resolve palette, palcolor,
#'    legend.position, and legend.direction for each split.
#' 7. **Per-split dispatch** — each split is passed to
#'    `DensityHistoPlotAtomic(type = "histogram", ...)` with its resolved
#'    parameters (including `bins`, `binwidth`, `use_trend`, `add_trend`,
#'    `trend_skip_zero`, `trend_alpha`, `trend_linewidth`, `trend_pt_size`).
#'    Title defaults to the split level name unless `title` is a function.
#' 8. **Combination** — `combine_plots()` assembles the list of plots via
#'    `patchwork::wrap_plots()`, applying `nrow`, `ncol`, `byrow`, `axes`,
#'    `axis_titles`, `guides`, and `design`.
#'
#' @rdname densityhistoplot
#' @inheritParams common_args
#' @inheritParams DensityHistoPlotAtomic
#' @return A `ggplot` object (single plot), a `patchwork` / `wrap_plots` object
#'   (when `split_by` is provided and `combine = TRUE`), or a list of `ggplot`
#'   objects (when `split_by` is provided and `combine = FALSE`).
#' @export
#' @examples
#' set.seed(8525)
#' data <- data.frame(
#'     x = sample(setdiff(1:100, c(30:36, 50:55, 70:77)), 1000, replace = TRUE),
#'     group = factor(rep(c("A", "B", NA, "D"), each = 250), levels = LETTERS[1:4]),
#'     facet = sample(c("F1", "F2"), 1000, replace = TRUE)
#' )
#'
#' # basic histogram
#' Histogram(data, x = "x")
#' Histogram(data, x = "x", group_by = "group")
#'
#' # NA / empty level handling
#' Histogram(data, x = "x", group_by = "group", keep_na = TRUE, keep_empty = 'level')
#'
#' # add_bars and trend overlays
#' Histogram(data, x = "x", split_by = "facet", add_bars = TRUE)
#' Histogram(data, x = "x", group_by = "group", add_trend = TRUE)
#' Histogram(data, x = "x", group_by = "group", add_trend = TRUE, trend_skip_zero = TRUE)
#'
#' # use_trend replaces bars entirely
#' Histogram(data, x = "x", group_by = "group", split_by = "facet",
#'  use_trend = TRUE, trend_pt_size = 3)
#'
#' # per-split palettes
#' Histogram(data, x = "x", group_by = "group", split_by = "facet",
#'  palette = c(F1 = "Paired", F2 = "Spectral"))
Histogram <- function(
    data,
    x,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    xtrans = "identity",
    ytrans = "identity",
    split_by = NULL,
    split_by_sep = "_",
    flip = FALSE,
    bins = NULL,
    binwidth = NULL,
    trend_skip_zero = FALSE,
    add_bars = FALSE,
    bar_height = 0.025,
    bar_alpha = 1,
    bar_width = .1,
    position = "identity",
    keep_na = FALSE,
    keep_empty = FALSE,
    use_trend = FALSE,
    add_trend = FALSE,
    trend_alpha = 1,
    trend_linewidth = 0.8,
    trend_pt_size = 1.5,
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = .5,
    theme = "theme_this",
    theme_args = list(),
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    expand = c(bottom = 0, left = 0, right = 0),
    facet_by = NULL,
    facet_scales = "free_y",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    legend.position = ifelse(is.null(group_by), "none", "right"),
    legend.direction = "vertical",
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    keep_na <- check_keep_na(keep_na, c(group_by, split_by, facet_by))
    keep_empty <- check_keep_empty(keep_empty, c(group_by, split_by, facet_by))
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }

    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            DensityHistoPlotAtomic(
                datas[[nm]],
                x = x,
                group_by = group_by,
                group_by_sep = group_by_sep,
                group_name = group_name,
                type = "histogram",
                flip = flip,
                xtrans = xtrans,
                ytrans = ytrans,
                use_trend = use_trend,
                trend_skip_zero = trend_skip_zero,
                add_trend = add_trend,
                trend_alpha = trend_alpha,
                trend_linewidth = trend_linewidth,
                trend_pt_size = trend_pt_size,
                add_bars = add_bars,
                bar_height = bar_height,
                bar_alpha = bar_alpha,
                bar_width = bar_width,
                bins = bins,
                binwidth = binwidth,
                expand = expand,
                position = position,
                keep_na = keep_na,
                keep_empty = keep_empty,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                alpha = alpha,
                theme = theme,
                theme_args = theme_args,
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                aspect.ratio = aspect.ratio,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
data <- data.frame(
    x = c(rnorm(500, mean = 5, sd = 2), rnorm(500, mean = 8, sd = 1.5)),
    group = factor(rep(c("A", "B"), each = 500)),
    facet = factor(rep(c("F1", "F2"), 500))
)

# DensityPlot tests
test_that("DensityPlot returns a ggplot for basic usage", {
    p <- DensityPlot(data, x = "x")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("DensityPlot with group_by works", {
    p <- DensityPlot(data, x = "x", group_by = "group")
    expect_s3_class(p, "ggplot")
})

test_that("DensityPlot sets title and labels", {
    p <- DensityPlot(data, x = "x", title = "Density", xlab = "Value")
    expect_equal(p$labels$title, "Density")
})

test_that("DensityPlot with split_by returns combined", {
    p <- DensityPlot(data, x = "x", split_by = "facet", combine = TRUE)
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("DensityPlot with split_by and combine = FALSE returns list", {
    plots <- DensityPlot(data, x = "x", split_by = "facet", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
})

test_that("DensityPlot with multiple split_by columns returns a list with one plot per combination", {
    plots <- suppressMessages(DensityPlot(data, x = "x", split_by = c("group", "facet"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("DensityPlot with multiple split_by columns returns combined plot", {
    p <- suppressMessages(DensityPlot(data, x = "x", split_by = c("group", "facet"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("DensityPlot with facet_by works", {
    p <- DensityPlot(data, x = "x", facet_by = "facet")
    expect_s3_class(p, "ggplot")
})

test_that("DensityPlot with add_bars = TRUE works", {
    p <- DensityPlot(data, x = "x", group_by = "group", add_bars = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("DensityPlot with flip = TRUE works", {
    p <- DensityPlot(data, x = "x", flip = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("DensityPlot width/height attributes are numeric", {
    p <- DensityPlot(data, x = "x")
    expect_true(is.numeric(attr(p, "width")))
    expect_true(is.numeric(attr(p, "height")))
})

# Histogram tests
test_that("Histogram returns a ggplot for basic usage", {
    p <- Histogram(data, x = "x")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("Histogram with group_by works", {
    p <- Histogram(data, x = "x", group_by = "group")
    expect_s3_class(p, "ggplot")
})

test_that("Histogram with binwidth works", {
    p <- Histogram(data, x = "x", binwidth = 0.5)
    expect_s3_class(p, "ggplot")
})

test_that("Histogram with bins works", {
    p <- Histogram(data, x = "x", bins = 20)
    expect_s3_class(p, "ggplot")
})

test_that("Histogram with add_bars = TRUE works", {
    p <- Histogram(data, x = "x", group_by = "group", add_bars = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("Histogram with add_trend = TRUE works", {
    p <- Histogram(data, x = "x", group_by = "group", add_trend = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("Histogram with split_by and combine = FALSE returns list", {
    plots <- Histogram(data, x = "x", split_by = "facet", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
})

test_that("Histogram with multiple split_by columns returns a list with one plot per combination", {
    plots <- suppressMessages(Histogram(data, x = "x", split_by = c("group", "facet"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("Histogram sets title", {
    p <- Histogram(data, x = "x", title = "Histogram Title")
    expect_equal(p$labels$title, "Histogram Title")
})

test_that("Histogram width/height attributes are numeric", {
    p <- Histogram(data, x = "x")
    expect_true(is.numeric(attr(p, "width")))
    expect_true(is.numeric(attr(p, "height")))
})

# RidgePlot tests
test_that("RidgePlot returns a ggplot for basic usage", {
    skip_if_not_installed("ggridges")
    p <- RidgePlot(data, x = "x", group_by = "group")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("RidgePlot sets title", {
    skip_if_not_installed("ggridges")
    p <- RidgePlot(data, x = "x", group_by = "group", title = "Ridge Title")
    expect_equal(p$labels$title, "Ridge Title")
})

test_that("RidgePlot with multiple split_by columns returns a list with one plot per combination", {
    skip_if_not_installed("ggridges")
    plots <- suppressMessages(RidgePlot(data, x = "x", group_by = "group", split_by = c("group", "facet"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("RidgePlot with multiple split_by columns returns combined plot", {
    skip_if_not_installed("ggridges")
    p <- suppressMessages(RidgePlot(data, x = "x", group_by = "group", split_by = c("group", "facet"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

#' Atomic clustree plot
#'
#' Core implementation for clustree (clustering tree) plots. This internal
#' function renders a tree-based visualisation showing how cluster assignments
#' change across increasing clustering resolutions. It is called by the
#' exported \code{ClustreePlot()} function.
#'
#' The function expects a data frame containing columns named with a common
#' \code{prefix} followed by numeric resolution values (e.g. \code{"res_0.1"},
#' \code{"res_0.3"}, \code{"res_0.5"}). Each column represents cluster
#' assignments at a given resolution, and the tree visualises how cells (rows)
#' transition between clusters as resolution increases.
#'
#' The core layout is produced via \code{clustree::clustree()}, which computes
#' a \code{ggraph} layout (default: \code{"sugiyama"}) linking clusters across
#' adjacent resolutions. Nodes represent clusters and edges represent cell
#' transitions; edge width and colour encode the number of transitioning cells.
#' Cluster labels are overlaid on each node via \code{ggplot2::geom_text()}.
#'
#' Key features:
#' \itemize{
#'   \item \strong{Resolution-level node colouring} — each resolution level
#'   receives a distinct colour from \code{palette} via
#'   \code{ggplot2::scale_color_manual()}.
#'   \item \strong{Edge gradient} — edges are coloured by transition count
#'   using \code{ggraph::scale_edge_color_gradientn()} with the
#'   \code{edge_palette} gradient.
#'   \item \strong{Flip support} — \code{flip = TRUE} places resolutions on
#'   the x-axis with cluster IDs as y-axis row labels for left-to-right
#'   reading.
#'   \item \strong{Automatic dimension calculation} — plot height and width
#'   are sized based on the number of resolutions, number of clusters, and
#'   legend configuration, with different formulae for flipped vs. non-flipped
#'   layouts.
#' }
#'
#' @section Architecture:
#'
#' \strong{ClustreePlotAtomic} executes the following steps:
#' \enumerate{
#'   \item \strong{Column selection} — selects all columns whose names start
#'   with \code{prefix} via \code{dplyr::starts_with()}.
#'   \item \strong{NA filtering} — removes rows with any missing values via
#'   \code{stats::complete.cases()}.
#'   \item \strong{Expansion normalisation} — \code{\link{norm_expansion}()}
#'   parses the \code{expand} argument for continuous x and y axes.
#'   \item \strong{Empty data guard} — stops with an informative error if no
#'   columns match the prefix or no complete rows remain.
#'   \item \strong{Resolution parsing} — extracts the suffix after
#'   \code{prefix} from each column name and strips leading separators
#'   (\code{"_"} or \code{"."}) if present. Values are converted to numeric
#'   and sorted. Column names are rewritten to use the original prefix for
#'   consistent \code{clustree} processing.
#'   \item \strong{Cluster counting} — computes the number of unique clusters
#'   at the highest resolution (\code{max_clusters}), used for dimension
#'   calculation in the non-flipped layout.
#'   \item \strong{Argument merging} — builds a call to
#'   \code{clustree::clustree()} by merging user-supplied \code{...} with
#'   sensible defaults for \code{node_alpha}, \code{edge_width},
#'   \code{show_axis}, \code{layout} (\code{"sugiyama"}),
#'   \code{node_size_range} (\code{c(6, 12)}), \code{node_text_size}
#'   (\code{3}), and \code{node_text_colour} (\code{"black"}).
#'   \item \strong{clustree execution} — calls
#'   \code{gglogger::register(clustree::clustree)} with the merged arguments
#'   to produce the base \code{ggplot} object.
#'   \item \strong{Node labels} — overlays \code{ggplot2::geom_text()} with
#'   the cluster label from the \code{clustree} output, using the configured
#'   text size and colour.
#'   \item \strong{Resolution colour scale} — applies
#'   \code{ggplot2::scale_color_manual()} with one colour per resolution
#'   level using \code{palette_this()}. The colour legend is suppressed (the
#'   edge gradient legend serves as the primary guide).
#'   \item \strong{Edge colour scale} — applies
#'   \code{ggraph::scale_edge_color_gradientn()} with the
#'   \code{edge_palette} palette, encoding transition counts via a
#'   colour-bar guide with black frame and ticks.
#'   \item \strong{Theme and labels} — applies the selected theme and sets
#'   \code{title}, \code{subtitle}, x-axis label, and y-axis label (default
#'   y-label is the \code{prefix} with trailing separator stripped).
#'   \item \strong{Flip branch} (\code{flip = TRUE}):
#'     \itemize{
#'       \item Resolutions are placed on the x-axis (horizontal); clusters
#'       become y-axis rows.
#'       \item Width scales with \code{max(3, 0.5 + nres * 1.0)}; height
#'       scales with \code{max(3, 0.5 + max_clusters * 0.5)} (or via
#'       aspect.ratio).
#'       \item \code{scale_y_reverse()} + \code{coord_flip()} reorients the
#'       layout so resolutions read left-to-right.
#'       \item Horizontal grid lines are dashed grey80; y-axis text and
#'       ticks are suppressed.
#'     }
#'   \item \strong{Non-flip branch} (\code{flip = FALSE}, default):
#'     \itemize{
#'       \item Resolutions are on the y-axis (rows); clusters spread across
#'       the x-axis (columns).
#'       \item Width scales with \code{max(3, 0.5 + max_clusters * 0.5)};
#'       height scales with \code{max(3, 0.5 + nres * 1.0)}
#'       (or via aspect.ratio).
#'       \item \code{coord_cartesian(clip = "off")} and
#'       \code{scale_y_continuous()} show resolution labels on the y-axis.
#'       \item Vertical grid lines are dashed grey80; x-axis text and ticks
#'       are suppressed.
#'     }
#'   \item \strong{Dimension storage} — \code{\link{calculate_plot_dimensions}()}
#'   computes \code{height} and \code{width} attributes, stored on the
#'   \code{ggplot} object.
#' }
#'
#' @inheritParams common_args
#' @param prefix A character string specifying the common prefix of the
#'   resolution columns in \code{data}. All columns whose names start with
#'   this prefix are selected as resolution columns. The suffix after the
#'   prefix is parsed as a numeric resolution value. Supports \code{"_"}
#'   and \code{"."} as separators between the prefix and the resolution
#'   value (e.g. \code{"res_0.5"} or \code{"p.0.5"}).
#' @param flip A logical value. If \code{TRUE}, the tree is flipped so that
#'   resolutions are displayed on the x-axis (left to right) and cluster
#'   assignments are shown as row labels on the y-axis.
#'   Default: \code{FALSE}.
#' @param alpha A numeric value in \code{[0, 1]} specifying the transparency
#'   of the nodes in the clustree plot. Only used when \code{node_alpha} is
#'   not explicitly provided via \code{...}. Default: \code{0.85}.
#' @param edge_palette A character string specifying the palette name for
#'   the edge colour gradient. Edges are coloured by the number of
#'   transitioning cells between clusters at adjacent resolutions, using
#'   \code{ggraph::scale_edge_color_gradientn()}. Default: \code{"Spectral"}.
#' @param edge_palcolor A character vector of custom colours for the edge
#'   colour gradient. When \code{NULL} (the default), colours are derived
#'   from \code{edge_palette}.
#' @param ... Additional arguments passed to \code{clustree::clustree()}.
#'   Commonly used overrides include \code{node_size_range},
#'   \code{node_text_size}, \code{layout} (default: \code{"sugiyama"}),
#'   \code{show_axis}, and \code{node_text_colour}. Note that \code{x}
#'   (the data) and \code{prefix} are set internally and cannot be
#'   overridden here.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'   attributes.
#' @keywords internal
#' @importFrom stats complete.cases
#' @importFrom dplyr %>% select starts_with
#' @importFrom ggplot2 scale_color_manual coord_cartesian element_line element_blank labs coord_flip
#' @importFrom ggplot2 scale_y_reverse coord_flip
#' @importFrom ggrepel geom_text_repel
ClustreePlotAtomic <- function(
    data,
    prefix,
    flip = FALSE,
    alpha = 0.85,
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    edge_palette = "Spectral",
    edge_palcolor = NULL,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    expand = c(0.1, 0.1),
    theme = "theme_this",
    theme_args = list(),
    ...
) {
    # if (!requireNamespace("ggraph", quietly = TRUE)) {
    #     stop("ggraph is not available for ClustreePlot. Please install it first.")
    # }
    # ! Unknown guide: edge_colourbar
    # suppressMessages(suppressWarnings(suppressPackageStartupMessages(library(ggraph))))
    # if (!requireNamespace("clustree", quietly = TRUE)) {
    #     stop("clustree is not available for ClustreePlot. Please install it first.")
    # }

    data <- data %>% select(starts_with(prefix))
    data <- data[complete.cases(data), , drop = FALSE]
    expand <- norm_expansion(
        expand,
        x_type = "continuous",
        y_type = "continuous"
    )

    if (ncol(data) == 0 || nrow(data) == 0) {
        stop("No data found with prefix '", prefix, "'")
    }
    resolutions <- substring(colnames(data), nchar(prefix) + 1)
    nres <- length(resolutions)
    if (all(startsWith(resolutions, "_"))) {
        prefix <- paste0(prefix, "_")
        resolutions <- substring(resolutions, 2)
    } else if (all(startsWith(resolutions, "."))) {
        prefix <- paste0(prefix, ".")
        resolutions <- substring(resolutions, 2)
    }
    resolutions <- as.numeric(resolutions)
    # make sure res.0.50 to be access by res.0.5
    colnames(data) <- paste0(prefix, resolutions)
    resolutions <- sort(resolutions)
    nres <- length(resolutions)
    max_clusters <- length(unique(data[[paste0(prefix, max(resolutions))]]))
    clustree_args <- list(...)
    clustree_args$x <- data
    clustree_args$prefix <- prefix
    clustree_args$node_alpha <- clustree_args$node_alpha %||% alpha %||% 0.85
    clustree_args$edge_width <- clustree_args$edge_width %||% 0.9
    clustree_args$show_axis <- clustree_args$show_axis %||% TRUE
    clustree_args$layout <- clustree_args$layout %||% "sugiyama"
    clustree_args$node_size_range <- clustree_args$node_size_range %||% c(6, 12)
    clustree_args$node_text_size <- clustree_args$node_text_size %||% 3
    clustree_args$node_text_colour <- clustree_args$node_text_colour %||%
        clustree_args$node_text_color %||%
        "black"
    clustree <- gglogger::register(clustree::clustree)

    p <- suppressMessages(do_call(clustree, clustree_args))

    # make glow effect
    # ggrepel::geom_text_repel with bg.colour is not working well with coord_flip
    # theta <- seq(pi / 8, 2 * pi, length.out = 16)
    # for (i in theta) {
    #     dx <- cos(i) * clustree_args$node_text_size / 10
    #     dy <- sin(i) * clustree_args$node_text_size / 10
    #     p <- p + geom_text(aes(label = cluster, x = x + dx, y = y + dy), fontface = "bold",
    #         size = clustree_args$node_text_size, color = "white")
    # }
    # p <- p + geom_text(aes(label = cluster, x = x + 0.1, y = y + 0.1),
    #     size = clustree_args$node_text_size, color = "white")
    # p <- p + geom_text(aes(label = cluster, x = x + 0.1, y = y - 0.1),
    #     size = clustree_args$node_text_size, color = "white")
    # p <- p + geom_text(aes(label = cluster, x = x - 0.1, y = y - 0.1),
    #     size = clustree_args$node_text_size, color = "white")
    # p <- p + geom_text(aes(label = cluster, x = x - 0.1, y = y - 0.1),
    #     size = clustree_args$node_text_size, color = "white")

    p <- suppressMessages({
        p +
            geom_text(
                aes(label = !!sym("cluster"), x = !!sym("x"), y = !!sym("y")),
                size = clustree_args$node_text_size,
                color = clustree_args$node_text_colour
            ) +
            scale_color_manual(
                values = palette_this(
                    n = nres,
                    palette = palette,
                    palcolor = palcolor,
                    keep_names = FALSE,
                    reverse = palreverse
                ),
                guide = "none"
            ) +
            ggraph::scale_edge_color_gradientn(
                name = "count",
                n.breaks = 5,
                colors = palette_this(
                    palette = edge_palette,
                    palcolor = edge_palcolor,
                    reverse = palreverse
                ),
                na.value = "grey80",
                guide = ggraph::guide_edge_colorbar(
                    frame.colour = "black",
                    ticks.colour = "black",
                    title.hjust = 0
                )
            ) +
            do_call(theme, theme_args) +
            ggplot2::theme(
                aspect.ratio = aspect.ratio,
                axis.title.x = element_text(),
                axis.title.y = element_text(),
                legend.position = legend.position,
                legend.direction = legend.direction,
                # panel.border = element_blank()
            ) +
            labs(
                title = title,
                subtitle = subtitle,
                x = xlab %||% "",
                y = ylab %||% sub("\\.|_$", "", prefix)
            ) +
            scale_x_continuous(expand = expand$x)
    })

    if (isTRUE(flip)) {
        # After flip: resolutions become x-axis columns, clusters become y-axis rows
        # Width driven by nres; height by max_clusters (or aspect.ratio applied to width)
        content_width_fl <- max(3, 0.5 + nres * 1.0)
        content_height_fl <- if (!is.null(aspect.ratio)) {
            max(3, min(content_width_fl * aspect.ratio, 12))
        } else {
            max(3, 0.5 + max_clusters * 0.5)
        }
        dims <- calculate_plot_dimensions(
            base_height = content_height_fl,
            aspect.ratio = NULL,
            n_x = nres,
            x_scale_factor = 1.0,
            legend.position = legend.position,
            legend.direction = legend.direction
        )
        attr(p, "height") <- dims$height
        attr(p, "width") <- dims$width
        p <- suppressMessages({
            p +
                scale_y_reverse(
                    breaks = nres:1,
                    labels = resolutions,
                    expand = expand$y
                ) +
                coord_flip(clip = "off") +
                ggplot2::theme(
                    panel.grid.major.x = element_line(
                        colour = "grey80",
                        linetype = 2
                    ),
                    panel.grid.major.y = element_blank(),
                    axis.line.y = element_blank(),
                    axis.text.y = element_blank(),
                    axis.ticks.y = element_blank()
                )
        })
    } else {
        # Non-flip: resolutions on y-axis (rows), clusters spread on x-axis (columns)
        # Width driven by max_clusters; height by nres (or aspect.ratio applied to width)
        content_width_nf <- max(3, 0.5 + max_clusters * 0.5)
        content_height_nf <- if (!is.null(aspect.ratio)) {
            max(3, min(content_width_nf * aspect.ratio, 12))
        } else {
            max(3, 0.5 + nres * 1.0)
        }
        dims <- calculate_plot_dimensions(
            base_height = content_height_nf,
            aspect.ratio = NULL,
            n_x = max_clusters,
            x_scale_factor = 0.5,
            legend.position = legend.position,
            legend.direction = legend.direction
        )
        attr(p, "height") <- dims$height
        attr(p, "width") <- dims$width
        p <- suppressMessages({
            p +
                coord_cartesian(clip = "off") +
                scale_y_continuous(
                    breaks = nres:1,
                    labels = resolutions,
                    expand = expand$y
                ) +
                ggplot2::theme(
                    panel.grid.major.x = element_blank(),
                    panel.grid.major.y = element_line(
                        colour = "grey80",
                        linetype = 2
                    ),
                    axis.line.x = element_blank(),
                    axis.text.x = element_blank(),
                    axis.ticks.x = element_blank()
                )
        })
    }

    p
}

#' Clustree Plot
#'
#' @description
#' Creates a clustree (clustering tree) plot visualising how cluster
#' assignments change across increasing clustering resolutions. The plot
#' helps identify stable clustering solutions and understand the hierarchical
#' relationships among clusters at different resolution thresholds.
#'
#' The function expects a data frame with columns named by a common
#' \code{prefix} followed by numeric resolution values (e.g.
#' \code{"res_0.1"}, \code{"res_0.3"}, \code{"res_0.5"}). Each column
#' contains cluster labels (factor or character) for every observation at
#' that resolution.
#'
#' Internally, the function uses \code{clustree::clustree()} to compute a
#' \code{ggraph}-based tree layout where nodes are clusters and edges
#' represent cells transitioning between clusters at adjacent resolutions.
#' Edge colour and width encode the number of transitioning cells.
#'
#' Key features:
#' \itemize{
#'   \item \strong{Resolution-level node colouring} — each resolution
#'   receives a distinct colour from the selected \code{palette}.
#'   \item \strong{Edge gradient} — edges are coloured by transition count
#'   using a separate \code{edge_palette} colour gradient.
#'   \item \strong{Flip support} — \code{flip = TRUE} places resolutions
#'   on the x-axis for left-to-right reading.
#'   \item \strong{Split by groups} — \code{split_by} generates per-group
#'   clustree plots that are combined via \code{patchwork}.
#'   \item \strong{Automatic dimensions} — plot height and width are
#'   automatically computed based on the number of resolutions, clusters,
#'   and the legend configuration.
#' }
#'
#' @section split_by Workflow (ClustreePlot):
#'
#' When \code{split_by} is provided, the following pipeline executes:
#' \enumerate{
#'   \item \strong{Argument validation} — \code{validate_common_args()}
#'   checks the \code{seed} value and sets the random seed.
#'   \item \strong{Theme resolution} — \code{process_theme()} resolves the
#'   \code{theme} string or function to a theme function.
#'   \item \strong{Split column validation} — \code{check_columns()}
#'   resolves \code{split_by} with \code{force_factor = TRUE,
#'   allow_multi = TRUE, concat_multi = TRUE}.
#'   \item \strong{Data splitting} — splits \code{data} by \code{split_by}
#'   levels (unused levels dropped), preserving factor level order.
#'   \item \strong{Per-split palette / colour / legend} —
#'   \code{check_palette()}, \code{check_palcolor()}, and
#'   \code{check_legend()} resolve per-split overrides for
#'   \code{palette}, \code{palcolor}, \code{legend.position}, and
#'   \code{legend.direction}.
#'   \item \strong{Per-split title} — when \code{title} is a function, it
#'   receives the default title (the split level name) and can return a
#'   custom string; otherwise \code{title \%||\% split_level} is used.
#'   \item \strong{Dispatch} — each split subset is passed to
#'   \code{\link{ClustreePlotAtomic}} with the per-split parameters.
#'   \item \strong{Combination} — \code{\link{combine_plots}()} assembles
#'   the list of plots via \code{patchwork::wrap_plots}, honouring
#'   \code{nrow}/\code{ncol}/\code{byrow}/\code{design}.
#'   \item \strong{Combined data} — when \code{combine = TRUE}, the
#'   combined plot's \code{data} (\code{p$data}) exposes the node layout
#'   of every split, each row tagged with the \code{split_by} level, plus
#'   a combined graph of all splits' nodes and links (\code{"graph"}
#'   attribute) and the original links (\code{"edges"} attribute).
#' }
#'
#' @inheritParams ClustreePlotAtomic
#' @inheritParams common_args
#' @param split_by The column(s) to split data by and generate separate
#'   clustree plots for each level. Each split level produces an independent
#'   clustree plot via \code{\link{ClustreePlotAtomic}}.
#' @param split_by_sep A character string used to concatenate multiple
#'   \code{split_by} column values when \code{split_by} specifies more than
#'   one column. Default: \code{"_"}.
#' @param seed The random seed for reproducibility. Passed to
#'   \code{validate_common_args()}. Default: \code{8525}.
#' @param combine A logical value. If \code{TRUE} (the default), the list
#'   of per-split plots is combined into a single \code{patchwork} object.
#'   If \code{FALSE}, returns the raw list of \code{ggplot} objects.
#' @param nrow,ncol,byrow Integers controlling the layout of combined plots
#'   via \code{patchwork::wrap_plots()}. \code{byrow = TRUE} (default)
#'   fills the layout row-wise. Ignored when \code{design} is provided.
#' @param axes,axis_titles Strings controlling how axes and axis titles are
#'   handled across combined plots. Passed to \code{\link{combine_plots}()}.
#'   See \code{?patchwork::wrap_plots} for options (\code{"keep"},
#'   \code{"collect"}, \code{"collect_x"}, \code{"collect_y"}).
#' @param guides A string controlling guide collection across combined
#'   plots. Passed to \code{\link{combine_plots}()}.
#' @param design A custom layout specification for combined plots. Passed
#'   to \code{\link{combine_plots}()}. When specified, \code{nrow},
#'   \code{ncol}, and \code{byrow} are ignored.
#' @return A \code{ggplot} object (single plot), a \code{patchwork} object
#'   (when \code{combine = TRUE} with \code{split_by}), or a \code{list} of
#'   \code{ggplot} objects (when \code{combine = FALSE}).
#' @export
#' @examples
#' \donttest{
#' set.seed(8525)
#' N <- 100
#' data <- data.frame(
#'     p.0.4 = sample(LETTERS[1:5], N, replace = TRUE),
#'     p.0.5 = sample(LETTERS[1:6], N, replace = TRUE),
#'     p.0.6 = sample(LETTERS[1:7], N, replace = TRUE),
#'     p.0.7 = sample(LETTERS[1:8], N, replace = TRUE),
#'     p.0.8 = sample(LETTERS[1:9], N, replace = TRUE),
#'     p.0.9 = sample(LETTERS[1:10], N, replace = TRUE),
#'     p.1 = sample(LETTERS[1:30], N, replace = TRUE),
#'     split = sample(1:2, N, replace = TRUE)
#' )
#'
#' # --- Basic clustree plot ---
#' ClustreePlot(data, prefix = "p")
#'
#' # --- Flipped layout (resolutions on x-axis) ---
#' ClustreePlot(data, prefix = "p", flip = TRUE)
#'
#' # --- Split by group ---
#' ClustreePlot(data, prefix = "p", split_by = "split")
#'
#' # --- Split by group with per-split palettes ---
#' ClustreePlot(data, prefix = "p", split_by = "split",
#'              palette = c("1" = "Set1", "2" = "Paired"))
#' }
ClustreePlot <- function(
    data,
    prefix,
    flip = FALSE,
    split_by = NULL,
    split_by_sep = "_",
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    edge_palette = "Spectral",
    edge_palcolor = NULL,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    expand = c(0.1, 0.1),
    theme = "theme_this",
    theme_args = list(),
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    seed = 8525,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed)
    theme <- process_theme(theme)
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }

    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            ClustreePlotAtomic(
                datas[[nm]],
                prefix = prefix,
                flip = flip,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                edge_palette = edge_palette,
                edge_palcolor = edge_palcolor,
                expand = expand,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                theme = theme,
                theme_args = theme_args,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
# ClustreePlot expects data with columns named with a common prefix
# followed by resolution values. E.g., "res.0.1", "res.0.3", "res.0.5"
n_cells <- 100
clustree_data <- data.frame(
    res_0.1 = sample(c("A", "B"), n_cells, replace = TRUE),
    res_0.3 = sample(c("A", "B", "C"), n_cells, replace = TRUE),
    res_0.5 = sample(c("A", "B", "C", "D"), n_cells, replace = TRUE),
    res_0.8 = sample(c("A", "B", "C", "D", "E"), n_cells, replace = TRUE),
    group = factor(rep(c("G1", "G2"), each = n_cells / 2))
)

test_that("ClustreePlot basic usage works", {
    skip_if_not_installed("clustree")
    p <- ClustreePlot(clustree_data, prefix = "res_")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("ClustreePlot with title and subtitle works", {
    skip_if_not_installed("clustree")
    p <- ClustreePlot(clustree_data, prefix = "res_",
                      title = "Clustree Title", subtitle = "Subtitle")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "Clustree Title")
    expect_equal(p$labels$subtitle, "Subtitle")
})

test_that("ClustreePlot with flip = TRUE works", {
    skip_if_not_installed("clustree")
    p <- ClustreePlot(clustree_data, prefix = "res_", flip = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("ClustreePlot with split_by returns patchwork", {
    skip_if_not_installed("clustree")
    p <- ClustreePlot(clustree_data, prefix = "res_",
                      split_by = "group", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("ClustreePlot with combine = FALSE returns list", {
    skip_if_not_installed("clustree")
    plots <- ClustreePlot(clustree_data, prefix = "res_",
                          split_by = "group", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("ClustreePlot with multiple split_by columns returns a list with one plot per combination", {
    skip_if_not_installed("clustree")
    data2 <- clustree_data
    data2$split2 <- factor(rep(c("s1", "s2"), 50))
    plots <- suppressMessages(ClustreePlot(data2, prefix = "res_",
                                           split_by = c("group", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("ClustreePlot with multiple split_by columns returns combined plot", {
    skip_if_not_installed("clustree")
    data2 <- clustree_data
    data2$split2 <- factor(rep(c("s1", "s2"), 50))
    p <- suppressMessages(ClustreePlot(data2, prefix = "res_",
                                       split_by = c("group", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("ClustreePlot split_by keeps link endpoints aligned with nodes when splits differ", {
    skip_if_not_installed("clustree")
    # ggraph-based plots (same class as Network) carry their layout in
    # the plot data: links in the combined plot must stay anchored to
    # each split's own node positions instead of resolving against the
    # first split's layout.
    p <- ClustreePlot(clustree_data, prefix = "res_",
                      split_by = "group", combine = TRUE)
    # The combined plot exposes the layout of every split, each row
    # tagged with its split level, and a combined graph attribute.
    expect_true("group" %in% colnames(p$data))
    expect_equal(sort(unique(p$data$group)), c("G1", "G2"))
    expect_false(is.null(attr(p$data, "graph")))
    expect_true("group" %in% colnames(attr(p$data, "edges")))
    b <- ggplot2::ggplot_build(p)
    # The edge layer is the one carrying the interpolation index; the
    # node layer is the one with point shape/colour aesthetics.
    edge_layer <- Filter(function(d) "index" %in% colnames(d), b$data)
    node_layer <- Filter(function(d) "shape" %in% colnames(d), b$data)
    expect_length(edge_layer, 1)
    expect_length(node_layer, 1)
    pos <- as.matrix(node_layer[[1]][, c("x", "y")])
    check_endpoint <- function(x, y) {
        any(apply(pos, 1, function(v) sqrt(sum((c(x, y) - v)^2))) < 1e-6)
    }
    edges <- edge_layer[[1]]
    for (g in unique(edges$group)) {
        d <- edges[edges$group == g, ]
        expect_true(check_endpoint(d$x[1], d$y[1]), info = paste("edge", g, "start"))
        expect_true(check_endpoint(d$x[nrow(d)], d$y[nrow(d)]), info = paste("edge", g, "end"))
    }
})

test_that("ClustreePlot with invalid prefix errors informatively", {
    skip_if_not_installed("clustree")
    expect_error(ClustreePlot(clustree_data, prefix = "nonexistent_"), "nonexistent_")
})

#' Atomic area plot (internal)
#'
#' @description
#' Core implementation for drawing a single stacked area plot.  This is the
#' workhorse behind the exported \code{\link{AreaPlot}} function — it takes a
#' **single** data frame (no \code{split_by} support) and returns a
#' \code{ggplot} object.  The plot shows how one or more groups' numeric values
#' (or counts) accumulate across a discrete x-axis, with each group rendered
#' as a filled area stacked from baseline.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{Column resolution} — \code{x}, \code{y}, \code{group_by},
#'         and \code{facet_by} are validated and transformed via
#'         \code{\link{check_columns}}.  Multi-column inputs for \code{x} and
#'         \code{group_by} are concatenated into single columns using their
#'         respective separators (\code{x_sep}, \code{group_by_sep}).
#'   \item \strong{NA / empty-level handling} — \code{\link{process_keep_na_empty}()}
#'         applies \code{keep_na} and \code{keep_empty} policies.  Per-column
#'         \code{keep_empty} settings are extracted for \code{x},
#'         \code{group_by}, and \code{facet_by} independently.  The facet
#'         columns must have identical \code{keep_empty} values.
#'   \item \strong{Count aggregation} — when \code{y = NULL}, the count of
#'         observations in each unique (\code{x}, \code{group_by},
#'         \code{facet_by}) combination is computed as a new \code{.count}
#'         column.  Factor levels are preserved after aggregation.
#'   \item \strong{Proportion scaling} — when \code{scale_y = TRUE}, the
#'         y-values are divided by the sum within each (\code{x},
#'         \code{facet_by}) group, producing a proportion (0–1).  Percent
#'         labels are used automatically on the y-axis.
#'   \item \strong{Empty-fill guard} — when \code{group_by = NULL} (no
#'         grouping), a dummy \code{.fill} factor is created so the single
#'         area still draws with the first palette colour.  The legend is
#'         suppressed (\code{legend.position = "none"}).
#'   \item \strong{Colour mapping} — \code{\link{palette_this}()} assigns
#'         colours to all \code{group_by} levels, including \code{NA}
#'         (defaulting to \code{"grey80"}).
#'   \item \strong{Data completion} — \code{\link[tidyr]{complete}()} pads
#'         all \code{x} × \code{group_by} (× \code{facet_by}) combinations
#'         with \code{y = 0}.  This prevents \code{\link[ggplot2]{geom_area}()}
#'         from interpolating across missing groups, which would otherwise
#'         cause stacked areas to exceed the correct total.
#'   \item \strong{x-axis numeric mapping} — the discrete x variable is
#'         converted to a numeric position column (\code{.x_numeric}) so
#'         \code{geom_area()} can draw continuous area fills between x
#'         positions.  \code{NA} levels are placed at position
#'         \code{n_levels + 1}.
#'   \item \strong{Plot assembly} — the \code{ggplot} object is built with
#'         \code{geom_area(position = position_stack(vjust = 0.5))},
#'         \code{scale_x_discrete()} (breaks from factor levels),
#'         \code{scale_y_continuous()} (percent labels when scaled), and
#'         \code{scale_fill_manual()} with per-group colours.  The fill
#'         scale \code{drop} argument is controlled by
#'         \code{keep_empty_group}.
#'   \item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
#'         computes plot height and width from the x-axis category count,
#'         \code{aspect.ratio}, and legend metrics.  The resulting
#'         \code{height} / \code{width} attributes are stored on the
#'         \code{ggplot} object.
#'   \item \strong{Faceting} — \code{\link{facet_plot}()} wraps the plot
#'         with \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is
#'         provided, respecting the \code{keep_empty} setting for facet
#'         variables.
#' }
#'
#' @inheritParams common_args
#' @param x A character string specifying the column name to plot on the
#'  x-axis.  Must be character or factor.  Multiple columns can be provided;
#'  they are concatenated with \code{x_sep} as the separator.
#' @param x_sep A character string used to join multiple \code{x} columns.
#'  Default \code{"_"}.  Ignored when \code{x} is a single column.
#' @param y A character string specifying the numeric column for the y-axis.
#'  When \code{NULL}, the count of observations in each (\code{x},
#'  \code{group_by}, \code{facet_by}) combination is used.
#' @param scale_y A logical value.  When \code{TRUE}, y-values are scaled to
#'  proportions within each (\code{x}, \code{facet_by}) group so that each
#'  x position stacks to 1.0.  The y-axis labels switch from numeric to
#'  percent format automatically.
#' @param group_by A character vector of column names to fill the areas by.
#'  Each unique combination becomes a separate stacked area.  Multiple
#'  columns are concatenated with \code{group_by_sep}.  When \code{NULL},
#'  a single filled area is drawn (no grouping) and the legend is hidden.
#' @param group_by_sep A character string to separate concatenated
#'  \code{group_by} columns.  Default \code{"_"}.
#' @param group_name A character string used as the fill legend title.
#'  When \code{NULL}, the \code{group_by} column name is used.
#' @keywords internal
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @importFrom rlang syms :=
#' @importFrom dplyr summarise n %>%
#' @importFrom ggplot2 geom_area scale_x_discrete scale_y_continuous scale_fill_manual
#' @importFrom ggplot2 labs theme element_line element_text position_stack waiver
AreaPlotAtomic <- function(
    data,
    x,
    y = NULL,
    x_sep = "_",
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    scale_y = FALSE,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    x_text_angle = 0,
    aspect.ratio = 1,
    legend.position = waiver(),
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_na = FALSE,
    keep_empty = FALSE,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    x <- check_columns(
        data,
        x,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = x_sep
    )
    y <- check_columns(data, y)
    group_by <- check_columns(
        data,
        group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = group_by_sep
    )
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )
    data <- process_keep_na_empty(data, keep_na, keep_empty)
    # TRUE: unused levels will be kept on X axis
    # FALSE/level: unused levels will be dropped
    keep_empty_x <- keep_empty[[x]]
    # TRUE: unused levels will be kept in group_by
    # FALSE: unused levels will be dropped
    # level: unused levels will be dropped, but the group colors will be identified using all levels
    keep_empty_group <- if (!is.null(group_by)) keep_empty[[group_by]] else NULL
    # TRUE: unused levels will be kept in facet_by
    # FALSE/level: unused levels will be dropped
    # 2-column facet_by is not supported yet
    keep_empty_facet <- if (!is.null(facet_by)) {
        keep_empty[[facet_by[1]]]
    } else {
        NULL
    }
    if (length(facet_by) > 1) {
        stopifnot(
            "[AreaPlot] `keep_empty` for `facet_by` variables must be identical." = identical(
                keep_empty_facet,
                keep_empty[[facet_by[2]]]
            )
        )
    }

    orig_data <- data
    if (is.null(y)) {
        y <- ".count"
        data <- data %>%
            dplyr::group_by(!!!syms(unique(c(x, group_by, facet_by)))) %>%
            summarise(.count = n(), .groups = "drop")
        # keep the levels
        for (col in unique(c(x, group_by, facet_by))) {
            data[[col]] <- factor(
                data[[col]],
                levels = levels(orig_data[[col]])
            )
        }
    }

    if (isTRUE(scale_y)) {
        data <- data %>%
            dplyr::group_by(!!!syms(unique(c(x, facet_by)))) %>%
            mutate(!!sym(y) := !!sym(y) / sum(!!sym(y))) %>%
            ungroup()
        # keep the levels
        for (col in unique(c(x, group_by, facet_by))) {
            data[[col]] <- factor(
                data[[col]],
                levels = levels(orig_data[[col]])
            )
        }
    }
    rm(orig_data)

    if (is.null(group_by)) {
        data$.fill <- factor("")
        group_by <- ".fill"
        legend.position <- ifelse(
            inherits(legend.position, "waiver"),
            "none",
            "right"
        )
    } else {
        legend.position <- ifelse(
            inherits(legend.position, "waiver"),
            "right",
            legend.position
        )
    }
    if (!isTRUE(keep_empty_x)) {
        data[[x]] <- droplevels(data[[x]])
    }

    x_vals <- levels(data[[x]])
    if (anyNA(data[[x]])) {
        x_vals <- c(x_vals, NA)
    }

    # group_by_vals <- if (!isTRUE(keep_empty_group) || !anyNA(data[[group_by]])) {
    #     levels(data[[group_by]])
    # } else {
    #     c(levels(data[[group_by]]), NA)
    # }
    group_by_vals <- levels(data[[group_by]])
    if (anyNA(data[[group_by]])) {
        group_by_vals <- c(group_by_vals, NA)
    }
    group_colors <- palette_this(
        group_by_vals,
        palette = palette,
        palcolor = palcolor,
        NA_keep = TRUE,
        reverse = palreverse
    )

    just <- calc_just(x_text_angle)

    # Complete all x * group_by (and facet_by) combinations so that geom_area
    # doesn't interpolate across missing groups, which would cause stacked
    # areas to exceed the correct total.
    complete_vars <- unique(c(x, group_by, facet_by))
    complete_fill <- setNames(list(0), y)
    data_complete <- data %>%
        tidyr::complete(!!!syms(complete_vars), fill = complete_fill)
    # Restore factor levels that complete() may have altered
    for (col in complete_vars) {
        if (is.factor(data[[col]])) {
            data_complete[[col]] <- factor(
                data_complete[[col]],
                levels = levels(data[[col]])
            )
        }
    }

    # Convert x to numeric, handling NA values by assigning them the next position
    data_complete$.x_numeric <- as.numeric(data_complete[[x]])
    if (anyNA(data_complete[[x]])) {
        data_complete$.x_numeric[is.na(data_complete[[
            x
        ]])] <- length(levels(data_complete[[x]])) + 1
    }

    p <- ggplot(
        data_complete,
        aes(x = !!sym(".x_numeric"), y = !!sym(y), fill = !!sym(group_by))
    ) +
        geom_area(
            alpha = alpha,
            color = "grey50",
            position = position_stack(vjust = 0.5),
            show.legend = TRUE
        ) +
        scale_x_discrete(
            expand = c(0, 0),
            breaks = x_vals,
            limits = x_vals,
            drop = isFALSE(keep_empty_x)
        ) +
        scale_y_continuous(
            expand = c(0, 0),
            labels = if (isFALSE(scale_y)) scales::number else scales::percent
        ) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% x,
            y = ylab %||% y
        ) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major = element_line(colour = "grey80", linetype = 2),
            axis.text.x = element_text(
                angle = x_text_angle,
                hjust = just$h,
                vjust = just$v
            )
        )

    if (isTRUE(keep_empty_group)) {
        p <- p +
            scale_fill_manual(
                name = group_name %||% group_by,
                values = group_colors,
                na.value = group_colors["NA"] %||% "grey80",
                breaks = group_by_vals,
                limits = group_by_vals,
                drop = FALSE
            )
    } else {
        p <- p +
            scale_fill_manual(
                name = group_name %||% group_by,
                values = group_colors,
                na.value = group_colors["NA"] %||% "grey80"
            )
    }

    # Calculate plot dimensions with aspect ratio consideration
    dims <- calculate_plot_dimensions(
        base_height = 4.5,
        aspect.ratio = aspect.ratio,
        n_x = length(x_vals),
        x_scale_factor = 0.5,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = length(group_by_vals),
        legend_nchar = max(nchar(as.character(group_by_vals)), na.rm = TRUE),
        flip = FALSE
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction,
        drop = !isTRUE(keep_empty_facet)
    )
}

#' Area plot
#'
#' @description
#' Draws a stacked area plot showing how one or more groups' numeric values
#' (or counts) accumulate across the progression of a discrete x-axis
#' variable.  Each group is rendered as a filled area stacked from baseline,
#' making it easy to compare both individual magnitudes and the total across
#' categories.
#'
#' The function supports \strong{count aggregation} (omit \code{y} to plot
#' observation counts per x-category), \strong{proportion scaling}
#' (\code{scale_y = TRUE} normalises each x position to 100\%), per-group
#' colour control, faceting, and splitting into separate sub-plots via
#' \code{split_by}.
#'
#' @section split_by workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
#'         normalise the \code{keep_na} / \code{keep_empty} arguments for all
#'         columns (\code{x}, \code{split_by}, \code{group_by}, \code{facet_by}).
#'   \item The \code{split_by} column is validated and its NA / empty levels
#'         are processed via \code{\link{process_keep_na_empty}()}.  It is
#'         then removed from the per-column \code{keep_na} / \code{keep_empty}
#'         lists.
#'   \item The data frame is split by \code{split_by} (preserving level
#'         order).  If \code{split_by} is \code{NULL}, the data is wrapped in
#'         a single-element list with name \code{"..."}.
#'   \item Per-split \code{palette}, \code{palcolor},
#'         \code{legend.position}, and \code{legend.direction} are resolved
#'         via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
#'         and \code{\link{check_legend}()}.
#'   \item \code{\link{AreaPlotAtomic}()} is called for each split.  If
#'         \code{title} is a function, it receives the split level name and
#'         can generate dynamic titles.
#'   \item Results are combined via \code{\link{combine_plots}()} (when
#'         \code{combine = TRUE}) or returned as a named list.
#' }
#'
#' @inheritParams common_args
#' @inheritParams AreaPlotAtomic
#' @param split_by The column(s) to split the data by and produce separate
#'  sub-plots.  Multiple columns are concatenated with \code{split_by_sep}.
#' @param split_by_sep A character string to separate concatenated
#'  \code{split_by} columns.  Default \code{"_"}.
#' @param seed A numeric seed for reproducibility.  Passed to
#'  \code{\link{validate_common_args}()}.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'  \code{patchwork} object.  When \code{FALSE}, returns a named list of
#'  individual \code{ggplot} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined layout
#'  (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param byrow Logical; fill the combined layout by row.  Default \code{TRUE}
#'  (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axes A character string specifying how axes should be treated across
#'  the combined layout (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axis_titles A character string specifying how axis titles should be
#'  treated across the combined layout.  Defaults to \code{axes}.
#' @param guides A character string specifying how guides (legends) should be
#'  collected across panels.  Default \code{"collect"} (passed to
#'  \code{\link{combine_plots}()}).
#' @param design A custom layout design for the combined plot (passed to
#'  \code{\link{combine_plots}()}).
#' @return A \code{ggplot} object, a \code{patchwork} object, or a named list
#'  of \code{ggplot} objects (when \code{combine = FALSE}), each with
#'  \code{height} and \code{width} attributes in inches.
#' @importFrom ggplot2 waiver
#' @export
#' @examples
#' \donttest{
#' set.seed(8525)
#' data <- data.frame(
#'     x = rep(c("A", "B", "C", "D"), 2),
#'     y = c(1, 3, 6, 4, 2, 5, 7, 8),
#'     group = rep(c("F1", "F2"), each = 4),
#'     split = rep(c("X", "Y"), 4)
#' )
#' # Basic stacked area
#' AreaPlot(data, x = "x", y = "y", group_by = "group")
#'
#' # Scaled to proportions
#' AreaPlot(data, x = "x", y = "y", group_by = "group",
#'          scale_y = TRUE)
#'
#' # Split into sub-plots (no group_by — single-colour fill)
#' AreaPlot(data, x = "x", y = "y", split_by = "group")
#'
#' # Per-split palettes
#' AreaPlot(data, x = "x", y = "y", split_by = "group",
#'          palette = c(F1 = "Blues", F2 = "Reds"))
#'
#' # Per-split legend positioning
#' AreaPlot(data, x = "x", y = "y", group_by = "group",
#'          split_by = "split",
#'          legend.direction = c(X = "horizontal", Y = "vertical"),
#'          legend.position = c(X = "top", Y = "right"))
#'
#' # How keep_na and keep_empty work
#' data <- data.frame(
#'     x = factor(rep(c("A", NA, "C", "D"), 3),
#'                levels = c("A", "B", "C", "D")),
#'     y = c(1, 3, 6, 4, 2, 5, 7, 8, 4, 2, 3, 5),
#'     group = factor(sample(rep(c("F1", NA, "F3"), each = 4)),
#'                    levels = c("F1", "F2", "F3")),
#'     split = factor(sample(rep(c("X", "Y", NA), 4)),
#'                    levels = c("X", "Y", "Z")),
#'     facet = factor(sample(rep(c("M", "N", NA), 4)),
#'                    levels = c("M", "N", "O"))
#' )
#'
#' # Default: NA and empty levels dropped
#' AreaPlot(data, x = "x", y = "y", group_by = "group")
#'
#' # Keep NA and empty levels
#' AreaPlot(data, x = "x", y = "y", group_by = "group",
#'          keep_na = TRUE, keep_empty = TRUE)
#'
#' # Keep NA, assign empty levels colours but don't show them
#' AreaPlot(data, x = "x", y = "y", group_by = "group",
#'          keep_na = TRUE, keep_empty = "level")
#'
#' # Drop NA, keep empty levels
#' AreaPlot(data, x = "x", y = "y", group_by = "group",
#'          keep_na = FALSE, keep_empty = TRUE)
#'
#' # Per-column keep_na / keep_empty via named lists
#' AreaPlot(data, x = "x", y = "y", group_by = "group",
#'          keep_na = list(x = TRUE, group = FALSE),
#'          keep_empty = list(x = FALSE, group = TRUE))
#' AreaPlot(data, x = "x", y = "y", group_by = "group",
#'          keep_na = list(x = FALSE, group = TRUE),
#'          keep_empty = list(x = TRUE, group = FALSE))
#' }
AreaPlot <- function(
    data,
    x,
    y = NULL,
    x_sep = "_",
    split_by = NULL,
    split_by_sep = "_",
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    scale_y = FALSE,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    x_text_angle = 0,
    aspect.ratio = 1,
    legend.position = waiver(),
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_na = FALSE,
    keep_empty = FALSE,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    keep_na <- check_keep_na(keep_na, c(x, split_by, group_by, facet_by))
    keep_empty <- check_keep_empty(
        keep_empty,
        c(x, split_by, group_by, facet_by)
    )
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            AreaPlotAtomic(
                datas[[nm]],
                x = x,
                y = y,
                x_sep = x_sep,
                group_by = group_by,
                group_by_sep = group_by_sep,
                group_name = group_name,
                scale_y = scale_y,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                alpha = alpha,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                x_text_angle = x_text_angle,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                keep_na = keep_na,
                keep_empty = keep_empty,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
# AreaPlot data: x = time points, group_by = group, y = value (if provided)
data <- data.frame(
    time = factor(rep(paste0("T", 1:6), each = 20), levels = paste0("T", 1:6)),
    group = factor(rep(c("A", "B", "C", "D"), 30)),
    value = runif(120, 0, 100),
    facet = factor(rep(c("f1", "f2"), 60))
)

test_that("AreaPlot returns a ggplot for basic usage", {
    p <- AreaPlot(data, x = "time", group_by = "group")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("AreaPlot works with y column", {
    p <- AreaPlot(data, x = "time", y = "value", group_by = "group")
    expect_s3_class(p, "ggplot")
})

test_that("AreaPlot works without group_by", {
    p <- AreaPlot(data, x = "time", y = "value")
    expect_s3_class(p, "ggplot")
})

test_that("AreaPlot sets title and labels", {
    p <- AreaPlot(data, x = "time", group_by = "group",
                  title = "Area Title", xlab = "Time", ylab = "Count")
    expect_equal(p$labels$title, "Area Title")
})

test_that("AreaPlot with scale_y = TRUE works", {
    p <- AreaPlot(data, x = "time", group_by = "group", scale_y = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("AreaPlot with split_by returns combined plot", {
    p <- AreaPlot(data, x = "time", group_by = "group",
                  split_by = "facet", combine = TRUE)
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("AreaPlot with split_by and combine = FALSE returns list", {
    plots <- AreaPlot(data, x = "time", group_by = "group",
                      split_by = "facet", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
})

test_that("AreaPlot with multiple split_by columns returns a list with one plot per combination", {
    data2 <- data
    data2$split2 <- factor(rep(c("s1", "s2"), each = 60))
    plots <- suppressMessages(AreaPlot(data2, x = "time", group_by = "group",
                                       split_by = c("facet", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("AreaPlot with multiple split_by columns returns combined plot", {
    data2 <- data
    data2$split2 <- factor(rep(c("s1", "s2"), each = 60))
    p <- suppressMessages(AreaPlot(data2, x = "time", group_by = "group",
                                   split_by = c("facet", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("AreaPlot with facet_by works", {
    p <- AreaPlot(data, x = "time", group_by = "group", facet_by = "facet")
    expect_s3_class(p, "ggplot")
})

test_that("AreaPlot width/height attributes are numeric", {
    p <- AreaPlot(data, x = "time", group_by = "group")
    expect_true(is.numeric(attr(p, "width")))
    expect_true(is.numeric(attr(p, "height")))
})

#' Atomic heatmap (internal)
#'
#' @description
#' Core implementation for drawing a single heatmap using
#' \code{ComplexHeatmap::Heatmap}.  This function takes pre-processed data
#' (already in long format with resolved row/column identifiers) and handles
#' colour mapping, cell rendering, annotation construction, and the final
#' \code{draw()} call.  It is the workhorse behind the exported
#' \code{\link{Heatmap}} function and is also reused by
#' \code{\link{LinkedHeatmapAtomic}} (via \code{return_ht = TRUE}).
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{Annotation resolution} — row and column annotation
#'         parameters (side, type, palette, params, aggregation) are
#'         resolved with alias support (\code{.row}, \code{.col},
#'         \code{.rows.split}, etc.) so that built-in name/split annotations
#'         and user-defined annotations can be configured uniformly.
#'   \item \strong{Keep-NA / keep-empty handling} — \code{keep_na} and
#'         \code{keep_empty} are processed per column to control whether
#'         \code{NA} values and empty factor levels are preserved in the
#'         data before constructing the heatmap matrix.
#'   \item \strong{Flip} — when \code{flip = TRUE}, row and column
#'         parameters are swapped transparently so the caller does not need
#'         to manually swap every argument.
#'   \item \strong{Cell dimension calculation} — cell width and height are
#'         pre-computed from \code{cell_type}, \code{aspect.ratio},
#'         \code{base_size}, and the unique row/column counts (accounting
#'         for split groups).  These are passed as explicit
#'         \code{"inches"} units to \code{ComplexHeatmap::Heatmap()} so
#'         cells have guaranteed physical sizes.
#'   \item \strong{Colour mapping} — the main colour scale is built from
#'         \code{palette} / \code{palcolor} with optional quantile-based
#'         or explicit cutoff clamping.
#'   \item \strong{Cell function dispatch} — depending on
#'         \code{cell_type}, the appropriate \code{cell_fun} or
#'         \code{layer_fun} is installed (tile, bars, label, mark,
#'         label+mark, dot, violin, boxplot, pie).
#'   \item \strong{Annotation construction} — row and column annotations
#'         are built via \code{ComplexHeatmap::HeatmapAnnotation()} using
#'         the resolved type and parameters.
#'   \item \strong{Drawing} — the heatmap is drawn via
#'         \code{ComplexHeatmap::draw()} with controlled padding and legend
#'         collection.  When \code{return_ht = TRUE}, the prepared
#'         \code{Heatmap} object is returned without drawing.
#'   \item \strong{Dimension attributes} — the returned object carries
#'         \code{height} / \code{width} attributes (in inches) and
#'         \code{cell_w} / \code{cell_h} attributes for downstream layout
#'         calculations.
#' }
#'
#' @inheritParams common_args
#' @inheritParams process_heatmap_data
#' @param data A data frame in long format. Each row represents one
#'  observation; columns specify row/column membership and the value to
#'  encode as colour.
#' @param values_fill A value used to fill missing cells in the matrix.
#'  Default \code{NA}.  Missing values prevent clustering when not filled.
#' @param border A logical value indicating whether to draw borders around
#'  the heatmap.  If \code{TRUE}, slice borders are also drawn.  Default
#'  \code{TRUE}.
#' @param title The global (column) title of the heatmap.
#' @param lower_quantile,upper_quantile,lower_cutoff,upper_cutoff
#'  Quantile or explicit cutoffs for clipping the colour scale.  Applied
#'  to aggregated values for \code{tile} / \code{label} cell types; applied
#'  to raw values for \code{bars} / \code{violin} / \code{boxplot} types.
#' @param cluster_columns Logical; cluster the columns.  If \code{TRUE} and
#'  \code{columns_split_by} is provided, clustering is applied within each
#'  split group.
#' @param cluster_rows Logical; cluster the rows.  If \code{TRUE} and
#'  \code{rows_split_by} is provided, clustering is applied within each
#'  split group.
#' @param legend_items A named numeric vector specifying custom legend
#'  entries for the main colour scale.  Names become the displayed labels.
#' @param legend_discrete Logical; if \code{TRUE}, treat the main colour
#'  scale as discrete.
#' @param show_row_names Logical or character vector; show row names.  If
#'  \code{TRUE}, the legend of the row group annotation is hidden.
#'  \code{FALSE} only hides the in-place names.  Alternatively, a character
#'  vector combining the display modes \code{"inplace"} (in-place names,
#'  same as \code{TRUE}), \code{"legend"} (row group annotation with
#'  legend), \code{"simple"} (row group annotation without legend),
#'  \code{"anno"} (alias \code{"annotation"}; label annotation showing the
#'  row names as text) and \code{"none"} (no names, annotation, or legend).
#'  Modes are lower-priority defaults: explicit \code{row_annotation}
#'  entries (e.g. \code{.row}) still win.
#' @param show_column_names Logical or character vector; show column names.
#'  See \code{show_row_names} for the display modes.
#' @param column_title Character string/vector used as the column group
#'  annotation title.  A character vector consisting entirely of the display
#'  modes \code{"inplace"} (per-slice split titles), \code{"legend"} (split
#'  annotation with legend, no title), \code{"simple"} (split annotation,
#'  no legend/title), \code{"anno"} (alias \code{"annotation"}; label block
#'  annotation) and \code{"none"} (no title, annotation, or legend)
#'  controls how the split titles are displayed.  A literal title identical
#'  to a mode keyword is interpreted as a mode.
#' @param row_title Character string/vector used as the row group
#'  annotation title.  See \code{column_title} for the display modes.
#' @param na_col Colour for \code{NA} cells.  Default \code{"grey85"}.
#' @param row_names_side Side for row names.  Default \code{"right"}.
#' @param column_names_side Side for column names.  Default \code{"bottom"}.
#' @param bars_sample Fraction of each cell's observations (0 < x <= 1;
#'  \code{1} uses all data) or a whole count > 1 sampled per cell when
#'  \code{cell_type = "bars"}.  Column widths are proportional to the
#'  number of bars drawn per column, so a numeric count gives equal
#'  column widths and a fraction gives widths proportional to the
#'  column data sizes.  Proportional widths are skipped when
#'  \code{columns_split_by} is given, and column clustering is
#'  disabled in proportional mode.  Default 1 (all data).
#' @param flip Logical; if \code{TRUE}, swap rows and columns
#'  transparently.  The caller does not need to swap row- and
#'  column-related arguments manually.
#' @param pie_palette,pie_palcolor Palette and custom colours for pie slice
#'  fill colours.
#' @param pie_values A function or string (convertible via
#'  \code{\link[base]{match.arg}}) to compute the value represented by
#'  each pie slice.  Default \code{"length"} counts observations per
#'  group.
#' @param pie_size A numeric value or function returning the pie radius.
#'  When a function, it receives the count of groups in the pie.
#' @param pie_size_name Legend title for the pie size.
#' @param label A function to compute text labels when
#'  \code{cell_type = "label"} (or \code{"label+mark"}).  Receives the
#'  aggregated value for a cell and optionally row/column indices and
#'  names.  See below for the full dispatch contract.
#' @param label_size Default point size for label text (used as fallback
#'  when the \code{label} function does not return a \code{size} field).
#' @param label_color Default colour for label text (fallback).
#' @param label_name Legend title for the label colour scale.  The legend
#'  is shown automatically when the \code{label} function returns a
#'  \code{legend} field for at least one cell.
#' @param mark A function to compute mark symbols when
#'  \code{cell_type = "mark"} (or \code{"label+mark"}).  Same dispatch
#'  contract as \code{label}.
#' @param mark_size Default mark stroke width (\code{lwd}) in pt (fallback).
#' @param mark_color Default mark colour (fallback).
#' @param mark_name Legend title for the mark colour scale.
#' @param layer_fun_callback A function to add custom graphical layers on
#'  top of each heatmap cell.  Receives \code{j}, \code{i}, \code{x},
#'  \code{y}, \code{w}, \code{h}, \code{fill}, \code{sr}, \code{sc}.
#'  See \code{\link[ComplexHeatmap]{Heatmap}} for details.
#' @param cell_type The type of cell to render.  One of \code{"tile"}
#'  (default), \code{"bars"}, \code{"label"}, \code{"mark"},
#'  \code{"label+mark"} (or \code{"mark+label"}), \code{"dot"},
#'  \code{"violin"}, \code{"boxplot"}, \code{"pie"}.  See the
#'  \strong{Cell types} section for details.
#' @param cell_agg A function to aggregate values within each cell when
#'  \code{cell_type = "tile"} or \code{"label"}.  Default is
#'  \code{\link[base]{mean}}.
#' @param add_bg Logical; if \code{TRUE}, add a background fill behind
#'  non-tile cell types.  Not used for \code{cell_type = "tile"} or
#'  \code{"bars"}.
#' @param bg_alpha Numeric in \eqn{[0, 1]} for background transparency.
#' @param violin_fill A character vector of colours to use as fill for
#'  violin plots when \code{cell_type = "violin"}.  If \code{NULL}, the
#'  annotation colour is used.
#' @param boxplot_fill A character vector of colours to use as fill for
#'  boxplots when \code{cell_type = "boxplot"}.  If \code{NULL}, the
#'  annotation colour is used.
#' @param dot_size Dot size when \code{cell_type = "dot"}.  Can be a
#'  numeric value or a function.
#' @param dot_size_name Legend title for the dot size.
#' @param column_annotation A structured list specifying column annotations.
#'  Each entry is a named list with sub-keys:
#'  \describe{
#'    \item{\code{col}}{Column name in \code{data} supplying the annotation
#'      values.  If omitted, the entry name is used as the column name.}
#'    \item{\code{side}}{\code{"top"} or \code{"bottom"}.}
#'    \item{\code{palette}}{Palette name (see \code{\link{show_palettes}}).}
#'    \item{\code{palcolor}}{Custom colour vector overriding \code{palette}.}
#'    \item{\code{type}}{Annotation type: \code{"auto"}, \code{"simple"},
#'      \code{"pie"}, \code{"ring"}, \code{"bar"}, \code{"violin"},
#'      \code{"boxplot"}, \code{"density"}, \code{"label"},
#'      \code{"rownames"}, \code{"points"}, \code{"lines"}.
#'      For split annotations, \code{"rownames"} (row splits) /
#'      \code{"colnames"} (column splits) is like \code{"label"} but labels
#'      each split block with the concatenated row/column names;
#'      \code{"names"} and \code{"dimnames"} are aliases.
#'      \code{params$sep} (default \code{" "}) controls the separator and
#'      \code{params$wrap_by} the number of names per line.
#'      For name annotations (the built-in \code{rows_by}/\code{columns_by}
#'      annotation), \code{"rownames"}/\code{"colnames"} renders the
#'      row/column names as text labels (this is what
#'      \code{show_row_names = "anno"} uses).}
#'    \item{\code{params}}{A list of additional parameters passed to the
#'      annotation constructor.  \code{FALSE} disables the annotation.
#'      \code{$show_legend} controls legend visibility.  See
#'      \code{\link[ComplexHeatmap]{HeatmapAnnotation}}.}
#'    \item{\code{agg}}{A function to aggregate values for the annotation.}
#'    \item{\code{name}}{Display name for the annotation, shown next to the
#'      annotation bar and used as the legend title.  \code{FALSE} hides the
#'      displayed name.  Defaults to the annotation key (column name).}
#'  }
#'  \strong{Shortcuts:}
#'  \itemize{
#'    \item \code{column_annotation = list(Score = "score")} is short for
#'      \code{list(Score = list(col = "score"))}.
#'    \item \code{column_annotation = TRUE} enables annotations with
#'      defaults.  \code{FALSE} disables all column annotations.
#'  }
#'  \strong{Special keys:}
#'  \itemize{
#'    \item \code{.default} — default values inherited by all entries.
#'      \code{params} is merged recursively; other keys are inherited
#'      only when the entry does not already specify them.
#'    \item \code{.col} / \code{.cols} / \code{.column} / \code{.columns} —
#'      alias for \code{columns_by} (the built-in name annotation).
#'    \item \code{.col.split} / \code{.cols.split} / \code{.column.split} /
#'      \code{.columns.split} — alias for \code{columns_split_by}
#'      (the built-in split annotation).
#'    \item \code{.row} / \code{.rows} — alias for \code{rows_by}.
#'    \item \code{.row.split} / \code{.rows.split} — alias for
#'      \code{rows_split_by}.
#'  }
#' @param column_annotation_side \strong{Deprecated}: use
#'  \code{column_annotation} with the \code{side} sub-key instead.
#' @param column_annotation_palette \strong{Deprecated}: use
#'  \code{column_annotation} with the \code{palette} sub-key instead.
#' @param column_annotation_palcolor \strong{Deprecated}: use
#'  \code{column_annotation} with the \code{palcolor} sub-key instead.
#' @param column_annotation_type \strong{Deprecated}: use
#'  \code{column_annotation} with the \code{type} sub-key instead.
#' @param column_annotation_params \strong{Deprecated}: use
#'  \code{column_annotation} with the \code{params} sub-key instead.
#' @param column_annotation_agg \strong{Deprecated}: use
#'  \code{column_annotation} with the \code{agg} sub-key instead.
#' @param row_annotation A structured list specifying row annotations.
#'  Same format as \code{column_annotation}.  Sides default to
#'  \code{"left"}.  Aliases: \code{.row}/\code{.rows} for
#'  \code{rows_by}, \code{.row.split}/\code{.rows.split} for
#'  \code{rows_split_by}.
#' @param row_annotation_side \strong{Deprecated}: use
#'  \code{row_annotation} with the \code{side} sub-key instead.
#' @param row_annotation_palette \strong{Deprecated}: use
#'  \code{row_annotation} with the \code{palette} sub-key instead.
#' @param row_annotation_palcolor \strong{Deprecated}: use
#'  \code{row_annotation} with the \code{palcolor} sub-key instead.
#' @param row_annotation_type \strong{Deprecated}: use
#'  \code{row_annotation} with the \code{type} sub-key instead.
#' @param row_annotation_params \strong{Deprecated}: use
#'  \code{row_annotation} with the \code{params} sub-key instead.
#' @param row_annotation_agg \strong{Deprecated}: use
#'  \code{row_annotation} with the \code{agg} sub-key instead.
#' @param add_reticle Logical; if \code{TRUE}, draw a reticle (crosshair
#'  pattern) over the heatmap.
#' @param reticle_color Colour for the reticle lines.
#' @param palette A character string naming a palette (see
#'  \code{\link{show_palettes}}) or a character vector of colours for the
#'  main heatmap colour scale.  Default \code{"RdBu"}.
#' @param palcolor A custom colour vector overriding \code{palette}.
#' @param alpha Alpha transparency for heatmap cells in \eqn{[0, 1]}.
#' @param padding Padding around the heatmap in CSS order (top, right,
#'  bottom, left).  Supports 1–4 values.  Default 15 (mm).  Note that
#'  this is different from \code{ComplexHeatmap::draw()}'s \code{padding}
#'  argument which uses bottom-left-top-right order.
#' @param base_size A positive numeric scalar used as a scaling factor for
#'  the overall heatmap size.  Default 1 (no scaling).  Values > 1 enlarge
#'  all cell dimensions proportionally.
#' @param aspect.ratio Height-to-width ratio of a single heatmap cell.
#'  When \code{NULL} (default), sensible per-\code{cell_type} defaults are
#'  used: 1 for \code{tile}/\code{label}/\code{dot}, 0.5 for \code{bars},
#'  and 2 for \code{violin}/\code{boxplot}/\code{pie}.  The ratio is
#'  constrained by the overall plot dimensions.
#' @param return_ht Logical; if \code{TRUE}, return the prepared
#'  \code{ComplexHeatmap::Heatmap} object without drawing.  Used
#'  internally by \code{\link{LinkedHeatmapAtomic}}.
#' @param draw_opts A named list of additional arguments passed to
#'  \code{\link[ComplexHeatmap]{draw,HeatmapList-method}}.  Internally
#'  managed arguments take precedence.
#' @param ... Additional arguments passed to
#'  \code{\link[ComplexHeatmap]{Heatmap}}.  When
#'  \code{row_names_max_width} is passed, a \code{unit} is expected but
#'  numeric values (default unit \code{"inches"}) or strings like
#'  \code{"5inches"} are also accepted.  Unmatched arguments produce a
#'  warning.
#'
#' @section Label / mark dispatch contract:
#' The \code{label} and \code{mark} functions can take 1, 3, or 5
#' arguments.  The first argument is always the aggregated value for a
#' single cell.  With 3 arguments, the second and third are the row and
#' column indices.  With 5 arguments, the fourth and fifth are the row
#' and column names.  The function should return one of:
#' \itemize{
#'   \item \code{NA} — nothing is drawn for this cell.
#'   \item A character scalar — used as the label text / mark type;
#'         default size and colour are used.
#'   \item A named list with fields \code{label}/\code{mark},
#'         \code{size}, \code{color}, \code{legend}, and \code{order}
#'         (all optional; smaller \code{order} appears first in the
#'         legend).
#' }
#' For label cells with custom indices, use
#' \code{ComplexHeatmap::pindex()} to translate between data and heatmap
#' coordinates.
#'
#' @section Supported mark types:
#' \itemize{
#'   \item \strong{Primitives:} \code{-} (h-line), \code{|} (v-line),
#'         \code{+} (cross), \code{/} (l-diag), \code{\\} (r-diag),
#'         \code{x} (both diags), \code{o} (circle), \code{()}
#'         (edge-touching circle), \code{<>} (diamond).
#'   \item \strong{With rectangular border:} \code{[]}, \code{[-]},
#'         \code{[|]}, \code{[+]}, \code{[/]}, \code{[\\]}, \code{[x]},
#'         \code{[o]}, \code{[()]}, \code{[<>]}.
#'   \item \strong{With full circle:} \code{(-)}, \code{(|)},
#'         \code{(+)}, \code{(/)}, \code{(\\)}, \code{(x)}, \code{(o)},
#'         \code{(<>)}.
#'   \item \strong{With diamond:} \code{<->}, \code{<|>}, \code{<+>},
#'         \code{</>}, \code{<\\>}, \code{<x>}, \code{<o>}.
#'   \item \strong{Octagon:} \code{\{\}}, \code{\{-\}}, \code{\{|\}},
#'         \code{\{+\}}, \code{\{/\}}, \code{\{\\\}}, \code{\{x\}},
#'         \code{\{o\}}, \code{\{()\}}, \code{\{<>\}}.
#'   \item \strong{Combinations:} e.g. \code{[(|)]}, \code{[(-)]},
#'         \code{[(+)]}, \code{[(/)]}, \code{[(\\]}, \code{[(x)]},
#'         \code{[(o)]}, \code{[(<>)]}.
#' }
#'
#' @section Annotations:
#' Row and column annotations are configured via the
#' \code{row_annotation} / \code{column_annotation} structured list
#' arguments.  Built-in name annotations (for \code{rows_by} /
#' \code{columns_by}) and split annotations (for \code{rows_split_by} /
#' \code{columns_split_by}) are added automatically and can be configured
#' using the aliases \code{.row} / \code{.col} and \code{.row.split} /
#' \code{.col.split} as keys in the annotation list.  Setting an alias
#' entry's \code{params} to \code{FALSE} disables that annotation.
#' \code{.default} provides default values inherited by all entries
#' (with recursive merge for \code{params}).  Ordering within each side:
#' name annotations are closest to the body, split annotations farthest
#' away, user-defined annotations in between.
#'
#' @note Removed parameters: \code{rows_palette}, \code{rows_palcolor},
#'  \code{columns_palette}, \code{columns_palcolor},
#'  \code{columns_split_palette}, \code{columns_split_palcolor},
#'  \code{rows_split_palette}, \code{rows_split_palcolor} — use
#'  \code{row_annotation}/\code{column_annotation} with
#'  sub-key \code{palette}/\code{palcolor} and alias
#'  \code{.row} / \code{.row.split} / \code{.col} / \code{.col.split}.
#'  Also removed:
#'  \code{row_name_annotation}, \code{row_name_legend},
#'  \code{column_name_annotation}, \code{column_name_legend} — set
#'  \code{row_annotation = list(.row = list(params = FALSE))} /
#'  \code{column_annotation = list(.col = list(params = FALSE))}.
#' @importFrom circlize colorRamp2
#' @importFrom dplyr group_by across ungroup %>% all_of summarise first everything group_map
#' @importFrom tidyr pivot_longer pivot_wider unite expand_grid
#' @importFrom ggplot2 ggplotGrob theme_void
#' @importFrom grid grid.rect grid.text grid.lines grid.points viewport gpar unit grid.draw is.unit
#' @importFrom grid convertUnit grid.grabExpr grid.segments grid.circle grid.polygon unit.c
#' @return An object of heatmap that is wrapped by `patchwork::wrap_plots()`
#' @keywords internal
HeatmapAtomic <- function(
    data,
    values_by,
    values_fill = NA,
    # data definition
    rows_by = NULL,
    rows_split_by = NULL,
    columns_by = NULL,
    columns_split_by = NULL,
    # palettes
    palette = "RdBu",
    palcolor = NULL,
    palreverse = FALSE,
    # cell_type: pies
    pie_size_name = "size",
    pie_size = NULL,
    pie_values = "length",
    pie_group_by = NULL,
    pie_palette = "Spectral",
    pie_palcolor = NULL,
    # cell_type: bars
    bars_sample = 1,
    # cell_type: label
    label = scales::label_number_auto(),
    label_size = 10,
    label_color = "black",
    label_name = "label",
    # cell_type: mark
    mark = identity,
    mark_color = "black",
    mark_size = 1,
    mark_name = "mark",
    # cell_type: violin
    violin_fill = NULL,
    # cell_type: boxplot
    boxplot_fill = NULL,
    # cell_type: dot
    dot_size = 8,
    dot_size_name = "size",
    # legend
    legend_items = NULL,
    legend_discrete = FALSE,
    legend.position = "right",
    legend.direction = "vertical",
    # values
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    # bg
    add_bg = FALSE,
    bg_alpha = 0.5,
    keep_na = FALSE,
    keep_empty = FALSE,
    # reticle
    add_reticle = FALSE,
    reticle_color = "grey",
    # passed to ComplexHeatmap::Heatmap
    cluster_columns = TRUE,
    cluster_rows = TRUE,
    show_row_names = NULL,
    show_column_names = NULL,
    border = TRUE,
    title = NULL,
    column_title = NULL,
    row_title = NULL,
    na_col = "grey85",
    row_names_side = "right",
    column_names_side = "bottom",
    row_annotation = NULL,
    row_annotation_side = NULL,
    row_annotation_palette = NULL,
    row_annotation_palcolor = NULL,
    row_annotation_type = NULL,
    row_annotation_params = NULL,
    row_annotation_agg = NULL,
    column_annotation = NULL,
    column_annotation_side = NULL,
    column_annotation_palette = NULL,
    column_annotation_palcolor = NULL,
    column_annotation_type = NULL,
    column_annotation_params = NULL,
    column_annotation_agg = NULL,
    # misc
    flip = FALSE,
    alpha = 1,
    seed = 8525,
    padding = 15,
    base_size = 1,
    aspect.ratio = NULL,
    draw_opts = list(),
    return_ht = FALSE,
    # cell customization
    layer_fun_callback = NULL,
    cell_type = "tile",
    cell_agg = NULL,
    ...
) {
    # Data was validated in `plotthis::Heatmap()`

    # Resolve annotation parameters via the unified .prep_annotations() helper.
    # This handles TRUE/FALSE shortcuts, deprecated arg merging with warnings,
    # .default inheritance (recursive for params), alias resolution, and validation.
    row_anno <- .prep_annotations(
        which = "row",
        annotation = row_annotation,
        annotation_side = row_annotation_side,
        annotation_palette = row_annotation_palette,
        annotation_palcolor = row_annotation_palcolor,
        annotation_type = row_annotation_type,
        annotation_params = row_annotation_params,
        annotation_agg = row_annotation_agg,
        row_key = rows_by,
        rsplit_key = rows_split_by,
        col_key = columns_by,
        csplit_key = columns_split_by,
        data = data
    )
    col_anno <- .prep_annotations(
        which = "column",
        annotation = column_annotation,
        annotation_side = column_annotation_side,
        annotation_palette = column_annotation_palette,
        annotation_palcolor = column_annotation_palcolor,
        annotation_type = column_annotation_type,
        annotation_params = column_annotation_params,
        annotation_agg = column_annotation_agg,
        row_key = rows_by,
        rsplit_key = rows_split_by,
        col_key = columns_by,
        csplit_key = columns_split_by,
        data = data
    )

    row_annotation <- row_anno$annotation
    row_annotation_type <- row_anno$annotation_type
    row_annotation_side <- row_anno$annotation_side
    row_annotation_palette <- row_anno$annotation_palette
    row_annotation_palcolor <- row_anno$annotation_palcolor
    row_annotation_agg <- row_anno$annotation_agg
    row_annotation_params <- row_anno$annotation_params
    row_annotation_name <- row_anno$annotation_name

    column_annotation <- col_anno$annotation
    column_annotation_type <- col_anno$annotation_type
    column_annotation_side <- col_anno$annotation_side
    column_annotation_palette <- col_anno$annotation_palette
    column_annotation_palcolor <- col_anno$annotation_palcolor
    column_annotation_agg <- col_anno$annotation_agg
    column_annotation_params <- col_anno$annotation_params
    column_annotation_name <- col_anno$annotation_name

    # Resolve display modes for row/column names ("inplace", "legend",
    # "simple", "anno", "none").  Modes are lower-priority defaults: they
    # only fill annotation config the user has not set (incl. .default).
    rn <- .resolve_show_modes(
        show_row_names,
        kind = "names",
        which = "row",
        by = rows_by,
        split_by = rows_split_by,
        annotation = row_annotation,
        annotation_type = row_annotation_type,
        annotation_params = row_annotation_params,
        annotation_name = row_annotation_name,
        legend.position = legend.position
    )
    row_annotation <- rn$annotation
    row_annotation_type <- rn$annotation_type
    row_annotation_params <- rn$annotation_params
    row_annotation_name <- rn$annotation_name
    row_by_eff <- rn$by_eff

    cn <- .resolve_show_modes(
        show_column_names,
        kind = "names",
        which = "column",
        by = columns_by,
        split_by = columns_split_by,
        annotation = column_annotation,
        annotation_type = column_annotation_type,
        annotation_params = column_annotation_params,
        annotation_name = column_annotation_name,
        legend.position = legend.position
    )
    column_annotation <- cn$annotation
    column_annotation_type <- cn$annotation_type
    column_annotation_params <- cn$annotation_params
    column_annotation_name <- cn$annotation_name
    col_by_eff <- cn$by_eff

    # Determine whether the name annotations are enabled.
    # A params entry of FALSE disables, but type "label" takes precedence
    # (setting the type implies the user wants the annotation visible).
    row_name_anno_enabled <- rn$enabled %||%
        (!is.null(rows_by) &&
            (!isFALSE(row_annotation[[rows_by]] %||% TRUE) ||
                row_annotation_type[[rows_by]] %||% "simple" == "label"))
    col_name_anno_enabled <- cn$enabled %||%
        (!is.null(columns_by) &&
            (!isFALSE(column_annotation[[columns_by]] %||% TRUE) ||
                column_annotation_type[[columns_by]] %||% "simple" == "label"))
    # "rownames"/"colnames" split annotations already show the concatenated
    # row/column names, so the heatmap row/column names are hidden by default
    split_names_types <- c("rownames", "colnames", "names", "dimnames")
    row_rownames_split <- !is.null(rows_split_by) &&
        row_annotation_type[[rows_split_by]] %in% split_names_types &&
        !isFALSE(row_annotation_params[[rows_split_by]])
    col_rownames_split <- !is.null(columns_split_by) &&
        column_annotation_type[[columns_split_by]] %in% split_names_types &&
        !isFALSE(column_annotation_params[[columns_split_by]])
    show_row_names <- rn$show %||%
        (!row_name_anno_enabled && !row_rownames_split)
    show_column_names <- cn$show %||%
        (!col_name_anno_enabled && !col_rownames_split)

    # Resolve split-title display modes (same vocabulary as names)
    rt <- .resolve_show_modes(
        row_title,
        kind = "titles",
        which = "row",
        by = rows_by,
        split_by = rows_split_by,
        annotation = row_annotation,
        annotation_type = row_annotation_type,
        annotation_params = row_annotation_params,
        annotation_name = row_annotation_name,
        legend.position = legend.position
    )
    row_annotation <- rt$annotation
    row_annotation_type <- rt$annotation_type
    row_annotation_params <- rt$annotation_params
    if (rt$explicit) {
        row_title <- rt$title
    }

    ct <- .resolve_show_modes(
        column_title,
        kind = "titles",
        which = "column",
        by = columns_by,
        split_by = columns_split_by,
        annotation = column_annotation,
        annotation_type = column_annotation_type,
        annotation_params = column_annotation_params,
        annotation_name = column_annotation_name,
        legend.position = legend.position
    )
    column_annotation <- ct$annotation
    column_annotation_type <- ct$annotation_type
    column_annotation_params <- ct$annotation_params
    if (ct$explicit) {
        column_title <- ct$title
    }

    # Convert to the format that ComplexHeatmap::Heatmap can understand
    if (
        !rt$explicit &&
            is.null(row_title) &&
            !is.null(rows_split_by) &&
            !isFALSE(row_annotation_params[[rows_split_by]])
    ) {
        # Let's check if we need row_title
        if (identical(row_annotation_type[[rows_split_by]], "auto")) {
            row_title <- TRUE
        } else {
            row_title <- (row_annotation_type[[rows_split_by]] %||% "simple") %in%
                c("simple", split_names_types)
        }
    }

    if (
        !ct$explicit &&
            is.null(column_title) &&
            !is.null(columns_split_by) &&
            !isFALSE(column_annotation_params[[columns_split_by]])
    ) {
        if (
            identical(
                column_annotation_type[[columns_split_by]] %||% "auto",
                "auto"
            )
        ) {
            column_title <- TRUE
        } else {
            column_title <- (column_annotation_type[[columns_split_by]] %||% "simple") %in%
                c("simple", split_names_types)
        }
    }

    if (isFALSE(column_title)) {
        column_title <- NULL
    }
    if (isFALSE(row_title)) {
        row_title <- NULL
    }
    if (isTRUE(column_title)) {
        column_title <- character(0)
    }
    if (isTRUE(row_title)) {
        row_title <- character(0)
    }
    stopifnot(
        "[Heatmap] 'padding' should be a numeric vector of length 1, 2, 3, or 4." = length(
            padding
        ) %in%
            1:4
    )

    get_col_fun <- function(lower, upper, a = alpha) {
        # If the lower and upper cutoff are the same, we need to adjust the upper cutoff
        if (upper == lower) {
            if (upper == 0) {
                upper <- 1e-3
            } else {
                upper <- upper + upper * 1e-3
            }
        }
        colorRamp2(
            seq(lower, upper, length = 100),
            palette_this(
                palette = palette,
                palcolor = palcolor,
                alpha = a,
                reverse = palreverse,
                transparent = FALSE
            )
        )
    }

    flip_side <- function(side) {
        match.arg(side, c("left", "right", "top", "bottom"))
        if (side == "left") {
            return("top")
        }
        if (side == "right") {
            return("bottom")
        }
        if (side == "top") {
            return("left")
        }
        if (side == "bottom") return("right")
    }

    # Initialize the heatmap arguments
    hmargs <- list(
        # name = name,   # error when name has irregular characters (e.g. "-")
        heatmap_legend_param = list(title = values_by),
        border = border,
        na_col = na_col,
        cluster_columns = if (flip) cluster_rows else cluster_columns,
        cluster_rows = if (flip) cluster_columns else cluster_rows,
        cluster_column_slices = FALSE,
        cluster_row_slices = FALSE,
        show_heatmap_legend = FALSE,
        show_row_names = if (flip) show_column_names else show_row_names,
        show_column_names = if (flip) show_row_names else show_column_names,
        row_names_side = if (flip) {
            flip_side(column_names_side)
        } else {
            row_names_side
        },
        column_names_side = if (flip) {
            flip_side(row_names_side)
        } else {
            column_names_side
        },
        column_title = column_title,
        row_title = row_title,
        ...
    )

    # Set the row_names_max_width based on the length of the row names
    if (!is.null(hmargs$row_names_max_width)) {
        if (is.character(hmargs$row_names_max_width)) {
            if (grepl("^[0-9]+(\\.[0-9]+)?$", hmargs$row_names_max_width)) {
                hmargs$row_names_max_width <- unit(
                    as.numeric(hmargs$row_names_max_width),
                    "inches"
                )
            }
            stopifnot(
                "[Heatmap] 'row_names_max_width' should be in a format of '2inches' or '2cm' if given as a string." = grepl(
                    "^[0-9]+(\\.[0-9]+\\s*)?(npc|cm|centimetre|centimeter|in|inch|inches|mm|points|picas|bigpts|cicero|scalepts|lines|char|native|snpc|strwidth|strheight|grobwidth|grobheight|null)$",
                    hmargs$row_names_max_width
                )
            )

            # convert to grid::unit
            hmargs$row_names_max_width <- unit(
                as.numeric(gsub("[^0-9.]", "", hmargs$row_names_max_width)),
                gsub("[0-9.]", "", hmargs$row_names_max_width)
            )
        } else if (is.numeric(hmargs$row_names_max_width)) {
            hmargs$row_names_max_width <- unit(
                hmargs$row_names_max_width,
                "inches"
            )
        }
    } else if (flip) {
        hmargs$row_names_max_width <- ComplexHeatmap::max_text_width(levels(data[[
            columns_by
        ]]))
    } else {
        hmargs$row_names_max_width <- ComplexHeatmap::max_text_width(levels(data[[
            rows_by
        ]]))
    }

    # Collect the legends
    legends <- list()

    # Set the default cell aggregation function for pie chart (will be plotted as the background)
    cell_agg <- cell_agg %||% ifelse(cell_type == "pie", "nansum", "nanmean")
    if (is.character(cell_agg)) {
        if (startsWith(cell_agg, "nan")) {
            fn <- match.fun(substring(cell_agg, 4))
            cell_agg <- function(x) fn(x[is.finite(x)])
        } else {
            cell_agg <- match.fun(cell_agg)
        }
    }

    # Extract the matrix for the heatmap (aggregated values, for e.g. tile, label, pie background, etc)
    # We also need it for bars, because ComplexHeatmap::Heatmap need the matrix to plot anyway
    # rows_split_by  rows_by  columns_split_by1::columns_by1 columns_split_by2::columns_by2 ...
    # rows_split_by1 rows_by1 0.1                            0.2                            ...
    # rows_split_by2 rows_by2 0.3                            0.4                            ...
    # ...
    hmargs$matrix <- data %>%
        group_by(
            !!!syms(unique(c(
                rows_split_by,
                rows_by,
                columns_split_by,
                columns_by
            )))
        ) %>%
        summarise(.value = cell_agg(!!sym(values_by)), .groups = "drop") %>%
        unite(
            ".columns",
            !!!syms(unique(c(columns_split_by, columns_by))),
            sep = " // "
        ) %>%
        unite(
            ".rows",
            !!!syms(unique(c(rows_split_by, rows_by))),
            sep = " // "
        ) %>%
        pivot_wider(
            names_from = ".columns",
            values_from = ".value",
            values_fill = values_fill
        ) %>%
        as.data.frame()

    rownames(hmargs$matrix) <- hmargs$matrix$.rows
    hmargs$matrix$.rows <- NULL
    hmargs$matrix <- as.matrix(hmargs$matrix)
    hmargs$matrix[is.na(hmargs$matrix)] <- values_fill

    columns_order <- data %>%
        tidyr::expand(!!!syms(unique(c(columns_split_by, columns_by)))) %>%
        unite(
            ".columns",
            !!!syms(unique(c(columns_split_by, columns_by))),
            sep = " // "
        ) %>%
        dplyr::pull(".columns") %>%
        unique() %>%
        intersect(colnames(hmargs$matrix))
    rows_order <- data %>%
        tidyr::expand(!!!syms(unique(c(rows_split_by, rows_by)))) %>%
        unite(
            ".rows",
            !!!syms(unique(c(rows_split_by, rows_by))),
            sep = " // "
        ) %>%
        dplyr::pull(".rows") %>%
        unique() %>%
        intersect(rownames(hmargs$matrix))

    hmargs$matrix <- hmargs$matrix[rows_order, columns_order, drop = FALSE]

    if (flip) {
        hmargs$matrix <- t(hmargs$matrix)
    }

    r_split_by <- if (flip) columns_split_by else rows_split_by
    c_split_by <- if (flip) rows_split_by else columns_split_by
    r_by <- if (flip) columns_by else rows_by
    c_by <- if (flip) rows_by else columns_by
    r_split_levels <- if (!is.null(r_split_by)) levels(data[[r_split_by]])
    c_split_levels <- if (!is.null(c_split_by)) levels(data[[c_split_by]])
    r_levels <- if (!is.null(r_by)) levels(data[[r_by]])
    c_levels <- if (!is.null(c_by)) levels(data[[c_by]])
    if (!is.null(r_split_by)) {
        row_split_labels <- strsplit(
            rownames(hmargs$matrix),
            " // ",
            fixed = TRUE
        )
        hmargs$row_split <- factor(
            sapply(row_split_labels, `[`, 1),
            levels = r_split_levels
        )
        hmargs$row_labels <- factor(
            sapply(row_split_labels, `[`, 2),
            levels = r_levels
        )
    } else {
        hmargs$row_labels <- factor(rownames(hmargs$matrix), levels = r_levels)
    }

    if (!is.null(c_split_by)) {
        column_split_labels <- strsplit(
            colnames(hmargs$matrix),
            " // ",
            fixed = TRUE
        )
        hmargs$column_split <- factor(
            sapply(column_split_labels, `[`, 1),
            levels = c_split_levels
        )
        hmargs$column_labels <- factor(
            sapply(column_split_labels, `[`, 2),
            levels = c_levels
        )
    } else {
        hmargs$column_labels <- factor(
            colnames(hmargs$matrix),
            levels = c_levels
        )
    }

    if (cell_type == "bars") {
        # where multiple values are used, operating on data
        lower_cutoff <- lower_cutoff %||%
            quantile(
                data[[values_by]][is.finite(data[[values_by]])],
                lower_quantile,
                na.rm = TRUE
            )
        upper_cutoff <- upper_cutoff %||%
            quantile(
                data[[values_by]][is.finite(data[[values_by]])],
                upper_quantile,
                na.rm = TRUE
            )
        data[[values_by]][data[[values_by]] < lower_cutoff] <- lower_cutoff
        data[[values_by]][data[[values_by]] > upper_cutoff] <- upper_cutoff
    } else {
        # where aggregated values are used
        lower_cutoff <- lower_cutoff %||%
            quantile(
                hmargs$matrix[is.finite(hmargs$matrix)],
                lower_quantile,
                na.rm = TRUE
            )
        upper_cutoff <- upper_cutoff %||%
            quantile(
                hmargs$matrix[is.finite(hmargs$matrix)],
                upper_quantile,
                na.rm = TRUE
            )
    }

    # Set the color function for the heatmap cells
    hmargs$col <- get_col_fun(lower_cutoff, upper_cutoff)

    # Indices for data in layer_fun
    indices <- if (flip) {
        # 1-1, 2-1, 1-2, 2-2, ...
        expand.grid(1:nrow(hmargs$matrix), 1:ncol(hmargs$matrix))
    } else {
        # 1-1, 1-2, 2-1, 2-2, ...
        expand_grid(1:nrow(hmargs$matrix), 1:ncol(hmargs$matrix))
    }
    indices <- paste(indices[[1]], indices[[2]], sep = "-")

    # Compose the main legend
    get_main_legend <- function(allow_discreate = TRUE) {
        if (identical(legend.position, "none")) {
            return(NULL)
        }
        if (!allow_discreate && isTRUE(legend_discrete)) {
            stop("[Heatmap] 'legend_discrete = TRUE' is not allowed.")
        }

        if (isTRUE(legend_discrete)) {
            if (is.null(legend_items)) {
                lgd_items <- sort(
                    unique(as.vector(hmargs$matrix)),
                    decreasing = TRUE
                )
                names(lgd_items) <- as.character(lgd_items)
            } else {
                lgd_items <- unlist(legend_items)
            }
            ComplexHeatmap::Legend(
                title = values_by,
                at = lgd_items,
                labels = names(lgd_items),
                legend_gp = gpar(fill = hmargs$col(lgd_items)),
                border = TRUE,
                direction = legend.direction
            )
        } else {
            ComplexHeatmap::Legend(
                title = values_by,
                col_fun = hmargs$col,
                border = TRUE,
                direction = legend.direction
            )
        }
    }

    if (cell_type == "pie") {
        if (is.null(pie_group_by)) {
            stop(
                "[Heatmap] Please provide 'pie_group_by' to use 'cell_type = 'pie'."
            )
        }
        pie_values <- pie_values %||% "length"
        if (is.character(pie_values)) {
            if (startsWith(pie_values, "nan")) {
                fn <- match.fun(substring(pie_values, 4))
                pie_values <- function(x) fn(x[is.finite(x)])
            } else {
                pie_values <- match.fun(pie_values)
            }
        }
        keep_empty_pie_group <- if (isFALSE(keep_empty)) {
            FALSE
        } else {
            keep_empty[[pie_group_by]]
        }
        pie_group_levels <- levels(data[[pie_group_by]])
        if (anyNA(data[[pie_group_by]])) {
            pie_group_levels <- c(pie_group_by, NA)
        }
        pie_data <- data %>%
            group_by(
                !!!syms(unique(c(
                    rows_split_by,
                    rows_by,
                    columns_split_by,
                    columns_by,
                    pie_group_by
                ))),
                .drop = FALSE
            ) %>%
            summarise(
                .value = pie_values(!!sym(values_by)),
                .groups = "drop"
            ) %>%
            group_by(
                !!!syms(unique(c(
                    rows_split_by,
                    rows_by,
                    columns_split_by,
                    columns_by
                )))
            ) %>%
            group_map(
                ~ data.frame(Var = .x[[pie_group_by]], Freq = .x$.value)
            )
        names(pie_data) <- indices

        pie_colors <- palette_this(
            pie_group_levels,
            palette = pie_palette,
            palcolor = pie_palcolor,
            NA_keep = TRUE
        )
        if (is.character((pie_size))) {
            if (startsWith(pie_size, "nan")) {
                fn <- match.fun(substring(pie_size, 4))
                pie_size <- function(x) fn(x[is.finite(x)])
            } else {
                pie_size <- match.fun(pie_size)
            }
        }
        hmargs$layer_fun <- function(j, i, x, y, w, h, fill, sr, sc) {
            layer_white_bg(j, i, x, y, w, h, fill)
            if (isTRUE(add_bg)) {
                layer_bg(j, x, x, y, w, h, fill, alpha = bg_alpha)
            }
            if (isTRUE(add_reticle)) {
                layer_reticle(j, i, x, y, w, h, fill, color = reticle_color)
            }
            layer_pie(
                j,
                i,
                x,
                y,
                w,
                h,
                fill,
                palette = pie_palette,
                palcolor = pie_palcolor,
                data = pie_data,
                pie_size = pie_size
            )
            if (is.function(layer_fun_callback)) {
                layer_fun_callback(j, i, x, y, w, h, fill, sr, sc)
            }
        }

        if (!identical(legend.position, "none") && is.function(pie_size)) {
            pie_sizes <- sapply(pie_data, function(d) {
                pie_size(sum(d$Freq, na.rm = TRUE))
            })
            pie_size_min <- min(pie_sizes, na.rm = TRUE)
            pie_size_max <- max(pie_sizes, na.rm = TRUE)
            legends$.pie_size <- ComplexHeatmap::Legend(
                title = pie_size_name,
                labels = scales::number(
                    (seq(
                        pie_size_min,
                        pie_size_max,
                        length.out = ifelse(pie_size_max > pie_size_min, 4, 1)
                    ))
                ),
                type = "points",
                pch = 21,
                size = unit(8, "mm") * seq(0.2, 1, length.out = 4),
                grid_height = unit(8, "mm") * seq(0.2, 1, length.out = 4) * 0.8,
                grid_width = unit(8, "mm"),
                legend_gp = gpar(fill = "grey30"),
                border = FALSE,
                background = "transparent",
                direction = legend.direction
            )
        }
        if (isTRUE(add_bg)) {
            legends$.heatmap <- get_main_legend()
        }
        if (!identical(legend.position, "none")) {
            legends$.pie <- ComplexHeatmap::Legend(
                title = pie_group_by,
                direction = legend.direction,
                border = TRUE,
                labels = pie_group_levels,
                legend_gp = gpar(fill = pie_colors)
            )
        }
    } else if (cell_type == "bars") {
        if (isTRUE(add_bg)) {
            stop("[Heatmap] Cannot use 'add_bg' with 'cell_type = 'bars'.")
        }
        if (isTRUE(add_reticle)) {
            stop("[Heatmap] Cannot use 'add_reticle' with 'cell_type = 'bars'.")
        }

        if (!is.numeric(bars_sample) || length(bars_sample) != 1 ||
            is.na(bars_sample) || bars_sample <= 0 ||
            (bars_sample > 1 && bars_sample != floor(bars_sample))) {
            stop(
                "[Heatmap] 'bars_sample' must be a fraction (0, 1] or ",
                "a whole count > 1."
            )
        }
        validate_common_args(seed)
        set.seed(seed)
        bars_data <- data %>%
            group_by(
                !!!syms(unique(c(
                    rows_split_by,
                    rows_by,
                    columns_split_by,
                    columns_by
                )))
            ) %>%
            group_map(~ {
                vals <- .x[[values_by]]
                if (bars_sample <= 1) {
                    # fraction (1 = all data)
                    sample(vals, ceiling(length(vals) * bars_sample))
                } else {
                    # whole count per cell
                    sample(vals, min(bars_sample, length(vals)))
                }
            })

        names(bars_data) <- indices

        # plot bars in each cell
        hmargs$layer_fun <- function(j, i, x, y, w, h, fill, sr, sc) {
            layer_white_bg(j, i, x, y, w, h, fill)
            layer_bars(
                j,
                i,
                x,
                y,
                w,
                h,
                fill,
                flip = flip,
                col_fun = hmargs$col,
                data = bars_data,
                alpha = alpha
            )
            if (is.function(layer_fun_callback)) {
                layer_fun_callback(j, i, x, y, w, h, fill, sr, sc)
            }
        }
        # Override the main legend
        legends$.heatmap <- get_main_legend(FALSE)
    } else if (cell_type == "dot") {
        if (is.character(dot_size)) {
            if (startsWith(dot_size, "nan")) {
                fn <- match.fun(substring(dot_size, 4))
                dot_size <- function(x) fn(x[is.finite(x)])
            } else {
                dot_size <- match.fun(dot_size)
            }
        }
        # Store raw values for each cell to pass to dot_size function later
        dot_data <- data %>%
            group_by(
                !!!syms(unique(c(
                    rows_split_by,
                    rows_by,
                    columns_split_by,
                    columns_by
                )))
            ) %>%
            summarise(.value = list(!!sym(values_by)), .groups = "drop") %>%
            unite(
                ".columns",
                !!!syms(unique(c(columns_split_by, columns_by))),
                sep = " // "
            ) %>%
            unite(
                ".rows",
                !!!syms(unique(c(rows_split_by, rows_by))),
                sep = " // "
            ) %>%
            pivot_wider(names_from = ".columns", values_from = ".value") %>%
            select(-!!sym(".rows")) %>%
            as.data.frame()

        if (flip) {
            dot_data <- t(dot_data)
        }

        if (
            !identical(legend.position, "none") &&
                is.function(dot_size) &&
                !is.null(dot_size_name)
        ) {
            # Optimized: only compute min/max for legend, not all sizes
            nargs <- length(formalArgs(dot_size))
            dot_size_min <- Inf
            dot_size_max <- -Inf

            for (idx in seq_along(indices)) {
                cell_key <- indices[idx]
                # Parse indices from the key "i-j"
                ij <- as.integer(strsplit(cell_key, "-")[[1]])
                cell_values <- dot_data[ij[1], ij[2]][[1]]

                size_val <- if (nargs == 1 || is.primitive(dot_size)) {
                    dot_size(cell_values)
                } else if (nargs == 3) {
                    dot_size(cell_values, ij[1], ij[2])
                } else if (nargs == 5) {
                    dot_size(
                        cell_values,
                        ij[1],
                        ij[2],
                        rownames(hmargs$matrix)[ij[1]],
                        colnames(hmargs$matrix)[ij[2]]
                    )
                } else {
                    stop(
                        "[Heatmap] 'dot_size' function should take 1, 3 or 5 arguments."
                    )
                }

                if (is.finite(size_val)) {
                    if (size_val < dot_size_min) {
                        dot_size_min <- size_val
                    }
                    if (size_val > dot_size_max) dot_size_max <- size_val
                }
            }

            legends$.dot_size <- ComplexHeatmap::Legend(
                title = dot_size_name,
                labels = scales::number(
                    (seq(
                        dot_size_min,
                        dot_size_max,
                        length.out = ifelse(dot_size_max > dot_size_min, 4, 1)
                    ))
                ),
                type = "points",
                pch = 21,
                size = unit(8, "mm") * seq(0.2, 1, length.out = 4),
                grid_height = unit(8, "mm") * seq(0.2, 1, length.out = 4) * 0.8,
                grid_width = unit(8, "mm"),
                legend_gp = gpar(fill = "grey30"),
                border = FALSE,
                background = "transparent",
                direction = legend.direction
            )
        }

        hmargs$layer_fun <- function(j, i, x, y, w, h, fill, sr, sc) {
            layer_white_bg(j, i, x, y, w, h, fill)
            if (isTRUE(add_bg)) {
                layer_bg(j, i, x, y, w, h, fill, alpha = bg_alpha)
            }
            if (isTRUE(add_reticle)) {
                layer_reticle(j, i, x, y, w, h, fill, color = reticle_color)
            }
            # Compute dot sizes based on function arguments
            if (is.function(dot_size)) {
                nargs <- length(formalArgs(dot_size))
                sizes <- numeric(length(i))
                for (idx in seq_along(i)) {
                    cell_values <- dot_data[[i[idx], j[idx]]]
                    if (nargs == 1 || is.primitive(dot_size)) {
                        sizes[idx] <- dot_size(cell_values)
                    } else if (nargs == 3) {
                        sizes[idx] <- dot_size(cell_values, i[idx], j[idx])
                    } else if (nargs == 5) {
                        sizes[idx] <- dot_size(
                            cell_values,
                            i[idx],
                            j[idx],
                            rownames(hmargs$matrix)[i[idx]],
                            colnames(hmargs$matrix)[j[idx]]
                        )
                    } else {
                        stop(
                            "[Heatmap] 'dot_size' function should take 1, 3 or 5 arguments."
                        )
                    }
                }
                layer_dot(
                    j,
                    i,
                    x,
                    y,
                    w,
                    h,
                    fill,
                    data = dot_data,
                    dot_size = sizes,
                    alpha = alpha
                )
            } else {
                layer_dot(
                    j,
                    i,
                    x,
                    y,
                    w,
                    h,
                    fill,
                    data = dot_data,
                    dot_size = dot_size,
                    alpha = alpha
                )
            }
            if (is.function(layer_fun_callback)) {
                layer_fun_callback(j, i, x, y, w, h, fill, sr, sc)
            }
        }
        # Override the main legend
        legends$.heatmap <- get_main_legend()
    } else if (cell_type %in% c("violin", "boxplot")) {
        # df with multiple values in each cell
        vdata <- data %>%
            group_by(
                !!!syms(unique(c(
                    rows_split_by,
                    rows_by,
                    columns_split_by,
                    columns_by
                )))
            ) %>%
            group_map(~ .x[[values_by]])

        names(vdata) <- indices
        vcolors <- if (cell_type == "violin") violin_fill else boxplot_fill

        hmargs$layer_fun <- function(j, i, x, y, w, h, fill, sr, sc) {
            layer_white_bg(j, i, x, y, w, h, fill)
            if (isTRUE(add_bg)) {
                layer_bg(j, i, x, y, w, h, fill, alpha = bg_alpha)
            }
            if (isTRUE(add_reticle)) {
                layer_reticle(j, i, x, y, w, h, fill, color = reticle_color)
            }
            layer_fn <- if (cell_type == "violin") ViolinPlot else BoxPlot
            layer_boxviolin(
                j,
                i,
                x,
                y,
                w,
                h,
                fill,
                flip = flip,
                data = vdata,
                colors = vcolors,
                fn = layer_fn
            )
            if (is.function(layer_fun_callback)) {
                layer_fun_callback(j, i, x, y, w, h, fill, sr, sc)
            }
        }
        if (!identical(legend.position, "none")) {
            if (!is.null(legend_items)) {
                stop(
                    "[Heatmap] Cannot use 'legend_items' with 'cell_type = 'violin' or 'boxplot'."
                )
            }
            if (isTRUE(legend_discrete)) {
                stop(
                    "[Heatmap] Cannot use 'legend_discrete = TRUE' with 'cell_type = 'violin' or 'boxplot'."
                )
            }
            if (is.null(vcolors)) {
                legends$.heatmap <- ComplexHeatmap::Legend(
                    title = values_by,
                    col_fun = hmargs$col,
                    border = TRUE,
                    direction = legend.direction
                )
            } else if (isTRUE(add_bg)) {
                legends$.heatmap <- ComplexHeatmap::Legend(
                    title = values_by,
                    col_fun = get_col_fun(lower_cutoff, upper_cutoff, bg_alpha),
                    border = TRUE,
                    direction = legend.direction
                )
            }
        }
    } else if (cell_type == "tile") {
        if (isTRUE(add_bg)) {
            stop("[Heatmap] Cannot use 'add_bg' with 'cell_type = 'tile'.")
        }
        if (isTRUE(add_reticle)) {
            stop("[Heatmap] Cannot use 'add_reticle' with 'cell_type = 'tile'.")
        }
        hmargs$rect_gp <- gpar(col = "grey80", lwd = 0.1)
        hmargs$layer_fun <- layer_fun_callback
        legends$.heatmap <- get_main_legend()
    } else if (cell_type == "label") {
        if (isTRUE(add_bg)) {
            stop("[Heatmap] Cannot use 'add_bg' with 'cell_type = 'tile'.")
        }
        if (isTRUE(add_reticle)) {
            stop("[Heatmap] Cannot use 'add_reticle' with 'cell_type = 'tile'.")
        }
        # Helper: call label() for a single cell value, dispatching by nargs
        .call_label <- if (is.function(label)) {
            nargs_label <- length(formalArgs(label))
            rnames <- rownames(hmargs$matrix)
            cnames <- colnames(hmargs$matrix)
            if (nargs_label == 1 || is.primitive(label)) {
                function(val, ri, ci) label(val)
            } else if (nargs_label == 3) {
                function(val, ri, ci) label(val, ri, ci)
            } else if (nargs_label == 5) {
                function(val, ri, ci) label(val, ri, ci, rnames[ri], cnames[ci])
            } else {
                stop(
                    "[Heatmap] 'label' function should take 1, 3 or 5 arguments."
                )
            }
        } else {
            function(val, ri, ci) val
        }

        # Helper: extract label text, supporting unnamed first list element
        .extract_lbl <- function(r) {
            if (!is.list(r)) {
                return(
                    if (length(r) == 1 && is.na(r)) {
                        NA_character_
                    } else {
                        as.character(r)
                    }
                )
            }
            if (!is.null(r$label)) {
                return(as.character(r$label))
            }
            nms <- names(r)
            unnamed_idx <- which(is.null(nms) | nms == "")
            if (length(unnamed_idx) > 0) {
                return(as.character(r[[unnamed_idx[1]]]))
            }
            NA_character_
        }

        # Pre-compute over whole matrix to auto-detect legend entries
        if (!identical(legend.position, "none")) {
            lgnd_seen <- list() # named list: legend_key -> list(text, color)
            for (ri in seq_len(nrow(hmargs$matrix))) {
                for (ci in seq_len(ncol(hmargs$matrix))) {
                    r <- .call_label(hmargs$matrix[ri, ci], ri, ci)
                    if (is.list(r) && !is.null(r$legend) && !is.na(r$legend)) {
                        key <- r$legend
                        if (is.null(lgnd_seen[[key]])) {
                            lbl_txt <- .extract_lbl(r)
                            lgnd_seen[[key]] <- list(
                                text = if (!is.na(lbl_txt)) lbl_txt else key,
                                color = r$color %||% label_color,
                                order = r$order %||% NA_integer_
                            )
                        }
                    }
                }
            }

            if (length(lgnd_seen) > 0) {
                # Sort by order field if any entries carry it
                orders <- sapply(lgnd_seen, function(e) {
                    e$order %||% NA_integer_
                })
                if (!all(is.na(orders))) {
                    # Entries without order are placed after explicitly ordered ones
                    orders[is.na(orders)] <- max(orders, na.rm = TRUE) +
                        seq_len(sum(is.na(orders)))
                    lgnd_seen <- lgnd_seen[order(orders)]
                }
                lgnd_graphics <- lapply(lgnd_seen, function(entry) {
                    txt <- entry$text
                    col <- entry$color
                    sz <- label_size
                    function(x, y, w, h, fill) {
                        grid.text(
                            txt,
                            x,
                            y,
                            gp = gpar(
                                fontsize = sz,
                                col = col,
                                fontface = "bold"
                            )
                        )
                    }
                })
                legends$.label <- ComplexHeatmap::Legend(
                    title = label_name,
                    labels = names(lgnd_seen),
                    graphics = lgnd_graphics,
                    direction = legend.direction
                )
            }
        }

        hmargs$layer_fun <- function(j, i, x, y, w, h, fill, sr, sc) {
            raw_vals <- ComplexHeatmap::pindex(hmargs$matrix, i, j)
            results <- lapply(seq_along(i), function(k) {
                .call_label(raw_vals[k], i[k], j[k])
            })

            lbl <- sapply(results, .extract_lbl)
            sizes <- sapply(results, function(r) {
                if (is.list(r)) r$size %||% label_size else label_size
            })
            colors <- sapply(results, function(r) {
                if (is.list(r)) r$color %||% label_color else label_color
            })

            inds <- which(!is.na(lbl))
            if (length(inds) > 0) {
                theta <- seq(pi / 8, 2 * pi, length.out = 16)
                for (k in inds) {
                    sz <- sizes[k]
                    lapply(theta, function(a) {
                        grid.text(
                            lbl[k],
                            x = x[k] + unit(cos(a) * sz / 30, "mm"),
                            y = y[k] + unit(sin(a) * sz / 30, "mm"),
                            gp = gpar(fontsize = sz, col = "white")
                        )
                    })
                    grid.text(
                        lbl[k],
                        x[k],
                        y[k],
                        gp = gpar(fontsize = sz, col = colors[k])
                    )
                }
            }
            if (is.function(layer_fun_callback)) {
                layer_fun_callback(j, i, x, y, w, h, fill, sr, sc)
            }
        }
        legends$.heatmap <- get_main_legend()
    } else if (cell_type == "mark") {
        if (isTRUE(add_bg)) {
            stop("[Heatmap] Cannot use 'add_bg' with 'cell_type = 'mark'.")
        }
        if (isTRUE(add_reticle)) {
            stop("[Heatmap] Cannot use 'add_reticle' with 'cell_type = 'mark'.")
        }

        # ── Mark dispatch helper ─────────────────────────────────────────────
        .call_mark <- if (is.function(mark)) {
            nargs_mark <- length(formalArgs(mark))
            rnames <- rownames(hmargs$matrix)
            cnames <- colnames(hmargs$matrix)
            if (nargs_mark == 1 || is.primitive(mark)) {
                function(val, ri, ci) mark(val)
            } else if (nargs_mark == 3) {
                function(val, ri, ci) mark(val, ri, ci)
            } else if (nargs_mark == 5) {
                function(val, ri, ci) mark(val, ri, ci, rnames[ri], cnames[ci])
            } else {
                stop(
                    "[Heatmap] 'mark' function should take 1, 3 or 5 arguments."
                )
            }
        } else {
            function(val, ri, ci) val
        }

        # ── Extract mark type string ────────────────────────────────────────
        .extract_mark <- function(r) {
            if (!is.list(r)) {
                return(
                    if (length(r) == 1 && is.na(r)) {
                        NA_character_
                    } else {
                        as.character(r)
                    }
                )
            }
            if (!is.null(r$mark)) {
                return(as.character(r$mark))
            }
            nms <- names(r)
            unnamed_idx <- which(is.null(nms) | nms == "")
            if (length(unnamed_idx) > 0) {
                return(as.character(r[[unnamed_idx[1]]]))
            }
            NA_character_
        }

        # ── Mark type parser ────────────────────────────────────────────────
        # Parses a mark string into a vector of drawing primitives.
        # Outer wrappers: [] adds "rect", () adds "circle_full", <> adds "diamond".
        # Wrappers are stripped left-to-right; the remainder is an inner primitive.
        # Inner primitives: -, |, +, /, \, x, o, (), <>
        .parse_mark_type <- function(m) {
            if (is.na(m) || m == "" || m == "NA") {
                return(character(0))
            }
            prims <- character(0)
            repeat {
                if (nchar(m) >= 2 && startsWith(m, "[") && endsWith(m, "]")) {
                    prims <- c(prims, "rect")
                    m <- substr(m, 2L, nchar(m) - 1L)
                } else if (
                    nchar(m) >= 3 && startsWith(m, "(") && endsWith(m, ")")
                ) {
                    # "(-)" wrapper, but "()" alone is a primitive — handled below
                    prims <- c(prims, "circle_full")
                    m <- substr(m, 2L, nchar(m) - 1L)
                } else if (
                    nchar(m) >= 3 && startsWith(m, "<") && endsWith(m, ">")
                ) {
                    # "<->" wrapper, but "<>" alone is a primitive — handled below
                    prims <- c(prims, "diamond")
                    m <- substr(m, 2L, nchar(m) - 1L)
                } else if (
                    nchar(m) >= 3 && startsWith(m, "{") && endsWith(m, "}")
                ) {
                    # "{-}" wrapper, but "{}" alone is a primitive — handled below
                    prims <- c(prims, "octagon")
                    m <- substr(m, 2L, nchar(m) - 1L)
                } else {
                    break
                }
            }
            inner <- if (m == "") {
                character(0)
            } else {
                switch(
                    m,
                    "-" = "hline",
                    "|" = "vline",
                    "+" = c("hline", "vline"),
                    "/" = "ldiag",
                    "\\" = "rdiag",
                    "x" = c("ldiag", "rdiag"),
                    "o" = "circle_gap",
                    "()" = "circle_full",
                    "<>" = "diamond",
                    "{}" = "octagon",
                    stop(paste0("[Heatmap] Unknown mark type: '", m, "'"))
                )
            }
            c(prims, inner)
        }

        # ── Vectorized primitive renderer ───────────────────────────────────
        # Draws one primitive for a batch of cells (same mark type, different positions).
        .draw_mark_prim <- function(prim, xv, yv, wv, hv, col, lwd) {
            switch(
                prim,
                rect = grid.rect(
                    x = xv,
                    y = yv,
                    width = wv,
                    height = hv,
                    gp = gpar(col = col, fill = NA, lwd = lwd)
                ),

                hline = grid.segments(
                    x0 = xv - wv * 0.5,
                    y0 = yv,
                    x1 = xv + wv * 0.5,
                    y1 = yv,
                    gp = gpar(col = col, lwd = lwd)
                ),

                vline = grid.segments(
                    x0 = xv,
                    y0 = yv - hv * 0.5,
                    x1 = xv,
                    y1 = yv + hv * 0.5,
                    gp = gpar(col = col, lwd = lwd)
                ),

                ldiag = grid.segments(
                    x0 = xv - wv * 0.5,
                    y0 = yv - hv * 0.5,
                    x1 = xv + wv * 0.5,
                    y1 = yv + hv * 0.5,
                    gp = gpar(col = col, lwd = lwd)
                ),

                rdiag = grid.segments(
                    x0 = xv - wv * 0.5,
                    y0 = yv + hv * 0.5,
                    x1 = xv + wv * 0.5,
                    y1 = yv - hv * 0.5,
                    gp = gpar(col = col, lwd = lwd)
                ),

                # circle: diameter = 3/4 of the shorter physical edge, true circle
                # Use mm so the radius is invariant to viewport aspect ratio
                circle_gap = {
                    .r_mm <- min(
                        convertUnit(wv[1L], "mm", valueOnly = TRUE),
                        convertUnit(hv[1L], "mm", valueOnly = TRUE)
                    ) *
                        0.25
                    grid.circle(
                        x = xv,
                        y = yv,
                        r = unit(.r_mm, "mm"),
                        gp = gpar(col = col, fill = NA, lwd = lwd)
                    )
                },

                # ellipse touching all 4 cell edges (no gap), handles non-square cells
                circle_full = {
                    .theta <- seq(0, 2 * pi, length.out = 65L)[seq_len(64L)]
                    .cos_t <- cos(.theta)
                    .sin_t <- sin(.theta)
                    for (k in seq_along(xv)) {
                        grid.polygon(
                            x = xv[k] + wv[k] * 0.5 * .cos_t,
                            y = yv[k] + hv[k] * 0.5 * .sin_t,
                            gp = gpar(
                                col = if (length(col) == 1L) col else col[k],
                                fill = NA,
                                lwd = if (length(lwd) == 1L) lwd else lwd[k]
                            )
                        )
                    }
                },

                # diamond — loop per cell (grid.polygon id approach avoids this)
                diamond = {
                    n <- length(xv)
                    hw <- wv * 0.5
                    hh <- hv * 0.5
                    for (k in seq_len(n)) {
                        grid.polygon(
                            x = unit.c(
                                xv[k],
                                xv[k] + hw[k],
                                xv[k],
                                xv[k] - hw[k]
                            ),
                            y = unit.c(
                                yv[k] + hh[k],
                                yv[k],
                                yv[k] - hh[k],
                                yv[k]
                            ),
                            gp = gpar(col = col[k], fill = NA, lwd = lwd[k])
                        )
                    }
                },

                # octagon: 4 sides on the cell edges, 4 small corner cuts (cut factor = 0.2)
                # The corner cuts are closer to the actual corners than a regular octagon would be
                octagon = {
                    n <- length(xv)
                    hw <- wv * 0.5
                    hh <- hv * 0.5
                    f <- 0.2 # corner cut factor (0 = full rectangle, 0.5 = diamond)
                    for (k in seq_len(n)) {
                        cx <- xv[k]
                        cy <- yv[k]
                        dx <- hw[k] * f
                        dy <- hh[k] * f
                        grid.polygon(
                            x = unit.c(
                                cx - hw[k] + dx,
                                cx + hw[k] - dx, # top edge: left → right
                                cx + hw[k],
                                cx + hw[k], # right edge: top → bottom
                                cx + hw[k] - dx,
                                cx - hw[k] + dx, # bottom edge: right → left
                                cx - hw[k],
                                cx - hw[k] # left edge: bottom → top
                            ),
                            y = unit.c(
                                cy + hh[k],
                                cy + hh[k], # top edge
                                cy + hh[k] - dy,
                                cy - hh[k] + dy, # right edge
                                cy - hh[k],
                                cy - hh[k], # bottom edge
                                cy - hh[k] + dy,
                                cy + hh[k] - dy # left edge
                            ),
                            gp = gpar(col = col[k], fill = NA, lwd = lwd[k])
                        )
                    }
                }
            )
        }

        # ── Legend icon builder ─────────────────────────────────────────────
        # Returns a function(x, y, w, h, fill) that draws the mark in a legend key.
        .mark_legend_icon <- function(mark_str, col, lwd) {
            prims <- .parse_mark_type(mark_str)
            function(x, y, w, h, fill) {
                for (p in prims) {
                    .draw_mark_prim(p, x, y, w, h, col, lwd)
                }
            }
        }

        # ── Pre-scan for legend entries ─────────────────────────────────────
        if (!identical(legend.position, "none")) {
            mkl_seen <- list()
            for (ri in seq_len(nrow(hmargs$matrix))) {
                for (ci in seq_len(ncol(hmargs$matrix))) {
                    r <- .call_mark(hmargs$matrix[ri, ci], ri, ci)
                    if (is.list(r) && !is.null(r$legend) && !is.na(r$legend)) {
                        key <- r$legend
                        if (is.null(mkl_seen[[key]])) {
                            mk_str <- .extract_mark(r)
                            mkl_seen[[key]] <- list(
                                mark = if (!is.na(mk_str)) mk_str else key,
                                color = r$color %||% mark_color,
                                size = r$size %||% mark_size,
                                order = r$order %||% NA_integer_
                            )
                        }
                    }
                }
            }

            if (length(mkl_seen) > 0) {
                # Sort by order
                orders <- sapply(mkl_seen, function(e) e$order %||% NA_integer_)
                if (!all(is.na(orders))) {
                    orders[is.na(orders)] <- max(orders, na.rm = TRUE) +
                        seq_len(sum(is.na(orders)))
                    mkl_seen <- mkl_seen[order(orders)]
                }
                mkl_graphics <- lapply(mkl_seen, function(entry) {
                    .mark_legend_icon(entry$mark, entry$color, entry$size)
                })
                legends$.mark <- ComplexHeatmap::Legend(
                    title = mark_name,
                    labels = names(mkl_seen),
                    graphics = mkl_graphics,
                    direction = legend.direction,
                    row_gap = unit(1, "mm")
                )
            }
        }

        # ── layer_fun ───────────────────────────────────────────────────────
        hmargs$layer_fun <- function(j, i, x, y, w, h, fill, sr, sc) {
            raw_vals <- ComplexHeatmap::pindex(hmargs$matrix, i, j)
            results <- lapply(seq_along(i), function(k) {
                .call_mark(raw_vals[k], i[k], j[k])
            })

            marks_vec <- sapply(results, .extract_mark)
            colors_vec <- sapply(results, function(r) {
                if (is.list(r)) r$color %||% mark_color else mark_color
            })
            sizes_vec <- sapply(results, function(r) {
                if (is.list(r)) r$size %||% mark_size else mark_size
            })

            # Draw each unique mark type as a batch
            unique_marks <- unique(marks_vec[!is.na(marks_vec)])
            for (mk in unique_marks) {
                km <- which(marks_vec == mk)
                prms <- .parse_mark_type(mk)
                for (prim in prms) {
                    .draw_mark_prim(
                        prim,
                        x[km],
                        y[km],
                        w[km],
                        h[km],
                        colors_vec[km],
                        sizes_vec[km]
                    )
                }
            }

            if (is.function(layer_fun_callback)) {
                layer_fun_callback(j, i, x, y, w, h, fill, sr, sc)
            }
        }
        legends$.heatmap <- get_main_legend()
    } else if (cell_type == "label+mark") {
        if (isTRUE(add_bg)) {
            stop(
                "[Heatmap] Cannot use 'add_bg' with 'cell_type = \"label+mark\"'."
            )
        }
        if (isTRUE(add_reticle)) {
            stop(
                "[Heatmap] Cannot use 'add_reticle' with 'cell_type = \"label+mark\"'."
            )
        }
        # ── Label dispatch helper ────────────────────────────────────────────
        .call_label <- if (is.function(label)) {
            nargs_label <- length(formalArgs(label))
            rnames <- rownames(hmargs$matrix)
            cnames <- colnames(hmargs$matrix)
            if (nargs_label == 1 || is.primitive(label)) {
                function(val, ri, ci) label(val)
            } else if (nargs_label == 3) {
                function(val, ri, ci) label(val, ri, ci)
            } else if (nargs_label == 5) {
                function(val, ri, ci) label(val, ri, ci, rnames[ri], cnames[ci])
            } else {
                stop(
                    "[Heatmap] 'label' function should take 1, 3 or 5 arguments."
                )
            }
        } else {
            function(val, ri, ci) val
        }

        # Helper: extract label text, supporting unnamed first list element
        .extract_lbl <- function(r) {
            if (!is.list(r)) {
                return(
                    if (length(r) == 1 && is.na(r)) {
                        NA_character_
                    } else {
                        as.character(r)
                    }
                )
            }
            if (!is.null(r$label)) {
                return(as.character(r$label))
            }
            nms <- names(r)
            unnamed_idx <- which(is.null(nms) | nms == "")
            if (length(unnamed_idx) > 0) {
                return(as.character(r[[unnamed_idx[1]]]))
            }
            NA_character_
        }

        # ── Mark dispatch helper ─────────────────────────────────────────────
        .call_mark <- if (is.function(mark)) {
            nargs_mark <- length(formalArgs(mark))
            rnames <- rownames(hmargs$matrix)
            cnames <- colnames(hmargs$matrix)
            if (nargs_mark == 1 || is.primitive(mark)) {
                function(val, ri, ci) mark(val)
            } else if (nargs_mark == 3) {
                function(val, ri, ci) mark(val, ri, ci)
            } else if (nargs_mark == 5) {
                function(val, ri, ci) mark(val, ri, ci, rnames[ri], cnames[ci])
            } else {
                stop(
                    "[Heatmap] 'mark' function should take 1, 3 or 5 arguments."
                )
            }
        } else {
            function(val, ri, ci) val
        }

        # ── Extract mark type string ────────────────────────────────────────
        .extract_mark <- function(r) {
            if (!is.list(r)) {
                return(
                    if (length(r) == 1 && is.na(r)) {
                        NA_character_
                    } else {
                        as.character(r)
                    }
                )
            }
            if (!is.null(r$mark)) {
                return(as.character(r$mark))
            }
            nms <- names(r)
            unnamed_idx <- which(is.null(nms) | nms == "")
            if (length(unnamed_idx) > 0) {
                return(as.character(r[[unnamed_idx[1]]]))
            }
            NA_character_
        }

        # ── Mark type parser ────────────────────────────────────────────────
        .parse_mark_type <- function(m) {
            if (is.na(m) || m == "" || m == "NA") {
                return(character(0))
            }
            prims <- character(0)
            repeat {
                if (nchar(m) >= 2 && startsWith(m, "[") && endsWith(m, "]")) {
                    prims <- c(prims, "rect")
                    m <- substr(m, 2L, nchar(m) - 1L)
                } else if (
                    nchar(m) >= 3 && startsWith(m, "(") && endsWith(m, ")")
                ) {
                    prims <- c(prims, "circle_full")
                    m <- substr(m, 2L, nchar(m) - 1L)
                } else if (
                    nchar(m) >= 3 && startsWith(m, "<") && endsWith(m, ">")
                ) {
                    prims <- c(prims, "diamond")
                    m <- substr(m, 2L, nchar(m) - 1L)
                } else if (
                    nchar(m) >= 3 && startsWith(m, "{") && endsWith(m, "}")
                ) {
                    # "{-}" wrapper, but "{}" alone is a primitive — handled below
                    prims <- c(prims, "octagon")
                    m <- substr(m, 2L, nchar(m) - 1L)
                } else {
                    break
                }
            }
            inner <- if (m == "") {
                character(0)
            } else {
                switch(
                    m,
                    "-" = "hline",
                    "|" = "vline",
                    "+" = c("hline", "vline"),
                    "/" = "ldiag",
                    "\\" = "rdiag",
                    "x" = c("ldiag", "rdiag"),
                    "o" = "circle_gap",
                    "()" = "circle_full",
                    "<>" = "diamond",
                    "{}" = "octagon",
                    stop(paste0("[Heatmap] Unknown mark type: '", m, "'"))
                )
            }
            c(prims, inner)
        }

        # ── Vectorized primitive renderer ───────────────────────────────────
        .draw_mark_prim <- function(prim, xv, yv, wv, hv, col, lwd) {
            switch(
                prim,
                rect = grid.rect(
                    x = xv,
                    y = yv,
                    width = wv,
                    height = hv,
                    gp = gpar(col = col, fill = NA, lwd = lwd)
                ),

                hline = grid.segments(
                    x0 = xv - wv * 0.5,
                    y0 = yv,
                    x1 = xv + wv * 0.5,
                    y1 = yv,
                    gp = gpar(col = col, lwd = lwd)
                ),

                vline = grid.segments(
                    x0 = xv,
                    y0 = yv - hv * 0.5,
                    x1 = xv,
                    y1 = yv + hv * 0.5,
                    gp = gpar(col = col, lwd = lwd)
                ),

                ldiag = grid.segments(
                    x0 = xv - wv * 0.5,
                    y0 = yv - hv * 0.5,
                    x1 = xv + wv * 0.5,
                    y1 = yv + hv * 0.5,
                    gp = gpar(col = col, lwd = lwd)
                ),

                rdiag = grid.segments(
                    x0 = xv - wv * 0.5,
                    y0 = yv + hv * 0.5,
                    x1 = xv + wv * 0.5,
                    y1 = yv - hv * 0.5,
                    gp = gpar(col = col, lwd = lwd)
                ),

                circle_gap = {
                    .r_mm <- min(
                        convertUnit(wv[1L], "mm", valueOnly = TRUE),
                        convertUnit(hv[1L], "mm", valueOnly = TRUE)
                    ) *
                        0.25
                    grid.circle(
                        x = xv,
                        y = yv,
                        r = unit(.r_mm, "mm"),
                        gp = gpar(col = col, fill = NA, lwd = lwd)
                    )
                },

                circle_full = {
                    .theta <- seq(0, 2 * pi, length.out = 65L)[seq_len(64L)]
                    .cos_t <- cos(.theta)
                    .sin_t <- sin(.theta)
                    for (k in seq_along(xv)) {
                        grid.polygon(
                            x = xv[k] + wv[k] * 0.5 * .cos_t,
                            y = yv[k] + hv[k] * 0.5 * .sin_t,
                            gp = gpar(
                                col = if (length(col) == 1L) col else col[k],
                                fill = NA,
                                lwd = if (length(lwd) == 1L) lwd else lwd[k]
                            )
                        )
                    }
                },

                diamond = {
                    n <- length(xv)
                    hw <- wv * 0.5
                    hh <- hv * 0.5
                    for (k in seq_len(n)) {
                        grid.polygon(
                            x = unit.c(
                                xv[k],
                                xv[k] + hw[k],
                                xv[k],
                                xv[k] - hw[k]
                            ),
                            y = unit.c(
                                yv[k] + hh[k],
                                yv[k],
                                yv[k] - hh[k],
                                yv[k]
                            ),
                            gp = gpar(col = col[k], fill = NA, lwd = lwd[k])
                        )
                    }
                },

                # octagon: 4 sides on the cell edges, 4 small corner cuts (cut factor = 0.2)
                # The corner cuts are closer to the actual corners than a regular octagon would be
                octagon = {
                    n <- length(xv)
                    hw <- wv * 0.5
                    hh <- hv * 0.5
                    f <- 0.5 # corner cut factor (0 = full rectangle, 0.5 = diamond)
                    for (k in seq_len(n)) {
                        cx <- xv[k]
                        cy <- yv[k]
                        dx <- hw[k] * f
                        dy <- hh[k] * f
                        grid.polygon(
                            x = unit.c(
                                cx - hw[k] + dx,
                                cx + hw[k] - dx, # top edge: left → right
                                cx + hw[k],
                                cx + hw[k], # right edge: top → bottom
                                cx + hw[k] - dx,
                                cx - hw[k] + dx, # bottom edge: right → left
                                cx - hw[k],
                                cx - hw[k] # left edge: bottom → top
                            ),
                            y = unit.c(
                                cy + hh[k],
                                cy + hh[k], # top edge
                                cy + hh[k] - dy,
                                cy - hh[k] + dy, # right edge
                                cy - hh[k],
                                cy - hh[k], # bottom edge
                                cy - hh[k] + dy,
                                cy + hh[k] - dy # left edge
                            ),
                            gp = gpar(col = col[k], fill = NA, lwd = lwd[k])
                        )
                    }
                }
            )
        }

        # ── Legend icon builder ─────────────────────────────────────────────
        .mark_legend_icon <- function(mark_str, col, lwd) {
            prims <- .parse_mark_type(mark_str)
            function(x, y, w, h, fill) {
                for (p in prims) {
                    .draw_mark_prim(p, x, y, w, h, col, lwd)
                }
            }
        }

        # ── Pre-scan for label legend entries ───────────────────────────────
        if (!identical(legend.position, "none")) {
            lgnd_seen <- list()
            for (ri in seq_len(nrow(hmargs$matrix))) {
                for (ci in seq_len(ncol(hmargs$matrix))) {
                    r <- .call_label(hmargs$matrix[ri, ci], ri, ci)
                    if (is.list(r) && !is.null(r$legend) && !is.na(r$legend)) {
                        key <- r$legend
                        if (is.null(lgnd_seen[[key]])) {
                            lbl_txt <- .extract_lbl(r)
                            lgnd_seen[[key]] <- list(
                                text = if (!is.na(lbl_txt)) lbl_txt else key,
                                color = r$color %||% label_color,
                                order = r$order %||% NA_integer_
                            )
                        }
                    }
                }
            }

            if (length(lgnd_seen) > 0) {
                orders <- sapply(lgnd_seen, function(e) {
                    e$order %||% NA_integer_
                })
                if (!all(is.na(orders))) {
                    orders[is.na(orders)] <- max(orders, na.rm = TRUE) +
                        seq_len(sum(is.na(orders)))
                    lgnd_seen <- lgnd_seen[order(orders)]
                }
                lgnd_graphics <- lapply(lgnd_seen, function(entry) {
                    txt <- entry$text
                    col <- entry$color
                    sz <- label_size
                    function(x, y, w, h, fill) {
                        grid.text(
                            txt,
                            x,
                            y,
                            gp = gpar(
                                fontsize = sz,
                                col = col,
                                fontface = "bold"
                            )
                        )
                    }
                })
                legends$.label <- ComplexHeatmap::Legend(
                    title = label_name,
                    labels = names(lgnd_seen),
                    graphics = lgnd_graphics,
                    direction = legend.direction
                )
            }

            # ── Pre-scan for mark legend entries ─────────────────────────────
            mkl_seen <- list()
            for (ri in seq_len(nrow(hmargs$matrix))) {
                for (ci in seq_len(ncol(hmargs$matrix))) {
                    r <- .call_mark(hmargs$matrix[ri, ci], ri, ci)
                    if (is.list(r) && !is.null(r$legend) && !is.na(r$legend)) {
                        key <- r$legend
                        if (is.null(mkl_seen[[key]])) {
                            mk_str <- .extract_mark(r)
                            mkl_seen[[key]] <- list(
                                mark = if (!is.na(mk_str)) mk_str else key,
                                color = r$color %||% mark_color,
                                size = r$size %||% mark_size,
                                order = r$order %||% NA_integer_
                            )
                        }
                    }
                }
            }

            if (length(mkl_seen) > 0) {
                orders <- sapply(mkl_seen, function(e) e$order %||% NA_integer_)
                if (!all(is.na(orders))) {
                    orders[is.na(orders)] <- max(orders, na.rm = TRUE) +
                        seq_len(sum(is.na(orders)))
                    mkl_seen <- mkl_seen[order(orders)]
                }
                mkl_graphics <- lapply(mkl_seen, function(entry) {
                    .mark_legend_icon(entry$mark, entry$color, entry$size)
                })
                legends$.mark <- ComplexHeatmap::Legend(
                    title = mark_name,
                    labels = names(mkl_seen),
                    graphics = mkl_graphics,
                    direction = legend.direction,
                    row_gap = unit(1, "mm")
                )
            }
        }

        # ── layer_fun: marks first (background), then labels (foreground) ───
        hmargs$layer_fun <- function(j, i, x, y, w, h, fill, sr, sc) {
            raw_vals <- ComplexHeatmap::pindex(hmargs$matrix, i, j)

            # Draw marks
            mk_results <- lapply(seq_along(i), function(k) {
                .call_mark(raw_vals[k], i[k], j[k])
            })
            marks_vec <- sapply(mk_results, .extract_mark)
            mk_colors <- sapply(mk_results, function(r) {
                if (is.list(r)) r$color %||% mark_color else mark_color
            })
            mk_sizes <- sapply(mk_results, function(r) {
                if (is.list(r)) r$size %||% mark_size else mark_size
            })

            unique_marks <- unique(marks_vec[!is.na(marks_vec)])
            for (mk in unique_marks) {
                km <- which(marks_vec == mk)
                prms <- .parse_mark_type(mk)
                for (prim in prms) {
                    .draw_mark_prim(
                        prim,
                        x[km],
                        y[km],
                        w[km],
                        h[km],
                        mk_colors[km],
                        mk_sizes[km]
                    )
                }
            }

            # Draw labels on top
            lbl_results <- lapply(seq_along(i), function(k) {
                .call_label(raw_vals[k], i[k], j[k])
            })
            lbl <- sapply(lbl_results, .extract_lbl)
            sizes <- sapply(lbl_results, function(r) {
                if (is.list(r)) r$size %||% label_size else label_size
            })
            colors <- sapply(lbl_results, function(r) {
                if (is.list(r)) r$color %||% label_color else label_color
            })

            inds <- which(!is.na(lbl))
            if (length(inds) > 0) {
                theta <- seq(pi / 8, 2 * pi, length.out = 16)
                for (k in inds) {
                    sz <- sizes[k]
                    lapply(theta, function(a) {
                        grid.text(
                            lbl[k],
                            x = x[k] + unit(cos(a) * sz / 30, "mm"),
                            y = y[k] + unit(sin(a) * sz / 30, "mm"),
                            gp = gpar(fontsize = sz, col = "white")
                        )
                    })
                    grid.text(
                        lbl[k],
                        x[k],
                        y[k],
                        gp = gpar(fontsize = sz, col = colors[k])
                    )
                }
            }

            if (is.function(layer_fun_callback)) {
                layer_fun_callback(j, i, x, y, w, h, fill, sr, sc)
            }
        }
        legends$.heatmap <- get_main_legend()
    }

    # Use actual matrix dimensions: split_by groups partition their axis items,
    # they don't multiply them. hmargs$matrix is already transposed when flip=TRUE.
    nrows <- nrow(hmargs$matrix)
    ncols <- ncol(hmargs$matrix)

    # ── Cell dimensions ──────────────────────────────────────────────────────
    # cell_w / cell_h: width and height of one cell in the *non-flipped* orientation
    # (inches).  cell_h = cell_w * effective_aspect_ratio.
    # Defaults are chosen so embedded sub-plots (violin, boxplot, pie) have enough
    # room, and bar cells are wider than they are tall.
    cell_w <- switch(
        cell_type,
        violin = 0.5,
        boxplot = 0.5,
        pie = 0.5,
        bars = 0.35,
        label = 0.6,
        mark = 0.25,
        `label+mark` = 0.6,
        0.25 # tile, dot
    )
    cell_w <- cell_w * base_size

    aspect.ratio <- aspect.ratio %||%
        switch(
            cell_type,
            violin = 2, # taller to accommodate violin shape
            boxplot = 2, # slightly taller to accommodate boxplot shape
            pie = 1, # square for pie charts
            bars = 0.5, # wider-than-tall for bars
            label = 0.6, # shorter to fit text better
            mark = 1, # square for marks
            `label+mark` = 0.6, # same as label
            1 # square by default for all other types
        )
    cell_h <- cell_w * aspect.ratio

    ncol_annos <- sum(cluster_columns, show_column_names) * 4
    ncol_annos <- ncol_annos +
        ifelse(
            is.null(columns_split_by) ||
                isFALSE(column_annotation_params[[columns_split_by]]),
            0,
            1
        ) +
        ifelse(!col_name_anno_enabled, 0, 1)
    res <- .setup_annos(
        which = "column",
        names_side = ifelse(flip, column_names_side, row_names_side),
        anno_title = column_title,
        show_names = show_column_names,
        annotation = column_annotation,
        annotation_type = column_annotation_type,
        annotation_side = column_annotation_side,
        annotation_palette = column_annotation_palette,
        annotation_palcolor = column_annotation_palcolor,
        annotation_agg = column_annotation_agg,
        annotation_params = column_annotation_params,
        annotation_name = column_annotation_name,
        split_by = columns_split_by,
        splits = if (flip) hmargs$row_split else hmargs$column_split,
        by = col_by_eff,
        by_labels = if (flip) hmargs$row_labels else hmargs$column_labels,
        flip = flip,
        legend.direction = legend.direction,
        legend.position = legend.position,
        data = data
    )
    column_annos <- res$annos
    legends <- c(legends, res$legends)
    # Dispatch annotations to sides based on per-annotation column_annotation_side
    all_keys <- names(column_annos)
    all_keys <- setdiff(
        all_keys,
        c("annotation_name_side", "show_annotation_name", "annotation_label")
    )
    builtin_keys <- unique(c(columns_split_by, columns_by))
    side_list <- column_annotation_side # already alias-resolved
    def_side <- if (is.list(side_list)) {
        side_list[[".default"]] %||% "top"
    } else {
        "top"
    }
    sna <- column_annos$show_annotation_name
    ala <- column_annos$annotation_label
    names_side <- column_annos$annotation_name_side
    top_annos <- list(
        annotation_name_side = names_side,
        show_annotation_name = list(),
        annotation_label = list()
    )
    bottom_annos <- list(
        annotation_name_side = names_side,
        show_annotation_name = list(),
        annotation_label = list()
    )
    for (k in all_keys) {
        if (is.list(side_list)) {
            side <- side_list[[k]] %||% def_side
        } else {
            side <- if (is.character(side_list)) side_list else def_side
        }
        if (side == "bottom") {
            bottom_annos[[k]] <- column_annos[[k]]
            if (!is.null(sna[[k]])) {
                bottom_annos$show_annotation_name[[k]] <- sna[[k]]
            }
            if (!is.null(ala[[k]])) {
                bottom_annos$annotation_label[[k]] <- ala[[k]]
            }
        } else {
            top_annos[[k]] <- column_annos[[k]]
            if (!is.null(sna[[k]])) {
                top_annos$show_annotation_name[[k]] <- sna[[k]]
            }
            if (!is.null(ala[[k]])) {
                top_annos$annotation_label[[k]] <- ala[[k]]
            }
        }
    }
    # Reorder: split annotation farthest from heatmap body, name annotation closest
    top_annos <- .reorder_anno_side(
        top_annos,
        col_by_eff,
        columns_split_by,
        side = "top"
    )
    bottom_annos <- .reorder_anno_side(
        bottom_annos,
        col_by_eff,
        columns_split_by,
        side = "bottom"
    )
    if (any(sapply(top_annos, inherits, "AnnotationFunction"))) {
        if (isTRUE(flip)) {
            hmargs$left_annotation <- do_call(
                ComplexHeatmap::rowAnnotation,
                top_annos
            )
        } else {
            hmargs$top_annotation <- do_call(
                ComplexHeatmap::HeatmapAnnotation,
                top_annos
            )
        }
    }
    if (any(sapply(bottom_annos, inherits, "AnnotationFunction"))) {
        if (isTRUE(flip)) {
            hmargs$right_annotation <- do_call(
                ComplexHeatmap::rowAnnotation,
                bottom_annos
            )
        } else {
            hmargs$bottom_annotation <- do_call(
                ComplexHeatmap::HeatmapAnnotation,
                bottom_annos
            )
        }
    }
    rm(top_annos, bottom_annos, column_annos)

    nrow_annos <- sum(cluster_rows, show_row_names) * 4
    nrow_annos <- nrow_annos +
        ifelse(
            is.null(rows_split_by) ||
                isFALSE(row_annotation_params[[rows_split_by]]),
            0,
            1
        ) +
        ifelse(!row_name_anno_enabled, 0, 1)
    res <- .setup_annos(
        which = "row",
        names_side = ifelse(flip, row_names_side, column_names_side),
        anno_title = row_title,
        show_names = show_row_names,
        annotation = row_annotation,
        annotation_type = row_annotation_type,
        annotation_side = row_annotation_side,
        annotation_palette = row_annotation_palette,
        annotation_palcolor = row_annotation_palcolor,
        annotation_agg = row_annotation_agg,
        annotation_params = row_annotation_params,
        annotation_name = row_annotation_name,
        split_by = rows_split_by,
        splits = if (flip) hmargs$column_split else hmargs$row_split,
        by = row_by_eff,
        by_labels = if (flip) hmargs$column_labels else hmargs$row_labels,
        flip = flip,
        legend.direction = legend.direction,
        legend.position = legend.position,
        data = data
    )
    row_annos <- res$annos
    legends <- c(legends, res$legends)
    # Dispatch annotations to sides based on per-annotation row_annotation_side
    all_keys <- names(row_annos)
    all_keys <- setdiff(
        all_keys,
        c("annotation_name_side", "show_annotation_name", "annotation_label")
    )
    side_list <- row_annotation_side # already alias-resolved
    def_side <- if (is.list(side_list)) {
        side_list[[".default"]] %||% "left"
    } else {
        "left"
    }
    sna <- row_annos$show_annotation_name
    ala <- row_annos$annotation_label
    names_side <- row_annos$annotation_name_side
    left_side <- list(
        annotation_name_side = names_side,
        show_annotation_name = list(),
        annotation_label = list()
    )
    right_side <- list(
        annotation_name_side = names_side,
        show_annotation_name = list(),
        annotation_label = list()
    )
    for (k in all_keys) {
        side <- if (is.list(side_list)) {
            side_list[[k]] %||% def_side
        } else if (is.character(side_list)) {
            side_list
        } else {
            def_side
        }
        if (side == "right") {
            right_side[[k]] <- row_annos[[k]]
            if (!is.null(sna[[k]])) {
                right_side$show_annotation_name[[k]] <- sna[[k]]
            }
            if (!is.null(ala[[k]])) {
                right_side$annotation_label[[k]] <- ala[[k]]
            }
        } else {
            left_side[[k]] <- row_annos[[k]]
            if (!is.null(sna[[k]])) {
                left_side$show_annotation_name[[k]] <- sna[[k]]
            }
            if (!is.null(ala[[k]])) {
                left_side$annotation_label[[k]] <- ala[[k]]
            }
        }
    }
    # Reorder: split annotation farthest from heatmap body, name annotation closest
    left_side <- .reorder_anno_side(
        left_side,
        row_by_eff,
        rows_split_by,
        side = "left"
    )
    right_side <- .reorder_anno_side(
        right_side,
        row_by_eff,
        rows_split_by,
        side = "bottom"
    )
    has_left <- any(sapply(left_side, inherits, "AnnotationFunction"))
    has_right <- any(sapply(right_side, inherits, "AnnotationFunction"))
    # Fix: when row annotations are on the right and row_names_side is "right",
    # ComplexHeatmap width() fails. Swap row_names_side to "left" to avoid the conflict.
    if (
        has_right &&
            !isTRUE(flip) &&
            hmargs$row_names_side == "right"
    ) {
        hmargs$row_names_side <- "left"
    }
    if (has_left) {
        if (isTRUE(flip)) {
            hmargs$top_annotation <- do_call(
                ComplexHeatmap::HeatmapAnnotation,
                left_side
            )
        } else {
            hmargs$left_annotation <- do_call(
                ComplexHeatmap::rowAnnotation,
                left_side
            )
        }
    }
    if (has_right) {
        if (isTRUE(flip)) {
            hmargs$bottom_annotation <- do_call(
                ComplexHeatmap::HeatmapAnnotation,
                right_side
            )
        } else {
            hmargs$right_annotation <- do_call(
                ComplexHeatmap::rowAnnotation,
                right_side
            )
        }
    }
    rm(row_annos, left_side, right_side, has_left, has_right)

    ## Fix for ComplexHeatmap annotation name / legend overlap bug:
    ## When show_row_names = FALSE but annotations have names on the right side
    ## (annotation_name_side = "right"), ComplexHeatmap does not allocate space for those
    ## annotation name labels, causing them to overlap the right-side legend.
    ## - flip = FALSE: column annotations (top_annotation) have names on row_names_side = "right"
    ## - flip = TRUE:  row annotations (top_annotation, from left_annos) have names on
    ##                 row_names_side = "right"; hmargs$row_names_side = flip_side(column_names_side)
    ## In both cases the offending names appear on the right, so inject a phantom invisible
    ## right annotation to reserve the required width.
    phantom_right_width_in <- 0
    if (
        !isTRUE(hmargs$show_row_names) &&
            hmargs$row_names_side == "right" &&
            legend.position == "right"
    ) {
        right_anno_names <- character(0)
        if (!isTRUE(flip)) {
            if (col_name_anno_enabled) {
                right_anno_names <- c(right_anno_names, col_by_eff)
            }
            if (
                !is.null(columns_split_by) &&
                    !isFALSE(column_annotation_params[[columns_split_by]])
            ) {
                right_anno_names <- c(right_anno_names, columns_split_by)
            }
            if (!is.null(column_annotation) && length(column_annotation) > 0) {
                col_anno_names <- if (is.list(column_annotation)) {
                    names(column_annotation)
                } else {
                    as.character(column_annotation)
                }
                right_anno_names <- c(right_anno_names, col_anno_names)
            }
        } else {
            if (row_name_anno_enabled) {
                right_anno_names <- c(right_anno_names, row_by_eff)
            }
            if (
                !is.null(rows_split_by) &&
                    !isFALSE(row_annotation_params[[rows_split_by]])
            ) {
                right_anno_names <- c(right_anno_names, rows_split_by)
            }
            if (!is.null(row_annotation) && length(row_annotation) > 0) {
                row_anno_names <- if (is.list(row_annotation)) {
                    names(row_annotation)
                } else {
                    as.character(row_annotation)
                }
                right_anno_names <- c(right_anno_names, row_anno_names)
            }
        }
        if (length(right_anno_names) > 0) {
            phantom_width <- ComplexHeatmap::max_text_width(right_anno_names)
            phantom_right_width_in <- convertUnit(
                phantom_width,
                "inches",
                valueOnly = TRUE
            )
            phantom_anno <- ComplexHeatmap::rowAnnotation(
                .gap = ComplexHeatmap::anno_empty(
                    border = FALSE,
                    width = phantom_width
                ),
                show_annotation_name = FALSE
            )
            if (is.null(hmargs$right_annotation)) {
                hmargs$right_annotation <- phantom_anno
            } else {
                hmargs$right_annotation <- hmargs$right_annotation +
                    phantom_anno
            }
        }
    }

    ## Set up the heatmap dimensions
    # Row names appear on left/right and add to width; only count when show_row_names is TRUE.
    # hmargs$show_row_names already accounts for flip (equals original show_column_names when
    # flip=TRUE, since original column labels become the row labels in the transposed matrix).
    # hmargs$row_names_max_width likewise accounts for flip (uses columns_by labels when flip=TRUE).
    rownames_width <- if (isTRUE(hmargs$show_row_names)) {
        max(
            convertUnit(
                hmargs$row_names_max_width,
                "inches",
                valueOnly = TRUE
            ) *
                0.5 -
                0.2,
            0
        )
    } else {
        0
    }

    # Estimate the dimension contribution of the original column labels.
    # Column labels are rendered rotated (vertical text), so their display height is proportional
    # to their text width. When flip=TRUE the matrix is transposed, so original column labels
    # end up as hmargs row labels (left/right) — but colnames_height lives inside col_overhead
    # which is routed to WIDTH in the flip case, so the geometry is still correct.
    # Always gate on the original show_column_names param (not hmargs$show_column_names, which
    # is swapped to show_row_names when flip=TRUE) to match the col_overhead accounting.
    # Use rownames(hmargs$matrix) when flip=TRUE because t() moved the original column labels
    # to row positions; use colnames(hmargs$matrix) otherwise.
    colnames_height <- if (isTRUE(show_column_names)) {
        orig_col_labels <- if (isTRUE(flip)) {
            rownames(hmargs$matrix)
        } else {
            colnames(hmargs$matrix)
        }
        col_max_width <- ComplexHeatmap::max_text_width(orig_col_labels)
        convertUnit(col_max_width, "inches", valueOnly = TRUE) * 0.25
    } else {
        0
    }

    # Annotation overhead per side:
    #   - row-side  (left/right): dendrogram + per-track bars → adds to width
    #   - col-side  (top/bottom): dendrogram + per-track bars → adds to height
    # nrow_annos / ncol_annos include: (cluster_* + show_*_names)*4 + n_splits + n_name_annos
    # We strip out the show_*_names contribution (already captured by rownames_width /
    # colnames_height above) and reduce the per-item coefficient to avoid double-counting.
    row_overhead <- (if (isTRUE(cluster_rows)) 0.5 else 0) +
        (nrow_annos - show_row_names * 4) * 0.15 +
        phantom_right_width_in
    col_overhead <- (if (isTRUE(cluster_columns)) 0.5 else 0) +
        (ncol_annos - show_column_names * 4) * 0.15 +
        colnames_height

    if (isTRUE(flip)) {
        body_width <- ncols * cell_h
        body_height <- nrows * cell_w
    } else {
        body_width <- ncols * cell_w
        body_height <- nrows * cell_h
    }

    padding <- if (inherits(padding, "unit")) padding else unit(padding, "mm")
    if (length(padding) == 1) {
        padding <- rep(padding, 4)
    } else if (length(padding) == 2) {
        # padding[1] -> top/bottom, padding[2] -> left/right
        padding <- rep(padding, 2)
    } else if (length(padding) == 3) {
        # padding[1] -> top, padding[2] -> left/right, padding[3] -> bottom
        padding <- c(padding, padding[2])
    }
    if (isTRUE(flip)) {
        width <- body_width +
            col_overhead +
            rownames_width +
            convertUnit(sum(padding[c(1, 3)]), "inches", valueOnly = TRUE)
        height <- body_height +
            row_overhead +
            convertUnit(sum(padding[c(2, 4)]), "inches", valueOnly = TRUE)
    } else {
        width <- body_width +
            row_overhead +
            rownames_width +
            convertUnit(sum(padding[c(2, 4)]), "inches", valueOnly = TRUE)
        height <- body_height +
            col_overhead +
            convertUnit(sum(padding[c(1, 3)]), "inches", valueOnly = TRUE)
    }
    # make padding from top, right, bottom, left to match the order in ComplexHeatmap::draw()
    # which is bottom, left, top and right.
    padding <- padding[c(3, 4, 1, 2)]

    # ── Precise legend size contribution (mirrors calculate_plot_dimensions) ──
    # Collect all candidate legend label strings to estimate max label width.
    # Sources:
    #   (a) main heatmap legend  – discrete item names, or formatted cutoff values
    #   (b) name annotations – row/column labels shown as legends when show_*_names=FALSE,
    #       and split annotations (rows_split_by / columns_split_by) which always show legends
    #   (c) extra row / column annotation legends
    .legend_label_cands <- character(0)
    if (isTRUE(legend_discrete) && !is.null(legend_items)) {
        .legend_label_cands <- c(.legend_label_cands, names(legend_items))
    } else {
        .legend_label_cands <- c(
            .legend_label_cands,
            formatC(c(lower_cutoff, upper_cutoff), digits = 3, format = "g")
        )
    }
    # Helper: extract label strings from a column of `data`
    .col_labels <- function(col) {
        if (is.null(col) || !col %in% colnames(data)) {
            return(character(0))
        }
        as.character(
            if (is.factor(data[[col]])) {
                levels(data[[col]])
            } else {
                unique(data[[col]])
            }
        )
    }
    # Name-annotation legends:
    #   – rows_by shows a legend when show_row_names=FALSE (and row name annotation is enabled)
    #   – columns_by shows a legend when show_column_names=FALSE (and col name annotation is enabled)
    #   – split_by annotations always show legends when present
    if (!isTRUE(show_row_names) && row_name_anno_enabled) {
        .legend_label_cands <- c(.legend_label_cands, .col_labels(row_by_eff))
    }
    if (!isTRUE(show_column_names) && col_name_anno_enabled) {
        .legend_label_cands <- c(.legend_label_cands, .col_labels(col_by_eff))
    }
    .legend_label_cands <- c(
        .legend_label_cands,
        .col_labels(rows_split_by),
        .col_labels(columns_split_by)
    )
    # Extra row / column annotation legends
    .scan_anno_labels <- function(anno) {
        if (is.null(anno)) {
            return(character(0))
        }
        cols <- if (is.list(anno)) unlist(anno) else as.character(anno)
        unlist(lapply(cols[cols %in% colnames(data)], .col_labels))
    }
    .legend_label_cands <- c(
        .legend_label_cands,
        .scan_anno_labels(row_annotation),
        .scan_anno_labels(column_annotation)
    )
    legend_nchar <- if (length(.legend_label_cands) > 0) {
        max(nchar(.legend_label_cands), na.rm = TRUE)
    } else {
        5L
    }
    # Number of distinct legend blocks rendered by ComplexHeatmap
    legend_n <- length(Filter(Negate(is.null), legends))

    # Use the same per-character metrics as calculate_plot_dimensions()
    legend_key_w <- 0.30 # key swatch width + internal margin
    legend_char_w <- 0.07 # inches per character of label text
    legend_row_h <- 0.30 # height of one stacked legend block
    legend_pad <- 0.35 # outer margin + optional title

    if (!identical(legend.position, "none")) {
        if (legend.position %in% c("right", "left")) {
            # Vertical panel on the side; width driven by label length.
            # When direction=="horizontal" the colorbar is rendered horizontally
            # (wider), so add an extra 0.5 in to account for the larger key.
            legend_extra <- if (legend.direction == "horizontal") 0.5 else 0
            legend_width <- max(
                1.0,
                legend_key_w + legend_nchar * legend_char_w
            ) +
                legend_extra
            width <- width + legend_width
        } else if (legend.direction == "horizontal") {
            # Each legend block goes on its own row at top / bottom.
            n_legend_rows <- max(1L, legend_n)
            legend_height <- max(1.0, legend_pad + n_legend_rows * legend_row_h)
            height <- height + legend_height
        } else {
            # Vertical blocks at top / bottom: add a legend-width column.
            legend_width <- max(
                1.0,
                legend_key_w + legend_nchar * legend_char_w
            )
            width <- width + legend_width
        }
    }
    # Fix body dimensions so ComplexHeatmap honours aspect.ratio regardless of canvas size.
    # Only set when the caller has not already provided explicit width/height via `...`.
    # Do NOT set the width and height so that the plot won't be truncated due to the device size limit; instead, we will set the width and height attributes on the returned object so that the downstream display method can choose how to handle it (e.g. scaling to fit the device).
    # if (is.null(hmargs$width)) {
    #     hmargs$width  <- unit(body_width,  "inches")
    # }
    # if (is.null(hmargs$height)) {
    #     hmargs$height <- unit(body_height, "inches")
    # }
    # Proportional column widths for bars: ComplexHeatmap ignores per-column
    # width vectors in the drawing, so replicate each logical column k_j
    # times and make each logical column its own slice (slice widths are
    # proportional to column counts per slice).  Uniform counts -> no
    # replication -> the existing code path is untouched.
    bars_mat_logical <- NULL
    if (cell_type == "bars") {
        user_split <- if (isTRUE(flip)) hmargs$row_split else hmargs$column_split
        if (is.null(user_split)) {
            n_orig <- if (isTRUE(flip)) {
                nrow(hmargs$matrix)
            } else {
                ncol(hmargs$matrix)
            }
            # per-logical-column bar counts (bars_data is keyed by
            # "matrix_row-matrix_col" indices of the original matrix)
            counts <- if (isTRUE(flip)) {
                vapply(seq_len(n_orig), function(i) {
                    sum(lengths(bars_data[startsWith(
                        names(bars_data), paste0(i, "-")
                    )]))
                }, integer(1))
            } else {
                vapply(seq_len(n_orig), function(j) {
                    sum(lengths(bars_data[endsWith(
                        names(bars_data), paste0("-", j)
                    )]))
                }, integer(1))
            }
            if (n_orig > 1 && all(counts > 0)) {
                gcd2 <- function(a, b) if (b == 0) a else gcd2(b, a %% b)
                k <- counts / Reduce(gcd2, counts)
                # ponytail: cap total columns for pathological count
                # spreads (e.g. 10000:1); raise if real data needs it
                if (sum(k) > 300) {
                    k <- pmax(1, round(k * 300 / sum(k)))
                }
                if (any(k > 1)) {
                    bars_mat_logical <- hmargs$matrix
                    idx <- rep(seq_len(n_orig), k)
                    jmap <- idx
                    bars_names_labs <- NULL
                    if (isTRUE(flip)) {
                        # original columns are matrix rows after flip
                        # (subsetting replicates rownames automatically)
                        hmargs$matrix <- hmargs$matrix[idx, , drop = FALSE]
                        hmargs$row_split <- factor(rep(seq_len(n_orig), k))
                        if (!is.null(hmargs$row_labels)) {
                            bars_names_labs <- as.character(hmargs$row_labels)
                            # keep the label at the middle sub-row so the
                            # name is centered on the slice
                            labs <- rep("", sum(k))
                            for (jj in seq_len(n_orig)) {
                                pos <- which(idx == jj)
                                labs[pos[ceiling(length(pos) / 2)]] <-
                                    bars_names_labs[jj]
                            }
                            hmargs$row_labels <- labs
                        }
                        hmargs$cluster_rows <- FALSE
                        if (!is.null(hmargs$left_annotation)) {
                            hmargs$left_annotation <- hmargs$left_annotation[idx]
                        }
                        if (!is.null(hmargs$right_annotation)) {
                            hmargs$right_annotation <- hmargs$right_annotation[idx]
                        }
                    } else {
                        # (subsetting replicates colnames automatically)
                        hmargs$matrix <- hmargs$matrix[, idx, drop = FALSE]
                        hmargs$column_split <- factor(rep(seq_len(n_orig), k))
                        if (!is.null(hmargs$column_labels)) {
                            bars_names_labs <- as.character(hmargs$column_labels)
                            # keep the label at the middle sub-column so the
                            # name is centered on the slice
                            labs <- rep("", sum(k))
                            for (jj in seq_len(n_orig)) {
                                pos <- which(idx == jj)
                                labs[pos[ceiling(length(pos) / 2)]] <-
                                    bars_names_labs[jj]
                            }
                            hmargs$column_labels <- labs
                        }
                        hmargs$cluster_columns <- FALSE
                        if (!is.null(hmargs$top_annotation)) {
                            hmargs$top_annotation <- hmargs$top_annotation[idx]
                        }
                        if (!is.null(hmargs$bottom_annotation)) {
                            hmargs$bottom_annotation <- hmargs$bottom_annotation[idx]
                        }
                    }
                    # Centered inplace names: slice titles are centered on
                    # their slice exactly (unlike column names, which always
                    # sit on a sub-column), so draw the names as titles on
                    # the names side, styled like the names
                    if (!is.null(bars_names_labs)) {
                        if (isTRUE(flip) && isTRUE(hmargs$show_row_names) &&
                            is.null(hmargs$row_title)) {
                            hmargs$row_title <- bars_names_labs
                            hmargs$row_title_side <- flip_side(column_names_side)
                            hmargs$row_title_gp <- hmargs$row_names_gp %||%
                                gpar(fontsize = 12)
                            hmargs$show_row_names <- FALSE
                        } else if (!isTRUE(flip) &&
                            isTRUE(hmargs$show_column_names) &&
                            is.null(hmargs$column_title)) {
                            hmargs$column_title <- bars_names_labs
                            hmargs$column_title_side <- column_names_side
                            hmargs$column_title_gp <- hmargs$column_names_gp %||%
                                gpar(fontsize = 12)
                            hmargs$show_column_names <- FALSE
                        }
                    }
                    # merged bar drawing: layer_fun is called once per slice
                    # (= logical column) with slice-relative npc coordinates,
                    # so each row's bars span the full slice width/height
                    hmargs$layer_fun <- if (isTRUE(flip)) {
                        function(j, i, x, y, w, h, fill, sr, sc) {
                            layer_white_bg(j, i, x, y, w, h, fill)
                            for (col in unique(j)) {
                                m <- which(j == col)
                                vals <- bars_data[[paste(
                                    col, jmap[i[m[1]]], sep = "-"
                                )]]
                                if (is.null(vals) || length(vals) == 0) next
                                ns <- length(vals)
                                grid.rect(
                                    x = x[m[1]], y = unit(0.5, "npc"),
                                    width = w[m[1]], height = unit(1, "npc"),
                                    gp = gpar(fill = "white", col = "white")
                                )
                                grid.rect(
                                    x = x[m[1]],
                                    y = unit((seq_len(ns) - 0.5) / ns, "npc"),
                                    width = w[m[1]],
                                    height = unit(rep(1 / ns, ns), "npc"),
                                    gp = gpar(
                                        fill = hmargs$col(vals),
                                        col = "transparent"
                                    )
                                )
                            }
                            if (is.function(layer_fun_callback)) {
                                layer_fun_callback(j, i, x, y, w, h, fill, sr, sc)
                            }
                        }
                    } else {
                        function(j, i, x, y, w, h, fill, sr, sc) {
                            layer_white_bg(j, i, x, y, w, h, fill)
                            for (row in unique(i)) {
                                m <- which(i == row)
                                vals <- bars_data[[paste(
                                    row, jmap[j[m[1]]], sep = "-"
                                )]]
                                if (is.null(vals) || length(vals) == 0) next
                                ns <- length(vals)
                                grid.rect(
                                    x = unit(0.5, "npc"), y = y[m[1]],
                                    width = unit(1, "npc"), height = h[m[1]],
                                    gp = gpar(fill = "white", col = "white")
                                )
                                grid.rect(
                                    x = unit((seq_len(ns) - 0.5) / ns, "npc"),
                                    y = y[m[1]],
                                    width = unit(rep(1 / ns, ns), "npc"),
                                    height = h[m[1]],
                                    gp = gpar(
                                        fill = hmargs$col(vals),
                                        col = "transparent"
                                    )
                                )
                            }
                            if (is.function(layer_fun_callback)) {
                                layer_fun_callback(j, i, x, y, w, h, fill, sr, sc)
                            }
                        }
                    }
                }
            }
        } else {
            warning(
                "[Heatmap] Proportional column widths for cell_type = \"bars\" ",
                "are skipped when 'columns_split_by' is given.",
                call. = FALSE
            )
        }
    }
    unknown_args <- setdiff(
        names(hmargs),
        methods::formalArgs(ComplexHeatmap::Heatmap)
    )
    if (length(unknown_args) > 0) {
        warning(
            "[Heatmap] Unknown arguments to ComplexHeatmap::Heatmap(): ",
            paste(unknown_args, collapse = ", ")
        )
        hmargs <- hmargs[setdiff(names(hmargs), unknown_args)]
    }
    p <- do_call(ComplexHeatmap::Heatmap, hmargs)
    # Move label/mark legends to the end so they appear last
    if (!is.null(legends$.label)) {
        legends <- c(legends[names(legends) != ".label"], legends[".label"])
    }
    if (!is.null(legends$.mark)) {
        legends <- c(legends[names(legends) != ".mark"], legends[".mark"])
    }
    if (return_ht) {
        p <- ComplexHeatmap::prepare(p)
        attr(p, "legends") <- legends
        return(p)
    }

    mat <- bars_mat_logical %||% p@matrix
    draw_args_fixed <- list(
        annotation_legend_list = legends,
        padding = draw_opts$padding %||% padding,
        column_title = title,
        align_heatmap_legend = draw_opts$align_heatmap_legend %||%
            "heatmap_center",
        align_annotation_legend = draw_opts$align_annotation_legend %||%
            "heatmap_center"
    )
    if (identical(legend.position, "none")) {
        draw_args_fixed$show_annotation_legend <- draw_opts$show_annotation_legend %||%
            FALSE
        draw_args_fixed$show_heatmap_legend <- draw_opts$show_heatmap_legend %||%
            FALSE
    } else {
        draw_args_fixed$annotation_legend_side <- draw_opts$annotation_legend_side %||%
            legend.position
        draw_args_fixed$heatmap_legend_side <- draw_opts$heatmap_legend_side %||%
            legend.position
    }
    draw_args <- utils::modifyList(draw_opts, draw_args_fixed)

    p <- grid::grid.grabExpr(do_call(
        ComplexHeatmap::draw,
        c(list(p), draw_args)
    ))
    p <- patchwork::wrap_plots(p)

    min_size_in <- 4
    max_size_in <- 64
    attr(p, "height") <- max(min(height, max_size_in), min_size_in)
    attr(p, "width") <- max(min(width, max_size_in), min_size_in)

    # keep the ratio
    ratio <- height / width
    if (ratio > 1) {
        if (attr(p, "height") == max_size_in) {
            attr(p, "width") <- attr(p, "height") / ratio
        } else if (attr(p, "width") == min_size_in) {
            attr(p, "height") <- attr(p, "width") * ratio
        }
    } else if (ratio < 1) {
        if (attr(p, "width") == max_size_in) {
            attr(p, "height") <- attr(p, "width") * ratio
        } else if (attr(p, "height") == min_size_in) {
            attr(p, "width") <- attr(p, "height") / ratio
        }
    }
    p$data <- as.data.frame(mat)

    p
}

#' Heatmap
#'
#' @description
#' Draw a heatmap to visualise data in matrix form.  This is the public,
#' exported interface — it accepts data in multiple input formats (matrix,
#' wide, or long), preprocesses it via \code{\link{process_heatmap_data}},
#' and delegates to \code{\link{HeatmapAtomic}} for rendering.  Commonly
#' used in biology to visualise gene expression, but applicable to any
#' matrix-structured data.
#'
#' @section Input formats:
#' The \code{in_form} parameter controls how the input \code{data} is
#' interpreted:
#' \itemize{
#'   \item \code{"auto"} (default) — detects the format automatically.
#'   \item \code{"matrix"} — \code{data} is a matrix with row and column
#'         names.  It is melted to long form internally.
#'   \item \code{"wide-rows"} — each row is a feature, columns are samples.
#'   \item \code{"wide-columns"} — each column is a feature, rows are
#'         samples.
#'   \item \code{"long"} — tidy/long format with one observation per row.
#' }
#'
#' @section Split-by support:
#' When \code{split_by} is provided, the data is partitioned into subsets
#' and an independent heatmap is produced for each level.  Results are
#' combined via \code{\link[patchwork]{wrap_plots}} according to
#' \code{nrow}, \code{ncol}, \code{byrow}, and \code{design}.  Per-split
#' \code{palette}, \code{palcolor}, \code{legend.position}, and
#' \code{legend.direction} can be specified as named lists keyed by split
#' level.
#'
#' @inheritParams HeatmapAtomic
#' @inheritParams process_heatmap_data
#' @inheritParams common_args
#' @param data A data frame or matrix.  When a matrix, it is melted to
#'  long format internally (requires row and column names).
#' @param ... Additional arguments passed to
#'  \code{\link{HeatmapAtomic}}, which in turn forwards them to
#'  \code{\link[ComplexHeatmap]{Heatmap}}.
#' @return A \code{patchwork} object (class \code{wrap_plots}) with
#'  \code{height} and \code{width} attributes (in inches).  When
#'  \code{combine = FALSE}, a named list of such objects, one per
#'  \code{split_by} level.
#' @export
#' @importFrom patchwork wrap_plots
#' @seealso \code{\link{HeatmapAtomic}}, \code{\link{LinkedHeatmap}},
#'  \code{\link{anno_simple}}, \code{\link{anno_points}},
#'  \code{\link{anno_lines}}, \code{\link{anno_pie}},
#'  \code{\link{anno_violin}}, \code{\link{anno_boxplot}},
#'  \code{\link{anno_density}}
#' @examples
#' \donttest{
#' set.seed(8525)
#'
#' matrix_data <- matrix(rnorm(60), nrow = 6, ncol = 10)
#' rownames(matrix_data) <- paste0("R", 1:6)
#' colnames(matrix_data) <- paste0("C", 1:10)
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     Heatmap(matrix_data)
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # use a different color palette
#'     # change the main legend title
#'     # show row names (legend will be hidden)
#'     # show column names
#'     # change the row name annotation name and side
#'     # change the column name annotation name
#'     Heatmap(matrix_data, palette = "viridis", values_by = "z-score",
#'        show_row_names = TRUE, show_column_names = TRUE,
#'        rows_name = "Features", row_names_side = "left",
#'        columns_name = "Samples")
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # flip the heatmap
#'     Heatmap(matrix_data, palette = "viridis", values_by = "z-score",
#'        show_row_names = TRUE, show_column_names = TRUE,
#'        rows_name = "Features", row_names_side = "left",
#'        columns_name = "Samples", flip = TRUE)
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # add annotations to the heatmap
#'     rows_data <- data.frame(
#'        rows = paste0("R", 1:6),
#'        group = sample(c("X", "Y", "Z"), 6, replace = TRUE)
#'     )
#'     Heatmap(matrix_data, rows_data = rows_data,
#'         row_annotation = list(Group = list(col = "group", palette = "Spectral"))
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     Heatmap(matrix_data, rows_data = rows_data,
#'         rows_split_by = "group"
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # use label annotation for split groups (shows group labels inside colored blocks)
#'     Heatmap(matrix_data, rows_data = rows_data,
#'         rows_split_by = "group",
#'         row_annotation = list(.row.split = list(
#'             type = "label",
#'             params = list(
#'                 border = FALSE,
#'                 labels_gp = grid::gpar(col = "white", fontsize = 12),
#'                 labels_rot = 0
#'             )
#'         ))
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # label annotation for column splits
#'     columns_data <- data.frame(
#'         columns = paste0("C", 1:10),
#'         batch = rep(c("A", "B"), each = 5)
#'     )
#'     Heatmap(matrix_data, columns_data = columns_data,
#'         columns_split_by = "batch",
#'         column_annotation = list(.col.split = list(type = "label"))
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # "rownames" annotation for row splits: show the concatenated row names
#'     # of each split group, wrapped every 3 names (row names and the split
#'     # title are hidden/shown by default accordingly)
#'     Heatmap(matrix_data, rows_data = rows_data,
#'         rows_split_by = "group",
#'         row_annotation = list(.row.split = list(
#'             type = "rownames",
#'             params = list(sep = ", ", wrap_by = 3)
#'         ))
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # same for column splits with type = "colnames"
#'     Heatmap(matrix_data, columns_data = columns_data,
#'         columns_split_by = "batch",
#'         column_annotation = list(.col.split = list(
#'             type = "colnames",
#'             params = list(wrap_by = 3)
#'         ))
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # split title modes: per-slice titles plus the split annotation
#'     # legend; and "anno" mode using label blocks instead of titles
#'     Heatmap(matrix_data, rows_data = rows_data,
#'         rows_split_by = "group",
#'         row_title = c("inplace", "legend")
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     Heatmap(matrix_data, rows_data = rows_data,
#'         rows_split_by = "group",
#'         row_title = "anno"
#'     )
#' }
#' rownames(matrix_data)[1] <- "R12345"
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # label annotation for name annotations: show row/column names as colored labels
#'     Heatmap(matrix_data, rows_data = rows_data,
#'         row_annotation = list(.row = list(
#'             type = "label", palette = "Set2", side = "right",
#'             params = list(labels_rot = 150)
#'         )),
#'         column_annotation = list(.col = list(
#'             type = "label", params = list(labels_rot = 90)
#'         ))
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # add labels to the heatmap
#'     Heatmap(matrix_data, rows_data = rows_data,
#'         rows_split_by = "group", cell_type = "label",
#'         base_size = 0.8,
#'         label = function(x) ifelse(
#'             x > 0, scales::number(x, accuracy = 0.01), NA
#'         )
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # add labels based on an external data
#'     pvalues <- matrix(runif(60, 0, 0.5), nrow = 6, ncol = 10)
#'     Heatmap(matrix_data, rows_data = rows_data,
#'         rows_split_by = "group", cell_type = "label",
#'         base_size = 0.8,
#'         label = function(x, i, j) {
#'             pv <- ComplexHeatmap::pindex(pvalues, i, j)
#'             ifelse(pv < 0.01, "***",
#'             ifelse(pv < 0.05, "**",
#'             ifelse(pv < 0.1, "*", NA)))
#'         }
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # Set label color, size, legend and order
#'     pvalues <- matrix(runif(60, 0, 0.5), nrow = 6, ncol = 10)
#'     Heatmap(matrix_data, rows_data = rows_data,
#'         rows_split_by = "group", cell_type = "label",
#'         base_size = 0.6,
#'         label_name = "Significance",
#'         label = function(x, i, j) {
#'             pv <- ComplexHeatmap::pindex(pvalues, i, j)
#'             if (pv < 0.01)
#'                list("***", color = "red", size = 12, legend = "p < 0.01", order = 1)
#'             else if (pv < 0.05)
#'                list("**", color = "orange", size = 10, legend = "p < 0.05", order = 3)
#'             else if (pv < 0.1)
#'                list("*", color = "yellow", size = 8, legend = "p < 0.1", order = 2)
#'             else NA
#'         }
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # add marks
#'     Heatmap(matrix_data, rows_data = rows_data,
#'         rows_split_by = "group", cell_type = "mark",
#'         mark = function(x, i, j) {
#'             pv <- ComplexHeatmap::pindex(pvalues, i, j)
#'             if(pv < 0.01) list("[x]", legend = "p < 0.01")
#'             else if (pv < 0.02) list("[o]", legend = "p < 0.02")
#'             else if (pv < 0.03) list("[-]", legend = "p < 0.03")
#'             else if (pv < 0.05) list("[()]", legend = "p < 0.05")
#'             else if (pv < 0.06) list("+", legend = "p < 0.06")
#'             else if (pv < 0.07) list("x", legend = "p < 0.07")
#'             else if (pv < 0.08) list("[/]", legend = "p < 0.08")
#'             else if (pv < 0.09) list("[\\]", legend = "p < 0.09")
#'             else NA
#'         }
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # add labels and marks
#'     Heatmap(matrix_data, rows_data = rows_data,
#'         rows_split_by = "group", cell_type = "mark+label",
#'         label = scales::label_number(accuracy = 0.01),
#'         mark = function(x, i, j) {
#'             pv <- ComplexHeatmap::pindex(pvalues, i, j)
#'             if(pv < 0.01) list("{}", legend = "p < 0.01")
#'             else if(pv < 0.05) list("[]", legend = "p < 0.05")
#'             else NA
#'         },
#'         mark_size = 1.5, mark_color = "red"
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # quickly simulate a GO board
#'     go <- matrix(sample(c(0, 1, NA), 81, replace = TRUE), ncol = 9)
#'
#'     Heatmap(
#'         go,
#'         # Do not cluster rows and columns and hide the name annotations
#'         # Use .row/.col aliases to disable the built-in name annotations
#'         cluster_rows = FALSE, cluster_columns = FALSE,
#'         row_annotation = list(.row = list(params = FALSE)),
#'         column_annotation = list(.col = list(params = FALSE)),
#'         show_row_names = FALSE, show_column_names = FALSE,
#'         # Set the legend items
#'         values_by = "Players", legend_discrete = TRUE,
#'         legend_items = c("Player 1" = 0, "Player 2" = 1),
#'         # Set the pawns
#'         cell_type = "dot", dot_size = function(x) ifelse(is.na(x), 0, 10),
#'         dot_size_name = NULL,  # hide the dot size legend
#'         palcolor = c("white", "black"),
#'         # Set the board
#'         add_reticle = TRUE,
#'         # Set the size of the board
#'         width = ggplot2::unit(105, "mm"), height = ggplot2::unit(105, "mm"))
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # Make the row/column name annotation thicker using the .row/.col aliases
#'     Heatmap(matrix_data,
#'         column_annotation = list(.col = list(params = list(height = 5))),
#'         row_annotation = list(.row = list(params = list(width = 5))))
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # Per-annotation side control: row name annotation on the right,
#'     # all other row annotations on the left (.default)
#'     rows_data2 <- data.frame(
#'         rows = sample(paste0("R", 1:6), 60, replace = TRUE),
#'         group = sample(c("X", "Y"), 60, replace = TRUE),
#'         score = runif(60)
#'     )
#'     Heatmap(matrix_data, rows_data = rows_data2,
#'         rows_split_by = "group",
#'         row_annotation = list(
#'             .default = list(side = "left"),
#'             .row = list(side = "right"),
#'             Score = "score"
#'         ),
#'         show_row_names = TRUE
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # Move all row annotations to the right side
#'     Heatmap(matrix_data, rows_data = rows_data2,
#'         rows_split_by = "group",
#'         row_annotation = list(
#'             .default = list(side = "right"),
#'             Score = "score"
#'         ),
#'         show_row_names = TRUE
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # Split and name annotations on opposite sides:
#'     # split annotation on the default left, name annotation on the right
#'     Heatmap(matrix_data, rows_data = rows_data2,
#'         rows_split_by = "group",
#'         row_annotation = list(
#'             .default = list(side = "left"),
#'             .row = list(side = "right")
#'         ),
#'         show_row_names = TRUE
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # Row name label annotation on the right side (text rotated 90° clockwise)
#'     Heatmap(matrix_data, rows_data = rows_data2,
#'         row_annotation = list(.row = list(
#'             type = "label", palette = "Set2", side = "right"
#'         )),
#'         show_row_names = TRUE
#'     )
#' }
#'
#' # Use long form data
#' N <- 500
#' data <- data.frame(
#'     value = rnorm(N),
#'     c = sample(letters[1:8], N, replace = TRUE),
#'     r = sample(LETTERS[1:5], N, replace = TRUE),
#'     p = sample(c("x", "y"), N, replace = TRUE),
#'     q = sample(c("X", "Y", "Z"), N, replace = TRUE),
#'     a = as.character(sample(1:5, N, replace = TRUE)),
#'     p1 = runif(N),
#'     p2 = runif(N)
#' )
#'
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     Heatmap(data, rows_by = "r", columns_by = "c", values_by = "value",
#'         rows_split_by = "p", columns_split_by = "q", show_column_names = TRUE)
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # display modes: in-place row names plus the row group annotation
#'     # legend; and "anno" mode showing the column names as text labels
#'     Heatmap(data, rows_by = "r", columns_by = "c", values_by = "value",
#'         show_row_names = c("inplace", "legend"),
#'         show_column_names = "anno"
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # split into multiple heatmaps
#'     Heatmap(data,
#'         values_by = "value", columns_by = "c", rows_by = "r", split_by = "p",
#'         upper_cutoff = 2, lower_cutoff = -2, legend.position = c("none", "right"),
#'         design = "AAAAAA#BBBBBBB"
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # cell_type = "bars" (default is "tile")
#'     Heatmap(data, values_by = "value", rows_by = "r", columns_by = "c",
#'         cell_type = "bars")
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # column widths are proportional to the number of bars per column
#'     # (columns c1 and c2 have 10 and 4 data points)
#'     bars_data <- data.frame(
#'         value = rnorm(14),
#'         r = c(rep("r1", 8), rep("r2", 6)),
#'         c = c(rep("c1", 6), rep("c2", 2), rep("c1", 4), rep("c2", 2))
#'     )
#'     Heatmap(bars_data, values_by = "value", rows_by = "r", columns_by = "c",
#'         cell_type = "bars")
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # a fraction uses that fraction of each cell's data
#'     Heatmap(bars_data, values_by = "value", rows_by = "r", columns_by = "c",
#'         cell_type = "bars", bars_sample = 0.5)
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # a whole count samples that many per cell -> equal column widths
#'     Heatmap(bars_data, values_by = "value", rows_by = "r", columns_by = "c",
#'         cell_type = "bars", bars_sample = 3)
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     p <- Heatmap(data, values_by = "value", rows_by = "r", columns_by = "c",
#'         cell_type = "dot", dot_size = length, dot_size_name = "data points",
#'         add_bg = TRUE, add_reticle = TRUE)
#'     p
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     dot_size_data <- as.matrix(p$data)
#'     # Make it big so we can see if we get the right indexing
#'     # for dot_size function
#'     dot_size_data["A", "a"] <- max(dot_size_data) * 2
#'
#'     Heatmap(data, values_by = "value", rows_by = "r", columns_by = "c",
#'         cell_type = "dot", dot_size_name = "data points",
#'         dot_size = function(x, i, j) ComplexHeatmap::pindex(dot_size_data, i, j),
#'         show_row_names = TRUE, show_column_names = TRUE,
#'         add_bg = TRUE, add_reticle = TRUE)
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     Heatmap(data, values_by = "value", rows_by = "r", columns_by = "c",
#'         cell_type = "pie", pie_group_by = "q", pie_size = sqrt,
#'         add_bg = TRUE, add_reticle = TRUE)
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     Heatmap(data, values_by = "value", rows_by = "r", columns_by = "c",
#'         cell_type = "violin", add_bg = TRUE, add_reticle = TRUE)
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     Heatmap(data, values_by = "value", rows_by = "r", columns_by = "c",
#'         cell_type = "boxplot", add_bg = TRUE, add_reticle = TRUE)
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     Heatmap(data,
#'         values_by = "value", rows_by = "r", columns_by = "c",
#'         column_annotation = list(
#'             r1 = list(col = "p", type = "ring",
#'                       params = list(height = grid::unit(10, "mm"), show_legend = FALSE)),
#'             r2 = list(col = "q", type = "bar"),
#'             r3 = list(col = "p1", type = "violin",
#'                       params = list(height = grid::unit(18, "mm")))
#'         ),
#'         row_annotation = list(
#'             .default = list(side = "right"),
#'             q = list(type = "pie", params = list(width = grid::unit(12, "mm"))),
#'             p2 = list(type = "density"),
#'             a = list(type = "simple")
#'         ),
#'         show_row_names = TRUE, show_column_names = TRUE
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     Heatmap(data,
#'         values_by = "value", rows_by = "r", columns_by = "c",
#'         split_by = "p", palette = list(x = "Reds", y = "Blues")
#'     )
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # implies in_form = "wide-rows"
#'     Heatmap(data, rows_by = c("p1", "p2"), columns_by = "c")
#' }
#' if (requireNamespace("cluster", quietly = TRUE)) {
#'     # implies wide-columns
#'     Heatmap(data, rows_by = "r", columns_by = c("p1", "p2"))
#' }
#' }
Heatmap <- function(
    data,
    values_by = NULL,
    values_fill = NA,
    name = NULL,
    # data definition
    in_form = c("auto", "matrix", "wide-columns", "wide-rows", "long"),
    split_by = NULL,
    split_by_sep = "_",
    rows_by = NULL,
    rows_by_sep = "_",
    rows_split_by = NULL,
    rows_split_by_sep = "_",
    columns_by = NULL,
    columns_by_sep = "_",
    columns_split_by = NULL,
    columns_split_by_sep = "_",
    rows_data = NULL,
    columns_data = NULL,
    keep_na = FALSE,
    keep_empty = FALSE,
    rows_orderby = NULL,
    columns_orderby = NULL,
    # names
    columns_name = NULL,
    columns_split_name = NULL,
    rows_name = NULL,
    rows_split_name = NULL,
    # palettes
    palette = "RdBu",
    palcolor = NULL,
    palreverse = FALSE,
    # cell_type: pies
    pie_size_name = "size",
    pie_size = NULL,
    pie_values = "length",
    pie_name = NULL,
    pie_group_by = NULL,
    pie_group_by_sep = "_",
    pie_palette = "Spectral",
    pie_palcolor = NULL,
    # cell_type: bars
    bars_sample = 1,
    # cell_type: label
    label = identity,
    label_size = 10,
    label_color = "black",
    label_name = "label",
    # cell_type: mark
    mark = identity,
    mark_color = "black",
    mark_size = 1,
    mark_name = "mark",
    # cell_type: violin
    violin_fill = NULL,
    # cell_type: boxplot
    boxplot_fill = NULL,
    # cell_type: dot
    dot_size = 8,
    dot_size_name = "size",
    # legend
    legend_items = NULL,
    legend_discrete = FALSE,
    legend.position = "right",
    legend.direction = "vertical",
    # values
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    # bg
    add_bg = FALSE,
    bg_alpha = 0.5,
    # reticle
    add_reticle = FALSE,
    reticle_color = "grey",
    # passed to ComplexHeatmap::Heatmap
    cluster_columns = NULL,
    cluster_rows = NULL,
    show_row_names = NULL,
    show_column_names = NULL,
    border = TRUE,
    title = NULL,
    column_title = NULL,
    row_title = NULL,
    na_col = "grey85",
    row_names_side = "right",
    column_names_side = "bottom",
    row_annotation = NULL,
    row_annotation_side = NULL,
    row_annotation_palette = NULL,
    row_annotation_palcolor = NULL,
    row_annotation_type = NULL,
    row_annotation_params = NULL,
    row_annotation_agg = NULL,
    column_annotation = NULL,
    column_annotation_side = NULL,
    column_annotation_palette = NULL,
    column_annotation_palcolor = NULL,
    column_annotation_type = NULL,
    column_annotation_params = NULL,
    column_annotation_agg = NULL,
    # misc
    flip = FALSE,
    alpha = 1,
    seed = 8525,
    padding = 15,
    base_size = 1,
    aspect.ratio = NULL,
    draw_opts = list(),
    # cell customization
    layer_fun_callback = NULL,
    cell_type = c(
        "tile",
        "bars",
        "label",
        "mark",
        "label+mark",
        "mark+label",
        "dot",
        "violin",
        "boxplot",
        "pie"
    ),
    cell_agg = NULL,
    # subplots
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed)
    in_form <- match.arg(in_form)
    cell_type <- match.arg(cell_type)
    cell_type <- sub("mark+label", "label+mark", cell_type, fixed = TRUE)

    if (!is.null(rows_orderby)) {
        cluster_rows <- cluster_rows %||% FALSE
        stopifnot(
            "[Heatmap] `rows_orderby` can't be used with `cluster_rows = TRUE`" = isFALSE(
                cluster_rows
            )
        )
    } else {
        cluster_rows <- cluster_rows %||% TRUE
    }

    if (!is.null(columns_orderby)) {
        cluster_columns <- cluster_columns %||% FALSE
        stopifnot(
            "[Heatmap] `columns_orderby` can't be used with `cluster_columns = TRUE`" = isFALSE(
                cluster_columns
            )
        )
    } else {
        cluster_columns <- cluster_columns %||% TRUE
    }

    hmdata <- process_heatmap_data(
        data,
        in_form = in_form,
        values_by = values_by,
        name = name,
        split_by = split_by,
        split_by_sep = split_by_sep,
        rows_orderby = rows_orderby,
        columns_orderby = columns_orderby,
        rows_by = rows_by,
        rows_by_sep = rows_by_sep,
        rows_name = rows_name,
        rows_split_by = rows_split_by,
        rows_split_by_sep = rows_split_by_sep,
        rows_split_name = rows_split_name,
        columns_by = columns_by,
        columns_by_sep = columns_by_sep,
        columns_name = columns_name,
        columns_split_by = columns_split_by,
        columns_split_by_sep = columns_split_by_sep,
        columns_split_name = columns_split_name,
        pie_group_by = pie_group_by,
        pie_group_by_sep = pie_group_by_sep,
        pie_name = pie_name,
        rows_data = rows_data,
        columns_data = columns_data,
        keep_na = keep_na,
        keep_empty = keep_empty
    )
    split_by <- split_by %||% "..."

    palette <- check_palette(palette, names(hmdata$data))
    palcolor <- check_palcolor(palcolor, names(hmdata$data))
    pie_palette <- check_palette(pie_palette, names(hmdata$data))
    pie_palcolor <- check_palcolor(pie_palcolor, names(hmdata$data))
    legend.direction <- check_legend(
        legend.direction,
        names(hmdata$data),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(hmdata$data),
        "legend.position"
    )

    plots <- lapply(
        names(hmdata$data),
        function(nm) {
            default_title <- if (
                length(hmdata$data) == 1 && identical(nm, "...")
            ) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }

            HeatmapAtomic(
                data = hmdata$data[[nm]],
                values_by = hmdata$values_by,
                values_fill = values_fill,
                rows_by = hmdata$rows_by,
                rows_split_by = hmdata$rows_split_by,
                columns_by = hmdata$columns_by,
                columns_split_by = hmdata$columns_split_by,

                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,

                pie_size_name = pie_size_name,
                pie_size = pie_size,
                pie_values = pie_values,
                pie_group_by = hmdata$pie_group_by,
                pie_palette = pie_palette[[nm]],
                pie_palcolor = pie_palcolor[[nm]],

                bars_sample = bars_sample,
                label = label,
                label_size = label_size,
                label_color = label_color,
                label_name = label_name,
                mark = mark,
                mark_color = mark_color,
                mark_size = mark_size,
                mark_name = mark_name,
                violin_fill = violin_fill,
                boxplot_fill = boxplot_fill,
                dot_size = dot_size,
                dot_size_name = dot_size_name,

                legend_items = legend_items,
                legend_discrete = legend_discrete,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],

                lower_quantile = lower_quantile,
                upper_quantile = upper_quantile,
                lower_cutoff = lower_cutoff,
                upper_cutoff = upper_cutoff,

                add_bg = add_bg,
                bg_alpha = bg_alpha,
                keep_na = hmdata$keep_na,
                keep_empty = hmdata$keep_empty,

                add_reticle = add_reticle,
                reticle_color = reticle_color,

                cluster_columns = cluster_columns,
                cluster_rows = cluster_rows,
                show_row_names = show_row_names,
                show_column_names = show_column_names,
                border = border,
                title = title,
                column_title = column_title,
                row_title = row_title,
                na_col = na_col,
                row_names_side = row_names_side,
                column_names_side = column_names_side,
                row_annotation = row_annotation,
                row_annotation_side = row_annotation_side,
                row_annotation_palette = row_annotation_palette,
                row_annotation_palcolor = row_annotation_palcolor,
                row_annotation_type = row_annotation_type,
                row_annotation_params = row_annotation_params,
                row_annotation_agg = row_annotation_agg,
                column_annotation = column_annotation,
                column_annotation_side = column_annotation_side,
                column_annotation_palette = column_annotation_palette,
                column_annotation_palcolor = column_annotation_palcolor,
                column_annotation_type = column_annotation_type,
                column_annotation_params = column_annotation_params,
                column_annotation_agg = column_annotation_agg,

                flip = flip,
                alpha = alpha,
                seed = seed,
                base_size = base_size,
                aspect.ratio = aspect.ratio,
                draw_opts = draw_opts,
                layer_fun_callback = layer_fun_callback,
                cell_type = cell_type,
                cell_agg = cell_agg,

                ...
            )
        }
    )
    names(plots) <- names(hmdata$data)

    p <- combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )

    # Return the plot object
    # When return_grob = FALSE, p is a HeatmapList object with auto-printing behavior
    # The initial draw() in HeatmapAtomic was captured to a null device to prevent
    # printing during assignment, so this returned object can print normally when
    # called directly (e.g., in a Jupyter cell)
    p
}

set.seed(8525)
# Matrix-form heatmap data
mat <- matrix(rnorm(50), nrow = 10, ncol = 5,
    dimnames = list(paste0("Gene", 1:10), paste0("Sample", 1:5))
)

# Long-form heatmap data
long_data <- data.frame(
    gene = rep(paste0("Gene", 1:8), each = 4),
    sample = rep(paste0("S", 1:4), 8),
    value = rnorm(32),
    group = rep(c("grp1", "grp2"), 16)
)

test_that("Heatmap returns an object for matrix input", {
    skip_if_not_installed("ComplexHeatmap")
    p <- Heatmap(mat)
    expect_true(!is.null(p))
    # Should have width/height attributes
    expect_true(!is.null(attr(p, "height")) || inherits(p, "HeatmapList") || inherits(p, "Heatmap"))
})

test_that("Heatmap works with long-form data", {
    skip_if_not_installed("ComplexHeatmap")
    p <- Heatmap(long_data, rows_by = "gene", columns_by = "sample", values_by = "value")
    expect_true(!is.null(p))
})

test_that("Heatmap with split_by and combine = FALSE returns list", {
    skip_if_not_installed("ComplexHeatmap")
    plots <- Heatmap(long_data, rows_by = "gene", columns_by = "sample",
                     values_by = "value", split_by = "group", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
})

test_that("Heatmap with multiple split_by columns returns a list with one plot per combination", {
    skip_if_not_installed("ComplexHeatmap")
    data2 <- long_data
    data2$split2 <- factor(rep(c("s1", "s2"), each = 16))
    plots <- suppressMessages(Heatmap(data2, rows_by = "gene", columns_by = "sample",
                                      values_by = "value", split_by = c("group", "split2"),
                                      combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
})

test_that("Heatmap with flip = TRUE works", {
    skip_if_not_installed("ComplexHeatmap")
    p <- Heatmap(mat, flip = TRUE)
    expect_true(!is.null(p))
})

test_that("Heatmap with cluster_rows = FALSE works", {
    skip_if_not_installed("ComplexHeatmap")
    p <- Heatmap(mat, cluster_rows = FALSE)
    expect_true(!is.null(p))
})

test_that("Heatmap with cluster_columns = FALSE works", {
    skip_if_not_installed("ComplexHeatmap")
    p <- Heatmap(mat, cluster_columns = FALSE)
    expect_true(!is.null(p))
})

test_that("Heatmap with custom palette works", {
    skip_if_not_installed("ComplexHeatmap")
    p <- Heatmap(mat, palette = "Spectral")
    expect_true(!is.null(p))
})

test_that("Heatmap with rownames split annotation works", {
    skip_if_not_installed("ComplexHeatmap")
    rows_data <- data.frame(
        rows = paste0("Gene", 1:10),
        group = rep(c("grp1", "grp2"), each = 5)
    )
    p <- Heatmap(mat, rows_data = rows_data, rows_split_by = "group",
        row_annotation = list(.row.split = list(
            type = "rownames",
            params = list(sep = ", ", wrap_by = 3)
        )))
    expect_true(!is.null(p))
})

test_that("Heatmap with rownames split annotation works with defaults", {
    skip_if_not_installed("ComplexHeatmap")
    rows_data <- data.frame(
        rows = paste0("Gene", 1:10),
        group = rep(c("grp1", "grp2"), each = 5)
    )
    p <- Heatmap(mat, rows_data = rows_data, rows_split_by = "group",
        row_annotation = list(.row.split = list(type = "rownames")))
    expect_true(!is.null(p))
})

test_that("Heatmap with colnames split annotation works for columns", {
    skip_if_not_installed("ComplexHeatmap")
    columns_data <- data.frame(
        columns = paste0("Sample", 1:5),
        batch = rep(c("b1", "b2"), length.out = 5)
    )
    p <- Heatmap(mat, columns_data = columns_data, columns_split_by = "batch",
        column_annotation = list(.col.split = list(
            type = "colnames",
            params = list(wrap_by = 2)
        )))
    expect_true(!is.null(p))
})

test_that("Heatmap with names/dimnames aliases for split annotations works", {
    skip_if_not_installed("ComplexHeatmap")
    rows_data <- data.frame(
        rows = paste0("Gene", 1:10),
        group = rep(c("grp1", "grp2"), each = 5)
    )
    p <- Heatmap(mat, rows_data = rows_data, rows_split_by = "group",
        row_annotation = list(.row.split = list(type = "names")))
    expect_true(!is.null(p))
    columns_data <- data.frame(
        columns = paste0("Sample", 1:5),
        batch = rep(c("b1", "b2"), length.out = 5)
    )
    p <- Heatmap(mat, columns_data = columns_data, columns_split_by = "batch",
        column_annotation = list(.col.split = list(type = "dimnames")))
    expect_true(!is.null(p))
})

test_that("Heatmap with show_row_names display modes works", {
    skip_if_not_installed("ComplexHeatmap")
    p <- Heatmap(long_data, rows_by = "gene", columns_by = "sample",
        values_by = "value", show_row_names = c("inplace", "legend"))
    expect_true(!is.null(p))
    p <- Heatmap(long_data, rows_by = "gene", columns_by = "sample",
        values_by = "value", show_row_names = "simple")
    expect_true(!is.null(p))
    p <- Heatmap(long_data, rows_by = "gene", columns_by = "sample",
        values_by = "value", show_row_names = "none")
    expect_true(!is.null(p))
})

test_that("Heatmap with anno display mode works with and without by", {
    skip_if_not_installed("ComplexHeatmap")
    # without a grouping column
    p <- Heatmap(mat, show_row_names = "anno")
    expect_true(!is.null(p))
    # with a grouping column
    p <- Heatmap(long_data, rows_by = "gene", columns_by = "sample",
        values_by = "value", show_row_names = "anno")
    expect_true(!is.null(p))
    # columns
    p <- Heatmap(long_data, rows_by = "gene", columns_by = "sample",
        values_by = "value", show_column_names = "annotation")
    expect_true(!is.null(p))
    # flip
    p <- Heatmap(mat, show_row_names = "anno", flip = TRUE)
    expect_true(!is.null(p))
})

test_that("Heatmap with explicit row_annotation overrides display modes", {
    skip_if_not_installed("ComplexHeatmap")
    # "none" mode + explicit .row config: the annotation still shows
    p <- Heatmap(long_data, rows_by = "gene", columns_by = "sample",
        values_by = "value", show_row_names = "none",
        row_annotation = list(.row = list(type = "label")))
    expect_true(!is.null(p))
})

test_that("Heatmap with split title display modes works", {
    skip_if_not_installed("ComplexHeatmap")
    rows_data <- data.frame(
        rows = paste0("Gene", 1:10),
        group = rep(c("grp1", "grp2"), each = 5)
    )
    columns_data <- data.frame(
        columns = paste0("Sample", 1:5),
        batch = rep(c("b1", "b2"), length.out = 5)
    )
    p <- Heatmap(mat, rows_data = rows_data, rows_split_by = "group",
        row_title = "legend")
    expect_true(!is.null(p))
    p <- Heatmap(mat, rows_data = rows_data, rows_split_by = "group",
        row_title = "none")
    expect_true(!is.null(p))
    p <- Heatmap(mat, rows_data = rows_data, rows_split_by = "group",
        row_title = "anno")
    expect_true(!is.null(p))
    p <- Heatmap(mat, rows_data = rows_data, rows_split_by = "group",
        row_title = c("inplace", "legend"))
    expect_true(!is.null(p))
    p <- Heatmap(mat, columns_data = columns_data, columns_split_by = "batch",
        column_title = "anno")
    expect_true(!is.null(p))
})

test_that("Heatmap errors on invalid display modes", {
    skip_if_not_installed("ComplexHeatmap")
    expect_error(Heatmap(mat, show_row_names = c("none", "legend")),
        "'none' cannot be combined")
    expect_error(Heatmap(mat, show_row_names = c("legend", "simple")),
        "'legend' and 'simple' cannot be combined")
    expect_error(Heatmap(mat, show_row_names = "bogus"),
        "Unknown display mode")
})

test_that("Heatmap annotation `name` aliases display name and legend title", {
    skip_if_not_installed("ComplexHeatmap")
    p <- Heatmap(long_data, rows_by = "gene", columns_by = "sample",
        values_by = "value",
        show_row_names = FALSE, show_column_names = FALSE,
        row_annotation = list(.row = list(name = "Gene Group")),
        column_annotation = list(.col = list(name = "Sample Group")),
        combine = FALSE, return_ht = TRUE)
    x <- p[[1]]
    # the legend title is the first text grob in the rendered legend
    legend_title <- function(key) {
        attr(x, "legends")[[key]]@grob$children[[1]]$label
    }
    expect_identical(
        slot(x, "left_annotation")@anno_list[["gene"]]@name_param$label,
        "Gene Group")
    expect_true(slot(x, "left_annotation")@anno_list[["gene"]]@name_param$show)
    expect_identical(
        slot(x, "top_annotation")@anno_list[["sample"]]@name_param$label,
        "Sample Group")
    expect_identical(legend_title("gene"), "Gene Group")
    expect_identical(legend_title("sample"), "Sample Group")
})

test_that("Heatmap annotation `name = FALSE` hides the displayed name", {
    skip_if_not_installed("ComplexHeatmap")
    p <- Heatmap(long_data, rows_by = "gene", columns_by = "sample",
        values_by = "value",
        show_row_names = FALSE, show_column_names = FALSE,
        row_annotation = list(.row = list(name = FALSE)),
        combine = FALSE, return_ht = TRUE)
    x <- p[[1]]
    expect_false(
        slot(x, "left_annotation")@anno_list[["gene"]]@name_param$show)
    expect_identical(
        attr(x, "legends")[["gene"]]@grob$children[[1]]$label, "gene")
})

test_that("Heatmap annotation `name` inherits from .default", {
    skip_if_not_installed("ComplexHeatmap")
    p <- Heatmap(long_data, rows_by = "gene", columns_by = "sample",
        values_by = "value",
        show_row_names = FALSE, show_column_names = FALSE,
        row_annotation = list(.default = list(name = "Group")),
        combine = FALSE, return_ht = TRUE)
    x <- p[[1]]
    expect_identical(
        slot(x, "left_annotation")@anno_list[["gene"]]@name_param$label,
        "Group")
})

test_that("show_row_names = 'none' keeps annotation configured via `name`", {
    skip_if_not_installed("ComplexHeatmap")
    p <- Heatmap(long_data, rows_by = "gene", columns_by = "sample",
        values_by = "value", show_row_names = "none",
        row_annotation = list(.default = list(name = "Group")),
        combine = FALSE, return_ht = TRUE)
    expect_true(!is.null(slot(p[[1]], "left_annotation")))
})

test_that("show_row_names = 'none' hides name display and legend of configured annotation", {
    skip_if_not_installed("ComplexHeatmap")
    # "none" = no names, annotation, or legend; a user-configured .row keeps
    # the annotation bar but the name display and legend are still hidden
    p <- Heatmap(long_data, rows_by = "gene", columns_by = "sample",
        values_by = "value", show_row_names = "none",
        show_column_names = "anno",
        row_annotation = list(.row = list(palette = "Paired")),
        combine = FALSE, return_ht = TRUE)
    x <- p[[1]]
    la <- slot(x, "left_annotation")
    expect_identical(names(la@anno_list), "gene")
    expect_false(la@anno_list[["gene"]]@name_param$show)
    expect_false("gene" %in% names(attr(x, "legends")))
    expect_true(".heatmap" %in% names(attr(x, "legends")))
})

test_that("Heatmap user-defined annotation `name` works", {
    skip_if_not_installed("ComplexHeatmap")
    rows_data <- data.frame(
        rows = paste0("Gene", 1:10),
        group = rep(c("g1", "g2"), each = 5)
    )
    p <- Heatmap(mat, rows_data = rows_data,
        row_annotation = list(Group = list(col = "group",
            name = "My Group", palette = "Spectral", agg = dplyr::first)),
        combine = FALSE, return_ht = TRUE)
    x <- p[[1]]
    expect_identical(
        slot(x, "left_annotation")@anno_list[["Group"]]@name_param$label,
        "My Group")
    expect_identical(
        attr(x, "legends")[["row.Group"]]@grob$children[[1]]$label,
        "My Group")
})

# Bars cell type: unbalanced fixture, column counts 10 vs 4 (gcd 2 -> k = c(5, 2))
bars_data <- data.frame(
    gene = rep(c("G1", "G2"), each = 7),
    sample = rep(rep(c("S1", "S2"), c(5, 2)), 2),
    value = rnorm(14)
)

test_that("Heatmap bars: default proportional column widths", {
    skip_if_not_installed("ComplexHeatmap")
    p <- Heatmap(bars_data, rows_by = "gene", columns_by = "sample",
        values_by = "value", cell_type = "bars", return_ht = TRUE)
    expect_identical(ncol(p@matrix), 7L)
    expect_identical(unname(lengths(p@column_order_list)), c(5L, 2L))
})

test_that("Heatmap bars: numeric bars_sample gives equal column widths", {
    skip_if_not_installed("ComplexHeatmap")
    p <- Heatmap(bars_data, rows_by = "gene", columns_by = "sample",
        values_by = "value", cell_type = "bars", bars_sample = 2,
        return_ht = TRUE)
    expect_identical(ncol(p@matrix), 2L)
    # uniform counts: no replication, single unsplit column block
    expect_identical(unname(lengths(p@column_order_list)), 2L)
})

test_that("Heatmap bars: fraction bars_sample samples per cell", {
    skip_if_not_installed("ComplexHeatmap")
    # column counts 10 vs 3; without sampling gcd(10, 3) = 1 -> 13 columns
    # (all four cells present: G1-S1 5, G2-S1 5, G1-S2 2, G2-S2 1)
    data3 <- data.frame(
        gene = c(rep("G1", 7), rep("G2", 6)),
        sample = c(rep("S1", 5), rep("S2", 2), rep("S1", 5), "S2"),
        value = rnorm(13)
    )
    p_full <- Heatmap(data3, rows_by = "gene", columns_by = "sample",
        values_by = "value", cell_type = "bars", return_ht = TRUE)
    expect_identical(ncol(p_full@matrix), 13L)
    # ceil(5*0.5)=3 + 3 vs ceil(3*0.5)=2 -> counts 6 vs 2 -> k = c(3, 1)
    p <- Heatmap(data3, rows_by = "gene", columns_by = "sample",
        values_by = "value", cell_type = "bars", bars_sample = 0.5,
        return_ht = TRUE)
    expect_identical(ncol(p@matrix), 4L)
    expect_identical(unname(lengths(p@column_order_list)), c(3L, 1L))
})

test_that("Heatmap bars: flip replicates rows", {
    skip_if_not_installed("ComplexHeatmap")
    p <- Heatmap(bars_data, rows_by = "gene", columns_by = "sample",
        values_by = "value", cell_type = "bars", flip = TRUE,
        return_ht = TRUE)
    expect_identical(nrow(p@matrix), 7L)
    expect_identical(unname(lengths(p@row_order_list)), c(5L, 2L))
})

test_that("Heatmap bars: invalid bars_sample errors", {
    skip_if_not_installed("ComplexHeatmap")
    for (bad in list(0, -1, 1.5, "a")) {
        expect_error(
            Heatmap(bars_data, rows_by = "gene", columns_by = "sample",
                values_by = "value", cell_type = "bars",
                bars_sample = bad, return_ht = TRUE),
            "bars_sample"
        )
    }
})

test_that("Heatmap bars: proportional widths work with user column annotations", {
    skip_if_not_installed("ComplexHeatmap")
    adata <- bars_data
    adata$ann <- rep(c("x", "y"), 7)
    p <- Heatmap(adata, rows_by = "gene", columns_by = "sample",
        values_by = "value", cell_type = "bars",
        column_annotation = "ann", return_ht = TRUE)
    expect_identical(ncol(p@matrix), 7L)
    expect_identical(unname(lengths(p@column_order_list)), c(5L, 2L))
})

test_that("Heatmap bars: smoke render and rows_split_by", {
    skip_if_not_installed("ComplexHeatmap")
    p <- Heatmap(bars_data, rows_by = "gene", columns_by = "sample",
        values_by = "value", cell_type = "bars")
    expect_true(!is.null(p))
    p2 <- Heatmap(bars_data, rows_by = "gene", columns_by = "sample",
        rows_split_by = "gene", values_by = "value", cell_type = "bars",
        return_ht = TRUE)
    expect_identical(ncol(p2@matrix), 7L)
    p3 <- Heatmap(bars_data, rows_by = "gene", columns_by = "sample",
        rows_split_by = "gene", values_by = "value", cell_type = "bars")
    expect_true(!is.null(p3))
})

#' A ggplot2 theme and palettes for plotthis
#' Borrowed from the `theme_this` function in the `SCP` pipeline
#'
#' @seealso \url{https://github.com/zhanghao-njmu/SCP}
#'
#' @param aspect.ratio The aspect ratio of the plot
#' @param base_size The base size of the text
#' If not specified, it will use the value from `getOption("theme_this.base_size", 12)`.
#' If you want to change the default base size, you can set the option `theme_this.base_size`.
#' This is applied to all plots using this theme.
#' @param font_family The font family of the text
#' If not specified, it will use the value from `getOption("theme_this.font_family")`.
#' If you want to change the default font family, you can set the option `theme_this.font_family`.
#' This is applied to all plots using this theme.
#' To list available font families, you can use the `systemfonts::system_fonts()` function.
#' @param ... Other arguments for `theme()`
#'
#' @return A ggplot2 theme
#' @importFrom methods formalArgs
#' @importFrom ggplot2 element_text element_rect margin theme element_blank unit
#' @export
theme_this <- function(
    aspect.ratio = NULL,
    base_size = NULL,
    font_family = NULL,
    ...
) {
    base_size <- base_size %||% getOption("theme_this.base_size", 12)
    font_family <- font_family %||% getOption("theme_this.font_family")
    text_size_scale <- base_size / 12
    args1 <- list(
        aspect.ratio = aspect.ratio,
        text = element_text(
            size = 12 * text_size_scale,
            family = font_family,
            color = "black"
        ),
        plot.title = element_text(
            size = 14 * text_size_scale,
            family = font_family,
            colour = "black",
            vjust = 1
        ),
        plot.subtitle = element_text(
            size = 13 * text_size_scale,
            family = font_family,
            hjust = 0,
            margin = margin(b = 3)
        ),
        plot.background = element_rect(fill = "white", color = "white"),
        plot.margin = margin(10, 10, 10, 10),
        axis.line = element_blank(),
        axis.title = element_text(
            size = 13 * text_size_scale,
            family = font_family,
            colour = "black"
        ),
        axis.text = element_text(
            size = 12 * text_size_scale,
            family = font_family,
            colour = "black"
        ),
        strip.text = element_text(
            size = 12.5 * text_size_scale,
            family = font_family,
            colour = "black",
            hjust = 0.5,
            margin = margin(3, 3, 3, 3)
        ),
        strip.background = element_rect(fill = "transparent", linetype = 0),
        strip.switch.pad.grid = unit(-1, "pt"),
        strip.switch.pad.wrap = unit(-1, "pt"),
        strip.placement = "outside",
        legend.title = element_text(
            size = 12 * text_size_scale,
            family = font_family,
            colour = "black",
            hjust = 0
        ),
        legend.text = element_text(
            size = 11 * text_size_scale,
            family = font_family,
            colour = "black"
        ),
        legend.key = element_rect(fill = "transparent", color = "transparent"),
        legend.key.size = unit(10, "pt"),
        legend.background = element_blank(),
        panel.background = element_rect(fill = "white", color = "white"),
        panel.border = element_rect(
            fill = "transparent",
            colour = "black",
            linewidth = 1
        )
    )
    args2 <- as.list(match.call())[-1]
    call.envir <- parent.frame(1)
    args2 <- lapply(args2, function(arg) {
        if (is.symbol(arg)) {
            eval(arg, envir = call.envir)
        } else if (is.call(arg)) {
            eval(arg, envir = call.envir)
        } else {
            arg
        }
    })
    for (n in names(args2)) {
        args1[[n]] <- args2[[n]]
    }
    args <- args1[names(args1) %in% formalArgs(theme)]
    do_call(what = theme, args = args)
}

#' Blank theme
#'
#' This function creates a theme with all elements blank except for axis lines and labels.
#' It can optionally add coordinate axes in the plot.
#'
#' @param add_coord Whether to add coordinate arrows. Default is \code{TRUE}.
#' @param xlen_npc The length of the x-axis arrow in "npc".
#' @param ylen_npc The length of the y-axis arrow in "npc".
#' @param xlab x-axis label.
#' @param ylab y-axis label.
#' @param lab_size Label size.
#' @param ... Arguments passed to the \code{\link[ggplot2]{theme}}.
#' @return A ggplot2 theme.
#' @examples
#' library(ggplot2)
#' p <- ggplot(mtcars, aes(x = wt, y = mpg, colour = factor(cyl))) +
#'     geom_point()
#' p + theme_blank()
#' p + theme_blank(xlab = "x-axis", ylab = "y-axis", lab_size = 16)
#' @importFrom methods formalArgs
#' @importFrom ggplot2 theme element_blank margin annotation_custom coord_cartesian
#' @importFrom grid grobTree gList linesGrob textGrob arrow gpar
#' @export
theme_blank <- function(
    add_coord = TRUE,
    xlen_npc = 0.15,
    ylen_npc = 0.15,
    xlab = "",
    ylab = "",
    lab_size = 12,
    ...
) {
    args1 <- list(
        panel.border = element_blank(),
        panel.grid = element_blank(),
        axis.title = element_blank(),
        axis.line = element_blank(),
        axis.ticks = element_blank(),
        axis.text = element_blank(),
        legend.background = element_blank(),
        legend.box.margin = margin(0, 0, 0, 0),
        legend.margin = margin(0, 0, 0, 0),
        legend.key.size = unit(10, "pt"),
        plot.margin = margin(
            lab_size + 2,
            lab_size + 2,
            lab_size + 2,
            lab_size + 2,
            unit = "points"
        )
    )
    args2 <- as.list(match.call())[-1]
    call.envir <- parent.frame(1)
    args2 <- lapply(args2, function(arg) {
        if (is.symbol(arg)) {
            eval(arg, envir = call.envir)
        } else if (is.call(arg)) {
            eval(arg, envir = call.envir)
        } else {
            arg
        }
    })
    for (n in names(args2)) {
        args1[[n]] <- args2[[n]]
    }
    args <- args1[names(args1) %in% formalArgs(theme)]
    out <- do_call(
        what = theme,
        args = args
    )
    if (isTRUE(add_coord)) {
        g <- grobTree(gList(
            linesGrob(
                x = unit(c(0, xlen_npc), "npc"),
                y = unit(c(0, 0), "npc"),
                arrow = arrow(length = unit(0.02, "npc")),
                gp = gpar(lwd = 2)
            ),
            textGrob(
                label = xlab,
                x = unit(0, "npc"),
                y = unit(0, "npc"),
                vjust = 4 / 3,
                hjust = 0,
                gp = gpar(fontsize = lab_size)
            ),
            linesGrob(
                x = unit(c(0, 0), "npc"),
                y = unit(c(0, ylen_npc), "npc"),
                arrow = arrow(length = unit(0.02, "npc")),
                gp = gpar(lwd = 2)
            ),
            textGrob(
                label = ylab,
                x = unit(0, "npc"),
                y = unit(0, "npc"),
                vjust = -2 / 3,
                hjust = 0,
                rot = 90,
                gp = gpar(fontsize = lab_size)
            )
        ))
        return(list(
            list(annotation_custom(g)),
            list(theme_this() + out),
            list(coord_cartesian(clip = "off"))
        ))
    } else {
        return(list(
            list(theme_this() + out)
        ))
    }
}

#' Box theme
#'
#' This function creates a theme with all elements blank except for axis lines
#' like a box around the plot.
#'
#' @param xlen_npc The length of the x-axis arrow in "npc".
#' @param ylen_npc The length of the y-axis arrow in "npc".
#' @param xlab x-axis label.
#' @param ylab y-axis label.
#' @param lab_size Label size.
#' @param ... Arguments passed to the \code{\link[ggplot2]{theme}}.
#' @return A ggplot2 theme.
#' @examples
#' library(ggplot2)
#' p <- ggplot(mtcars, aes(x = wt, y = mpg, colour = factor(cyl))) +
#'     geom_point()
#' p + theme_box()
#' @importFrom methods formalArgs
#' @importFrom ggplot2 theme element_blank margin annotation_custom coord_cartesian
#' @importFrom grid grobTree gList linesGrob textGrob arrow gpar
#' @export
theme_box <- function(
    xlen_npc = 0.15,
    ylen_npc = 0.15,
    xlab = "",
    ylab = "",
    lab_size = 12,
    ...
) {
    args1 <- list(
        panel.border = element_rect(
            fill = "transparent",
            colour = "black",
            linewidth = 1
        ),
        panel.grid = element_blank(),
        axis.title = element_blank(),
        axis.line = element_blank(),
        axis.ticks = element_blank(),
        axis.text = element_blank(),
        legend.background = element_blank(),
        legend.box.margin = margin(0, 0, 0, 0),
        legend.margin = margin(0, 0, 0, 0),
        legend.key.size = unit(10, "pt"),
        plot.margin = margin(
            lab_size + 2,
            lab_size + 2,
            lab_size + 2,
            lab_size + 2,
            unit = "points"
        )
    )
    args2 <- as.list(match.call())[-1]
    call.envir <- parent.frame(1)
    args2 <- lapply(args2, function(arg) {
        if (is.symbol(arg)) {
            eval(arg, envir = call.envir)
        } else if (is.call(arg)) {
            eval(arg, envir = call.envir)
        } else {
            arg
        }
    })
    for (n in names(args2)) {
        args1[[n]] <- args2[[n]]
    }
    args <- args1[names(args1) %in% formalArgs(theme)]
    out <- do_call(
        what = theme,
        args = args
    )

    return(list(
        list(theme_this() + out)
    ))
}

#' Color palettes collected in plotthis.
#'
#' @param x A vector of character/factor or numeric values. If missing, numeric values 1:n will be used as x.
#' @param n The number of colors to return for numeric values.
#' @param palette Palette name. All available palette names can be queried with \code{show_palettes()}.
#' @param palcolor Custom colors used to create a color palette.
#' @param type Type of \code{x}. Can be one of "auto", "discrete" or "continuous". The default is "auto", which automatically detects if \code{x} is a numeric value.
#' @param keep_names Whether to keep the names of the color vector.
#' @param alpha The alpha value of the colors. Default is 1.
#' @param matched If \code{TRUE}, will return a color vector of the same length as \code{x}.
#' @param reverse Whether to invert the colors.
#' @param NA_keep Whether to keep the color assignment to NA in \code{x}.
#' @param NA_color Color assigned to NA if NA_keep is \code{TRUE}.
#' @param transparent Whether to make the colors transparent when alpha < 1.
#'  When `TRUE`, [ggplot2::alpha()] is used to make the colors transparent.
#'  Otherwise, `adjcolors` is used to adjust the colors based on the alpha. The color will be not be actually transparent.
#'  For example, `ggplot2::alpha("red", 0.5) == "#FF000080"`; while `adjcolors("red", 0.5) == "#FF8080"`.
#' @return A vector of colors.
#' @importFrom grDevices colorRampPalette
#' @importFrom stats setNames
#' @export
#'
palette_this <- function(
    x,
    n = 100,
    palette = "Paired",
    palcolor = NULL,
    type = "auto",
    keep_names = TRUE,
    alpha = 1,
    matched = FALSE,
    reverse = FALSE,
    NA_keep = FALSE,
    NA_color = "grey80",
    transparent = TRUE
) {
    palette_list <- plotthis::palette_list
    if (missing(x)) {
        x <- 1:n
        type <- "continuous"
    }
    if (!palette %in% names(palette_list)) {
        stop(
            "The palette name (",
            palette,
            ") is invalid! You can check the available palette names with 'show_palettes()'. Or pass palette colors via the 'palcolor' parameter."
        )
    }

    # Store custom colors for later replacement
    custom_colors <- NULL
    if (is.list(palcolor)) {
        palcolor <- unlist(palcolor)
    }
    if (!is.null(palcolor) && length(palcolor) > 0 && !all(palcolor == "")) {
        custom_colors <- palcolor
    }

    # Use palette colors as base
    if (all(palcolor == "")) {
        palcolor <- palette_list[[palette]]
    }
    if (is.null(palcolor) || length(palcolor) == 0) {
        palcolor <- palette_list[[palette]]
    }

    # If custom_colors are provided, use palette as base
    if (!is.null(custom_colors)) {
        palcolor <- palette_list[[palette]]

        # For continuous colors without names, replace evenly in the base palette
        if (is.null(names(custom_colors))) {
            custom_len <- length(custom_colors)
            pal_len <- length(palcolor)
            if (custom_len > 0 && pal_len > 0) {
                # Calculate evenly spaced positions
                positions <- round(seq(1, pal_len, length.out = custom_len))
                # Replace at those positions where custom_colors is not NA
                for (i in seq_along(custom_colors)) {
                    if (!is.na(custom_colors[i])) {
                        palcolor[positions[i]] <- custom_colors[i]
                    }
                }
            }
            # Clear custom_colors for continuous since we've already applied them
            custom_colors_continuous <- NULL
        } else {
            custom_colors_continuous <- custom_colors
        }
    } else {
        custom_colors_continuous <- NULL
    }

    pal_n <- length(palcolor)

    if (!type %in% c("auto", "discrete", "continuous")) {
        stop("'type' must be one of 'auto','discrete' and 'continuous'.")
    }
    if (type == "auto") {
        if (is.numeric(x)) {
            type <- "continuous"
        } else {
            type <- "discrete"
        }
    }

    if (type == "discrete") {
        if (!is.factor(x)) {
            x <- factor(x, levels = unique(x))
        }
        n_x <- nlevels(x)
        if (isTRUE(attr(palcolor, "type") == "continuous")) {
            color <- colorRampPalette(palcolor)(n_x)
            names(color) <- levels(x)
        } else {
            color <- ifelse(
                rep(n_x, n_x) <= pal_n,
                palcolor[1:n_x],
                colorRampPalette(palcolor)(n_x)
            )
            names(color) <- levels(x)
        }

        # Replace colors with custom_colors if provided
        if (!is.null(custom_colors) && !is.null(names(custom_colors))) {
            matching_names <- intersect(names(custom_colors), names(color))
            if (length(matching_names) > 0) {
                color[matching_names] <- custom_colors[matching_names]
            }
        }

        if (any(is.na(x))) {
            # Check if custom_colors has "NA" key
            na_col <- if (
                !is.null(custom_colors) && "NA" %in% names(custom_colors)
            ) {
                custom_colors["NA"]
            } else {
                NA_color
            }
            color <- c(color, setNames(na_col, "NA"))
        }
        if (isTRUE(matched)) {
            color <- color[x]
            color[is.na(color)] <- NA_color
        }
    } else if (type == "continuous") {
        if (!is.numeric(x) && all(!is.na(x))) {
            stop(
                "'x' must be type of numeric when use continuous color palettes."
            )
        }
        if (all(is.na(x))) {
            values <- as.factor(rep(0, n))
        } else if (length(unique(na.omit(as.numeric(x)))) == 1) {
            values <- as.factor(rep(unique(na.omit(as.numeric(x))), n))
        } else {
            if (isTRUE(matched)) {
                values <- cut(
                    x,
                    breaks = seq(
                        min(x, na.rm = TRUE),
                        max(x, na.rm = TRUE),
                        length.out = n + 1
                    ),
                    include.lowest = TRUE
                )
            } else {
                values <- cut(
                    1:100,
                    breaks = seq(
                        min(x, na.rm = TRUE),
                        max(x, na.rm = TRUE),
                        length.out = n + 1
                    ),
                    include.lowest = TRUE
                )
            }
        }

        n_x <- nlevels(values)
        color <- ifelse(
            rep(n_x, n_x) <= pal_n,
            palcolor[1:n_x],
            colorRampPalette(palcolor)(n_x)
        )
        names(color) <- levels(values)

        if (any(is.na(x))) {
            # Check if custom_colors_continuous has "NA" key
            na_col <- if (
                !is.null(custom_colors_continuous) &&
                    "NA" %in% names(custom_colors_continuous)
            ) {
                custom_colors_continuous["NA"]
            } else if (
                !is.null(custom_colors) && "NA" %in% names(custom_colors)
            ) {
                custom_colors["NA"]
            } else {
                NA_color
            }
            color <- c(color, setNames(na_col, "NA"))
        }
        if (isTRUE(matched)) {
            if (all(is.na(x))) {
                color <- NA_color
            } else if (length(unique(na.omit(x))) == 1) {
                color <- color[as.character(unique(na.omit(x)))]
                color[is.na(color)] <- NA_color
            } else {
                color <- color[as.character(values)]
                color[is.na(color)] <- NA_color
            }
        }
    }

    if (isTRUE(reverse)) {
        if (!is.null(names(color))) {
            # Isolate NA-named entries so they don't misalign during reversal
            na_idx <- is.na(names(color)) | names(color) == "NA"
            if (any(na_idx)) {
                na_entries <- color[na_idx]
                non_na <- color[!na_idx]
                non_na <- setNames(rev(unname(non_na)), names(non_na))
                color <- c(non_na, na_entries)
            } else {
                color <- setNames(rev(unname(color)), names(color))
            }
        } else {
            color <- rev(color)
        }
    }
    if (!isTRUE(NA_keep)) {
        color <- color[names(color) != "NA"]
    }
    if (!isTRUE(keep_names)) {
        names(color) <- NULL
    }
    if (alpha < 1) {
        if (isTRUE(transparent)) {
            color <- ggplot2::alpha(color, alpha)
        } else {
            color <- adjcolors(color, alpha)
        }
    }

    return(color)
}

#' Show the color palettes
#'
#' This function displays color palettes using ggplot2.
#'
#' @param palettes A list of color palettes. If `NULL`, uses default palettes.
#' @param type A character vector specifying the type of palettes to include. Default is "discrete".
#' @param index A numeric vector specifying the indices of the palettes to include. Default is `NULL`.
#' @param palette_names A character vector specifying the names of the SCP palettes to include. Default is `NULL`.
#' @param return_names A logical value indicating whether to return the names of the selected palettes. Default is `TRUE`.
#' @param return_palettes A logical value indicating whether to return the colors of selected palettes. Default is `FALSE`.
#' @return A list of palette names or a list of palettes.
#'
#' @seealso \code{\link{palette_list}}
#'
#' @examples
#' show_palettes(palettes = list(c("red", "blue", "green"), c("yellow", "purple", "orange")))
#' all_palettes <- show_palettes(return_palettes = TRUE)
#' names(all_palettes)
#' all_palettes[["simspec"]]
#' show_palettes(index = 1:10)
#' show_palettes(type = "discrete", index = 1:10)
#' show_palettes(type = "continuous", index = 1:10)
#' show_palettes(
#'     palette_names = c("Paired", "nejm", "simspec", "Spectral", "jet"),
#'     return_palettes = TRUE
#' )
#'
#' @importFrom ggplot2 geom_col scale_fill_manual scale_x_continuous element_blank aes element_text margin element_rect unit theme
#' @export
show_palettes <- function(
    palettes = NULL,
    type = c("discrete", "continuous"),
    index = NULL,
    palette_names = NULL,
    return_names = TRUE,
    return_palettes = FALSE
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }

    if (!is.null(palettes)) {
        palette_list <- palettes
    } else {
        palette_list <- plotthis::palette_list[unlist(lapply(
            palette_list,
            function(x) isTRUE(attr(x, "type") %in% type)
        ))]
    }
    index <- index[index %in% seq_along(palette_list)]
    if (!is.null(index)) {
        palette_list <- palette_list[index]
    }
    if (is.null(names(palette_list))) {
        names(palette_list) <- seq_along(palette_list)
    }
    if (is.null(palette_names)) {
        palette_names <- palette_names %||% names(palette_list)
    }
    if (any(!palette_names %in% names(palette_list))) {
        stop(paste(
            "Can not find the palettes: ",
            paste0(
                palette_names[!palette_names %in% names(palette_list)],
                collapse = ","
            )
        ))
    }
    palette_list <- palette_list[palette_names]

    df <- data.frame(
        palette = rep(names(palette_list), sapply(palette_list, length)),
        color = unlist(palette_list)
    )
    df$palette <- factor(df$palette, levels = rev(unique(df$palette)))
    df$color_order <- factor(seq_len(nrow(df)), levels = seq_len(nrow(df)))
    df$proportion <- as.numeric(1 / table(df$palette)[df$palette])
    p <- ggplot(
        data = df,
        aes(
            y = !!sym("palette"),
            x = !!sym("proportion"),
            fill = !!sym("color_order")
        )
    ) +
        geom_col(show.legend = FALSE) +
        scale_fill_manual(values = df[["color"]]) +
        scale_x_continuous(expand = c(0, 0), trans = "reverse") +
        theme_this(
            axis.title = element_blank(),
            axis.ticks = element_blank(),
            axis.text.x = element_blank(),
            panel.border = element_blank()
        )
    print(p)

    if (isTRUE(return_palettes)) {
        return(palette_list)
    }
    if (isTRUE(return_names)) {
        return(palette_names)
    }
}


#' Theme element that add a box to the text
#'
#' Code grabbed from the `ggtext` package. See the original code at:
#' https://github.com/wilkelab/ggtext
#' This is used to create a text box around the text, primarily to be used in `CorPairsPlot`.
#'
#' @rdname element_textbox
#' @param family Font family
#' @param face Font face
#' @param size Font size (in pt)
#' @param colour,color Text color
#' @param fill Fill color of the enclosing box
#' @param box.colour,box.color Line color of the enclosing box (if different from the text color)
#' @param linetype Line type of the enclosing box (like `lty` in base R)
#' @param linewidth Line width of the enclosing box (measured in mm, just like `size` in
#'   [ggplot2::element_line()]).
#' @param hjust Horizontal justification
#' @param vjust Vertical justification
#' @param halign Horizontal justification
#' @param valign Vertical justification
#' @param lineheight Line height, in multiples of the font size
#' @param width,height Unit objects specifying the width and height
#'   of the textbox, as in [gridtext::textbox_grob()].
#' @param minwidth,minheight,maxwidth,maxheight Min and max values for width and height.
#'   Set to NULL to impose neither a minimum nor a maximum.
#' @param padding,margin Padding and margins around the text box.
#'   See [gridtext::textbox_grob()] for details.
#' @param r Unit value specifying the corner radius of the box
#' @param orientation Orientation of the text box. See [gridtext::textbox_grob()] for details.
#' @param debug Not implemented.
#' @param inherit.blank See [ggplot2::margin()] for details.
#' @return A ggplot2 theme element that can be used inside a [ggplot2::theme()] call.
#' @export
#' @importFrom grid unit
element_textbox <- function(
    family = NULL,
    face = NULL,
    size = NULL,
    colour = NULL,
    fill = NULL,
    box.colour = NULL,
    linetype = NULL,
    linewidth = NULL,
    hjust = NULL,
    vjust = NULL,
    halign = NULL,
    valign = NULL,
    lineheight = NULL,
    margin = NULL,
    padding = NULL,
    width = NULL,
    height = NULL,
    minwidth = NULL,
    maxwidth = NULL,
    minheight = NULL,
    maxheight = NULL,
    r = NULL,
    orientation = NULL,
    color = NULL,
    box.color = NULL,
    debug = FALSE,
    inherit.blank = FALSE
) {
    if (!is.null(color)) {
        colour <- color
    }

    if (!is.null(box.color)) {
        box.colour <- box.color
    }
    family <- family %||% getOption("theme_this.font_family")
    structure(
        list(
            family = family,
            face = face,
            size = size,
            colour = colour,
            fill = fill,
            box.colour = box.colour,
            linetype = linetype,
            linewidth = linewidth,
            hjust = hjust,
            vjust = vjust,
            halign = halign,
            valign = valign,
            lineheight = lineheight,
            margin = margin,
            padding = padding,
            width = width,
            height = height,
            minwidth = minwidth,
            maxwidth = maxwidth,
            minheight = minheight,
            maxheight = maxheight,
            r = r,
            orientation = orientation,
            debug = debug,
            inherit.blank = inherit.blank
        ),
        class = c("element_textbox", "element_text", "element")
    )
}

#' @rdname element_textbox
#' @param element A theme element created by [element_textbox()].
#' @param label Text to display in the textbox.
#' @param x,y Position of the textbox.
#' @param ... Other arguments passed to [gridtext::textbox_grob()].
#' @importFrom grid gpar
#' @importFrom gridtext textbox_grob
#' @importFrom ggplot2 zeroGrob .pt element_grob
#' @export
element_grob.element_textbox <- function(
    element,
    label = "",
    x = NULL,
    y = NULL,
    family = NULL,
    face = NULL,
    colour = NULL,
    size = NULL,
    hjust = NULL,
    vjust = NULL,
    lineheight = NULL,
    margin = NULL,
    ...
) {
    if (is.null(label)) {
        return(zeroGrob())
    }

    hj <- hjust %||% element$hjust
    vj <- vjust %||% element$vjust
    halign <- element$halign %||% 0
    valign <- element$valign %||% 1
    padding <- element$padding %||% ggplot2::margin(0, 0, 0, 0)
    margin <- margin %||% element$margin %||% ggplot2::margin(0, 0, 0, 0)
    orientation <- element$orientation %||% "upright"
    r <- element$r %||% unit(0, "pt")
    # The gp settings can override element_gp
    gp <- gpar(
        fontsize = size %||% element$size,
        col = colour %||% element$colour,
        fontfamily = family %||% element$family,
        fontface = face %||% element$face,
        lineheight = lineheight %||% element$lineheight
    )

    box_gp <- gpar(
        col = element$box.colour %||% gp$col,
        fill = element$fill %||% NA,
        lty = element$linetype %||% 0,
        lwd = (element$linewidth %||% 0.5) * .pt
    )

    textbox_grob(
        label,
        x = x,
        y = y,
        hjust = hj,
        vjust = vj,
        halign = halign,
        valign = valign,
        width = element$width,
        height = element$height,
        minwidth = element$minwidth,
        minheight = element$minheight,
        maxwidth = element$maxwidth,
        maxheight = element$maxheight,
        margin = margin,
        padding = padding,
        r = r,
        orientation = orientation,
        gp = gp,
        box_gp = box_gp
    )
}

# Basic functionality tests
test_that("palette_this works with discrete values", {
    colors <- palette_this(c("a", "b", "c"), palette = "Paired")
    expect_length(colors, 3)
    expect_named(colors, c("a", "b", "c"))
    expect_true(all(grepl("^#[0-9A-F]{6}$", colors)))
})

test_that("palette_this works with continuous values", {
    colors <- palette_this(1:10, n = 10, palette = "Spectral")
    expect_length(colors, 10)
    expect_true(all(grepl("^#[0-9A-F]{6}$", colors)))
})

test_that("palette_this auto-detects type", {
    discrete <- palette_this(c("a", "b", "c"), type = "auto")
    expect_named(discrete, c("a", "b", "c"))

    continuous <- palette_this(1:5, type = "auto", n = 5)
    expect_length(continuous, 5)
})

test_that("palette_this works with missing x", {
    colors <- palette_this(n = 50, palette = "Spectral")
    expect_length(colors, 50)
})

# Named palcolor tests (new behavior)
test_that("palette_this works with named palcolor - replaces specific colors", {
    colors <- palette_this(
        c("a", "b", "c"),
        palette = "Paired",
        palcolor = c("c" = "red", "b" = "blue", "a" = "green")
    )
    colors <- as.list(colors)[sort(names(colors))]
    expect_equal(colors, list(a = "green", b = "blue", c = "red"))
})

test_that("palette_this works with named palcolor but missed items - uses palette for missing", {
    colors <- palette_this(
        c("a", "b", "c"),
        palette = "Paired",
        palcolor = c("b" = "blue", "a" = "green")
    )
    expect_equal(colors[["a"]], "green")
    expect_equal(colors[["b"]], "blue")
    # c should get color from Paired palette
    expect_true(grepl("^#[0-9A-F]{6}$", colors[["c"]]))
})

test_that("palette_this partial named palcolor uses palette as base", {
    colors <- palette_this(
        c("a", "b", "c", "d"),
        palette = "Paired",
        palcolor = c("c" = "red")
    )
    expect_equal(colors[["c"]], "red")
    expect_named(colors, c("a", "b", "c", "d"))
    expect_true(all(grepl("^#[0-9A-F]{6}$", colors[c("a", "b", "d")])))
})

# Positional palcolor tests (continuous) - evenly distributed replacement
test_that("palette_this works with positional palcolor for continuous - evenly distributed", {
    # Get base palette colors
    base_palette <- plotthis::palette_list[["Spectral"]]

    # With 3 palcolor values, they should replace positions 1, middle, and last
    # in the base palette evenly
    colors_base <- palette_this(1:100, n = 100, palette = "Spectral")
    colors_custom <- palette_this(1:100, n = 100, palette = "Spectral", palcolor = c("red", NA, "blue"))

    # First and last should be different due to replacement
    expect_false(colors_custom[[1]] == colors_base[[1]])
    expect_false(colors_custom[[100]] == colors_base[[100]])
})

test_that("palette_this positional replacement evenly distributes in base palette", {
    # Test that replacement happens in the base palette, not final colors
    # With a 2-value palcolor, should replace first and last of base palette
    colors <- palette_this(1:100, n = 50, palette = "Spectral", palcolor = c("yellow", "darkred"))
    expect_length(colors, 50)
    # The gradient should go from yellow to darkred
    expect_true(grepl("^#[0-9A-F]{6}$", colors[[1]]))
    expect_true(grepl("^#[0-9A-F]{6}$", colors[[50]]))
})

test_that("palette_this evenly distributed replacement with 3 values", {
    # With 3 values: first, middle, last of base palette
    colors <- palette_this(1:100, n = 20, palette = "Spectral", palcolor = c("red", NA, "blue"))
    expect_length(colors, 20)
    # Should create a smooth gradient
    expect_true(all(grepl("^#[0-9A-F]{6}$", colors)))
})

# NA handling tests
test_that("palette_this handles NA values with default NA_color", {
    colors <- palette_this(c("a", "b", NA), palette = "Paired", NA_keep = TRUE)
    expect_length(colors, 3)
    expect_equal(colors[["NA"]], "grey80")
})

test_that("palette_this handles NA values with custom NA_color", {
    colors <- palette_this(c("a", "b", NA), palette = "Paired", NA_keep = TRUE, NA_color = "purple")
    expect_length(colors, 3)
    expect_equal(colors[["NA"]], "purple")
})

test_that("palette_this handles NA color from palcolor", {
    colors <- palette_this(
        c("a", "b", NA),
        palette = "Paired",
        palcolor = c("a" = "red", "NA" = "orange"),
        NA_keep = TRUE
    )
    expect_length(colors, 3)
    expect_equal(colors[["a"]], "red")
    expect_equal(colors[["NA"]], "orange")
})

test_that("palette_this NA_keep = FALSE removes NA color", {
    colors <- palette_this(c("a", "b", NA), palette = "Paired", NA_keep = FALSE)
    expect_length(colors, 2)
    expect_false("NA" %in% names(colors))
})

test_that("palette_this continuous with NA uses palcolor NA", {
    colors <- palette_this(
        c(1, 2, 3, NA),
        n = 3,
        palette = "Spectral",
        palcolor = c("NA" = "yellow"),
        NA_keep = TRUE
    )
    expect_equal(colors[["NA"]], "yellow")
})

# matched parameter tests
test_that("palette_this matched = TRUE returns color per value", {
    x <- c("a", "b", "c", "a", "b")
    colors <- palette_this(x, palette = "Paired", matched = TRUE)
    expect_length(colors, 5)
    expect_equal(colors[[1]], colors[[4]])
    expect_equal(colors[[2]], colors[[5]])
})

test_that("palette_this matched = FALSE returns unique colors", {
    x <- c("a", "b", "c", "a", "b")
    colors <- palette_this(x, palette = "Paired", matched = FALSE)
    expect_length(colors, 3)
    expect_named(colors, c("a", "b", "c"))
})

test_that("palette_this matched with NA values", {
    x <- c("a", "b", NA, "a")
    colors <- palette_this(x, palette = "Paired", matched = TRUE, NA_keep = TRUE)
    expect_length(colors, 4)
    expect_equal(colors[[3]], "grey80")
})

# reverse parameter tests
test_that("palette_this reverse = TRUE inverts color order", {
    colors_normal <- palette_this(c("a", "b", "c"), palette = "Paired", reverse = FALSE)
    colors_reversed <- palette_this(c("a", "b", "c"), palette = "Paired", reverse = TRUE)
    expect_equal(colors_normal[["a"]], colors_reversed[["c"]])
    expect_equal(colors_normal[["c"]], colors_reversed[["a"]])
})

test_that("palette_this reverse works with continuous", {
    colors_normal <- palette_this(n = 5, palette = "Spectral", reverse = FALSE)
    colors_reversed <- palette_this(n = 5, palette = "Spectral", reverse = TRUE)
    expect_equal(colors_normal[[1]], colors_reversed[[5]])
})

# keep_names parameter tests
test_that("palette_this keep_names = FALSE removes names", {
    colors <- palette_this(c("a", "b", "c"), palette = "Paired", keep_names = FALSE)
    expect_null(names(colors))
})

test_that("palette_this keep_names = TRUE keeps names", {
    colors <- palette_this(c("a", "b", "c"), palette = "Paired", keep_names = TRUE)
    expect_named(colors, c("a", "b", "c"))
})

# alpha parameter tests
test_that("palette_this applies alpha with transparent = TRUE", {
    colors <- palette_this(c("a", "b", "c"), palette = "Paired", alpha = 0.5, transparent = TRUE)
    expect_true(all(grepl("^#[0-9A-F]{8}$", colors)))  # 8 characters with alpha
})

test_that("palette_this applies alpha with transparent = FALSE", {
    colors <- palette_this(c("a", "b", "c"), palette = "Paired", alpha = 0.5, transparent = FALSE)
    expect_true(all(grepl("^#[0-9A-F]{6}$", colors)))  # 6 characters, adjusted but not transparent
})

test_that("palette_this alpha = 1 does not modify colors", {
    colors_alpha1 <- palette_this(c("a", "b", "c"), palette = "Paired", alpha = 1)
    colors_default <- palette_this(c("a", "b", "c"), palette = "Paired")
    expect_equal(colors_alpha1, colors_default)
})

# Factor handling tests
test_that("palette_this works with factor input", {
    x <- factor(c("a", "b", "c"), levels = c("c", "b", "a"))
    colors <- palette_this(x, palette = "Paired")
    expect_named(colors, c("c", "b", "a"))
})

test_that("palette_this preserves factor level order", {
    x <- factor(c("a", "b", "c"), levels = c("b", "c", "a"))
    colors <- palette_this(x, palette = "Paired")
    expect_equal(names(colors), c("b", "c", "a"))
})

# Continuous edge cases
test_that("palette_this handles all NA continuous values", {
    colors <- palette_this(c(NA, NA, NA), n = 3, palette = "Spectral", type = "continuous")
    expect_length(colors, 1)
})

test_that("palette_this handles single unique continuous value", {
    colors <- palette_this(c(5, 5, 5), n = 3, palette = "Spectral", type = "continuous")
    expect_length(colors, 1)
})

test_that("palette_this matched continuous returns correct length", {
    x <- c(1, 5, 10, 3, 7)
    colors <- palette_this(x, n = 5, palette = "Spectral", matched = TRUE, type = "continuous")
    expect_length(colors, 5)
})

# Error handling tests
test_that("palette_this throws error for invalid palette", {
    expect_error(
        palette_this(c("a", "b"), palette = "InvalidPalette"),
        "invalid"
    )
})

test_that("palette_this throws error for invalid type", {
    expect_error(
        palette_this(c("a", "b"), type = "invalid"),
        "must be one of"
    )
})

test_that("palette_this throws error for non-numeric continuous", {
    expect_error(
        palette_this(c("a", "b", "c"), type = "continuous"),
        "must be type of numeric"
    )
})

# List palcolor handling
test_that("palette_this handles list palcolor", {
    colors <- palette_this(
        c("a", "b", "c"),
        palette = "Paired",
        palcolor = list("a" = "red", "b" = "blue")
    )
    expect_equal(colors[["a"]], "red")
    expect_equal(colors[["b"]], "blue")
})

# Empty palcolor handling
test_that("palette_this handles empty string palcolor", {
    colors <- palette_this(c("a", "b", "c"), palette = "Paired", palcolor = "")
    expect_length(colors, 3)
    expect_named(colors, c("a", "b", "c"))
})

test_that("theme_this returns a ggplot2 theme", {
    t <- theme_this()
    expect_s3_class(t, "theme")
})

test_that("theme_this accepts aspect.ratio", {
    t <- theme_this(aspect.ratio = 1)
    expect_s3_class(t, "theme")
})

test_that("theme_this accepts base_size", {
    t <- theme_this(base_size = 14)
    expect_s3_class(t, "theme")
})

test_that("theme_blank returns a list of ggplot layers", {
    t <- theme_blank()
    expect_true(is.list(t))
    expect_true(length(t) > 0)
})

test_that("theme_blank with add_coord = FALSE returns a list", {
    t <- theme_blank(add_coord = FALSE)
    expect_true(is.list(t))
    expect_true(length(t) > 0)
})

test_that("theme_box returns a list of ggplot layers", {
    t <- theme_box()
    expect_true(is.list(t))
    expect_true(length(t) > 0)
})

test_that("theme_this can be applied to a ggplot", {
    p <- ggplot2::ggplot(data.frame(x = 1:5, y = 1:5), ggplot2::aes(x = x, y = y)) +
        ggplot2::geom_point() +
        theme_this()
    expect_s3_class(p, "ggplot")
})

test_that("theme_blank can be applied to a ggplot", {
    p <- ggplot2::ggplot(data.frame(x = 1:5, y = 1:5), ggplot2::aes(x = x, y = y)) +
        ggplot2::geom_point() +
        theme_blank()
    expect_s3_class(p, "ggplot")
})

test_that("show_palettes returns palette names (character)", {
    result <- show_palettes()
    expect_true(is.character(result))
    expect_true(length(result) > 0)
})

test_that("show_palettes with type = 'discrete' works", {
    result <- show_palettes(type = "discrete")
    expect_true(is.character(result))
    expect_true(length(result) > 0)
})

test_that("show_palettes with type = 'continuous' works", {
    result <- show_palettes(type = "continuous")
    expect_true(is.character(result))
    expect_true(length(result) > 0)
})

test_that("palette_list is a valid list", {
    expect_true(is.list(palette_list))
    expect_true(length(palette_list) > 0)
    expect_true("Paired" %in% names(palette_list))
    expect_true("Spectral" %in% names(palette_list))
})

#' Word cloud plot (internal)
#'
#' @description
#' Core implementation for drawing a word cloud plot. This is the workhorse
#' behind the exported \code{\link{WordCloudPlot}} function — it takes a
#' **single** data frame (no \code{split_by} support) and returns a
#' \code{ggplot} object rendered via
#' \code{\link[ggwordcloud]{geom_text_wordcloud}}.
#'
#' The function supports two input modes:
#' \itemize{
#'   \item \strong{Pre-tokenized words} (\code{word_by}) — each row contains
#'   a single word (or multiple words in a list column that is unnested).
#'   \item \strong{Sentence splitting} (\code{sentence_by}) — each row
#'   contains a sentence or phrase; the function splits the text into
#'   individual words after removing punctuation and lowercasing.
#' }
#'
#' Words are displayed with \strong{font size} proportional to a count
#' variable (\code{count_by}) and \strong{colour} based on a continuous
#' score variable (\code{score_by}). Text filtering options allow removal of
#' short words, common stop words, and bracketed patterns.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
#'         \code{ggplot2::ggplot} based on
#'         \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{Input validation} — ensures exactly one of \code{word_by}
#'         or \code{sentence_by} is specified. Errors if both are provided
#'         or neither is provided. When \code{sentence_by} is used,
#'         \code{count_by} must be \code{NULL} (counts are derived from
#'         occurrences after splitting).
#'   \item \strong{Column resolution} — \code{facet_by}, \code{count_by},
#'         and \code{score_by} are validated and transformed via
#'         \code{\link{check_columns}}.
#'   \item \strong{Default values} — when \code{score_by} is \code{NULL}, a
#'         synthetic \code{.score} column with value \code{1} is created.
#'         When \code{count_by} is \code{NULL}, a synthetic \code{.count}
#'         column with value \code{1} is created.
#'   \item \strong{Sentence splitting branch} (when \code{sentence_by} is set):
#'         \itemize{
#'           \item The \code{sentence_by} column is validated via
#'                 \code{\link{check_columns}}.
#'           \item Text is lowercased, punctuation is removed, and the
#'                 string is split into individual words on whitespace
#'                 boundaries.
#'           \item Words are unnested into separate rows via
#'                 \code{\link[tidyr]{unnest}}.
#'           \item Data is grouped by word (and \code{facet_by} columns
#'                 when present) and aggregated: \code{sum(count_by)} and
#'                 \code{score_agg(score_by)}. Separate aggregation
#'                 templates handle 0, 1, or 2 \code{facet_by} columns.
#'         }
#'   \item \strong{Word branch} (when \code{word_by} is set):
#'         \itemize{
#'           \item The \code{word_by} column is validated via
#'                 \code{\link{check_columns}}.
#'           \item Column values are unnested into separate rows (supports
#'                 list columns where a row may contain multiple words).
#'           \item Data is grouped by word (and \code{facet_by} columns
#'                 when present) and aggregated similarly to the sentence
#'                 branch.
#'         }
#'   \item \strong{Text filtering pipeline} — the aggregated data is
#'         processed sequentially:
#'         \itemize{
#'           \item Words matching the pattern \verb{[.*]} (bracketed
#'                 content) are removed.
#'           \item Words with fewer than \code{minchar} characters are
#'                 removed.
#'           \item Words listed in \code{words_excluded} are removed
#'                 (case-insensitive matching).
#'           \item Duplicate rows are removed.
#'           \item The top \code{top_words} words by score are retained
#'                 via \code{\link[dplyr]{slice_max}}.
#'         }
#'   \item \strong{Angle assignment} — each word is randomly assigned a
#'         rotation angle of 0 (60\% probability) or 90 degrees (40\%
#'         probability), creating the characteristic word cloud appearance.
#'   \item \strong{Colour scale preparation} —
#'         \code{\link{palette_this}()} with \code{type = "continuous"}
#'         generates a smooth gradient for the \code{score} variable. A
#'         \code{colors_value} sequence is created spanning
#'         \code{min(score)} to \code{quantile(score, 0.99)} for colour
#'         interpolation.
#'   \item \strong{Plot assembly} — the \code{ggplot} object is built with:
#'         \itemize{
#'           \item \code{ggwordcloud::geom_text_wordcloud()} renders the
#'                 words with \code{rm_outside = TRUE}, square shape, and
#'                 eccentricity of 1.
#'           \item \code{scale_color_gradientn()} maps the score to the
#'                 continuous colour gradient, with a colour-bar legend
#'                 (titled by \code{score_name} or \code{"Score"}).
#'           \item \code{scale_size()} maps the count to font size within
#'                 the \code{word_size} range, with a size legend (titled
#'                 by \code{count_name} or \code{"Count"}).
#'           \item \code{coord_flip()} swaps the axes (word cloud
#'                 convention).
#'           \item \code{do_call(theme, theme_args)} applies the selected
#'                 theme; \code{aspect.ratio}, \code{legend.position}, and
#'                 \code{legend.direction} are set directly.
#'         }
#'   \item \strong{Dimension calculation} —
#'         \code{\link{calculate_plot_dimensions}()} computes plot height
#'         and width using \code{base_height = 4.5}, \code{aspect.ratio},
#'         and legend metrics. The resulting \code{height} / \code{width}
#'         attributes are stored on the \code{ggplot} object.
#'   \item \strong{Faceting} — \code{\link{facet_plot}()} wraps the plot
#'         with \code{facet_wrap} or \code{facet_grid} if \code{facet_by}
#'         is provided, up to a maximum of 2 facet columns.
#' }
#'
#' @inheritParams common_args
#' @param word_by A character string specifying the column name containing
#'  pre-tokenized words. A character column is expected. Use this when your
#'  data already has one word per row (or a list of words per row). Mutually
#'  exclusive with \code{sentence_by}.
#' @param sentence_by A character string specifying the column name containing
#'  sentences or phrases to be split into individual words. A character column
#'  is expected. The text is lowercased and punctuation is removed before
#'  splitting on whitespace boundaries. Mutually exclusive with
#'  \code{word_by}.
#' @param count_by A character string specifying the numeric column for the
#'  count or frequency of each word. When \code{NULL} (the default), each
#'  occurrence counts as 1. Must be \code{NULL} when \code{sentence_by} is
#'  used, as counts are derived from the number of occurrences after splitting.
#' @param score_by A character string specifying the numeric column for the
#'  score of each word, mapped to the text colour via a continuous gradient.
#'  When \code{NULL} (the default), all words receive a score of 1 and are
#'  coloured at the low end of the palette.
#' @param count_name A character string for the size legend title. When
#'  \code{NULL} (the default), \code{"Count"} is used.
#' @param score_name A character string for the colour-bar legend title. When
#'  \code{NULL} (the default), \code{"Score"} is used.
#' @param words_excluded A character vector of words to exclude from the word
#'  cloud. Matching is case-insensitive. Defaults to
#'  \code{plotthis::words_excluded}, a built-in set of common English stop
#'  words.
#' @param score_agg A function to aggregate the scores when multiple
#'  observations of the same word exist. Default is \code{mean}. Other
#'  options include \code{sum}, \code{median}, or a custom function.
#' @param minchar A numeric value specifying the minimum number of characters
#'  a word must have to be included. Words with fewer characters are filtered
#'  out. Default: \code{2}.
#' @param word_size A numeric vector of length 2 specifying the range of font
#'  sizes (in mm) for the words. Passed to \code{scale_size(range =
#'  word_size)}. Default: \code{c(2, 8)}.
#' @param top_words A numeric value specifying the maximum number of words to
#'  display, selected by highest score. Default: \code{100}.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @keywords internal
#' @importFrom tidyr unnest
#' @importFrom dplyr all_of reframe slice_max distinct mutate group_by summarise n
WordCloudPlotAtomic <- function(
    data,
    word_by = NULL,
    sentence_by = NULL,
    count_by = NULL,
    score_by = NULL,
    count_name = NULL,
    score_name = NULL,
    words_excluded = plotthis::words_excluded,
    score_agg = mean,
    minchar = 2,
    word_size = c(2, 8),
    top_words = 100,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    alpha = 1,
    palreverse = FALSE,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    seed = 8525,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    if (sum(is.null(word_by), is.null(sentence_by)) != 1) {
        stop("Either 'word_by' or 'sentence_by' should be specified.")
    }
    if (!is.null(sentence_by) && !is.null(count_by)) {
        stop("Cannot specify 'count_by' when 'sentence_by' is specified.")
    }
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )
    count_by <- check_columns(data, count_by)
    score_by <- check_columns(data, score_by)
    if (is.null(score_by)) {
        data$.score <- 1
        score_by <- ".score"
    }
    if (is.null(count_by)) {
        data$.count <- 1
        count_by <- ".count"
    }

    if (!is.null(sentence_by)) {
        sentence_by <- check_columns(data, sentence_by)
        data <- data %>%
            mutate(
                word = strsplit(
                    tolower(gsub(
                        "[[:punct:]]",
                        "",
                        as.character(!!sym(sentence_by))
                    )),
                    "\\b\\s+\\b"
                )
            ) %>%
            unnest(cols = "word")

        if (length(facet_by) == 1) {
            data <- data %>%
                group_by(!!!syms(c("word", facet_by))) %>%
                reframe(
                    word = !!sym("word"),
                    !!sym(facet_by) := !!sym(facet_by),
                    count = sum(!!sym(count_by)),
                    score = score_agg(!!sym(score_by)),
                    .groups = "keep"
                )
        } else if (length(facet_by) == 2) {
            data <- data %>%
                group_by(!!!syms(c("word", facet_by))) %>%
                reframe(
                    word = !!sym("word"),
                    !!sym(facet_by[1]) := !!sym(facet_by[1]),
                    !!sym(facet_by[2]) := !!sym(facet_by[2]),
                    count = sum(!!sym(count_by)),
                    score = score_agg(!!sym(score_by)),
                    .groups = "keep"
                )
        } else {
            data <- data %>%
                group_by(!!sym("word")) %>%
                reframe(
                    word = !!sym("word"),
                    count = sum(!!sym(count_by)),
                    score = score_agg(!!sym(score_by)),
                    .groups = "keep"
                )
        }
    } else {
        word_by <- check_columns(data, word_by)
        data <- data %>% unnest(cols = all_of(word_by))

        if (length(facet_by) == 1) {
            data <- data %>%
                group_by(!!!syms(unique(c(word_by, facet_by)))) %>%
                reframe(
                    word = !!sym(word_by),
                    !!sym(facet_by) := !!sym(facet_by),
                    count = sum(!!sym(count_by)),
                    score = score_agg(!!sym(score_by)),
                    .groups = "keep"
                )
        } else if (length(facet_by) == 2) {
            data <- data %>%
                group_by(!!!syms(unique(c(word_by, facet_by)))) %>%
                reframe(
                    word = !!sym(word_by),
                    !!sym(facet_by[1]) := !!sym(facet_by[1]),
                    !!sym(facet_by[2]) := !!sym(facet_by[2]),
                    count = sum(!!sym(count_by)),
                    score = score_agg(!!sym(score_by)),
                    .groups = "keep"
                )
        } else {
            data <- data %>%
                group_by(!!sym(word_by)) %>%
                reframe(
                    word = !!sym(word_by),
                    count = sum(!!sym(count_by)),
                    score = score_agg(!!sym(score_by)),
                    .groups = "keep"
                )
        }
    }

    data <- data %>%
        filter(!grepl(pattern = "\\[.*\\]", x = !!sym("word"))) %>%
        filter(nchar(!!sym("word")) >= minchar) %>%
        filter(!tolower(!!sym("word")) %in% tolower(words_excluded)) %>%
        distinct() %>%
        slice_max(order_by = !!sym("score"), n = top_words) %>%
        mutate(
            angle = 90 * sample(c(0, 1), n(), replace = TRUE, prob = c(60, 40))
        ) %>%
        as.data.frame()

    colors <- palette_this(
        data$score,
        type = "continuous",
        palette = palette,
        palcolor = palcolor,
        matched = FALSE,
        reverse = palreverse
    )
    colors_value <- seq(
        min(data$score, na.rm = TRUE),
        quantile(data$score, 0.99, na.rm = TRUE) + 0.001,
        length.out = 100
    )
    p <- ggplot(
        data,
        aes(
            label = !!sym("word"),
            size = !!sym("count"),
            color = !!sym("score"),
            angle = !!sym("angle")
        )
    ) +
        ggwordcloud::geom_text_wordcloud(
            rm_outside = TRUE,
            eccentricity = 1,
            shape = "square",
            show.legend = TRUE,
            grid_margin = 3
        ) +
        scale_color_gradientn(
            name = score_name %||% "Score",
            colours = colors,
            values = scales::rescale(colors_value),
            guide = guide_colorbar(
                frame.colour = "black",
                ticks.colour = "black",
                title.hjust = 0
            )
        ) +
        scale_size(
            name = count_name %||% "Count",
            range = word_size,
            breaks = ceiling(seq(
                min(data$count, na.rm = TRUE),
                max(data$count, na.rm = TRUE),
                length.out = 3
            ))
        ) +
        guides(
            size = guide_legend(
                override.aes = list(colour = "black", label = "G"),
                order = 1
            )
        ) +
        labs(title = title, subtitle = subtitle) +
        coord_flip() +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction
        )

    dims <- calculate_plot_dimensions(
        base_height = 4.5,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    facet_plot(p, facet_by, facet_scales, facet_nrow, facet_ncol, facet_byrow)
}

#' Word cloud plot
#'
#' @description
#' Draws a word cloud plot that visualises word frequency and importance.
#' Words are displayed with font size proportional to a count variable and
#' colour based on a continuous score variable, using
#' \code{\link[ggwordcloud]{geom_text_wordcloud}} for rendering.
#'
#' The function supports \strong{pre-tokenised words} (via \code{word_by})
#' and \strong{sentence splitting} (via \code{sentence_by}). Sentences are
#' automatically lowercased, stripped of punctuation, and split into
#' individual words before aggregation. Common stop words can be excluded
#' via \code{words_excluded}, and the number of displayed words is
#' controlled by \code{top_words}.
#'
#' The word cloud can be \strong{faceted} (via \code{facet_by}) or
#' \strong{split} into separate sub-plots via \code{split_by}. When
#' \code{split_by} is used, each split level receives its own word cloud,
#' and the results are combined into a single layout via
#' \code{\link{combine_plots}}.
#'
#' @section split_by workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \code{\link{validate_common_args}()} validates the \code{seed}
#'         and \code{facet_by} constraints (maximum 2 facet columns).
#'   \item The \code{theme} argument is resolved via
#'         \code{\link{process_theme}()}.
#'   \item The \code{split_by} column(s) are validated and transformed via
#'         \code{\link{check_columns}()} with \code{force_factor = TRUE}
#'         and \code{concat_multi = TRUE}.
#'   \item The data frame is split by \code{split_by} (preserving factor
#'         level order). If \code{split_by} is \code{NULL}, the data is
#'         wrapped in a single-element list with name \code{"..."}.
#'   \item Per-split \code{palette}, \code{palcolor},
#'         \code{legend.position}, and \code{legend.direction} are resolved
#'         via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
#'         and \code{\link{check_legend}()}.
#'   \item \code{\link{WordCloudPlotAtomic}()} is called for each split. If
#'         \code{title} is a function, it receives the split level name and
#'         can generate dynamic titles per sub-plot.
#'   \item Results are combined via \code{\link{combine_plots}()} (when
#'         \code{combine = TRUE}) or returned as a named list (when
#'         \code{combine = FALSE}).
#' }
#'
#' @inheritParams common_args
#' @inheritParams WordCloudPlotAtomic
#' @return A \code{ggplot} object (when no \code{split_by} is used), a
#'  \code{patchwork} object (when \code{combine = TRUE} and
#'  \code{split_by} is used), or a named list of \code{ggplot} objects
#'  (when \code{combine = FALSE}), each with \code{height} and
#'  \code{width} attributes in inches.
#' @export
#' @examples
#' \donttest{
#' set.seed(8525)
#' data <- data.frame(
#'     word = c("apple", "banana", "cherry", "date", "elderberry",
#'              "fig", "grape", "honeydew", "kiwi", "lemon"),
#'     count = c(10, 20, 30, 40, 50, 15, 25, 35, 45, 55),
#'     score = c(1, 2, 3, 4, 5, 1.5, 2.5, 3.5, 4.5, 5.5),
#'     facet = rep(c("Group1", "Group2"), each = 5),
#'     split = rep(c("A", "B"), 5)
#' )
#'
#' # Basic word cloud with word, count, and score columns
#' WordCloudPlot(data, word_by = "word",
#'               count_by = "count", score_by = "score")
#'
#' # Word cloud using sentence_by (sentences split into words)
#' data_sent <- data.frame(
#'     sentence = c("The quick brown fox jumps over the lazy dog",
#'                  "A quick brown dog jumps over a lazy fox"),
#'     score = c(10, 5)
#' )
#' WordCloudPlot(data_sent, sentence_by = "sentence", score_by = "score")
#'
#' # Word cloud with faceting
#' WordCloudPlot(data, word_by = "word",
#'               count_by = "count", score_by = "score",
#'               facet_by = "facet")
#'
#' # Word cloud split by a grouping variable
#' WordCloudPlot(data, word_by = "word",
#'               count_by = "count", score_by = "score",
#'               split_by = "split")
#' }
WordCloudPlot <- function(
    data,
    word_by = NULL,
    sentence_by = NULL,
    count_by = NULL,
    score_by = NULL,
    count_name = NULL,
    score_name = NULL,
    split_by = NULL,
    split_by_sep = "_",
    words_excluded = plotthis::words_excluded,
    score_agg = mean,
    minchar = 2,
    word_size = c(2, 8),
    top_words = 100,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    theme = "theme_this",
    theme_args = list(),
    palette = "Spectral",
    palcolor = NULL,
    alpha = 1,
    palreverse = FALSE,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    theme <- process_theme(theme)
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            WordCloudPlotAtomic(
                datas[[nm]],
                word_by = word_by,
                sentence_by = sentence_by,
                count_by = count_by,
                score_by = score_by,
                count_name = count_name,
                score_name = score_name,
                words_excluded = words_excluded,
                score_agg = score_agg,
                minchar = minchar,
                word_size = word_size,
                top_words = top_words,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                alpha = alpha,
                palreverse = palreverse,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                seed = seed,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
# WordCloudPlot can work with word_by (individual word column)
# or sentence_by (text column that gets tokenized)
wc_data <- data.frame(
    word = c("apple", "banana", "cherry", "date", "elderberry",
             "fig", "grape", "honeydew", "kiwi", "lemon",
             "mango", "nectarine", "orange", "papaya", "quince"),
    count = c(50, 40, 30, 20, 60, 35, 45, 25, 55, 15, 70, 10, 80, 65, 12),
    score = runif(15, -2, 2),
    group = factor(rep(c("G1", "G2"), c(8, 7)))
)

test_that("WordCloudPlot with word_by and count_by works", {
    skip_if_not_installed("ggwordcloud")
    p <- WordCloudPlot(wc_data, word_by = "word", count_by = "count")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("WordCloudPlot with score_by works", {
    skip_if_not_installed("ggwordcloud")
    p <- WordCloudPlot(wc_data, word_by = "word", count_by = "count", score_by = "score")
    expect_s3_class(p, "ggplot")
})

test_that("WordCloudPlot with title and subtitle works", {
    skip_if_not_installed("ggwordcloud")
    p <- WordCloudPlot(wc_data, word_by = "word", count_by = "count",
                       title = "Word Cloud Title", subtitle = "Subtitle")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "Word Cloud Title")
    expect_equal(p$labels$subtitle, "Subtitle")
})

test_that("WordCloudPlot with split_by returns patchwork", {
    skip_if_not_installed("ggwordcloud")
    p <- WordCloudPlot(wc_data, word_by = "word", count_by = "count",
                       split_by = "group", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("WordCloudPlot with combine = FALSE returns list", {
    skip_if_not_installed("ggwordcloud")
    plots <- WordCloudPlot(wc_data, word_by = "word", count_by = "count",
                           split_by = "group", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("WordCloudPlot with multiple split_by columns returns a list with one plot per combination", {
    skip_if_not_installed("ggwordcloud")
    data2 <- wc_data
    data2$split2 <- factor(rep(c("s1", "s2"), length.out = 15))
    plots <- suppressMessages(WordCloudPlot(data2, word_by = "word", count_by = "count",
                                            split_by = c("group", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("WordCloudPlot with multiple split_by columns returns combined plot", {
    skip_if_not_installed("ggwordcloud")
    data2 <- wc_data
    data2$split2 <- factor(rep(c("s1", "s2"), length.out = 15))
    p <- suppressMessages(WordCloudPlot(data2, word_by = "word", count_by = "count",
                                        split_by = c("group", "split2"), combine = TRUE))
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("WordCloudPlot with top_words limit works", {
    skip_if_not_installed("ggwordcloud")
    p <- WordCloudPlot(wc_data, word_by = "word", count_by = "count", top_words = 10)
    expect_s3_class(p, "ggplot")
})

test_that("WordCloudPlot with sentence_by works", {
    skip_if_not_installed("ggwordcloud")
    sentence_data <- data.frame(
        text = c(
            "the quick brown fox jumps over the lazy dog",
            "to be or not to be that is the question",
            "all that glitters is not gold",
            "a rose by any other name would smell as sweet"
        )
    )
    p <- WordCloudPlot(sentence_data, sentence_by = "text")
    expect_s3_class(p, "ggplot")
})

#' Atomic chord plot (internal)
#'
#' @description
#' Core implementation for drawing a chord diagram using
#' \code{circlize::chordDiagram()}.  This is the workhorse behind the
#' exported \code{\link{ChordPlot}} function — it takes a **single** data
#' frame (no \code{split_by} support) and returns a \code{patchwork}
#' wrapped element.
#'
#' Chord diagrams visualise relationships (flows) between two sets of
#' categories arranged around a circle.  The width of each link is
#' proportional to the value (\code{y}) connecting its source and target
#' nodes.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{Column resolution} — \code{from}, \code{to}, and
#'         \code{y} are validated and transformed via
#'         \code{\link{check_columns}}.  Multi-column \code{from} and
#'         \code{to} are concatenated with their respective separators
#'         (\code{from_sep}, \code{to_sep}).
#'   \item \strong{Count aggregation} — when \code{y = NULL}, the count
#'         of observations per (\code{from}, \code{to}) pair is computed
#'         as a new \code{.y} column.  Factor levels from the original
#'         data are preserved.
#'   \item \strong{NA / empty-level handling} —
#'         \code{\link{process_keep_na_empty}()} applies \code{keep_na}
#'         and \code{keep_empty} policies.
#'   \item \strong{Flip} — when \code{flip = TRUE}, the \code{from} and
#'         \code{to} columns are swapped, effectively reversing the link
#'         direction.
#'   \item \strong{Colour mapping} — \code{\link{palette_this}()} assigns
#'         colours to all unique \code{from} and \code{to} values (the grid
#'         sectors).  \code{NA} values are mapped to a literal \code{"NA"}
#'         level and assigned a colour from the palette.
#'   \item \strong{Link colour resolution} — \code{links_color} controls
#'         whether each connecting ribbon takes its colour from the
#'         \code{"from"} side (default) or the \code{"to"} side.
#'   \item \strong{circlize rendering} — two rendering paths exist,
#'         selected by \code{labels_rot}:
#'         \itemize{
#'           \item \strong{Horizontal labels} (\code{labels_rot = FALSE}):
#'                 track 1 has a fixed height of 1 mm and uses
#'                 \code{niceFacing = TRUE} for automatic label rotation.
#'                 Track 2 adds axis ticks on the outside for wide-enough
#'                 sectors.
#'           \item \strong{Rotated labels} (\code{labels_rot = TRUE}):
#'                 track 1 height is computed from the maximum string width
#'                 of all node names, and labels are rendered with
#'                 \code{facing = "clockwise"}.  Track 2 adds axis ticks.
#'         }
#'         Links use arrow heads (\code{link.arr.type = "big.arrow"}) and
#'         differentiated height (\code{direction.type = c("diffHeight",
#'         "arrows")}).  Both tracks set \code{bg.border = NA} to prevent
#'         border lines from appearing between sectors.
#'   \item \strong{Patchwork integration} — the circlize formula is wrapped
#'         via \code{\link[patchwork]{wrap_elements}()} so it can be
#'         composed with other ggplot objects.  Title and subtitle are added
#'         via \code{\link[patchwork]{plot_annotation}()}.
#'   \item \strong{Dimension attributes} — \code{height} and \code{width}
#'         attributes (in inches) are set to a base size (7) scaled up by
#'         2, 4, or 6 depending on the maximum label character width when
#'         \code{labels_rot = TRUE}.
#' }
#'
#' @inheritParams common_args
#' @param from A character string (or vector) specifying the column name(s)
#'  for the source nodes.  Character/factor columns are expected.  Multiple
#'  columns are concatenated with \code{from_sep}.
#' @param from_sep A character string to join multiple \code{from} columns.
#'  Default \code{"_"}.
#' @param to A character string (or vector) specifying the column name(s)
#'  for the target nodes.  Character/factor columns are expected.  Multiple
#'  columns are concatenated with \code{to_sep}.
#' @param to_sep A character string to join multiple \code{to} columns.
#'  Default \code{"_"}.
#' @param y A character string specifying the numeric column whose values
#'  determine link thickness.  When \code{NULL}, the count of observations
#'  per (\code{from}, \code{to}) pair is used.
#' @param flip Logical; if \code{TRUE}, swap the source and target nodes,
#'  reversing the link direction.
#' @param links_color A character string controlling which node's colour
#'  each link ribbon takes: \code{"from"} (default) or \code{"to"}.
#' @param labels_rot Logical; if \code{TRUE}, rotate sector labels by 90
#'  degrees (clockwise).  Default \code{FALSE} uses \code{niceFacing} for
#'  automatic orientation.
#' @param alpha Numeric transparency for the link ribbons (0–1).
#'  Default \code{0.5}.
#' @return A \code{patchwork} wrapped element with \code{height} and
#'  \code{width} attributes (in inches) attached.  The original data is
#'  stored in the \code{p$data} field.
#' @importFrom dplyr %>% group_by summarise n select
#' @importFrom patchwork wrap_elements plot_annotation
#' @keywords internal
ChordPlotAtomic <- function(
    data,
    y = NULL,
    from = NULL,
    from_sep = "_",
    to = NULL,
    to_sep = "_",
    flip = FALSE,
    links_color = c("from", "to"),
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 0.5,
    labels_rot = FALSE,
    title = NULL,
    subtitle = NULL,
    keep_na = FALSE,
    keep_empty = FALSE,
    ...
) {
    # if (!requireNamespace("circlize", quietly = TRUE)) {
    #     stop("circlize is required for chord plot.")
    # }
    links_color <- match.arg(links_color)
    from <- check_columns(
        data,
        from,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = from_sep
    )
    to <- check_columns(
        data,
        to,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = to_sep
    )
    y <- check_columns(data, y)
    if (is.null(y)) {
        from_levels <- levels(data[[from]])
        to_levels <- levels(data[[to]])
        data <- data %>%
            group_by(!!sym(from), !!sym(to)) %>%
            summarise(.y = n(), .groups = "drop")

        data[[from]] <- factor(data[[from]], levels = from_levels)
        data[[to]] <- factor(data[[to]], levels = to_levels)

        y <- ".y"
    }

    data <- process_keep_na_empty(data, keep_na, keep_empty)
    if (isTRUE(flip)) {
        data <- data %>%
            select(from = !!sym(to), to = !!sym(from), value = !!sym(y))
    } else {
        data <- data %>%
            select(from = !!sym(from), to = !!sym(to), value = !!sym(y))
    }

    data <- data[order(data$from, data$to), , drop = FALSE]
    # data$from <- as.character(data$from)
    # data$to <- as.character(data$to)

    # froms <- unique(data$from)
    # tos <- unique(data$to)
    from_vals <- levels(data$from)
    if (anyNA(data$from)) {
        from_vals <- c(from_vals, NA)
    }
    to_vals <- levels(data$to)
    if (anyNA(data$to)) {
        to_vals <- c(to_vals, NA)
    }

    grid_cols <- palette_this(
        unique(c(from_vals, to_vals)),
        palette = palette,
        palcolor = palcolor,
        NA_keep = TRUE,
        reverse = palreverse
    )
    names(grid_cols)[is.na(names(grid_cols))] <- "NA"
    if (anyNA(from_vals)) {
        from_vals[is.na(from_vals)] <- "NA"
        levels(data$from) <- from_vals
        data$from[is.na(data$from)] <- "NA"
    }
    if (anyNA(to_vals)) {
        to_levels[is.na(to_vals)] <- "NA"
        levels(data$to) <- to_vals
        data$to[is.na(data$to)] <- "NA"
    }
    link_cols <- grid_cols[as.character(data[[links_color]])]

    circlize::circos.clear()
    circlize::circos.par(track.margin = c(0.01, 0.02))

    if (!isTRUE(labels_rot)) {
        p <- ~ {
            circlize::chordDiagram(
                data,
                grid.col = grid_cols,
                col = link_cols,
                transparency = 1 - alpha,
                direction = 1,
                annotationTrack = "grid",
                direction.type = c("diffHeight", "arrows"),
                link.arr.type = "big.arrow",
                link.arr.length = .04,
                preAllocateTracks = list(
                    list(track.height = circlize::mm_h(1)),
                    list(track.height = circlize::mm_h(.1))
                )
            )
            circlize::circos.track(
                track.index = 1,
                panel.fun = function(x, y) {
                    circlize::circos.text(
                        circlize::CELL_META$xcenter,
                        circlize::CELL_META$ylim[1] + 5.5,
                        circlize::CELL_META$sector.index,
                        niceFacing = TRUE,
                        adj = c(0.5, 0.5)
                    )
                },
                bg.border = NA
            )
            circlize::circos.track(
                track.index = 2,
                panel.fun = function(x, y) {
                    for (si in circlize::get.all.sector.index()) {
                        start.degree <- circlize::get.cell.meta.data(
                            "cell.start.degree",
                            sector.index = si
                        )
                        end.degree <- circlize::get.cell.meta.data(
                            "cell.end.degree",
                            sector.index = si
                        )
                        if (abs(end.degree - start.degree) > 2) {
                            # otherwise: patchwork wrap_elements 'x' and 'units' must have length > 0
                            circlize::circos.axis(
                                h = "top",
                                labels.cex = 0.7,
                                labels.niceFacing = TRUE,
                                sector.index = si
                            )
                        }
                    }
                },
                bg.border = NA
            ) # here set bg.border to NA is important
        }
    } else {
        allnames <- unique(c(from_vals, to_vals))
        p <- ~ {
            circlize::chordDiagram(
                data,
                grid.col = grid_cols,
                col = link_cols,
                transparency = 1 - alpha,
                direction = 1,
                annotationTrack = "grid",
                direction.type = c("diffHeight", "arrows"),
                link.arr.type = "big.arrow",
                link.arr.length = .04,
                preAllocateTracks = list(
                    list(track.height = max(strwidth(allnames))),
                    list(track.height = circlize::mm_h(.1))
                )
            )
            circlize::circos.track(
                track.index = 1,
                panel.fun = function(x, y) {
                    circlize::circos.text(
                        circlize::CELL_META$xcenter,
                        circlize::CELL_META$ylim[1] + .15,
                        circlize::CELL_META$sector.index,
                        facing = "clockwise",
                        niceFacing = TRUE,
                        adj = c(0, 0.5)
                    )
                },
                bg.border = NA
            )
            circlize::circos.track(
                track.index = 2,
                panel.fun = function(x, y) {
                    for (si in circlize::get.all.sector.index()) {
                        start.degree <- circlize::get.cell.meta.data(
                            "cell.start.degree",
                            sector.index = si
                        )
                        end.degree <- circlize::get.cell.meta.data(
                            "cell.end.degree",
                            sector.index = si
                        )
                        if (abs(end.degree - start.degree) > 2) {
                            # otherwise: patchwork wrap_elements 'x' and 'units' must have length > 0
                            circlize::circos.axis(
                                h = "top",
                                labels.cex = 0.7,
                                labels.niceFacing = TRUE,
                                sector.index = si
                            )
                        }
                    }
                },
                bg.border = NA
            ) # here set bg.border to NA is important
        }
    }
    p <- wrap_elements(full = p)

    if (!is.null(title) || !is.null(subtitle)) {
        p <- p +
            plot_annotation(
                title = title,
                subtitle = subtitle,
                theme = do_call(theme, theme_args)
            )
    }
    # allow to access data from the plot object
    p$data <- data

    base_size <- 7
    if (isTRUE(labels_rot)) {
        maxchar <- max(c(nchar(from_vals), nchar(to_vals)))
        if (maxchar < 16) {
            base_size <- base_size + 2
        } else if (maxchar < 32) {
            base_size <- base_size + 4
        } else {
            base_size <- base_size + 6
        }
    }

    attr(p, "height") <- base_size
    attr(p, "width") <- base_size

    p
}

#' Chord / Circos plot
#'
#' @description
#' Draws a chord diagram (also known as a circos plot) to visualise
#' relationships between two categorical variables.  Categories are arranged
#' around a circle, and connecting ribbons (links) represent the flow or
#' association between source and target nodes.  The width of each link is
#' proportional to the associated numeric value or observation count.
#'
#' The function supports \strong{count aggregation} (omit \code{y} to plot
#' observation counts per pair), \strong{link colouring} by source or target
#' node, \strong{label rotation} options, and splitting into separate
#' sub-diagrams via \code{split_by}.
#'
#' \code{CircosPlot} is an alias of \code{ChordPlot}.
#'
#' @section split_by workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
#'         normalise the \code{keep_na} / \code{keep_empty} arguments for all
#'         columns (\code{split_by}, \code{from}, \code{to}).
#'   \item The \code{split_by} column is validated and its NA / empty levels
#'         are processed.  It is then removed from the per-column lists.
#'   \item The data is split by \code{split_by} (preserving level order).  If
#'         \code{split_by} is \code{NULL}, the data is wrapped in a
#'         single-element list with name \code{"..."}.
#'   \item Per-split \code{palette} and \code{palcolor} are resolved via
#'         \code{\link{check_palette}()} and \code{\link{check_palcolor}()}.
#'   \item \code{\link{ChordPlotAtomic}()} is called for each split.
#'         When \code{title} is a function, it receives the split level name
#'         for dynamic titles.
#'   \item Results are combined via \code{\link{combine_plots}()}.
#' }
#'
#' @inheritParams common_args
#' @inheritParams ChordPlotAtomic
#' @param split_by The column(s) to split the data by for separate sub-diagrams.
#'  Multiple columns are concatenated with \code{split_by_sep}.
#' @param split_by_sep A character string to separate concatenated
#'  \code{split_by} columns.  Default \code{"_"}.
#' @param seed A numeric seed for reproducibility.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'  \code{patchwork} object.  When \code{FALSE}, returns a named list of
#'  individual wrapped elements.
#' @param ncol,nrow Integer number of columns / rows for the combined layout.
#' @param byrow Logical; fill the combined layout by row (default \code{TRUE}).
#' @param axes,axis_titles Character strings for axis handling in the
#'  combined layout.
#' @param guides Character string for legend collection across panels.
#' @param design A custom layout design for the combined plot.
#' @return A \code{patchwork} object or a named list of wrapped elements
#'  (when \code{combine = FALSE}), each with \code{height} and \code{width}
#'  attributes in inches.
#' @rdname chordplot
#' @export
#' @examples
#' \donttest{
#' set.seed(8525)
#' data <- data.frame(
#'     nodes1 = sample(c("Soure1", "Source2", "Source3"), 10, replace = TRUE),
#'     nodes2 = sample(letters[1:3], 10, replace = TRUE),
#'     y = sample(1:5, 10, replace = TRUE)
#' )
#'
#' # Basic chord diagram (counts)
#' ChordPlot(data, from = "nodes1", to = "nodes2")
#'
#' # Links coloured by target + rotated labels
#' ChordPlot(data, from = "nodes1", to = "nodes2",
#'           links_color = "to", labels_rot = TRUE)
#'
#' # With explicit y values (link thickness)
#' ChordPlot(data, from = "nodes1", to = "nodes2", y = "y")
#'
#' # Split by a column — one diagram per split level
#' ChordPlot(data, from = "nodes1", to = "nodes2", split_by = "y")
#'
#' # Per-split palettes
#' ChordPlot(data, from = "nodes1", to = "nodes2", split_by = "y",
#'           palette = c("1" = "Reds", "2" = "Blues",
#'                       "3" = "Greens", "4" = "Purp"))
#'
#' # Flip source/target direction
#' ChordPlot(data, from = "nodes1", to = "nodes2", flip = TRUE)
#' }
ChordPlot <- function(
    data,
    y = NULL,
    from = NULL,
    from_sep = "_",
    to = NULL,
    to_sep = "_",
    split_by = NULL,
    split_by_sep = "_",
    flip = FALSE,
    links_color = c("from", "to"),
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 0.5,
    labels_rot = FALSE,
    title = NULL,
    subtitle = NULL,
    seed = 8525,
    keep_na = FALSE,
    keep_empty = FALSE,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed)
    keep_na <- check_keep_na(keep_na, c(split_by, from, to))
    keep_empty <- check_keep_empty(keep_empty, c(split_by, from, to))
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }

    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            ChordPlotAtomic(
                datas[[nm]],
                y = y,
                from = from,
                from_sep = from_sep,
                to = to,
                to_sep = to_sep,
                flip = flip,
                links_color = links_color,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                alpha = alpha,
                labels_rot = labels_rot,
                title = title,
                subtitle = subtitle,
                keep_na = keep_na,
                keep_empty = keep_empty,
                ...
            )
        }
    )
    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

#' @rdname chordplot
#' @description
#' \code{CircosPlot} is an alias for \code{\link{ChordPlot}}.
#' @export
CircosPlot <- ChordPlot

set.seed(8525)
# ChordPlot expects long-format data with from/to columns
# or can compute counts automatically
chord_data <- data.frame(
    from = factor(c("A", "A", "B", "B", "C", "C", "A", "B", "C"),
                  levels = c("A", "B", "C")),
    to   = factor(c("X", "Y", "X", "Z", "Y", "Z", "Z", "Y", "X"),
                  levels = c("X", "Y", "Z")),
    value = c(10, 20, 15, 8, 12, 18, 5, 9, 14),
    group = factor(c("G1", "G1", "G1", "G1", "G2", "G2", "G2", "G2", "G2"))
)

test_that("ChordPlot basic usage works", {
    p <- ChordPlot(chord_data, from = "from", to = "to")
    expect_true(!is.null(p))
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("ChordPlot with y (value) column works", {
    p <- ChordPlot(chord_data, from = "from", to = "to", y = "value")
    expect_true(!is.null(p))
})

test_that("ChordPlot with title and subtitle works", {
    p <- ChordPlot(chord_data, from = "from", to = "to",
                   title = "Chord Title", subtitle = "Subtitle")
    expect_true(!is.null(p))
    expect_true(!is.null(attr(p, "height")))
})

test_that("ChordPlot with split_by returns patchwork", {
    p <- ChordPlot(chord_data, from = "from", to = "to",
                   split_by = "group", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("ChordPlot with combine = FALSE returns list", {
    plots <- ChordPlot(chord_data, from = "from", to = "to",
                       split_by = "group", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
})

test_that("ChordPlot with multiple split_by columns returns a list with one plot per combination", {
    data2 <- chord_data
    data2$split2 <- factor(rep(c("s1", "s2"), length.out = 9))
    plots <- suppressMessages(ChordPlot(data2, from = "from", to = "to",
                                        split_by = c("group", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("ChordPlot with multiple split_by columns returns combined plot", {
    data2 <- chord_data
    data2$split2 <- factor(rep(c("s1", "s2"), length.out = 9))
    p <- suppressMessages(ChordPlot(data2, from = "from", to = "to",
                                    split_by = c("group", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("ChordPlot with flip = TRUE works", {
    p <- ChordPlot(chord_data, from = "from", to = "to", flip = TRUE)
    expect_true(!is.null(p))
})

test_that("ChordPlot with links_color = 'to' works", {
    p <- ChordPlot(chord_data, from = "from", to = "to", links_color = "to")
    expect_true(!is.null(p))
})

test_that("CircosPlot is an alias for ChordPlot", {
    p <- CircosPlot(chord_data, from = "from", to = "to")
    expect_true(!is.null(p))
})

#' Atomic radar/spider plot (internal)
#'
#' @description
#' Core implementation for drawing a single radar (or spider) chart.  This
#' is the workhorse behind the exported \code{\link{RadarPlot}} and
#' \code{\link{SpiderPlot}} functions — it takes a **single** data frame
#' (no \code{split_by} support) and returns a \code{ggplot} object.
#'
#' Radar charts display multivariate data on a two-dimensional polar
#' coordinate system where each variable (x-axis category) is placed
#' along a radial axis evenly spaced around the circle.  The data values
#' are connected by lines forming a polygon (or multiple polygons when
#' \code{group_by} is provided).  When \code{polygon = FALSE} (the
#' default) the grid is rendered as concentric circles (classic radar
#' chart); when \code{polygon = TRUE} straight polygonal grid lines are
#' used (spider chart variant).
#'
#' The polar coordinate system is implemented via a local
#' \code{coord_radar()} helper that extends
#' \code{ggplot2::CoordPolar} with \code{is_linear = TRUE}, allowing
#' discrete x-axis positions to be mapped to evenly spaced angular
#' coordinates.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{Column resolution} — \code{x}, \code{y},
#'         \code{group_by}, and \code{facet_by} are validated and
#'         transformed via \code{\link{check_columns}}.  Multi-column
#'         inputs for \code{x} and \code{group_by} are concatenated
#'         using \code{x_sep} and \code{group_by_sep}.
#'   \item \strong{Group setup} — When \code{group_by} is \code{NULL},
#'         a dummy \code{.group} factor is created so polygons still
#'         draw.  The legend is suppressed.  When \code{groups} is
#'         provided, data is filtered to only those group levels.
#'   \item \strong{Count aggregation} — When \code{y = NULL}, the count
#'         of observations in each unique (\code{x}, \code{group_by},
#'         \code{facet_by}) combination is computed.  Factor levels
#'         are preserved after aggregation.
#'   \item \strong{Proportion scaling} — The \code{scale_y} argument
#'         controls y-value normalisation: \code{"group"} scales within
#'         each group, \code{"global"} within each facet (or the whole
#'         dataset), \code{"x"} within each x category, and
#'         \code{"none"} leaves values raw.  Percent labels are used
#'         for scaled modes.
#'   \item \strong{NA / empty-level handling} —
#'         \code{\link{process_keep_na_empty}()} applies \code{keep_na}
#'         and \code{keep_empty} policies.  Per-column \code{keep_empty}
#'         settings for \code{x}, \code{group_by}, and \code{facet_by}
#'         are extracted; facet columns must share the same value.
#'   \item \strong{Coordinate setup} — The local \code{coord_radar()}
#'         creates a \code{CoordPolar} subclass with
#'         \code{is_linear = TRUE}, placing discrete x positions at
#'         evenly spaced angles.
#'   \item \strong{Y-axis range} — \code{y_min}, \code{y_max}, and
#'         \code{y_nbreaks} determine the radial axis limits and
#'         the number of concentric grid lines.
#'   \item \strong{Colour mapping} — \code{\link{palette_this}()}
#'         assigns colours to all \code{group_by} levels, including
#'         \code{NA} (defaulting to \code{"grey80"}).
#'   \item \strong{Background layer} — When \code{polygon = TRUE}, a
#'         polygonal background fills the area between \code{y_min} and
#'         \code{y_max}.  When \code{polygon = FALSE}, a smooth circular
#'         background is interpolated via 360 sample points.
#'   \item \strong{Radial grid lines} — \code{geom_path()} draws
#'         radial lines from \code{y_min} to \code{y_max} at each
#'         x-axis position, respecting facets.
#'   \item \strong{Polygon grid} — When \code{polygon = TRUE},
#'         concentric polygonal grid lines are drawn at each break
#'         level via \code{geom_polygon()}.
#'   \item \strong{Data rendering} — \code{geom_polygon()} draws the
#'         filled (or unfilled) polygons for each group.
#'         \code{geom_point()} adds points at each observation.
#'         Radial axis labels are rendered as text at a fixed
#'         angular offset.
#'   \item \strong{Scale configuration} — \code{scale_y_continuous()}
#'         sets the radial axis limits and breaks.
#'         \code{scale_x_discrete()} positions the category labels.
#'         Colour scales use \code{scale_fill_manual()} and
#'         \code{scale_color_manual()} with \code{drop} controlled
#'         by \code{keep_empty_group}.
#'   \item \strong{Circular grid lines} — When \code{polygon = FALSE},
#'         dashed circular grid lines are styled via
#'         \code{panel.grid.major.y}.
#'   \item \strong{Dimension calculation} —
#'         \code{\link{calculate_plot_dimensions}()} computes plot
#'         height and width from \code{aspect.ratio}, legend metrics,
#'         and a base height.
#'   \item \strong{Faceting} — \code{\link{facet_plot}()} wraps the
#'         plot with \code{facet_wrap} / \code{facet_grid} if
#'         \code{facet_by} is supplied, respecting the
#'         \code{keep_empty} setting for facet variables.
#' }
#'
#' @inheritParams common_args
#' @param x A character string specifying the column name of the data
#'  frame to plot on the x-axis (the radial categories).  Must be
#'  character or factor.  Multiple columns can be provided; they are
#'  concatenated with \code{x_sep} as the separator.
#' @param x_sep A character string used to join multiple \code{x}
#'  columns.  Default \code{"_"}.  Ignored when \code{x} is a single
#'  column.
#' @param y A character string specifying the numeric column for the
#'  radial axis.  When \code{NULL}, the count of observations in each
#'  (\code{x}, \code{group_by}, \code{facet_by}) combination is used.
#' @param group_by A character vector of column names to group the
#'  data into separate filled polygons.  Each unique combination
#'  becomes a distinct polygon layer.  Multiple columns are
#'  concatenated with \code{group_by_sep}.  When \code{NULL}, a
#'  single polygon is drawn with no legend.
#' @param group_by_sep A character string used to join multiple
#'  \code{group_by} columns.  Default \code{"_"}.
#' @param group_name A character string used as the colour/fill legend
#'  title.  When \code{NULL}, the \code{group_by} column name is used.
#' @param groups A character vector of group values (in the
#'  \code{group_by} column) to include in the plot.  When \code{NULL},
#'  all groups are included.  This can control which groups appear and
#'  their legend order.  Implies \code{keep_empty = FALSE} for the
#'  \code{group_by} column: groups not present in the data are not
#'  shown in the legend.
#' @param scale_y How should the radial axis be scaled?  Default is
#'  \code{"group"}.  Options are \code{"group"}, \code{"global"},
#'  \code{"x"}, and \code{"none"}.
#'  \itemize{
#'    \item{\code{"group"} — scaled to the fraction within each group.}
#'    \item{\code{"global"} — scaled to the fraction of the total.}
#'    \item{\code{"x"} — scaled to the fraction within each x-axis category.}
#'    \item{\code{"none"} — raw counts or values, no scaling.}
#'  }
#' @param y_min A numeric value setting the minimum of the radial
#'  axis.  Default \code{0}.
#' @param y_max A numeric value setting the maximum of the radial
#'  axis.  When \code{NULL}, the maximum data value is used.
#' @param y_nbreaks A numeric value for the number of breaks
#'  (concentric grid lines) on the radial axis.  Default \code{4}.
#' @param polygon A logical value.  When \code{TRUE}, the background
#'  grid is drawn as a polygon (spider chart style); when
#'  \code{FALSE} (default), concentric circles are used (radar chart
#'  style).
#' @param bg_color A character string specifying the background fill
#'  colour.  Default \code{"grey80"}.
#' @param bg_alpha A numeric value for the transparency of the
#'  background fill.  Default \code{0.1}.
#' @param fill A logical value.  When \code{TRUE} (default), the data
#'  polygons are filled with the group colour.  When \code{FALSE},
#'  only outlines are drawn.
#' @param linewidth A numeric value for the width of the polygon
#'  outline lines.  Default \code{1}.
#' @param pt_size A numeric value for the size of the data point
#'  markers.  Default \code{4}.
#' @param max_charwidth A numeric value for the maximum character
#'  width of x-axis labels before wrapping.  Default \code{16}.
#' @keywords internal
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @importFrom rlang syms
#' @importFrom dplyr summarise n %>% distinct ungroup
#' @importFrom tidyr complete
#' @importFrom ggplot2 geom_polygon geom_point geom_text scale_y_continuous scale_x_discrete scale_fill_manual
#' @importFrom ggplot2 scale_color_manual coord_polar labs theme element_blank element_line element_text element_rect
#' @importFrom ggplot2 ggproto CoordPolar waiver
RadarPlotAtomic <- function(
    data,
    x,
    x_sep = "_",
    group_by = NULL,
    group_by_sep = "_",
    y = NULL,
    group_name = NULL,
    groups = NULL,
    scale_y = c("group", "global", "x", "none"),
    y_min = 0,
    y_max = NULL,
    y_nbreaks = 4,
    polygon = FALSE,
    fill = TRUE,
    linewidth = 1,
    pt_size = 4,
    max_charwidth = 16,
    bg_color = "grey80",
    bg_alpha = 0.1,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    alpha = 0.2,
    aspect.ratio = 1,
    legend.position = waiver(),
    legend.direction = "vertical",
    keep_na = FALSE,
    keep_empty = FALSE,
    title = NULL,
    subtitle = NULL,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    x <- check_columns(
        data,
        x,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = x_sep
    )
    y <- check_columns(data, y)
    group_by <- check_columns(
        data,
        group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = group_by_sep
    )
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )

    if (is.null(group_by)) {
        data$.group <- factor("")
        group_by <- ".group"
        legend.position <- ifelse(
            inherits(legend.position, "waiver"),
            "none",
            "right"
        )
    } else {
        if (!is.null(groups)) {
            data <- data[data[[group_by]] %in% groups, , drop = FALSE]
            data[[group_by]] <- droplevels(data[[group_by]])
        }
        legend.position <- ifelse(
            inherits(legend.position, "waiver"),
            "right",
            legend.position
        )
    }

    orig_data <- data
    if (is.null(y)) {
        y <- ".count"
        data <- data %>%
            dplyr::group_by(!!!syms(unique(c(x, group_by, facet_by)))) %>%
            summarise(.count = n(), .groups = "drop")

        for (col in unique(c(x, group_by, facet_by))) {
            data[[col]] <- factor(
                data[[col]],
                levels = levels(orig_data[[col]])
            )
        }
    }

    scale_y <- match.arg(scale_y)
    if (scale_y == "group") {
        data <- data %>%
            dplyr::group_by(!!!syms(unique(c(group_by, facet_by)))) %>%
            mutate(!!sym(y) := !!sym(y) / sum(!!sym(y))) %>%
            ungroup()
        for (col in unique(c(group_by, facet_by))) {
            data[[col]] <- factor(
                data[[col]],
                levels = levels(orig_data[[col]])
            )
        }
    } else if (scale_y == "global") {
        if (!is.null(facet_by)) {
            data <- data %>%
                dplyr::group_by(!!!syms(unique(facet_by))) %>%
                mutate(!!sym(y) := !!sym(y) / sum(!!sym(y))) %>%
                ungroup()
            for (col in unique(facet_by)) {
                data[[col]] <- factor(
                    data[[col]],
                    levels = levels(orig_data[[col]])
                )
            }
        } else {
            data <- data %>%
                mutate(!!sym(y) := !!sym(y) / sum(!!sym(y)))
        }
    } else if (scale_y == "x") {
        data <- data %>%
            dplyr::group_by(!!!syms(unique(c(x, facet_by)))) %>%
            mutate(!!sym(y) := !!sym(y) / sum(!!sym(y))) %>%
            ungroup()
        for (col in unique(c(x, facet_by))) {
            data[[col]] <- factor(
                data[[col]],
                levels = levels(orig_data[[col]])
            )
        }
    }
    rm(orig_data)

    data <- process_keep_na_empty(data, keep_na, keep_empty)
    keep_empty_x <- keep_empty[[x]]
    keep_empty_group <- if (!is.null(group_by)) keep_empty[[group_by]] else NULL
    keep_empty_facet <- if (!is.null(facet_by)) {
        keep_empty[[facet_by[1]]]
    } else {
        NULL
    }
    if (length(facet_by) > 1) {
        stopifnot(
            "[RadarPLot/SpiderPlot] `keep_empty` for `facet_by` variables must be identical." = identical(
                keep_empty_facet,
                keep_empty[[facet_by[2]]]
            )
        )
    }

    # We need all levels of x in each group and facet
    # fill the y value with 0 if the level is missing
    # complete_fill = list(0)
    # names(complete_fill) = y
    # data <- data %>%
    #     dplyr::group_by(!!!syms(unique(c(group_by, facet_by)))) %>%
    #     complete(!!sym(x), fill = complete_fill) %>%
    #     ungroup()

    coord_radar <- function(theta = "x", start = 0, direction = 1) {
        theta <- match.arg(theta, c("x", "y"))
        r <- if (theta == "x") "y" else "x"
        ggproto(
            "CoordRadar",
            CoordPolar,
            theta = theta,
            r = r,
            start = start,
            clip = "off",
            direction = sign(direction),
            is_linear = function(coord) TRUE
        )
    }

    y_min <- y_min %||% min(data[[y]])
    y_max <- y_max %||% max(data[[y]])
    breaks <- seq(y_min, y_max, length.out = y_nbreaks)
    group_vals <- levels(data[[group_by]])
    if (anyNA(data[[group_by]])) {
        group_vals <- c(group_vals, NA)
    }
    group_colors <- palette_this(
        group_vals,
        palette = palette,
        palcolor = palcolor,
        NA_keep = TRUE,
        reverse = palreverse
    )

    # Add background (circle or polygon based on polygon parameter)
    x_levels <- levels(data[[x]])
    if (anyNA(data[[x]])) {
        # Otherwise there will be an angle shift if NA is present
        x_levels <- c(x_levels, "NA")
        levels(data[[x]]) <- x_levels
        data[[x]][is.na(data[[x]])] <- "NA"
    }
    n_x <- length(x_levels)
    p <- ggplot(
        data,
        aes(
            x = !!sym(x),
            y = !!sym(y),
            group = !!sym(group_by),
            color = !!sym(group_by)
        )
    )

    if (polygon) {
        # Polygonal background - connects discrete x-axis points
        bg_df <- data.frame(
            x = factor(
                c(x_levels, x_levels[1], rev(x_levels), rev(x_levels)[1]),
                levels = x_levels
            ),
            y = c(rep(y_max, n_x), y_max, rep(y_min, n_x), y_min),
            bg_group = 1
        )

        p <- p +
            geom_polygon(
                data = bg_df,
                aes(x = !!sym("x"), y = !!sym("y"), group = !!sym("bg_group")),
                fill = bg_color,
                alpha = bg_alpha,
                color = NA,
                inherit.aes = FALSE
            )
    } else {
        # Circular background - smooth interpolated circle
        # In polar coordinates with discrete x, we need to go from 0.5 to n_x+0.5
        n_smooth <- 360
        x_positions <- seq(0.5, n_x + 0.5, length.out = n_smooth + 1)

        bg_df <- data.frame(
            x_pos = c(x_positions, rev(x_positions)),
            y = c(rep(y_max, n_smooth + 1), rep(y_min, n_smooth + 1)),
            bg_group = 1
        )

        p <- p +
            geom_polygon(
                data = bg_df,
                aes(
                    x = !!sym("x_pos"),
                    y = !!sym("y"),
                    group = !!sym("bg_group")
                ),
                fill = bg_color,
                alpha = bg_alpha,
                color = NA,
                inherit.aes = FALSE
            )
    }

    # Add radial grid lines that only extend from y_min to y_max
    if (!is.null(facet_by)) {
        grid_df <- data %>% select(!!!syms(facet_by)) %>% distinct()
        grid_df <- expand_grid(grid_df, x = x_levels)
    } else {
        grid_df <- data.frame(x = x_levels)
    }
    # Create two points for each radial line
    n_lines <- nrow(grid_df)
    grid_df <- grid_df[rep(seq_len(n_lines), each = 2), , drop = FALSE]
    grid_df$y <- rep(c(y_min, y_max), times = n_lines)
    # Create a unique identifier for grouping each radial line
    grid_df$line_id <- rep(seq_len(n_lines), each = 2)

    p <- p +
        geom_path(
            data = grid_df,
            aes(x = !!sym("x"), y = !!sym("y"), group = !!sym("line_id")),
            colour = "grey80",
            linetype = 1,
            inherit.aes = FALSE
        )
    if (isTRUE(polygon)) {
        # Create proper polygon grid at each break level
        for (i in seq_along(breaks)) {
            if (!is.null(facet_by)) {
                poly_grid_df <- data %>%
                    select(!!!syms(facet_by)) %>%
                    distinct()
                poly_grid_df <- expand_grid(poly_grid_df, x = x_levels)
            } else {
                poly_grid_df <- data.frame(x = x_levels)
            }
            poly_grid_df$y <- breaks[i]
            poly_grid_df$poly_id <- i

            p <- p +
                geom_polygon(
                    data = poly_grid_df,
                    aes(
                        x = !!sym("x"),
                        y = !!sym("y"),
                        group = !!sym("poly_id")
                    ),
                    fill = "transparent",
                    color = "grey80",
                    linetype = 2,
                    inherit.aes = FALSE
                )
        }
    }

    if (isTRUE(fill)) {
        p <- p +
            geom_polygon(
                aes(fill = !!sym(group_by)),
                alpha = alpha,
                linewidth = linewidth,
                show.legend = TRUE
            )
    } else {
        p <- p +
            geom_polygon(
                alpha = alpha,
                linewidth = linewidth,
                fill = "transparent"
            )
    }

    if (!is.null(facet_by)) {
        labels_df <- data %>% select(!!!syms(facet_by)) %>% distinct()
        labels_df <- expand_grid(labels_df, breaks)
    } else {
        labels_df <- data.frame(breaks = breaks)
    }

    mc <- min(max(nchar(levels(data[[x]]))), max_charwidth)
    panel.spacing <- theme_args$panel.spacing %||% unit(mc * 2, "points")
    theme_args$panel.spacing <- NULL
    p <- p +
        geom_point(
            aes(colour = !!sym(group_by), fill = !!sym(group_by)),
            size = pt_size,
            shape = 21,
            show.legend = TRUE
        ) +
        geom_text(
            data = labels_df,
            mapping = aes(
                y = !!sym("breaks"),
                label = if (scale_y == "none") {
                    scales::number(!!sym("breaks"))
                } else {
                    scales::percent(!!sym("breaks"))
                }
            ),
            x = pi / (length(breaks) - 0.88),
            color = "grey20",
            inherit.aes = FALSE
        ) +
        scale_y_continuous(
            limits = c(y_min, y_max),
            breaks = breaks,
            expand = c(.1, 0, 0, 0)
        ) +
        scale_x_discrete(
            labels = scales::label_wrap(max_charwidth),
            drop = !isTRUE(keep_empty_x)
        ) +
        # scale_fill_manual(
        #     name = group_name %||% group_by,
        #     values = palette_this(levels(data[[group_by]]), palette = palette, palcolor = palcolor)
        # ) +
        # scale_color_manual(
        #     values = palette_this(levels(data[[group_by]]), palette = palette, palcolor = palcolor)
        # ) +
        coord_radar(start = -pi / nlevels(data[[x]])) +
        labs(title = title, subtitle = subtitle) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            axis.text.y = element_blank(),
            axis.ticks = element_blank(),
            axis.title = element_blank(),
            panel.grid = element_blank(),
            panel.border = element_blank(),
            panel.spacing = panel.spacing
        )

    if (isTRUE(keep_empty_group)) {
        p <- p +
            scale_fill_manual(
                name = group_name %||% group_by,
                values = group_colors,
                na.value = group_colors["NA"] %||% "grey80",
                breaks = group_vals,
                limits = group_vals,
                drop = FALSE
            ) +
            scale_color_manual(
                values = group_colors,
                na.value = group_colors["NA"] %||% "grey80",
                breaks = group_vals,
                limits = group_vals,
                drop = FALSE
            )
    } else {
        p <- p +
            scale_fill_manual(
                name = group_name %||% group_by,
                values = group_colors
            ) +
            scale_color_manual(
                values = group_colors
            )
    }

    if (isFALSE(polygon)) {
        p <- p +
            ggplot2::theme(
                panel.grid.major.y = element_line(
                    colour = c(rep("grey80", length(breaks)), NA),
                    linetype = 2
                )
            )
    }

    dims <- calculate_plot_dimensions(
        base_height = 4.5,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = length(group_vals),
        legend_nchar = max(nchar(as.character(group_vals)), na.rm = TRUE)
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction,
        drop = !isTRUE(keep_empty_facet)
    )
}

#' Radar plot / Spider plot
#'
#' @description
#' Draws a radar chart (concentric circular grid) or spider chart
#' (polygonal grid) displaying multivariate data in a two-dimensional
#' polar coordinate system.  Each x-axis category is placed at an
#' evenly spaced angular position around the chart, and numeric values
#' are plotted along the radial axis.
#'
#' The function supports \strong{count aggregation} (omit \code{y} to
#' plot observation counts), \strong{proportion scaling} (via
#' \code{scale_y}), per-group colour control, faceting, and splitting
#' into separate sub-plots via \code{split_by}.
#'
#' \code{\link{SpiderPlot}} is an alias that renders the same data with
#' polygonal grid lines (spider chart style) by using
#' \code{polygon = TRUE}.
#'
#' @section split_by Workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \code{\link{check_keep_na}()} and
#'         \code{\link{check_keep_empty}()} normalise the
#'         \code{keep_na} / \code{keep_empty} arguments for all
#'         relevant columns (\code{x}, \code{split_by},
#'         \code{group_by}, \code{facet_by}).
#'   \item The \code{split_by} column is validated and its NA / empty
#'         levels are processed via
#'         \code{\link{process_keep_na_empty}()}.  It is then removed
#'         from the per-column \code{keep_na} / \code{keep_empty}
#'         lists.
#'   \item The data frame is split by \code{split_by} (preserving
#'         level order).  If \code{split_by} is \code{NULL}, the data
#'         is wrapped in a single-element list with name \code{"..."}.
#'   \item Per-split \code{palette}, \code{palcolor},
#'         \code{legend.position}, and \code{legend.direction} are
#'         resolved via \code{\link{check_palette}()},
#'         \code{\link{check_palcolor}()}, and
#'         \code{\link{check_legend}()}.
#'   \item \code{\link{RadarPlotAtomic}()} is called for each split
#'         with \code{polygon = FALSE}.  If \code{title} is a
#'         function, it receives the split level name and can generate
#'         dynamic titles.
#'   \item Results are combined via \code{\link{combine_plots}()}
#'         (when \code{combine = TRUE}) or returned as a named list.
#' }
#'
#' @rdname radarplot
#' @inheritParams common_args
#' @inheritParams RadarPlotAtomic
#' @param split_by The column(s) to split the data by and produce
#'  separate sub-plots.  Multiple columns are concatenated with
#'  \code{split_by_sep}.
#' @param split_by_sep A character string to separate concatenated
#'  \code{split_by} columns.  Default \code{"_"}.
#' @param seed A numeric seed for reproducibility.  Passed to
#'  \code{\link{validate_common_args}()}.
#' @param combine Logical; when \code{TRUE} (default), returns a
#'  combined \code{patchwork} object.  When \code{FALSE}, returns a
#'  named list of individual \code{ggplot} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined
#'  layout (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param byrow Logical; fill the combined layout by row.  Default
#'  \code{TRUE} (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axes A character string specifying how axes should be
#'  treated across the combined layout (passed to
#'  \code{\link[patchwork]{wrap_plots}}).
#' @param axis_titles A character string specifying how axis titles
#'  should be treated across the combined layout.  Defaults to
#'  \code{axes}.
#' @param guides A character string specifying how guides (legends)
#'  should be collected across panels (passed to
#'  \code{\link{combine_plots}()}).
#' @param design A custom layout design for the combined plot (passed
#'  to \code{\link{combine_plots}()}).
#' @importFrom ggplot2 waiver
#' @return A \code{ggplot} object (when \code{combine = TRUE} and
#'  \code{split_by} is \code{NULL}), a \code{patchwork} object (when
#'  \code{combine = TRUE} and \code{split_by} is provided), or a named
#'  list of \code{ggplot} objects (when \code{combine = FALSE}), each
#'  with \code{height} and \code{width} attributes in inches.
#' @export
RadarPlot <- function(
    data,
    x,
    x_sep = "_",
    group_by = NULL,
    group_by_sep = "_",
    y = NULL,
    group_name = NULL,
    groups = NULL,
    scale_y = c("group", "global", "x", "none"),
    y_min = 0,
    y_max = NULL,
    y_nbreaks = 4,
    bg_color = "grey80",
    bg_alpha = 0.1,
    fill = TRUE,
    linewidth = 1,
    pt_size = 4,
    max_charwidth = 16,
    split_by = NULL,
    split_by_sep = "_",
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    alpha = 0.2,
    aspect.ratio = 1,
    legend.position = waiver(),
    legend.direction = "vertical",
    keep_na = FALSE,
    keep_empty = FALSE,
    title = NULL,
    subtitle = NULL,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    keep_na <- check_keep_na(keep_na, c(x, group_by, facet_by, split_by))
    keep_empty <- check_keep_empty(
        keep_empty,
        c(x, group_by, facet_by, split_by)
    )
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            RadarPlotAtomic(
                datas[[nm]],
                x = x,
                x_sep = x_sep,
                group_by = group_by,
                group_by_sep = group_by_sep,
                y = y,
                group_name = group_name,
                groups = groups,
                scale_y = scale_y,
                y_min = y_min,
                y_max = y_max,
                y_nbreaks = y_nbreaks,
                polygon = FALSE,
                fill = fill,
                linewidth = linewidth,
                pt_size = pt_size,
                max_charwidth = max_charwidth,
                bg_color = bg_color,
                bg_alpha = bg_alpha,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                alpha = alpha,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                keep_na = keep_na,
                keep_empty = keep_empty,
                title = title,
                subtitle = subtitle,
                ...
            )
        }
    )
    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

#' @rdname radarplot
#' @description
#' `SpiderPlot` is a variant of \code{\link{RadarPlot}} that renders the chart with
#' straight polygonal grid lines (spider chart) instead of concentric
#' circles.  Internally, it calls \code{\link{RadarPlotAtomic}} with
#' \code{polygon = TRUE} but is otherwise identical to
#' \code{\link{RadarPlot}} in behaviour and parameters.
#'
#' @inheritParams common_args
#' @inheritParams RadarPlotAtomic
#' @param split_by The column(s) to split the data by and produce
#'  separate sub-plots.  Multiple columns are concatenated with
#'  \code{split_by_sep}.
#' @param split_by_sep A character string to separate concatenated
#'  \code{split_by} columns.  Default \code{"_"}.
#' @param seed A numeric seed for reproducibility.  Passed to
#'  \code{\link{validate_common_args}()}.
#' @param combine Logical; when \code{TRUE} (default), returns a
#'  combined \code{patchwork} object.  When \code{FALSE}, returns a
#'  named list of individual \code{ggplot} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined
#'  layout (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param byrow Logical; fill the combined layout by row.  Default
#'  \code{TRUE} (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axes A character string specifying how axes should be
#'  treated across the combined layout (passed to
#'  \code{\link[patchwork]{wrap_plots}}).
#' @param axis_titles A character string specifying how axis titles
#'  should be treated across the combined layout.  Defaults to
#'  \code{axes}.
#' @param guides A character string specifying how guides (legends)
#'  should be collected across panels (passed to
#'  \code{\link{combine_plots}()}).
#' @param design A custom layout design for the combined plot (passed
#'  to \code{\link{combine_plots}()}).
#' @export
#' @importFrom ggplot2 waiver
#' @return A \code{ggplot} object (when \code{combine = TRUE} and
#'  \code{split_by} is \code{NULL}), a \code{patchwork} object (when
#'  \code{combine = TRUE} and \code{split_by} is provided), or a named
#'  list of \code{ggplot} objects (when \code{combine = FALSE}), each
#'  with \code{height} and \code{width} attributes in inches.
#' @examples
#' \donttest{
#' set.seed(8525)
#'
#' # --- Radar chart with observation counts ---
#' data <- data.frame(
#'     x = factor(
#'         c(rep("A", 20), rep("B", 30), rep(NA, 30), rep("D", 40), rep("E", 50)),
#'         levels = LETTERS[1:5]
#'     ),
#'     group = factor(
#'         sample(c("G1", NA, "G3", "G4"), 170, replace = TRUE),
#'         levels = c("G1", "G2", "G3", "G4")
#'     )
#' )
#'
#' # Basic radar chart
#' RadarPlot(data, x = "x")
#'
#' # Keep NA and empty factor levels
#' RadarPlot(data, x = "x", keep_na = TRUE, keep_empty = TRUE)
#'
#' # Custom background colour
#' RadarPlot(data, x = "x", bg_color = "lightpink")
#'
#' # Raw counts (no proportion scaling)
#' RadarPlot(data, x = "x", scale_y = "none")
#'
#' # Grouped by a variable
#' RadarPlot(data, x = "x", group_by = "group", keep_na = TRUE)
#'
#' # Faceted by a variable
#' RadarPlot(data, x = "x", facet_by = "group")
#'
#' # Spider chart variant (polygonal grid)
#' SpiderPlot(data, x = "x")
#' SpiderPlot(data, x = "x", group_by = "group")
#'
#' # --- Radar chart with explicit y values ---
#' data <- data.frame(
#'     x = rep(LETTERS[1:5], 2),
#'     y = c(1, 3, 6, 4, 2, 5, 7, 8, 9, 10),
#'     group = rep(c("G1", "G2"), each = 5)
#' )
#'
#' # Grouped radar with raw values
#' RadarPlot(data, x = "x", y = "y", scale_y = "none", group_by = "group")
#'
#' # Faceted radar
#' RadarPlot(data, x = "x", y = "y", facet_by = "group")
#'
#' # Split into separate sub-plots
#' RadarPlot(data, x = "x", y = "y", split_by = "group")
#'
#' # Per-split palettes
#' RadarPlot(data, x = "x", y = "y", split_by = "group",
#'           palette = c(G1 = "Set1", G2 = "Paired"))
#' }
SpiderPlot <- function(
    data,
    x,
    x_sep = "_",
    group_by = NULL,
    group_by_sep = "_",
    y = NULL,
    group_name = NULL,
    groups = NULL,
    scale_y = c("group", "global", "x", "none"),
    y_min = 0,
    y_max = NULL,
    y_nbreaks = 4,
    bg_color = "grey80",
    bg_alpha = 0.1,
    fill = TRUE,
    linewidth = 1,
    pt_size = 4,
    max_charwidth = 16,
    split_by = NULL,
    split_by_sep = "_",
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    alpha = 0.2,
    aspect.ratio = 1,
    legend.position = waiver(),
    legend.direction = "vertical",
    keep_na = FALSE,
    keep_empty = FALSE,
    title = NULL,
    subtitle = NULL,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    keep_na <- check_keep_na(keep_na, c(x, group_by, facet_by, split_by))
    keep_empty <- check_keep_empty(
        keep_empty,
        c(x, group_by, facet_by, split_by)
    )
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            RadarPlotAtomic(
                datas[[nm]],
                x = x,
                x_sep = x_sep,
                group_by = group_by,
                group_by_sep = group_by_sep,
                y = y,
                group_name = group_name,
                groups = groups,
                scale_y = scale_y,
                y_min = y_min,
                y_max = y_max,
                y_nbreaks = y_nbreaks,
                polygon = TRUE,
                bg_color = bg_color,
                bg_alpha = bg_alpha,
                fill = fill,
                linewidth = linewidth,
                pt_size = pt_size,
                max_charwidth = max_charwidth,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                alpha = alpha,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                keep_na = keep_na,
                keep_empty = keep_empty,
                title = title,
                subtitle = subtitle,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
# Long-format data for radar/spider plot:
# x = trait, y = numeric value, group_by = group
radar_data <- data.frame(
    trait = rep(c("Str", "Spd", "Def", "Int", "Luck"), times = 6),
    value = c(80, 60, 70, 90, 50, 70, 85, 65, 75, 55,
              60, 90, 55, 80, 70, 75, 80, 70, 85, 65,
              50, 70, 80, 60, 90, 65, 75, 60, 70, 75),
    group = rep(c("A", "B", "C"), each = 10),
    split = factor(rep(c("S1", "S2"), each = 15))
)

test_that("RadarPlot basic usage works", {
    p <- RadarPlot(radar_data, x = "trait", y = "value")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("RadarPlot with group_by works", {
    p <- RadarPlot(radar_data, x = "trait", y = "value", group_by = "group")
    expect_s3_class(p, "ggplot")
})

test_that("RadarPlot with title and subtitle works", {
    p <- RadarPlot(radar_data, x = "trait", y = "value", group_by = "group",
                   title = "Radar Title", subtitle = "Subtitle")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "Radar Title")
    expect_equal(p$labels$subtitle, "Subtitle")
})

test_that("RadarPlot with split_by returns patchwork", {
    p <- RadarPlot(radar_data, x = "trait", y = "value", group_by = "group",
                   split_by = "split", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("RadarPlot with combine = FALSE returns list", {
    plots <- RadarPlot(radar_data, x = "trait", y = "value", group_by = "group",
                       split_by = "split", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("RadarPlot with multiple split_by columns returns a list with one plot per combination", {
    radar_data2 <- radar_data
    radar_data2$split2 <- factor(rep(c("s1", "s2"), 15))
    plots <- suppressMessages(RadarPlot(radar_data2, x = "trait", y = "value", group_by = "group",
                                        split_by = c("split", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("RadarPlot with multiple split_by columns returns combined plot", {
    radar_data2 <- radar_data
    radar_data2$split2 <- factor(rep(c("s1", "s2"), 15))
    p <- suppressMessages(RadarPlot(radar_data2, x = "trait", y = "value", group_by = "group",
                                    split_by = c("split", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("RadarPlot with scale_y = 'global' works", {
    p <- RadarPlot(radar_data, x = "trait", y = "value", group_by = "group",
                   scale_y = "global")
    expect_s3_class(p, "ggplot")
})

test_that("RadarPlot with fill = FALSE works", {
    p <- RadarPlot(radar_data, x = "trait", y = "value", group_by = "group",
                   fill = FALSE)
    expect_s3_class(p, "ggplot")
})

test_that("SpiderPlot basic usage works", {
    p <- SpiderPlot(radar_data, x = "trait", y = "value", group_by = "group")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("SpiderPlot with multiple split_by columns returns a list with one plot per combination", {
    radar_data2 <- radar_data
    radar_data2$split2 <- factor(rep(c("s1", "s2"), 15))
    plots <- suppressMessages(SpiderPlot(radar_data2, x = "trait", y = "value", group_by = "group",
                                         split_by = c("split", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("SpiderPlot with multiple split_by columns returns combined plot", {
    radar_data2 <- radar_data
    radar_data2$split2 <- factor(rep(c("s1", "s2"), 15))
    p <- suppressMessages(SpiderPlot(radar_data2, x = "trait", y = "value", group_by = "group",
                                     split_by = c("split", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("SpiderPlot with title works", {
    p <- SpiderPlot(radar_data, x = "trait", y = "value", title = "Spider Title")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "Spider Title")
})

test_that("SpiderPlot with split_by works", {
    p <- SpiderPlot(radar_data, x = "trait", y = "value", group_by = "group",
                    split_by = "split", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

#' Atomic trend plot (internal)
#'
#' @description
#' Core implementation for drawing a single trend plot. This is the workhorse
#' behind the exported \code{\link{TrendPlot}} function -- it takes a **single**
#' data frame (no \code{split_by} support) and returns a \code{ggplot} object.
#' A trend plot combines stacked bars (\code{\link[ggplot2]{geom_col}()}) with
#' a semi-transparent area background (\code{\link[ggplot2]{geom_area}()}) to
#' show how one or more groups accumulate across a discrete x-axis. This hybrid
#' style sits between a bar plot and an area plot, preserving the discrete
#' category separation of bars while softening the visual with an area fill.
#'
#' The function supports count aggregation (omit \code{y} to plot observation
#' counts per x-category), proportion scaling (\code{scale_y = TRUE} normalises
#' each x position to 100\%), per-group colour control, and faceting.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{Column resolution} -- \code{x}, \code{y}, \code{group_by},
#'         and \code{facet_by} are validated and transformed via
#'         \code{\link{check_columns}}. Multi-column inputs for \code{x} and
#'         \code{group_by} are concatenated into single columns using their
#'         respective separators (\code{x_sep}, \code{group_by_sep}).
#'   \item \strong{Count aggregation} -- when \code{y = NULL}, the count of
#'         observations in each unique (\code{x}, \code{group_by},
#'         \code{facet_by}) combination is computed as a new \code{.count}
#'         column. Factor levels are preserved after aggregation.
#'   \item \strong{Proportion scaling} -- when \code{scale_y = TRUE}, the
#'         y-values are divided by the sum within each (\code{x},
#'         \code{facet_by}) group, producing a proportion (0--1). Percent
#'         labels are used automatically on the y-axis.
#'   \item \strong{NA / empty-level handling} --
#'         \code{\link{process_keep_na_empty}()} applies \code{keep_na} and
#'         \code{keep_empty} policies. Per-column \code{keep_empty} settings
#'         are extracted for \code{x}, \code{group_by}, and \code{facet_by}
#'         independently. The facet columns must have identical
#'         \code{keep_empty} values. Note that \code{keep_empty = TRUE} is
#'         not supported globally because empty categories would break the
#'         continuity of the x-axis.
#'   \item \strong{Group fill setup} -- when \code{group_by = NULL}, a dummy
#'         \code{.fill} factor is created so the single group still draws with
#'         the first palette colour. The legend is suppressed
#'         (\code{legend.position = "none"}).
#'   \item \strong{Data completion} -- \code{\link[tidyr]{complete}()} pads
#'         all \code{x} by \code{group_by} (by \code{facet_by})
#'         combinations with \code{y = 0}. This prevents
#'         \code{\link[ggplot2]{geom_area}()} from interpolating across missing
#'         groups, which would otherwise cause stacked areas to exceed the
#'         correct total.
#'   \item \strong{Area layer construction} -- the completed data is expanded
#'         by duplicating each row and offsetting the x-axis positions by
#'         \eqn{\pm 0.2} on a numeric scale, creating discrete-width area
#'         strips that align perfectly with the bars.
#'   \item \strong{Colour mapping} -- \code{\link{palette_this}()} assigns
#'         colours to all \code{group_by} levels, including \code{NA}
#'         (defaulting to \code{"grey80"}).
#'   \item \strong{Plot assembly} -- the \code{ggplot} object is built with
#'         \code{geom_area()} (at half \code{alpha}, grey outline) as a
#'         background fill, overlaid with \code{geom_col()} (full \code{alpha},
#'         black outline, width 0.4) as the foreground bars. Both layers use
#'         \code{position_stack(vjust = 0.5)}. The x-axis uses
#'         \code{scale_x_discrete()} and the y-axis uses
#'         \code{scale_y_continuous()} (percent labels when scaled). The
#'         fill scale \code{drop} argument is controlled by
#'         \code{keep_empty_group}.
#'   \item \strong{Dimension calculation} --
#'         \code{\link{calculate_plot_dimensions}()} computes plot height and
#'         width from the x-axis category count, \code{aspect.ratio}, and
#'         legend metrics. The base height increases for plots with 10 or more
#'         x categories (from 4.5 to 6.5 inches). The resulting \code{height}
#'         and \code{width} attributes are stored on the \code{ggplot} object.
#'   \item \strong{Faceting} -- \code{\link{facet_plot}()} wraps the plot with
#'         \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is
#'         provided, respecting the \code{keep_empty} setting for facet
#'         variables.
#' }
#'
#' @inheritParams common_args
#' @param x A character string specifying the column name to plot on the
#'  x-axis. Must be character or factor. Multiple columns can be provided;
#'  they are concatenated with \code{x_sep} as the separator.
#' @param x_sep A character string used to join multiple \code{x} columns.
#'  Default \code{"_"}. Ignored when \code{x} is a single column.
#' @param y A character string specifying the numeric column for the y-axis.
#'  When \code{NULL}, the count of observations in each (\code{x},
#'  \code{group_by}, \code{facet_by}) combination is used.
#' @param scale_y A logical value. When \code{TRUE}, y-values are scaled to
#'  proportions within each (\code{x}, \code{facet_by}) group so that each
#'  x position stacks to 1.0. The y-axis labels switch from numeric to
#'  percent format automatically.
#' @param group_by A character vector of column names to fill the bars and
#'  areas by. Each unique combination becomes a separate stacked segment.
#'  Multiple columns are concatenated with \code{group_by_sep}. When
#'  \code{NULL}, a single filled bar/area is drawn and the legend is hidden.
#' @param group_by_sep A character string to separate concatenated
#'  \code{group_by} columns. Default \code{"_"}.
#' @param group_name A character string used as the fill legend title.
#'  When \code{NULL}, the \code{group_by} column name is used.
#' @keywords internal
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @importFrom rlang syms :=
#' @importFrom dplyr summarise n %>%
#' @importFrom tidyr complete
#' @importFrom ggplot2 geom_area geom_col scale_x_discrete scale_y_continuous scale_fill_manual
#' @importFrom ggplot2 labs theme element_line element_text position_stack waiver
TrendPlotAtomic <- function(
    data,
    x,
    y = NULL,
    x_sep = "_",
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    scale_y = FALSE,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    x_text_angle = 0,
    aspect.ratio = 1,
    legend.position = waiver(),
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_empty = FALSE,
    keep_na = FALSE,
    ...
) {
    if (isTRUE(keep_empty)) {
        stop(
            "[TrendPlot] 'keep_empty' = TRUE is not supported for TrendPlot as it would break the continuity of the plot."
        )
    }
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    x <- check_columns(
        data,
        x,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = x_sep
    )
    y <- check_columns(data, y)
    group_by <- check_columns(
        data,
        group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = group_by_sep
    )
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )

    orig_data <- data
    if (is.null(y)) {
        y <- ".count"
        data <- data %>%
            dplyr::group_by(!!!syms(unique(c(x, group_by, facet_by)))) %>%
            summarise(.count = n(), .groups = "drop")

        for (col in unique(c(x, group_by, facet_by))) {
            data[[col]] <- factor(
                data[[col]],
                levels = levels(orig_data[[col]])
            )
        }
    }

    if (isTRUE(scale_y)) {
        data <- data %>%
            dplyr::group_by(!!!syms(unique(c(x, facet_by)))) %>%
            mutate(!!sym(y) := !!sym(y) / sum(!!sym(y))) %>%
            ungroup()

        for (col in unique(c(x, facet_by))) {
            data[[col]] <- factor(
                data[[col]],
                levels = levels(orig_data[[col]])
            )
        }
    }
    data <- process_keep_na_empty(data, keep_na, keep_empty)
    keep_empty_x <- keep_empty[[x]]
    keep_empty_group <- if (!is.null(group_by)) keep_empty[[group_by]] else NULL
    keep_empty_facet <- if (!is.null(facet_by)) {
        keep_empty[[facet_by[1]]]
    } else {
        NULL
    }
    if (length(facet_by) > 1) {
        stopifnot(
            "[TrendPlot] `keep_empty` for `facet_by` variables must be identical." = identical(
                keep_empty_facet,
                keep_empty[[facet_by[2]]]
            )
        )
    }

    if (is.null(group_by)) {
        data$.fill <- factor("")
        group_by <- ".fill"
        legend.position <- ifelse(
            inherits(legend.position, "waiver"),
            "none",
            "right"
        )
    } else {
        # data[[group_by]] <- droplevels(data[[group_by]])
        # # fill up some missing group_by values for each x, and fill it with 0 for y
        # fill_levels <- levels(data[[group_by]])
        # complete_fill <- list(0)
        # names(complete_fill) <- y
        # data <- data %>%
        #     dplyr::group_by(!!!syms(unique(c(group_by, facet_by)))) %>%
        #     complete(!!sym(x), fill = complete_fill) %>%
        #     ungroup()
        legend.position <- ifelse(
            inherits(legend.position, "waiver"),
            "right",
            legend.position
        )
    }
    # Complete all x * group_by (and facet_by) combinations for the area layer
    # so that geom_area doesn't interpolate across missing groups, which would
    # cause stacked areas to exceed the correct total.
    complete_vars <- unique(c(x, group_by, facet_by))
    complete_fill <- setNames(list(0), y)
    data_for_area <- data %>%
        tidyr::complete(!!!syms(complete_vars), fill = complete_fill)
    # Restore factor levels that complete() may have altered
    for (col in complete_vars) {
        if (is.factor(data[[col]])) {
            data_for_area[[col]] <- factor(
                data_for_area[[col]],
                levels = levels(data[[col]])
            )
        }
    }

    nr <- nrow(data_for_area)
    dat_area <- data_for_area[rep(seq_len(nr), each = 2), , drop = FALSE]
    # Convert factor to numeric, handling NA values
    x_numeric <- as.numeric(dat_area[[x]])
    # For NA values in factors, assign them the position after all levels
    na_pos <- nlevels(dat_area[[x]]) + 1
    x_numeric[is.na(x_numeric)] <- na_pos
    dat_area[[x]] <- x_numeric
    dat_area[seq(1, nr * 2, 2), x] <- dat_area[seq(1, nr * 2, 2), x] - 0.2
    dat_area[seq(2, nr * 2, 2), x] <- dat_area[seq(2, nr * 2, 2), x] + 0.2

    group_vals <- levels(data[[group_by]])
    if (anyNA(data[[group_by]])) {
        group_vals <- c(group_vals, NA)
    }
    group_cols <- palette_this(
        group_vals,
        palette = palette,
        palcolor = palcolor,
        NA_keep = TRUE,
        reverse = palreverse
    )

    position <- position_stack(vjust = 0.5)

    just <- calc_just(x_text_angle)
    p <- ggplot(data, aes(x = !!sym(x), y = !!sym(y), fill = !!sym(group_by))) +
        geom_area(
            data = dat_area,
            mapping = aes(x = !!sym(x), fill = !!sym(group_by)),
            alpha = alpha / 2,
            color = "grey50",
            position = position,
            show.legend = TRUE
        ) +
        geom_col(
            aes(fill = !!sym(group_by)),
            width = 0.4,
            color = "black",
            alpha = alpha,
            position = position,
            show.legend = TRUE
        ) +
        scale_x_discrete(expand = c(0, 0), drop = !isTRUE(keep_empty_x)) +
        scale_y_continuous(
            expand = c(0, 0),
            labels = if (isFALSE(scale_y)) scales::number else scales::percent
        ) +
        # scale_fill_manual(
        #     name = group_name %||% group_by,
        #     values = palette_this(levels(data[[group_by]]), palette = palette, palcolor = palcolor)) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% "",
            y = ylab %||% y
        ) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major = element_line(colour = "grey80", linetype = 2),
            axis.text.x = element_text(
                angle = x_text_angle,
                hjust = just$h,
                vjust = just$v
            )
        )

    if (isTRUE(keep_empty_group)) {
        p <- p +
            scale_fill_manual(
                name = group_name %||% group_by,
                values = group_cols,
                breaks = group_vals,
                limits = group_vals,
                drop = FALSE
            )
    } else {
        p <- p +
            scale_fill_manual(
                name = group_name %||% group_by,
                values = group_cols
            )
    }

    xs <- levels(data[[x]])
    if (anyNA(data[[x]])) {
        xs <- c(xs, NA)
    }
    dims <- calculate_plot_dimensions(
        base_height = ifelse(length(xs) < 10, 4.5, 6.5),
        aspect.ratio = aspect.ratio,
        n_x = length(xs),
        x_scale_factor = ifelse(length(xs) < 10, 0.8, 0.25),
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = length(group_vals),
        legend_nchar = max(nchar(as.character(group_vals)), na.rm = TRUE)
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction,
        drop = !isTRUE(keep_empty_facet)
    )
}


#' Trend plot
#'
#' @description
#' Draws a trend plot combining stacked bars with a semi-transparent area
#' background to show how one or more groups accumulate across a discrete
#' x-axis variable. This hybrid style sits between a bar plot and an area
#' plot: it preserves the discrete category separation of bars while
#' softening the visual with an area fill, making trends across categories
#' easier to perceive.
#'
#' The function supports \strong{count aggregation} (omit \code{y} to plot
#' observation counts per x-category), \strong{proportion scaling}
#' (\code{scale_y = TRUE} normalises each x position to 100\%), per-group
#' colour control, faceting, and splitting into separate sub-plots via
#' \code{split_by}.
#'
#' @seealso \code{\link{AreaPlot}} for a pure stacked area plot.
#'
#' @section split_by workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
#'         normalise the \code{keep_na} / \code{keep_empty} arguments for all
#'         columns (\code{x}, \code{split_by}, \code{group_by}, \code{facet_by}).
#'   \item The \code{split_by} column is validated and its NA / empty levels
#'         are processed via \code{\link{process_keep_na_empty}()}. It is
#'         then removed from the per-column \code{keep_na} / \code{keep_empty}
#'         lists.
#'   \item The data frame is split by \code{split_by} (preserving level
#'         order). If \code{split_by} is \code{NULL}, the data is wrapped in
#'         a single-element list with name \code{"..."}.
#'   \item Per-split \code{palette}, \code{palcolor},
#'         \code{legend.position}, and \code{legend.direction} are resolved
#'         via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
#'         and \code{\link{check_legend}()}.
#'   \item \code{\link{TrendPlotAtomic}()} is called for each split. If
#'         \code{title} is a function, it receives the split level name and
#'         can generate dynamic titles.
#'   \item Results are combined via \code{\link{combine_plots}()} (when
#'         \code{combine = TRUE}) or returned as a named list.
#' }
#'
#' @inheritParams common_args
#' @inheritParams TrendPlotAtomic
#' @param split_by The column(s) to split the data by and produce separate
#'  sub-plots. Multiple columns are concatenated with \code{split_by_sep}.
#' @param split_by_sep A character string to separate concatenated
#'  \code{split_by} columns. Default \code{"_"}.
#' @param seed A numeric seed for reproducibility. Passed to
#'  \code{\link{validate_common_args}()}.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'  \code{patchwork} object. When \code{FALSE}, returns a named list of
#'  individual \code{ggplot} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined layout
#'  (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param byrow Logical; fill the combined layout by row. Default \code{TRUE}
#'  (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axes A character string specifying how axes should be treated across
#'  the combined layout (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axis_titles A character string specifying how axis titles should be
#'  treated across the combined layout. Defaults to \code{axes}.
#' @param guides A character string specifying how guides (legends) should be
#'  collected across panels. Default \code{"collect"} (passed to
#'  \code{\link{combine_plots}()}).
#' @param design A custom layout design for the combined plot (passed to
#'  \code{\link{combine_plots}()}).
#' @return A \code{ggplot} object, a \code{patchwork} object, or a named list
#'  of \code{ggplot} objects (when \code{combine = FALSE}), each with
#'  \code{height} and \code{width} attributes in inches.
#' @export
#' @importFrom ggplot2 waiver
#' @examples
#' \donttest{
#' data <- data.frame(
#'     x = factor(rep(c("A", "B", NA, "D"), 3), levels = LETTERS[1:4]),
#'     y = c(1, 3, 6, 5, 4, 2, 5, 7, 8, 9, 4, 8),
#'     group = factor(rep(c("F1", NA, "F3"), each = 4), levels = c("F1", "F2", "F3"))
#' )
#'
#' # Basic trend plot with grouping
#' TrendPlot(data, x = "x", y = "y", group_by = "group")
#'
#' # Scaled to proportions
#' TrendPlot(data, x = "x", y = "y", group_by = "group",
#'          scale_y = TRUE)
#'
#' # Split into sub-plots (no group_by -- single-colour fill)
#' TrendPlot(data, x = "x", y = "y", split_by = "group")
#'
#' # Per-split palettes
#' TrendPlot(data, x = "x", y = "y", split_by = "group",
#'           palette = c(F1 = "Set1", F3 = "Dark2"))
#'
#' # How keep_na and keep_empty work
#' TrendPlot(data, x = "x", y = "y", group_by = "group",
#'          keep_na = TRUE, keep_empty = TRUE)
#' TrendPlot(data, x = "x", y = "y", group_by = "group",
#'          keep_na = TRUE, keep_empty = list(x = FALSE, group = 'level'))
#'
#' # Faceting
#' TrendPlot(data, x = "x", y = "y", facet_by = "group",
#'          keep_na = TRUE, keep_empty = list(x = FALSE, group = 'level'))
#' }
TrendPlot <- function(
    data,
    x,
    y = NULL,
    x_sep = "_",
    split_by = NULL,
    split_by_sep = "_",
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    scale_y = FALSE,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    x_text_angle = 0,
    aspect.ratio = 1,
    legend.position = waiver(),
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_na = FALSE,
    keep_empty = FALSE,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    keep_na <- check_keep_na(keep_na, c(x, group_by, split_by, facet_by))
    keep_empty <- check_keep_empty(
        keep_empty,
        c(x, group_by, split_by, facet_by)
    )
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            TrendPlotAtomic(
                datas[[nm]],
                x = x,
                y = y,
                x_sep = x_sep,
                group_by = group_by,
                group_by_sep = group_by_sep,
                group_name = group_name,
                scale_y = scale_y,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                alpha = alpha,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                x_text_angle = x_text_angle,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                keep_na = keep_na,
                keep_empty = keep_empty,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

#' Atomic volcano plot (internal)
#'
#' @description
#' Core implementation for drawing a single volcano plot. This is the
#' workhorse behind the exported \code{\link{VolcanoPlot}} function — it takes
#' a **single** data frame (no \code{split_by} support) and returns a
#' \code{ggplot} object. The plot displays statistical significance (typically
#' -log10 adjusted p-value) on the y-axis versus magnitude of change (log2
#' fold change) on the x-axis, with points coloured by significance category
#' or a user-supplied variable. Top features can be automatically labelled
#' via \code{\link[ggrepel]{geom_text_repel}()}, and specific points can be
#' highlighted.
#'
#' The function categorises points into three groups based on cutoff
#' thresholds: \code{"sig_pos_x"} (points exceeding both the positive
#' x-cutoff and y-cutoff), \code{"sig_neg_x"} (points exceeding both the
#' negative x-cutoff and y-cutoff), and \code{"insig"} (all remaining
#' points). When \code{color_by = NULL}, this categorisation drives point
#' colouring; otherwise the supplied column controls the colour scale.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
#'         \code{ggplot2::ggplot} based on
#'         \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{Input validation} — \code{trim} (must be length-2 in
#'         \code{[0, 1]}) and \code{xlim} (must be length-2 or \code{NULL})
#'         are validated via \code{stopifnot()}. \code{trim} is sorted.
#'   \item \strong{Column resolution} — \code{x}, \code{y}, \code{color_by},
#'         \code{facet_by}, and \code{label_by} are validated and transformed
#'         via \code{\link{check_columns}}. Multi-column \code{facet_by} is
#'         concatenated with \code{force_factor = TRUE}.
#'   \item \strong{y-axis transformation} — the y-column is transformed by
#'         \code{ytrans} (default: \code{"-log10"}). The \code{y_cutoff}
#'         value is also transformed.
#'   \item \strong{x_cutoff defaulting} — if \code{x_cutoff} is
#'         \code{NULL}, it is set to \code{0} (suppressing the x-cutoff
#'         legend line).
#'   \item \strong{Category assignment} — a \code{.category} factor with
#'         levels \code{c("sig_neg_x", "insig", "sig_pos_x")} is created:
#'         \itemize{
#'           \item When \code{y_cutoff} is non-\code{NULL}: points with
#'           \code{|x| > x_cutoff} AND \code{y > y_cutoff} are significant.
#'           \item When \code{y_cutoff} is \code{NULL}: only the x-cutoff
#'           determines significance.
#'         }
#'   \item \strong{Color resolution} — three cases:
#'         \itemize{
#'           \item \code{color_by = NULL}: uses \code{.category} as a
#'           discrete colour column; the legend is suppressed.
#'           \item Character/factor column: discrete colour scale via
#'           \code{\link{palette_this}()} and
#'           \code{scale_color_manual()}, guide suppressed.
#'           \item Numeric column: continuous gradient via
#'           \code{scale_color_gradientn()} with a framed colour-bar legend.
#'         }
#'   \item \strong{Flip negatives} — when \code{flip_negatives = TRUE},
#'         the y-values of points with negative x are multiplied by -1,
#'         creating a mirrored volcano where both up- and down-regulated
#'         features show their significance on the same side of the y-axis.
#'   \item \strong{Label column} — \code{.label} is populated from
#'         \code{label_by} or \code{rownames(data)}.
#'   \item \strong{Trim / winsorize} — x-values beyond the trim quantile
#'         bounds are clamped. When both bounds are nonzero and of opposite
#'         sign, they are symmetrised to the smaller absolute value.
#'         Outlying points are marked in \code{.outlier}.
#'   \item \strong{Label selection} — two modes:
#'         \itemize{
#'           \item Explicit \code{labels}: the specified rows (by name or
#'           index) are marked for labelling.
#'           \item Automatic: top \code{nlabel} points (by Euclidean distance
#'           to origin) are selected per \code{sign(x)} group, and per facet
#'           level if \code{facet_by} is set.
#'         }
#'         All labels are filtered to exclude \code{"insig"} points.
#'   \item \strong{Data split} — data is split into \code{pos_data}
#'         (\code{x >= 0}) and \code{neg_data} (\code{x < 0}) so that
#'         \code{ggrepel} labels can nudge in opposite directions (positive
#'         points nudge left, negative points nudge right).
#'   \item \strong{Outlier jitter} — outlier points are rendered separately
#'         with \code{position_jitter()} to reduce overplotting.
#'   \item \strong{Base ggplot} — \code{geom_point()} layers for positive,
#'         negative, and outlier data, with colour mapped to
#'         \code{color_by}.
#'   \item \strong{Colour scale} — discrete:
#'         \code{scale_color_manual()} with \code{"insig"} forced to
#'         \code{"grey"} (when \code{palcolor} is \code{NULL}); continuous:
#'         \code{scale_color_gradientn()} with palette re-scaled so that
#'         the colour-bar is centred at 0.
#'   \item \strong{Highlight} — when \code{highlight} is provided, two
#'         additional \code{geom_point()} layers (non-outliers and outliers
#'         with jitter) overlay the highlighted points in
#'         \code{highlight_color}.
#'   \item \strong{x-cutoff lines} — vertical dashed lines at
#'         \code{+/- x_cutoff} via \code{geom_vline()} with
#'         \code{\link[ggnewscale]{new_scale_color}()}, labelled by
#'         \code{x_cutoff_name}. Suppressed when \code{x_cutoff} is
#'         \code{NULL} or \code{0}.
#'   \item \strong{y-cutoff lines} — horizontal dashed line(s) at
#'         \code{y_cutoff} (or \code{+/- y_cutoff} when
#'         \code{flip_negatives = TRUE}) via \code{geom_hline()} with
#'         \code{\link[ggnewscale]{new_scale_color}()}, labelled by
#'         \code{y_cutoff_name}.
#'   \item \strong{Flip-negatives axis} — when \code{flip_negatives = TRUE},
#'         a solid \code{geom_hline(yintercept = 0)} is added and
#'         \code{scale_y_continuous(labels = abs)} formats the y-axis.
#'   \item \strong{x-axis limits} — optional \code{xlim} passed to
#'         \code{ggplot2::xlim()}.
#'   \item \strong{Reference line and labels} — a grey80 dashed vertical
#'         line at \code{x = 0}, followed by \code{geom_text_repel()} for
#'         positive and negative labelled points with separate x-nudges.
#'   \item \strong{Labels and theme} — \code{labs()},
#'         \code{coord_cartesian(clip = "off")},
#'         \code{do_call(theme, theme_args)}, and theme elements for
#'         \code{aspect.ratio}, \code{legend.position}, and
#'         \code{legend.direction}.
#'   \item \strong{Dimension calculation} —
#'         \code{\link{calculate_plot_dimensions}()} computes \code{height}
#'         and \code{width} attributes from \code{base_height = 5},
#'         \code{aspect.ratio}, and legend geometry.
#'   \item \strong{Faceting} — \code{\link{facet_plot}()} wraps the plot
#'         with \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is
#'         provided.
#' }
#'
#' @inheritParams common_args
#' @param ytrans A function or a function name (as a string) to transform
#'  the y-axis values before plotting. The transformed values are used for
#'  both the y-axis and cutoff comparisons. Default: \code{"-log10"}
#'  (converts p-values to a -log10 scale). Other named functions can be
#'  passed as strings, e.g. \code{"sqrt"}.
#' @param color_by A character string specifying the column name to colour
#'  the points by. When \code{NULL} (default), points are automatically
#'  categorised as \code{"sig_pos_x"}, \code{"sig_neg_x"}, or
#'  \code{"insig"} based on \code{x_cutoff} and \code{y_cutoff}, and the
#'  colour legend is suppressed. When a column name is provided, the colour
#'  mapping follows the column type — discrete (character/factor) uses
#'  \code{scale_color_manual()} with the specified \code{palette};
#'  numeric (continuous) uses \code{scale_color_gradientn()}.
#' @param color_name A character string for the colour legend title when
#'  \code{color_by} is a numeric column. When \code{NULL} (default), the
#'  \code{color_by} column name is used.
#' @param flip_negatives A logical value. When \code{TRUE}, y-values of
#'  points with negative x-values are multiplied by -1, creating a mirrored
#'  volcano plot where both up- and down-regulated features show their
#'  significance on the same side of the y-axis. A horizontal line at
#'  \code{y = 0} and absolute-value axis labels are added. Default:
#'  \code{FALSE}.
#' @param trim A numeric vector of length 2 specifying quantile bounds for
#'  winsorizing the x-axis values. Values below the first quantile are
#'  clamped to that quantile; values above the second quantile are clamped
#'  to that quantile. Both values must be in \code{[0, 1]}. When both bounds
#'  are nonzero and of opposite sign, they are symmetrised to the smaller
#'  absolute value. Default: \code{c(0, 1)} (no trimming).
#' @param xlim A numeric vector of length 2 to set the x-axis limits.
#'  Passed to \code{\link[ggplot2]{xlim}()}. When \code{NULL} (default),
#'  limits are determined automatically from the data.
#' @param x_cutoff A numeric value specifying the x-axis significance
#'  cutoff. Both the negative and positive of this value are used as
#'  vertical threshold lines. When \code{NULL} or \code{0}, no x-cutoff
#'  line is drawn. Default: \code{NULL}.
#' @param y_cutoff A numeric value specifying the y-axis significance
#'  cutoff in the **original** (untransformed) scale. The value is
#'  transformed by \code{ytrans} before plotting. When \code{NULL}, no
#'  y-cutoff line is drawn and the category assignment uses only the
#'  x-cutoff. Default: \code{0.05}.
#' @param x_cutoff_name A character string for the x-cutoff legend entry.
#'  When \code{"none"}, the legend for the x-cutoff line is suppressed
#'  entirely (the line is still drawn). When \code{NULL} (default), a
#'  label of the form \code{"<x> = +/-<value>"} is generated.
#' @param y_cutoff_name A character string for the y-cutoff legend entry.
#'  When \code{"none"}, the legend for the y-cutoff line is suppressed
#'  entirely (the line is still drawn). When \code{NULL} (default), a
#'  label of the form \code{"<ylab> = <value>"} is generated.
#' @param x_cutoff_color A character string specifying the colour of the
#'  x-axis cutoff line(s). Default: \code{"red2"}.
#' @param y_cutoff_color A character string specifying the colour of the
#'  y-axis cutoff line(s). Default: \code{"blue2"}.
#' @param x_cutoff_linetype A character string specifying the linetype of
#'  the x-axis cutoff line(s). Default: \code{"dashed"}.
#' @param y_cutoff_linetype A character string specifying the linetype of
#'  the y-axis cutoff line(s). Default: \code{"dashed"}.
#' @param x_cutoff_linewidth A numeric value specifying the linewidth of
#'  the x-axis cutoff line(s). Default: \code{0.5}.
#' @param y_cutoff_linewidth A numeric value specifying the linewidth of
#'  the y-axis cutoff line(s). Default: \code{0.5}.
#' @param pt_size A numeric value specifying the point size for all data
#'  points. Default: \code{2}.
#' @param pt_alpha A numeric value in \code{[0, 1]} specifying the
#'  transparency of all data points. Default: \code{0.5}.
#' @param pt_shape A numeric value specifying the point shape. Default: \code{21}
#'  (filled circle with border). Shapes 21--25 support separate fill and border
#'  colour aesthetics; all other shapes use a single colour aesthetic. In
#'  raster mode, all points are drawn as filled circles (shape is ignored).
#' @param pt_border_color Controls the point border colour. For shapes 21--25:
#'  \itemize{
#'    \item \code{TRUE} (default) -- border colour tracks the \code{color_by}
#'    gradient / palette.
#'    \item A colour string (e.g. \code{"black"}) -- constant colour border.
#'  }
#'  \code{FALSE} or \code{NULL} disables the border. For shapes without a fill
#'  aesthetic (not 21--25), this parameter has no effect. In raster mode the
#'  border is drawn as a slightly larger disc behind each point (the shape is
#'  always a circle there), and \code{TRUE} falls back to a border disc in the
#'  \code{color_by} colour.
#' @param pt_border_size A numeric value specifying the point border size
#'  (stroke width, in mm). \code{0} disables the border. Default: \code{0.5}.
#' @param nlabel An integer specifying the number of top features to label
#'  automatically. Points are ranked by Euclidean distance to the origin
#'  within each \code{sign(x)} group (and per facet level if
#'  \code{facet_by} is set). Only non-insignificant points receive labels.
#'  Default: \code{5}.
#' @param labels A character vector of row names or integer indices
#'  specifying which points to label. Overrides automatic \code{nlabel}
#'  selection. When \code{NULL} (default), top \code{nlabel} points are
#'  chosen automatically.
#' @param label_by A character string specifying the column whose values
#'  are used as label text. When \code{NULL} (default), row names of the
#'  data frame are used.
#' @param label_size A numeric value specifying the font size of the
#'  labels. Default: \code{3}.
#' @param label_fg A character string specifying the text colour of the
#'  labels. Default: \code{"black"}.
#' @param label_bg A character string specifying the background colour of
#'  the label boxes (passed to \code{geom_text_repel(bg.color = ...)}).
#'  Default: \code{"white"}.
#' @param label_bg_r A numeric value specifying the corner radius of the
#'  label background boxes (passed to \code{geom_text_repel(bg.r = ...)}).
#'  Default: \code{0.1}.
#' @param highlight A character vector of row names or integer indices
#'  specifying which points to highlight with an overlaid point layer in
#'  \code{highlight_color}. When \code{NULL} (default), no highlighting is
#'  applied.
#' @param highlight_color A character string specifying the colour of the
#'  highlight points. Default: \code{"red"}.
#' @param highlight_size A numeric value specifying the point size of the
#'  highlight layer. Default: \code{2}.
#' @param highlight_alpha A numeric value in \code{[0, 1]} specifying the
#'  transparency of the highlight points. Default: \code{1}.
#' @param highlight_stroke A numeric value specifying the stroke width of
#'  the highlight point borders. Default: \code{0.5}.
#' @param raster A logical value. If `TRUE`, points are rendered via
#'  `scattermore::geom_scattermore()` for efficient rasterised plotting.
#'  Default is `NULL`, which auto-enables when `nrow(data) > 1e3`.
#' @param raster_dpi A numeric vector of length 2 `[x_dpi, y_dpi]` specifying the raster
#'  resolution in pixels. Passed to `scattermore::geom_scattermore(pixels = ...)`.
#'  Default is `c(512, 512)`. If a single value is provided it is recycled to both dimensions.
#'
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @keywords internal
#' @importFrom dplyr %>% case_when mutate arrange row_number group_by ungroup desc filter
VolcanoPlotAtomic <- function(
    data,
    x,
    y,
    ytrans = "-log10",
    color_by = NULL,
    color_name = NULL,
    flip_negatives = FALSE,
    x_cutoff = NULL,
    y_cutoff = 0.05,
    trim = c(0, 1),
    xlim = NULL,
    x_cutoff_name = NULL,
    y_cutoff_name = NULL,
    x_cutoff_color = "red2",
    y_cutoff_color = "blue2",
    x_cutoff_linetype = "dashed",
    y_cutoff_linetype = "dashed",
    x_cutoff_linewidth = 0.5,
    y_cutoff_linewidth = 0.5,
    pt_size = 2,
    pt_alpha = 0.5,
    pt_shape = 21,
    pt_border_color = TRUE,
    pt_border_size = 0.5,
    nlabel = 5,
    labels = NULL,
    label_by = NULL,
    label_size = 3,
    label_fg = "black",
    label_bg = "white",
    label_bg_r = 0.1,
    highlight = NULL,
    highlight_color = "red",
    highlight_size = 2,
    highlight_alpha = 1,
    highlight_stroke = 0.5,
    raster = NULL,
    raster_dpi = c(512, 512),
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    theme = "theme_this",
    theme_args = list(),
    palette = "Spectral",
    palcolor = NULL,
    palreverse = FALSE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    seed = 8525,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    stopifnot(
        "[VolcanoPlot] 'trim' must be a numeric vector of length 2 and both values must be in the range [0, 1]." = length(
            trim
        ) ==
            2 &&
            all(trim >= 0 & trim <= 1)
    )
    stopifnot(
        "[VolcanoPlot] 'xlim' must be a numeric vector of length 2." = is.null(
            xlim
        ) ||
            length(xlim) == 2
    )
    trim <- sort(trim)
    x <- check_columns(data, x)
    y <- check_columns(data, y)
    color_by <- check_columns(data, color_by)
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )
    label_by <- check_columns(data, label_by)

    # Resolve function-name strings (e.g. "-log10", "sqrt") to functions
    if (is.null(ytrans)) {
        ytrans <- identity
    } else if (is.character(ytrans)) {
        if (startsWith(ytrans, "-")) {
            ytrans_str <- trimws(substring(ytrans, 2))
            ytrans <- function(x) -1 * match.fun(ytrans_str)(x)
        } else {
            ytrans <- match.fun(trimws(ytrans))
        }
    }

    data[[y]] <- ytrans(data[[y]])
    x_cutoff <- x_cutoff %||% 0
    y_cutoff <- ytrans(y_cutoff)

    if (!is.null(y_cutoff)) {
        data <- data %>%
            mutate(
                .category = factor(
                    case_when(
                        !!sym(x) > x_cutoff & !!sym(y) > y_cutoff ~ "sig_pos_x",
                        !!sym(x) < -x_cutoff &
                            !!sym(y) > y_cutoff ~ "sig_neg_x",
                        TRUE ~ "insig"
                    ),
                    levels = c("sig_neg_x", "insig", "sig_pos_x")
                )
            )
    } else {
        data <- data %>%
            mutate(
                .category = factor(
                    case_when(
                        !!sym(x) > x_cutoff ~ "sig_pos_x",
                        !!sym(x) < -x_cutoff ~ "sig_neg_x",
                        TRUE ~ "insig"
                    ),
                    levels = c("sig_neg_x", "insig", "sig_pos_x")
                )
            )
    }

    if (is.null(color_by)) {
        color_by <- ".category"
        color_type <- "discrete"
    } else if (is.character(data[[color_by]]) || is.factor(data[[color_by]])) {
        color_type <- "discrete"
    } else {
        color_type <- "continuous"
    }

    if (flip_negatives) {
        data[data[[x]] < 0, y] <- -data[data[[x]] < 0, y]
    }
    data$.label <- if (is.null(label_by)) rownames(data) else data[[label_by]]
    data$.show_label <- FALSE

    x_upper <- quantile(data[[x]][is.finite(data[[x]])], c(trim[2], 1))
    x_lower <- quantile(data[[x]][is.finite(data[[x]])], c(trim[1], 0))
    x_upper <- ifelse(x_upper[1] > 0, x_upper[1], x_upper[2])
    x_lower <- ifelse(x_lower[1] < 0, x_lower[1], x_lower[2])
    if (x_upper > 0 & x_lower < 0) {
        value_range <- min(abs(c(x_upper, x_lower)), na.rm = TRUE)
        x_upper <- value_range
        x_lower <- -value_range
    }
    data$.outlier <- data[[x]] > x_upper | data[[x]] < x_lower
    data[[x]][data[[x]] > x_upper] <- x_upper
    data[[x]][data[[x]] < x_lower] <- x_lower

    if (!is.null(labels)) {
        data[labels, ".show_label"] <- TRUE
    } else {
        # calculate the distance to the origin
        data$.distance <- sqrt(data[[x]]^2 + data[[y]]^2)
        if (!is.null(facet_by)) {
            data <- data %>%
                group_by(!!!syms(facet_by), sign(data[[x]])) %>%
                arrange(desc(!!sym(".distance"))) %>%
                mutate(.show_label = row_number() <= nlabel) %>%
                ungroup()
        } else {
            data <- data %>%
                group_by(sign(data[[x]])) %>%
                arrange(desc(!!sym(".distance"))) %>%
                mutate(.show_label = row_number() <= nlabel) %>%
                ungroup()
        }
    }
    data <- data %>%
        mutate(
            .show_label = !!sym(".show_label") & !!sym(".category") != "insig"
        ) %>%
        as.data.frame()
    # rownames(data) <- data$.label

    # x_nudge is not an aesthetic, we can't specify it separately for negative and positive values
    # so we plot the points in two layers
    # one for negative values and one for positive values
    # and set the x_nudge for each layer
    pos_data <- data[data[[x]] >= 0, , drop = FALSE]
    neg_data <- data[data[[x]] < 0, , drop = FALSE]

    outlier_data <- data[data$.outlier, , drop = FALSE]

    # x_nudges will be the same for all facets
    pos_x_nudge <- -diff(range(pos_data[[x]])) * 0.05
    neg_x_nudge <- diff(range(neg_data[[x]])) * 0.05
    jitter <- position_jitter(width = 0.2, height = 0.2, seed = seed)

    raster <- raster %||% (nrow(data) > 1e3)
    if (length(raster_dpi) == 1) {
        raster_dpi <- rep(raster_dpi, 2)
    }
    dims <- calculate_plot_dimensions(
        base_height = 5,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction
    )
    # scattermore's `pointsize` is a radius in raster pixels; convert the
    # ggplot point size (diameter in mm) so raster points match geom_point
    raster_pointsize <- max(
        1,
        round(pt_size * raster_dpi[1] / (25.4 * 1.25 * dims$width))
    )

    shape_has_fill <- pt_shape %in% 21:25
    has_border <- shape_has_fill && !is.null(pt_border_color) && !isFALSE(pt_border_color)
    # vector border colour: TRUE maps the aesthetic, FALSE/NULL hides the border
    vector_border <- if (isTRUE(pt_border_color)) {
        NULL
    } else if (isFALSE(pt_border_color) || is.null(pt_border_color)) {
        "transparent"
    } else {
        pt_border_color
    }

    p <- ggplot(mapping = aes(x = !!sym(x), y = !!sym(y)))
    if (isTRUE(raster)) {
        # scattermore can only stamp filled discs: no fill aesthetic, no
        # border. The border is a slightly larger disc stamped behind, with
        # the `color_by` colour riding the colour aesthetic. An opaque disc
        # of the panel background is stamped between border and fill —
        # otherwise the border colour would tint the translucent interior.
        # same mm -> raster-px conversion as raster_pointsize (including the
        # 1.25 calibration and plot width), so the border keeps its vector
        # proportion of the point instead of dwarfing it
        border_px <- if (has_border && pt_border_size > 0) {
            max(1, round(pt_border_size * raster_dpi[1] / (25.4 * 1.25 * dims$width)))
        } else {
            0
        }
        panel_fill <- do_call(theme, theme_args)$panel.background$fill %||% "white"
        raster_layer <- function(dat, position = NULL) {
            position <- position %||% "identity"
            fill_layer <- scattermore::geom_scattermore(
                data = dat,
                mapping = aes(color = !!sym(color_by)),
                pointsize = raster_pointsize,
                alpha = pt_alpha,
                pixels = raster_dpi,
                position = position
            )
            if (border_px == 0) {
                return(list(fill_layer))
            }
            disc_args <- list(
                data = dat,
                pointsize = raster_pointsize + border_px,
                alpha = 1,
                pixels = raster_dpi,
                position = position
            )
            if (isTRUE(pt_border_color)) {
                disc_args$mapping <- aes(color = !!sym(color_by))
            } else {
                disc_args$color <- vector_border
            }
            list(
                do_call(scattermore::geom_scattermore, disc_args),
                scattermore::geom_scattermore(
                    data = dat,
                    color = panel_fill,
                    pointsize = raster_pointsize,
                    alpha = 1,
                    pixels = raster_dpi,
                    position = position
                ),
                fill_layer
            )
        }
        p <- p +
            raster_layer(pos_data) +
            raster_layer(neg_data) +
            raster_layer(outlier_data, position = jitter)
    } else {
        point_layer <- function(dat, position = NULL) {
            args <- list(
                data = dat,
                size = pt_size,
                alpha = pt_alpha,
                shape = pt_shape
            )
            if (shape_has_fill) {
                if (isTRUE(pt_border_color)) {
                    # border tracks the fill gradient / palette (single layer)
                    args$mapping <- aes(fill = !!sym(color_by), color = !!sym(color_by))
                    args$stroke <- pt_border_size
                } else {
                    args$mapping <- aes(fill = !!sym(color_by))
                    # stroke 0 suppresses the point's own outline; a constant
                    # colour border is drawn by the ring layer on top. NA would
                    # hide the whole point, and "transparent" becomes opaque
                    # white through scales::alpha() when pt_alpha = 1.
                    args$color <- "transparent"
                    args$stroke <- 0
                }
            } else {
                args$mapping <- aes(color = !!sym(color_by))
            }
            if (!is.null(position)) {
                args$position <- position
            }
            layers <- list(do_call(geom_point, args))
            if (has_border && !isTRUE(pt_border_color) && pt_border_size > 0) {
                # opaque border ring layer on top, so the border stays visible
                # under translucent fills (same look as the raster mode)
                border_args <- list(
                    data = dat,
                    size = pt_size,
                    shape = pt_shape,
                    alpha = 1,
                    fill = NA,
                    color = vector_border,
                    stroke = pt_border_size
                )
                if (!is.null(position)) {
                    border_args$position <- position
                }
                layers[[2]] <- do_call(geom_point, border_args)
            }
            layers
        }
        p <- p +
            point_layer(pos_data) +
            point_layer(neg_data) +
            point_layer(outlier_data, position = jitter)
    }

    if (color_type == "discrete") {
        colors <- palette_this(
            levels(data[[color_by]]),
            palette = palette,
            palcolor = palcolor,
            reverse = palreverse
        )
        if (is.null(palcolor)) {
            colors['insig'] <- "grey"
        }
        if (shape_has_fill && !isTRUE(raster)) {
            p <- p + scale_fill_manual(values = colors, guide = "none")
            if (isTRUE(pt_border_color)) {
                p <- p + scale_color_manual(values = colors, guide = "none")
            }
        } else {
            p <- p + scale_color_manual(values = colors, guide = "none")
        }
    } else {
        grad_args <- list(
            colors = palette_this(
                palette = palette,
                palcolor = palcolor,
                reverse = palreverse
            ),
            values = scales::rescale(unique(c(
                min(c(unlist(data[[color_by]]), 0), na.rm = TRUE),
                0,
                max(unlist(data[[color_by]]), na.rm = TRUE)
            ))),
            guide = guide_colorbar(
                title = color_name %||% color_by,
                frame.colour = "black",
                ticks.colour = "black",
                title.hjust = 0,
                order = 1
            )
        )
        if (shape_has_fill && !isTRUE(raster)) {
            p <- p + do_call(scale_fill_gradientn, grad_args)
            if (isTRUE(pt_border_color)) {
                grad_args$guide <- guide_none()
                p <- p + do_call(scale_color_gradientn, grad_args)
            }
        } else {
            p <- p + do_call(scale_color_gradientn, grad_args)
        }
    }

    if (!is.null(highlight)) {
        p <- p +
            geom_point(
                data = data[match(data$.label, highlight, nomatch = 0), , drop = FALSE] %>%
                    filter(!!sym(".outlier") == FALSE),
                color = highlight_color,
                size = highlight_size,
                alpha = highlight_alpha,
                stroke = highlight_stroke
            ) +
            geom_point(
                data = data[match(data$.label, highlight, nomatch = 0), , drop = FALSE] %>%
                    filter(!!sym(".outlier")),
                color = highlight_color,
                size = highlight_size,
                alpha = highlight_alpha,
                stroke = highlight_stroke,
                position = jitter
            )
    }

    if (!is.null(x_cutoff) && x_cutoff != 0) {
        if (identical(x_cutoff_name, "none")) {
            guide <- guide_none()
        } else {
            guide <- guide_legend(
                override.aes = list(alpha = 0.8, size = 5),
                order = 2,
                theme = ggplot2::theme(legend.margin = margin(0, 0, -10, 4.5))
            )
        }
        vline_df <- data.frame(xintercept = c(-x_cutoff, x_cutoff))
        if (!is.null(facet_by)) {
            vline_df <- expand_grid(
                vline_df,
                data[, facet_by, drop = FALSE] %>% distinct()
            )
        }
        p <- p +
            new_scale_color() +
            geom_vline(
                data = vline_df,
                mapping = aes(
                    xintercept = !!sym("xintercept"),
                    color = x_cutoff_name %||%
                        paste0(
                            x,
                            " = +/-",
                            scales::number(x_cutoff, accuracy = 0.01)
                        )
                ),
                alpha = 0.4,
                linetype = x_cutoff_linetype,
                linewidth = x_cutoff_linewidth,
            ) +
            scale_color_manual(
                name = NULL,
                values = x_cutoff_color,
                guide = guide
            )
    }

    if (!is.null(y_cutoff)) {
        if (isTRUE(flip_negatives)) {
            yintercept <- c(-y_cutoff, y_cutoff)
        } else {
            yintercept <- y_cutoff
        }
        if (identical(y_cutoff_name, "none")) {
            guide <- guide_none()
        } else {
            guide <- guide_legend(
                override.aes = list(alpha = 0.8, size = 5),
                order = 3
            )
        }
        hline_df <- data.frame(yintercept = yintercept)
        if (!is.null(facet_by)) {
            hline_df <- expand_grid(
                hline_df,
                data[, facet_by, drop = FALSE] %>% distinct()
            )
        }
        p <- p +
            new_scale_color() +
            geom_hline(
                data = hline_df,
                mapping = aes(
                    yintercept = !!sym("yintercept"),
                    color = y_cutoff_name %||%
                        paste0(
                            ylab %||% y,
                            " = ",
                            scales::number(y_cutoff, accuracy = 0.01)
                        )
                ),
                alpha = 0.4,
                linetype = y_cutoff_linetype,
                linewidth = y_cutoff_linewidth
            ) +
            scale_color_manual(
                name = NULL,
                values = y_cutoff_color,
                guide = guide
            )
    }

    if (isTRUE(flip_negatives)) {
        p <- p +
            geom_hline(yintercept = 0, color = "black", linetype = 1) +
            scale_y_continuous(labels = abs)
    }
    if (!is.null(xlim)) {
        p <- p + ggplot2::xlim(xlim)
    }

    p <- p +
        geom_vline(xintercept = 0, color = "grey80", linetype = 2) +
        geom_text_repel(
            data = pos_data[pos_data$.show_label, , drop = FALSE],
            aes(label = !!sym(".label")),
            nudge_x = pos_x_nudge,
            color = label_fg,
            bg.color = label_bg,
            bg.r = label_bg_r,
            size = label_size,
            min.segment.length = 0,
            segment.color = "grey40",
            max.overlaps = 100
        ) +
        geom_text_repel(
            data = neg_data[neg_data$.show_label, , drop = FALSE],
            aes(label = !!sym(".label")),
            nudge_x = neg_x_nudge,
            color = label_fg,
            bg.color = label_bg,
            bg.r = label_bg_r,
            size = label_size,
            min.segment.length = 0,
            segment.color = "grey40",
            max.overlaps = 100
        ) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% x,
            y = ylab %||% y
        ) +
        coord_cartesian(clip = "off")

    p <- p +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction
        )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction
    )
}

#' Volcano plot
#'
#' @description
#' Produces a volcano plot — a scatter plot that displays statistical
#' significance (typically -log10 adjusted p-value) on the y-axis versus
#' magnitude of change (log2 fold change) on the x-axis. Points are coloured
#' automatically by significance category (\code{"sig_pos_x"},
#' \code{"sig_neg_x"}, \code{"insig"}) or by a user-supplied column. The
#' most significant features can be labelled automatically via
#' \code{\link[ggrepel]{geom_text_repel}()}, and specific points can be
#' highlighted.
#'
#' The function supports \strong{automatic labelling} of top features (by
#' distance to origin), \strong{mirrored layout} via
#' \code{flip_negatives}, \strong{x-axis trimming} to reduce the influence
#' of extreme values, \strong{faceting}, and \strong{splitting} into
#' separate sub-plots via \code{split_by} with per-split colour palette and
#' legend control.
#'
#' @section split_by Workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item The \code{split_by} column(s) are validated via
#'         \code{\link{check_columns}()} with \code{force_factor = TRUE} and
#'         \code{concat_multi = TRUE} (multiple columns are concatenated
#'         with \code{split_by_sep}).
#'   \item The data frame is split by \code{split_by} (preserving factor
#'         level order). If \code{split_by} is \code{NULL}, the data is
#'         wrapped in a single-element list with name \code{"..."}.
#'   \item Per-split \code{palette}, \code{palcolor},
#'         \code{legend.position}, and \code{legend.direction} are resolved
#'         via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
#'         and \code{\link{check_legend}()}.
#'   \item \code{\link{VolcanoPlotAtomic}()} is called for each split. If
#'         \code{title} is a function, it receives the split level name and
#'         can generate dynamic titles.
#'   \item Results are combined via \code{\link{combine_plots}()} (when
#'         \code{combine = TRUE}) or returned as a named list.
#' }
#'
#' @inheritParams common_args
#' @inheritParams VolcanoPlotAtomic
#' @param split_by The column(s) to split the data by and produce separate
#'  sub-plots. Multiple columns are concatenated with \code{split_by_sep}.
#' @param split_by_sep A character string to separate concatenated
#'  \code{split_by} columns. Default \code{"_"}.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'  \code{patchwork} object. When \code{FALSE}, returns a named list of
#'  individual \code{ggplot} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined layout
#'  (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param byrow Logical; fill the combined layout by row. Default
#'  \code{TRUE} (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axes A character string specifying how axes should be treated
#'  across the combined layout (passed to
#'  \code{\link[patchwork]{wrap_plots}}).
#' @param axis_titles A character string specifying how axis titles should
#'  be treated across the combined layout. Defaults to \code{axes}.
#' @param guides A character string specifying how guides (legends) should
#'  be collected across panels. Default \code{"collect"} (passed to
#'  \code{\link{combine_plots}()}).
#' @param design A custom layout design for the combined plot (passed to
#'  \code{\link{combine_plots}()}).
#' @return A \code{ggplot} object, a \code{patchwork} object, or a named
#'  list of \code{ggplot} objects (when \code{combine = FALSE}), each with
#'  \code{height} and \code{width} attributes in inches.
#' @export
#' @examples
#' \donttest{
#' set.seed(8525)
#' n <- 200
#' n_de <- 150
#'
#' ## Non-DE genes
#' fc_null <- rnorm(n - n_de, 0, 0.35)
#' z_null  <- rnorm(n - n_de, 0, 1)
#'
#' ## DE genes
#' fc_de <- rnorm(
#'   n_de,
#'   mean = sample(c(-1, 1), n_de, replace = TRUE),
#'   sd = 0.45
#' )
#'
#' ## Make significance related to effect size,
#' ## but with substantial variation
#' z_de <- fc_de * rnorm(n_de, 4.5, 0.8)
#'
#' avg_log2FC <- c(fc_null, fc_de)
#' z <- c(z_null, z_de)
#'
#' p_val <- 2 * pnorm(-abs(z))
#' p_val_adj <- p.adjust(p_val, method = "BH")
#'
#' ## Shuffle genes
#' i <- sample(n)
#'
#' data <- data.frame(
#'   avg_log2FC = avg_log2FC[i],
#'   p_val_adj = p_val_adj[i],
#'   gene = paste0("gene", seq_len(n))[i],
#'   pct_diff = rnorm(n, 0, 1),
#'   group = sample(LETTERS[1:2], n, replace = TRUE)
#' )
#'
#' # --- Basic usage ---
#' VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", color_by = "pct_diff",
#'    y_cutoff_name = "-log10(0.05)")
#' # --- With gene labels ---
#' VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", color_by = "pct_diff",
#'    y_cutoff_name = "-log10(0.05)", label_by = "gene")
#' # --- Mirrored layout ---
#' VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", y_cutoff_name = "none",
#'    flip_negatives = TRUE, label_by = "gene")
#' # --- With faceting ---
#' VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", y_cutoff_name = "none",
#'    flip_negatives = TRUE, facet_by = "group", label_by = "gene")
#' # --- With splitting ---
#' VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", y_cutoff_name = "none",
#'    flip_negatives = TRUE, split_by = "group", label_by = "gene")
#' # --- With highlighting ---
#' VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", y_cutoff_name = "none",
#'    highlight = c("gene196", "gene151"), label_by = "gene")
#' # --- Per-split palettes ---
#' VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", color_by = "pct_diff",
#'    y_cutoff_name = "-log10(0.05)", split_by = "group", label_by = "gene",
#'    palette = c(A = "Spectral", B = "PuOr"))
#' # Trim extreme x-values (winsorize to 40% and 50% quantiles), for demo purposes
#' VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", color_by = "pct_diff",
#'    y_cutoff_name = "-log10(0.05)", trim = c(0.4, 0.5))
#' }
VolcanoPlot <- function(
    data,
    x,
    y,
    ytrans = "-log10",
    color_by = NULL,
    color_name = NULL,
    xlim = NULL,
    flip_negatives = FALSE,
    x_cutoff = NULL,
    y_cutoff = 0.05,
    split_by = NULL,
    split_by_sep = "_",
    label_by = NULL,
    x_cutoff_name = NULL,
    y_cutoff_name = NULL,
    x_cutoff_color = "red2",
    y_cutoff_color = "blue2",
    x_cutoff_linetype = "dashed",
    y_cutoff_linetype = "dashed",
    x_cutoff_linewidth = 0.5,
    y_cutoff_linewidth = 0.5,
    pt_size = 2,
    pt_alpha = 0.5,
    pt_shape = 21,
    pt_border_color = TRUE,
    pt_border_size = 0.5,
    nlabel = 5,
    labels = NULL,
    label_size = 3,
    label_fg = "black",
    label_bg = "white",
    label_bg_r = 0.1,
    highlight = NULL,
    highlight_color = "red",
    highlight_size = 2,
    highlight_alpha = 1,
    highlight_stroke = 0.5,
    raster = NULL,
    raster_dpi = c(512, 512),
    trim = c(0, 1),
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    theme = "theme_this",
    theme_args = list(),
    palette = "Spectral",
    palcolor = NULL,
    palreverse = FALSE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    theme <- process_theme(theme)
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            VolcanoPlotAtomic(
                datas[[nm]],
                x = x,
                y = y,
                ytrans = ytrans,
                color_by = color_by,
                color_name = color_name,
                flip_negatives = flip_negatives,
                x_cutoff = x_cutoff,
                y_cutoff = y_cutoff,
                trim = trim,
                xlim = xlim,
                label_by = label_by,
                x_cutoff_name = x_cutoff_name,
                y_cutoff_name = y_cutoff_name,
                x_cutoff_color = x_cutoff_color,
                y_cutoff_color = y_cutoff_color,
                x_cutoff_linetype = x_cutoff_linetype,
                y_cutoff_linetype = y_cutoff_linetype,
                x_cutoff_linewidth = x_cutoff_linewidth,
                y_cutoff_linewidth = y_cutoff_linewidth,
                pt_size = pt_size,
                pt_alpha = pt_alpha,
                pt_shape = pt_shape,
                pt_border_color = pt_border_color,
                pt_border_size = pt_border_size,
                nlabel = nlabel,
                labels = labels,
                label_size = label_size,
                label_fg = label_fg,
                label_bg = label_bg,
                label_bg_r = label_bg_r,
                highlight = highlight,
                highlight_color = highlight_color,
                highlight_size = highlight_size,
                highlight_alpha = highlight_alpha,
                highlight_stroke = highlight_stroke,
                raster = raster,
                raster_dpi = raster_dpi,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                seed = seed,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
n <- 200
data <- data.frame(
    logFC = rnorm(n, mean = 0, sd = 2),
    pvalue = runif(n, min = 1e-10, max = 1),
    gene = paste0("gene", seq_len(n)),
    group = factor(rep(c("g1", "g2"), each = n / 2))
)
rownames(data) <- data$gene

test_that("VolcanoPlot returns a ggplot for basic usage", {
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0)
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("VolcanoPlot sets title and labels", {
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                     title = "Volcano", xlab = "log2FC", ylab = "-log10(p)")
    expect_equal(p$labels$title, "Volcano")
    expect_equal(p$labels$x, "log2FC")
    expect_equal(p$labels$y, "-log10(p)")
})

test_that("VolcanoPlot with x_cutoff and y_cutoff works", {
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                     x_cutoff = 1, y_cutoff = 0.05)
    expect_s3_class(p, "ggplot")
})

test_that("VolcanoPlot with nlabel > 0 works", {
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 5)
    expect_s3_class(p, "ggplot")
})

test_that("VolcanoPlot with specific labels works", {
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                     labels = rownames(data)[1:3])
    expect_s3_class(p, "ggplot")
})

test_that("VolcanoPlot with label_by works", {
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 3, label_by = "gene")
    expect_s3_class(p, "ggplot")
})

test_that("VolcanoPlot with color_by (discrete) works", {
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0, color_by = "group")
    expect_s3_class(p, "ggplot")
})

test_that("VolcanoPlot with split_by returns combined plot", {
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                     split_by = "group", combine = TRUE)
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("VolcanoPlot with split_by and combine = FALSE returns list", {
    plots <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                         split_by = "group", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
})

test_that("VolcanoPlot with multiple split_by columns returns a list with one plot per combination", {
    data2 <- data
    data2$split2 <- factor(rep(c("s1", "s2"), 100))
    plots <- suppressMessages(VolcanoPlot(data2, x = "logFC", y = "pvalue", nlabel = 0,
                                          split_by = c("group", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("VolcanoPlot with multiple split_by columns returns combined plot", {
    data2 <- data
    data2$split2 <- factor(rep(c("s1", "s2"), 100))
    p <- suppressMessages(VolcanoPlot(data2, x = "logFC", y = "pvalue", nlabel = 0,
                                      split_by = c("group", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("VolcanoPlot with facet_by works", {
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                     facet_by = "group")
    expect_s3_class(p, "ggplot")
})

test_that("VolcanoPlot with trim works", {
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                     trim = c(0.01, 0.99))
    expect_s3_class(p, "ggplot")
})

test_that("VolcanoPlot errors on invalid trim", {
    expect_error(
        VolcanoPlot(data, x = "logFC", y = "pvalue", trim = c(0, 1, 0.5)),
        "trim"
    )
})

test_that("VolcanoPlot with flip_negatives works", {
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                     flip_negatives = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("VolcanoPlot with highlight works", {
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                     highlight = rownames(data)[1:5])
    expect_s3_class(p, "ggplot")
})

test_that("VolcanoPlot width/height attributes are numeric", {
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0)
    expect_true(is.numeric(attr(p, "width")))
    expect_true(is.numeric(attr(p, "height")))
})

test_that("VolcanoPlot with raster = TRUE uses scattermore layers", {
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0, raster = TRUE)
    expect_s3_class(p, "ggplot")
    expect_true(any(vapply(p$layers, function(l) inherits(l$geom, "GeomScattermore"), logical(1))))
    # highlight stays as sharp vector points
    p2 <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                      raster = TRUE, highlight = rownames(data)[1:5])
    expect_s3_class(p2, "ggplot")
})

test_that("VolcanoPlot raster border uses scattermore discs following pt_border_color/pt_border_size", {
    n_scatter <- function(p) sum(vapply(p$layers, function(l) inherits(l$geom, "GeomScattermore"), logical(1)))
    # default shape 21 + TRUE border: border disc + panel-colour erase disc +
    # translucent fill dot per data subset
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0, raster = TRUE)
    expect_equal(n_scatter(p), 9)
    # border disabled (FALSE / size 0 / shape without fill): fill dots only
    p0 <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                      raster = TRUE, pt_border_color = FALSE)
    expect_equal(n_scatter(p0), 3)
    p1 <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                      raster = TRUE, pt_shape = 16)
    expect_equal(n_scatter(p1), 3)
    p3 <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                      raster = TRUE, pt_border_size = 0)
    expect_equal(n_scatter(p3), 3)
    # constant colour border: opaque border disc with that colour, larger than
    # the translucent fill dot; the erase disc matches the panel background
    p2 <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                      raster = TRUE, pt_border_color = "black")
    expect_equal(n_scatter(p2), 9)
    discs <- p2$layers[vapply(p2$layers, function(l) {
        inherits(l$geom, "GeomScattermore") && is.null(l$mapping$colour) &&
            identical(l$aes_params$colour, "black") && identical(l$aes_params$alpha, 1)
    }, logical(1))]
    expect_length(discs, 3)
    fills <- p2$layers[vapply(p2$layers, function(l) {
        inherits(l$geom, "GeomScattermore") && !is.null(l$mapping$colour) &&
            identical(l$aes_params$alpha, 0.5)
    }, logical(1))]
    expect_length(fills, 3)
    expect_true(all(mapply(
        function(d, f) d$geom_params$pointsize > f$geom_params$pointsize,
        discs, fills
    )))
    erases <- p2$layers[vapply(p2$layers, function(l) {
        inherits(l$geom, "GeomScattermore") && is.null(l$mapping$colour) &&
            !identical(l$aes_params$colour, "black") && identical(l$aes_params$alpha, 1)
    }, logical(1))]
    expect_length(erases, 3)
})

test_that("VolcanoPlot pt_shape/pt_border_color control vector point layers", {
    # default shape 21 + TRUE border: single layer with fill+colour mapped
    p <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                     color_by = "group")
    point_layers <- p$layers[vapply(p$layers, function(l) inherits(l$geom, "GeomPoint"), logical(1))]
    expect_true(any(vapply(point_layers, function(l) !is.null(l$mapping$fill), logical(1))))
    expect_true(any(vapply(point_layers, function(l) !is.null(l$mapping$colour), logical(1))))
    # shape without fill: colour aesthetic instead
    p1 <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                      color_by = "group", pt_shape = 16)
    point_layers1 <- p1$layers[vapply(p1$layers, function(l) inherits(l$geom, "GeomPoint"), logical(1))]
    expect_true(any(vapply(point_layers1, function(l) !is.null(l$mapping$colour), logical(1))))
    # constant border colour: translucent fill + opaque ring on top
    p2 <- VolcanoPlot(data, x = "logFC", y = "pvalue", nlabel = 0,
                      color_by = "group", pt_border_color = "black")
    point_layers2 <- p2$layers[vapply(p2$layers, function(l) inherits(l$geom, "GeomPoint"), logical(1))]
    expect_true(any(vapply(point_layers2, function(l) identical(l$aes_params$colour, "black"), logical(1))))
    expect_true(any(vapply(point_layers2, function(l) identical(l$aes_params$alpha, 1), logical(1))))
    expect_true(any(vapply(point_layers2, function(l) identical(l$aes_params$stroke, 0.5), logical(1))))
})

test_that("VolcanoPlot raster auto-enables above 1e3 points", {
    big <- rbind(data, data, data, data, data, data, data, data, data, data, data, data)[seq_len(1200), ]
    p <- VolcanoPlot(big, x = "logFC", y = "pvalue", nlabel = 0)
    expect_true(any(vapply(p$layers, function(l) inherits(l$geom, "GeomScattermore"), logical(1))))
    # single raster_dpi value is recycled
    p2 <- VolcanoPlot(big, x = "logFC", y = "pvalue", nlabel = 0, raster_dpi = 256)
    expect_s3_class(p2, "ggplot")
})

#' Single bar plot (no groups)
#'
#' @description
#' Core implementation for drawing a bar plot without grouping — each x-axis
#' category is a single bar.  This is the simpler code path dispatched by
#' \code{\link{BarPlotAtomic}} when \code{group_by = NULL}.  Bars can be
#' filled by a categorical variable, a continuous variable (numeric colour
#' gradient), or a solid colour.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{Column resolution} — \code{x}, \code{y}, and
#'         \code{facet_by} are validated and transformed via
#'         \code{\link{check_columns}}.  Multi-column \code{x} is concatenated
#'         with \code{x_sep}.
#'   \item \strong{Count aggregation} — when \code{y = NULL}, the count of
#'         observations per (\code{x}, \code{facet_by}) combination is
#'         computed as a new \code{.y} column.  Factor levels are preserved.
#'   \item \strong{Fill resolution} — \code{fill_by} can be \code{TRUE}
#'         (use x values), \code{FALSE}/\code{NULL} (solid fill), a
#'         categorical column name (discrete colour scale), or a numeric
#'         column name (continuous gradient).  Discrete fills use
#'         \code{\link{palette_this}()} for colour assignment; numeric fills
#'         use \code{\link{prepare_continuous_color_scale}()} with quantile
#'         / cutoff clamping and \code{scale_fill_gradientn()}.
#'   \item \strong{Background stripes} — when \code{add_bg = TRUE},
#'         \code{\link{bg_layer}()} adds alternating horizontal or vertical
#'         stripe fills behind the bars.
#'   \item \strong{Labels} — when \code{label} is set, values are displayed
#'         on or near the bar tops via \code{\link[ggrepel]{geom_text_repel}()}
#'         (non-flipped) or \code{geom_text()} (flipped).  The y-position is
#'         nudged by \code{label_nudge} × the data range.
#'   \item \strong{Trend line} — when \code{add_trend = TRUE}, a line and
#'         points are overlaid across the bar tops.
#'   \item \strong{Horizontal reference line} — \code{add_line} draws a
#'         horizontal line at the specified y-value, with a colour legend
#'         entry named by \code{line_name}.
#'   \item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
#'         computes plot height and width from the x-axis category count,
#'         label character widths, legend metrics, and flip state.  When
#'         flipped, height scales with the number of x categories.
#'   \item \strong{Coordinate transform} — when \code{flip = TRUE},
#'         \code{coord_flip()} swaps axes; otherwise \code{coord_cartesian()}
#'         applies \code{y_min} / \code{y_max} limits.  Free-scale faceting
#'         skips limit constraints.
#' }
#'
#' @inheritParams common_args
#' @param x A character vector of column name(s) for the x-axis.
#'  Character/factor columns are expected.  Multiple columns are
#'  concatenated with \code{x_sep}.
#' @param x_sep A character string to join multiple \code{x} columns.
#'  Default \code{"_"}.
#' @param y A character string specifying the numeric column for the y-axis.
#'  Default \code{NULL} — the count of observations per x category is used.
#' @param flip Logical; if \code{TRUE}, swap the x and y axes.
#' @param label A column name (or \code{TRUE}) for text labels on bars.
#'  When \code{TRUE}, the y-axis values are labelled.  When a column name,
#'  the values in that column are used.
#' @param label_nudge A numeric value controlling the distance between labels
#'  and the bar top, expressed as a fraction of the data range.
#' @param label_fg A character string specifying the label text colour.
#' @param label_size A numeric value specifying the label text size.
#' @param label_bg A character string specifying the label background colour.
#' @param label_bg_r A numeric value specifying the label background corner
#'  radius.
#' @param add_bg Logical; add alternating background stripes behind the bars.
#' @param bg_palette Palette for the background stripes.
#' @param bg_palcolor Custom colours for the background stripes.
#' @param bg_alpha Alpha transparency for the background stripes.
#' @param fill_by A variable used to fill the bars.  Can be \code{TRUE}
#'  (default; fill by x-axis values), \code{FALSE} (solid fill), or a column
#'  name (categorical or numeric).  When \code{group_by} is used in
#'  \code{\link{BarPlotAtomic}}, this parameter is ignored.
#' @param fill_name A character string for the fill legend title.
#' @param width A numeric value specifying the bar width (0–1).
#' @param add_line A numeric y-intercept for a horizontal reference line.
#' @param line_color Colour of the reference line.
#' @param line_width Width of the reference line.
#' @param line_type Linetype of the reference line (e.g., 1 = solid, 2 = dashed).
#' @param line_name Legend name for the reference line.
#' @param add_trend Logical; add a trend line and points connecting the bar tops.
#' @param trend_color Colour of the trend line.
#' @param trend_linewidth Width of the trend line.
#' @param trend_ptsize Size of the trend line points.
#' @param y_min,y_max Numeric limits for the y-axis (or x-axis when flipped).
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @keywords internal
#' @importFrom rlang sym %||%
#' @importFrom dplyr %>% group_by summarise n
#' @importFrom tidyr complete
#' @importFrom ggplot2 aes geom_bar geom_text scale_fill_manual labs scale_x_discrete scale_y_continuous guide_legend guide_colorbar
#' @importFrom ggplot2 element_line waiver coord_flip scale_color_manual guide_legend coord_cartesian
#' @importFrom ggrepel geom_text_repel
#' @importFrom scales rescale
BarPlotSingle <- function(
    data,
    x,
    x_sep = "_",
    y = NULL,
    flip = FALSE,
    facet_by = NULL,
    facet_scales = "fixed",
    label = NULL,
    label_nudge = 0.02,
    label_fg = "black",
    label_size = 4,
    label_bg = "white",
    label_bg_r = 0.1,
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    theme = "theme_this",
    theme_args = list(),
    palette = NULL,
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    x_text_angle = 0,
    aspect.ratio = 1,
    y_min = NULL,
    y_max = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    add_line = NULL,
    line_color = "red2",
    line_width = .6,
    line_type = 2,
    line_name = NULL,
    add_trend = FALSE,
    trend_color = "black",
    trend_linewidth = 1,
    trend_ptsize = 2.5,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_na = FALSE,
    keep_empty = FALSE,
    expand = waiver(),
    fill_by = TRUE,
    fill_name = NULL,
    width = 0.9,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    if (inherits(width, "waiver")) {
        width <- 0.9
    }

    x <- check_columns(
        data,
        x,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = x_sep
    )
    y <- check_columns(data, y)
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )
    if (is.null(y)) {
        orig_data <- data
        data <- data %>%
            group_by(!!!syms(unique(c(x, facet_by)))) %>%
            summarise(.y = n(), .groups = "drop")
        # keep the factor levels
        for (col in unique(c(x, facet_by))) {
            data[[col]] <- factor(
                data[[col]],
                levels = levels(orig_data[[col]])
            )
        }
        rm(orig_data)
        y <- ".y"
    }
    if (isTRUE(label)) {
        label <- y
    }
    if (isTRUE(fill_by)) {
        fill_by <- x
    }
    if (is.null(fill_by) || isFALSE(fill_by)) {
        fill_by <- ".fill"
        data[[fill_by]] <- factor("")
        fill_guide <- "none"
        fill_is_numeric <- FALSE
        palette <- palette %||% "Paired"
    } else {
        fill_by <- check_columns(data, fill_by, allow_multi = TRUE)
        if (is.numeric(data[[fill_by]])) {
            fill_guide <- guide_colorbar(
                frame.colour = "black",
                ticks.colour = "black",
                title.hjust = 0
            )
            fill_is_numeric <- TRUE
            palette <- palette %||% "Spectral"
        } else {
            fill_by <- check_columns(data, fill_by, force_factor = TRUE)
            fill_guide <- guide_legend(order = 1)
            fill_is_numeric <- FALSE
            palette <- palette %||% "Paired"
        }
    }

    label <- check_columns(data, label)
    if (inherits(expand, "waiver")) {
        expand <- c(0, 0, 0, 0)
        if (!is.null(label)) {
            if (any(data[[y]] > 0)) {
                # what is the best ratio for the label nudge?
                # based on maximum y value?
                expand[1] <- 0.05 + label_nudge * 0.5
            }
            if (any(data[[y]] < 0)) {
                expand[3] <- 0.05 + label_nudge * 0.5
            }
        }
    }
    expand <- norm_expansion(expand, x_type = "discrete", y_type = "continuous")
    data <- process_keep_na_empty(data, keep_na, keep_empty)

    keep_empty_x <- keep_empty[[x]]
    keep_empty_fill <- keep_empty[[fill_by]]

    if (!fill_is_numeric) {
        fill_vals <- levels(data[[fill_by]])
        if (anyNA(data[[fill_by]])) {
            fill_vals <- c(fill_vals, NA)
        }
        colors <- palette_this(
            fill_vals,
            palette = palette,
            palcolor = palcolor,
            NA_keep = TRUE,
            reverse = palreverse
        )
    } else {
        colors <- palette_this(
            palette = palette,
            palcolor = palcolor,
            reverse = palreverse
        )
    }

    feat_colors_value <- NULL
    if (fill_is_numeric) {
        result <- prepare_continuous_color_scale(
            data,
            fill_by,
            lower_quantile = lower_quantile,
            upper_quantile = upper_quantile,
            lower_cutoff = lower_cutoff,
            upper_cutoff = upper_cutoff
        )
        data <- result$data
        feat_colors_value <- result$feat_colors_value
    }

    p <- ggplot(data, aes(x = !!sym(x), y = !!sym(y), fill = !!sym(fill_by)))

    if (isTRUE(add_bg)) {
        p <- p +
            bg_layer(
                data,
                x,
                isTRUE(keep_empty_x),
                bg_palette,
                bg_palcolor,
                bg_alpha,
                facet_by
            )
    }

    just <- calc_just(x_text_angle)

    p <- p + geom_col(alpha = alpha, width = width, show.legend = TRUE)

    if (!is.null(label)) {
        # nudge doesn't work as aes, let's adjust the y
        label_data <- data
        label_data <- label_data[order(label_data[[x]]), , drop = FALSE]
        label_data$.label <- label_data[[label]]
        yr <- diff(range(data[[y]], na.rm = TRUE))
        label_data$.sign <- label_data[[y]] > 0
        label_data[[y]] <- label_data[[y]] +
            yr * label_nudge * ifelse(label_data$.sign, 1, -1)

        if (!isTRUE(flip)) {
            p <- p +
                geom_text_repel(
                    data = label_data,
                    mapping = aes(label = !!sym(".label")),
                    color = label_fg,
                    size = label_size,
                    hjust = if (flip) ifelse(label_data$.sign, 0, 1) else 0.5,
                    bg.color = label_bg,
                    bg.r = label_bg_r,
                    direction = "y",
                    force = 0,
                    min.segment.length = 0,
                    max.overlaps = 100,
                    segment.color = 'transparent'
                )
        } else {
            p <- p +
                geom_text(
                    data = label_data,
                    mapping = aes(label = !!sym(".label")),
                    color = label_fg,
                    size = label_size,
                    hjust = if (flip) ifelse(label_data$.sign, 0, 1) else 0.5,
                    show.legend = FALSE
                )
        }
    }
    p <- p +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% x,
            y = ylab %||% y
        ) +
        scale_x_discrete(expand = expand$x, drop = !isTRUE(keep_empty_x)) +
        scale_y_continuous(expand = expand$y) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major = element_line(colour = "grey80", linetype = 2),
            axis.text.x = element_text(
                angle = x_text_angle,
                hjust = just$h,
                vjust = just$v
            )
        )

    if (!fill_is_numeric) {
        if (isTRUE(keep_empty_fill)) {
            p <- p +
                scale_fill_manual(
                    name = fill_name %||% fill_by,
                    na.value = colors['NA'] %||% 'grey80',
                    values = colors,
                    guide = fill_guide,
                    breaks = fill_vals,
                    limits = fill_vals,
                    drop = FALSE
                )
        } else {
            p <- p +
                scale_fill_manual(
                    name = fill_name %||% fill_by,
                    na.value = colors['NA'] %||% 'grey80',
                    values = colors,
                    guide = fill_guide
                )
        }
    } else {
        p <- p +
            scale_fill_gradientn(
                name = fill_name %||% fill_by,
                colors = colors,
                n.breaks = 3,
                values = if (!is.null(feat_colors_value)) {
                    scales::rescale(feat_colors_value)
                } else {
                    waiver()
                },
                limits = if (!is.null(feat_colors_value)) {
                    range(feat_colors_value)
                } else {
                    NULL
                },
                guide = fill_guide
            )
    }

    if (isTRUE(add_trend)) {
        p <- p +
            geom_line(
                aes(group = 1),
                position = position_dodge(width = 0.9),
                color = trend_color,
                linewidth = trend_linewidth
            ) +
            geom_point(
                position = position_dodge(width = 0.9),
                color = "black",
                fill = "white",
                size = trend_ptsize,
                shape = 21
            )
    }

    if (!is.null(add_line)) {
        p <- p +
            geom_hline(
                aes(
                    color = line_name %||% paste0(y, " = ", add_line),
                    yintercept = add_line
                ),
                linetype = line_type,
                linewidth = line_width
            ) +
            scale_color_manual(
                name = NULL,
                values = line_color,
                guide = guide_legend(order = 2)
            )
    }

    facet_free <- !is.null(facet_by) &&
        (identical(facet_scales, "free") ||
            (!flip && identical(facet_scales, "free_y")) ||
            (flip && identical(facet_scales, "free_x")))
    if (isTRUE(flip) && !facet_free) {
        p <- p + coord_flip(ylim = c(y_min, y_max))
    } else if (isTRUE(flip)) {
        p <- p + coord_flip()
    } else if (!facet_free) {
        p <- p + coord_cartesian(ylim = c(y_min, y_max))
    }

    x_maxchars <- max(nchar(unlist(strsplit(levels(data[[x]]), "\n"))))
    dims <- calculate_plot_dimensions(
        base_height = if (isTRUE(flip)) {
            max(.5 + nlevels(data[[x]]) * .8, 4)
        } else {
            3.5 + max(x_maxchars * 0.1, 1)
        },
        aspect.ratio = aspect.ratio,
        n_x = if (isTRUE(flip)) NULL else nlevels(data[[x]]),
        x_scale_factor = 0.8,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = if (!fill_is_numeric) length(fill_vals) else 1,
        legend_nchar = if (!fill_is_numeric) {
            max(nchar(as.character(fill_vals)), na.rm = TRUE)
        } else {
            5
        },
        flip = isTRUE(flip)
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width + if (isTRUE(flip)) x_maxchars * 0.1 else 0

    p
}

#' Bar plot with groups
#'
#' @description
#' Core implementation for drawing a grouped bar plot.  Each x-axis category
#' is split into side-by-side (dodge) or stacked bars according to the
#' \code{group_by} variable.  This is the grouped code path dispatched by
#' \code{\link{BarPlotAtomic}} when \code{group_by} is provided.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{Column resolution} — \code{group_by}, \code{facet_by},
#'         \code{x}, and \code{y} are validated and transformed via
#'         \code{\link{check_columns}}.
#'   \item \strong{Count aggregation} — when \code{y = NULL}, the count of
#'         observations per (\code{x}, \code{group_by}, \code{facet_by})
#'         combination is computed.  Factor levels are preserved.
#'   \item \strong{Proportion scaling} — when \code{scale_y = TRUE},
#'         y-values are divided by the sum within each (\code{x},
#'         \code{facet_by}) group so that each x position stacks to 100\%.
#'   \item \strong{Position resolution} — \code{position = "auto"} chooses
#'         \code{"dodge"} for \eqn{\le}5 groups or \code{"stack"} for >5 groups.
#'         Explicit \code{"dodge"} and \code{"stack"} are also accepted.
#'   \item \strong{Expand calculation} — for stacked bars, expansion is
#'         computed from y-range and label presence.  For dodged bars,
#'         expansion is minimal.  The \code{\link{norm_expansion}()} utility
#'         normalises the final expansion vector.
#'   \item \strong{Colour mapping} — \code{\link{palette_this}()} assigns
#'         per-group colours.  The fill scale \code{drop} argument is
#'         controlled by \code{keep_empty_group}.
#'   \item \strong{Labels} — when \code{label} is set:
#'         \itemize{
#'           \item For \strong{stacked} bars, cumulative label positions are
#'                 computed so each label is centred within its segment.
#'           \item For \strong{dodged} bars, labels are nudged above/below
#'                 the bar top by \code{label_nudge} × the data range.
#'           \item \code{\link[ggrepel]{geom_text_repel}()} is used for
#'                 automatic overlap avoidance.
#'         }
#'   \item \strong{Trend line} — when \code{add_trend = TRUE}, lines connect
#'         bar tops.  When \code{trend_color} is \code{NULL}, each group gets
#'         its own coloured line; otherwise a single colour is used.
#'   \item \strong{Horizontal reference line} — \code{add_line} draws a
#'         \code{geom_hline()} at the specified y-value.
#'   \item \strong{Dimension calculation} — width accounts for the number of
#'         x categories × number of groups (dodge) or just x categories
#'         (stack).  \code{calculate_plot_dimensions()} adjusts for
#'         \code{flip} and legend metrics.
#'   \item \strong{Coordinate transform} — \code{coord_flip()} or
#'         \code{coord_cartesian()} with \code{y_min} / \code{y_max}.
#' }
#'
#' @inheritParams common_args
#' @inheritParams BarPlotSingle
#' @param group_by A character vector of column name(s) to group the bars by.
#'  Each unique combination becomes a separate bar segment.  Multiple columns
#'  are concatenated with \code{group_by_sep}.  Required.
#' @param group_by_sep A character string to separate concatenated
#'  \code{group_by} columns.  Default \code{"_"}.
#' @param group_name A character string for the group fill legend title.
#'  When \code{NULL}, the \code{group_by} column name is used.
#' @param scale_y A logical value.  When \code{TRUE}, y-values are scaled to
#'  proportions within each x position so that each position's total is 100\%.
#'  Only applicable when \code{position = "stack"}.
#' @param position A character string specifying the bar layout:
#'  \code{"auto"} (default: dodge when \eqn{\le}5 groups, stack otherwise),
#'  \code{"dodge"} (side-by-side), or \code{"stack"} (stacked on top of
#'  each other).
#' @param position_dodge_preserve A character string passed to
#'  \code{\link[ggplot2]{position_dodge2}()}: \code{"total"} preserves the
#'  overall bar group width; \code{"single"} preserves individual bar widths.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @keywords internal
#' @importFrom rlang sym syms %||%
#' @importFrom dplyr %>% summarise n mutate ungroup case_when last first
#' @importFrom ggplot2 aes geom_bar scale_fill_manual labs position_dodge2 coord_flip guide_legend scale_color_manual
BarPlotGrouped <- function(
    data,
    x,
    x_sep = "_",
    y = NULL,
    scale_y = FALSE,
    flip = FALSE,
    group_by,
    group_by_sep = "_",
    group_name = NULL,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    label = NULL,
    label_nudge = 0.02,
    label_fg = "black",
    label_size = 4,
    label_bg = "white",
    label_bg_r = 0.1,
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    alpha = 1,
    x_text_angle = 0,
    aspect.ratio = 1,
    add_line = NULL,
    line_color = "red2",
    line_width = .6,
    line_type = 2,
    line_name = NULL,
    add_trend = FALSE,
    trend_color = "black",
    trend_linewidth = 1,
    trend_ptsize = 2.5,
    position = "auto",
    position_dodge_preserve = "total",
    y_min = NULL,
    y_max = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_na = FALSE,
    keep_empty = FALSE,
    expand = waiver(),
    width = 0.8,
    facet_by = NULL,
    facet_scales = "fixed",
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    group_by <- check_columns(
        data,
        group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = group_by_sep
    )
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )
    x <- check_columns(
        data,
        x,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = x_sep
    )
    y <- check_columns(data, y)
    orig_data <- data
    if (is.null(y)) {
        data <- data %>%
            dplyr::group_by(!!!syms(unique(c(x, group_by, facet_by)))) %>%
            summarise(.y = n(), .groups = "drop")
        # keep the factor levels
        for (col in unique(c(x, group_by, facet_by))) {
            data[[col]] <- factor(
                data[[col]],
                levels = levels(orig_data[[col]])
            )
        }
        y <- ".y"
    }
    if (isTRUE(label)) {
        label <- y
    }
    label <- check_columns(data, label)
    if (isTRUE(scale_y)) {
        y_scaled <- paste0(y, "_scaled")
        data <- data %>%
            dplyr::group_by(!!!syms(unique(c(x, facet_by)))) %>%
            mutate(!!sym(y_scaled) := !!sym(y) / sum(!!sym(y))) %>%
            ungroup()

        for (col in unique(c(x, facet_by))) {
            data[[col]] <- factor(
                data[[col]],
                levels = levels(orig_data[[col]])
            )
        }
    }
    rm(orig_data)

    if (inherits(width, "waiver")) {
        width <- 0.8
    }

    data <- process_keep_na_empty(data, keep_na, keep_empty)

    keep_empty_x <- keep_empty[[x]]
    keep_empty_group <- keep_empty[[group_by]]
    group_vals <- levels(data[[group_by]])
    if (anyNA(data[[group_by]])) {
        group_vals <- c(group_vals, NA)
    }

    p <- ggplot(
        data,
        aes(
            x = !!sym(x),
            y = !!sym(ifelse(scale_y, y_scaled, y)),
            fill = !!sym(group_by)
        )
    )
    if (isTRUE(add_bg)) {
        p <- p +
            bg_layer(
                data,
                x,
                isTRUE(keep_empty_x),
                bg_palette,
                bg_palcolor,
                bg_alpha,
                facet_by
            )
    }

    colors <- palette_this(
        group_vals,
        palette = palette,
        palcolor = palcolor,
        NA_keep = TRUE,
        reverse = palreverse
    )
    just <- calc_just(x_text_angle)
    if (position == "auto") {
        position <- if (length(colors) <= 5) {
            position_dodge2(preserve = position_dodge_preserve, width = width)
        } else {
            "stack"
        }
    } else if (position == "dodge") {
        position <- position_dodge2(
            preserve = position_dodge_preserve,
            width = width
        )
    }
    if (inherits(expand, "waiver")) {
        if (identical(position, "stack")) {
            expand <- c(top = 0, bottom = 0)
            if (!is.null(label)) {
                if (any(data[[y]] > 0)) {
                    expand['top'] <- 0.05 + label_nudge * 0.5
                }
                if (any(data[[y]] < 0)) {
                    expand['bottom'] <- 0.05 + label_nudge * 0.5
                }
            }
        } else if (min(data[[y]], na.rm = TRUE) >= 0) {
            expand <- c(bottom = 0)
        } else if (max(data[[y]], na.rm = TRUE) <= 0) {
            expand <- c(top = 0)
        } else {
            expand <- NULL
        }
    }
    expand <- norm_expansion(expand, x_type = "discrete", y_type = "continuous")

    p <- p +
        geom_col(
            alpha = alpha,
            position = position,
            width = width,
            show.legend = TRUE
        ) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% x,
            y = ylab %||% y
        ) +
        scale_x_discrete(expand = expand$x, drop = !isTRUE(keep_empty_x)) +
        scale_y_continuous(expand = expand$y) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major = element_line(colour = "grey80", linetype = 2),
            axis.text.x = element_text(
                angle = x_text_angle,
                hjust = just$h,
                vjust = just$v
            )
        )

    if (isTRUE(keep_empty_group)) {
        p <- p +
            scale_fill_manual(
                name = group_name %||% group_by,
                values = colors,
                na.value = colors["NA"] %||% "grey80",
                guide = guide_legend(order = 1),
                breaks = group_vals,
                limits = group_vals,
                drop = FALSE
            )
    } else {
        p <- p +
            scale_fill_manual(
                name = group_name %||% group_by,
                values = colors,
                na.value = colors["NA"] %||% "grey80",
                guide = guide_legend(order = 1)
            )
    }

    if (isTRUE(add_trend)) {
        if (is.null(trend_color)) {
            p <- p +
                geom_line(
                    aes(group = !!sym(group_by), color = !!sym(group_by)),
                    position = position_dodge(width = 0.9),
                    linewidth = trend_linewidth,
                    show.legend = FALSE
                ) +
                scale_color_manual(values = colors)
        } else {
            p <- p +
                geom_line(
                    aes(group = !!sym(group_by)),
                    position = position_dodge(width = 0.9),
                    color = trend_color,
                    linewidth = trend_linewidth
                )
        }
        p <- p +
            geom_point(
                position = position_dodge(width = 0.9),
                color = "black",
                fill = "white",
                size = trend_ptsize,
                shape = 21
            )
    }

    if (!is.null(add_line)) {
        p <- p +
            geom_hline(
                aes(
                    color = line_name %||% as.character(add_line),
                    yintercept = add_line
                ),
                linetype = line_type,
                linewidth = line_width
            ) +
            scale_color_manual(
                name = NULL,
                values = line_color,
                guide = guide_legend(order = 2)
            )
    }

    if (!is.null(label)) {
        label_data <- data
        label_data <- label_data[
            order(label_data[[x]], label_data[[group_by]]),
            ,
            drop = FALSE
        ]
        label_data$.label <- label_data[[label]]
        .y <- ifelse(scale_y, y_scaled, y)
        label_data$.sign <- label_data[[.y]] > 0
        yr <- diff(range(data[[.y]], na.rm = TRUE))
        if (identical(position, "stack")) {
            # caculate the y-axis of the labels
            label_data <- mutate(
                label_data,
                !!sym(.y) := ifelse(
                    !!sym(".sign"),
                    cumsum(rev(!!sym(.y))) + yr * label_nudge,
                    cumsum(rev(!!sym(.y))) - yr * label_nudge
                ),
                .label = rev(!!sym(".label")),
                .by = unique(c(x, ".sign", facet_by))
            )

            p <- p +
                geom_text_repel(
                    data = label_data,
                    mapping = aes(label = !!sym(".label")),
                    color = label_fg,
                    size = label_size,
                    hjust = if (flip) ifelse(label_data$.sign, 0, 1) else 0.5,
                    bg.color = label_bg,
                    bg.r = label_bg_r,
                    direction = "y",
                    force = 0,
                    min.segment.length = 0,
                    max.overlaps = 100,
                    segment.color = 'transparent'
                )
        } else {
            label_data[[y]] <- label_data[[y]] +
                yr * label_nudge * ifelse(label_data$.sign, 1, -1)

            p <- p +
                geom_text_repel(
                    data = label_data,
                    mapping = aes(label = !!sym(".label")),
                    color = label_fg,
                    size = label_size,
                    hjust = if (flip) ifelse(label_data$.sign, 0, 1) else 0.5,
                    bg.color = label_bg,
                    bg.r = label_bg_r,
                    position = position,
                    direction = "y",
                    force = 0,
                    min.segment.length = 0,
                    max.overlaps = 100,
                    segment.color = 'transparent'
                )
        }
    }

    facet_free <- !is.null(facet_by) &&
        (identical(facet_scales, "free") ||
            (!flip && identical(facet_scales, "free_y")) ||
            (flip && identical(facet_scales, "free_x")))
    if (isTRUE(flip) && facet_free) {
        p <- p + coord_flip()
    } else if (isTRUE(flip)) {
        p <- p + coord_flip(ylim = c(y_min, y_max))
    } else if (!facet_free) {
        p <- p + coord_cartesian(ylim = c(y_min, y_max))
    }

    n_groups <- length(unique(data[[group_by]]))
    dims <- calculate_plot_dimensions(
        base_height = 4.5,
        aspect.ratio = aspect.ratio,
        n_x = if (is.character(position) && position == "stack") {
            nlevels(data[[x]])
        } else {
            nlevels(data[[x]]) * n_groups
        },
        x_scale_factor = if (is.character(position) && position == "stack") {
            0.8
        } else {
            0.5
        },
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = n_groups,
        legend_nchar = max(nchar(levels(data[[group_by]]))),
        flip = isTRUE(flip)
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    p
}

#' Atomic bar plot (internal)
#'
#' @description
#' Dispatcher that routes to \code{\link{BarPlotSingle}} (when
#' \code{group_by = NULL}) or \code{\link{BarPlotGrouped}} (when
#' \code{group_by} is provided).  This is the core implementation layer —
#' it takes a **single** data frame (no \code{split_by} support) and returns
#' a \code{ggplot} object with faceting applied.
#'
#' @section Dispatch logic:
#' \itemize{
#'   \item \strong{Without \code{group_by}} — delegates to
#'         \code{BarPlotSingle}.  \code{fill_by} controls bar colouring:
#'         \code{TRUE} (default) fills by x-axis values, \code{FALSE} uses a
#'         single colour, a character column produces discrete colours, and
#'         a numeric column produces a continuous gradient.
#'   \item \strong{With \code{group_by}} — delegates to
#'         \code{BarPlotGrouped}.  \code{fill_by} must match \code{group_by}
#'         or be left as default; an explicit mismatch raises a stop error.
#'         \code{position} controls dodge vs. stack layout with automatic
#'         selection based on group count (\eqn{\le}5 → dodge, >5 → stack).
#' }
#'
#' After the delegate returns, \code{\link{facet_plot}()} wraps the result
#' with \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is provided.
#'
#' @inheritParams common_args
#' @inheritParams BarPlotSingle
#' @inheritParams BarPlotGrouped
#' @param fill_by A variable used to fill the bars.  Both categorical and
#'  numeric columns are accepted:
#'  \itemize{
#'    \item \code{TRUE} (default) — fill by the x-axis values.
#'    \item \code{FALSE} — solid fill (first palette colour).
#'    \item A column name (character/factor) — discrete colour scale.
#'    \item A column name (numeric) — continuous gradient with quantile
#'          / cutoff controls.
#'  }
#'  Ignored when \code{group_by} is provided (fill is determined by
#'  \code{group_by}).
#' @param fill_name A character string for the fill legend title.  Only
#'  applies when \code{group_by = NULL} and the fill is from \code{fill_by}.
#' @param facet_args A list of additional arguments passed to the faceting
#'  function (e.g., \code{scales}, \code{labeller}).
#' @return A \code{ggplot} object, possibly faceted, with \code{height}
#'  and \code{width} attributes (in inches) attached.
#' @importFrom ggplot2 waiver
#' @keywords internal
BarPlotAtomic <- function(
    data,
    x,
    x_sep = "_",
    y = NULL,
    scale_y = FALSE,
    flip = FALSE,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    fill_by = TRUE,
    fill_name = NULL,
    label_nudge = 0.02,
    label = NULL,
    label_fg = "black",
    label_size = 4,
    label_bg = "white",
    label_bg_r = 0.1,
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    theme = "theme_this",
    theme_args = list(),
    palette = NULL,
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    x_text_angle = 0,
    aspect.ratio = 1,
    add_line = NULL,
    line_color = "red2",
    line_width = .6,
    line_type = 2,
    line_name = NULL,
    add_trend = FALSE,
    trend_color = "black",
    trend_linewidth = 1,
    trend_ptsize = 2,
    position = "auto",
    position_dodge_preserve = "total",
    y_min = NULL,
    y_max = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_na = FALSE,
    keep_empty = FALSE,
    expand = waiver(),
    width = waiver(),
    facet_by = NULL,
    facet_scales = "fixed",
    facet_nrow = NULL,
    facet_ncol = NULL,
    facet_byrow = TRUE,
    facet_args = list(),
    ...
) {
    if (is.null(group_by)) {
        p <- BarPlotSingle(
            data,
            x,
            x_sep,
            y,
            label = label,
            label_nudge = label_nudge,
            label_fg = label_fg,
            label_size = label_size,
            label_bg = label_bg,
            label_bg_r = label_bg_r,
            facet_by = facet_by,
            facet_scales = facet_scales,
            flip = flip,
            line_name = line_name,
            theme = theme,
            theme_args = theme_args,
            palette = palette,
            palcolor = palcolor,
            palreverse = palreverse,
            alpha = alpha,
            lower_quantile = lower_quantile,
            upper_quantile = upper_quantile,
            lower_cutoff = lower_cutoff,
            upper_cutoff = upper_cutoff,
            x_text_angle = x_text_angle,
            aspect.ratio = aspect.ratio,
            add_bg = add_bg,
            add_line = add_line,
            line_color = line_color,
            line_width = line_width,
            line_type = line_type,
            add_trend = add_trend,
            trend_color = trend_color,
            trend_linewidth = trend_linewidth,
            trend_ptsize = trend_ptsize,
            legend.position = legend.position,
            legend.direction = legend.direction,
            y_min = y_min,
            y_max = y_max,
            title = title,
            subtitle = subtitle,
            xlab = xlab,
            ylab = ylab,
            keep_na = keep_na,
            keep_empty = keep_empty,
            expand = expand,
            fill_by = fill_by,
            fill_name = fill_name,
            width = width,
            ...
        )
    } else {
        stopifnot(
            "[BarPlot] `fill_by` cannot be applied when `group_by` is specified." = !missing(
                fill_by
            ) ||
                identical(fill_by, group_by)
        )
        p <- BarPlotGrouped(
            data,
            x,
            x_sep,
            y,
            label = label,
            label_nudge = label_nudge,
            label_fg = label_fg,
            label_size = label_size,
            label_bg = label_bg,
            label_bg_r = label_bg_r,
            scale_y = scale_y,
            group_by = group_by,
            group_by_sep = group_by_sep,
            group_name = group_name,
            facet_by = facet_by,
            facet_scales = facet_scales,
            flip = flip,
            line_name = line_name,
            add_bg = add_bg,
            bg_palette = bg_palette,
            bg_palcolor = bg_palcolor,
            bg_alpha = bg_alpha,
            theme = theme,
            theme_args = theme_args,
            palette = palette %||% "Paired",
            palcolor = palcolor,
            palreverse = palreverse,
            alpha = alpha,
            x_text_angle = x_text_angle,
            aspect.ratio = aspect.ratio,
            position = position,
            position_dodge_preserve = position_dodge_preserve,
            y_min = y_min,
            y_max = y_max,
            add_line = add_line,
            line_color = line_color,
            line_width = line_width,
            line_type = line_type,
            add_trend = add_trend,
            trend_color = trend_color,
            trend_linewidth = trend_linewidth,
            trend_ptsize = trend_ptsize,
            legend.position = legend.position,
            legend.direction = legend.direction,
            title = title,
            subtitle = subtitle,
            xlab = xlab,
            ylab = ylab,
            keep_na = keep_na,
            keep_empty = keep_empty,
            expand = expand,
            width = width,
            ...
        )
    }
    keep_empty_facet <- if (!is.null(facet_by)) {
        keep_empty[[facet_by[1]]]
    } else {
        NULL
    }
    if (length(facet_by) > 1) {
        stopifnot(
            "[BarPlot] `keep_empty` for `facet_by` variables must be identical." = identical(
                keep_empty_facet,
                keep_empty[[facet_by[2]]]
            )
        )
    }

    facet_args$plot <- p
    facet_args["facet_by"] <- list(facet_by)
    facet_args["facet_scales"] <- list(facet_scales)
    facet_args["drop"] <- list(!isTRUE(keep_empty_facet))
    facet_args["nrow"] <- list(facet_nrow)
    facet_args["ncol"] <- list(facet_ncol)
    facet_args["byrow"] <- list(facet_byrow)
    facet_args["legend.position"] <- list(legend.position)
    facet_args["legend.direction"] <- list(legend.direction)
    do_call(facet_plot, facet_args)
}

#' Bar / SplitBar / Waterfall Plot
#'
#' @description
#' `BarPlot` draws bar plots with flexible fill, grouping, labelling, and annotation
#' options.  Supports both simple single-colour bars and grouped bars (dodged
#' or stacked).  Bars can be filled by a categorical variable (discrete
#' colour scale), a continuous variable (colour gradient), or a fixed colour.
#'
#' The function supports \strong{count aggregation} (omit \code{y} to plot
#' observation counts), \strong{proportion scaling} (\code{scale_y = TRUE}
#' for grouped bars), background stripes (\code{add_bg}), bar labels, trend
#' lines, horizontal reference lines, and splitting into separate sub-plots
#' via \code{split_by}.
#'
#' @section split_by workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
#'         normalise the \code{keep_na} / \code{keep_empty} arguments for all
#'         columns (\code{x}, \code{split_by}, \code{facet_by}, \code{group_by}).
#'   \item The \code{split_by} column is validated and its NA / empty levels
#'         are processed.  It is then removed from the per-column
#'         \code{keep_na} / \code{keep_empty} lists.
#'   \item The data is split by \code{split_by} (preserving level order).  If
#'         \code{split_by} is \code{NULL}, the data is wrapped in a
#'         single-element list with name \code{"..."}.
#'   \item Per-split \code{palette}, \code{palcolor},
#'         \code{legend.position}, and \code{legend.direction} are resolved
#'         via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
#'         and \code{\link{check_legend}()}.
#'   \item \code{\link{BarPlotAtomic}()} is called for each split.
#'   \item Results are combined via \code{\link{combine_plots}()} (when
#'         \code{combine = TRUE}) or returned as a named list.
#' }
#'
#' @inheritParams common_args
#' @inheritParams BarPlotAtomic
#' @param split_by The column(s) to split the data by and produce separate
#'  sub-plots.  Multiple columns are concatenated with \code{split_by_sep}.
#' @param split_by_sep A character string to separate concatenated
#'  \code{split_by} columns.  Default \code{"_"}.
#' @param seed A numeric seed for reproducibility.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'  \code{patchwork} object.  When \code{FALSE}, returns a named list of
#'  individual \code{ggplot} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined layout.
#' @param byrow Logical; fill the combined layout by row (default \code{TRUE}).
#' @param axes,axis_titles Character strings for how axes and axis titles are
#'  treated across the combined layout.
#' @param guides A character string specifying how legends are collected
#'  across panels.
#' @param design A custom layout design for the combined plot.
#' @rdname barplot
#' @return A \code{ggplot} object, a \code{patchwork} object, or a named list
#'  of \code{ggplot} objects (when \code{combine = FALSE}), each with
#'  \code{height} and \code{width} attributes in inches.
#' @export
#' @importFrom ggplot2 waiver
#' @examples
#' \donttest{
#' data <- data.frame(
#'     x = c("A", "B", "C", "D", "E", "F", "G", "H"),
#'     y = c(10, 8, 16, 4, 6, 12, 14, 2),
#'     group = c("G1", "G1", "G2", "G2", "G3", "G3", "G4", "G4"),
#'     facet = c("F1", "F2", "F3", "F4", "F1", "F2", "F3", "F4")
#' )
#'
#' # Single-colour bars
#' BarPlot(data, x = "x", y = "y")
#'
#' # Solid fill (no colour mapping)
#' BarPlot(data, x = "x", y = "y", fill_by = FALSE)
#'
#' # Label bar tops
#' BarPlot(data, x = "x", y = "y", label = TRUE)
#' BarPlot(data, x = "x", y = "y", label = "facet", label_nudge = 0)
#'
#' # Grouped bars
#' BarPlot(data, x = "group", y = "y", group_by = "x")
#'
#' # Dodged bars with background stripes
#' BarPlot(data,
#'     x = "group", y = "y", group_by = "x",
#'     position = "dodge", add_bg = TRUE)
#'
#' # split_by with faceting
#' BarPlot(data,
#'     x = "x", y = "y", split_by = "group",
#'     facet_by = "facet", position = "dodge", facet_ncol = 1)
#'
#' # split_by with collected guides
#' BarPlot(data,
#'     x = "x", y = "y", split_by = "group", facet_by = "facet",
#'     position = "dodge", facet_ncol = 1, guides = 'collect')
#'
#' # Per-split palettes
#' BarPlot(data,
#'     x = "x", y = "y", split_by = "group",
#'     palette = list(G1 = "Reds", G2 = "Blues", G3 = "Greens", G4 = "Purp"),
#'     facet_by = "facet", position = "dodge", facet_ncol = 1)
#'
#' # Background stripe palette
#' BarPlot(data,
#'     x = "group", y = "y", group_by = "x",
#'     position = "dodge", add_bg = TRUE, bg_palette = "Spectral")
#'
#' # Count bars (y = NULL)
#' BarPlot(data, x = "group", ylab = "count")
#'
#' # Flipped axes
#' BarPlot(data, x = "group", flip = TRUE, ylab = "count")
#'
#' # Numeric fill_by with colour gradient
#' BarPlot(data, x = "x", y = "y", fill_by = "y", flip = TRUE)
#'
#' # Control fill colour scale limits (quantile)
#' BarPlot(data, x = "x", y = "y", fill_by = "y", flip = TRUE,
#'         lower_quantile = 0.1, upper_quantile = 0.9)
#'
#' # Control fill colour scale limits (explicit cutoff)
#' BarPlot(data, x = "x", y = "y", fill_by = "y", flip = TRUE,
#'         lower_cutoff = 5, upper_cutoff = 12)
#'
#' # keep_na and keep_empty examples
#' data <- data.frame(
#'     x = factor(c("A", "B", "C", "D", "E", "F", NA, "H"),
#'                levels = LETTERS[1:10]),
#'     y = c(10, 8, 16, 4, 6, NA, 14, 2),
#'     group = factor(c("G1", "G1", "G2", NA, "G3", "G3", "G5", "G5"),
#'                    levels = c("G1", "G2", "G3", "G4", "G5")),
#'     facet = factor(c("F1", NA, "F3", "F4", "F1", "F2", "F3", "F4"),
#'                    levels = c("F1", "F2", "F3", "F4", "F5"))
#' )
#'
#' # Default: NA and empty levels dropped
#' BarPlot(data, x = "x", y = "y")
#'
#' # Keep both NA and empty levels
#' BarPlot(data, x = "x", y = "y",
#'         keep_na = TRUE, keep_empty = TRUE)
#'
#' # With faceting
#' BarPlot(data, x = "x", y = "y",
#'         keep_na = TRUE, keep_empty = TRUE, facet_by = "facet")
#'
#' # Keep NA, hide empty levels but reserve their colours
#' BarPlot(data, x = "x", y = "y",
#'         keep_na = TRUE, keep_empty = 'level')
#'
#' # Per-column keep_na / keep_empty
#' BarPlot(data, x = "x", y = "y",
#'         keep_na = list(x = TRUE), keep_empty = list(x = FALSE))
#'
#' # Grouped bars with keep_na / keep_empty
#' BarPlot(data, x = "group", y = "y", group_by = "x")
#' BarPlot(data, x = "group", y = "y", group_by = "x",
#'         keep_na = TRUE, keep_empty = TRUE)
#' BarPlot(data, x = "group", y = "y", group_by = "x",
#'         keep_na = TRUE, keep_empty = TRUE, facet_by = "facet")
#'
#' # Per-column on grouped bars
#' BarPlot(data, x = "group", y = "y", group_by = "x",
#'         keep_na = list(x = TRUE, group = FALSE),
#'         keep_empty = list(x = FALSE, group = TRUE))
#' }
BarPlot <- function(
    data,
    x,
    x_sep = "_",
    y = NULL,
    flip = FALSE,
    fill_by = TRUE,
    fill_name = NULL,
    line_name = NULL,
    label_nudge = 0.02,
    label = NULL,
    label_fg = "black",
    label_size = 4,
    label_bg = "white",
    label_bg_r = 0.1,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    split_by = NULL,
    split_by_sep = "_",
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    facet_args = list(),
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    add_line = NULL,
    line_color = "red2",
    line_width = .6,
    line_type = 2,
    add_trend = FALSE,
    trend_color = "black",
    trend_linewidth = 1,
    trend_ptsize = 2,
    theme = "theme_this",
    theme_args = list(),
    palette = NULL,
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    x_text_angle = 0,
    aspect.ratio = 1,
    y_min = NULL,
    y_max = NULL,
    position = "auto",
    position_dodge_preserve = "total",
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_empty = FALSE,
    keep_na = FALSE,
    expand = waiver(),
    width = waiver(),
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    seed = 8525,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    keep_na <- check_keep_na(keep_na, c(x, split_by, facet_by, group_by))
    keep_empty <- check_keep_empty(
        keep_empty,
        c(x, split_by, facet_by, group_by)
    )
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL

        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }

    palette <- if (is.null(palette)) {
        list()
    } else {
        check_palette(palette, names(datas))
    }
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            title <- title %||%
                (if (length(datas) == 1 && identical(nm, "...")) NULL else nm)
            BarPlotAtomic(
                datas[[nm]],
                label = label,
                label_nudge = label_nudge,
                label_fg = label_fg,
                label_size = label_size,
                label_bg = label_bg,
                label_bg_r = label_bg_r,
                x = x,
                x_sep = x_sep,
                y = y,
                flip = flip,
                group_by = group_by,
                group_by_sep = group_by_sep,
                group_name = group_name,
                fill_by = fill_by,
                fill_name = fill_name,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                alpha = alpha,
                lower_quantile = lower_quantile,
                upper_quantile = upper_quantile,
                lower_cutoff = lower_cutoff,
                upper_cutoff = upper_cutoff,
                add_bg = add_bg,
                bg_palette = bg_palette,
                bg_palcolor = bg_palcolor,
                bg_alpha = bg_alpha,
                x_text_angle = x_text_angle,
                aspect.ratio = aspect.ratio,
                line_name = line_name,
                add_line = add_line,
                line_color = line_color,
                line_width = line_width,
                line_type = line_type,
                add_trend = add_trend,
                trend_color = trend_color,
                trend_linewidth = trend_linewidth,
                trend_ptsize = trend_ptsize,
                position = position,
                position_dodge_preserve = position_dodge_preserve,
                y_min = y_min,
                y_max = y_max,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                keep_empty = keep_empty,
                keep_na = keep_na,
                expand = expand,
                width = width,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                facet_args = facet_args,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

#' Atomic split bar plot (internal)
#'
#' @description
#' Core implementation for drawing a split (divergent / waterfall-style) bar
#' plot.  Bars extend left (negative values) and right (positive values) from
#' a central zero line, with the y-axis listing categories.  Bars can be
#' coloured by a categorical or continuous fill variable, and their opacity
#' can encode an additional numeric variable via \code{alpha_by}.
#'
#' This is the workhorse behind the exported \code{\link{SplitBarPlot}}
#' (also aliased as \code{\link{WaterfallPlot}}).
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{Column resolution} — \code{x}, \code{y}, \code{fill_by},
#'         \code{alpha_by}, and \code{facet_by} are validated via
#'         \code{\link{check_columns}}.  Multi-column \code{y} and
#'         \code{fill_by} are concatenated with their respective separators.
#'   \item \strong{Direction assignment} — a \code{direction_name} column
#'         is created from the sign of \code{x}, assigning each row to the
#'         positive or negative group (customisable via
#'         \code{direction_pos_name} / \code{direction_neg_name}).
#'   \item \strong{Fill resolution} — if \code{fill_by} is \code{NULL},
#'         the direction column is used as the fill (two-colour palette).
#'         Categorical fills use a discrete colour scale; numeric fills use
#'         \code{\link{prepare_continuous_color_scale}()} with quantile /
#'         cutoff clamping and \code{scale_fill_gradientn()}.
#'   \item \strong{Alpha by} — when \code{alpha_by} is provided, bar
#'         opacity encodes an additional numeric variable via
#'         \code{scale_alpha_continuous()}.
#'   \item \strong{Ordering} — \code{order_y} is a named list controlling
#'         the vertical ordering of bars.  The keys \code{"+"} and \code{"-"}
#'         specify separate orderings for positive and negative bars; key
#'         \code{"*"} applies a single ordering to all bars.  Values are
#'         character vectors of ordering criteria: \code{"x_asc"},
#'         \code{"x_desc"}, \code{"alpha_asc"}, \code{"alpha_desc"}.
#'   \item \strong{Empty level padding} — when \code{keep_empty_y = TRUE},
#'         missing y-levels are padded per facet (or globally) with
#'         zero-height bars so they still appear on the axis.
#'   \item \strong{Text labels} — category names are drawn beside the bars
#'         via \code{geom_text()} at the zero line.  When flipped, labels are
#'         rotated 90°.  Long labels are wrapped via
#'         \code{\link[stringr]{str_wrap}()} using \code{max_charwidth}.
#'   \item \strong{Plot assembly} — the ggplot is built with
#'         \code{geom_vline(xintercept = 0)} for the centre line,
#'         \code{geom_col()} for bars, and the appropriate fill / alpha
#'         scales.
#'   \item \strong{Dimension calculation} — \code{calculate_plot_dimensions()}
#'         computes dimensions from the number of y categories, scaled by
#'         \code{bar_height / 4}.  Flipping adjusts the aspect ratio.
#'   \item \strong{Faceting} — \code{\link{facet_plot}()} wraps the result,
#'         defaulting to \code{facet_scales = "free_y"}.
#' }
#'
#' @inheritParams common_args
#' @param x A character string specifying the numeric column for the
#'  x-axis.  Values determine bar direction: positive → right, negative →
#'  left.
#' @param y A character string (or vector) specifying the column(s) for the
#'  category axis.  Each unique value becomes a bar.  Multiple columns are
#'  concatenated with \code{y_sep}.
#' @param y_sep A character string to join multiple \code{y} columns.
#'  Default \code{"_"}.
#' @param flip Logical; if \code{TRUE}, swap the x and y axes (bars become
#'  vertical rather than horizontal).
#' @param alpha_by A character string naming a numeric column to encode as
#'  bar opacity.  Default \code{NULL} (all bars fully opaque).
#' @param alpha_reverse Logical; if \code{TRUE}, reverse the alpha scale
#'  direction (solid for low values, transparent for high).
#' @param alpha_name A character string for the alpha legend title.
#' @param order_y A named list controlling the vertical ordering of bars.
#'  Keys are \code{"+"} (positive bars), \code{"-"} (negative bars), or
#'  \code{"*"} (all bars).  Values are character vectors of ordering
#'  criteria: \code{"x_asc"}, \code{"x_desc"}, \code{"alpha_asc"},
#'  \code{"alpha_desc"}.  Default orders positive bars by descending x and
#'  descending alpha; negative bars by descending x and ascending alpha.
#' @param bar_height A numeric value (0–1) specifying the bar height as a
#'  fraction of the available category slot.
#' @param lineheight A numeric value controlling the line height of wrapped
#'  category labels.
#' @param max_charwidth An integer specifying the maximum character width
#'  for wrapping category labels.
#' @param fill_by A character string (or vector) naming the column(s) for
#'  bar fill colour.  If \code{NULL}, the direction (positive/negative) is
#'  used.  Can be categorical or numeric.
#' @param fill_by_sep A character string to join multiple \code{fill_by}
#'  columns.  Default \code{"_"}.
#' @param fill_name A character string for the fill legend title.
#' @param direction_name A character string naming the internal direction
#'  column (used in legends).  Default \code{"direction"}.
#' @param direction_pos_name A character string labelling the positive
#'  direction in the legend.  Default \code{"positive"}.
#' @param direction_neg_name A character string labelling the negative
#'  direction in the legend.  Default \code{"negative"}.
#' @param x_min,x_max Numeric limits for the x-axis.  When \code{NULL},
#'  symmetric limits are computed from the maximum absolute x-value.
#' @return A \code{ggplot} object, possibly faceted, with \code{height}
#'  and \code{width} attributes (in inches) attached.
#' @keywords internal
#' @importFrom stringr str_wrap
#' @importFrom forcats fct_relabel
#' @importFrom dplyr .data
#' @importFrom ggplot2 aes geom_vline geom_col geom_text scale_fill_manual labs scale_y_discrete scale_alpha_continuous
#' @importFrom ggplot2 scale_alpha_continuous guide_none guide_legend
#' @importFrom scales rescale
SplitBarPlotAtomic <- function(
    data,
    x,
    y,
    y_sep = "_",
    flip = FALSE,
    alpha_by = NULL,
    alpha_reverse = FALSE,
    alpha_name = NULL,
    order_y = list(
        "+" = c("x_desc", "alpha_desc"),
        "-" = c("x_desc", "alpha_asc")
    ),
    bar_height = 0.9,
    lineheight = 0.5,
    max_charwidth = 80,
    fill_by = NULL,
    fill_by_sep = "_",
    fill_name = NULL,
    direction_name = "direction",
    direction_pos_name = "positive",
    direction_neg_name = "negative",
    theme = "theme_this",
    theme_args = list(),
    palette = "Spectral",
    palcolor = NULL,
    palreverse = FALSE,
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    facet_by = NULL,
    facet_scales = "free_y",
    facet_nrow = NULL,
    facet_ncol = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    x_min = NULL,
    x_max = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_empty = FALSE,
    keep_na = FALSE,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    x <- check_columns(data, x)
    y <- check_columns(
        data,
        y,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = y_sep
    )
    fill_by <- check_columns(data, fill_by, allow_multi = TRUE)
    fill_by <- fill_by %||% direction_name
    data[[direction_name]] <- ifelse(
        data[[x]] > 0,
        direction_pos_name,
        direction_neg_name
    )
    data[[direction_name]] <- factor(
        data[[direction_name]],
        levels = c(direction_pos_name, direction_neg_name)
    )
    if (is.numeric(data[[fill_by]])) {
        fill_by_numeric <- TRUE
    } else {
        fill_by_numeric <- FALSE
        fill_by <- check_columns(
            data,
            fill_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = fill_by_sep
        )
    }

    feat_colors_value <- NULL
    if (fill_by_numeric) {
        result <- prepare_continuous_color_scale(
            data,
            fill_by,
            lower_quantile = lower_quantile,
            upper_quantile = upper_quantile,
            lower_cutoff = lower_cutoff,
            upper_cutoff = upper_cutoff
        )
        data <- result$data
        feat_colors_value <- result$feat_colors_value
    }

    alpha_by <- check_columns(data, alpha_by)
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )

    if (is.null(alpha_by)) {
        data$.alpha <- 1
        alpha_by <- ".alpha"
        alpha_guide <- guide_none()
    } else {
        alpha_guide <- guide_legend(order = 2)
    }

    if (!is.null(order_y)) {
        if (!is.list(order_y)) {
            order_y <- list("*" = order_y)
        }
        if (length(order_y) != 1 && length(order_y) != 2) {
            stop("'order_y' must be a list of length 1 or 2.")
        }
        if (length(order_y) == 1 && names(order_y) != "*") {
            stop(
                "The name of the 'order_y' list must be '*' when it has only one element."
            )
        }
        if (length(order_y) == 2 && !all(c("+", "-") %in% names(order_y))) {
            stop(
                "The names of the 'order_y' list must be '+' and '-' when it has two elements."
            )
        }
        for (o in order_y) {
            sapply(
                o,
                match.arg,
                c("x_asc", "x_desc", "alpha_asc", "alpha_desc")
            )
        }

        order_df <- function(df, dirs) {
            order_list <- list()
            for (dir in dirs) {
                if (dir == "x_asc") {
                    order_list[[dir]] <- df[[x]]
                } else if (dir == "x_desc") {
                    order_list[[dir]] <- -df[[x]]
                } else if (dir == "alpha_asc") {
                    order_list[[dir]] <- df[[alpha_by]]
                } else if (dir == "alpha_desc") {
                    order_list[[dir]] <- -df[[alpha_by]]
                }
            }
            df[do_call(order, order_list), , drop = FALSE]
        }
        if (length(order_y) == 1) {
            # *
            data <- order_df(data, order_y[[1]])
        } else {
            # +, -
            data[[direction_name]][is.na(data[[x]])] <- "positive"
            data[[x]][is.na(data[[x]])] <- 0
            data_pos <- order_df(
                data[data[[direction_name]] == direction_pos_name, ],
                order_y[["+"]]
            )
            data_neg <- order_df(
                data[data[[direction_name]] == direction_neg_name, ],
                order_y[["-"]]
            )
            data <- rbind(data_pos, data_neg)
            rm(data_pos, data_neg)
        }
        ordered_levels <- rev(unique(as.character(data[[y]])))
        ordered_levels <- c(
            ordered_levels,
            setdiff(levels(data[[y]]), unique(data[[y]]))
        )
        data[[y]] <- factor(data[[y]], levels = ordered_levels)
    }
    data <- process_keep_na_empty(data, keep_na, keep_empty)
    data[[y]] <- fct_relabel(data[[y]], str_wrap, width = max_charwidth)

    # true: unused levels are kept on Y axis
    # false: unused levels are dropped
    # level: unused levels are dropped, but kept for determine fill_by colors
    keep_empty_y <- keep_empty[[y]]
    keep_empty_fill <- if (!fill_by_numeric) keep_empty[[fill_by]] else NULL
    keep_empty_facet <- if (!is.null(facet_by)) {
        keep_empty[[facet_by[1]]]
    } else {
        NULL
    }
    if (length(facet_by) > 1) {
        stopifnot(
            "[SplitBarPlot] `keep_empty` for `facet_by` variables must be identical." = identical(
                keep_empty_facet,
                keep_empty[[facet_by[2]]]
            )
        )
    }
    if (isTRUE(keep_empty_y)) {
        # Determine all possible y values globally (levels + NA if present anywhere)
        all_y_vals_global <- unique(c(
            levels(data[[y]]),
            if (anyNA(data[[y]])) NA else NULL
        ))

        # Create facet grouping (or single group if no facets)
        if (!is.null(facet_by)) {
            facet_cols <- if (length(facet_by) > 1) {
                interaction(data[facet_by], drop = TRUE, sep = "___")
            } else {
                data[[facet_by[1]]]
            }
        } else {
            facet_cols <- factor(rep("__single__", nrow(data)))
        }

        # Add missing levels per facet (or globally if no facets)
        data_list <- split(data, facet_cols)
        data_list <- lapply(names(data_list), function(facet_val) {
            facet_data <- data_list[[facet_val]]
            y_vals <- unique(facet_data[[y]])
            missing_levels <- setdiff(all_y_vals_global, y_vals)

            if (length(missing_levels) > 0) {
                add_data <- data.frame(matrix(
                    NA,
                    nrow = length(missing_levels),
                    ncol = ncol(facet_data)
                ))
                colnames(add_data) <- colnames(facet_data)
                add_data[[y]] <- missing_levels
                add_data[[x]] <- 0
                add_data[[direction_name]] <- "positive"

                # Set facet variables if they exist
                if (!is.null(facet_by)) {
                    if (length(facet_by) > 1) {
                        facet_vals <- strsplit(facet_val, "___", fixed = TRUE)[[
                            1
                        ]]
                        for (i in seq_along(facet_by)) {
                            add_data[[facet_by[i]]] <- facet_vals[i]
                        }
                    } else {
                        add_data[[facet_by[1]]] <- facet_val
                    }
                }

                facet_data <- rbind(facet_data, add_data)
            }
            facet_data
        })
        data <- do_call(rbind, data_list)
        rownames(data) <- NULL
    }

    x_min <- x_min %||% -max(abs(data[[x]]))
    x_max <- x_max %||% max(abs(data[[x]]))

    p <- ggplot(data, aes(x = !!sym(x), y = !!sym(y))) +
        geom_vline(xintercept = 0) +
        geom_col(
            aes(fill = !!sym(fill_by), alpha = !!sym(alpha_by)),
            color = "black",
            width = bar_height,
            show.legend = TRUE
        ) +
        scale_alpha_continuous(
            name = alpha_name %||% alpha_by,
            range = if (alpha_reverse) c(1, 0.1) else c(0.1, 1),
            guide = alpha_guide
        )

    if (!fill_by_numeric) {
        fill_vals <- if (!isFALSE(keep_empty_fill) || !anyNA(data[[fill_by]])) {
            levels(data[[fill_by]])
        } else {
            c(levels(data[[fill_by]]), NA)
        }
        fill_colors <- palette_this(
            fill_vals,
            palette = palette,
            palcolor = palcolor,
            NA_keep = TRUE,
            reverse = palreverse
        )

        if (isTRUE(keep_empty_fill)) {
            p <- p +
                scale_fill_manual(
                    name = fill_name %||% fill_by,
                    values = fill_colors,
                    na.value = fill_colors['NA'] %||% "grey80",
                    guide = guide_legend(order = 1),
                    breaks = fill_vals,
                    limits = fill_vals,
                    drop = FALSE
                )
        } else {
            p <- p +
                scale_fill_manual(
                    name = fill_name %||% fill_by,
                    values = fill_colors,
                    na.value = fill_colors['NA'] %||% "grey80",
                    guide = guide_legend(order = 1)
                )
        }
    } else {
        p <- p +
            scale_fill_gradientn(
                name = fill_name %||% fill_by,
                n.breaks = 3,
                colors = palette_this(
                    palette = palette,
                    palcolor = palcolor,
                    reverse = palreverse
                ),
                values = if (!is.null(feat_colors_value)) {
                    scales::rescale(feat_colors_value)
                } else {
                    waiver()
                },
                limits = if (!is.null(feat_colors_value)) {
                    range(feat_colors_value)
                } else {
                    NULL
                },
                na.value = "grey80",
                guide = guide_colorbar(
                    frame.colour = "black",
                    ticks.colour = "black",
                    title.hjust = 0
                )
            )
    }

    if (isTRUE(flip)) {
        p <- p +
            geom_text(
                aes(
                    x = 0,
                    y = !!sym(y),
                    # nudge_x is not an aesthetic
                    label = ifelse(
                        is.na(!!sym(y)),
                        " NA ",
                        ifelse(
                            .data[[x]] >= 0,
                            gsub("(\\n|$)", " \\1", !!sym(y)),
                            gsub("(^|\\n)", "\\1 ", !!sym(y))
                        )
                    ),
                    hjust = ifelse(.data[[x]] >= 0, 1, 0),
                ),
                color = "black",
                lineheight = lineheight,
                angle = 90
            ) +
            ggplot2::theme(
                axis.text.x = element_blank(),
                axis.ticks.x = element_blank()
            )
    } else {
        p <- p +
            geom_text(
                aes(
                    x = 0,
                    y = !!sym(y),
                    # nudge_y is not an aesthetic
                    label = ifelse(
                        is.na(!!sym(y)),
                        " NA ",
                        ifelse(
                            .data[[x]] >= 0,
                            gsub("(\\n|$)", " \\1", !!sym(y)),
                            gsub("(^|\\n)", "\\1 ", !!sym(y))
                        )
                    ),
                    hjust = ifelse(.data[[x]] >= 0, 1, 0),
                ),
                color = "black",
                lineheight = lineheight
            ) +
            ggplot2::theme(
                axis.text.y = element_blank(),
                axis.ticks.y = element_blank()
            )
    }

    p <- p +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% x,
            y = ylab %||% y
        ) +
        # scale_x_continuous(expand = expand$x) +
        scale_y_discrete() +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction
        )

    dims <- calculate_plot_dimensions(
        base_height = 5.5,
        aspect.ratio = aspect.ratio,
        n_y = nlevels(data[[y]]),
        y_scale_factor = bar_height / 4,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = if (!fill_by_numeric) length(fill_vals) else 1,
        legend_nchar = if (!fill_by_numeric) {
            max(nchar(as.character(fill_vals)), na.rm = TRUE)
        } else {
            5
        },
        flip = isTRUE(flip)
    )

    if (isTRUE(flip)) {
        p <- p + coord_flip(xlim = c(x_min, x_max))
    } else {
        p <- p + coord_cartesian(xlim = c(x_min, x_max))
    }
    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction,
        drop = !isTRUE(keep_empty_facet)
    )
}

#' @rdname barplot
#'
#' @description
#' `SplitBarPlot` (also known as `WaterfallPlot`) draws a divergent bar plot
#' where bars extend left (negative values) and right (positive values) from a
#' central zero line.  The bar fill colour and opacity can encode additional
#' variables, and the vertical ordering of categories is fully customisable.
#'
#' The function supports \strong{split_by} to produce separate panels,
#' \strong{facet_by} for grouped views within panels, and
#' \strong{alpha_by} for encoding a secondary numeric variable via opacity.
#'
#' @section split_by workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
#'         normalise the \code{keep_na} / \code{keep_empty} arguments.
#'   \item The \code{split_by} column is validated and its NA / empty levels
#'         are processed.  It is then removed from the per-column lists.
#'   \item The data is split by \code{split_by} (preserving level order).
#'   \item Per-split \code{palette}, \code{palcolor},
#'         \code{legend.position}, and \code{legend.direction} are resolved.
#'   \item \code{\link{SplitBarPlotAtomic}()} is called for each split.
#'         When \code{title} is a function, it receives the split level name
#'         for dynamic title generation.
#'   \item Results are combined via \code{\link{combine_plots}()}.
#' }
#'
#' @inheritParams common_args
#' @inheritParams SplitBarPlotAtomic
#' @param split_by The column(s) to split the data by for separate sub-plots.
#' @param split_by_sep Separator for concatenated \code{split_by} columns.
#' @param seed A numeric seed for reproducibility.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'  \code{patchwork} object.  When \code{FALSE}, returns a named list of
#'  \code{ggplot} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined layout.
#' @param byrow Logical; fill the combined layout by row (default \code{TRUE}).
#' @param axes,axis_titles Character strings for axis handling in the
#'  combined layout.
#' @param guides Character string for legend collection across panels.
#' @param design A custom layout design for the combined plot.
#' @return A \code{ggplot} object, a \code{patchwork} object, or a named list
#'  of \code{ggplot} objects (when \code{combine = FALSE}), each with
#'  \code{height} and \code{width} attributes in inches.
#' @export
#' @examples
#' \donttest{
#' set.seed(8525)
#' data <- data.frame(
#'     word = c("apple", "banana", "cherry", "date", "elderberry",
#'              "It is a very long term with a lot of words"),
#'     count = c(-10, 20, -30, 40, 50, 34),
#'     score = c(1, 2, 3, 4, 5, 3.2),
#'     group = c("A", "A", "B", "B", "C", "C")
#' )
#'
#' # Basic split bar plot with alpha encoding
#' SplitBarPlot(data, x = "count", y = "word", alpha_by = "score")
#'
#' # Control label wrapping
#' SplitBarPlot(data, x = "count", y = "word", alpha_by = "score",
#'              max_charwidth = 30, lineheight = 1.1)
#'
#' # Fill by categorical variable
#' SplitBarPlot(data, x = "count", y = "word", fill_by = "group")
#'
#' # Faceting
#' SplitBarPlot(data, x = "count", y = "word", facet_by = "group",
#'              fill_name = "Direction")
#'
#' # Per-split palettes
#' SplitBarPlot(data, x = "count", y = "word", alpha_by = "score",
#'              split_by = "group",
#'              palette = c(A = "Reds", B = "Blues", C = "Greens"))
#'
#' # keep_na and keep_empty examples
#' data <- data.frame(
#'     word = factor(c("apple", "banana", "cherry", NA, "elderberry",
#'          "It is a very long term with a lot of words"),
#'          levels = c("apple", "banana", "cherry", "date", "elderberry",
#'              "unused", "It is a very long term with a lot of words")),
#'     count = c(-10, 20, NA, 40, 10, 34),
#'     score = c(1, 2, 3, 4, 5, 3.2),
#'     group = factor(sample(c("A", "A", "B", "B", "C", "C")),
#'          levels = c("A", "B", "C", "D"))
#' )
#'
#' # Default: NA and empty levels dropped
#' SplitBarPlot(data, x = "count", y = "word", alpha_by = "score")
#'
#' # Keep NA and empty levels
#' SplitBarPlot(data, x = "count", y = "word", alpha_by = "score",
#'              keep_na = TRUE, keep_empty = TRUE)
#'
#' # Keep with faceting
#' SplitBarPlot(data, x = "count", y = "word", alpha_by = "score",
#'              keep_na = TRUE, keep_empty = TRUE, facet_by = "group")
#'
#' # Keep NA, hide empty levels (reserve colours)
#' SplitBarPlot(data, x = "count", y = "word", alpha_by = "score",
#'              keep_na = TRUE, keep_empty = "level")
#'
#' # Per-column control
#' SplitBarPlot(data, x = "count", y = "word", alpha_by = "score",
#'              keep_na = list(word = FALSE), keep_empty = list(word = TRUE))
#'
#' # Control fill colour scale limits
#' SplitBarPlot(data, x = "count", y = "word", fill_by = "score",
#'              lower_cutoff = 1, upper_cutoff = 4)
#' }
SplitBarPlot <- function(
    data,
    x,
    y,
    y_sep = "_",
    flip = FALSE,
    split_by = NULL,
    split_by_sep = "_",
    alpha_by = NULL,
    alpha_reverse = FALSE,
    alpha_name = NULL,
    order_y = list(
        "+" = c("x_desc", "alpha_desc"),
        "-" = c("x_desc", "alpha_asc")
    ),
    bar_height = 0.9,
    lineheight = 0.5,
    max_charwidth = 80,
    fill_by = NULL,
    fill_by_sep = "_",
    fill_name = NULL,
    direction_name = "direction",
    direction_pos_name = "positive",
    direction_neg_name = "negative",
    theme = "theme_this",
    theme_args = list(),
    palette = "Spectral",
    palcolor = NULL,
    palreverse = FALSE,
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    facet_by = NULL,
    facet_scales = "free_y",
    facet_nrow = NULL,
    facet_ncol = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    x_min = NULL,
    x_max = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    keep_empty = FALSE,
    keep_na = FALSE,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    seed = 8525,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    keep_na <- check_keep_na(keep_na, c(y, split_by, facet_by, fill_by))
    keep_empty <- check_keep_empty(
        keep_empty,
        c(y, split_by, facet_by, fill_by)
    )
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL

        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }

    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            SplitBarPlotAtomic(
                datas[[nm]],
                x = x,
                y = y,
                y_sep = y_sep,
                flip = flip,
                alpha_by = alpha_by,
                alpha_reverse = alpha_reverse,
                alpha_name = alpha_name,
                order_y = order_y,
                bar_height = bar_height,
                lineheight = lineheight,
                max_charwidth = max_charwidth,
                fill_by = fill_by,
                fill_by_sep = fill_by_sep,
                fill_name = fill_name,
                direction_name = direction_name,
                direction_pos_name = direction_pos_name,
                direction_neg_name = direction_neg_name,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                lower_quantile = lower_quantile,
                upper_quantile = upper_quantile,
                lower_cutoff = lower_cutoff,
                upper_cutoff = upper_cutoff,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_nrow = facet_nrow,
                facet_ncol = facet_ncol,
                facet_byrow = facet_byrow,
                aspect.ratio = aspect.ratio,
                x_min = x_min,
                x_max = x_max,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                keep_empty = keep_empty,
                keep_na = keep_na,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

#' @rdname barplot
#' @inheritParams SplitBarPlot
#' @export
WaterfallPlot <- SplitBarPlot

set.seed(8525)
data <- data.frame(
    x = factor(rep(c("A", "B", "C", "D"), each = 25), levels = c("A", "B", "C", "D")),
    y = rnorm(100),
    group1 = factor(rep(c("g1", "g2"), 50)),
    group2 = factor(rep(c("f1", "f2"), each = 50))
)

test_that("BarPlot returns a ggplot object for basic usage", {
    p <- BarPlot(data, x = "x")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("BarPlot works with y column", {
    p <- BarPlot(data, x = "x", y = "y")
    expect_s3_class(p, "ggplot")
})

test_that("BarPlot works with group_by", {
    p <- BarPlot(data, x = "x", y = "y", group_by = "group1")
    expect_s3_class(p, "ggplot")
})

test_that("BarPlot with split_by returns patchwork when combine = TRUE", {
    p <- BarPlot(data, x = "x", y = "y", split_by = "group2", combine = TRUE)
    expect_s3_class(p, c("patchwork", "gg"), exact = FALSE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("BarPlot with split_by and combine = FALSE returns a list", {
    plots <- BarPlot(data, x = "x", y = "y", split_by = "group2", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
    expect_s3_class(plots[[2]], "ggplot")
})

test_that("BarPlot with multiple split_by columns returns a list with one plot per combination", {
    plots <- suppressMessages(BarPlot(data, x = "x", y = "y", split_by = c("group1", "group2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("BarPlot with multiple split_by columns returns combined plot", {
    p <- suppressMessages(BarPlot(data, x = "x", y = "y", split_by = c("group1", "group2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("BarPlot sets title and subtitle", {
    p <- BarPlot(data, x = "x", title = "Test Title", subtitle = "Test Subtitle")
    expect_s3_class(p, "ggplot")
    labels <- p$labels
    expect_equal(labels$title, "Test Title")
    expect_equal(labels$subtitle, "Test Subtitle")
})

test_that("BarPlot sets xlab and ylab", {
    p <- BarPlot(data, x = "x", xlab = "X Label", ylab = "Y Label")
    expect_s3_class(p, "ggplot")
    labels <- p$labels
    expect_equal(labels$x, "X Label")
    expect_equal(labels$y, "Y Label")
})

test_that("BarPlot works with flip = TRUE", {
    p <- BarPlot(data, x = "x", y = "y", flip = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("BarPlot works with facet_by", {
    p <- BarPlot(data, x = "x", y = "y", facet_by = "group2")
    expect_s3_class(p, "ggplot")
})

test_that("BarPlot works with position = 'stack'", {
    p <- BarPlot(data, x = "x", group_by = "group1", position = "stack")
    expect_s3_class(p, "ggplot")
})

test_that("BarPlot works with fill_by = FALSE", {
    p <- BarPlot(data, x = "x", fill_by = FALSE)
    expect_s3_class(p, "ggplot")
})

test_that("BarPlot works with add_line parameter", {
    p <- BarPlot(data, x = "x", y = "y", add_line = 0)
    expect_s3_class(p, "ggplot")
})

test_that("BarPlot works with add_trend = TRUE", {
    p <- BarPlot(data, x = "x", y = "y", add_trend = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("BarPlot works with custom palette", {
    p <- BarPlot(data, x = "x", palette = "Set1")
    expect_s3_class(p, "ggplot")
})

test_that("BarPlot width attribute is numeric", {
    p <- BarPlot(data, x = "x")
    expect_true(is.numeric(attr(p, "width")))
    expect_true(is.numeric(attr(p, "height")))
})

test_that("BarPlot works with multiple split levels", {
    data2 <- data
    data2$group3 <- factor(rep(c("h1", "h2", "h3"), length.out = nrow(data2)))
    plots <- BarPlot(data2, x = "x", split_by = "group3", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 3)
})

test_that("LollipopPlot returns a ggplot", {
    p <- LollipopPlot(data, x = "y", y = "x")
    expect_s3_class(p, "ggplot")
})

test_that("WaterfallPlot returns a ggplot", {
    wf_data <- data.frame(
        x = c(0.5, -0.3, 0.8, -1.2, 0.4),
        y = c("GeneA", "GeneB", "GeneC", "GeneD", "GeneE")
    )
    p <- WaterfallPlot(wf_data, x = "x", y = "y")
    expect_s3_class(p, "ggplot")
})

test_that("SplitBarPlot returns a ggplot", {
    wf_data <- data.frame(
        x = c(0.5, -0.3, 0.8, -1.2, 0.4),
        y = c("GeneA", "GeneB", "GeneC", "GeneD", "GeneE")
    )
    p <- SplitBarPlot(wf_data, x = "x", y = "y")
    expect_s3_class(p, "ggplot")
})

test_that("SplitBarPlot with multiple split_by columns returns a list with one plot per combination", {
    wf_data <- data.frame(
        x = c(0.5, -0.3, 0.8, -1.2, 0.4),
        y = c("GeneA", "GeneB", "GeneC", "GeneD", "GeneE"),
        group1 = factor(c("g1", "g1", "g1", "g2", "g2")),
        group2 = factor(c("f1", "f1", "f2", "f1", "f2"))
    )
    plots <- suppressMessages(SplitBarPlot(wf_data, x = "x", y = "y",
        split_by = c("group1", "group2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("SplitBarPlot with multiple split_by columns returns combined plot", {
    wf_data <- data.frame(
        x = c(0.5, -0.3, 0.8, -1.2, 0.4),
        y = c("GeneA", "GeneB", "GeneC", "GeneD", "GeneE"),
        group1 = factor(c("g1", "g1", "g1", "g2", "g2")),
        group2 = factor(c("f1", "f1", "f2", "f1", "f2"))
    )
    p <- suppressMessages(SplitBarPlot(wf_data, x = "x", y = "y",
        split_by = c("group1", "group2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("WaterfallPlot with multiple split_by columns returns a list with one plot per combination", {
    wf_data <- data.frame(
        x = c(0.5, -0.3, 0.8, -1.2, 0.4),
        y = c("GeneA", "GeneB", "GeneC", "GeneD", "GeneE"),
        group1 = factor(c("g1", "g1", "g1", "g2", "g2")),
        group2 = factor(c("f1", "f1", "f2", "f1", "f2"))
    )
    plots <- suppressMessages(WaterfallPlot(wf_data, x = "x", y = "y",
        split_by = c("group1", "group2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("WaterfallPlot with multiple split_by columns returns combined plot", {
    wf_data <- data.frame(
        x = c(0.5, -0.3, 0.8, -1.2, 0.4),
        y = c("GeneA", "GeneB", "GeneC", "GeneD", "GeneE"),
        group1 = factor(c("g1", "g1", "g1", "g2", "g2")),
        group2 = factor(c("f1", "f1", "f2", "f1", "f2"))
    )
    p <- suppressMessages(WaterfallPlot(wf_data, x = "x", y = "y",
        split_by = c("group1", "group2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("BarPlot with numeric fill_by and quantile works", {
    p <- BarPlot(data, x = "x", y = "y", fill_by = "y",
        lower_quantile = 0.1, upper_quantile = 0.9)
    expect_s3_class(p, "ggplot")
})

test_that("BarPlot with numeric fill_by and explicit cutoffs works", {
    p <- BarPlot(data, x = "x", y = "y", fill_by = "y",
        lower_cutoff = -0.5, upper_cutoff = 0.5)
    expect_s3_class(p, "ggplot")
})

test_that("SplitBarPlot with quantile works", {
    wf_data <- data.frame(
        x = c(0.5, -0.3, 0.8, -1.2, 0.4),
        y = c("GeneA", "GeneB", "GeneC", "GeneD", "GeneE"),
        score = c(1, 2, 3, 4, 5)
    )
    p <- SplitBarPlot(wf_data, x = "x", y = "y", fill_by = "score",
        lower_quantile = 0.1, upper_quantile = 0.9)
    expect_s3_class(p, "ggplot")
})

#' @title Flip y-coordinates for spatial data objects
#'
#' @description
#' These internal S3 methods flip the y-coordinates of \code{SpatRaster},
#' \code{SpatVector}, and \code{data.frame} objects.
#'
#' For rasters, the raster is flipped vertically and its extent is negated.
#' For vectors, the y-coordinates of all geometries are negated.
#' For data frames, the specified y column is negated.
#'
#' @param data A \code{SpatRaster}, \code{SpatVector}, or \code{data.frame}.
#' @param y A character string specifying the y column name for \code{data.frame}
#'   input. Ignored for \code{SpatRaster} and \code{SpatVector}. Default is \code{"y"}.
#' @param ... Additional arguments (not used).
#'
#' @return
#' For \code{SpatRaster} input, a \code{SpatRaster} flipped vertically with
#' negated y-extent. For \code{SpatVector} input, a \code{SpatVector} with
#' negated y-coordinates. For \code{data.frame} input, a data frame with the
#' specified \code{y} column negated.
#'
#' @details
#' These functions are intended for internal use to facilitate coordinate
#' transformations. When visualizing spatial data, it is often necessary to
#' flip the y-axis to put the origin at the top-left corner. However,
#' \code{\link[ggplot2]{geom_sf}()} does not work with
#' \code{\link[ggplot2]{scale_y_reverse}()}. See
#' \href{https://github.com/tidyverse/ggplot2/issues/4021#issuecomment-650787582}{this
#' GitHub comment} for details. These functions negate the y-coordinates so
#' that the axis labels can be displayed with reversed sign, mimicking
#' \code{scale_y_reverse()}.
#'
#' @keywords internal
#' @rdname dot_flip_y
.flip_y <- function(data, ...) {
    UseMethod(".flip_y", data)
}

#' @keywords internal
#' @rdname dot_flip_y
.flip_y.SpatRaster <- function(data, ...) {
    # Flip the raster vertically
    out <- terra::flip(data, direction = "vertical")

    # Adjust the y-coordinates to go from -nrow to 0
    nrows <- terra::nrow(data)
    ext_orig <- terra::ext(data)

    # Create new extent with flipped y coordinates
    new_ext <- terra::ext(
        ext_orig[1],
        ext_orig[2], # keep x coordinates
        -1 * ext_orig[4],
        -1 * ext_orig[3] # flip y coordinates
    )

    # Set the new extent
    terra::ext(out) <- new_ext

    return(out)
}

#' @keywords internal
#' @rdname dot_flip_y
.flip_y.SpatVector <- function(data, ...) {
    # Manual coordinate transformation approach (more reliable)
    # Get all geometries and transform coordinates
    coords_list <- terra::geom(data)
    coords_list[, "y"] <- -coords_list[, "y"]

    # Recreate the SpatVector with flipped coordinates
    data_flipped <- terra::vect(
        coords_list,
        type = terra::geomtype(data),
        atts = terra::values(data)
    )

    return(data_flipped)
}

#' @keywords internal
#' @rdname dot_flip_y
.flip_y.data.frame <- function(data, y = "y", ...) {
    data[[y]] <- -data[[y]]
    return(data)
}

#' Prepare the extent for spatial plots
#'
#' @param ext A numeric vector of length 4 specifying the extent as `c(xmin, xmax, ymin, ymax)`,
#' or a `SpatExtent` object from the `terra` package.
#' @return A `SpatExtent` object if `ext` is a numeric vector, or the original `SpatExtent` if it is already one.
#' NULL is returned if `ext` is NULL.
#' @keywords internal
.prepare_extent <- function(ext) {
    if (is.null(ext)) {
        return(NULL)
    }
    if (is.numeric(ext) && length(ext) == 4) {
        # Convert numeric vector to SpatExtent
        ext <- terra::ext(ext[1], ext[2], ext[3], ext[4])
    }
    if (!inherits(ext, "SpatExtent")) {
        stop(
            "'ext' must be a numeric vector of length 4 or a SpatExtent object."
        )
    }
    return(ext)
}

#' Wrap spatial plot if plotted independently
#'
#' This function is used to wrap spatial plots if they are plotted independently
#' with return_layer = FALSE.
#'
#' @param layers A list of ggplot layers to be wrapped.
#' @param ext A numeric vector of length 4 specifying the extent as `c(xmin, xmax, ymin, ymax)`. Default is NULL.
#' @param flip_y Whether to flip the y-axis direction. Default is TRUE.
#' @param legend.position The position of the legend. Default is "right".
#' @param legend.direction The direction of the legend. Default is "vertical".
#' @param title The title of the plot. Default is NULL.
#' @param subtitle The subtitle of the plot. Default is NULL.
#' @param xlab The x-axis label. Default is NULL.
#' @param ylab The y-axis label. Default is NULL.
#' @param theme The theme to be used for the plot. Default is "theme_box".
#' @param theme_args A list of arguments to be passed to the theme function. Default is an empty list.
#' @return A ggplot object with the specified layers.
#' @importFrom ggplot2 coord_sf labs scale_y_continuous
#' @keywords internal
.wrap_spatial_layers <- function(
    layers,
    ext = NULL,
    flip_y = TRUE,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    theme = "theme_box",
    theme_args = list()
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }

    xlim <- ylim <- NULL
    # Set default width and height based on extent proportions
    if (!is.null(ext)) {
        ext_width <- ext[2] - ext[1]
        ext_height <- ext[4] - ext[3]
        aspect_ratio <- ext_width / ext_height

        # Base height of 6, adjust width proportionally
        base_height <- 6
        base_width <- base_height * aspect_ratio

        # Ensure reasonable bounds
        if (base_width > 12) {
            base_width <- 12
            base_height <- base_width / aspect_ratio
        } else if (base_width < 3) {
            base_width <- 3
            base_height <- base_width / aspect_ratio
        }

        xlim <- c(ext[1], ext[2])
        if (flip_y) {
            ylim <- c(-ext[4], -ext[3]) # Flip y-axis
        } else {
            ylim <- c(ext[3], ext[4])
        }
    } else {
        # Default dimensions when no extent specified
        base_height <- 6
        base_width <- 6
    }

    # Adjust for legend position (similar to DimPlot)
    if (!identical(legend.position, "none")) {
        if (legend.position %in% c("right", "left")) {
            base_width <- base_width + 1
        } else if (legend.direction == "horizontal") {
            base_height <- base_height + 1
        } else {
            base_width <- base_width + 2
        }
    }
    p <- ggplot() +
        layers +
        coord_sf(expand = 0, xlim = xlim, ylim = ylim) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab,
            y = ylab
        ) +
        do_call(process_theme(theme), theme_args) +
        ggplot2::theme(
            legend.position = legend.position,
            legend.direction = legend.direction
        )

    if (flip_y) {
        p <- p +
            scale_y_continuous(labels = function(x) {
                sub("-", "\u2212", as.character(-x))
            })
    }

    attr(p, "height") <- base_height
    attr(p, "width") <- base_width

    return(p)
}

#' Spatial visualization functions for terra objects and point data
#'
#' These functions provide publication-quality spatial visualizations built on
#' \pkg{ggplot2}, supporting raster images, categorical masks, vector shapes,
#' and point data from the \pkg{terra} package or standard data frames.
#'
#' \describe{
#'   \item{\code{SpatImagePlot}}{Render a \code{SpatRaster} as a raster image.
#'     Supports single-layer continuous values (with gradient fill) and 3-channel
#'     RGB data (with automatic color identity scaling).}
#'   \item{\code{SpatMasksPlot}}{Render a \code{SpatRaster} as a categorical
#'     mask overlay. Zero-valued cells are treated as transparent background;
#'     optional polygon borders can be added around mask regions.}
#'   \item{\code{SpatShapesPlot}}{Render spatial shapes (polygons) from a
#'     \code{SpatVector} or a data frame of vertex coordinates. Supports
#'     single and multi-column fill mapping with automatic faceting.}
#'   \item{\code{SpatPointsPlot}}{Render spatial points from a data frame with
#'     support for color/size/fill mapping, hex binning, rasterized rendering,
#'     network/graph overlays, labels, and point highlighting.}
#' }
#'
#' @section Rendering Pipeline for SpatImagePlot:
#' \enumerate{
#'   \item \strong{Extent cropping} — if \code{ext} is provided, the
#'     \code{SpatRaster} is cropped via \code{terra::crop()}. An error is
#'     raised if no cells remain within the extent.
#'   \item \strong{Auto-rasterization} — if the raster exceeds 1e6 cells (or
#'     \code{raster = TRUE}), the raster is aggregated via
#'     \code{terra::aggregate()} to a target resolution of \code{raster_dpi}.
#'   \item \strong{Y-axis flipping} — when \code{flip_y = TRUE}, the raster
#'     is flipped vertically via \code{\link{.flip_y}()} and its y-extent is
#'     negated so that axis labels can be displayed with reversed sign.
#'   \item \strong{RGB detection} — if the raster has exactly 3 layers, they
#'     are treated as red/green/blue channels: each channel is rescaled to
#'     0--255 and combined into a hex color via \code{rgb()}. A
#'     \code{scale_fill_identity()} is used with no legend guide.
#'   \item \strong{Single-layer rendering} — otherwise, the raster is
#'     converted to an x/y/value data frame and rendered via
#'     \code{geom_raster()} with \code{scale_fill_gradientn()}.
#'   \item \strong{Layer return or full assembly} — if
#'     \code{return_layer = TRUE}, the list of layers (with a \code{"scales"}
#'     attribute set to \code{"fill"}) is returned. Otherwise,
#'     \code{\link{.wrap_spatial_layers}()} assembles a complete ggplot with
#'     \code{coord_sf(expand = 0)}, theme, labels, legend, and dimension
#'     attributes.
#' }
#'
#' @section Rendering Pipeline for SpatMasksPlot:
#' \enumerate{
#'   \item \strong{Extent cropping} — if \code{ext} is provided, the
#'     \code{SpatRaster} is cropped via \code{terra::crop()}.
#'   \item \strong{Y-axis flipping} — when \code{flip_y = TRUE}, the raster
#'     is flipped vertically and its y-extent is negated.
#'   \item \strong{Background masking} — cells with value 0 are set to
#'     \code{NA} so they render as transparent via
#'     \code{na.value = "transparent"}.
#'   \item \strong{Raster layer} — the mask is converted to an x/y/value data
#'     frame and rendered via \code{geom_raster()} with a gradient fill scale.
#'   \item \strong{Optional borders} — when \code{add_border = TRUE}, the
#'     mask values are polygonized via \code{terra::as.polygons()}, converted
#'     to sf, and overlaid as unfilled \code{geom_sf()} with the specified
#'     \code{border_color}, \code{border_size}, and \code{border_alpha}.
#'   \item \strong{Layer return or full assembly} — if
#'     \code{return_layer = TRUE}, the layers are returned; otherwise,
#'     \code{\link{.wrap_spatial_layers}()} creates the complete ggplot.
#' }
#'
#' @section Rendering Pipeline for SpatPointsPlot:
#' \enumerate{
#'   \item \strong{Column resolution} — \code{x} and \code{y} coordinates are
#'     resolved from the data, auto-detecting common column names
#'     (\code{"x"}, \code{"X"}, \code{"sdimx"}, etc. for x; \code{"y"},
#'     \code{"Y"}, \code{"sdimy"}, etc. for y) when not explicitly provided.
#'   \item \strong{Extent cropping} — if \code{ext} is provided, rows
#'     outside the extent are filtered.
#'   \item \strong{Y-axis flipping} — when \code{flip_y = TRUE}, the y
#'     coordinate column is negated via \code{\link{.flip_y}()}.
#'   \item \strong{Multi-column faceting} — when \code{color_by} has multiple
#'     columns (all numeric), the data is reshaped to long format with a
#'     \code{.facet_var} column and faceted via \code{facet_plot()}.
#'   \item \strong{Cutoff winsorization} — for numeric \code{color_by},
#'     values outside \code{[lower_cutoff, upper_cutoff]} (derived from
#'     quantiles or explicit values) are clamped to the nearest cutoff.
#'   \item \strong{Graph / network edges} — if \code{graph} is provided,
#'     edges are resolved from an adjacency matrix, data.frame, or data
#'     attribute, and rendered as \code{geom_segment()} segments with line
#'     width proportional to edge weight.
#'   \item \strong{Main point layer} — one of three rendering modes:
#'     \describe{
#'       \item{\emph{Regular}}{(\code{hex = FALSE}, \code{raster = FALSE}) —
#'         \code{geom_point()} with shape, size, color, and fill aesthetic
#'         mappings. Shapes 21--25 support separate fill and border colors.}
#'       \item{\emph{Hex}}{(\code{hex = TRUE}) — \code{geom_hex()} or
#'         \code{stat_summary_hex()} for binned aggregation of numeric
#'         \code{color_by} values.}
#'       \item{\emph{Raster}}{(\code{raster = TRUE}) —
#'         \code{scattermore::geom_scattermore()} for efficient rendering of
#'         large datasets (>1e6 rows).}
#'     }
#'   \item \strong{Highlight} — if \code{highlight} is specified,
#'     highlighted points are overlaid as larger, colored markers using
#'     \code{geom_point()} or \code{scattermore::geom_scattermore()}.
#'   \item \strong{Labels} — if \code{label = TRUE} and \code{color_by} is a
#'     categorical column, group centroid positions are computed via
#'     \code{aggregate()} with the \code{label_pos} function, and labels are
#'     rendered via \code{ggrepel::geom_text_repel()} with optional
#'     background styling and repulsion.
#'   \item \strong{Layer return or full assembly} — if
#'     \code{return_layer = TRUE}, the layers are returned; otherwise,
#'     \code{\link{.wrap_spatial_layers}()} creates the complete ggplot and
#'     \code{\link{facet_plot}()} is applied when multi-column faceting
#'     is active.
#' }
#'
#' @rdname spatialplots
#' @concept spatial
#' @inheritParams common_args
#' @param data A \code{SpatRaster}, \code{SpatVector}, or \code{data.frame}
#'   depending on the function. See individual function descriptions.
#' @param x A character string specifying the x coordinate column for
#'   \code{SpatPointsPlot} and \code{SpatShapesPlot} (when \code{data} is a
#'   data frame). Auto-detected from common column names when \code{NULL}.
#' @param y A character string specifying the y coordinate column for
#'   \code{SpatPointsPlot} and \code{SpatShapesPlot} (when \code{data} is a
#'   data frame). Auto-detected from common column names when \code{NULL}.
#' @param group A character string specifying the grouping column for
#'   \code{SpatShapesPlot} when \code{data} is a data.frame. Each unique
#'   value in this column defines a separate polygon.
#' @param ext A numeric vector of length 4 \code{c(xmin, xmax, ymin, ymax)}
#'   or a \code{terra::SpatExtent} object. Default is \code{NULL} (use full
#'   extent).
#' @param flip_y Whether to negate the y-coordinates so the axis labels can
#'   be displayed with reversed sign. Default is \code{TRUE}.
#' @param fill_by A character string or vector specifying the column(s) to
#'   map to fill color in \code{SpatShapesPlot}. When multiple columns are
#'   provided (all must be numeric), the data is reshaped and faceted.
#'   When a single string that does not match a column name, it is treated
#'   as a fixed fill color.
#' @param fill_name A character string for the fill legend title.
#' @param color_by A character string or vector specifying the column(s) to
#'   map to point color in \code{SpatPointsPlot}. Multiple numeric columns
#'   trigger faceting.
#' @param color_name A character string for the color legend title.
#' @param size_by A character string specifying the column to map to point
#'   size in \code{SpatPointsPlot}. Mutually exclusive with \code{size}.
#' @param size A numeric value for a fixed point size in
#'   \code{SpatPointsPlot}. Alias for \code{size_by} when given a scalar.
#' @param size_name A character string for the size legend title.
#' @param shape A numeric value (21--25 for border shapes) or character
#'   string specifying the point shape in \code{SpatPointsPlot}.
#'   Default is \code{16} (filled circle).
#' @param add_border Whether to add polygon borders around mask regions in
#'   \code{SpatMasksPlot}. Default is \code{TRUE}.
#' @param border_color A character string for the border color. Default is
#'   \code{"black"}. When \code{TRUE} in \code{SpatShapesPlot} or
#'   \code{SpatPointsPlot}, the border maps to the same variable as fill.
#' @param border_size A numeric value for the border line width.
#'   Default is \code{0.5}.
#' @param border_alpha A numeric value for the border transparency.
#'   Default is \code{1}.
#' @param raster Whether to rasterize for efficient rendering of large
#'   datasets. Default is \code{NULL} (auto-detect: rasterize when
#'   \code{ncell > 1e6} or \code{nrow > 1e6}). Uses
#'   \code{scattermore::geom_scattermore()} for points.
#' @param raster_dpi A numeric vector of length 1 or 2 specifying the
#'   raster output resolution in pixels. Default is \code{c(512, 512)}.
#' @param hex Whether to use hexagonal binning in \code{SpatPointsPlot}.
#'   Requires a numeric \code{color_by}. Default is \code{FALSE}.
#' @param hex_linewidth A numeric value for the hex bin border width.
#'   Default is \code{0.5}.
#' @param hex_count Whether to show hex bin count as point opacity.
#'   Default is \code{FALSE}.
#' @param hex_bins A numeric value for the number of hex bins.
#'   Default is \code{50}.
#' @param hex_binwidth A numeric value for the hex bin width.
#'   Default is \code{NULL}.
#' @param label Whether to show group labels in \code{SpatPointsPlot}.
#'   Default is \code{FALSE}. Requires a categorical \code{color_by}.
#' @param label_size A numeric value for the label text size.
#'   Default is \code{4}.
#' @param label_fg A character string for the label text color.
#'   Default is \code{"white"}.
#' @param label_bg A character string for the label background color.
#'   Default is \code{"black"}.
#' @param label_bg_r A numeric value for the label background corner
#'   radius ratio. Default is \code{0.1}.
#' @param label_repel Whether to repel labels from each other and from
#'   data points. Default is \code{FALSE}.
#' @param label_repulsion A numeric value for the repulsion force.
#'   Default is \code{20}.
#' @param label_pt_size A numeric value for the label anchor point size.
#'   Default is \code{1}.
#' @param label_pt_color A character string for the label anchor point
#'   color. Default is \code{"black"}.
#' @param label_segment_color A character string for the label connector
#'   line color. Default is \code{"black"}.
#' @param label_insitu Whether to place raw group names as labels
#'   instead of numeric indices. Default is \code{FALSE}.
#' @param label_pos A character string or function specifying how label
#'   positions are computed per group. Options: \code{"median"} (default),
#'   \code{"mean"}, \code{"center"}, \code{"first"}, \code{"last"},
#'   \code{"random"}, \code{"min"}, \code{"max"}, or a custom function
#'   that takes a numeric vector and returns a single value.
#' @param highlight A character vector of row names to highlight, a filter
#'   expression string (e.g., \code{'cat == "A"'}), or \code{TRUE}
#'   (highlight all). Highlighted points are overlaid with larger markers.
#' @param highlight_alpha A numeric value for highlight transparency.
#'   Default is \code{1}.
#' @param highlight_size A numeric value for the highlight marker size.
#'   Default is \code{1}.
#' @param highlight_color A character string for the highlight marker
#'   color. Default is \code{"black"}.
#' @param highlight_stroke A numeric value for the highlight stroke width
#'   (added to \code{border_size}). Default is \code{0.8}.
#' @param graph Graph/network edge data for \code{SpatPointsPlot}. Can be
#'   a square adjacency matrix with row/column names matching \code{data},
#'   a data.frame with edge coordinates (see \code{graph_x} etc.), a column
#'   name or index, or \code{"@graph"} (extracts from the \code{data}
#'   attribute named \code{"graph"}). Default is \code{NULL}.
#' @param graph_x A character string for the x start coordinate column
#'   in the graph data.
#' @param graph_y A character string for the y start coordinate column
#'   in the graph data.
#' @param graph_xend A character string for the x end coordinate column
#'   in the graph data.
#' @param graph_yend A character string for the y end coordinate column
#'   in the graph data.
#' @param graph_value A character string for the edge weight column
#'   in the graph data.
#' @param edge_size A numeric vector of length 2 specifying the line width
#'   range for graph edges. Default is \code{c(0.05, 0.5)}.
#' @param edge_alpha A numeric value for graph edge transparency.
#'   Default is \code{0.1}.
#' @param edge_color A character string for the graph edge color.
#'   Default is \code{"grey40"}.
#' @param return_layer Whether to return a list of ggplot layers instead
#'   of a complete plot. Default is \code{FALSE}. When \code{TRUE}, the
#'   returned list has a \code{"scales"} attribute for layer conflict
#'   detection, allowing multiple spatial layers in a single custom ggplot.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'   attributes when \code{return_layer = FALSE} (the default). When
#'   \code{return_layer = TRUE}, a list of \code{ggplot} layers with a
#'   \code{"scales"} attribute is returned. When faceted via multiple
#'   \code{fill_by} or \code{color_by} columns, a faceted \code{ggplot}
#'   is returned.
#' @importFrom rlang %||% sym parse_exprs
#' @importFrom ggplot2 scale_fill_gradientn aes guide_colorbar element_blank
#' @importFrom ggplot2 geom_sf ylim labs coord_sf geom_hex stat_summary_hex
#' @importFrom ggplot2 geom_segment scale_linewidth_continuous
#' @export
#' @examples
#' \donttest{
#' set.seed(8525)
#' # --- SpatImagePlot ---
#' # Generate a sample SpatRaster
#' r <- terra::rast(
#'     nrows = 50, ncols = 40, vals = runif(2000),
#'     xmin = 0, xmax = 40, ymin = 0, ymax = 50,
#'     crs = ""
#' )
#' SpatImagePlot(r)
#' SpatImagePlot(r, raster = TRUE, raster_dpi = 20)
#' SpatImagePlot(r, alpha = 0.5, theme = "theme_blank",
#'     theme_args = list(add_coord = FALSE), fill_name = "value")
#' SpatImagePlot(r, ext = c(0, 10, 0, 10), flip_y = FALSE, palette = "viridis")
#'
#' # --- SpatMasksPlot ---
#' m <- terra::rast(
#'    nrows = 50, ncols = 40,
#'    vals = sample(c(1:5, NA), 2000, replace = TRUE, prob = c(rep(0.04, 5), 0.8)),
#'    xmin = 0, xmax = 40, ymin = 0, ymax = 50,
#'    crs = ""
#' )
#' SpatMasksPlot(m, border_color = "red")
#' SpatMasksPlot(m, ext = c(0, 15, 0, 20), add_border = FALSE,
#'     palreverse = TRUE, fill_name = "value")
#'
#' # --- SpatShapesPlot ---
#' polygons <- data.frame(
#'    id = paste0("poly_", 1:10),
#'    cat = sample(LETTERS[1:3], 10, replace = TRUE),
#'    feat1 = rnorm(10),
#'    feat2 = rnorm(10),
#'    geometry = c(
#'        'POLYGON((64.6 75.3,66.0 70.5,66.4 70.2,67.0 69.8,72.8 70.4,64.6 75.3))',
#'        'POLYGON((56.7 63.0,52.3 65.6,48.0 63.2,51.2 55.7,57.1 59.2,56.7 63.0))',
#'        'POLYGON((9.9 16.5,9.3 15.9,8.0 13.1,11.5 7.8,17.8 11.3,9.9 16.5))',
#'        'POLYGON((64.9 37.2,60.3 37.4,57.6 31.7,58.9 29.3,64.0 28.1,64.9 37.2))',
#'        'POLYGON((30.5 49.1,22.4 46.5,22.4 43.9,30.9 41.9,31.6 42.9,30.5 49.1))',
#'        'POLYGON((78.3 57.8,70.5 61.6,71.6 52.7,72.2 52.5,77.4 54.5,78.3 57.8))',
#'        'POLYGON((41.8 23.8,41.3 25.9,41.0 26.4,36.5 28.7,35.8 28.6,41.8 23.8))',
#'        'POLYGON((15.7 75.9,14.2 74.4,15.7 67.5,23.0 69.8,23.4 71.7,15.7 75.9))',
#'        'POLYGON((80.7 37.4,75.3 31.3,77.1 28.5,82.5 28.0,83.1 28.5,80.7 37.4))',
#'        'POLYGON((15.5 37.8,14.4 38.6,7.3 32.6,8.3 30.9,15.1 30.2,15.5 37.8))'
#'    )
#' )
#'
#' polygons <- terra::vect(polygons, crs = "EPSG:4326", geom = "geometry")
#'
#' SpatShapesPlot(polygons)
#' SpatShapesPlot(polygons, ext = c(0, 20, 0, 20))
#' SpatShapesPlot(polygons, highlight = 'cat == "A"', highlight_color = "red2")
#' SpatShapesPlot(polygons, border_color = "red", border_size = 2)
#' SpatShapesPlot(polygons, fill_by = "cat", fill_name = "category")
#' # Let border color be determined by fill
#' SpatShapesPlot(polygons, fill_by = "cat", alpha = 0.6, border_color = TRUE)
#' SpatShapesPlot(polygons, fill_by = "feat1")
#' SpatShapesPlot(polygons, fill_by = c("feat1", "feat2"), palette = "RdYlBu")
#'
#' # --- SpatPointsPlot ---
#' # create some random points in the above polygons
#' points <- data.frame(
#'   id = paste0("point_", 1:30),
#'   gene = sample(LETTERS[1:3], 30, replace = TRUE),
#'   feat1 = runif(30, 0, 100),
#'   feat2 = runif(30, 0, 100),
#'   size = runif(30, 1, 5),
#'   x = c(
#'     61.6, 14.3, 12.7, 49.6, 74.9, 58.9, 13.9, 24.7, 16.9, 15.6,
#'     72.4, 60.1, 75.4, 14.9, 80.3, 78.8, 16.7, 27.6, 48.9, 52.5,
#'     12.9, 11.8, 50.4, 25.6, 10.4, 51.9, 73.4, 26.8, 50.4, 60.0
#'   ),
#'   y = c(
#'     32.1, 12.8, 33.2, 59.9, 57.8, 31.9, 10.1, 46.8, 75.3, 69.0,
#'     60.0, 29.4, 54.2, 34.2, 35.3, 33.1, 74.7, 48.0, 63.2, 59.2,
#'     9.2, 15.1, 64.5, 47.1, 11.4, 60.1, 54.1, 44.5, 61.9, 30.3
#'   )
#' )
#'
#' SpatPointsPlot(points)
#' SpatPointsPlot(points, color_by = "gene", size_by = "size", shape = 22,
#'   border_size = 1)
#' SpatPointsPlot(points, raster = TRUE, raster_dpi = 30, color_by = "feat1")
#' SpatPointsPlot(points, color_by = c("feat1", "feat2"), size_by = "size")
#' SpatPointsPlot(points, color_by = "feat1", upper_cutoff = 50)
#' SpatPointsPlot(points, color_by = "feat1", hex = TRUE)
#' SpatPointsPlot(points, color_by = "gene", label = TRUE)
#' SpatPointsPlot(points, color_by = "gene", highlight = 1:20,
#'   highlight_color = "red2", highlight_stroke = 0.8)
#'
#' # --- Graph/Network functionality ---
#' # Create a simple adjacency matrix for demonstration
#' set.seed(8525)
#' graph_mat <- matrix(0, nrow = 30, ncol = 30)
#' # Add some random connections with weights
#' for(i in 1:30) {
#'   neighbors <- sample(setdiff(1:30, i), size = sample(2:5, 1))
#'   graph_mat[i, neighbors] <- runif(length(neighbors), 0.1, 1)
#' }
#' rownames(graph_mat) <- colnames(graph_mat) <- rownames(points)
#' attr(points, "graph") <- graph_mat
#'
#' SpatPointsPlot(points, color_by = "gene", graph = "@graph",
#'   edge_color = "grey60", edge_alpha = 0.3)
#' SpatPointsPlot(points, color_by = "feat1", graph = graph_mat,
#'   edge_size = c(0.1, 1), edge_alpha = 0.5)
#'
#' # --- Use the `return_layer` argument to get the ggplot layers
#' ext = c(0, 40, 0, 50)
#' ggplot2::ggplot() +
#'   SpatImagePlot(r, return_layer = TRUE, alpha = 0.2, ext = ext) +
#'   SpatShapesPlot(polygons, return_layer = TRUE, ext = ext, fill_by = "white") +
#'   SpatPointsPlot(points, return_layer = TRUE, ext = ext, color_by = "feat1") +
#'   theme_box() +
#'   ggplot2::coord_sf(expand = 0) +
#'   ggplot2::scale_y_continuous(labels = function(x) -x)
#'
#' }
SpatImagePlot <- function(
    data,
    ext = NULL,
    raster = NULL,
    raster_dpi = NULL,
    flip_y = TRUE,
    palette = "turbo",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    fill_name = NULL,
    return_layer = FALSE,
    theme = "theme_box",
    theme_args = list(),
    legend.position = ifelse(return_layer, "none", "right"),
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525
) {
    set.seed(seed)
    stopifnot(
        "'data' must be a SpatRaster object" = inherits(data, "SpatRaster")
    )

    ext <- .prepare_extent(ext)
    if (!is.null(ext)) {
        data <- terra::crop(data, ext)
        if (terra::ncell(data) == 0) {
            stop(
                "[SpatImagePlot] No data in the specified extent. Please check your extent."
            )
        }
    }

    raster <- raster %||% (terra::ncell(data) > 1e6)
    if (raster) {
        # Calculate appropriate aggregation factor based on desired output resolution
        if (is.null(raster_dpi)) {
            raster_dpi <- c(512, 512)
        }
        if (length(raster_dpi) == 1) {
            raster_dpi <- rep(raster_dpi, 2)
        }

        # Calculate aggregation factors to achieve target resolution
        current_dims <- c(terra::ncol(data), terra::nrow(data))
        agg_factors <- pmax(1, round(current_dims / raster_dpi))

        # Aggregate raster to reduce resolution
        data <- terra::aggregate(data, fact = agg_factors, fun = "mean")
    }

    if (flip_y) {
        data <- .flip_y(data)
    }

    if (dim(data)[3] == 3) {
        names(data) <- c("red", "green", "blue")
        data <- terra::as.data.frame(data, xy = TRUE)
        data$red <- scales::rescale(data$red, to = c(0, 255))
        data$green <- scales::rescale(data$green, to = c(0, 255))
        data$blue <- scales::rescale(data$blue, to = c(0, 255))
        data$value <- rgb(data$red, data$green, data$blue, maxColorValue = 255)
        colnames(data)[1:2] <- c("x", "y")
        fill_identity <- TRUE
    } else {
        data <- terra::as.data.frame(data, xy = TRUE)
        colnames(data) <- c("x", "y", "value")
        fill_identity <- FALSE
    }

    layers <- list(
        ggplot2::geom_raster(
            data = data,
            aes(x = !!sym("x"), y = !!sym("y"), fill = !!sym("value")),
            alpha = alpha
        )
    )
    if (fill_identity) {
        layers <- c(layers, list(ggplot2::scale_fill_identity(guide = "none")))
    } else {
        layers <- c(
            layers,
            list(
                scale_fill_gradientn(
                    name = fill_name %||% names(data)[1],
                    n.breaks = 4,
                    colors = palette_this(
                        palette = palette,
                        n = 256,
                        reverse = palreverse,
                        palcolor = palcolor
                    ),
                    guide = if (identical(legend.position, "none")) {
                        "none"
                    } else {
                        guide_colorbar(
                            frame.colour = "black",
                            ticks.colour = "black",
                            title.hjust = 0,
                            alpha = alpha
                        )
                    },
                    na.value = "transparent"
                )
            )
        )
    }
    # If we add another layers, we need the ggnewscale::new_scale_fill() to avoid conflicts
    attr(layers, "scales") <- "fill"

    if (return_layer) {
        return(layers)
    }

    .wrap_spatial_layers(
        layers,
        ext = ext,
        flip_y = flip_y,
        legend.position = legend.position,
        legend.direction = legend.direction,
        title = title,
        subtitle = subtitle,
        xlab = xlab,
        ylab = ylab,
        theme = theme,
        theme_args = theme_args
    )
}

#' @rdname spatialplots
#' @concept spatial
#' @export
SpatMasksPlot <- function(
    data,
    ext = NULL,
    flip_y = TRUE,
    add_border = TRUE,
    border_color = "black",
    border_size = 0.5,
    border_alpha = 1,
    palette = "turbo",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    fill_name = NULL,
    return_layer = FALSE,
    theme = "theme_box",
    theme_args = list(),
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525
) {
    set.seed(seed)
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }

    stopifnot(
        "'data' must be a SpatRaster object" = inherits(data, "SpatRaster")
    )

    ext <- .prepare_extent(ext)
    if (!is.null(ext)) {
        data <- terra::crop(data, ext)
        if (terra::ncell(data) == 0) {
            stop(
                "[SpatMasksPlot] No data in the specified extent. Please check your extent."
            )
        }
    }

    if (flip_y) {
        data <- .flip_y(data)
    }

    # Set background (0 values) to NA for transparency
    data[data == 0] <- NA

    layers <- list(
        # Convert to data frame for proper transparency handling
        ggplot2::geom_raster(
            data = stats::setNames(
                terra::as.data.frame(data, xy = TRUE),
                c("x", "y", "value")
            ),
            aes(x = !!sym("x"), y = !!sym("y"), fill = !!sym("value")),
            alpha = alpha
        )
    )

    if (add_border) {
        # Convert mask values to polygons for cleaner visualization
        polys <- terra::as.polygons(data, dissolve = TRUE)
        layers <- c(
            layers,
            list(ggplot2::geom_sf(
                data = sf::st_as_sf(polys),
                fill = NA,
                color = border_color,
                linewidth = border_size,
                alpha = border_alpha
            ))
        )
    }

    layers <- c(
        layers,
        list(scale_fill_gradientn(
            name = fill_name %||% names(data)[1],
            n.breaks = 4,
            colors = palette_this(
                palette = palette,
                n = 256,
                reverse = palreverse,
                palcolor = palcolor
            ),
            guide = if (identical(legend.position, "none")) {
                "none"
            } else {
                guide_colorbar(
                    frame.colour = "black",
                    ticks.colour = "black",
                    title.hjust = 0
                )
            },
            na.value = "transparent"
        ))
    )
    # If we add another layers, we need the ggnewscale::new_scale_fill() to avoid conflicts
    attr(layers, "scales") <- "fill"

    if (return_layer) {
        return(layers)
    }

    .wrap_spatial_layers(
        layers,
        ext = ext,
        flip_y = flip_y,
        legend.position = legend.position,
        legend.direction = legend.direction,
        title = title,
        subtitle = subtitle,
        xlab = xlab,
        ylab = ylab,
        theme = theme,
        theme_args = theme_args
    )
}

#' @rdname spatialplots
#' @concept spatial
#' @export
SpatShapesPlot <- function(
    data,
    x = NULL,
    y = NULL,
    group = NULL,
    ext = NULL,
    flip_y = TRUE,
    fill_by = NULL,
    border_color = "black",
    border_size = 0.5,
    border_alpha = 1,
    palette = NULL,
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    fill_name = NULL,
    highlight = NULL,
    highlight_alpha = 1,
    highlight_size = 1,
    highlight_color = "black",
    highlight_stroke = 0.8,
    facet_scales = "fixed",
    facet_nrow = NULL,
    facet_ncol = NULL,
    facet_byrow = TRUE,
    return_layer = FALSE,
    theme = "theme_box",
    theme_args = list(),
    legend.position = ifelse(return_layer, "none", "right"),
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525
) {
    UseMethod("SpatShapesPlot", data)
}

#' @rdname spatialplots
#' @concept spatial
#' @export
SpatShapesPlot.SpatVector <- function(
    data,
    x = NULL,
    y = NULL,
    group = NULL,
    ext = NULL,
    flip_y = TRUE,
    fill_by = NULL,
    border_color = "black",
    border_size = 0.5,
    border_alpha = 1,
    palette = NULL,
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    fill_name = NULL,
    highlight = NULL,
    highlight_alpha = 1,
    highlight_size = 1,
    highlight_color = "black",
    highlight_stroke = 0.8,
    facet_scales = "fixed",
    facet_nrow = NULL,
    facet_ncol = NULL,
    facet_byrow = TRUE,
    return_layer = FALSE,
    theme = "theme_box",
    theme_args = list(),
    legend.position = ifelse(return_layer, "none", "right"),
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525
) {
    set.seed(seed)
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }

    ext <- .prepare_extent(ext)
    if (!is.null(ext)) {
        data <- terra::crop(data, ext)
        if (terra::nrow(data) == 0 || terra::ncol(data) == 0) {
            stop(
                "[SpatShapesPlot] No data in the specified extent. Please check your extent."
            )
        }
    }

    if (flip_y) {
        data <- .flip_y(data)
    }

    # Convert to sf object for ggplot
    data_sf <- sf::st_as_sf(data)

    # Handle multiple fill_by columns for faceting
    if (!is.null(fill_by) && length(fill_by) > 1) {
        # Check that all fill_by columns exist and are numeric
        missing_cols <- fill_by[!fill_by %in% names(data_sf)]
        if (length(missing_cols) > 0) {
            stop(
                "Columns not found in data: ",
                paste(missing_cols, collapse = ", ")
            )
        }

        non_numeric_cols <- fill_by[
            !sapply(fill_by, function(col) is.numeric(data_sf[[col]]))
        ]
        if (length(non_numeric_cols) > 0) {
            stop(
                "Multiple fill_by columns must be numeric. Non-numeric columns: ",
                paste(non_numeric_cols, collapse = ", ")
            )
        }

        # Reshape data for faceting: convert to long format
        data_long <- data_sf
        # Create a list to store the reshaped data
        facet_data_list <- list()

        for (col in fill_by) {
            temp_data <- data_sf
            temp_data$.fill <- temp_data[[col]]
            temp_data$.facet_var <- col
            facet_data_list[[col]] <- temp_data
        }

        # Combine all facet data
        data_sf <- do_call(rbind, facet_data_list)

        # Convert .facet_var to factor with levels in original fill_by order
        data_sf$.facet_var <- factor(data_sf$.facet_var, levels = fill_by)

        # Set fill_by to the new column name
        fill_by <- ".fill"
        facet_by <- ".facet_var"
    } else {
        # Single column case
        if (
            !is.null(fill_by) && is.character(fill_by) && length(fill_by) == 1
        ) {
            if (fill_by %in% names(data_sf)) {
                # It's a column name
                fill_by <- check_columns(
                    data_sf,
                    fill_by,
                    force_factor = !is.numeric(data_sf[[fill_by]])
                )
            } else {
                # It's a fixed color - don't treat as column
                fill_by <- fill_by
            }
        }
        facet_by <- NULL
    }

    # Set default palette based on data type
    if (is.null(palette) && !is.null(fill_by) && fill_by %in% names(data_sf)) {
        if (is.numeric(data_sf[[fill_by]])) {
            palette <- "turbo"
        } else {
            palette <- "Paired"
        }
    }
    palette <- palette %||% "turbo" # fallback default

    # Determine geometry type for appropriate geom
    # Assuming polygons
    # geom_type <- unique(sf::st_geometry_type(data_sf))

    # Build aesthetic mappings
    aes_mapping <- aes()
    fill_by_is_column <- !is.null(fill_by) &&
        is.character(fill_by) &&
        length(fill_by) == 1 &&
        fill_by %in% names(data_sf)

    if (fill_by_is_column) {
        aes_mapping$fill <- sym(fill_by)
    }

    # Handle border_color mapping
    border_aes <- "none"
    if (isTRUE(border_color) && fill_by_is_column) {
        # Use same variable for both fill and color
        aes_mapping$colour <- sym(fill_by)
        border_aes <- "same_as_fill"
    } else if (is.character(border_color) && length(border_color) == 1) {
        # Single color string - will be applied as fixed color
        border_aes <- "fixed_color"
    } else if (isFALSE(border_color)) {
        # No border
        border_aes <- "none"
    }

    # POLYGON, MULTIPOLYGON, or other
    # Build geom parameters conditionally
    geom_params <- list(
        data = data_sf,
        mapping = aes_mapping,
        alpha = alpha,
        linewidth = border_size
    )

    # Only add color if not mapped and needed
    if (!("colour" %in% names(aes_mapping))) {
        if (border_aes == "fixed_color") {
            geom_params$color <- adjcolors(border_color, border_alpha)
        } else if (border_aes == "none") {
            geom_params$color <- NA
        }
    }

    # Only add fill if not mapped and needed
    if (!("fill" %in% names(aes_mapping))) {
        if (!fill_by_is_column && !is.null(fill_by)) {
            geom_params$fill <- fill_by
        }
    }

    geom_layer <- do_call(ggplot2::geom_sf, geom_params)
    layers <- list(geom_layer)

    # Add appropriate scales
    if (fill_by_is_column) {
        if (is.numeric(data_sf[[fill_by]])) {
            # Numeric data - use gradient scale
            layers <- c(
                layers,
                list(
                    scale_fill_gradientn(
                        name = fill_name %||% fill_by,
                        n.breaks = 4,
                        colors = palette_this(
                            palette = palette,
                            n = 256,
                            reverse = palreverse,
                            palcolor = palcolor
                        ),
                        guide = if (identical(legend.position, "none")) {
                            "none"
                        } else {
                            guide_colorbar(
                                frame.colour = "black",
                                ticks.colour = "black",
                                title.hjust = 0
                            )
                        },
                        na.value = "transparent"
                    )
                )
            )
        } else {
            # Categorical data - use manual/discrete scale
            layers <- c(
                layers,
                list(
                    ggplot2::scale_fill_manual(
                        name = fill_name %||% fill_by,
                        values = palette_this(
                            levels(data_sf[[fill_by]]),
                            palette = palette,
                            reverse = palreverse,
                            palcolor = palcolor
                        ),
                        guide = if (identical(legend.position, "none")) {
                            "none"
                        } else {
                            "legend"
                        },
                        na.value = "transparent"
                    )
                )
            )
        }
    }

    # Add color scale only when border_color = TRUE (same as fill)
    if (border_aes == "same_as_fill") {
        if (is.numeric(data_sf[[fill_by]])) {
            # Numeric data - use gradient scale
            layers <- c(
                layers,
                list(
                    scale_color_gradientn(
                        n.breaks = 4,
                        colors = palette_this(
                            palette = palette,
                            n = 256,
                            reverse = palreverse,
                            palcolor = palcolor,
                            alpha = border_alpha
                        ),
                        guide = if (identical(legend.position, "none")) {
                            "none"
                        } else {
                            guide_colorbar(
                                frame.colour = "black",
                                ticks.colour = "black",
                                title.hjust = 0
                            )
                        },
                        na.value = "transparent"
                    )
                )
            )
        } else {
            # Categorical data - use manual/discrete scale
            layers <- c(
                layers,
                list(
                    ggplot2::scale_color_manual(
                        values = palette_this(
                            levels(data_sf[[fill_by]]),
                            palette = palette,
                            reverse = palreverse,
                            palcolor = palcolor
                        ),
                        guide = "none", # Always hide guide for border color
                        na.value = "transparent"
                    )
                )
            )
        }
    }

    # Set scale attribute for layers conflicts
    scales_used <- c(
        if (!is.null(fill_by)) "fill",
        if (border_aes == "same_as_fill") "color"
    )

    # Adding the highlight
    if (!isFALSE(highlight) && !is.null(highlight)) {
        if (isTRUE(highlight)) {
            hi_sf <- data_sf
        } else if (length(highlight) == 1 && is.character(highlight)) {
            hi_sf <- dplyr::filter(data_sf, !!!parse_exprs(highlight))
        } else {
            all_inst <- rownames(data_sf) %||% 1:nrow(data_sf)
            if (!any(highlight %in% all_inst)) {
                stop("No highlight items found in the data (rownames).")
            }
            if (!all(highlight %in% all_inst)) {
                warning(
                    "Not all highlight items found in the data (rownames).",
                    immediate. = TRUE
                )
            }
            hi_sf <- data_sf[intersect(highlight, all_inst), , drop = FALSE]
            rm(all_inst)
        }
        if (nrow(hi_sf) > 0) {
            # Add highlight border
            layers <- c(
                layers,
                list(
                    ggplot2::geom_sf(
                        data = hi_sf,
                        fill = NA,
                        color = highlight_color,
                        linewidth = border_size + highlight_stroke,
                        alpha = highlight_alpha
                    )
                )
            )
            scales_used <- c(scales_used, "color")
        }
    }

    # Set scale attribute for layers conflicts
    attr(layers, "scales") <- unique(scales_used)

    if (return_layer) {
        return(layers)
    }

    p <- .wrap_spatial_layers(
        layers,
        ext = ext,
        flip_y = flip_y,
        legend.position = legend.position,
        legend.direction = legend.direction,
        title = title,
        subtitle = subtitle,
        xlab = xlab,
        ylab = ylab,
        theme = theme,
        theme_args = theme_args
    )

    if (!is.null(facet_by)) {
        p <- facet_plot(
            p,
            facet_by,
            facet_scales,
            facet_nrow,
            facet_ncol,
            facet_byrow,
            legend.position = legend.position,
            legend.direction = legend.direction
        )
    }

    p
}

#' @rdname spatialplots
#' @concept spatial
#' @export
SpatShapesPlot.data.frame <- function(
    data,
    x,
    y,
    group,
    ext = NULL,
    flip_y = TRUE,
    fill_by = "grey90",
    border_color = "black",
    border_size = 0.5,
    border_alpha = 1,
    palette = NULL,
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    fill_name = NULL,
    highlight = NULL,
    highlight_alpha = 1,
    highlight_size = 1,
    highlight_color = "black",
    highlight_stroke = 0.8,
    facet_scales = "fixed",
    facet_nrow = NULL,
    facet_ncol = NULL,
    facet_byrow = TRUE,
    return_layer = FALSE,
    theme = "theme_box",
    theme_args = list(),
    legend.position = ifelse(return_layer, "none", "right"),
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525
) {
    set.seed(seed)
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }

    x <- check_columns(data, x)
    y <- check_columns(data, y)
    group <- check_columns(data, group, force_factor = TRUE)

    ext <- .prepare_extent(ext)
    if (!is.null(ext)) {
        data <- data[
            data[[x]] >= ext[1] &
                data[[x]] <= ext[2] &
                data[[y]] >= ext[3] &
                data[[y]] <= ext[4],
            ,
            drop = FALSE
        ]
        if (nrow(data) == 0) {
            stop(
                "[SpatShapesPlot] No data in the specified extent. Please check your extent."
            )
        }
    }

    if (flip_y) {
        data <- .flip_y(data)
    }

    # Handle multiple fill_by columns for faceting
    if (!is.null(fill_by) && length(fill_by) > 1) {
        # Check that all fill_by columns exist and are numeric
        missing_cols <- fill_by[!fill_by %in% names(data)]
        if (length(missing_cols) > 0) {
            stop(
                "[SpatShapesPlot] Columns not found in data: ",
                paste(missing_cols, collapse = ", ")
            )
        }

        non_numeric_cols <- fill_by[
            !sapply(fill_by, function(col) is.numeric(data[[col]]))
        ]
        if (length(non_numeric_cols) > 0) {
            stop(
                "[SpatShapesPlot] Multiple fill_by columns must be numeric. Non-numeric columns: ",
                paste(non_numeric_cols, collapse = ", ")
            )
        }

        # Reshape data for faceting: convert to long format
        data_long <- data
        # Create a list to store the reshaped data
        facet_data_list <- list()

        for (col in fill_by) {
            temp_data <- data
            temp_data$.fill <- temp_data[[col]]
            temp_data$.facet_var <- col
            facet_data_list[[col]] <- temp_data
        }

        # Combine all facet data
        data <- do_call(rbind, facet_data_list)

        # Convert .facet_var to factor with levels in original fill_by order
        data$.facet_var <- factor(data$.facet_var, levels = fill_by)

        # Set fill_by to the new column name
        fill_by <- ".fill"
        facet_by <- ".facet_var"
    } else {
        # Single column case
        if (
            !is.null(fill_by) && is.character(fill_by) && length(fill_by) == 1
        ) {
            if (fill_by %in% names(data)) {
                # It's a column name
                fill_by <- check_columns(
                    data,
                    fill_by,
                    force_factor = !is.numeric(data[[fill_by]])
                )
            } else {
                # It's a fixed color - don't treat as column
                fill_by <- fill_by
            }
        }
        facet_by <- NULL
    }

    # Set default palette based on data type
    if (is.null(palette) && !is.null(fill_by) && fill_by %in% names(data)) {
        if (is.numeric(data[[fill_by]])) {
            palette <- "turbo"
        } else {
            palette <- "Paired"
        }
    }
    palette <- palette %||% "turbo" # fallback default

    # Determine geometry type for appropriate geom
    # Assuming polygons
    # geom_type <- unique(sf::st_geometry_type(data))

    # Build aesthetic mappings
    aes_mapping <- aes(x = !!sym(x), y = !!sym(y), group = !!sym(group))
    fill_by_is_column <- !is.null(fill_by) &&
        is.character(fill_by) &&
        length(fill_by) == 1 &&
        fill_by %in% names(data)

    if (fill_by_is_column) {
        aes_mapping$fill <- sym(fill_by)
    }

    # Handle border_color mapping
    border_aes <- "none"
    if (isTRUE(border_color) && fill_by_is_column) {
        # Use same variable for both fill and color
        aes_mapping$colour <- sym(fill_by)
        border_aes <- "same_as_fill"
    } else if (is.character(border_color) && length(border_color) == 1) {
        # Single color string - will be applied as fixed color
        border_aes <- "fixed_color"
    } else if (isFALSE(border_color)) {
        # No border
        border_aes <- "none"
    }

    # POLYGON, MULTIPOLYGON, or other
    # Build geom parameters conditionally
    geom_params <- list(
        data = data,
        mapping = aes_mapping,
        alpha = alpha,
        linewidth = border_size
    )

    # Only add color if not mapped and needed
    if (!("colour" %in% names(aes_mapping))) {
        if (border_aes == "fixed_color") {
            geom_params$color <- adjcolors(border_color, border_alpha)
        } else if (border_aes == "none") {
            geom_params$color <- NA
        }
    }

    # Only add fill if not mapped and needed
    if (!("fill" %in% names(aes_mapping))) {
        if (!fill_by_is_column && !is.null(fill_by)) {
            geom_params$fill <- fill_by
        }
    }

    geom_layer <- do_call(ggplot2::geom_polygon, geom_params)
    layers <- list(geom_layer)

    # Add appropriate scales
    if (fill_by_is_column) {
        if (is.numeric(data[[fill_by]])) {
            # Numeric data - use gradient scale
            layers <- c(
                layers,
                list(
                    scale_fill_gradientn(
                        name = fill_name %||% fill_by,
                        n.breaks = 4,
                        colors = palette_this(
                            palette = palette,
                            n = 256,
                            reverse = palreverse,
                            palcolor = palcolor
                        ),
                        guide = if (identical(legend.position, "none")) {
                            "none"
                        } else {
                            guide_colorbar(
                                frame.colour = "black",
                                ticks.colour = "black",
                                title.hjust = 0
                            )
                        },
                        na.value = "transparent"
                    )
                )
            )
        } else {
            # Categorical data - use manual/discrete scale
            layers <- c(
                layers,
                list(
                    ggplot2::scale_fill_manual(
                        name = fill_name %||% fill_by,
                        values = palette_this(
                            levels(data[[fill_by]]),
                            palette = palette,
                            reverse = palreverse,
                            palcolor = palcolor
                        ),
                        guide = if (identical(legend.position, "none")) {
                            "none"
                        } else {
                            "legend"
                        },
                        na.value = "transparent"
                    )
                )
            )
        }
    }

    # Add color scale only when border_color = TRUE (same as fill)
    if (border_aes == "same_as_fill") {
        if (is.numeric(data[[fill_by]])) {
            # Numeric data - use gradient scale
            layers <- c(
                layers,
                list(
                    scale_color_gradientn(
                        n.breaks = 4,
                        colors = palette_this(
                            palette = palette,
                            n = 256,
                            reverse = palreverse,
                            palcolor = palcolor,
                            alpha = border_alpha
                        ),
                        guide = if (identical(legend.position, "none")) {
                            "none"
                        } else {
                            guide_colorbar(
                                frame.colour = "black",
                                ticks.colour = "black",
                                title.hjust = 0
                            )
                        },
                        na.value = "transparent"
                    )
                )
            )
        } else {
            # Categorical data - use manual/discrete scale
            layers <- c(
                layers,
                list(
                    ggplot2::scale_color_manual(
                        values = palette_this(
                            levels(data[[fill_by]]),
                            palette = palette,
                            reverse = palreverse,
                            palcolor = palcolor
                        ),
                        guide = "none", # Always hide guide for border color
                        na.value = "transparent"
                    )
                )
            )
        }
    }

    # Set scale attribute for layers conflicts
    scales_used <- c(
        if (!is.null(fill_by)) "fill",
        if (border_aes == "same_as_fill") "color"
    )

    # Adding the highlight
    if (!isFALSE(highlight) && !is.null(highlight)) {
        if (isTRUE(highlight)) {
            hi_data <- data
        } else if (length(highlight) == 1 && is.character(highlight)) {
            hi_data <- dplyr::filter(data, !!!parse_exprs(highlight))
        } else {
            all_inst <- rownames(data) %||% 1:nrow(data)
            if (!any(highlight %in% all_inst)) {
                stop("No highlight items found in the data (rownames).")
            }
            if (!all(highlight %in% all_inst)) {
                warning(
                    "Not all highlight items found in the data (rownames).",
                    immediate. = TRUE
                )
            }
            hi_data <- data[intersect(highlight, all_inst), , drop = FALSE]
            rm(all_inst)
        }
        if (nrow(hi_data) > 0) {
            # Add highlight border
            layers <- c(
                layers,
                list(
                    ggplot2::geom_polygon(
                        data = hi_data,
                        mapping = aes(
                            x = !!sym(x),
                            y = !!sym(y),
                            group = !!sym(group)
                        ),
                        fill = NA,
                        color = highlight_color,
                        linewidth = border_size + highlight_stroke,
                        alpha = highlight_alpha
                    )
                )
            )
            scales_used <- c(scales_used, "color")
        }
    }

    # Set scale attribute for layers conflicts
    attr(layers, "scales") <- unique(scales_used)

    if (return_layer) {
        return(layers)
    }

    p <- .wrap_spatial_layers(
        layers,
        ext = ext,
        flip_y = flip_y,
        legend.position = legend.position,
        legend.direction = legend.direction,
        title = title,
        subtitle = subtitle,
        xlab = xlab,
        ylab = ylab,
        theme = theme,
        theme_args = theme_args
    )

    if (!is.null(facet_by)) {
        p <- facet_plot(
            p,
            facet_by,
            facet_scales,
            facet_nrow,
            facet_ncol,
            facet_byrow,
            legend.position = legend.position,
            legend.direction = legend.direction
        )
    }

    p
}

#' @rdname spatialplots
#' @concept spatial
#' @export
SpatPointsPlot <- function(
    data,
    x = NULL,
    y = NULL,
    ext = NULL,
    flip_y = TRUE,
    color_by = NULL,
    size_by = NULL,
    size = NULL,
    fill_by = NULL,
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    palette = NULL,
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    color_name = NULL,
    size_name = NULL,
    shape = 16,
    border_color = "black",
    border_size = 0.5,
    border_alpha = 1,
    raster = NULL,
    raster_dpi = c(512, 512),
    hex = FALSE,
    hex_linewidth = 0.5,
    hex_count = FALSE,
    hex_bins = 50,
    hex_binwidth = NULL,
    label = FALSE,
    label_size = 4,
    label_fg = "white",
    label_bg = "black",
    label_bg_r = 0.1,
    label_repel = FALSE,
    label_repulsion = 20,
    label_pt_size = 1,
    label_pt_color = "black",
    label_segment_color = "black",
    label_insitu = FALSE,
    label_pos = c(
        "median",
        "mean",
        "max",
        "min",
        "first",
        "last",
        "center",
        "random"
    ),
    highlight = NULL,
    highlight_alpha = 1,
    highlight_size = 1,
    highlight_color = "black",
    highlight_stroke = 0.8,
    graph = NULL,
    graph_x = NULL,
    graph_y = NULL,
    graph_xend = NULL,
    graph_yend = NULL,
    graph_value = NULL,
    edge_size = c(0.05, 0.5),
    edge_alpha = 0.1,
    edge_color = "grey40",
    facet_scales = "fixed",
    facet_nrow = NULL,
    facet_ncol = NULL,
    facet_byrow = TRUE,
    return_layer = FALSE,
    theme = "theme_box",
    theme_args = list(),
    legend.position = ifelse(return_layer, "none", "right"),
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525
) {
    set.seed(seed)
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    stopifnot(
        "'size_by' and 'size' should not be both specified" = is.null(
            size_by
        ) ||
            is.null(size)
    )
    stopifnot(
        "'size' must be a single numeric value" = is.null(size) ||
            (is.numeric(size) && length(size) == 1)
    )
    size_by <- size_by %||% size

    stopifnot("'data' must be a data.frame" = is.data.frame(data))
    if (!is.null(fill_by) && !is.null(color_by) && fill_by != color_by) {
        stop(
            "Can't use both 'fill_by' and 'color_by'. Always use 'color_by' for points."
        )
    }
    color_by <- color_by %||% fill_by

    # Determine x and y columns
    if (is.null(x)) {
        x_candidates <- c(
            "x",
            "X",
            "sdimx",
            "coord_x",
            "spatial_x",
            "longitude",
            "lon",
            "lng"
        )
        x <- x_candidates[x_candidates %in% names(data)][1]
        if (is.na(x)) {
            stop(
                "Could not find x coordinate column. Please specify 'x' parameter."
            )
        }
    }

    if (is.null(y)) {
        y_candidates <- c(
            "y",
            "Y",
            "sdimy",
            "coord_y",
            "spatial_y",
            "latitude",
            "lat"
        )
        y <- y_candidates[y_candidates %in% names(data)][1]
        if (is.na(y)) {
            stop(
                "Could not find y coordinate column. Please specify 'y' parameter."
            )
        }
    }
    x <- check_columns(data, x)
    y <- check_columns(data, y)

    # Set up raster parameters similar to DimPlot
    if (length(raster_dpi) == 1) {
        raster_dpi <- rep(raster_dpi, 2)
    }
    raster_is_null <- is.null(raster)
    raster <- raster %||% (nrow(data) > 1e6)
    if (isTRUE(raster) && raster_is_null) {
        warning(
            "[SpatPointsPlot] Rasterization is enabled by default for large datasets (nrow > 1e6). Set 'raster = FALSE' to disable."
        )
    }

    # Apply extent cropping if specified
    ext <- .prepare_extent(ext)
    if (!is.null(ext)) {
        data <- data[
            data[[x]] >= ext[1] &
                data[[x]] <= ext[2] &
                data[[y]] >= ext[3] &
                data[[y]] <= ext[4],
            ,
            drop = FALSE
        ]
        if (nrow(data) == 0) {
            stop(
                "[SpatPointsPlot] No data in the specified extent. Please check your extent."
            )
        }
    }

    # Apply y-axis flipping
    if (flip_y) {
        data <- .flip_y(data, y = y)
    }

    # Handle multiple color_by columns for faceting
    if (!is.null(color_by) && length(color_by) > 1) {
        # Check that all color_by columns exist and are numeric
        missing_cols <- color_by[!color_by %in% names(data)]
        if (length(missing_cols) > 0) {
            stop(
                "Columns not found in data: ",
                paste(missing_cols, collapse = ", ")
            )
        }

        non_numeric_cols <- color_by[
            !sapply(color_by, function(col) is.numeric(data[[col]]))
        ]
        if (length(non_numeric_cols) > 0) {
            stop(
                "Multiple color_by columns must be numeric. Non-numeric columns: ",
                paste(non_numeric_cols, collapse = ", ")
            )
        }

        # Reshape data for faceting: convert to long format
        facet_data_list <- list()

        for (col in color_by) {
            temp_data <- data
            temp_data$.color <- temp_data[[col]]
            temp_data$.facet_var <- col
            facet_data_list[[col]] <- temp_data
        }

        # Combine all facet data
        data <- do_call(rbind, facet_data_list)

        # Convert .facet_var to factor with levels in original color_by order
        data$.facet_var <- factor(data$.facet_var, levels = color_by)

        # Set color_by to the new column name
        color_by <- ".color"
        facet_by <- ".facet_var"
    } else {
        # Single column case
        if (
            !is.null(color_by) &&
                is.character(color_by) &&
                length(color_by) == 1
        ) {
            if (color_by %in% names(data)) {
                # It's a column name
                color_by <- check_columns(
                    data,
                    color_by,
                    force_factor = !is.numeric(data[[color_by]])
                )
            } else {
                # It's a fixed color - don't treat as column
                color_by <- color_by
            }
        }
        facet_by <- NULL
    }

    if (
        !is.null(color_by) &&
            color_by %in% names(data) &&
            is.numeric(data[[color_by]])
    ) {
        lower_cutoff <- lower_cutoff %||%
            quantile(
                data[[color_by]][is.finite(data[[color_by]])],
                lower_quantile,
                na.rm = TRUE
            )
        upper_cutoff <- upper_cutoff %||%
            quantile(
                data[[color_by]][is.finite(data[[color_by]])],
                upper_quantile,
                na.rm = TRUE
            )
        if (upper_cutoff == lower_cutoff) {
            if (upper_cutoff == 0) {
                upper_cutoff <- 1e-3
            } else {
                upper_cutoff <- upper_cutoff + upper_cutoff * 1e-3
            }
        }
        data[[color_by]][data[[color_by]] < lower_cutoff] <- lower_cutoff
        data[[color_by]][data[[color_by]] > upper_cutoff] <- upper_cutoff
    }

    # Set default palette based on data type
    if (is.null(palette) && !is.null(color_by) && color_by %in% names(data)) {
        if (is.numeric(data[[color_by]])) {
            palette <- "turbo"
        } else {
            palette <- "Paired"
        }
    }
    palette <- palette %||% "turbo" # fallback default

    # Check and prepare aesthetic columns
    color_by_is_column <- !is.null(color_by) &&
        is.character(color_by) &&
        length(color_by) == 1 &&
        color_by %in% names(data)
    size_by_is_column <- !is.null(size_by) &&
        is.character(size_by) &&
        length(size_by) == 1 &&
        size_by %in% names(data)

    scales_used <- c()
    # Build aesthetic mappings
    aes_mapping <- aes(x = !!sym(x), y = !!sym(y))

    # Only add aesthetic mappings when they are actual column mappings
    if (size_by_is_column && !is.null(size_by) && size_by %in% names(data)) {
        aes_mapping$size <- sym(size_by)
        scales_used <- c(scales_used, "size")
    }

    # Check if shape supports borders (shapes 21-25)
    has_border <- shape %in% 21:25

    # Handle aesthetic mapping based on shape type
    if (has_border) {
        # For border shapes (21-25), color_by maps to fill when it's a column
        if (
            color_by_is_column &&
                !is.null(color_by) &&
                color_by %in% names(data) &&
                !isTRUE(raster)
        ) {
            aes_mapping$fill <- sym(color_by)
            scales_used <- c(scales_used, "fill")

            # Handle border_color logic for border shapes
            if (isTRUE(border_color)) {
                # Use same variable for both fill and color (border)
                aes_mapping$colour <- sym(color_by)
                border_aes <- "same_as_fill"
                scales_used <- c(scales_used, "color")
            } else {
                border_aes <- if (is.character(border_color)) {
                    "fixed_color"
                } else {
                    "none"
                }
            }
        } else {
            # No column mapping for fill
            border_aes <- if (is.character(border_color)) {
                "fixed_color"
            } else {
                "none"
            }
        }
    } else {
        # For non-border shapes, color_by maps to color when it's a column
        if (
            color_by_is_column &&
                !is.null(color_by) &&
                color_by %in% names(data)
        ) {
            aes_mapping$colour <- sym(color_by)
            scales_used <- c(scales_used, "color")
        }
        border_aes <- "none"
    }

    # Create geom layers with raster and hex support
    layers <- list()

    ## Adding the graph/network
    if (!is.null(graph)) {
        is_coord_graph <- !is.null(graph_x) &&
            !is.null(graph_y) &&
            !is.null(graph_xend) &&
            !is.null(graph_yend) &&
            !is.null(graph_value)

        if (
            is.character(graph) && length(graph) == 1 && startsWith(graph, "@")
        ) {
            graph <- substring(graph, 2)
            net_mat <- attr(data, graph)
        } else if (is.matrix(graph) || is.data.frame(graph)) {
            net_mat <- graph
        } else if (is.numeric(graph)) {
            graph <- colnames(data)[graph]
            net_mat <- data[graph]
        } else if (is.character(graph)) {
            net_mat <- data[graph]
        } else {
            stop(
                "The 'graph' should be a matrix, data.frame, indexes, or column names."
            )
        }
        if (is_coord_graph) {
            graph_x <- check_columns(net_mat, graph_x)
            graph_y <- check_columns(net_mat, graph_y)
            graph_xend <- check_columns(net_mat, graph_xend)
            graph_yend <- check_columns(net_mat, graph_yend)
            graph_value <- check_columns(net_mat, graph_value)
            net_mat <- data.frame(
                x = net_mat[[graph_x]],
                y = net_mat[[graph_y]],
                xend = net_mat[[graph_xend]],
                yend = net_mat[[graph_yend]],
                value = net_mat[[graph_value]]
            )
        } else {
            if (!is.matrix(net_mat)) {
                net_mat <- as.matrix(net_mat)
            }

            if (!is.null(rownames(net_mat)) && !is.null(colnames(net_mat))) {
                net_mat <- net_mat[rownames(data), rownames(data)]
            } else if (nrow(net_mat) != ncol(net_mat)) {
                stop(
                    "[SpatPointsPlot] The graph matrix should be square (same number of rows and columns)."
                )
            } else if (nrow(net_mat) != nrow(data)) {
                stop(
                    "[SpatPointsPlot] The graph matrix should have the same number of rows as the input data."
                )
            }
            net_mat[net_mat == 0] <- NA
            net_mat[upper.tri(net_mat)] <- NA

            handle_single_facet_value <- function(mat) {
                ndf <- reshape2::melt(
                    mat,
                    na.rm = TRUE,
                    stringsAsFactors = FALSE
                )
                ndf$value <- as.numeric(ndf$value)
                ndf$Var1 <- as.character(ndf$Var1)
                ndf$Var2 <- as.character(ndf$Var2)
                ndf$x <- data[ndf$Var1, x]
                ndf$y <- data[ndf$Var1, y]
                ndf$xend <- data[ndf$Var2, x]
                ndf$yend <- data[ndf$Var2, y]
                return(ndf)
            }

            if (!is.null(facet_by)) {
                net_mat <- do_call(
                    rbind,
                    lapply(split(data, data[, facet_by]), function(d) {
                        d <- handle_single_facet_value(net_mat[
                            rownames(d),
                            rownames(d)
                        ])
                        d[, facet_by] <- d[1, facet_by]
                        d
                    })
                )
            } else {
                net_mat <- handle_single_facet_value(net_mat)
            }
        }

        layers <- c(
            layers,
            list(
                ggplot2::geom_segment(
                    data = net_mat,
                    mapping = aes(
                        x = !!sym("x"),
                        y = !!sym("y"),
                        xend = !!sym("xend"),
                        yend = !!sym("yend"),
                        linewidth = !!sym("value")
                    ),
                    color = edge_color,
                    alpha = edge_alpha,
                    show.legend = FALSE
                ),
                ggplot2::scale_linewidth_continuous(range = edge_size)
            )
        )
    }

    # Create the main point layers
    if (isTRUE(hex)) {
        # Hex functionality - only for numeric color_by
        if (is.null(color_by)) {
            stop(
                "Hex plotting requires a 'color_by' column to aggregate values."
            )
        }
        if (
            !is.null(color_by) &&
                color_by_is_column &&
                !is.numeric(data[[color_by]])
        ) {
            stop(
                "Hex plotting only works with numeric 'color_by' values. Use regular points for categorical data."
            )
        }

        if (isTRUE(hex_count)) {
            if (is.null(color_by) || !color_by_is_column) {
                stop(
                    "Don't know how to count for the hex when 'color_by' is not provided."
                )
            }
            geom_layer <- ggplot2::geom_hex(
                data = data,
                mapping = aes(
                    x = !!sym(x),
                    y = !!sym(y),
                    color = !!sym(color_by),
                    fill = !!sym(color_by),
                    alpha = after_stat(!!sym("count"))
                ),
                linewidth = hex_linewidth,
                bins = hex_bins,
                binwidth = hex_binwidth
            )
            scales_used <- c(scales_used, "fill", "color")
        } else {
            # Hex without color mapping - use stat_summary_hex for aggregation
            data_na <- data[is.na(data[[color_by]]), , drop = FALSE]
            if (nrow(data_na) > 0) {
                geom_layer <- list(
                    ggplot2::geom_hex(
                        data = data_na,
                        mapping = aes(x = !!sym(x), y = !!sym(y)),
                        fill = "grey80",
                        linewidth = hex_linewidth,
                        bins = hex_bins,
                        binwidth = hex_binwidth,
                        alpha = alpha / 2
                    ),
                    ggplot2::stat_summary_hex(
                        data = data[!is.na(data[[color_by]]), , drop = FALSE],
                        mapping = aes(
                            x = !!sym(x),
                            y = !!sym(y),
                            z = !!sym(color_by)
                        ),
                        linewidth = hex_linewidth,
                        bins = hex_bins,
                        binwidth = hex_binwidth,
                        alpha = alpha
                    )
                )
            } else {
                geom_layer <- ggplot2::stat_summary_hex(
                    data = data,
                    mapping = aes(
                        x = !!sym(x),
                        y = !!sym(y),
                        z = !!sym(color_by)
                    ),
                    linewidth = hex_linewidth,
                    bins = hex_bins,
                    binwidth = hex_binwidth,
                    alpha = alpha
                )
                scales_used <- c(scales_used, "fill")
            }
        }
    } else if (isTRUE(raster)) {
        if (raster_is_null && !identical(raster_dpi, c(512, 512))) {
            message(
                "[SpatPointsPlot] 'raster' is enabled. Point size (size_by) is ignored, try 'raster_dpi' to control resolution."
            )
        }
        # Use scattermore for rasterized plotting
        if (!color_by_is_column) {
            # No color mapping - use fixed color
            fixed_color <- if (!is.null(color_by) && is.character(color_by)) {
                color_by
            } else {
                "black"
            }
            geom_layer <- scattermore::geom_scattermore(
                data = data,
                mapping = aes(x = !!sym(x), y = !!sym(y)),
                color = fixed_color,
                alpha = alpha,
                pixels = raster_dpi
            )
        } else if (has_border) {
            # Border shapes with color mapping - need two layers
            geom_layer <- list(
                scattermore::geom_scattermore(
                    data = data,
                    mapping = aes(x = !!sym(x), y = !!sym(y)),
                    color = if (border_aes == "fixed_color") {
                        adjcolors(border_color, border_alpha)
                    } else {
                        "black"
                    },
                    alpha = alpha,
                    pixels = raster_dpi
                ),
                scattermore::geom_scattermore(
                    data = data,
                    mapping = aes(
                        x = !!sym(x),
                        y = !!sym(y),
                        color = !!sym(color_by)
                    ),
                    alpha = alpha,
                    pixels = raster_dpi
                )
            )
            scales_used <- c(scales_used, "color")
        } else {
            # Non-border shapes with color mapping
            geom_layer <- list(scattermore::geom_scattermore(
                data = data,
                mapping = aes(
                    x = !!sym(x),
                    y = !!sym(y),
                    color = !!sym(color_by)
                ),
                alpha = alpha,
                pixels = raster_dpi
            ))
        }
    } else {
        # Standard geom_point - build parameters conditionally
        geom_params <- list(
            data = data,
            mapping = aes_mapping,
            alpha = alpha,
            shape = shape
        )
        if (has_border) {
            geom_params$stroke <- border_size
        }

        # Only add size parameter when not mapped
        if (!size_by_is_column && is.numeric(size_by)) {
            geom_params$size <- size_by
        }

        # Only add color parameter when not mapped
        if (!("colour" %in% names(aes_mapping))) {
            if (has_border && border_aes == "fixed_color") {
                geom_params$color <- adjcolors(border_color, border_alpha)
            } else if (has_border && border_aes == "none") {
                geom_params$color <- NA
            } else if (
                !has_border &&
                    !color_by_is_column &&
                    !is.null(color_by) &&
                    is.character(color_by)
            ) {
                geom_params$color <- color_by
            } else if (!has_border && !color_by_is_column) {
                geom_params$color <- "black"
            }
        }

        # Only add fill parameter when not mapped
        if (!("fill" %in% names(aes_mapping))) {
            if (
                has_border &&
                    !color_by_is_column &&
                    !is.null(color_by) &&
                    is.character(color_by)
            ) {
                geom_params$fill <- color_by
            }
        }

        geom_layer <- do_call(ggplot2::geom_point, geom_params)
    }

    if (inherits(geom_layer, c("gg", "Layer"))) {
        layers <- c(layers, list(geom_layer))
    } else {
        layers <- c(layers, geom_layer)
    }

    # because the points could be very small, we need to set a larger point size for the legend
    # to make it visible.
    # should we make this configurable?
    legend_point_size <- 3

    # Add appropriate scales
    if (color_by_is_column && !is.null(color_by)) {
        if ("fill" %in% scales_used) {
            if (is.numeric(data[[color_by]])) {
                layers <- c(
                    layers,
                    list(
                        scale_fill_gradientn(
                            name = color_name %||% color_by,
                            n.breaks = 4,
                            colors = palette_this(
                                palette = palette,
                                n = 256,
                                reverse = palreverse,
                                palcolor = palcolor
                            ),
                            guide = if (identical(legend.position, "none")) {
                                "none"
                            } else {
                                guide_colorbar(
                                    frame.colour = "black",
                                    ticks.colour = "black",
                                    title.hjust = 0
                                )
                            },
                            na.value = "transparent"
                        )
                    )
                )
            } else {
                layers <- c(
                    layers,
                    list(
                        ggplot2::scale_fill_manual(
                            name = color_name %||% color_by,
                            values = palette_this(
                                levels(data[[color_by]]),
                                palette = palette,
                                reverse = palreverse,
                                palcolor = palcolor
                            ),
                            guide = if (identical(legend.position, "none")) {
                                "none"
                            } else {
                                ggplot2::guide_legend(
                                    override.aes = list(
                                        size = legend_point_size
                                    )
                                )
                            },
                            na.value = "transparent"
                        )
                    )
                )
            }
        }
        if ("color" %in% scales_used) {
            if (is.numeric(data[[color_by]])) {
                if (has_border && border_aes == "same_as_fill") {
                    colors <- palette_this(
                        palette = palette,
                        n = 256,
                        reverse = palreverse,
                        palcolor = palcolor,
                        alpha = border_alpha,
                        transparent = FALSE
                    )
                } else {
                    colors <- palette_this(
                        palette = palette,
                        n = 256,
                        reverse = palreverse,
                        palcolor = palcolor
                    )
                }
                layers <- c(
                    layers,
                    list(
                        scale_color_gradientn(
                            name = color_name %||% color_by,
                            n.breaks = 4,
                            colors = colors,
                            guide = if (
                                identical(legend.position, "none") ||
                                    "fill" %in% scales_used
                            ) {
                                "none"
                            } else {
                                guide_colorbar(
                                    frame.colour = "black",
                                    ticks.colour = "black",
                                    title.hjust = 0
                                )
                            },
                            na.value = "transparent"
                        )
                    )
                )
            } else {
                if (has_border && border_aes == "same_as_fill") {
                    # Use same colors as fill
                    colors <- palette_this(
                        levels(data[[color_by]]),
                        palette = palette,
                        reverse = palreverse,
                        palcolor = palcolor,
                        alpha = border_alpha,
                        transparent = FALSE
                    )
                } else {
                    colors <- palette_this(
                        levels(data[[color_by]]),
                        palette = palette,
                        reverse = palreverse,
                        palcolor = palcolor
                    )
                }
                layers <- c(
                    layers,
                    list(
                        ggplot2::scale_color_manual(
                            name = color_name %||% color_by,
                            values = colors,
                            guide = if (
                                identical(legend.position, "none") ||
                                    "fill" %in% scales_used
                            ) {
                                "none"
                            } else {
                                ggplot2::guide_legend(
                                    override.aes = list(
                                        size = legend_point_size
                                    )
                                )
                            },
                            na.value = "transparent"
                        )
                    )
                )
            }
        }
    }

    if ("size" %in% scales_used) {
        layers <- c(
            layers,
            list(
                ggplot2::scale_size_continuous(
                    name = size_name %||% size_by,
                    guide = if (identical(legend.position, "none")) {
                        "none"
                    } else {
                        "legend"
                    }
                )
            )
        )
    }

    # Adding the highlight (similar to DimPlot)
    if (!isFALSE(highlight) && !is.null(highlight)) {
        if (isTRUE(hex)) {
            stop("Highlight is not supported for hex plot.")
        }
        if (isTRUE(highlight)) {
            hi_df <- data
        } else if (length(highlight) == 1 && is.character(highlight)) {
            hi_df <- dplyr::filter(data, !!!parse_exprs(highlight))
        } else {
            all_inst <- rownames(data) %||% 1:nrow(data)
            if (!any(highlight %in% all_inst)) {
                stop("No highlight items found in the data (rownames).")
            }
            if (!all(highlight %in% all_inst)) {
                warning(
                    "Not all highlight items found in the data (rownames).",
                    immediate. = TRUE
                )
            }
            hi_df <- data[intersect(highlight, all_inst), , drop = FALSE]
            rm(all_inst)
        }
        if (nrow(hi_df) > 0) {
            if (isTRUE(raster)) {
                layers <- c(
                    layers,
                    list(
                        scattermore::geom_scattermore(
                            data = hi_df,
                            aes(x = !!sym(x), y = !!sym(y)),
                            color = highlight_color,
                            pointsize = floor(highlight_size) +
                                highlight_stroke,
                            alpha = highlight_alpha,
                            pixels = raster_dpi
                        ),
                        scattermore::geom_scattermore(
                            data = hi_df,
                            aes(
                                x = !!sym(x),
                                y = !!sym(y),
                                color = !!sym(color_by)
                            ),
                            pointsize = floor(highlight_size),
                            alpha = highlight_alpha,
                            pixels = raster_dpi
                        )
                    )
                )
                scales_used <- c(scales_used, "color")
            } else {
                layers <- c(
                    layers,
                    list(
                        geom_point(
                            data = hi_df,
                            aes(x = !!sym(x), y = !!sym(y)),
                            color = highlight_color,
                            size = highlight_size + highlight_stroke,
                            alpha = highlight_alpha
                        ),
                        geom_point(
                            data = hi_df,
                            aes(
                                x = !!sym(x),
                                y = !!sym(y),
                                color = !!sym(color_by)
                            ),
                            size = highlight_size,
                            alpha = highlight_alpha
                        )
                    )
                )
                scales_used <- c(scales_used, "color")
            }
        }
    }

    # Force label to be TRUE when label_repel or label_insitu is TRUE (similar to DimPlot)
    if ((isTRUE(label_repel) || isTRUE(label_insitu)) && !isTRUE(label)) {
        message(
            "Forcing label to be TRUE when label_repel or label_insitu is TRUE."
        )
        label <- TRUE
    }

    # Adding the labels (similar to DimPlot)
    if (isTRUE(label)) {
        if (is.null(color_by) || !color_by_is_column) {
            stop(
                "Adding labels requires 'color_by' to be specified as a categorical column."
            )
        }
        if (is.numeric(data[[color_by]])) {
            stop(
                "Adding labels is not supported for numeric 'color_by' values. Use categorical data."
            )
        }
        if (isTRUE(hex)) {
            stop("Adding labels is not supported for hex plots.")
        }

        if (is.character(label_pos)) {
            label_pos <- match.arg(label_pos)
            if (label_pos == "median") {
                label_pos <- function(x) median(x, na.rm = TRUE)
            } else if (label_pos == "mean") {
                label_pos <- function(x) mean(x, na.rm = TRUE)
            } else if (label_pos == "first") {
                label_pos <- function(x) x[1]
            } else if (label_pos == "last") {
                label_pos <- function(x) x[length(x)]
            } else if (label_pos == "random") {
                label_pos <- function(x) sample(x, 1)
            } else if (label_pos == "center") {
                label_pos <- function(x) mean(range(x, na.rm = TRUE))
            } else if (label_pos == "min") {
                label_pos <- function(x) min(x, na.rm = TRUE)
            } else if (label_pos == "max") {
                label_pos <- function(x) max(x, na.rm = TRUE)
            } else {
                stop(
                    "Invalid label position specified. Use 'median', 'mean', 'first', 'last', 'random', or 'center'."
                )
            }
        }

        if (!is.null(facet_by)) {
            label_df <- aggregate(
                data[, c(x, y)],
                by = list(data[[color_by]], data[[facet_by]]),
                FUN = label_pos
            )
            colnames(label_df)[1:2] <- c(".label", facet_by)
        } else {
            label_df <- aggregate(
                data[, c(x, y)],
                by = list(data[[color_by]]),
                FUN = label_pos
            )
            colnames(label_df)[1] <- ".label"
        }
        label_df <- label_df[!is.na(label_df[, ".label"]), , drop = FALSE]

        if (!isTRUE(label_insitu)) {
            label_df[, ".label"] <- seq_len(nrow(label_df))
        }

        if (isTRUE(label_repel)) {
            layers <- c(
                layers,
                list(
                    geom_point(
                        data = label_df,
                        mapping = aes(x = !!sym(x), y = !!sym(y)),
                        color = label_pt_color,
                        size = label_pt_size
                    ),
                    ggrepel::geom_text_repel(
                        data = label_df,
                        aes(
                            x = !!sym(x),
                            y = !!sym(y),
                            label = !!sym(".label")
                        ),
                        point.size = label_pt_size,
                        max.overlaps = 100,
                        force = label_repulsion,
                        color = label_fg,
                        bg.color = label_bg,
                        bg.r = label_bg_r,
                        size = label_size,
                        inherit.aes = FALSE
                    )
                )
            )
        } else {
            layers <- c(
                layers,
                list(
                    ggrepel::geom_text_repel(
                        data = label_df,
                        aes(
                            x = !!sym(x),
                            y = !!sym(y),
                            label = !!sym(".label")
                        ),
                        fontface = "bold",
                        min.segment.length = 0,
                        segment.color = label_segment_color,
                        point.size = NA,
                        max.overlaps = 100,
                        force = 0,
                        color = label_fg,
                        bg.color = label_bg,
                        bg.r = label_bg_r,
                        size = label_size,
                        inherit.aes = FALSE
                    )
                )
            )
        }
    }

    # Set scale attribute for layers conflicts
    attr(layers, "scales") <- unique(scales_used)

    if (return_layer) {
        return(layers)
    }

    p <- .wrap_spatial_layers(
        layers,
        ext = ext,
        flip_y = flip_y,
        legend.position = legend.position,
        legend.direction = legend.direction,
        title = title,
        subtitle = subtitle,
        xlab = xlab,
        ylab = ylab,
        theme = theme,
        theme_args = theme_args
    )

    if (!is.null(facet_by)) {
        p <- facet_plot(
            p,
            facet_by,
            facet_scales,
            facet_nrow,
            facet_ncol,
            facet_byrow,
            legend.position = legend.position,
            legend.direction = legend.direction
        )
    }

    p
}

#' Atomic linked heatmap (internal)
#'
#' @description
#' Draws two heatmaps side-by-side with spline link lines connecting matching
#' rows across the left and right heatmaps.  This is the core implementation
#' layer — it takes a **single** data frame and a full set of left/right
#' parameters, delegates to \code{\link{HeatmapAtomic}} (with
#' \code{return_ht = TRUE}) to obtain prepared \code{ComplexHeatmap::Heatmap}
#' objects, extracts exact dimension information, and then assembles
#' everything into a composite \code{grid} layout.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{Parameter resolution} — extra arguments passed via
#'         \code{...} are split into left-specific and right-specific groups
#'         by prefix (\code{left_} vs \code{right_}).  Each group is merged
#'         with the shared defaults using \code{\%||\%} fallback chains,
#'         giving the caller full per-side override capability.
#'   \item \strong{Cell dimension pre-computation} — the internal helper
#'         \code{.get_dim_pre()} calculates exact cell width and height
#'         (in \code{"inches"} units) from \code{cell_type},
#'         \code{aspect.ratio}, \code{base_size}, and the unique row/column
#'         counts (including split groups).  These are passed as explicit
#'         \code{width} / \code{height} arguments to
#'         \code{ComplexHeatmap::Heatmap()} so that cells have a guaranteed
#'         physical size rather than using \code{null} units.
#'   \item \strong{Delegation to \code{HeatmapAtomic}} — both the left and
#'         right heatmaps are created by calling
#'         \code{do_call(HeatmapAtomic, ...)} with \code{return_ht = TRUE}.
#'         This returns the prepared \code{Heatmap} object (with
#'         \code{cell_w}/\code{cell_h} attributes) without drawing it.
#'   \item \strong{Dimension extraction} — exact component heights and widths
#'         are read from the prepared \code{Heatmap} objects via
#'         \code{ComplexHeatmap:::component_height()} and
#'         \code{ComplexHeatmap:::component_width()}.  These 9-element vectors
#'         break down every part of the heatmap (title, dendrogram, names,
#'         annotations, body) so the composite layout can be sized precisely.
#'   \item \strong{Link table construction} — rows in the left and right
#'         heatmaps are matched by grouping the input data on the row
#'         identifier columns (plus optional \code{rows_split_by}).  Each
#'         link's position is mapped to the ordered row index in its
#'         respective heatmap (respecting clustering order).
#'   \item \strong{Legend collection} — legends from both heatmaps are
#'         collected, packed via \code{ComplexHeatmap::packLegend()}, and
#'         placed according to \code{legend.position} (left, right, top, or
#'         bottom).
#'   \item \strong{Composite drawing} — a \code{grid.layout} is built
#'         dynamically based on the legend position.  Each heatmap is drawn
#'         inside a centered, exact-size viewport so
#'         \code{ComplexHeatmap::draw()} sizes the body to the pre-computed
#'         row/column counts.  Link lines are drawn as \code{grid.xspline()}
#'         curves in a dedicated gap column between the two heatmaps.
#'   \item \strong{Dimension attributes} — the final \code{patchwork}-wrapped
#'         grob carries \code{height} and \code{width} attributes (in inches)
#'         for consistent rendering in downstream layouts.
#' }
#'
#' @param data A data frame in long format. Each row represents one
#'  observation; columns specify row/column membership for both left and
#'  right heatmaps as well as the values to encode as color.
#' @param left_values_by,right_values_by Column name whose values determine
#'  the fill color of cells in the left / right heatmap.  These are the
#'  primary data columns visualized by the colour scale.
#' @param left_rows_by,right_rows_by Column name that defines the rows of
#'  the left / right heatmap.  Each unique value becomes one row.
#' @param left_columns_by,right_columns_by Column name that defines the
#'  columns of the left / right heatmap.  Each unique value becomes one
#'  column.
#' @param left_columns_split_by,right_columns_split_by Optional column name
#'  to split the columns of the left / right heatmap into groups (passed to
#'  \code{ComplexHeatmap::Heatmap} as \code{column_split}).
#' @param left_pie_group_by,right_pie_group_by When \code{cell_type = "pie"},
#'  column(s) to group values by within each pie cell for the left / right
#'  heatmap.  Multiple columns are concatenated.
#' @param rows_split_by Optional column name to split the rows of **both**
#'  heatmaps into groups (passed as \code{row_split}).  When provided, row
#'  names in the link table are prefixed with the split level to
#'  disambiguate rows across splits.
#' @param values_fill A value used to fill missing cells in the matrix
#'  (passed to \code{HeatmapAtomic}).  Default is \code{NA} (cells with no
#'  data are left empty).
#'
#' @param palette A character string naming a palette (see
#'  \code{\link{show_palettes}}) or a character vector of colours for the
#'  main heatmap colour scale.  Default \code{"RdBu"}.  Applied to both
#'  heatmaps unless overridden per-side via \code{...}.
#' @param palcolor A custom colour vector that overrides \code{palette} for
#'  the main heatmap colour scale.  Applied to both heatmaps unless
#'  overridden per-side.
#' @param palreverse Logical; if \code{TRUE}, reverse the palette direction.
#'
#' @param pie_size_name Legend title for the pie size when
#'  \code{cell_type = "pie"}.
#' @param pie_size A numeric value or function returning the pie radius.
#'  When a function, it receives the count of groups in the pie and should
#'  return a radius.
#' @param pie_values A function or string (convertible via
#'  \code{\link[base]{match.arg}}) to compute the value represented by each
#'  pie slice.  Default \code{"length"} counts observations per group.
#' @param pie_palette,pie_palcolor Palette and custom colours for pie slice
#'  fill colours.
#'
#' @param bars_sample Fraction of each cell's observations (0 < x <= 1;
#'  \code{1} uses all data) or a whole count > 1 sampled per cell when
#'  \code{cell_type = "bars"}.  Column widths are proportional to the
#'  number of bars drawn per column.  Default 1 (all data).
#'
#' @param label A function to compute text labels when
#'  \code{cell_type = "label"} (or \code{"label+mark"}).  Receives the
#'  aggregated value for a cell and optionally row/column indices and names.
#'  See \code{\link{HeatmapAtomic}} for the full dispatch contract.
#' @param label_size Default point size for label text (used as fallback
#'  when the \code{label} function does not return a \code{size} field).
#' @param label_color Default colour for label text (used as fallback when
#'  the \code{label} function does not return a \code{color} field).
#' @param label_name Legend title for the label colour scale.
#'
#' @param mark A function to compute mark symbols when
#'  \code{cell_type = "mark"} (or \code{"label+mark"}).  Same dispatch
#'  contract as \code{label}.  See \code{\link{HeatmapAtomic}} for supported
#'  mark types.
#' @param mark_color Default mark colour (fallback).
#' @param mark_size Default mark stroke width in pt (fallback).
#' @param mark_name Legend title for the mark colour scale.
#'
#' @param violin_fill A character vector of colours to use as fill for
#'  violin plots when \code{cell_type = "violin"}.  If \code{NULL}, the
#'  annotation colour is used.
#' @param boxplot_fill A character vector of colours to use as fill for
#'  boxplots when \code{cell_type = "boxplot"}.  If \code{NULL}, the
#'  annotation colour is used.
#' @param dot_size Dot size when \code{cell_type = "dot"}.  Can be a
#'  numeric value or a function.
#' @param dot_size_name Legend title for the dot size.
#'
#' @param legend_items A named numeric vector specifying custom legend
#'  entries for the main colour scale.  Names become the displayed labels.
#' @param legend_discrete Logical; if \code{TRUE}, treat the main colour
#'  scale as discrete.
#' @param legend.position A character string specifying where to place the
#'  combined legend: \code{"right"} (default), \code{"left"}, \code{"top"},
#'  \code{"bottom"}, or \code{"none"}.
#' @param legend.direction Legend stacking direction:
#'  \code{"vertical"} (default) or \code{"horizontal"}.
#'
#' @param lower_quantile,upper_quantile Quantiles used for clipping the
#'  colour scale when \code{lower_cutoff} / \code{upper_cutoff} are
#'  \code{NULL}.  Defaults are 0 and 0.99 respectively.
#' @param lower_cutoff,upper_cutoff Explicit cutoffs for the colour scale.
#'  Values outside the range are clamped (winsorized).  Override
#'  \code{lower_quantile} / \code{upper_quantile} when set.
#'
#' @param add_bg Logical; if \code{TRUE}, add a background fill behind
#'  non-tile cell types.  Not used for \code{cell_type = "tile"} or
#'  \code{"bars"}.
#' @param bg_alpha Numeric in \eqn{[0, 1]} for background transparency.
#' @param keep_na,keep_empty Passed through to \code{HeatmapAtomic}.
#'  See \code{\link{common_args}} for details.
#'
#' @param add_reticle Logical; if \code{TRUE}, draw a reticle (crosshair
#'  pattern) over the heatmap.
#' @param reticle_color Colour for the reticle lines.
#'
#' @param left_cluster_rows,right_cluster_rows Logical controlling whether
#'  rows are clustered in the left / right heatmap.  \code{NULL} (default)
#'  lets \code{HeatmapAtomic} decide based on
#'  \code{rows_orderby}/\code{cluster_rows}.
#' @param cluster_columns Logical; cluster columns in both heatmaps.
#'  \code{NULL} lets \code{HeatmapAtomic} decide.
#' @param show_row_names,show_column_names Logical; show row/column names.
#'  May also be a character vector of display modes; see
#'  \code{\link{Heatmap}}.
#' @param border Logical; draw a border around each heatmap.  Default
#'  \code{TRUE}.
#' @param title A character string for the overall plot title.  A function
#'  can be used to generate a dynamic title from the default.
#'  Note that, `left_title` and `right_title` are used to set the title for each heatmap,
#'  and `title` is used to set the overall title for the combined plot.
#' @param title_params A list of parameters passed to \code{grid::grid.text()} to control the title appearance.
#'  Default is \code{list(gp = gpar(fontsize = 14, fontface = "bold"))}.
#' @param column_title,row_title Character title displayed above the columns
#'  / beside the rows of each heatmap.  May also be a character vector of
#'  display modes; see \code{\link{Heatmap}}.
#' @param na_col Colour used for \code{NA} cells.  Default \code{"grey85"}.
#' @param column_names_side Side for column names.  Default
#'  \code{"bottom"}.
#'
#' @param column_annotation A structured list specifying column annotations.
#'  See \code{\link{HeatmapAtomic}} for the full specification.
#' @param column_annotation_side \strong{Deprecated}: use
#'  \code{column_annotation} with the \code{side} sub-key instead.
#' @param column_annotation_palette \strong{Deprecated}: use
#'  \code{column_annotation} with the \code{palette} sub-key instead.
#' @param column_annotation_palcolor \strong{Deprecated}: use
#'  \code{column_annotation} with the \code{palcolor} sub-key instead.
#' @param column_annotation_type \strong{Deprecated}: use
#'  \code{column_annotation} with the \code{type} sub-key instead.
#' @param column_annotation_params \strong{Deprecated}: use
#'  \code{column_annotation} with the \code{params} sub-key instead.
#' @param column_annotation_agg \strong{Deprecated}: use
#'  \code{column_annotation} with the \code{agg} sub-key instead.
#'
#' @param row_annotation A structured list specifying row annotations.
#'  See \code{\link{HeatmapAtomic}} for the full specification.
#' @param row_annotation_side \strong{Deprecated}: use
#'  \code{row_annotation} with the \code{side} sub-key instead.
#' @param row_annotation_palette \strong{Deprecated}: use
#'  \code{row_annotation} with the \code{palette} sub-key instead.
#' @param row_annotation_palcolor \strong{Deprecated}: use
#'  \code{row_annotation} with the \code{palcolor} sub-key instead.
#' @param row_annotation_type \strong{Deprecated}: use
#'  \code{row_annotation} with the \code{type} sub-key instead.
#' @param row_annotation_params \strong{Deprecated}: use
#'  \code{row_annotation} with the \code{params} sub-key instead.
#' @param row_annotation_agg \strong{Deprecated}: use
#'  \code{row_annotation} with the \code{agg} sub-key instead.
#'
#' @param links_span Width (in inches) of the gap column between the two
#'  heatmaps where link curves are drawn.  Default 0.5.
#' @param link_width_by Optional column name in \code{data} whose values
#'  determine the stroke width of each link line (e.g. interaction
#'  strength).  Values are min-max scaled to \eqn{[0, 1]} and multiplied by
#'  \code{link_width_scale}.
#'  You can also pass a numeric value to use a constant width for all links.
#' @param link_width_scale Numeric scaling factor applied to the normalised
#'  link intensity values to produce final line widths (\code{lwd}).
#'  Default 5.
#' @param link_color Colour of the link spline curves.  Default
#'  \code{"grey30"}.
#' @param link_alpha Alpha transparency of link curves in \eqn{[0, 1]}.
#'  Default 0.8.
#'
#' @param alpha Alpha transparency for heatmap cells in \eqn{[0, 1]}.
#' @param seed Random seed for reproducibility.  Default 8525.
#' @param padding Padding around the heatmap in CSS order (top, right,
#'  bottom, left).  Supports 1–4 values.  Default 15 (mm).
#' @param base_size A positive numeric scalar used as a scaling factor for
#'  the overall heatmap size.  Default 1 (no scaling).  Values > 1 enlarge
#'  all cell dimensions proportionally.
#' @param aspect.ratio Height-to-width ratio of a single heatmap cell.
#'  When \code{NULL} (default), sensible defaults are chosen per
#'  \code{cell_type} (e.g. 1 for tiles, 0.5 for bars, 2 for violins).
#' @param draw_opts A named list of additional arguments passed to
#'  \code{\link[ComplexHeatmap]{draw,HeatmapList-method}}.  Internally
#'  managed arguments (\code{padding}, \code{show_heatmap_legend}, etc.)
#'  take precedence.
#' @param layer_fun_callback A function to add custom graphical layers on
#'  top of each heatmap cell.  Receives \code{j}, \code{i}, \code{x},
#'  \code{y}, \code{w}, \code{h}, \code{fill}, \code{sr}, \code{sc}.
#'  See \code{\link[ComplexHeatmap]{Heatmap}} for details.
#' @param cell_type The type of cell to render.  One of \code{"tile"}
#'  (default), \code{"bars"}, \code{"label"}, \code{"mark"},
#'  \code{"label+mark"} (or \code{"mark+label"}), \code{"dot"},
#'  \code{"violin"}, \code{"boxplot"}, \code{"pie"}.  Different cell types
#'  use different \code{cell_fun} / \code{layer_fun} implementations.
#' @param cell_agg A function to aggregate values within each cell when
#'  \code{cell_type = "tile"} or \code{"label"}.  Default is
#'  \code{\link[base]{mean}}.
#' @param ... Additional arguments passed to
#'  \code{\link[ComplexHeatmap]{Heatmap}}.  Arguments prefixed with
#'  \code{left_} or \code{right_} are routed to the corresponding heatmap
#'  (the prefix is stripped before passing).  Unprefixed arguments are
#'  passed to both heatmaps but can be overridden by a prefixed version.
#'  Notable examples:
#'  \itemize{
#'    \item \code{left_name}, \code{right_name} — legend titles for the
#'          left / right colour scales.
#'    \item \code{left_row_gap}, \code{right_row_gap} — per-side row gaps.
#'    \item \code{left_column_gap}, \code{right_column_gap} — per-side
#'          column gaps.
#'    \item \code{row_names_max_width}, \code{column_names_max_height} —
#'          passed through to \code{ComplexHeatmap::Heatmap}.
#'  }
#'
#' @return A \code{patchwork}-wrapped grob (class \code{wrap_plots}) with
#'  \code{height} and \code{width} attributes (in inches).  The original
#'  \code{data} is stored in \code{p$data}.
#' @keywords internal
#' @importFrom grid pushViewport popViewport grid.layout grid.xspline
#' @importFrom grid unit convertUnit grid.grabExpr grid.newpage
LinkedHeatmapAtomic <- function(
    data,
    left_values_by,
    right_values_by,
    left_rows_by,
    right_rows_by,
    left_columns_by,
    right_columns_by,
    left_columns_split_by = NULL,
    right_columns_split_by = NULL,
    left_pie_group_by = NULL,
    right_pie_group_by = NULL,

    rows_split_by = NULL,
    values_fill = NA,

    # palettes
    palette = "RdBu",
    palcolor = NULL,
    palreverse = FALSE,

    # cell_type: pies
    pie_size_name = "size",
    pie_size = NULL,
    pie_values = "length",
    pie_palette = "Spectral",
    pie_palcolor = NULL,

    # cell_type: bars
    bars_sample = 1,
    # cell_type: label
    label = identity,
    label_size = 10,
    label_color = "black",
    label_name = "label",
    # cell_type: mark
    mark = identity,
    mark_color = "black",
    mark_size = 1,
    mark_name = "mark",
    # cell_type: violin
    violin_fill = NULL,
    # cell_type: boxplot
    boxplot_fill = NULL,
    # cell_type: dot
    dot_size = 8,
    dot_size_name = "size",
    # legend
    legend_items = NULL,
    legend_discrete = FALSE,
    legend.position = "right",
    legend.direction = "vertical",
    # values
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    # bg
    add_bg = FALSE,
    bg_alpha = 0.5,
    keep_na = FALSE,
    keep_empty = FALSE,
    # reticle
    add_reticle = FALSE,
    reticle_color = "grey",

    left_cluster_rows = NULL,
    right_cluster_rows = NULL,
    cluster_columns = NULL,
    show_row_names = NULL,
    show_column_names = NULL,
    border = TRUE,
    title = NULL,
    title_params = NULL,
    column_title = NULL,
    row_title = NULL,
    na_col = "grey85",
    column_names_side = "bottom",
    row_annotation = NULL,
    row_annotation_side = NULL,
    row_annotation_palette = NULL,
    row_annotation_palcolor = NULL,
    row_annotation_type = NULL,
    row_annotation_params = NULL,
    row_annotation_agg = NULL,
    column_annotation = NULL,
    column_annotation_side = NULL,
    column_annotation_palette = NULL,
    column_annotation_palcolor = NULL,
    column_annotation_type = NULL,
    column_annotation_params = NULL,
    column_annotation_agg = NULL,

    links_span = 0.5,
    link_width_by = NULL,
    link_width_scale = 5,
    link_color = "grey30",
    link_alpha = 0.8,

    alpha = 1,
    seed = 8525,
    padding = 15,
    base_size = 1,
    aspect.ratio = NULL,
    draw_opts = list(),
    # cell customization
    layer_fun_callback = NULL,
    cell_type = c(
        "tile",
        "bars",
        "label",
        "mark",
        "label+mark",
        "mark+label",
        "dot",
        "violin",
        "boxplot",
        "pie"
    ),
    cell_agg = NULL,
    ...
) {
    set.seed(seed)

    rest_args <- list(...)
    # The arguments in rest_args that don't begin with "right_" prefix
    left_args <- rest_args[!grepl("^right_", names(rest_args))]
    # Replace all left_ prefix and then remove the duplicate arguments that have
    # been passed to left_args
    # The duplicated ones to remove are the ones without `left_` prefix
    # For example, `left_row_names_side` and `row_names_side`, the one
    # to be removed is `row_names_side`
    left_names_with_prefix <- names(left_args)[grepl(
        "^left_",
        names(left_args)
    )]
    left_names_without_prefix <- gsub("^left_", "", left_names_with_prefix)
    left_args <- left_args[!names(left_args) %in% left_names_without_prefix]
    names(left_args) <- gsub("^left_", "", names(left_args))

    right_args <- rest_args[!grepl("^left_", names(rest_args))]
    right_names_with_prefix <- names(right_args)[grepl(
        "^right_",
        names(right_args)
    )]
    right_names_without_prefix <- gsub("^right_", "", right_names_with_prefix)
    right_args <- right_args[!names(right_args) %in% right_names_without_prefix]
    names(right_args) <- gsub("^right_", "", names(right_args))

    # ── Resolve left/right parameters ──
    left_args$data <- data
    left_args$values_by <- left_values_by
    left_args$values_fill <- left_args$values_fill %||% values_fill
    left_args$rows_by <- left_rows_by
    left_args$rows_split_by <- rows_split_by
    left_args$columns_by <- left_columns_by
    left_args$columns_split_by <- left_columns_split_by
    left_args$palette <- left_args$palette %||% palette
    left_args$palcolor <- left_args$palcolor %||% palcolor
    left_args$palreverse <- left_args$palreverse %||% palreverse
    left_args$pie_size_name <- left_args$pie_size_name %||% pie_size_name
    left_args$pie_size <- left_args$pie_size %||% pie_size
    left_args$pie_values <- left_args$pie_values %||% pie_values
    left_args$pie_group_by <- left_pie_group_by
    left_args$pie_palette <- left_args$pie_palette %||% pie_palette
    left_args$pie_palcolor <- left_args$pie_palcolor %||% pie_palcolor
    left_args$bars_sample <- left_args$bars_sample %||% bars_sample
    left_args$label <- left_args$label %||% label
    left_args$label_size <- left_args$label_size %||% label_size
    left_args$label_color <- left_args$label_color %||% label_color
    left_args$label_name <- left_args$label_name %||% label_name
    left_args$mark <- left_args$mark %||% mark
    left_args$mark_color <- left_args$mark_color %||% mark_color
    left_args$mark_size <- left_args$mark_size %||% mark_size
    left_args$mark_name <- left_args$mark_name %||% mark_name
    left_args$violin_fill <- left_args$violin_fill %||% violin_fill
    left_args$boxplot_fill <- left_args$boxplot_fill %||% boxplot_fill
    left_args$dot_size <- left_args$dot_size %||% dot_size
    left_args$dot_size_name <- left_args$dot_size_name %||% dot_size_name
    left_args$legend_items <- left_args$legend_items %||% legend_items
    left_args$legend_discrete <- left_args$legend_discrete %||% legend_discrete
    left_args$legend.position <- "right"
    left_args$legend.direction <- left_args$legend.direction %||%
        legend.direction
    left_args$lower_quantile <- left_args$lower_quantile %||% lower_quantile
    left_args$upper_quantile <- left_args$upper_quantile %||% upper_quantile
    left_args$lower_cutoff <- left_args$lower_cutoff %||% lower_cutoff
    left_args$upper_cutoff <- left_args$upper_cutoff %||% upper_cutoff
    left_args$add_bg <- left_args$add_bg %||% add_bg
    left_args$bg_alpha <- left_args$bg_alpha %||% bg_alpha
    left_args$add_reticle <- left_args$add_reticle %||% add_reticle
    left_args$reticle_color <- left_args$reticle_color %||% reticle_color
    left_args$cluster_columns <- left_args$cluster_columns %||% cluster_columns
    left_args$cluster_rows <- left_cluster_rows
    left_args$show_row_names <- left_args$show_row_names %||% show_row_names
    left_args$show_column_names <- left_args$show_column_names %||%
        show_column_names
    left_args$border <- left_args$border %||% border
    left_args$column_title <- left_args$column_title %||% column_title
    left_args$row_title <- left_args$row_title %||% row_title
    left_args$na_col <- left_args$na_col %||% na_col
    left_args$column_names_side <- left_args$column_names_side %||%
        column_names_side
    left_args$row_annotation <- left_args[["row_annotation"]] %||%
        row_annotation
    left_args$row_annotation_side <- left_args$row_annotation_side %||%
        row_annotation_side
    left_args$row_annotation_palette <- left_args$row_annotation_palette %||%
        row_annotation_palette
    left_args$row_annotation_palcolor <- left_args$row_annotation_palcolor %||%
        row_annotation_palcolor
    left_args$row_annotation_type <- left_args$row_annotation_type %||%
        row_annotation_type
    left_args$row_annotation_params <- left_args$row_annotation_params %||%
        row_annotation_params
    left_args$row_annotation_agg <- left_args$row_annotation_agg %||%
        row_annotation_agg
    left_args$column_annotation <- left_args[["column_annotation"]] %||%
        column_annotation
    left_args$column_annotation_side <- left_args$column_annotation_side %||%
        column_annotation_side
    left_args$column_annotation_palette <- left_args$column_annotation_palette %||%
        column_annotation_palette
    left_args$column_annotation_palcolor <- left_args$column_annotation_palcolor %||%
        column_annotation_palcolor
    left_args$column_annotation_type <- left_args$column_annotation_type %||%
        column_annotation_type
    left_args$column_annotation_params <- left_args$column_annotation_params %||%
        column_annotation_params
    left_args$column_annotation_agg <- left_args$column_annotation_agg %||%
        column_annotation_agg
    # flip
    left_args$alpha <- left_args$alpha %||% alpha
    left_args$padding <- left_args$padding %||% padding
    left_args$base_size <- left_args$base_size %||% base_size
    left_args$aspect.ratio <- left_args$aspect.ratio %||% aspect.ratio
    left_args$draw_opts <- left_args$draw_opts %||% draw_opts
    left_args$layer_fun_callback <- left_args$layer_fun_callback %||%
        layer_fun_callback
    left_args$cell_type <- left_args$cell_type %||% cell_type
    left_args$cell_agg <- left_args$cell_agg %||% cell_agg
    left_args$return_ht <- TRUE

    right_args$data <- data
    right_args$values_by <- right_values_by
    right_args$values_fill <- right_args$values_fill %||% values_fill
    right_args$rows_by <- right_rows_by
    right_args$rows_split_by <- rows_split_by
    right_args$columns_by <- right_columns_by
    right_args$columns_split_by <- right_columns_split_by
    right_args$palette <- right_args$palette %||% palette
    right_args$palcolor <- right_args$palcolor %||% palcolor
    right_args$palreverse <- right_args$palreverse %||% palreverse
    right_args$pie_size_name <- right_args$pie_size_name %||% pie_size_name
    right_args$pie_size <- right_args$pie_size %||% pie_size
    right_args$pie_values <- right_args$pie_values %||% pie_values
    right_args$pie_group_by <- right_pie_group_by
    right_args$pie_palette <- right_args$pie_palette %||% pie_palette
    right_args$pie_palcolor <- right_args$pie_palcolor %||% pie_palcolor
    right_args$bars_sample <- right_args$bars_sample %||% bars_sample
    right_args$label <- right_args$label %||% label
    right_args$label_size <- right_args$label_size %||% label_size
    right_args$label_color <- right_args$label_color %||% label_color
    right_args$label_name <- right_args$label_name %||% label_name
    right_args$mark <- right_args$mark %||% mark
    right_args$mark_color <- right_args$mark_color %||% mark_color
    right_args$mark_size <- right_args$mark_size %||% mark_size
    right_args$mark_name <- right_args$mark_name %||% mark_name
    right_args$violin_fill <- right_args$violin_fill %||% violin_fill
    right_args$boxplot_fill <- right_args$boxplot_fill %||% boxplot_fill
    right_args$dot_size <- right_args$dot_size %||% dot_size
    right_args$dot_size_name <- right_args$dot_size_name %||% dot_size_name
    right_args$legend_items <- right_args$legend_items %||% legend_items
    right_args$legend_discrete <- right_args$legend_discrete %||%
        legend_discrete
    right_args$legend.position <- "right"
    right_args$legend.direction <- right_args$legend.direction %||%
        legend.direction
    right_args$lower_quantile <- right_args$lower_quantile %||% lower_quantile
    right_args$upper_quantile <- right_args$upper_quantile %||% upper_quantile
    right_args$lower_cutoff <- right_args$lower_cutoff %||% lower_cutoff
    right_args$upper_cutoff <- right_args$upper_cutoff %||% upper_cutoff
    right_args$add_bg <- right_args$add_bg %||% add_bg
    right_args$bg_alpha <- right_args$bg_alpha %||% bg_alpha
    right_args$add_reticle <- right_args$add_reticle %||% add_reticle
    right_args$reticle_color <- right_args$reticle_color %||% reticle_color
    right_args$cluster_columns <- right_args$cluster_columns %||%
        cluster_columns
    right_args$cluster_rows <- right_cluster_rows
    right_args$show_row_names <- right_args$show_row_names %||% show_row_names
    right_args$show_column_names <- right_args$show_column_names %||%
        show_column_names
    right_args$border <- right_args$border %||% border
    right_args$column_title <- right_args$column_title %||% column_title
    right_args$row_title <- right_args$row_title %||% row_title
    right_args$na_col <- right_args$na_col %||% na_col
    right_args$column_names_side <- right_args$column_names_side %||%
        column_names_side
    right_args$row_annotation <- right_args[["row_annotation"]] %||%
        row_annotation
    right_args$row_annotation_side <- right_args$row_annotation_side %||%
        row_annotation_side
    right_args$row_annotation_palette <- right_args$row_annotation_palette %||%
        row_annotation_palette
    right_args$row_annotation_palcolor <- right_args$row_annotation_palcolor %||%
        row_annotation_palcolor
    right_args$row_annotation_type <- right_args$row_annotation_type %||%
        row_annotation_type
    right_args$row_annotation_params <- right_args$row_annotation_params %||%
        row_annotation_params
    right_args$row_annotation_agg <- right_args$row_annotation_agg %||%
        row_annotation_agg
    right_args$column_annotation <- right_args[["column_annotation"]] %||%
        column_annotation
    right_args$column_annotation_side <- right_args$column_annotation_side %||%
        column_annotation_side
    right_args$column_annotation_palette <- right_args$column_annotation_palette %||%
        column_annotation_palette
    right_args$column_annotation_palcolor <- right_args$column_annotation_palcolor %||%
        column_annotation_palcolor
    right_args$column_annotation_type <- right_args$column_annotation_type %||%
        column_annotation_type
    right_args$column_annotation_params <- right_args$column_annotation_params %||%
        column_annotation_params
    right_args$column_annotation_agg <- right_args$column_annotation_agg %||%
        column_annotation_agg
    # flip
    right_args$alpha <- right_args$alpha %||% alpha
    right_args$padding <- right_args$padding %||% padding
    right_args$base_size <- right_args$base_size %||% base_size
    right_args$aspect.ratio <- right_args$aspect.ratio %||% aspect.ratio
    right_args$draw_opts <- right_args$draw_opts %||% draw_opts
    right_args$layer_fun_callback <- right_args$layer_fun_callback %||%
        layer_fun_callback
    right_args$cell_type <- right_args$cell_type %||% cell_type
    right_args$cell_agg <- right_args$cell_agg %||% cell_agg
    right_args$return_ht <- TRUE

    # ── Pre-compute cell dimensions (same formula as HeatmapAtomic) ──
    # Pass explicit width/height to guarantee exact cell sizing.
    # Without this, ComplexHeatmap uses null units and cells fill
    # whatever space is available in the viewport.
    .get_dim_pre <- function(args) {
        ct <- sub("mark+label", "label+mark", args$cell_type, fixed = TRUE)
        cell_w <- switch(
            ct,
            violin = 0.5,
            boxplot = 0.5,
            pie = 0.5,
            bars = 0.35,
            label = 0.6,
            mark = 0.25,
            `label+mark` = 0.6,
            0.25
        )
        aspect_default <- switch(
            ct,
            violin = 2,
            boxplot = 2,
            pie = 1,
            bars = 0.5,
            label = 0.6,
            mark = 1,
            `label+mark` = 0.6,
            1
        )

        bs <- args$base_size %||% base_size %||% 1
        ar <- args$aspect.ratio %||% aspect_default
        cell_w_pre <- cell_w * bs
        cell_h_pre <- cell_w_pre * ar

        if (!is.null(args$rows_split_by)) {
            n_row_splits <- length(unique(args$data[[args$rows_split_by]]))
            n_rows <- args$data[,
                c(args$rows_by, args$rows_split_by),
                drop = FALSE
            ] %>%
                dplyr::n_distinct()
        } else {
            n_row_splits <- 1
            n_rows <- args$data[[args$rows_by]] %>% dplyr::n_distinct()
        }

        if (!is.null(args$columns_split_by)) {
            n_col_splits <- length(unique(args$data[[args$columns_split_by]]))
            n_cols <- args$data[,
                c(args$columns_by, args$columns_split_by),
                drop = FALSE
            ] %>%
                dplyr::n_distinct()
        } else {
            n_col_splits <- 1
            n_cols <- args$data[[args$columns_by]] %>% dplyr::n_distinct()
        }

        row_gap <- args$row_gap %||%
            convertUnit(unit(1, "mm"), "inches", valueOnly = TRUE)
        height <- unit(
            n_rows * cell_h_pre + (n_row_splits - 1) * row_gap,
            "inches"
        )

        column_gap <- args$column_gap %||%
            convertUnit(unit(1, "mm"), "inches", valueOnly = TRUE)
        width <- unit(
            n_cols * cell_w_pre + (n_col_splits - 1) * column_gap,
            "inches"
        )

        list(width = width, height = height)
    }
    left_dim_pre <- .get_dim_pre(left_args)
    right_dim_pre <- .get_dim_pre(right_args)

    left_args$width <- left_dim_pre$width
    left_args$height <- left_dim_pre$height
    right_args$width <- right_dim_pre$width
    right_args$height <- right_dim_pre$height

    left_ht <- do_call(HeatmapAtomic, left_args)
    right_ht <- do_call(HeatmapAtomic, right_args)

    n_left_rows <- nrow(left_ht@matrix)
    n_left_cols <- ncol(left_ht@matrix)
    n_right_rows <- nrow(right_ht@matrix)
    n_right_cols <- ncol(right_ht@matrix)

    # Get the order of the rows in the left heatmap after clustering (if any)
    left_row_order <- left_ht@row_order
    right_row_order <- right_ht@row_order

    # ── Extract exact dimensions from ComplexHeatmap prepared objects ──
    # component_height() returns a 9-element list (all internal gaps included):
    #   [1] column_title_top    [2] column_dend_top    [3] column_names_top
    #   [4] column_anno_top     [5] heatmap_body       [6] column_anno_bot
    #   [7] column_names_bot    [8] column_dend_bot    [9] column_title_bot
    # component_width() returns a 9-element list:
    #   [1] row_title_left      [2] row_dend_left      [3] row_names_left
    #   [4] row_anno_left       [5] heatmap_body       [6] row_anno_right
    #   [7] row_names_right     [8] row_dend_right     [9] row_title_right
    .ch_to_in <- function(comp_list) {
        vapply(
            comp_list,
            function(u) {
                if (is.null(u)) {
                    return(0)
                }
                convertUnit(u, "inches", valueOnly = TRUE)
            },
            numeric(1)
        )
    }

    component_height <- utils::getFromNamespace(
        "component_height",
        "ComplexHeatmap"
    )
    component_width <- utils::getFromNamespace(
        "component_width",
        "ComplexHeatmap"
    )
    left_ch <- .ch_to_in(component_height(left_ht))
    right_ch <- .ch_to_in(component_height(right_ht))
    left_cw <- .ch_to_in(component_width(left_ht))
    right_cw <- .ch_to_in(component_width(right_ht))

    # Body dimensions (component [5] is the body)
    left_body_h <- left_ch[5]
    right_body_h <- right_ch[5]
    left_body_w <- left_cw[5]
    right_body_w <- right_cw[5]

    # body_top_offset = sum of all components ABOVE the body [1:4]
    body_top_offset_left <- sum(left_ch[1:4])
    body_top_offset_right <- sum(right_ch[1:4])

    # below_h = sum of all components BELOW the body [6:9]
    left_below_h <- sum(left_ch[6:9])
    right_below_h <- sum(right_ch[6:9])

    # ── Estimate draw-time column_title heights ──
    # column_title passed to ComplexHeatmap::draw() is a LIST-level title
    # that sits ABOVE the components already captured by component_height().
    # We must add its height to body_top_offset & total_h so body NPC
    # positions remain correct.
    .est_title_h <- function(title_text) {
        if (is.null(title_text) || identical(title_text, "")) return(0)
        gp <- ComplexHeatmap::ht_opt("heatmap_column_title_gp") %||%
            gpar(fontsize = 14, fontface = "bold")
        txt_h <- grid::convertHeight(
            grid::grobHeight(grid::textGrob(title_text, gp = gp)),
            "inches", valueOnly = TRUE
        )
        # ~2.5mm above + below is the internal padding ComplexHeatmap
        # applies to column titles (no separate ht_opt key for it).
        pad_h <- grid::convertUnit(unit(5, "mm"), "inches", valueOnly = TRUE)
        txt_h + pad_h
    }
    left_title_h <- .est_title_h(left_args$title)
    right_title_h <- .est_title_h(right_args$title)

    body_top_offset_left  <- body_top_offset_left  + left_title_h
    body_top_offset_right <- body_top_offset_right + right_title_h

    # Total heatmap dimensions
    left_total_w <- sum(left_cw)
    left_total_h <- sum(left_ch) + left_title_h
    right_total_w <- sum(right_cw)
    right_total_h <- sum(right_ch) + right_title_h

    total_w <- left_total_w + links_span + right_total_w
    total_h <- max(left_total_h, right_total_h)

    # ── Legend flag (needed later) ──
    legend_gap <- 0.15
    show_legend <- !identical(legend.position, "none")

    # ── Build link table ──
    left_row_names_ordered <- rownames(left_ht@matrix)[left_row_order]
    right_row_names_ordered <- rownames(right_ht@matrix)[right_row_order]

    link_table <- data %>%
        dplyr::group_by(
            !!!syms(unique(c(left_rows_by, right_rows_by, rows_split_by)))
        )

    if (is.character(link_width_by)) {
        link_table <- link_table %>%
            dplyr::summarise(
                !!sym(link_width_by) := mean(
                    !!sym(link_width_by),
                    na.rm = TRUE
                ),
                .groups = "drop"
            )
    } else {
        link_width_by <- link_width_by %||% 1
        link_table <- link_table %>%
            dplyr::summarise(.link_width = link_width_by, .groups = "drop")
        link_width_by <- ".link_width"
    }

    if (!is.null(rows_split_by)) {
        link_table <- link_table %>%
            dplyr::mutate(
                left_rows = paste0(
                    !!sym(rows_split_by),
                    " // ",
                    !!sym(left_rows_by)
                ),
                right_rows = paste0(
                    !!sym(rows_split_by),
                    " // ",
                    !!sym(right_rows_by)
                )
            )
    } else {
        link_table <- link_table %>%
            dplyr::mutate(
                left_rows = !!sym(left_rows_by),
                right_rows = !!sym(right_rows_by)
            )
    }
    link_table <- link_table %>%
        mutate(
            pos_left = match(
                as.character(!!sym("left_rows")),
                left_row_names_ordered
            ),
            pos_right = match(
                as.character(!!sym("right_rows")),
                right_row_names_ordered
            )
        ) %>%
        filter(!is.na(.data$pos_left), !is.na(.data$pos_right))

    raw_intensity <- link_table[[link_width_by]]
    if (max(raw_intensity, na.rm = TRUE) > min(raw_intensity, na.rm = TRUE)) {
        link_table$intensity <- (raw_intensity -
            min(raw_intensity, na.rm = TRUE)) /
            (max(raw_intensity, na.rm = TRUE) -
                min(raw_intensity, na.rm = TRUE))
    } else {
        link_table$intensity <- raw_intensity
    }
    link_table$intensity[is.na(link_table$intensity)] <- 0

    # ── Collect legends (before link position computation so legend
    #     dimensions can be factored into plot_h for top/bottom) ──
    combined_legend <- NULL
    legend_w <- 0
    legend_h <- 0

    if (show_legend) {
        legends <- c(attr(left_ht, "legends"), attr(right_ht, "legends"))

        if (length(legends) > 0) {
            combined_legend <- ComplexHeatmap::packLegend(
                list = legends,
                direction = legend.direction,
                gap = unit(4, "mm")
            )
            # Estimate legend width from label text widths.
            # Discrete legends have @labels slot; continuous (colorRamp2)
            # legends do not — use fixed estimate for those.
            label_widths <- vapply(
                legends,
                function(lgd) {
                    if (
                        methods::.hasSlot(lgd, "labels") &&
                            is.character(lgd@labels) &&
                            length(lgd@labels) > 0
                    ) {
                        convertUnit(
                            ComplexHeatmap::max_text_width(lgd@labels),
                            "inches",
                            valueOnly = TRUE
                        )
                    } else {
                        0.6 # continuous legend: fixed label width
                    }
                },
                numeric(1)
            )
            max_label_w <- max(label_widths, 0.5)
            title_w <- max(
                vapply(
                    legends,
                    function(lgd) {
                        if (
                            methods::.hasSlot(lgd, "title") &&
                                is.character(lgd@title) &&
                                nchar(lgd@title) > 0
                        ) {
                            convertUnit(
                                ComplexHeatmap::max_text_width(lgd@title),
                                "inches",
                                valueOnly = TRUE
                            )
                        } else {
                            0
                        }
                    },
                    numeric(1)
                ),
                0
            )
            legend_w <- max(max_label_w, title_w) + 0.6

            # Estimate legend height:
            # ~0.7 in per legend (title + colour bar + labels + gaps)
            # When vertical they stack; when horizontal single row.
            n_legends <- length(legends)
            mm_gap_in <- convertUnit(unit(4, "mm"), "inches", valueOnly = TRUE)
            if (legend.direction == "vertical") {
                legend_h <- n_legends * 0.7 + (n_legends - 1) * mm_gap_in
            } else {
                legend_h <- 0.7
            }
        }
    }

    # ── Plot dimensions (heatmap body + legend + title) ──
    # Resolve title graphical parameters and height
    if (!is.null(title)) {
        title_params <- title_params %||% list()
        title_params$label <- title
        title_params$x <- title_params$x %||% 0.5
        if (is.numeric(title_params$x)) {
            title_params$x <- unit(title_params$x, "npc")
        }
        title_params$y <- title_params$y %||% 0.5
        if (is.numeric(title_params$y)) {
            title_params$y <- unit(title_params$y, "npc")
        }
        title_params$gp <- title_params$gp %||% gpar()
        title_params$gp$fontsize <- title_params$gp$fontsize %||% 14
        title_params$gp$fontface <- title_params$gp$fontface %||% "bold"
        title_h <- 0.5 # inches reserved for title
    } else {
        title_h <- 0
    }
    plot_w <- total_w
    plot_h <- total_h + title_h
    if (show_legend) {
        plot_w <- total_w + legend_gap + legend_w
        plot_h <- total_h + legend_gap + legend_h + title_h
    }

    # ── Link position computation ──
    # NPC coords use total_h (heatmap-only height) because each heatmap
    # viewport receives that exact height from the grid layout.
    # Each heatmap is drawn inside a centered viewport of its exact native
    # size, so the body top position includes the centering offset:
    #   body_top_px = (total_h - heatmap_total_h) / 2 + body_top_offset

    left_center_offset <- (total_h - left_total_h) / 2
    right_center_offset <- (total_h - right_total_h) / 2

    left_body_top_npc <- 1 -
        (left_center_offset + body_top_offset_left) / total_h
    left_body_bot_npc <- 1 -
        (left_center_offset + body_top_offset_left + left_body_h) / total_h
    left_body_range <- left_body_top_npc - left_body_bot_npc

    right_body_top_npc <- 1 -
        (right_center_offset + body_top_offset_right) / total_h
    right_body_bot_npc <- 1 -
        (right_center_offset + body_top_offset_right + right_body_h) / total_h
    right_body_range <- right_body_top_npc - right_body_bot_npc

    compute_y <- function(pos, n_rows, top_npc, range_npc) {
        top_npc - (pos - 0.5) / n_rows * range_npc
    }

    # ── Draw composite ──
    # Build grid layout dynamically based on legend.position.
    # Heatmaps always occupy one row of 3 columns (left | gap | right).
    # The legend gets an extra column (left/right) or an extra row (top/bottom).
    # When an overall title is provided, a title row is added at the top.
    pos_left <- legend.position == "left"
    pos_right <- legend.position == "right"
    pos_top <- legend.position == "top"
    pos_bottom <- legend.position == "bottom"

    n_hm_cols <- 3L
    hm_col_widths <- unit(c(left_total_w, links_span, right_total_w), "inches")
    hm_col_left <- 1L
    hm_col_gap <- 2L
    hm_col_right <- 3L

    if (show_legend && (pos_left || pos_right)) {
        n_cols <- 4L
        if (pos_left) {
            col_widths <- unit(
                c(
                    legend_w + legend_gap,
                    left_total_w,
                    links_span,
                    right_total_w
                ),
                "inches"
            )
            lg_col <- 1L
            hm_col_left <- 2L
            hm_col_gap <- 3L
            hm_col_right <- 4L
        } else {
            # right
            col_widths <- unit(
                c(
                    left_total_w,
                    links_span,
                    right_total_w,
                    legend_w + legend_gap
                ),
                "inches"
            )
            lg_col <- 4L
        }
        n_rows <- 1L
        row_heights <- unit(total_h, "inches")
        hm_row <- 1L
        lg_row <- 1L
    } else if (show_legend && (pos_top || pos_bottom)) {
        n_cols <- 3L
        col_widths <- hm_col_widths
        n_rows <- 2L
        if (pos_top) {
            row_heights <- unit(c(legend_h + legend_gap, total_h), "inches")
            lg_row <- 1L
            hm_row <- 2L
        } else {
            # bottom
            row_heights <- unit(c(total_h, legend_h + legend_gap), "inches")
            hm_row <- 1L
            lg_row <- 2L
        }
    } else {
        n_cols <- 3L
        col_widths <- hm_col_widths
        n_rows <- 1L
        row_heights <- unit(total_h, "inches")
        hm_row <- 1L
    }

    # Adjust layout for overall title (added as a top row)
    if (!is.null(title)) {
        n_rows <- n_rows + 1L
        row_heights <- unit.c(unit(title_h, "inches"), row_heights)
        hm_row <- hm_row + 1L
        if (show_legend && (pos_left || pos_right || pos_top || pos_bottom)) {
            lg_row <- lg_row + 1L
        }
    }

    p <- grid.grabExpr({
        grid.newpage()
        pushViewport(viewport(
            layout = grid.layout(
                n_rows,
                n_cols,
                widths = col_widths,
                heights = row_heights
            )
        ))

        # Overall title
        if (!is.null(title)) {
            pushViewport(viewport(
                layout.pos.row = 1,
                layout.pos.col = 1:n_cols
            ))
            # TODO: a better way to control title placement and alignment
            do_call(grid.text, title_params)
            popViewport()
        }

        # Left heatmap — centered exact-size viewport so ComplexHeatmap
        # sizes the body to exactly n_left_rows × cell_h
        pushViewport(viewport(
            layout.pos.row = hm_row,
            layout.pos.col = hm_col_left
        ))
        pushViewport(viewport(
            x = unit(0, "npc"),
            y = unit(0.5, "npc"),
            width = unit(left_total_w, "inches"),
            height = unit(left_total_h, "inches"),
            just = c("left", "centre")
        ))
        ComplexHeatmap::draw(
            left_ht,
            column_title = left_args$title,
            newpage = FALSE,
            show_heatmap_legend = FALSE,
            show_annotation_legend = FALSE,
            padding = unit(c(0, 0, 0, 0), "mm")
        )
        popViewport(2)

        # Right heatmap — centered exact-size viewport so ComplexHeatmap
        # sizes the body to exactly n_right_rows × cell_h
        pushViewport(viewport(
            layout.pos.row = hm_row,
            layout.pos.col = hm_col_right
        ))
        pushViewport(viewport(
            x = unit(0, "npc"),
            y = unit(0.5, "npc"),
            width = unit(right_total_w, "inches"),
            height = unit(right_total_h, "inches"),
            just = c("left", "centre")
        ))
        ComplexHeatmap::draw(
            right_ht,
            column_title = right_args$title,
            newpage = FALSE,
            show_heatmap_legend = FALSE,
            show_annotation_legend = FALSE,
            padding = unit(c(0, 0, 0, 0), "mm")
        )
        popViewport(2)

        # Legend
        if (!is.null(combined_legend)) {
            if (pos_left || pos_right) {
                pushViewport(viewport(
                    layout.pos.row = lg_row,
                    layout.pos.col = lg_col
                ))
                ComplexHeatmap::draw(
                    combined_legend,
                    x = unit(0.05, "npc"),
                    just = "left"
                )
            } else {
                # top / bottom: legend spans all heatmap columns
                pushViewport(viewport(
                    layout.pos.row = lg_row,
                    layout.pos.col = 1:3
                ))
                ComplexHeatmap::draw(
                    combined_legend,
                    x = unit(0.5, "npc"),
                    just = "centre"
                )
            }
            popViewport()
        }

        # Links in middle column
        pushViewport(viewport(
            layout.pos.row = hm_row,
            layout.pos.col = hm_col_gap
        ))
        if (nrow(link_table) > 0) {
            for (k in seq_len(nrow(link_table))) {
                y_left <- compute_y(
                    link_table$pos_left[k],
                    n_left_rows,
                    left_body_top_npc,
                    left_body_range
                )
                y_right <- compute_y(
                    link_table$pos_right[k],
                    n_right_rows,
                    right_body_top_npc,
                    right_body_range
                )
                grid.xspline(
                    x = unit(c(0.15, 0.5, 0.85), "npc"),
                    y = unit(
                        c(y_left, mean(c(y_left, y_right)), y_right),
                        "npc"
                    ),
                    shape = 0.5,
                    gp = gpar(
                        lwd = link_table$intensity[k] * link_width_scale,
                        col = link_color,
                        alpha = link_alpha
                    )
                )
            }
        }
        popViewport()
    })

    p <- patchwork::wrap_plots(p)

    # ── Dimension attributes ──
    # Legend dimensions are fixed (don't scale with the heatmap body), so we
    # compute the aspect ratio from the body only, clamp body dimensions
    # (reserving space for the legend), apply ratio correction, then add
    # legend space back to get the final display dimensions.
    min_size_in <- 4
    max_size_in <- 64
    display_h <- max(min(plot_h, max_size_in), min_size_in)
    display_w <- max(min(plot_w, max_size_in), min_size_in)
    ratio <- plot_h / plot_w
    if (ratio > 1 && display_h == max_size_in) {
        display_w <- display_h / ratio
    } else if (ratio < 1 && display_w == max_size_in) {
        display_h <- display_w * ratio
    }
    attr(p, "height") <- display_h + 1
    attr(p, "width") <- display_w + 1

    p$data <- data

    p
}

#' Linked Heatmap
#'
#' @description
#' Draw two heatmaps side-by-side with spline link lines connecting matching
#' rows across the two heatmaps.  This is the public, exported interface for
#' creating linked-heatmap visualisations.
#'
#' A typical use case is visualising ligand–receptor interactions: the
#' left heatmap shows ligand expression (rows = ligands, columns = cell
#' sources), the right heatmap shows receptor expression (rows = receptors,
#' columns = cell targets), and link curves connect each ligand to its
#' cognate receptor(s).
#'
#' @section Left / right specification:
#' Parameters that differ between the two heatmaps are prefixed with
#' \code{left_} or \code{right_}.  Shared parameters (e.g. \code{palette},
#' \code{cell_type}, \code{cluster_columns}) apply to both sides but can be
#' overridden per-side via the \code{...} argument.  Every parameter listed
#' below can also be passed with a \code{left_} or \code{right_} prefix in
#' \code{...} for full per-side control.
#'
#' The \code{...} argument is also forwarded to
#' \code{\link[ComplexHeatmap]{Heatmap}} after prefix-stripping, allowing
#' direct access to \code{ComplexHeatmap} parameters (e.g.
#' \code{left_row_names_gp}, \code{right_column_names_rot}).
#'
#' @section Split-by support:
#' When \code{split_by} is provided, the data is partitioned into subsets
#' and an independent linked-heatmap pair is produced for each level.  The
#' results are combined via \code{\link[patchwork]{wrap_plots}} according to
#' \code{nrow}, \code{ncol}, \code{byrow}, and \code{design}.
#' Per-split \code{palette}, \code{palcolor}, and \code{legend.position} can
#' be specified as named lists keyed by split level.
#'
#' @section Dimension calculation:
#' Cell dimensions are pre-computed from \code{cell_type},
#' \code{aspect.ratio}, \code{base_size}, and the unique row/column counts
#' in the data (accounting for split groups).  These exact dimensions are
#' passed to \code{ComplexHeatmap::Heatmap} so cells have guaranteed
#' physical sizes, ensuring that the two heatmaps' bodies align precisely
#' and that link line endpoints land on the correct rows.  The final
#' \code{height} / \code{width} attributes on the returned object include
#' legend space and are clamped to \eqn{[4, 64]} inches with aspect-ratio
#' correction.
#'
#' @inheritParams LinkedHeatmapAtomic
#' @inheritParams common_args
#' @param values_by Default column name for heatmap cell values.  Used as
#'  fallback when \code{left_values_by} / \code{right_values_by} are not
#'  explicitly provided via \code{...}.
#' @param name Default legend title for the colour scale.  Used as fallback
#'  when \code{left_name} / \code{right_name} are not provided via
#'  \code{...}.  The suffixes \code{" (left)"} / \code{" (right)"} are
#'  appended automatically.
#' @param rows_by Default column for rows in both heatmaps.  Used as
#'  fallback for \code{left_rows_by} / \code{right_rows_by}.
#' @param rows_by_sep Separator for concatenated \code{rows_by} columns.
#' @param rows_split_by_sep Separator for concatenated
#'  \code{rows_split_by} columns.
#' @param columns_by Default column for columns in both heatmaps.  Used as
#'  fallback for \code{left_columns_by} / \code{right_columns_by}.
#' @param columns_by_sep Separator for concatenated \code{columns_by}
#'  columns.
#' @param columns_split_by Default column to split columns into groups.
#'  Used as fallback for \code{left_columns_split_by} /
#'  \code{right_columns_split_by}.
#' @param columns_split_by_sep Separator for concatenated
#'  \code{columns_split_by} columns.
#' @param rows_data,columns_data Optional data frames providing additional
#'  row / column metadata for annotations.  Passed through to
#'  \code{HeatmapAtomic}.
#' @param rows_orderby,columns_orderby Column name to order rows / columns
#'  by (disables clustering when set).
#' @param columns_name Display name for the column annotation.
#' @param columns_split_name Display name for the column split annotation.
#' @param rows_name Display name for the row annotation.
#' @param rows_split_name Display name for the row split annotation.
#' @param pie_name Default name for the pie legend.  Used as fallback for
#'  \code{left_pie_name} / \code{right_pie_name}.
#' @param pie_group_by Default column(s) for pie grouping.  Used as fallback
#'  for \code{left_pie_group_by} / \code{right_pie_group_by}.
#' @param pie_group_by_sep Separator for concatenated \code{pie_group_by}
#'  columns.
#' @param cluster_rows Default clustering setting for rows.  Used as
#'  fallback for \code{left_cluster_rows} / \code{right_cluster_rows}.
#' @param row_names_side Default side for row names.  Used as fallback for
#'  \code{left_row_names_side} / \code{right_row_names_side}.  Default
#'  \code{"right"}.
#' @param row_annotation_side \strong{Deprecated}: use
#'  \code{row_annotation} with the \code{side} sub-key instead.  Used as
#'  fallback for \code{left_row_annotation_side} /
#'  \code{right_row_annotation_side}.  Default \code{"left"}.
#' @param flip Logical; must be \code{FALSE} for linked heatmaps (flipping
#'  is not supported).  Default \code{FALSE}.
#' @param ... Additional arguments passed to \code{\link{LinkedHeatmapAtomic}}.
#'  All parameters listed above (and those inherited from
#'  \code{LinkedHeatmapAtomic}) can be specified with \code{left_} or
#'  \code{right_} prefixes for per-side control (e.g.
#'  \code{left_palette = "Blues"}, \code{right_palette = "Reds"}).
#'  Unprefixed arguments apply to both sides.  Also forwarded to
#'  \code{\link[ComplexHeatmap]{Heatmap}} after prefix stripping.
#'
#' @return A \code{patchwork} object (class \code{wrap_plots}) with
#'  \code{height} and \code{width} attributes (in inches).  When
#'  \code{combine = FALSE}, a named list of such objects, one per
#'  \code{split_by} level.
#' @export
#' @seealso \code{\link{Heatmap}}
#' @examples
#' \donttest{
#' set.seed(8525)
#' # Define sparse ligand-receptor pairs
#' pairs_df <- data.frame(
#'     ligand = c("Ligand1", "Ligand2", "Ligand3", "Ligand4", "Ligand5",
#'                "Ligand1", "Ligand3", "Ligand5"),
#'     receptor = c("Receptor1", "Receptor2", "Receptor1", "Receptor3", "Receptor4",
#'                  "Receptor5", "Receptor2", "Receptor5"),
#'     stringsAsFactors = FALSE
#' )
#' sources <- paste0("Source", 1:4)
#' targets <- paste0("Target", 1:6)
#'
#' # Expand pairs across all sources and targets
#' data <- merge(
#'     merge(pairs_df, data.frame(source = sources, stringsAsFactors = FALSE)),
#'     data.frame(target = targets, stringsAsFactors = FALSE)
#' )
#' data$split <- sample(c("A", "B"), nrow(data), replace = TRUE)
#' data$ligand_expr <- runif(nrow(data), 0, 10)
#' data$receptor_expr <- runif(nrow(data), 0, 10)
#' data$intensity <- runif(nrow(data), 0, 1)
#'
#' if (requireNamespace("ComplexHeatmap", quietly = TRUE)) {
#'     LinkedHeatmap(
#'         data,
#'         show_column_names = TRUE,
#'         column_names_side = "bottom",
#'         show_row_names = FALSE,
#'         left_row_names_side = "right",
#'         left_rows_by = "ligand",
#'         left_columns_by = "source",
#'         left_values_by = "ligand_expr",
#'         left_name = "Ligand",
#'         left_row_annotation = list(
#'             .row = list(
#'                 type = "label",
#'                 params = list(labels_rot = 0),
#'                 side = "right"
#'             )
#'         ),
#'         right_cluster_rows = FALSE,
#'         right_row_names_side = "left",
#'         right_row_annotation = list(
#'             .row = list(
#'                 type = "label",
#'                 params = list(labels_rot = 0),
#'                 side = "left"
#'             )
#'         ),
#'         right_rows_by = "receptor",
#'         right_columns_by = "target",
#'         right_values_by = "receptor_expr",
#'         right_name = "Receptor",
#'         link_width_by = "intensity"
#'     )
#' }
#' }
LinkedHeatmap <- function(
    data,
    values_by,
    values_fill = NA,
    name = NULL,
    split_by = NULL,
    split_by_sep = "_",
    # data definition
    rows_by = NULL,
    rows_by_sep = "_",
    rows_split_by = NULL,
    rows_split_by_sep = "_",
    columns_by = NULL,
    columns_by_sep = "_",
    columns_split_by = NULL,
    columns_split_by_sep = "_",
    rows_data = NULL,
    columns_data = NULL,
    keep_na = FALSE,
    keep_empty = FALSE,
    rows_orderby = NULL,
    columns_orderby = NULL,
    # names
    columns_name = NULL,
    columns_split_name = NULL,
    rows_name = NULL,
    rows_split_name = NULL,
    # palettes
    palette = "RdBu",
    palcolor = NULL,
    palreverse = FALSE,
    # cell_type: pies
    pie_size_name = "size",
    pie_size = NULL,
    pie_values = "length",
    pie_name = NULL,
    pie_group_by = NULL,
    pie_group_by_sep = "_",
    pie_palette = "Spectral",
    pie_palcolor = NULL,
    # cell_type: bars
    bars_sample = 1,
    # cell_type: label
    label = identity,
    label_size = 10,
    label_color = "black",
    label_name = "label",
    # cell_type: mark
    mark = identity,
    mark_color = "black",
    mark_size = 1,
    mark_name = "mark",
    # cell_type: violin
    violin_fill = NULL,
    # cell_type: boxplot
    boxplot_fill = NULL,
    # cell_type: dot
    dot_size = 8,
    dot_size_name = "size",
    # legend
    legend_items = NULL,
    legend_discrete = FALSE,
    legend.position = "right",
    legend.direction = "vertical",
    # values
    lower_quantile = 0,
    upper_quantile = 0.99,
    lower_cutoff = NULL,
    upper_cutoff = NULL,
    # bg
    add_bg = FALSE,
    bg_alpha = 0.5,
    # reticle
    add_reticle = FALSE,
    reticle_color = "grey",
    # passed to ComplexHeatmap::Heatmap
    cluster_columns = NULL,
    cluster_rows = NULL,
    show_row_names = NULL,
    show_column_names = NULL,
    border = TRUE,
    title = NULL,
    title_params = NULL,
    column_title = NULL,
    row_title = NULL,
    na_col = "grey85",
    row_names_side = "right",
    column_names_side = "bottom",
    row_annotation = NULL,
    row_annotation_side = NULL,
    row_annotation_palette = NULL,
    row_annotation_palcolor = NULL,
    row_annotation_type = NULL,
    row_annotation_params = NULL,
    row_annotation_agg = NULL,
    column_annotation = NULL,
    column_annotation_side = NULL,
    column_annotation_palette = NULL,
    column_annotation_palcolor = NULL,
    column_annotation_type = NULL,
    column_annotation_params = NULL,
    column_annotation_agg = NULL,
    # links
    link_width_by = NULL,
    link_width_scale = 5,
    link_color = "grey40",
    link_alpha = 0.6,

    # misc
    flip = FALSE,
    alpha = 1,
    seed = 8525,
    padding = 15,
    base_size = 1,
    aspect.ratio = NULL,
    draw_opts = list(),
    # cell customization
    layer_fun_callback = NULL,
    cell_type = c(
        "tile",
        "bars",
        "label",
        "mark",
        "label+mark",
        "mark+label",
        "dot",
        "violin",
        "boxplot",
        "pie"
    ),
    cell_agg = NULL,
    # subplots
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    # ── Validate ──
    validate_common_args(seed)
    stopifnot(
        "[LinkedHeatmap] `flip` is not supported for linked heatmaps; please set `flip = FALSE`" = isFALSE(
            flip
        )
    )

    cell_type <- match.arg(cell_type)
    cell_type <- sub("mark+label", "label+mark", cell_type, fixed = TRUE)

    if (is.character(link_width_by)) {
        link_width_by <- check_columns(data, link_width_by)
    }

    args <- list(...)
    left_rows_orderby <- args$left_rows_orderby %||% rows_orderby
    if (!is.null(left_rows_orderby)) {
        left_cluster_rows <- args$left_cluster_rows %||% cluster_rows %||% FALSE
        stopifnot(
            "[LinkedHeatmap] `left_rows_orderby` can't be used with `left_cluster_rows/cluster_rows = TRUE`" = !left_cluster_rows
        )
    } else {
        left_cluster_rows <- args$left_cluster_rows %||% cluster_rows %||% TRUE
    }

    right_rows_orderby <- args$right_rows_orderby %||% rows_orderby
    if (!is.null(right_rows_orderby)) {
        right_cluster_rows <- args$right_cluster_rows %||%
            cluster_rows %||%
            FALSE
        stopifnot(
            "[LinkedHeatmap] `right_rows_orderby` can't be used with `right_cluster_rows/cluster_rows = TRUE`" = !right_cluster_rows
        )
    } else {
        right_cluster_rows <- args$right_cluster_rows %||%
            cluster_rows %||%
            TRUE
    }

    # ── Preprocess data ──
    hmdata <- process_linkedheatmap_data(
        data,
        split_by = split_by,
        split_by_sep = split_by_sep,
        rows_split_by = rows_split_by,
        rows_split_by_sep = rows_split_by_sep,
        rows_split_name = rows_split_name,
        left_values_by = args$left_values_by %||% values_by,
        left_name = args$left_name %||%
            if (is.null(name)) NULL else paste0(name, " (left)"),
        left_rows_by = args$left_rows_by %||% rows_by,
        left_rows_by_sep = args$left_rows_by_sep %||% rows_by_sep,
        left_rows_name = args$left_rows_name %||% rows_name,
        left_rows_orderby = left_rows_orderby,
        left_columns_orderby = args$left_columns_orderby %||% columns_orderby,
        left_columns_by = args$left_columns_by %||% columns_by,
        left_columns_by_sep = args$left_columns_by_sep %||% columns_by_sep,
        left_columns_name = args$left_columns_name %||% columns_name,
        left_columns_split_by = args$left_columns_split_by %||%
            columns_split_by,
        left_columns_split_by_sep = args$left_columns_split_by_sep %||%
            columns_split_by_sep,
        left_columns_split_name = args$left_columns_split_name %||%
            columns_split_name,
        left_pie_group_by = args$left_pie_group_by %||% pie_group_by,
        left_pie_group_by_sep = args$left_pie_group_by_sep %||%
            pie_group_by_sep,
        left_pie_name = args$left_pie_name %||% pie_name,
        left_rows_data = args$left_rows_data %||% rows_data,
        left_columns_data = args$left_columns_data %||% columns_data,
        right_values_by = args$right_values_by %||% values_by,
        right_name = args$right_name %||%
            if (is.null(name)) NULL else paste0(name, " (right)"),
        right_rows_by = args$right_rows_by %||% rows_by,
        right_rows_by_sep = args$right_rows_by_sep %||% rows_by_sep,
        right_rows_name = args$right_rows_name %||% rows_name,
        right_rows_orderby = right_rows_orderby,
        right_columns_orderby = args$right_columns_orderby %||% columns_orderby,
        right_columns_by = args$right_columns_by %||% columns_by,
        right_columns_by_sep = args$right_columns_by_sep %||% columns_by_sep,
        right_columns_name = args$right_columns_name %||% columns_name,
        right_columns_split_by = args$right_columns_split_by %||%
            columns_split_by,
        right_columns_split_by_sep = args$right_columns_split_by_sep %||%
            columns_split_by_sep,
        right_columns_split_name = args$right_columns_split_name %||%
            columns_split_name,
        right_pie_group_by = args$right_pie_group_by %||% pie_group_by,
        right_pie_group_by_sep = args$right_pie_group_by_sep %||%
            pie_group_by_sep,
        right_pie_name = args$right_pie_name %||% pie_name,
        right_rows_data = args$right_rows_data %||% rows_data,
        right_columns_data = args$right_columns_data %||% columns_data,
        keep_na = keep_na,
        keep_empty = keep_empty
    )

    if (length(split_by) > 1) {
        split_by <- paste(split_by, collapse = split_by_sep)
    } else {  # NULL
        split_by <- "..."
    }

    palette <- check_palette(palette, names(hmdata$data))
    palcolor <- check_palcolor(palcolor, names(hmdata$data))
    pie_palette <- check_palette(pie_palette, names(hmdata$data))
    pie_palcolor <- check_palcolor(pie_palcolor, names(hmdata$data))

    # ── Validate per-split legend params ──
    legend.position <- check_legend(
        legend.position,
        names(hmdata$data),
        "legend.position"
    )
    legend.direction <- check_legend(
        legend.direction,
        names(hmdata$data),
        "legend.direction"
    )

    # ── Build per-split plots ──
    plots <- lapply(names(hmdata$data), function(nm) {
        default_title <- if (length(hmdata$data) == 1 && identical(nm, "...")) {
            NULL
        } else {
            nm
        }
        if (is.function(title)) {
            title <- title(default_title)
        } else {
            title <- title %||% default_title
        }

        args_atomic <- args

        args_atomic$data <- hmdata$data[[nm]]

        args_atomic$left_values_by <- hmdata$left_values_by
        args_atomic$right_values_by <- hmdata$right_values_by
        args_atomic$left_rows_by <- hmdata$left_rows_by
        args_atomic$right_rows_by <- hmdata$right_rows_by
        args_atomic$rows_split_by <- hmdata$rows_split_by
        args_atomic$left_columns_by <- hmdata$left_columns_by
        args_atomic$right_columns_by <- hmdata$right_columns_by
        args_atomic$left_columns_split_by <- hmdata$left_columns_split_by
        args_atomic$right_columns_split_by <- hmdata$right_columns_split_by
        args_atomic$left_pie_group_by <- hmdata$left_pie_group_by
        args_atomic$right_pie_group_by <- hmdata$right_pie_group_by

        args_atomic$palette <- palette[[nm]]
        args_atomic$palcolor <- palcolor[[nm]]
        args_atomic$palreverse <- palreverse

        args_atomic$pie_size_name <- pie_size_name
        args_atomic$pie_size <- pie_size
        args_atomic$pie_values <- pie_values
        args_atomic$pie_palette <- pie_palette[[nm]]
        args_atomic$pie_palcolor <- pie_palcolor[[nm]]

        args_atomic$bars_sample <- bars_sample
        args_atomic$label <- label
        args_atomic$label_size <- label_size
        args_atomic$label_color <- label_color
        args_atomic$mark <- mark
        args_atomic$mark_color <- mark_color
        args_atomic$mark_size <- mark_size
        args_atomic$violin_fill <- violin_fill
        args_atomic$boxplot_fill <- boxplot_fill
        args_atomic$dot_size <- dot_size
        args_atomic$dot_size_name <- dot_size_name

        args_atomic$legend_items <- legend_items
        args_atomic$legend_discrete <- legend_discrete
        args_atomic$legend.position <- legend.position[[nm]]
        args_atomic$legend.direction <- legend.direction[[nm]]

        args_atomic$lower_quantile <- lower_quantile
        args_atomic$upper_quantile <- upper_quantile
        args_atomic$lower_cutoff <- lower_cutoff
        args_atomic$upper_cutoff <- upper_cutoff

        args_atomic$add_bg <- add_bg
        args_atomic$bg_alpha <- bg_alpha
        args_atomic$keep_na <- hmdata$keep_na
        args_atomic$keep_empty <- hmdata$keep_empty

        args_atomic$add_reticle <- add_reticle
        args_atomic$reticle_color <- reticle_color

        args_atomic$left_cluster_rows <- left_cluster_rows
        args_atomic$right_cluster_rows <- right_cluster_rows
        args_atomic$cluster_columns <- cluster_columns
        args_atomic$show_row_names <- show_row_names
        args_atomic$show_column_names <- show_column_names
        args_atomic$border <- border
        args_atomic$title <- title
        args_atomic$title_params <- title_params
        args_atomic$column_title <- column_title
        args_atomic$row_title <- row_title
        args_atomic$na_col <- na_col
        args_atomic$row_names_side <- row_names_side
        args_atomic$column_names_side <- column_names_side
        args_atomic$row_annotation <- row_annotation
        args_atomic$row_annotation_side <- row_annotation_side
        args_atomic$row_annotation_palette <- row_annotation_palette
        args_atomic$row_annotation_palcolor <- row_annotation_palcolor
        args_atomic$row_annotation_type <- row_annotation_type
        args_atomic$row_annotation_params <- row_annotation_params
        args_atomic$row_annotation_agg <- row_annotation_agg
        args_atomic$column_annotation <- column_annotation
        args_atomic$column_annotation_side <- column_annotation_side
        args_atomic$column_annotation_palette <- column_annotation_palette
        args_atomic$column_annotation_palcolor <- column_annotation_palcolor
        args_atomic$column_annotation_type <- column_annotation_type
        args_atomic$column_annotation_params <- column_annotation_params
        args_atomic$column_annotation_agg <- column_annotation_agg

        args_atomic$link_width_by <- link_width_by
        args_atomic$link_width_scale <- link_width_scale
        args_atomic$link_color <- link_color
        args_atomic$link_alpha <- link_alpha

        args_atomic$alpha <- alpha
        args_atomic$seed <- seed
        args_atomic$base_size <- base_size
        args_atomic$aspect.ratio <- aspect.ratio
        args_atomic$draw_opts <- draw_opts
        args_atomic$layer_fun_callback <- layer_fun_callback
        args_atomic$cell_type <- cell_type
        args_atomic$cell_agg <- cell_agg

        args_atomic$values_fill <- values_fill

        do_call(LinkedHeatmapAtomic, args_atomic)
    })

    names(plots) <- names(hmdata$data)

    # ── Combine ──
    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
# Linked heatmap data: ligand-receptor pairs across sources/targets
pairs_df <- data.frame(
    ligand = rep(paste0("Ligand", 1:4), each = 2),
    receptor = paste0("Receptor", 1:8),
    stringsAsFactors = FALSE
)
sources <- paste0("Source", 1:3)
targets <- paste0("Target", 1:4)
link_data <- merge(
    merge(pairs_df, data.frame(source = sources, stringsAsFactors = FALSE)),
    data.frame(target = targets, stringsAsFactors = FALSE)
)
link_data$split <- sample(c("A", "B"), nrow(link_data), replace = TRUE)
link_data$ligand_expr <- runif(nrow(link_data), 0, 10)
link_data$receptor_expr <- runif(nrow(link_data), 0, 10)
link_data$intensity <- runif(nrow(link_data), 0, 1)

test_that("LinkedHeatmap returns an object", {
    skip_if_not_installed("ComplexHeatmap")
    p <- LinkedHeatmap(
        link_data,
        left_rows_by = "ligand",
        left_columns_by = "source",
        left_values_by = "ligand_expr",
        left_name = "Ligand",
        right_rows_by = "receptor",
        right_columns_by = "target",
        right_values_by = "receptor_expr",
        right_name = "Receptor",
        link_width_by = "intensity"
    )
    expect_true(!is.null(p))
})

test_that("LinkedHeatmap works with name and split title display modes", {
    skip_if_not_installed("ComplexHeatmap")
    p <- LinkedHeatmap(
        link_data,
        left_rows_by = "ligand",
        left_columns_by = "source",
        left_values_by = "ligand_expr",
        left_name = "Ligand",
        right_rows_by = "receptor",
        right_columns_by = "target",
        right_values_by = "receptor_expr",
        right_name = "Receptor",
        link_width_by = "intensity",
        rows_split_by = "split",
        show_row_names = "legend",
        row_title = c("inplace", "legend")
    )
    expect_true(!is.null(p))
})

#' A list of palettes for use in data visualization
#'
#' @concept data
#' @examples
#' \dontrun{
#' if (interactive()) {
#'   library(stringr)
#'   library(RColorBrewer)
#'   library(Redmonder)
#'   library(rcartocolor)
#'   library(nord)
#'   library(viridis)
#'   library(pals)
#'   library(dichromat)
#'   library(jcolors)
#'   library(scales)
#'   library(ggthemes)
#'   syspals <- utils::getFromNamespace("syspals", "pals")
#'   brewer.pal.info <- RColorBrewer::brewer.pal.info
#'   ggsci_db <- utils::getFromNamespace("ggsci_db", "ggsci")
#'   redmonder.pal.info <- Redmonder::redmonder.pal.info
#'   metacartocolors <- rcartocolor::metacartocolors
#'   rownames(metacartocolors) <- metacartocolors$Name
#'   nord_palettes <- nord::nord_palettes
#'   viridis_names <- c("magma", "inferno", "plasma", "viridis", "cividis", "rocket",
#'      "mako", "turbo")
#'   viridis_palettes <- lapply(stats::setNames(viridis_names, viridis_names),
#'      function(x) viridis::viridis(100, option = x))
#'   ocean_names <- names(syspals)[grep("ocean", names(syspals))]
#'   ocean_palettes <- syspals[ocean_names]
#'   dichromat_palettes <- dichromat::colorschemes
#'   jcolors_names <- paste0("jcolors-", c("default", "pal2", "pal3", "pal4", "pal5",
#'      "pal6", "pal7", "pal8", "pal9", "pal10", "pal11", "pal12", "rainbow"))
#'   custom_names <- c("jet", "simspec", "GdRd")
#'   custom_palettes <- list(
#'     oompaBase::jetColors(N = 100),
#'     c("#c22b86", "#f769a1", "#fcc5c1", "#253777", "#1d92c0", "#9ec9e1", "#015b33",
#'         "#42aa5e", "#d9f0a2", "#E66F00", "#f18c28", "#FFBB61"),
#'     c("gold", "red3")
#'   )
#'   names(custom_palettes) <- custom_names
#'   seurat_discrete_palettes <- list(
#'      alphabet = c(
#'        "#F0A0FF", "#0075DC", "#993F00", "#4C005C", "#191919", "#005C31",
#'        "#2BCE48", "#FFCC99", "#808080", "#94FFB5", "#8F7C00", "#9DCC00",
#'        "#C20088", "#003380", "#FFA405", "#FFA8BB", "#426600", "#FF0010",
#'        "#5EF1F2", "#00998F", "#E0FF66", "#740AFF", "#990000", "#FFFF80",
#'        "#FFE100", "#FF5005"
#'      ),
#'      alphabet2 = c(
#'        "#AA0DFE", "#3283FE", "#85660D", "#782AB6", "#565656", "#1C8356",
#'        "#16FF32", "#F7E1A0", "#E2E2E2", "#1CBE4F", "#C4451C", "#DEA0FD",
#'        "#FE00FA", "#325A9B", "#FEAF16", "#F8A19F", "#90AD1C", "#F6222E",
#'        "#1CFFCE", "#2ED9FF", "#B10DA1", "#C075A6", "#FC1CBF", "#B00068",
#'        "#FBE426", "#FA0087"
#'      ),
#'      glasbey = c(
#'        "#0000FF", "#FF0000", "#00FF00", "#000033", "#FF00B6", "#005300",
#'        "#FFD300", "#009FFF", "#9A4D42", "#00FFBE", "#783FC1", "#1F9698",
#'        "#FFACFD", "#B1CC71", "#F1085C", "#FE8F42", "#DD00FF", "#201A01",
#'        "#720055", "#766C95", "#02AD24", "#C8FF00", "#886C00", "#FFB79F",
#'        "#858567", "#A10300", "#14F9FF", "#00479E", "#DC5E93", "#93D4FF",
#'        "#004CFF", "#F2F318"
#'      ),
#'      polychrome = c(
#'        "#5A5156", "#E4E1E3", "#F6222E", "#FE00FA", "#16FF32", "#3283FE",
#'        "#FEAF16", "#B00068", "#1CFFCE", "#90AD1C", "#2ED9FF", "#DEA0FD",
#'        "#AA0DFE", "#F8A19F", "#325A9B", "#C4451C", "#1C8356", "#85660D",
#'        "#B10DA1", "#FBE426", "#1CBE4F", "#FA0087", "#FC1CBF", "#F7E1A0",
#'        "#C075A6", "#782AB6", "#AAF400", "#BDCDFF", "#822E1C", "#B5EFB5",
#'        "#7ED7D1", "#1C7F93", "#D85FF7", "#683B79", "#66B0FF", "#3B00FB"
#'      ),
#'      stepped = c(
#'        "#990F26", "#B33E52", "#CC7A88", "#E6B8BF", "#99600F", "#B3823E",
#'        "#CCAA7A", "#E6D2B8", "#54990F", "#78B33E", "#A3CC7A", "#CFE6B8",
#'        "#0F8299", "#3E9FB3", "#7ABECC", "#B8DEE6", "#3D0F99", "#653EB3",
#'        "#967ACC", "#C7B8E6", "#333333", "#666666", "#999999", "#CCCCCC"
#'      ),
#'      parade = c(
#'        '#ff6969', '#9b37ff', '#cd3737', '#69cdff', '#ffff69', '#69cdcd',
#'        '#9b379b', '#3737cd', '#ffff9b', '#cdff69', '#ff9b37', '#37ffff',
#'        '#9b69ff', '#37cd69', '#ff3769', '#ff3737', '#37ff9b', '#cdcd37',
#'        '#3769cd', '#37cdff', '#9b3737', '#ff699b', '#9b9bff', '#cd9b37',
#'        '#69ff37', '#cd3769', '#cd69cd', '#cd6937', '#3737ff', '#cdcd69',
#'        '#ff9b69', '#cd37cd', '#9bff37', '#cd379b', '#cd6969', '#69ff9b',
#'        '#ff379b', '#9bff9b', '#6937ff', '#69cd37', '#cdff37', '#9bff69',
#'        '#9b37cd', '#ff37ff', '#ff37cd', '#ffff37', '#37cd9b', '#379bff',
#'        '#ffcd37', '#379b37', '#ff9bff', '#379b9b', '#69ffcd', '#379bcd',
#'        '#ff69ff', '#ff9b9b', '#37ff69', '#ff6937', '#6969ff', '#699bff',
#'        '#ffcd69', '#69ffff', '#37ff37', '#6937cd', '#37cd37', '#3769ff',
#'        '#cd69ff', '#6969cd', '#9bcd37', '#69ff69', '#37cdcd', '#cd37ff',
#'        '#37379b', '#37ffcd', '#69cd69', '#ff69cd', '#9bffff', '#9b9b37'
#'      )
#'   )
#'   seurat_continuous_palettes <- list(
#'     seurat = hue_pal()(16),
#'     seurat.16 = hue_pal()(16),
#'     seurat.32 = hue_pal()(32),
#'     seurat.64 = hue_pal()(64)
#'   )
#'   stripe_palettes <- list(
#'     stripe = rep(c("white", "grey60"), 8),
#'     stripe.16 = rep(c("white", "grey60"), 8),
#'     stripe.32 = rep(c("white", "grey60"), 16),
#'     stripe.64 = rep(c("white", "grey60"), 32)
#'   )
#'   tableau_palettes <- list()
#'   orig_tableau_palettes <- ggthemes::ggthemes_data[["tableau"]][["color-palettes"]]
#'   for (g in names(orig_tableau_palettes)) {
#'     for (pal in names(orig_tableau_palettes[[g]])) {
#'       palcolors <- as.list(orig_tableau_palettes[[g]][[pal]])
#'       if (!is.null(palcolors$name)) {
#'         tableau_palettes[[pal]] <- stats::setNames(palcolors$value, palcolors$name)
#'       } else {
#'         tableau_palettes[[pal]] <- palcolors$value
#'       }
#'     }
#'   }
#'
#'   palette_list <- list()
#'   all_colors <- c(
#'     rownames(brewer.pal.info), names(ggsci_db), rownames(redmonder.pal.info),
#'     rownames(metacartocolors), names(nord_palettes), names(viridis_palettes),
#'     ocean_names, names(dichromat_palettes), jcolors_names, names(seurat_discrete_palettes),
#'     names(seurat_continuous_palettes), custom_names, names(stripe_palettes),
#'     names(tableau_palettes)
#'   )
#'   for (pal in all_colors) {
#'     if (!pal %in% all_colors) {
#'       stop(paste0("Invalid pal Must be one of ", paste0(all_colors, collapse = ",")))
#'     }
#'     if (pal %in% rownames(brewer.pal.info)) {
#'       pal_n <- brewer.pal.info[pal, "maxcolors"]
#'       pal_category <- brewer.pal.info[pal, "category"]
#'       if (pal_category == "div") {
#'         palcolor <- rev(brewer.pal(name = pal, n = pal_n))
#'       } else {
#'         if (pal == "Paired") {
#'           palcolor <- brewer.pal(12, "Paired")[c(1:4, 7, 8, 5, 6, 9, 10, 11, 12)]
#'         } else {
#'           palcolor <- brewer.pal(name = pal, n = pal_n)
#'         }
#'       }
#'       if (pal_category == "qual") {
#'         attr(palcolor, "type") <- "discrete"
#'       } else {
#'         attr(palcolor, "type") <- "continuous"
#'       }
#'     } else if (pal %in% names(ggsci_db)) {
#'       if (pal %in% c("d3", "uchicago", "material")) {
#'         for (subpal in names(ggsci_db[[pal]])) {
#'           palcolor <- ggsci_db[[pal]][[subpal]]
#'           if (pal == "material") {
#'             attr(palcolor, "type") <- "continuous"
#'           } else {
#'             attr(palcolor, "type") <- "discrete"
#'           }
#'           palette_list[[paste0(pal, "-", subpal)]] <- palcolor
#'         }
#'         next
#'       } else {
#'         palcolor <- ggsci_db[[pal]][[1]]
#'         if (pal == "gsea") {
#'           attr(palcolor, "type") <- "continuous"
#'         } else {
#'           attr(palcolor, "type") <- "discrete"
#'         }
#'       }
#'     } else if (pal %in% rownames(redmonder.pal.info)) {
#'       pal_n <- redmonder.pal.info[pal, "maxcolors"]
#'       pal_category <- redmonder.pal.info[pal, "category"]
#'       if (pal_category == "div") {
#'         palcolor <- rev(redmonder.pal(name = pal, n = pal_n))
#'       } else {
#'         palcolor <- redmonder.pal(name = pal, n = pal_n)
#'       }
#'       if (pal_category == "qual") {
#'         attr(palcolor, "type") <- "discrete"
#'       } else {
#'         attr(palcolor, "type") <- "continuous"
#'       }
#'     } else if (pal %in% rownames(metacartocolors)) {
#'       pal_n <- metacartocolors[pal, "Max_n"]
#'       palcolor <- carto_pal(name = pal, n = pal_n)
#'       if (pal_category == "qualitative") {
#'         attr(palcolor, "type") <- "discrete"
#'       } else {
#'         attr(palcolor, "type") <- "continuous"
#'       }
#'     } else if (pal %in% names(nord_palettes)) {
#'       palcolor <- nord_palettes[[pal]]
#'       attr(palcolor, "type") <- "discrete"
#'     } else if (pal %in% names(viridis_palettes)) {
#'       palcolor <- viridis_palettes[[pal]]
#'       attr(palcolor, "type") <- "continuous"
#'     } else if (pal %in% names(ocean_palettes)) {
#'       palcolor <- ocean_palettes[[pal]]
#'       attr(palcolor, "type") <- "continuous"
#'     } else if (pal %in% names(dichromat_palettes)) {
#'       palcolor <- dichromat_palettes[[pal]]
#'       if (pal %in% c("Categorical.12", "SteppedSequential.5")) {
#'         attr(palcolor, "type") <- "discrete"
#'       } else {
#'         attr(palcolor, "type") <- "continuous"
#'       }
#'     } else if (pal %in% jcolors_names) {
#'       palcolor <- jcolors(palette = gsub("jcolors-", "", pal))
#'       if (pal %in% paste0("jcolors-", c("pal10", "pal11", "pal12", "rainbow"))) {
#'         attr(palcolor, "type") <- "continuous"
#'       } else {
#'         attr(palcolor, "type") <- "discrete"
#'       }
#'     } else if (pal %in% custom_names) {
#'       palcolor <- custom_palettes[[pal]]
#'       if (pal %in% c("jet")) {
#'         attr(palcolor, "type") <- "continuous"
#'       } else {
#'         attr(palcolor, "type") <- "discrete"
#'       }
#'     } else if (pal %in% names(seurat_discrete_palettes)) {
#'       palcolor <- seurat_discrete_palettes[[pal]]
#'       attr(palcolor, "type") <- "discrete"
#'     } else if (pal %in% names(seurat_continuous_palettes)) {
#'       palcolor <- seurat_continuous_palettes[[pal]]
#'       attr(palcolor, "type") <- "continuous"
#'     } else if (pal %in% names(stripe_palettes)) {
#'       palcolor <- stripe_palettes[[pal]]
#'       attr(palcolor, "type") <- "discrete"
#'     } else if (pal %in% names(tableau_palettes)) {
#'       palcolor <- tableau_palettes[[pal]]
#'       attr(palcolor, "type") <- "discrete"
#'     }
#'     palette_list[[pal]] <- palcolor
#'   }
#' }
#' }
#' @name palette_list
NULL

#' Excluded words in keyword enrichment analysis and extraction
#'
#' The variable "words_excluded" represents the words that are excluded during keyword enrichment analysis or keyword extraction process. These mainly include words that are excessively redundant or of little value.
#' @concept data
#' @examples
#' \dontrun{
#' if (interactive()) {
#'   words_excluded <- c(
#'     "the", "is", "and", "or", "a", "in", "on", "under", "between", "of", "through",
#'     "via", "along", "that", "for", "with", "within", "without", "cell", "cellular",
#'     "dna", "rna", "protein", "peptide", "amino", "acid", "development", "involved",
#'     "organization", "system", "regulation", "regulated", "positive", "negative",
#'     "response", "process", "processing", "small", "large", "change", "disease"
#'   )
#' }
#' }
#' @name words_excluded
NULL

#' An example of clusterProfiler enrichment result
#' @concept data
#' @examples
#' \dontrun{
#' if (interactive()) {
#'   data(geneList, package="DOSE")
#'   de <- names(geneList)[abs(geneList) > 1.5]
#'   enrich_example <- clusterProfiler::enrichPathway(gene=de, pvalueCutoff = 0.05, readable=TRUE)
#'   enrich_example <- as.data.frame(enrich_example)
#' }
#' }
#' @name enrich_example
NULL

#' An example of clusterProfiler enrichment result with multiple databases
#' @concept data
#' @examples
#' \dontrun{
#' if (interactive()) {
#'   data(enrich_example, package="plotthis")
#'   enrich_example$Database <- "DB1"
#'   enrich_example2 <- enrich_example
#'   enrich_example2$Database <- "DB2"
#'   enrich_example2$ID <- paste0(enrich_example2$ID, "_DB2")
#'   enrich_example2$Description <- paste0(enrich_example2$Description, " (DB2)")
#'   enrich_multidb_example <- rbind(enrich_example, enrich_example2)
#' }
#' }
#' @name enrich_multidb_example
NULL

#' An example of GSEA result from fgsea package
#' @concept data
#' @examples
#' \dontrun{
#' if (interactive()) {
#'  set.seed(1234)
#'  data(geneList, package="DOSE")
#'  gsea_example <- DOSE::gseDO(geneList)
#'  gene_ranks <- gsea_example@geneList
#'  gene_sets <- gsea_example@geneSets
#'  gsea_example_pos <- gsea_example[gsea_example$p.adjust < 0.05 & gsea_example$NES > 0, ]
#'  gsea_example_neg <- gsea_example[gsea_example$p.adjust < 0.05 & gsea_example$NES < 0, ]
#'  gsea_example <- rbind(
#'      gsea_example_pos[sample(1:nrow(gsea_example_pos), 5), ],
#'      gsea_example_pos[sample(1:nrow(gsea_example_pos), 5), ]
#'  )
#'
#'  attr(gsea_example, "gene_ranks") <- gene_ranks
#'  attr(gsea_example, "gene_sets") <- gene_sets[gsea_example$ID]
#' }
#' }
#' @name gsea_example
NULL

#' An example data for dimensionality reduction plot
#'
#' @description
#' This dataset is generated from the scvelo (scv.datasets.pancreas()) with
#' the scvelo run on the dataset.
#' Then the cell embeddings and velocity embeddings are extracted (200 downsampled), which are the first
#' 4 columns of the data frame.
#' The fifth column is the group identifier (clusters), and the sixth column is a fake
#' grouping variable used to visualize stats, facetting, etc.
#' An attribute "graph" is added to the data frame, which is a square matrix of the
#' cell-cell distances, which is used for the graph (network) on dimensionality reduction
#' plots.
#'
#' @concept data
#' @name dim_example
NULL

#' NetworkAtomic
#'
#' @description
#' Core implementation for rendering a single network graph from a links
#' (edge list) data frame and an optional nodes (vertex metadata) data frame.
#' This is the workhorse behind the exported \code{\link{Network}} function --
#' it takes a **single** pair of links/nodes data frames (no
#' \code{split_by} support) and returns a \code{ggplot} object.
#'
#' The graph is constructed via \code{\link[igraph]{graph_from_data_frame}},
#' laid out using igraph layout algorithms (\code{"circle"}, \code{"tree"},
#' \code{"grid"}, or any named igraph layout such as \code{"fr"} or
#' \code{"kk"}), and rendered with \code{\link[ggraph]{ggraph}} using
#' \code{\link[ggraph]{geom_edge_arc}} for links and
#' \code{\link[ggplot2]{geom_point}} for nodes.
#'
#' Key features include:
#' \itemize{
#'   \item \strong{Directed / undirected graphs} with optional arrow heads.
#'   \item \strong{Link styling}: variable width (weight), linetype, and
#'         colour, each constant or mapped from a column. Edge colours
#'         can follow source nodes, target nodes, or a dedicated column.
#'   \item \strong{Node styling}: variable size, shape, colour, and fill,
#'         each constant or column-mapped.
#'   \item \strong{Community detection} via igraph clustering algorithms
#'         with convex hull, ellipse, rectangle, or circle marks from
#'         \code{\link[ggforce]{ggforce}}.
#'   \item \strong{Automatic labels} via
#'         \code{\link[ggrepel]{geom_text_repel}}.
#'   \item \strong{Self-loop edges} drawn as direction-sensitive arcs.
#' }
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{Column resolution} -- Default column names for
#'         \code{from}, \code{to}, and \code{node_by} are assigned when
#'         \code{NULL}, using the first or second column of the respective
#'         data frame. Each is validated via \code{\link{check_columns}};
#'         multi-column inputs are concatenated with their respective
#'         separator (\code{from_sep}, \code{to_sep}, \code{node_by_sep})
#'         and force-converted to factors. Resolved columns are renamed to
#'         the canonical names \code{"from"}, \code{"to"}, and \code{"name"}.
#'   \item \strong{Graph construction} -- An \code{\link[igraph]{igraph}}
#'         object is built via \code{\link[igraph]{graph_from_data_frame}}
#'         using the links data frame and (optionally) the nodes data frame.
#'         If \code{nodes} is a string starting with \code{"@"}, it is
#'         extracted from the corresponding attribute on the links object.
#'   \item \strong{Layout computation} -- The layout is resolved:
#'         \code{"circle"} -> \code{\link[igraph]{layout_in_circle}},
#'         \code{"tree"} -> \code{\link[igraph]{layout_as_tree}},
#'         \code{"grid"} -> \code{\link[igraph]{layout_on_grid}}.
#'         Any other character string is prefixed with \code{"layout_with_"}
#'         and looked up in the igraph namespace (e.g. \code{"fr"} ->
#'         \code{layout_with_fr}). If \code{layout} is already an
#'         \code{igraph_layout_spec} object it is applied directly via
#'         \code{\link[igraph]{layout_}}.
#'   \item \strong{Data extraction} -- Vertex and edge data frames are
#'         extracted from the igraph object via
#'         \code{\link[igraph]{as_data_frame}}. Layout coordinates are
#'         added as \code{x} and \code{y} columns to the vertex data.
#'   \item \strong{Node aesthetic assembly} -- Each of \code{size},
#'         \code{color}, \code{shape}, and \code{fill} is resolved as
#'         either a constant value (numeric or character) or a mapping to
#'         a column of the node data. A per-aesthetic \code{"guide"} /
#'         \code{"none"} flag is tracked for legend construction.
#'   \item \strong{Link aesthetic assembly} -- \code{linewidth} (weight),
#'         \code{linetype}, and \code{color} are resolved similarly.
#'         \itemize{
#'           \item When \code{link_color_by = "from"}, edge colours are
#'                 derived from the source node's fill (if node shape is
#'                 filled, 21--25) or colour; when \code{"to"}, from the
#'                 target node.
#'           \item When \code{link_color_by} names a column in the links
#'                 data frame (other than \code{"from"}/\code{"to"}), it
#'                 is mapped directly.
#'         }
#'         If \code{directed = TRUE}, a \code{\link[ggplot2]{arrow}} is
#'         added and \code{end_cap} is set via \code{\link[ggraph]{circle}}
#'         to prevent arrow overlap with nodes. Self-loop edges receive a
#'         \code{direction} aesthetic.
#'   \item \strong{Plot initialisation} -- A \code{\link[ggraph]{ggraph}}
#'         plot is created with \code{layout = "manual"} and vertex
#'         coordinates from the layout.
#'   \item \strong{Clustering and marks} -- When \code{cluster != "none"},
#'         the specified igraph community-detection algorithm is run
#'         (\code{cluster_fast_greedy}, \code{cluster_walktrap},
#'         \code{cluster_edge_betweenness}, \code{cluster_infomap}, or a
#'         custom function). Membership overrides the aesthetic specified
#'         by \code{cluster_scale} (\code{"fill"}, \code{"color"}, or
#'         \code{"shape"}) with a warning that the previous setting is
#'         discarded. If \code{add_mark = TRUE}, a mark enclosure
#'         (hull / ellipse / rect / circle) is drawn per cluster via the
#'         corresponding \code{\link[ggforce]{ggforce}} geom.
#'   \item \strong{Link rendering} -- \code{\link[ggraph]{geom_edge_arc}}
#'         draws the edges (with configurable \code{link_curvature}) and
#'         \code{\link[ggraph]{geom_edge_loop}} handles self-loop edges.
#'   \item \strong{Link scales} -- Conditional on their guide status,
#'         \code{scale_edge_width_continuous},
#'         \code{scale_edge_linetype_discrete}, and
#'         \code{scale_edge_color_manual} /
#'         \code{scale_edge_color_gradientn} are added for weight,
#'         linetype, and colour legends respectively.
#'   \item \strong{Node rendering} -- \code{\link[ggplot2]{geom_point}}
#'         draws the nodes with the assembled aesthetics.
#'   \item \strong{Node scales} -- Conditional scale additions:
#'         \code{scale_size_continuous} (range from
#'         \code{node_size_range}), \code{scale_color_manual},
#'         \code{scale_shape_manual}, and \code{scale_fill_manual}, each
#'         with their legend title and guide overrides.
#'   \item \strong{Labels} -- When \code{add_label = TRUE}, node
#'         identifiers are rendered via
#'         \code{\link[ggrepel]{geom_text_repel}} using
#'         \code{label_fg}, \code{label_bg}, and \code{label_bg_r}.
#'   \item \strong{Final theme and dimensions} -- Coordinate expansion,
#'         axis labels, theme application, and legend positioning are
#'         applied. Plot height and width are computed via
#'         \code{\link{calculate_plot_dimensions}} and stored as
#'         attributes.
#' }
#'
#' @inheritParams common_args
#' @param links A data frame containing the edge list. Must contain the
#'  \code{from} and \code{to} columns specifying source and target node
#'  identifiers. Additional columns can be referenced by other parameters
#'  (e.g., \code{link_weight_by}, \code{link_type_by},
#'  \code{link_color_by}).
#' @param nodes An optional data frame of node metadata. When provided,
#'  columns such as \code{node_size_by}, \code{node_color_by},
#'  \code{node_shape_by}, and \code{node_fill_by} can reference its
#'  columns. When \code{NULL}, the node set is inferred from the unique
#'  values in the \code{from} and \code{to} columns. If a single character
#'  string starting with \code{"@"}, the nodes data frame is extracted
#'  from the corresponding attribute of \code{links} (e.g.
#'  \code{"@nodes"} extracts \code{attr(links, "nodes")}).
#' @param from A character string specifying the column name in
#'  \code{links} for the source node identifiers. Defaults to
#'  \code{"from"}, or the first column of \code{links} if that column
#'  name does not exist. Multiple columns can be provided; they are
#'  concatenated with \code{from_sep}.
#' @param from_sep A character string to join multiple \code{from}
#'  columns. Default \code{"_"}. Ignored when \code{from} is a single
#'  column.
#' @param to A character string specifying the column name in
#'  \code{links} for the target node identifiers. Defaults to
#'  \code{"to"}, or the second column of \code{links} if that column
#'  name does not exist. Multiple columns can be provided; they are
#'  concatenated with \code{to_sep}.
#' @param to_sep A character string to join multiple \code{to} columns.
#'  Default \code{"_"}. Ignored when \code{to} is a single column.
#' @param node_by A character string specifying the column name in
#'  \code{nodes} for the node identifiers. These must match the values
#'  in the \code{from} / \code{to} columns of \code{links}. Defaults to
#'  \code{"name"}, or the first column of \code{nodes} if that column
#'  name does not exist. Multiple columns can be provided; they are
#'  concatenated with \code{node_by_sep}.
#' @param node_by_sep A character string to join multiple
#'  \code{node_by} columns. Default \code{"_"}. Ignored when
#'  \code{node_by} is a single column.
#' @param link_weight_by A numeric value or a character string. If
#'  numeric, all edges receive that constant line width. If a column
#'  name, the edge line width is mapped to that column. Default
#'  \code{2}.
#' @param link_weight_name A character string for the link weight legend
#'  title. When \code{NULL} (default), the column name from
#'  \code{link_weight_by} is used. Only relevant when
#'  \code{link_weight_by} is a column name.
#' @param link_type_by A character string or a column name specifying
#'  the edge linetype. Can be \code{"solid"}, \code{"dashed"},
#'  \code{"dotted"}, etc. If a column name from \code{links} is
#'  supplied, the linetype is mapped to that column (with a version
#'  check for ggplot2 4.0.0, where mapping is unsupported and a
#'  warning is issued). Default \code{"solid"}.
#' @param link_type_name A character string for the link linetype legend
#'  title. When \code{NULL} (default), the column name from
#'  \code{link_type_by} is used. Only relevant when
#'  \code{link_type_by} is a column name.
#' @param node_size_by A numeric value or a character string. If
#'  numeric, all nodes receive that constant point size. If a column
#'  name, the size is mapped to that column. Default \code{15}.
#' @param node_size_name A character string for the node size legend
#'  title. When \code{NULL} (default), the column name from
#'  \code{node_size_by} is used. Only relevant when
#'  \code{node_size_by} is a column name.
#' @param node_color_by A character string specifying the node colour.
#'  If a colour name or hex code (e.g. \code{"black"}), all nodes
#'  receive that constant colour. If a column name from \code{nodes} is
#'  supplied, the colour is mapped to that column. Default
#'  \code{"black"}.
#' @param node_color_name A character string for the node colour legend
#'  title. When \code{NULL} (default), the column name from
#'  \code{node_color_by} is used. Only relevant when
#'  \code{node_color_by} is a column name.
#' @param node_shape_by A numeric value or a character string. If
#'  numeric, all nodes receive that constant shape (see
#'  \code{\link[ggplot2]{shape}}). If a column name, the shape is
#'  mapped to that column (cast to factor). Default \code{21} (filled
#'  circle with border).
#' @param node_shape_name A character string for the node shape legend
#'  title. When \code{NULL} (default), the column name from
#'  \code{node_shape_by} is used. Only relevant when
#'  \code{node_shape_by} is a column name.
#' @param node_fill_by A character string specifying the node fill
#'  colour. If a colour name or hex code (e.g. \code{"grey20"}), all
#'  nodes receive that constant fill. If a column name from
#'  \code{nodes} is supplied, the fill is mapped to that column.
#'  Default \code{"grey20"}.
#' @param node_fill_name A character string for the node fill legend
#'  title. When \code{NULL} (default), the column name from
#'  \code{node_fill_by} is used. Only relevant when
#'  \code{node_fill_by} is a column name.
#' @param link_alpha A numeric value specifying the transparency
#'  (alpha) of the edge lines. Between \code{0} (invisible) and
#'  \code{1} (opaque). Default \code{1}.
#' @param node_alpha A numeric value specifying the fill transparency
#'  of the nodes. Only applies when \code{node_shape_by} is one of the
#'  filled shapes (21--25). Default \code{0.95}.
#' @param node_stroke A numeric value specifying the border stroke
#'  width of the node points. Default \code{1.5}.
#' @param cluster_scale A character string specifying which node
#'  aesthetic is overridden by cluster membership. One of
#'  \code{"fill"}, \code{"color"}, or \code{"shape"}. The value is
#'  matched via \code{\link{match.arg}}; default is \code{"fill"}.
#' @param node_size_range A numeric vector of length 2 giving the
#'  minimum and maximum node size (in ggplot2 point units) when
#'  \code{node_size_by} is a column name. Default \code{c(5, 20)}.
#' @param link_weight_range A numeric vector of length 2 giving the
#'  minimum and maximum edge line width (in mm) when
#'  \code{link_weight_by} is a column name. Default
#'  \code{c(0.5, 5)}.
#' @param link_arrow_offset A numeric value (in points) specifying the
#'  offset distance for the arrow end cap from the target node.
#'  Prevents arrow heads from overlapping the node points. Only
#'  relevant when \code{directed = TRUE}. Default \code{20}.
#' @param link_curvature A numeric value controlling the curvature of
#'  the edges. \code{0} (default) produces straight edges; positive
#'  values curve them away from the direct path.
#' @param link_color_by A character string controlling how edge colour
#'  is determined. Options:
#'  \itemize{
#'    \item \code{"from"} (default) -- colour follows the source node's
#'          fill or colour aesthetic.
#'    \item \code{"to"} -- colour follows the target node's fill or
#'          colour.
#'    \item A column name from \code{links} -- colour is mapped
#'          directly to that column.
#'  }
#' @param link_color_name A character string for the edge colour legend
#'  title. Only used when \code{link_color_by} is a column name (not
#'  \code{"from"} or \code{"to"}). When \code{NULL} (default), the
#'  column name is used.
#' @param link_palette A character string specifying the palette for
#'  edge colours when they are mapped. When \code{link_color_by} is
#'  \code{"from"} or \code{"to"}, defaults to the node
#'  \code{palette}. Otherwise defaults to \code{"Set1"}.
#' @param link_palcolor A character vector specifying custom colours
#'  for the edge palette. When \code{link_color_by} is \code{"from"}
#'  or \code{"to"}, defaults to the node \code{palcolor}. Otherwise
#'  defaults to \code{NULL}.
#' @param directed A logical value. When \code{TRUE}, edges are drawn
#'  with arrow heads and an end-cap offset. Default \code{TRUE}.
#' @param layout A character string or an \code{igraph_layout_spec}
#'  object specifying the node placement algorithm. Built-in shortcuts:
#'  \code{"circle"} (circular layout), \code{"tree"} (hierarchical
#'  tree), \code{"grid"} (grid layout). Any other string is prefixed
#'  with \code{"layout_with_"} and called as an igraph function (e.g.
#'  \code{"fr"} for Fruchterman--Reingold, \code{"kk"} for
#'  Kamada--Kawai). Default \code{"circle"}.
#' @param cluster A character string specifying the community detection
#'  algorithm. One of \code{"none"}, \code{"fast_greedy"},
#'  \code{"walktrap"}, \code{"edge_betweenness"}, \code{"infomap"}, or
#'  a custom clustering function from igraph. When not \code{"none"},
#'  cluster membership overrides the aesthetic selected by
#'  \code{cluster_scale}. Default \code{"none"}.
#' @param add_mark A logical value. When \code{TRUE} (and
#'  \code{cluster != "none"}), an enclosure mark is drawn around each
#'  cluster's nodes. Default \code{FALSE}.
#' @param mark_expand A \code{\link[grid]{unit}} object specifying the
#'  extra space around points within a cluster mark. Default
#'  \code{unit(10, "mm")}.
#' @param mark_type A character string specifying the mark geometry.
#'  One of \code{"hull"}, \code{"ellipse"}, \code{"rect"}, or
#'  \code{"circle"}, corresponding to ggforce's \code{geom_mark_hull},
#'  \code{geom_mark_ellipse}, \code{geom_mark_rect}, and
#'  \code{geom_mark_circle}. The value is matched via
#'  \code{\link{match.arg}}; default is \code{"hull"}.
#' @param mark_alpha A numeric value for the fill transparency of
#'  cluster marks. Default \code{0.1}.
#' @param mark_linetype A numeric or character value specifying the
#'  border line type of the cluster marks. Default \code{1} (solid).
#' @param add_label A logical value. When \code{TRUE} (default), node
#'  identifiers are drawn as repulsive text labels via
#'  \code{\link[ggrepel]{geom_text_repel}}.
#' @param label_size A numeric value for the font size of node labels.
#'  Scaled by the theme base size. Default \code{3}.
#' @param label_fg A character string specifying the text colour of
#'  node labels. Default \code{"white"}.
#' @param label_bg A character string specifying the background colour
#'  of node labels. Default \code{"black"}.
#' @param label_bg_r A numeric value specifying the background box
#'  radius (as a fraction of label height). Passed to
#'  \code{\link[ggrepel]{geom_text_repel}}'s \code{bg.r} argument.
#'  Default \code{0.1}.
#' @param arrow A \code{\link[ggplot2]{arrow}} object for the link
#'  arrow heads. Only used when \code{directed = TRUE}. Default is
#'  \code{arrow(type = "closed", length = unit(0.1, "inches"))}.
#' @param ... Not used.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches).
#' @keywords internal
#' @importFrom utils getFromNamespace
#' @importFrom dplyr %>% rename relocate
#' @importFrom ggplot2 aes geom_point scale_x_continuous scale_y_continuous
#' @importFrom ggplot2 scale_size_continuous guide_legend
#' @importFrom ggrepel geom_text_repel
#' @importFrom ggnewscale new_scale_color
NetworkAtomic <- function(
    links,
    nodes = NULL,
    from = NULL,
    from_sep = "_",
    to = NULL,
    to_sep = "_",
    node_by = NULL,
    node_by_sep = "_",
    link_weight_by = 2,
    link_weight_name = NULL,
    link_type_by = "solid",
    link_type_name = NULL,
    node_size_by = 15,
    node_size_name = NULL,
    node_color_by = "black",
    node_color_name = NULL,
    node_shape_by = 21,
    node_shape_name = NULL,
    node_fill_by = "grey20",
    node_fill_name = NULL,
    link_alpha = 1,
    node_alpha = 0.95,
    node_stroke = 1.5,
    cluster_scale = c("fill", "color", "shape"),
    node_size_range = c(5, 20),
    link_weight_range = c(0.5, 5),
    link_arrow_offset = 20,
    link_curvature = 0,
    link_color_by = "from",
    link_color_name = NULL,
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    link_palette = ifelse(link_color_by %in% c("from", "to"), palette, "Set1"),
    link_palcolor = if (link_color_by %in% c("from", "to")) palcolor else NULL,
    directed = TRUE,
    layout = "circle",
    cluster = "none",
    add_mark = FALSE,
    mark_expand = ggplot2::unit(10, "mm"),
    mark_type = c("hull", "ellipse", "rect", "circle"),
    mark_alpha = 0.1,
    mark_linetype = 1,
    add_label = TRUE,
    label_size = 3,
    label_fg = "white",
    label_bg = "black",
    label_bg_r = 0.1,
    arrow = ggplot2::arrow(
        type = "closed",
        length = ggplot2::unit(0.1, "inches")
    ),
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    aspect.ratio = 1,
    theme = "theme_this",
    theme_args = list(),
    legend.position = "right",
    legend.direction = "vertical",
    ...
) {
    cluster_scale <- match.arg(cluster_scale)

    base_size <- theme_args$base_size %||% 12
    text_size_scale <- base_size / 12

    if (is.character(nodes) && length(nodes) == 1 && startsWith(nodes, "@")) {
        nodes <- attr(links, substring(nodes, 2))
    }

    modify_list <- getFromNamespace("modify_list", "ggplot2")
    if (is.null(from)) {
        from <- "from"
        if (!from %in% colnames(links)) {
            colnames(links)[1] <- from
        }
    }
    if (is.null(to)) {
        to <- "to"
        if (!to %in% colnames(links)) {
            colnames(links)[2] <- to
        }
    }
    if (is.null(node_by) && !is.null(nodes)) {
        node_by <- "name"
        if (!node_by %in% colnames(nodes)) {
            colnames(nodes)[1] <- node_by
        }
    }

    from <- check_columns(
        links,
        from,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = from_sep
    )
    to <- check_columns(
        links,
        to,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = to_sep
    )
    links <- links %>%
        rename(from = from, to = to) %>%
        relocate(!!sym("from"), !!sym("to"))

    if (!is.null(nodes)) {
        node_by <- check_columns(
            nodes,
            node_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = node_by_sep
        )
        nodes <- nodes %>% rename(name = node_by) %>% relocate(!!sym("name"))
    }

    graph <- igraph::graph_from_data_frame(
        d = links,
        vertices = nodes,
        directed = directed
    )
    if (inherits(layout, "igraph_layout_spec")) {
        layout <- igraph::layout_(graph, layout)
    } else {
        if (layout %in% c("circle", "tree", "grid")) {
            layout <- switch(
                layout,
                "circle" = igraph::layout_in_circle(graph),
                "tree" = igraph::layout_as_tree(graph),
                "grid" = igraph::layout_on_grid(graph)
            )
        } else {
            lofun <- getFromNamespace(paste0("layout_with_", layout), "igraph")
            layout <- lofun(graph)
        }
    }

    df <- igraph::as_data_frame(graph, what = "both")
    df_nodes <- df$vertices
    df_nodes$x <- layout[, 1]
    df_nodes$y <- layout[, 2]
    df_edges <- df$edges
    rm(df)

    ## NODES
    node_layer_args <- list(
        mapping = list(aes(x = !!sym("x"), y = !!sym("y"))),
        stroke = node_stroke
    )

    if (is.numeric(node_size_by)) {
        node_layer_args$size <- node_size_by
        node_size_by_guide <- "none"
    } else {
        node_layer_args$mapping[[length(node_layer_args$mapping) + 1]] <- aes(
            size = !!sym(node_size_by)
        )
        node_size_by_guide <- "guide"
    }

    if (!node_color_by %in% colnames(df_nodes)) {
        node_layer_args$color <- node_color_by
        node_color_by_guide <- "none"
    } else {
        node_color_by <- check_columns(
            df_nodes,
            node_color_by,
            force_factor = TRUE
        )
        node_layer_args$mapping[[length(node_layer_args$mapping) + 1]] <- aes(
            color = !!sym(node_color_by)
        )
        node_color_by_guide <- "guide"
    }

    if (is.numeric(node_shape_by)) {
        node_layer_args$shape <- node_shape_by
        node_shape_by_guide <- "none"
    } else {
        node_shape_by <- check_columns(
            df_nodes,
            node_shape_by,
            force_factor = TRUE
        )
        node_layer_args$mapping[[length(node_layer_args$mapping) + 1]] <- aes(
            shape = as.factor(!!sym(node_shape_by))
        )
        node_shape_by_guide <- "guide"
    }

    if (!node_fill_by %in% colnames(df_nodes)) {
        node_layer_args$fill <- node_fill_by
        node_fill_by_guide <- "none"
    } else {
        node_fill_by <- check_columns(
            df_nodes,
            node_fill_by,
            force_factor = TRUE
        )
        node_layer_args$mapping[[length(node_layer_args$mapping) + 1]] <- aes(
            fill = !!sym(node_fill_by)
        )
        node_fill_by_guide <- "guide"
    }

    # LINKS
    link_layer_args <- list(
        alpha = link_alpha,
        check_overlap = TRUE,
        mapping = list()
    )
    if (isTRUE(directed)) {
        link_layer_args$arrow <- arrow
    }

    if (
        link_color_by %in%
            colnames(df_edges) &&
            !link_color_by %in% c("from", "to")
    ) {
        link_color_by_guide <- "guide"
        link_layer_args$mapping[[length(link_layer_args$mapping) + 1]] <- aes(
            color = !!sym(link_color_by)
        )
    } else {
        link_color_by_guide <- "none"
        if (node_shape_by %in% 21:25) {
            if (node_fill_by_guide == "none") {
                link_layer_args$color <- node_fill_by
            } else {
                link_layer_args$mapping[[
                    length(link_layer_args$mapping) + 1
                ]] <- aes(color = I(!!sym(paste0("node.", node_fill_by))))
                # map node_fill_by from nodes data to links data
                df_edges[[paste0("node.", node_fill_by)]] <- palette_this(
                    levels(df_nodes[[node_fill_by]]),
                    palette = link_palette,
                    palcolor = link_palcolor,
                    reverse = palreverse
                )[df_nodes[df_edges[[link_color_by]], node_fill_by]]
                graph <- igraph::graph_from_data_frame(
                    d = df_edges,
                    vertices = df_nodes,
                    directed = directed
                )
            }
        } else {
            if (node_color_by_guide == "none") {
                link_layer_args$color <- node_color_by
            } else {
                link_layer_args$mapping[[
                    length(link_layer_args$mapping) + 1
                ]] <- aes(color = I(!!sym(paste0("node.", node_color_by))))
                # map node_color_by from nodes data to links data
                df_edges[[paste0("node.", node_color_by)]] <- palette_this(
                    levels(df_nodes[[node_color_by]]),
                    palette = link_palette,
                    palcolor = link_palcolor,
                    reverse = palreverse
                )[df_nodes[df_edges[[link_color_by]], node_color_by]]
                graph <- igraph::graph_from_data_frame(
                    d = df_edges,
                    vertices = df_nodes,
                    directed = directed
                )
            }
        }
    }

    if (is.numeric(link_weight_by)) {
        link_layer_args$linewidth <- link_weight_by
        link_weight_by_guide <- "none"
    } else {
        link_layer_args$mapping[[length(link_layer_args$mapping) + 1]] <- aes(
            linewidth = !!sym(link_weight_by)
        )
        link_weight_by_guide <- "guide"
    }

    if (!link_type_by %in% colnames(df_edges)) {
        link_layer_args$linetype <- link_type_by
        link_type_by_guide <- "none"
    } else {
        if (
            utils::compareVersion(
                as.character(utils::packageVersion("ggplot2")),
                "4.0.0"
            ) !=
                0
        ) {
            # This is fixed in ggplot2 4.0.1
            link_layer_args$mapping[[
                length(link_layer_args$mapping) + 1
            ]] <- aes(linetype = factor(!!sym(link_type_by)))
            link_type_by_guide <- "guide"
        } else {
            warning(
                "Using `link_type_by` with ggplot2 == 4.0.0 (ggraph <= 2.2.2) is not supported. Set to 'solid'. See https://github.com/thomasp85/ggraph/issues/394 for details."
            )
            link_layer_args$linetype <- "solid"
            link_type_by_guide <- "none"
        }
    }

    if (isTRUE(directed)) {
        link_layer_args$end_cap <- ggraph::circle(link_arrow_offset, "pt")
    }
    link_loop_layer_args <- link_layer_args
    link_loop_layer_args$mapping[[
        length(link_loop_layer_args$mapping) + 1
    ]] <- aes(direction = (!!sym("from") - 1) * 360 / length(graph))
    # link_layer_args$data <- df_edges[df_edges$from != df_edges$to, , drop = FALSE]
    link_layer_args$strength <- link_curvature
    link_layer_args$mapping <- Reduce(modify_list, link_layer_args$mapping)
    link_loop_layer_args$mapping <- Reduce(
        modify_list,
        link_loop_layer_args$mapping
    )

    # Start building the plot
    p <- ggraph::ggraph(
        graph,
        layout = "manual",
        x = df_nodes$x,
        y = df_nodes$y
    )
    if (cluster != "none") {
        clfun <- getFromNamespace(paste0("cluster_", cluster), "igraph")
        clusters <- clfun(graph)
        df_nodes$cluster <- factor(
            paste0("c", clusters$membership),
            levels = paste0("c", unique(sort(clusters$membership)))
        )
        if (cluster_scale == "fill") {
            if (!identical(node_fill_by, "grey20")) {
                warning(
                    "`cluster_scale = 'fill'` overrides `node_fill_by` when 'cluster' is enabled"
                )
            }
            node_fill_by <- "cluster"
        } else if (cluster_scale == "color") {
            if (!identical(node_color_by, "black")) {
                warning(
                    "`cluster_scale = 'color'` overrides `node_color_by` when 'cluster' is enabled"
                )
            }
            node_color_by <- "cluster"
        } else if (cluster_scale == "shape") {
            if (!identical(node_shape_by, 21)) {
                warning(
                    "`cluster_scale = 'shape'` overrides `node_shape_by` when 'cluster' is enabled"
                )
            }
            node_shape_by <- "cluster"
        }

        mark_type <- match.arg(mark_type)
        mark_fun <- switch(
            mark_type,
            hull = ggforce::geom_mark_hull,
            ellipse = ggforce::geom_mark_ellipse,
            rect = ggforce::geom_mark_rect,
            circle = ggforce::geom_mark_circle
        )
        p <- p +
            mark_fun(
                data = df_nodes,
                mapping = aes(
                    x = !!sym("x"),
                    y = !!sym("y"),
                    color = !!sym("cluster"),
                    fill = !!sym("cluster")
                ),
                expand = mark_expand,
                alpha = mark_alpha,
                linetype = mark_linetype,
                show.legend = FALSE
            ) +
            scale_fill_manual(
                values = palette_this(
                    levels(df_nodes$cluster),
                    palette = palette,
                    palcolor = palcolor,
                    reverse = palreverse
                )
            ) +
            scale_color_manual(
                values = palette_this(
                    levels(df_nodes$cluster),
                    palette = palette,
                    palcolor = palcolor,
                    reverse = palreverse
                )
            ) +
            new_scale_fill() +
            new_scale_color()
    }

    ## Combine link layer
    p <- p +
        do_call(ggraph::geom_edge_arc, link_layer_args) +
        do_call(ggraph::geom_edge_loop, link_loop_layer_args)

    if (link_weight_by_guide == "guide") {
        p <- p +
            ggraph::scale_edge_width_continuous(
                range = link_weight_range,
                breaks = scales::pretty_breaks(n = 4),
                guide = guide_legend(
                    title = link_weight_name %||% link_weight_by,
                    order = 10
                )
            )
    }

    if (link_type_by_guide == "guide") {
        p <- p +
            ggraph::scale_edge_linetype_discrete(
                guide = guide_legend(
                    title = link_type_name %||% link_type_by,
                    order = 11
                )
            )
    }

    if (link_color_by_guide == "guide") {
        if (is.numeric(df_edges[[link_color_by]])) {
            p <- p +
                ggraph::scale_edge_color_gradientn(
                    n.breaks = 5,
                    colors = palette_this(
                        palette = link_palette,
                        palcolor = link_palcolor,
                        reverse = palreverse
                    ),
                    na.value = "grey80",
                    guide = ggraph::guide_edge_colorbar(
                        title = link_color_name %||% link_color_by,
                        frame.colour = "black",
                        ticks.colour = "black",
                        title.hjust = 0,
                        order = 12
                    )
                )
        } else {
            lc_values <- if (is.factor(df_edges[[link_color_by]])) {
                levels(df_edges[[link_color_by]])
            } else {
                unique(df_edges[[link_color_by]])
            }
            p <- p +
                ggraph::scale_edge_color_manual(
                    values = palette_this(
                        lc_values,
                        palette = link_palette,
                        palcolor = link_palcolor,
                        reverse = palreverse
                    ),
                    guide = guide_legend(
                        title = link_color_name %||% link_color_by,
                        order = 12
                    )
                )
        }
    }

    # Compose node layer
    node_layer_args$data <- df_nodes
    node_layer_args$mapping <- Reduce(modify_list, node_layer_args$mapping)
    p <- p + do_call(geom_point, node_layer_args)

    if (node_size_by_guide == "guide") {
        p <- p +
            scale_size_continuous(
                range = node_size_range,
                breaks = scales::pretty_breaks(n = 4),
                guide = guide_legend(
                    title = node_size_name %||% node_size_by,
                    order = 1,
                    override.aes = list(
                        size = scales::rescale(
                            sort(df_nodes[[node_size_by]]),
                            c(1, 6)
                        )
                    )
                )
            )
    }

    if (node_color_by_guide == "guide") {
        p <- p +
            scale_color_manual(
                values = palette_this(
                    levels(df_nodes[[node_color_by]]),
                    palette = palette,
                    palcolor = palcolor,
                    reverse = palreverse
                ),
                guide = guide_legend(
                    title = node_color_name %||% node_color_by,
                    order = 2,
                    override.aes = list(size = 4)
                )
            )
    }

    if (node_shape_by_guide == "guide") {
        p <- p +
            scale_shape_manual(
                guide = guide_legend(
                    title = node_shape_name %||% node_shape_by,
                    order = 3,
                    override.aes = list(size = 4)
                )
            )
    }

    if (node_fill_by_guide == "guide") {
        p <- p +
            scale_fill_manual(
                values = palette_this(
                    levels(df_nodes[[node_fill_by]]),
                    palette = palette,
                    palcolor = palcolor,
                    alpha = node_alpha,
                    reverse = palreverse
                ),
                guide = guide_legend(
                    title = node_fill_name %||% node_fill_by,
                    order = 4,
                    override.aes = list(size = 4)
                )
            )
    }

    ## NODE LABELS
    if (isTRUE(add_label)) {
        p <- p +
            geom_text_repel(
                data = df_nodes,
                mapping = aes(
                    x = !!sym("x"),
                    y = !!sym("y"),
                    label = !!sym("name")
                ),
                segment.color = "transparent",
                point.size = NA,
                max.overlaps = 100,
                color = label_fg,
                bg.color = label_bg,
                bg.r = label_bg_r,
                size = label_size * text_size_scale
            )
    }

    p <- p +
        scale_x_continuous(expand = c(0, .4)) +
        scale_y_continuous(expand = c(0, .4)) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% "",
            y = ylab %||% ""
        ) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            legend.key.size = unit(0.5, "cm"),
            legend.key.width = unit(0.8, "cm")
        )

    dims <- calculate_plot_dimensions(
        base_height = 5,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    p
}

#' Network
#'
#' @description
#' Draws a network graph from a links (edge list) data frame and an optional
#' nodes (vertex metadata) data frame. The graph is constructed via
#' \code{\link[igraph]{igraph}}, laid out with igraph layout algorithms, and
#' rendered with \code{\link[ggraph]{ggraph}}. Supports directed or
#' undirected edges, variable link widths/linetypes/colours, node
#' sizes/shapes/colours/fills, community detection with enclosure marks,
#' automatic node labels, and a wide range of layout options.
#'
#' When \code{links} (and optionally \code{nodes}) contain a
#' \code{split_by} column, separate sub-plots are generated for each split
#' level and combined via \code{\link[patchwork]{patchwork}}. Unlike most
#' other plot types, \code{Network} operates on two data frames; splitting
#' may affect both.
#'
#' @section split_by workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \strong{Column validation} -- The \code{split_by} column is
#'         validated in \code{links} via \code{\link{check_columns}},
#'         force-converted to a factor, and empty levels are dropped.
#'   \item \strong{Node split} -- If \code{split_nodes = TRUE} and
#'         \code{nodes} is provided, the same \code{split_by} column is
#'         validated in \code{nodes}. It must be identical in name to the
#'         links \code{split_by} or an error is raised. Empty levels are
#'         also dropped.
#'   \item \strong{Data splitting} -- The \code{links} data frame is
#'         split by the \code{split_by} levels into a named list,
#'         preserving factor level order.
#'   \item \strong{Attach node splits} -- If \code{split_nodes = TRUE},
#'         the \code{nodes} data frame is split identically. Each split's
#'         node data is attached as the \code{"nodes"} attribute on the
#'         corresponding links split.
#'   \item \strong{Dispatch to atomic} -- \code{\link{NetworkAtomic}} is
#'         called for each split. The \code{nodes} argument is passed as
#'         \code{"@nodes"} when \code{split_nodes = TRUE} so that it is
#'         extracted from the attribute. If \code{title} is a function, it
#'         receives the split level name for dynamic title generation.
#'   \item \strong{Combination} -- Results are combined via
#'         \code{\link{combine_plots}()} (when \code{combine = TRUE}) or
#'         returned as a named list of \code{ggplot} objects.
#'   \item \strong{Combined data} -- When \code{combine = TRUE}, the
#'         combined plot's \code{data} (\code{p$data}) exposes the node
#'         layout of every split, each row tagged with the
#'         \code{split_by} level. The \code{"graph"} attribute carries a
#'         combined graph of all splits' nodes and links, and the
#'         original link rows (tagged with the \code{split_by} column)
#'         are available in the \code{"edges"} attribute.
#' }
#'
#' @inheritParams common_args
#' @inheritParams NetworkAtomic
#' @param split_nodes A logical value. When \code{TRUE} and
#'  \code{split_by} is provided, the \code{nodes} data frame is split by
#'  the same \code{split_by} column in addition to the links. Both data
#'  frames must have a column with the same name as \code{split_by}.
#'  Default \code{FALSE}.
#' @return A \code{ggplot} object (no \code{split_by}), a
#'  \code{patchwork} object (\code{combine = TRUE}), or a named list of
#'  \code{ggplot} objects (\code{combine = FALSE}), each with
#'  \code{height} and \code{width} attributes in inches.
#' @export
#' @examples
#' \donttest{
#' # Create example data
#' actors <- data.frame(
#'   name = c("Alice", "Bob", "Cecil", "David", "Esmeralda"),
#'   age = c(48, 33, 45, 34, 21),
#'   shape = c(21, 22, 21, 22, 23),
#'   gender = c("F", "M", "F", "M", "F")
#' )
#' relations <- data.frame(
#'   from = c("Bob", "Cecil", "Cecil", "David", "David", "Esmeralda", "Bob", "Alice",
#'      "Cecil", "David"),
#'   to = c("Alice", "Bob", "Alice", "Alice", "Bob", "Alice", "Bob", "Alice", "Cecil",
#'      "David"),
#'   friendship = c(4, 5, 5, 2, 1, 1, 2, 1, 3, 4),
#'   type = c(1, 1, 1, 1, 1, 2, 2, 2, 2, 2)
#' )
#'
#' # Basic network
#' Network(relations, actors)
#'
#' # Blank theme with no coordinate axes
#' Network(relations, actors, theme = "theme_blank",
#'         theme_args = list(add_coord = FALSE))
#'
#' # Mapped aesthetics with custom layout
#' Network(relations, actors,
#'         link_weight_by = "friendship",
#'         node_size_by = "age",
#'         link_weight_name = "FRIENDSHIP",
#'         node_fill_by = "gender",
#'         link_color_by = "to",
#'         link_type_by = "type",
#'         node_color_by = "black",
#'         layout = "circle",
#'         link_curvature = 0.2)
#'
#' # Tree layout with clustering and marks
#' Network(relations, actors, layout = "tree",
#'         directed = FALSE, cluster = "fast_greedy",
#'         add_mark = TRUE)
#'
#' # Split by a column
#' Network(relations, actors, split_by = "type")
#' }
Network <- function(
    links,
    nodes = NULL,
    split_by = NULL,
    split_by_sep = "_",
    split_nodes = FALSE,
    from = NULL,
    from_sep = "_",
    to = NULL,
    to_sep = "_",
    node_by = NULL,
    node_by_sep = "_",
    link_weight_by = 2,
    link_weight_name = NULL,
    link_type_by = "solid",
    link_type_name = NULL,
    node_size_by = 15,
    node_size_name = NULL,
    node_color_by = "black",
    node_color_name = NULL,
    node_shape_by = 21,
    node_shape_name = NULL,
    node_fill_by = "grey20",
    node_fill_name = NULL,
    link_alpha = 1,
    node_alpha = 0.95,
    node_stroke = 1.5,
    cluster_scale = c("fill", "color", "shape"),
    node_size_range = c(5, 20),
    link_weight_range = c(0.5, 5),
    link_arrow_offset = 20,
    link_curvature = 0,
    link_color_by = "from",
    link_color_name = NULL,
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    link_palette = ifelse(link_color_by %in% c("from", "to"), palette, "Set1"),
    link_palcolor = if (link_color_by %in% c("from", "to")) palcolor else NULL,
    directed = TRUE,
    layout = "circle",
    cluster = "none",
    add_mark = FALSE,
    mark_expand = ggplot2::unit(10, "mm"),
    mark_type = c("hull", "ellipse", "rect", "circle"),
    mark_alpha = 0.1,
    mark_linetype = 1,
    add_label = TRUE,
    label_size = 3,
    label_fg = "white",
    label_bg = "black",
    label_bg_r = 0.1,
    arrow = ggplot2::arrow(
        type = "closed",
        length = ggplot2::unit(0.1, "inches")
    ),
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    aspect.ratio = 1,
    theme = "theme_this",
    theme_args = list(),
    legend.position = "right",
    legend.direction = "vertical",
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed = seed)
    theme <- process_theme(theme)
    if (is.null(split_by)) {
        split_nodes <- FALSE
    }

    l_split_by <- check_columns(
        links,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )
    if (!is.null(l_split_by)) {
        links[[l_split_by]] <- droplevels(links[[l_split_by]])
    }

    if (isTRUE(split_nodes) && !is.null(nodes)) {
        if (
            is.character(nodes) && length(nodes) == 1 && startsWith(nodes, "@")
        ) {
            nodes <- attr(links, substring(nodes, 2))
        }
        n_split_by <- check_columns(
            nodes,
            split_by,
            force_factor = TRUE,
            allow_multi = TRUE,
            concat_multi = TRUE,
            concat_sep = split_by_sep
        )
        if (!is.null(n_split_by)) {
            nodes[[n_split_by]] <- droplevels(nodes[[n_split_by]])
        }

        if (!identical(l_split_by, n_split_by)) {
            stop(
                "The `split_by` columns in `links` and `nodes` must be the same."
            )
        }
    }

    if (!is.null(split_by)) {
        linkss <- split(links, links[[l_split_by]])
        if (isTRUE(split_nodes) && !is.null(nodes)) {
            nodess <- split(nodes, nodes[[n_split_by]])
            nms <- names(nodess)
            linkss <
                lapply(nms, function(nm) {
                    dat <- linkss[[nm]]
                    attr(dat, "nodes") <- nodess[[nm]]
                    dat
                })
            names(linkss) <- nms
        } else {
            linkss <- linkss[levels(links[[l_split_by]])]
        }
    } else {
        linkss <- list(links)
        split_by <- names(linkss) <- "..."
    }

    if (isTRUE(split_nodes) && !is.null(nodes)) {
        nodes <- "@nodes"
    }

    plots <- lapply(names(linkss), function(nm) {
        default_title <- if (length(linkss) == 1 && identical(nm, "...")) {
            NULL
        } else {
            nm
        }
        if (is.function(title)) {
            title <- title(default_title)
        } else {
            title <- title %||% default_title
        }

        NetworkAtomic(
            links = linkss[[nm]],
            nodes = nodes,
            from = from,
            from_sep = from_sep,
            to = to,
            to_sep = to_sep,
            node_by = node_by,
            node_by_sep = node_by_sep,
            link_weight_by = link_weight_by,
            link_weight_name = link_weight_name,
            link_type_by = link_type_by,
            link_type_name = link_type_name,
            node_size_by = node_size_by,
            node_size_name = node_size_name,
            node_color_by = node_color_by,
            node_color_name = node_color_name,
            node_shape_by = node_shape_by,
            node_shape_name = node_shape_name,
            node_fill_by = node_fill_by,
            node_fill_name = node_fill_name,
            link_alpha = link_alpha,
            node_alpha = node_alpha,
            node_stroke = node_stroke,
            cluster_scale = cluster_scale,
            node_size_range = node_size_range,
            link_weight_range = link_weight_range,
            link_arrow_offset = link_arrow_offset,
            link_curvature = link_curvature,
            link_color_by = link_color_by,
            link_color_name = link_color_name,
            palette = palette,
            palcolor = palcolor,
            palreverse = palreverse,
            link_palette = link_palette,
            link_palcolor = link_palcolor,
            directed = directed,
            layout = layout,
            cluster = cluster,
            add_mark = add_mark,
            mark_expand = mark_expand,
            mark_type = mark_type,
            mark_alpha = mark_alpha,
            mark_linetype = mark_linetype,
            add_label = add_label,
            label_size = label_size,
            label_fg = label_fg,
            label_bg = label_bg,
            label_bg_r = label_bg_r,
            arrow = arrow,
            title = title,
            subtitle = subtitle,
            xlab = xlab,
            ylab = ylab,
            aspect.ratio = aspect.ratio,
            theme = theme,
            theme_args = theme_args,
            legend.position = legend.position,
            legend.direction = legend.direction,
            ...
        )
    })

    names(plots) <- names(linkss)

    combine_plots(
        plots,
        combine = combine,
        split_by = l_split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
# Network plot expects a links data frame with from/to columns
net_links <- data.frame(
    from = c("A", "A", "B", "C", "C", "D", "E", "E"),
    to   = c("B", "C", "D", "D", "E", "E", "A", "B"),
    weight = c(2, 3, 1, 4, 2, 1, 3, 2),
    type = factor(c("strong", "weak", "strong", "strong", "weak", "weak", "strong", "weak")),
    group = factor(c("G1", "G1", "G1", "G2", "G2", "G2", "G1", "G2"))
)

test_that("Network basic usage works", {
    p <- Network(net_links, from = "from", to = "to")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("Network with title and subtitle works", {
    p <- Network(net_links, from = "from", to = "to",
                 title = "Network Title", subtitle = "Subtitle")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "Network Title")
    expect_equal(p$labels$subtitle, "Subtitle")
})

test_that("Network with link_weight_by works", {
    p <- Network(net_links, from = "from", to = "to", link_weight_by = "weight")
    expect_s3_class(p, "ggplot")
})

test_that("Network with link_color_by = 'to' works", {
    p <- Network(net_links, from = "from", to = "to", link_color_by = "to")
    expect_s3_class(p, "ggplot")
})

test_that("Network with layout = 'fr' works", {
    p <- Network(net_links, from = "from", to = "to", layout = "fr")
    expect_s3_class(p, "ggplot")
})

test_that("Network with split_by returns patchwork", {
    p <- Network(net_links, from = "from", to = "to",
                 split_by = "group", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("Network with combine = FALSE returns list", {
    plots <- Network(net_links, from = "from", to = "to",
                     split_by = "group", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("Network with multiple split_by columns returns a list with one plot per combination", {
    plots <- suppressMessages(Network(net_links, from = "from", to = "to",
                                      split_by = c("group", "type"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("Network with multiple split_by columns returns combined plot", {
    p <- suppressMessages(Network(net_links, from = "from", to = "to",
                                  split_by = c("group", "type"), combine = TRUE))
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("Network split_by keeps link endpoints aligned with nodes when splits differ", {
    # G1 has 5 nodes (A, B, C, D, E) but G2 only 4 (A missing): links in
    # the combined plot must stay anchored to each split's own node
    # positions instead of resolving against the first split's layout.
    p <- Network(net_links, from = "from", to = "to",
                 split_by = "group", combine = TRUE)
    # The combined plot exposes the layout of every split, each row
    # tagged with its split level, plus a combined graph carrying all
    # nodes and links.
    expect_true("group" %in% colnames(p$data))
    expect_equal(nrow(p$data), 9) # 5 nodes (G1) + 4 nodes (G2)
    expect_equal(sort(unique(p$data$group)), c("G1", "G2"))
    g <- attr(p$data, "graph")
    expect_equal(igraph::vcount(g), 9)
    expect_equal(igraph::ecount(g), 8)
    # The combined links table is exposed too, tagged with the split
    # column.
    eg <- attr(p$data, "edges")
    expect_equal(nrow(eg), 8)
    expect_true(all(c("from", "to", "weight", "group") %in% colnames(eg)))
    expect_equal(sort(unique(eg$group)), c("G1", "G2"))
    b <- ggplot2::ggplot_build(p)
    # Expected node positions for the last split (G2): layout_in_circle
    # of the graph with nodes C, D, E, B.
    g2 <- igraph::graph_from_data_frame(net_links[net_links$group == "G2", c("from", "to")])
    pos <- igraph::layout_in_circle(g2)
    check_endpoint <- function(x, y) {
        any(apply(pos, 1, function(v) sqrt(sum((c(x, y) - v)^2))) < 1e-6)
    }
    # The edge layer is the one carrying xend/yend (interpolated arcs).
    edge_layer <- Filter(function(d) "xend" %in% colnames(d) && nrow(d) > 0, b$data)
    expect_length(edge_layer, 1)
    edges <- edge_layer[[1]]
    expect_equal(nrow(edges), 400) # 4 edges x 100 interpolated points
    for (g in unique(edges$group)) {
        d <- edges[edges$group == g, ]
        expect_true(check_endpoint(d$x[1], d$y[1]), info = paste("edge", g, "start"))
        expect_true(check_endpoint(d$xend[1], d$yend[1]), info = paste("edge", g, "end"))
    }
})

test_that("Network with directed = FALSE works", {
    p <- Network(net_links, from = "from", to = "to", directed = FALSE)
    expect_s3_class(p, "ggplot")
})

test_that("Network with add_label = FALSE works", {
    p <- Network(net_links, from = "from", to = "to", add_label = FALSE)
    expect_s3_class(p, "ggplot")
})

#' Atomic pie chart (internal)
#'
#' @description
#' Core implementation for drawing a single pie chart.  This is the
#' workhorse behind the exported \code{\link{PieChart}} function — it takes a
#' **single** data frame (no \code{split_by} support) and returns a
#' \code{ggplot} object.  The chart maps the proportion of each x-axis
#' category to the angle of a pie slice using
#' \code{\link[ggplot2]{coord_polar}()}, with optional faceting and
#' per-slice labels.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
#'         \code{ggplot2::ggplot} based on
#'         \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{Column resolution} — \code{x} is forced to factor;
#'         \code{y} is validated (numeric, optional).
#'   \item \strong{Count aggregation} — when \code{y = NULL}, the count of
#'         observations per (\code{x}, \code{facet_by}) combination is
#'         computed as a new \code{.y} column.  Factor levels are preserved
#'         after aggregation.
#'   \item \strong{Label resolution} — when \code{label = TRUE}, the label
#'         column is set to \code{y}.  The \code{label} column is then
#'         validated.
#'   \item \strong{Facet column resolution} — \code{facet_by} columns are
#'         validated as factors, allowing up to two columns.
#'   \item \strong{NA / empty-level handling} —
#'         \code{\link{process_keep_na_empty}()} applies \code{keep_na} and
#'         \code{keep_empty} policies.  Per-column \code{keep_empty} settings
#'         are extracted for \code{x} and \code{facet_by}.  When more than one
#'         facet column is provided, their \code{keep_empty} values must be
#'         identical.
#'   \item \strong{Clockwise ordering} — when \code{clockwise = TRUE}
#'         (default), the levels of \code{x} are reversed so that the first
#'         slice starts from the top and proceeds clockwise.  The data is then
#'         sorted by the (possibly reversed) \code{x} factor levels.
#'   \item \strong{Position calculation} — if faceted, grouping by facet
#'         variables; computes cumulative sums (\code{csum}) and label
#'         midpoint positions (\code{pos}) for each slice.
#'   \item \strong{Colour mapping} — \code{\link{palette_this}()} assigns
#'         colours to all \code{x} levels, including \code{NA} (defaulting to
#'         \code{"grey80"}).
#'   \item \strong{Plot assembly} — the \code{ggplot} object is built with
#'         \code{geom_col(width = 1)} and
#'         \code{coord_polar(theta = "y")} to create the circular pie
#'         layout.  The fill scale uses \code{scale_fill_manual()} with
#'         per-slice colours; the \code{drop} argument is controlled by
#'         \code{keep_empty_x}.
#'   \item \strong{Labels} — when \code{label} is not \code{NULL},
#'         \code{\link[ggrepel]{geom_label_repel}()} adds text labels at
#'         the computed midpoint positions (\code{pos}).
#'   \item \strong{Dimension calculation} —
#'         \code{\link{calculate_plot_dimensions}()} computes plot height and
#'         width from \code{aspect.ratio} and legend metrics.  The resulting
#'         \code{height} / \code{width} attributes are stored on the
#'         \code{ggplot} object.
#'   \item \strong{Faceting} — \code{\link{facet_plot}()} wraps the plot
#'         with \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is
#'         provided, respecting the \code{keep_empty} setting for facet
#'         variables.
#' }
#'
#' @inheritParams common_args
#' @param x A character string specifying the column name for the x-axis
#'  (categories).  Must be character or factor.  Each unique value becomes a
#'  pie slice.
#' @param y A character string specifying the numeric column for the y-axis.
#'  When \code{NULL} (default), the count of observations in each
#'  (\code{x}, \code{facet_by}) combination is used and stored as
#'  \code{.y}.
#' @param label A character string specifying the column to use for slice
#'  labels.  \code{NULL} (default) hides labels.  When \code{TRUE}, the
#'  \code{y} values are used as labels.  When \code{y = NULL}, use
#'  \code{".y"} to label with the computed counts.
#' @param clockwise A logical value.  When \code{TRUE} (default), the pie
#'  slices are ordered clockwise starting from the top.  When \code{FALSE},
#'  slices are ordered counter-clockwise.
#' @keywords internal
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @importFrom rlang sym
#' @importFrom dplyr lead if_else mutate %>% group_by summarise n
#' @importFrom tidyr complete
#' @importFrom ggplot2 coord_polar geom_col scale_fill_manual labs element_blank guide_legend
#' @importFrom ggrepel geom_label_repel
PieChartAtomic <- function(
    data,
    x,
    y = NULL,
    label = y,
    clockwise = TRUE,
    keep_na = FALSE,
    keep_empty = FALSE,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    facet_by = NULL,
    facet_scales = "free_y",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    base_size <- theme_args$base_size %||% 12
    text_size_scale <- base_size / 12

    x <- check_columns(data, x, force_factor = TRUE)
    y <- check_columns(data, y)
    orig_data <- data
    if (is.null(y)) {
        data <- data %>%
            group_by(!!!syms(unique(c(x, facet_by)))) %>%
            summarise(.y = n(), .groups = "drop")

        for (col in unique(c(x, facet_by))) {
            data[[col]] <- factor(
                data[[col]],
                levels = levels(orig_data[[col]])
            )
        }
        y <- ".y"
    }
    if (isTRUE(label)) {
        label <- y
    }

    label <- check_columns(data, label)
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )

    data <- process_keep_na_empty(data, keep_na, keep_empty)
    keep_empty_x <- keep_empty[[x]]
    keep_empty_facet <- if (!is.null(facet_by)) {
        keep_empty[[facet_by[1]]]
    } else {
        NULL
    }
    if (length(facet_by) > 1) {
        stopifnot(
            "[PieChart] `keep_empty` for `facet_by` variables must be identical." = identical(
                keep_empty_facet,
                keep_empty[[facet_by[2]]]
            )
        )
    }

    # order the data by the levels of x
    if (isTRUE(clockwise)) {
        data[[x]] <- factor(data[[x]], levels = rev(levels(data[[x]])))
    }
    data <- data[order(data[[x]]), , drop = FALSE]

    if (!is.null(facet_by)) {
        data <- data %>%
            group_by(!!!syms(facet_by)) %>%
            mutate(
                csum = rev(cumsum(rev(!!sym(y)))),
                pos = !!sym(y) / 2 + lead(!!sym("csum"), 1),
                pos = if_else(is.na(!!sym("pos")), !!sym(y) / 2, !!sym("pos"))
            ) %>%
            ungroup()

        for (col in facet_by) {
            data[[col]] <- factor(
                data[[col]],
                levels = levels(orig_data[[col]])
            )
        }
    } else {
        data <- data %>%
            mutate(
                csum = rev(cumsum(rev(!!sym(y)))),
                pos = !!sym(y) / 2 + lead(!!sym("csum"), 1),
                pos = if_else(is.na(!!sym("pos")), !!sym(y) / 2, !!sym("pos"))
            )
    }
    rm(orig_data)

    x_vals <- levels(data[[x]])
    if (isTRUE(clockwise)) {
        x_vals <- rev(x_vals)
    }
    if (anyNA(data[[x]])) {
        x_vals <- c(x_vals, NA)
    }

    colors <- palette_this(
        x_vals,
        palette = palette,
        palcolor = palcolor,
        NA_keep = TRUE,
        reverse = palreverse
    )

    p <- ggplot(data, aes(x = "", y = !!sym(y), fill = !!sym(x))) +
        geom_col(
            width = 1,
            alpha = alpha,
            color = "white",
            show.legend = TRUE
        ) +
        # scale_fill_manual(name = x, drop = !keep_empty, values = colors, guide = guide_legend(reverse = clockwise)) +
        labs(title = title, subtitle = subtitle, x = xlab, y = ylab) +
        coord_polar(theta = "y") +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            axis.text = element_blank(),
            axis.ticks = element_blank(),
            axis.title = element_blank(),
            panel.border = element_blank()
        )

    if (isTRUE(keep_empty_x)) {
        p <- p +
            scale_fill_manual(
                name = x,
                values = colors,
                na.value = colors["NA"] %||% "grey80",
                breaks = rev(x_vals),
                limits = rev(x_vals),
                drop = FALSE,
                guide = guide_legend(reverse = clockwise)
            )
    } else {
        p <- p +
            scale_fill_manual(
                name = x,
                values = colors,
                na.value = colors["NA"] %||% "grey80",
                breaks = rev(x_vals),
                guide = guide_legend(reverse = clockwise)
            )
    }

    if (!is.null(label)) {
        p <- p +
            geom_label_repel(
                aes(
                    y = !!sym("pos"),
                    label = ifelse(
                        is.na(!!sym(label)),
                        "NA",
                        as.character(!!sym(label))
                    )
                ),
                nudge_x = 0.1,
                color = "grey20",
                fill = "#fcfcfc",
                size = text_size_scale * 3
            )
    }

    dims <- calculate_plot_dimensions(
        base_height = 4.5,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = length(x_vals),
        legend_nchar = max(nchar(as.character(x_vals)), na.rm = TRUE)
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction,
        drop = !isTRUE(keep_empty_facet)
    )
}

#' Pie chart
#'
#' @description
#' Draws a pie chart illustrating the numerical proportion of each group
#' relative to the whole.  Each slice corresponds to a level of the x-axis
#' variable and its angle is proportional to the y-axis value (or the
#' observation count when \code{y} is omitted).
#'
#' The function supports \strong{count aggregation} (omit \code{y} to plot
#' observation counts per x-category), \strong{slice labels} via
#' \code{ggrepel::geom_label_repel()}, clockwise or counter-clockwise slice
#' ordering, faceting, and splitting into separate sub-plots via
#' \code{split_by}.
#'
#' @section split_by workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \code{\link{validate_common_args}()} validates the \code{seed}
#'         and \code{facet_by} settings.
#'   \item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
#'         normalise the \code{keep_na} / \code{keep_empty} arguments for all
#'         columns (\code{x}, \code{split_by}, \code{facet_by}).
#'   \item \code{\link{process_theme}()} resolves the theme function.
#'   \item The \code{x} column is forced to factor; \code{y} is validated.
#'   \item The \code{split_by} column is validated and its NA / empty levels
#'         are processed via \code{\link{process_keep_na_empty}()}.  It is
#'         then removed from the per-column \code{keep_na} / \code{keep_empty}
#'         lists.
#'   \item The data frame is split by \code{split_by} (preserving level
#'         order).  If \code{split_by} is \code{NULL}, the data is wrapped in
#'         a single-element list with name \code{"..."}.
#'   \item Per-split \code{palette}, \code{palcolor},
#'         \code{legend.position}, and \code{legend.direction} are resolved
#'         via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
#'         and \code{\link{check_legend}()}.
#'   \item \code{\link{PieChartAtomic}()} is called for each split.  If
#'         \code{title} is a function, it receives the split level name and
#'         can generate dynamic titles.
#'   \item Results are combined via \code{\link{combine_plots}()} (when
#'         \code{combine = TRUE}) or returned as a named list.
#' }
#'
#' @export
#' @inheritParams PieChartAtomic
#' @inheritParams common_args
#' @param split_by The column(s) to split the data by and produce separate
#'  sub-plots.  Multiple columns are concatenated with \code{split_by_sep}.
#' @param split_by_sep A character string to separate concatenated
#'  \code{split_by} columns.  Default \code{"_"}.
#' @param seed A numeric seed for reproducibility.  Passed to
#'  \code{\link{validate_common_args}()}.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'  \code{patchwork} object.  When \code{FALSE}, returns a named list of
#'  individual \code{ggplot} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined layout
#'  (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param byrow Logical; fill the combined layout by row.  Default \code{TRUE}
#'  (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axes A character string specifying how axes should be treated across
#'  the combined layout (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axis_titles A character string specifying how axis titles should be
#'  treated across the combined layout.  Defaults to \code{axes}.
#' @param guides A character string specifying how guides (legends) should be
#'  collected across panels.  Default \code{"collect"} (passed to
#'  \code{\link{combine_plots}()}).
#' @param design A custom layout design for the combined plot (passed to
#'  \code{\link{combine_plots}()}).
#' @return A \code{ggplot} object, a \code{patchwork} object, or a named list
#'  of \code{ggplot} objects (when \code{combine = FALSE}), each with
#'  \code{height} and \code{width} attributes in inches.
#' @examples
#' \donttest{
#' data <- data.frame(
#'    x = factor(c("A", "B", "C", NA, "E", "F", "G", "H"), levels = LETTERS[1:8]),
#'    y = c(10, 8, 16, 4, 6, 12, 14, 2),
#'    group = factor(c("G1", "G1", NA, NA, "G3", "G3", "G4", "G4"),
#'        levels = c("G1", "G2", "G3", "G4")),
#'    facet = factor(c("F1", NA, "F3", "F4", "F1", NA, "F3", "F4"),
#'        levels = c("F1", "F2", "F3", "F4"))
#' )
#'
#' # Basic pie chart
#' PieChart(data, x = "x", y = "y")
#'
#' # Keep NA and empty levels
#' PieChart(data, x = "x", y = "y", keep_na = TRUE, keep_empty = TRUE)
#'
#' # Counter-clockwise ordering
#' PieChart(data, x = "x", y = "y", clockwise = FALSE)
#' PieChart(data, x = "x", y = "y", clockwise = FALSE,
#'          keep_na = TRUE, keep_empty = TRUE)
#'
#' # With slice labels
#' PieChart(data, x = "x", y = "y", label = "group")
#'
#' # Faceting
#' PieChart(data, x = "x", y = "y", facet_by = "facet")
#' PieChart(data, x = "x", y = "y", facet_by = c("facet", "group"),
#'     keep_empty = "level")
#' PieChart(data, x = "x", y = "y", facet_by = c("facet", "group"),
#'     keep_empty = TRUE)
#'
#' # Split into sub-plots
#' PieChart(data, x = "x", y = "y", split_by = "group")
#'
#' # Per-split palettes
#' PieChart(data, x = "x", y = "y", split_by = "group",
#'          palette = list(G1 = "Reds", G2 = "Blues", G3 = "Greens", G4 = "Purp"))
#'
#' # Y from count
#' PieChart(data, x = "group")
#'
#' # Y from count with label
#' PieChart(data, x = "group", label = ".y")
#' }
PieChart <- function(
    data,
    x,
    y = NULL,
    label = y,
    split_by = NULL,
    split_by_sep = "_",
    clockwise = TRUE,
    facet_by = NULL,
    facet_scales = "free_y",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    aspect.ratio = 1,
    keep_na = FALSE,
    keep_empty = FALSE,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    seed = 8525,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    keep_na <- check_keep_na(keep_na, c(x, split_by, facet_by))
    keep_empty <- check_keep_empty(keep_empty, c(x, split_by, facet_by))
    theme <- process_theme(theme)

    x <- check_columns(data, x, force_factor = TRUE)
    y <- check_columns(data, y)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            PieChartAtomic(
                datas[[nm]],
                x = x,
                y = y,
                label = label,
                split_by = split_by,
                clockwise = clockwise,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                keep_na = keep_na,
                keep_empty = keep_empty,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                alpha = alpha,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
data <- data.frame(
    category = factor(sample(c("A", "B", "C", "D"), 100, replace = TRUE)),
    value = runif(100, 1, 10),
    group = factor(rep(c("g1", "g2"), 50))
)

# PieChart tests
test_that("PieChart returns a ggplot for basic usage", {
    p <- PieChart(data, x = "category")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("PieChart works with y column for values", {
    p <- PieChart(data, x = "category", y = "value")
    expect_s3_class(p, "ggplot")
})

test_that("PieChart sets title", {
    p <- PieChart(data, x = "category", title = "Pie Title")
    expect_equal(p$labels$title, "Pie Title")
})

test_that("PieChart with split_by returns combined plot", {
    p <- PieChart(data, x = "category", split_by = "group", combine = TRUE)
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("PieChart with split_by and combine = FALSE returns list", {
    plots <- PieChart(data, x = "category", split_by = "group", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("PieChart with multiple split_by columns returns a list with one plot per combination", {
    data2 <- data
    data2$split2 <- factor(rep(c("s1", "s2"), each = 50))
    plots <- suppressWarnings(suppressMessages(PieChart(data2, x = "category",
                                                        split_by = c("group", "split2"), combine = FALSE)))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("PieChart with multiple split_by columns returns combined plot", {
    data2 <- data
    data2$split2 <- factor(rep(c("s1", "s2"), each = 50))
    p <- suppressWarnings(suppressMessages(PieChart(data2, x = "category",
                                                    split_by = c("group", "split2"), combine = TRUE)))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("PieChart with facet_by works", {
    p <- PieChart(data, x = "category", facet_by = "group")
    expect_s3_class(p, "ggplot")
})

test_that("PieChart with clockwise = FALSE works", {
    p <- PieChart(data, x = "category", clockwise = FALSE)
    expect_s3_class(p, "ggplot")
})

test_that("PieChart with custom palette works", {
    p <- PieChart(data, x = "category", palette = "Set2")
    expect_s3_class(p, "ggplot")
})

test_that("PieChart width/height attributes are numeric", {
    p <- PieChart(data, x = "category")
    expect_true(is.numeric(attr(p, "width")))
    expect_true(is.numeric(attr(p, "height")))
})

# RingPlot tests
test_that("RingPlot returns a ggplot for basic usage", {
    p <- RingPlot(data, x = "category")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("RingPlot works with y column", {
    p <- RingPlot(data, x = "category", y = "value")
    expect_s3_class(p, "ggplot")
})

test_that("RingPlot sets title", {
    p <- RingPlot(data, x = "category", title = "Ring Title")
    expect_equal(p$labels$title, "Ring Title")
})

test_that("RingPlot with split_by and combine = FALSE returns list", {
    plots <- RingPlot(data, x = "category", split_by = "group", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
})

test_that("RingPlot with multiple split_by columns returns a list with one plot per combination", {
    data2 <- data
    data2$split2 <- factor(rep(c("s1", "s2"), each = 50))
    plots <- suppressWarnings(suppressMessages(RingPlot(data2, x = "category",
                                                        split_by = c("group", "split2"), combine = FALSE)))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("RingPlot width/height attributes are numeric", {
    p <- RingPlot(data, x = "category")
    expect_true(is.numeric(attr(p, "width")))
    expect_true(is.numeric(attr(p, "height")))
})

#' Atomic jitter plot (internal)
#'
#' @description
#' Core implementation for drawing a single jittered point plot.  This is the
#' workhorse behind the exported \code{\link{JitterPlot}} function — it takes a
#' **single** data frame (no \code{split_by} support) and returns a
#' \code{ggplot} object.  The plot displays individual data points with random
#' jitter along the x-axis (and optionally the y-axis) to reveal the
#' distribution of values within each x category, making it especially useful
#' for visualising overlapping discrete data.
#'
#' The function provides extensive annotation features including point labelling
#' (via \code{\link[ggrepel]{geom_text_repel}} with automatic top-n selection
#' using a configurable distance metric), point highlighting, optional dodging
#' via \code{group_by}, and horizontal reference lines.  It supports both long
#' and wide input formats, quantile-based axis limits, and x-axis reordering by
#' y-value summaries (mean or median).
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
#'         \code{ggplot2::ggplot} based on
#'         \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{in_form handling} — when \code{in_form = "wide"}, the x
#'         columns are pivoted to long format via
#'         \code{\link[tidyr]{pivot_longer}()}, creating a key column
#'         \code{.x} and a value column \code{.y}.
#'   \item \strong{Column resolution} — \code{x}, \code{y}, \code{group_by},
#'         and \code{facet_by} are validated and transformed via
#'         \code{\link{check_columns}}.  Multi-column inputs for \code{x} and
#'         \code{group_by} are concatenated into single columns using their
#'         respective separators (\code{x_sep}, \code{group_by_sep}).
#'   \item \strong{NA / empty-level handling} — \code{\link{process_keep_na_empty}()}
#'         applies \code{keep_na} and \code{keep_empty} policies.  Per-column
#'         \code{keep_empty} settings are extracted for \code{x},
#'         \code{group_by}, and \code{facet_by} independently.  When
#'         \code{facet_by} has more than one column, the \code{keep_empty}
#'         values for all facet columns must be identical.
#'   \item \strong{sort_x resolution} — \code{match.arg()} normalises the
#'         \code{sort_x} argument, and per-group (\code{x}, \code{group_by},
#'         \code{facet_by}) means and medians of \code{y} are computed for
#'         subsequent reordering.
#'   \item \strong{y-axis limit computation} — \code{y_max} and \code{y_min}
#'         support raw numeric values or quantile strings (e.g. \code{"q95"}),
#'         which are resolved via \code{\link[stats]{quantile}()}.  These
#'         limits are used for fixed \code{coord_cartesian} / \code{coord_flip}.
#'   \item \strong{Highlight flag} — when \code{highlight} is provided, a
#'         logical column \code{.highlight} is created.  Values can be
#'         \code{TRUE} (all points), a numeric vector of row indices, a single
#'         character string parsed as an \R expression, or a character vector
#'         matched against row names.
#'   \item \strong{Label preparation} — labels are resolved from
#'         \code{label_by} or row names.  When \code{labels} is \code{NULL}
#'         and \code{nlabel > 0}, the top \code{nlabel} points per x-group are
#'         selected by descending order of \code{order_by} (default:
#'         \code{-({y}^2 + {size_by}^2)}, the radial distance from zero in
#'         y-size space, analogous to VolcanoPlot).  When \code{facet_by} is
#'         active, the ranking is nested within each facet panel.
#'   \item \strong{x-axis reordering} — when \code{sort_x} is not
#'         \code{"none"}, the x-axis factor levels are reordered by the
#'         per-group mean or median via \code{\link[stats]{reorder}()}.
#'   \item \strong{Flip handling} — when \code{flip = TRUE}, the x-axis
#'         factor levels are reversed and the aspect ratio is inverted.
#'   \item \strong{Colour assignment} — \code{\link{palette_this}()} assigns
#'         colours to the levels of \code{group_by} (or \code{x} when
#'         \code{group_by} is \code{NULL}), including \code{NA} levels which
#'         are relabelled as \code{"NA"}.
#'   \item \strong{Background layer} — when \code{add_bg = TRUE}, alternating
#'         background stripes are drawn via \code{\link{bg_layer}()} using the
#'         x-axis level order.
#'   \item \strong{Position jitter} — when \code{group_by} is \code{NULL},
#'         \code{\link[ggplot2]{position_jitter}()} is used.  When
#'         \code{group_by} is provided, \code{\link[ggplot2]{position_jitterdodge}()}
#'         is used so points are dodged by group before jittering.
#'   \item \strong{Pre-compute jittered positions} — a temporary \code{ggplot}
#'         is built and rendered via \code{ggplot_build()} to extract the
#'         actual jittered coordinates.  These are stored as \code{.x_jittered}
#'         and \code{.y_jittered} columns and used by the label layer because
#'         \code{geom_text_repel} does not natively respect
#'         \code{position_jitter} / \code{position_jitterdodge}.
#'   \item \strong{Point layer} — \code{geom_point()} is built with the
#'         appropriate aesthetic mapping: fill (for shapes 21–25) or colour
#'         (for other shapes) mapped to the grouping variable.  Border
#'         behaviour for shapes 21–25 is controlled by \code{border}
#'         (constant string, mapped to group colour, or \code{NA}).  When
#'         \code{size_by} is a column, size is mapped via \code{aes(size)}.
#'   \item \strong{Discrete colour/fill scales} —
#'         \code{scale_fill_manual()} or \code{scale_color_manual()} with the
#'         assigned palette colours.  When \code{keep_empty} is \code{TRUE}
#'         for the colour column, \code{drop = FALSE} and explicit
#'         \code{breaks} / \code{limits} preserve empty levels.  An additional
#'         \code{scale_color_manual()} is added for border colour when
#'         \code{border = TRUE}.
#'   \item \strong{Size scale} — when \code{size_by} is a column,
#'         \code{scale_size_area(max_size = 6)} is applied.  If
#'         \code{size_trans} is provided, the legend labels show the original
#'         (untransformed) values while the mapping uses transformed values.
#'   \item \strong{Highlight overlay} — if any points are highlighted, a
#'         second \code{geom_point()} layer is drawn on top using the
#'         \code{highlight_color}, \code{highlight_size}, and
#'         \code{highlight_alpha} settings.  The highlight layer does not
#'         affect the legend.
#'   \item \strong{Label layer} — if any points are selected for labelling,
#'         \code{\link[ggrepel]{geom_text_repel}()} is rendered at the
#'         pre-computed jittered coordinates.
#'   \item \strong{Horizontal reference lines} — when \code{add_hline} is
#'         provided, \code{geom_hline()} is drawn using the specified line
#'         aesthetics.
#'   \item \strong{Axes and labels} — \code{scale_x_discrete()} (with
#'         \code{drop = !isTRUE(keep_empty_x)}),
#'         \code{scale_y_continuous(trans = y_trans, n.breaks = y_nbreaks)},
#'         and \code{labs()} set the axis scale properties and titles.
#'   \item \strong{Coordinate system} — when \code{flip = TRUE},
#'         \code{coord_flip()} is used with (or without) fixed y-limits
#'         depending on free-scale faceting.  When \code{flip = FALSE},
#'         \code{coord_cartesian()} sets the y-axis limits unless free scales
#'         are in use.
#'   \item \strong{Theme} — \code{do_call(theme, theme_args)} applies the
#'         selected theme, with additional \code{aspect.ratio}, x-axis text
#'         rotation, legend position, and legend direction adjustments.
#'         Panel grid lines are styled: major y-axis lines (dashed grey) for
#'         non-flipped plots; major x-axis lines for flipped plots.
#'   \item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
#'         computes plot height and width from the x-axis category count,
#'         \code{aspect.ratio}, dodge group count, legend metrics, and label
#'         character width.  The minimum dimensions account for x-axis label
#'         length.  The resulting \code{height} / \code{width} attributes are
#'         stored on the \code{ggplot} object.  For flipped plots, the width
#'         is set to at least \code{max(width, height)}.
#'   \item \strong{Faceting} — \code{\link{facet_plot}()} wraps the plot
#'         with \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is
#'         provided, respecting the \code{keep_empty} setting for facet
#'         variables.
#' }
#'
#' @inheritParams common_args
#' @param x A character string specifying the column name for the x-axis.
#'  Must be character or factor.  Multiple columns can be provided; they are
#'  concatenated with \code{x_sep} as the separator.  When \code{in_form} is
#'  \code{"wide"}, the \code{x} columns are used as key columns and pivoted
#'  to long format (they are not concatenated).
#' @param x_sep A character string used to join multiple \code{x} columns.
#'  Default \code{"_"}.  Ignored when \code{x} is a single column or when
#'  \code{in_form} is \code{"wide"}.
#' @param y A character string specifying the numeric column for the y-axis.
#'  Required when \code{in_form} is \code{"long"} (default).  When
#'  \code{in_form} is \code{"wide"}, \code{y} is not required — the values
#'  under the \code{x} columns are used as y-values.
#' @param in_form A character string specifying the input data format.  Either
#'  \code{"long"} (default) or \code{"wide"}.  In \code{"long"} format,
#'  \code{x} and \code{y} are separate columns.  In \code{"wide"} format, the
#'  \code{x} columns contain the y-values and are pivoted to a key-value pair.
#' @param sort_x A character string controlling x-axis level reordering by
#'  y-value summaries.  One of \code{"none"}, \code{"mean_asc"},
#'  \code{"mean_desc"}, \code{"mean"}, \code{"median_asc"},
#'  \code{"median_desc"}, \code{"median"}.  \code{"none"} leaves the levels
#'  as-is.  \code{"mean_asc"} / \code{"mean"} sorts by ascending mean of y.
#'  \code{"mean_desc"} sorts by descending mean.  \code{"median_asc"} /
#'  \code{"median"} sorts by ascending median.  \code{"median_desc"} sorts by
#'  descending median.  Default: \code{"none"}.
#' @param flip A logical value.  When \code{TRUE}, the x and y axes are
#'  swapped via \code{coord_flip} and the x-axis factor levels are reversed.
#'  Dimension calculation accounts for the flip.  Default: \code{FALSE}.
#' @param group_by A character vector of column names for dodging the points.
#'  Each unique combination becomes a separate dodge group and the points are
#'  offset horizontally via \code{position_jitterdodge} to reduce overlap.
#'  Multiple columns are concatenated with \code{group_by_sep}.  When
#'  \code{NULL} (default), no dodging is applied — only jitter via
#'  \code{position_jitter}.
#' @param group_by_sep A character string used to join multiple
#'  \code{group_by} columns.  Default \code{"_"}.
#' @param group_name A character string for the dodge-group legend title.
#'  When \code{NULL} (default), the \code{group_by} column name is used.
#' @param add_bg A logical value.  When \code{TRUE}, alternating background
#'  stripes are drawn behind the points via \code{\link{bg_layer}()}, using
#'  the x-axis level order.  Default: \code{FALSE}.
#' @param bg_palette A character string specifying the palette for the
#'  background stripe colours.  Passed to \code{\link{bg_layer}()}.
#'  Default: \code{"stripe"}.
#' @param bg_palcolor A character vector of colours for the background
#'  stripes.  Passed to \code{\link{bg_layer}()}.  When \code{NULL} (default),
#'  colours are derived from \code{bg_palette}.
#' @param bg_alpha A numeric value in \code{[0, 1]} for the transparency of
#'  the background stripes.  Default: \code{0.2}.
#' @param shape A numeric value specifying the point shape (ggplot2 point
#'  shape codes).  Shapes 21–25 are filled shapes with borders; for these
#'  shapes the border behaviour is controlled by \code{border}.  Default:
#'  \code{21} (filled circle).
#' @param border Controls the border of points when the shape has a border
#'  (21–25).  If \code{TRUE}, the border colour follows the point fill colour
#'  (same as the group colour).  If a single colour string (e.g.
#'  \code{"black"}), uses that constant border colour for all points.  If
#'  \code{FALSE}, no border is drawn (\code{NA}).  Default: \code{"black"}.
#' @param size_by A numeric column name or a single numeric value controlling
#'  point size.  When a column name is provided, sizes are scaled using
#'  \code{scale_size_area(max_size = 6)} and a size legend is shown.  When a
#'  single numeric value, all points use that constant size.  Default:
#'  \code{2}.
#' @param size_name A character string for the size legend title.  When
#'  \code{NULL} (default) and \code{size_by} is a column, the column name is
#'  used.  Ignored when \code{size_by} is a single numeric value.
#' @param size_trans A function or a function name (as a string) to transform
#'  the \code{size_by} values for size mapping.  The transformed values
#'  determine the point size on the plot, but the legend labels show the
#'  original (untransformed) values.  When \code{NULL} (default), no
#'  transformation is applied.
#' @param alpha A numeric value in \code{[0, 1]} controlling point
#'  transparency.  Default: \code{1}.
#' @param jitter_width A numeric value controlling the amount of horizontal
#'  jitter (in x-axis units).  Passed to \code{position_jitter} /
#'  \code{position_jitterdodge}.  Default: \code{0.5}.
#' @param jitter_height A numeric value controlling the amount of vertical
#'  jitter (in y-axis units).  Passed to \code{position_jitter} /
#'  \code{position_jitterdodge}.  Default: \code{0}.
#' @param y_max,y_min Numeric values or quantile strings (e.g. \code{"q95"},
#'  \code{"q5"}) for y-axis limits used in \code{coord_cartesian} /
#'  \code{coord_flip}.  When \code{NULL} (default), the data range is used.
#'  When a quantile string, the corresponding quantile of the y-values is
#'  computed via \code{\link[stats]{quantile}()}.
#' @param y_trans A character string specifying a transformation for the
#'  y-axis (e.g. \code{"log10"}, \code{"sqrt"}).  Passed to
#'  \code{\link[ggplot2]{scale_y_continuous}}.  Default: \code{"identity"}.
#' @param y_nbreaks A numeric value hinting at the number of break intervals
#'  for the y-axis.  Passed to \code{scale_y_continuous}.  Default: \code{4}.
#' @param labels A vector of row names or row indices specifying which points
#'  to label.  When \code{NULL} (default) and \code{nlabel > 0}, the top
#'  \code{nlabel} points per x-group are selected automatically.
#' @param label_by A character string naming a column whose values are used
#'  as label text.  When \code{NULL} (default), row names are used as labels.
#' @param nlabel An integer specifying the number of points to label per
#'  x-group when \code{labels} is \code{NULL}.  Points are selected by
#'  descending order of \code{order_by}.  Default: \code{5}.  Set to
#'  \code{0} to suppress automatic labelling.
#' @param order_by A string expression passed to \code{\link[dplyr]{arrange}()}
#'  to determine which points are labelled.  Evaluated within each x-group
#'  (and facet panel when \code{facet_by} is set).  Default:
#'  \code{"-({y}^2 + {size_by}^2)"}, which selects points farthest from the
#'  origin in y-size radial distance, analogous to VolcanoPlot.
#' @param label_size,label_fg,label_bg,label_bg_r Label aesthetics for
#'  \code{\link[ggrepel]{geom_text_repel}}.  \code{label_size}: text size
#'  (default \code{3}).  \code{label_fg}: text colour (default
#'  \code{"black"}).  \code{label_bg}: background (halo) colour for the label
#'  text (default \code{"white"}).  \code{label_bg_r}: background border
#'  radius (default \code{0.1}).
#' @param add_hline One or more numeric values specifying y-values at which
#'  to draw horizontal reference lines.  When \code{NULL} (default), no
#'  reference lines are drawn.
#' @param hline_type A character string specifying the line type for the
#'  horizontal reference line(s).  Default: \code{"solid"}.
#' @param hline_width A numeric value specifying the line width for the
#'  horizontal reference line(s).  Default: \code{0.5}.
#' @param hline_color A character string specifying the colour for the
#'  horizontal reference line(s).  Default: \code{"black"}.
#' @param hline_alpha A numeric value in \code{[0, 1]} specifying the alpha
#'  (transparency) for the horizontal reference line(s).  Default: \code{1}.
#' @param highlight A specification of which points to highlight.  Can be:
#'  \code{TRUE} (highlight all points), a numeric vector of row indices, a
#'  single character string parsed as an \R expression, or a character vector
#'  of row names.  When a point is highlighted, an overlay \code{geom_point}
#'  is drawn on top with \code{highlight_color}, \code{highlight_size}, and
#'  \code{highlight_alpha}.  Default: \code{NULL} (no highlighting).
#' @param highlight_color A character string specifying the colour of
#'  highlighted points.  Default: \code{"red2"}.
#' @param highlight_size A numeric value specifying the size of highlighted
#'  points.  Default: \code{1}.
#' @param highlight_alpha A numeric value in \code{[0, 1]} specifying the
#'  transparency of highlighted points.  Default: \code{1}.
#' @param raster A logical value. If `TRUE`, points are rendered via
#'  `scattermore::geom_scattermore()` for efficient rasterised plotting.
#'  Default is `NULL`, which auto-enables when `nrow(data) > 1e3`.
#'  Ignored (with a warning) when `size_by` is mapped to a column, since
#'  scattermore cannot vary the point size per point.
#' @param raster_dpi A numeric vector of length 2 `[x_dpi, y_dpi]` specifying the raster
#'  resolution in pixels. Passed to `scattermore::geom_scattermore(pixels = ...)`.
#'  Default is `c(512, 512)`. If a single value is provided it is recycled to both dimensions.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @keywords internal
#' @importFrom stats median quantile
#' @importFrom rlang sym syms parse_expr %||%
#' @importFrom dplyr mutate ungroup first
#' @importFrom ggplot2 geom_point labs theme element_line element_text coord_flip
#' @importFrom ggplot2 position_jitterdodge scale_fill_manual scale_color_manual scale_y_continuous scale_size_area guide_legend guide_colorbar
#' @importFrom ggrepel geom_text_repel
JitterPlotAtomic <- function(
    data,
    x,
    x_sep = "_",
    y = NULL,
    in_form = c("long", "wide"),
    keep_na = FALSE,
    keep_empty = FALSE,
    sort_x = c(
        "none",
        "mean_asc",
        "mean_desc",
        "mean",
        "median_asc",
        "median_desc",
        "median"
    ),
    flip = FALSE,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    x_text_angle = 0,
    order_by = "-({y}^2 + {size_by}^2)",
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    aspect.ratio = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    shape = 21,
    border = "black",
    size_by = 2,
    size_name = NULL,
    size_trans = NULL,
    y_nbreaks = 4,
    jitter_width = 0.5,
    jitter_height = 0,
    y_max = NULL,
    y_min = NULL,
    y_trans = "identity",
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    add_hline = NULL,
    hline_type = "solid",
    hline_width = 0.5,
    hline_color = "black",
    hline_alpha = 1,
    labels = NULL,
    label_by = NULL,
    nlabel = 5,
    label_size = 3,
    label_fg = "black",
    label_bg = "white",
    label_bg_r = 0.1,
    highlight = NULL,
    highlight_color = "red2",
    highlight_size = 1,
    highlight_alpha = 1,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525,
    raster = NULL,
    raster_dpi = c(512, 512),
    ...
) {
    set.seed(seed)
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    in_form <- match.arg(in_form)
    if (in_form == "wide") {
        data <- data %>%
            tidyr::pivot_longer(cols = x, names_to = ".x", values_to = ".y")
        x <- ".x"
        y <- ".y"
    }

    x <- check_columns(
        data,
        x,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = x_sep
    )
    y <- check_columns(data, y)
    group_by <- check_columns(
        data,
        group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = group_by_sep
    )
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )

    data <- process_keep_na_empty(data, keep_na, keep_empty)
    keep_empty_x <- keep_empty[[x]]
    keep_empty_group <- if (!is.null(group_by)) keep_empty[[group_by]] else NULL
    keep_empty_facet <- if (!is.null(facet_by)) {
        keep_empty[[facet_by[1]]]
    } else {
        NULL
    }
    if (length(facet_by) > 1) {
        stopifnot(
            "[JitterPlot] `keep_empty` for `facet_by` variables must be identical." = identical(
                keep_empty_facet,
                keep_empty[[facet_by[2]]]
            )
        )
    }

    sort_x <- match.arg(sort_x)
    data <- data %>%
        dplyr::group_by(!!!syms(unique(c(x, group_by, facet_by)))) %>%
        mutate(.y_mean = mean(!!sym(y)), .y_median = median(!!sym(y))) %>%
        ungroup()

    # keep factor levels
    for (col in unique(c(x, group_by, facet_by))) {
        data[[col]] <- factor(data[[col]], levels = levels(data[[col]]))
    }

    values <- data[[y]][is.finite(data[[y]])]
    if (is.character(y_max)) {
        q_max <- as.numeric(sub("(^q)(\\d+)", "\\2", y_max)) / 100
        y_max_use <- stats::quantile(values, q_max, na.rm = TRUE)
    } else {
        y_max_use <- max(values, na.rm = TRUE)
    }
    if (is.null(y_min)) {
        y_min_use <- min(values, na.rm = TRUE)
    } else if (is.character(y_min)) {
        q_min <- as.numeric(sub("(^q)(\\d+)", "\\2", y_min)) / 100
        y_min_use <- stats::quantile(values, q_min, na.rm = TRUE)
    } else {
        y_min_use <- y_min
    }
    rm(values)

    # Highlight flag
    if (!is.null(highlight)) {
        if (isTRUE(highlight)) {
            data$.highlight <- TRUE
        } else if (is.numeric(highlight)) {
            data$.highlight <- 1:nrow(data) %in% highlight
        } else if (is.character(highlight) && length(highlight) == 1) {
            data <- dplyr::mutate(data, .highlight = !!parse_expr(highlight))
        } else if (is.null(rownames(data))) {
            stop(
                "No row names in the data, please provide a vector of indexes to highlight."
            )
        } else {
            data$.highlight <- rownames(data) %in% highlight
        }
    } else {
        data$.highlight <- FALSE
    }

    # Labels (similar to VolcanoPlot)
    # Labels using distance = y^2 + size^2
    data$.label <- if (is.null(label_by)) rownames(data) else data[[label_by]]
    data$.show_label <- FALSE
    if (!is.null(labels)) {
        data[labels, ".show_label"] <- TRUE
    } else if (nlabel > 0) {
        if (!is.null(facet_by)) {
            data <- data %>%
                dplyr::group_by(!!!syms(facet_by), !!sym(x)) %>%
                dplyr::arrange(!!rlang::parse_expr(glue::glue(order_by))) %>%
                dplyr::mutate(.show_label = dplyr::row_number() <= nlabel) %>%
                dplyr::ungroup()
        } else {
            data <- data %>%
                dplyr::group_by(!!sym(x)) %>%
                dplyr::arrange(!!rlang::parse_expr(glue::glue(order_by))) %>%
                dplyr::mutate(.show_label = dplyr::row_number() <= nlabel) %>%
                dplyr::ungroup()
        }
        for (col in unique(c(x, facet_by))) {
            data[[col]] <- factor(data[[col]], levels = levels(data[[col]]))
        }
    }
    data <- as.data.frame(data)

    if (sort_x == "mean" || sort_x == "mean_asc") {
        data[[x]] <- stats::reorder(data[[x]], data$.y_mean)
    } else if (sort_x == "mean_desc") {
        data[[x]] <- stats::reorder(data[[x]], -data$.y_mean)
    } else if (sort_x == "median" || sort_x == "median_asc") {
        data[[x]] <- stats::reorder(data[[x]], data$.y_median)
    } else if (sort_x == "median_desc") {
        data[[x]] <- stats::reorder(data[[x]], -data$.y_median)
    }

    if (isTRUE(flip)) {
        data[[x]] <- factor(data[[x]], levels = rev(levels(data[[x]])))
        aspect.ratio <- 1 / aspect.ratio
        if (length(aspect.ratio) == 0 || is.na(aspect.ratio)) {
            aspect.ratio <- NULL
        }
    }

    color_col <- ifelse(is.null(group_by), x, group_by)
    keep_empty_col <- if (color_col == x) keep_empty_x else keep_empty_group
    col_levels <- levels(data[[color_col]])
    if (anyNA(data[[color_col]])) {
        col_levels <- c(col_levels, NA)
    }
    colors <- palette_this(
        col_levels,
        palette = palette,
        palcolor = palcolor,
        NA_keep = TRUE,
        reverse = palreverse
    )
    if (anyNA(col_levels)) {
        # To prevent ggrepel text segments being too long
        # It seem it thinks the NA is an empty area for it to place labels
        names(colors)[is.na(names(colors))] <- "NA"
        col_levels[is.na(col_levels)] <- "NA"
        levels(data[[color_col]]) <- col_levels
        data[[color_col]][is.na(data[[color_col]])] <- "NA"
    }
    # Similar when x has NA, but color_col is group
    if (color_col != x && anyNA(data[[x]])) {
        x_levels <- c(levels(data[[x]]), "NA")
        levels(data[[x]]) <- x_levels
        data[[x]][is.na(data[[x]])] <- "NA"
    }
    # Base
    p <- ggplot(data, aes(x = !!sym(x), y = !!sym(y)))
    if (isTRUE(add_bg)) {
        p <- p +
            bg_layer(
                data,
                x,
                isTRUE(keep_empty_x),
                bg_palette,
                bg_palcolor,
                bg_alpha,
                facet_by
            )
    }

    # Positioner (jitter + optional dodge)
    if (is.null(group_by)) {
        pos <- position_jitter(
            width = jitter_width,
            height = jitter_height,
            seed = seed
        )
    } else {
        pos <- position_jitterdodge(
            jitter.width = jitter_width,
            jitter.height = jitter_height,
            dodge.width = 0.9,
            seed = seed
        )
    }

    # Pre-calculate jittered positions for labels (geom_text_repel doesn't respect position adjustments)
    if (any(data$.show_label)) {
        # Create a temporary plot to extract jittered positions
        temp_data <- data
        temp_mapping <- aes(x = !!sym(x), y = !!sym(y))
        if (!is.null(group_by)) {
            # position_jitterdodge requires a visual aesthetic (fill/colour), not just group
            temp_mapping$fill <- aes(fill = !!sym(group_by))$fill
        }
        temp_plot <- ggplot2::ggplot(temp_data, temp_mapping) +
            geom_point(position = pos) +
            scale_x_discrete(drop = !isTRUE(keep_empty_x))
        temp_build <- ggplot2::ggplot_build(temp_plot)
        jittered_coords <- temp_build$data[[1]]
        data$.x_jittered <- jittered_coords$x
        data$.y_jittered <- jittered_coords$y
    }

    # Build point layer, color by x
    has_fill <- shape %in% 21:25
    raster <- raster %||% (nrow(data) > 1e3)
    if (isTRUE(raster) && !is.numeric(size_by)) {
        warning(
            "[JitterPlot] `raster` is ignored when `size_by` is mapped to a column; ",
            "falling back to vector points.",
            call. = FALSE
        )
        raster <- FALSE
    }
    if (isTRUE(raster)) {
        # scattermore can only stamp filled discs (no fill aesthetic, no
        # border): the colour rides the `color` aesthetic. A border is a
        # slightly larger disc stamped behind, with an opaque disc of the
        # panel background in between, so the border colour doesn't tint the
        # translucent interior. All layers share `pos`, so every disc gets
        # the same jitter offsets.
        dims_r <- calculate_plot_dimensions(
            base_height = 5,
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction
        )
        # scattermore's `pointsize` is a radius in raster pixels; convert the
        # ggplot point size (diameter in mm) so raster points match geom_point
        raster_pointsize <- max(
            1,
            round(size_by * raster_dpi[1] / (25.4 * dims_r$width))
        )
        if (length(raster_dpi) == 1) {
            raster_dpi <- rep(raster_dpi, 2)
        }
        # the border follows the vector default stroke (0.5 mm) through the
        # same conversion, so it keeps its vector proportion of the point
        border_px <- if (has_fill && !is.null(border) && !isFALSE(border)) {
            max(1, round(0.5 * raster_dpi[1] / (25.4 * dims_r$width)))
        } else {
            0
        }
        panel_fill <- do_call(theme, theme_args)$panel.background$fill %||% "white"
        fill_mapping <- aes(color = !!sym(color_col))
        if (!is.null(group_by)) {
            # required for position_jitterdodge to dodge by group
            fill_mapping <- Reduce(
                utils::getFromNamespace("modify_list", "ggplot2"),
                list(fill_mapping, aes(group = !!sym(group_by)))
            )
        }
        fill_layer <- scattermore::geom_scattermore(
            data = data,
            mapping = fill_mapping,
            pointsize = raster_pointsize,
            alpha = alpha,
            pixels = raster_dpi,
            position = pos
        )
        if (border_px == 0) {
            p <- p + fill_layer
        } else {
            disc_args <- list(
                data = data,
                pointsize = raster_pointsize + border_px,
                alpha = 1,
                pixels = raster_dpi,
                position = pos
            )
            if (isTRUE(border)) {
                disc_args$mapping <- aes(color = !!sym(color_col))
            } else {
                disc_args$color <- border
            }
            p <- p +
                do_call(scattermore::geom_scattermore, disc_args) +
                scattermore::geom_scattermore(
                    data = data,
                    color = panel_fill,
                    pointsize = raster_pointsize,
                    alpha = 1,
                    pixels = raster_dpi,
                    position = pos
                ) +
                fill_layer
        }
    } else {
        point_args <- list(
            shape = shape,
            position = pos,
            alpha = alpha
        )
        mapping <- list(aes())

        if (has_fill) {
            mapping[[length(mapping) + 1]] <- aes(fill = !!sym(color_col))
            # border handling
            if (isTRUE(border)) {
                mapping[[length(mapping) + 1]] <- aes(color = !!sym(color_col))
            } else if (is.character(border) && length(border) == 1) {
                point_args$color <- border
            } else {
                point_args$color <- NA
            }
        } else {
            # shapes without fill
            mapping[[length(mapping) + 1]] <- aes(color = !!sym(color_col))
        }

        # size handling
        if (is.numeric(size_by)) {
            point_args$size <- size_by
        } else {
            size_by <- check_columns(data, size_by)
            # transform size values while keeping original values for legend labels
            f <- if (is.null(size_trans)) {
                identity
            } else if (is.function(size_trans)) {
                size_trans
            } else {
                get(as.character(size_trans), inherits = TRUE)
            }
            data$.size_raw <- data[[size_by]]
            data$.size_mapped <- f(data$.size_raw)
            mapping[[length(mapping) + 1]] <- aes(size = !!sym(".size_mapped"))
        }

        # Group for dodging only (no legend)
        if (!is.null(group_by)) {
            mapping[[length(mapping) + 1]] <- aes(group = !!sym(group_by))
        }

        modify_list <- utils::getFromNamespace("modify_list", "ggplot2")
        point_args$mapping <- Reduce(modify_list, mapping)
        point_args$data <- data
        point_args$show.legend <- TRUE
        p <- p + do_call(geom_point, point_args)
    }

    # Discrete color/fill scales by x
    if (isTRUE(raster)) {
        # scattermore uses the colour aesthetic (no fill); keep the same
        # palette and legend title as the vector version
        if (isTRUE(keep_empty_col)) {
            p <- p +
                scale_color_manual(
                    name = color_col,
                    values = colors,
                    na.value = colors['NA'] %||% "grey80",
                    breaks = col_levels,
                    limits = col_levels,
                    drop = FALSE
                )
        } else {
            p <- p +
                scale_color_manual(
                    name = color_col,
                    values = colors,
                    na.value = colors['NA'] %||% "grey80"
                )
        }
    } else if (has_fill) {
        if (isTRUE(keep_empty_col)) {
            p <- p +
                scale_fill_manual(
                    name = color_col,
                    values = colors,
                    na.value = colors['NA'] %||% "grey80",
                    breaks = col_levels,
                    limits = col_levels,
                    drop = FALSE
                )
        } else {
            p <- p +
                scale_fill_manual(
                    name = color_col,
                    values = colors,
                    na.value = colors['NA'] %||% "grey80"
                )
        }
        if (isTRUE(border)) {
            p <- p +
                scale_color_manual(
                    values = colors,
                    guide = "none",
                    na.value = colors['NA'] %||% "grey80"
                )
        }
    } else {
        if (isTRUE(keep_empty_col)) {
            p <- p +
                scale_color_manual(
                    name = color_col,
                    values = colors,
                    na.value = colors['NA'] %||% "grey80",
                    breaks = col_levels,
                    limits = col_levels,
                    drop = FALSE
                )
        } else {
            p <- p +
                scale_color_manual(
                    name = color_col,
                    values = colors,
                    na.value = colors['NA'] %||% "grey80"
                )
        }
    }

    # Size scale when mapped
    if (!is.numeric(size_by)) {
        # compute breaks on raw, but use transformed positions for sizes
        f <- if (is.null(size_trans)) {
            identity
        } else if (is.function(size_trans)) {
            size_trans
        } else {
            get(as.character(size_trans), inherits = TRUE)
        }
        raw_vals <- data$.size_raw
        raw_breaks <- unique(scales::pretty_breaks(n = 4)(raw_vals))
        mapped_breaks <- tryCatch(f(raw_breaks), error = function(e) raw_breaks)
        p <- p +
            scale_size_area(
                max_size = 6,
                breaks = mapped_breaks,
                labels = raw_breaks
            ) +
            guides(
                size = guide_legend(
                    title = size_name %||% size_by,
                    override.aes = list(fill = "grey30", shape = shape),
                    order = 1
                )
            )
    }

    # Highlight overlay on top (does not affect legends)
    if (any(data$.highlight)) {
        hi_df <- data[data$.highlight, , drop = FALSE]
        if (has_fill) {
            p <- p +
                geom_point(
                    data = hi_df,
                    mapping = if (!is.null(group_by)) {
                        aes(x = !!sym(x), y = !!sym(y), group = !!sym(group_by))
                    } else {
                        aes(x = !!sym(x), y = !!sym(y))
                    },
                    shape = shape,
                    fill = highlight_color,
                    color = "transparent",
                    position = pos,
                    size = if (is.numeric(size_by)) {
                        highlight_size
                    } else {
                        highlight_size
                    },
                    alpha = highlight_alpha
                )
        } else {
            p <- p +
                geom_point(
                    data = hi_df,
                    mapping = if (!is.null(group_by)) {
                        aes(x = !!sym(x), y = !!sym(y), group = !!sym(group_by))
                    } else {
                        aes(x = !!sym(x), y = !!sym(y))
                    },
                    shape = shape,
                    color = highlight_color,
                    position = pos,
                    size = if (is.numeric(size_by)) {
                        highlight_size
                    } else {
                        highlight_size
                    },
                    alpha = highlight_alpha
                )
        }
    }

    # Labels layer
    if (any(data$.show_label)) {
        p <- p +
            geom_text_repel(
                data = data[data$.show_label, , drop = FALSE],
                mapping = aes(
                    x = !!sym(".x_jittered"),
                    y = !!sym(".y_jittered"),
                    label = !!sym(".label")
                ),
                color = label_fg,
                bg.color = label_bg,
                bg.r = label_bg_r,
                size = label_size,
                min.segment.length = 0,
                segment.color = "grey40",
                max.overlaps = 100
            )
    }
    # Optional horizontal reference lines
    if (!is.null(add_hline)) {
        p <- p +
            ggplot2::geom_hline(
                yintercept = add_hline,
                linetype = hline_type,
                linewidth = hline_width,
                color = hline_color,
                alpha = hline_alpha
            )
    }

    just <- calc_just(x_text_angle)
    p <- p +
        scale_x_discrete(drop = !isTRUE(keep_empty_x)) +
        scale_y_continuous(trans = y_trans, n.breaks = y_nbreaks) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% x,
            y = ylab %||% y
        )

    x_maxchars <- max(nchar(levels(data[[x]])))
    nx <- nlevels(data[[x]])
    nd <- ifelse(is.null(group_by), 1, nlevels(data[[group_by]]))
    facet_free <- !is.null(facet_by) &&
        (identical(facet_scales, "free") ||
            (!flip && identical(facet_scales, "free_y")) ||
            (flip && identical(facet_scales, "free_x")))

    if (isTRUE(flip)) {
        strip_position <- "top"
        p <- p +
            ggplot2::theme(
                strip.text.y = element_text(angle = 0),
                panel.grid.major.y = element_line(color = "grey", linetype = 2)
            )
        if (facet_free) {
            p <- p + coord_flip()
        } else {
            p <- p + coord_flip(ylim = c(y_min_use, y_max_use))
        }
    } else {
        strip_position <- "top"
        p <- p +
            ggplot2::theme(
                strip.text.x = element_text(angle = 0),
                panel.grid.major.x = element_line(color = "grey", linetype = 2)
            )
        if (!facet_free) {
            p <- p + ggplot2::coord_cartesian(ylim = c(y_min_use, y_max_use))
        }
    }

    p <- p +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            axis.text.x = element_text(
                angle = x_text_angle,
                hjust = just$h,
                vjust = just$v
            ),
            legend.position = legend.position,
            legend.direction = legend.direction
        )

    if (isTRUE(flip)) {
        label_min_width <- max(3, 2.2 + x_maxchars * 0.05)
        dims <- calculate_plot_dimensions(
            base_height = label_min_width,
            aspect.ratio = aspect.ratio,
            n_y = nx * nd,
            y_scale_factor = 0.5,
            legend.position = legend.position,
            legend.direction = legend.direction,
            legend_n = nd,
            legend_nchar = max(nchar(col_levels)),
            flip = TRUE
        )
        height <- dims$height
        width <- max(dims$width, label_min_width)
    } else {
        label_min_height <- 2 + x_maxchars * 0.05
        dims <- calculate_plot_dimensions(
            base_height = label_min_height,
            aspect.ratio = aspect.ratio,
            n_x = nx * nd,
            x_scale_factor = 0.5,
            legend.position = legend.position,
            legend.direction = legend.direction,
            legend_n = nd,
            legend_nchar = max(nchar(col_levels)),
            flip = FALSE
        )
        height <- max(dims$height, label_min_height)
        width <- dims$width
    }

    attr(p, "height") <- height
    attr(p, "width") <- max(width, height)

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        strip.position = strip_position,
        legend.position = legend.position,
        legend.direction = legend.direction,
        drop = !isTRUE(keep_empty_facet)
    )
}

#' Jitter plot
#'
#' @description
#' Draws a jittered point plot showing the distribution of numeric y-values
#' across a discrete x-axis.  Each data point is rendered with random jitter
#' along the x-axis (and optionally the y-axis) to reduce overplotting, making
#' it easy to visualise data density, spread, and outliers within each category.
#'
#' The function supports \strong{x-axis reordering} by y-value summaries
#' (mean or median), \strong{group dodging} via \code{group_by} to compare
#' subgroups side-by-side, \strong{point labelling} with automatic top-n
#' selection using a configurable distance metric (default: radial distance
#' \code{y^2 + size^2}), \strong{point highlighting} for emphasis, optional
#' \strong{horizontal reference lines}, and \strong{wide-format input} via
#' \code{in_form}.  Colour control, faceting, and splitting into separate
#' sub-plots via \code{split_by} are supported.
#'
#' @section split_by workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \code{\link{validate_common_args}()} validates the \code{seed}.
#'   \item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
#'         normalise the \code{keep_na} / \code{keep_empty} arguments for all
#'         relevant columns (\code{x}, \code{split_by}, \code{group_by},
#'         \code{facet_by}).
#'   \item The \code{split_by} column is validated via
#'         \code{\link{check_columns}()} with \code{force_factor = TRUE}.
#'         Multiple \code{split_by} columns are concatenated with
#'         \code{split_by_sep}.
#'   \item If \code{split_by} is not \code{NULL}, the data frame is split
#'         (preserving factor level order).  If \code{split_by} is
#'         \code{NULL}, the data is wrapped in a single-element list with
#'         name \code{"..."}.
#'   \item Per-split \code{palette}, \code{palcolor},
#'         \code{legend.position}, and \code{legend.direction} are resolved
#'         via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
#'         and \code{\link{check_legend}()}.
#'   \item \code{\link{JitterPlotAtomic}()} is called for each split.  If
#'         \code{title} is a function, it receives the split level name and
#'         can generate dynamic titles.
#'   \item Results are combined via \code{\link{combine_plots}()} (when
#'         \code{combine = TRUE}) or returned as a named list.
#' }
#'
#' @rdname jitterplot
#' @export
#' @inheritParams JitterPlotAtomic
#' @inheritParams common_args
#' @param split_by The column(s) to split the data by and produce separate
#'  sub-plots.  Multiple columns are concatenated with \code{split_by_sep}.
#' @param split_by_sep A character string to separate concatenated
#'  \code{split_by} columns.  Default \code{"_"}.
#' @param seed A numeric seed for reproducibility.  Passed to
#'  \code{\link{validate_common_args}()}.  Default: \code{8525}.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'  \code{patchwork} object.  When \code{FALSE}, returns a named list of
#'  individual \code{ggplot} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined layout
#'  (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param byrow Logical; fill the combined layout by row.  Default
#'  \code{TRUE} (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axes A character string specifying how axes should be treated
#'  across the combined layout (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axis_titles A character string specifying how axis titles should be
#'  treated across the combined layout.  Defaults to \code{axes}.
#' @param guides A character string specifying how guides (legends) should be
#'  collected across panels (passed to \code{\link{combine_plots}()}).
#' @param design A custom layout design for the combined plot (passed to
#'  \code{\link{combine_plots}()}).
#' @return A \code{ggplot} object (when \code{split_by} is \code{NULL}), a
#'  \code{patchwork} object (when \code{split_by} is provided and
#'  \code{combine = TRUE}), or a named list of \code{ggplot} objects (when
#'  \code{combine = FALSE}).  All \code{ggplot} objects have \code{height}
#'  and \code{width} attributes in inches.
#' @examples
#' \donttest{
#' set.seed(8525)
#' n <- 180
#' x <- factor(
#'     sample(c("A", NA, LETTERS[3:5]), n, replace = TRUE),
#'     levels = c("A", "B", "C", "D", "E")
#' )
#' group <- factor(
#'     sample(c("G1", NA, "G3"), n, replace = TRUE),
#'     levels = c("G1", "G2", "G3")
#' )
#' size <- rexp(n, rate = 1)
#' id <- paste0("pt", seq_len(n))
#' y <- rnorm(n, mean = ifelse(is.na(group), 0, ifelse(group == "G1", 0.5, -0.5))) +
#'      as.numeric(ifelse(is.na(x), 0, x))/10
#' df <- data.frame(
#'   x = x,
#'   y = y,
#'   group = group,
#'   size = size,
#'   id = id
#' )
#'
#' # Basic
#' JitterPlot(df, x = "x", y = "y")
#'
#' # Keep empty x levels and NA
#' JitterPlot(df, x = "x", y = "y", keep_na = TRUE, keep_empty = TRUE)
#'
#' # Map size with transform; legend shows original values
#' JitterPlot(df, x = "x", y = "y", size_by = "size", size_name = "Abundance",
#'     size_trans = sqrt, order_by = "-y^2")
#'
#' # Dodge by group and add a horizontal line
#' JitterPlot(df, x = "x", y = "y", group_by = "group",
#'   add_hline = 0, hline_type = "dashed", hline_color = "red2")
#'
#' # Keep the empty levels only for color coding
#' # Note the G3 is not blue (which is taken by unused level G2)
#' JitterPlot(df, x = "x", y = "y", group_by = "group",
#'     keep_na = TRUE, keep_empty = 'level')
#'
#' # Label top points by distance (y^2 + size^2)
#' JitterPlot(df, x = "x", y = "y", size_by = "size", label_by = "id", nlabel = 3)
#'
#' # Flip axes
#' JitterPlot(df, x = "x", y = "y", flip = TRUE)
#' }
JitterPlot <- function(
    data,
    x,
    x_sep = "_",
    y = NULL,
    in_form = c("long", "wide"),
    split_by = NULL,
    split_by_sep = "_",
    keep_na = FALSE,
    keep_empty = FALSE,
    sort_x = c(
        "none",
        "mean_asc",
        "mean_desc",
        "mean",
        "median_asc",
        "median_desc",
        "median"
    ),
    flip = FALSE,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    x_text_angle = 0,
    order_by = "-({y}^2 + {size_by}^2)",
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    aspect.ratio = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    shape = 21,
    border = "black",
    size_by = 2,
    size_name = NULL,
    size_trans = NULL,
    y_nbreaks = 4,
    jitter_width = 0.5,
    jitter_height = 0,
    y_max = NULL,
    y_min = NULL,
    y_trans = "identity",
    add_bg = FALSE,
    bg_palette = "stripe",
    bg_palcolor = NULL,
    bg_alpha = 0.2,
    add_hline = NULL,
    hline_type = "solid",
    hline_width = 0.5,
    hline_color = "black",
    hline_alpha = 1,
    labels = NULL,
    label_by = NULL,
    nlabel = 5,
    label_size = 3,
    label_fg = "black",
    label_bg = "white",
    label_bg_r = 0.1,
    highlight = NULL,
    highlight_color = "red2",
    highlight_size = 1,
    highlight_alpha = 1,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed)
    keep_na <- check_keep_na(keep_na, c(x, split_by, group_by, facet_by))
    keep_empty <- check_keep_empty(
        keep_empty,
        c(x, split_by, group_by, facet_by)
    )
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL
        datas <- split(data, data[[split_by]])
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }

    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            JitterPlotAtomic(
                datas[[nm]],
                x = x,
                x_sep = x_sep,
                y = y,
                in_form = in_form,
                keep_na = keep_na,
                keep_empty = keep_empty,
                sort_x = sort_x,
                flip = flip,
                group_by = group_by,
                group_by_sep = group_by_sep,
                group_name = group_name,
                x_text_angle = x_text_angle,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                alpha = alpha,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                shape = shape,
                border = border,
                order_by = order_by,
                size_by = size_by,
                size_name = size_name,
                size_trans = size_trans,
                y_nbreaks = y_nbreaks,
                jitter_width = jitter_width,
                jitter_height = jitter_height,
                y_max = y_max,
                y_min = y_min,
                y_trans = y_trans,
                add_bg = add_bg,
                bg_palette = bg_palette,
                bg_palcolor = bg_palcolor,
                bg_alpha = bg_alpha,
                add_hline = add_hline,
                hline_type = hline_type,
                hline_width = hline_width,
                hline_color = hline_color,
                hline_alpha = hline_alpha,
                labels = labels,
                label_by = label_by,
                nlabel = nlabel,
                label_size = label_size,
                label_fg = label_fg,
                label_bg = label_bg,
                label_bg_r = label_bg_r,
                highlight = highlight,
                highlight_color = highlight_color,
                highlight_size = highlight_size,
                highlight_alpha = highlight_alpha,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                seed = seed,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
data <- data.frame(
    x = factor(rep(c("A", "B", "C", "D"), each = 25), levels = c("A", "B", "C", "D")),
    y = rnorm(100),
    size_col = runif(100, 1, 10),
    group1 = factor(rep(c("g1", "g2"), 50)),
    group2 = factor(rep(c("f1", "f2"), each = 50))
)

test_that("JitterPlot returns a ggplot for basic usage", {
    p <- JitterPlot(data, x = "x", y = "y")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("JitterPlot sets title and labels", {
    p <- JitterPlot(data, x = "x", y = "y",
                    title = "Jitter", xlab = "X", ylab = "Y")
    expect_equal(p$labels$title, "Jitter")
    expect_equal(p$labels$x, "X")
    expect_equal(p$labels$y, "Y")
})

test_that("JitterPlot with group_by works", {
    p <- JitterPlot(data, x = "x", y = "y", group_by = "group1")
    expect_s3_class(p, "ggplot")
})

test_that("JitterPlot with split_by and combine = FALSE returns list", {
    plots <- JitterPlot(data, x = "x", y = "y", split_by = "group2", combine = FALSE)
    expect_true(is.list(plots))
    expect_length(plots, 2)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("JitterPlot with multiple split_by columns returns a list with one plot per combination", {
    plots <- suppressMessages(JitterPlot(data, x = "x", y = "y", split_by = c("group1", "group2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("JitterPlot with facet_by works", {
    p <- JitterPlot(data, x = "x", y = "y", facet_by = "group2")
    expect_s3_class(p, "ggplot")
})

test_that("JitterPlot with flip = TRUE works", {
    p <- JitterPlot(data, x = "x", y = "y", flip = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("JitterPlot with size_by column works", {
    p <- JitterPlot(data, x = "x", y = "y", size_by = "size_col")
    expect_s3_class(p, "ggplot")
})

test_that("JitterPlot with sort_x works", {
    p <- JitterPlot(data, x = "x", y = "y", sort_x = "mean_asc")
    expect_s3_class(p, "ggplot")
})

test_that("JitterPlot with add_bg = TRUE works", {
    p <- JitterPlot(data, x = "x", y = "y", add_bg = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("JitterPlot with highlight expression works", {
    p <- JitterPlot(data, x = "x", y = "y", highlight = "y > 1")
    expect_s3_class(p, "ggplot")
})

test_that("JitterPlot width/height attributes are numeric", {
    p <- JitterPlot(data, x = "x", y = "y")
    expect_true(is.numeric(attr(p, "width")))
    expect_true(is.numeric(attr(p, "height")))
})

test_that("JitterPlot raster = TRUE uses scattermore layers", {
    n_sc <- function(p) sum(vapply(p$layers, function(l) {
        inherits(l$geom, "GeomScattermore")
    }, logical(1)))
    # default border "black": border disc + panel-colour erase disc +
    # translucent fill dot
    p <- JitterPlot(data, x = "x", y = "y", raster = TRUE)
    expect_equal(n_sc(p), 3)
    expect_false(any(vapply(p$layers, function(l) {
        inherits(l$geom, "GeomPoint")
    }, logical(1))))
    # highlight stays as sharp vector points on top
    p2 <- JitterPlot(data, x = "x", y = "y", raster = TRUE,
                     highlight = c(1, 5, 10))
    expect_equal(n_sc(p2), 3)
    expect_true(any(vapply(p2$layers, function(l) {
        inherits(l$geom, "GeomPoint")
    }, logical(1))))
})

test_that("JitterPlot raster border follows border param", {
    n_sc <- function(p) sum(vapply(p$layers, function(l) {
        inherits(l$geom, "GeomScattermore")
    }, logical(1)))
    # border disabled or shape without fill: fill dots only
    p0 <- JitterPlot(data, x = "x", y = "y", raster = TRUE, border = FALSE)
    expect_equal(n_sc(p0), 1)
    p1 <- JitterPlot(data, x = "x", y = "y", raster = TRUE, shape = 16)
    expect_equal(n_sc(p1), 1)
    # constant colour border: opaque border disc with that colour, larger
    # than the translucent fill dot; the erase disc matches the panel
    p2 <- JitterPlot(data, x = "x", y = "y", raster = TRUE, border = "black")
    expect_equal(n_sc(p2), 3)
    disc <- p2$layers[vapply(p2$layers, function(l) {
        inherits(l$geom, "GeomScattermore") && is.null(l$mapping$colour) &&
            identical(l$aes_params$colour, "black") && identical(l$aes_params$alpha, 1)
    }, logical(1))]
    expect_length(disc, 1)
    fill <- p2$layers[vapply(p2$layers, function(l) {
        inherits(l$geom, "GeomScattermore") && !is.null(l$mapping$colour)
    }, logical(1))]
    expect_length(fill, 1)
    expect_true(disc[[1]]$geom_params$pointsize > fill[[1]]$geom_params$pointsize)
    erase <- p2$layers[vapply(p2$layers, function(l) {
        inherits(l$geom, "GeomScattermore") && is.null(l$mapping$colour) &&
            !identical(l$aes_params$colour, "black") && identical(l$aes_params$alpha, 1)
    }, logical(1))]
    expect_length(erase, 1)
})

test_that("JitterPlot raster works with group_by, split_by and facet_by", {
    n_sc <- function(p) sum(vapply(p$layers, function(l) {
        inherits(l$geom, "GeomScattermore")
    }, logical(1)))
    p <- JitterPlot(data, x = "x", y = "y", group_by = "group1", raster = TRUE)
    expect_equal(n_sc(p), 3)
    expect_s3_class(p, "ggplot")
    plots <- JitterPlot(data, x = "x", y = "y", raster = TRUE,
                        split_by = "group2", combine = FALSE)
    expect_length(plots, 2)
    expect_true(all(vapply(plots, n_sc, numeric(1)) == 3))
    p2 <- JitterPlot(data, x = "x", y = "y", raster = TRUE, facet_by = "group1")
    expect_s3_class(p2, "ggplot")
})

test_that("JitterPlot raster falls back to vector points when size_by is mapped", {
    expect_warning(
        p <- JitterPlot(data, x = "x", y = "y", raster = TRUE, size_by = "size_col"),
        "raster"
    )
    expect_true(any(vapply(p$layers, function(l) {
        inherits(l$geom, "GeomPoint")
    }, logical(1))))
    expect_false(any(vapply(p$layers, function(l) {
        inherits(l$geom, "GeomScattermore")
    }, logical(1))))
})

test_that("JitterPlot raster auto-enables above 1e3 points", {
    big <- do.call(rbind, rep(list(data), 12))[seq_len(1200), ]
    p <- JitterPlot(big, x = "x", y = "y")
    expect_true(any(vapply(p$layers, function(l) {
        inherits(l$geom, "GeomScattermore")
    }, logical(1))))
    # single raster_dpi value is recycled
    p2 <- JitterPlot(big, x = "x", y = "y", raster_dpi = 256)
    expect_s3_class(p2, "ggplot")
})

#' Common arguments for plots
#'
#' @name common_args
#'
#' @param data A data frame.
#' @param x A character string specifying the column name of the data frame to plot for the x-axis.
#' @param y A character string specifying the column name of the data frame to plot for the y-axis.
#' @param group_by Columns to group the data for plotting
#'   For those plotting functions that do not support multiple groups,
#'   They will be concatenated into one column, using \code{group_by_sep} as the separator
#' @param group_by_sep The separator for multiple group_by columns. See \code{group_by}
#' @param split_by The column(s) to split data by and plot separately.
#'   When \code{combine = TRUE}, the combined plot's data (\code{p$data})
#'   contains the rows of every split, tagged with the \code{split_by}
#'   column. For ggraph-based plots (e.g. \code{\link{Network}},
#'   \code{\link{ClustreePlot}}), the node layout of every split is
#'   exposed with the split column, and the original link rows (also
#'   tagged with the \code{split_by} column) are available as the
#'   \code{"edges"} attribute of the data.
#' @param split_by_sep The separator for multiple split_by columns. See \code{split_by}
#' @param keep_na A logical value or a character to replace the NA values in the data.
#'   It can also take a named list to specify different behavior for different columns.
#'   If TRUE or NA, NA values will be replaced with NA.
#'   If FALSE, NA values will be removed from the data before plotting.
#'   If a character string is provided, NA values will be replaced with the provided string.
#'   If a named vector/list is provided, the names should be the column names to apply the behavior to,
#'   and the values should be one of TRUE, FALSE, or a character string.
#'   Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
#'   for example, the `x`, `group_by`, `fill_by`, etc.
#' @param keep_empty One of FALSE, TRUE and "level". It can also take a named list to specify
#' different behavior for different columns. Without a named list, the behavior applies to the
#' categorical/character columns used on the plot, for example, the `x`, `group_by`, `fill_by`, etc.
#' \itemize{
#'   \item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
#'   \item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
#'   \item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
#'     But they will be assigned colors from the palette to maintain consistency across multiple plots.
#'    Alias: \code{levels}}
#' }
#' @param theme A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
#'   Default is "theme_this".
#' @param theme_args A list of arguments to pass to the theme function.
#' @param palette A character string specifying the palette to use.
#'   A named list or vector can be used to specify the palettes for different `split_by` values.
#' @param palcolor A character string specifying the color to use in the palette.
#'   A named list can be used to specify the colors for different `split_by` values.
#'   If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).
#' @param palreverse A logical value indicating whether to reverse the palette. Default is FALSE.
#' @param lower_quantile,upper_quantile Lower and upper quantiles for the continuous color/fill scale.
#'   The actual cutoffs are determined by these quantiles when \code{lower_cutoff} and
#'   \code{upper_cutoff} are \code{NULL}. Defaults: \code{lower_quantile = 0}, \code{upper_quantile = 0.99}.
#' @param lower_cutoff,upper_cutoff Explicit lower and upper cutoffs for the continuous color/fill scale.
#'   When \code{NULL} (the default), the cutoffs are determined by \code{lower_quantile} and
#'   \code{upper_quantile} via \code{\link[stats]{quantile}}. Values outside the
#'   \code{[lower_cutoff, upper_cutoff]} range are clamped (winsorized) to the nearest cutoff value.
#' @param alpha A numeric value specifying the transparency of the plot.
#' @param x_text_angle A numeric value specifying the angle of the x-axis text.
#' @param aspect.ratio A numeric value specifying the aspect ratio of the plot.
#' @param title A character string specifying the title of the plot.
#'  A function can be used to generate the title based on the default title.
#'  This is useful when split_by is used and the title needs to be dynamic.
#' @param subtitle A character string specifying the subtitle of the plot.
#' @param xlab A character string specifying the x-axis label.
#' @param ylab A character string specifying the y-axis label.
#' @param legend.position A character string specifying the position of the legend.
#'   if `waiver()`, for single groups, the legend will be "none", otherwise "right".
#' @param legend.direction A character string specifying the direction of the legend.
#' @param expand The values to expand the x and y axes. It is like CSS padding.
#'   When a single value is provided, it is used for both axes on both sides.
#'   When two values are provided, the first value is used for the top/bottom side and the second value is used for the left/right side.
#'   When three values are provided, the first value is used for the top side, the second value is used for the left/right side, and the third value is used for the bottom side.
#'   When four values are provided, the values are used for the top, right, bottom, and left sides, respectively.
#'   You can also use a named vector to specify the values for each side.
#'   When the axis is discrete, the values will be applied as 'add' to the 'expansion' function.
#'   When the axis is continuous, the values will be applied as 'mult' to the 'expansion' function.
#'   See also \url{https://ggplot2.tidyverse.org/reference/expansion.html}
#'
#' @param facet_by A character string specifying the column name of the data frame to facet the plot.
#'   Otherwise, the data will be split by \code{split_by} and generate multiple plots
#'   and combine them into one using \code{patchwork::wrap_plots}
#' @param facet_scales Whether to scale the axes of facets. Default is "fixed"
#'   Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}
#' @param facet_nrow A numeric value specifying the number of rows in the facet.
#'   When facet_by is a single column and facet_wrap is used.
#' @param facet_ncol A numeric value specifying the number of columns in the facet.
#'   When facet_by is a single column and facet_wrap is used.
#' @param facet_byrow A logical value indicating whether to fill the plots by row. Default is TRUE.
#' @param combine Whether to combine the plots into one when facet is FALSE. Default is TRUE.
#' @param nrow A numeric value specifying the number of rows in the facet.
#' @param ncol A numeric value specifying the number of columns in the facet.
#' @param byrow A logical value indicating whether to fill the plots by row.
#' @param axes A string specifying how axes should be treated. Passed to [`patchwork::wrap_plots()`].
#'   Only relevant when `split_by` is used and `combine` is TRUE.
#'   Options are:
#'   * 'keep' will retain all axes in individual plots.
#'   * 'collect' will remove duplicated axes when placed in the same run of rows or columns of the layout.
#'   * 'collect_x' and 'collect_y' will remove duplicated x-axes in the columns or duplicated y-axes in the rows respectively.
#' @param axis_titles A string specifying how axis titltes should be treated. Passed to [`patchwork::wrap_plots()`].
#'   Only relevant when `split_by` is used and `combine` is TRUE.
#'   Options are:
#'   * 'keep' will retain all axis titles in individual plots.
#'   * 'collect' will remove duplicated titles in one direction and merge titles in the opposite direction.
#'   * 'collect_x' and 'collect_y' control this for x-axis titles and y-axis titles respectively.
#' @param guides A string specifying how guides should be treated in the layout. Passed to [`patchwork::wrap_plots()`].
#'   Only relevant when `split_by` is used and `combine` is TRUE.
#'   Options are:
#'   * 'collect' will collect guides below to the given nesting level, removing duplicates.
#'   * 'keep' will stop collection at this level and let guides be placed alongside their plot.
#'   * 'auto' will allow guides to be collected if a upper level tries, but place them alongside the plot if not.
#' @param design Specification of the location of areas in the layout, passed to [`patchwork::wrap_plots()`].
#'   Only relevant when `split_by` is used and `combine` is TRUE. When specified, `nrow`, `ncol`, and `byrow` are ignored.
#'   See [`patchwork::wrap_plots()`] for more details.
#' @param seed The random seed to use. Default is 8525.
#' @param ... Additional arguments.
#' @keywords internal
NULL

#' Validate common arguments
#' @keywords internal
#' @inheritParams common_args
validate_common_args <- function(
    seed,
    facet_by = NULL,
    plot_type = NULL,
    split_by = NULL,
    split_by_sep = "_",
    group_by = NULL,
    group_by_sep = "_",
    facet_scales = "fixed",
    facet_nrow = NULL,
    facet_ncol = NULL,
    facet_byrow = TRUE,
    theme = "theme_this",
    theme_args = list(),
    palette = NULL,
    palcolor = NULL,
    expand = NULL,
    keep_na = FALSE,
    keep_empty = FALSE,
    alpha = 1,
    x_text_angle = 0,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    ...
) {
    if (!is.numeric(seed)) {
        stop("'seed' must be a numeric value.")
    }

    set.seed(seed)

    if (length(facet_by) > 2) {
        stop(
            "Too many columns specified in 'facet_by', only up to 2 columns are allowed."
        )
    }

    invisible(NULL)
}

#' Adjust_network_layout
#'
#' @keywords internal
#' @importFrom stats dist
adjust_network_layout <- function(
    graph,
    layout,
    width,
    height = 2,
    scale = 100,
    iter = 100
) {
    w <- width / 2
    layout[, 1] <- layout[, 1] / diff(range(layout[, 1])) * scale
    layout[, 2] <- layout[, 2] / diff(range(layout[, 2])) * scale

    adjusted <- c()
    # for (i in seq_len(iter)) {
    for (v in order(igraph::degree(graph), decreasing = TRUE)) {
        adjusted <- c(adjusted, v)
        neighbors <- as.numeric(igraph::neighbors(graph, igraph::V(graph)[v]))
        neighbors <- setdiff(neighbors, adjusted)
        x <- layout[v, 1]
        y <- layout[v, 2]
        r <- w[v]
        for (neighbor in neighbors) {
            nx <- layout[neighbor, 1]
            ny <- layout[neighbor, 2]
            ndist <- sqrt((nx - x)^2 + (ny - y)^2)
            nr <- w[neighbor]
            expect <- r + nr
            if (ndist < expect) {
                dx <- (x - nx) * (expect - ndist) / ndist
                dy <- (y - ny) * (expect - ndist) / ndist
                layout[neighbor, 1] <- nx - dx
                layout[neighbor, 2] <- ny - dy
                adjusted <- c(adjusted, neighbor)
            }
        }
    }
    # }

    for (i in seq_len(iter)) {
        dist_matrix <- as.matrix(dist(layout))
        nearest_neighbors <- apply(
            dist_matrix,
            2,
            function(x) which(x == min(x[x > 0])),
            simplify = FALSE
        )
        # nearest_neighbors <- apply(dist_matrix, 2, function(x) {
        #   head(order(x), 3)[-1]
        # }, simplify = FALSE)
        for (v in sample(seq_len(nrow(layout)))) {
            neighbors <- unique(nearest_neighbors[[v]])
            x <- layout[v, 1]
            y <- layout[v, 2]
            r <- w[v]
            for (neighbor in neighbors) {
                nx <- layout[neighbor, 1]
                ny <- layout[neighbor, 2]
                nr <- w[neighbor]
                if (abs(nx - x) < (r + nr) && abs(ny - y) < height) {
                    dx <- r + nr - (nx - x)
                    dy <- height - (ny - y)
                    if (sample(c(1, 0), 1) == 1) {
                        dx <- 0
                    } else {
                        dy <- 0
                    }
                    layout[neighbor, 1] <- nx - dx
                    layout[neighbor, 2] <- ny - dy
                }
            }
        }
    }
    return(layout)
}

#' Atomic enrichment map (internal)
#'
#' @description
#' Core implementation for drawing a single enrichment map -- a gene-set
#' similarity network where nodes represent enriched terms, node fill colour
#' encodes cluster membership, node size encodes the number of genes per
#' term, and edge thickness encodes the number of overlapping genes between
#' pairs of terms.  This is the workhorse behind the exported
#' \code{\link{EnrichMap}} function -- it takes a **single** data frame (no
#' \code{split_by} support) and returns a \code{ggplot} object.
#'
#' The function constructs an undirected graph from the term-term gene
#' overlap matrix, computes a force-directed layout, detects term clusters
#' via igraph community detection, and renders the result as a labelled
#' network plot with ggforce hull annotations around each cluster.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{ggplot dispatch} -- selects \code{gglogger::ggplot} or
#'         \code{ggplot2::ggplot} based on
#'         \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{Data format conversion} -- if \code{data} inherits from
#'         \code{"enrichResult"}, converts via \code{as.data.frame()}; if
#'         \code{in_form == "enrichr"}, calls
#'         \code{\link{prepare_enrichr_result}()} to rename columns and
#'         infer GeneRatio/BgRatio.
#'   \item \strong{Column validation} -- \code{\link{check_columns}()}
#'         verifies that \code{Description}, \code{GeneRatio},
#'         \code{pvalue}, \code{p.adjust}, and \code{geneID} are present.
#'   \item \strong{Top-term selection} -- when \code{top_term} is not
#'         \code{NULL}, selects the top N terms by the chosen \code{metric}
#'         via \code{dplyr::slice_min()}.
#'   \item \strong{Metric transformation} -- computes
#'         \eqn{-\log_{10}(metric)} as the node scoring variable.  Splits
#'         \code{geneID} on \code{"/"} to obtain per-term gene lists.
#'   \item \strong{ID assignment} -- uses the \code{ID} column if present;
#'         otherwise falls back to row names or generates synthetic IDs
#'         (\code{"GS1"}, \code{"GS2"}, \dots).
#'   \item \strong{Edge construction} -- creates all pairwise combinations
#'         (\code{utils::combn()}) of term IDs and computes the overlap
#'         count (intersection of geneID elements) as edge weight.  Edges
#'         with weight 0 are dropped.
#'   \item \strong{igraph graph construction} --
#'         \code{igraph::graph_from_data_frame()} builds an undirected
#'         graph from the edge list, with node attributes from the term
#'         data.
#'   \item \strong{Layout computation} -- dispatches to the chosen igraph
#'         layout function (\code{layout_with_*}) or one of the built-in
#'         shortcuts (\code{"circle"}, \code{"tree"}, \code{"grid"}).
#'   \item \strong{Cluster detection} -- runs the chosen igraph clustering
#'         algorithm (\code{cluster_*}) to group related terms into
#'         modules.
#'   \item \strong{Node coordinates} -- extracts vertex positions and
#'         cluster assignments into a data frame with \code{dim1},
#'         \code{dim2}, and \code{clusters} columns.
#'   \item \strong{Keyword extraction} -- when \code{show_keyword = TRUE},
#'         tokenizes \code{Description} text, filters words by
#'         \code{minchar} and \code{words_excluded}, scores remaining
#'         words by summed \code{metric} per cluster, and keeps the top
#'         \code{nlabel} keywords.  Otherwise, the term \code{Description}
#'         texts themselves are used as keywords.
#'   \item \strong{Gene keyword extraction} -- always computes per-cluster
#'         gene-level keywords (top \code{nlabel} genes by summed
#'         \code{metric}) for the feature legend mode.
#'   \item \strong{Mark layer} -- \code{ggforce::geom_mark_*()} (ellipse,
#'         rect, circle, or text) draws cluster hulls with labels
#'         containing the cluster ID and either term or feature keywords.
#'   \item \strong{Edge rendering} -- \code{ggplot2::geom_segment()} draws
#'         edges with line width proportional to overlap weight.
#'   \item \strong{Node rendering} -- \code{ggplot2::geom_point(shape = 21)}
#'         draws nodes sized by \code{Count} and filled by cluster
#'         membership.
#'   \item \strong{Scale configuration} -- \code{scale_size()} for node
#'         size, \code{scale_linewidth()} for edge width,
#'         \code{scale_fill_manual()} for cluster colours with custom
#'         legend labels (term keywords or gene keywords depending on
#'         \code{label}).
#'   \item \strong{Theme and dimensions} -- applies the resolved theme,
#'         sets aspect ratio and legend position, then calls
#'         \code{\link{calculate_plot_dimensions}()} to attach
#'         \code{height}/\code{width} attributes.
#' }
#'
#' @inheritParams common_args
#' @param data A data frame containing enrichment results in clusterProfiler
#'  format with at least the columns: \code{Description}, \code{GeneRatio},
#'  \code{pvalue}, \code{p.adjust}, and \code{geneID}.
#'  \itemize{
#'    \item \code{ID}, \code{qvalue}, \code{BgRatio}, and \code{Count} are
#'          optional.
#'    \item \code{Description} is the term description, displayed as the
#'          default keyword.
#'    \item \code{GeneRatio} is the fraction of input genes annotated to
#'          the term (e.g. \code{"10/500"}).
#'    \item \code{BgRatio} is the fraction of background genes annotated
#'          to the term (e.g. \code{"50/20000"}).
#'    \item \code{Count}, if given, must equal the numerator of
#'          \code{GeneRatio}.
#'    \item \code{geneID} contains gene symbols separated by \code{"/"}.
#'  }
#' @param in_form A character string specifying the input format.
#'  When \code{"auto"} (default), the function infers the format from the
#'  column names: clusterProfiler columns (\code{pvalue}, \code{p.adjust},
#'  \code{qvalue}) or Enrichr columns (\code{P.value},
#'  \code{Adjusted.P.value}).  Other options are \code{"clusterProfiler"}
#'  and \code{"enrichr"}.
#' @param top_term An integer specifying the maximum number of terms to
#'  include.  Terms are ranked by \code{metric} (ascending).  Default
#'  \code{100}.
#' @param metric A character string specifying the significance metric
#'  used for top-term selection and node scoring: \code{"p.adjust"}
#'  (default) or \code{"pvalue"}.  The value is transformed as
#'  \eqn{-\log_{10}(metric)}.
#' @param layout A character string naming the igraph layout algorithm.
#'  Built-in shortcuts: \code{"circle"}, \code{"tree"}, \code{"grid"}.
#'  Otherwise, the suffix passed to \code{layout_with_<layout>} in igraph
#'  (e.g. \code{"fr"} for Fruchterman-Reingold, \code{"kk"} for
#'  Kamada-Kawai).  Default \code{"fr"}.
#' @param minchar An integer specifying the minimum character length for
#'  words to be included as keywords when \code{show_keyword = TRUE}.
#'  Default \code{2}.
#' @param cluster A character string naming the igraph community detection
#'  algorithm.  The suffix passed to \code{cluster_<cluster>} in igraph
#'  (e.g. \code{"fast_greedy"}, \code{"walktrap"},
#'  \code{"edge_betweenness"}, \code{"infomap"}).  Default
#'  \code{"fast_greedy"}.
#' @param show_keyword A logical value.  When \code{TRUE}, the
#'  \code{Description} text is tokenized and the most significant words
#'  per cluster are shown as keywords.  When \code{FALSE} (default), the
#'  original term descriptions are used as labels.
#' @param nlabel An integer specifying the number of keywords or term
#'  descriptions to show per cluster in the legend labels.  Default
#'  \code{4}.
#' @param character_width An integer specifying the maximum width (in
#'  characters) at which keyword labels are wrapped via
#'  \code{strwrap(width = character_width)}.  Default \code{50}.
#' @param mark A character string naming the ggforce hull function.
#'  One of \code{"ellipse"} (default), \code{"rect"}, \code{"circle"}, or
#'  \code{"text"} -- passed as the suffix to \code{geom_mark_<mark>}.
#' @param label A character string specifying what information to display
#'  in the legend labels.  Either \code{"term"} (default; shows top term
#'  descriptions/keywords per cluster) or \code{"feature"} (shows top
#'  gene symbols per cluster).
#' @param labelsize A numeric value specifying the font size of the
#'  cluster labels drawn by the ggforce mark layer.  Default \code{5}.
#' @param expand A numeric vector of length 2 (or 4) specifying the
#'  expansion of the x and y axes, processed via
#'  \code{\link{norm_expansion}()}.  Default \code{c(0.4, 0.4)}.
#' @param words_excluded A character vector of words to exclude from
#'  keyword extraction when \code{show_keyword = TRUE}.  Defaults to
#'  \code{plotthis::words_excluded}.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @keywords internal
#' @importFrom utils combn
#' @importFrom dplyr slice_min slice_head reframe distinct group_by arrange reframe desc
#' @importFrom scales breaks_extended
#' @importFrom ggplot2 geom_segment geom_point labs scale_size guides scale_linewidth scale_fill_manual scale_x_continuous
#' @importFrom ggplot2 scale_y_continuous
EnrichMapAtomic <- function(
    data,
    in_form = "clusterProfiler",
    top_term = 100,
    metric = "p.adjust",
    layout = "fr",
    minchar = 2,
    cluster = "fast_greedy",
    show_keyword = FALSE,
    nlabel = 4,
    character_width = 50,
    words_excluded = plotthis::words_excluded,
    mark = "ellipse",
    label = c("term", "feature"),
    labelsize = 5,
    expand = c(0.4, 0.4),
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525,
    ...
) {
    # if (!requireNamespace("igraph", quietly = TRUE)) {
    #     stop("The 'igraph' package is required 'EnrichMap'. Please install it first.")
    # }
    # if (!requireNamespace("ggforce", quietly = TRUE)) {
    #     stop("The 'ggforce' package is required 'EnrichMap'. Please install it first.")
    # }
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    if (inherits(data, "enrichResult")) {
        data <- as.data.frame(data)
    }
    if (in_form == "enrichr") {
        data <- prepare_enrichr_result(data)
    }

    # Check the columns
    check_columns(data, "Description", force_factor = TRUE)
    check_columns(data, "GeneRatio")
    check_columns(data, "pvalue")
    check_columns(data, "p.adjust")
    check_columns(data, "geneID")
    label <- match.arg(label)
    expand <- norm_expansion(
        expand,
        x_type = "continuous",
        y_type = "continuous"
    )

    if (!is.null(top_term)) {
        data <- slice_min(data, !!sym(metric), n = top_term, with_ties = FALSE)
    }

    data$metric <- -log10(data[[metric]])
    data$geneID <- strsplit(data$geneID, "/")
    if (is.null(data$ID) && is.null(rownames(data))) {
        data$ID <- paste0("GS", seq_len(nrow(data)))
    } else if (is.null(data$ID)) {
        data$ID <- rownames(data)
    }

    nodes <- data
    edges <- as.data.frame(t(combn(nodes$ID, 2)))
    colnames(edges) <- c("from", "to")
    edges$weight <- mapply(
        function(x, y) {
            length(intersect(
                unlist(data[x, "geneID"]),
                unlist(data[y, "geneID"])
            ))
        },
        edges$from,
        edges$to
    )
    edges <- edges[edges$weight > 0, , drop = FALSE]
    nodes <- nodes[c("ID", setdiff(colnames(nodes), "ID"))]
    graph <- igraph::graph_from_data_frame(
        d = edges,
        vertices = nodes,
        directed = FALSE
    )
    if (layout %in% c("circle", "tree", "grid")) {
        layout <- switch(
            layout,
            "circle" = igraph::layout_in_circle(graph),
            "tree" = igraph::layout_as_tree(graph),
            "grid" = igraph::layout_on_grid(graph)
        )
    } else {
        lofun <- getFromNamespace(paste0("layout_with_", layout), "igraph")
        layout <- lofun(graph)
    }

    clfun <- getFromNamespace(paste0("cluster_", cluster), "igraph")
    clusters <- clfun(graph)

    df_graph <- igraph::as_data_frame(graph, what = "both")
    df_nodes <- df_graph$vertices
    df_nodes$dim1 <- layout[, 1]
    df_nodes$dim2 <- layout[, 2]
    df_nodes$clusters <- factor(
        paste0("C", clusters$membership),
        paste0("C", unique(sort(clusters$membership)))
    )

    if (isTRUE(show_keyword)) {
        df_keyword1 <- df_nodes %>%
            mutate(
                keyword = strsplit(
                    tolower(as.character(!!sym("Description"))),
                    "\\s|\\n",
                    perl = TRUE
                )
            ) %>%
            unnest(cols = "keyword") %>%
            group_by(!!sym("keyword"), !!sym("clusters")) %>%
            reframe(
                keyword = tools::toTitleCase(!!sym("keyword")),
                score = sum(!!sym("metric")),
                count = n(),
                .groups = "keep"
            ) %>%
            filter(!grepl(pattern = "\\[.*\\]", x = !!sym("keyword"))) %>%
            filter(nchar(!!sym("keyword")) >= minchar) %>%
            filter(!tolower(!!sym("keyword")) %in% tolower(words_excluded)) %>%
            distinct() %>%
            group_by(!!sym("clusters")) %>%
            arrange(desc(!!sym("score"))) %>%
            slice_head(n = nlabel) %>%
            reframe(keyword = paste0(!!sym("keyword"), collapse = "/")) %>%
            as.data.frame()
        rownames(df_keyword1) <- as.character(df_keyword1$clusters)
        df_keyword1$keyword <- str_wrap(
            df_keyword1$keyword,
            width = character_width
        )
        df_keyword1$label <- paste0(
            df_keyword1$clusters,
            ":\n",
            df_keyword1$keyword
        )
    } else {
        # if (label == "term") {
        df_nodes$Description <- paste0(
            "- ",
            str_wrap(df_nodes$Description, width = character_width, exdent = 2)
        )
        # }
        df_keyword1 <- df_nodes %>%
            group_by(!!sym("clusters")) %>%
            arrange(desc(!!sym("metric"))) %>%
            reframe(keyword = !!sym("Description")) %>%
            distinct() %>%
            group_by(!!sym("clusters")) %>%
            slice_head(n = nlabel) %>%
            reframe(keyword = paste0(!!sym("keyword"), collapse = "\n")) %>%
            as.data.frame()

        rownames(df_keyword1) <- as.character(df_keyword1$clusters)
        df_keyword1$label <- paste0(
            df_keyword1$clusters,
            ":\n",
            df_keyword1$keyword
        )
    }

    df_keyword2 <- df_nodes %>%
        mutate(keyword = !!sym("geneID")) %>%
        unnest(cols = "keyword") %>%
        group_by(!!sym("keyword"), !!sym("clusters")) %>%
        reframe(
            keyword = !!sym("keyword"),
            score = sum(!!sym("metric")),
            count = n(),
            .groups = "keep"
        ) %>%
        distinct() %>%
        group_by(!!sym("clusters")) %>%
        arrange(desc(!!sym("score"))) %>%
        slice_head(n = nlabel) %>%
        reframe(keyword = paste0(!!sym("keyword"), collapse = "|")) %>%
        as.data.frame()

    rownames(df_keyword2) <- as.character(df_keyword2$clusters)
    df_keyword2$keyword <- str_wrap(
        df_keyword2$keyword,
        width = character_width
    )
    df_keyword2$label <- paste0(
        df_keyword2$clusters,
        ":\n",
        df_keyword2$keyword
    )

    df_nodes$keyword1 <- df_keyword1[as.character(df_nodes$clusters), "keyword"]
    df_nodes$keyword2 <- df_keyword2[as.character(df_nodes$clusters), "keyword"]

    df_edges <- df_graph$edges
    df_edges$from_dim1 <- df_nodes[df_edges$from, "dim1"]
    df_edges$from_dim2 <- df_nodes[df_edges$from, "dim2"]
    df_edges$to_dim1 <- df_nodes[df_edges$to, "dim1"]
    df_edges$to_dim2 <- df_nodes[df_edges$to, "dim2"]

    markfun <- getFromNamespace(paste0("geom_mark_", mark), "ggforce")
    mark_layer <- markfun(
        data = df_nodes,
        aes(
            x = !!sym("dim1"),
            y = !!sym("dim2"),
            color = !!sym("clusters"),
            fill = !!sym("clusters"),
            label = !!sym("clusters"),
            description = !!sym(ifelse(label == "term", "keyword1", "keyword2"))
        ),
        expand = unit(3, "mm"),
        alpha = 0.1,
        label.margin = margin(1, 1, 1, 1, "mm"),
        label.fontsize = labelsize * 2,
        label.fill = "grey95",
        label.minwidth = unit(character_width, "in"),
        label.buffer = unit(0, "mm"),
        con.size = 1,
        con.cap = 0
    )

    p <- ggplot() +
        mark_layer +
        geom_segment(
            data = df_edges,
            aes(
                x = !!sym("from_dim1"),
                y = !!sym("from_dim2"),
                xend = !!sym("to_dim1"),
                yend = !!sym("to_dim2"),
                linewidth = !!sym("weight")
            ),
            alpha = 0.1,
            lineend = "round"
        ) +
        geom_point(
            data = df_nodes,
            aes(
                x = !!sym("dim1"),
                y = !!sym("dim2"),
                size = !!sym("Count"),
                fill = !!sym("clusters")
            ),
            color = "black",
            shape = 21
        ) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% "",
            y = ylab %||% ""
        ) +
        scale_size(name = "Count", range = c(2, 6), breaks_extended(n = 4)) +
        guides(
            size = guide_legend(
                override.aes = list(fill = "grey30", shape = 21),
                order = 1
            )
        ) +
        scale_linewidth(
            name = "Intersection",
            range = c(0.3, 3),
            breaks_extended(n = 4)
        ) +
        guides(
            linewidth = guide_legend(
                override.aes = list(alpha = 1, color = "grey"),
                order = 2
            )
        ) +
        scale_fill_manual(
            name = switch(label, "term" = "Feature", "feature" = "Term"),
            values = palette_this(
                levels(df_nodes$clusters),
                palette = palette,
                palcolor = palcolor,
                reverse = palreverse
            ),
            labels = if (label == "term") {
                df_keyword2[levels(df_nodes$clusters), "label"]
            } else {
                df_keyword1[levels(df_nodes$clusters), "label"]
            },
            na.value = "grey80",
            aesthetics = c("colour", "fill")
        ) +
        guides(
            fill = guide_legend(
                override.aes = list(alpha = 1, color = "black", shape = NA),
                byrow = TRUE,
                theme = ggplot2::theme(legend.key.spacing.y = unit(0.1, "cm")),
                order = 3
            )
        ) +
        guides(color = guide_none()) +
        scale_x_continuous(expand = expand$x) +
        scale_y_continuous(expand = expand$y) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction
        )

    dims <- calculate_plot_dimensions(
        base_height = 8,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction
    )
    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    p
}

#' Atomic enrichment network (internal)
#'
#' @description
#' Core implementation for drawing a single enrichment network -- a
#' term-gene bipartite graph where enriched terms and their member genes
#' are shown as interconnected nodes.  Term nodes are displayed as numbered
#' filled circles with a colour-coded legend; gene nodes are displayed as
#' labelled rectangles coloured by a blend of the term colours they belong
#' to.  This is the workhorse behind the exported \code{\link{EnrichNetwork}}
#' function -- it takes a **single** data frame (no \code{split_by} support)
#' and returns a \code{ggplot} object.
#'
#' The function constructs a bipartite graph between terms and genes,
#' computes a force-directed layout, optionally adjusts node positions to
#' reduce overlap, blends term colours for shared genes, and renders the
#' result as a labelled network.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{ggplot dispatch} -- selects \code{gglogger::ggplot} or
#'         \code{ggplot2::ggplot} based on
#'         \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{Data format conversion} -- if \code{data} inherits from
#'         \code{"enrichResult"}, converts via \code{as.data.frame()}.
#'   \item \strong{Column validation} -- \code{\link{check_columns}()}
#'         verifies that \code{Description}, \code{GeneRatio},
#'         \code{pvalue}, \code{p.adjust}, and \code{geneID} are present.
#'   \item \strong{Top-term selection} -- when \code{top_term} is not
#'         \code{NULL}, selects the top N terms by \code{metric} via
#'         \code{dplyr::slice_min()}.
#'   \item \strong{Metric and description preparation} -- computes
#'         \eqn{-\log_{10}(metric)} as the scoring variable.  Wraps
#'         \code{Description} text to \code{character_width} and parses
#'         \code{geneID} by splitting on \code{"/"}.
#'   \item \strong{Gene-term unnesting} -- unnests the \code{geneID}
#'         column to produce a gene-term mapping table.
#'   \item \strong{Bipartite node construction} -- creates nodes for both
#'         terms (\code{class = "term"}) and genes (\code{class = "gene"}),
#'         carrying the \code{Database} attribute if present.
#'   \item \strong{Bipartite edge construction} -- edges connect each term
#'         to its member genes with uniform weight (\code{weight = 1}).
#'   \item \strong{igraph graph construction} --
#'         \code{igraph::graph_from_data_frame()} builds an undirected
#'         bipartite graph.
#'   \item \strong{Layout computation} -- dispatches to the chosen igraph
#'         layout function (\code{layout_with_*}) or built-in shortcuts
#'         (\code{"circle"}, \code{"tree"}, \code{"grid"}).
#'   \item \strong{Layout adjustment} -- when \code{layoutadjust = TRUE}
#'         (default), calls \code{\link{adjust_network_layout}()} to push
#'         overlapping nodes apart based on label width and a repulsion
#'         simulation.
#'   \item \strong{Node coordinates} -- extracts vertex positions into
#'         \code{dim1} and \code{dim2} columns.
#'   \item \strong{Colour computation} -- palette colours are assigned to
#'         term nodes.  Gene node colours are computed by blending the
#'         colours of all connected terms via the specified
#'         \code{blendmode} using \code{blend_colors()}.
#'   \item \strong{Label colour} -- each node's text colour is chosen as
#'         black or white based on the luminance of its fill colour (sum
#'         of RGB channels > 510).
#'   \item \strong{Numeric labels} -- term nodes receive sequential integer
#'         labels; gene nodes display their gene symbol.
#'   \item \strong{Custom legend key} -- \code{draw_key_cust()} renders
#'         term legend entries as numbered circles via
#'         \code{ggrepel::shadowtextGrob()}.
#'   \item \strong{Edge rendering} -- \code{ggplot2::geom_segment()} draws
#'         edges coloured by the source term (legend suppressed).
#'   \item \strong{Gene node rendering} -- \code{ggplot2::geom_label()}
#'         displays gene symbols with fill colour blended from connected
#'         terms.
#'   \item \strong{Term node rendering} -- two \code{geom_point()} layers
#'         draw black-outlined circles filled with the term colour, with
#'         \code{draw_key_cust} as the key glyph.
#'   \item \strong{Term labels} -- \code{ggrepel::geom_text_repel()}
#'         places the numeric term labels with white text on a black
#'         background.
#'   \item \strong{Scale configuration} -- \code{scale_color_identity()}
#'         and \code{scale_fill_identity()} with a manual legend mapping
#'         term colours to term descriptions.
#'   \item \strong{Theme and dimensions} -- applies the resolved theme,
#'         sets aspect ratio and legend position, then calls
#'         \code{\link{calculate_plot_dimensions}()} to attach
#'         \code{height}/\code{width} attributes.
#' }
#'
#' @inheritParams common_args
#' @param data A data frame containing enrichment results in clusterProfiler
#'  format (see \code{\link{EnrichMapAtomic}} for the expected columns).
#' @param top_term An integer specifying the maximum number of terms to
#'  include.  Terms are ranked by \code{metric} (ascending).  Default
#'  \code{6}.
#' @param metric A character string specifying the significance metric for
#'  top-term selection: \code{"p.adjust"} (default) or \code{"pvalue"}.
#' @param character_width An integer specifying the maximum width (in
#'  characters) at which term descriptions are wrapped via
#'  \code{strwrap(width = character_width)}.  Default \code{50}.
#' @param layout A character string naming the igraph layout algorithm.
#'  Built-in shortcuts: \code{"circle"}, \code{"tree"}, \code{"grid"}.
#'  Otherwise, the suffix passed to \code{layout_with_<layout>} in igraph.
#'  Default \code{"fr"}.
#' @param layoutadjust A logical value.  When \code{TRUE} (default),
#'  applies \code{\link{adjust_network_layout}()} after the initial layout
#'  to reduce node overlap based on label width and a repulsion simulation.
#' @param adjscale A numeric value controlling the scale of the layout
#'  adjustment.  Passed as the \code{scale} argument to
#'  \code{\link{adjust_network_layout}()}.  Default \code{60}.
#' @param adjiter A numeric value controlling the number of iterations for
#'  the layout adjustment.  Passed as the \code{iter} argument to
#'  \code{\link{adjust_network_layout}()}.  Default \code{100}.
#' @param blendmode A character string specifying how gene colours are
#'  computed from the colours of the terms they belong to.  One of
#'  \code{"blend"} (default), \code{"average"}, \code{"multiply"}, or
#'  \code{"screen"}.  Passed to \code{blend_colors()}.
#' @param labelsize A numeric value specifying the font size of the
#'  numeric term labels displayed via \code{ggrepel::geom_text_repel()}.
#'  Default \code{5}.
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @keywords internal
#' @importFrom utils getFromNamespace
#' @importFrom grDevices col2rgb
#' @importFrom ggplot2 scale_color_identity scale_fill_identity guides guide_legend draw_key_point .pt element_text
EnrichNetworkAtomic <- function(
    data,
    top_term = 6,
    metric = "p.adjust",
    character_width = 50,
    layout = "fr",
    layoutadjust = TRUE,
    adjscale = 60,
    adjiter = 100,
    blendmode = "blend",
    labelsize = 5,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    # if (!requireNamespace("igraph", quietly = TRUE)) {
    #     stop("The 'igraph' package is required 'EnrichMap'. Please install it first.")
    # }
    # if (!requireNamespace("ggforce", quietly = TRUE)) {
    #     stop("The 'ggforce' package is required 'EnrichMap'. Please install it first.")
    # }
    if (inherits(data, "enrichResult")) {
        data <- as.data.frame(data)
    }
    # Check the columns
    check_columns(data, "Description", force_factor = TRUE)
    check_columns(data, "GeneRatio")
    check_columns(data, "pvalue")
    check_columns(data, "p.adjust")
    check_columns(data, "geneID")

    if (!is.null(top_term)) {
        data <- slice_min(data, !!sym(metric), n = top_term, with_ties = FALSE)
    }

    data$metric <- -log10(data[[metric]])
    data$Description <- str_wrap(data$Description, width = character_width)
    data$Description <- factor(
        data$Description,
        levels = unique(data$Description)
    )
    data$geneID <- strsplit(data$geneID, "/")
    df_unnest <- unnest(data, cols = "geneID")

    nodes <- rbind(
        data.frame(
            "ID" = data$Description,
            class = "term",
            metric = data$metric
        ),
        data.frame("ID" = unique(df_unnest$geneID), class = "gene", metric = 0)
    )
    nodes$Database <- data$Database[1]
    edges <- as.data.frame(df_unnest[, c("Description", "geneID")])
    colnames(edges) <- c("from", "to")
    edges$weight <- 1
    graph <- igraph::graph_from_data_frame(
        d = edges,
        vertices = nodes,
        directed = FALSE
    )
    if (layout %in% c("circle", "tree", "grid")) {
        layout <- switch(
            layout,
            "circle" = igraph::layout_in_circle(graph),
            "tree" = igraph::layout_as_tree(graph),
            "grid" = igraph::layout_on_grid(graph)
        )
    } else {
        lofun <- getFromNamespace(paste0("layout_with_", layout), "igraph")
        layout <- lofun(graph)
    }
    df_graph <- igraph::as_data_frame(graph, what = "both")

    df_nodes <- df_graph$vertices
    if (isTRUE(layoutadjust)) {
        width <- nchar(df_nodes$name)
        width[df_nodes$class == "term"] <- 8
        layout <- adjust_network_layout(
            graph = graph,
            layout = layout,
            width = width,
            height = 2,
            scale = adjscale,
            iter = adjiter
        )
    }
    df_nodes$dim1 <- layout[, 1]
    df_nodes$dim2 <- layout[, 2]

    df_edges <- df_graph$edges
    df_edges$from_dim1 <- df_nodes[df_edges$from, "dim1"]
    df_edges$from_dim2 <- df_nodes[df_edges$from, "dim2"]
    df_edges$to_dim1 <- df_nodes[df_edges$to, "dim1"]
    df_edges$to_dim2 <- df_nodes[df_edges$to, "dim2"]

    colors <- palette_this(
        levels(data$Description),
        palette = palette,
        palcolor = palcolor,
        reverse = palreverse
    )
    df_edges$color <- colors[df_edges$from]
    node_colors <- aggregate(
        df_unnest$Description,
        by = list(df_unnest$geneID),
        FUN = function(x) blend_colors(colors = colors[x], mode = blendmode)
    )
    colors <- c(colors, setNames(node_colors[, 2], node_colors[, 1]))
    label_colors <- ifelse(colSums(col2rgb(colors)) > 255 * 2, "black", "white")
    df_nodes$color <- colors[df_nodes$name]
    df_nodes$label_color <- label_colors[df_nodes$name]
    df_nodes$label <- NA
    df_nodes[levels(data$Description), "label"] <- seq_len(nlevels(
        data$Description
    ))

    draw_key_cust <- function(df, params, size) {
        data_text <- df
        data_text$label <- which(
            levels(data$Description) %in%
                names(colors)[colors == data_text$fill]
        )
        data_text$colour <- "black"
        data_text$alpha <- 1
        data_text$size <- 11 / .pt
        grid::grobTree(
            draw_key_point(df, list(color = "white", shape = 21)),
            getFromNamespace("shadowtextGrob", "ggrepel")(
                label = data_text$label,
                bg.colour = "black",
                bg.r = 0.1,
                gp = grid::gpar(col = "white", fontface = "bold")
            )
        )
    }

    p <- ggplot() +
        geom_segment(
            data = df_edges,
            aes(
                x = !!sym("from_dim1"),
                y = !!sym("from_dim2"),
                xend = !!sym("to_dim1"),
                yend = !!sym("to_dim2"),
                color = !!sym("color")
            ),
            alpha = 1,
            lineend = "round",
            show.legend = FALSE
        ) +
        geom_label(
            data = df_nodes[df_nodes$class == "gene", ],
            aes(
                x = !!sym("dim1"),
                y = !!sym("dim2"),
                label = !!sym("name"),
                fill = !!sym("color"),
                color = !!sym("label_color")
            ),
            size = 3,
            show.legend = FALSE
        ) +
        geom_point(
            data = df_nodes[df_nodes$class == "term", ],
            aes(x = !!sym("dim1"), y = !!sym("dim2")),
            size = 8,
            color = "black",
            fill = "black",
            stroke = 1,
            shape = 21,
            show.legend = FALSE
        ) +
        geom_point(
            data = df_nodes[df_nodes$class == "term", ],
            aes(x = !!sym("dim1"), y = !!sym("dim2"), fill = !!sym("color")),
            size = 7,
            color = "white",
            stroke = 1,
            shape = 21,
            key_glyph = draw_key_cust
        ) +
        geom_text_repel(
            data = df_nodes[df_nodes$class == "term", ],
            aes(x = !!sym("dim1"), y = !!sym("dim2"), label = !!sym("label")),
            fontface = "bold",
            min.segment.length = 0,
            segment.color = "black",
            point.size = NA,
            max.overlaps = 100,
            force = 0,
            color = "white",
            bg.color = "black",
            bg.r = 0.1,
            size = labelsize
        ) +
        scale_color_identity(guide = "none") +
        scale_fill_identity(
            name = "Term",
            guide = "legend",
            labels = levels(data$Description),
            breaks = colors[levels(data$Description)]
        ) +
        guides(
            color = guide_legend(override.aes = list(color = "transparent"))
        ) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% "",
            y = ylab %||% ""
        ) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction
        )

    dims <- calculate_plot_dimensions(
        base_height = 8,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction
    )
    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    p
}

#' Process the enrichment results from Enrichr
#'
#' @param data A data frame containing the result by Enrichr.
#' @param dbname A character string specifying the name of the database column.
#' @param n_input An integer specifying the number of input genes.
#' Enrichr result doesn't ship with the number of input genes.
#' You can either provide the number directly or we will infer it. See details.
#' @return A data frame that can be used in `EnrichMap`.
#' @keywords internal
#' @details
#' In order to use the `EnrichMap` and `EnrichNetwork` functions and other visualization functions in `plotthis`,
#' the enrichment results from Enrichr need to be processed by the `prepare_enrichr_result` function.
#' The following columns are renamed:
#' * `Term` -> `Description`
#' * `Genes` -> `geneID` (separated replaced by `/`)
#' * `P.value` -> `pvalue`
#' * `Adjusted.P.value` -> `p.adjust`
#' Additionally, GeneRatio and BgRatio columns are inferred.
#' From [enrichr's documentation](https://maayanlab.cloud/Enrichr/help#background), the oddsRatio is defined as:
#' `oddsRatio = (A * (D - B - C + A) / max((B - A) * (C - A), 1)`, where A is the overlapping genes; B is the total genes in the gene set;
#' C (n_input) is the genes in input list; D is the total genes in the background.
#' D is not provided by Enrichr. To infer it, `D = oddsRatio * max((B - A) * (C - A), 1) / A + B + C - A`.
#' * `Overlap = A / B` (from Enrichr)
#' * `GeneRatio = A / C` (from ClusterProfiler)
#' * `BgRatio = B / D` (from ClusterProfiler)
#' `C (n_input)`, if not provided, will be inferred when `D` for all terms are equal.
#' When starting inferrence, the minimum value to try will be unique genes in `data$Genes`/`data$geneID`.
prepare_enrichr_result <- function(data, dbname = "Database", n_input = NULL) {
    if (inherits(data, "list")) {
        data <- lapply(names(data), function(x) {
            df <- prepare_enrichr_result(data[[x]])
            df[[dbname]] <- x
            df
        })
        return(do_call(rbind, data))
    }
    # Calculate GeneRatio and BgRatio
    A_B <- strsplit(data$Overlap, "/", fixed = TRUE)
    A <- as.numeric(sapply(A_B, `[`, 1))
    B <- as.numeric(sapply(A_B, `[`, 2))
    data$geneID <- data$Genes
    data$Genes <- NULL
    data$geneID <- gsub(";", "/", data$geneID, fixed = TRUE)
    if (is.null(n_input)) {
        C <- length(unique(unlist(strsplit(
            unlist(data$geneID),
            "/",
            fixed = TRUE
        ))))
        while (TRUE) {
            D <- data$Odds.Ratio * pmax((B - A) * (C - A), 1) / A + B + C - A
            if (isTRUE(all.equal(D, rep(D[1], length(D))))) {
                break
            }
            if (D[1] > 100000) {
                stop(
                    "Failed to infer the number of input genes. Please provide it manually."
                )
            }
            C <- C + 1
        }
    } else {
        C <- n_input
        D <- data$Odds.Ratio * pmax((B - A) * (C - A), 1) / A + B + C - A
    }
    D <- round(D)
    data$Count <- A
    data$GeneRatio <- paste(A, C, sep = "/")
    data$BgRatio <- paste(B, D, sep = "/")
    data$Overlap <- NULL
    data$Odds.Ratio <- NULL
    data$Description <- data$Term
    data$Term <- NULL
    data$pvalue <- data[["P.value"]]
    data[["P.value"]] <- NULL
    data$p.adjust <- data[["Adjusted.P.value"]]
    data[["Adjusted.P.value"]] <- NULL

    data
}

#' Enrichment Map and Enrichment Network
#'
#' @description
#' \code{EnrichMap} draws an enrichment map -- a gene-set similarity network
#' where each node is an enriched term, node size encodes the number of
#' associated genes, node fill colour encodes cluster membership (detected
#' via igraph community detection), and edge thickness encodes the number
#' of overlapping genes between term pairs.  The plot uses a force-directed
#' layout to arrange terms, and ggforce hull annotations group terms into
#' clusters.  Keyword or term-description labels appear in the legend.
#'
#' \code{EnrichNetwork} draws an enrichment network -- a term-gene bipartite
#' graph where term nodes are shown as numbered circles and gene nodes as
#' labelled rectangles.  Gene node colours are blended from the colours of
#' all terms they belong to.  A force-directed layout positions the nodes,
#' with optional overlap adjustment for better readability.
#'
#' Both functions accept enrichment results from clusterProfiler or Enrichr
#' (the latter is auto-detected and preprocessed via
#' \code{\link{prepare_enrichr_result}()}).
#'
#' @section split_by Workflow (EnrichMap):
#'
#' When \code{split_by} is provided, \code{EnrichMap()} executes the
#' following pipeline:
#' \enumerate{
#'   \item \strong{Argument validation} --
#'         \code{\link{validate_common_args}()} checks the seed.
#'   \item \strong{Input format detection} -- \code{match.arg()} resolves
#'         \code{in_form}; \code{"auto"} mode infers the format from
#'         column names.
#'   \item \strong{Enrichr preprocessing} -- when format is \code{"enrichr"},
#'         calls \code{\link{prepare_enrichr_result}()} to rename columns
#'         and infer GeneRatio/BgRatio.
#'   \item \strong{Split column resolution} --
#'         \code{\link{check_columns}()} validates \code{split_by}
#'         (force_factor, allow_multi, concat_multi).
#'   \item \strong{Data splitting} -- splits \code{data} by
#'         \code{split_by} levels, preserving factor level order.
#'   \item \strong{Per-split palette/colour} --
#'         \code{\link{check_palette}()} and
#'         \code{\link{check_palcolor}()} resolve per-split palette and
#'         colour overrides.
#'   \item \strong{Per-split legend} -- \code{\link{check_legend}()}
#'         resolves \code{legend.position} and \code{legend.direction}
#'         per split.
#'   \item \strong{Per-split title} -- when \code{title} is a function,
#'         it receives the default title (the split level name); otherwise
#'         \code{title \%||\% split_level} is used.
#'   \item \strong{Dispatch} -- each split subset is passed to
#'         \code{\link{EnrichMapAtomic}} with its resolved parameters.
#'   \item \strong{Combination} -- \code{\link{combine_plots}()} assembles
#'         the list of plots via \code{patchwork::wrap_plots}, honouring
#'         \code{nrow}/\code{ncol}/\code{byrow}/\code{axes}/
#'         \code{axis_titles}/\code{guides}/\code{design}.
#' }
#'
#' @rdname enrichmap1
#' @inheritParams common_args
#' @inheritParams EnrichMapAtomic
#' @param data A data frame containing enrichment results in clusterProfiler
#'  format with at least the columns: \code{Description}, \code{GeneRatio},
#'  \code{pvalue}, \code{p.adjust}, and \code{geneID}.
#'  \itemize{
#'    \item \code{ID}, \code{qvalue}, \code{BgRatio}, and \code{Count} are
#'          optional.
#'    \item \code{Description} is the term description.
#'    \item \code{GeneRatio} is the fraction of input genes annotated to
#'          the term (e.g. \code{"10/500"}).
#'    \item \code{BgRatio} is the fraction of background genes annotated
#'          to the term (e.g. \code{"50/20000"}).
#'    \item \code{Count}, if given, must equal the numerator of
#'          \code{GeneRatio}.
#'  }
#'  If you have enrichment results from multiple databases, you can combine
#'  them into one data frame and add a column (e.g. \code{Database}) to
#'  indicate the source.  Use \code{split_by = "Database"} to plot them
#'  side by side.
#' @param in_form A character string specifying the input format.
#'  When \code{"auto"} (default), the function infers the format from the
#'  column names: clusterProfiler columns (\code{pvalue}, \code{p.adjust},
#'  \code{qvalue}) or Enrichr columns (\code{P.value},
#'  \code{Adjusted.P.value}).  Other options are \code{"clusterProfiler"},
#'  \code{"clusterprofiler"}, and \code{"enrichr"}.
#' @return A \code{ggplot} object (single plot), a \code{patchwork} /
#'  \code{wrap_plots} object (when \code{split_by} is provided and
#'  \code{combine = TRUE}), or a list of \code{ggplot} objects (when
#'  \code{split_by} is provided and \code{combine = FALSE}).
#' @export
#' @examples
#' \donttest{
#' data(enrich_example)
#' EnrichMap(enrich_example)
#' EnrichMap(enrich_example, label = "feature")
#' EnrichMap(enrich_example, show_keyword = TRUE, label = "term")
#' EnrichMap(enrich_example, show_keyword = TRUE, label = "feature")
#'
#' data(enrich_multidb_example)
#' EnrichMap(enrich_multidb_example, split_by = "Database")
#' EnrichMap(enrich_multidb_example, split_by = "Database",
#'           palette = list(DB1 = "Paired", DB2 = "Set1"))
#' }
EnrichMap <- function(
    data,
    in_form = c("auto", "clusterProfiler", "clusterprofiler", "enrichr"),
    split_by = NULL,
    split_by_sep = "_",
    top_term = 10,
    metric = "p.adjust",
    layout = "fr",
    minchar = 2,
    cluster = "fast_greedy",
    show_keyword = FALSE,
    nlabel = 4,
    character_width = 50,
    mark = "ellipse",
    label = c("term", "feature"),
    labelsize = 5,
    expand = c(0.4, 0.4),
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed)
    theme <- process_theme(theme)
    in_form <- match.arg(in_form)
    in_form <- tolower(in_form)
    if (in_form == "auto") {
        if (all(c("pvalue", "p.adjust", "qvalue") %in% colnames(data))) {
            in_form <- "clusterProfiler"
        } else if (all(c("P.value", "Adjusted.P.value") %in% colnames(data))) {
            in_form <- "enrichr"
        } else {
            stop("Cannot infer the input format. Please provide it manually.")
        }
    }
    if (in_form == "enrichr") {
        data <- prepare_enrichr_result(data)
    }

    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            EnrichMapAtomic(
                datas[[nm]],
                top_term = top_term,
                metric = metric,
                layout = layout,
                minchar = minchar,
                cluster = cluster,
                show_keyword = show_keyword,
                nlabel = nlabel,
                character_width = character_width,
                mark = mark,
                label = label,
                labelsize = labelsize,
                expand = expand,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                alpha = alpha,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                seed = seed,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

#' @section split_by Workflow (EnrichNetwork):
#'
#' When \code{split_by} is provided, \code{EnrichNetwork()} executes the
#' same pipeline as \code{EnrichMap()} above, but dispatches each split
#' subset to \code{\link{EnrichNetworkAtomic}}.
#'
#' @rdname enrichmap1
#' @inheritParams common_args
#' @inheritParams EnrichNetworkAtomic
#' @param data A data frame containing enrichment results in clusterProfiler
#'  format (see \code{\link{EnrichMap}} for the expected columns).
#'  If you have enrichment results from multiple databases, you can combine
#'  them into one data frame and add a column (e.g. \code{Database}) to
#'  indicate the source.  Use \code{split_by = "Database"} to plot them
#'  side by side.
#' @param in_form A character string specifying the input format.
#'  When \code{"auto"} (default), the function infers the format from the
#'  column names.  Other options are \code{"clusterProfiler"},
#'  \code{"clusterprofiler"}, and \code{"enrichr"}.
#' @return A \code{ggplot} object (single plot), a \code{patchwork} /
#'  \code{wrap_plots} object (when \code{split_by} is provided and
#'  \code{combine = TRUE}), or a list of \code{ggplot} objects (when
#'  \code{split_by} is provided and \code{combine = FALSE}).
#' @export
#' @examples
#' \donttest{
#' EnrichNetwork(enrich_example, top_term = 5)
#' }
EnrichNetwork <- function(
    data,
    in_form = c("auto", "clusterProfiler", "clusterprofiler", "enrichr"),
    split_by = NULL,
    split_by_sep = "_",
    top_term = 10,
    metric = "p.adjust",
    character_width = 50,
    layout = "fr",
    layoutadjust = TRUE,
    adjscale = 60,
    adjiter = 100,
    blendmode = "blend",
    labelsize = 5,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed)
    theme <- process_theme(theme)
    in_form <- match.arg(in_form)
    in_form <- tolower(in_form)
    if (in_form == "auto") {
        if (all(c("pvalue", "p.adjust", "qvalue") %in% colnames(data))) {
            in_form <- "clusterProfiler"
        } else if (all(c("P.value", "Adjusted.P.value") %in% colnames(data))) {
            in_form <- "enrichr"
        } else {
            stop("Cannot infer the input format. Please provide it manually.")
        }
    }
    if (in_form == "enrichr") {
        data <- prepare_enrichr_result(data)
    }

    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            EnrichNetworkAtomic(
                datas[[nm]],
                top_term = top_term,
                metric = metric,
                character_width = character_width,
                layout = layout,
                layoutadjust = layoutadjust,
                adjscale = adjscale,
                adjiter = adjiter,
                blendmode = blendmode,
                labelsize = labelsize,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                alpha = alpha,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                seed = seed,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

data("enrich_example", package = "plotthis")
data("enrich_multidb_example", package = "plotthis")

test_that("EnrichMap basic usage with clusterProfiler data works", {
    p <- EnrichMap(enrich_example)
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("EnrichMap with title and subtitle works", {
    p <- EnrichMap(enrich_example, title = "EnrichMap Title", subtitle = "Subtitle")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "EnrichMap Title")
    expect_equal(p$labels$subtitle, "Subtitle")
})

test_that("EnrichMap with top_term parameter works", {
    p <- EnrichMap(enrich_example, top_term = 5)
    expect_s3_class(p, "ggplot")
})

test_that("EnrichMap with split_by returns patchwork", {
    p <- EnrichMap(enrich_multidb_example, split_by = "Database", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("EnrichMap with split_by combine = FALSE returns list", {
    plots <- EnrichMap(enrich_multidb_example, split_by = "Database", combine = FALSE)
    expect_true(is.list(plots))
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("EnrichMap with multiple split_by columns returns a list with one plot per combination", {
    em <- enrich_multidb_example
    em$split2 <- factor(rep(c("s1", "s2"), length.out = nrow(em)))
    plots <- suppressMessages(EnrichMap(em, split_by = c("Database", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("EnrichMap with multiple split_by columns returns combined plot", {
    em <- enrich_multidb_example
    em$split2 <- factor(rep(c("s1", "s2"), length.out = nrow(em)))
    p <- suppressMessages(EnrichMap(em, split_by = c("Database", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

test_that("EnrichMap with cluster = 'walktrap' works", {
    p <- EnrichMap(enrich_example, cluster = "walktrap")
    expect_s3_class(p, "ggplot")
})

test_that("EnrichMap with show_keyword = TRUE works", {
    p <- EnrichMap(enrich_example, show_keyword = TRUE)
    expect_s3_class(p, "ggplot")
})

test_that("EnrichNetwork basic usage works", {
    p <- EnrichNetwork(enrich_example)
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("EnrichNetwork with title works", {
    p <- EnrichNetwork(enrich_example, title = "EnrichNetwork Title")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "EnrichNetwork Title")
})

test_that("EnrichNetwork with top_term parameter works", {
    p <- EnrichNetwork(enrich_example, top_term = 5)
    expect_s3_class(p, "ggplot")
})

test_that("EnrichNetwork with split_by returns patchwork", {
    p <- EnrichNetwork(enrich_multidb_example, split_by = "Database", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("EnrichNetwork with combine = FALSE returns list", {
    plots <- EnrichNetwork(enrich_multidb_example, split_by = "Database", combine = FALSE)
    expect_true(is.list(plots))
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("EnrichNetwork with multiple split_by columns returns a list with one plot per combination", {
    em <- enrich_multidb_example
    em$split2 <- factor(rep(c("s1", "s2"), length.out = nrow(em)))
    plots <- suppressMessages(EnrichNetwork(em, split_by = c("Database", "split2"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 4)
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("EnrichNetwork with multiple split_by columns returns combined plot", {
    em <- enrich_multidb_example
    em$split2 <- factor(rep(c("s1", "s2"), length.out = nrow(em)))
    p <- suppressMessages(EnrichNetwork(em, split_by = c("Database", "split2"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

#' Atomic rarefaction / extrapolation plot (internal)
#'
#' @description
#' Core implementation for drawing a single rarefaction (or extrapolation)
#' curve from biodiversity data.  This is the workhorse behind the exported
#' \code{\link{RarefactionPlot}} — it takes a **single** fortifed data frame
#' (produced by \code{\link[ggplot2]{fortify}()} on an \code{iNEXT} object)
#' and returns a \code{ggplot} object.
#'
#' The function renders three types of curves:
#' \enumerate{
#'   \item \strong{Sample-size-based} (\code{type = 1}) — species diversity
#'         as a function of sample size (number of individuals or sampling
#'         units), with rarefaction (interpolation) and extrapolation segments.
#'   \item \strong{Sample completeness} (\code{type = 2}) — sample coverage
#'         as a function of sample size.
#'   \item \strong{Coverage-based} (\code{type = 3}) — species diversity as a
#'         function of sample coverage.
#' }
#'
#' Observed data points are marked with \code{geom_point()} (shape per group),
#' the rarefaction / extrapolation lines are drawn with \code{geom_line()}
#' using a solid/dashed linetype to distinguish the two phases, and
#' confidence intervals are rendered as semi-transparent ribbons when
#' \code{se = TRUE} and the fortifed data contains \code{y.lwr} / \code{y.upr}
#' columns.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
#'         \code{ggplot2::ggplot} based on
#'         \code{getOption("plotthis.gglogger.enabled")}.
#'   \item \strong{Axis labels} — set according to the combination of
#'         \code{type} and the data's \code{datatype} attribute
#'         (\code{"abundance"} vs. \code{"incidence"}):
#'         \itemize{
#'           \item \code{type = 2}: x = "Number of individuals" or
#'                 "Number of sampling units", y = "Sample coverage".
#'           \item \code{type = 3 || 4}: x = "Sample coverage",
#'                 y = "Species diversity".
#'           \item \code{type = 1} (abundance): x = "Number of individuals",
#'                 y = "Species diversity".
#'           \item \code{type = 1} (incidence): x = "Number of sampling units",
#'                 y = "Species diversity".
#'         }
#'   \item \strong{Group name fallback} — if \code{group_name} is \code{NULL},
#'         it is set to the value of \code{group_by} (i.e. \code{"q"} or
#'         \code{"group"}).
#'   \item \strong{Base ggplot} — initialises \code{ggplot(data, aes(x, y,
#'         color = group_by))}.
#'   \item \strong{Observed data points} — \code{geom_point()} on rows where
#'         \code{Method == "Observed"}, with shape mapped to the group variable.
#'   \item \strong{Colour scale} — \code{scale_color_manual()} using
#'         \code{\link{palette_this}()}.  The legend is suppressed when
#'         \code{group_by} is the dummy \code{".group"}.
#'   \item \strong{Shape scale} — \code{scale_shape_discrete()} with the same
#'         legend-suppression logic.
#'   \item \strong{Rarefaction / extrapolation lines} — \code{geom_line()}
#'         with linetype mapped to the \code{lty} column (\code{"Rarefaction"}
#'         vs. \code{"Extrapolation"}).
#'   \item \strong{Linetype scale} — \code{scale_linetype_manual()} with
#'         \code{solid} and \code{dashed} values, legend key width set to
#'         1 cm.
#'   \item \strong{Theme and labels} — \code{do_call(theme, theme_args)},
#'         \code{panel.grid.major} (grey80 dashed), aspect ratio, legend
#'         position / direction, and title / subtitle.
#'   \item \strong{Confidence ribbon} — when \code{se = TRUE}, a
#'         \code{geom_ribbon(aes(ymin = y.lwr, ymax = y.upr, fill = group_by))}
#'         is added with the same per-group palette at transparency
#'         \code{alpha}.
#'   \item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
#'         computes \code{height} / \code{width} attributes from
#'         \code{base_height = 4.5}, \code{aspect.ratio}, and legend metrics.
#'   \item \strong{Faceting} — \code{\link{facet_plot}()} wraps the plot with
#'         \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is
#'         provided.
#' }
#'
#' @inheritParams common_args
#' @param data A data frame produced by \code{\link[ggplot2]{fortify}()} on an
#'  \code{iNEXT} object.  Expected to contain columns \code{x} (sample size or
#'  coverage), \code{y} (diversity or coverage estimate), \code{Method}
#'  (\code{"Observed"}, \code{"Rarefaction"}, \code{"Extrapolation"}),
#'  \code{y.lwr} / \code{y.upr} (confidence bounds, optional), \code{datatype}
#'  (\code{"abundance"} or \code{"incidence"}), \code{group} (assemblage name,
#'  renamed from \code{Assemblage}), and \code{q} (diversity order, renamed
#'  from \code{Order.q}).
#' @param type An integer specifying the curve type: \code{1} for
#'  sample-size-based rarefaction/extrapolation, \code{2} for sample
#'  completeness, or \code{3} for coverage-based rarefaction/extrapolation.
#'  A vector of types can be passed and the data will be fortifed for all of
#'  them; faceting or splitting then separates the panels.  Default: \code{1}.
#' @param se A logical value indicating whether to display confidence intervals
#'  as semi-transparent ribbons around the estimated curve.  When \code{NULL}
#'  (the default), it resolves to \code{TRUE} if the fortifed data contains
#'  \code{y.lwr} and \code{y.upr} columns, and \code{FALSE} otherwise.
#' @param group_by A character vector specifying how to group the data for
#'  colouring the lines.  Must be one or both of \code{"q"} (diversity order)
#'  and \code{"group"} (assemblage/site).  Multiple values are concatenated
#'  with \code{group_by_sep}.  When \code{NULL}, a dummy \code{".group"}
#'  column is created and the legend is hidden.  Default: \code{"group"}.
#' @param group_name A character string used as the title for the colour
#'  (and shape) legend.  When \code{NULL} (the default), the value of
#'  \code{group_by} is used.
#' @param pt_size A numeric value specifying the size of the observed-data
#'  points.  Default: \code{3}.
#' @param line_width A numeric value specifying the width of the rarefaction /
#'  extrapolation lines.  Default: \code{1}.
#' @param palette A character string specifying the colour palette to use.
#'  A named list or vector can be used to specify palettes for different
#'  \code{split_by} levels in the exported \code{\link{RarefactionPlot}}.
#'  Default: \code{"Spectral"}.
#' @param alpha A numeric value between 0 and 1 specifying the transparency
#'  of the confidence-interval ribbon fill.  Default: \code{0.2}.
#' @param ... Additional arguments (currently unused in the atomic function;
#'  accepted for forward compatibility with the exported wrapper).
#' @keywords internal
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @importFrom dplyr rename
#' @importFrom ggplot2 fortify geom_point geom_line geom_ribbon scale_color_manual scale_linetype_manual
#' @importFrom ggplot2 element_line element_text scale_shape_discrete unit
RarefactionPlotAtomic <- function(
    data,
    type = 1,
    se = TRUE,
    group_by = "group",
    group_name = NULL,
    pt_size = 3,
    line_width = 1,
    theme = "theme_this",
    theme_args = list(),
    palette = "Spectral",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 0.2,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    datatype <- unique(data$datatype)
    if (type == 2L) {
        if (identical(datatype, "abundance")) {
            xlab <- xlab %||% "Number of individuals"
            ylab <- ylab %||% "Sample coverage"
        } else {
            xlab <- xlab %||% "Number of sampling units"
            ylab <- ylab %||% "Sample coverage"
        }
    } else if (type == 3L || type == 4L) {
        xlab <- xlab %||% "Sample coverage"
        ylab <- ylab %||% "Species diversity"
    } else if (identical(datatype, "abundance")) {
        xlab <- xlab %||% "Number of individuals"
        ylab <- ylab %||% "Species diversity"
    } else {
        xlab <- xlab %||% "Number of sampling units"
        ylab <- ylab %||% "Species diversity"
    }

    group_name <- group_name %||% group_by
    p <- ggplot(
        data,
        aes(x = !!sym("x"), y = !!sym("y"), color = !!sym(group_by))
    ) +
        geom_point(
            aes(shape = !!sym(group_by)),
            size = pt_size,
            data = data[data$Method == "Observed", , drop = FALSE]
        ) +
        scale_color_manual(
            name = group_name,
            values = palette_this(
                levels(data[[group_by]]),
                palette = palette,
                palcolor = palcolor,
                reverse = palreverse
            ),
            guide = ifelse(identical(group_by, ".group"), "none", "legend")
        ) +
        scale_shape_discrete(
            name = group_name,
            guide = ifelse(identical(group_by, ".group"), "none", "legend")
        ) +
        geom_line(aes(linetype = !!sym("lty")), linewidth = line_width) +
        scale_linetype_manual(
            name = "",
            values = c("solid", "dashed"),
            # make items wider
            guide = guide_legend(
                theme = ggplot2::theme(legend.key.width = unit(1, "cm"))
            )
        ) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major = element_line(colour = "grey80", linetype = 2)
        ) +
        labs(title = title, subtitle = subtitle, x = xlab, y = ylab)

    if (isTRUE(se)) {
        p <- p +
            geom_ribbon(
                aes(
                    ymin = !!sym("y.lwr"),
                    ymax = !!sym("y.upr"),
                    fill = !!sym(group_by)
                ),
                color = "transparent",
                alpha = alpha
            ) +
            scale_fill_manual(
                name = group_name,
                values = palette_this(
                    levels(data[[group_by]]),
                    palette = palette,
                    palcolor = palcolor,
                    reverse = palreverse
                ),
                guide = ifelse(identical(group_by, ".group"), "none", "legend")
            )
    }

    dims <- calculate_plot_dimensions(
        base_height = 4.5,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = nlevels(data[[group_by]]),
        legend_nchar = max(nchar(levels(data[[group_by]])))
    )
    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction
    )
}


#' Rarefaction / extrapolation plot
#'
#' @description
#' Draws rarefaction and extrapolation curves for biodiversity data using the
#' \code{iNEXT} package.  Accepts raw species-abundance / incidence-frequency
#' lists (which are passed to \code{\link[iNEXT]{iNEXT}()} for estimation) or
#' pre-computed \code{iNEXT} objects.
#'
#' The function supports three curve types (sample-size-based, sample
#' completeness, and coverage-based), diversity orders (\code{q}), per-group
#' colouring, faceting, and splitting into separate sub-plots via
#' \code{split_by}.  Observed data are marked with points, rarefaction lines
#' are solid, and extrapolation segments are dashed.  Confidence intervals are
#' shown as semi-transparent ribbons.
#'
#' @section split_by workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \code{\link{validate_common_args}()} checks the \code{seed} and
#'         \code{facet_by} validity.
#'   \item The \code{type} argument is validated (must be one or more of
#'         1, 2, 3).
#'   \item \code{group_by}, \code{split_by}, and \code{facet_by} are validated
#'         for allowed values (\code{"q"} and/or \code{"group"}) and checked
#'         for mutual exclusivity — no parameter may overlap with another.
#'   \item If \code{data} is not an \code{iNEXT} object, it is passed to
#'         \code{\link[iNEXT]{iNEXT}()} with \code{...} (which may contain
#'         \code{q}, \code{datatype}, \code{nboot}, etc.).
#'   \item The \code{iNEXT} object is fortifed via
#'         \code{\link[ggplot2]{fortify}()} for the requested \code{type}s.
#'         Columns \code{Assemblage} and \code{Order.q} are renamed to
#'         \code{group} and \code{q}, respectively.
#'   \item The \code{se} parameter is resolved: if \code{NULL} it becomes
#'         \code{TRUE} when the fortifed data contains \code{y.lwr} /
#'         \code{y.upr} columns.
#'   \item A \code{lty} column is created (factor with levels
#'         \code{"Rarefaction"} and \code{"Extrapolation"}) to distinguish the
#'         two line phases via solid / dashed linetypes.
#'   \item \code{group_by}, \code{split_by}, and \code{facet_by} are processed
#'         via \code{\link{check_columns}()} with \code{force_factor = TRUE}
#'         and multi-column concatenation.
#'   \item If \code{group_by} is \code{NULL}, a dummy \code{".group"} column
#'         is created and the legend is hidden.
#'   \item The data is split by \code{split_by} (preserving level order).  If
#'         \code{split_by} is \code{NULL}, the data is wrapped in a
#'         single-element list with name \code{"..."}.
#'   \item Per-split \code{palette}, \code{palcolor},
#'         \code{legend.position}, and \code{legend.direction} are resolved
#'         via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
#'         and \code{\link{check_legend}()}.
#'   \item \code{\link{RarefactionPlotAtomic}()} is called for each split.
#'         If \code{title} is a function, it receives the split level name and
#'         can generate dynamic titles.
#'   \item Results are combined via \code{\link{combine_plots}()} (when
#'         \code{combine = TRUE}) or returned as a named list.
#' }
#'
#' @inheritParams common_args
#' @inheritParams RarefactionPlotAtomic
#' @param split_by A character vector specifying how to split the data into
#'  separate sub-plots.  Must be one or both of \code{"q"} (diversity order)
#'  and \code{"group"} (assemblage/site).  Multiple values are concatenated
#'  with \code{split_by_sep}.  Cannot overlap with \code{group_by} or
#'  \code{facet_by}.  Default: \code{NULL}.
#' @param split_by_sep A character string used to join multiple
#'  \code{split_by} column values when \code{split_by} has length > 1.
#'  Default: \code{"_"}.
#' @param group_by_sep A character string used to join multiple
#'  \code{group_by} column values when \code{group_by} has length > 1.
#'  Also used by the exported function for the group concatenation.
#'  Default: \code{"_"}.
#' @param seed A numeric seed for reproducibility.  Passed to
#'  \code{\link{validate_common_args}()}.  Default: \code{8525}.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'  \code{patchwork} object.  When \code{FALSE}, returns a named list of
#'  individual \code{ggplot} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined layout
#'  (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param byrow Logical; fill the combined layout by row.  Default \code{TRUE}.
#' @param axes A character string specifying how axes should be treated across
#'  the combined layout (passed to \code{\link{combine_plots}()}).
#' @param axis_titles A character string specifying how axis titles should be
#'  treated across the combined layout.  Defaults to \code{axes}.
#' @param guides A character string specifying how guides (legends) should be
#'  collected across panels (passed to \code{\link{combine_plots}()}).
#' @param design A custom layout design for the combined plot (passed to
#'  \code{\link{combine_plots}()}).
#' @param ... Additional arguments passed to \code{\link[iNEXT]{iNEXT}} when
#'  \code{data} is not already an \code{iNEXT} object.  Common options include
#'  \code{q} (diversity order, default \code{c(0, 1, 2)}), \code{datatype}
#'  (\code{"abundance"} or \code{"incidence"}), and \code{nboot} (number of
#'  bootstrap replicates).
#' @return A \code{ggplot} object (single split), a \code{patchwork} object
#'  (multiple splits with \code{combine = TRUE}), or a named list of
#'  \code{ggplot} objects (when \code{combine = FALSE}), each with
#'  \code{height} and \code{width} attributes in inches.
#' @export
#' @examples
#' \donttest{
#' set.seed(8525)
#' spider <- list(
#'    Girdled = c(46, 22, 17, 15, 15, 9, 8, 6, 6, 4, rep(2, 4), rep(1, 12)),
#'    Logged = c(88, 22, 16, 15, 13, 10, 8, 8, 7, 7, 7, 5, 4, 4, 4, 3, 3, 3, 3,
#'      2, 2, 2, 2, rep(1, 14))
#' )
#'
#' # Basic sample-size-based rarefaction (type = 1)
#' RarefactionPlot(spider)
#'
#' # Multiple diversity orders with faceting
#' RarefactionPlot(spider, q = c(0, 1, 2), facet_by = "q")
#'
#' # Multiple diversity orders split into sub-plots
#' RarefactionPlot(spider, q = c(0, 1, 2), split_by = "q")
#'
#' # Per-split palettes
#' RarefactionPlot(spider, q = c(0, 1, 2), split_by = "q",
#'                 palette = c("0" = "Paired", "1" = "Set1", "2" = "Dark2"))
#'
#' # Coverage-based rarefaction (type = 3) with
#' # group_by = "q" and facet_by = "group"
#' RarefactionPlot(spider, q = c(0, 1, 2), group_by = "q",
#'  facet_by = "group", palette = "Set1", type = 3)
#' }
RarefactionPlot <- function(
    data,
    type = 1,
    se = NULL,
    group_by = "group",
    group_by_sep = "_",
    group_name = NULL,
    split_by = NULL,
    split_by_sep = "_",
    theme = "theme_this",
    theme_args = list(),
    palette = "Spectral",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 0.2,
    pt_size = 3,
    line_width = 1,
    facet_by = NULL,
    facet_scales = "fixed",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    theme <- process_theme(theme)
    stopifnot(
        "Invalid 'type' value. It must be 1, 2, or 3 or combination of them." = length(
            type
        ) >
            0 &&
            length(setdiff(type, 1:3)) == 0
    )
    # check group_by, split_by, facet_by.
    # If given, they should be "q" or "group". Only facet_by could be both.
    stopifnot(
        "Invalid 'group_by' value. It must be 'q' and/or 'group'." = is.null(
            group_by
        ) ||
            all(group_by %in% c("q", "group"))
    )
    stopifnot(
        "Invalid 'split_by' value. It must be 'q' and/or 'group'." = is.null(
            split_by
        ) ||
            all(split_by %in% c("q", "group"))
    )
    stopifnot(
        "Invalid 'facet_by' value. It must be 'q' and/or 'group'." = is.null(
            facet_by
        ) ||
            all(facet_by %in% c("q", "group"))
    )
    # They should not overlap with each other.
    # If they are not NULL, they should be different.
    stopifnot(
        "Invalid 'group_by'/'split_by'. They should not overlap." = is.null(
            group_by
        ) ||
            is.null(split_by) ||
            length(intersect(group_by, split_by)) == 0
    )
    stopifnot(
        "Invalid 'group_by'/'facet_by'. They should not overlap." = is.null(
            group_by
        ) ||
            is.null(facet_by) ||
            length(intersect(group_by, facet_by)) == 0
    )
    stopifnot(
        "Invalid 'split_by'/'facet_by'. They should not overlap." = is.null(
            split_by
        ) ||
            is.null(facet_by) ||
            length(intersect(split_by, facet_by)) == 0
    )

    if (!inherits(data, "iNEXT")) {
        data <- iNEXT::iNEXT(data, ...)
    }

    data <- suppressWarnings({
        fortify(data, type = type)
    })
    # rename Assemblage to group and Order.q to q
    data <- rename(data, group = "Assemblage", q = "Order.q")

    se <- se %||% ('y.lwr' %in% names(data))

    data$Method2 <- data$Method
    data$Method2[data$Method2 == "Observed"] <- "Rarefaction"
    data$lty <- factor(data$Method2, levels = c("Rarefaction", "Extrapolation"))

    group_by <- check_columns(
        data,
        group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = group_by_sep
    )

    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )
    if (!is.null(facet_by)) {
        data[[facet_by]] <- droplevels(data[[facet_by]])
    }

    if (is.null(group_by)) {
        group_by <- ".group"
        data$.group <- factor("")
    }

    if (!is.null(split_by)) {
        data[[split_by]] <- droplevels(data[[split_by]])
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                ifelse(
                    identical(split_by, "q"),
                    paste(split_by, nm, sep = " = "),
                    nm
                )
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            RarefactionPlotAtomic(
                datas[[nm]],
                type = type,
                se = se,
                group_by = group_by,
                group_name = group_name,
                pt_size = pt_size,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                line_width = line_width,
                alpha = alpha,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                aspect.ratio = aspect.ratio,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

set.seed(8525)
# RarefactionPlot requires iNEXT package
# Input: a list of abundance vectors per sample/group
rarefy_data <- list(
    GroupA = c(10, 5, 3, 8, 2, 1, 7, 4, 6, 9, 3, 2, 5, 1, 4),
    GroupB = c(8, 4, 2, 6, 3, 1, 5, 7, 3, 2, 4, 9, 6, 1, 3)
)

test_that("RarefactionPlot basic usage with list input works", {
    skip_if_not_installed("iNEXT")
    p <- RarefactionPlot(rarefy_data)
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("RarefactionPlot with type = 2 (Shannon diversity) works", {
    skip_if_not_installed("iNEXT")
    p <- RarefactionPlot(rarefy_data, type = 2)
    expect_s3_class(p, "ggplot")
})

test_that("RarefactionPlot with title and subtitle works", {
    skip_if_not_installed("iNEXT")
    p <- RarefactionPlot(rarefy_data, title = "Rarefaction Title", subtitle = "Subtitle")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "Rarefaction Title")
    expect_equal(p$labels$subtitle, "Subtitle")
})

test_that("RarefactionPlot with group_by = 'q' and split_by = 'group' works", {
    skip_if_not_installed("iNEXT")
    p <- RarefactionPlot(rarefy_data, group_by = "group", split_by = "q", combine = TRUE)
    expect_true(inherits(p, "patchwork") || inherits(p, "gg"))
})

test_that("RarefactionPlot with combine = FALSE returns list when split_by = 'q'", {
    skip_if_not_installed("iNEXT")
    plots <- RarefactionPlot(rarefy_data, group_by = "group", split_by = "q", combine = FALSE)
    expect_true(is.list(plots))
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("RarefactionPlot with facet_by = 'q' works", {
    skip_if_not_installed("iNEXT")
    p <- RarefactionPlot(rarefy_data, group_by = "group", facet_by = "q")
    expect_s3_class(p, "ggplot")
})

test_that("RarefactionPlot with iNEXT object input works", {
    skip_if_not_installed("iNEXT")
    inext_obj <- iNEXT::iNEXT(rarefy_data)
    p <- RarefactionPlot(inext_obj)
    expect_s3_class(p, "ggplot")
})

test_that("RarefactionPlot with multiple split_by columns returns a list with one plot per combination", {
    skip_if_not_installed("iNEXT")
    plots <- suppressMessages(RarefactionPlot(rarefy_data, q = c(0, 1, 2),
                                              group_by = NULL, split_by = c("q", "group"), combine = FALSE))
    expect_true(is.list(plots))
    expect_length(plots, 6)   # 3 q values x 2 groups
    expect_s3_class(plots[[1]], "ggplot")
})

test_that("RarefactionPlot with multiple split_by columns returns combined plot", {
    skip_if_not_installed("iNEXT")
    p <- suppressMessages(RarefactionPlot(rarefy_data, q = c(0, 1, 2),
                                          group_by = NULL, split_by = c("q", "group"), combine = TRUE))
    expect_true(inherits(p, "gg") || inherits(p, "patchwork"))
})

#' Atomic ring plot (internal)
#'
#' @description
#' Core implementation for drawing a single ring (multi-layer donut) plot.
#' This is the workhorse behind the exported \code{\link{RingPlot}} function —
#' it takes a **single** data frame (no \code{split_by} support) and returns a
#' \code{ggplot} object.  Each level of \code{x} becomes a concentric ring,
#' and within each ring the \code{group_by} variable divides the ring into
#' filled segments drawn via \code{geom_col()} with \code{coord_polar("y")}.
#'
#' When \code{y = NULL}, the count of observations per combination is used.
#' y-values are always normalised to proportions (0–1) within each
#' (\code{x}, \code{facet_by}) group so that each complete ring sums to 1.
#'
#' @section Architecture:
#' \enumerate{
#'   \item \strong{Column resolution} — \code{x}, \code{group_by}, \code{y},
#'         and \code{facet_by} are validated and transformed via
#'         \code{\link{check_columns}}.  If \code{x} is \code{NULL}, a dummy
#'         \code{.x} column (value \code{1}) is created, producing a single-ring
#'         (pie-chart-like) plot.  If \code{group_by} is \code{NULL}, a dummy
#'         \code{.group_by} factor is created and the legend is suppressed.
#'         Multi-column \code{group_by} inputs are concatenated with
#'         \code{group_by_sep}.
#'   \item \strong{NA / empty-level handling} — \code{\link{process_keep_na_empty}()}
#'         applies \code{keep_na} and \code{keep_empty} policies.  Per-column
#'         \code{keep_empty} settings are extracted for \code{x},
#'         \code{group_by}, and \code{facet_by} independently.  The facet
#'         columns must have identical \code{keep_empty} values.
#'   \item \strong{Count aggregation} — when \code{y = NULL}, the count of
#'         observations in each unique (\code{x}, \code{group_by},
#'         \code{facet_by}) combination is computed as a new \code{.y} column.
#'         Factor levels are preserved after aggregation.
#'   \item \strong{Proportion scaling} — y-values are divided by the sum within
#'         each (\code{x}, \code{facet_by}) group, normalising each ring to
#'         sum to 1.0.  This is essential for polar coordinates where each ring
#'         represents a full 360 degrees.
#'   \item \strong{Ring resolution} — The levels of \code{x} are reversed so
#'         the outermost ring appears first in the data (rings are drawn
#'         outward from the origin).  \code{NA} levels are appended.
#'   \item \strong{Label logic} — When \code{label = NULL}, labels are
#'         automatically suppressed for single-ring plots and shown for
#'         multi-ring plots.  Labels are placed at the inner edge of each ring
#'         via \code{geom_label()}.
#'   \item \strong{Clockwise ordering} — When \code{clockwise = TRUE} (default),
#'         the \code{group_by} levels are reversed so segments appear clockwise
#'         from the top.  The legend guide also reverses its order.
#'   \item \strong{Colour mapping} — \code{\link{palette_this}()} assigns
#'         colours to all \code{group_by} levels, including \code{NA}
#'         (defaulting to \code{"grey80"}).
#'   \item \strong{Plot assembly} — The \code{ggplot} object is built with
#'         \code{geom_col()} (ring segments), \code{coord_polar("y")} (polar
#'         transform), \code{scale_x_discrete()} (with a leading space level
#'         as the donut hole), and \code{scale_fill_manual()} with per-group
#'         colours.  All axes, ticks, grid lines, and panel borders are removed
#'         for a clean appearance.  The fill scale \code{drop} argument is
#'         controlled by \code{keep_empty_group}.
#'   \item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
#'         computes plot height and width from \code{aspect.ratio} and legend
#'         metrics.  The resulting \code{height} / \code{width} attributes are
#'         stored on the \code{ggplot} object.
#'   \item \strong{Faceting} — \code{\link{facet_plot}()} wraps the plot with
#'         \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is
#'         provided, respecting the \code{keep_empty} setting for facet
#'         variables.
#' }
#'
#' @inheritParams common_args
#' @param x A character string specifying the column to define the rings
#'  (concentric layers) of the plot.  Each unique value becomes one ring.
#'  When \code{NULL}, a single-ring (pie-chart-like) plot is produced.
#'  Character or factor columns are expected.  Multiple columns are not
#'  supported for \code{x} in ring plots.
#' @param y A character string specifying the numeric column for segment
#'  sizing.  When \code{NULL} (default), the count of observations in each
#'  (\code{x}, \code{group_by}, \code{facet_by}) combination is used.
#'  Values are always normalised to proportions within each ring.
#' @param group_by A character vector of column names to divide each ring
#'  into filled segments.  Each unique combination becomes one segment per
#'  ring.  Multiple columns are concatenated with \code{group_by_sep}.
#'  When \code{NULL}, a single unsegmented ring is drawn and the legend
#'  is hidden.
#' @param group_by_sep A character string to join multiple \code{group_by}
#'  columns.  Default \code{"_"}.  Ignored when \code{group_by} is a single
#'  column.
#' @param group_name A character string used as the fill legend title.
#'  When \code{NULL}, the \code{group_by} column name is used.
#' @param label A logical value controlling whether ring labels are shown.
#'  Labels display the \code{x} values (ring names) at the inner edge of
#'  each ring.  Default \code{NULL} auto-selects: \code{FALSE} for
#'  single-ring plots, \code{TRUE} for multi-ring plots.
#' @param clockwise A logical value.  When \code{TRUE} (default), segments
#'  within each ring are drawn clockwise (top-to-right).  When
#'  \code{FALSE}, they are drawn counter-clockwise.  The legend also
#'  reverses to match the drawing order.
#' @keywords internal
#' @return A \code{ggplot} object with \code{height} and \code{width}
#'  attributes (in inches) attached.
#' @importFrom rlang sym syms
#' @importFrom dplyr %>% summarise n ungroup mutate
#' @importFrom ggplot2 geom_col scale_fill_manual scale_x_discrete geom_label
RingPlotAtomic <- function(
    data,
    x = NULL,
    y = NULL,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    label = NULL,
    clockwise = TRUE,
    keep_na = FALSE,
    keep_empty = FALSE,
    facet_by = NULL,
    facet_scales = "free_y",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    aspect.ratio = 1,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    seed = 8525,
    ...
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    base_size <- theme_args$base_size %||% 12
    text_size_scale <- base_size / 12

    if (is.null(x)) {
        x <- ".x"
        data$.x <- 1
    }
    x <- check_columns(data, x, force_factor = TRUE)
    group_by <- check_columns(
        data,
        group_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = group_by_sep
    )
    facet_by <- check_columns(
        data,
        facet_by,
        force_factor = TRUE,
        allow_multi = TRUE
    )
    if (is.null(group_by)) {
        group_by <- ".group_by"
        data[[group_by]] <- factor("")
        group_guide <- "none"
    } else {
        group_guide <- guide_legend(reverse = clockwise)
    }

    data <- process_keep_na_empty(data, keep_na, keep_empty)
    keep_empty_x <- keep_empty[[x]]
    keep_empty_group <- if (!is.null(group_by)) keep_empty[[group_by]] else NULL
    keep_empty_facet <- if (!is.null(facet_by)) {
        keep_empty[[facet_by[1]]]
    } else {
        NULL
    }
    if (length(facet_by) > 1) {
        stopifnot(
            "[RingPlot] `keep_empty` for `facet_by` variables must be identical." = identical(
                keep_empty_facet,
                keep_empty[[facet_by[2]]]
            )
        )
    }

    orig_data <- data
    if (is.null(y)) {
        data <- data %>%
            dplyr::group_by(!!!syms(unique(c(x, group_by, facet_by)))) %>%
            summarise(.y = n(), .groups = "drop")
        for (col in unique(c(x, facet_by))) {
            data[[col]] <- factor(
                data[[col]],
                levels = levels(orig_data[[col]])
            )
        }
        y <- ".y"
    } else {
        y <- check_columns(data, y)
    }
    # make sure each ring sums to 1
    data <- data %>%
        dplyr::group_by(!!!syms(unique(c(x, facet_by)))) %>%
        mutate(!!sym(y) := !!sym(y) / sum(!!sym(y))) %>%
        ungroup()
    for (col in unique(x, facet_by)) {
        data[[col]] <- factor(data[[col]], levels = levels(orig_data[[col]]))
    }
    rm(orig_data)

    rings <- rev(levels(data[[x]]))
    if (anyNA(data[[x]])) {
        rings <- c(rings, NA)
    }
    if (length(rings) == 1 && is.null(label)) {
        label <- FALSE
    } else if (length(rings) > 1 && is.null(label)) {
        label <- TRUE
    }
    if (isTRUE(clockwise)) {
        data[[group_by]] <- factor(
            data[[group_by]],
            levels = rev(levels(data[[group_by]]))
        )
        group_vals <- rev(levels(data[[group_by]]))
        if (anyNA(data[[group_by]])) {
            group_vals <- c(group_vals, NA)
        }
    } else {
        group_vals <- levels(data[[group_by]])
        if (anyNA(data[[group_by]])) {
            group_vals <- c(group_vals, NA)
        }
    }
    data <- data[order(data[[group_by]]), , drop = FALSE]
    colors <- palette_this(
        group_vals,
        palette = palette,
        palcolor = palcolor,
        NA_keep = TRUE,
        reverse = palreverse
    )

    p <- ggplot(data, aes(x = !!sym(x), y = !!sym(y), fill = !!sym(group_by))) +
        geom_col(
            width = 0.9,
            color = "white",
            alpha = alpha,
            show.legend = TRUE
        ) +
        coord_polar("y", start = 0) +
        scale_x_discrete(limits = c(" ", rings), drop = !isTRUE(keep_empty_x)) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            axis.text = element_blank(),
            axis.ticks = element_blank(),
            axis.title = element_blank(),
            panel.border = element_blank()
        ) +
        labs(
            title = title,
            subtitle = subtitle,
            x = xlab %||% "",
            y = ylab %||% ""
        )

    if (isTRUE(keep_empty_group)) {
        p <- p +
            scale_fill_manual(
                name = group_name %||% group_by,
                values = colors,
                na.value = colors["NA"] %||% "grey80",
                breaks = rev(group_vals),
                limits = rev(group_vals),
                drop = FALSE,
                guide = group_guide
            )
    } else {
        p <- p +
            scale_fill_manual(
                name = group_name %||% group_by,
                values = colors,
                na.value = colors["NA"] %||% "grey80",
                breaks = rev(group_vals),
                drop = TRUE,
                guide = group_guide
            )
    }

    if (label) {
        # Create label data frame with all levels (including empty ones)
        label_df <- data.frame(
            x = rings[!is.na(rings)],
            y = 0,
            stringsAsFactors = FALSE
        )
        if (anyNA(rings)) {
            label_df <- rbind(
                label_df,
                data.frame(x = NA, y = 0, stringsAsFactors = FALSE)
            )
        }
        names(label_df)[1] <- x
        label_df[[x]] <- factor(label_df[[x]], levels = rings)
        label_df$label_text <- ifelse(
            is.na(label_df[[x]]),
            "NA",
            as.character(label_df[[x]])
        )

        p <- p +
            geom_label(
                aes(label = !!sym("label_text"), x = !!sym(x), y = 0),
                data = label_df,
                inherit.aes = FALSE,
                color = "grey20",
                size = text_size_scale * 3
            )
    }

    dims <- calculate_plot_dimensions(
        base_height = 4.5,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction,
        legend_n = length(group_vals),
        legend_nchar = max(nchar(as.character(group_vals)), na.rm = TRUE)
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    facet_plot(
        p,
        facet_by,
        facet_scales,
        facet_nrow,
        facet_ncol,
        facet_byrow,
        legend.position = legend.position,
        legend.direction = legend.direction,
        drop = !isTRUE(keep_empty_facet)
    )
}

#' Ring plot (multi-layer donut chart)
#'
#' @description
#' Draws a ring plot (multi-layer donut chart) where each level of \code{x}
#' becomes a concentric ring divided into filled segments by \code{group_by}.
#' The plot is built with \code{geom_col()} under \code{coord_polar("y")},
#' producing a publication-quality ring chart with automatic count aggregation,
#' per-group colour assignment, faceting, and splitting into sub-plots.
#'
#' When \code{x = NULL}, a single-ring plot is produced (functionally
#' equivalent to a pie chart via \code{\link{PieChart}}).
#'
#' @section split_by workflow:
#' When \code{split_by} is provided:
#' \enumerate{
#'   \item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
#'         normalise the \code{keep_na} / \code{keep_empty} arguments for all
#'         columns (\code{x}, \code{group_by}, \code{split_by}, \code{facet_by}).
#'   \item The \code{split_by} column is validated and its NA / empty levels
#'         are processed via \code{\link{process_keep_na_empty}()}.  It is
#'         then removed from the per-column \code{keep_na} / \code{keep_empty}
#'         lists.
#'   \item The data frame is split by \code{split_by} (preserving level
#'         order).  If \code{split_by} is \code{NULL}, the data is wrapped in
#'         a single-element list with name \code{"..."}.
#'   \item Per-split \code{palette}, \code{palcolor},
#'         \code{legend.position}, and \code{legend.direction} are resolved
#'         via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
#'         and \code{\link{check_legend}()}.
#'   \item \code{\link{RingPlotAtomic}()} is called for each split.  If
#'         \code{title} is a function, it receives the split level name and
#'         can generate dynamic titles.
#'   \item Results are combined via \code{\link{combine_plots}()} (when
#'         \code{combine = TRUE}) or returned as a named list.
#' }
#'
#' @seealso \code{\link{PieChart}}
#' @inheritParams common_args
#' @inheritParams RingPlotAtomic
#' @param split_by The column(s) to split the data by and produce separate
#'  sub-plots.  Multiple columns are concatenated with \code{split_by_sep}.
#' @param split_by_sep A character string to separate concatenated
#'  \code{split_by} columns.  Default \code{"_"}.
#' @param seed A numeric seed for reproducibility.  Passed to
#'  \code{\link{validate_common_args}()}.
#' @param combine Logical; when \code{TRUE} (default), returns a combined
#'  \code{patchwork} object.  When \code{FALSE}, returns a named list of
#'  individual \code{ggplot} objects.
#' @param ncol,nrow Integer number of columns / rows for the combined layout
#'  (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param byrow Logical; fill the combined layout by row.  Default \code{TRUE}
#'  (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axes A character string specifying how axes should be treated across
#'  the combined layout (passed to \code{\link[patchwork]{wrap_plots}}).
#' @param axis_titles A character string specifying how axis titles should be
#'  treated across the combined layout.  Defaults to \code{axes}.
#' @param guides A character string specifying how guides (legends) should be
#'  collected across panels.  Default \code{"collect"} (passed to
#'  \code{\link{combine_plots}()}).
#' @param design A custom layout design for the combined plot (passed to
#'  \code{\link{combine_plots}()}).
#' @return A \code{ggplot} object, a \code{patchwork} object, or a named list
#'  of \code{ggplot} objects (when \code{combine = FALSE}), each with
#'  \code{height} and \code{width} attributes in inches.
#' @export
#' @examples
#' \donttest{
#' # Basic single-ring plot (pie-chart-like)
#' RingPlot(datasets::iris, group_by = "Species")
#'
#' # Multi-ring plot with faceting
#' RingPlot(datasets::mtcars, x = "cyl", group_by = "carb", facet_by = "vs")
#'
#' # Split into sub-plots with per-split palettes
#' RingPlot(datasets::mtcars, x = "cyl", group_by = "carb", split_by = "vs",
#'         palette = c("0" = "Set1", "1" = "Paired"))
#'
#' # Custom data with NA and empty levels
#' data <- data.frame(
#'   x = factor(c("A", "B", NA, "D", "A", "B", NA, "D"), levels = c("A", "B", "C", "D")),
#'   y = c(1, 2, 5, 3, 4, 5, 2, 6),
#'   group = factor(c("a", "a", "a", NA, NA, "c", "c", "c"), levels = c("a", "b", "c"))
#' )
#'
#' # Default: NA and empty levels dropped
#' RingPlot(data, x = "x", y = "y", group_by = "group")
#'
#' # Keep NA and empty levels
#' RingPlot(data, x = "x", y = "y", group_by = "group",
#'         keep_na = TRUE, keep_empty = TRUE)
#'
#' # Per-column keep_na / keep_empty via named lists
#' RingPlot(data, x = "x", y = "y", group_by = "group",
#'         keep_na = TRUE, keep_empty = list(x = FALSE, group = 'level'))
#' }
RingPlot <- function(
    data,
    x = NULL,
    y = NULL,
    group_by = NULL,
    group_by_sep = "_",
    group_name = NULL,
    label = NULL,
    split_by = NULL,
    split_by_sep = "_",
    facet_by = NULL,
    facet_scales = "free_y",
    facet_ncol = NULL,
    facet_nrow = NULL,
    facet_byrow = TRUE,
    theme = "theme_this",
    theme_args = list(),
    palette = "Paired",
    palcolor = NULL,
    palreverse = FALSE,
    alpha = 1,
    aspect.ratio = 1,
    keep_na = FALSE,
    keep_empty = FALSE,
    legend.position = "right",
    legend.direction = "vertical",
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    combine = TRUE,
    nrow = NULL,
    ncol = NULL,
    byrow = TRUE,
    seed = 8525,
    axes = NULL,
    axis_titles = axes,
    guides = NULL,
    design = NULL,
    ...
) {
    validate_common_args(seed, facet_by = facet_by)
    keep_na <- check_keep_na(keep_na, c(x, group_by, split_by, facet_by))
    keep_empty <- check_keep_empty(
        keep_empty,
        c(x, group_by, split_by, facet_by)
    )
    theme <- process_theme(theme)
    split_by_orig <- split_by
    split_by <- check_columns(
        data,
        split_by,
        force_factor = TRUE,
        allow_multi = TRUE,
        concat_multi = TRUE,
        concat_sep = split_by_sep
    )

    if (!is.null(split_by)) {
        data <- process_keep_na_empty(data, keep_na, keep_empty, col = split_by)
        keep_na[split_by_orig] <- NULL
        keep_empty[split_by_orig] <- NULL
        datas <- split(data, data[[split_by]])
        # keep the order of levels
        datas <- datas[levels(data[[split_by]])]
    } else {
        datas <- list(data)
        split_by <- names(datas) <- "..."
    }
    palette <- check_palette(palette, names(datas))
    palcolor <- check_palcolor(palcolor, names(datas))
    legend.direction <- check_legend(
        legend.direction,
        names(datas),
        "legend.direction"
    )
    legend.position <- check_legend(
        legend.position,
        names(datas),
        "legend.position"
    )

    plots <- lapply(
        names(datas),
        function(nm) {
            default_title <- if (length(datas) == 1 && identical(nm, "...")) {
                NULL
            } else {
                nm
            }
            if (is.function(title)) {
                title <- title(default_title)
            } else {
                title <- title %||% default_title
            }
            RingPlotAtomic(
                datas[[nm]],
                x = x,
                y = y,
                label = label,
                group_by = group_by,
                group_by_sep = group_by_sep,
                group_name = group_name,
                facet_by = facet_by,
                facet_scales = facet_scales,
                facet_ncol = facet_ncol,
                facet_nrow = facet_nrow,
                facet_byrow = facet_byrow,
                theme = theme,
                theme_args = theme_args,
                palette = palette[[nm]],
                palcolor = palcolor[[nm]],
                palreverse = palreverse,
                alpha = alpha,
                aspect.ratio = aspect.ratio,
                keep_na = keep_na,
                keep_empty = keep_empty,
                legend.position = legend.position[[nm]],
                legend.direction = legend.direction[[nm]],
                title = title,
                subtitle = subtitle,
                xlab = xlab,
                ylab = ylab,
                seed = seed,
                ...
            )
        }
    )

    names(plots) <- names(datas)

    combine_plots(
        plots,
        combine = combine,
        split_by = split_by,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow,
        axes = axes,
        axis_titles = axis_titles,
        guides = guides,
        design = design
    )
}

#' Compute velocity on a regular grid from sparse cell embeddings
#'
#' Computes velocity vectors on a regular grid by averaging cell-level velocities
#' within a Gaussian-weighted neighborhood of each grid point. This function is
#' adapted from the scvelo Python implementation at
#' \url{https://github.com/theislab/scvelo/blob/master/scvelo/plotting/velocity_embedding_grid.py}.
#'
#' @param embedding A numeric matrix of dimension n_obs x n_dim containing the low-dimensional embedding coordinates of each cell.
#' @param v_embedding A numeric matrix of dimension n_obs x n_dim containing the velocity vectors for each cell.
#' @param density A numeric value specifying the density of the grid points along each dimension. Higher values produce a finer grid. Default is 1.
#' @param smooth A numeric value specifying the standard deviation multiplier for the Gaussian kernel used to weight neighboring cells when averaging velocities onto grid points. Default is 0.5.
#' @param n_neighbors An integer value specifying the number of nearest neighbors to consider for each grid point when computing the weighted average velocity. Default is \code{ceiling(n_obs / 50)}.
#' @param min_mass A numeric value specifying the minimum mass threshold. Grid points with total weight below this threshold are filtered out. When \code{adjust_for_stream = TRUE}, this is interpreted on a logarithmic scale (\code{10^(min_mass - 6)}). Default is 1.
#' @param scale A numeric value specifying the scaling factor to apply to the resulting grid velocity vectors. Only used when \code{adjust_for_stream = FALSE}. Default is 1.
#' @param adjust_for_stream A logical value indicating whether to adjust the output for streamline rendering. When \code{TRUE}, returns a 2D array format with different filtering logic suitable for \code{metR::geom_streamline}. Default is \code{FALSE}.
#' @param cutoff_perc A numeric value specifying the percentile cutoff for removing low-density grid points. Only used when \code{adjust_for_stream = TRUE}. Default is 5.
#' @return A list with components \code{x_grid} (grid point coordinates) and \code{v_grid} (velocity vectors at each grid point). When \code{adjust_for_stream = TRUE}, \code{x_grid} is a 2-row matrix of unique coordinates and \code{v_grid} is a 3D array.
#' @keywords internal
.compute_velocity_on_grid <- function(
    embedding,
    v_embedding,
    density = NULL,
    smooth = NULL,
    n_neighbors = NULL,
    min_mass = NULL,
    scale = 1,
    adjust_for_stream = FALSE,
    cutoff_perc = NULL
) {
    n_obs <- nrow(embedding)
    n_dim <- ncol(embedding)

    density <- density %||% 1
    smooth <- smooth %||% 0.5
    n_neighbors <- n_neighbors %||% ceiling(n_obs / 50)
    min_mass <- min_mass %||% 1
    cutoff_perc <- cutoff_perc %||% 5

    grs <- list()
    for (dim_i in 1:n_dim) {
        m <- min(embedding[, dim_i], na.rm = TRUE)
        M <- max(embedding[, dim_i], na.rm = TRUE)
        # m <- m - 0.01 * abs(M - m)
        # M <- M + 0.01 * abs(M - m)
        gr <- seq(m, M, length.out = ceiling(50 * density))
        grs <- c(grs, list(gr))
    }
    x_grid <- as.matrix(expand.grid(grs))

    d <- proxyC::dist(
        x = Matrix::Matrix(as.matrix(embedding), sparse = TRUE),
        y = Matrix::Matrix(x_grid, sparse = TRUE),
        method = "euclidean",
        use_nan = TRUE
    )

    neighbors <- t(as.matrix(apply(d, 2, function(x) {
        order(x, decreasing = FALSE)[1:n_neighbors]
    })))
    dists <- t(as.matrix(apply(d, 2, function(x) {
        x[order(x, decreasing = FALSE)[1:n_neighbors]]
    })))

    weight <- stats::dnorm(
        dists,
        sd = mean(sapply(grs, function(g) g[2] - g[1])) * smooth
    )
    p_mass <- p_mass_v <- rowSums(weight)
    p_mass_v[p_mass_v < 1] <- 1

    neighbors_emb <- array(
        as.matrix(v_embedding)[neighbors, seq_len(ncol(v_embedding))],
        dim = c(dim(neighbors), dim(v_embedding)[2])
    )
    v_grid <- apply((neighbors_emb * c(weight)), c(1, 3), sum)
    v_grid <- v_grid / p_mass_v

    if (isTRUE(adjust_for_stream)) {
        x_grid <- matrix(
            c(unique(x_grid[, 1]), unique(x_grid[, 2])),
            nrow = 2,
            byrow = TRUE
        )
        ns <- floor(sqrt(length(v_grid[, 1])))
        v_grid <- array(t(v_grid), dim = c(2, ns, ns))

        mass <- sqrt(apply(v_grid**2, c(2, 3), sum))
        min_mass <- 10**(min_mass - 6) # default min_mass = 1e-5
        min_mass[min_mass > max(mass, na.rm = TRUE) * 0.9] <- max(
            mass,
            na.rm = TRUE
        ) *
            0.9
        cutoff <- array(t(mass), dim = c(ns, ns)) < min_mass

        lens <- t(apply(apply(abs(neighbors_emb), c(1, 3), mean), 1, sum))
        lens <- array(t(lens), dim = c(ns, ns))

        cutoff <- cutoff | lens < quantile(lens, cutoff_perc / 100)
        v_grid[1, , ][cutoff] <- NA
    } else {
        min_mass <- min_mass * quantile(p_mass, 0.99) / 100
        x_grid <- x_grid[p_mass > min_mass, , drop = FALSE]
        v_grid <- v_grid[p_mass > min_mass, , drop = FALSE]
        if (!is.null(scale)) {
            v_grid <- v_grid * scale
        }
    }

    return(list(x_grid = x_grid, v_grid = v_grid))
}

#' Cell velocity plot
#'
#' Plots RNA velocity vectors on a low-dimensional embedding (e.g., UMAP, t-SNE)
#' to visualize the direction and magnitude of cellular state transitions. Supports
#' three visualization modes: raw arrows at each cell position, arrows on a regular
#' grid, and streamline paths. Optionally colors arrows by cell metadata groups.
#'
#' @section Rendering Pipeline:
#' The \code{VelocityPlot} function proceeds through the following steps:
#' \enumerate{
#'   \item{\strong{Input validation} --- Verifies that \code{embedding} and \code{v_embedding} are matrices or data frames of equal dimensions, that \code{group_by} matches the number of rows (if provided), and that \code{split_by} is \code{NULL} (unsupported and raises an error).}
#'   \item{\strong{Axis label resolution} --- Uses the column names of \code{embedding} as axis labels, falling back to \code{"Reduction 1"} and \code{"Reduction 2"} when column names are \code{NULL}.}
#'   \item{\strong{Grouping setup} --- Converts \code{group_by} to a factor and applies \code{keep_na} / \code{keep_empty} logic to filter or recode missing values and empty factor levels.}
#'   \item{\strong{Plot-type dispatch} --- Branches on \code{plot_type}:}
#'   \itemize{
#'     \item{\strong{raw} --- Optionally subsamples cells when \code{density < 1}, scales velocity vectors by \code{scale}, computes arrow lengths proportional to the embedding range, and renders \code{\link[ggplot2]{geom_segment}} with arrowheads. When \code{group_by} is provided, arrows are colored by group using \code{group_palette}.}
#'     \item{\strong{grid} --- Delegates to \code{\link{.compute_velocity_on_grid}} to interpolate the sparse cell velocities onto a regular grid, then renders \code{\link[ggplot2]{geom_segment}} with arrowheads at each grid point. \code{group_by} is ignored with a warning.}
#'     \item{\strong{stream} --- Delegates to \code{\link{.compute_velocity_on_grid}} with \code{adjust_for_stream = TRUE}, then renders smooth streamline paths via \code{\link[metR]{geom_streamline}}. When \code{streamline_color} is provided, streamlines use a fixed color with a background stroke; when \code{NULL}, streamlines are colored by velocity magnitude using \code{streamline_palette}. \code{group_by} is ignored with a warning.}
#'   }
#'   \item{\strong{Layer return or plot assembly} --- If \code{return_layer = TRUE}, returns the list of ggplot layers. Otherwise, constructs a full \code{ggplot} object with labels, theme, aspect ratio, legend configuration, and \code{height} / \code{width} attributes via \code{\link{calculate_plot_dimensions}()}.}
#' }
#'
#' @inheritParams common_args
#' @param embedding A matrix or data frame of dimension n_obs x n_dim specifying the low-dimensional embedding coordinates (e.g., UMAP, t-SNE) of the cells. The first two columns are used for the x and y axes.
#' @param v_embedding A matrix or data frame of dimension n_obs x n_dim specifying the velocity vectors for each cell. Must have the same dimensions as \code{embedding}.
#' @param plot_type A character string specifying the visualization method. \code{"raw"} plots arrows directly from each cell's embedding position. \code{"grid"} averages velocities onto a regular grid and plots arrows at grid points. \code{"stream"} computes smooth streamline paths from the gridded velocity field. Default is \code{"raw"}.
#' @param split_by Not supported for VelocityPlot. Setting this parameter will raise an error.
#' @param group_by An optional vector of the same length as the number of rows in \code{embedding} specifying a grouping variable for cells. When provided, arrows are colored by group using \code{group_palette}. Only applies to \code{plot_type = "raw"}; ignored with a warning for \code{"grid"} and \code{"stream"}. Default is \code{NULL}.
#' @param group_name A character string specifying the legend title for the grouping variable. Default is \code{"Group"}.
#' @param group_palette A character string specifying the color palette to use for the grouping variable. Passed to \code{\link{palette_this}}. Default is \code{"Paired"}.
#' @param group_palcolor An optional character vector of specific colors for the grouping variable. If \code{NULL}, colors are generated from \code{group_palette}. Default is \code{NULL}.
#' @param n_neighbors An integer value specifying the number of nearest neighbors for computing grid velocities. Only used when \code{plot_type} is \code{"grid"} or \code{"stream"}. Default is \code{ceiling(nrow(embedding) / 50)}.
#' @param density A numeric value specifying the grid density along each dimension. Only used when \code{plot_type} is \code{"grid"} or \code{"stream"}. For \code{plot_type = "raw"}, when \code{density} is between 0 and 1, it specifies the fraction of cells to randomly subsample. Default is 1.
#' @param smooth A numeric value specifying the standard deviation multiplier for the Gaussian kernel when averaging cell velocities onto grid points. Only used when \code{plot_type} is \code{"grid"} or \code{"stream"}. Default is 0.5.
#' @param scale A numeric value specifying the scaling factor for the velocity vectors. Applied to raw and grid arrows. For \code{plot_type = "stream"}, this is fixed to 1 internally. Default is 1.
#' @param min_mass A numeric value specifying the minimum mass threshold for retaining grid points. Only used when \code{plot_type} is \code{"grid"} or \code{"stream"}. Default is 1.
#' @param cutoff_perc A numeric value specifying the percentile cutoff for removing low-density grid points. Only used when \code{plot_type} is \code{"stream"}. Default is 5.
#' @param arrow_angle A numeric value specifying the angle of the arrowheads in degrees. Applied to \code{\link[grid]{arrow}} when \code{plot_type} is \code{"raw"} or \code{"grid"}. Default is 20.
#' @param arrow_color A character string specifying the color of the velocity arrows. For \code{plot_type = "stream"}, this sets only the arrowhead color. Default is \code{"black"}.
#' @param arrow_alpha A numeric value between 0 and 1 specifying the transparency of the velocity arrows. Only used when \code{plot_type = "raw"} or \code{"grid"}; for \code{plot_type = "stream"}, use \code{streamline_alpha} instead. Default is 1.
#' @param keep_na A logical or character value specifying how to handle NA values in \code{group_by}. Unlike other plot functions, VelocityPlot does not support named lists for per-column control. See \code{keep_na} in \code{common_args} for details of supported values. Default is \code{FALSE}.
#' @param keep_empty One of \code{FALSE}, \code{TRUE}, or \code{"level"} specifying how to handle empty factor levels in \code{group_by}. Unlike other plot functions, VelocityPlot does not support named lists for per-column control. See \code{keep_empty} in \code{common_args} for details. Default is \code{FALSE}.
#' @param streamline_l A numeric value specifying the integration length of the streamlines. Passed to \code{\link[metR]{geom_streamline}} as the \code{L} parameter. Default is 5.
#' @param streamline_minl A numeric value specifying the minimum streamline length. Shorter streamlines are not drawn. Passed to \code{\link[metR]{geom_streamline}} as the \code{min.L} parameter. Default is 1.
#' @param streamline_res A numeric value specifying the resolution of the streamline integration. Passed to \code{\link[metR]{geom_streamline}} as the \code{res} parameter. Default is 1.
#' @param streamline_n A numeric value specifying the number of streamlines to draw. Passed to \code{\link[metR]{geom_streamline}} as the \code{n} parameter. Default is 15.
#' @param streamline_width A numeric vector of length 2 specifying the range of line widths for streamlines. Passed to \code{scale_size(range = ...)}. Only used when \code{streamline_color} is \code{NULL}. Default is \code{c(0, 0.8)}.
#' @param streamline_alpha A numeric value between 0 and 1 specifying the transparency of the velocity streamlines. Default is 1.
#' @param streamline_color An optional character string specifying a fixed color for streamlines. When \code{NULL} (the default), streamlines are colored by velocity magnitude using \code{streamline_palette}.
#' @param streamline_palette A character string specifying the color palette for streamline velocity magnitude. Passed to \code{\link{palette_this}}. Only used when \code{streamline_color} is \code{NULL}. Default is \code{"RdYlBu"}.
#' @param streamline_palcolor An optional character vector of specific colors for the streamline velocity gradient. If \code{NULL}, colors are generated from \code{streamline_palette}. Default is \code{NULL}.
#' @param streamline_bg_color A character string specifying the background (outline) color applied to streamlines to create a stroke effect. Default is \code{"white"}.
#' @param streamline_bg_stroke A numeric value specifying the additional line width of the background stroke relative to the foreground streamline. Default is 0.5.
#' @param return_layer A logical value indicating whether to return only the ggplot layers instead of the full assembled plot. When \code{TRUE}, returns a list of ggplot layers suitable for combining with other ggplot objects. Default is \code{FALSE}.
#' @return A ggplot object representing the cell velocity plot, with \code{height} and \code{width} attributes set for consistent rendering. If \code{return_layer = TRUE}, returns a list of ggplot layers instead.
#' @importFrom rlang %||% sym
#' @importFrom ggplot2 geom_segment scale_color_gradientn scale_size after_stat
#' @export
#' @seealso \code{\link{DimPlot}} \code{\link{FeatureDimPlot}}
#' @examples
#' \donttest{
#' data(dim_example)
#' dim_example$clusters[dim_example$clusters == "Ductal"] <- NA
#'
#' # Basic velocity plot with group coloring
#' VelocityPlot(dim_example[, 1:2], dim_example[, 3:4], group_by = dim_example$clusters)
#'
#' # Handle NA groups with keep_na / keep_empty
#' VelocityPlot(dim_example[, 1:2], dim_example[, 3:4], group_by = dim_example$clusters,
#'     keep_na = TRUE, keep_empty = TRUE)
#' VelocityPlot(dim_example[, 1:2], dim_example[, 3:4], group_by = dim_example$clusters,
#'     keep_na = TRUE, keep_empty = 'level')
#' VelocityPlot(dim_example[, 1:2], dim_example[, 3:4], group_by = dim_example$clusters,
#'     keep_na = TRUE, keep_empty = FALSE)
#' }
VelocityPlot <- function(
    embedding,
    v_embedding,
    plot_type = c("raw", "grid", "stream"),
    split_by = NULL,
    group_by = NULL,
    group_name = "Group",
    group_palette = "Paired",
    group_palcolor = NULL,
    n_neighbors = NULL,
    density = 1,
    smooth = 0.5,
    scale = 1,
    min_mass = 1,
    cutoff_perc = 5,
    arrow_angle = 20,
    arrow_color = "black",
    arrow_alpha = 1,
    keep_na = FALSE,
    keep_empty = FALSE,
    streamline_l = 5,
    streamline_minl = 1,
    streamline_res = 1,
    streamline_n = 15,
    streamline_width = c(0, 0.8),
    streamline_alpha = 1,
    streamline_color = NULL,
    streamline_palette = "RdYlBu",
    streamline_palcolor = NULL,
    palreverse = FALSE,
    streamline_bg_color = "white",
    streamline_bg_stroke = 0.5,
    aspect.ratio = 1,
    title = "Cell velocity",
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    theme = "theme_this",
    theme_args = list(),
    return_layer = FALSE,
    seed = 8525
) {
    ggplot <- if (getOption("plotthis.gglogger.enabled", FALSE)) {
        gglogger::ggplot
    } else {
        ggplot2::ggplot
    }
    stopifnot(
        "[VelocityPlot] 'split_by' is not supported yet" = is.null(split_by)
    )
    stopifnot(
        "[VelocityPlot] 'keep_na' supports only atomic values (logical or character)" = is.atomic(
            keep_na
        ) &&
            (is.logical(keep_na) || is.character(keep_na))
    )
    stopifnot(
        "[VelocityPlot] 'keep_empty' supports only atomic values (logical or 'level'/'levels')" = is.atomic(
            keep_empty
        ) &&
            (is.logical(keep_empty) ||
                (is.character(keep_empty) &&
                    keep_empty %in% c("level", "levels")))
    )
    stopifnot(
        "[VelocityPlot] 'embedding' must be a matrix or data.frame" = is.matrix(
            embedding
        ) ||
            is.data.frame(embedding)
    )
    stopifnot(
        "[VelocityPlot] 'v_embedding' must be a matrix or data.frame" = is.matrix(
            v_embedding
        ) ||
            is.data.frame(v_embedding)
    )
    stopifnot(
        "[VelocityPlot] 'embedding' and 'v_embedding' must have the same dimensions" = all(
            dim(embedding) == dim(v_embedding)
        )
    )
    stopifnot(
        "[VelocityPlot] 'group_by' must be NULL or a vector of the same length as the number of rows in 'embedding'" = is.null(
            group_by
        ) ||
            (length(group_by) == nrow(embedding))
    )

    set.seed(seed)

    plot_type <- match.arg(plot_type)
    reduc_names <- colnames(embedding)
    if (is.null(reduc_names)) {
        xlab <- xlab %||% "Reduction 1"
        ylab <- ylab %||% "Reduction 2"
    } else {
        xlab <- xlab %||% reduc_names[1]
        ylab <- ylab %||% reduc_names[2]
    }

    if (identical(theme, "theme_blank")) {
        theme_args[["xlab"]] <- xlab
        theme_args[["ylab"]] <- ylab
    }
    n_neighbors <- n_neighbors %||% ceiling(nrow(embedding) / 50)

    if (!is.null(group_by)) {
        group_by <- as.factor(group_by)
    }

    if (
        !is.null(group_by) &&
            anyNA(group_by) &&
            !isTRUE(keep_na) &&
            !is.na(keep_na)
    ) {
        if (isFALSE(keep_na)) {
            row_idxes <- is.na(group_by)
            group_by <- group_by[!row_idxes]
            embedding <- embedding[!row_idxes, , drop = FALSE]
            v_embedding <- v_embedding[!row_idxes, , drop = FALSE]
        } else {
            keep_na <- as.character(keep_na)
            levels(group_by) <- c(levels(group_by), keep_na)
            group_by[is.na(group_by)] <- keep_na
        }
    }
    if (!is.null(group_by) && isFALSE(keep_empty)) {
        group_by <- droplevels(group_by)
    }

    if (plot_type == "raw") {
        if (!is.null(density) && (density > 0 && density < 1)) {
            s <- ceiling(density * nrow(embedding))
            ix_choice <- sample(
                seq_len(nrow(embedding)),
                size = s,
                replace = FALSE
            )
            embedding <- embedding[ix_choice, ]
            v_embedding <- v_embedding[ix_choice, ]
        }
        if (!is.null(scale)) {
            v_embedding <- v_embedding * scale
        }
        df <- cbind.data.frame(embedding, v_embedding)
        colnames(df) <- c("x", "y", "u", "v")
        df$length <- sqrt(df[["u"]]^2 + df[["v"]]^2)
        global_size <- sqrt(
            max(df$x, na.rm = TRUE)^2 + max(df$y, na.rm = TRUE)^2
        )
        df$length_perc <- df$length / global_size

        if (!is.null(group_by)) {
            df[[group_name]] <- group_by
            group_vals <- levels(group_by)
            if (anyNA(group_by)) {
                group_vals <- c(group_vals, NA)
            }
            group_cols <- palette_this(
                group_vals,
                palette = group_palette,
                palcolor = group_palcolor,
                NA_keep = TRUE,
                reverse = palreverse
            )
            velocity_layer <- list(
                geom_segment(
                    data = df,
                    mapping = aes(
                        x = !!sym("x"),
                        y = !!sym("y"),
                        xend = !!sym("x") + !!sym("u"),
                        yend = !!sym("y") + !!sym("v"),
                        color = !!sym(group_name)
                    ),
                    alpha = arrow_alpha,
                    arrow = if (
                        utils::compareVersion(
                            as.character(utils::packageVersion("ggplot2")),
                            "4"
                        ) !=
                            0
                    ) {
                        arrow(
                            length = unit(df$length_perc, "npc"),
                            type = "closed",
                            angle = arrow_angle
                        )
                    } else {
                        warning(
                            "[VelocityPlot] 'arrow()' in ggplot2 == 4.0.0 does not support varying lengths. Using fixed length instead. See https://github.com/tidyverse/ggplot2/issues/6594 for details."
                        )
                        NULL
                    },
                    lineend = "round",
                    linejoin = "mitre",
                    inherit.aes = FALSE,
                    show.legend = TRUE
                )
            )
            if (isTRUE(keep_empty)) {
                velocity_layer[[
                    length(velocity_layer) + 1
                ]] <- scale_color_manual(
                    name = group_name,
                    values = group_cols,
                    na.value = group_cols["NA"] %||% "grey80",
                    breaks = group_vals,
                    limits = group_vals,
                    drop = FALSE,
                    guide = guide_legend(
                        title.hjust = 0,
                        order = 1,
                        override.aes = list(linewidth = 2, alpha = 1)
                    )
                )
            } else {
                velocity_layer[[
                    length(velocity_layer) + 1
                ]] <- scale_color_manual(
                    name = group_name,
                    values = group_cols,
                    na.value = group_cols["NA"] %||% "grey80",
                    guide = guide_legend(
                        title.hjust = 0,
                        order = 1,
                        override.aes = list(linewidth = 2, alpha = 1)
                    )
                )
            }
            attr(velocity_layer, "scales") <- unique(c(
                attr(velocity_layer, "scales"),
                "color"
            ))
        } else {
            velocity_layer <- list(
                geom_segment(
                    data = df,
                    aes(
                        x = !!sym("x"),
                        y = !!sym("y"),
                        xend = !!sym("x") + !!sym("u"),
                        yend = !!sym("y") + !!sym("v")
                    ),
                    color = arrow_color,
                    alpha = arrow_alpha,
                    arrow = arrow(
                        length = unit(df$length_perc, "npc"),
                        type = "closed",
                        angle = arrow_angle
                    ),
                    lineend = "round",
                    linejoin = "mitre",
                    inherit.aes = FALSE
                )
            )
        }
    } else if (plot_type == "grid") {
        if (!is.null(group_by)) {
            warning(
                "[VelocityPlot] Ignoring 'group_by', which is not supported for 'grid' plot type."
            )
        }
        res <- .compute_velocity_on_grid(
            embedding,
            v_embedding,
            density = density,
            smooth = smooth,
            n_neighbors = n_neighbors,
            min_mass = min_mass,
            scale = scale
        )
        x_grid <- res$x_grid
        v_grid <- res$v_grid

        df <- cbind.data.frame(x_grid, v_grid)
        colnames(df) <- c("x", "y", "u", "v")
        df$length <- sqrt(df[["u"]]^2 + df[["v"]]^2)
        global_size <- sqrt(
            max(df$x, na.rm = TRUE)^2 + max(df$y, na.rm = TRUE)^2
        )
        df$length_perc <- df$length / global_size
        velocity_layer <- list(
            geom_segment(
                data = df,
                aes(
                    x = !!sym("x"),
                    y = !!sym("y"),
                    xend = !!sym("x") + !!sym("u"),
                    yend = !!sym("y") + !!sym("v")
                ),
                color = arrow_color,
                alpha = arrow_alpha,
                arrow = arrow(
                    length = unit(df$length_perc, "npc"),
                    type = "closed",
                    angle = arrow_angle
                ),
                lineend = "round",
                linejoin = "mitre",
                inherit.aes = FALSE
            )
        )
    } else if (plot_type == "stream") {
        if (!is.null(group_by)) {
            warning(
                "[VelocityPlot] Ignoring 'group_by', which is not supported for 'stream' plot type."
            )
        }
        res <- .compute_velocity_on_grid(
            embedding,
            v_embedding,
            density = density,
            smooth = smooth,
            n_neighbors = n_neighbors,
            min_mass = min_mass,
            scale = 1,
            cutoff_perc = cutoff_perc,
            adjust_for_stream = TRUE
        )
        x_grid <- res$x_grid
        v_grid <- res$v_grid

        df <- expand.grid(x_grid[1, ], x_grid[2, ])
        colnames(df) <- c("x", "y")
        u <- reshape2::melt(t(v_grid[1, , ]))
        v <- reshape2::melt(t(v_grid[2, , ]))
        df[, "u"] <- u$value
        df[, "v"] <- v$value
        df[is.na(df)] <- 0

        if (!is.null(streamline_color)) {
            velocity_layer <- list(
                # Streamline background layer for stroke effect
                metR::geom_streamline(
                    data = df,
                    aes(
                        x = !!sym("x"),
                        y = !!sym("y"),
                        dx = !!sym("u"),
                        dy = !!sym("v")
                    ),
                    L = streamline_l,
                    min.L = streamline_minl,
                    res = streamline_res,
                    n = streamline_n,
                    linewidth = max(streamline_width, na.rm = TRUE) +
                        streamline_bg_stroke,
                    color = streamline_bg_color,
                    alpha = streamline_alpha,
                    arrow.type = "closed",
                    arrow.angle = arrow_angle,
                    lineend = "round",
                    linejoin = "mitre",
                    inherit.aes = FALSE
                ),
                # Streamline foreground layer with fixed color
                metR::geom_streamline(
                    data = df,
                    aes(
                        x = !!sym("x"),
                        y = !!sym("y"),
                        dx = !!sym("u"),
                        dy = !!sym("v")
                    ),
                    L = streamline_l,
                    min.L = streamline_minl,
                    res = streamline_res,
                    n = streamline_n,
                    linewidth = max(streamline_width, na.rm = TRUE),
                    color = streamline_color,
                    alpha = streamline_alpha,
                    arrow.type = "closed",
                    arrow.angle = arrow_angle,
                    lineend = "round",
                    linejoin = "mitre",
                    inherit.aes = FALSE
                ),
                # Streamline arrowhead layer
                metR::geom_streamline(
                    data = df,
                    aes(
                        x = !!sym("x"),
                        y = !!sym("y"),
                        dx = !!sym("u"),
                        dy = !!sym("v")
                    ),
                    L = streamline_l,
                    min.L = streamline_minl,
                    res = streamline_res,
                    n = streamline_n,
                    linetype = 0,
                    color = arrow_color,
                    arrow.type = "closed",
                    arrow.angle = arrow_angle,
                    lineend = "round",
                    linejoin = "mitre",
                    inherit.aes = FALSE
                )
            )
        } else {
            velocity_layer <- list(
                # Streamline background layer for stroke effect
                metR::geom_streamline(
                    data = df,
                    aes(
                        x = !!sym("x"),
                        y = !!sym("y"),
                        dx = !!sym("u"),
                        dy = !!sym("v")
                    ),
                    L = streamline_l,
                    min.L = streamline_minl,
                    res = streamline_res,
                    n = streamline_n,
                    linewidth = max(streamline_width, na.rm = TRUE) +
                        streamline_bg_stroke,
                    color = streamline_bg_color,
                    alpha = streamline_alpha,
                    arrow.type = "closed",
                    arrow.angle = arrow_angle,
                    lineend = "round",
                    linejoin = "mitre",
                    inherit.aes = FALSE
                ),
                # Streamline foreground layer colored by velocity magnitude
                metR::geom_streamline(
                    data = df,
                    aes(
                        x = !!sym("x"),
                        y = !!sym("y"),
                        dx = !!sym("u"),
                        dy = !!sym("v"),
                        linewidth = after_stat(!!sym("step")),
                        color = sqrt(
                            after_stat(!!sym("dx"))^2 +
                                after_stat(!!sym("dy"))^2
                        )
                    ),
                    L = streamline_l,
                    min.L = streamline_minl,
                    res = streamline_res,
                    n = streamline_n,
                    alpha = streamline_alpha,
                    arrow = NULL,
                    lineend = "round",
                    linejoin = "mitre",
                    inherit.aes = FALSE
                ),
                # Streamline arrowhead layer
                metR::geom_streamline(
                    data = df,
                    aes(
                        x = !!sym("x"),
                        y = !!sym("y"),
                        dx = !!sym("u"),
                        dy = !!sym("v")
                    ),
                    L = streamline_l,
                    min.L = streamline_minl,
                    res = streamline_res,
                    n = streamline_n,
                    linetype = 0,
                    color = arrow_color,
                    arrow.type = "closed",
                    arrow.angle = arrow_angle,
                    lineend = "round",
                    linejoin = "mitre",
                    inherit.aes = FALSE
                ),
                # Color scale for velocity magnitude
                scale_color_gradientn(
                    name = "Velocity",
                    n.breaks = 4,
                    colors = palette_this(
                        palette = streamline_palette,
                        palcolor = streamline_palcolor,
                        reverse = palreverse
                    ),
                    guide = guide_colorbar(
                        frame.colour = "black",
                        ticks.colour = "black",
                        title.hjust = 0,
                        order = 1
                    )
                ),
                # Size scale for streamline width
                scale_size(range = range(streamline_width), guide = "none")
            )
            attr(velocity_layer, "scales") <- unique(c(
                attr(velocity_layer, "scales"),
                "color"
            ))
        }
    }

    if (isTRUE(return_layer)) {
        return(velocity_layer)
    }

    p <- ggplot() +
        velocity_layer +
        labs(title = title, subtitle = subtitle, x = xlab, y = ylab) +
        do_call(theme, theme_args) +
        ggplot2::theme(
            aspect.ratio = aspect.ratio,
            legend.position = legend.position,
            legend.direction = legend.direction
        )

    dims <- calculate_plot_dimensions(
        base_height = 6,
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction
    )

    attr(p, "height") <- dims$height
    attr(p, "width") <- dims$width

    return(p)
}

set.seed(8525)
# VelocityPlot takes embedding and v_embedding (both matrix/data.frame)
# group_by is a vector (not a column name)
n_cells <- 100
embedding <- data.frame(
    UMAP1 = rnorm(n_cells),
    UMAP2 = rnorm(n_cells)
)
v_embedding <- data.frame(
    UMAP1 = rnorm(n_cells, sd = 0.1),
    UMAP2 = rnorm(n_cells, sd = 0.1)
)
groups <- factor(sample(c("TypeA", "TypeB", "TypeC"), n_cells, replace = TRUE))

test_that("VelocityPlot with plot_type = 'raw' works", {
    p <- VelocityPlot(embedding, v_embedding, plot_type = "raw")
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "height")))
    expect_true(!is.null(attr(p, "width")))
})

test_that("VelocityPlot with group_by works", {
    p <- VelocityPlot(embedding, v_embedding, plot_type = "raw", group_by = groups)
    expect_s3_class(p, "ggplot")
})

test_that("VelocityPlot with plot_type = 'grid' works", {
    p <- VelocityPlot(embedding, v_embedding, plot_type = "grid")
    expect_s3_class(p, "ggplot")
})

test_that("VelocityPlot with plot_type = 'stream' works", {
    p <- VelocityPlot(embedding, v_embedding, plot_type = "stream")
    expect_s3_class(p, "ggplot")
})

test_that("VelocityPlot with title and subtitle works", {
    p <- VelocityPlot(embedding, v_embedding, plot_type = "raw",
                      title = "Velocity Title", subtitle = "Subtitle")
    expect_s3_class(p, "ggplot")
    expect_equal(p$labels$title, "Velocity Title")
    expect_equal(p$labels$subtitle, "Subtitle")
})

test_that("VelocityPlot with matrix inputs works", {
    emb_mat <- as.matrix(embedding)
    v_mat <- as.matrix(v_embedding)
    p <- VelocityPlot(emb_mat, v_mat, plot_type = "raw")
    expect_s3_class(p, "ggplot")
})

test_that("VelocityPlot errors on mismatched dimensions", {
    expect_error(VelocityPlot(embedding, v_embedding[1:50, ], plot_type = "raw"))
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{norm_expansion}
\alias{norm_expansion}
\title{Expand the plot area with CSS-like padding}
\usage{
norm_expansion(
  expand,
  x_type,
  y_type,
  continuous_default = c(0.05, 0),
  discrete_default = c(0, 0.6)
)
}
\arguments{
\item{expand}{A numeric vector of length 1, 2, 3, or 4
The values to expand the x and y axes. It is like CSS padding.
When a single value is provided, it is used for both axes on both sides.
When two values are provided, the first value is used for the top/bottom side and the second value is used for the left/right side.
When three values are provided, the first value is used for the top side, the second value is used for the left/right side, and the third value is used for the bottom side.
When four values are provided, the values are used for the top, right, bottom, and left sides, respectively.
You can also use a named vector to specify the values for each side.
When the axis is discrete, the values will be applied as 'add' to the 'expansion' function.
When the axis is continuous, the values will be applied as 'mult' to the 'expansion' function.
See also \url{https://ggplot2.tidyverse.org/reference/expansion.html}}

\item{x_type}{The type of x-axis, either "continuous" or "discrete"}

\item{y_type}{The type of y-axis, either "continuous" or "discrete"}
}
\value{
A list with x and y values for expand
}
\description{
Expand the plot area with CSS-like padding
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/venndiagram.R
\name{prepare_venn_data}
\alias{prepare_venn_data}
\title{Prepare input data for Venn diagram rendering}
\usage{
prepare_venn_data(
  data,
  in_form = "auto",
  group_by = NULL,
  group_by_sep = "_",
  id_by = NULL
)
}
\arguments{
\item{data}{A data frame, a named list, or a \code{VennPlotData} object
to be prepared for Venn diagram rendering.  See the \strong{Input formats}
section for the expected structure of each format.}

\item{in_form}{A character string specifying the input format.  One of
\code{"auto"} (default; detect automatically via
\code{\link{detect_venn_datatype}()}), \code{"long"}, \code{"wide"},
\code{"list"}, or \code{"venn"}.}

\item{group_by}{A character string (or vector) specifying the column name(s)
identifying set membership.  For long-format data, a single column defines
the set; multiple columns are concatenated with \code{group_by_sep}.  For
wide-format data, these are the set columns (must be logical or 0/1); when
\code{NULL}, all columns are used as sets.  Ignored for list and
\code{VennPlotData} input.}

\item{group_by_sep}{A character string used to concatenate multiple
\code{group_by} columns when \code{in_form = "long"}.  Default \code{"_"}.}

\item{id_by}{A character string specifying the column name that identifies
individual elements.  Required for long-format data; ignored otherwise.}
}
\value{
A \code{VennPlotData} object suitable for rendering by
\pkg{ggVennDiagram}.
}
\description{
Converts data in any supported input format into a \code{VennPlotData}
object suitable for rendering by \pkg{ggVennDiagram}.  When
\code{in_form = "auto"} (the default), the format is detected via
\code{\link{detect_venn_datatype}()}.
}
\section{Input formats}{

\enumerate{
\item \strong{Long format} — a data frame with a grouping column
(\code{group_by}) identifying the set and an ID column
(\code{id_by}) identifying each element.  Multiple \code{group_by}
columns are concatenated with \code{group_by_sep}.
\preformatted{
group_by id_by
A        a1
A        a2
B        a1
B        a3
}
\item \strong{Wide format} — a data frame where each column represents a
set and each row an element.  Values must be logical or
\code{0}/\code{1}.  Columns specified in \code{group_by} define the
sets; if \code{group_by} is \code{NULL}, all columns are used.
\preformatted{
A    B
TRUE TRUE
TRUE FALSE
FALSE TRUE
}
\item \strong{List format} — a named list where each element is a vector
of identifiers belonging to that set.
\preformatted{
list(A = c("a1", "a2"), B = c("a1", "a3"))
}
\item \strong{VennPlotData} — a pre-computed object returned by a
previous call to \code{prepare_venn_data()}.  Returned as-is.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/radarplot.R
\name{RadarPlotAtomic}
\alias{RadarPlotAtomic}
\title{Atomic radar/spider plot (internal)}
\usage{
RadarPlotAtomic(
  data,
  x,
  x_sep = "_",
  group_by = NULL,
  group_by_sep = "_",
  y = NULL,
  group_name = NULL,
  groups = NULL,
  scale_y = c("group", "global", "x", "none"),
  y_min = 0,
  y_max = NULL,
  y_nbreaks = 4,
  polygon = FALSE,
  fill = TRUE,
  linewidth = 1,
  pt_size = 4,
  max_charwidth = 16,
  bg_color = "grey80",
  bg_alpha = 0.1,
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  alpha = 0.2,
  aspect.ratio = 1,
  legend.position = waiver(),
  legend.direction = "vertical",
  keep_na = FALSE,
  keep_empty = FALSE,
  title = NULL,
  subtitle = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name of the data
frame to plot on the x-axis (the radial categories).  Must be
character or factor.  Multiple columns can be provided; they are
concatenated with \code{x_sep} as the separator.}

\item{x_sep}{A character string used to join multiple \code{x}
columns.  Default \code{"_"}.  Ignored when \code{x} is a single
column.}

\item{group_by}{A character vector of column names to group the
data into separate filled polygons.  Each unique combination
becomes a distinct polygon layer.  Multiple columns are
concatenated with \code{group_by_sep}.  When \code{NULL}, a
single polygon is drawn with no legend.}

\item{group_by_sep}{A character string used to join multiple
\code{group_by} columns.  Default \code{"_"}.}

\item{y}{A character string specifying the numeric column for the
radial axis.  When \code{NULL}, the count of observations in each
(\code{x}, \code{group_by}, \code{facet_by}) combination is used.}

\item{group_name}{A character string used as the colour/fill legend
title.  When \code{NULL}, the \code{group_by} column name is used.}

\item{groups}{A character vector of group values (in the
\code{group_by} column) to include in the plot.  When \code{NULL},
all groups are included.  This can control which groups appear and
their legend order.  Implies \code{keep_empty = FALSE} for the
\code{group_by} column: groups not present in the data are not
shown in the legend.}

\item{scale_y}{How should the radial axis be scaled?  Default is
\code{"group"}.  Options are \code{"group"}, \code{"global"},
\code{"x"}, and \code{"none"}.
\itemize{
\item{\code{"group"} — scaled to the fraction within each group.}
\item{\code{"global"} — scaled to the fraction of the total.}
\item{\code{"x"} — scaled to the fraction within each x-axis category.}
\item{\code{"none"} — raw counts or values, no scaling.}
}}

\item{y_min}{A numeric value setting the minimum of the radial
axis.  Default \code{0}.}

\item{y_max}{A numeric value setting the maximum of the radial
axis.  When \code{NULL}, the maximum data value is used.}

\item{y_nbreaks}{A numeric value for the number of breaks
(concentric grid lines) on the radial axis.  Default \code{4}.}

\item{polygon}{A logical value.  When \code{TRUE}, the background
grid is drawn as a polygon (spider chart style); when
\code{FALSE} (default), concentric circles are used (radar chart
style).}

\item{fill}{A logical value.  When \code{TRUE} (default), the data
polygons are filled with the group colour.  When \code{FALSE},
only outlines are drawn.}

\item{linewidth}{A numeric value for the width of the polygon
outline lines.  Default \code{1}.}

\item{pt_size}{A numeric value for the size of the data point
markers.  Default \code{4}.}

\item{max_charwidth}{A numeric value for the maximum character
width of x-axis labels before wrapping.  Default \code{16}.}

\item{bg_color}{A character string specifying the background fill
colour.  Default \code{"grey80"}.}

\item{bg_alpha}{A numeric value for the transparency of the
background fill.  Default \code{0.1}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a single radar (or spider) chart.  This
is the workhorse behind the exported \code{\link{RadarPlot}} and
\code{\link{SpiderPlot}} functions — it takes a \strong{single} data frame
(no \code{split_by} support) and returns a \code{ggplot} object.

Radar charts display multivariate data on a two-dimensional polar
coordinate system where each variable (x-axis category) is placed
along a radial axis evenly spaced around the circle.  The data values
are connected by lines forming a polygon (or multiple polygons when
\code{group_by} is provided).  When \code{polygon = FALSE} (the
default) the grid is rendered as concentric circles (classic radar
chart); when \code{polygon = TRUE} straight polygonal grid lines are
used (spider chart variant).

The polar coordinate system is implemented via a local
\code{coord_radar()} helper that extends
\code{ggplot2::CoordPolar} with \code{is_linear = TRUE}, allowing
discrete x-axis positions to be mapped to evenly spaced angular
coordinates.
}
\section{Architecture}{

\enumerate{
\item \strong{Column resolution} — \code{x}, \code{y},
\code{group_by}, and \code{facet_by} are validated and
transformed via \code{\link{check_columns}}.  Multi-column
inputs for \code{x} and \code{group_by} are concatenated
using \code{x_sep} and \code{group_by_sep}.
\item \strong{Group setup} — When \code{group_by} is \code{NULL},
a dummy \code{.group} factor is created so polygons still
draw.  The legend is suppressed.  When \code{groups} is
provided, data is filtered to only those group levels.
\item \strong{Count aggregation} — When \code{y = NULL}, the count
of observations in each unique (\code{x}, \code{group_by},
\code{facet_by}) combination is computed.  Factor levels
are preserved after aggregation.
\item \strong{Proportion scaling} — The \code{scale_y} argument
controls y-value normalisation: \code{"group"} scales within
each group, \code{"global"} within each facet (or the whole
dataset), \code{"x"} within each x category, and
\code{"none"} leaves values raw.  Percent labels are used
for scaled modes.
\item \strong{NA / empty-level handling} —
\code{\link{process_keep_na_empty}()} applies \code{keep_na}
and \code{keep_empty} policies.  Per-column \code{keep_empty}
settings for \code{x}, \code{group_by}, and \code{facet_by}
are extracted; facet columns must share the same value.
\item \strong{Coordinate setup} — The local \code{coord_radar()}
creates a \code{CoordPolar} subclass with
\code{is_linear = TRUE}, placing discrete x positions at
evenly spaced angles.
\item \strong{Y-axis range} — \code{y_min}, \code{y_max}, and
\code{y_nbreaks} determine the radial axis limits and
the number of concentric grid lines.
\item \strong{Colour mapping} — \code{\link{palette_this}()}
assigns colours to all \code{group_by} levels, including
\code{NA} (defaulting to \code{"grey80"}).
\item \strong{Background layer} — When \code{polygon = TRUE}, a
polygonal background fills the area between \code{y_min} and
\code{y_max}.  When \code{polygon = FALSE}, a smooth circular
background is interpolated via 360 sample points.
\item \strong{Radial grid lines} — \code{geom_path()} draws
radial lines from \code{y_min} to \code{y_max} at each
x-axis position, respecting facets.
\item \strong{Polygon grid} — When \code{polygon = TRUE},
concentric polygonal grid lines are drawn at each break
level via \code{geom_polygon()}.
\item \strong{Data rendering} — \code{geom_polygon()} draws the
filled (or unfilled) polygons for each group.
\code{geom_point()} adds points at each observation.
Radial axis labels are rendered as text at a fixed
angular offset.
\item \strong{Scale configuration} — \code{scale_y_continuous()}
sets the radial axis limits and breaks.
\code{scale_x_discrete()} positions the category labels.
Colour scales use \code{scale_fill_manual()} and
\code{scale_color_manual()} with \code{drop} controlled
by \code{keep_empty_group}.
\item \strong{Circular grid lines} — When \code{polygon = FALSE},
dashed circular grid lines are styled via
\code{panel.grid.major.y}.
\item \strong{Dimension calculation} —
\code{\link{calculate_plot_dimensions}()} computes plot
height and width from \code{aspect.ratio}, legend metrics,
and a base height.
\item \strong{Faceting} — \code{\link{facet_plot}()} wraps the
plot with \code{facet_wrap} / \code{facet_grid} if
\code{facet_by} is supplied, respecting the
\code{keep_empty} setting for facet variables.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corplot.R
\name{CorPlotAtomic}
\alias{CorPlotAtomic}
\title{Atomic Correlation Plot}
\usage{
CorPlotAtomic(
  data,
  x,
  y,
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  pt_size = 2,
  pt_shape = 16,
  alpha = 1,
  raster = FALSE,
  raster_dpi = c(512, 512),
  highlight = NULL,
  highlight_color = "black",
  highlight_size = 1,
  highlight_alpha = 1,
  highlight_stroke = 0.8,
  anno_items = c("eq", "r2", "p"),
  anno_size = 3,
  anno_fg = "black",
  anno_bg = "white",
  anno_bg_r = 0.1,
  anno_position = c("topleft", "topright", "bottomleft", "bottomright", "tl", "tr", "bl",
    "br"),
  add_smooth = TRUE,
  smooth_color = "red2",
  smooth_width = 1.5,
  smooth_se = FALSE,
  theme = "theme_this",
  theme_args = list(),
  palette = ifelse(is.null(group_by), "Spectral", "Paired"),
  palcolor = NULL,
  palreverse = FALSE,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  legend.position = waiver(),
  legend.direction = "vertical",
  seed = 8525,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name for the x-axis.
Must be numeric.}

\item{y}{A character string specifying the column name for the y-axis.
Must be numeric.}

\item{group_by}{A character vector of column names to colour the points
by.  Each unique combination becomes a separate group in the legend.
Multiple columns are concatenated with \code{group_by_sep}.  When
\code{NULL}, all points are a single colour and the legend is hidden.}

\item{group_by_sep}{A character string to separate concatenated
\code{group_by} columns.  Default \code{"_"}.}

\item{group_name}{A character string used as the colour legend title.
When \code{NULL}, the \code{group_by} column name is used.}

\item{pt_size}{A numeric value specifying the size of the points.
Default: \code{2}.}

\item{pt_shape}{A numeric value specifying the shape of the points
(see \code{\link[ggplot2]{geom_point}}).  Default: \code{16} (filled
circle).}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{raster}{A logical value.  When \code{TRUE}, uses
\code{scattermore::geom_scattermore()} for efficient rendering of
large datasets.  Default: \code{FALSE}.}

\item{raster_dpi}{An integer vector of length 1 or 2 specifying the
raster resolution in (width, height) pixels.  When a single value is
provided, it is recycled.  Default: \code{c(512, 512)}.}

\item{highlight}{Specifies which points to emphasise.  Can be:
\itemize{
\item \code{TRUE} — highlight all points.
\item A character expression (e.g. \code{'Species == "setosa"'}) —
evaluated via \code{dplyr::filter}.
\item A character vector — matched against rownames of the data.
\item A numeric vector — treated as row indices.
}
Default: \code{NULL} (no highlighting).}

\item{highlight_color}{A character string specifying the colour of the
highlighted point borders.  Default: \code{"black"}.}

\item{highlight_size}{A numeric value specifying the size of the
highlighted points (the inner fill).  Default: \code{1}.}

\item{highlight_alpha}{A numeric value specifying the alpha transparency
of the highlighted points.  Default: \code{1}.}

\item{highlight_stroke}{A numeric value specifying the stroke width of
the highlighted point borders.  The outer layer size is
\code{highlight_size + highlight_stroke}.  Default: \code{0.8}.}

\item{anno_items}{A character vector specifying which statistics to
display as text annotation.  Available items: \code{"eq"} (regression
equation), \code{"r2"} (R-squared), \code{"p"} (p-value),
\code{"spearman"}, \code{"pearson"}, \code{"kendall"},
\code{"n"} (observation count).  Default: \code{c("eq", "r2", "p")}.}

\item{anno_size}{A numeric value specifying the font size of the
annotation text (scaled by \code{base_size / 12}).  Default: \code{3}.}

\item{anno_fg}{A character string specifying the colour of the
annotation text.  Default: \code{"black"}.}

\item{anno_bg}{A character string specifying the background colour of
the annotation text boxes.  Default: \code{"white"}.}

\item{anno_bg_r}{A numeric value specifying the corner radius of the
annotation text background boxes.  Default: \code{0.1}.}

\item{anno_position}{A character string specifying the corner position
of the annotation text.  One of \code{"topleft"} (alias \code{"tl"}),
\code{"topright"} (\code{"tr"}), \code{"bottomleft"} (\code{"bl"}),
\code{"bottomright"} (\code{"br"}).}

\item{add_smooth}{A logical value.  When \code{TRUE} (default), a linear
regression line (\code{geom_smooth(method = "lm")}) is added.}

\item{smooth_color}{A character string specifying the colour of the
regression line.  Default: \code{"red2"}.}

\item{smooth_width}{A numeric value specifying the linewidth of the
regression line.  Default: \code{1.5}.}

\item{smooth_se}{A logical value.  When \code{TRUE}, a standard error
band is drawn around the regression line.  Default: \code{FALSE}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{seed}{The random seed to use. Default is 8525.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a scatter plot of two variables with a
linear regression line, optional correlation statistics, and point
highlighting.  This is the workhorse behind the exported
\code{\link{CorPlot}} — it takes a \strong{single} data frame (no
\code{split_by} support) and returns a \code{ggplot} object with
faceting applied.

The function supports \strong{group-based colouring} (\code{group_by}),
point highlighting by expression or rowname, multiple annotation items
(regression equation, R-squared, p-value, Spearman/Pearson/Kendall
correlation, N), raster rendering for large datasets, configurable
regression line style, and faceting.
}
\section{Architecture}{

\enumerate{
\item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{Parameter normalisation} — \code{match.arg()} resolves
\code{anno_position} abbreviations (\code{"tl"}, \code{"tr"},
\code{"bl"}, \code{"br"}) to their full forms.
\code{raster_dpi} is expanded to length 2 when given as a scalar.
\item \strong{Column resolution} — \code{x}, \code{y}, and
\code{group_by} are validated via \code{\link{check_columns}}.
Multi-column \code{group_by} is concatenated with
\code{group_by_sep}.
\item \strong{Grouping fallback} — when \code{group_by = NULL}, a dummy
\code{.group} column is created and the legend is suppressed
(\code{legend.position = "none"}).
\item \strong{Annotation data calculation} — the data is grouped by
\code{facet_by} and a linear model (\code{lm(y ~ x)}) is fitted
per group.  Each requested \code{anno_items} is computed:
\itemize{
\item \code{"eq"} — regression equation \code{y = a + bx}.
\item \code{"r2"} — R-squared of the model.
\item \code{"p"} — p-value of the x coefficient.
\item \code{"spearman"} — Spearman's rho.
\item \code{"pearson"} — Pearson's r.
\item \code{"kendall"} — Kendall's tau.
\item \code{"n"} — number of observations.
}
The results are stored in an \code{annodata} data frame for
\code{geom_text_repel}.
\item \strong{Highlight parsing} — the \code{highlight} argument is
resolved into a \code{.highlight} logical column:
\itemize{
\item \code{TRUE} — highlights all points.
\item A character expression — evaluated via
\code{dplyr::filter} to select rows.
\item A character vector — matched against rownames of the data.
\item A numeric vector — treated as row indices.
}
\item \strong{Point rendering} — two branches:
\itemize{
\item \strong{Raster mode} (\code{raster = TRUE}) — uses
\code{scattermore::geom_scattermore()} for efficient
rendering of large datasets.  Highlighted points are drawn
in two layers (stroke + fill).
\item \strong{Vector mode} (\code{raster = FALSE}, default) —
uses \code{ggplot2::geom_point()} with configurable size,
shape, and alpha.  Highlighted points get an outer stroke
via a second point layer.
}
\item \strong{Regression line} — \code{geom_smooth(method = "lm")}
draws the linear regression line with optional standard error band
(\code{smooth_se}).
\item \strong{Annotation text} — \code{ggrepel::geom_text_repel()}
places the computed annotations at the specified
\code{anno_position} corner, with background styling.
\item \strong{Colour scale} — \code{scale_color_manual()} maps group
levels to colours via \code{\link{palette_this}()}.
\item \strong{Labels and theme} — \code{labs()} sets titles and axis
labels.  The theme is applied via \code{do_call()}, with
\code{aspect.ratio}, \code{legend.position}, and
\code{legend.direction} enforced.
\item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
computes \code{height} and \code{width} attributes from
\code{base_height = 4.5}, \code{aspect.ratio}, legend metrics, and
the number of group levels.
\item \strong{Faceting} — \code{\link{facet_plot}()} applies
\code{facet_wrap} / \code{facet_grid} if \code{facet_by} is
provided.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dimplot.R
\name{DimPlotAtomic}
\alias{DimPlotAtomic}
\title{Atomic Dimension Reduction Plot without splitting the data}
\usage{
DimPlotAtomic(
  data,
  dims = 1:2,
  group_by = NULL,
  group_by_sep = "_",
  features = NULL,
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  pt_size = NULL,
  pt_alpha = 1,
  bg_color = "grey80",
  bg_cutoff = NULL,
  color_name = "",
  label_insitu = FALSE,
  show_stat = !identical(theme, "theme_blank"),
  label = FALSE,
  label_size = 4,
  label_fg = "white",
  label_bg = "black",
  label_bg_r = 0.1,
  label_repel = FALSE,
  label_repulsion = 20,
  label_pt_size = 1,
  label_pt_color = "black",
  label_segment_color = "black",
  order = c("as-is", "reverse", "high-top", "low-top", "random"),
  highlight = NULL,
  highlight_alpha = 1,
  highlight_size = 1,
  highlight_color = "black",
  highlight_stroke = 0.8,
  add_mark = FALSE,
  mark_type = c("hull", "ellipse", "rect", "circle"),
  mark_expand = unit(3, "mm"),
  mark_alpha = 0.1,
  mark_linetype = 1,
  stat_by = NULL,
  stat_plot_type = c("pie", "ring", "bar", "line"),
  stat_plot_size = 0.1,
  stat_palette = "Set1",
  stat_args = list(),
  graph = NULL,
  edge_size = c(0.05, 0.5),
  edge_alpha = 0.1,
  edge_color = "grey40",
  add_density = FALSE,
  density_color = "grey80",
  density_filled = FALSE,
  density_filled_palette = "Greys",
  density_filled_palcolor = NULL,
  lineages = NULL,
  lineages_trim = c(0.01, 0.99),
  lineages_span = 0.75,
  lineages_palette = "Dark2",
  lineages_palcolor = NULL,
  lineages_arrow = ggplot2::arrow(length = unit(0.1, "inches")),
  lineages_linewidth = 1,
  lineages_line_bg = "white",
  lineages_line_bg_stroke = 0.5,
  lineages_whiskers = FALSE,
  lineages_whiskers_linewidth = 0.5,
  lineages_whiskers_alpha = 0.5,
  velocity = NULL,
  velocity_plot_type = c("raw", "grid", "stream"),
  velocity_n_neighbors = NULL,
  velocity_density = 1,
  velocity_smooth = 0.5,
  velocity_scale = 1,
  velocity_min_mass = 1,
  velocity_cutoff_perc = 5,
  velocity_group_palette = "Set2",
  velocity_group_palcolor = NULL,
  arrow_angle = 20,
  arrow_color = "black",
  streamline_l = 5,
  streamline_minl = 1,
  streamline_res = 1,
  streamline_n = 15,
  arrow_alpha = 1,
  streamline_width = c(0, 0.8),
  streamline_alpha = 1,
  streamline_color = NULL,
  streamline_palette = "RdYlBu",
  streamline_palcolor = NULL,
  streamline_bg_color = "white",
  streamline_bg_stroke = 0.5,
  keep_na = FALSE,
  keep_empty = FALSE,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_nrow = NULL,
  facet_ncol = NULL,
  facet_byrow = TRUE,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  theme = "theme_this",
  theme_args = list(),
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  raster = NULL,
  raster_dpi = c(512, 512),
  hex = FALSE,
  hex_linewidth = 0.5,
  hex_count = !is.null(group_by),
  hex_bins = 50,
  hex_binwidth = NULL,
  palette = ifelse(is.null(features), "Paired", "Spectral"),
  palcolor = NULL,
  palreverse = FALSE,
  seed = 8525,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{dims}{A character vector of the column names to plot on the x, y (and optionally z) axes or a numeric vector
of the column indices. When 3 dimensions are provided, a 3D interactive plot is created using plotly.
Supported in 3D: group_by, features, labels, highlight, lineages, graph/network, show_stat, order.
Not supported in 3D: add_mark, stat_by, add_density, velocity, hex, facet_by, raster.}

\item{group_by}{A character string of the column name to group the data by for discrete colouring.
A character/factor column is expected. If multiple columns are provided, the columns will be concatenated with \code{group_by_sep}.}

\item{group_by_sep}{A character string to concatenate the columns in \code{group_by}, if multiple columns are provided.}

\item{features}{A character vector of the column names to plot as features (continuous colouring).
When multiple features are provided and \code{facet_by} is not set, the data is pivoted to long format and
faceted by feature name.}

\item{lower_quantile, upper_quantile}{Lower and upper quantiles for the continuous color/fill scale.
The actual cutoffs are determined by these quantiles when \code{lower_cutoff} and
\code{upper_cutoff} are \code{NULL}. Defaults: \code{lower_quantile = 0}, \code{upper_quantile = 0.99}.}

\item{lower_cutoff, upper_cutoff}{Explicit lower and upper cutoffs for the continuous color/fill scale.
When \code{NULL} (the default), the cutoffs are determined by \code{lower_quantile} and
\code{upper_quantile} via \code{\link[stats]{quantile}}. Values outside the
\code{[lower_cutoff, upper_cutoff]} range are clamped (winsorized) to the nearest cutoff value.}

\item{pt_size}{A numeric value for point size, or a character string naming a numeric column
in \code{data} to map point sizes to. If \code{NULL} (default), auto-calculated as
\code{min(3000 / nrow(data), 0.6)}. Column-name mapping only applies to 2D plots and
is ignored with a message when \code{raster = TRUE}.}

\item{pt_alpha}{A numeric value in \verb{[0, 1]} for the point transparency. Default is \code{1}.}

\item{bg_color}{A character string specifying the colour used for NA-valued points and
background context points drawn from other facets. Default is \code{"grey80"}.}

\item{bg_cutoff}{A numeric threshold. Feature values with absolute value below this
cutoff are set to \code{NA} (and therefore rendered in \code{bg_color}). Default is \code{NULL}.}

\item{color_name}{A character string used as the title for the continuous colour bar
in feature mode. Default is \code{""}.}

\item{label_insitu}{A logical value. If \code{TRUE}, the raw group names are placed at the group median
coordinates instead of numeric indices. Forces \code{label = TRUE}. Default is \code{FALSE}.}

\item{show_stat}{A logical value. If \code{TRUE} (default), the number of points per group is shown
in the legend labels and subtitle. Ignored when \code{theme = "theme_blank"}.}

\item{label}{A logical value. If \code{TRUE}, group labels (numeric indices by default, or group
names when \code{label_insitu = TRUE}) are placed at the median coordinates of each group.
Forced to \code{TRUE} when \code{label_repel} or \code{label_insitu} is set.}

\item{label_size}{A numeric value for the label text size. Passed to \code{ggrepel::geom_text_repel()}.
Default is \code{4}.}

\item{label_fg}{A character string for the label text (foreground) colour. Default is \code{"white"}.}

\item{label_bg}{A character string for the label background / outline colour. Default is \code{"black"}.}

\item{label_bg_r}{A numeric value for the background fill ratio of the label bounding box.
Passed to \code{ggrepel::geom_text_repel(bg.r = ...)}. Default is \code{0.1}.}

\item{label_repel}{A logical value. If \code{TRUE}, labels are repelled from each other with force
\code{label_repulsion}. A visible point anchor is drawn. Forces \code{label = TRUE}.}

\item{label_repulsion}{A numeric value for the repulsion force when \code{label_repel = TRUE}.
Passed to \code{ggrepel::geom_text_repel(force = ...)}. Default is \code{20}.}

\item{label_pt_size}{A numeric value for the size of the anchor point drawn when
\code{label_repel = TRUE}. Default is \code{1}.}

\item{label_pt_color}{A character string for the colour of the label anchor point. Default is \code{"black"}.}

\item{label_segment_color}{A character string for the colour of the line segment connecting
the label to the anchor. Used in non-repel mode (\code{label_repel = FALSE}) where
\code{min.segment.length = 0}. Default is \code{"black"}.}

\item{order}{A character string controlling the draw order of points:
\itemize{
\item \code{"as-is"} (default) — the row order in the data is preserved.
\item \code{"reverse"} — rows are reversed.
\item \code{"high-top"} — points with high values (last factor levels for group_by) are drawn last (on top).
\item \code{"low-top"} — points with low values (first factor levels) are drawn last.
\item \code{"random"} — rows are randomly shuffled.
}

For \code{high-top} and \code{low-top}, NA values are always plotted at the bottom.
When applied to \code{group_by}, only the draw order changes — legend colours and order
are unaffected. Within the same level, point order is preserved. For precise
control, set factor levels before plotting.
See \url{https://github.com/pwwang/scplotter/issues/29#issuecomment-3009694130} for examples.}

\item{highlight}{A specification for highlighted points:
\itemize{
\item \code{NULL} (default): no highlighting.
\item \code{TRUE}: highlight all points (adds a dark outline around every point).
\item A character string: a dplyr filter expression (e.g., \code{"clusters == 'Ductal'"}).
\item A character vector: row names to highlight.
\item A numeric vector: row indices to highlight.
}}

\item{highlight_alpha}{A numeric value in \verb{[0, 1]} for the transparency of highlighted points.
Default is \code{1}.}

\item{highlight_size}{A numeric value for the size of the inner (coloured) highlight point.
Default is \code{1}.}

\item{highlight_color}{A character string for the colour of the outer highlight ring.
Default is \code{"black"}.}

\item{highlight_stroke}{A numeric value for the thickness of the outer highlight ring (the
difference between the outer ring size and \code{highlight_size}). Default is \code{0.8}.}

\item{add_mark}{A logical value. If \code{TRUE}, group boundaries are drawn around points using
\code{ggforce} marks. Requires \code{group_by}. Only supported in 2D.}

\item{mark_type}{A character string specifying the mark shape. Options:
\code{"hull"} (convex hull, default), \code{"ellipse"}, \code{"rect"}, or \code{"circle"}.}

\item{mark_expand}{A unit value for the outward expansion of the mark boundary.
Passed to \code{ggforce::geom_mark_*(expand = ...)}. Default is \code{unit(3, "mm")}.}

\item{mark_alpha}{A numeric value in \verb{[0, 1]} for the transparency of the mark fill.
Default is \code{0.1}.}

\item{mark_linetype}{A numeric value for the line type of the mark boundary. Default is \code{1} (solid).}

\item{stat_by}{A character string naming a column used to compute per-group statistical summary
mini-plots embedded at group centroid positions. Only supported with \code{group_by} (not \code{features}).
Only supported in 2D without \code{facet_by}.}

\item{stat_plot_type}{A character string specifying the mini-plot type. Options:
\code{"pie"} (default), \code{"ring"}, \code{"bar"}, or \code{"line"}.}

\item{stat_plot_size}{A numeric value for the size of the stat mini-plot, expressed as a
fraction of the axis range. Default is \code{0.1}.}

\item{stat_args}{A list of additional arguments passed to the stat plot function
(e.g., \code{list(palette = "Set1")}). Default is \code{list(palette = "Set1")}.}

\item{graph}{A specification for network / graph edges to overlay. Sources:
\itemize{
\item A character string starting with \code{"@"} (e.g., \code{"@graph"}): extracts the attribute
named \code{"graph"} from \code{attributes(data)}.
\item A \code{Graph} object (e.g., Seurat): coerced to dense matrix via \code{as.matrix()}.
\item A \code{matrix}, \code{data.frame}, or \code{dgCMatrix}: used directly as the adjacency matrix.
\item Numeric indices or character column names: extracts columns from \code{data}.
Edges are drawn for non-zero, lower-triangle entries. Requires \code{data} to have row names
matching the matrix dimnames.
}}

\item{edge_size}{A numeric vector of length 2 specifying the range \verb{[min, max]} for
\code{scale_linewidth_continuous(range = ...)} applied to edge widths. Default is \code{c(0.05, 0.5)}.}

\item{edge_alpha}{A numeric value in \verb{[0, 1]} for the transparency of graph edges.
Default is \code{0.1}.}

\item{edge_color}{A character string for the colour of graph edges. Default is \code{"grey40"}.}

\item{add_density}{A logical value. If \code{TRUE}, a 2D density layer is overlaid. Only supported in 2D.}

\item{density_color}{A character string for the colour of the density contour lines.
Used when \code{density_filled = FALSE}. Default is \code{"grey80"}.}

\item{density_filled}{A logical value. If \code{TRUE}, the density is rendered as a filled raster
(\code{stat_density_2d(geom = "raster")}) instead of contour lines. A separate fill scale is used.}

\item{density_filled_palette}{A character string naming the palette for the filled density layer.
Default is \code{"Greys"}.}

\item{density_filled_palcolor}{A character vector of specific colours for the filled density
palette. Default is \code{NULL} (auto-resolved from \code{density_filled_palette}).}

\item{lineages}{A character vector of column names representing pseudotime / trajectory lineages.
Each column is fitted with a LOESS smooth (\verb{span = lineages_span, degree = 2}) across the
2D embedding, after trimming the top and bottom \code{lineages_trim} quantiles.
Only supported in 2D without \code{facet_by}.}

\item{lineages_trim}{A numeric vector of length 2 specifying the lower and upper quantile
thresholds \verb{[0, 1]} for trimming lineage values before LOESS fitting.
Default is \code{c(0.01, 0.99)}.}

\item{lineages_span}{A numeric value passed as \code{span} to \code{stats::loess()} controlling the
smoothness of the lineage curve. Smaller values follow the data more closely.
Default is \code{0.75}.}

\item{lineages_palette}{A character string naming the palette for lineage colours.
Default is \code{"Dark2"}.}

\item{lineages_palcolor}{A character vector of specific colours for lineage curves.
Default is \code{NULL} (auto-resolved from \code{lineages_palette}).}

\item{lineages_arrow}{A ggplot2 \code{arrow} specification applied to the end of lineage paths.
Default is \code{arrow(length = unit(0.1, "inches"))}.}

\item{lineages_linewidth}{A numeric value for the width of the lineage curve lines.
Default is \code{1}.}

\item{lineages_line_bg}{A character string for the colour of the background (wider) stroke
drawn behind each lineage curve for improved visibility. Default is \code{"white"}.}

\item{lineages_line_bg_stroke}{A numeric value for the additional width of the background
stroke relative to \code{lineages_linewidth}. The background line has total width
\code{lineages_linewidth + lineages_line_bg_stroke}. Default is \code{0.5}.}

\item{lineages_whiskers}{A logical value. If \code{TRUE}, short line segments connect the
smoothed lineage curve to the original data coordinates of the fitted points.
Default is \code{FALSE}.}

\item{lineages_whiskers_linewidth}{A numeric value for the width of the whisker lines.
Default is \code{0.5}.}

\item{lineages_whiskers_alpha}{A numeric value in \verb{[0, 1]} for the transparency of the
whisker lines. Default is \code{0.5}.}

\item{velocity}{A specification for RNA-velocity arrows. Can be:
\itemize{
\item \code{NULL} (default): no velocity overlay.
\item A character / integer vector: column names or indices in \code{data} for the velocity embedding.
\item A data frame or matrix: the velocity embedding itself (must align with \code{data} rows).
Only supported in 2D without \code{facet_by}.
}}

\item{velocity_plot_type}{A character string specifying the velocity rendering style.
Options: \code{"raw"} (arrows from embedding), \code{"grid"} (grid-based arrows), or \code{"stream"}
(streamlines). Default is \code{"raw"}.}

\item{velocity_n_neighbors}{A numeric value for the number of neighbours used in the
velocity grid computation. Default is \code{NULL} (auto).}

\item{velocity_density}{A numeric value for the velocity kernel density bandwidth.
Default is \code{1}.}

\item{velocity_smooth}{A numeric value for the velocity smoothing parameter.
Default is \code{0.5}.}

\item{velocity_scale}{A numeric value for scaling the velocity arrows. Default is \code{1}.}

\item{velocity_min_mass}{A numeric value for the minimum cell mass threshold in velocity
grid computation. Default is \code{1}.}

\item{velocity_cutoff_perc}{A numeric value for the velocity cutoff percentage.
Default is \code{5}.}

\item{velocity_group_palette}{A character string naming the palette for velocity group
colours (used in \code{"raw"} plot type). Default is \code{"Set2"}.}

\item{velocity_group_palcolor}{A character vector of specific colours for velocity groups.
Default is \code{NULL} (auto-resolved from \code{velocity_group_palette}).}

\item{arrow_angle}{A numeric value specifying the angle of the arrowheads in degrees. Applied to \code{\link[grid]{arrow}} when \code{plot_type} is \code{"raw"} or \code{"grid"}. Default is 20.}

\item{arrow_color}{A character string specifying the color of the velocity arrows. For \code{plot_type = "stream"}, this sets only the arrowhead color. Default is \code{"black"}.}

\item{streamline_l}{A numeric value specifying the integration length of the streamlines. Passed to \code{\link[metR]{geom_streamline}} as the \code{L} parameter. Default is 5.}

\item{streamline_minl}{A numeric value specifying the minimum streamline length. Shorter streamlines are not drawn. Passed to \code{\link[metR]{geom_streamline}} as the \code{min.L} parameter. Default is 1.}

\item{streamline_res}{A numeric value specifying the resolution of the streamline integration. Passed to \code{\link[metR]{geom_streamline}} as the \code{res} parameter. Default is 1.}

\item{streamline_n}{A numeric value specifying the number of streamlines to draw. Passed to \code{\link[metR]{geom_streamline}} as the \code{n} parameter. Default is 15.}

\item{arrow_alpha}{A numeric value between 0 and 1 specifying the transparency of the velocity arrows. Only used when \code{plot_type = "raw"} or \code{"grid"}; for \code{plot_type = "stream"}, use \code{streamline_alpha} instead. Default is 1.}

\item{streamline_width}{A numeric vector of length 2 specifying the range of line widths for streamlines. Passed to \code{scale_size(range = ...)}. Only used when \code{streamline_color} is \code{NULL}. Default is \code{c(0, 0.8)}.}

\item{streamline_alpha}{A numeric value between 0 and 1 specifying the transparency of the velocity streamlines. Default is 1.}

\item{streamline_color}{An optional character string specifying a fixed color for streamlines. When \code{NULL} (the default), streamlines are colored by velocity magnitude using \code{streamline_palette}.}

\item{streamline_palette}{A character string specifying the color palette for streamline velocity magnitude. Passed to \code{\link{palette_this}}. Only used when \code{streamline_color} is \code{NULL}. Default is \code{"RdYlBu"}.}

\item{streamline_palcolor}{An optional character vector of specific colors for the streamline velocity gradient. If \code{NULL}, colors are generated from \code{streamline_palette}. Default is \code{NULL}.}

\item{streamline_bg_color}{A character string specifying the background (outline) color applied to streamlines to create a stroke effect. Default is \code{"white"}.}

\item{streamline_bg_stroke}{A numeric value specifying the additional line width of the background stroke relative to the foreground streamline. Default is 0.5.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{raster}{A logical value. If \code{TRUE}, points are rendered via
\code{scattermore::geom_scattermore()} for efficient rasterised plotting.
Default is \code{NULL}, which auto-enables when \code{nrow(data) > 1e5}.}

\item{raster_dpi}{A numeric vector of length 2 \verb{[x_dpi, y_dpi]} specifying the raster
resolution in pixels. Passed to \code{scattermore::geom_scattermore(pixels = ...)}.
Default is \code{c(512, 512)}. If a single value is provided it is recycled to both dimensions.}

\item{hex}{A logical value. If \code{TRUE}, points are rendered as hexagonal bins via
\code{geom_hex()} / \code{stat_summary_hex()}. Not supported with highlight.
Default is \code{FALSE}. Only supported in 2D.}

\item{hex_linewidth}{A numeric value for the width of the hexagon boundary lines.
Default is \code{0.5}.}

\item{hex_count}{A logical value. If \code{TRUE} and \code{group_by} is set, hex fill alpha
is mapped to \code{after_stat(count)} so denser bins are more opaque.
For \code{features} mode hex_count is ignored. Default is \code{!is.null(group_by)}.}

\item{hex_bins}{A numeric value for the number of hex bins along each axis.
Passed to \code{geom_hex(bins = ...)}. Default is \code{50}.}

\item{hex_binwidth}{A numeric value for the width of individual hex bins.
Passed to \code{geom_hex(binwidth = ...)}. Takes precedence over \code{hex_bins} when set.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{seed}{The random seed to use. Default is 8525.}

\item{...}{Additional arguments.}
}
\value{
A ggplot object or a plotly object (when 3 dimensions are provided)
}
\description{
Core implementation for dimension reduction visualisation. This is the
internal workhorse dispatched by both \code{DimPlot()} (group-based) and
\code{FeatureDimPlot()} (continuous feature expression). It renders a 2D
scatter plot of ordination axes with extensive annotation capabilities,
and automatically delegates to \code{DimPlotAtomic3D()} for interactive 3D
plots when three \code{dims} are provided.

The function supports two primary colouring modes:
\itemize{
\item \strong{group_by} — discrete factor colouring with a legend of group levels,
optional density/statistical overlays, and group mark / label annotations.
\item \strong{features} — continuous numeric colouring with a gradient colour bar,
multi-feature faceting via \code{tidyr::pivot_longer()}, and optional cutoff
/ quantile trimming via \code{prepare_continuous_color_scale()}.
}

Additional annotation layers include: graph / network edges drawn as
segments between connected nodes, 2D density contours (filled or outline),
group marks (hull, ellipse, rect, circle via \code{ggforce}), lineage curves
(LOESS-smoothed paths with optional whiskers), velocity / RNA-velocity
arrows (raw, grid, or stream via \code{VelocityPlot()}), statistical summary
mini-plots (pie, ring, bar, line) embedded at group centroids, background
points from other facets (faded context), and group labels with repulsion
(via \code{ggrepel::geom_text_repel()}).

Rendering scales automatically: scatter points for small datasets,
\code{scattermore::geom_scattermore()} raster for \code{n > 1e5}, or hex-binned
aggregation (\code{stat_summary_hex()} / \code{geom_hex()}). Legend assembly uses
\code{cowplot::get_plot_component()} with independent guide-boxes for base
groups, lineages, velocity, and stat-by annotations — combined via
\code{rbind} / \code{cbind} and re-inserted with \code{add_grob()}.
}
\section{Architecture}{

\strong{DimPlotAtomic} executes the following steps:
\enumerate{
\item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or \code{ggplot2::ggplot}.
\item \strong{Order validation} — \code{match.arg(order)}.
\item \strong{Dimension resolution} — converts numeric indices to column names,
validates exactly 2 or 3 dims via \code{check_columns()}.
\item \strong{Column resolution} — validates \code{group_by} (force_factor, concat_multi),
\code{features}, and \code{facet_by}. Requires at least one of \code{group_by} or
\code{features}. Blocks \code{facet_by} when multiple features are present (features
themselves become facets).
\item \strong{Auto defaults} — \code{pt_size} defaults to \code{min(3000 / nrow(data), 0.6)}
when \code{NULL}. If a character string is given, it is treated as a column name
for variable point sizing (2D only, ignored in raster mode);
\code{raster} auto-enables when \code{nrow(data) > 1e5}; \code{theme_blank} suppresses
axis labels.
\item \strong{Label force-enable} — if \code{label_repel} or \code{label_insitu} is \code{TRUE},
\code{label} is forced to \code{TRUE} with a message.
\item \strong{NA / empty handling} — \code{process_keep_na_empty()} filters the data;
\code{keep_empty} values extracted for group_by and facet_by.
\item \strong{group_by preprocessing} — group values / colours resolved via
\code{palette_this()}. NA levels mapped to literal \code{"NA"} string.
\code{label_use} is constructed based on \code{label}, \code{label_insitu}, and
\code{show_stat} combinations. Facet labeller is set to \code{as_labeller()} with
per-facet N annotations when \code{show_stat = TRUE}.
\item \strong{Multi-feature pivot} — when \code{length(features) > 1}, data is pivoted
to long format via \code{tidyr::pivot_longer()} with \code{.feature} as the facet
variable and \code{.value} as the numeric column.
\item \strong{Continuous colour scale} — when \code{features} is set,
\code{prepare_continuous_color_scale()} applies quantile/cutoff trimming
and returns anchor values for the gradient.
\item \strong{Point ordering} — applies the selected \code{order} strategy (as-is,
reverse, high-top, low-top, random) by sorting or shuffling rows.
\item \strong{3D branch} — if \code{length(dims) == 3}, emits a warning for unsupported
features, optionally down-samples with stratified group sampling (when
\code{raster = TRUE}), and delegates to \code{DimPlotAtomic3D()}.
\item \strong{Group marks} (2D only, when \code{add_mark = TRUE}) — dispatches to
\code{ggforce::geom_mark_hull/ellipse/rect/circle}, with \code{new_scale_fill()}
/ \code{new_scale_color()} to isolate mark scales.
\item \strong{Graph / network} (2D only) — resolves the graph source (same logic
as 3D), melts the matrix, handles faceted data by per-facet edge
splitting, and adds \code{geom_segment(aes(linewidth = value))} with
\code{scale_linewidth_continuous()}.
\item \strong{Density overlay} (2D only) — filled (\code{stat_density_2d(geom = "raster")})
or outline (\code{geom_density_2d()}).
\item \strong{Base scales and theme} — x / y limits from data range,
\code{do_call(theme, theme_args)}, aspect ratio and legend position.
\item \strong{Background points for facet context} — for faceted plots (excluding
multi-feature auto-faceting), points from other facets are added as
faded background (raster / hex / point depending on settings).
\item \strong{Main point layer} — dispatches by rendering mode:
\itemize{
\item \emph{raster}: \code{scattermore::geom_scattermore()} with separate NA / non-NA
group layers.
\item \emph{hex}: \code{geom_hex()} (group_by with optional count alpha, or
\code{stat_summary_hex()} for features). Raises \code{has_fill = TRUE}.
\item \emph{standard}: \code{geom_point()}.
}
\item \strong{Highlight} (2D only) — resolves highlight specification (TRUE, filter
expression, row names, indices), errors on hex + highlight combo, renders
highlight points with stroke (outer ring + inner colour).
\item \strong{Colour scales} — \code{scale_color_manual()} for group_by (with
\code{keep_empty}-aware \code{breaks}/\code{limits}/\code{drop}), or
\code{scale_color_gradientn()} for features. \code{scale_fill_manual()} /
\code{scale_fill_gradientn()} added when \code{has_fill} is TRUE.
\item \strong{Base legend} — extracted via \code{cowplot::get_plot_component("guide-box-bottom")}.
\item \strong{Lineages} (2D only, no facet_by) — per-lineage LOESS fitting
(\verb{span = lineages_span, degree = 2}) with optional whiskers connecting
smoothed to raw coordinates. Rendered as \code{geom_path()} with background
stroke + foreground colour, plus \code{scale_color_manual()}. Legend extracted
as a separate guide-box.
\item \strong{Velocity} (2D only, no facet_by) — delegates to \code{VelocityPlot()} with
\code{return_layer = TRUE}. Adds \code{new_scale_color()} when the velocity layer
has its own colour scale. Legend extracted separately.
\item \strong{Stat-by mini-plots} (2D only, no facet_by, no features) — dispatches
to \code{PieChart()} / \code{RingPlot()} / \code{BarPlot()} / \code{LinePlot()} via
\code{do_call()}. Each mini-plot is rendered as \code{annotation_custom()} at the
group's median coordinates, scaled by \code{stat_plot_size * range}.
Legend extracted from the stat plot.
\item \strong{Group labels} (2D only, no features) — \code{geom_text_repel()} with
optional repulsion (\code{force = label_repulsion}) or fixed segments
(\code{force = 0}, bold, \code{min.segment.length = 0}). Labels positioned at
group median coordinates (computed per facet if applicable).
\item \strong{Legend assembly} — when additional legends exist (lineages, velocity,
stat_by), they are combined with the base legend via \code{cbind} (vertical
direction) or \code{rbind} (horizontal). The combined legend is re-inserted
via \code{add_grob(gtable, legend, legend.position)}.
\item \strong{Dimension calculation} — \code{calculate_plot_dimensions(base_height = 5.5, aspect.ratio, legend_n, legend_nchar)} sets \code{height} / \code{width} attributes.
\item \strong{Faceting} — \code{facet_plot()} is called only when no additional legends
were assembled (otherwise the combined-legend grob is returned directly).
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/densityplot.R
\name{DensityHistoPlotAtomic}
\alias{DensityHistoPlotAtomic}
\title{Atomic density/histogram plot}
\usage{
DensityHistoPlotAtomic(
  data,
  x,
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  xtrans = "identity",
  ytrans = "identity",
  type = c("density", "histogram"),
  bins = NULL,
  binwidth = NULL,
  flip = FALSE,
  keep_na = FALSE,
  keep_empty = FALSE,
  add_bars = FALSE,
  bar_height = 0.025,
  bar_alpha = 1,
  bar_width = 0.1,
  position = "identity",
  use_trend = FALSE,
  add_trend = FALSE,
  trend_alpha = 1,
  trend_linewidth = 0.8,
  trend_pt_size = 1.5,
  trend_skip_zero = FALSE,
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 0.5,
  theme = "theme_this",
  theme_args = list(),
  aspect.ratio = 1,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  expand = c(bottom = 0, left = 0, right = 0),
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  legend.position = ifelse(is.null(group_by), "none", "right"),
  legend.direction = "vertical",
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name for the x-axis values.
A numeric column is expected.}

\item{group_by}{A character string specifying the column(s) to group the data
by. Multiple columns are concatenated with \code{group_by_sep}. Each group
receives a distinct fill and outline colour.}

\item{group_by_sep}{A character string used to join multiple \code{group_by}
column values into a single factor level. Default: \code{"_"}.}

\item{group_name}{A character string used as the legend title for the
\code{group_by} aesthetic. When \code{NULL} (default), the (possibly concatenated)
\code{group_by} column name is used.}

\item{xtrans}{A character string specifying the transformation applied to
the x-axis. Passed to \code{ggplot2::scale_x_continuous(transform = ...)}.
Supported values include \code{"identity"} (default), \code{"log10"}, \code{"log2"},
\code{"sqrt"}, \code{"reverse"}, etc.}

\item{ytrans}{A character string specifying the transformation applied to
the y-axis. Passed to \code{ggplot2::scale_y_continuous(transform = ...)}.
Used by \code{trend_skip_zero} to correctly interpolate across zero bins on
a transformed scale. Default: \code{"identity"}.}

\item{type}{A character string specifying the plot type. \code{"density"} (default)
renders \code{geom_density()}; \code{"histogram"} renders \code{geom_histogram()} with
optional trend overlays.}

\item{bins}{A numeric value specifying the number of bins for the histogram.
Ignored when \code{type = "density"}. Defaults to \code{30} when neither \code{bins} nor
\code{binwidth} is provided.}

\item{binwidth}{A numeric value specifying the width of individual bins for
the histogram. Ignored when \code{type = "density"}. Takes precedence over
\code{bins} when both are set.}

\item{flip}{A logical value. If \code{TRUE}, the x and y axes are swapped via
\code{coord_flip()}. Dimension calculation accounts for the flip.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{add_bars}{A logical value. If \code{TRUE}, a data-distribution rug is
drawn along the y = 0 axis using \code{geom_linerange()}. Each group's bars
are vertically offset to avoid overlap.}

\item{bar_height}{A numeric value specifying the height (in data units,
relative to the maximum y) of the rug bars added by \code{add_bars}.
The actual pixel height scales with \code{max_y}. Default: \code{0.025}.}

\item{bar_alpha}{A numeric value in \verb{[0, 1]} for the transparency of the
rug bars. Default: \code{1}.}

\item{bar_width}{A numeric value passed as the \code{linewidth} aesthetic of
\code{geom_linerange()}. Controls the thickness of each rug tick.
Default: \code{0.1}.}

\item{position}{A character string specifying the position adjustment for
the bars or density curves. Default: \code{"identity"}, which shows the
actual count / density per group (unlike \code{ggplot2}'s default \code{"stack"}).
Other options: \code{"stack"}, \code{"dodge"}, \code{"fill"}.}

\item{use_trend}{A logical value. If \code{TRUE}, the histogram bars are replaced
entirely by a trend line (points + connecting line). Only applies when
\code{type = "histogram"}.}

\item{add_trend}{A logical value. If \code{TRUE}, a trend line is overlaid on
top of the histogram bars. Only applies when \code{type = "histogram"}.}

\item{trend_alpha}{A numeric value in \verb{[0, 1]} controlling the transparency
of the trend points and line. Default: \code{1}.}

\item{trend_linewidth}{A numeric value for the thickness of the trend line.
Default: \code{0.8}.}

\item{trend_pt_size}{A numeric value for the size of the trend points.
Default: \code{1.5}.}

\item{trend_skip_zero}{A logical value. If \code{TRUE}, bins with zero count are
set to \code{NA} before the trend line is computed, and \code{zoo::na.approx()} is
used to interpolate across the gaps — producing a continuous curve even
when some bins are empty. Requires \code{ytrans} to be correctly specified.
Only applies when \code{type = "histogram"} and \code{use_trend} or \code{add_trend} is
active.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{expand}{The values to expand the x and y axes. It is like CSS padding.
When a single value is provided, it is used for both axes on both sides.
When two values are provided, the first value is used for the top/bottom side and the second value is used for the left/right side.
When three values are provided, the first value is used for the top side, the second value is used for the left/right side, and the third value is used for the bottom side.
When four values are provided, the values are used for the top, right, bottom, and left sides, respectively.
You can also use a named vector to specify the values for each side.
When the axis is discrete, the values will be applied as 'add' to the 'expansion' function.
When the axis is continuous, the values will be applied as 'mult' to the 'expansion' function.
See also \url{https://ggplot2.tidyverse.org/reference/expansion.html}}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{...}{Additional arguments.}
}
\description{
Core implementation for density and histogram plots. This is the internal workhorse
dispatched by both the \code{DensityPlot()} and \code{Histogram()} public wrappers. It
renders a grouped density curve or histogram, with optional data-distribution
bars along the y=0 axis, trend-line interpolation (histogram only), and full
faceting support.

The function supports two plotting modes selected by \code{type}:
\itemize{
\item \strong{density} — renders \code{ggplot2::geom_density()} for each group.
\item \strong{histogram} — renders \code{ggplot2::geom_histogram()} with optional trend
overlays (\code{use_trend}, \code{add_trend}), including zero-skip interpolation
(\code{trend_skip_zero}) that uses \code{zoo::na.approx()} to bridge gaps where a
bin has zero observations under a transformed y-axis.
}

When \code{add_bars = TRUE}, a data rug is drawn along the bottom of the plot using
\code{ggplot2::geom_linerange()}. Each group's bars are offset vertically so they
stack without overlapping.
}
\section{Architecture}{

\strong{DensityHistoPlotAtomic} executes the following steps:
\enumerate{
\item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or \code{ggplot2::ggplot}
based on \code{getOption("plotthis.gglogger.enabled")}.
\item \strong{Type check} — \code{match.arg(type, c("density", "histogram"))}.
\item \strong{Expansion normalization} — \code{norm_expansion()} converts the \code{expand}
vector into CSS-padding-style x/y components.
\item \strong{Column resolution} — \code{check_columns()} validates \code{x}, \code{group_by}
(force_factor, allow_multi, concat_multi), and \code{facet_by}.
\item \strong{Default group} — when \code{group_by} is \code{NULL}, a synthetic \code{.group}
factor with a single empty-string level is created so the colour-mapping
pipeline runs uniformly.
\item \strong{Histogram bin default} — if \code{type = "histogram"} and neither \code{bins}
nor \code{binwidth} is set, \code{bins = 30} with a message.
\item \strong{Add-bars pre-calculation} — when \code{add_bars = TRUE}:
\itemize{
\item For density: max y = \code{max(density(x)$y) * 1.5}.
\item For histogram: max y = max bin count from \code{cut(x, s)}.
\item Computes \code{.ymin} and \code{.ymax} per row, offset by \code{bar_height * max_y}
for each group so rugs stack without colliding.
}
\item \strong{NA / empty handling} — \code{process_keep_na_empty()} filters data and
\code{keep_empty} values are extracted for group and facet dimensions.
\item \strong{Palette resolution} — \code{palette_this()} maps group levels to colours.
\item \strong{Base ggplot + scales} — initialises \code{ggplot(data, aes(x, fill, color))},
then adds \code{scale_fill_manual()} / \code{scale_color_manual()}. When
\code{keep_empty_group} is \code{TRUE}, \code{drop = FALSE}, \code{breaks}, and \code{limits}
are set to preserve empty factor levels.
\item \strong{Geometry layer}:
\itemize{
\item \emph{Histogram (no trend)}: \code{geom_histogram(alpha, bins, binwidth, position)}.
\item \emph{Histogram (use_trend / add_trend)}: adds \code{stat_bin(geom = "point")}
for trend points + \code{stat_bin(geom = "line")} for the trend curve.
When \code{trend_skip_zero = TRUE}, an \code{after_stat()} expression sets zero
counts to \code{NA}, transforms y, applies \code{zoo::na.approx()} per
\code{..group..}, and inverts — producing a continuous trend that
interpolates over empty bins.
\item \emph{Density}: \code{geom_density(alpha, position)}.
}
\item \strong{Add-bars geometry} — if \code{add_bars = TRUE}, \code{geom_linerange()} draws
vertical ticks at the pre-computed \code{.ymin} / \code{.ymax} positions.
\item \strong{Scales, theme, labels} — x/y continuous scales with transforms,
theme applied via \code{do_call(theme, theme_args)}, and axis / title
labels (default y-lab: "Count" for histogram, "Density" for density).
\item \strong{Flip} — optional \code{coord_flip()}.
\item \strong{Dimension calculation} — \code{calculate_plot_dimensions(base_height = 3.5, aspect.ratio, legend, flip)} sets \code{height} / \code{width} attributes.
\item \strong{Faceting} — \code{facet_plot()} applies \code{facet_grid} / \code{facet_wrap}.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ringplot.R
\name{RingPlotAtomic}
\alias{RingPlotAtomic}
\title{Atomic ring plot (internal)}
\usage{
RingPlotAtomic(
  data,
  x = NULL,
  y = NULL,
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  label = NULL,
  clockwise = TRUE,
  keep_na = FALSE,
  keep_empty = FALSE,
  facet_by = NULL,
  facet_scales = "free_y",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column to define the rings
(concentric layers) of the plot.  Each unique value becomes one ring.
When \code{NULL}, a single-ring (pie-chart-like) plot is produced.
Character or factor columns are expected.  Multiple columns are not
supported for \code{x} in ring plots.}

\item{y}{A character string specifying the numeric column for segment
sizing.  When \code{NULL} (default), the count of observations in each
(\code{x}, \code{group_by}, \code{facet_by}) combination is used.
Values are always normalised to proportions within each ring.}

\item{group_by}{A character vector of column names to divide each ring
into filled segments.  Each unique combination becomes one segment per
ring.  Multiple columns are concatenated with \code{group_by_sep}.
When \code{NULL}, a single unsegmented ring is drawn and the legend
is hidden.}

\item{group_by_sep}{A character string to join multiple \code{group_by}
columns.  Default \code{"_"}.  Ignored when \code{group_by} is a single
column.}

\item{group_name}{A character string used as the fill legend title.
When \code{NULL}, the \code{group_by} column name is used.}

\item{label}{A logical value controlling whether ring labels are shown.
Labels display the \code{x} values (ring names) at the inner edge of
each ring.  Default \code{NULL} auto-selects: \code{FALSE} for
single-ring plots, \code{TRUE} for multi-ring plots.}

\item{clockwise}{A logical value.  When \code{TRUE} (default), segments
within each ring are drawn clockwise (top-to-right).  When
\code{FALSE}, they are drawn counter-clockwise.  The legend also
reverses to match the drawing order.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{seed}{The random seed to use. Default is 8525.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a single ring (multi-layer donut) plot.
This is the workhorse behind the exported \code{\link{RingPlot}} function —
it takes a \strong{single} data frame (no \code{split_by} support) and returns a
\code{ggplot} object.  Each level of \code{x} becomes a concentric ring,
and within each ring the \code{group_by} variable divides the ring into
filled segments drawn via \code{geom_col()} with \code{coord_polar("y")}.

When \code{y = NULL}, the count of observations per combination is used.
y-values are always normalised to proportions (0–1) within each
(\code{x}, \code{facet_by}) group so that each complete ring sums to 1.
}
\section{Architecture}{

\enumerate{
\item \strong{Column resolution} — \code{x}, \code{group_by}, \code{y},
and \code{facet_by} are validated and transformed via
\code{\link{check_columns}}.  If \code{x} is \code{NULL}, a dummy
\code{.x} column (value \code{1}) is created, producing a single-ring
(pie-chart-like) plot.  If \code{group_by} is \code{NULL}, a dummy
\code{.group_by} factor is created and the legend is suppressed.
Multi-column \code{group_by} inputs are concatenated with
\code{group_by_sep}.
\item \strong{NA / empty-level handling} — \code{\link{process_keep_na_empty}()}
applies \code{keep_na} and \code{keep_empty} policies.  Per-column
\code{keep_empty} settings are extracted for \code{x},
\code{group_by}, and \code{facet_by} independently.  The facet
columns must have identical \code{keep_empty} values.
\item \strong{Count aggregation} — when \code{y = NULL}, the count of
observations in each unique (\code{x}, \code{group_by},
\code{facet_by}) combination is computed as a new \code{.y} column.
Factor levels are preserved after aggregation.
\item \strong{Proportion scaling} — y-values are divided by the sum within
each (\code{x}, \code{facet_by}) group, normalising each ring to
sum to 1.0.  This is essential for polar coordinates where each ring
represents a full 360 degrees.
\item \strong{Ring resolution} — The levels of \code{x} are reversed so
the outermost ring appears first in the data (rings are drawn
outward from the origin).  \code{NA} levels are appended.
\item \strong{Label logic} — When \code{label = NULL}, labels are
automatically suppressed for single-ring plots and shown for
multi-ring plots.  Labels are placed at the inner edge of each ring
via \code{geom_label()}.
\item \strong{Clockwise ordering} — When \code{clockwise = TRUE} (default),
the \code{group_by} levels are reversed so segments appear clockwise
from the top.  The legend guide also reverses its order.
\item \strong{Colour mapping} — \code{\link{palette_this}()} assigns
colours to all \code{group_by} levels, including \code{NA}
(defaulting to \code{"grey80"}).
\item \strong{Plot assembly} — The \code{ggplot} object is built with
\code{geom_col()} (ring segments), \code{coord_polar("y")} (polar
transform), \code{scale_x_discrete()} (with a leading space level
as the donut hole), and \code{scale_fill_manual()} with per-group
colours.  All axes, ticks, grid lines, and panel borders are removed
for a clean appearance.  The fill scale \code{drop} argument is
controlled by \code{keep_empty_group}.
\item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
computes plot height and width from \code{aspect.ratio} and legend
metrics.  The resulting \code{height} / \code{width} attributes are
stored on the \code{ggplot} object.
\item \strong{Faceting} — \code{\link{facet_plot}()} wraps the plot with
\code{facet_wrap} / \code{facet_grid} if \code{facet_by} is
provided, respecting the \code{keep_empty} setting for facet
variables.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{gsea_example}
\alias{gsea_example}
\title{An example of GSEA result from fgsea package}
\description{
An example of GSEA result from fgsea package
}
\examples{
\dontrun{
if (interactive()) {
 set.seed(1234)
 data(geneList, package="DOSE")
 gsea_example <- DOSE::gseDO(geneList)
 gene_ranks <- gsea_example@geneList
 gene_sets <- gsea_example@geneSets
 gsea_example_pos <- gsea_example[gsea_example$p.adjust < 0.05 & gsea_example$NES > 0, ]
 gsea_example_neg <- gsea_example[gsea_example$p.adjust < 0.05 & gsea_example$NES < 0, ]
 gsea_example <- rbind(
     gsea_example_pos[sample(1:nrow(gsea_example_pos), 5), ],
     gsea_example_pos[sample(1:nrow(gsea_example_pos), 5), ]
 )

 attr(gsea_example, "gene_ranks") <- gene_ranks
 attr(gsea_example, "gene_sets") <- gene_sets[gsea_example$ID]
}
}
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/barplot.R
\name{BarPlotAtomic}
\alias{BarPlotAtomic}
\title{Atomic bar plot (internal)}
\usage{
BarPlotAtomic(
  data,
  x,
  x_sep = "_",
  y = NULL,
  scale_y = FALSE,
  flip = FALSE,
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  fill_by = TRUE,
  fill_name = NULL,
  label_nudge = 0.02,
  label = NULL,
  label_fg = "black",
  label_size = 4,
  label_bg = "white",
  label_bg_r = 0.1,
  add_bg = FALSE,
  bg_palette = "stripe",
  bg_palcolor = NULL,
  bg_alpha = 0.2,
  theme = "theme_this",
  theme_args = list(),
  palette = NULL,
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  x_text_angle = 0,
  aspect.ratio = 1,
  add_line = NULL,
  line_color = "red2",
  line_width = 0.6,
  line_type = 2,
  line_name = NULL,
  add_trend = FALSE,
  trend_color = "black",
  trend_linewidth = 1,
  trend_ptsize = 2,
  position = "auto",
  position_dodge_preserve = "total",
  y_min = NULL,
  y_max = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  keep_na = FALSE,
  keep_empty = FALSE,
  expand = waiver(),
  width = waiver(),
  facet_by = NULL,
  facet_scales = "fixed",
  facet_nrow = NULL,
  facet_ncol = NULL,
  facet_byrow = TRUE,
  facet_args = list(),
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis.}

\item{x_sep}{A character string to join multiple \code{x} columns.
Default \code{"_"}.}

\item{y}{A character string specifying the column name of the data frame to plot for the y-axis.}

\item{scale_y}{A logical value.  When \code{TRUE}, y-values are scaled to
proportions within each x position so that each position's total is 100\\%.
Only applicable when \code{position = "stack"}.}

\item{flip}{Logical; if \code{TRUE}, swap the x and y axes.}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{group_name}{A character string for the group fill legend title.
When \code{NULL}, the \code{group_by} column name is used.}

\item{fill_by}{A variable used to fill the bars.  Both categorical and
numeric columns are accepted:
\itemize{
\item \code{TRUE} (default) — fill by the x-axis values.
\item \code{FALSE} — solid fill (first palette colour).
\item A column name (character/factor) — discrete colour scale.
\item A column name (numeric) — continuous gradient with quantile
/ cutoff controls.
}
Ignored when \code{group_by} is provided (fill is determined by
\code{group_by}).}

\item{fill_name}{A character string for the fill legend title.  Only
applies when \code{group_by = NULL} and the fill is from \code{fill_by}.}

\item{label_nudge}{A numeric value controlling the distance between labels
and the bar top, expressed as a fraction of the data range.}

\item{label}{A column name (or \code{TRUE}) for text labels on bars.
When \code{TRUE}, the y-axis values are labelled.  When a column name,
the values in that column are used.}

\item{label_fg}{A character string specifying the label text colour.}

\item{label_size}{A numeric value specifying the label text size.}

\item{label_bg}{A character string specifying the label background colour.}

\item{label_bg_r}{A numeric value specifying the label background corner
radius.}

\item{add_bg}{Logical; add alternating background stripes behind the bars.}

\item{bg_palette}{Palette for the background stripes.}

\item{bg_palcolor}{Custom colours for the background stripes.}

\item{bg_alpha}{Alpha transparency for the background stripes.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{lower_quantile, upper_quantile}{Lower and upper quantiles for the continuous color/fill scale.
The actual cutoffs are determined by these quantiles when \code{lower_cutoff} and
\code{upper_cutoff} are \code{NULL}. Defaults: \code{lower_quantile = 0}, \code{upper_quantile = 0.99}.}

\item{lower_cutoff, upper_cutoff}{Explicit lower and upper cutoffs for the continuous color/fill scale.
When \code{NULL} (the default), the cutoffs are determined by \code{lower_quantile} and
\code{upper_quantile} via \code{\link[stats]{quantile}}. Values outside the
\code{[lower_cutoff, upper_cutoff]} range are clamped (winsorized) to the nearest cutoff value.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{add_line}{A numeric y-intercept for a horizontal reference line.}

\item{line_color}{Colour of the reference line.}

\item{line_width}{Width of the reference line.}

\item{line_type}{Linetype of the reference line (e.g., 1 = solid, 2 = dashed).}

\item{line_name}{Legend name for the reference line.}

\item{add_trend}{Logical; add a trend line and points connecting the bar tops.}

\item{trend_color}{Colour of the trend line.}

\item{trend_linewidth}{Width of the trend line.}

\item{trend_ptsize}{Size of the trend line points.}

\item{position}{A character string specifying the bar layout:
\code{"auto"} (default: dodge when \eqn{\le}5 groups, stack otherwise),
\code{"dodge"} (side-by-side), or \code{"stack"} (stacked on top of
each other).}

\item{position_dodge_preserve}{A character string passed to
\code{\link[ggplot2]{position_dodge2}()}: \code{"total"} preserves the
overall bar group width; \code{"single"} preserves individual bar widths.}

\item{y_min, y_max}{Numeric limits for the y-axis (or x-axis when flipped).}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{expand}{The values to expand the x and y axes. It is like CSS padding.
When a single value is provided, it is used for both axes on both sides.
When two values are provided, the first value is used for the top/bottom side and the second value is used for the left/right side.
When three values are provided, the first value is used for the top side, the second value is used for the left/right side, and the third value is used for the bottom side.
When four values are provided, the values are used for the top, right, bottom, and left sides, respectively.
You can also use a named vector to specify the values for each side.
When the axis is discrete, the values will be applied as 'add' to the 'expansion' function.
When the axis is continuous, the values will be applied as 'mult' to the 'expansion' function.
See also \url{https://ggplot2.tidyverse.org/reference/expansion.html}}

\item{width}{A numeric value specifying the bar width (0–1).}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{facet_args}{A list of additional arguments passed to the faceting
function (e.g., \code{scales}, \code{labeller}).}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object, possibly faceted, with \code{height}
and \code{width} attributes (in inches) attached.
}
\description{
Dispatcher that routes to \code{\link{BarPlotSingle}} (when
\code{group_by = NULL}) or \code{\link{BarPlotGrouped}} (when
\code{group_by} is provided).  This is the core implementation layer —
it takes a \strong{single} data frame (no \code{split_by} support) and returns
a \code{ggplot} object with faceting applied.
}
\section{Dispatch logic}{

\itemize{
\item \strong{Without \code{group_by}} — delegates to
\code{BarPlotSingle}.  \code{fill_by} controls bar colouring:
\code{TRUE} (default) fills by x-axis values, \code{FALSE} uses a
single colour, a character column produces discrete colours, and
a numeric column produces a continuous gradient.
\item \strong{With \code{group_by}} — delegates to
\code{BarPlotGrouped}.  \code{fill_by} must match \code{group_by}
or be left as default; an explicit mismatch raises a stop error.
\code{position} controls dodge vs. stack layout with automatic
selection based on group count (\eqn{\le}5 → dodge, >5 → stack).
}

After the delegate returns, \code{\link{facet_plot}()} wraps the result
with \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is provided.
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lineplot.R
\name{LinePlotGrouped}
\alias{LinePlotGrouped}
\title{Multi-series line plot (internal)}
\usage{
LinePlotGrouped(
  data,
  x,
  y = NULL,
  group_by,
  group_by_sep = "_",
  facet_by = NULL,
  add_bg = FALSE,
  bg_palette = "stripe",
  bg_palcolor = NULL,
  bg_alpha = 0.2,
  add_errorbars = FALSE,
  errorbar_width = 0.1,
  errorbar_alpha = 1,
  errorbar_color = "grey30",
  errorbar_linewidth = 0.75,
  errorbar_min = NULL,
  errorbar_max = NULL,
  errorbar_sd = NULL,
  highlight = NULL,
  highlight_size = pt_size - 0.75,
  highlight_color = "red2",
  highlight_alpha = 0.8,
  pt_alpha = 1,
  pt_size = 5,
  add_hline = FALSE,
  hline_type = "solid",
  hline_width = 0.5,
  hline_color = "black",
  hline_alpha = 1,
  line_type = "solid",
  line_width = 1,
  line_alpha = 0.8,
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  x_text_angle = 0,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  keep_empty = FALSE,
  keep_na = FALSE,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis.}

\item{y}{A character string specifying the column name of the data frame to plot for the y-axis.}

\item{group_by}{A character vector of column names to group the data by.
Each unique combination becomes a separate line. Multiple columns are
concatenated with \code{group_by_sep}. When \code{NULL}, the
\code{LinePlotSingle} path is used instead.}

\item{group_by_sep}{A character string used to join multiple
\code{group_by} column values into a single group identifier. Default
\code{"_"}.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{add_bg}{A logical value. When TRUE, alternating background stripes
are drawn via \code{\link{bg_layer}()}. Default FALSE.}

\item{bg_palette}{A character string specifying the palette for the
background stripe colours. Default \code{"stripe"}.}

\item{bg_palcolor}{A character vector of colours for the background
stripes. When NULL (default), colours are derived from
\code{bg_palette}.}

\item{bg_alpha}{A numeric value in \code{[0, 1]} for the transparency
of background stripes. Default 0.2.}

\item{add_errorbars}{A logical value. When TRUE, error bars are added via
\code{\link[ggplot2]{geom_errorbar}()}. Requires \code{errorbar_sd} or
\code{errorbar_min}/\code{errorbar_max}. Default FALSE.}

\item{errorbar_width}{A numeric value for the width of the error bar
caps. Default 0.1.}

\item{errorbar_alpha}{A numeric value in \code{[0, 1]} for the transparency of
error bars. Default 1.}

\item{errorbar_color}{A character string for the colour of the error
bars. When \code{"line"}, error bars are coloured the same as the lines
(by x when \code{color_line_by_x = TRUE}, or single colour otherwise).
Default \code{"grey30"}.}

\item{errorbar_linewidth}{A numeric value for the line width of error
bars. Default 0.75.}

\item{errorbar_min}{A character string naming the column with the lower
error bar bound. Ignored when \code{errorbar_sd} is provided.}

\item{errorbar_max}{A character string naming the column with the upper
error bar bound. Ignored when \code{errorbar_sd} is provided.}

\item{errorbar_sd}{A character string naming the column with the standard
deviation. When \code{errorbar_min} and \code{errorbar_max} are not
provided, error bars are computed as y +/- \code{errorbar_sd}.}

\item{highlight}{A vector of row indices, row names, a single string
expression (e.g. \code{"y > 10"}) filtering rows to highlight, or TRUE
to highlight all points. When NULL (default), no highlighting is
applied.}

\item{highlight_size}{A numeric value for the size of highlighted points.
Defaults to \code{pt_size - 0.75}.}

\item{highlight_color}{A character string for the colour of highlighted
points. Default \code{"red2"}.}

\item{highlight_alpha}{A numeric value in \code{[0, 1]} for the transparency of
highlighted points. Default 0.8.}

\item{pt_alpha}{A numeric value in \code{[0, 1]} for the transparency of points.
Default 1.}

\item{pt_size}{A numeric value for the point size. Default 5.}

\item{add_hline}{A numeric value specifying the y-intercept of a
horizontal reference line. When FALSE (default), no line is drawn.}

\item{hline_type}{A character string specifying the line type of the
horizontal reference line. Default \code{"solid"}.}

\item{hline_width}{A numeric value for the width of the horizontal
reference line. Default 0.5.}

\item{hline_color}{A character string for the colour of the horizontal
reference line. Default \code{"black"}.}

\item{hline_alpha}{A numeric value in \code{[0, 1]} for the transparency of the
horizontal reference line. Default 1.}

\item{line_type}{A character string specifying the line type. Default
\code{"solid"}.}

\item{line_width}{A numeric value for the line width (in mm). Default 1.}

\item{line_alpha}{A numeric value in \code{[0, 1]} for the transparency of the
line. Default 0.8.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches).
}
\description{
Core implementation for drawing a multi-series line plot.  This is the
workhorse behind \code{\link{LinePlotAtomic}} for grouped data -- it takes a
\strong{single} data frame with a \code{group_by} column and returns a
\code{ggplot} object.  Each group receives its own line rendered in a
distinct colour from the palette, with optional error bars, highlighted
points, per-group horizontal reference lines, and background stripes.
}
\section{Architecture}{

\enumerate{
\item \strong{ggplot dispatch} -- selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{Column resolution} -- \code{x} (forced to factor),
\code{y}, \code{group_by} (multi-column concatenation supported),
and \code{facet_by} are validated via
\code{\link{check_columns}}.
\item \strong{Count aggregation} -- when \code{y = NULL}, the count of
observations per (\code{x}, \code{group_by}, \code{facet_by})
combination is computed and used as \code{y}.
\item \strong{Error bar validation} -- if \code{add_errorbars = TRUE},
checks that either \code{errorbar_min}/\code{errorbar_max} or
\code{errorbar_sd} is provided.
\item \strong{NA / empty-level handling} --
\code{\link{process_keep_na_empty}()} applies \code{keep_na} and
\code{keep_empty} policies.
\item \strong{Highlight parsing} -- \code{highlight} (indices, row
names, expression string, or \code{TRUE}) is resolved to a subset
of data rows.
\item \strong{Background layer} -- when \code{add_bg = TRUE},
alternating background stripes are drawn via
\code{\link{bg_layer}()}.
\item \strong{Colour assignment} -- \code{\link{palette_this}()}
assigns colours to all \code{group_by} levels (including
\code{NA}, defaulting to \code{"grey80"}).
\item \strong{Horizontal reference line} -- when \code{add_hline} is
set, one or more \code{geom_hline()} are added.  If
\code{hline_color = TRUE}, each horizontal line is coloured to
match the corresponding group colour, and \code{add_hline} can
be a named vector/ list mapping groups to intercept values.
\item \strong{Line rendering} -- lines are coloured and grouped by
\code{group_by} via \code{\link[ggplot2]{geom_line}()} with
\code{aes(color = group_by, group = group_by)} and
\code{scale_color_manual()}.
\item \strong{Error bars} -- when \code{add_errorbars = TRUE}, three
colour modes are supported: follow the line colour
(\code{"line"}), track \code{group_by} colours, or use a
constant colour.  Error bars are added via
\code{\link[ggplot2]{geom_errorbar}()}.
\item \strong{Point rendering} -- points are filled per group via
\code{\link[ggplot2]{geom_point}()} with
\code{aes(fill = group_by)} and \code{scale_fill_manual()}.
\item \strong{Highlight overlay} -- an additional \code{geom_point()}
layer draws highlighted rows in the specified colour and size.
\item \strong{Plot assembly} -- \code{scale_x_discrete()},
\code{labs()}, theme application, axis styling (angle, grid
lines), and legend positioning.
\item \strong{Dimension calculation} --
\code{\link{calculate_plot_dimensions}()} computes \code{height}
and \code{width} attributes (in inches) from x-axis category
count, aspect ratio, and legend metrics (legend entries reflect
group levels rather than x levels).
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corplot.R
\name{CorPairsPlotAtomic}
\alias{CorPairsPlotAtomic}
\title{Atomic Correlation Pairs Plot}
\usage{
CorPairsPlotAtomic(
  data,
  columns = NULL,
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  diag_type = NULL,
  diag_args = list(),
  layout = c(".\\\\", "\\\\.", "/.", "./"),
  cor_method = c("pearson", "spearman", "kendall"),
  cor_palette = "RdBu",
  cor_palcolor = NULL,
  cor_size = 3,
  cor_format = "corr: {round(corr, 2)}",
  cor_fg = "black",
  cor_bg = "white",
  cor_bg_r = 0.1,
  theme = "theme_this",
  theme_args = list(),
  palette = ifelse(is.null(group_by), "Spectral", "Paired"),
  palcolor = NULL,
  palreverse = FALSE,
  title = NULL,
  subtitle = NULL,
  facet_by = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  seed = 8525,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{columns}{A character vector of column names to include in the
pairs plot.  When \code{NULL} (default), all columns except
\code{group_by} are used.  At least two columns are required.}

\item{group_by}{A character vector of column names to colour the
scatter points by.  Each unique combination becomes a separate group.
Required for \code{diag_type = "violin"} and \code{diag_type = "box"}.
Multiple columns are concatenated with \code{group_by_sep}.}

\item{group_by_sep}{A character string to separate concatenated
\code{group_by} columns.  Default \code{"_"}.}

\item{group_name}{A character string used as the colour legend title
in the scatter plots.  When \code{NULL}, the \code{group_by} column
name is used.}

\item{diag_type}{A character string specifying the plot type for
diagonal cells.  One of \code{"density"}, \code{"violin"},
\code{"histogram"}, \code{"box"}, or \code{"none"} (diagonal line).
Default: \code{"density"} (no \code{group_by}) or \code{"violin"}
(with \code{group_by}).}

\item{diag_args}{A named list of additional arguments passed to the
diagonal plot function (\code{\link{DensityPlot}},
\code{\link{ViolinPlot}}, \code{\link{Histogram}}, or
\code{\link{BoxPlot}}).  Default: \code{list()}.}

\item{layout}{A character string specifying the layout orientation.
One of the following codes (dot = scatter, backslash/slash =
diagonal): \code{.\\}, \code{\\\.}, \code{/.}, \code{./}.
Default: \code{.\\}.}

\item{cor_method}{A character string specifying the correlation method
for the fill tiles.  One of \code{"pearson"}, \code{"spearman"},
\code{"kendall"}.  Default: \code{"pearson"}.}

\item{cor_palette}{A character string specifying the colour palette for
the correlation fill tiles.  Default: \code{"RdBu"}.}

\item{cor_palcolor}{A character vector of custom colours used to create
the correlation tile palette.  When \code{NULL}, the palette's default
colours are used.}

\item{cor_size}{A numeric value specifying the font size of the
correlation text in the fill tiles.  Default: \code{3}.}

\item{cor_format}{A character string specifying a glue template for
formatting the correlation text.  The template is evaluated by
\code{glue::glue()} with access to \code{corr} (the correlation value),
\code{x}, and \code{y} (the column names).  Default:
\code{"corr: \\{round(corr, 2)\\}"}.}

\item{cor_fg}{A character string specifying the colour of the
correlation text.  Default: \code{"black"}.}

\item{cor_bg}{A character string specifying the background colour of
the correlation text boxes.  Default: \code{"white"}.}

\item{cor_bg_r}{A numeric value specifying the corner radius of the
correlation text background boxes.  Default: \code{0.1}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{seed}{The random seed to use. Default is 8525.}

\item{...}{Additional arguments passed to the underlying
\code{\link{CorPlot}} for the scatter plot cells.}
}
\value{
A \code{patchwork} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a correlation pairs (scatterplot matrix)
grid.  This is the workhorse behind the exported
\code{\link{CorPairsPlot}} — it takes a \strong{single} data frame (no
\code{split_by} support) and returns a \code{patchwork} object.

The grid arranges all pairwise scatter plots of selected columns.  The
upper or lower triangle displays correlation tiles (fill) while the
opposite triangle displays scatter plots with regression lines.  The
diagonal can show density plots, violin plots, histograms, box plots,
or a simple diagonal line.

The function supports \strong{four layout orientations} (\code{layout}),
\strong{three correlation methods} (\code{cor_method}), configurable
diagonal plots using other plotthis functions, and custom correlation
tile formatting.
}
\details{
\code{theme} and \code{theme_args} are supported and passed to
each individual cell plot.
}
\section{Architecture}{

\enumerate{
\item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{facet_by guard} — raises an error if \code{facet_by} is
provided (not supported in pairs plots; use \code{split_by}
instead).
\item \strong{Column resolution} — when \code{columns} is \code{NULL},
all columns (except \code{group_by}) are used.  Otherwise,
\code{columns} are validated via \code{\link{check_columns}()}.
At least two columns are required.
\item \strong{Default diag_type} — when \code{diag_type} is \code{NULL},
it defaults to \code{"density"} (no \code{group_by}) or
\code{"violin"} (with \code{group_by}).
\item \strong{Layout determination} — \code{get_plot_info()} is called
for each cell \code{(i, j)} to determine:
\itemize{
\item \strong{Type}: \code{layout} (diagonal line),
\code{"cor"} (scatter plot with regression),
\code{"fill"} (correlation tile).
\item \strong{Axis/label positions}: which sides of the cell
should show x-axis, y-axis, x-label, and y-label, based
on the cell's location in the matrix and the chosen
layout.
}
\item \strong{Cell rendering} — \code{get_plot()} draws each cell
based on its \code{info$type}:
\itemize{
\item \strong{Diagonal, \code{diag_type = "none"}} — draws an
X or backslash diagonal line via \code{geom_line()}.
\item \strong{Diagonal, \code{"density"}} — delegates to
\code{\link{DensityPlot}()}.
\item \strong{Diagonal, \code{"violin"}} — delegates to
\code{\link{ViolinPlot}()}; requires \code{group_by}.
\item \strong{Diagonal, \code{"histogram"}} — delegates to
\code{\link{Histogram}()}.
\item \strong{Diagonal, \code{"box"}} — delegates to
\code{\link{BoxPlot}()}; requires \code{group_by}.
\item \strong{Off-diagonal, \code{"cor"}} — delegates to
\code{\link{CorPlot}()} for the scatter plot with
regression line.
\item \strong{Off-diagonal, \code{"fill"}} — draws a
\code{geom_tile()} filled by the correlation value
with \code{scale_fill_gradientn()} and displays the
formatted correlation via
\code{ggrepel::geom_text_repel()}.
}
\item \strong{Axis/label positioning} — axes, axis titles, and labels
are positioned at the margins of the full grid based on
\code{info$xaxis}, \code{info$yaxis}, \code{info$xlab},
\code{info$ylab} using \code{scale_x_*}/\code{scale_y_*} with
appropriate \code{position} arguments.
\item \strong{Layout assembly} — all cell plots are arranged into a
matrix via \code{patchwork::wrap_plots()} with
\code{ncol = length(columns)}.  Guides are collected, and the
title/subtitle are added via \code{plot_annotation()}.  The
result is wrapped in \code{patchwork::wrap_elements()} so the
title displays correctly.
\item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
computes \code{height} and \code{width} attributes from
\code{base_height = sqrt(length(columns)) * 4}, a fixed aspect
ratio of 1, and legend metrics.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{enrich_multidb_example}
\alias{enrich_multidb_example}
\title{An example of clusterProfiler enrichment result with multiple databases}
\description{
An example of clusterProfiler enrichment result with multiple databases
}
\examples{
\dontrun{
if (interactive()) {
  data(enrich_example, package="plotthis")
  enrich_example$Database <- "DB1"
  enrich_example2 <- enrich_example
  enrich_example2$Database <- "DB2"
  enrich_example2$ID <- paste0(enrich_example2$ID, "_DB2")
  enrich_example2$Description <- paste0(enrich_example2$Description, " (DB2)")
  enrich_multidb_example <- rbind(enrich_example, enrich_example2)
}
}
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theming.R
\name{element_textbox}
\alias{element_textbox}
\alias{element_grob.element_textbox}
\title{Theme element that add a box to the text}
\usage{
element_textbox(
  family = NULL,
  face = NULL,
  size = NULL,
  colour = NULL,
  fill = NULL,
  box.colour = NULL,
  linetype = NULL,
  linewidth = NULL,
  hjust = NULL,
  vjust = NULL,
  halign = NULL,
  valign = NULL,
  lineheight = NULL,
  margin = NULL,
  padding = NULL,
  width = NULL,
  height = NULL,
  minwidth = NULL,
  maxwidth = NULL,
  minheight = NULL,
  maxheight = NULL,
  r = NULL,
  orientation = NULL,
  color = NULL,
  box.color = NULL,
  debug = FALSE,
  inherit.blank = FALSE
)

\method{element_grob}{element_textbox}(
  element,
  label = "",
  x = NULL,
  y = NULL,
  family = NULL,
  face = NULL,
  colour = NULL,
  size = NULL,
  hjust = NULL,
  vjust = NULL,
  lineheight = NULL,
  margin = NULL,
  ...
)
}
\arguments{
\item{family}{Font family}

\item{face}{Font face}

\item{size}{Font size (in pt)}

\item{colour, color}{Text color}

\item{fill}{Fill color of the enclosing box}

\item{box.colour, box.color}{Line color of the enclosing box (if different from the text color)}

\item{linetype}{Line type of the enclosing box (like \code{lty} in base R)}

\item{linewidth}{Line width of the enclosing box (measured in mm, just like \code{size} in
\code{\link[ggplot2:element]{ggplot2::element_line()}}).}

\item{hjust}{Horizontal justification}

\item{vjust}{Vertical justification}

\item{halign}{Horizontal justification}

\item{valign}{Vertical justification}

\item{lineheight}{Line height, in multiples of the font size}

\item{padding, margin}{Padding and margins around the text box.
See \code{\link[gridtext:textbox_grob]{gridtext::textbox_grob()}} for details.}

\item{width, height}{Unit objects specifying the width and height
of the textbox, as in \code{\link[gridtext:textbox_grob]{gridtext::textbox_grob()}}.}

\item{minwidth, minheight, maxwidth, maxheight}{Min and max values for width and height.
Set to NULL to impose neither a minimum nor a maximum.}

\item{r}{Unit value specifying the corner radius of the box}

\item{orientation}{Orientation of the text box. See \code{\link[gridtext:textbox_grob]{gridtext::textbox_grob()}} for details.}

\item{debug}{Not implemented.}

\item{inherit.blank}{See \code{\link[ggplot2:element]{ggplot2::margin()}} for details.}

\item{element}{A theme element created by \code{\link[=element_textbox]{element_textbox()}}.}

\item{label}{Text to display in the textbox.}

\item{x, y}{Position of the textbox.}

\item{...}{Other arguments passed to \code{\link[gridtext:textbox_grob]{gridtext::textbox_grob()}}.}
}
\value{
A ggplot2 theme element that can be used inside a \code{\link[ggplot2:theme]{ggplot2::theme()}} call.
}
\description{
Code grabbed from the \code{ggtext} package. See the original code at:
https://github.com/wilkelab/ggtext
This is used to create a text box around the text, primarily to be used in \code{CorPairsPlot}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rarefractionplot.R
\name{RarefactionPlot}
\alias{RarefactionPlot}
\title{Rarefaction / extrapolation plot}
\usage{
RarefactionPlot(
  data,
  type = 1,
  se = NULL,
  group_by = "group",
  group_by_sep = "_",
  group_name = NULL,
  split_by = NULL,
  split_by_sep = "_",
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 0.2,
  pt_size = 3,
  line_width = 1,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{type}{An integer specifying the curve type: \code{1} for
sample-size-based rarefaction/extrapolation, \code{2} for sample
completeness, or \code{3} for coverage-based rarefaction/extrapolation.
A vector of types can be passed and the data will be fortifed for all of
them; faceting or splitting then separates the panels.  Default: \code{1}.}

\item{se}{A logical value indicating whether to display confidence intervals
as semi-transparent ribbons around the estimated curve.  When \code{NULL}
(the default), it resolves to \code{TRUE} if the fortifed data contains
\code{y.lwr} and \code{y.upr} columns, and \code{FALSE} otherwise.}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{A character string used to join multiple
\code{group_by} column values when \code{group_by} has length > 1.
Also used by the exported function for the group concatenation.
Default: \code{"_"}.}

\item{group_name}{A character string used as the title for the colour
(and shape) legend.  When \code{NULL} (the default), the value of
\code{group_by} is used.}

\item{split_by}{A character vector specifying how to split the data into
separate sub-plots.  Must be one or both of \code{"q"} (diversity order)
and \code{"group"} (assemblage/site).  Multiple values are concatenated
with \code{split_by_sep}.  Cannot overlap with \code{group_by} or
\code{facet_by}.  Default: \code{NULL}.}

\item{split_by_sep}{A character string used to join multiple
\code{split_by} column values when \code{split_by} has length > 1.
Default: \code{"_"}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{pt_size}{A numeric value specifying the size of the observed-data
points.  Default: \code{3}.}

\item{line_width}{A numeric value specifying the width of the rarefaction /
extrapolation lines.  Default: \code{1}.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{seed}{A numeric seed for reproducibility.  Passed to
\code{\link{validate_common_args}()}.  Default: \code{8525}.}

\item{combine}{Logical; when \code{TRUE} (default), returns a combined
\code{patchwork} object.  When \code{FALSE}, returns a named list of
individual \code{ggplot} objects.}

\item{ncol, nrow}{Integer number of columns / rows for the combined layout
(passed to \code{\link[patchwork]{wrap_plots}}).}

\item{byrow}{Logical; fill the combined layout by row.  Default \code{TRUE}.}

\item{axes}{A character string specifying how axes should be treated across
the combined layout (passed to \code{\link{combine_plots}()}).}

\item{axis_titles}{A character string specifying how axis titles should be
treated across the combined layout.  Defaults to \code{axes}.}

\item{guides}{A character string specifying how guides (legends) should be
collected across panels (passed to \code{\link{combine_plots}()}).}

\item{design}{A custom layout design for the combined plot (passed to
\code{\link{combine_plots}()}).}

\item{...}{Additional arguments passed to \code{\link[iNEXT]{iNEXT}} when
\code{data} is not already an \code{iNEXT} object.  Common options include
\code{q} (diversity order, default \code{c(0, 1, 2)}), \code{datatype}
(\code{"abundance"} or \code{"incidence"}), and \code{nboot} (number of
bootstrap replicates).}
}
\value{
A \code{ggplot} object (single split), a \code{patchwork} object
(multiple splits with \code{combine = TRUE}), or a named list of
\code{ggplot} objects (when \code{combine = FALSE}), each with
\code{height} and \code{width} attributes in inches.
}
\description{
Draws rarefaction and extrapolation curves for biodiversity data using the
\code{iNEXT} package.  Accepts raw species-abundance / incidence-frequency
lists (which are passed to \code{\link[iNEXT]{iNEXT}()} for estimation) or
pre-computed \code{iNEXT} objects.

The function supports three curve types (sample-size-based, sample
completeness, and coverage-based), diversity orders (\code{q}), per-group
colouring, faceting, and splitting into separate sub-plots via
\code{split_by}.  Observed data are marked with points, rarefaction lines
are solid, and extrapolation segments are dashed.  Confidence intervals are
shown as semi-transparent ribbons.
}
\section{split_by workflow}{

When \code{split_by} is provided:
\enumerate{
\item \code{\link{validate_common_args}()} checks the \code{seed} and
\code{facet_by} validity.
\item The \code{type} argument is validated (must be one or more of
1, 2, 3).
\item \code{group_by}, \code{split_by}, and \code{facet_by} are validated
for allowed values (\code{"q"} and/or \code{"group"}) and checked
for mutual exclusivity — no parameter may overlap with another.
\item If \code{data} is not an \code{iNEXT} object, it is passed to
\code{\link[iNEXT]{iNEXT}()} with \code{...} (which may contain
\code{q}, \code{datatype}, \code{nboot}, etc.).
\item The \code{iNEXT} object is fortifed via
\code{\link[ggplot2]{fortify}()} for the requested \code{type}s.
Columns \code{Assemblage} and \code{Order.q} are renamed to
\code{group} and \code{q}, respectively.
\item The \code{se} parameter is resolved: if \code{NULL} it becomes
\code{TRUE} when the fortifed data contains \code{y.lwr} /
\code{y.upr} columns.
\item A \code{lty} column is created (factor with levels
\code{"Rarefaction"} and \code{"Extrapolation"}) to distinguish the
two line phases via solid / dashed linetypes.
\item \code{group_by}, \code{split_by}, and \code{facet_by} are processed
via \code{\link{check_columns}()} with \code{force_factor = TRUE}
and multi-column concatenation.
\item If \code{group_by} is \code{NULL}, a dummy \code{".group"} column
is created and the legend is hidden.
\item The data is split by \code{split_by} (preserving level order).  If
\code{split_by} is \code{NULL}, the data is wrapped in a
single-element list with name \code{"..."}.
\item Per-split \code{palette}, \code{palcolor},
\code{legend.position}, and \code{legend.direction} are resolved
via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
and \code{\link{check_legend}()}.
\item \code{\link{RarefactionPlotAtomic}()} is called for each split.
If \code{title} is a function, it receives the split level name and
can generate dynamic titles.
\item Results are combined via \code{\link{combine_plots}()} (when
\code{combine = TRUE}) or returned as a named list.
}
}

\examples{
\donttest{
set.seed(8525)
spider <- list(
   Girdled = c(46, 22, 17, 15, 15, 9, 8, 6, 6, 4, rep(2, 4), rep(1, 12)),
   Logged = c(88, 22, 16, 15, 13, 10, 8, 8, 7, 7, 7, 5, 4, 4, 4, 3, 3, 3, 3,
     2, 2, 2, 2, rep(1, 14))
)

# Basic sample-size-based rarefaction (type = 1)
RarefactionPlot(spider)

# Multiple diversity orders with faceting
RarefactionPlot(spider, q = c(0, 1, 2), facet_by = "q")

# Multiple diversity orders split into sub-plots
RarefactionPlot(spider, q = c(0, 1, 2), split_by = "q")

# Per-split palettes
RarefactionPlot(spider, q = c(0, 1, 2), split_by = "q",
                palette = c("0" = "Paired", "1" = "Set1", "2" = "Dark2"))

# Coverage-based rarefaction (type = 3) with
# group_by = "q" and facet_by = "group"
RarefactionPlot(spider, q = c(0, 1, 2), group_by = "q",
 facet_by = "group", palette = "Set1", type = 3)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap-utils.R
\name{.prep_annotations}
\alias{.prep_annotations}
\title{Prepare and normalize annotation arguments}
\usage{
.prep_annotations(
  which = c("row", "column"),
  annotation = NULL,
  annotation_side = NULL,
  annotation_palette = NULL,
  annotation_palcolor = NULL,
  annotation_type = NULL,
  annotation_params = NULL,
  annotation_agg = NULL,
  row_key = NULL,
  rsplit_key = NULL,
  col_key = NULL,
  csplit_key = NULL,
  data = NULL
)
}
\arguments{
\item{which}{\code{"row"} or \code{"column"}.}

\item{annotation}{The new structured annotation list.}

\item{annotation_side}{Deprecated. Old side argument.}

\item{annotation_palette}{Deprecated. Old palette argument.}

\item{annotation_palcolor}{Deprecated. Old palcolor argument.}

\item{annotation_type}{Deprecated. Old type argument.}

\item{annotation_params}{Deprecated. Old params argument.}

\item{annotation_agg}{Deprecated. Old agg argument.}

\item{row_key}{The actual column name for \code{rows_by}.}

\item{rsplit_key}{The actual column name for \code{rows_split_by}.}

\item{col_key}{The actual column name for \code{columns_by}.}

\item{csplit_key}{The actual column name for \code{columns_split_by}.}

\item{data}{The data frame used for column validation.}
}
\value{
A list with components \code{annotation}, \code{annotation_type},
\code{annotation_side}, \code{annotation_palette}, \code{annotation_palcolor},
\code{annotation_agg}, \code{annotation_params}, \code{annotation_name},
and \code{enabled}.
}
\description{
Consolidates the new structured annotation list format with backward-compatible
deprecation support for old flat arguments. Handles \code{TRUE}/\code{FALSE}
shortcuts, \code{.default} inheritance (with recursive \code{params} merge),
alias resolution, and shortcut expansion.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/volcanoplot.R
\name{VolcanoPlotAtomic}
\alias{VolcanoPlotAtomic}
\title{Atomic volcano plot (internal)}
\usage{
VolcanoPlotAtomic(
  data,
  x,
  y,
  ytrans = "-log10",
  color_by = NULL,
  color_name = NULL,
  flip_negatives = FALSE,
  x_cutoff = NULL,
  y_cutoff = 0.05,
  trim = c(0, 1),
  xlim = NULL,
  x_cutoff_name = NULL,
  y_cutoff_name = NULL,
  x_cutoff_color = "red2",
  y_cutoff_color = "blue2",
  x_cutoff_linetype = "dashed",
  y_cutoff_linetype = "dashed",
  x_cutoff_linewidth = 0.5,
  y_cutoff_linewidth = 0.5,
  pt_size = 2,
  pt_alpha = 0.5,
  pt_shape = 21,
  pt_border_color = TRUE,
  pt_border_size = 0.5,
  nlabel = 5,
  labels = NULL,
  label_by = NULL,
  label_size = 3,
  label_fg = "black",
  label_bg = "white",
  label_bg_r = 0.1,
  highlight = NULL,
  highlight_color = "red",
  highlight_size = 2,
  highlight_alpha = 1,
  highlight_stroke = 0.5,
  raster = NULL,
  raster_dpi = c(512, 512),
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  palreverse = FALSE,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  seed = 8525,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis.}

\item{y}{A character string specifying the column name of the data frame to plot for the y-axis.}

\item{ytrans}{A function or a function name (as a string) to transform
the y-axis values before plotting. The transformed values are used for
both the y-axis and cutoff comparisons. Default: \code{"-log10"}
(converts p-values to a -log10 scale). Other named functions can be
passed as strings, e.g. \code{"sqrt"}.}

\item{color_by}{A character string specifying the column name to colour
the points by. When \code{NULL} (default), points are automatically
categorised as \code{"sig_pos_x"}, \code{"sig_neg_x"}, or
\code{"insig"} based on \code{x_cutoff} and \code{y_cutoff}, and the
colour legend is suppressed. When a column name is provided, the colour
mapping follows the column type — discrete (character/factor) uses
\code{scale_color_manual()} with the specified \code{palette};
numeric (continuous) uses \code{scale_color_gradientn()}.}

\item{color_name}{A character string for the colour legend title when
\code{color_by} is a numeric column. When \code{NULL} (default), the
\code{color_by} column name is used.}

\item{flip_negatives}{A logical value. When \code{TRUE}, y-values of
points with negative x-values are multiplied by -1, creating a mirrored
volcano plot where both up- and down-regulated features show their
significance on the same side of the y-axis. A horizontal line at
\code{y = 0} and absolute-value axis labels are added. Default:
\code{FALSE}.}

\item{x_cutoff}{A numeric value specifying the x-axis significance
cutoff. Both the negative and positive of this value are used as
vertical threshold lines. When \code{NULL} or \code{0}, no x-cutoff
line is drawn. Default: \code{NULL}.}

\item{y_cutoff}{A numeric value specifying the y-axis significance
cutoff in the \strong{original} (untransformed) scale. The value is
transformed by \code{ytrans} before plotting. When \code{NULL}, no
y-cutoff line is drawn and the category assignment uses only the
x-cutoff. Default: \code{0.05}.}

\item{trim}{A numeric vector of length 2 specifying quantile bounds for
winsorizing the x-axis values. Values below the first quantile are
clamped to that quantile; values above the second quantile are clamped
to that quantile. Both values must be in \code{[0, 1]}. When both bounds
are nonzero and of opposite sign, they are symmetrised to the smaller
absolute value. Default: \code{c(0, 1)} (no trimming).}

\item{xlim}{A numeric vector of length 2 to set the x-axis limits.
Passed to \code{\link[ggplot2]{xlim}()}. When \code{NULL} (default),
limits are determined automatically from the data.}

\item{x_cutoff_name}{A character string for the x-cutoff legend entry.
When \code{"none"}, the legend for the x-cutoff line is suppressed
entirely (the line is still drawn). When \code{NULL} (default), a
label of the form \code{"<x> = +/-<value>"} is generated.}

\item{y_cutoff_name}{A character string for the y-cutoff legend entry.
When \code{"none"}, the legend for the y-cutoff line is suppressed
entirely (the line is still drawn). When \code{NULL} (default), a
label of the form \code{"<ylab> = <value>"} is generated.}

\item{x_cutoff_color}{A character string specifying the colour of the
x-axis cutoff line(s). Default: \code{"red2"}.}

\item{y_cutoff_color}{A character string specifying the colour of the
y-axis cutoff line(s). Default: \code{"blue2"}.}

\item{x_cutoff_linetype}{A character string specifying the linetype of
the x-axis cutoff line(s). Default: \code{"dashed"}.}

\item{y_cutoff_linetype}{A character string specifying the linetype of
the y-axis cutoff line(s). Default: \code{"dashed"}.}

\item{x_cutoff_linewidth}{A numeric value specifying the linewidth of
the x-axis cutoff line(s). Default: \code{0.5}.}

\item{y_cutoff_linewidth}{A numeric value specifying the linewidth of
the y-axis cutoff line(s). Default: \code{0.5}.}

\item{pt_size}{A numeric value specifying the point size for all data
points. Default: \code{2}.}

\item{pt_alpha}{A numeric value in \code{[0, 1]} specifying the
transparency of all data points. Default: \code{0.5}.}

\item{pt_shape}{A numeric value specifying the point shape. Default: \code{21}
(filled circle with border). Shapes 21--25 support separate fill and border
colour aesthetics; all other shapes use a single colour aesthetic. In
raster mode, all points are drawn as filled circles (shape is ignored).}

\item{pt_border_color}{Controls the point border colour. For shapes 21--25:
\itemize{
\item \code{TRUE} (default) -- border colour tracks the \code{color_by}
gradient / palette.
\item A colour string (e.g. \code{"black"}) -- constant colour border.
}
\code{FALSE} or \code{NULL} disables the border. For shapes without a fill
aesthetic (not 21--25), this parameter has no effect. In raster mode the
border is drawn as a slightly larger disc behind each point (the shape is
always a circle there), and \code{TRUE} falls back to a border disc in the
\code{color_by} colour.}

\item{pt_border_size}{A numeric value specifying the point border size
(stroke width, in mm). \code{0} disables the border. Default: \code{0.5}.}

\item{nlabel}{An integer specifying the number of top features to label
automatically. Points are ranked by Euclidean distance to the origin
within each \code{sign(x)} group (and per facet level if
\code{facet_by} is set). Only non-insignificant points receive labels.
Default: \code{5}.}

\item{labels}{A character vector of row names or integer indices
specifying which points to label. Overrides automatic \code{nlabel}
selection. When \code{NULL} (default), top \code{nlabel} points are
chosen automatically.}

\item{label_by}{A character string specifying the column whose values
are used as label text. When \code{NULL} (default), row names of the
data frame are used.}

\item{label_size}{A numeric value specifying the font size of the
labels. Default: \code{3}.}

\item{label_fg}{A character string specifying the text colour of the
labels. Default: \code{"black"}.}

\item{label_bg}{A character string specifying the background colour of
the label boxes (passed to \code{geom_text_repel(bg.color = ...)}).
Default: \code{"white"}.}

\item{label_bg_r}{A numeric value specifying the corner radius of the
label background boxes (passed to \code{geom_text_repel(bg.r = ...)}).
Default: \code{0.1}.}

\item{highlight}{A character vector of row names or integer indices
specifying which points to highlight with an overlaid point layer in
\code{highlight_color}. When \code{NULL} (default), no highlighting is
applied.}

\item{highlight_color}{A character string specifying the colour of the
highlight points. Default: \code{"red"}.}

\item{highlight_size}{A numeric value specifying the point size of the
highlight layer. Default: \code{2}.}

\item{highlight_alpha}{A numeric value in \code{[0, 1]} specifying the
transparency of the highlight points. Default: \code{1}.}

\item{highlight_stroke}{A numeric value specifying the stroke width of
the highlight point borders. Default: \code{0.5}.}

\item{raster}{A logical value. If \code{TRUE}, points are rendered via
\code{scattermore::geom_scattermore()} for efficient rasterised plotting.
Default is \code{NULL}, which auto-enables when \code{nrow(data) > 1e3}.}

\item{raster_dpi}{A numeric vector of length 2 \verb{[x_dpi, y_dpi]} specifying the raster
resolution in pixels. Passed to \code{scattermore::geom_scattermore(pixels = ...)}.
Default is \code{c(512, 512)}. If a single value is provided it is recycled to both dimensions.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{seed}{The random seed to use. Default is 8525.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a single volcano plot. This is the
workhorse behind the exported \code{\link{VolcanoPlot}} function — it takes
a \strong{single} data frame (no \code{split_by} support) and returns a
\code{ggplot} object. The plot displays statistical significance (typically
-log10 adjusted p-value) on the y-axis versus magnitude of change (log2
fold change) on the x-axis, with points coloured by significance category
or a user-supplied variable. Top features can be automatically labelled
via \code{\link[ggrepel]{geom_text_repel}()}, and specific points can be
highlighted.

The function categorises points into three groups based on cutoff
thresholds: \code{"sig_pos_x"} (points exceeding both the positive
x-cutoff and y-cutoff), \code{"sig_neg_x"} (points exceeding both the
negative x-cutoff and y-cutoff), and \code{"insig"} (all remaining
points). When \code{color_by = NULL}, this categorisation drives point
colouring; otherwise the supplied column controls the colour scale.
}
\section{Architecture}{

\enumerate{
\item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{Input validation} — \code{trim} (must be length-2 in
\code{[0, 1]}) and \code{xlim} (must be length-2 or \code{NULL})
are validated via \code{stopifnot()}. \code{trim} is sorted.
\item \strong{Column resolution} — \code{x}, \code{y}, \code{color_by},
\code{facet_by}, and \code{label_by} are validated and transformed
via \code{\link{check_columns}}. Multi-column \code{facet_by} is
concatenated with \code{force_factor = TRUE}.
\item \strong{y-axis transformation} — the y-column is transformed by
\code{ytrans} (default: \code{"-log10"}). The \code{y_cutoff}
value is also transformed.
\item \strong{x_cutoff defaulting} — if \code{x_cutoff} is
\code{NULL}, it is set to \code{0} (suppressing the x-cutoff
legend line).
\item \strong{Category assignment} — a \code{.category} factor with
levels \code{c("sig_neg_x", "insig", "sig_pos_x")} is created:
\itemize{
\item When \code{y_cutoff} is non-\code{NULL}: points with
\code{|x| > x_cutoff} AND \code{y > y_cutoff} are significant.
\item When \code{y_cutoff} is \code{NULL}: only the x-cutoff
determines significance.
}
\item \strong{Color resolution} — three cases:
\itemize{
\item \code{color_by = NULL}: uses \code{.category} as a
discrete colour column; the legend is suppressed.
\item Character/factor column: discrete colour scale via
\code{\link{palette_this}()} and
\code{scale_color_manual()}, guide suppressed.
\item Numeric column: continuous gradient via
\code{scale_color_gradientn()} with a framed colour-bar legend.
}
\item \strong{Flip negatives} — when \code{flip_negatives = TRUE},
the y-values of points with negative x are multiplied by -1,
creating a mirrored volcano where both up- and down-regulated
features show their significance on the same side of the y-axis.
\item \strong{Label column} — \code{.label} is populated from
\code{label_by} or \code{rownames(data)}.
\item \strong{Trim / winsorize} — x-values beyond the trim quantile
bounds are clamped. When both bounds are nonzero and of opposite
sign, they are symmetrised to the smaller absolute value.
Outlying points are marked in \code{.outlier}.
\item \strong{Label selection} — two modes:
\itemize{
\item Explicit \code{labels}: the specified rows (by name or
index) are marked for labelling.
\item Automatic: top \code{nlabel} points (by Euclidean distance
to origin) are selected per \code{sign(x)} group, and per facet
level if \code{facet_by} is set.
}
All labels are filtered to exclude \code{"insig"} points.
\item \strong{Data split} — data is split into \code{pos_data}
(\code{x >= 0}) and \code{neg_data} (\code{x < 0}) so that
\code{ggrepel} labels can nudge in opposite directions (positive
points nudge left, negative points nudge right).
\item \strong{Outlier jitter} — outlier points are rendered separately
with \code{position_jitter()} to reduce overplotting.
\item \strong{Base ggplot} — \code{geom_point()} layers for positive,
negative, and outlier data, with colour mapped to
\code{color_by}.
\item \strong{Colour scale} — discrete:
\code{scale_color_manual()} with \code{"insig"} forced to
\code{"grey"} (when \code{palcolor} is \code{NULL}); continuous:
\code{scale_color_gradientn()} with palette re-scaled so that
the colour-bar is centred at 0.
\item \strong{Highlight} — when \code{highlight} is provided, two
additional \code{geom_point()} layers (non-outliers and outliers
with jitter) overlay the highlighted points in
\code{highlight_color}.
\item \strong{x-cutoff lines} — vertical dashed lines at
\code{+/- x_cutoff} via \code{geom_vline()} with
\code{\link[ggnewscale]{new_scale_color}()}, labelled by
\code{x_cutoff_name}. Suppressed when \code{x_cutoff} is
\code{NULL} or \code{0}.
\item \strong{y-cutoff lines} — horizontal dashed line(s) at
\code{y_cutoff} (or \code{+/- y_cutoff} when
\code{flip_negatives = TRUE}) via \code{geom_hline()} with
\code{\link[ggnewscale]{new_scale_color}()}, labelled by
\code{y_cutoff_name}.
\item \strong{Flip-negatives axis} — when \code{flip_negatives = TRUE},
a solid \code{geom_hline(yintercept = 0)} is added and
\code{scale_y_continuous(labels = abs)} formats the y-axis.
\item \strong{x-axis limits} — optional \code{xlim} passed to
\code{ggplot2::xlim()}.
\item \strong{Reference line and labels} — a grey80 dashed vertical
line at \code{x = 0}, followed by \code{geom_text_repel()} for
positive and negative labelled points with separate x-nudges.
\item \strong{Labels and theme} — \code{labs()},
\code{coord_cartesian(clip = "off")},
\code{do_call(theme, theme_args)}, and theme elements for
\code{aspect.ratio}, \code{legend.position}, and
\code{legend.direction}.
\item \strong{Dimension calculation} —
\code{\link{calculate_plot_dimensions}()} computes \code{height}
and \code{width} attributes from \code{base_height = 5},
\code{aspect.ratio}, and legend geometry.
\item \strong{Faceting} — \code{\link{facet_plot}()} wraps the plot
with \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is
provided.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rarefractionplot.R
\name{RarefactionPlotAtomic}
\alias{RarefactionPlotAtomic}
\title{Atomic rarefaction / extrapolation plot (internal)}
\usage{
RarefactionPlotAtomic(
  data,
  type = 1,
  se = TRUE,
  group_by = "group",
  group_name = NULL,
  pt_size = 3,
  line_width = 1,
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 0.2,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame produced by \code{\link[ggplot2]{fortify}()} on an
\code{iNEXT} object.  Expected to contain columns \code{x} (sample size or
coverage), \code{y} (diversity or coverage estimate), \code{Method}
(\code{"Observed"}, \code{"Rarefaction"}, \code{"Extrapolation"}),
\code{y.lwr} / \code{y.upr} (confidence bounds, optional), \code{datatype}
(\code{"abundance"} or \code{"incidence"}), \code{group} (assemblage name,
renamed from \code{Assemblage}), and \code{q} (diversity order, renamed
from \code{Order.q}).}

\item{type}{An integer specifying the curve type: \code{1} for
sample-size-based rarefaction/extrapolation, \code{2} for sample
completeness, or \code{3} for coverage-based rarefaction/extrapolation.
A vector of types can be passed and the data will be fortifed for all of
them; faceting or splitting then separates the panels.  Default: \code{1}.}

\item{se}{A logical value indicating whether to display confidence intervals
as semi-transparent ribbons around the estimated curve.  When \code{NULL}
(the default), it resolves to \code{TRUE} if the fortifed data contains
\code{y.lwr} and \code{y.upr} columns, and \code{FALSE} otherwise.}

\item{group_by}{A character vector specifying how to group the data for
colouring the lines.  Must be one or both of \code{"q"} (diversity order)
and \code{"group"} (assemblage/site).  Multiple values are concatenated
with \code{group_by_sep}.  When \code{NULL}, a dummy \code{".group"}
column is created and the legend is hidden.  Default: \code{"group"}.}

\item{group_name}{A character string used as the title for the colour
(and shape) legend.  When \code{NULL} (the default), the value of
\code{group_by} is used.}

\item{pt_size}{A numeric value specifying the size of the observed-data
points.  Default: \code{3}.}

\item{line_width}{A numeric value specifying the width of the rarefaction /
extrapolation lines.  Default: \code{1}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the colour palette to use.
A named list or vector can be used to specify palettes for different
\code{split_by} levels in the exported \code{\link{RarefactionPlot}}.
Default: \code{"Spectral"}.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value between 0 and 1 specifying the transparency
of the confidence-interval ribbon fill.  Default: \code{0.2}.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{...}{Additional arguments (currently unused in the atomic function;
accepted for forward compatibility with the exported wrapper).}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a single rarefaction (or extrapolation)
curve from biodiversity data.  This is the workhorse behind the exported
\code{\link{RarefactionPlot}} — it takes a \strong{single} fortifed data frame
(produced by \code{\link[ggplot2]{fortify}()} on an \code{iNEXT} object)
and returns a \code{ggplot} object.

The function renders three types of curves:
\enumerate{
\item \strong{Sample-size-based} (\code{type = 1}) — species diversity
as a function of sample size (number of individuals or sampling
units), with rarefaction (interpolation) and extrapolation segments.
\item \strong{Sample completeness} (\code{type = 2}) — sample coverage
as a function of sample size.
\item \strong{Coverage-based} (\code{type = 3}) — species diversity as a
function of sample coverage.
}

Observed data points are marked with \code{geom_point()} (shape per group),
the rarefaction / extrapolation lines are drawn with \code{geom_line()}
using a solid/dashed linetype to distinguish the two phases, and
confidence intervals are rendered as semi-transparent ribbons when
\code{se = TRUE} and the fortifed data contains \code{y.lwr} / \code{y.upr}
columns.
}
\section{Architecture}{

\enumerate{
\item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{Axis labels} — set according to the combination of
\code{type} and the data's \code{datatype} attribute
(\code{"abundance"} vs. \code{"incidence"}):
\itemize{
\item \code{type = 2}: x = "Number of individuals" or
"Number of sampling units", y = "Sample coverage".
\item \code{type = 3 || 4}: x = "Sample coverage",
y = "Species diversity".
\item \code{type = 1} (abundance): x = "Number of individuals",
y = "Species diversity".
\item \code{type = 1} (incidence): x = "Number of sampling units",
y = "Species diversity".
}
\item \strong{Group name fallback} — if \code{group_name} is \code{NULL},
it is set to the value of \code{group_by} (i.e. \code{"q"} or
\code{"group"}).
\item \strong{Base ggplot} — initialises \code{ggplot(data, aes(x, y,
        color = group_by))}.
\item \strong{Observed data points} — \code{geom_point()} on rows where
\code{Method == "Observed"}, with shape mapped to the group variable.
\item \strong{Colour scale} — \code{scale_color_manual()} using
\code{\link{palette_this}()}.  The legend is suppressed when
\code{group_by} is the dummy \code{".group"}.
\item \strong{Shape scale} — \code{scale_shape_discrete()} with the same
legend-suppression logic.
\item \strong{Rarefaction / extrapolation lines} — \code{geom_line()}
with linetype mapped to the \code{lty} column (\code{"Rarefaction"}
vs. \code{"Extrapolation"}).
\item \strong{Linetype scale} — \code{scale_linetype_manual()} with
\code{solid} and \code{dashed} values, legend key width set to
1 cm.
\item \strong{Theme and labels} — \code{do_call(theme, theme_args)},
\code{panel.grid.major} (grey80 dashed), aspect ratio, legend
position / direction, and title / subtitle.
\item \strong{Confidence ribbon} — when \code{se = TRUE}, a
\code{geom_ribbon(aes(ymin = y.lwr, ymax = y.upr, fill = group_by))}
is added with the same per-group palette at transparency
\code{alpha}.
\item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
computes \code{height} / \code{width} attributes from
\code{base_height = 4.5}, \code{aspect.ratio}, and legend metrics.
\item \strong{Faceting} — \code{\link{facet_plot}()} wraps the plot with
\code{facet_wrap} / \code{facet_grid} if \code{facet_by} is
provided.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scatterplot.R
\name{ScatterPlotAtomic}
\alias{ScatterPlotAtomic}
\title{Atomic scatter plot (internal)}
\usage{
ScatterPlotAtomic(
  data,
  x,
  y,
  size_by = 2,
  size_name = NULL,
  color_by = NULL,
  color_name = NULL,
  palreverse = FALSE,
  theme = "theme_this",
  theme_args = list(),
  alpha = ifelse(shape \%in\% 21:25, 0.65, 1),
  shape = 21,
  border_color = "black",
  border_size = 0.5,
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  xtrans = "identity",
  ytrans = "identity",
  highlight = NULL,
  highlight_shape = 16,
  highlight_size = 3,
  highlight_color = "red",
  highlight_alpha = 1,
  palette = ifelse(!is.null(color_by) && !is.numeric(data[[color_by]]), "Paired",
    "Spectral"),
  palcolor = NULL,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the numeric column to use for the
x-axis. A numeric column is expected.}

\item{y}{A character string specifying the numeric column to use for the
y-axis. A numeric column is expected.}

\item{size_by}{Either a numeric constant (uniform dot size) or a character
string naming a numeric column whose values control dot size via
\code{scale_size_area(max_size = 6)}. Default: \code{2}.}

\item{size_name}{A character string for the size legend title. When
\code{NULL} (default), the \code{size_by} column name is used. Ignored when
\code{size_by} is a numeric constant.}

\item{color_by}{A character string naming a column whose values control
dot colour. Can be numeric (continuous gradient via
\code{scale_fill_gradientn()} / \code{scale_color_gradientn()}) or
factor/character (discrete palette via
\code{scale_fill_manual()} / \code{scale_color_manual()}). For shapes
21--25, the colour is applied to the fill aesthetic. When \code{NULL}
(default), all dots are rendered in a single colour derived from the
palette.}

\item{color_name}{A character string for the colour legend title. When
\code{NULL} (default), the \code{color_by} column name is used.}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{shape}{A numeric value specifying the point shape. Default: \code{21}
(filled circle with border). Shapes 21--25 support separate fill and border
colour aesthetics; all other shapes use a single colour aesthetic.}

\item{border_color}{Controls the point border colour. For shapes 21--25:
\itemize{
\item \code{"black"} (default) -- constant black border.
\item A colour string (e.g. \code{"red"}, \code{"#FF0000"}) -- constant
colour border.
\item \code{TRUE} -- border colour tracks the \code{color_by} gradient
/ palette via \code{scale_color_gradientn()} /
\code{scale_color_manual()}.
}
For shapes without a fill aesthetic (not 21--25), this parameter has no
effect.}

\item{border_size}{A numeric value specifying the point border size. Default: \code{0.5}.}

\item{lower_quantile, upper_quantile}{Lower and upper quantiles for the continuous color/fill scale.
The actual cutoffs are determined by these quantiles when \code{lower_cutoff} and
\code{upper_cutoff} are \code{NULL}. Defaults: \code{lower_quantile = 0}, \code{upper_quantile = 0.99}.}

\item{lower_cutoff, upper_cutoff}{Explicit lower and upper cutoffs for the continuous color/fill scale.
When \code{NULL} (the default), the cutoffs are determined by \code{lower_quantile} and
\code{upper_quantile} via \code{\link[stats]{quantile}}. Values outside the
\code{[lower_cutoff, upper_cutoff]} range are clamped (winsorized) to the nearest cutoff value.}

\item{xtrans}{A character string specifying the transformation for the
x-axis, passed to \code{scale_x_continuous(trans = ...)}. Common options:
\code{"identity"} (default), \code{"log10"}, \code{"log2"},
\code{"sqrt"}, \code{"reverse"}. See \code{ggplot2::continuous_scale()}
for a full list.}

\item{ytrans}{A character string specifying the transformation for the
y-axis, passed to \code{scale_y_continuous(trans = ...)}. Common options:
\code{"identity"} (default), \code{"log10"}, \code{"log2"},
\code{"sqrt"}, \code{"reverse"}.}

\item{highlight}{Specifies which points to highlight with an overlaid
\code{geom_point()} layer. Accepted values:
\itemize{
\item \code{NULL} (default) -- no highlighting.
\item \code{TRUE} -- all points are highlighted.
\item A numeric vector -- row indices of points to highlight.
\item A single character string -- an R expression (e.g.
\code{"x > 0"}) that is parsed with \code{rlang::parse_expr()} and
evaluated via \code{\link[dplyr]{filter}()} to select rows.
\item A character vector -- rownames of points to highlight. An error
is thrown if the data has no rownames.
}}

\item{highlight_shape}{A numeric value specifying the point shape for
highlighted points. Default: \code{16} (filled circle). Shapes 21--25
use the \code{fill} aesthetic; other shapes use \code{color}.}

\item{highlight_size}{A numeric value specifying the size of highlighted
points. Default: \code{3}.}

\item{highlight_color}{A character string specifying the colour of
highlighted points. Default: \code{"red"}.}

\item{highlight_alpha}{A numeric value in \code{[0, 1]} specifying the
transparency of highlighted points. Default: \code{1}.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a single scatter plot. This is the workhorse
behind the exported \code{\link{ScatterPlot}} function -- it takes a
\strong{single} data frame (no \code{split_by} support) and returns a
\code{ggplot} object. The plot positions points by \code{x} and \code{y}
coordinates (both numeric), with optional size encoding via a third numeric
variable (\code{size_by}), and colour encoding (\code{color_by}) that can be
numeric (continuous gradient) or categorical (discrete palette).

Key features:
\itemize{
\item \strong{Variable point size} -- \code{size_by} accepts either a
numeric constant (uniform dot size) or a column name (size scales with
value via \code{scale_size_area()}).
\item \strong{Colour modes} -- \code{color_by} can be numeric (rendered as
a continuous gradient via \code{scale_fill_gradientn()} or
\code{scale_color_gradientn()}) or factor/character (discrete palette
via \code{scale_fill_manual()}/\code{scale_color_manual()}).
\item \strong{Shape support} -- shapes 21--25 support separate fill and
border (\code{color}) aesthetics; all other shapes use only the colour
aesthetic. The \code{border_color} parameter can be a constant colour,
\code{TRUE} (track the fill gradient), or omitted.
\item \strong{Colour scale trimming} -- \code{lower_quantile} /
\code{upper_quantile} (or explicit \code{lower_cutoff} /
\code{upper_cutoff}) trim/clamp the continuous colour scale extremes.
\item \strong{Axis transformation} -- \code{xtrans} and \code{ytrans}
apply arbitrary scale transformations (e.g. \code{"log10"}, \code{"sqrt"})
to the x and y axes via \code{scale_x_continuous()} /
\code{scale_y_continuous()}.
\item \strong{Point highlighting} -- \code{highlight} accepts indices,
rownames, logical \code{TRUE}, or a string expression to select points
that are overlaid with a distinct shape, size, colour, and alpha.
}
}
\section{Architecture}{

\enumerate{
\item \strong{ggplot dispatch} -- selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{Column resolution} -- \code{facet_by} is validated and
forced to factor via \code{\link{check_columns}()}. \code{size_by} is
validated when it is a column name; numeric constants are kept as-is.
\code{color_by} is validated.
\item \strong{Dummy colour guard} -- when \code{color_by = NULL}, a
synthetic \code{.color_by} column (constant \code{""}) is created and the
colour legend is suppressed.
\item \strong{Categorical colour coercion} -- when \code{color_by} is a
non-numeric column, it is forced to factor via
\code{\link{check_columns}()} with \code{force_factor = TRUE}.
\item \strong{Continuous colour scale preparation} -- when
\code{color_by} is numeric,
\code{\link{prepare_continuous_color_scale}()} computes quantile-trimmed or
cutoff-clamped colour range endpoints (\code{feat_colors_value}) and
winsorizes out-of-range data.
\item \strong{Highlight resolution} -- \code{highlight} is processed into
a subset data frame (\code{hidata}) via one of four paths:
\code{TRUE} (all points), numeric index vector, a single string expression
(parsed with \code{rlang::parse_expr()} and applied via
\code{\link[dplyr]{filter}()}), or a character vector of rownames.
An error is thrown if rownames are used on data without row names.
\item \strong{Shape fill detection} -- determines whether the point shape
(\code{shape}) supports a separate fill aesthetic (shapes 21--25).
\item \strong{Base ggplot} -- initialises \code{ggplot(data, aes(x, y))}.
\item \strong{Point layer configuration} -- the aesthetic mapping and
constant aesthetics for \code{geom_point()} are assembled:
\itemize{
\item \emph{Fill vs colour}: shapes 21--25 map \code{fill} (and
optionally \code{color} when \code{border_color = TRUE}); other shapes
map \code{color}.
\item \emph{Size}: a numeric \code{size_by} is passed as a constant
aesthetic; a column name is mapped to the \code{size} aesthetic.
}
\item \strong{Size scale} -- when \code{size_by} is a column name,
\code{scale_size_area(max_size = 6)} is added with a size legend
(order = 1, title = \code{size_name \%||\% size_by}).
\item \strong{Fill / colour scale (4 branches)}:
\itemize{
\item \emph{Shape with fill + numeric colour}: \code{scale_fill_gradientn()}
with \code{feat_colors_value} rescaling and either a colour-bar legend
or suppressed guide. When \code{border_color = TRUE}, a matching
\code{scale_color_gradientn()} is added.
\item \emph{Shape with fill + factor colour}: \code{scale_fill_manual()}
with \code{\link{palette_this}()} colours and a discrete legend. When
\code{border_color = TRUE}, a matching \code{scale_color_manual()} is
added.
\item \emph{Shape without fill + numeric colour}: \code{scale_color_gradientn()}
with continuous colour-bar legend.
\item \emph{Shape without fill + factor colour}: \code{scale_color_manual()}
with discrete legend.
}
\item \strong{Highlight overlay} -- when \code{hidata} is non-\code{NULL},
a second \code{geom_point()} layer is added using \code{highlight_shape},
\code{highlight_color}, \code{highlight_size}, and \code{highlight_alpha}.
The fill or colour aesthetic is chosen based on whether
\code{highlight_shape} is 21--25.
\item \strong{Axis scales} -- \code{scale_x_continuous(trans = xtrans)}
and \code{scale_y_continuous(trans = ytrans)} apply the requested
transformations.
\item \strong{Labels and theme} -- \code{labs(title, subtitle, x, y)} sets
plot annotations. \code{do_call(theme, theme_args)} applies the theme.
\code{panel.grid.major} is set to grey80 dashed lines.
\item \strong{Dimension calculation} -- \code{\link{calculate_plot_dimensions}()}
computes plot \code{height} and \code{width} from \code{base_height = 5},
\code{aspect.ratio}, and legend metrics (number of legend items and
maximum label character width).
\item \strong{Faceting} -- \code{\link{facet_plot}()} wraps the plot with
\code{facet_wrap} / \code{facet_grid} if \code{facet_by} is provided.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/barplot.R
\name{BarPlot}
\alias{BarPlot}
\alias{SplitBarPlot}
\alias{WaterfallPlot}
\title{Bar / SplitBar / Waterfall Plot}
\usage{
BarPlot(
  data,
  x,
  x_sep = "_",
  y = NULL,
  flip = FALSE,
  fill_by = TRUE,
  fill_name = NULL,
  line_name = NULL,
  label_nudge = 0.02,
  label = NULL,
  label_fg = "black",
  label_size = 4,
  label_bg = "white",
  label_bg_r = 0.1,
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  split_by = NULL,
  split_by_sep = "_",
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  facet_args = list(),
  add_bg = FALSE,
  bg_palette = "stripe",
  bg_palcolor = NULL,
  bg_alpha = 0.2,
  add_line = NULL,
  line_color = "red2",
  line_width = 0.6,
  line_type = 2,
  add_trend = FALSE,
  trend_color = "black",
  trend_linewidth = 1,
  trend_ptsize = 2,
  theme = "theme_this",
  theme_args = list(),
  palette = NULL,
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  x_text_angle = 0,
  aspect.ratio = 1,
  y_min = NULL,
  y_max = NULL,
  position = "auto",
  position_dodge_preserve = "total",
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  keep_empty = FALSE,
  keep_na = FALSE,
  expand = waiver(),
  width = waiver(),
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  seed = 8525,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)

SplitBarPlot(
  data,
  x,
  y,
  y_sep = "_",
  flip = FALSE,
  split_by = NULL,
  split_by_sep = "_",
  alpha_by = NULL,
  alpha_reverse = FALSE,
  alpha_name = NULL,
  order_y = list(`+` = c("x_desc", "alpha_desc"), `-` = c("x_desc", "alpha_asc")),
  bar_height = 0.9,
  lineheight = 0.5,
  max_charwidth = 80,
  fill_by = NULL,
  fill_by_sep = "_",
  fill_name = NULL,
  direction_name = "direction",
  direction_pos_name = "positive",
  direction_neg_name = "negative",
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  palreverse = FALSE,
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  facet_by = NULL,
  facet_scales = "free_y",
  facet_nrow = NULL,
  facet_ncol = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  x_min = NULL,
  x_max = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  keep_empty = FALSE,
  keep_na = FALSE,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  seed = 8525,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)

WaterfallPlot(
  data,
  x,
  y,
  y_sep = "_",
  flip = FALSE,
  split_by = NULL,
  split_by_sep = "_",
  alpha_by = NULL,
  alpha_reverse = FALSE,
  alpha_name = NULL,
  order_y = list(`+` = c("x_desc", "alpha_desc"), `-` = c("x_desc", "alpha_asc")),
  bar_height = 0.9,
  lineheight = 0.5,
  max_charwidth = 80,
  fill_by = NULL,
  fill_by_sep = "_",
  fill_name = NULL,
  direction_name = "direction",
  direction_pos_name = "positive",
  direction_neg_name = "negative",
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  palreverse = FALSE,
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  facet_by = NULL,
  facet_scales = "free_y",
  facet_nrow = NULL,
  facet_ncol = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  x_min = NULL,
  x_max = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  keep_empty = FALSE,
  keep_na = FALSE,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  seed = 8525,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis.}

\item{x_sep}{A character string to join multiple \code{x} columns.
Default \code{"_"}.}

\item{y}{A character string specifying the column name of the data frame to plot for the y-axis.}

\item{flip}{Logical; if \code{TRUE}, swap the x and y axes.}

\item{fill_by}{A variable used to fill the bars.  Both categorical and
numeric columns are accepted:
\itemize{
\item \code{TRUE} (default) — fill by the x-axis values.
\item \code{FALSE} — solid fill (first palette colour).
\item A column name (character/factor) — discrete colour scale.
\item A column name (numeric) — continuous gradient with quantile
/ cutoff controls.
}
Ignored when \code{group_by} is provided (fill is determined by
\code{group_by}).}

\item{fill_name}{A character string for the fill legend title.  Only
applies when \code{group_by = NULL} and the fill is from \code{fill_by}.}

\item{line_name}{Legend name for the reference line.}

\item{label_nudge}{A numeric value controlling the distance between labels
and the bar top, expressed as a fraction of the data range.}

\item{label}{A column name (or \code{TRUE}) for text labels on bars.
When \code{TRUE}, the y-axis values are labelled.  When a column name,
the values in that column are used.}

\item{label_fg}{A character string specifying the label text colour.}

\item{label_size}{A numeric value specifying the label text size.}

\item{label_bg}{A character string specifying the label background colour.}

\item{label_bg_r}{A numeric value specifying the label background corner
radius.}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{group_name}{A character string for the group fill legend title.
When \code{NULL}, the \code{group_by} column name is used.}

\item{split_by}{The column(s) to split the data by for separate sub-plots.}

\item{split_by_sep}{Separator for concatenated \code{split_by} columns.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{facet_args}{A list of additional arguments passed to the faceting
function (e.g., \code{scales}, \code{labeller}).}

\item{add_bg}{Logical; add alternating background stripes behind the bars.}

\item{bg_palette}{Palette for the background stripes.}

\item{bg_palcolor}{Custom colours for the background stripes.}

\item{bg_alpha}{Alpha transparency for the background stripes.}

\item{add_line}{A numeric y-intercept for a horizontal reference line.}

\item{line_color}{Colour of the reference line.}

\item{line_width}{Width of the reference line.}

\item{line_type}{Linetype of the reference line (e.g., 1 = solid, 2 = dashed).}

\item{add_trend}{Logical; add a trend line and points connecting the bar tops.}

\item{trend_color}{Colour of the trend line.}

\item{trend_linewidth}{Width of the trend line.}

\item{trend_ptsize}{Size of the trend line points.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{lower_quantile, upper_quantile}{Lower and upper quantiles for the continuous color/fill scale.
The actual cutoffs are determined by these quantiles when \code{lower_cutoff} and
\code{upper_cutoff} are \code{NULL}. Defaults: \code{lower_quantile = 0}, \code{upper_quantile = 0.99}.}

\item{lower_cutoff, upper_cutoff}{Explicit lower and upper cutoffs for the continuous color/fill scale.
When \code{NULL} (the default), the cutoffs are determined by \code{lower_quantile} and
\code{upper_quantile} via \code{\link[stats]{quantile}}. Values outside the
\code{[lower_cutoff, upper_cutoff]} range are clamped (winsorized) to the nearest cutoff value.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{y_min, y_max}{Numeric limits for the y-axis (or x-axis when flipped).}

\item{position}{A character string specifying the bar layout:
\code{"auto"} (default: dodge when \eqn{\le}5 groups, stack otherwise),
\code{"dodge"} (side-by-side), or \code{"stack"} (stacked on top of
each other).}

\item{position_dodge_preserve}{A character string passed to
\code{\link[ggplot2]{position_dodge2}()}: \code{"total"} preserves the
overall bar group width; \code{"single"} preserves individual bar widths.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{expand}{The values to expand the x and y axes. It is like CSS padding.
When a single value is provided, it is used for both axes on both sides.
When two values are provided, the first value is used for the top/bottom side and the second value is used for the left/right side.
When three values are provided, the first value is used for the top side, the second value is used for the left/right side, and the third value is used for the bottom side.
When four values are provided, the values are used for the top, right, bottom, and left sides, respectively.
You can also use a named vector to specify the values for each side.
When the axis is discrete, the values will be applied as 'add' to the 'expansion' function.
When the axis is continuous, the values will be applied as 'mult' to the 'expansion' function.
See also \url{https://ggplot2.tidyverse.org/reference/expansion.html}}

\item{width}{A numeric value specifying the bar width (0–1).}

\item{combine}{Logical; when \code{TRUE} (default), returns a combined
\code{patchwork} object.  When \code{FALSE}, returns a named list of
\code{ggplot} objects.}

\item{ncol, nrow}{Integer number of columns / rows for the combined layout.}

\item{byrow}{Logical; fill the combined layout by row (default \code{TRUE}).}

\item{seed}{A numeric seed for reproducibility.}

\item{axes, axis_titles}{Character strings for axis handling in the
combined layout.}

\item{guides}{Character string for legend collection across panels.}

\item{design}{A custom layout design for the combined plot.}

\item{...}{Additional arguments.}

\item{y_sep}{A character string to join multiple \code{y} columns.
Default \code{"_"}.}

\item{alpha_by}{A character string naming a numeric column to encode as
bar opacity.  Default \code{NULL} (all bars fully opaque).}

\item{alpha_reverse}{Logical; if \code{TRUE}, reverse the alpha scale
direction (solid for low values, transparent for high).}

\item{alpha_name}{A character string for the alpha legend title.}

\item{order_y}{A named list controlling the vertical ordering of bars.
Keys are \code{"+"} (positive bars), \code{"-"} (negative bars), or
\code{"*"} (all bars).  Values are character vectors of ordering
criteria: \code{"x_asc"}, \code{"x_desc"}, \code{"alpha_asc"},
\code{"alpha_desc"}.  Default orders positive bars by descending x and
descending alpha; negative bars by descending x and ascending alpha.}

\item{bar_height}{A numeric value (0–1) specifying the bar height as a
fraction of the available category slot.}

\item{lineheight}{A numeric value controlling the line height of wrapped
category labels.}

\item{max_charwidth}{An integer specifying the maximum character width
for wrapping category labels.}

\item{fill_by_sep}{A character string to join multiple \code{fill_by}
columns.  Default \code{"_"}.}

\item{direction_name}{A character string naming the internal direction
column (used in legends).  Default \code{"direction"}.}

\item{direction_pos_name}{A character string labelling the positive
direction in the legend.  Default \code{"positive"}.}

\item{direction_neg_name}{A character string labelling the negative
direction in the legend.  Default \code{"negative"}.}

\item{x_min, x_max}{Numeric limits for the x-axis.  When \code{NULL},
symmetric limits are computed from the maximum absolute x-value.}
}
\value{
A \code{ggplot} object, a \code{patchwork} object, or a named list
of \code{ggplot} objects (when \code{combine = FALSE}), each with
\code{height} and \code{width} attributes in inches.

A \code{ggplot} object, a \code{patchwork} object, or a named list
of \code{ggplot} objects (when \code{combine = FALSE}), each with
\code{height} and \code{width} attributes in inches.
}
\description{
\code{BarPlot} draws bar plots with flexible fill, grouping, labelling, and annotation
options.  Supports both simple single-colour bars and grouped bars (dodged
or stacked).  Bars can be filled by a categorical variable (discrete
colour scale), a continuous variable (colour gradient), or a fixed colour.

The function supports \strong{count aggregation} (omit \code{y} to plot
observation counts), \strong{proportion scaling} (\code{scale_y = TRUE}
for grouped bars), background stripes (\code{add_bg}), bar labels, trend
lines, horizontal reference lines, and splitting into separate sub-plots
via \code{split_by}.

\code{SplitBarPlot} (also known as \code{WaterfallPlot}) draws a divergent bar plot
where bars extend left (negative values) and right (positive values) from a
central zero line.  The bar fill colour and opacity can encode additional
variables, and the vertical ordering of categories is fully customisable.

The function supports \strong{split_by} to produce separate panels,
\strong{facet_by} for grouped views within panels, and
\strong{alpha_by} for encoding a secondary numeric variable via opacity.
}
\section{split_by workflow}{

When \code{split_by} is provided:
\enumerate{
\item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
normalise the \code{keep_na} / \code{keep_empty} arguments for all
columns (\code{x}, \code{split_by}, \code{facet_by}, \code{group_by}).
\item The \code{split_by} column is validated and its NA / empty levels
are processed.  It is then removed from the per-column
\code{keep_na} / \code{keep_empty} lists.
\item The data is split by \code{split_by} (preserving level order).  If
\code{split_by} is \code{NULL}, the data is wrapped in a
single-element list with name \code{"..."}.
\item Per-split \code{palette}, \code{palcolor},
\code{legend.position}, and \code{legend.direction} are resolved
via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
and \code{\link{check_legend}()}.
\item \code{\link{BarPlotAtomic}()} is called for each split.
\item Results are combined via \code{\link{combine_plots}()} (when
\code{combine = TRUE}) or returned as a named list.
}


When \code{split_by} is provided:
\enumerate{
\item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
normalise the \code{keep_na} / \code{keep_empty} arguments.
\item The \code{split_by} column is validated and its NA / empty levels
are processed.  It is then removed from the per-column lists.
\item The data is split by \code{split_by} (preserving level order).
\item Per-split \code{palette}, \code{palcolor},
\code{legend.position}, and \code{legend.direction} are resolved.
\item \code{\link{SplitBarPlotAtomic}()} is called for each split.
When \code{title} is a function, it receives the split level name
for dynamic title generation.
\item Results are combined via \code{\link{combine_plots}()}.
}
}

\examples{
\donttest{
data <- data.frame(
    x = c("A", "B", "C", "D", "E", "F", "G", "H"),
    y = c(10, 8, 16, 4, 6, 12, 14, 2),
    group = c("G1", "G1", "G2", "G2", "G3", "G3", "G4", "G4"),
    facet = c("F1", "F2", "F3", "F4", "F1", "F2", "F3", "F4")
)

# Single-colour bars
BarPlot(data, x = "x", y = "y")

# Solid fill (no colour mapping)
BarPlot(data, x = "x", y = "y", fill_by = FALSE)

# Label bar tops
BarPlot(data, x = "x", y = "y", label = TRUE)
BarPlot(data, x = "x", y = "y", label = "facet", label_nudge = 0)

# Grouped bars
BarPlot(data, x = "group", y = "y", group_by = "x")

# Dodged bars with background stripes
BarPlot(data,
    x = "group", y = "y", group_by = "x",
    position = "dodge", add_bg = TRUE)

# split_by with faceting
BarPlot(data,
    x = "x", y = "y", split_by = "group",
    facet_by = "facet", position = "dodge", facet_ncol = 1)

# split_by with collected guides
BarPlot(data,
    x = "x", y = "y", split_by = "group", facet_by = "facet",
    position = "dodge", facet_ncol = 1, guides = 'collect')

# Per-split palettes
BarPlot(data,
    x = "x", y = "y", split_by = "group",
    palette = list(G1 = "Reds", G2 = "Blues", G3 = "Greens", G4 = "Purp"),
    facet_by = "facet", position = "dodge", facet_ncol = 1)

# Background stripe palette
BarPlot(data,
    x = "group", y = "y", group_by = "x",
    position = "dodge", add_bg = TRUE, bg_palette = "Spectral")

# Count bars (y = NULL)
BarPlot(data, x = "group", ylab = "count")

# Flipped axes
BarPlot(data, x = "group", flip = TRUE, ylab = "count")

# Numeric fill_by with colour gradient
BarPlot(data, x = "x", y = "y", fill_by = "y", flip = TRUE)

# Control fill colour scale limits (quantile)
BarPlot(data, x = "x", y = "y", fill_by = "y", flip = TRUE,
        lower_quantile = 0.1, upper_quantile = 0.9)

# Control fill colour scale limits (explicit cutoff)
BarPlot(data, x = "x", y = "y", fill_by = "y", flip = TRUE,
        lower_cutoff = 5, upper_cutoff = 12)

# keep_na and keep_empty examples
data <- data.frame(
    x = factor(c("A", "B", "C", "D", "E", "F", NA, "H"),
               levels = LETTERS[1:10]),
    y = c(10, 8, 16, 4, 6, NA, 14, 2),
    group = factor(c("G1", "G1", "G2", NA, "G3", "G3", "G5", "G5"),
                   levels = c("G1", "G2", "G3", "G4", "G5")),
    facet = factor(c("F1", NA, "F3", "F4", "F1", "F2", "F3", "F4"),
                   levels = c("F1", "F2", "F3", "F4", "F5"))
)

# Default: NA and empty levels dropped
BarPlot(data, x = "x", y = "y")

# Keep both NA and empty levels
BarPlot(data, x = "x", y = "y",
        keep_na = TRUE, keep_empty = TRUE)

# With faceting
BarPlot(data, x = "x", y = "y",
        keep_na = TRUE, keep_empty = TRUE, facet_by = "facet")

# Keep NA, hide empty levels but reserve their colours
BarPlot(data, x = "x", y = "y",
        keep_na = TRUE, keep_empty = 'level')

# Per-column keep_na / keep_empty
BarPlot(data, x = "x", y = "y",
        keep_na = list(x = TRUE), keep_empty = list(x = FALSE))

# Grouped bars with keep_na / keep_empty
BarPlot(data, x = "group", y = "y", group_by = "x")
BarPlot(data, x = "group", y = "y", group_by = "x",
        keep_na = TRUE, keep_empty = TRUE)
BarPlot(data, x = "group", y = "y", group_by = "x",
        keep_na = TRUE, keep_empty = TRUE, facet_by = "facet")

# Per-column on grouped bars
BarPlot(data, x = "group", y = "y", group_by = "x",
        keep_na = list(x = TRUE, group = FALSE),
        keep_empty = list(x = FALSE, group = TRUE))
}
\donttest{
set.seed(8525)
data <- data.frame(
    word = c("apple", "banana", "cherry", "date", "elderberry",
             "It is a very long term with a lot of words"),
    count = c(-10, 20, -30, 40, 50, 34),
    score = c(1, 2, 3, 4, 5, 3.2),
    group = c("A", "A", "B", "B", "C", "C")
)

# Basic split bar plot with alpha encoding
SplitBarPlot(data, x = "count", y = "word", alpha_by = "score")

# Control label wrapping
SplitBarPlot(data, x = "count", y = "word", alpha_by = "score",
             max_charwidth = 30, lineheight = 1.1)

# Fill by categorical variable
SplitBarPlot(data, x = "count", y = "word", fill_by = "group")

# Faceting
SplitBarPlot(data, x = "count", y = "word", facet_by = "group",
             fill_name = "Direction")

# Per-split palettes
SplitBarPlot(data, x = "count", y = "word", alpha_by = "score",
             split_by = "group",
             palette = c(A = "Reds", B = "Blues", C = "Greens"))

# keep_na and keep_empty examples
data <- data.frame(
    word = factor(c("apple", "banana", "cherry", NA, "elderberry",
         "It is a very long term with a lot of words"),
         levels = c("apple", "banana", "cherry", "date", "elderberry",
             "unused", "It is a very long term with a lot of words")),
    count = c(-10, 20, NA, 40, 10, 34),
    score = c(1, 2, 3, 4, 5, 3.2),
    group = factor(sample(c("A", "A", "B", "B", "C", "C")),
         levels = c("A", "B", "C", "D"))
)

# Default: NA and empty levels dropped
SplitBarPlot(data, x = "count", y = "word", alpha_by = "score")

# Keep NA and empty levels
SplitBarPlot(data, x = "count", y = "word", alpha_by = "score",
             keep_na = TRUE, keep_empty = TRUE)

# Keep with faceting
SplitBarPlot(data, x = "count", y = "word", alpha_by = "score",
             keep_na = TRUE, keep_empty = TRUE, facet_by = "group")

# Keep NA, hide empty levels (reserve colours)
SplitBarPlot(data, x = "count", y = "word", alpha_by = "score",
             keep_na = TRUE, keep_empty = "level")

# Per-column control
SplitBarPlot(data, x = "count", y = "word", alpha_by = "score",
             keep_na = list(word = FALSE), keep_empty = list(word = TRUE))

# Control fill colour scale limits
SplitBarPlot(data, x = "count", y = "word", fill_by = "score",
             lower_cutoff = 1, upper_cutoff = 4)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\name{NetworkAtomic}
\alias{NetworkAtomic}
\title{NetworkAtomic}
\usage{
NetworkAtomic(
  links,
  nodes = NULL,
  from = NULL,
  from_sep = "_",
  to = NULL,
  to_sep = "_",
  node_by = NULL,
  node_by_sep = "_",
  link_weight_by = 2,
  link_weight_name = NULL,
  link_type_by = "solid",
  link_type_name = NULL,
  node_size_by = 15,
  node_size_name = NULL,
  node_color_by = "black",
  node_color_name = NULL,
  node_shape_by = 21,
  node_shape_name = NULL,
  node_fill_by = "grey20",
  node_fill_name = NULL,
  link_alpha = 1,
  node_alpha = 0.95,
  node_stroke = 1.5,
  cluster_scale = c("fill", "color", "shape"),
  node_size_range = c(5, 20),
  link_weight_range = c(0.5, 5),
  link_arrow_offset = 20,
  link_curvature = 0,
  link_color_by = "from",
  link_color_name = NULL,
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  link_palette = ifelse(link_color_by \%in\% c("from", "to"), palette, "Set1"),
  link_palcolor = if (link_color_by \%in\% c("from", "to")) palcolor else NULL,
  directed = TRUE,
  layout = "circle",
  cluster = "none",
  add_mark = FALSE,
  mark_expand = ggplot2::unit(10, "mm"),
  mark_type = c("hull", "ellipse", "rect", "circle"),
  mark_alpha = 0.1,
  mark_linetype = 1,
  add_label = TRUE,
  label_size = 3,
  label_fg = "white",
  label_bg = "black",
  label_bg_r = 0.1,
  arrow = ggplot2::arrow(type = "closed", length = ggplot2::unit(0.1, "inches")),
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  aspect.ratio = 1,
  theme = "theme_this",
  theme_args = list(),
  legend.position = "right",
  legend.direction = "vertical",
  ...
)
}
\arguments{
\item{links}{A data frame containing the edge list. Must contain the
\code{from} and \code{to} columns specifying source and target node
identifiers. Additional columns can be referenced by other parameters
(e.g., \code{link_weight_by}, \code{link_type_by},
\code{link_color_by}).}

\item{nodes}{An optional data frame of node metadata. When provided,
columns such as \code{node_size_by}, \code{node_color_by},
\code{node_shape_by}, and \code{node_fill_by} can reference its
columns. When \code{NULL}, the node set is inferred from the unique
values in the \code{from} and \code{to} columns. If a single character
string starting with \code{"@"}, the nodes data frame is extracted
from the corresponding attribute of \code{links} (e.g.
\code{"@nodes"} extracts \code{attr(links, "nodes")}).}

\item{from}{A character string specifying the column name in
\code{links} for the source node identifiers. Defaults to
\code{"from"}, or the first column of \code{links} if that column
name does not exist. Multiple columns can be provided; they are
concatenated with \code{from_sep}.}

\item{from_sep}{A character string to join multiple \code{from}
columns. Default \code{"_"}. Ignored when \code{from} is a single
column.}

\item{to}{A character string specifying the column name in
\code{links} for the target node identifiers. Defaults to
\code{"to"}, or the second column of \code{links} if that column
name does not exist. Multiple columns can be provided; they are
concatenated with \code{to_sep}.}

\item{to_sep}{A character string to join multiple \code{to} columns.
Default \code{"_"}. Ignored when \code{to} is a single column.}

\item{node_by}{A character string specifying the column name in
\code{nodes} for the node identifiers. These must match the values
in the \code{from} / \code{to} columns of \code{links}. Defaults to
\code{"name"}, or the first column of \code{nodes} if that column
name does not exist. Multiple columns can be provided; they are
concatenated with \code{node_by_sep}.}

\item{node_by_sep}{A character string to join multiple
\code{node_by} columns. Default \code{"_"}. Ignored when
\code{node_by} is a single column.}

\item{link_weight_by}{A numeric value or a character string. If
numeric, all edges receive that constant line width. If a column
name, the edge line width is mapped to that column. Default
\code{2}.}

\item{link_weight_name}{A character string for the link weight legend
title. When \code{NULL} (default), the column name from
\code{link_weight_by} is used. Only relevant when
\code{link_weight_by} is a column name.}

\item{link_type_by}{A character string or a column name specifying
the edge linetype. Can be \code{"solid"}, \code{"dashed"},
\code{"dotted"}, etc. If a column name from \code{links} is
supplied, the linetype is mapped to that column (with a version
check for ggplot2 4.0.0, where mapping is unsupported and a
warning is issued). Default \code{"solid"}.}

\item{link_type_name}{A character string for the link linetype legend
title. When \code{NULL} (default), the column name from
\code{link_type_by} is used. Only relevant when
\code{link_type_by} is a column name.}

\item{node_size_by}{A numeric value or a character string. If
numeric, all nodes receive that constant point size. If a column
name, the size is mapped to that column. Default \code{15}.}

\item{node_size_name}{A character string for the node size legend
title. When \code{NULL} (default), the column name from
\code{node_size_by} is used. Only relevant when
\code{node_size_by} is a column name.}

\item{node_color_by}{A character string specifying the node colour.
If a colour name or hex code (e.g. \code{"black"}), all nodes
receive that constant colour. If a column name from \code{nodes} is
supplied, the colour is mapped to that column. Default
\code{"black"}.}

\item{node_color_name}{A character string for the node colour legend
title. When \code{NULL} (default), the column name from
\code{node_color_by} is used. Only relevant when
\code{node_color_by} is a column name.}

\item{node_shape_by}{A numeric value or a character string. If
numeric, all nodes receive that constant shape (see
\code{\link[ggplot2]{shape}}). If a column name, the shape is
mapped to that column (cast to factor). Default \code{21} (filled
circle with border).}

\item{node_shape_name}{A character string for the node shape legend
title. When \code{NULL} (default), the column name from
\code{node_shape_by} is used. Only relevant when
\code{node_shape_by} is a column name.}

\item{node_fill_by}{A character string specifying the node fill
colour. If a colour name or hex code (e.g. \code{"grey20"}), all
nodes receive that constant fill. If a column name from
\code{nodes} is supplied, the fill is mapped to that column.
Default \code{"grey20"}.}

\item{node_fill_name}{A character string for the node fill legend
title. When \code{NULL} (default), the column name from
\code{node_fill_by} is used. Only relevant when
\code{node_fill_by} is a column name.}

\item{link_alpha}{A numeric value specifying the transparency
(alpha) of the edge lines. Between \code{0} (invisible) and
\code{1} (opaque). Default \code{1}.}

\item{node_alpha}{A numeric value specifying the fill transparency
of the nodes. Only applies when \code{node_shape_by} is one of the
filled shapes (21--25). Default \code{0.95}.}

\item{node_stroke}{A numeric value specifying the border stroke
width of the node points. Default \code{1.5}.}

\item{cluster_scale}{A character string specifying which node
aesthetic is overridden by cluster membership. One of
\code{"fill"}, \code{"color"}, or \code{"shape"}. The value is
matched via \code{\link{match.arg}}; default is \code{"fill"}.}

\item{node_size_range}{A numeric vector of length 2 giving the
minimum and maximum node size (in ggplot2 point units) when
\code{node_size_by} is a column name. Default \code{c(5, 20)}.}

\item{link_weight_range}{A numeric vector of length 2 giving the
minimum and maximum edge line width (in mm) when
\code{link_weight_by} is a column name. Default
\code{c(0.5, 5)}.}

\item{link_arrow_offset}{A numeric value (in points) specifying the
offset distance for the arrow end cap from the target node.
Prevents arrow heads from overlapping the node points. Only
relevant when \code{directed = TRUE}. Default \code{20}.}

\item{link_curvature}{A numeric value controlling the curvature of
the edges. \code{0} (default) produces straight edges; positive
values curve them away from the direct path.}

\item{link_color_by}{A character string controlling how edge colour
is determined. Options:
\itemize{
\item \code{"from"} (default) -- colour follows the source node's
fill or colour aesthetic.
\item \code{"to"} -- colour follows the target node's fill or
colour.
\item A column name from \code{links} -- colour is mapped
directly to that column.
}}

\item{link_color_name}{A character string for the edge colour legend
title. Only used when \code{link_color_by} is a column name (not
\code{"from"} or \code{"to"}). When \code{NULL} (default), the
column name is used.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{link_palette}{A character string specifying the palette for
edge colours when they are mapped. When \code{link_color_by} is
\code{"from"} or \code{"to"}, defaults to the node
\code{palette}. Otherwise defaults to \code{"Set1"}.}

\item{link_palcolor}{A character vector specifying custom colours
for the edge palette. When \code{link_color_by} is \code{"from"}
or \code{"to"}, defaults to the node \code{palcolor}. Otherwise
defaults to \code{NULL}.}

\item{directed}{A logical value. When \code{TRUE}, edges are drawn
with arrow heads and an end-cap offset. Default \code{TRUE}.}

\item{layout}{A character string or an \code{igraph_layout_spec}
object specifying the node placement algorithm. Built-in shortcuts:
\code{"circle"} (circular layout), \code{"tree"} (hierarchical
tree), \code{"grid"} (grid layout). Any other string is prefixed
with \code{"layout_with_"} and called as an igraph function (e.g.
\code{"fr"} for Fruchterman--Reingold, \code{"kk"} for
Kamada--Kawai). Default \code{"circle"}.}

\item{cluster}{A character string specifying the community detection
algorithm. One of \code{"none"}, \code{"fast_greedy"},
\code{"walktrap"}, \code{"edge_betweenness"}, \code{"infomap"}, or
a custom clustering function from igraph. When not \code{"none"},
cluster membership overrides the aesthetic selected by
\code{cluster_scale}. Default \code{"none"}.}

\item{add_mark}{A logical value. When \code{TRUE} (and
\code{cluster != "none"}), an enclosure mark is drawn around each
cluster's nodes. Default \code{FALSE}.}

\item{mark_expand}{A \code{\link[grid]{unit}} object specifying the
extra space around points within a cluster mark. Default
\code{unit(10, "mm")}.}

\item{mark_type}{A character string specifying the mark geometry.
One of \code{"hull"}, \code{"ellipse"}, \code{"rect"}, or
\code{"circle"}, corresponding to ggforce's \code{geom_mark_hull},
\code{geom_mark_ellipse}, \code{geom_mark_rect}, and
\code{geom_mark_circle}. The value is matched via
\code{\link{match.arg}}; default is \code{"hull"}.}

\item{mark_alpha}{A numeric value for the fill transparency of
cluster marks. Default \code{0.1}.}

\item{mark_linetype}{A numeric or character value specifying the
border line type of the cluster marks. Default \code{1} (solid).}

\item{add_label}{A logical value. When \code{TRUE} (default), node
identifiers are drawn as repulsive text labels via
\code{\link[ggrepel]{geom_text_repel}}.}

\item{label_size}{A numeric value for the font size of node labels.
Scaled by the theme base size. Default \code{3}.}

\item{label_fg}{A character string specifying the text colour of
node labels. Default \code{"white"}.}

\item{label_bg}{A character string specifying the background colour
of node labels. Default \code{"black"}.}

\item{label_bg_r}{A numeric value specifying the background box
radius (as a fraction of label height). Passed to
\code{\link[ggrepel]{geom_text_repel}}'s \code{bg.r} argument.
Default \code{0.1}.}

\item{arrow}{A \code{\link[ggplot2]{arrow}} object for the link
arrow heads. Only used when \code{directed = TRUE}. Default is
\code{arrow(type = "closed", length = unit(0.1, "inches"))}.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{...}{Not used.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches).
}
\description{
Core implementation for rendering a single network graph from a links
(edge list) data frame and an optional nodes (vertex metadata) data frame.
This is the workhorse behind the exported \code{\link{Network}} function --
it takes a \strong{single} pair of links/nodes data frames (no
\code{split_by} support) and returns a \code{ggplot} object.

The graph is constructed via \code{\link[igraph]{graph_from_data_frame}},
laid out using igraph layout algorithms (\code{"circle"}, \code{"tree"},
\code{"grid"}, or any named igraph layout such as \code{"fr"} or
\code{"kk"}), and rendered with \code{\link[ggraph]{ggraph}} using
\code{\link[ggraph]{geom_edge_arc}} for links and
\code{\link[ggplot2]{geom_point}} for nodes.

Key features include:
\itemize{
\item \strong{Directed / undirected graphs} with optional arrow heads.
\item \strong{Link styling}: variable width (weight), linetype, and
colour, each constant or mapped from a column. Edge colours
can follow source nodes, target nodes, or a dedicated column.
\item \strong{Node styling}: variable size, shape, colour, and fill,
each constant or column-mapped.
\item \strong{Community detection} via igraph clustering algorithms
with convex hull, ellipse, rectangle, or circle marks from
\code{\link[ggforce]{ggforce}}.
\item \strong{Automatic labels} via
\code{\link[ggrepel]{geom_text_repel}}.
\item \strong{Self-loop edges} drawn as direction-sensitive arcs.
}
}
\section{Architecture}{

\enumerate{
\item \strong{Column resolution} -- Default column names for
\code{from}, \code{to}, and \code{node_by} are assigned when
\code{NULL}, using the first or second column of the respective
data frame. Each is validated via \code{\link{check_columns}};
multi-column inputs are concatenated with their respective
separator (\code{from_sep}, \code{to_sep}, \code{node_by_sep})
and force-converted to factors. Resolved columns are renamed to
the canonical names \code{"from"}, \code{"to"}, and \code{"name"}.
\item \strong{Graph construction} -- An \code{\link[igraph]{igraph}}
object is built via \code{\link[igraph]{graph_from_data_frame}}
using the links data frame and (optionally) the nodes data frame.
If \code{nodes} is a string starting with \code{"@"}, it is
extracted from the corresponding attribute on the links object.
\item \strong{Layout computation} -- The layout is resolved:
\code{"circle"} -> \code{\link[igraph]{layout_in_circle}},
\code{"tree"} -> \code{\link[igraph]{layout_as_tree}},
\code{"grid"} -> \code{\link[igraph]{layout_on_grid}}.
Any other character string is prefixed with \code{"layout_with_"}
and looked up in the igraph namespace (e.g. \code{"fr"} ->
\code{layout_with_fr}). If \code{layout} is already an
\code{igraph_layout_spec} object it is applied directly via
\code{\link[igraph]{layout_}}.
\item \strong{Data extraction} -- Vertex and edge data frames are
extracted from the igraph object via
\code{\link[igraph]{as_data_frame}}. Layout coordinates are
added as \code{x} and \code{y} columns to the vertex data.
\item \strong{Node aesthetic assembly} -- Each of \code{size},
\code{color}, \code{shape}, and \code{fill} is resolved as
either a constant value (numeric or character) or a mapping to
a column of the node data. A per-aesthetic \code{"guide"} /
\code{"none"} flag is tracked for legend construction.
\item \strong{Link aesthetic assembly} -- \code{linewidth} (weight),
\code{linetype}, and \code{color} are resolved similarly.
\itemize{
\item When \code{link_color_by = "from"}, edge colours are
derived from the source node's fill (if node shape is
filled, 21--25) or colour; when \code{"to"}, from the
target node.
\item When \code{link_color_by} names a column in the links
data frame (other than \code{"from"}/\code{"to"}), it
is mapped directly.
}
If \code{directed = TRUE}, a \code{\link[ggplot2]{arrow}} is
added and \code{end_cap} is set via \code{\link[ggraph]{circle}}
to prevent arrow overlap with nodes. Self-loop edges receive a
\code{direction} aesthetic.
\item \strong{Plot initialisation} -- A \code{\link[ggraph]{ggraph}}
plot is created with \code{layout = "manual"} and vertex
coordinates from the layout.
\item \strong{Clustering and marks} -- When \code{cluster != "none"},
the specified igraph community-detection algorithm is run
(\code{cluster_fast_greedy}, \code{cluster_walktrap},
\code{cluster_edge_betweenness}, \code{cluster_infomap}, or a
custom function). Membership overrides the aesthetic specified
by \code{cluster_scale} (\code{"fill"}, \code{"color"}, or
\code{"shape"}) with a warning that the previous setting is
discarded. If \code{add_mark = TRUE}, a mark enclosure
(hull / ellipse / rect / circle) is drawn per cluster via the
corresponding \code{\link[ggforce]{ggforce}} geom.
\item \strong{Link rendering} -- \code{\link[ggraph]{geom_edge_arc}}
draws the edges (with configurable \code{link_curvature}) and
\code{\link[ggraph]{geom_edge_loop}} handles self-loop edges.
\item \strong{Link scales} -- Conditional on their guide status,
\code{scale_edge_width_continuous},
\code{scale_edge_linetype_discrete}, and
\code{scale_edge_color_manual} /
\code{scale_edge_color_gradientn} are added for weight,
linetype, and colour legends respectively.
\item \strong{Node rendering} -- \code{\link[ggplot2]{geom_point}}
draws the nodes with the assembled aesthetics.
\item \strong{Node scales} -- Conditional scale additions:
\code{scale_size_continuous} (range from
\code{node_size_range}), \code{scale_color_manual},
\code{scale_shape_manual}, and \code{scale_fill_manual}, each
with their legend title and guide overrides.
\item \strong{Labels} -- When \code{add_label = TRUE}, node
identifiers are rendered via
\code{\link[ggrepel]{geom_text_repel}} using
\code{label_fg}, \code{label_bg}, and \code{label_bg_r}.
\item \strong{Final theme and dimensions} -- Coordinate expansion,
axis labels, theme application, and legend positioning are
applied. Plot height and width are computed via
\code{\link{calculate_plot_dimensions}} and stored as
attributes.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scatterplot.R
\name{ScatterPlot}
\alias{ScatterPlot}
\title{Scatter Plot}
\usage{
ScatterPlot(
  data,
  x,
  y,
  size_by = 2,
  size_name = NULL,
  color_by = NULL,
  color_name = NULL,
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  palreverse = FALSE,
  split_by = NULL,
  split_by_sep = "_",
  shape = 21,
  alpha = ifelse(shape \%in\% 21:25, 0.65, 1),
  border_color = "black",
  border_size = 0.5,
  highlight = NULL,
  highlight_shape = 16,
  highlight_size = 3,
  highlight_color = "red",
  highlight_alpha = 1,
  theme = "theme_this",
  theme_args = list(),
  palette = ifelse(!is.null(color_by) && !is.numeric(data[[color_by]]), "Paired",
    "Spectral"),
  palcolor = NULL,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  seed = 8525,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis.}

\item{y}{A character string specifying the column name of the data frame to plot for the y-axis.}

\item{size_by}{Either a numeric constant (uniform dot size) or a character
string naming a numeric column whose values control dot size via
\code{scale_size_area(max_size = 6)}. Default: \code{2}.}

\item{size_name}{A character string for the size legend title. When
\code{NULL} (default), the \code{size_by} column name is used. Ignored when
\code{size_by} is a numeric constant.}

\item{color_by}{A character string naming a column whose values control
dot colour. Can be numeric (continuous gradient via
\code{scale_fill_gradientn()} / \code{scale_color_gradientn()}) or
factor/character (discrete palette via
\code{scale_fill_manual()} / \code{scale_color_manual()}). For shapes
21--25, the colour is applied to the fill aesthetic. When \code{NULL}
(default), all dots are rendered in a single colour derived from the
palette.}

\item{color_name}{A character string for the colour legend title. When
\code{NULL} (default), the \code{color_by} column name is used.}

\item{lower_quantile, upper_quantile}{Lower and upper quantiles for the continuous color/fill scale.
The actual cutoffs are determined by these quantiles when \code{lower_cutoff} and
\code{upper_cutoff} are \code{NULL}. Defaults: \code{lower_quantile = 0}, \code{upper_quantile = 0.99}.}

\item{lower_cutoff, upper_cutoff}{Explicit lower and upper cutoffs for the continuous color/fill scale.
When \code{NULL} (the default), the cutoffs are determined by \code{lower_quantile} and
\code{upper_quantile} via \code{\link[stats]{quantile}}. Values outside the
\code{[lower_cutoff, upper_cutoff]} range are clamped (winsorized) to the nearest cutoff value.}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{split_by}{The column(s) to split data by and generate separate scatter
plots for each level. The split column is processed before splitting;
multiple columns are concatenated with \code{split_by_sep}.}

\item{split_by_sep}{A character string used to concatenate multiple
\code{split_by} column values. Default: \code{"_"}.}

\item{shape}{A numeric value specifying the point shape. Default: \code{21}
(filled circle with border). Shapes 21--25 support separate fill and border
colour aesthetics; all other shapes use a single colour aesthetic.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{border_color}{Controls the point border colour. For shapes 21--25:
\itemize{
\item \code{"black"} (default) -- constant black border.
\item A colour string (e.g. \code{"red"}, \code{"#FF0000"}) -- constant
colour border.
\item \code{TRUE} -- border colour tracks the \code{color_by} gradient
/ palette via \code{scale_color_gradientn()} /
\code{scale_color_manual()}.
}
For shapes without a fill aesthetic (not 21--25), this parameter has no
effect.}

\item{border_size}{A numeric value specifying the point border size. Default: \code{0.5}.}

\item{highlight}{Specifies which points to highlight with an overlaid
\code{geom_point()} layer. Accepted values:
\itemize{
\item \code{NULL} (default) -- no highlighting.
\item \code{TRUE} -- all points are highlighted.
\item A numeric vector -- row indices of points to highlight.
\item A single character string -- an R expression (e.g.
\code{"x > 0"}) that is parsed with \code{rlang::parse_expr()} and
evaluated via \code{\link[dplyr]{filter}()} to select rows.
\item A character vector -- rownames of points to highlight. An error
is thrown if the data has no rownames.
}}

\item{highlight_shape}{A numeric value specifying the point shape for
highlighted points. Default: \code{16} (filled circle). Shapes 21--25
use the \code{fill} aesthetic; other shapes use \code{color}.}

\item{highlight_size}{A numeric value specifying the size of highlighted
points. Default: \code{3}.}

\item{highlight_color}{A character string specifying the colour of
highlighted points. Default: \code{"red"}.}

\item{highlight_alpha}{A numeric value in \code{[0, 1]} specifying the
transparency of highlighted points. Default: \code{1}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{combine}{A logical value. If \code{TRUE} (the default), the list of
per-split plots is combined into a single \code{patchwork} object via
\code{\link{combine_plots}()}. If \code{FALSE}, returns the raw list of
\code{ggplot} objects.}

\item{nrow, ncol, byrow}{Integers controlling the layout of combined plots via
\code{\link{combine_plots}()}. \code{byrow = TRUE} (default) fills the
layout row-wise.}

\item{seed}{The random seed for reproducibility. Passed to
\code{\link{validate_common_args}()}. Default: \code{8525}.}

\item{axes, axis_titles}{Strings controlling how axes and axis titles are
handled across combined plots. Passed to \code{\link{combine_plots}()}.
See \code{?patchwork::wrap_plots} for options (\code{"keep"},
\code{"collect"}, \code{"collect_x"}, \code{"collect_y"}).}

\item{guides}{A string controlling guide collection across combined plots.
Passed to \code{\link{combine_plots}()}.}

\item{design}{A custom layout specification for combined plots. Passed to
\code{\link{combine_plots}()}. When specified, \code{nrow}, \code{ncol},
and \code{byrow} are ignored.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object (single plot), a \code{patchwork} object
(when \code{combine = TRUE} with \code{split_by}), or a named list of
\code{ggplot} objects (when \code{combine = FALSE}), each with
\code{height} and \code{width} attributes in inches.
}
\description{
Draws a scatter plot with optional size encoding, colour encoding
(continuous gradient or discrete palette), point highlighting, and axis
transformations. This is the user-facing wrapper around
\code{\link{ScatterPlotAtomic}} that adds \code{split_by} support
(generating separate sub-plots per group) and combines them via
\code{patchwork}.

Key features:
\itemize{
\item \strong{Variable point size} -- \code{size_by} accepts either a
numeric constant or a column name.
\item \strong{Colour modes} -- numeric \code{color_by} produces a
continuous gradient; factor/character \code{color_by} produces a discrete
palette.
\item \strong{Colour scale trimming} -- \code{lower_quantile} /
\code{upper_quantile} (or explicit \code{lower_cutoff} /
\code{upper_cutoff}) trim/clamp continuous colour scale extremes.
\item \strong{Border modes} -- \code{border_color} can be a constant
colour, \code{TRUE} (track the fill gradient), or omitted.
\item \strong{Point highlighting} -- \code{highlight} accepts indices,
rownames, logical \code{TRUE}, or a string expression.
\item \strong{Axis transformation} -- \code{xtrans} / \code{ytrans}
support log, sqrt, and other scale transformations.
\item \strong{Split sub-plots} -- \code{split_by} produces one scatter
plot per group level, combined into a single \code{patchwork} layout.
}
}
\section{split_by Workflow}{

When \code{split_by} is provided:
\enumerate{
\item \strong{Seed validation} -- \code{\link{validate_common_args}()}
sets the random seed for reproducibility.
\item \strong{Theme resolution} -- \code{\link{process_theme}()} resolves
the \code{theme} string or function.
\item \strong{Split column resolution} -- \code{\link{check_columns}()}
validates \code{split_by} (force_factor, allow_multi, concat_multi).
\item \strong{Data splitting} -- unused factor levels are dropped and the
data is split into a named list (preserving factor level order). When
\code{split_by = NULL}, a single-element list named \code{"..."} is used.
\item \strong{Per-split palette / colour} -- \code{\link{check_palette}()}
and \code{\link{check_palcolor}()} resolve per-split palette and colour
overrides.
\item \strong{Per-split legend} -- \code{\link{check_legend}()} resolves
\code{legend.position} and \code{legend.direction} per split.
\item \strong{Per-split title} -- when \code{title} is a function, it
receives the default title (the split level name) and can return a custom
string; otherwise \code{title \%||\% split_level} is used.
\item \strong{Dispatch} -- each split subset is passed to
\code{\link{ScatterPlotAtomic}()}.
\item \strong{Combination} -- \code{\link{combine_plots}()} assembles the
list of plots via \code{patchwork::wrap_plots}, honouring
\code{nrow}/\code{ncol}/\code{byrow}/\code{design}.
}
}

\examples{
set.seed(8525)

data <- data.frame(
   x = rnorm(20),
   y = rnorm(20),
   w = abs(rnorm(20)),
   t = sample(c("A", "B"), 20, replace = TRUE)
)

# --- Basic scatter plot ---
ScatterPlot(data, x = "x", y = "y")

# --- Highlight points ---
ScatterPlot(data, x = "x", y = "y", highlight = 'x > 0')

# --- Size encoding (column name) ---
ScatterPlot(data, x = "x", y = "y", size_by = "w")

# --- Colour encoding (numeric gradient) ---
ScatterPlot(data, x = "x", y = "y", color_by = "w")

# --- Colour encoding (categorical) with border ---
ScatterPlot(data, x = "x", y = "y", size_by = "w", color_by = "t",
 border_color = "red")

# --- Border colour tracks fill gradient ---
ScatterPlot(data, x = "x", y = "y", size_by = "w", color_by = "t",
 border_color = TRUE)

# --- Shape without fill (single colour aesthetic) ---
ScatterPlot(data, x = "x", y = "y", size_by = "w", color_by = "t",
 shape = 1, palette = "Set1")

# --- split_by with per-split palcolor ---
ScatterPlot(data, x = "x", y = "y", split_by = "t",
            palcolor = list(A = "blue", B = "red"))

# --- Colour scale limits (quantile-based) ---
ScatterPlot(data, x = "x", y = "y", color_by = "w",
            lower_quantile = 0.1, upper_quantile = 0.9)

# --- Colour scale limits (explicit cutoffs) ---
ScatterPlot(data, x = "x", y = "y", color_by = "w",
            lower_cutoff = 0, upper_cutoff = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap-utils.R
\name{process_heatmap_data}
\alias{process_heatmap_data}
\title{Process/normalize data passed to \code{\link[=Heatmap]{Heatmap()}}}
\usage{
process_heatmap_data(
  data,
  in_form,
  values_by,
  name,
  split_by,
  split_by_sep,
  rows_orderby,
  columns_orderby,
  rows_by,
  rows_by_sep,
  rows_name,
  rows_split_by,
  rows_split_by_sep,
  rows_split_name,
  columns_by,
  columns_by_sep,
  columns_name,
  columns_split_by,
  columns_split_by_sep,
  columns_split_name,
  pie_group_by,
  pie_group_by_sep,
  pie_name,
  rows_data,
  columns_data,
  keep_na,
  keep_empty
)
}
\arguments{
\item{data}{A data frame or matrix containing the data to be plotted.
Based on the \code{in_form}, the data can have the following formats:
\itemize{
\item \code{matrix}: A matrix with rows and columns directly representing the heatmap.
\item \code{long}: A data frame in long format with columns for values, rows, and columns.
\item \code{wide-rows}: A data frame in wide format with columns for heatmap rows and values,
and a single column for heatmap columns.
\item \code{wide-columns}: A data frame in wide format with columns for heatmap columns and values,
and a single column for heatmap rows.
\item \code{auto}: Automatically inferred from the data format.
When \code{data} is a matrix, \code{in_form} is set to \code{"matrix"}. When \code{columns_by} has more than one column,
\code{in_form} is set to \code{"wide-columns"}. When \code{rows_by} has more than one column,
\code{in_form} is set to \code{"wide-rows"}. Otherwise, it is set to \code{"long"}.
}}

\item{in_form}{The format of the data. Can be one of \code{"matrix"}, \code{"long"}, \code{"wide-rows"}, \code{"wide-columns"}, or \code{"auto"}.
Defaults to \code{"auto"}.}

\item{values_by}{A character of column name in \code{data} that contains the values to be plotted.
This is required when \code{in_form} is \code{"long"}. For other formats, the values are pivoted into a column named by \code{values_by}.}

\item{name}{A character string to name the heatmap (will be used to rename \code{values_by}).}

\item{split_by}{A character of column name in \code{data} that contains the split information to split into multiple heatmaps.
This is used to create a list of heatmaps, one for each level of the split.
Defaults to \code{NULL}, meaning no split.}

\item{split_by_sep}{A character string to concat multiple columns in \code{split_by}.}

\item{rows_orderby}{A expression (in character) to specify how to order rows. It will be evaluated in the context of the data frame used for rows (after grouping by rows_split_by and rows_by). The expression should return a vector of the same length as the number of rows in the data frame. The default is NULL, which means no specific ordering.
Can't be used with cluster_rows = TRUE.
This is applied before renaming rows_by to rows_name.}

\item{columns_orderby}{A expression (in character) to specify how to order columns. It will be evaluated in the context of the data frame used for columns (after grouping by columns
split_by and columns_by). The expression should return a vector of the same length as the number of rows in the data frame. The default is NULL, which means no specific ordering.
Can't be used with cluster_columns = TRUE.
This is applied before renaming columns_by to columns_name.}

\item{rows_by}{A vector of column names in \code{data} that contains the row information.
This is used to create the rows of the heatmap.
When \code{in_form} is \code{"long"} or \code{"wide-columns"}, this is requied, and multiple columns can be specified,
which will be concatenated by \code{rows_by_sep} into a single column.}

\item{rows_by_sep}{A character string to concat multiple columns in \code{rows_by}.}

\item{rows_name}{A character string to rename the column created by \code{rows_by}, which will be reflected in the name of the annotation or legend.}

\item{rows_split_by}{A character of column name in \code{data} that contains the split information for rows.}

\item{rows_split_by_sep}{A character string to concat multiple columns in \code{rows_split_by}.}

\item{rows_split_name}{A character string to rename the column created by \code{rows_split_by}, which will be reflected in the name of the annotation or legend.}

\item{columns_by}{A vector of column names in \code{data} that contains the column information.
This is used to create the columns of the heatmap.
When \code{in_form} is \code{"long"} or \code{"wide-rows"}, this is required, and multiple columns can be specified,
which will be concatenated by \code{columns_by_sep} into a single column.}

\item{columns_by_sep}{A character string to concat multiple columns in \code{columns_by}.}

\item{columns_name}{A character string to rename the column created by \code{columns_by}, which will be reflected in the name of the annotation or legend.}

\item{columns_split_by}{A character of column name in \code{data} that contains the split information for columns.}

\item{columns_split_by_sep}{A character string to concat multiple columns in \code{columns_split_by}.}

\item{columns_split_name}{A character string to rename the column created by \code{columns_split_by}, which will be reflected in the name of the annotation or legend.}

\item{pie_group_by}{A character of column name in \code{data} that contains the group information for pie charts.
This is used to create pie charts in the heatmap when \code{cell_type} is \code{"pie"}.}

\item{pie_group_by_sep}{A character string to concat multiple columns in \code{pie_group_by}.}

\item{pie_name}{A character string to rename the column created by \code{pie_group_by}, which will be reflected in the name of the annotation or legend.}

\item{rows_data}{A data frame containing additional data for rows, which can be used to add annotations to the heatmap.
It will be joined to the main data by \code{rows_by} and \code{split_by} if \code{split_by} exists in \code{rows_data}.
This is useful for adding additional information to the rows of the heatmap.}

\item{columns_data}{A data frame containing additional data for columns, which can be used to add annotations to the heatmap.
It will be joined to the main data by \code{columns_by} and \code{split_by} if \code{split_by} exists in \code{columns_data}.
This is useful for adding additional information to the columns of the heatmap.}

\item{keep_na}{Whether we should keep NA groups in rows, columns and split_by variables. Default is FALSE.
FALSE to remove NA groups; TRUE to keep NA groups.
A vector of column names can also be provided to specify which columns to keep NA groups.
Note that the record will be removed if any of the grouping columns has NA and is not specified to keep NA.}
}
\value{
A list containing the processed data and metadata:
\itemize{
\item \code{data}: A list of data frames, one for each level of \code{split_by}. If no \code{split_by} is provided, the name will be \code{"..."}.
Each data frame is in the long format.
\item \code{values_by}: The name of the column containing the values to be plotted.
\item \code{rows_by}: The name of the column containing the row information.
\item \code{rows_split_by}: The name of the column containing the row split information.
\item \code{columns_by}: The name of the column containing the column information.
\item \code{columns_split_by}: The name of the column containing the column split information.
\item \code{pie_group_by}: The name of the column containing the pie group information.
}
}
\description{
This function is used to process the data passed to \code{\link[=Heatmap]{Heatmap()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sankeyplot.R
\name{SankeyPlotAtomic}
\alias{SankeyPlotAtomic}
\title{Atomic Sankey / alluvial diagram (internal)}
\usage{
SankeyPlotAtomic(
  data,
  in_form = c("auto", "long", "lodes", "wide", "alluvia", "counts"),
  x,
  x_sep = "_",
  y = NULL,
  stratum = NULL,
  stratum_sep = "_",
  alluvium = NULL,
  alluvium_sep = "_",
  flow = FALSE,
  nodes_color = "grey30",
  links_fill_by = NULL,
  links_fill_by_sep = "_",
  links_name = NULL,
  links_color = "gray80",
  nodes_palette = "Paired",
  nodes_palcolor = NULL,
  palreverse = FALSE,
  nodes_alpha = 1,
  nodes_label = FALSE,
  nodes_width = 0.25,
  nodes_label_miny = 0,
  nodes_legend = c("auto", "separate", "merge", "none"),
  expand = c(0, 0, 0, 0),
  links_palette = "Paired",
  links_palcolor = NULL,
  links_alpha = 0.6,
  legend.box = "vertical",
  keep_empty = TRUE,
  x_text_angle = 0,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  flip = FALSE,
  theme = "theme_this",
  theme_args = list(),
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{in_form}{A character string specifying the input data format.
One of \code{"auto"} (default), \code{"long"}, \code{"lodes"},
\code{"wide"}, \code{"alluvia"}, or \code{"counts"}.
\code{"long"} is an alias for \code{"lodes"}; \code{"wide"} is an alias for
\code{"alluvia"}.  See the \code{data} parameter of \code{\link{SankeyPlot}}
for format descriptions.}

\item{x}{A character vector of column name(s) for the x-axis categories.
Each unique value or concatenated pair represents a time point, state, or
position along the horizontal axis.  Behaviour depends on \code{in_form}:
for \code{"lodes"} at least one column is expected; for \code{"alluvia"}
and \code{"counts"} at least two columns are required.  In the latter two
cases \code{x_sep} is not used.}

\item{x_sep}{A character string to join multiple \code{x} columns when
\code{in_form} is \code{"lodes"} or auto-determined as lodes.
Default \code{"_"}.}

\item{y}{A character string specifying the numeric column for the y-axis
(frequency / value).  When \code{NULL} (default), the count of observations
per combination of \code{x}, \code{stratum}, \code{alluvium},
\code{links_fill_by}, and \code{facet_by} is computed automatically.
Ignored when \code{in_form} is \code{"counts"}.}

\item{stratum}{A character string specifying the column that defines the
node categories at each x-axis position.  Each unique value becomes a
stratum (node block) at each x position.  When \code{NULL}, defaults to
\code{links_fill_by}.  Multiple columns are concatenated with
\code{stratum_sep}.  Ignored in \code{"alluvia"} format.}

\item{stratum_sep}{A character string to join multiple \code{stratum}
columns.  Default \code{"_"}.}

\item{alluvium}{A character string specifying the column that identifies
individual flows (alluvia) across x-axis positions.  Each unique value
represents a single observational unit tracked across positions.  When
\code{NULL} in \code{"counts"} format, an auto-generated identifier is
created.  Multiple columns are concatenated with \code{alluvium_sep}.
Ignored in \code{"alluvia"} format.}

\item{alluvium_sep}{A character string to join multiple \code{alluvium}
columns.  Default \code{"_"}.}

\item{flow}{A logical value.  When \code{FALSE} (default),
\code{\link[ggalluvial]{geom_alluvium}()} is used for the links.  When
\code{TRUE}, \code{\link[ggalluvial]{geom_flow}()} is used instead, which
draws the flows with a directional gradient between x positions.}

\item{nodes_color}{A character string specifying the border colour of the
node (stratum) rectangles.  Use the special value \code{".fill"} to match
the border colour to the node fill colour.  Default \code{"grey30"}.}

\item{links_fill_by}{A character string specifying the column that
determines the fill colour of the links (alluvia / flows).  When
\code{NULL} in \code{"lodes"} format, defaults to \code{alluvium}.  In
\code{"counts"} format with the \code{"."} prefix, this parameter is
required.  Multiple columns are concatenated with
\code{links_fill_by_sep}.}

\item{links_fill_by_sep}{A character string to join multiple
\code{links_fill_by} columns.  Default \code{"_"}.}

\item{links_name}{A character string for the legend title of the link fill
scale.  When \code{NULL} (default), the \code{links_fill_by} column name
is used.}

\item{links_color}{A character string specifying the border colour of the
links (alluvia / flows).  Use the special value \code{".fill"} to match
the link border colour to the link fill colour.  Default \code{"gray80"}.}

\item{nodes_palette}{A character string specifying the colour palette for
the node (stratum) fill.  Passed to \code{\link{palette_this}()}.
Default \code{"Paired"}.}

\item{nodes_palcolor}{A character vector of custom colours for the node
fill, used as \code{palcolor} in \code{\link{palette_this}()}.  When
\code{NULL} (default), the palette colours are used directly.}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{nodes_alpha}{A numeric value in \eqn{[0, 1]} controlling the
transparency of the node (stratum) fill.  Default \code{1}.}

\item{nodes_label}{A logical value.  When \code{TRUE}, stratum labels are
drawn inside each node using \code{\link[ggplot2]{geom_label}()} with
\code{\link[ggalluvial]{StatStratum}}.  Default \code{FALSE}.}

\item{nodes_width}{A numeric value (typically 0–1) specifying the width of
the node (stratum) rectangles as a fraction of the x-axis spacing.
Default \code{0.25}.}

\item{nodes_label_miny}{A numeric value specifying the minimum y
(frequency) threshold for displaying node labels.  Nodes with y-values
below this threshold are not labelled.  Default \code{0}.}

\item{nodes_legend}{Controls how the node legend is displayed.  One of:
\describe{
\item{\code{"auto"} (default)}{Automatically determined:
if \code{nodes_label = TRUE}, or if \code{stratum} is identical to
\code{links_fill_by} with matching colours, the legend is hidden.
Otherwise, overlapping stratum values across x positions are checked:
any overlap produces a merged legend; no overlap produces separate
legends per x position.}
\item{\code{"merge"}}{A single merged legend for all nodes.}
\item{\code{"separate"}}{One legend per x-axis position, generated via
separate \code{\link[ggplot2]{scale_fill_manual}()} layers.}
\item{\code{"none"}}{No node legend is shown.}
}}

\item{expand}{The values to expand the x and y axes. It is like CSS padding.
When a single value is provided, it is used for both axes on both sides.
When two values are provided, the first value is used for the top/bottom side and the second value is used for the left/right side.
When three values are provided, the first value is used for the top side, the second value is used for the left/right side, and the third value is used for the bottom side.
When four values are provided, the values are used for the top, right, bottom, and left sides, respectively.
You can also use a named vector to specify the values for each side.
When the axis is discrete, the values will be applied as 'add' to the 'expansion' function.
When the axis is continuous, the values will be applied as 'mult' to the 'expansion' function.
See also \url{https://ggplot2.tidyverse.org/reference/expansion.html}}

\item{links_palette}{A character string specifying the colour palette for
the link fill.  Passed to \code{\link{palette_this}()}.
Default \code{"Paired"}.}

\item{links_palcolor}{A character vector of custom colours for the link
fill, used as \code{palcolor} in \code{\link{palette_this}()}.  When
\code{NULL} (default), the palette colours are used directly.}

\item{links_alpha}{A numeric value in \eqn{[0, 1]} controlling the
transparency of the link fill.  Default \code{0.6}.}

\item{legend.box}{A character string specifying the arrangement of legend
boxes, either \code{"vertical"} (default) or \code{"horizontal"}.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{flip}{A logical value.  When \code{TRUE},
\code{\link[ggplot2]{coord_flip}()} is applied to swap the x and y axes.
Default \code{FALSE}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{...}{Additional arguments passed to
\code{\link[ggalluvial]{geom_alluvium}()} or
\code{\link[ggalluvial]{geom_flow}()}, depending on the \code{flow}
setting.  For \code{geom_flow} with a distinct \code{links_fill_by}
column, passing \code{stat = "alluvium"} preserves the fill variable.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a Sankey (alluvial) diagram using
\code{\link[ggalluvial]{geom_alluvium}} (or \code{\link[ggalluvial]{geom_flow}})
and \code{\link[ggalluvial]{geom_stratum}}.  This function takes a \strong{single}
data frame (no \code{split_by} support) and returns a \code{ggplot} object
with faceting applied.

The function supports five input formats (\code{in_form}) which control how
the data columns are interpreted.  Nodes (strata) are rendered as vertical
blocks whose fill colour and alpha can be customised independently of the
links (alluvia / flows) connecting them.  Link colours can match node
colours or use a separate palette; link borders can be set to a fixed colour
or to follow the fill colour (\code{links_color = ".fill"}).

Automatic legend resolution determines whether nodes on different x-axis
positions receive a merged legend or separate legends, based on overlaps
between stratum values across positions.
}
\section{Architecture}{

\enumerate{
\item \strong{Format resolution} — \code{in_form} is matched and aliased
(\code{"long"} → \code{"lodes"}, \code{"wide"} → \code{"alluvia"}).
\item \strong{Data parsing by format} — one of five code paths executes:
\itemize{
\item \strong{Lodes / long} — \code{x}, \code{stratum}, \code{alluvium},
and \code{links_fill_by} are validated via
\code{\link{check_columns}()}.  Multi-column inputs are
concatenated with their respective separators.  When \code{y} is
\code{NULL}, counts are computed per (\code{x}, \code{stratum},
\code{alluvium}, \code{links_fill_by}, \code{facet_by}).
\item \strong{Counts} (x without \code{"."} prefix) — numeric \code{x}
columns are pivoted to long form via
\code{\link[tidyr]{pivot_longer}()}, creating the y-axis from the
cell values.  \code{alluvium} and \code{stratum} default to
\code{links_fill_by}.
\item \strong{Counts with source node} (x with \code{"."} as first
element, \code{is_flowcounts}) — same as counts but also injects
\code{links_fill_by} values as an additional first column of
nodes, visualising how flows originate from source groups.
\item \strong{Alluvia / wide} — validated via
\code{\link[ggalluvial]{is_alluvia_form}()} and converted to lodes
form via \code{\link[ggalluvial]{to_lodes_form}()}.
\code{stratum} and \code{alluvium} are ignored.
\item \strong{Auto / fallback} — when no other branch matches, the lodes
path is attempted and validated via
\code{\link[ggalluvial]{is_lodes_form}()}.
}
\item \strong{Palette assignment} — \code{\link{palette_this}()} resolves
colours for both nodes (\code{nodes_palette} / \code{nodes_palcolor})
and links (\code{links_palette} / \code{links_palcolor}).
\item \strong{Flow-counts guide logic} — when \code{is_flowcounts} is
\code{TRUE}, the links guide is suppressed if the first-column node
colours match the link colours, avoiding redundant legends.  When
the palettes are identical but colours differ (too few colours in
the palette), the first-column node colours are reused.
\item \strong{Legend auto-detection} — when \code{nodes_legend = "auto"}:
if \code{nodes_label = TRUE} or if \code{stratum} and
\code{links_fill_by} share identical values and colours, the nodes
legend is hidden.  Otherwise, overlapping stratum values across
x-axis positions are checked: any overlap triggers a merged legend;
no overlap produces separate legends per position.
\item \strong{Base ggplot} — constructed with \code{aes(x = x,
        stratum = stratum, alluvium = alluvium, y = y)}.
\item \strong{Separate node fill layers} — when \code{stratum} differs
from \code{links_fill_by} and \code{nodes_legend = "separate"}, a
\code{\link[ggplot2]{geom_col}()} +
\code{\link[ggplot2]{scale_fill_manual}()} layer pair is added per
x-axis position, each followed by
\code{\link[ggnewscale]{new_scale_fill}()} to produce independent
legends.
\item \strong{Link rendering} — \code{\link[ggalluvial]{geom_alluvium}()}
(default) or \code{\link[ggalluvial]{geom_flow}()} when
\code{flow = TRUE}.  When \code{links_color = ".fill"}, the colour
aesthetic is mapped to \code{links_fill_by} and the colour scale
guide is suppressed.  For \code{geom_flow} with a distinct
\code{stratum} and \code{links_fill_by}, \code{stat = "alluvium"}
is forced to preserve the fill variable through the flow stat
transformation.
\item \strong{Link fill scale} — \code{\link[ggplot2]{scale_fill_manual}()}
with \code{links_colors} and the resolved \code{links_guide}.
\item \strong{Node rendering} — \code{\link[ggalluvial]{geom_stratum}()}
with \code{nodes_color} (border) and \code{nodes_alpha}.  When
\code{nodes_color = ".fill"}, the colour aesthetic maps to
\code{stratum}.
\item \strong{Node fill scale} — \code{\link[ggplot2]{scale_fill_manual}()}
with \code{nodes_colors}.  The guide is \code{"none"} when
\code{nodes_legend} is \code{"none"} or \code{"separate"}
(already handled by per-position layers); otherwise \code{"legend"}.
\item \strong{Labels} — when \code{nodes_label = TRUE},
\code{\link[ggplot2]{geom_label}()} with
\code{stat = \link[ggalluvial]{StatStratum}} and
\code{min.y = nodes_label_miny}.
\item \strong{Axes, theme, labels} — \code{\link[ggplot2]{scale_x_discrete}()},
\code{\link[ggplot2]{scale_y_continuous}()}, custom theme, plot
title / subtitle, axis labels, and aesthetic adjustments (aspect
ratio, legend position / direction, grid removal, x-text angle).
\item \strong{Coordinate flip} — when \code{flip = TRUE},
\code{\link[ggplot2]{coord_flip}()} swaps the axes.
\item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
computes \code{height} and \code{width} attributes from the number
of x-axis positions, legend metrics, and flip state, scaled by
\code{aspect.ratio}.
\item \strong{Faceting} — \code{\link{facet_plot}()} wraps the result
when \code{facet_by} is provided.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{adjcolors}
\alias{adjcolors}
\title{Convert a color with arbitrary transparency to a fixed color}
\usage{
adjcolors(colors, alpha)
}
\arguments{
\item{colors}{Color vectors.}

\item{alpha}{Alpha level ranging from 0 to 1.}
}
\value{
The colors with the specified alpha level.
}
\description{
This function takes a vector of colors and an alpha level and converts the colors
to fixed colors with the specified alpha level.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corplot.R
\name{CorPairsPlot}
\alias{CorPairsPlot}
\title{Correlation pairs (scatterplot matrix)}
\usage{
CorPairsPlot(
  data,
  columns = NULL,
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  split_by = NULL,
  split_by_sep = "_",
  diag_type = NULL,
  diag_args = list(),
  layout = c(".\\\\", "\\\\.", "/.", "./"),
  cor_method = c("pearson", "spearman", "kendall"),
  cor_palette = "RdBu",
  cor_palcolor = NULL,
  cor_size = 3,
  cor_format = "corr: {round(corr, 2)}",
  cor_fg = "black",
  cor_bg = "white",
  cor_bg_r = 0.1,
  theme = "theme_this",
  theme_args = list(),
  palette = ifelse(is.null(group_by), "Spectral", "Paired"),
  palcolor = NULL,
  palreverse = FALSE,
  title = NULL,
  subtitle = NULL,
  facet_by = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{columns}{A character vector of column names to include in the
pairs plot.  When \code{NULL} (default), all columns except
\code{group_by} are used.  At least two columns are required.}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{group_name}{A character string used as the colour legend title
in the scatter plots.  When \code{NULL}, the \code{group_by} column
name is used.}

\item{split_by}{The column(s) to split the data by and produce separate
sub-plots.  Multiple columns are concatenated with \code{split_by_sep}.}

\item{split_by_sep}{A character string to separate concatenated
\code{split_by} columns.  Default \code{"_"}.}

\item{diag_type}{A character string specifying the plot type for
diagonal cells.  One of \code{"density"}, \code{"violin"},
\code{"histogram"}, \code{"box"}, or \code{"none"} (diagonal line).
Default: \code{"density"} (no \code{group_by}) or \code{"violin"}
(with \code{group_by}).}

\item{diag_args}{A named list of additional arguments passed to the
diagonal plot function (\code{\link{DensityPlot}},
\code{\link{ViolinPlot}}, \code{\link{Histogram}}, or
\code{\link{BoxPlot}}).  Default: \code{list()}.}

\item{layout}{A character string specifying the layout orientation.
One of the following codes (dot = scatter, backslash/slash =
diagonal): \code{.\\}, \code{\\\.}, \code{/.}, \code{./}.
Default: \code{.\\}.}

\item{cor_method}{A character string specifying the correlation method
for the fill tiles.  One of \code{"pearson"}, \code{"spearman"},
\code{"kendall"}.  Default: \code{"pearson"}.}

\item{cor_palette}{A character string specifying the colour palette for
the correlation fill tiles.  Default: \code{"RdBu"}.}

\item{cor_palcolor}{A character vector of custom colours used to create
the correlation tile palette.  When \code{NULL}, the palette's default
colours are used.}

\item{cor_size}{A numeric value specifying the font size of the
correlation text in the fill tiles.  Default: \code{3}.}

\item{cor_format}{A character string specifying a glue template for
formatting the correlation text.  The template is evaluated by
\code{glue::glue()} with access to \code{corr} (the correlation value),
\code{x}, and \code{y} (the column names).  Default:
\code{"corr: \\{round(corr, 2)\\}"}.}

\item{cor_fg}{A character string specifying the colour of the
correlation text.  Default: \code{"black"}.}

\item{cor_bg}{A character string specifying the background colour of
the correlation text boxes.  Default: \code{"white"}.}

\item{cor_bg_r}{A numeric value specifying the corner radius of the
correlation text background boxes.  Default: \code{0.1}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{seed}{A numeric seed for reproducibility.}

\item{combine}{Logical; when \code{TRUE} (default), returns a combined
\code{patchwork} object.  When \code{FALSE}, returns a named list of
individual \code{patchwork} objects.}

\item{ncol, nrow}{Integer number of columns / rows for the combined
layout.}

\item{byrow}{Logical; fill the combined layout by row.  Default
\code{TRUE}.}

\item{axes}{A character string specifying how axes should be treated
across the combined layout.}

\item{axis_titles}{A character string specifying how axis titles should
be treated across the combined layout.  Defaults to \code{axes}.}

\item{guides}{A character string specifying how guides (legends) should
be collected across panels.}

\item{design}{A custom layout design for the combined plot.}

\item{...}{Additional arguments.}
}
\value{
A \code{patchwork} object (when \code{combine = TRUE}) or a
named list of \code{patchwork} objects (when \code{combine = FALSE}),
each with \code{height} and \code{width} attributes in inches.
}
\description{
Draws a grid of pairwise scatter plots for selected numeric columns,
arranged in a scatterplot matrix layout.  The upper or lower triangle
displays correlation tiles while the opposite triangle shows scatter
plots with regression lines.  Diagonal cells can show density plots,
violin plots, histograms, box plots, or a simple diagonal line.

\strong{NOTE:} The \code{facet_by} parameter is \strong{not supported}
in CorPairsPlot (an error is raised if provided).  Use
\code{split_by} instead to create separate correlation pair matrices
per group.

The function supports \strong{four layout orientations} (\code{layout}),
\strong{three correlation methods}, configurable diagonal plots via
other plotthis functions, custom correlation tile formatting, and
splitting into separate sub-plots via \code{split_by}.
}
\section{split_by workflow}{

When \code{split_by} is provided:
\enumerate{
\item The \code{split_by} column is validated via
\code{\link{check_columns}()} with \code{force_factor = TRUE}.
Empty levels are dropped.
\item The data frame is split by \code{split_by} (preserving level
order).  If \code{split_by} is \code{NULL}, the data is wrapped
in a single-element list with name \code{"..."}.  The
\code{split_by} column is removed from each split's data before
plotting.
\item Per-split \code{palette}, \code{palcolor},
\code{legend.position}, and \code{legend.direction} are resolved
via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
and \code{\link{check_legend}()}.
\item \code{\link{CorPairsPlotAtomic}()} is called for each split.
When \code{title} is a function, it receives the split level
name and can generate dynamic titles.
\item Results are combined via \code{\link{combine_plots}()} (when
\code{combine = TRUE}) or returned as a named list.
}
}

\examples{
\donttest{
set.seed(8525)
data <- data.frame(x = rnorm(100))
data$y <- rnorm(100, 10, sd = 0.5)
data$z <- -data$x + data$y + rnorm(100, 20, 1)
data$g <- sample(1:4, 100, replace = TRUE)

# Histogram diagonal, slash layout
CorPairsPlot(data, diag_type = "histogram",
    diag_args = list(bins = 30, palette = "Paired"),
    layout = "/.")

# No diagonal with axis title styling
CorPairsPlot(data, group_by = "g", diag_type = "none", layout = "./",
    theme_args = list(axis.title = element_textbox(
        color = "black", box.color = "grey20", size = 16, halign = 0.5,
        fill = "grey90", linetype = 1,
        width = grid::unit(1, "npc"),
        padding = ggplot2::margin(5, 5, 5, 5))))

# Violin diagonal with custom format
CorPairsPlot(data, group_by = "g", diag_type = "violin", layout = "\\\\.",
    cor_format = "{x}\n{y}\ncorr: {round(corr, 2)}")

# Per-split with bottom legend
CorPairsPlot(data, split_by = "g", diag_type = "none", layout = ".\\\\",
    legend.position = "bottom", legend.direction = "horizontal",
    group_name = "group")

# Per-split with custom palette colours
CorPairsPlot(data, split_by = "g",
    palcolor = list("1" = "red", "2" = "blue", "3" = "green",
                    "4" = "yellow"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap-utils.R
\name{.apply_orderby}
\alias{.apply_orderby}
\title{Apply ordering to a data frame based on a specified expression}
\usage{
.apply_orderby(df, orderby, by, sby, sby_levels)
}
\arguments{
\item{df}{A data frame to be ordered.}

\item{orderby}{A character string representing the expression to order by.}

\item{by}{A character string representing the column name to group by.}

\item{sby}{A character string representing the column name to group by for secondary ordering.}

\item{sby_levels}{A character vector representing the levels of the secondary grouping column.}
}
\value{
A data frame ordered based on the specified expression.
}
\description{
Apply ordering to a data frame based on a specified expression
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dimplot.R
\name{DimPlot}
\alias{DimPlot}
\alias{FeatureDimPlot}
\title{DimPlot / FeatureDimPlot}
\usage{
DimPlot(
  data,
  dims = 1:2,
  group_by,
  group_by_sep = "_",
  split_by = NULL,
  split_by_sep = "_",
  pt_size = NULL,
  pt_alpha = 1,
  bg_color = "grey80",
  label_insitu = FALSE,
  show_stat = !identical(theme, "theme_blank"),
  label = FALSE,
  label_size = 4,
  label_fg = "white",
  label_bg = "black",
  label_bg_r = 0.1,
  label_repel = FALSE,
  label_repulsion = 20,
  label_pt_size = 1,
  label_pt_color = "black",
  label_segment_color = "black",
  order = c("as-is", "reverse", "high-top", "low-top", "random"),
  highlight = NULL,
  highlight_alpha = 1,
  highlight_size = 1,
  highlight_color = "black",
  highlight_stroke = 0.8,
  add_mark = FALSE,
  mark_type = c("hull", "ellipse", "rect", "circle"),
  mark_expand = unit(3, "mm"),
  mark_alpha = 0.1,
  mark_linetype = 1,
  stat_by = NULL,
  stat_plot_type = c("pie", "ring", "bar", "line"),
  stat_plot_size = 0.1,
  stat_args = list(palette = "Set1"),
  graph = NULL,
  edge_size = c(0.05, 0.5),
  edge_alpha = 0.1,
  edge_color = "grey40",
  add_density = FALSE,
  density_color = "grey80",
  density_filled = FALSE,
  density_filled_palette = "Greys",
  density_filled_palcolor = NULL,
  lineages = NULL,
  lineages_trim = c(0.01, 0.99),
  lineages_span = 0.75,
  lineages_palette = "Dark2",
  lineages_palcolor = NULL,
  lineages_arrow = arrow(length = unit(0.1, "inches")),
  lineages_linewidth = 1,
  lineages_line_bg = "white",
  lineages_line_bg_stroke = 0.5,
  lineages_whiskers = FALSE,
  lineages_whiskers_linewidth = 0.5,
  lineages_whiskers_alpha = 0.5,
  velocity = NULL,
  velocity_plot_type = c("raw", "grid", "stream"),
  velocity_n_neighbors = NULL,
  velocity_density = 1,
  velocity_smooth = 0.5,
  velocity_scale = 1,
  velocity_min_mass = 1,
  velocity_cutoff_perc = 5,
  velocity_group_palette = "Set2",
  velocity_group_palcolor = NULL,
  arrow_angle = 20,
  arrow_color = "black",
  arrow_alpha = 1,
  streamline_l = 5,
  streamline_minl = 1,
  streamline_res = 1,
  streamline_n = 15,
  streamline_width = c(0, 0.8),
  streamline_alpha = 1,
  streamline_color = NULL,
  streamline_palette = "RdYlBu",
  streamline_palcolor = NULL,
  streamline_bg_color = "white",
  streamline_bg_stroke = 0.5,
  keep_na = FALSE,
  keep_empty = FALSE,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_nrow = NULL,
  facet_ncol = NULL,
  facet_byrow = TRUE,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  theme = "theme_this",
  theme_args = list(),
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  raster = NULL,
  raster_dpi = c(512, 512),
  hex = FALSE,
  hex_linewidth = 0.5,
  hex_count = TRUE,
  hex_bins = 50,
  hex_binwidth = NULL,
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)

FeatureDimPlot(
  data,
  dims = 1:2,
  features,
  split_by = NULL,
  split_by_sep = "_",
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  pt_size = NULL,
  pt_alpha = 1,
  bg_color = "grey80",
  bg_cutoff = NULL,
  label_insitu = FALSE,
  show_stat = !identical(theme, "theme_blank"),
  color_name = "",
  label = FALSE,
  label_size = 4,
  label_fg = "white",
  label_bg = "black",
  label_bg_r = 0.1,
  label_repel = FALSE,
  label_repulsion = 20,
  label_pt_size = 1,
  label_pt_color = "black",
  label_segment_color = "black",
  order = c("as-is", "reverse", "high-top", "low-top", "random"),
  highlight = NULL,
  highlight_alpha = 1,
  highlight_size = 1,
  highlight_color = "black",
  highlight_stroke = 0.8,
  add_mark = FALSE,
  mark_type = c("hull", "ellipse", "rect", "circle"),
  mark_expand = unit(3, "mm"),
  mark_alpha = 0.1,
  mark_linetype = 1,
  keep_na = FALSE,
  keep_empty = FALSE,
  stat_by = NULL,
  stat_plot_type = c("pie", "ring", "bar", "line"),
  stat_plot_size = 0.1,
  stat_args = list(palette = "Set1"),
  graph = NULL,
  edge_size = c(0.05, 0.5),
  edge_alpha = 0.1,
  edge_color = "grey40",
  add_density = FALSE,
  density_color = "grey80",
  density_filled = FALSE,
  density_filled_palette = "Greys",
  density_filled_palcolor = NULL,
  lineages = NULL,
  lineages_trim = c(0.01, 0.99),
  lineages_span = 0.75,
  lineages_palette = "Dark2",
  lineages_palcolor = NULL,
  lineages_arrow = arrow(length = unit(0.1, "inches")),
  lineages_linewidth = 1,
  lineages_line_bg = "white",
  lineages_line_bg_stroke = 0.5,
  lineages_whiskers = FALSE,
  lineages_whiskers_linewidth = 0.5,
  lineages_whiskers_alpha = 0.5,
  velocity = NULL,
  velocity_plot_type = c("raw", "grid", "stream"),
  velocity_n_neighbors = NULL,
  velocity_density = 1,
  velocity_smooth = 0.5,
  velocity_scale = 1,
  velocity_min_mass = 1,
  velocity_cutoff_perc = 5,
  velocity_group_palette = "Set2",
  velocity_group_palcolor = NULL,
  arrow_angle = 20,
  arrow_color = "black",
  arrow_alpha = 1,
  streamline_l = 5,
  streamline_minl = 1,
  streamline_res = 1,
  streamline_n = 15,
  streamline_width = c(0, 0.8),
  streamline_alpha = 1,
  streamline_color = NULL,
  streamline_palette = "RdYlBu",
  streamline_palcolor = NULL,
  streamline_bg_color = "white",
  streamline_bg_stroke = 0.5,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_nrow = NULL,
  facet_ncol = NULL,
  facet_byrow = TRUE,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  theme = "theme_this",
  theme_args = list(),
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  raster = NULL,
  raster_dpi = c(512, 512),
  hex = FALSE,
  hex_linewidth = 0.5,
  hex_count = FALSE,
  hex_bins = 50,
  hex_binwidth = NULL,
  palette = "Spectral",
  palcolor = NULL,
  palreverse = FALSE,
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{dims}{A character vector of the column names to plot on the x, y (and optionally z) axes or a numeric vector
of the column indices. When 3 dimensions are provided, a 3D interactive plot is created using plotly.
Supported in 3D: group_by, features, labels, highlight, lineages, graph/network, show_stat, order.
Not supported in 3D: add_mark, stat_by, add_density, velocity, hex, facet_by, raster.}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{split_by}{A character vector of column names to split the data by and
plot separately. If \code{TRUE}, the data is split by \code{features} — each feature
is plotted in its own panel. Use this instead of \code{facet_by} when you need
independent layout control (\code{nrow}, \code{ncol}) or per-feature palettes via
\code{split_by = TRUE} combined with \code{palette} / \code{palcolor}.}

\item{split_by_sep}{The separator for multiple split_by columns. See \code{split_by}}

\item{pt_size}{A numeric value for point size, or a character string naming a numeric column
in \code{data} to map point sizes to. If \code{NULL} (default), auto-calculated as
\code{min(3000 / nrow(data), 0.6)}. Column-name mapping only applies to 2D plots and
is ignored with a message when \code{raster = TRUE}.}

\item{pt_alpha}{A numeric value in \verb{[0, 1]} for the point transparency. Default is \code{1}.}

\item{bg_color}{A character string specifying the colour used for NA-valued points and
background context points drawn from other facets. Default is \code{"grey80"}.}

\item{label_insitu}{A logical value. If \code{TRUE}, the raw group names are placed at the group median
coordinates instead of numeric indices. Forces \code{label = TRUE}. Default is \code{FALSE}.}

\item{show_stat}{A logical value. If \code{TRUE} (default), the number of points per group is shown
in the legend labels and subtitle. Ignored when \code{theme = "theme_blank"}.}

\item{label}{A logical value. If \code{TRUE}, group labels (numeric indices by default, or group
names when \code{label_insitu = TRUE}) are placed at the median coordinates of each group.
Forced to \code{TRUE} when \code{label_repel} or \code{label_insitu} is set.}

\item{label_size}{A numeric value for the label text size. Passed to \code{ggrepel::geom_text_repel()}.
Default is \code{4}.}

\item{label_fg}{A character string for the label text (foreground) colour. Default is \code{"white"}.}

\item{label_bg}{A character string for the label background / outline colour. Default is \code{"black"}.}

\item{label_bg_r}{A numeric value for the background fill ratio of the label bounding box.
Passed to \code{ggrepel::geom_text_repel(bg.r = ...)}. Default is \code{0.1}.}

\item{label_repel}{A logical value. If \code{TRUE}, labels are repelled from each other with force
\code{label_repulsion}. A visible point anchor is drawn. Forces \code{label = TRUE}.}

\item{label_repulsion}{A numeric value for the repulsion force when \code{label_repel = TRUE}.
Passed to \code{ggrepel::geom_text_repel(force = ...)}. Default is \code{20}.}

\item{label_pt_size}{A numeric value for the size of the anchor point drawn when
\code{label_repel = TRUE}. Default is \code{1}.}

\item{label_pt_color}{A character string for the colour of the label anchor point. Default is \code{"black"}.}

\item{label_segment_color}{A character string for the colour of the line segment connecting
the label to the anchor. Used in non-repel mode (\code{label_repel = FALSE}) where
\code{min.segment.length = 0}. Default is \code{"black"}.}

\item{order}{A character string controlling the draw order of points:
\itemize{
\item \code{"as-is"} (default) — the row order in the data is preserved.
\item \code{"reverse"} — rows are reversed.
\item \code{"high-top"} — points with high values (last factor levels for group_by) are drawn last (on top).
\item \code{"low-top"} — points with low values (first factor levels) are drawn last.
\item \code{"random"} — rows are randomly shuffled.
}

For \code{high-top} and \code{low-top}, NA values are always plotted at the bottom.
When applied to \code{group_by}, only the draw order changes — legend colours and order
are unaffected. Within the same level, point order is preserved. For precise
control, set factor levels before plotting.
See \url{https://github.com/pwwang/scplotter/issues/29#issuecomment-3009694130} for examples.}

\item{highlight}{A specification for highlighted points:
\itemize{
\item \code{NULL} (default): no highlighting.
\item \code{TRUE}: highlight all points (adds a dark outline around every point).
\item A character string: a dplyr filter expression (e.g., \code{"clusters == 'Ductal'"}).
\item A character vector: row names to highlight.
\item A numeric vector: row indices to highlight.
}}

\item{highlight_alpha}{A numeric value in \verb{[0, 1]} for the transparency of highlighted points.
Default is \code{1}.}

\item{highlight_size}{A numeric value for the size of the inner (coloured) highlight point.
Default is \code{1}.}

\item{highlight_color}{A character string for the colour of the outer highlight ring.
Default is \code{"black"}.}

\item{highlight_stroke}{A numeric value for the thickness of the outer highlight ring (the
difference between the outer ring size and \code{highlight_size}). Default is \code{0.8}.}

\item{add_mark}{A logical value. If \code{TRUE}, group boundaries are drawn around points using
\code{ggforce} marks. Requires \code{group_by}. Only supported in 2D.}

\item{mark_type}{A character string specifying the mark shape. Options:
\code{"hull"} (convex hull, default), \code{"ellipse"}, \code{"rect"}, or \code{"circle"}.}

\item{mark_expand}{A unit value for the outward expansion of the mark boundary.
Passed to \code{ggforce::geom_mark_*(expand = ...)}. Default is \code{unit(3, "mm")}.}

\item{mark_alpha}{A numeric value in \verb{[0, 1]} for the transparency of the mark fill.
Default is \code{0.1}.}

\item{mark_linetype}{A numeric value for the line type of the mark boundary. Default is \code{1} (solid).}

\item{stat_by}{A character string naming a column used to compute per-group statistical summary
mini-plots embedded at group centroid positions. Only supported with \code{group_by} (not \code{features}).
Only supported in 2D without \code{facet_by}.}

\item{stat_plot_type}{A character string specifying the mini-plot type. Options:
\code{"pie"} (default), \code{"ring"}, \code{"bar"}, or \code{"line"}.}

\item{stat_plot_size}{A numeric value for the size of the stat mini-plot, expressed as a
fraction of the axis range. Default is \code{0.1}.}

\item{stat_args}{A list of additional arguments passed to the stat plot function
(e.g., \code{list(palette = "Set1")}). Default is \code{list(palette = "Set1")}.}

\item{graph}{A specification for network / graph edges to overlay. Sources:
\itemize{
\item A character string starting with \code{"@"} (e.g., \code{"@graph"}): extracts the attribute
named \code{"graph"} from \code{attributes(data)}.
\item A \code{Graph} object (e.g., Seurat): coerced to dense matrix via \code{as.matrix()}.
\item A \code{matrix}, \code{data.frame}, or \code{dgCMatrix}: used directly as the adjacency matrix.
\item Numeric indices or character column names: extracts columns from \code{data}.
Edges are drawn for non-zero, lower-triangle entries. Requires \code{data} to have row names
matching the matrix dimnames.
}}

\item{edge_size}{A numeric vector of length 2 specifying the range \verb{[min, max]} for
\code{scale_linewidth_continuous(range = ...)} applied to edge widths. Default is \code{c(0.05, 0.5)}.}

\item{edge_alpha}{A numeric value in \verb{[0, 1]} for the transparency of graph edges.
Default is \code{0.1}.}

\item{edge_color}{A character string for the colour of graph edges. Default is \code{"grey40"}.}

\item{add_density}{A logical value. If \code{TRUE}, a 2D density layer is overlaid. Only supported in 2D.}

\item{density_color}{A character string for the colour of the density contour lines.
Used when \code{density_filled = FALSE}. Default is \code{"grey80"}.}

\item{density_filled}{A logical value. If \code{TRUE}, the density is rendered as a filled raster
(\code{stat_density_2d(geom = "raster")}) instead of contour lines. A separate fill scale is used.}

\item{density_filled_palette}{A character string naming the palette for the filled density layer.
Default is \code{"Greys"}.}

\item{density_filled_palcolor}{A character vector of specific colours for the filled density
palette. Default is \code{NULL} (auto-resolved from \code{density_filled_palette}).}

\item{lineages}{A character vector of column names representing pseudotime / trajectory lineages.
Each column is fitted with a LOESS smooth (\verb{span = lineages_span, degree = 2}) across the
2D embedding, after trimming the top and bottom \code{lineages_trim} quantiles.
Only supported in 2D without \code{facet_by}.}

\item{lineages_trim}{A numeric vector of length 2 specifying the lower and upper quantile
thresholds \verb{[0, 1]} for trimming lineage values before LOESS fitting.
Default is \code{c(0.01, 0.99)}.}

\item{lineages_span}{A numeric value passed as \code{span} to \code{stats::loess()} controlling the
smoothness of the lineage curve. Smaller values follow the data more closely.
Default is \code{0.75}.}

\item{lineages_palette}{A character string naming the palette for lineage colours.
Default is \code{"Dark2"}.}

\item{lineages_palcolor}{A character vector of specific colours for lineage curves.
Default is \code{NULL} (auto-resolved from \code{lineages_palette}).}

\item{lineages_arrow}{A ggplot2 \code{arrow} specification applied to the end of lineage paths.
Default is \code{arrow(length = unit(0.1, "inches"))}.}

\item{lineages_linewidth}{A numeric value for the width of the lineage curve lines.
Default is \code{1}.}

\item{lineages_line_bg}{A character string for the colour of the background (wider) stroke
drawn behind each lineage curve for improved visibility. Default is \code{"white"}.}

\item{lineages_line_bg_stroke}{A numeric value for the additional width of the background
stroke relative to \code{lineages_linewidth}. The background line has total width
\code{lineages_linewidth + lineages_line_bg_stroke}. Default is \code{0.5}.}

\item{lineages_whiskers}{A logical value. If \code{TRUE}, short line segments connect the
smoothed lineage curve to the original data coordinates of the fitted points.
Default is \code{FALSE}.}

\item{lineages_whiskers_linewidth}{A numeric value for the width of the whisker lines.
Default is \code{0.5}.}

\item{lineages_whiskers_alpha}{A numeric value in \verb{[0, 1]} for the transparency of the
whisker lines. Default is \code{0.5}.}

\item{velocity}{A specification for RNA-velocity arrows. Can be:
\itemize{
\item \code{NULL} (default): no velocity overlay.
\item A character / integer vector: column names or indices in \code{data} for the velocity embedding.
\item A data frame or matrix: the velocity embedding itself (must align with \code{data} rows).
Only supported in 2D without \code{facet_by}.
}}

\item{velocity_plot_type}{A character string specifying the velocity rendering style.
Options: \code{"raw"} (arrows from embedding), \code{"grid"} (grid-based arrows), or \code{"stream"}
(streamlines). Default is \code{"raw"}.}

\item{velocity_n_neighbors}{A numeric value for the number of neighbours used in the
velocity grid computation. Default is \code{NULL} (auto).}

\item{velocity_density}{A numeric value for the velocity kernel density bandwidth.
Default is \code{1}.}

\item{velocity_smooth}{A numeric value for the velocity smoothing parameter.
Default is \code{0.5}.}

\item{velocity_scale}{A numeric value for scaling the velocity arrows. Default is \code{1}.}

\item{velocity_min_mass}{A numeric value for the minimum cell mass threshold in velocity
grid computation. Default is \code{1}.}

\item{velocity_cutoff_perc}{A numeric value for the velocity cutoff percentage.
Default is \code{5}.}

\item{velocity_group_palette}{A character string naming the palette for velocity group
colours (used in \code{"raw"} plot type). Default is \code{"Set2"}.}

\item{velocity_group_palcolor}{A character vector of specific colours for velocity groups.
Default is \code{NULL} (auto-resolved from \code{velocity_group_palette}).}

\item{arrow_angle}{A numeric value specifying the angle of the arrowheads in degrees. Applied to \code{\link[grid]{arrow}} when \code{plot_type} is \code{"raw"} or \code{"grid"}. Default is 20.}

\item{arrow_color}{A character string specifying the color of the velocity arrows. For \code{plot_type = "stream"}, this sets only the arrowhead color. Default is \code{"black"}.}

\item{arrow_alpha}{A numeric value between 0 and 1 specifying the transparency of the velocity arrows. Only used when \code{plot_type = "raw"} or \code{"grid"}; for \code{plot_type = "stream"}, use \code{streamline_alpha} instead. Default is 1.}

\item{streamline_l}{A numeric value specifying the integration length of the streamlines. Passed to \code{\link[metR]{geom_streamline}} as the \code{L} parameter. Default is 5.}

\item{streamline_minl}{A numeric value specifying the minimum streamline length. Shorter streamlines are not drawn. Passed to \code{\link[metR]{geom_streamline}} as the \code{min.L} parameter. Default is 1.}

\item{streamline_res}{A numeric value specifying the resolution of the streamline integration. Passed to \code{\link[metR]{geom_streamline}} as the \code{res} parameter. Default is 1.}

\item{streamline_n}{A numeric value specifying the number of streamlines to draw. Passed to \code{\link[metR]{geom_streamline}} as the \code{n} parameter. Default is 15.}

\item{streamline_width}{A numeric vector of length 2 specifying the range of line widths for streamlines. Passed to \code{scale_size(range = ...)}. Only used when \code{streamline_color} is \code{NULL}. Default is \code{c(0, 0.8)}.}

\item{streamline_alpha}{A numeric value between 0 and 1 specifying the transparency of the velocity streamlines. Default is 1.}

\item{streamline_color}{An optional character string specifying a fixed color for streamlines. When \code{NULL} (the default), streamlines are colored by velocity magnitude using \code{streamline_palette}.}

\item{streamline_palette}{A character string specifying the color palette for streamline velocity magnitude. Passed to \code{\link{palette_this}}. Only used when \code{streamline_color} is \code{NULL}. Default is \code{"RdYlBu"}.}

\item{streamline_palcolor}{An optional character vector of specific colors for the streamline velocity gradient. If \code{NULL}, colors are generated from \code{streamline_palette}. Default is \code{NULL}.}

\item{streamline_bg_color}{A character string specifying the background (outline) color applied to streamlines to create a stroke effect. Default is \code{"white"}.}

\item{streamline_bg_stroke}{A numeric value specifying the additional line width of the background stroke relative to the foreground streamline. Default is 0.5.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{raster}{A logical value. If \code{TRUE}, points are rendered via
\code{scattermore::geom_scattermore()} for efficient rasterised plotting.
Default is \code{NULL}, which auto-enables when \code{nrow(data) > 1e5}.}

\item{raster_dpi}{A numeric vector of length 2 \verb{[x_dpi, y_dpi]} specifying the raster
resolution in pixels. Passed to \code{scattermore::geom_scattermore(pixels = ...)}.
Default is \code{c(512, 512)}. If a single value is provided it is recycled to both dimensions.}

\item{hex}{A logical value. If \code{TRUE}, points are rendered as hexagonal bins via
\code{geom_hex()} / \code{stat_summary_hex()}. Not supported with highlight.
Default is \code{FALSE}. Only supported in 2D.}

\item{hex_linewidth}{A numeric value for the width of the hexagon boundary lines.
Default is \code{0.5}.}

\item{hex_count}{A logical value. If \code{TRUE} and \code{group_by} is set, hex fill alpha
is mapped to \code{after_stat(count)} so denser bins are more opaque.
For \code{features} mode hex_count is ignored. Default is \code{!is.null(group_by)}.}

\item{hex_bins}{A numeric value for the number of hex bins along each axis.
Passed to \code{geom_hex(bins = ...)}. Default is \code{50}.}

\item{hex_binwidth}{A numeric value for the width of individual hex bins.
Passed to \code{geom_hex(binwidth = ...)}. Takes precedence over \code{hex_bins} when set.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{seed}{The random seed to use. Default is 8525.}

\item{combine}{Whether to combine the plots into one when facet is FALSE. Default is TRUE.}

\item{nrow}{A numeric value specifying the number of rows in the facet.}

\item{ncol}{A numeric value specifying the number of columns in the facet.}

\item{byrow}{A logical value indicating whether to fill the plots by row.}

\item{axes}{A string specifying how axes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axes in individual plots.
\item 'collect' will remove duplicated axes when placed in the same run of rows or columns of the layout.
\item 'collect_x' and 'collect_y' will remove duplicated x-axes in the columns or duplicated y-axes in the rows respectively.
}}

\item{axis_titles}{A string specifying how axis titltes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axis titles in individual plots.
\item 'collect' will remove duplicated titles in one direction and merge titles in the opposite direction.
\item 'collect_x' and 'collect_y' control this for x-axis titles and y-axis titles respectively.
}}

\item{guides}{A string specifying how guides should be treated in the layout. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'collect' will collect guides below to the given nesting level, removing duplicates.
\item 'keep' will stop collection at this level and let guides be placed alongside their plot.
\item 'auto' will allow guides to be collected if a upper level tries, but place them alongside the plot if not.
}}

\item{design}{Specification of the location of areas in the layout, passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE. When specified, \code{nrow}, \code{ncol}, and \code{byrow} are ignored.
See \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}} for more details.}

\item{...}{Additional arguments.}

\item{features}{A character vector of the column names to plot as features (continuous colouring).
When multiple features are provided and \code{facet_by} is not set, the data is pivoted to long format and
faceted by feature name.}

\item{lower_quantile, upper_quantile}{Lower and upper quantiles for the continuous color/fill scale.
The actual cutoffs are determined by these quantiles when \code{lower_cutoff} and
\code{upper_cutoff} are \code{NULL}. Defaults: \code{lower_quantile = 0}, \code{upper_quantile = 0.99}.}

\item{lower_cutoff, upper_cutoff}{Explicit lower and upper cutoffs for the continuous color/fill scale.
When \code{NULL} (the default), the cutoffs are determined by \code{lower_quantile} and
\code{upper_quantile} via \code{\link[stats]{quantile}}. Values outside the
\code{[lower_cutoff, upper_cutoff]} range are clamped (winsorized) to the nearest cutoff value.}

\item{bg_cutoff}{A numeric threshold. Feature values with absolute value below this
cutoff are set to \code{NA} (and therefore rendered in \code{bg_color}). Default is \code{NULL}.}

\item{color_name}{A character string used as the title for the continuous colour bar
in feature mode. Default is \code{""}.}
}
\value{
A \code{ggplot} object (single plot), a \code{patchwork} / \code{wrap_plots} object
(when \code{split_by} is provided and \code{combine = TRUE}), or a list of \code{ggplot}
objects (when \code{split_by} is provided and \code{combine = FALSE}).
When \code{dims} has 3 elements, a \strong{plotly} object is returned instead.

A \code{ggplot} object (single plot), a \code{patchwork} / \code{wrap_plots} object
(when \code{split_by} is provided and \code{combine = TRUE}), or a list of \code{ggplot}
objects (when \code{split_by} is provided and \code{combine = FALSE}).
When \code{dims} has 3 elements, a \strong{plotly} object is returned instead.
}
\description{
\code{DimPlot} visualizes dimension reduction data (PCA, t-SNE, UMAP, etc.) as a 2D or 3D
scatter plot. \code{DimPlot()} colours points by a discrete grouping variable
(e.g., clusters), while \code{FeatureDimPlot()} colours points by a continuous
numeric feature (e.g., gene expression, lineage scores).

Both functions share the same internal engine (\code{DimPlotAtomic}) and support
an extensive set of annotation layers: group boundary marks, network/graph
edges, 2D density contours, lineage/trajectory curves, RNA-velocity arrows
(raw, grid, or stream), statistical summary mini-plots at group centroids,
point highlighting, background context points from other facets, and
flexible label positioning.

When \code{dims} has 3 elements, both functions automatically return an
interactive \strong{plotly} 3D scatter plot (via \code{DimPlotAtomic3D}). Certain
2D-only features are silently ignored in 3D mode (see \verb{@param dims} for
the full list).

Rendering scales with dataset size: standard \code{geom_point()} for small data,
automatic rasterisation via \code{scattermore::geom_scattermore()} when
\code{nrow(data) > 1e5}, or hex-bin aggregation (\code{geom_hex()} /
\code{stat_summary_hex()}).

\code{FeatureDimPlot} to visualize feature expression on dimension reduction plots.
Colours points by a
continuous numeric variable (e.g., gene expression, module score, lineage
pseudotime) using a gradient colour scale, with optional quantile trimming
and background cutoff.

When multiple \code{features} are provided and \code{facet_by} is not set, the data
is automatically pivoted to long format and faceted by feature name.
\code{split_by = TRUE} dispatches each feature to a separate plot for
independent layout control. \code{split_by} as a column name splits by that
column's levels, producing one plot per level with per-split palette
support.

For detailed split_by workflows, see the main \code{DimPlot / FeatureDimPlot}
documentation (\verb{@section split_by Workflow (FeatureDimPlot)}).
}
\section{split_by Workflow (DimPlot)}{


When \code{split_by} is specified, \code{DimPlot()} executes the following pipeline:
\enumerate{
\item \strong{Argument validation} — \code{validate_common_args()} checks the seed and
blocks \code{split_by} + \code{velocity} combinations.
\item \strong{NA / empty normalisation} — \code{check_keep_na()} / \code{check_keep_empty()}
convert \code{keep_na} / \code{keep_empty} to per-column lists.
\item \strong{Theme resolution} — \code{process_theme()} resolves the theme string to a
theme function.
\item \strong{Split column resolution} — \code{check_columns()} validates \code{split_by}
(force_factor, concat_multi).
\item \strong{Pre-filtering} — \code{process_keep_na_empty()} removes NA / empty levels
from the split column, then \code{data} is split by \code{split_by} levels (order
preserved). When \code{graph} references an attribute (\verb{@graph}), the graph
matrix is also subset per split.
\item \strong{Per-split parameter resolution} — \code{check_palette()},
\code{check_palcolor()}, \code{check_legend()} resolve palette, palcolor,
legend.position, and legend.direction for each split.
\item \strong{Per-split dispatch} — each split is passed to \code{DimPlotAtomic()} with
its resolved parameters. Title defaults to the split level name unless
\code{title} is a function.
\item \strong{Combination} — \code{combine_plots()} assembles the list of plots via
\code{patchwork::wrap_plots()}, applying \code{nrow}, \code{ncol}, \code{byrow}, \code{axes},
\code{axis_titles}, \code{guides}, and \code{design}.
}
}

\section{split_by Workflow (FeatureDimPlot)}{


\code{FeatureDimPlot()} supports two forms of splitting:

\strong{A. \code{split_by = TRUE} (split by features)}
\enumerate{
\item Each feature in \code{features} is dispatched individually to
\code{DimPlotAtomic()}, producing one plot per feature. The plot title
defaults to the feature name.
\item Plots are combined via \code{combine_plots()} with \code{split_by = ".features"}.
}

\strong{B. \code{split_by} as a column name (split by data column)}
\enumerate{
\item Data is split by the named column's levels (same pipeline as DimPlot
steps 1–8 above). Graph attribute splitting is supported.
}
}

\examples{
\donttest{
data(dim_example)

# basic dim plot
DimPlot(dim_example, group_by = "clusters")
DimPlot(dim_example, group_by = "clusters", theme = "theme_blank")
DimPlot(dim_example, group_by = "clusters", theme = ggplot2::theme_classic,
    theme_args = list(base_size = 16), palette = "seurat")

# raster and highlighting
DimPlot(dim_example, group_by = "clusters", raster = TRUE, raster_dpi = 50)
DimPlot(dim_example, group_by = "clusters", highlight = 1:20,
    highlight_color = "black", highlight_stroke = 2)
DimPlot(dim_example, group_by = "clusters", highlight = TRUE, facet_by = "group",
    theme = "theme_blank")

# labels
DimPlot(dim_example, group_by = "clusters", label = TRUE,
    label_size = 5, label_bg_r = 0.2)
DimPlot(dim_example, group_by = "clusters", label = TRUE, label_fg = "red",
    label_bg = "yellow", label_size = 5)
DimPlot(dim_example, group_by = "clusters", label = TRUE, label_insitu = TRUE)

# group marks
DimPlot(dim_example, group_by = "clusters", add_mark = TRUE)
DimPlot(dim_example, group_by = "clusters", add_mark = TRUE, mark_linetype = 2)
DimPlot(dim_example, group_by = "clusters", add_mark = TRUE, mark_type = "ellipse")

# density overlays
DimPlot(dim_example, group_by = "clusters", add_density = TRUE)
DimPlot(dim_example, group_by = "clusters", add_density = TRUE, density_filled = TRUE)
DimPlot(dim_example, group_by = "clusters", add_density = TRUE, density_filled = TRUE,
    density_filled_palette = "Blues", highlight = TRUE)

# statistics at group centroids
DimPlot(dim_example, group_by = "clusters", stat_by = "group")
DimPlot(dim_example, group_by = "clusters", stat_by = "group",
    stat_plot_type = "bar", stat_plot_size = 0.06)

# hex bins
DimPlot(dim_example, group_by = "clusters", hex = TRUE)
DimPlot(dim_example, group_by = "clusters", hex = TRUE, hex_bins = 20)
DimPlot(dim_example, group_by = "clusters", hex = TRUE, hex_count = FALSE)

# graph / network edges
DimPlot(dim_example, group_by = "clusters", graph = "@graph", edge_color = "grey80")

# lineages / trajectories
DimPlot(dim_example, group_by = "clusters", lineages = c("stochasticbasis_1", "stochasticbasis_2"))
DimPlot(dim_example, group_by = "clusters", lineages = c("stochasticbasis_1", "stochasticbasis_2"),
    lineages_whiskers = TRUE, lineages_whiskers_linewidth = 0.1)
DimPlot(dim_example, group_by = "clusters", lineages = c("stochasticbasis_1", "stochasticbasis_2"),
    lineages_span = 0.4)

# split_by
DimPlot(dim_example, group_by = "clusters",  split_by = "group",
    palette = list(A = "Paired", B = "Set1"))

# velocity
DimPlot(dim_example, group_by = "clusters", velocity = c("stochasticbasis_1", "stochasticbasis_2"),
    pt_alpha = 0)
DimPlot(dim_example, group_by = "clusters", velocity = 3:4,
    velocity_plot_type = "grid", arrow_alpha = 0.6)
DimPlot(dim_example, group_by = "clusters", velocity = 3:4,
    velocity_plot_type = "stream")

# 3D plots (returns a plotly object)
DimPlot(dim_example, dims = 1:3, group_by = "clusters")
DimPlot(dim_example, dims = 1:3, group_by = "clusters", label = TRUE,
    label_insitu = TRUE)
DimPlot(dim_example, dims = c("basis_1", "basis_2", "stochasticbasis_1"),
    group_by = "clusters", graph = "@graph", edge_color = "grey80")

# keep_na and keep_empty
dim_example$clusters[dim_example$clusters == "Ductal"] <- NA

DimPlot(dim_example, group_by = "clusters", keep_na = FALSE, keep_empty = TRUE)
DimPlot(dim_example, group_by = "clusters", keep_na = TRUE, keep_empty = TRUE)
DimPlot(dim_example, group_by = "clusters", keep_na = TRUE, keep_empty = FALSE)
}
\donttest{
data(dim_example)

# single feature
FeatureDimPlot(dim_example, features = "stochasticbasis_1", pt_size = 2)
FeatureDimPlot(dim_example, features = "stochasticbasis_1", pt_size = 2, bg_cutoff = 0)
FeatureDimPlot(dim_example, features = "stochasticbasis_1", raster = TRUE, raster_dpi = 30)

# multiple features (auto-pivoted to long, faceted by feature)
FeatureDimPlot(dim_example, features = c("stochasticbasis_1", "stochasticbasis_2"),
 pt_size = 2)

# single feature with facet_by (facet_by works when only 1 feature)
FeatureDimPlot(dim_example, features = c("stochasticbasis_1"), pt_size = 2,
 facet_by = "group")

# multiple features with split_by for independent layout
FeatureDimPlot(dim_example, features = c("stochasticbasis_1", "stochasticbasis_2"),
 split_by = "group", nrow = 2)

# highlight and hex
FeatureDimPlot(dim_example, features = c("stochasticbasis_1", "stochasticbasis_2"),
 highlight = TRUE)
FeatureDimPlot(dim_example, features = c("stochasticbasis_1", "stochasticbasis_2"),
 hex = TRUE, hex_bins = 15)
FeatureDimPlot(dim_example, features = c("stochasticbasis_1", "stochasticbasis_2"),
 hex = TRUE, hex_bins = 15, split_by = "group", palette = list(A = "Reds", B = "Blues"))

# 3D plots (returns a plotly object)
FeatureDimPlot(dim_example, dims = 1:3, features = "stochasticbasis_2", pt_size = 2)
FeatureDimPlot(dim_example, dims = c("basis_1", "basis_2", "stochasticbasis_1"),
 features = "stochasticbasis_2")
}
}
\seealso{
\code{\link{VelocityPlot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{prepare_continuous_color_scale}
\alias{prepare_continuous_color_scale}
\title{Prepare continuous color scale limits with quantile/cutoff controls}
\usage{
prepare_continuous_color_scale(
  data,
  column,
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  bg_cutoff = NULL
)
}
\arguments{
\item{data}{A data frame.}

\item{column}{The column name in \code{data} to use for the color scale.}

\item{lower_quantile, upper_quantile}{Lower and upper quantiles for the continuous color/fill scale.
The actual cutoffs are determined by these quantiles when \code{lower_cutoff} and
\code{upper_cutoff} are \code{NULL}. Defaults: \code{lower_quantile = 0}, \code{upper_quantile = 0.99}.}

\item{lower_cutoff, upper_cutoff}{Explicit lower and upper cutoffs for the continuous color/fill scale.
When \code{NULL} (the default), the cutoffs are determined by \code{lower_quantile} and
\code{upper_quantile} via \code{\link[stats]{quantile}}. Values outside the
\code{[lower_cutoff, upper_cutoff]} range are clamped (winsorized) to the nearest cutoff value.}

\item{bg_cutoff}{Optional numeric cutoff — values \code{<= bg_cutoff} are
set to \code{NA} before computing cutoffs. Default is \code{NULL}.}
}
\value{
A list with components:
\item{data}{The modified data frame (with winsorized \code{column}).}
\item{feat_colors_value}{Numeric vector of 100 evenly-spaced values from
\code{lower_cutoff} to \code{upper_cutoff}.}
\item{limits}{Numeric vector of length 2 giving the range
\code{c(lower_cutoff, upper_cutoff)}.}
}
\description{
Computes the lower and upper cutoffs for a continuous color/fill scale,
applies data winsorization (clamping), and returns the results needed
by \code{\link[ggplot2]{scale_fill_gradientn}} or
\code{\link[ggplot2]{scale_color_gradientn}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap-utils.R
\name{layer_white_bg}
\alias{layer_white_bg}
\alias{layer_bg}
\alias{layer_reticle}
\alias{layer_dot}
\alias{layer_bars}
\alias{layer_pie}
\alias{layer_boxviolin}
\title{Heatmap layer functions used to draw on the heatmap cells}
\usage{
layer_white_bg(j, i, x, y, w, h, fill)

layer_bg(j, i, x, y, w, h, fill, alpha)

layer_reticle(j, i, x, y, w, h, fill, color)

layer_dot(
  j,
  i,
  x,
  y,
  w,
  h,
  fill,
  data,
  dot_size,
  alpha,
  row_names = NULL,
  col_names = NULL
)

layer_bars(j, i, x, y, w, h, fill, flip, col_fun, data, alpha)

layer_pie(j, i, x, y, w, h, fill, palette, palcolor, data, pie_size)

layer_boxviolin(j, i, x, y, w, h, fill, flip, data, colors, fn)
}
\arguments{
\item{j}{An integer specifying the column index}

\item{i}{An integer specifying the row index}

\item{x}{A numeric vector specifying the x position}

\item{y}{A numeric vector specifying the y position}

\item{w}{A numeric vector specifying the width}

\item{h}{A numeric vector specifying the height}

\item{fill}{A character vector specifying the fill color}

\item{alpha}{A numeric value between 0 and 1 specifying the transparency of the fill color}

\item{color}{A character vector specifying the color of the reticle}

\item{data}{A dataframe used to create the annotation. Different from the data used to
create the heatmap itself, which is aggregated data. This dataframe is the original data,
where each cell could have multiple values.}

\item{dot_size}{A numeric value specifying the size of the dot or a function to calculate the size
from the values in the cell. The function can take 1, 3, or 5 arguments: the first argument is
the values in the cell before aggregation; the 2nd and 3rd arguments are the row and column
indices; the 4th and 5th arguments are the row and column names.}

\item{row_names}{Row names from the heatmap matrix.}

\item{col_names}{Column names from the heatmap matrix.}

\item{col_fun}{A function to calculate the color of the bars}

\item{colors}{A character vector specifying the fill color of the violin plot.
If not provided, the fill color of row/column annotation will be used}
}
\description{
Heatmap layer functions used to draw on the heatmap cells
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/upsetplot.R
\name{UpsetPlotAtomic}
\alias{UpsetPlotAtomic}
\title{Atomic UpSet plot (internal)}
\usage{
UpsetPlotAtomic(
  data,
  in_form = "auto",
  group_by = NULL,
  group_by_sep = "_",
  id_by = NULL,
  label = TRUE,
  label_fg = "black",
  label_size = NULL,
  label_bg = "white",
  label_bg_r = 0.1,
  palette = "material-indigo",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  specific = TRUE,
  combmatrix_gap = 6,
  theme = "theme_this",
  theme_args = list(),
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  aspect.ratio = 0.6,
  legend.position = "right",
  legend.direction = "vertical",
  levels = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{in_form}{A character string specifying the input format.  One of
\code{"auto"} (default; detect from \code{data} structure),
\code{"long"}, \code{"wide"}, \code{"list"}, or \code{"upset"}.}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{id_by}{A character string specifying the column name for instance
identifiers.  Required for long format; optional for wide format (a
synthetic \code{.id} column is created if omitted).}

\item{label}{A logical value.  When \code{TRUE} (default), count labels are
displayed above each intersection bar via \code{\link[ggrepel]{geom_text_repel}()}.}

\item{label_fg}{A character string specifying the colour of the label text.
Default: \code{"black"}.}

\item{label_size}{A numeric value specifying the size of the label text.
Default: \code{NULL} (computed from \code{base_size / 12 * 3.5}).}

\item{label_bg}{A character string specifying the background fill colour of
the label.  Default: \code{"white"}.}

\item{label_bg_r}{A numeric value specifying the corner radius of the label
background, passed to \code{geom_text_repel(bg.r)}.  Default: \code{0.1}.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{specific}{A logical value.  When \code{TRUE} (default), only specific
intersections are returned (elements belonging exclusively to the shown
set combination).  When \code{FALSE}, all overlapping items are included.
See \url{https://github.com/gaospecial/ggVennDiagram/issues/64}.}

\item{combmatrix_gap}{A numeric value specifying the gap between rows of the
combination matrix, measured at \code{base_size = 12}.  The actual gap is
scaled by \code{text_size_scale = base_size / 12}.  Default: \code{6}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{levels}{A character vector of set levels to include, in display order.
When \code{NULL} (default), all detected sets are shown.  Accepted for
compatibility but not currently used by the internal implementation.}

\item{...}{Additional arguments passed to \code{\link[ggupset]{scale_x_upset}}.
Useful options include \code{n_sets} and \code{n_intersections} to limit
the number of shown sets / intersections.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width} attributes
(in inches).
}
\description{
Core implementation for drawing a single UpSet plot.  This is the internal
workhorse behind the exported \code{\link{UpsetPlot}} function --- it takes a
\strong{single} data partition (no \code{split_by} support) and returns a
\code{ggplot} object.  The plot comprises three visual components:
\itemize{
\item A \strong{horizontal bar chart} (top) showing the size of each
intersection, with bars filled by a gradient of intersection count.
\item A \strong{set-size bar chart} (left) showing the total number of
elements in each set (added automatically by \code{ggupset}).
\item A \strong{combination matrix} (bottom) where rows are sets, columns
are intersections, and dots indicate set membership.  Connected dots
trace the intersection pattern.
}
}
\section{Architecture}{

\enumerate{
\item \strong{ggplot dispatch} --- selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{Data preparation} --- \code{\link{prepare_upset_data}()}
converts the input into the internal \code{UpsetPlotData} format
with an \code{Intersection} list-column (set-membership labels per
element) and a \code{"group_order"} attribute for set ordering.
\item \strong{Text scale} --- \code{base_size} from \code{theme_args}
(default 12) drives a \code{text_size_scale} that proportionally
scales the combination-matrix row gap, label sizes, and point sizes.
\item \strong{Bar geom} --- \code{geom_bar()} counts rows per intersection.
Bars are filled by count via \code{scale_fill_gradientn()} with
colours from \code{\link{palette_this}()}.
\item \strong{Bar labels} --- when \code{label = TRUE},
\code{\link[ggrepel]{geom_text_repel}()} places count labels above
each bar, with configurable foreground, background, and size.
\item \strong{Scale x upset} --- \code{ggupset::scale_x_upset(...)} is
called, returning two components: a \code{ScaleUpset} (index \code{[[1]]})
and a \code{CoordCombMatrix} (index \code{[[2]]}).  Only the scale is
applied; the bundled coord is discarded to avoid a "Coordinate
system already present" conflict.
\item \strong{Labels and theme} --- \code{labs()} sets title, subtitle,
and axis text.  \code{do_call(theme, theme_args)} applies the chosen
theme.  Aspect ratio, legend position, legend direction, and grid
lines are configured.
\item \strong{Combination matrix} --- \code{ggupset::axis_combmatrix()}
builds the lower panel via an \code{override_plotting_function}:
\itemize{
\item Reorders rows to match \code{group_order} (reversed for
top-to-bottom set listing).
\item Alternating row backgrounds via \code{geom_rect()} with
white / \code{#F7F7F7} fills.
\item Set-membership dots (\code{geom_point()}) in black for
present and \code{#E0E0E0} for absent.
\item Connecting lines (\code{geom_line()}) through observed
dots to trace the intersection pattern.
}
\item \strong{Matrix theme} --- \code{ggupset::theme_combmatrix()}
applies the combination-matrix appearance with
\code{combmatrix_gap} between rows (scaled by
\code{text_size_scale}).
\item \strong{Dimension calculation} ---
\code{\link{calculate_plot_dimensions}()} computes height from
\code{n_sets} and width from \code{n_intersections}, both passed
via \code{...} (default 99).  \code{aspect.ratio} is used as a
standard H/W coupling with \code{x_scale_factor = 0.6}.  An
additional width adjustment accounts for set-label character width.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxviolinplot.R
\name{BoxViolinPlotAtomic}
\alias{BoxViolinPlotAtomic}
\title{Atomic Box / Violin / Bar / Beeswarm plot (internal)}
\usage{
BoxViolinPlotAtomic(
  data,
  x,
  x_sep = "_",
  y = NULL,
  base = c("box", "violin", "bar", "none"),
  in_form = c("long", "wide"),
  sort_x = NULL,
  flip = FALSE,
  keep_empty = FALSE,
  keep_na = FALSE,
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  paired_by = NULL,
  x_text_angle = ifelse(isTRUE(flip), 0, 45),
  step_increase = 0.1,
  position_dodge_preserve = "total",
  fill_mode = ifelse(!is.null(group_by), "dodge", "x"),
  palreverse = FALSE,
  symnum_args = NULL,
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  alpha = 1,
  aspect.ratio = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  add_point = FALSE,
  pt_color = if (isTRUE(add_beeswarm)) NULL else "grey30",
  pt_size = NULL,
  pt_alpha = 1,
  y_nbreaks = 4,
  jitter_width = NULL,
  jitter_height = 0,
  stack = FALSE,
  y_max = NULL,
  y_min = NULL,
  y_trans = "identity",
  y_brackets = NULL,
  add_beeswarm = FALSE,
  beeswarm_method = "swarm",
  beeswarm_cex = 1,
  beeswarm_priority = "ascending",
  beeswarm_dodge = 0.9,
  add_box = FALSE,
  box_color = "black",
  box_width = 0.1,
  box_ptsize = 2.5,
  add_errorbar = "SEM",
  errorbar_color = "grey20",
  errorbar_width = 0.4,
  errorbar_linewidth = 0.6,
  add_trend = FALSE,
  trend_color = NULL,
  trend_linewidth = 1,
  trend_ptsize = 2,
  add_stat = NULL,
  stat_name = NULL,
  stat_color = "black",
  stat_size = 1,
  stat_stroke = 1,
  stat_shape = 25,
  add_bg = FALSE,
  bg_palette = "stripe",
  bg_palcolor = NULL,
  bg_alpha = 0.2,
  add_line = NULL,
  line_color = "red2",
  line_width = 0.6,
  line_type = 2,
  highlight = NULL,
  highlight_color = "red2",
  highlight_size = 1,
  highlight_alpha = 1,
  comparisons = NULL,
  ref_group = NULL,
  pairwise_method = "wilcox.test",
  multiplegroup_comparisons = FALSE,
  multiple_method = "kruskal.test",
  sig_label = "p.format",
  sig_labelsize = 3.5,
  hide_ns = FALSE,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string of column name(s) for the x-axis.
Character/factor columns are expected.  Multiple columns are
concatenated with \code{x_sep}.}

\item{x_sep}{A character string to join multiple \code{x} columns.
Default \code{"_"}.}

\item{y}{A character string of the numeric column for the y-axis.
Not required when \code{in_form = "wide"} (data values are taken from
the \code{x} columns).}

\item{base}{A character string specifying the primary plot type:
\code{"box"} (box plot), \code{"violin"} (violin plot), \code{"bar"}
(mean bars with optional error bars), or \code{"none"} (no primary
geometry, used by beeswarm plots).}

\item{in_form}{A character string: \code{"long"} (default) or
\code{"wide"}.  In wide form, \code{x} columns are pivoted to long
format.}

\item{sort_x}{An R expression string (e.g., \code{"mean(y)"}) to order
x-axis categories.  Default \code{NULL} keeps the original order.
When \code{keep_empty_x} is \code{TRUE}, empty levels are placed last.}

\item{flip}{Logical; if \code{TRUE}, swap the x and y axes.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{group_by}{A character vector of column name(s) to dodge the
boxes/violins by.  Multiple columns are concatenated with
\code{group_by_sep}.}

\item{group_by_sep}{A character string to separate concatenated
\code{group_by} columns.  Default \code{"_"}.}

\item{group_name}{A character string for the dodge legend title.}

\item{paired_by}{A character string naming a column that identifies
paired observations.  Forces \code{add_point = TRUE} and connects
paired observations with lines.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{step_increase}{Fractional step increase for stacking significance
brackets when multiple comparisons exist.}

\item{position_dodge_preserve}{Passed to
\code{\link[ggplot2]{position_dodge}()}: \code{"total"} preserves the
overall group width; \code{"single"} preserves individual element width.}

\item{fill_mode}{A character string controlling fill colour mapping:
\code{"dodge"} (fill by \code{group_by}, discrete),
\code{"x"} (fill by x-axis categories, discrete),
\code{"mean"} or \code{"median"} (fill by pre-computed statistic,
continuous gradient).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{symnum_args}{A list of arguments passed to
\code{\link[stats]{symnum}} for symbolic p-value coding.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{add_point}{Logical; add jittered or beeswarm points to the plot.}

\item{pt_color}{Colour of the points.  When \code{add_beeswarm = TRUE}
and \code{pt_color} is \code{NULL}, points are coloured by the fill
variable.}

\item{pt_size}{Numeric size of the points.  Default computed from
data size: \code{min(3000 / nrow(data), 0.6)}.}

\item{pt_alpha}{Numeric transparency of the points.}

\item{y_nbreaks}{Integer number of y-axis breaks.}

\item{jitter_width}{Numeric width of the jitter.  Defaults to
\code{0.5}, but set to \code{0} when \code{paired_by} is provided.}

\item{jitter_height}{Numeric height of the jitter.  Default \code{0}.}

\item{stack}{Logical; stack facetted panels in a compact layout with
shared strip labels.}

\item{y_max, y_min}{Numeric y-axis limits, or quantile notation strings
(e.g., \code{"q95"} for the 95th percentile, \code{"q5"} for the
5th percentile).}

\item{y_trans}{A character string for y-axis transformation
(e.g., \code{"log10"}).}

\item{y_brackets}{Numeric y-axis position for significance brackets
(or p-value labels for multiple comparisons).  If NULL, the brackets are placed above the maximum y-value.}

\item{add_beeswarm}{Logical; use \code{ggbeeswarm::geom_beeswarm()} for
non-overlapping point layout instead of jitter.  Requires the
\code{ggbeeswarm} package.}

\item{beeswarm_method}{Beeswarm layout method: \code{"swarm"},
\code{"compactswarm"}, \code{"hex"}, \code{"square"}, or
\code{"center"}.}

\item{beeswarm_cex}{Numeric scaling for point spacing.  Larger values
spread points more.}

\item{beeswarm_priority}{Point layout priority: \code{"ascending"},
\code{"descending"}, \code{"density"}, or \code{"random"}.}

\item{beeswarm_dodge}{Numeric dodge width for beeswarm points when
\code{group_by} is provided.  Default \code{0.9}.}

\item{add_box}{Logical; overlay a box plot on the primary geometry.
Mutually exclusive with \code{base = "box"} and \code{base = "bar"}.}

\item{box_color}{Colour of the overlaid box plot outline and fill.}

\item{box_width}{Width of the overlaid box plot.}

\item{box_ptsize}{Size of the median point in the overlaid box plot.}

\item{add_errorbar}{Type of error bars for bar plots (\code{base = "bar"}):
\code{"SEM"} (standard error of the mean, default), \code{"SD"}
(standard deviation), \code{"CI"} or \code{"CI95"} (95\\% confidence
interval), or \code{"none"}.  Silently ignored for non-bar bases.}

\item{errorbar_color}{Colour of the error bar lines and caps.}

\item{errorbar_width}{Width of the error bar caps.}

\item{errorbar_linewidth}{Line width of the error bars.}

\item{add_trend}{Logical; add trend lines connecting group medians.}

\item{trend_color}{Colour of the trend line.  When \code{NULL} and
\code{group_by} is present, lines are coloured per group.}

\item{trend_linewidth}{Width of the trend line.}

\item{trend_ptsize}{Size of the trend line points.}

\item{add_stat}{A summary function (e.g., \code{mean}, \code{median}) to
display as a point with a shape legend entry.}

\item{stat_name}{Legend title for the stat summary shape.}

\item{stat_color}{Colour of the stat summary point.}

\item{stat_size}{Size of the stat summary point.}

\item{stat_stroke}{Stroke width of the stat summary point.}

\item{stat_shape}{Shape (an integer) for the stat summary point.  Uses
\code{scale_shape_identity()} so the shape is rendered directly.}

\item{add_bg}{Logical; add alternating background stripes.}

\item{bg_palette}{Palette for the background stripes.}

\item{bg_palcolor}{Custom colours for the background stripes.}

\item{bg_alpha}{Alpha transparency for the background stripes.}

\item{add_line}{A numeric y-intercept for a horizontal reference line.}

\item{line_color}{Colour of the reference line.}

\item{line_width}{Width of the reference line.}

\item{line_type}{Linetype of the reference line.}

\item{highlight}{A specification of points to highlight: \code{TRUE}
(all), a numeric index vector, a logical expression string, or a
character vector of row names.}

\item{highlight_color}{Colour of highlighted points.}

\item{highlight_size}{Size of highlighted points.}

\item{highlight_alpha}{Alpha of highlighted points.}

\item{comparisons}{A logical value (\code{TRUE} for all pairs) or a list
of two-element vectors specifying pairwise comparisons.  Only available
when \code{fill_mode = "dodge"} (i.e., \code{group_by} is present).}

\item{ref_group}{A character string specifying the reference group for
comparisons.}

\item{pairwise_method}{Method for pairwise tests.  Default
\code{"wilcox.test"}.}

\item{multiplegroup_comparisons}{Logical; perform an omnibus test
(e.g., Kruskal-Wallis) across all groups.}

\item{multiple_method}{Method for the omnibus test.  Default
\code{"kruskal.test"}.}

\item{sig_label}{Label format for significance annotations.  For
pairwise comparisons: \code{"p.format"}, \code{"p.signif"}, or a
\pkg{glue} template (e.g., \code{"p = {p}"}).  For multiple-group
tests: \code{"p.format"} or \code{"p.signif"}.}

\item{sig_labelsize}{Size of the significance label text.}

\item{hide_ns}{Logical; hide non-significant comparison labels.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{seed}{The random seed to use. Default is 8525.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object, possibly faceted, with \code{height}
and \code{width} attributes (in inches) attached.
}
\description{
Core implementation for drawing box plots, violin plots, bar plots (mean
± error bars), or beeswarm plots.  This is the workhorse behind
\code{\link{BoxPlot}}, \code{\link{ViolinPlot}}, and
\code{\link{BeeswarmPlot}} — it takes a \strong{single} data frame (no
\code{split_by} support) and returns a \code{ggplot} object.

The \code{base} parameter selects the primary geometry:
\itemize{
\item \code{"box"} — \code{geom_boxplot()}
\item \code{"violin"} — \code{geom_violin()}
\item \code{"bar"} — \code{stat_summary(fun = mean, geom = "col")}
with optional error bars (SEM, SD, or CI).
\item \code{"none"} — no primary geometry (used by
\code{\link{BeeswarmPlot}} to draw beeswarm points alone).
}
}
\section{Architecture}{

\enumerate{
\item \strong{Wide-to-long conversion} — when \code{in_form = "wide"},
data is pivoted via \code{tidyr::pivot_longer()}.  The
\code{keep_na} / \code{keep_empty} lists are filtered to the new
column names.
\item \strong{Column resolution} — \code{x}, \code{y}, \code{group_by},
\code{facet_by}, and \code{paired_by} are validated via
\code{\link{check_columns}}.
\item \strong{NA / empty-level handling} — per-column
\code{keep_empty} settings are extracted for \code{x},
\code{group_by}, and \code{facet_by} independently.
\item \strong{Beeswarm validation} — if \code{add_beeswarm = TRUE},
the \code{ggbeeswarm} package is required.  Beeswarm is disabled
(with a warning) when \code{paired_by} is provided.
\item \strong{Paired data validation} — structural checks ensure each
(\code{x}, \code{paired_by}) combination has exactly 2
observations (one per group when \code{group_by} is present).
Paired observations force \code{add_point = TRUE}.
\item \strong{Summary statistics} — \code{.y_mean} and
\code{.y_median} are pre-computed per (\code{x}, \code{group_by},
\code{facet_by}) for use in trend lines and fill modes.
\item \strong{y-axis limits} — \code{y_max} / \code{y_min} accept
numeric values or quantile notation (\code{"q95"},
\code{"q5"}).  For bar plots, the limit is extended upward by
the error bar extent.
\item \strong{Highlight} — \code{highlight} can be \code{TRUE} (all
points), a numeric index vector, a logical expression string, or
a character vector of row names.
\item \strong{sort_x} — an R expression string (e.g.,
\code{"mean(y)"}) evaluated per x-level to reorder categories.
\item \strong{Flip transformation} — when \code{flip = TRUE}, factor
levels are reversed and \code{aspect.ratio} is inverted.
\item \strong{Base geometry} — the primary geom is added:
\code{geom_boxplot()}, \code{geom_violin()}, or
\code{stat_summary(fun = mean, geom = "col")}.  Error bars
(SEM / SD / CI) are layered on bar plots via a custom
\code{stat_summary(fun.data = ...)}.
\item \strong{Fill mode} — \code{fill_mode} controls colour mapping:
\itemize{
\item \code{"dodge"} — fill by \code{group_by} (discrete).
\item \code{"x"} — fill by x-axis categories (discrete).
\item \code{"mean"} / \code{"median"} — fill by pre-computed
mean/median (continuous gradient).
}
\item \strong{Box overlay} — when \code{add_box = TRUE} on a non-box
base, a box plot is overlaid via \code{ggnewscale::new_scale_fill()}
with a white fill/black outline.
\item \strong{Statistical comparisons} — two pathways:
\itemize{
\item \strong{Pairwise} (\code{comparisons}) — uses
\code{ggpubr::geom_pwc()} with automatic or explicit
comparison pairs.  Data is preprocessed to avoid test
failures from zero-variance or all-NA groups.
\item \strong{Multiple-group} (\code{multiplegroup_comparisons})
— uses \code{ggpubr::stat_compare_means()} for omnibus
tests (e.g., Kruskal-Wallis).
}
After comparison layers are added, \code{y_max_use} is expanded to
accommodate significance brackets.
\item \strong{Points} — jittered points (\code{geom_point()} with
\code{position_jitterdodge}) or beeswarm points
(\code{ggbeeswarm::geom_beeswarm()}).  Paired observations
add connecting lines (\code{geom_line()}) between matched
subjects.
\item \strong{Trend lines} — \code{stat_summary(fun = first)} draws
lines connecting group medians.  When \code{trend_color} is
\code{NULL} and \code{group_by} is present, lines are coloured
per group.
\item \strong{Reference lines} — \code{geom_hline()} at the specified
y-intercept.
\item \strong{Stat summary points} — a custom \code{stat_summary()}
point layer displaying a user-specified summary statistic
(e.g., mean) with a shape legend entry.
\item \strong{Stack layout} — when \code{stack = TRUE}, facets are
arranged with shared strip labels and negative panel spacing for
a compact stacked appearance.
\item \strong{Dimension calculation} — \code{calculate_plot_dimensions()}
accounts for the number of x-levels × dodge groups, flip state,
and stack layout adjustments.  Minimum dimensions are enforced
from label character widths.
\item \strong{Faceting} — \code{\link{facet_plot}()} wraps the result
with the appropriate strip position based on flip/stack state.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{calculate_plot_dimensions}
\alias{calculate_plot_dimensions}
\title{Calculate plot dimensions with aspect ratio consideration}
\usage{
calculate_plot_dimensions(
  base_height = 4.5,
  aspect.ratio = 1,
  n_x = NULL,
  n_y = NULL,
  x_scale_factor = 0.5,
  y_scale_factor = 0.5,
  legend.position = "right",
  legend.direction = "vertical",
  legend_n = 1,
  legend_nchar = 5,
  flip = FALSE,
  min_width = 3,
  min_height = 3,
  max_width = 12,
  max_height = 12
)
}
\arguments{
\item{base_height}{Base height for the plot (before legend adjustments). Default is 4.5.}

\item{aspect.ratio}{Aspect ratio (height/width). If NULL, width is calculated independently.}

\item{n_x}{Number of categories on x-axis (for width scaling)}

\item{n_y}{Number of categories on y-axis (for height scaling)}

\item{x_scale_factor}{Scaling factor per x-axis category. Default is 0.5.}

\item{y_scale_factor}{Scaling factor per y-axis category. Default is 0.5.}

\item{legend.position}{Position of legend ("none", "right", "left", "top", "bottom")}

\item{legend.direction}{Direction of legend ("vertical" or "horizontal")}

\item{flip}{Whether the plot is flipped (inverts aspect ratio)}

\item{min_width}{Minimum width in inches. Default is 3.}

\item{min_height}{Minimum height in inches. Default is 3.}

\item{max_width}{Maximum width in inches. Default is 12.}

\item{max_height}{Maximum height in inches. Default is 12.}
}
\value{
A list with height and width components
}
\description{
This function calculates plot height and width taking into account:
\itemize{
\item Content-based scaling (number of items on axes)
\item Aspect ratio constraints
\item Legend position and direction
\item Minimum and maximum dimension bounds
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roccurve.R
\name{ROCCurve}
\alias{ROCCurve}
\title{ROC curve}
\usage{
ROCCurve(
  data,
  truth_by,
  score_by,
  pos_label = NULL,
  split_by = NULL,
  split_by_sep = "_",
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  x_axis_reverse = FALSE,
  percent = FALSE,
  ci = NULL,
  n_cuts = 0,
  cutoffs_at = NULL,
  cutoffs_labels = NULL,
  cutoffs_accuracy = 0.001,
  cutoffs_pt_size = 5,
  cutoffs_pt_shape = 4,
  cutoffs_pt_stroke = 1,
  cutoffs_labal_fg = "black",
  cutoffs_label_size = 4,
  cutoffs_label_bg = "white",
  cutoffs_label_bg_r = 0.1,
  show_auc = c("auto", "none", "legend", "plot"),
  auc_accuracy = 0.01,
  auc_size = 4,
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  legend.position = waiver(),
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = ifelse(x_axis_reverse, "Specificity", "1 - Specificity"),
  ylab = "Sensitivity",
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  seed = 8525,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{truth_by}{A character string naming the column that contains the true
class labels (binary outcome, 0/1 or TRUE/FALSE).}

\item{score_by}{A character vector of column names containing the predicted
scores (classifier output values). When multiple columns are provided, each
column becomes a separate ROC curve grouped by a \code{.group} identifier.
When multiple columns are used, \code{group_by} must be NULL.}

\item{pos_label}{A character string specifying the positive class label in
\code{truth_by}. When NULL (default), the labels are handled by the
\code{plotROC} package: if \code{truth_by} is a factor, the last level is
used; otherwise it is coerced to a factor with a warning.}

\item{split_by}{The column(s) to split the data by and produce separate
ROC curve plots for each level. The \code{split_by} column is removed from
the per-split data to avoid interfering with ROC analysis. Multiple columns
are concatenated with \code{split_by_sep}.}

\item{split_by_sep}{A character string used to separate concatenated
\code{split_by} columns. Default: \code{"_"}.}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{group_name}{A character string to use as the legend title for the
ROC curve groups. When NULL (default), the \code{group_by} column name
is used.}

\item{x_axis_reverse}{A logical value. If TRUE, the x-axis is reversed (from
1 to 0), displaying specificity instead of 1 - specificity. The x-axis
label automatically changes to \code{"Specificity"}. Default: FALSE.}

\item{percent}{A logical value. If TRUE, the x and y axes are displayed as
percentages (0 to 100). Default: FALSE.}

\item{ci}{A list of arguments passed to \code{plotROC::geom_rocci()} to add
confidence intervals to the ROC curve. When NULL (default), no confidence
intervals are drawn. Example: \code{ci = list(sig.level = 0.05)}.}

\item{n_cuts}{An integer specifying the number of evenly-spaced quantile-based
cutoff points to annotate on the ROC curve. Quantiles are computed from the
\code{score_by} distribution. Default: \code{0} (no quantile cutoffs).
Ignored when \code{cutoffs_at} is non-NULL.}

\item{cutoffs_at}{A vector of user-supplied cutoff values to annotate as
points on the ROC curve. When non-NULL, overrides \code{n_cuts}. Accepts
raw numeric score thresholds and/or named method strings from the
\code{\link[OptimalCutpoints]{optimal.cutpoints}} package for automatic
optimal cutoff identification. Both \code{cutoffs_at} and
\code{cutoffs.labels} are passed to \code{plotROC::geom_roc()}.
Supported method values are:
\itemize{
\item \code{"CB"} (cost-benefit method);
\item \code{"MCT"} (minimises Misclassification Cost Term);
\item \code{"MinValueSp"} (a minimum value set for Specificity);
\item \code{"MinValueSe"} (a minimum value set for Sensitivity);
\item \code{"ValueSe"} (a value set for Sensitivity);
\item \code{"MinValueSpSe"} (a minimum value set for Specificity and Sensitivity);
\item \code{"MaxSp"} (maximises Specificity);
\item \code{"MaxSe"} (maximises Sensitivity);
\item \code{"MaxSpSe"} (maximises Sensitivity and Specificity simultaneously);
\item \code{"MaxProdSpSe"} (maximises the product of Sensitivity and Specificity);
\item \code{"ROC01"} (minimises distance between ROC plot and point (0,1));
\item \code{"SpEqualSe"} (Sensitivity = Specificity);
\item \code{"Youden"} (Youden Index);
\item \code{"MaxEfficiency"} (maximises Efficiency/Accuracy);
\item \code{"Minimax"} (minimises the most frequent error);
\item \code{"MaxDOR"} (maximises Diagnostic Odds Ratio);
\item \code{"MaxKappa"} (maximises Kappa Index);
\item \code{"MinValueNPV"} (a minimum value set for Negative Predictive Value);
\item \code{"MinValuePPV"} (a minimum value set for Positive Predictive Value);
\item \code{"ValueNPV"} (a value set for Negative Predictive Value);
\item \code{"ValuePPV"} (a value set for Positive Predictive Value);
\item \code{"MinValueNPVPPV"} (a minimum value set for Predictive Values);
\item \code{"PROC01"} (minimises distance between PROC plot and point (0,1));
\item \code{"NPVEqualPPV"} (Negative Predictive Value = Positive Predictive Value);
\item \code{"MaxNPVPPV"} (maximises Positive and Negative Predictive Values simultaneously);
\item \code{"MaxSumNPVPPV"} (maximises the sum of the Predictive Values);
\item \code{"MaxProdNPVPPV"} (maximises the product of Predictive Values);
\item \code{"ValueDLR.Negative"} (a value set for Negative Diagnostic Likelihood Ratio);
\item \code{"ValueDLR.Positive"} (a value set for Positive Diagnostic Likelihood Ratio);
\item \code{"MinPvalue"} (minimises p-value of the Chi-squared test);
\item \code{"ObservedPrev"} (closest value to observed prevalence);
\item \code{"MeanPrev"} (closest value to the mean of the test values);
\item \code{"PrevalenceMatching"} (predicted prevalence equals observed prevalence).
}}

\item{cutoffs_labels}{A character vector of user-supplied labels for the
cutoff points. Must be the same length as \code{cutoffs_at}. When NULL,
labels are generated automatically (score value or method name).}

\item{cutoffs_accuracy}{A numeric value controlling the rounding precision of
automatically generated cutoff labels. Default: \code{0.01}.}

\item{cutoffs_pt_size}{A numeric value specifying the size of the cutoff
point markers. Default: \code{5}.}

\item{cutoffs_pt_shape}{A numeric value specifying the shape of the cutoff
point markers. Default: \code{4} (cross).}

\item{cutoffs_pt_stroke}{A numeric value specifying the stroke width of the
cutoff point markers. Default: \code{1}.}

\item{cutoffs_labal_fg}{A character string specifying the text colour of
the cutoff labels. Default: \code{"black"}.}

\item{cutoffs_label_size}{A numeric value specifying the font size of the
cutoff labels. Default: \code{4}.}

\item{cutoffs_label_bg}{A character string specifying the background colour
of the cutoff labels. Default: \code{"white"}.}

\item{cutoffs_label_bg_r}{A numeric value specifying the background radius
of the cutoff labels (passed to \code{ggrepel::geom_text_repel()}).
Default: \code{0.1}.}

\item{show_auc}{A character string specifying the display mode for AUC values:
\itemize{
\item \code{"auto"} (default): Automatically determine the position. When
there is a single group or \code{facet_by} is provided, AUC is placed on
the plot; otherwise AUC is placed in the legend.
\item \code{"none"}: Do not display AUC values.
\item \code{"legend"}: Display AUC values in the legend labels.
\item \code{"plot"}: Display AUC values as text on the plot.
}}

\item{auc_accuracy}{A numeric value controlling the rounding precision of
AUC values in labels. Default: \code{0.01}.}

\item{auc_size}{A numeric value specifying the font size of AUC labels when
displayed on the plot. Default: \code{4}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{combine}{A logical value. When TRUE (default), the list of per-split
plots is combined into a single \code{patchwork} object with
\code{attr(p, "auc")} and \code{attr(p, "cutoffs")} containing the
aggregated results. When FALSE, returns a named list of individual
\code{ggplot} objects.}

\item{nrow, ncol}{Integer values specifying the number of rows and columns
in the combined plot layout. Passed to \code{\link[patchwork]{wrap_plots}}.}

\item{byrow}{A logical value. If TRUE (default), the combined layout is
filled row-wise. Passed to \code{\link[patchwork]{wrap_plots}}.}

\item{seed}{A numeric seed for reproducibility. Default: \code{8525}.
Passed to \code{\link{validate_common_args}()}.}

\item{axes}{A character string specifying how axes are treated across the
combined layout. Passed to \code{\link{combine_plots}()}. Options:
\code{"keep"}, \code{"collect"}, \code{"collect_x"}, \code{"collect_y"}.}

\item{axis_titles}{A character string specifying how axis titles are treated
across the combined layout. Defaults to \code{axes}. Passed to
\code{\link{combine_plots}()}.}

\item{guides}{A character string specifying how legends are collected across
panels in the combined layout. Passed to \code{\link{combine_plots}()}.}

\item{design}{A custom layout specification for the combined plot. Passed
to \code{\link{combine_plots}()}. When specified, \code{nrow},
\code{ncol}, and \code{byrow} are ignored.}

\item{...}{Additional arguments.}
}
\value{
A \code{patchwork} object (when \code{combine = TRUE}) with
\code{attr(p, "auc")} and \code{attr(p, "cutoffs")} data frames
containing aggregated AUC values and cutoff information across all
splits. When \code{combine = FALSE}, returns a named list of
\code{ggplot} objects, each with their own \code{attr(p[[i]], "auc")}
and \code{attr(p[[i]], "cutoffs")}.
}
\description{
Draws one or more Receiver Operating Characteristic (ROC) curves for
evaluating binary classifier performance. The function wraps
\code{\link{ROCCurveAtomic}} with \code{split_by} handling, providing the
ability to generate separate ROC curves per split level and combine them
via \code{\link[patchwork]{wrap_plots}}.
}
\details{
Key features:
\itemize{
\item \strong{Multiple classifiers} — compare several prediction scores
side-by-side by providing multiple \code{score_by} columns.
\item \strong{AUC display} — area under the curve values shown on the
plot or in the legend, with configurable precision.
\item \strong{Optimal cutoffs} — identify and annotate optimal cutoff
points using any of the 30+ methods from the \code{OptimalCutpoints}
package, or supply custom numeric thresholds.
\item \strong{Confidence intervals} — add ROC confidence bands via
\code{plotROC::geom_rocci()}.
\item \strong{Axis orientation} — reverse x-axis to show specificity or
display axes as percentages.
\item \strong{Splitting and faceting} — split data into sub-plots via
\code{split_by} or facet within a single plot via \code{facet_by}.
}
}
\section{split_by Workflow}{

When \code{split_by} is provided, the following pipeline executes:
\enumerate{
\item \strong{Validation} — \code{\link{validate_common_args}()} checks
the random seed and \code{facet_by} configuration.
\item \strong{Column resolution} — \code{\link{check_columns}()} resolves
\code{split_by} (force_factor, allow_multi, concat_multi).
\item \strong{Data splitting} — Unused factor levels in \code{split_by}
are dropped via \code{droplevels()}, and the data is split by
\code{split_by} levels (preserving factor level order). If
\code{split_by} is NULL, the data is wrapped in a single-element list
named \code{"..."}.
\item \strong{Per-split resolution} — \code{\link{check_palette}()},
\code{\link{check_palcolor}()}, and \code{\link{check_legend}()} resolve
per-split palette, colour, legend.position, and legend.direction
overrides.
\item \strong{Per-split dispatch} — For each split:
\itemize{
\item Title resolution: if \code{title} is a function, it receives
the split level name; otherwise \code{title \%||\% split_level} is
used.
\item The \code{split_by} column is removed from the per-split data
frame to avoid conflicts with the ROC analysis.
\item \code{\link{ROCCurveAtomic}()} is called with the per-split
palette, palcolor, legend.position, and legend.direction.
}
\item \strong{Combination} — \code{\link{combine_plots}()} assembles the
list of plots via \code{patchwork::wrap_plots}, honouring
\code{nrow}/\code{ncol}/\code{byrow}/\code{design}.
\item \strong{AUC / cutoff collection} — When \code{combine = TRUE}, the
per-split \code{auc} and \code{cutoffs} attributes are collected into
combined data frames with a \code{split_by} column identifying the source
split, and stored as \code{attr(p, "auc")} and
\code{attr(p, "cutoffs")}.
}
}

\examples{
set.seed(8525)

D.ex <- rbinom(200, size = 1, prob = .5)
M1 <- rnorm(200, mean = D.ex, sd = .65)
M2 <- rnorm(200, mean = D.ex, sd = 1.5)
gender <- c("Male", "Female")[rbinom(200, 1, .49) + 1]

data <- data.frame(D = D.ex, D.str = c("Healthy", "Ill")[D.ex + 1],
  gender = gender, M1 = M1, M2 = M2)

# --- Basic ROC curve ---
ROCCurve(data, truth_by = "D", score_by = "M1")

# --- Will warn about the positive label ---
ROCCurve(data, truth_by = "D.str", score_by = "M1")

# --- Decreasing direction ---
ROCCurve(data, truth_by = "D", score_by = "M1", increasing = FALSE)

# --- Multiple ROC curves (multiple classifiers) ---
ROCCurve(data, truth_by = "D", score_by = c("M1", "M2"), group_name = "Method")

# --- Grouping by a column ---
ROCCurve(data, truth_by = "D", score_by = "M1", group_by = "gender", show_auc = "plot")

# --- Reverse x-axis and display as percentages ---
ROCCurve(data, truth_by = "D", score_by = "M1", x_axis_reverse = TRUE, percent = TRUE)

# --- Custom n_cuts and single colour ---
ROCCurve(data, truth_by = "D", score_by = "M1", n_cuts = 10, palcolor = "black")

# --- Add confidence intervals ---
ROCCurve(data, truth_by = "D", score_by = "M1", ci = list(sig.level = .01))

# --- Facet by a column ---
ROCCurve(data, truth_by = "D", score_by = "M1", facet_by = "gender")

# --- Show cutoffs ---
ROCCurve(data, truth_by = "D", score_by = "M1", cutoffs_at = c(0, "ROC01", "SpEqualSe"))

# --- Split by a column ---
p <- ROCCurve(data, truth_by = "D", score_by = "M1", split_by = "gender",
   cutoffs_at = c(0.2, "MaxSpSe"))
p
# Retrieve the AUC values
attr(p, "auc")
# Retrieve the cutoffs
attr(p, "cutoffs")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/barplot.R
\name{SplitBarPlotAtomic}
\alias{SplitBarPlotAtomic}
\title{Atomic split bar plot (internal)}
\usage{
SplitBarPlotAtomic(
  data,
  x,
  y,
  y_sep = "_",
  flip = FALSE,
  alpha_by = NULL,
  alpha_reverse = FALSE,
  alpha_name = NULL,
  order_y = list(`+` = c("x_desc", "alpha_desc"), `-` = c("x_desc", "alpha_asc")),
  bar_height = 0.9,
  lineheight = 0.5,
  max_charwidth = 80,
  fill_by = NULL,
  fill_by_sep = "_",
  fill_name = NULL,
  direction_name = "direction",
  direction_pos_name = "positive",
  direction_neg_name = "negative",
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  palreverse = FALSE,
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  facet_by = NULL,
  facet_scales = "free_y",
  facet_nrow = NULL,
  facet_ncol = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  x_min = NULL,
  x_max = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  keep_empty = FALSE,
  keep_na = FALSE,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the numeric column for the
x-axis.  Values determine bar direction: positive → right, negative →
left.}

\item{y}{A character string (or vector) specifying the column(s) for the
category axis.  Each unique value becomes a bar.  Multiple columns are
concatenated with \code{y_sep}.}

\item{y_sep}{A character string to join multiple \code{y} columns.
Default \code{"_"}.}

\item{flip}{Logical; if \code{TRUE}, swap the x and y axes (bars become
vertical rather than horizontal).}

\item{alpha_by}{A character string naming a numeric column to encode as
bar opacity.  Default \code{NULL} (all bars fully opaque).}

\item{alpha_reverse}{Logical; if \code{TRUE}, reverse the alpha scale
direction (solid for low values, transparent for high).}

\item{alpha_name}{A character string for the alpha legend title.}

\item{order_y}{A named list controlling the vertical ordering of bars.
Keys are \code{"+"} (positive bars), \code{"-"} (negative bars), or
\code{"*"} (all bars).  Values are character vectors of ordering
criteria: \code{"x_asc"}, \code{"x_desc"}, \code{"alpha_asc"},
\code{"alpha_desc"}.  Default orders positive bars by descending x and
descending alpha; negative bars by descending x and ascending alpha.}

\item{bar_height}{A numeric value (0–1) specifying the bar height as a
fraction of the available category slot.}

\item{lineheight}{A numeric value controlling the line height of wrapped
category labels.}

\item{max_charwidth}{An integer specifying the maximum character width
for wrapping category labels.}

\item{fill_by}{A character string (or vector) naming the column(s) for
bar fill colour.  If \code{NULL}, the direction (positive/negative) is
used.  Can be categorical or numeric.}

\item{fill_by_sep}{A character string to join multiple \code{fill_by}
columns.  Default \code{"_"}.}

\item{fill_name}{A character string for the fill legend title.}

\item{direction_name}{A character string naming the internal direction
column (used in legends).  Default \code{"direction"}.}

\item{direction_pos_name}{A character string labelling the positive
direction in the legend.  Default \code{"positive"}.}

\item{direction_neg_name}{A character string labelling the negative
direction in the legend.  Default \code{"negative"}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{lower_quantile, upper_quantile}{Lower and upper quantiles for the continuous color/fill scale.
The actual cutoffs are determined by these quantiles when \code{lower_cutoff} and
\code{upper_cutoff} are \code{NULL}. Defaults: \code{lower_quantile = 0}, \code{upper_quantile = 0.99}.}

\item{lower_cutoff, upper_cutoff}{Explicit lower and upper cutoffs for the continuous color/fill scale.
When \code{NULL} (the default), the cutoffs are determined by \code{lower_quantile} and
\code{upper_quantile} via \code{\link[stats]{quantile}}. Values outside the
\code{[lower_cutoff, upper_cutoff]} range are clamped (winsorized) to the nearest cutoff value.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{x_min, x_max}{Numeric limits for the x-axis.  When \code{NULL},
symmetric limits are computed from the maximum absolute x-value.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object, possibly faceted, with \code{height}
and \code{width} attributes (in inches) attached.
}
\description{
Core implementation for drawing a split (divergent / waterfall-style) bar
plot.  Bars extend left (negative values) and right (positive values) from
a central zero line, with the y-axis listing categories.  Bars can be
coloured by a categorical or continuous fill variable, and their opacity
can encode an additional numeric variable via \code{alpha_by}.

This is the workhorse behind the exported \code{\link{SplitBarPlot}}
(also aliased as \code{\link{WaterfallPlot}}).
}
\section{Architecture}{

\enumerate{
\item \strong{Column resolution} — \code{x}, \code{y}, \code{fill_by},
\code{alpha_by}, and \code{facet_by} are validated via
\code{\link{check_columns}}.  Multi-column \code{y} and
\code{fill_by} are concatenated with their respective separators.
\item \strong{Direction assignment} — a \code{direction_name} column
is created from the sign of \code{x}, assigning each row to the
positive or negative group (customisable via
\code{direction_pos_name} / \code{direction_neg_name}).
\item \strong{Fill resolution} — if \code{fill_by} is \code{NULL},
the direction column is used as the fill (two-colour palette).
Categorical fills use a discrete colour scale; numeric fills use
\code{\link{prepare_continuous_color_scale}()} with quantile /
cutoff clamping and \code{scale_fill_gradientn()}.
\item \strong{Alpha by} — when \code{alpha_by} is provided, bar
opacity encodes an additional numeric variable via
\code{scale_alpha_continuous()}.
\item \strong{Ordering} — \code{order_y} is a named list controlling
the vertical ordering of bars.  The keys \code{"+"} and \code{"-"}
specify separate orderings for positive and negative bars; key
\code{"*"} applies a single ordering to all bars.  Values are
character vectors of ordering criteria: \code{"x_asc"},
\code{"x_desc"}, \code{"alpha_asc"}, \code{"alpha_desc"}.
\item \strong{Empty level padding} — when \code{keep_empty_y = TRUE},
missing y-levels are padded per facet (or globally) with
zero-height bars so they still appear on the axis.
\item \strong{Text labels} — category names are drawn beside the bars
via \code{geom_text()} at the zero line.  When flipped, labels are
rotated 90°.  Long labels are wrapped via
\code{\link[stringr]{str_wrap}()} using \code{max_charwidth}.
\item \strong{Plot assembly} — the ggplot is built with
\code{geom_vline(xintercept = 0)} for the centre line,
\code{geom_col()} for bars, and the appropriate fill / alpha
scales.
\item \strong{Dimension calculation} — \code{calculate_plot_dimensions()}
computes dimensions from the number of y categories, scaled by
\code{bar_height / 4}.  Flipping adjusts the aspect ratio.
\item \strong{Faceting} — \code{\link{facet_plot}()} wraps the result,
defaulting to \code{facet_scales = "free_y"}.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wordcloudplot.R
\name{WordCloudPlot}
\alias{WordCloudPlot}
\title{Word cloud plot}
\usage{
WordCloudPlot(
  data,
  word_by = NULL,
  sentence_by = NULL,
  count_by = NULL,
  score_by = NULL,
  count_name = NULL,
  score_name = NULL,
  split_by = NULL,
  split_by_sep = "_",
  words_excluded = plotthis::words_excluded,
  score_agg = mean,
  minchar = 2,
  word_size = c(2, 8),
  top_words = 100,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  alpha = 1,
  palreverse = FALSE,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{word_by}{A character string specifying the column name containing
pre-tokenized words. A character column is expected. Use this when your
data already has one word per row (or a list of words per row). Mutually
exclusive with \code{sentence_by}.}

\item{sentence_by}{A character string specifying the column name containing
sentences or phrases to be split into individual words. A character column
is expected. The text is lowercased and punctuation is removed before
splitting on whitespace boundaries. Mutually exclusive with
\code{word_by}.}

\item{count_by}{A character string specifying the numeric column for the
count or frequency of each word. When \code{NULL} (the default), each
occurrence counts as 1. Must be \code{NULL} when \code{sentence_by} is
used, as counts are derived from the number of occurrences after splitting.}

\item{score_by}{A character string specifying the numeric column for the
score of each word, mapped to the text colour via a continuous gradient.
When \code{NULL} (the default), all words receive a score of 1 and are
coloured at the low end of the palette.}

\item{count_name}{A character string for the size legend title. When
\code{NULL} (the default), \code{"Count"} is used.}

\item{score_name}{A character string for the colour-bar legend title. When
\code{NULL} (the default), \code{"Score"} is used.}

\item{split_by}{The column(s) to split data by and plot separately.
When \code{combine = TRUE}, the combined plot's data (\code{p$data})
contains the rows of every split, tagged with the \code{split_by}
column. For ggraph-based plots (e.g. \code{\link{Network}},
\code{\link{ClustreePlot}}), the node layout of every split is
exposed with the split column, and the original link rows (also
tagged with the \code{split_by} column) are available as the
\code{"edges"} attribute of the data.}

\item{split_by_sep}{The separator for multiple split_by columns. See \code{split_by}}

\item{words_excluded}{A character vector of words to exclude from the word
cloud. Matching is case-insensitive. Defaults to
\code{plotthis::words_excluded}, a built-in set of common English stop
words.}

\item{score_agg}{A function to aggregate the scores when multiple
observations of the same word exist. Default is \code{mean}. Other
options include \code{sum}, \code{median}, or a custom function.}

\item{minchar}{A numeric value specifying the minimum number of characters
a word must have to be included. Words with fewer characters are filtered
out. Default: \code{2}.}

\item{word_size}{A numeric vector of length 2 specifying the range of font
sizes (in mm) for the words. Passed to \code{scale_size(range =
 word_size)}. Default: \code{c(2, 8)}.}

\item{top_words}{A numeric value specifying the maximum number of words to
display, selected by highest score. Default: \code{100}.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{seed}{The random seed to use. Default is 8525.}

\item{combine}{Whether to combine the plots into one when facet is FALSE. Default is TRUE.}

\item{nrow}{A numeric value specifying the number of rows in the facet.}

\item{ncol}{A numeric value specifying the number of columns in the facet.}

\item{byrow}{A logical value indicating whether to fill the plots by row.}

\item{axes}{A string specifying how axes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axes in individual plots.
\item 'collect' will remove duplicated axes when placed in the same run of rows or columns of the layout.
\item 'collect_x' and 'collect_y' will remove duplicated x-axes in the columns or duplicated y-axes in the rows respectively.
}}

\item{axis_titles}{A string specifying how axis titltes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axis titles in individual plots.
\item 'collect' will remove duplicated titles in one direction and merge titles in the opposite direction.
\item 'collect_x' and 'collect_y' control this for x-axis titles and y-axis titles respectively.
}}

\item{guides}{A string specifying how guides should be treated in the layout. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'collect' will collect guides below to the given nesting level, removing duplicates.
\item 'keep' will stop collection at this level and let guides be placed alongside their plot.
\item 'auto' will allow guides to be collected if a upper level tries, but place them alongside the plot if not.
}}

\item{design}{Specification of the location of areas in the layout, passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE. When specified, \code{nrow}, \code{ncol}, and \code{byrow} are ignored.
See \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}} for more details.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object (when no \code{split_by} is used), a
\code{patchwork} object (when \code{combine = TRUE} and
\code{split_by} is used), or a named list of \code{ggplot} objects
(when \code{combine = FALSE}), each with \code{height} and
\code{width} attributes in inches.
}
\description{
Draws a word cloud plot that visualises word frequency and importance.
Words are displayed with font size proportional to a count variable and
colour based on a continuous score variable, using
\code{\link[ggwordcloud]{geom_text_wordcloud}} for rendering.

The function supports \strong{pre-tokenised words} (via \code{word_by})
and \strong{sentence splitting} (via \code{sentence_by}). Sentences are
automatically lowercased, stripped of punctuation, and split into
individual words before aggregation. Common stop words can be excluded
via \code{words_excluded}, and the number of displayed words is
controlled by \code{top_words}.

The word cloud can be \strong{faceted} (via \code{facet_by}) or
\strong{split} into separate sub-plots via \code{split_by}. When
\code{split_by} is used, each split level receives its own word cloud,
and the results are combined into a single layout via
\code{\link{combine_plots}}.
}
\section{split_by workflow}{

When \code{split_by} is provided:
\enumerate{
\item \code{\link{validate_common_args}()} validates the \code{seed}
and \code{facet_by} constraints (maximum 2 facet columns).
\item The \code{theme} argument is resolved via
\code{\link{process_theme}()}.
\item The \code{split_by} column(s) are validated and transformed via
\code{\link{check_columns}()} with \code{force_factor = TRUE}
and \code{concat_multi = TRUE}.
\item The data frame is split by \code{split_by} (preserving factor
level order). If \code{split_by} is \code{NULL}, the data is
wrapped in a single-element list with name \code{"..."}.
\item Per-split \code{palette}, \code{palcolor},
\code{legend.position}, and \code{legend.direction} are resolved
via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
and \code{\link{check_legend}()}.
\item \code{\link{WordCloudPlotAtomic}()} is called for each split. If
\code{title} is a function, it receives the split level name and
can generate dynamic titles per sub-plot.
\item Results are combined via \code{\link{combine_plots}()} (when
\code{combine = TRUE}) or returned as a named list (when
\code{combine = FALSE}).
}
}

\examples{
\donttest{
set.seed(8525)
data <- data.frame(
    word = c("apple", "banana", "cherry", "date", "elderberry",
             "fig", "grape", "honeydew", "kiwi", "lemon"),
    count = c(10, 20, 30, 40, 50, 15, 25, 35, 45, 55),
    score = c(1, 2, 3, 4, 5, 1.5, 2.5, 3.5, 4.5, 5.5),
    facet = rep(c("Group1", "Group2"), each = 5),
    split = rep(c("A", "B"), 5)
)

# Basic word cloud with word, count, and score columns
WordCloudPlot(data, word_by = "word",
              count_by = "count", score_by = "score")

# Word cloud using sentence_by (sentences split into words)
data_sent <- data.frame(
    sentence = c("The quick brown fox jumps over the lazy dog",
                 "A quick brown dog jumps over a lazy fox"),
    score = c(10, 5)
)
WordCloudPlot(data_sent, sentence_by = "sentence", score_by = "score")

# Word cloud with faceting
WordCloudPlot(data, word_by = "word",
              count_by = "count", score_by = "score",
              facet_by = "facet")

# Word cloud split by a grouping variable
WordCloudPlot(data, word_by = "word",
              count_by = "count", score_by = "score",
              split_by = "split")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_columns}
\alias{check_columns}
\title{Check the columns if columns found in the data}
\usage{
check_columns(
  df,
  columns,
  force_factor = FALSE,
  allow_multi = FALSE,
  concat_multi = FALSE,
  concat_sep = "_"
)
}
\arguments{
\item{df}{A data frame}

\item{columns}{A character vector of column names}

\item{force_factor}{Whether to force the columns to be factors}

\item{allow_multi}{Whether to allow multiple columns}

\item{concat_multi}{Whether to concatenate multiple columns}

\item{concat_sep}{The separator to use for concatenation}
}
\value{
A character string of the valid column
}
\description{
Check the columns if columns found in the data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theming.R
\name{theme_box}
\alias{theme_box}
\title{Box theme}
\usage{
theme_box(
  xlen_npc = 0.15,
  ylen_npc = 0.15,
  xlab = "",
  ylab = "",
  lab_size = 12,
  ...
)
}
\arguments{
\item{xlen_npc}{The length of the x-axis arrow in "npc".}

\item{ylen_npc}{The length of the y-axis arrow in "npc".}

\item{xlab}{x-axis label.}

\item{ylab}{y-axis label.}

\item{lab_size}{Label size.}

\item{...}{Arguments passed to the \code{\link[ggplot2]{theme}}.}
}
\value{
A ggplot2 theme.
}
\description{
This function creates a theme with all elements blank except for axis lines
like a box around the plot.
}
\examples{
library(ggplot2)
p <- ggplot(mtcars, aes(x = wt, y = mpg, colour = factor(cyl))) +
    geom_point()
p + theme_box()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{Heatmap}
\alias{Heatmap}
\title{Heatmap}
\usage{
Heatmap(
  data,
  values_by = NULL,
  values_fill = NA,
  name = NULL,
  in_form = c("auto", "matrix", "wide-columns", "wide-rows", "long"),
  split_by = NULL,
  split_by_sep = "_",
  rows_by = NULL,
  rows_by_sep = "_",
  rows_split_by = NULL,
  rows_split_by_sep = "_",
  columns_by = NULL,
  columns_by_sep = "_",
  columns_split_by = NULL,
  columns_split_by_sep = "_",
  rows_data = NULL,
  columns_data = NULL,
  keep_na = FALSE,
  keep_empty = FALSE,
  rows_orderby = NULL,
  columns_orderby = NULL,
  columns_name = NULL,
  columns_split_name = NULL,
  rows_name = NULL,
  rows_split_name = NULL,
  palette = "RdBu",
  palcolor = NULL,
  palreverse = FALSE,
  pie_size_name = "size",
  pie_size = NULL,
  pie_values = "length",
  pie_name = NULL,
  pie_group_by = NULL,
  pie_group_by_sep = "_",
  pie_palette = "Spectral",
  pie_palcolor = NULL,
  bars_sample = 1,
  label = identity,
  label_size = 10,
  label_color = "black",
  label_name = "label",
  mark = identity,
  mark_color = "black",
  mark_size = 1,
  mark_name = "mark",
  violin_fill = NULL,
  boxplot_fill = NULL,
  dot_size = 8,
  dot_size_name = "size",
  legend_items = NULL,
  legend_discrete = FALSE,
  legend.position = "right",
  legend.direction = "vertical",
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  add_bg = FALSE,
  bg_alpha = 0.5,
  add_reticle = FALSE,
  reticle_color = "grey",
  cluster_columns = NULL,
  cluster_rows = NULL,
  show_row_names = NULL,
  show_column_names = NULL,
  border = TRUE,
  title = NULL,
  column_title = NULL,
  row_title = NULL,
  na_col = "grey85",
  row_names_side = "right",
  column_names_side = "bottom",
  row_annotation = NULL,
  row_annotation_side = NULL,
  row_annotation_palette = NULL,
  row_annotation_palcolor = NULL,
  row_annotation_type = NULL,
  row_annotation_params = NULL,
  row_annotation_agg = NULL,
  column_annotation = NULL,
  column_annotation_side = NULL,
  column_annotation_palette = NULL,
  column_annotation_palcolor = NULL,
  column_annotation_type = NULL,
  column_annotation_params = NULL,
  column_annotation_agg = NULL,
  flip = FALSE,
  alpha = 1,
  seed = 8525,
  padding = 15,
  base_size = 1,
  aspect.ratio = NULL,
  draw_opts = list(),
  layer_fun_callback = NULL,
  cell_type = c("tile", "bars", "label", "mark", "label+mark", "mark+label", "dot",
    "violin", "boxplot", "pie"),
  cell_agg = NULL,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame or matrix.  When a matrix, it is melted to
long format internally (requires row and column names).}

\item{values_by}{A character of column name in \code{data} that contains the values to be plotted.
This is required when \code{in_form} is \code{"long"}. For other formats, the values are pivoted into a column named by \code{values_by}.}

\item{values_fill}{A value used to fill missing cells in the matrix.
Default \code{NA}.  Missing values prevent clustering when not filled.}

\item{name}{A character string to name the heatmap (will be used to rename \code{values_by}).}

\item{in_form}{The format of the data. Can be one of \code{"matrix"}, \code{"long"}, \code{"wide-rows"}, \code{"wide-columns"}, or \code{"auto"}.
Defaults to \code{"auto"}.}

\item{split_by}{A character of column name in \code{data} that contains the split information to split into multiple heatmaps.
This is used to create a list of heatmaps, one for each level of the split.
Defaults to \code{NULL}, meaning no split.}

\item{split_by_sep}{A character string to concat multiple columns in \code{split_by}.}

\item{rows_by}{A vector of column names in \code{data} that contains the row information.
This is used to create the rows of the heatmap.
When \code{in_form} is \code{"long"} or \code{"wide-columns"}, this is requied, and multiple columns can be specified,
which will be concatenated by \code{rows_by_sep} into a single column.}

\item{rows_by_sep}{A character string to concat multiple columns in \code{rows_by}.}

\item{rows_split_by}{A character of column name in \code{data} that contains the split information for rows.}

\item{rows_split_by_sep}{A character string to concat multiple columns in \code{rows_split_by}.}

\item{columns_by}{A vector of column names in \code{data} that contains the column information.
This is used to create the columns of the heatmap.
When \code{in_form} is \code{"long"} or \code{"wide-rows"}, this is required, and multiple columns can be specified,
which will be concatenated by \code{columns_by_sep} into a single column.}

\item{columns_by_sep}{A character string to concat multiple columns in \code{columns_by}.}

\item{columns_split_by}{A character of column name in \code{data} that contains the split information for columns.}

\item{columns_split_by_sep}{A character string to concat multiple columns in \code{columns_split_by}.}

\item{rows_data}{A data frame containing additional data for rows, which can be used to add annotations to the heatmap.
It will be joined to the main data by \code{rows_by} and \code{split_by} if \code{split_by} exists in \code{rows_data}.
This is useful for adding additional information to the rows of the heatmap.}

\item{columns_data}{A data frame containing additional data for columns, which can be used to add annotations to the heatmap.
It will be joined to the main data by \code{columns_by} and \code{split_by} if \code{split_by} exists in \code{columns_data}.
This is useful for adding additional information to the columns of the heatmap.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{rows_orderby}{A expression (in character) to specify how to order rows. It will be evaluated in the context of the data frame used for rows (after grouping by rows_split_by and rows_by). The expression should return a vector of the same length as the number of rows in the data frame. The default is NULL, which means no specific ordering.
Can't be used with cluster_rows = TRUE.
This is applied before renaming rows_by to rows_name.}

\item{columns_orderby}{A expression (in character) to specify how to order columns. It will be evaluated in the context of the data frame used for columns (after grouping by columns
split_by and columns_by). The expression should return a vector of the same length as the number of rows in the data frame. The default is NULL, which means no specific ordering.
Can't be used with cluster_columns = TRUE.
This is applied before renaming columns_by to columns_name.}

\item{columns_name}{A character string to rename the column created by \code{columns_by}, which will be reflected in the name of the annotation or legend.}

\item{columns_split_name}{A character string to rename the column created by \code{columns_split_by}, which will be reflected in the name of the annotation or legend.}

\item{rows_name}{A character string to rename the column created by \code{rows_by}, which will be reflected in the name of the annotation or legend.}

\item{rows_split_name}{A character string to rename the column created by \code{rows_split_by}, which will be reflected in the name of the annotation or legend.}

\item{palette}{A character string naming a palette (see
\code{\link{show_palettes}}) or a character vector of colours for the
main heatmap colour scale.  Default \code{"RdBu"}.}

\item{palcolor}{A custom colour vector overriding \code{palette}.}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{pie_size_name}{Legend title for the pie size.}

\item{pie_size}{A numeric value or function returning the pie radius.
When a function, it receives the count of groups in the pie.}

\item{pie_values}{A function or string (convertible via
\code{\link[base]{match.arg}}) to compute the value represented by
each pie slice.  Default \code{"length"} counts observations per
group.}

\item{pie_name}{A character string to rename the column created by \code{pie_group_by}, which will be reflected in the name of the annotation or legend.}

\item{pie_group_by}{A character of column name in \code{data} that contains the group information for pie charts.
This is used to create pie charts in the heatmap when \code{cell_type} is \code{"pie"}.}

\item{pie_group_by_sep}{A character string to concat multiple columns in \code{pie_group_by}.}

\item{pie_palette, pie_palcolor}{Palette and custom colours for pie slice
fill colours.}

\item{bars_sample}{Fraction of each cell's observations (0 < x <= 1;
\code{1} uses all data) or a whole count > 1 sampled per cell when
\code{cell_type = "bars"}.  Column widths are proportional to the
number of bars drawn per column, so a numeric count gives equal
column widths and a fraction gives widths proportional to the
column data sizes.  Proportional widths are skipped when
\code{columns_split_by} is given, and column clustering is
disabled in proportional mode.  Default 1 (all data).}

\item{label}{A function to compute text labels when
\code{cell_type = "label"} (or \code{"label+mark"}).  Receives the
aggregated value for a cell and optionally row/column indices and
names.  See below for the full dispatch contract.}

\item{label_size}{Default point size for label text (used as fallback
when the \code{label} function does not return a \code{size} field).}

\item{label_color}{Default colour for label text (fallback).}

\item{label_name}{Legend title for the label colour scale.  The legend
is shown automatically when the \code{label} function returns a
\code{legend} field for at least one cell.}

\item{mark}{A function to compute mark symbols when
\code{cell_type = "mark"} (or \code{"label+mark"}).  Same dispatch
contract as \code{label}.}

\item{mark_color}{Default mark colour (fallback).}

\item{mark_size}{Default mark stroke width (\code{lwd}) in pt (fallback).}

\item{mark_name}{Legend title for the mark colour scale.}

\item{violin_fill}{A character vector of colours to use as fill for
violin plots when \code{cell_type = "violin"}.  If \code{NULL}, the
annotation colour is used.}

\item{boxplot_fill}{A character vector of colours to use as fill for
boxplots when \code{cell_type = "boxplot"}.  If \code{NULL}, the
annotation colour is used.}

\item{dot_size}{Dot size when \code{cell_type = "dot"}.  Can be a
numeric value or a function.}

\item{dot_size_name}{Legend title for the dot size.}

\item{legend_items}{A named numeric vector specifying custom legend
entries for the main colour scale.  Names become the displayed labels.}

\item{legend_discrete}{Logical; if \code{TRUE}, treat the main colour
scale as discrete.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{lower_quantile, upper_quantile, lower_cutoff, upper_cutoff}{Quantile or explicit cutoffs for clipping the colour scale.  Applied
to aggregated values for \code{tile} / \code{label} cell types; applied
to raw values for \code{bars} / \code{violin} / \code{boxplot} types.}

\item{add_bg}{Logical; if \code{TRUE}, add a background fill behind
non-tile cell types.  Not used for \code{cell_type = "tile"} or
\code{"bars"}.}

\item{bg_alpha}{Numeric in \eqn{[0, 1]} for background transparency.}

\item{add_reticle}{Logical; if \code{TRUE}, draw a reticle (crosshair
pattern) over the heatmap.}

\item{reticle_color}{Colour for the reticle lines.}

\item{cluster_columns}{Logical; cluster the columns.  If \code{TRUE} and
\code{columns_split_by} is provided, clustering is applied within each
split group.}

\item{cluster_rows}{Logical; cluster the rows.  If \code{TRUE} and
\code{rows_split_by} is provided, clustering is applied within each
split group.}

\item{show_row_names}{Logical or character vector; show row names.  If
\code{TRUE}, the legend of the row group annotation is hidden.
\code{FALSE} only hides the in-place names.  Alternatively, a character
vector combining the display modes \code{"inplace"} (in-place names,
same as \code{TRUE}), \code{"legend"} (row group annotation with
legend), \code{"simple"} (row group annotation without legend),
\code{"anno"} (alias \code{"annotation"}; label annotation showing the
row names as text) and \code{"none"} (no names, annotation, or legend).
Modes are lower-priority defaults: explicit \code{row_annotation}
entries (e.g. \code{.row}) still win.}

\item{show_column_names}{Logical or character vector; show column names.
See \code{show_row_names} for the display modes.}

\item{border}{A logical value indicating whether to draw borders around
the heatmap.  If \code{TRUE}, slice borders are also drawn.  Default
\code{TRUE}.}

\item{title}{The global (column) title of the heatmap.}

\item{column_title}{Character string/vector used as the column group
annotation title.  A character vector consisting entirely of the display
modes \code{"inplace"} (per-slice split titles), \code{"legend"} (split
annotation with legend, no title), \code{"simple"} (split annotation,
no legend/title), \code{"anno"} (alias \code{"annotation"}; label block
annotation) and \code{"none"} (no title, annotation, or legend)
controls how the split titles are displayed.  A literal title identical
to a mode keyword is interpreted as a mode.}

\item{row_title}{Character string/vector used as the row group
annotation title.  See \code{column_title} for the display modes.}

\item{na_col}{Colour for \code{NA} cells.  Default \code{"grey85"}.}

\item{row_names_side}{Side for row names.  Default \code{"right"}.}

\item{column_names_side}{Side for column names.  Default \code{"bottom"}.}

\item{row_annotation}{A structured list specifying row annotations.
Same format as \code{column_annotation}.  Sides default to
\code{"left"}.  Aliases: \code{.row}/\code{.rows} for
\code{rows_by}, \code{.row.split}/\code{.rows.split} for
\code{rows_split_by}.}

\item{row_annotation_side}{\strong{Deprecated}: use
\code{row_annotation} with the \code{side} sub-key instead.}

\item{row_annotation_palette}{\strong{Deprecated}: use
\code{row_annotation} with the \code{palette} sub-key instead.}

\item{row_annotation_palcolor}{\strong{Deprecated}: use
\code{row_annotation} with the \code{palcolor} sub-key instead.}

\item{row_annotation_type}{\strong{Deprecated}: use
\code{row_annotation} with the \code{type} sub-key instead.}

\item{row_annotation_params}{\strong{Deprecated}: use
\code{row_annotation} with the \code{params} sub-key instead.}

\item{row_annotation_agg}{\strong{Deprecated}: use
\code{row_annotation} with the \code{agg} sub-key instead.}

\item{column_annotation}{A structured list specifying column annotations.
Each entry is a named list with sub-keys:
\describe{
\item{\code{col}}{Column name in \code{data} supplying the annotation
values.  If omitted, the entry name is used as the column name.}
\item{\code{side}}{\code{"top"} or \code{"bottom"}.}
\item{\code{palette}}{Palette name (see \code{\link{show_palettes}}).}
\item{\code{palcolor}}{Custom colour vector overriding \code{palette}.}
\item{\code{type}}{Annotation type: \code{"auto"}, \code{"simple"},
\code{"pie"}, \code{"ring"}, \code{"bar"}, \code{"violin"},
\code{"boxplot"}, \code{"density"}, \code{"label"},
\code{"rownames"}, \code{"points"}, \code{"lines"}.
For split annotations, \code{"rownames"} (row splits) /
\code{"colnames"} (column splits) is like \code{"label"} but labels
each split block with the concatenated row/column names;
\code{"names"} and \code{"dimnames"} are aliases.
\code{params$sep} (default \code{" "}) controls the separator and
\code{params$wrap_by} the number of names per line.
For name annotations (the built-in \code{rows_by}/\code{columns_by}
annotation), \code{"rownames"}/\code{"colnames"} renders the
row/column names as text labels (this is what
\code{show_row_names = "anno"} uses).}
\item{\code{params}}{A list of additional parameters passed to the
annotation constructor.  \code{FALSE} disables the annotation.
\code{$show_legend} controls legend visibility.  See
\code{\link[ComplexHeatmap]{HeatmapAnnotation}}.}
\item{\code{agg}}{A function to aggregate values for the annotation.}
\item{\code{name}}{Display name for the annotation, shown next to the
annotation bar and used as the legend title.  \code{FALSE} hides the
displayed name.  Defaults to the annotation key (column name).}
}
\strong{Shortcuts:}
\itemize{
\item \code{column_annotation = list(Score = "score")} is short for
\code{list(Score = list(col = "score"))}.
\item \code{column_annotation = TRUE} enables annotations with
defaults.  \code{FALSE} disables all column annotations.
}
\strong{Special keys:}
\itemize{
\item \code{.default} — default values inherited by all entries.
\code{params} is merged recursively; other keys are inherited
only when the entry does not already specify them.
\item \code{.col} / \code{.cols} / \code{.column} / \code{.columns} —
alias for \code{columns_by} (the built-in name annotation).
\item \code{.col.split} / \code{.cols.split} / \code{.column.split} /
\code{.columns.split} — alias for \code{columns_split_by}
(the built-in split annotation).
\item \code{.row} / \code{.rows} — alias for \code{rows_by}.
\item \code{.row.split} / \code{.rows.split} — alias for
\code{rows_split_by}.
}}

\item{column_annotation_side}{\strong{Deprecated}: use
\code{column_annotation} with the \code{side} sub-key instead.}

\item{column_annotation_palette}{\strong{Deprecated}: use
\code{column_annotation} with the \code{palette} sub-key instead.}

\item{column_annotation_palcolor}{\strong{Deprecated}: use
\code{column_annotation} with the \code{palcolor} sub-key instead.}

\item{column_annotation_type}{\strong{Deprecated}: use
\code{column_annotation} with the \code{type} sub-key instead.}

\item{column_annotation_params}{\strong{Deprecated}: use
\code{column_annotation} with the \code{params} sub-key instead.}

\item{column_annotation_agg}{\strong{Deprecated}: use
\code{column_annotation} with the \code{agg} sub-key instead.}

\item{flip}{Logical; if \code{TRUE}, swap rows and columns
transparently.  The caller does not need to swap row- and
column-related arguments manually.}

\item{alpha}{Alpha transparency for heatmap cells in \eqn{[0, 1]}.}

\item{seed}{The random seed to use. Default is 8525.}

\item{padding}{Padding around the heatmap in CSS order (top, right,
bottom, left).  Supports 1–4 values.  Default 15 (mm).  Note that
this is different from \code{ComplexHeatmap::draw()}'s \code{padding}
argument which uses bottom-left-top-right order.}

\item{base_size}{A positive numeric scalar used as a scaling factor for
the overall heatmap size.  Default 1 (no scaling).  Values > 1 enlarge
all cell dimensions proportionally.}

\item{aspect.ratio}{Height-to-width ratio of a single heatmap cell.
When \code{NULL} (default), sensible per-\code{cell_type} defaults are
used: 1 for \code{tile}/\code{label}/\code{dot}, 0.5 for \code{bars},
and 2 for \code{violin}/\code{boxplot}/\code{pie}.  The ratio is
constrained by the overall plot dimensions.}

\item{draw_opts}{A named list of additional arguments passed to
\code{\link[ComplexHeatmap]{draw,HeatmapList-method}}.  Internally
managed arguments take precedence.}

\item{layer_fun_callback}{A function to add custom graphical layers on
top of each heatmap cell.  Receives \code{j}, \code{i}, \code{x},
\code{y}, \code{w}, \code{h}, \code{fill}, \code{sr}, \code{sc}.
See \code{\link[ComplexHeatmap]{Heatmap}} for details.}

\item{cell_type}{The type of cell to render.  One of \code{"tile"}
(default), \code{"bars"}, \code{"label"}, \code{"mark"},
\code{"label+mark"} (or \code{"mark+label"}), \code{"dot"},
\code{"violin"}, \code{"boxplot"}, \code{"pie"}.  See the
\strong{Cell types} section for details.}

\item{cell_agg}{A function to aggregate values within each cell when
\code{cell_type = "tile"} or \code{"label"}.  Default is
\code{\link[base]{mean}}.}

\item{combine}{Whether to combine the plots into one when facet is FALSE. Default is TRUE.}

\item{nrow}{A numeric value specifying the number of rows in the facet.}

\item{ncol}{A numeric value specifying the number of columns in the facet.}

\item{byrow}{A logical value indicating whether to fill the plots by row.}

\item{axes}{A string specifying how axes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axes in individual plots.
\item 'collect' will remove duplicated axes when placed in the same run of rows or columns of the layout.
\item 'collect_x' and 'collect_y' will remove duplicated x-axes in the columns or duplicated y-axes in the rows respectively.
}}

\item{axis_titles}{A string specifying how axis titltes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axis titles in individual plots.
\item 'collect' will remove duplicated titles in one direction and merge titles in the opposite direction.
\item 'collect_x' and 'collect_y' control this for x-axis titles and y-axis titles respectively.
}}

\item{guides}{A string specifying how guides should be treated in the layout. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'collect' will collect guides below to the given nesting level, removing duplicates.
\item 'keep' will stop collection at this level and let guides be placed alongside their plot.
\item 'auto' will allow guides to be collected if a upper level tries, but place them alongside the plot if not.
}}

\item{design}{Specification of the location of areas in the layout, passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE. When specified, \code{nrow}, \code{ncol}, and \code{byrow} are ignored.
See \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}} for more details.}

\item{...}{Additional arguments passed to
\code{\link{HeatmapAtomic}}, which in turn forwards them to
\code{\link[ComplexHeatmap]{Heatmap}}.}
}
\value{
A \code{patchwork} object (class \code{wrap_plots}) with
\code{height} and \code{width} attributes (in inches).  When
\code{combine = FALSE}, a named list of such objects, one per
\code{split_by} level.
}
\description{
Draw a heatmap to visualise data in matrix form.  This is the public,
exported interface — it accepts data in multiple input formats (matrix,
wide, or long), preprocesses it via \code{\link{process_heatmap_data}},
and delegates to \code{\link{HeatmapAtomic}} for rendering.  Commonly
used in biology to visualise gene expression, but applicable to any
matrix-structured data.
}
\section{Input formats}{

The \code{in_form} parameter controls how the input \code{data} is
interpreted:
\itemize{
\item \code{"auto"} (default) — detects the format automatically.
\item \code{"matrix"} — \code{data} is a matrix with row and column
names.  It is melted to long form internally.
\item \code{"wide-rows"} — each row is a feature, columns are samples.
\item \code{"wide-columns"} — each column is a feature, rows are
samples.
\item \code{"long"} — tidy/long format with one observation per row.
}
}

\section{Split-by support}{

When \code{split_by} is provided, the data is partitioned into subsets
and an independent heatmap is produced for each level.  Results are
combined via \code{\link[patchwork]{wrap_plots}} according to
\code{nrow}, \code{ncol}, \code{byrow}, and \code{design}.  Per-split
\code{palette}, \code{palcolor}, \code{legend.position}, and
\code{legend.direction} can be specified as named lists keyed by split
level.
}

\examples{
\donttest{
set.seed(8525)

matrix_data <- matrix(rnorm(60), nrow = 6, ncol = 10)
rownames(matrix_data) <- paste0("R", 1:6)
colnames(matrix_data) <- paste0("C", 1:10)
if (requireNamespace("cluster", quietly = TRUE)) {
    Heatmap(matrix_data)
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # use a different color palette
    # change the main legend title
    # show row names (legend will be hidden)
    # show column names
    # change the row name annotation name and side
    # change the column name annotation name
    Heatmap(matrix_data, palette = "viridis", values_by = "z-score",
       show_row_names = TRUE, show_column_names = TRUE,
       rows_name = "Features", row_names_side = "left",
       columns_name = "Samples")
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # flip the heatmap
    Heatmap(matrix_data, palette = "viridis", values_by = "z-score",
       show_row_names = TRUE, show_column_names = TRUE,
       rows_name = "Features", row_names_side = "left",
       columns_name = "Samples", flip = TRUE)
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # add annotations to the heatmap
    rows_data <- data.frame(
       rows = paste0("R", 1:6),
       group = sample(c("X", "Y", "Z"), 6, replace = TRUE)
    )
    Heatmap(matrix_data, rows_data = rows_data,
        row_annotation = list(Group = list(col = "group", palette = "Spectral"))
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    Heatmap(matrix_data, rows_data = rows_data,
        rows_split_by = "group"
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # use label annotation for split groups (shows group labels inside colored blocks)
    Heatmap(matrix_data, rows_data = rows_data,
        rows_split_by = "group",
        row_annotation = list(.row.split = list(
            type = "label",
            params = list(
                border = FALSE,
                labels_gp = grid::gpar(col = "white", fontsize = 12),
                labels_rot = 0
            )
        ))
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # label annotation for column splits
    columns_data <- data.frame(
        columns = paste0("C", 1:10),
        batch = rep(c("A", "B"), each = 5)
    )
    Heatmap(matrix_data, columns_data = columns_data,
        columns_split_by = "batch",
        column_annotation = list(.col.split = list(type = "label"))
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # "rownames" annotation for row splits: show the concatenated row names
    # of each split group, wrapped every 3 names (row names and the split
    # title are hidden/shown by default accordingly)
    Heatmap(matrix_data, rows_data = rows_data,
        rows_split_by = "group",
        row_annotation = list(.row.split = list(
            type = "rownames",
            params = list(sep = ", ", wrap_by = 3)
        ))
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # same for column splits with type = "colnames"
    Heatmap(matrix_data, columns_data = columns_data,
        columns_split_by = "batch",
        column_annotation = list(.col.split = list(
            type = "colnames",
            params = list(wrap_by = 3)
        ))
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # split title modes: per-slice titles plus the split annotation
    # legend; and "anno" mode using label blocks instead of titles
    Heatmap(matrix_data, rows_data = rows_data,
        rows_split_by = "group",
        row_title = c("inplace", "legend")
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    Heatmap(matrix_data, rows_data = rows_data,
        rows_split_by = "group",
        row_title = "anno"
    )
}
rownames(matrix_data)[1] <- "R12345"
if (requireNamespace("cluster", quietly = TRUE)) {
    # label annotation for name annotations: show row/column names as colored labels
    Heatmap(matrix_data, rows_data = rows_data,
        row_annotation = list(.row = list(
            type = "label", palette = "Set2", side = "right",
            params = list(labels_rot = 150)
        )),
        column_annotation = list(.col = list(
            type = "label", params = list(labels_rot = 90)
        ))
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # add labels to the heatmap
    Heatmap(matrix_data, rows_data = rows_data,
        rows_split_by = "group", cell_type = "label",
        base_size = 0.8,
        label = function(x) ifelse(
            x > 0, scales::number(x, accuracy = 0.01), NA
        )
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # add labels based on an external data
    pvalues <- matrix(runif(60, 0, 0.5), nrow = 6, ncol = 10)
    Heatmap(matrix_data, rows_data = rows_data,
        rows_split_by = "group", cell_type = "label",
        base_size = 0.8,
        label = function(x, i, j) {
            pv <- ComplexHeatmap::pindex(pvalues, i, j)
            ifelse(pv < 0.01, "***",
            ifelse(pv < 0.05, "**",
            ifelse(pv < 0.1, "*", NA)))
        }
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # Set label color, size, legend and order
    pvalues <- matrix(runif(60, 0, 0.5), nrow = 6, ncol = 10)
    Heatmap(matrix_data, rows_data = rows_data,
        rows_split_by = "group", cell_type = "label",
        base_size = 0.6,
        label_name = "Significance",
        label = function(x, i, j) {
            pv <- ComplexHeatmap::pindex(pvalues, i, j)
            if (pv < 0.01)
               list("***", color = "red", size = 12, legend = "p < 0.01", order = 1)
            else if (pv < 0.05)
               list("**", color = "orange", size = 10, legend = "p < 0.05", order = 3)
            else if (pv < 0.1)
               list("*", color = "yellow", size = 8, legend = "p < 0.1", order = 2)
            else NA
        }
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # add marks
    Heatmap(matrix_data, rows_data = rows_data,
        rows_split_by = "group", cell_type = "mark",
        mark = function(x, i, j) {
            pv <- ComplexHeatmap::pindex(pvalues, i, j)
            if(pv < 0.01) list("[x]", legend = "p < 0.01")
            else if (pv < 0.02) list("[o]", legend = "p < 0.02")
            else if (pv < 0.03) list("[-]", legend = "p < 0.03")
            else if (pv < 0.05) list("[()]", legend = "p < 0.05")
            else if (pv < 0.06) list("+", legend = "p < 0.06")
            else if (pv < 0.07) list("x", legend = "p < 0.07")
            else if (pv < 0.08) list("[/]", legend = "p < 0.08")
            else if (pv < 0.09) list("[\\\\]", legend = "p < 0.09")
            else NA
        }
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # add labels and marks
    Heatmap(matrix_data, rows_data = rows_data,
        rows_split_by = "group", cell_type = "mark+label",
        label = scales::label_number(accuracy = 0.01),
        mark = function(x, i, j) {
            pv <- ComplexHeatmap::pindex(pvalues, i, j)
            if(pv < 0.01) list("{}", legend = "p < 0.01")
            else if(pv < 0.05) list("[]", legend = "p < 0.05")
            else NA
        },
        mark_size = 1.5, mark_color = "red"
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # quickly simulate a GO board
    go <- matrix(sample(c(0, 1, NA), 81, replace = TRUE), ncol = 9)

    Heatmap(
        go,
        # Do not cluster rows and columns and hide the name annotations
        # Use .row/.col aliases to disable the built-in name annotations
        cluster_rows = FALSE, cluster_columns = FALSE,
        row_annotation = list(.row = list(params = FALSE)),
        column_annotation = list(.col = list(params = FALSE)),
        show_row_names = FALSE, show_column_names = FALSE,
        # Set the legend items
        values_by = "Players", legend_discrete = TRUE,
        legend_items = c("Player 1" = 0, "Player 2" = 1),
        # Set the pawns
        cell_type = "dot", dot_size = function(x) ifelse(is.na(x), 0, 10),
        dot_size_name = NULL,  # hide the dot size legend
        palcolor = c("white", "black"),
        # Set the board
        add_reticle = TRUE,
        # Set the size of the board
        width = ggplot2::unit(105, "mm"), height = ggplot2::unit(105, "mm"))
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # Make the row/column name annotation thicker using the .row/.col aliases
    Heatmap(matrix_data,
        column_annotation = list(.col = list(params = list(height = 5))),
        row_annotation = list(.row = list(params = list(width = 5))))
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # Per-annotation side control: row name annotation on the right,
    # all other row annotations on the left (.default)
    rows_data2 <- data.frame(
        rows = sample(paste0("R", 1:6), 60, replace = TRUE),
        group = sample(c("X", "Y"), 60, replace = TRUE),
        score = runif(60)
    )
    Heatmap(matrix_data, rows_data = rows_data2,
        rows_split_by = "group",
        row_annotation = list(
            .default = list(side = "left"),
            .row = list(side = "right"),
            Score = "score"
        ),
        show_row_names = TRUE
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # Move all row annotations to the right side
    Heatmap(matrix_data, rows_data = rows_data2,
        rows_split_by = "group",
        row_annotation = list(
            .default = list(side = "right"),
            Score = "score"
        ),
        show_row_names = TRUE
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # Split and name annotations on opposite sides:
    # split annotation on the default left, name annotation on the right
    Heatmap(matrix_data, rows_data = rows_data2,
        rows_split_by = "group",
        row_annotation = list(
            .default = list(side = "left"),
            .row = list(side = "right")
        ),
        show_row_names = TRUE
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # Row name label annotation on the right side (text rotated 90° clockwise)
    Heatmap(matrix_data, rows_data = rows_data2,
        row_annotation = list(.row = list(
            type = "label", palette = "Set2", side = "right"
        )),
        show_row_names = TRUE
    )
}

# Use long form data
N <- 500
data <- data.frame(
    value = rnorm(N),
    c = sample(letters[1:8], N, replace = TRUE),
    r = sample(LETTERS[1:5], N, replace = TRUE),
    p = sample(c("x", "y"), N, replace = TRUE),
    q = sample(c("X", "Y", "Z"), N, replace = TRUE),
    a = as.character(sample(1:5, N, replace = TRUE)),
    p1 = runif(N),
    p2 = runif(N)
)

if (requireNamespace("cluster", quietly = TRUE)) {
    Heatmap(data, rows_by = "r", columns_by = "c", values_by = "value",
        rows_split_by = "p", columns_split_by = "q", show_column_names = TRUE)
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # display modes: in-place row names plus the row group annotation
    # legend; and "anno" mode showing the column names as text labels
    Heatmap(data, rows_by = "r", columns_by = "c", values_by = "value",
        show_row_names = c("inplace", "legend"),
        show_column_names = "anno"
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # split into multiple heatmaps
    Heatmap(data,
        values_by = "value", columns_by = "c", rows_by = "r", split_by = "p",
        upper_cutoff = 2, lower_cutoff = -2, legend.position = c("none", "right"),
        design = "AAAAAA#BBBBBBB"
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # cell_type = "bars" (default is "tile")
    Heatmap(data, values_by = "value", rows_by = "r", columns_by = "c",
        cell_type = "bars")
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # column widths are proportional to the number of bars per column
    # (columns c1 and c2 have 10 and 4 data points)
    bars_data <- data.frame(
        value = rnorm(14),
        r = c(rep("r1", 8), rep("r2", 6)),
        c = c(rep("c1", 6), rep("c2", 2), rep("c1", 4), rep("c2", 2))
    )
    Heatmap(bars_data, values_by = "value", rows_by = "r", columns_by = "c",
        cell_type = "bars")
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # a fraction uses that fraction of each cell's data
    Heatmap(bars_data, values_by = "value", rows_by = "r", columns_by = "c",
        cell_type = "bars", bars_sample = 0.5)
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # a whole count samples that many per cell -> equal column widths
    Heatmap(bars_data, values_by = "value", rows_by = "r", columns_by = "c",
        cell_type = "bars", bars_sample = 3)
}
if (requireNamespace("cluster", quietly = TRUE)) {
    p <- Heatmap(data, values_by = "value", rows_by = "r", columns_by = "c",
        cell_type = "dot", dot_size = length, dot_size_name = "data points",
        add_bg = TRUE, add_reticle = TRUE)
    p
}
if (requireNamespace("cluster", quietly = TRUE)) {
    dot_size_data <- as.matrix(p$data)
    # Make it big so we can see if we get the right indexing
    # for dot_size function
    dot_size_data["A", "a"] <- max(dot_size_data) * 2

    Heatmap(data, values_by = "value", rows_by = "r", columns_by = "c",
        cell_type = "dot", dot_size_name = "data points",
        dot_size = function(x, i, j) ComplexHeatmap::pindex(dot_size_data, i, j),
        show_row_names = TRUE, show_column_names = TRUE,
        add_bg = TRUE, add_reticle = TRUE)
}
if (requireNamespace("cluster", quietly = TRUE)) {
    Heatmap(data, values_by = "value", rows_by = "r", columns_by = "c",
        cell_type = "pie", pie_group_by = "q", pie_size = sqrt,
        add_bg = TRUE, add_reticle = TRUE)
}
if (requireNamespace("cluster", quietly = TRUE)) {
    Heatmap(data, values_by = "value", rows_by = "r", columns_by = "c",
        cell_type = "violin", add_bg = TRUE, add_reticle = TRUE)
}
if (requireNamespace("cluster", quietly = TRUE)) {
    Heatmap(data, values_by = "value", rows_by = "r", columns_by = "c",
        cell_type = "boxplot", add_bg = TRUE, add_reticle = TRUE)
}
if (requireNamespace("cluster", quietly = TRUE)) {
    Heatmap(data,
        values_by = "value", rows_by = "r", columns_by = "c",
        column_annotation = list(
            r1 = list(col = "p", type = "ring",
                      params = list(height = grid::unit(10, "mm"), show_legend = FALSE)),
            r2 = list(col = "q", type = "bar"),
            r3 = list(col = "p1", type = "violin",
                      params = list(height = grid::unit(18, "mm")))
        ),
        row_annotation = list(
            .default = list(side = "right"),
            q = list(type = "pie", params = list(width = grid::unit(12, "mm"))),
            p2 = list(type = "density"),
            a = list(type = "simple")
        ),
        show_row_names = TRUE, show_column_names = TRUE
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    Heatmap(data,
        values_by = "value", rows_by = "r", columns_by = "c",
        split_by = "p", palette = list(x = "Reds", y = "Blues")
    )
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # implies in_form = "wide-rows"
    Heatmap(data, rows_by = c("p1", "p2"), columns_by = "c")
}
if (requireNamespace("cluster", quietly = TRUE)) {
    # implies wide-columns
    Heatmap(data, rows_by = "r", columns_by = c("p1", "p2"))
}
}
}
\seealso{
\code{\link{HeatmapAtomic}}, \code{\link{LinkedHeatmap}},
\code{\link{anno_simple}}, \code{\link{anno_points}},
\code{\link{anno_lines}}, \code{\link{anno_pie}},
\code{\link{anno_violin}}, \code{\link{anno_boxplot}},
\code{\link{anno_density}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/jitterplot.R
\name{JitterPlotAtomic}
\alias{JitterPlotAtomic}
\title{Atomic jitter plot (internal)}
\usage{
JitterPlotAtomic(
  data,
  x,
  x_sep = "_",
  y = NULL,
  in_form = c("long", "wide"),
  keep_na = FALSE,
  keep_empty = FALSE,
  sort_x = c("none", "mean_asc", "mean_desc", "mean", "median_asc", "median_desc",
    "median"),
  flip = FALSE,
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  x_text_angle = 0,
  order_by = "-({y}^2 + {size_by}^2)",
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  aspect.ratio = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  shape = 21,
  border = "black",
  size_by = 2,
  size_name = NULL,
  size_trans = NULL,
  y_nbreaks = 4,
  jitter_width = 0.5,
  jitter_height = 0,
  y_max = NULL,
  y_min = NULL,
  y_trans = "identity",
  add_bg = FALSE,
  bg_palette = "stripe",
  bg_palcolor = NULL,
  bg_alpha = 0.2,
  add_hline = NULL,
  hline_type = "solid",
  hline_width = 0.5,
  hline_color = "black",
  hline_alpha = 1,
  labels = NULL,
  label_by = NULL,
  nlabel = 5,
  label_size = 3,
  label_fg = "black",
  label_bg = "white",
  label_bg_r = 0.1,
  highlight = NULL,
  highlight_color = "red2",
  highlight_size = 1,
  highlight_alpha = 1,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525,
  raster = NULL,
  raster_dpi = c(512, 512),
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name for the x-axis.
Must be character or factor.  Multiple columns can be provided; they are
concatenated with \code{x_sep} as the separator.  When \code{in_form} is
\code{"wide"}, the \code{x} columns are used as key columns and pivoted
to long format (they are not concatenated).}

\item{x_sep}{A character string used to join multiple \code{x} columns.
Default \code{"_"}.  Ignored when \code{x} is a single column or when
\code{in_form} is \code{"wide"}.}

\item{y}{A character string specifying the numeric column for the y-axis.
Required when \code{in_form} is \code{"long"} (default).  When
\code{in_form} is \code{"wide"}, \code{y} is not required — the values
under the \code{x} columns are used as y-values.}

\item{in_form}{A character string specifying the input data format.  Either
\code{"long"} (default) or \code{"wide"}.  In \code{"long"} format,
\code{x} and \code{y} are separate columns.  In \code{"wide"} format, the
\code{x} columns contain the y-values and are pivoted to a key-value pair.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{sort_x}{A character string controlling x-axis level reordering by
y-value summaries.  One of \code{"none"}, \code{"mean_asc"},
\code{"mean_desc"}, \code{"mean"}, \code{"median_asc"},
\code{"median_desc"}, \code{"median"}.  \code{"none"} leaves the levels
as-is.  \code{"mean_asc"} / \code{"mean"} sorts by ascending mean of y.
\code{"mean_desc"} sorts by descending mean.  \code{"median_asc"} /
\code{"median"} sorts by ascending median.  \code{"median_desc"} sorts by
descending median.  Default: \code{"none"}.}

\item{flip}{A logical value.  When \code{TRUE}, the x and y axes are
swapped via \code{coord_flip} and the x-axis factor levels are reversed.
Dimension calculation accounts for the flip.  Default: \code{FALSE}.}

\item{group_by}{A character vector of column names for dodging the points.
Each unique combination becomes a separate dodge group and the points are
offset horizontally via \code{position_jitterdodge} to reduce overlap.
Multiple columns are concatenated with \code{group_by_sep}.  When
\code{NULL} (default), no dodging is applied — only jitter via
\code{position_jitter}.}

\item{group_by_sep}{A character string used to join multiple
\code{group_by} columns.  Default \code{"_"}.}

\item{group_name}{A character string for the dodge-group legend title.
When \code{NULL} (default), the \code{group_by} column name is used.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{order_by}{A string expression passed to \code{\link[dplyr]{arrange}()}
to determine which points are labelled.  Evaluated within each x-group
(and facet panel when \code{facet_by} is set).  Default:
\code{"-({y}^2 + {size_by}^2)"}, which selects points farthest from the
origin in y-size radial distance, analogous to VolcanoPlot.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value in \code{[0, 1]} controlling point
transparency.  Default: \code{1}.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{shape}{A numeric value specifying the point shape (ggplot2 point
shape codes).  Shapes 21–25 are filled shapes with borders; for these
shapes the border behaviour is controlled by \code{border}.  Default:
\code{21} (filled circle).}

\item{border}{Controls the border of points when the shape has a border
(21–25).  If \code{TRUE}, the border colour follows the point fill colour
(same as the group colour).  If a single colour string (e.g.
\code{"black"}), uses that constant border colour for all points.  If
\code{FALSE}, no border is drawn (\code{NA}).  Default: \code{"black"}.}

\item{size_by}{A numeric column name or a single numeric value controlling
point size.  When a column name is provided, sizes are scaled using
\code{scale_size_area(max_size = 6)} and a size legend is shown.  When a
single numeric value, all points use that constant size.  Default:
\code{2}.}

\item{size_name}{A character string for the size legend title.  When
\code{NULL} (default) and \code{size_by} is a column, the column name is
used.  Ignored when \code{size_by} is a single numeric value.}

\item{size_trans}{A function or a function name (as a string) to transform
the \code{size_by} values for size mapping.  The transformed values
determine the point size on the plot, but the legend labels show the
original (untransformed) values.  When \code{NULL} (default), no
transformation is applied.}

\item{y_nbreaks}{A numeric value hinting at the number of break intervals
for the y-axis.  Passed to \code{scale_y_continuous}.  Default: \code{4}.}

\item{jitter_width}{A numeric value controlling the amount of horizontal
jitter (in x-axis units).  Passed to \code{position_jitter} /
\code{position_jitterdodge}.  Default: \code{0.5}.}

\item{jitter_height}{A numeric value controlling the amount of vertical
jitter (in y-axis units).  Passed to \code{position_jitter} /
\code{position_jitterdodge}.  Default: \code{0}.}

\item{y_max, y_min}{Numeric values or quantile strings (e.g. \code{"q95"},
\code{"q5"}) for y-axis limits used in \code{coord_cartesian} /
\code{coord_flip}.  When \code{NULL} (default), the data range is used.
When a quantile string, the corresponding quantile of the y-values is
computed via \code{\link[stats]{quantile}()}.}

\item{y_trans}{A character string specifying a transformation for the
y-axis (e.g. \code{"log10"}, \code{"sqrt"}).  Passed to
\code{\link[ggplot2]{scale_y_continuous}}.  Default: \code{"identity"}.}

\item{add_bg}{A logical value.  When \code{TRUE}, alternating background
stripes are drawn behind the points via \code{\link{bg_layer}()}, using
the x-axis level order.  Default: \code{FALSE}.}

\item{bg_palette}{A character string specifying the palette for the
background stripe colours.  Passed to \code{\link{bg_layer}()}.
Default: \code{"stripe"}.}

\item{bg_palcolor}{A character vector of colours for the background
stripes.  Passed to \code{\link{bg_layer}()}.  When \code{NULL} (default),
colours are derived from \code{bg_palette}.}

\item{bg_alpha}{A numeric value in \code{[0, 1]} for the transparency of
the background stripes.  Default: \code{0.2}.}

\item{add_hline}{One or more numeric values specifying y-values at which
to draw horizontal reference lines.  When \code{NULL} (default), no
reference lines are drawn.}

\item{hline_type}{A character string specifying the line type for the
horizontal reference line(s).  Default: \code{"solid"}.}

\item{hline_width}{A numeric value specifying the line width for the
horizontal reference line(s).  Default: \code{0.5}.}

\item{hline_color}{A character string specifying the colour for the
horizontal reference line(s).  Default: \code{"black"}.}

\item{hline_alpha}{A numeric value in \code{[0, 1]} specifying the alpha
(transparency) for the horizontal reference line(s).  Default: \code{1}.}

\item{labels}{A vector of row names or row indices specifying which points
to label.  When \code{NULL} (default) and \code{nlabel > 0}, the top
\code{nlabel} points per x-group are selected automatically.}

\item{label_by}{A character string naming a column whose values are used
as label text.  When \code{NULL} (default), row names are used as labels.}

\item{nlabel}{An integer specifying the number of points to label per
x-group when \code{labels} is \code{NULL}.  Points are selected by
descending order of \code{order_by}.  Default: \code{5}.  Set to
\code{0} to suppress automatic labelling.}

\item{label_size, label_fg, label_bg, label_bg_r}{Label aesthetics for
\code{\link[ggrepel]{geom_text_repel}}.  \code{label_size}: text size
(default \code{3}).  \code{label_fg}: text colour (default
\code{"black"}).  \code{label_bg}: background (halo) colour for the label
text (default \code{"white"}).  \code{label_bg_r}: background border
radius (default \code{0.1}).}

\item{highlight}{A specification of which points to highlight.  Can be:
\code{TRUE} (highlight all points), a numeric vector of row indices, a
single character string parsed as an \R expression, or a character vector
of row names.  When a point is highlighted, an overlay \code{geom_point}
is drawn on top with \code{highlight_color}, \code{highlight_size}, and
\code{highlight_alpha}.  Default: \code{NULL} (no highlighting).}

\item{highlight_color}{A character string specifying the colour of
highlighted points.  Default: \code{"red2"}.}

\item{highlight_size}{A numeric value specifying the size of highlighted
points.  Default: \code{1}.}

\item{highlight_alpha}{A numeric value in \code{[0, 1]} specifying the
transparency of highlighted points.  Default: \code{1}.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{seed}{The random seed to use. Default is 8525.}

\item{raster}{A logical value. If \code{TRUE}, points are rendered via
\code{scattermore::geom_scattermore()} for efficient rasterised plotting.
Default is \code{NULL}, which auto-enables when \code{nrow(data) > 1e3}.
Ignored (with a warning) when \code{size_by} is mapped to a column, since
scattermore cannot vary the point size per point.}

\item{raster_dpi}{A numeric vector of length 2 \verb{[x_dpi, y_dpi]} specifying the raster
resolution in pixels. Passed to \code{scattermore::geom_scattermore(pixels = ...)}.
Default is \code{c(512, 512)}. If a single value is provided it is recycled to both dimensions.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a single jittered point plot.  This is the
workhorse behind the exported \code{\link{JitterPlot}} function — it takes a
\strong{single} data frame (no \code{split_by} support) and returns a
\code{ggplot} object.  The plot displays individual data points with random
jitter along the x-axis (and optionally the y-axis) to reveal the
distribution of values within each x category, making it especially useful
for visualising overlapping discrete data.

The function provides extensive annotation features including point labelling
(via \code{\link[ggrepel]{geom_text_repel}} with automatic top-n selection
using a configurable distance metric), point highlighting, optional dodging
via \code{group_by}, and horizontal reference lines.  It supports both long
and wide input formats, quantile-based axis limits, and x-axis reordering by
y-value summaries (mean or median).
}
\section{Architecture}{

\enumerate{
\item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{in_form handling} — when \code{in_form = "wide"}, the x
columns are pivoted to long format via
\code{\link[tidyr]{pivot_longer}()}, creating a key column
\code{.x} and a value column \code{.y}.
\item \strong{Column resolution} — \code{x}, \code{y}, \code{group_by},
and \code{facet_by} are validated and transformed via
\code{\link{check_columns}}.  Multi-column inputs for \code{x} and
\code{group_by} are concatenated into single columns using their
respective separators (\code{x_sep}, \code{group_by_sep}).
\item \strong{NA / empty-level handling} — \code{\link{process_keep_na_empty}()}
applies \code{keep_na} and \code{keep_empty} policies.  Per-column
\code{keep_empty} settings are extracted for \code{x},
\code{group_by}, and \code{facet_by} independently.  When
\code{facet_by} has more than one column, the \code{keep_empty}
values for all facet columns must be identical.
\item \strong{sort_x resolution} — \code{match.arg()} normalises the
\code{sort_x} argument, and per-group (\code{x}, \code{group_by},
\code{facet_by}) means and medians of \code{y} are computed for
subsequent reordering.
\item \strong{y-axis limit computation} — \code{y_max} and \code{y_min}
support raw numeric values or quantile strings (e.g. \code{"q95"}),
which are resolved via \code{\link[stats]{quantile}()}.  These
limits are used for fixed \code{coord_cartesian} / \code{coord_flip}.
\item \strong{Highlight flag} — when \code{highlight} is provided, a
logical column \code{.highlight} is created.  Values can be
\code{TRUE} (all points), a numeric vector of row indices, a single
character string parsed as an \R expression, or a character vector
matched against row names.
\item \strong{Label preparation} — labels are resolved from
\code{label_by} or row names.  When \code{labels} is \code{NULL}
and \code{nlabel > 0}, the top \code{nlabel} points per x-group are
selected by descending order of \code{order_by} (default:
\code{-({y}^2 + {size_by}^2)}, the radial distance from zero in
y-size space, analogous to VolcanoPlot).  When \code{facet_by} is
active, the ranking is nested within each facet panel.
\item \strong{x-axis reordering} — when \code{sort_x} is not
\code{"none"}, the x-axis factor levels are reordered by the
per-group mean or median via \code{\link[stats]{reorder}()}.
\item \strong{Flip handling} — when \code{flip = TRUE}, the x-axis
factor levels are reversed and the aspect ratio is inverted.
\item \strong{Colour assignment} — \code{\link{palette_this}()} assigns
colours to the levels of \code{group_by} (or \code{x} when
\code{group_by} is \code{NULL}), including \code{NA} levels which
are relabelled as \code{"NA"}.
\item \strong{Background layer} — when \code{add_bg = TRUE}, alternating
background stripes are drawn via \code{\link{bg_layer}()} using the
x-axis level order.
\item \strong{Position jitter} — when \code{group_by} is \code{NULL},
\code{\link[ggplot2]{position_jitter}()} is used.  When
\code{group_by} is provided, \code{\link[ggplot2]{position_jitterdodge}()}
is used so points are dodged by group before jittering.
\item \strong{Pre-compute jittered positions} — a temporary \code{ggplot}
is built and rendered via \code{ggplot_build()} to extract the
actual jittered coordinates.  These are stored as \code{.x_jittered}
and \code{.y_jittered} columns and used by the label layer because
\code{geom_text_repel} does not natively respect
\code{position_jitter} / \code{position_jitterdodge}.
\item \strong{Point layer} — \code{geom_point()} is built with the
appropriate aesthetic mapping: fill (for shapes 21–25) or colour
(for other shapes) mapped to the grouping variable.  Border
behaviour for shapes 21–25 is controlled by \code{border}
(constant string, mapped to group colour, or \code{NA}).  When
\code{size_by} is a column, size is mapped via \code{aes(size)}.
\item \strong{Discrete colour/fill scales} —
\code{scale_fill_manual()} or \code{scale_color_manual()} with the
assigned palette colours.  When \code{keep_empty} is \code{TRUE}
for the colour column, \code{drop = FALSE} and explicit
\code{breaks} / \code{limits} preserve empty levels.  An additional
\code{scale_color_manual()} is added for border colour when
\code{border = TRUE}.
\item \strong{Size scale} — when \code{size_by} is a column,
\code{scale_size_area(max_size = 6)} is applied.  If
\code{size_trans} is provided, the legend labels show the original
(untransformed) values while the mapping uses transformed values.
\item \strong{Highlight overlay} — if any points are highlighted, a
second \code{geom_point()} layer is drawn on top using the
\code{highlight_color}, \code{highlight_size}, and
\code{highlight_alpha} settings.  The highlight layer does not
affect the legend.
\item \strong{Label layer} — if any points are selected for labelling,
\code{\link[ggrepel]{geom_text_repel}()} is rendered at the
pre-computed jittered coordinates.
\item \strong{Horizontal reference lines} — when \code{add_hline} is
provided, \code{geom_hline()} is drawn using the specified line
aesthetics.
\item \strong{Axes and labels} — \code{scale_x_discrete()} (with
\code{drop = !isTRUE(keep_empty_x)}),
\code{scale_y_continuous(trans = y_trans, n.breaks = y_nbreaks)},
and \code{labs()} set the axis scale properties and titles.
\item \strong{Coordinate system} — when \code{flip = TRUE},
\code{coord_flip()} is used with (or without) fixed y-limits
depending on free-scale faceting.  When \code{flip = FALSE},
\code{coord_cartesian()} sets the y-axis limits unless free scales
are in use.
\item \strong{Theme} — \code{do_call(theme, theme_args)} applies the
selected theme, with additional \code{aspect.ratio}, x-axis text
rotation, legend position, and legend direction adjustments.
Panel grid lines are styled: major y-axis lines (dashed grey) for
non-flipped plots; major x-axis lines for flipped plots.
\item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
computes plot height and width from the x-axis category count,
\code{aspect.ratio}, dodge group count, legend metrics, and label
character width.  The minimum dimensions account for x-axis label
length.  The resulting \code{height} / \code{width} attributes are
stored on the \code{ggplot} object.  For flipped plots, the width
is set to at least \code{max(width, height)}.
\item \strong{Faceting} — \code{\link{facet_plot}()} wraps the plot
with \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is
provided, respecting the \code{keep_empty} setting for facet
variables.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{enrich_example}
\alias{enrich_example}
\title{An example of clusterProfiler enrichment result}
\description{
An example of clusterProfiler enrichment result
}
\examples{
\dontrun{
if (interactive()) {
  data(geneList, package="DOSE")
  de <- names(geneList)[abs(geneList) > 1.5]
  enrich_example <- clusterProfiler::enrichPathway(gene=de, pvalueCutoff = 0.05, readable=TRUE)
  enrich_example <- as.data.frame(enrich_example)
}
}
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{bg_layer}
\alias{bg_layer}
\title{Get a ggplot layer for background}
\usage{
bg_layer(
  data,
  x,
  keep_empty,
  palette,
  palcolor,
  alpha,
  facet_by,
  direction = "vertical"
)
}
\arguments{
\item{data}{A data frame}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis}

\item{keep_empty}{A character string specifying whether to keep empty levels}

\item{palette}{A character string specifying the palette to use}

\item{palcolor}{A character string specifying the color to use in the palette}

\item{alpha}{A numeric value specifying the transparency of the plot}

\item{facet_by}{A character string specifying the column name(s) of the data frame to facet the plot}

\item{direction}{A character string specifying the direction for the background}
}
\value{
A ggplot layer for background
}
\description{
Get a ggplot layer for background
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{calc_just}
\alias{calc_just}
\title{Calculate hjust and vjust based on angle}
\usage{
calc_just(angle)
}
\arguments{
\item{angle}{A numeric value of the angle}
}
\value{
A list with h and v values
}
\description{
Calculate hjust and vjust based on angle
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clustreeplot.R
\name{ClustreePlot}
\alias{ClustreePlot}
\title{Clustree Plot}
\usage{
ClustreePlot(
  data,
  prefix,
  flip = FALSE,
  split_by = NULL,
  split_by_sep = "_",
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  edge_palette = "Spectral",
  edge_palcolor = NULL,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  expand = c(0.1, 0.1),
  theme = "theme_this",
  theme_args = list(),
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  seed = 8525,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{prefix}{A character string specifying the common prefix of the
resolution columns in \code{data}. All columns whose names start with
this prefix are selected as resolution columns. The suffix after the
prefix is parsed as a numeric resolution value. Supports \code{"_"}
and \code{"."} as separators between the prefix and the resolution
value (e.g. \code{"res_0.5"} or \code{"p.0.5"}).}

\item{flip}{A logical value. If \code{TRUE}, the tree is flipped so that
resolutions are displayed on the x-axis (left to right) and cluster
assignments are shown as row labels on the y-axis.
Default: \code{FALSE}.}

\item{split_by}{The column(s) to split data by and generate separate
clustree plots for each level. Each split level produces an independent
clustree plot via \code{\link{ClustreePlotAtomic}}.}

\item{split_by_sep}{A character string used to concatenate multiple
\code{split_by} column values when \code{split_by} specifies more than
one column. Default: \code{"_"}.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{edge_palette}{A character string specifying the palette name for
the edge colour gradient. Edges are coloured by the number of
transitioning cells between clusters at adjacent resolutions, using
\code{ggraph::scale_edge_color_gradientn()}. Default: \code{"Spectral"}.}

\item{edge_palcolor}{A character vector of custom colours for the edge
colour gradient. When \code{NULL} (the default), colours are derived
from \code{edge_palette}.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{expand}{The values to expand the x and y axes. It is like CSS padding.
When a single value is provided, it is used for both axes on both sides.
When two values are provided, the first value is used for the top/bottom side and the second value is used for the left/right side.
When three values are provided, the first value is used for the top side, the second value is used for the left/right side, and the third value is used for the bottom side.
When four values are provided, the values are used for the top, right, bottom, and left sides, respectively.
You can also use a named vector to specify the values for each side.
When the axis is discrete, the values will be applied as 'add' to the 'expansion' function.
When the axis is continuous, the values will be applied as 'mult' to the 'expansion' function.
See also \url{https://ggplot2.tidyverse.org/reference/expansion.html}}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{combine}{A logical value. If \code{TRUE} (the default), the list
of per-split plots is combined into a single \code{patchwork} object.
If \code{FALSE}, returns the raw list of \code{ggplot} objects.}

\item{nrow, ncol, byrow}{Integers controlling the layout of combined plots
via \code{patchwork::wrap_plots()}. \code{byrow = TRUE} (default)
fills the layout row-wise. Ignored when \code{design} is provided.}

\item{seed}{The random seed for reproducibility. Passed to
\code{validate_common_args()}. Default: \code{8525}.}

\item{axes, axis_titles}{Strings controlling how axes and axis titles are
handled across combined plots. Passed to \code{\link{combine_plots}()}.
See \code{?patchwork::wrap_plots} for options (\code{"keep"},
\code{"collect"}, \code{"collect_x"}, \code{"collect_y"}).}

\item{guides}{A string controlling guide collection across combined
plots. Passed to \code{\link{combine_plots}()}.}

\item{design}{A custom layout specification for combined plots. Passed
to \code{\link{combine_plots}()}. When specified, \code{nrow},
\code{ncol}, and \code{byrow} are ignored.}

\item{...}{Additional arguments passed to \code{clustree::clustree()}.
Commonly used overrides include \code{node_size_range},
\code{node_text_size}, \code{layout} (default: \code{"sugiyama"}),
\code{show_axis}, and \code{node_text_colour}. Note that \code{x}
(the data) and \code{prefix} are set internally and cannot be
overridden here.}
}
\value{
A \code{ggplot} object (single plot), a \code{patchwork} object
(when \code{combine = TRUE} with \code{split_by}), or a \code{list} of
\code{ggplot} objects (when \code{combine = FALSE}).
}
\description{
Creates a clustree (clustering tree) plot visualising how cluster
assignments change across increasing clustering resolutions. The plot
helps identify stable clustering solutions and understand the hierarchical
relationships among clusters at different resolution thresholds.

The function expects a data frame with columns named by a common
\code{prefix} followed by numeric resolution values (e.g.
\code{"res_0.1"}, \code{"res_0.3"}, \code{"res_0.5"}). Each column
contains cluster labels (factor or character) for every observation at
that resolution.

Internally, the function uses \code{clustree::clustree()} to compute a
\code{ggraph}-based tree layout where nodes are clusters and edges
represent cells transitioning between clusters at adjacent resolutions.
Edge colour and width encode the number of transitioning cells.

Key features:
\itemize{
\item \strong{Resolution-level node colouring} — each resolution
receives a distinct colour from the selected \code{palette}.
\item \strong{Edge gradient} — edges are coloured by transition count
using a separate \code{edge_palette} colour gradient.
\item \strong{Flip support} — \code{flip = TRUE} places resolutions
on the x-axis for left-to-right reading.
\item \strong{Split by groups} — \code{split_by} generates per-group
clustree plots that are combined via \code{patchwork}.
\item \strong{Automatic dimensions} — plot height and width are
automatically computed based on the number of resolutions, clusters,
and the legend configuration.
}
}
\section{split_by Workflow (ClustreePlot)}{


When \code{split_by} is provided, the following pipeline executes:
\enumerate{
\item \strong{Argument validation} — \code{validate_common_args()}
checks the \code{seed} value and sets the random seed.
\item \strong{Theme resolution} — \code{process_theme()} resolves the
\code{theme} string or function to a theme function.
\item \strong{Split column validation} — \code{check_columns()}
resolves \code{split_by} with \code{force_factor = TRUE,
  allow_multi = TRUE, concat_multi = TRUE}.
\item \strong{Data splitting} — splits \code{data} by \code{split_by}
levels (unused levels dropped), preserving factor level order.
\item \strong{Per-split palette / colour / legend} —
\code{check_palette()}, \code{check_palcolor()}, and
\code{check_legend()} resolve per-split overrides for
\code{palette}, \code{palcolor}, \code{legend.position}, and
\code{legend.direction}.
\item \strong{Per-split title} — when \code{title} is a function, it
receives the default title (the split level name) and can return a
custom string; otherwise \code{title \%||\% split_level} is used.
\item \strong{Dispatch} — each split subset is passed to
\code{\link{ClustreePlotAtomic}} with the per-split parameters.
\item \strong{Combination} — \code{\link{combine_plots}()} assembles
the list of plots via \code{patchwork::wrap_plots}, honouring
\code{nrow}/\code{ncol}/\code{byrow}/\code{design}.
\item \strong{Combined data} — when \code{combine = TRUE}, the
combined plot's \code{data} (\code{p$data}) exposes the node layout
of every split, each row tagged with the \code{split_by} level, plus
a combined graph of all splits' nodes and links (\code{"graph"}
attribute) and the original links (\code{"edges"} attribute).
}
}

\examples{
\donttest{
set.seed(8525)
N <- 100
data <- data.frame(
    p.0.4 = sample(LETTERS[1:5], N, replace = TRUE),
    p.0.5 = sample(LETTERS[1:6], N, replace = TRUE),
    p.0.6 = sample(LETTERS[1:7], N, replace = TRUE),
    p.0.7 = sample(LETTERS[1:8], N, replace = TRUE),
    p.0.8 = sample(LETTERS[1:9], N, replace = TRUE),
    p.0.9 = sample(LETTERS[1:10], N, replace = TRUE),
    p.1 = sample(LETTERS[1:30], N, replace = TRUE),
    split = sample(1:2, N, replace = TRUE)
)

# --- Basic clustree plot ---
ClustreePlot(data, prefix = "p")

# --- Flipped layout (resolutions on x-axis) ---
ClustreePlot(data, prefix = "p", flip = TRUE)

# --- Split by group ---
ClustreePlot(data, prefix = "p", split_by = "split")

# --- Split by group with per-split palettes ---
ClustreePlot(data, prefix = "p", split_by = "split",
             palette = c("1" = "Set1", "2" = "Paired"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theming.R
\name{theme_this}
\alias{theme_this}
\title{A ggplot2 theme and palettes for plotthis
Borrowed from the \code{theme_this} function in the \code{SCP} pipeline}
\usage{
theme_this(aspect.ratio = NULL, base_size = NULL, font_family = NULL, ...)
}
\arguments{
\item{aspect.ratio}{The aspect ratio of the plot}

\item{base_size}{The base size of the text
If not specified, it will use the value from \code{getOption("theme_this.base_size", 12)}.
If you want to change the default base size, you can set the option \code{theme_this.base_size}.
This is applied to all plots using this theme.}

\item{font_family}{The font family of the text
If not specified, it will use the value from \code{getOption("theme_this.font_family")}.
If you want to change the default font family, you can set the option \code{theme_this.font_family}.
This is applied to all plots using this theme.
To list available font families, you can use the \code{systemfonts::system_fonts()} function.}

\item{...}{Other arguments for \code{theme()}}
}
\value{
A ggplot2 theme
}
\description{
A ggplot2 theme and palettes for plotthis
Borrowed from the \code{theme_this} function in the \code{SCP} pipeline
}
\seealso{
\url{https://github.com/zhanghao-njmu/SCP}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap-utils.R
\name{.resolve_anno_aliases}
\alias{.resolve_anno_aliases}
\title{Resolve annotation aliases:
.row         -> rows_by
.rows.split  -> rows_split_by
.col/.column -> columns_by
.col.split/.column.split -> columns_split_by}
\usage{
.resolve_anno_aliases(lst, row_key, rsplit_key, col_key, csplit_key)
}
\arguments{
\item{lst}{A list of annotations, which may contain aliases for rows_by, rows_split_by, columns_by, and columns_split_by.}

\item{row_key}{The actual key for rows_by in the list.}

\item{rsplit_key}{The actual key for rows_split_by in the list.}

\item{col_key}{The actual key for columns_by in the list.}

\item{csplit_key}{The actual key for columns_split_by in the list.}
}
\value{
A list of annotations with aliases resolved to the actual keys.
}
\description{
Resolve annotation aliases:
.row         -> rows_by
.rows.split  -> rows_split_by
.col/.column -> columns_by
.col.split/.column.split -> columns_split_by
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrich.R
\name{prepare_enrichr_result}
\alias{prepare_enrichr_result}
\title{Process the enrichment results from Enrichr}
\usage{
prepare_enrichr_result(data, dbname = "Database", n_input = NULL)
}
\arguments{
\item{data}{A data frame containing the result by Enrichr.}

\item{dbname}{A character string specifying the name of the database column.}

\item{n_input}{An integer specifying the number of input genes.
Enrichr result doesn't ship with the number of input genes.
You can either provide the number directly or we will infer it. See details.}
}
\value{
A data frame that can be used in \code{EnrichMap}.
}
\description{
Process the enrichment results from Enrichr
}
\details{
In order to use the \code{EnrichMap} and \code{EnrichNetwork} functions and other visualization functions in \code{plotthis},
the enrichment results from Enrichr need to be processed by the \code{prepare_enrichr_result} function.
The following columns are renamed:
\itemize{
\item \code{Term} -> \code{Description}
\item \code{Genes} -> \code{geneID} (separated replaced by \code{/})
\item \code{P.value} -> \code{pvalue}
\item \code{Adjusted.P.value} -> \code{p.adjust}
Additionally, GeneRatio and BgRatio columns are inferred.
From \href{https://maayanlab.cloud/Enrichr/help#background}{enrichr's documentation}, the oddsRatio is defined as:
\verb{oddsRatio = (A * (D - B - C + A) / max((B - A) * (C - A), 1)}, where A is the overlapping genes; B is the total genes in the gene set;
C (n_input) is the genes in input list; D is the total genes in the background.
D is not provided by Enrichr. To infer it, \code{D = oddsRatio * max((B - A) * (C - A), 1) / A + B + C - A}.
\item \code{Overlap = A / B} (from Enrichr)
\item \code{GeneRatio = A / C} (from ClusterProfiler)
\item \code{BgRatio = B / D} (from ClusterProfiler)
\code{C (n_input)}, if not provided, will be inferred when \code{D} for all terms are equal.
When starting inferrence, the minimum value to try will be unique genes in \code{data$Genes}/\code{data$geneID}.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/common_args.R
\name{validate_common_args}
\alias{validate_common_args}
\title{Validate common arguments}
\usage{
validate_common_args(
  seed,
  facet_by = NULL,
  plot_type = NULL,
  split_by = NULL,
  split_by_sep = "_",
  group_by = NULL,
  group_by_sep = "_",
  facet_scales = "fixed",
  facet_nrow = NULL,
  facet_ncol = NULL,
  facet_byrow = TRUE,
  theme = "theme_this",
  theme_args = list(),
  palette = NULL,
  palcolor = NULL,
  expand = NULL,
  keep_na = FALSE,
  keep_empty = FALSE,
  alpha = 1,
  x_text_angle = 0,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  ...
)
}
\arguments{
\item{seed}{The random seed to use. Default is 8525.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{split_by}{The column(s) to split data by and plot separately.
When \code{combine = TRUE}, the combined plot's data (\code{p$data})
contains the rows of every split, tagged with the \code{split_by}
column. For ggraph-based plots (e.g. \code{\link{Network}},
\code{\link{ClustreePlot}}), the node layout of every split is
exposed with the split column, and the original link rows (also
tagged with the \code{split_by} column) are available as the
\code{"edges"} attribute of the data.}

\item{split_by_sep}{The separator for multiple split_by columns. See \code{split_by}}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{expand}{The values to expand the x and y axes. It is like CSS padding.
When a single value is provided, it is used for both axes on both sides.
When two values are provided, the first value is used for the top/bottom side and the second value is used for the left/right side.
When three values are provided, the first value is used for the top side, the second value is used for the left/right side, and the third value is used for the bottom side.
When four values are provided, the values are used for the top, right, bottom, and left sides, respectively.
You can also use a named vector to specify the values for each side.
When the axis is discrete, the values will be applied as 'add' to the 'expansion' function.
When the axis is continuous, the values will be applied as 'mult' to the 'expansion' function.
See also \url{https://ggplot2.tidyverse.org/reference/expansion.html}}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{combine}{Whether to combine the plots into one when facet is FALSE. Default is TRUE.}

\item{nrow}{A numeric value specifying the number of rows in the facet.}

\item{ncol}{A numeric value specifying the number of columns in the facet.}

\item{byrow}{A logical value indicating whether to fill the plots by row.}

\item{...}{Additional arguments.}
}
\description{
Validate common arguments
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{combine_plots}
\alias{combine_plots}
\title{Combine plots into one}
\usage{
combine_plots(
  plots,
  combine = TRUE,
  split_by = NULL,
  nrow = NULL,
  ncol = NULL,
  byrow = NULL,
  axes = NULL,
  axis_titles = NULL,
  guides = NULL,
  design = NULL,
  recalc_size = TRUE
)
}
\arguments{
\item{plots}{A list of plots}

\item{combine}{Whether to combine the plots into one}

\item{split_by}{The column name to split the plots by. When provided,
the combined data from all sub-plots is available via \code{p$data}.}

\item{nrow}{The number of rows in the combined plot}

\item{ncol}{The number of columns in the combined plot}

\item{byrow}{Whether to fill the plots by row}

\item{recalc_size}{Whether to re-calculate the size of the combined plot}
}
\value{
The faceted plot. If guess_size is TRUE, attr(p, "height") and attr(p, "width") will be set
}
\description{
Combine plots into one
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trendplot.R
\name{TrendPlot}
\alias{TrendPlot}
\title{Trend plot}
\usage{
TrendPlot(
  data,
  x,
  y = NULL,
  x_sep = "_",
  split_by = NULL,
  split_by_sep = "_",
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  scale_y = FALSE,
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  x_text_angle = 0,
  aspect.ratio = 1,
  legend.position = waiver(),
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  keep_na = FALSE,
  keep_empty = FALSE,
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis.}

\item{y}{A character string specifying the column name of the data frame to plot for the y-axis.}

\item{x_sep}{A character string used to join multiple \code{x} columns.
Default \code{"_"}. Ignored when \code{x} is a single column.}

\item{split_by}{The column(s) to split the data by and produce separate
sub-plots. Multiple columns are concatenated with \code{split_by_sep}.}

\item{split_by_sep}{A character string to separate concatenated
\code{split_by} columns. Default \code{"_"}.}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{group_name}{A character string used as the fill legend title.
When \code{NULL}, the \code{group_by} column name is used.}

\item{scale_y}{A logical value. When \code{TRUE}, y-values are scaled to
proportions within each (\code{x}, \code{facet_by}) group so that each
x position stacks to 1.0. The y-axis labels switch from numeric to
percent format automatically.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{seed}{A numeric seed for reproducibility. Passed to
\code{\link{validate_common_args}()}.}

\item{combine}{Logical; when \code{TRUE} (default), returns a combined
\code{patchwork} object. When \code{FALSE}, returns a named list of
individual \code{ggplot} objects.}

\item{ncol, nrow}{Integer number of columns / rows for the combined layout
(passed to \code{\link[patchwork]{wrap_plots}}).}

\item{byrow}{Logical; fill the combined layout by row. Default \code{TRUE}
(passed to \code{\link[patchwork]{wrap_plots}}).}

\item{axes}{A character string specifying how axes should be treated across
the combined layout (passed to \code{\link[patchwork]{wrap_plots}}).}

\item{axis_titles}{A character string specifying how axis titles should be
treated across the combined layout. Defaults to \code{axes}.}

\item{guides}{A character string specifying how guides (legends) should be
collected across panels. Default \code{"collect"} (passed to
\code{\link{combine_plots}()}).}

\item{design}{A custom layout design for the combined plot (passed to
\code{\link{combine_plots}()}).}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object, a \code{patchwork} object, or a named list
of \code{ggplot} objects (when \code{combine = FALSE}), each with
\code{height} and \code{width} attributes in inches.
}
\description{
Draws a trend plot combining stacked bars with a semi-transparent area
background to show how one or more groups accumulate across a discrete
x-axis variable. This hybrid style sits between a bar plot and an area
plot: it preserves the discrete category separation of bars while
softening the visual with an area fill, making trends across categories
easier to perceive.

The function supports \strong{count aggregation} (omit \code{y} to plot
observation counts per x-category), \strong{proportion scaling}
(\code{scale_y = TRUE} normalises each x position to 100\\%), per-group
colour control, faceting, and splitting into separate sub-plots via
\code{split_by}.
}
\section{split_by workflow}{

When \code{split_by} is provided:
\enumerate{
\item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
normalise the \code{keep_na} / \code{keep_empty} arguments for all
columns (\code{x}, \code{split_by}, \code{group_by}, \code{facet_by}).
\item The \code{split_by} column is validated and its NA / empty levels
are processed via \code{\link{process_keep_na_empty}()}. It is
then removed from the per-column \code{keep_na} / \code{keep_empty}
lists.
\item The data frame is split by \code{split_by} (preserving level
order). If \code{split_by} is \code{NULL}, the data is wrapped in
a single-element list with name \code{"..."}.
\item Per-split \code{palette}, \code{palcolor},
\code{legend.position}, and \code{legend.direction} are resolved
via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
and \code{\link{check_legend}()}.
\item \code{\link{TrendPlotAtomic}()} is called for each split. If
\code{title} is a function, it receives the split level name and
can generate dynamic titles.
\item Results are combined via \code{\link{combine_plots}()} (when
\code{combine = TRUE}) or returned as a named list.
}
}

\examples{
\donttest{
data <- data.frame(
    x = factor(rep(c("A", "B", NA, "D"), 3), levels = LETTERS[1:4]),
    y = c(1, 3, 6, 5, 4, 2, 5, 7, 8, 9, 4, 8),
    group = factor(rep(c("F1", NA, "F3"), each = 4), levels = c("F1", "F2", "F3"))
)

# Basic trend plot with grouping
TrendPlot(data, x = "x", y = "y", group_by = "group")

# Scaled to proportions
TrendPlot(data, x = "x", y = "y", group_by = "group",
         scale_y = TRUE)

# Split into sub-plots (no group_by -- single-colour fill)
TrendPlot(data, x = "x", y = "y", split_by = "group")

# Per-split palettes
TrendPlot(data, x = "x", y = "y", split_by = "group",
          palette = c(F1 = "Set1", F3 = "Dark2"))

# How keep_na and keep_empty work
TrendPlot(data, x = "x", y = "y", group_by = "group",
         keep_na = TRUE, keep_empty = TRUE)
TrendPlot(data, x = "x", y = "y", group_by = "group",
         keep_na = TRUE, keep_empty = list(x = FALSE, group = 'level'))

# Faceting
TrendPlot(data, x = "x", y = "y", facet_by = "group",
         keep_na = TRUE, keep_empty = list(x = FALSE, group = 'level'))
}
}
\seealso{
\code{\link{AreaPlot}} for a pure stacked area plot.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap-utils.R
\name{.reorder_anno_side}
\alias{.reorder_anno_side}
\title{Reorder annotation list so split annotation (split_by) is farthest from heatmap body
and name annotation (by) is closest. User annotations stay in between.
For all annotation sides, ComplexHeatmap renders first-element = farthest,
last-element = closest to the heatmap body.}
\usage{
.reorder_anno_side(x, by, split_by, side)
}
\arguments{
\item{x}{A list of annotations}

\item{by}{The name of the annotation used for row/column names}

\item{split_by}{The name of the annotation used for splitting rows/columns}

\item{side}{The annotation side ("top", "bottom", "left", "right")}
}
\value{
A reordered list of annotations
}
\description{
Reorder annotation list so split annotation (split_by) is farthest from heatmap body
and name annotation (by) is closest. User annotations stay in between.
For all annotation sides, ComplexHeatmap renders first-element = farthest,
last-element = closest to the heatmap body.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{dim_example}
\alias{dim_example}
\title{An example data for dimensionality reduction plot}
\description{
This dataset is generated from the scvelo (scv.datasets.pancreas()) with
the scvelo run on the dataset.
Then the cell embeddings and velocity embeddings are extracted (200 downsampled), which are the first
4 columns of the data frame.
The fifth column is the group identifier (clusters), and the sixth column is a fake
grouping variable used to visualize stats, facetting, etc.
An attribute "graph" is added to the data frame, which is a square matrix of the
cell-cell distances, which is used for the graph (network) on dimensionality reduction
plots.
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roccurve.R
\name{get_cutoffs_data}
\alias{get_cutoffs_data}
\title{Prepare cutoff data for ROC curve annotation}
\usage{
get_cutoffs_data(
  data,
  truth_by,
  score_by,
  cat_by,
  cutoffs_at = NULL,
  cutoffs_labels = NULL,
  cutoffs_accuracy = 0.001,
  n_cuts = 0,
  increasing = TRUE
)
}
\arguments{
\item{data}{A data frame with the truth and score columns.}

\item{truth_by}{A character string of the column name that contains the true
class labels (binary, 0/1 or TRUE/FALSE).}

\item{score_by}{A character string of the column name that contains the predicted scores.}

\item{cat_by}{A character string of the column name to categorise/group the data.
When specified, cutoffs are calculated separately for each category level,
enabling per-group or per-facet cutoff annotation.}

\item{cutoffs_at}{A vector of user-supplied cutoff values to plot as points.
When non-NULL, overrides \code{n_cuts}. Supports both raw numeric values
and method names from \code{\link[OptimalCutpoints]{optimal.cutpoints}}.}

\item{cutoffs_labels}{A character vector of user-supplied labels for the
cutoffs. Must be the same length as \code{cutoffs_at}. When NULL, labels
are generated automatically.}

\item{cutoffs_accuracy}{A numeric value specifying the rounding precision
for automatically generated cutoff labels. Default: \code{0.001}.}

\item{n_cuts}{An integer specifying the number of evenly-spaced quantile-based
cutoff points. Ignored when \code{cutoffs_at} is non-NULL. Default: \code{0}
(no quantile cutoffs).}

\item{increasing}{A logical value. If TRUE (default), higher scores indicate
the positive class; if FALSE, lower scores indicate the positive class.}
}
\value{
A data frame with columns \code{cutoff}, \code{x} (1 - specificity),
\code{y} (sensitivity), \code{label}, and \code{cat_by} (the category name).
}
\description{
Internal helper that computes the sensitivity and specificity values for a
set of user-supplied cutoff points (or computed via OptimalCutpoints methods)
across categories defined by group_by and/or facet_by columns. Used by
\code{ROCCurveAtomic()} to annotate the ROC curve with cutoff markers and labels.
}
\details{
Cutoffs can be specified either as numeric values (raw score thresholds) or as
method names from the \code{OptimalCutpoints} package for automatic optimal
cutoff identification. When \code{n_cuts > 0}, \code{n_cuts} evenly-spaced
quantile values of the score distribution are used as cutoffs.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatialplot.R
\name{.flip_y}
\alias{.flip_y}
\alias{.flip_y.SpatRaster}
\alias{.flip_y.SpatVector}
\alias{.flip_y.data.frame}
\title{Flip y-coordinates for spatial data objects}
\usage{
.flip_y(data, ...)

\method{.flip_y}{SpatRaster}(data, ...)

\method{.flip_y}{SpatVector}(data, ...)

\method{.flip_y}{data.frame}(data, y = "y", ...)
}
\arguments{
\item{data}{A \code{SpatRaster}, \code{SpatVector}, or \code{data.frame}.}

\item{...}{Additional arguments (not used).}

\item{y}{A character string specifying the y column name for \code{data.frame}
input. Ignored for \code{SpatRaster} and \code{SpatVector}. Default is \code{"y"}.}
}
\value{
For \code{SpatRaster} input, a \code{SpatRaster} flipped vertically with
negated y-extent. For \code{SpatVector} input, a \code{SpatVector} with
negated y-coordinates. For \code{data.frame} input, a data frame with the
specified \code{y} column negated.
}
\description{
These internal S3 methods flip the y-coordinates of \code{SpatRaster},
\code{SpatVector}, and \code{data.frame} objects.

For rasters, the raster is flipped vertically and its extent is negated.
For vectors, the y-coordinates of all geometries are negated.
For data frames, the specified y column is negated.
}
\details{
These functions are intended for internal use to facilitate coordinate
transformations. When visualizing spatial data, it is often necessary to
flip the y-axis to put the origin at the top-left corner. However,
\code{\link[ggplot2]{geom_sf}()} does not work with
\code{\link[ggplot2]{scale_y_reverse}()}. See
\href{https://github.com/tidyverse/ggplot2/issues/4021#issuecomment-650787582}{this
GitHub comment} for details. These functions negate the y-coordinates so
that the axis labels can be displayed with reversed sign, mimicking
\code{scale_y_reverse()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theming.R
\name{show_palettes}
\alias{show_palettes}
\title{Show the color palettes}
\usage{
show_palettes(
  palettes = NULL,
  type = c("discrete", "continuous"),
  index = NULL,
  palette_names = NULL,
  return_names = TRUE,
  return_palettes = FALSE
)
}
\arguments{
\item{palettes}{A list of color palettes. If \code{NULL}, uses default palettes.}

\item{type}{A character vector specifying the type of palettes to include. Default is "discrete".}

\item{index}{A numeric vector specifying the indices of the palettes to include. Default is \code{NULL}.}

\item{palette_names}{A character vector specifying the names of the SCP palettes to include. Default is \code{NULL}.}

\item{return_names}{A logical value indicating whether to return the names of the selected palettes. Default is \code{TRUE}.}

\item{return_palettes}{A logical value indicating whether to return the colors of selected palettes. Default is \code{FALSE}.}
}
\value{
A list of palette names or a list of palettes.
}
\description{
This function displays color palettes using ggplot2.
}
\examples{
show_palettes(palettes = list(c("red", "blue", "green"), c("yellow", "purple", "orange")))
all_palettes <- show_palettes(return_palettes = TRUE)
names(all_palettes)
all_palettes[["simspec"]]
show_palettes(index = 1:10)
show_palettes(type = "discrete", index = 1:10)
show_palettes(type = "continuous", index = 1:10)
show_palettes(
    palette_names = c("Paired", "nejm", "simspec", "Spectral", "jet"),
    return_palettes = TRUE
)

}
\seealso{
\code{\link{palette_list}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_palcolor}
\alias{check_palcolor}
\title{check_palcolor
Check if the palcolor can be properly used}
\usage{
check_palcolor(palcolor, datas_name)
}
\arguments{
\item{palcolor}{palcolor}

\item{datas_name}{names of the split data}
}
\value{
named list containing color names
}
\description{
check_palcolor
Check if the palcolor can be properly used
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/barplot.R
\name{BarPlotGrouped}
\alias{BarPlotGrouped}
\title{Bar plot with groups}
\usage{
BarPlotGrouped(
  data,
  x,
  x_sep = "_",
  y = NULL,
  scale_y = FALSE,
  flip = FALSE,
  group_by,
  group_by_sep = "_",
  group_name = NULL,
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  label = NULL,
  label_nudge = 0.02,
  label_fg = "black",
  label_size = 4,
  label_bg = "white",
  label_bg_r = 0.1,
  add_bg = FALSE,
  bg_palette = "stripe",
  bg_palcolor = NULL,
  bg_alpha = 0.2,
  alpha = 1,
  x_text_angle = 0,
  aspect.ratio = 1,
  add_line = NULL,
  line_color = "red2",
  line_width = 0.6,
  line_type = 2,
  line_name = NULL,
  add_trend = FALSE,
  trend_color = "black",
  trend_linewidth = 1,
  trend_ptsize = 2.5,
  position = "auto",
  position_dodge_preserve = "total",
  y_min = NULL,
  y_max = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  keep_na = FALSE,
  keep_empty = FALSE,
  expand = waiver(),
  width = 0.8,
  facet_by = NULL,
  facet_scales = "fixed",
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis.}

\item{x_sep}{A character string to concatenate the columns in \code{x}, if multiple columns are provided.}

\item{y}{A character string specifying the column name of the data frame to plot for the y-axis.}

\item{scale_y}{A logical value indicating whether to scale the total y values in each group to 100\%.
Only works when group_by is specified.}

\item{flip}{A logical value indicating whether to flip the x and y axes.}

\item{group_by}{A character vector specifying the column as the group_by of the plot.
A character/factor column is expected.}

\item{group_by_sep}{A character string to concatenate the columns in \code{group_by}, if multiple columns are provided.}

\item{group_name}{A character string to specify the name of the group_by in the legend.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{label}{A column name for the values to be displayed on the top of the bars.
If TRUE, the y values will be displayed.}

\item{label_nudge}{A numeric value to nudge the labels (the distance between the label and the top of the bar).}

\item{label_fg}{A character string indicating the color of the label.}

\item{label_size}{A numeric value indicating the size of the label.}

\item{label_bg}{A character string indicating the background color of the label.}

\item{label_bg_r}{A numeric value indicating the radius of the background.}

\item{add_bg}{A logical value indicating whether to add a background to the plot.}

\item{bg_palette}{A character string indicating the palette to use for the background.}

\item{bg_palcolor}{A character string indicating the color to use for the background.}

\item{bg_alpha}{A numeric value indicating the alpha of the background.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{add_line}{A numeric value indicating the y value to add a horizontal line.}

\item{line_color}{A character string indicating the color of the line.}

\item{line_width}{A numeric value indicating the size of the line.}

\item{line_type}{A numeric value indicating the type of the line.}

\item{line_name}{A character string indicating the name of the line.}

\item{add_trend}{A logical value to add trend line to the plot.}

\item{trend_color}{A character string to specify the color of the trend line.}

\item{trend_linewidth}{A numeric value to specify the width of the trend line.}

\item{trend_ptsize}{A numeric value to specify the size of the trend line points.}

\item{position}{A character string indicating the position of the bars.
If "auto", the position will be "stack" if group_by has more than 5 levels, otherwise "dodge".
"fill" is also a valid option. Only works when group_by is not NULL.}

\item{position_dodge_preserve}{Should dodging preserve the "total" width of all elements at a position, or the width of a "single" element?}

\item{y_min}{A numeric value to specify the minimum value of the y axis.}

\item{y_max}{A numeric value to specify the maximum value of the y axis.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{expand}{The values to expand the x and y axes. It is like CSS padding.
When a single value is provided, it is used for both axes on both sides.
When two values are provided, the first value is used for the top/bottom side and the second value is used for the left/right side.
When three values are provided, the first value is used for the top side, the second value is used for the left/right side, and the third value is used for the bottom side.
When four values are provided, the values are used for the top, right, bottom, and left sides, respectively.
You can also use a named vector to specify the values for each side.
When the axis is discrete, the values will be applied as 'add' to the 'expansion' function.
When the axis is continuous, the values will be applied as 'mult' to the 'expansion' function.
See also \url{https://ggplot2.tidyverse.org/reference/expansion.html}}

\item{width}{A numeric value specifying the width of the bars.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{...}{Additional arguments.}
}
\value{
A ggplot object.
}
\description{
Create a bar plot with groups.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rgba_to_rgb}
\alias{rgba_to_rgb}
\title{Convert RGBA to RGB}
\usage{
rgba_to_rgb(RGBA, BackGround = c(1, 1, 1))
}
\description{
Convert RGBA to RGB
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dimplot.R
\name{DimPlotAtomic3D}
\alias{DimPlotAtomic3D}
\title{Internal helper to create a 3D dimension reduction plot using plotly}
\usage{
DimPlotAtomic3D(
  data,
  dims,
  group_by = NULL,
  features = NULL,
  colorby,
  colors = NULL,
  feat_colors_value = NULL,
  label_use = NULL,
  labels_tb = NULL,
  keep_empty_group = FALSE,
  bg_color = "grey80",
  color_name = "",
  pt_size = NULL,
  pt_alpha = 1,
  show_stat = TRUE,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  label = FALSE,
  label_insitu = FALSE,
  label_size = 4,
  label_fg = "white",
  label_bg = "black",
  highlight = NULL,
  highlight_color = "black",
  highlight_size = 1,
  highlight_stroke = 0.8,
  highlight_alpha = 1,
  graph = NULL,
  edge_size = c(0.05, 0.5),
  edge_alpha = 0.1,
  edge_color = "grey40",
  lineages = NULL,
  lineages_trim = c(0.01, 0.99),
  lineages_span = 0.75,
  lineages_palette = "Dark2",
  lineages_palcolor = NULL,
  palette = "Spectral",
  palcolor = NULL,
  palreverse = FALSE,
  n_sampled = NULL
)
}
\arguments{
\item{data}{A data frame.}

\item{dims}{A character vector of the column names to plot on the x, y (and optionally z) axes or a numeric vector
of the column indices. When 3 dimensions are provided, a 3D interactive plot is created using plotly.
Supported in 3D: group_by, features, labels, highlight, lineages, graph/network, show_stat, order.
Not supported in 3D: add_mark, stat_by, add_density, velocity, hex, facet_by, raster.}

\item{group_by}{A character string of the column name to group the data by for discrete colouring.
A character/factor column is expected. If multiple columns are provided, the columns will be concatenated with \code{group_by_sep}.}

\item{features}{A character vector of the column names to plot as features (continuous colouring).
When multiple features are provided and \code{facet_by} is not set, the data is pivoted to long format and
faceted by feature name.}

\item{colorby}{A character string naming the column used for colour mapping
(either the \code{group_by} factor or the \code{features} numeric column).}

\item{colors}{A named character vector mapping group levels to hex colours.
Only used in group_by mode.}

\item{feat_colors_value}{A numeric vector of the colour-scale anchor values
for feature mode, as returned by \code{prepare_continuous_color_scale()}.}

\item{label_use}{A character vector of formatted legend labels (with
optional count annotations) used in group_by mode.}

\item{labels_tb}{A table of group counts for trace iteration in group_by
mode.}

\item{keep_empty_group}{A logical value. If \code{TRUE}, empty factor levels are
preserved; traces with zero rows are still added to the legend.}

\item{bg_color}{A character string specifying the colour used for NA-valued points and
background context points drawn from other facets. Default is \code{"grey80"}.}

\item{color_name}{A character string used as the title for the continuous colour bar
in feature mode. Default is \code{""}.}

\item{pt_size}{A numeric value for point size, or a character string naming a numeric column
in \code{data} to map point sizes to. If \code{NULL} (default), auto-calculated as
\code{min(3000 / nrow(data), 0.6)}. Column-name mapping only applies to 2D plots and
is ignored with a message when \code{raster = TRUE}.}

\item{pt_alpha}{A numeric value in \verb{[0, 1]} for the point transparency. Default is \code{1}.}

\item{show_stat}{A logical value. If \code{TRUE} (default), the number of points per group is shown
in the legend labels and subtitle. Ignored when \code{theme = "theme_blank"}.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{label}{A logical value. If \code{TRUE}, group labels (numeric indices by default, or group
names when \code{label_insitu = TRUE}) are placed at the median coordinates of each group.
Forced to \code{TRUE} when \code{label_repel} or \code{label_insitu} is set.}

\item{label_insitu}{A logical value. If \code{TRUE}, the raw group names are placed at the group median
coordinates instead of numeric indices. Forces \code{label = TRUE}. Default is \code{FALSE}.}

\item{label_size}{A numeric value for the label text size. Passed to \code{ggrepel::geom_text_repel()}.
Default is \code{4}.}

\item{label_fg}{A character string for the label text (foreground) colour. Default is \code{"white"}.}

\item{label_bg}{A character string for the label background / outline colour. Default is \code{"black"}.}

\item{highlight}{A specification for highlighted points:
\itemize{
\item \code{NULL} (default): no highlighting.
\item \code{TRUE}: highlight all points (adds a dark outline around every point).
\item A character string: a dplyr filter expression (e.g., \code{"clusters == 'Ductal'"}).
\item A character vector: row names to highlight.
\item A numeric vector: row indices to highlight.
}}

\item{highlight_color}{A character string for the colour of the outer highlight ring.
Default is \code{"black"}.}

\item{highlight_size}{A numeric value for the size of the inner (coloured) highlight point.
Default is \code{1}.}

\item{highlight_stroke}{A numeric value for the thickness of the outer highlight ring (the
difference between the outer ring size and \code{highlight_size}). Default is \code{0.8}.}

\item{highlight_alpha}{A numeric value in \verb{[0, 1]} for the transparency of highlighted points.
Default is \code{1}.}

\item{graph}{A specification for network / graph edges to overlay. Sources:
\itemize{
\item A character string starting with \code{"@"} (e.g., \code{"@graph"}): extracts the attribute
named \code{"graph"} from \code{attributes(data)}.
\item A \code{Graph} object (e.g., Seurat): coerced to dense matrix via \code{as.matrix()}.
\item A \code{matrix}, \code{data.frame}, or \code{dgCMatrix}: used directly as the adjacency matrix.
\item Numeric indices or character column names: extracts columns from \code{data}.
Edges are drawn for non-zero, lower-triangle entries. Requires \code{data} to have row names
matching the matrix dimnames.
}}

\item{edge_size}{A numeric vector of length 2 specifying the range \verb{[min, max]} for
\code{scale_linewidth_continuous(range = ...)} applied to edge widths. Default is \code{c(0.05, 0.5)}.}

\item{edge_alpha}{A numeric value in \verb{[0, 1]} for the transparency of graph edges.
Default is \code{0.1}.}

\item{edge_color}{A character string for the colour of graph edges. Default is \code{"grey40"}.}

\item{lineages}{A character vector of column names representing pseudotime / trajectory lineages.
Each column is fitted with a LOESS smooth (\verb{span = lineages_span, degree = 2}) across the
2D embedding, after trimming the top and bottom \code{lineages_trim} quantiles.
Only supported in 2D without \code{facet_by}.}

\item{lineages_trim}{A numeric vector of length 2 specifying the lower and upper quantile
thresholds \verb{[0, 1]} for trimming lineage values before LOESS fitting.
Default is \code{c(0.01, 0.99)}.}

\item{lineages_span}{A numeric value passed as \code{span} to \code{stats::loess()} controlling the
smoothness of the lineage curve. Smaller values follow the data more closely.
Default is \code{0.75}.}

\item{lineages_palette}{A character string naming the palette for lineage colours.
Default is \code{"Dark2"}.}

\item{lineages_palcolor}{A character vector of specific colours for lineage curves.
Default is \code{NULL} (auto-resolved from \code{lineages_palette}).}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{n_sampled}{An integer indicating how many points were retained after
down-sampling (displayed in the subtitle). \code{NULL} if no sampling occurred.}
}
\description{
Renders an interactive 3D scatter plot via \strong{plotly} for dimension reduction
data with three ordination axes. Called automatically by \code{DimPlotAtomic()}
when \code{length(dims) == 3}.

The function supports two visualisation modes:
\itemize{
\item \strong{group_by} — discrete group colouring with one trace per group, matching
the 2D legend behaviour via \code{label_use}.
\item \strong{features} — continuous colour scale with a \code{plotly} colorscale bar.
}

Graphs / networks are rendered as 3D line segments (with NA separators for
line breaks). Lineage curves are fitted via \code{stats::loess()} per lineage
column and drawn as smoothed paths. For datasets exceeding 100 000 points,
per-point hover text is disabled to reduce the JSON payload.
}
\section{Architecture}{

\strong{DimPlotAtomic3D} executes the following steps:
\enumerate{
\item \strong{plotly availability check} — errors early if the \code{plotly} package is
not installed.
\item \strong{Axis labels} — defaults \code{xlab} / \code{ylab} / \code{zlab} from \code{dims}.
\item \strong{Marker size scaling} — multiplies \code{pt_size} by 2 to convert from
ggplot2 mm units to plotly pixel units.
\item \strong{Large-data detection} — disables per-point hover text when
\code{nrow(data) > 1e5}.
\item \strong{Base plotly object} — \code{plotly::plot_ly()} initialisation.
\item \strong{Graph / network edges} (if \code{graph} is provided) — resolves the graph
source (\verb{@attribute}, Graph object, matrix, data.frame, column indices,
or column names), aligns row/column names with \code{data}, sets upper-triangle
and zero entries to \code{NA}, melts into an edge data frame, builds x/y/z
coordinate vectors (interleaved with NA for line breaks), and adds
\code{scatter3d} line traces.
\item \strong{Main scatter traces}:
\itemize{
\item \emph{group_by mode}: iterates over \code{labels_tb} entries, creating one trace
per group with the resolved colour. NA groups use \code{bg_color}. Legend
labels use \code{label_use} (with count annotations if \code{show_stat = TRUE}).
\item \emph{features mode}: builds a plotly-style colorscale from the continuous
palette. NA points are rendered first in \code{bg_color} without a colour
bar; non-NA points follow with a \code{showscale = TRUE} colour bar.
}
\item \strong{Highlight} — if \code{highlight} is specified, highlighted points are
plotted as open circles with increased marker size (to account for
stroke). Supports \code{TRUE} (all points), a filter expression string, or
a vector of row names / numeric indices.
\item \strong{Lineages} — each lineage column is validated, trimmed to
\verb{[lineages_trim[1], lineages_trim[2]]} quantiles, ordered by value,
and a \code{loess(span = lineages_span, degree = 2)} smooth is fitted per
dimension. The smoothed curve is rendered as a 3D line trace.
\item \strong{Labels} — when \code{label = TRUE} and \code{group_by} is set, group median
coordinates are computed via \code{aggregate()}. In 3D, plotly does not
support text background/outline, so \code{label_bg} is used directly as
the text colour. Text size is scaled by 3× for plotly.
\item \strong{Layout} — subtitle is composited with an optional "Showing N sampled
points" note. The full title is rendered as HTML (\verb{<br><sup>...</sup>}).
The scene sets \code{aspectratio = list(x = 1, y = 1, z = 1)} and axis
titles.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sankeyplot.R
\name{SankeyPlot}
\alias{SankeyPlot}
\alias{AlluvialPlot}
\title{Sankey / Alluvial Plot}
\usage{
SankeyPlot(
  data,
  in_form = c("auto", "long", "lodes", "wide", "alluvia", "counts"),
  x,
  x_sep = "_",
  y = NULL,
  stratum = NULL,
  stratum_sep = "_",
  alluvium = NULL,
  alluvium_sep = "_",
  split_by = NULL,
  split_by_sep = "_",
  keep_empty = TRUE,
  flow = FALSE,
  expand = c(0, 0, 0, 0),
  nodes_legend = c("auto", "separate", "merge", "none"),
  nodes_color = "grey30",
  links_fill_by = NULL,
  links_fill_by_sep = "_",
  links_name = NULL,
  links_color = "gray80",
  nodes_palette = "Paired",
  nodes_palcolor = NULL,
  nodes_alpha = 1,
  nodes_label = FALSE,
  nodes_label_miny = 0,
  nodes_width = 0.25,
  links_palette = "Paired",
  links_palcolor = NULL,
  palreverse = FALSE,
  links_alpha = 0.6,
  legend.box = "vertical",
  x_text_angle = 0,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  flip = FALSE,
  theme = "theme_this",
  theme_args = list(),
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)

AlluvialPlot(
  data,
  in_form = c("auto", "long", "lodes", "wide", "alluvia", "counts"),
  x,
  x_sep = "_",
  y = NULL,
  stratum = NULL,
  stratum_sep = "_",
  alluvium = NULL,
  alluvium_sep = "_",
  split_by = NULL,
  split_by_sep = "_",
  keep_empty = TRUE,
  flow = FALSE,
  expand = c(0, 0, 0, 0),
  nodes_legend = c("auto", "separate", "merge", "none"),
  nodes_color = "grey30",
  links_fill_by = NULL,
  links_fill_by_sep = "_",
  links_name = NULL,
  links_color = "gray80",
  nodes_palette = "Paired",
  nodes_palcolor = NULL,
  nodes_alpha = 1,
  nodes_label = FALSE,
  nodes_label_miny = 0,
  nodes_width = 0.25,
  links_palette = "Paired",
  links_palcolor = NULL,
  palreverse = FALSE,
  links_alpha = 0.6,
  legend.box = "vertical",
  x_text_angle = 0,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  flip = FALSE,
  theme = "theme_this",
  theme_args = list(),
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{in_form}{A character string specifying the input data format.
One of \code{"auto"} (default), \code{"long"}, \code{"lodes"},
\code{"wide"}, \code{"alluvia"}, or \code{"counts"}.
\code{"long"} is an alias for \code{"lodes"}; \code{"wide"} is an alias for
\code{"alluvia"}.  See the \code{data} parameter of \code{\link{SankeyPlot}}
for format descriptions.}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis.}

\item{x_sep}{A character string to join multiple \code{x} columns when
\code{in_form} is \code{"lodes"} or auto-determined as lodes.
Default \code{"_"}.}

\item{y}{A character string specifying the column name of the data frame to plot for the y-axis.}

\item{stratum}{A character string specifying the column that defines the
node categories at each x-axis position.  Each unique value becomes a
stratum (node block) at each x position.  When \code{NULL}, defaults to
\code{links_fill_by}.  Multiple columns are concatenated with
\code{stratum_sep}.  Ignored in \code{"alluvia"} format.}

\item{stratum_sep}{A character string to join multiple \code{stratum}
columns.  Default \code{"_"}.}

\item{alluvium}{A character string specifying the column that identifies
individual flows (alluvia) across x-axis positions.  Each unique value
represents a single observational unit tracked across positions.  When
\code{NULL} in \code{"counts"} format, an auto-generated identifier is
created.  Multiple columns are concatenated with \code{alluvium_sep}.
Ignored in \code{"alluvia"} format.}

\item{alluvium_sep}{A character string to join multiple \code{alluvium}
columns.  Default \code{"_"}.}

\item{split_by}{The column(s) to split data by and plot separately.
When \code{combine = TRUE}, the combined plot's data (\code{p$data})
contains the rows of every split, tagged with the \code{split_by}
column. For ggraph-based plots (e.g. \code{\link{Network}},
\code{\link{ClustreePlot}}), the node layout of every split is
exposed with the split column, and the original link rows (also
tagged with the \code{split_by} column) are available as the
\code{"edges"} attribute of the data.}

\item{split_by_sep}{The separator for multiple split_by columns. See \code{split_by}}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{flow}{A logical value.  When \code{FALSE} (default),
\code{\link[ggalluvial]{geom_alluvium}()} is used for the links.  When
\code{TRUE}, \code{\link[ggalluvial]{geom_flow}()} is used instead, which
draws the flows with a directional gradient between x positions.}

\item{expand}{The values to expand the x and y axes. It is like CSS padding.
When a single value is provided, it is used for both axes on both sides.
When two values are provided, the first value is used for the top/bottom side and the second value is used for the left/right side.
When three values are provided, the first value is used for the top side, the second value is used for the left/right side, and the third value is used for the bottom side.
When four values are provided, the values are used for the top, right, bottom, and left sides, respectively.
You can also use a named vector to specify the values for each side.
When the axis is discrete, the values will be applied as 'add' to the 'expansion' function.
When the axis is continuous, the values will be applied as 'mult' to the 'expansion' function.
See also \url{https://ggplot2.tidyverse.org/reference/expansion.html}}

\item{nodes_legend}{Controls how the node legend is displayed.  One of:
\describe{
\item{\code{"auto"} (default)}{Automatically determined:
if \code{nodes_label = TRUE}, or if \code{stratum} is identical to
\code{links_fill_by} with matching colours, the legend is hidden.
Otherwise, overlapping stratum values across x positions are checked:
any overlap produces a merged legend; no overlap produces separate
legends per x position.}
\item{\code{"merge"}}{A single merged legend for all nodes.}
\item{\code{"separate"}}{One legend per x-axis position, generated via
separate \code{\link[ggplot2]{scale_fill_manual}()} layers.}
\item{\code{"none"}}{No node legend is shown.}
}}

\item{nodes_color}{A character string specifying the border colour of the
node (stratum) rectangles.  Use the special value \code{".fill"} to match
the border colour to the node fill colour.  Default \code{"grey30"}.}

\item{links_fill_by}{A character string specifying the column that
determines the fill colour of the links (alluvia / flows).  When
\code{NULL} in \code{"lodes"} format, defaults to \code{alluvium}.  In
\code{"counts"} format with the \code{"."} prefix, this parameter is
required.  Multiple columns are concatenated with
\code{links_fill_by_sep}.}

\item{links_fill_by_sep}{A character string to join multiple
\code{links_fill_by} columns.  Default \code{"_"}.}

\item{links_name}{A character string for the legend title of the link fill
scale.  When \code{NULL} (default), the \code{links_fill_by} column name
is used.}

\item{links_color}{A character string specifying the border colour of the
links (alluvia / flows).  Use the special value \code{".fill"} to match
the link border colour to the link fill colour.  Default \code{"gray80"}.}

\item{nodes_palette}{A character string specifying the colour palette for
the node (stratum) fill.  Passed to \code{\link{palette_this}()}.
Default \code{"Paired"}.}

\item{nodes_palcolor}{A character vector of custom colours for the node
fill, used as \code{palcolor} in \code{\link{palette_this}()}.  When
\code{NULL} (default), the palette colours are used directly.}

\item{nodes_alpha}{A numeric value in \eqn{[0, 1]} controlling the
transparency of the node (stratum) fill.  Default \code{1}.}

\item{nodes_label}{A logical value.  When \code{TRUE}, stratum labels are
drawn inside each node using \code{\link[ggplot2]{geom_label}()} with
\code{\link[ggalluvial]{StatStratum}}.  Default \code{FALSE}.}

\item{nodes_label_miny}{A numeric value specifying the minimum y
(frequency) threshold for displaying node labels.  Nodes with y-values
below this threshold are not labelled.  Default \code{0}.}

\item{nodes_width}{A numeric value (typically 0–1) specifying the width of
the node (stratum) rectangles as a fraction of the x-axis spacing.
Default \code{0.25}.}

\item{links_palette}{A character string specifying the colour palette for
the link fill.  Passed to \code{\link{palette_this}()}.
Default \code{"Paired"}.}

\item{links_palcolor}{A character vector of custom colours for the link
fill, used as \code{palcolor} in \code{\link{palette_this}()}.  When
\code{NULL} (default), the palette colours are used directly.}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{links_alpha}{A numeric value in \eqn{[0, 1]} controlling the
transparency of the link fill.  Default \code{0.6}.}

\item{legend.box}{A character string specifying the arrangement of legend
boxes, either \code{"vertical"} (default) or \code{"horizontal"}.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{flip}{A logical value.  When \code{TRUE},
\code{\link[ggplot2]{coord_flip}()} is applied to swap the x and y axes.
Default \code{FALSE}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{seed}{The random seed to use. Default is 8525.}

\item{combine}{Whether to combine the plots into one when facet is FALSE. Default is TRUE.}

\item{nrow}{A numeric value specifying the number of rows in the facet.}

\item{ncol}{A numeric value specifying the number of columns in the facet.}

\item{byrow}{A logical value indicating whether to fill the plots by row.}

\item{axes}{A string specifying how axes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axes in individual plots.
\item 'collect' will remove duplicated axes when placed in the same run of rows or columns of the layout.
\item 'collect_x' and 'collect_y' will remove duplicated x-axes in the columns or duplicated y-axes in the rows respectively.
}}

\item{axis_titles}{A string specifying how axis titltes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axis titles in individual plots.
\item 'collect' will remove duplicated titles in one direction and merge titles in the opposite direction.
\item 'collect_x' and 'collect_y' control this for x-axis titles and y-axis titles respectively.
}}

\item{guides}{A string specifying how guides should be treated in the layout. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'collect' will collect guides below to the given nesting level, removing duplicates.
\item 'keep' will stop collection at this level and let guides be placed alongside their plot.
\item 'auto' will allow guides to be collected if a upper level tries, but place them alongside the plot if not.
}}

\item{design}{Specification of the location of areas in the layout, passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE. When specified, \code{nrow}, \code{ncol}, and \code{byrow} are ignored.
See \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}} for more details.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object (single panel, no \code{split_by}), a
\code{patchwork} object (when \code{combine = TRUE} and
\code{split_by} is used), or a named list of \code{ggplot} objects
(when \code{combine = FALSE}).  Each plot carries \code{height} and
\code{width} attributes in inches.
}
\description{
Draws Sankey (alluvial) diagrams to visualise flow, movement, or change
from one categorical state to another across discrete positions (time
points, stages, or groups).  The plot consists of \strong{nodes} (vertical
blocks, or strata) representing categories at each position, and
\strong{links} (alluvia / flows) representing the observation units that
move between categories across positions.

The function accepts data in several formats, controlled by \code{in_form}:
\describe{
\item{\code{"lodes"} / \code{"long"}}{Each row is an observation at one
x-position, with columns for \code{x}, \code{stratum}, \code{alluvium},
and optionally \code{y}.}
\item{\code{"alluvia"} / \code{"wide"}}{Each row is an observation unit
tracked across all positions; \code{x} columns represent the categories
at each position.  Converted internally via
\code{\link[ggalluvial]{to_lodes_form}()}.}
\item{\code{"counts"}}{Numeric columns under each \code{x} represent
frequencies.  When the first element of \code{x} is \code{"."}, the
\code{links_fill_by} values are injected as an additional first column
of nodes, visualising the source distribution of flows.}
\item{\code{"auto"} (default)}{Automatically detects the format:
numeric multi-column \code{x} → \code{"counts"};
multi-column \code{x} passing \code{is_alluvia_form} → \code{"alluvia"};
otherwise → \code{"lodes"}.}
}

Supports \strong{split_by} to produce separate sub-plots for different
subsets of the data, \strong{facet_by} for within-plot faceting, and
independent styling of nodes and links (colours, alpha, borders, labels,
and legend behaviour).

\code{AlluvialPlot} is an alias of \code{SankeyPlot}.
}
\section{split_by workflow}{

When \code{split_by} is provided:
\enumerate{
\item The \code{split_by} column(s) are validated and coerced to factors
via \code{\link{check_columns}()}.  Multi-column \code{split_by}
is concatenated with \code{split_by_sep}.
\item Empty factor levels are dropped from \code{split_by}.
\item The data is split by \code{split_by} level (preserving level order).
If \code{split_by} is \code{NULL}, the data is wrapped in a
single-element list with name \code{"..."}.
\item \code{\link{SankeyPlotAtomic}()} is called for each split, with
\code{title} resolved per level (supports function-valued titles).
\item Results are combined via \code{\link{combine_plots}()} (when
\code{combine = TRUE}) or returned as a named list.
}
}

\examples{
\donttest{
# Examples from ggalluvial datasets
set.seed(8525)

data(UCBAdmissions, package = "datasets")
UCBAdmissions <- as.data.frame(UCBAdmissions)
SankeyPlot(as.data.frame(UCBAdmissions), x = c("Gender", "Dept"),
    y = "Freq", nodes_width = 1/12, links_fill_by = "Admit", nodes_label = TRUE,
    nodes_palette = "simspec", links_palette = "Set1", links_alpha = 0.5,
    nodes_palcolor = "black", links_color = "transparent")

data(HairEyeColor, package = "datasets")
SankeyPlot(as.data.frame(HairEyeColor), x = c("Hair", "Eye", "Sex"),
    y = "Freq", links_fill_by = "Eye", nodes_width = 1/8, nodes_alpha = 0.4,
    flip = TRUE, reverse = FALSE, knot.pos = 0, links_color = "transparent",
    ylab = "Freq", links_alpha = 0.5, links_name = "Eye (links)", links_palcolor = c(
        Brown = "#70493D", Hazel = "#E2AC76", Green = "#3F752B", Blue = "#81B0E4"))

data(Refugees, package = "alluvial")
country_regions <- c(
    Afghanistan = "Middle East",
    Burundi = "Central Africa",
    `Congo DRC` = "Central Africa",
    Iraq = "Middle East",
    Myanmar = "Southeast Asia",
    Palestine = "Middle East",
    Somalia = "Horn of Africa",
    Sudan = "Central Africa",
    Syria = "Middle East",
    Vietnam = "Southeast Asia"
)
Refugees$region <- country_regions[Refugees$country]
SankeyPlot(Refugees, x = "year", y = "refugees", alluvium = "country",
    links_fill_by = "country", links_color = ".fill", links_alpha = 0.75,
    links_palette = "Set3", facet_by = "region", x_text_angle = -45, nodes_legend = "none",
    theme_args = list(strip.background = ggplot2::element_rect(fill="grey80")),
    decreasing = FALSE, nodes_width = 0, nodes_color = "transparent", ylab = "refugees",
    title = "Refugee volume by country and region of origin")

data(majors, package = "ggalluvial")
majors$curriculum <- as.factor(majors$curriculum)
SankeyPlot(majors, x = "semester", stratum = "curriculum", alluvium = "student",
    links_fill_by = "curriculum", flow = TRUE, stat = "alluvium", nodes_palette = "Set2",
    links_palette = "Set2")

data(vaccinations, package = "ggalluvial")
vaccinations <- transform(vaccinations,
    response = factor(response, rev(levels(response))))
SankeyPlot(vaccinations, x = "survey", stratum = "response", alluvium = "subject",
    y = "freq", links_fill_by = "response", nodes_label = TRUE, nodes_alpha = 0.5,
    nodes_palette = "seurat", links_palette = "seurat", links_alpha = 0.5,
    legend.position = "none", flow = TRUE, expand = c(0, 0, 0, .15), stat = "alluvium",
    title = "vaccination survey responses at three points in time")

data(Titanic, package = "datasets")
SankeyPlot(as.data.frame(Titanic), x = c("Class", "Sex"), y = "Freq",
    links_fill_by = "Survived", flow = TRUE, facet_by = "Age", facet_scales = "free_y",
    nodes_label = TRUE, expand = c(0.05, 0), xlab = "", links_palette = "Set1",
    nodes_palcolor = "white", nodes_label_miny = 10)

# Simulated examples
df <- data.frame(
    Clone = paste0("clone", 1:10),
    Timepoint1 = sample(c(rep(0, 30), 1:100), 10),
    Timepoint2 = sample(c(rep(0, 30), 1:100), 10)
)
SankeyPlot(df, x = c("Timepoint1", "Timepoint2"), alluvium = "Clone",
    links_color = ".fill")

df <- data.frame(
    Clone = rep(paste0("clone", 1:6), each = 2),
    Timepoint1 = sample(c(rep(0, 30), 1:100), 6),
    Timepoint2 = sample(c(rep(0, 30), 1:100), 6),
    Group = rep(c("A", "B"), 6)
)
SankeyPlot(df, x = c(".", "Timepoint1", "Timepoint2"),
    stratum = "Group", links_fill_by = "Clone", links_color = ".fill")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/upsetplot.R
\name{detect_upset_datatype}
\alias{detect_upset_datatype}
\title{Detect the type of the input data for an UpSet plot}
\usage{
detect_upset_datatype(data, group_by = NULL, id_by = NULL)
}
\arguments{
\item{data}{A data frame or a named list of element vectors.}

\item{group_by}{A character string specifying the column name for the set-definition groups.
Only meaningful when \code{data} is a data frame.}

\item{id_by}{A character string specifying the column name for instance identifiers.
Required when \code{group_by} is a single column and \code{data} is a data frame.}
}
\value{
A character string, one of \code{"long"}, \code{"wide"}, \code{"list"},
or \code{"upset"}.
\describe{
\item{\code{"long"}}{Long-format data --- each row records one (set, element) pair.}
\item{\code{"wide"}}{Wide-format data --- each row is an element and each set has its own logical/0-1 column.}
\item{\code{"list"}}{A named list of element vectors.}
\item{\code{"upset"}}{Already an \code{UpsetPlotData} object.}
}
}
\description{
Inspects the structure of \code{data} and returns a classifier string that
downstream functions use to select the parsing strategy.  When \code{data}
is an \code{UpsetPlotData} object (from \code{\link{prepare_upset_data}()})
it short-circuits to \code{"upset"} immediately.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qqplot.R
\name{QQPlotAtomic}
\alias{QQPlotAtomic}
\title{Atomic QQ/PP plot}
\usage{
QQPlotAtomic(
  data,
  val,
  val_trans = NULL,
  type = c("qq", "pp"),
  band = NULL,
  line = list(),
  point = list(),
  fill_name = "Bands",
  band_alpha = 0.5,
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  palreverse = FALSE,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  legend.position = waiver(),
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  seed = 8525,
  xlim = NULL,
  ylim = NULL,
  xlab = ifelse(type == "qq", "Theoretical Quantiles", "Probability Points"),
  ylab = ifelse(type == "qq", "Sample Quantiles", "Cumulative Probability"),
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{val}{A character string naming the numeric column whose distribution
is compared against the theoretical distribution.}

\item{val_trans}{A transformation function applied to the \code{val}
column before plotting. For example, \code{log} or \code{sqrt}.
Default: \code{NULL} (no transformation).}

\item{type}{A character string specifying the plot type. Either
\code{"qq"} (quantile-quantile, the default) or \code{"pp"}
(probability-probability). Partial matching is supported.}

\item{band}{A list of arguments passed to \code{\link[qqplotr]{stat_qq_band}}
or \code{\link[qqplotr]{stat_pp_band}}, depending on \code{type}.
Set to \code{TRUE} or an empty list to use default arguments.
Set to \code{NULL} (the default) to suppress bands entirely.
To add multiple bands, provide a list of lists, each containing
arguments for one band (e.g. different \code{bandType} or
\code{distribution}). Each band can also include a custom
\code{mapping} aesthetic to control its fill colour legend entry.}

\item{line}{A list of arguments passed to \code{\link[qqplotr]{stat_qq_line}}
or \code{\link[qqplotr]{stat_pp_line}}, depending on \code{type}.
Default: \code{list()} (adds a reference line with default arguments).
Set to \code{NULL} to omit the line entirely.}

\item{point}{A list of arguments passed to \code{\link[qqplotr]{stat_qq_point}}
or \code{\link[qqplotr]{stat_pp_point}}, depending on \code{type}.
Default: \code{list()} (adds points with default arguments).
Set to \code{NULL} to omit points (not recommended).}

\item{fill_name}{A character string for the fill legend title used
when bands are present. Default: \code{"Bands"}.}

\item{band_alpha}{A numeric value in \code{[0, 1]} setting the
transparency of all bands. Individual bands can override this via
\code{alpha} inside the \code{band} argument list.
Default: \code{0.5}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{seed}{A numeric value for the random seed used internally.
Default: \code{8525}. Passed to \code{\link{set.seed}}.}

\item{xlim}{A numeric vector of length 2 specifying the x-axis limits.
Default: \code{NULL} (use data range).}

\item{ylim}{A numeric vector of length 2 specifying the y-axis limits.
Default: \code{NULL} (use data range).}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches).
}
\description{
Core implementation for drawing a single quantile-quantile (QQ) or
probability-probability (PP) plot. This is the internal workhorse
dispatched by the exported \code{\link{QQPlot}} function -- it takes a
\strong{single} data frame (no \code{split_by} support) and returns a
\code{ggplot} object. The function compares the empirical distribution
of a numeric variable against a theoretical distribution (default:
standard normal) via the \pkg{qqplotr} package.

Two plot types are supported via the \code{type} parameter:
\itemize{
\item \strong{QQ plot} (\code{type = "qq"}, the default) -- plots
sample quantiles against theoretical quantiles. Deviations from the
reference line indicate departures from the assumed distribution
(skewness, heavy tails, outliers).
\item \strong{PP plot} (\code{type = "pp"}) -- plots empirical
cumulative probability against theoretical cumulative probability.
PP plots are more sensitive to deviations in the centre of the
distribution, while QQ plots are more sensitive at the tails.
}

The function can overlay \strong{confidence bands} (\code{band}) around
the reference line using several methods (pointwise confidence intervals,
Kolmogorov-Smirnov, Tukey's simultaneous intervals, or bootstrap).
Multiple bands can be combined and each receives a separate fill colour
from the \code{palette}.
}
\section{Architecture}{


\strong{QQPlotAtomic} executes the following steps:
\enumerate{
\item \strong{ggplot dispatch} -- selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{Type validation} -- \code{match.arg()} resolves
\code{type} to \code{"qq"} or \code{"pp"}.
\item \strong{Input validation} -- \code{stopifnot()} checks that
\code{band} is \code{TRUE}, a list, or \code{NULL}; \code{line} and
\code{point} are lists or \code{NULL}; and \code{xlim}/\code{ylim}
are numeric vectors of length 2 or \code{NULL}.
\item \strong{Column resolution} -- \code{\link{check_columns}()}
validates the \code{val} column.
\item \strong{Value transformation} -- when \code{val_trans} is
provided, it is applied to the \code{val} column (e.g. log-transform).
\item \strong{Base ggplot} -- initialises
\code{ggplot(data, aes(sample = !!sym(val)))}. The \code{sample}
aesthetic is the standard interface for \pkg{qqplotr}.
\item \strong{Band rendering} -- when \code{band} is not \code{NULL}:
\enumerate{
\item Selects the band stat function:
\code{qqplotr::stat_qq_band} for QQ plots or
\code{qqplotr::stat_pp_band} for PP plots.
\item Converts \code{band = TRUE} to an empty list (default
arguments).
\item Normalises a single band (non-list or named list) into a
list-of-lists format.
\item Iterates over bands, assigning each a default fill aesthetic
(\code{"Band_1"}, \code{"Band_2"}, ..., up to a maximum of 10).
The user can override the fill mapping via \code{mapping} inside
each band's argument list.
\item Sets \code{alpha} per band, falling back to the
\code{band_alpha} parameter.
\item Adds each band to the plot via \code{do_call(band_fn, bnd)}.
}
\item \strong{Legend position resolution} -- if no bands were rendered
or all bands use default \code{"Band_"} names, the legend defaults to
\code{"none"} (when \code{legend.position} is a \code{waiver});
otherwise defaults to \code{"right"}.
\item \strong{Reference line} -- when \code{line} is not \code{NULL},
adds \code{qqplotr::stat_qq_line} (QQ) or
\code{qqplotr::stat_pp_line} (PP) via \code{do_call()}.
\item \strong{Points} -- when \code{point} is not \code{NULL}, adds
\code{qqplotr::stat_qq_point} (QQ) or
\code{qqplotr::stat_pp_point} (PP) via \code{do_call()}.
\item \strong{Fill colour scale} -- when bands are present,
\code{scale_fill_manual()} is added with colours resolved via
\code{\link{palette_this}()} using the band names, \code{palette},
\code{palcolor}, and \code{palreverse}. The legend title is set to
\code{fill_name}.
\item \strong{Axis limits} -- \code{ggplot2::xlim()} and
\code{ggplot2::ylim()} are applied if \code{xlim} or \code{ylim} are
set.
\item \strong{Labels and theme} -- \code{labs(title, subtitle, x, y)}
with fallback to \code{val} for axis labels; \code{do_call(theme,
  theme_args)}; \code{ggplot2::theme()} with \code{aspect.ratio},
\code{panel.grid.major} (grey80, dashed), \code{legend.position},
and \code{legend.direction}.
\item \strong{Dimension calculation} --
\code{\link{calculate_plot_dimensions}()} with \code{base_height =
  4.5}, \code{aspect.ratio}, and legend metrics (number of bands, band
name character width).
\item \strong{Faceting} -- \code{\link{facet_plot}()} applies
\code{facet_wrap} / \code{facet_grid} if \code{facet_by} is provided.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_keep_na}
\alias{check_keep_na}
\title{check_keep_na
Check and normalize keep_na parameter}
\usage{
check_keep_na(keep_na, cols = NA)
}
\arguments{
\item{keep_na}{keep_na}

\item{cols}{column names if keep_na is a single value}
}
\value{
normalized keep_na
}
\description{
check_keep_na
Check and normalize keep_na parameter
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/venndiagram.R
\name{detect_venn_datatype}
\alias{detect_venn_datatype}
\title{Detect the input data format for Venn diagram processing}
\usage{
detect_venn_datatype(data, group_by = NULL, id_by = NULL)
}
\arguments{
\item{data}{A data frame, a named list, or a \code{VennPlotData} object.
Data frames are classified as \code{"long"} when a single non-\code{NULL}
\code{group_by} is provided, or \code{"wide"} when \code{group_by} has two
or more columns or is \code{NULL}.}

\item{group_by}{A character string specifying the column name(s) identifying
the set membership.  A single non-\code{NULL} column indicates long format.
Two or more columns, or \code{NULL}, indicate wide format.}

\item{id_by}{A character string specifying the column name that identifies
individual elements.  Required when \code{group_by} is a single column and
\code{data} is a data frame.  Ignored for other input types.}
}
\value{
A character string indicating the detected data format:
\code{"long"}, \code{"wide"}, \code{"list"}, or \code{"venn"}.  Stops with
an error if the data does not match any recognised type.
}
\description{
Examines the structure of \code{data} and determines which of the four
supported formats it conforms to:
\itemize{
\item \code{"long"} — a data frame with one row per element-set pair.
\item \code{"wide"} — a data frame with logical/0-1 columns per set.
\item \code{"list"} — a named list of element vectors per set.
\item \code{"venn"} — a pre-computed \code{VennPlotData} object.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{blend_rgblist}
\alias{blend_rgblist}
\title{Blend a list of colors}
\usage{
blend_rgblist(Clist, mode = "blend", RGB_BackGround = c(1, 1, 1))
}
\description{
Blend a list of colors
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_keep_empty}
\alias{check_keep_empty}
\title{check_keep_empty
Check and normalize keep_empty parameter}
\usage{
check_keep_empty(keep_empty, cols = NA)
}
\arguments{
\item{keep_empty}{keep_empty}

\item{cols}{column names if keep_empty is a single value}
}
\value{
normalized keep_empty
}
\description{
check_keep_empty
Check and normalize keep_empty parameter
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap-utils.R
\name{.resolve_show_modes}
\alias{.resolve_show_modes}
\title{Resolve display modes for row/column names and split titles}
\usage{
.resolve_show_modes(
  show,
  kind = c("names", "titles"),
  which = c("row", "column"),
  by = NULL,
  split_by = NULL,
  annotation = list(),
  annotation_type = list(),
  annotation_params = list(),
  annotation_name = list(),
  legend.position = "right"
)
}
\arguments{
\item{show}{The value of \code{show_row_names}/\code{show_column_names} or
\code{row_title}/\code{column_title}}

\item{kind}{Which arguments are being resolved: "names" or "titles"}

\item{which}{The direction ("row" or "column")}

\item{by}{The name-annotation column (\code{rows_by}/\code{columns_by})}

\item{split_by}{The split-annotation column (\code{rows_split_by}/\code{columns_split_by})}

\item{annotation, annotation_type, annotation_params}{The prepared annotation
components returned by \code{\link{.prep_annotations}}}

\item{legend.position}{The legend position}
}
\value{
A list with the possibly-updated annotation components plus the
resolved \code{show} flag, \code{title}, \code{explicit}, \code{by_eff},
and \code{enabled}
}
\description{
\code{show_row_names}/\code{show_column_names} (kind = "names") and
\code{row_title}/\code{column_title} (kind = "titles") accept a character vector
combining the modes "inplace", "legend", "simple", "anno" (alias
"annotation"), and "none".  Modes are lower-priority defaults: they only
fill annotation type/params the user has not configured (per-key or via
\code{.default}).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/venndiagram.R
\name{VennDiagram}
\alias{VennDiagram}
\title{Venn / Euler diagram}
\usage{
VennDiagram(
  data,
  in_form = c("auto", "long", "wide", "list", "venn"),
  split_by = NULL,
  split_by_sep = "_",
  group_by = NULL,
  group_by_sep = "_",
  id_by = NULL,
  label = "count",
  label_fg = "black",
  label_size = NULL,
  label_bg = "white",
  label_bg_r = 0.1,
  fill_mode = "count",
  palreverse = FALSE,
  fill_name = NULL,
  palette = ifelse(fill_mode == "set", "Paired", "Blues"),
  palcolor = NULL,
  alpha = 1,
  theme = "theme_this",
  theme_args = list(),
  title = NULL,
  subtitle = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  aspect.ratio = 1,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  seed = 8525,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{in_form}{A character string specifying the input format.  One of
\code{"auto"} (default; detect automatically via
\code{\link{detect_venn_datatype}()}), \code{"long"}, \code{"wide"},
\code{"list"}, or \code{"venn"}.}

\item{split_by}{The column(s) to split the data by and produce separate
Venn diagrams per subgroup.  Only supported when \code{data} is a data
frame.  Multiple columns are concatenated with \code{split_by_sep}.}

\item{split_by_sep}{A character string to separate concatenated
\code{split_by} columns.  Default \code{"_"}.}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{id_by}{A character string specifying the column name that identifies
individual elements.  Required for long-format data; ignored otherwise.}

\item{label}{A character string or function controlling the text shown in
each intersection region.  One of:
\itemize{
\item \code{"count"} (default) — the raw count of elements in that region.
\item \code{"percent"} — the percentage of the total element count.
\item \code{"both"} — count and percentage on separate lines.
\item \code{"none"} — no region labels are drawn.
\item A \strong{function} — receives a data frame with columns
\code{"id"}, \code{"X"}, \code{"Y"}, \code{"name"}, \code{"item"},
and \code{"count"}, and must return a character vector of labels.
}}

\item{label_fg}{A character string specifying the colour of the label text.}

\item{label_size}{A numeric value specifying the font size of the label
text.  When \code{NULL} (the default), auto-sized at 3.5 for region labels
and 4 for set labels, scaled by \code{base_size / 12}.}

\item{label_bg}{A character string specifying the background colour of the
label text (passed to \code{\link[ggrepel]{geom_text_repel}()} as
\code{bg.color}).  Default \code{"white"}.}

\item{label_bg_r}{A numeric value specifying the corner radius of the label
background rectangle (passed as \code{bg.r}).  Default \code{0.1}.}

\item{fill_mode}{A character string specifying how intersection regions are
coloured.  One of:
\itemize{
\item \code{"count"} — continuous gradient based on element count
(default palette: \code{"Spectral"}).
\item \code{"count_rev"} — continuous gradient with reversed count order.
\item \code{"set"} — discrete blended colours per set combination
(default palette: \code{"Paired"}).  No legend is drawn.
}}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{fill_name}{A character string for the colour bar legend title when
\code{fill_mode} is \code{"count"} or \code{"count_rev"}.  Ignored when
\code{fill_mode = "set"}.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{combine}{Logical; when \code{TRUE} (default), returns a combined
\code{patchwork} object.  When \code{FALSE}, returns a named list of
individual \code{ggplot} objects.}

\item{ncol, nrow}{Integer number of columns / rows for the combined layout
when \code{combine = TRUE}.}

\item{byrow}{Logical; fill the combined layout by row (default \code{TRUE}).}

\item{seed}{A numeric seed for reproducibility.  Default \code{8525}.}

\item{axes, axis_titles}{Character strings specifying how axes and axis
titles are handled across the combined layout.}

\item{guides}{A character string specifying how legends are collected
across panels in the combined layout.}

\item{design}{A custom layout specification for the combined plot.
Passed to \code{\link[patchwork]{wrap_plots}()}.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object (single split), a \code{patchwork} object
(combined sub-plots), or a named list of \code{ggplot} objects (when
\code{combine = FALSE}), each with \code{height} and \code{width}
attributes in inches.
}
\description{
Draws Venn or Euler diagrams that visualise the overlap relationships among
multiple sets.  Supports four input formats: long (one row per element-set
pair), wide (logical/0-1 columns per set), a named list (element vectors
per set), and a pre-computed \code{VennPlotData} object.

Intersection regions can be filled by a continuous colour gradient encoding
the element count (\code{fill_mode = "count"} / \code{"count_rev"}) or by
blended set colours (\code{fill_mode = "set"}).  Region labels can display
counts, percentages, both, or a custom function.  Set labels always show
the set name and its total element count.

Use \code{split_by} to produce separate Venn diagrams for each level of a
grouping variable.  Note that \code{split_by} is only supported when
\code{data} is a data frame (list and \code{VennPlotData} inputs cannot be
split).
}
\section{split_by Workflow}{

When a non-\code{NULL} \code{split_by} is provided and the input is a
data frame:
\enumerate{
\item \strong{Validation} — an error is raised if \code{data} is not a
data frame (list and \code{VennPlotData} input cannot be split).
\item \strong{Column resolution} — \code{split_by} is validated and
optionally concatenated via \code{\link{check_columns}()} with
\code{force_factor = TRUE} and \code{allow_multi = TRUE}.
\item \strong{Data splitting} — the data frame is split by the unique
levels of the \code{split_by} column, preserving factor level
order.  Empty levels are dropped via \code{droplevels()}.
\item \strong{Per-split colour and legend resolution} —
\code{\link{check_palette}()}, \code{\link{check_palcolor}()},
and \code{\link{check_legend}()} resolve per-split palettes,
custom colours, legend positions, and legend directions.
\item \strong{Atomic dispatch} — \code{\link{VennDiagramAtomic}()} is
called for each subset.  When \code{title} is a function, it
receives the split level name for dynamic title generation.
\item \strong{Combination} — results are passed to
\code{\link{combine_plots}()} which returns a combined
\code{patchwork} object (when \code{combine = TRUE}) or a named
list of individual ggplot objects (when \code{combine = FALSE}).
}
}

\examples{
\donttest{
set.seed(8525)
data <- list(
    A = sort(sample(letters, 8)),
    B = sort(sample(letters, 8)),
    C = sort(sample(letters, 8)),
    D = sort(sample(letters, 8))
)

# Basic Venn diagram with count labels
VennDiagram(data)

# Fill by set membership (blended colours)
VennDiagram(data, fill_mode = "set")

# Show both count and percentage
VennDiagram(data, label = "both")

# Custom label function using set names
VennDiagram(data, label = function(df) df$name)

# Custom palette and transparency
VennDiagram(data, palette = "material-indigo", alpha = 0.6)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corplot.R
\name{CorPlot}
\alias{CorPlot}
\title{Correlation scatter plot}
\usage{
CorPlot(
  data,
  x,
  y,
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  split_by = NULL,
  split_by_sep = "_",
  pt_size = 2,
  pt_shape = 16,
  raster = FALSE,
  alpha = 1,
  raster_dpi = c(512, 512),
  highlight = NULL,
  highlight_color = "black",
  highlight_size = 1,
  highlight_alpha = 1,
  highlight_stroke = 0.8,
  anno_items = c("eq", "r2", "p"),
  anno_size = 3,
  anno_fg = "black",
  anno_bg = "white",
  anno_bg_r = 0.1,
  anno_position = c("topleft", "topright", "bottomleft", "bottomright", "tl", "tr", "bl",
    "br"),
  add_smooth = TRUE,
  smooth_color = "red2",
  smooth_width = 1.5,
  smooth_se = FALSE,
  theme = "theme_this",
  theme_args = list(),
  palette = ifelse(is.null(group_by), "Spectral", "Paired"),
  palcolor = NULL,
  palreverse = FALSE,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  legend.position = waiver(),
  legend.direction = "vertical",
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis.}

\item{y}{A character string specifying the column name of the data frame to plot for the y-axis.}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{group_name}{A character string used as the colour legend title.
When \code{NULL}, the \code{group_by} column name is used.}

\item{split_by}{The column(s) to split the data by and produce separate
sub-plots.  Multiple columns are concatenated with \code{split_by_sep}.}

\item{split_by_sep}{A character string to separate concatenated
\code{split_by} columns.  Default \code{"_"}.}

\item{pt_size}{A numeric value specifying the size of the points.
Default: \code{2}.}

\item{pt_shape}{A numeric value specifying the shape of the points
(see \code{\link[ggplot2]{geom_point}}).  Default: \code{16} (filled
circle).}

\item{raster}{A logical value.  When \code{TRUE}, uses
\code{scattermore::geom_scattermore()} for efficient rendering of
large datasets.  Default: \code{FALSE}.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{raster_dpi}{An integer vector of length 1 or 2 specifying the
raster resolution in (width, height) pixels.  When a single value is
provided, it is recycled.  Default: \code{c(512, 512)}.}

\item{highlight}{Specifies which points to emphasise.  Can be:
\itemize{
\item \code{TRUE} — highlight all points.
\item A character expression (e.g. \code{'Species == "setosa"'}) —
evaluated via \code{dplyr::filter}.
\item A character vector — matched against rownames of the data.
\item A numeric vector — treated as row indices.
}
Default: \code{NULL} (no highlighting).}

\item{highlight_color}{A character string specifying the colour of the
highlighted point borders.  Default: \code{"black"}.}

\item{highlight_size}{A numeric value specifying the size of the
highlighted points (the inner fill).  Default: \code{1}.}

\item{highlight_alpha}{A numeric value specifying the alpha transparency
of the highlighted points.  Default: \code{1}.}

\item{highlight_stroke}{A numeric value specifying the stroke width of
the highlighted point borders.  The outer layer size is
\code{highlight_size + highlight_stroke}.  Default: \code{0.8}.}

\item{anno_items}{A character vector specifying which statistics to
display as text annotation.  Available items: \code{"eq"} (regression
equation), \code{"r2"} (R-squared), \code{"p"} (p-value),
\code{"spearman"}, \code{"pearson"}, \code{"kendall"},
\code{"n"} (observation count).  Default: \code{c("eq", "r2", "p")}.}

\item{anno_size}{A numeric value specifying the font size of the
annotation text (scaled by \code{base_size / 12}).  Default: \code{3}.}

\item{anno_fg}{A character string specifying the colour of the
annotation text.  Default: \code{"black"}.}

\item{anno_bg}{A character string specifying the background colour of
the annotation text boxes.  Default: \code{"white"}.}

\item{anno_bg_r}{A numeric value specifying the corner radius of the
annotation text background boxes.  Default: \code{0.1}.}

\item{anno_position}{A character string specifying the corner position
of the annotation text.  One of \code{"topleft"} (alias \code{"tl"}),
\code{"topright"} (\code{"tr"}), \code{"bottomleft"} (\code{"bl"}),
\code{"bottomright"} (\code{"br"}).}

\item{add_smooth}{A logical value.  When \code{TRUE} (default), a linear
regression line (\code{geom_smooth(method = "lm")}) is added.}

\item{smooth_color}{A character string specifying the colour of the
regression line.  Default: \code{"red2"}.}

\item{smooth_width}{A numeric value specifying the linewidth of the
regression line.  Default: \code{1.5}.}

\item{smooth_se}{A logical value.  When \code{TRUE}, a standard error
band is drawn around the regression line.  Default: \code{FALSE}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{seed}{A numeric seed for reproducibility.  Passed to
\code{\link{validate_common_args}()}.}

\item{combine}{Logical; when \code{TRUE} (default), returns a combined
\code{patchwork} object.  When \code{FALSE}, returns a named list of
individual \code{ggplot} objects.}

\item{ncol, nrow}{Integer number of columns / rows for the combined
layout (passed to \code{\link[patchwork]{wrap_plots}}).}

\item{byrow}{Logical; fill the combined layout by row.  Default
\code{TRUE}.}

\item{axes}{A character string specifying how axes should be treated
across the combined layout (passed to
\code{\link[patchwork]{wrap_plots}}).}

\item{axis_titles}{A character string specifying how axis titles should
be treated across the combined layout.  Defaults to \code{axes}.}

\item{guides}{A character string specifying how guides (legends) should
be collected across panels (passed to \code{\link{combine_plots}()}).}

\item{design}{A custom layout design for the combined plot (passed to
\code{\link{combine_plots}()}).}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object (when \code{split_by} is \code{NULL}),
a \code{patchwork} object (when \code{combine = TRUE}), or a named
list of \code{ggplot} objects (when \code{combine = FALSE}), each
with \code{height} and \code{width} attributes in inches.
}
\description{
Draws a scatter plot of two numeric variables with a linear regression
line, optional correlation statistics, and point highlighting.  This is
the public entry point that wraps \code{\link{CorPlotAtomic}} with
\code{split_by} support.

Key features include \strong{group-based colouring} (\code{group_by}),
\strong{point highlighting} by expression, rowname, or index,
\strong{annotation items} (regression equation, R-squared, p-value,
Spearman/Pearson/Kendall rho, N), \strong{raster rendering} for large
datasets, \strong{faceting} (\code{facet_by}), and \strong{splitting}
into separate sub-plots via \code{split_by}.
}
\section{split_by workflow}{

When \code{split_by} is provided:
\enumerate{
\item The \code{split_by} column is validated via
\code{\link{check_columns}()} with \code{force_factor = TRUE}.
Empty levels are dropped (\code{droplevels()}).
\item The data frame is split by \code{split_by} (preserving level
order).  If \code{split_by} is \code{NULL}, the data is wrapped
in a single-element list with name \code{"..."}.
\item Per-split \code{palette}, \code{palcolor},
\code{legend.position}, and \code{legend.direction} are resolved
via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
and \code{\link{check_legend}()}.
\item \code{\link{CorPlotAtomic}()} is called for each split.  When
\code{title} is a function, it receives the split level name and
can generate dynamic titles.
\item Results are combined via \code{\link{combine_plots}()} (when
\code{combine = TRUE}) or returned as a named list.
}
}

\examples{
\donttest{
data(iris)

# Basic scatter with group colours
CorPlot(iris, "Sepal.Length", "Sepal.Width", group_by = "Species")

# Highlight a specific group with custom stroke
CorPlot(iris, "Sepal.Length", "Sepal.Width", group_by = "Species",
    highlight = 'Species == "setosa"', highlight_stroke = 1.5,
    anno_items = c("eq", "pearson"), anno_position = "bottomright")

# Faceted by species
CorPlot(iris, "Sepal.Length", "Sepal.Width", facet_by = "Species",
    facet_scales = "free")

# Per-split palettes
CorPlot(iris, "Sepal.Length", "Sepal.Width", split_by = "Species",
    palette = c(setosa = "Set1", versicolor = "Dark2", virginica = "Paired"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/jitterplot.R
\name{JitterPlot}
\alias{JitterPlot}
\title{Jitter plot}
\usage{
JitterPlot(
  data,
  x,
  x_sep = "_",
  y = NULL,
  in_form = c("long", "wide"),
  split_by = NULL,
  split_by_sep = "_",
  keep_na = FALSE,
  keep_empty = FALSE,
  sort_x = c("none", "mean_asc", "mean_desc", "mean", "median_asc", "median_desc",
    "median"),
  flip = FALSE,
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  x_text_angle = 0,
  order_by = "-({y}^2 + {size_by}^2)",
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  aspect.ratio = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  shape = 21,
  border = "black",
  size_by = 2,
  size_name = NULL,
  size_trans = NULL,
  y_nbreaks = 4,
  jitter_width = 0.5,
  jitter_height = 0,
  y_max = NULL,
  y_min = NULL,
  y_trans = "identity",
  add_bg = FALSE,
  bg_palette = "stripe",
  bg_palcolor = NULL,
  bg_alpha = 0.2,
  add_hline = NULL,
  hline_type = "solid",
  hline_width = 0.5,
  hline_color = "black",
  hline_alpha = 1,
  labels = NULL,
  label_by = NULL,
  nlabel = 5,
  label_size = 3,
  label_fg = "black",
  label_bg = "white",
  label_bg_r = 0.1,
  highlight = NULL,
  highlight_color = "red2",
  highlight_size = 1,
  highlight_alpha = 1,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name for the x-axis.
Must be character or factor.  Multiple columns can be provided; they are
concatenated with \code{x_sep} as the separator.  When \code{in_form} is
\code{"wide"}, the \code{x} columns are used as key columns and pivoted
to long format (they are not concatenated).}

\item{x_sep}{A character string used to join multiple \code{x} columns.
Default \code{"_"}.  Ignored when \code{x} is a single column or when
\code{in_form} is \code{"wide"}.}

\item{y}{A character string specifying the numeric column for the y-axis.
Required when \code{in_form} is \code{"long"} (default).  When
\code{in_form} is \code{"wide"}, \code{y} is not required — the values
under the \code{x} columns are used as y-values.}

\item{in_form}{A character string specifying the input data format.  Either
\code{"long"} (default) or \code{"wide"}.  In \code{"long"} format,
\code{x} and \code{y} are separate columns.  In \code{"wide"} format, the
\code{x} columns contain the y-values and are pivoted to a key-value pair.}

\item{split_by}{The column(s) to split the data by and produce separate
sub-plots.  Multiple columns are concatenated with \code{split_by_sep}.}

\item{split_by_sep}{A character string to separate concatenated
\code{split_by} columns.  Default \code{"_"}.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{sort_x}{A character string controlling x-axis level reordering by
y-value summaries.  One of \code{"none"}, \code{"mean_asc"},
\code{"mean_desc"}, \code{"mean"}, \code{"median_asc"},
\code{"median_desc"}, \code{"median"}.  \code{"none"} leaves the levels
as-is.  \code{"mean_asc"} / \code{"mean"} sorts by ascending mean of y.
\code{"mean_desc"} sorts by descending mean.  \code{"median_asc"} /
\code{"median"} sorts by ascending median.  \code{"median_desc"} sorts by
descending median.  Default: \code{"none"}.}

\item{flip}{A logical value.  When \code{TRUE}, the x and y axes are
swapped via \code{coord_flip} and the x-axis factor levels are reversed.
Dimension calculation accounts for the flip.  Default: \code{FALSE}.}

\item{group_by}{A character vector of column names for dodging the points.
Each unique combination becomes a separate dodge group and the points are
offset horizontally via \code{position_jitterdodge} to reduce overlap.
Multiple columns are concatenated with \code{group_by_sep}.  When
\code{NULL} (default), no dodging is applied — only jitter via
\code{position_jitter}.}

\item{group_by_sep}{A character string used to join multiple
\code{group_by} columns.  Default \code{"_"}.}

\item{group_name}{A character string for the dodge-group legend title.
When \code{NULL} (default), the \code{group_by} column name is used.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{order_by}{A string expression passed to \code{\link[dplyr]{arrange}()}
to determine which points are labelled.  Evaluated within each x-group
(and facet panel when \code{facet_by} is set).  Default:
\code{"-({y}^2 + {size_by}^2)"}, which selects points farthest from the
origin in y-size radial distance, analogous to VolcanoPlot.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value in \code{[0, 1]} controlling point
transparency.  Default: \code{1}.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{shape}{A numeric value specifying the point shape (ggplot2 point
shape codes).  Shapes 21–25 are filled shapes with borders; for these
shapes the border behaviour is controlled by \code{border}.  Default:
\code{21} (filled circle).}

\item{border}{Controls the border of points when the shape has a border
(21–25).  If \code{TRUE}, the border colour follows the point fill colour
(same as the group colour).  If a single colour string (e.g.
\code{"black"}), uses that constant border colour for all points.  If
\code{FALSE}, no border is drawn (\code{NA}).  Default: \code{"black"}.}

\item{size_by}{A numeric column name or a single numeric value controlling
point size.  When a column name is provided, sizes are scaled using
\code{scale_size_area(max_size = 6)} and a size legend is shown.  When a
single numeric value, all points use that constant size.  Default:
\code{2}.}

\item{size_name}{A character string for the size legend title.  When
\code{NULL} (default) and \code{size_by} is a column, the column name is
used.  Ignored when \code{size_by} is a single numeric value.}

\item{size_trans}{A function or a function name (as a string) to transform
the \code{size_by} values for size mapping.  The transformed values
determine the point size on the plot, but the legend labels show the
original (untransformed) values.  When \code{NULL} (default), no
transformation is applied.}

\item{y_nbreaks}{A numeric value hinting at the number of break intervals
for the y-axis.  Passed to \code{scale_y_continuous}.  Default: \code{4}.}

\item{jitter_width}{A numeric value controlling the amount of horizontal
jitter (in x-axis units).  Passed to \code{position_jitter} /
\code{position_jitterdodge}.  Default: \code{0.5}.}

\item{jitter_height}{A numeric value controlling the amount of vertical
jitter (in y-axis units).  Passed to \code{position_jitter} /
\code{position_jitterdodge}.  Default: \code{0}.}

\item{y_max, y_min}{Numeric values or quantile strings (e.g. \code{"q95"},
\code{"q5"}) for y-axis limits used in \code{coord_cartesian} /
\code{coord_flip}.  When \code{NULL} (default), the data range is used.
When a quantile string, the corresponding quantile of the y-values is
computed via \code{\link[stats]{quantile}()}.}

\item{y_trans}{A character string specifying a transformation for the
y-axis (e.g. \code{"log10"}, \code{"sqrt"}).  Passed to
\code{\link[ggplot2]{scale_y_continuous}}.  Default: \code{"identity"}.}

\item{add_bg}{A logical value.  When \code{TRUE}, alternating background
stripes are drawn behind the points via \code{\link{bg_layer}()}, using
the x-axis level order.  Default: \code{FALSE}.}

\item{bg_palette}{A character string specifying the palette for the
background stripe colours.  Passed to \code{\link{bg_layer}()}.
Default: \code{"stripe"}.}

\item{bg_palcolor}{A character vector of colours for the background
stripes.  Passed to \code{\link{bg_layer}()}.  When \code{NULL} (default),
colours are derived from \code{bg_palette}.}

\item{bg_alpha}{A numeric value in \code{[0, 1]} for the transparency of
the background stripes.  Default: \code{0.2}.}

\item{add_hline}{One or more numeric values specifying y-values at which
to draw horizontal reference lines.  When \code{NULL} (default), no
reference lines are drawn.}

\item{hline_type}{A character string specifying the line type for the
horizontal reference line(s).  Default: \code{"solid"}.}

\item{hline_width}{A numeric value specifying the line width for the
horizontal reference line(s).  Default: \code{0.5}.}

\item{hline_color}{A character string specifying the colour for the
horizontal reference line(s).  Default: \code{"black"}.}

\item{hline_alpha}{A numeric value in \code{[0, 1]} specifying the alpha
(transparency) for the horizontal reference line(s).  Default: \code{1}.}

\item{labels}{A vector of row names or row indices specifying which points
to label.  When \code{NULL} (default) and \code{nlabel > 0}, the top
\code{nlabel} points per x-group are selected automatically.}

\item{label_by}{A character string naming a column whose values are used
as label text.  When \code{NULL} (default), row names are used as labels.}

\item{nlabel}{An integer specifying the number of points to label per
x-group when \code{labels} is \code{NULL}.  Points are selected by
descending order of \code{order_by}.  Default: \code{5}.  Set to
\code{0} to suppress automatic labelling.}

\item{label_size, label_fg, label_bg, label_bg_r}{Label aesthetics for
\code{\link[ggrepel]{geom_text_repel}}.  \code{label_size}: text size
(default \code{3}).  \code{label_fg}: text colour (default
\code{"black"}).  \code{label_bg}: background (halo) colour for the label
text (default \code{"white"}).  \code{label_bg_r}: background border
radius (default \code{0.1}).}

\item{highlight}{A specification of which points to highlight.  Can be:
\code{TRUE} (highlight all points), a numeric vector of row indices, a
single character string parsed as an \R expression, or a character vector
of row names.  When a point is highlighted, an overlay \code{geom_point}
is drawn on top with \code{highlight_color}, \code{highlight_size}, and
\code{highlight_alpha}.  Default: \code{NULL} (no highlighting).}

\item{highlight_color}{A character string specifying the colour of
highlighted points.  Default: \code{"red2"}.}

\item{highlight_size}{A numeric value specifying the size of highlighted
points.  Default: \code{1}.}

\item{highlight_alpha}{A numeric value in \code{[0, 1]} specifying the
transparency of highlighted points.  Default: \code{1}.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{seed}{A numeric seed for reproducibility.  Passed to
\code{\link{validate_common_args}()}.  Default: \code{8525}.}

\item{combine}{Logical; when \code{TRUE} (default), returns a combined
\code{patchwork} object.  When \code{FALSE}, returns a named list of
individual \code{ggplot} objects.}

\item{ncol, nrow}{Integer number of columns / rows for the combined layout
(passed to \code{\link[patchwork]{wrap_plots}}).}

\item{byrow}{Logical; fill the combined layout by row.  Default
\code{TRUE} (passed to \code{\link[patchwork]{wrap_plots}}).}

\item{axes}{A character string specifying how axes should be treated
across the combined layout (passed to \code{\link[patchwork]{wrap_plots}}).}

\item{axis_titles}{A character string specifying how axis titles should be
treated across the combined layout.  Defaults to \code{axes}.}

\item{guides}{A character string specifying how guides (legends) should be
collected across panels (passed to \code{\link{combine_plots}()}).}

\item{design}{A custom layout design for the combined plot (passed to
\code{\link{combine_plots}()}).}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object (when \code{split_by} is \code{NULL}), a
\code{patchwork} object (when \code{split_by} is provided and
\code{combine = TRUE}), or a named list of \code{ggplot} objects (when
\code{combine = FALSE}).  All \code{ggplot} objects have \code{height}
and \code{width} attributes in inches.
}
\description{
Draws a jittered point plot showing the distribution of numeric y-values
across a discrete x-axis.  Each data point is rendered with random jitter
along the x-axis (and optionally the y-axis) to reduce overplotting, making
it easy to visualise data density, spread, and outliers within each category.

The function supports \strong{x-axis reordering} by y-value summaries
(mean or median), \strong{group dodging} via \code{group_by} to compare
subgroups side-by-side, \strong{point labelling} with automatic top-n
selection using a configurable distance metric (default: radial distance
\code{y^2 + size^2}), \strong{point highlighting} for emphasis, optional
\strong{horizontal reference lines}, and \strong{wide-format input} via
\code{in_form}.  Colour control, faceting, and splitting into separate
sub-plots via \code{split_by} are supported.
}
\section{split_by workflow}{

When \code{split_by} is provided:
\enumerate{
\item \code{\link{validate_common_args}()} validates the \code{seed}.
\item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
normalise the \code{keep_na} / \code{keep_empty} arguments for all
relevant columns (\code{x}, \code{split_by}, \code{group_by},
\code{facet_by}).
\item The \code{split_by} column is validated via
\code{\link{check_columns}()} with \code{force_factor = TRUE}.
Multiple \code{split_by} columns are concatenated with
\code{split_by_sep}.
\item If \code{split_by} is not \code{NULL}, the data frame is split
(preserving factor level order).  If \code{split_by} is
\code{NULL}, the data is wrapped in a single-element list with
name \code{"..."}.
\item Per-split \code{palette}, \code{palcolor},
\code{legend.position}, and \code{legend.direction} are resolved
via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
and \code{\link{check_legend}()}.
\item \code{\link{JitterPlotAtomic}()} is called for each split.  If
\code{title} is a function, it receives the split level name and
can generate dynamic titles.
\item Results are combined via \code{\link{combine_plots}()} (when
\code{combine = TRUE}) or returned as a named list.
}
}

\examples{
\donttest{
set.seed(8525)
n <- 180
x <- factor(
    sample(c("A", NA, LETTERS[3:5]), n, replace = TRUE),
    levels = c("A", "B", "C", "D", "E")
)
group <- factor(
    sample(c("G1", NA, "G3"), n, replace = TRUE),
    levels = c("G1", "G2", "G3")
)
size <- rexp(n, rate = 1)
id <- paste0("pt", seq_len(n))
y <- rnorm(n, mean = ifelse(is.na(group), 0, ifelse(group == "G1", 0.5, -0.5))) +
     as.numeric(ifelse(is.na(x), 0, x))/10
df <- data.frame(
  x = x,
  y = y,
  group = group,
  size = size,
  id = id
)

# Basic
JitterPlot(df, x = "x", y = "y")

# Keep empty x levels and NA
JitterPlot(df, x = "x", y = "y", keep_na = TRUE, keep_empty = TRUE)

# Map size with transform; legend shows original values
JitterPlot(df, x = "x", y = "y", size_by = "size", size_name = "Abundance",
    size_trans = sqrt, order_by = "-y^2")

# Dodge by group and add a horizontal line
JitterPlot(df, x = "x", y = "y", group_by = "group",
  add_hline = 0, hline_type = "dashed", hline_color = "red2")

# Keep the empty levels only for color coding
# Note the G3 is not blue (which is taken by unused level G2)
JitterPlot(df, x = "x", y = "y", group_by = "group",
    keep_na = TRUE, keep_empty = 'level')

# Label top points by distance (y^2 + size^2)
JitterPlot(df, x = "x", y = "y", size_by = "size", label_by = "id", nlabel = 3)

# Flip axes
JitterPlot(df, x = "x", y = "y", flip = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxviolinplot.R
\name{BoxViolinPlot}
\alias{BoxViolinPlot}
\title{Box / Violin / Bar / Beeswarm plot (internal)}
\usage{
BoxViolinPlot(
  data,
  x,
  x_sep = "_",
  y = NULL,
  base = c("box", "violin", "bar", "none"),
  in_form = c("long", "wide"),
  split_by = NULL,
  split_by_sep = "_",
  symnum_args = NULL,
  sort_x = NULL,
  flip = FALSE,
  keep_empty = FALSE,
  keep_na = FALSE,
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  paired_by = NULL,
  x_text_angle = ifelse(isTRUE(flip), 0, 45),
  step_increase = 0.1,
  fill_mode = ifelse(!is.null(group_by), "dodge", "x"),
  palreverse = FALSE,
  position_dodge_preserve = "total",
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  alpha = 1,
  aspect.ratio = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  add_point = FALSE,
  pt_color = if (isTRUE(add_beeswarm)) NULL else "grey30",
  pt_size = NULL,
  pt_alpha = 1,
  jitter_width = NULL,
  jitter_height = 0,
  stack = FALSE,
  y_max = NULL,
  y_min = NULL,
  y_brackets = NULL,
  add_beeswarm = FALSE,
  beeswarm_method = "swarm",
  beeswarm_cex = 1,
  beeswarm_priority = "ascending",
  beeswarm_dodge = 0.9,
  add_box = FALSE,
  box_color = "black",
  box_width = 0.1,
  box_ptsize = 2.5,
  add_errorbar = "SEM",
  errorbar_color = "grey20",
  errorbar_width = 0.4,
  errorbar_linewidth = 0.6,
  add_trend = FALSE,
  trend_color = NULL,
  trend_linewidth = 1,
  trend_ptsize = 2,
  add_stat = NULL,
  stat_name = NULL,
  stat_color = "black",
  stat_size = 1,
  stat_stroke = 1,
  stat_shape = 25,
  add_bg = FALSE,
  bg_palette = "stripe",
  bg_palcolor = NULL,
  bg_alpha = 0.2,
  add_line = NULL,
  line_color = "red2",
  line_width = 0.6,
  line_type = 2,
  highlight = NULL,
  highlight_color = "red2",
  highlight_size = 1,
  highlight_alpha = 1,
  comparisons = NULL,
  ref_group = NULL,
  pairwise_method = "wilcox.test",
  multiplegroup_comparisons = FALSE,
  multiple_method = "kruskal.test",
  sig_label = "p.format",
  sig_labelsize = 3.5,
  hide_ns = FALSE,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis.}

\item{x_sep}{A character string to join multiple \code{x} columns.
Default \code{"_"}.}

\item{y}{A character string specifying the column name of the data frame to plot for the y-axis.}

\item{base}{A character string specifying the primary plot type:
\code{"box"} (box plot), \code{"violin"} (violin plot), \code{"bar"}
(mean bars with optional error bars), or \code{"none"} (no primary
geometry, used by beeswarm plots).}

\item{in_form}{A character string: \code{"long"} (default) or
\code{"wide"}.  In wide form, \code{x} columns are pivoted to long
format.}

\item{split_by}{The column(s) to split the data by for separate sub-plots.}

\item{split_by_sep}{Separator for concatenated \code{split_by} columns.}

\item{symnum_args}{A list of arguments passed to
\code{\link[stats]{symnum}} for symbolic p-value coding.}

\item{sort_x}{An R expression string (e.g., \code{"mean(y)"}) to order
x-axis categories.  Default \code{NULL} keeps the original order.
When \code{keep_empty_x} is \code{TRUE}, empty levels are placed last.}

\item{flip}{Logical; if \code{TRUE}, swap the x and y axes.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{group_name}{A character string for the dodge legend title.}

\item{paired_by}{A character string naming a column that identifies
paired observations.  Forces \code{add_point = TRUE} and connects
paired observations with lines.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{step_increase}{Fractional step increase for stacking significance
brackets when multiple comparisons exist.}

\item{fill_mode}{A character string controlling fill colour mapping:
\code{"dodge"} (fill by \code{group_by}, discrete),
\code{"x"} (fill by x-axis categories, discrete),
\code{"mean"} or \code{"median"} (fill by pre-computed statistic,
continuous gradient).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{position_dodge_preserve}{Passed to
\code{\link[ggplot2]{position_dodge}()}: \code{"total"} preserves the
overall group width; \code{"single"} preserves individual element width.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{add_point}{Logical; add jittered or beeswarm points to the plot.}

\item{pt_color}{Colour of the points.  When \code{add_beeswarm = TRUE}
and \code{pt_color} is \code{NULL}, points are coloured by the fill
variable.}

\item{pt_size}{Numeric size of the points.  Default computed from
data size: \code{min(3000 / nrow(data), 0.6)}.}

\item{pt_alpha}{Numeric transparency of the points.}

\item{jitter_width}{Numeric width of the jitter.  Defaults to
\code{0.5}, but set to \code{0} when \code{paired_by} is provided.}

\item{jitter_height}{Numeric height of the jitter.  Default \code{0}.}

\item{stack}{Logical; stack facetted panels in a compact layout with
shared strip labels.}

\item{y_max, y_min}{Numeric y-axis limits, or quantile notation strings
(e.g., \code{"q95"} for the 95th percentile, \code{"q5"} for the
5th percentile).}

\item{y_brackets}{Numeric y-axis position for significance brackets
(or p-value labels for multiple comparisons).  If NULL, the brackets are placed above the maximum y-value.}

\item{add_beeswarm}{Logical; use \code{ggbeeswarm::geom_beeswarm()} for
non-overlapping point layout instead of jitter.  Requires the
\code{ggbeeswarm} package.}

\item{beeswarm_method}{Beeswarm layout method: \code{"swarm"},
\code{"compactswarm"}, \code{"hex"}, \code{"square"}, or
\code{"center"}.}

\item{beeswarm_cex}{Numeric scaling for point spacing.  Larger values
spread points more.}

\item{beeswarm_priority}{Point layout priority: \code{"ascending"},
\code{"descending"}, \code{"density"}, or \code{"random"}.}

\item{beeswarm_dodge}{Numeric dodge width for beeswarm points when
\code{group_by} is provided.  Default \code{0.9}.}

\item{add_box}{Logical; overlay a box plot on the primary geometry.
Mutually exclusive with \code{base = "box"} and \code{base = "bar"}.}

\item{box_color}{Colour of the overlaid box plot outline and fill.}

\item{box_width}{Width of the overlaid box plot.}

\item{box_ptsize}{Size of the median point in the overlaid box plot.}

\item{add_errorbar}{Type of error bars for bar plots (\code{base = "bar"}):
\code{"SEM"} (standard error of the mean, default), \code{"SD"}
(standard deviation), \code{"CI"} or \code{"CI95"} (95\\% confidence
interval), or \code{"none"}.  Silently ignored for non-bar bases.}

\item{errorbar_color}{Colour of the error bar lines and caps.}

\item{errorbar_width}{Width of the error bar caps.}

\item{errorbar_linewidth}{Line width of the error bars.}

\item{add_trend}{Logical; add trend lines connecting group medians.}

\item{trend_color}{Colour of the trend line.  When \code{NULL} and
\code{group_by} is present, lines are coloured per group.}

\item{trend_linewidth}{Width of the trend line.}

\item{trend_ptsize}{Size of the trend line points.}

\item{add_stat}{A summary function (e.g., \code{mean}, \code{median}) to
display as a point with a shape legend entry.}

\item{stat_name}{Legend title for the stat summary shape.}

\item{stat_color}{Colour of the stat summary point.}

\item{stat_size}{Size of the stat summary point.}

\item{stat_stroke}{Stroke width of the stat summary point.}

\item{stat_shape}{Shape (an integer) for the stat summary point.  Uses
\code{scale_shape_identity()} so the shape is rendered directly.}

\item{add_bg}{Logical; add alternating background stripes.}

\item{bg_palette}{Palette for the background stripes.}

\item{bg_palcolor}{Custom colours for the background stripes.}

\item{bg_alpha}{Alpha transparency for the background stripes.}

\item{add_line}{A numeric y-intercept for a horizontal reference line.}

\item{line_color}{Colour of the reference line.}

\item{line_width}{Width of the reference line.}

\item{line_type}{Linetype of the reference line.}

\item{highlight}{A specification of points to highlight: \code{TRUE}
(all), a numeric index vector, a logical expression string, or a
character vector of row names.}

\item{highlight_color}{Colour of highlighted points.}

\item{highlight_size}{Size of highlighted points.}

\item{highlight_alpha}{Alpha of highlighted points.}

\item{comparisons}{A logical value (\code{TRUE} for all pairs) or a list
of two-element vectors specifying pairwise comparisons.  Only available
when \code{fill_mode = "dodge"} (i.e., \code{group_by} is present).}

\item{ref_group}{A character string specifying the reference group for
comparisons.}

\item{pairwise_method}{Method for pairwise tests.  Default
\code{"wilcox.test"}.}

\item{multiplegroup_comparisons}{Logical; perform an omnibus test
(e.g., Kruskal-Wallis) across all groups.}

\item{multiple_method}{Method for the omnibus test.  Default
\code{"kruskal.test"}.}

\item{sig_label}{Label format for significance annotations.  For
pairwise comparisons: \code{"p.format"}, \code{"p.signif"}, or a
\pkg{glue} template (e.g., \code{"p = {p}"}).  For multiple-group
tests: \code{"p.format"} or \code{"p.signif"}.}

\item{sig_labelsize}{Size of the significance label text.}

\item{hide_ns}{Logical; hide non-significant comparison labels.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{seed}{A numeric seed for reproducibility.}

\item{combine}{Logical; when \code{TRUE} (default), returns a combined
\code{patchwork} object.  When \code{FALSE}, returns a named list of
\code{ggplot} objects.}

\item{ncol, nrow}{Integer number of columns / rows for the combined layout.}

\item{byrow}{Logical; fill the combined layout by row (default \code{TRUE}).}

\item{axes, axis_titles}{Character strings for axis handling in the
combined layout.}

\item{guides}{Character string for legend collection across panels.}

\item{design}{A custom layout design for the combined plot.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object, a \code{patchwork} object, or a named list
of \code{ggplot} objects (when \code{combine = FALSE}), each with
\code{height} and \code{width} attributes in inches.
}
\description{
Internal wrapper that handles \code{split_by} processing and dispatches
to \code{\link{BoxViolinPlotAtomic}} for each split.  This is the
intermediate layer between the three public APIs
(\code{\link{BoxPlot}}, \code{\link{ViolinPlot}}, and
\code{\link{BeeswarmPlot}}) and the core implementation.
}
\section{split_by workflow}{

When \code{split_by} is provided:
\enumerate{
\item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
normalise the \code{keep_na} / \code{keep_empty} arguments.
\item The \code{split_by} column is validated and its NA / empty levels
are processed.  It is then removed from the per-column lists.
\item The data is split by \code{split_by} (preserving level order).
If \code{split_by} is \code{NULL}, the data is wrapped in a
single-element list with name \code{"..."}.
\item Per-split \code{palette}, \code{palcolor},
\code{legend.position}, and \code{legend.direction} are resolved.
\item \code{\link{BoxViolinPlotAtomic}()} is called for each split.
When \code{title} is a function, it receives the split level name
for dynamic titles.
\item Results are combined via \code{\link{combine_plots}()}.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/upsetplot.R
\name{prepare_upset_data}
\alias{prepare_upset_data}
\title{Prepare data for an UpSet plot}
\usage{
prepare_upset_data(
  data,
  in_form = "auto",
  group_by = NULL,
  group_by_sep = "_",
  id_by = NULL,
  specific = TRUE
)
}
\arguments{
\item{data}{A data frame, a named list of element vectors, or an
\code{UpsetPlotData} object.}

\item{in_form}{A character string specifying the input format.  One of
\code{"auto"} (default; detect from \code{data} structure),
\code{"long"}, \code{"wide"}, \code{"list"}, or \code{"upset"}.}

\item{group_by}{A character vector of column name(s) defining the sets.
In long format, this is the column with set labels.  In wide format,
these are the set-membership columns.  When \code{NULL} in wide format,
all columns except \code{id_by} are used.  Ignored for list and upset
inputs.}

\item{group_by_sep}{A character string to concatenate multiple
\code{group_by} columns when \code{in_form = "long"}.  Default: \code{"_"}.}

\item{id_by}{A character string specifying the column name for instance
identifiers.  Required for long format; optional for wide format (a
synthetic \code{.id} column is created if omitted).}

\item{specific}{A logical value.  When \code{TRUE} (default), only specific
intersections are returned (elements belonging exclusively to the shown
set combination).  When \code{FALSE}, all overlapping items are included.
See \url{https://github.com/gaospecial/ggVennDiagram/issues/64}.}
}
\value{
An \code{UpsetPlotData} object --- a data frame with an
\code{Intersection} list-column of set labels and one row per element.
A \code{"group_order"} attribute preserves the original set order.
}
\description{
Converts raw input (long data frame, wide data frame, named list, or an
existing \code{UpsetPlotData} object) into the internal format expected by
\code{\link{UpsetPlotAtomic}()}.  This is the data-preparation workhorse
that handles all input format parsing.
}
\section{Input formats}{


\strong{Long format} --- one row per (set, element) pair:
\preformatted{
group_by id_by
A        a1
A        a2
B        a1
B        a3
}
Requires both \code{group_by} and \code{id_by}.

\strong{Wide format} --- each row is an element, each set has a logical or
0/1 membership column:
\preformatted{
A      B
TRUE   TRUE
TRUE   FALSE
FALSE  TRUE
}
The set columns are identified by \code{group_by} (or all columns except
\code{id_by} when \code{group_by = NULL}).

\strong{List format} --- a named list of element vectors:
\preformatted{
list(A = c("a1", "a2"), B = c("a1", "a3"))
}

\strong{UpsetPlotData} --- a pre-processed object (returned by this function)
is returned unchanged (with a warning if \code{group_by} is also provided).
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap-utils.R
\name{.check_annotation}
\alias{.check_annotation}
\title{Check the annotation specification
The annotations should not be any of by or split_by, and the columns must exist in the data}
\usage{
.check_annotation(data, annotation, by, split_by, which)
}
\arguments{
\item{data}{The data for Heatmap plot}

\item{annotation}{The annotation mappings}

\item{by}{The column for rows or columns}

\item{split_by}{The column for row or column splitting}

\item{which}{If the annotation is for row or column}
}
\description{
Check the annotation specification
The annotations should not be any of by or split_by, and the columns must exist in the data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/densityplot.R
\name{RidgePlotAtomic}
\alias{RidgePlotAtomic}
\title{Atomic ridge plot}
\usage{
RidgePlotAtomic(
  data,
  x = NULL,
  in_form = c("long", "wide"),
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  add_vline = NULL,
  vline_type = "solid",
  vline_color = TRUE,
  vline_width = 0.5,
  vline_alpha = 1,
  flip = FALSE,
  alpha = 0.8,
  scale = NULL,
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  x_text_angle = 90,
  x_min = NULL,
  x_max = NULL,
  keep_na = FALSE,
  keep_empty = FALSE,
  reverse = FALSE,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  legend.position = "none",
  legend.direction = "vertical",
  ...
)
}
\arguments{
\item{data}{A data frame. Accepted in two forms:
\itemize{
\item \strong{long} (\code{in_form = "long"}): a numeric column (named by \code{x}) and a
factor column (named by \code{group_by}) whose levels become y-axis ridges.
\item \strong{wide} (\code{in_form = "wide"}): multiple numeric columns listed in
\code{group_by} are gathered into \code{.x} / \code{.group} via \code{tidyr::pivot_longer()}.
}}

\item{x}{A character string specifying the column name for the numeric
values plotted on the x-axis. When \code{in_form = "wide"}, \code{x} should be
\code{NULL}; the gathered values are stored in a synthetic \code{.x} column.}

\item{in_form}{A character string specifying whether \code{data} is in
\code{"long"} (default) or \code{"wide"} format.}

\item{group_by}{A character string specifying the column(s) whose levels
define the individual ridges on the y-axis. Multiple columns are
concatenated with \code{group_by_sep}. In wide mode, these are the column
names to gather.}

\item{group_by_sep}{A character string used to join multiple \code{group_by}
column values into a single factor level. In wide form the columns are
not concatenated (each becomes its own ridge). Default: \code{"_"}.}

\item{group_name}{A character string used as the legend title for the
\code{group_by} fill aesthetic. Defaults to the (concatenated) \code{group_by}
column name.}

\item{add_vline}{A specification for vertical reference lines:
\itemize{
\item \code{NULL} or \code{FALSE}: no lines.
\item \code{TRUE}: draw a line at the mean of each group.
\item A numeric vector: draw the same lines for all groups.
\item A named list of numeric vectors: per-group lines, where names should
match \code{group_by} levels.
}}

\item{vline_type}{A character string specifying the line type for the
vertical reference lines. Passed as \code{linetype} to \code{geom_vline()}.
Default: \code{"solid"}.}

\item{vline_color}{The colour of the vertical reference lines:
\itemize{
\item A literal colour value or vector (recycled): applied directly.
\item \code{TRUE} (default): each line is coloured with a darkened blend of
the corresponding ridge fill colour, computed via
\code{blend_colors(mode = "multiply")}.
}}

\item{vline_width}{A numeric value for the thickness of the vertical
reference lines. Passed as \code{linewidth} to \code{geom_vline()}.
Default: \code{0.5}.}

\item{vline_alpha}{A numeric value in \verb{[0, 1]} for the transparency of
the vertical reference lines. Default: \code{1}.}

\item{flip}{A logical value. If \code{TRUE}, the axes are swapped via
\code{coord_flip()}. X-axis text angle and grid-line placement are adjusted
accordingly.}

\item{alpha}{A numeric value in \verb{[0, 1]} for the transparency of the
ridge fill. Default: \code{0.8}.}

\item{scale}{A numeric value controlling the vertical overlap of ridges.
Passed to \code{ggridges::geom_density_ridges(scale = ...)}. Smaller values
increase overlap. When \code{NULL}, ggridges auto-computes the scale.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{x_text_angle}{A numeric value specifying the angle (in degrees) for
x-axis text when \code{flip = TRUE}. Used with \code{calc_just()} to compute
optimal \code{hjust} / \code{vjust}. Default: \code{90}.}

\item{x_min, x_max}{Numeric limits for the x-axis. When \code{NULL} (default),
limits are determined from the data range. Passed to \code{scale_x_continuous(limits = ...)}.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{reverse}{A logical value. If \code{TRUE}, the y-axis group order is
reversed. NA groups are renamed to the literal string \code{"NA"} and
placed at the end.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{...}{Additional arguments passed to \code{ggridges::geom_density_ridges()}
(bandwidth, jittered_points, quantile_lines, etc.).}
}
\description{
Core implementation for ridge (joy) plots. Renders overlapping density curves
for each group on the y-axis using \code{ggridges::geom_density_ridges()}, with
optional vertical reference lines and wide-to-long data conversion.

The function accepts data in two forms:
\itemize{
\item \strong{long form} (default) — a numeric \code{x} column plus a \code{group_by} factor
column whose levels become the y-axis ridges.
\item \strong{wide form} — multiple numeric columns named in \code{group_by} are gathered
via \code{tidyr::pivot_longer()} into \code{.x} / \code{.group} columns, then processed
identically to the long form.
}

Vertical reference lines (\code{add_vline}) can be specified as a numeric vector
(same lines for all groups), a named list (per-group values), or \code{TRUE}
(group means). When \code{vline_color = TRUE}, each line is coloured with a
darkened blend of the corresponding ridge fill.
}
\section{Architecture}{

\strong{RidgePlotAtomic} executes the following steps:
\enumerate{
\item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or \code{ggplot2::ggplot}.
\item \strong{Wide-to-long conversion} — when \code{in_form = "wide"}, calls
\code{tidyr::pivot_longer()} on the \code{group_by} columns, producing \code{.group}
(factor) and \code{.x} (values). \code{x} and \code{group_by} are redirected to these
synthetic columns.
\item \strong{Column resolution} — \code{check_columns()} validates \code{x}, \code{group_by}
(force_factor, allow_multi, concat_multi), and \code{facet_by}.
\item \strong{Default group} — when \code{group_by} is \code{NULL}, a synthetic \code{.group}
factor with a single space character level is created so the fill
pipeline runs uniformly.
\item \strong{Reverse ordering} — if \code{reverse = TRUE}, factor levels of \code{group_by}
are reversed, flipping the y-axis ridge order.
\item \strong{NA / empty handling} — \code{process_keep_na_empty()} filters data. When
\code{reverse = TRUE} and any group value is \code{NA}, the NA level is renamed
to the literal string \code{"NA"} and moved to the end of the factor so
colour mapping and display remain consistent.
\item \strong{Palette resolution} — \code{palette_this()} maps group levels to fill
colours.
\item \strong{Base ggplot} — initialises \code{ggplot(data, aes(x, y, fill))} with
\code{group_by} on the y-axis.
\item \strong{Ridge geometry} — \code{ggridges::geom_density_ridges(alpha, scale)}.
When \code{scale} is \code{NULL}, ggridges auto-computes the overlap factor.
\item \strong{Vertical reference lines} — if \code{add_vline} is not \code{NULL} / \code{FALSE}:
\itemize{
\item \code{add_vline = TRUE} → computes \code{tapply(x, group_by, mean)}.
\item \code{vline_color = TRUE} → resolves per-group line colours by
darkening each fill colour via \code{blend_colors(mode = "multiply")}.
Named list elements are matched to factor levels.
\item Adds \code{geom_vline(xintercept, linetype, linewidth, color, alpha)}.
}
\item \strong{Scales and labels} — \code{scale_y_discrete(drop = !keep_empty_group)},
\code{scale_x_continuous()}, and \code{labs()}.
\item \strong{Fill scale} — \code{scale_fill_manual()}. When \code{keep_empty_group = TRUE},
\code{drop = FALSE}, \code{breaks}, and \code{limits} are set to preserve empty
factor levels.
\item \strong{Flip-aware theme} — when \code{flip = TRUE}:
\itemize{
\item \code{coord_flip()} is applied.
\item x-axis text angle is set from \code{x_text_angle} with computed
\code{hjust} / \code{vjust} via \code{calc_just()}.
\item Major grid lines are drawn on the x-axis.
\item When \code{flip = FALSE}, y-axis text is right-aligned and grid lines
appear on the y-axis.
}
\item \strong{Theme application} — \code{do_call(theme, theme_args)} applies the
resolved theme function, then \code{aspect.ratio} and legend position are
set.
\item \strong{Dimension calculation} — \code{calculate_plot_dimensions(base_height = 1, n_y = nlevels(group_by), y_scale_factor = 1, aspect.ratio, legend, flip)} sets \code{height} / \code{width} attributes. The base height of 1 unit
per ridge keeps individual ridges compact.
\item \strong{Faceting} — \code{facet_plot()} applies \code{facet_grid} / \code{facet_wrap}, with
\code{drop = !keep_empty_facet}.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{process_theme}
\alias{process_theme}
\title{Process theme to allow 'ggplot2::theme_minimal' to work}
\usage{
process_theme(theme)
}
\arguments{
\item{theme}{The theme to process}
}
\value{
The processed theme
}
\description{
Process theme to allow 'ggplot2::theme_minimal' to work
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatialplot.R
\name{SpatImagePlot}
\alias{SpatImagePlot}
\alias{SpatMasksPlot}
\alias{SpatShapesPlot}
\alias{SpatShapesPlot.SpatVector}
\alias{SpatShapesPlot.data.frame}
\alias{SpatPointsPlot}
\title{Spatial visualization functions for terra objects and point data}
\usage{
SpatImagePlot(
  data,
  ext = NULL,
  raster = NULL,
  raster_dpi = NULL,
  flip_y = TRUE,
  palette = "turbo",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  fill_name = NULL,
  return_layer = FALSE,
  theme = "theme_box",
  theme_args = list(),
  legend.position = ifelse(return_layer, "none", "right"),
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525
)

SpatMasksPlot(
  data,
  ext = NULL,
  flip_y = TRUE,
  add_border = TRUE,
  border_color = "black",
  border_size = 0.5,
  border_alpha = 1,
  palette = "turbo",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  fill_name = NULL,
  return_layer = FALSE,
  theme = "theme_box",
  theme_args = list(),
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525
)

SpatShapesPlot(
  data,
  x = NULL,
  y = NULL,
  group = NULL,
  ext = NULL,
  flip_y = TRUE,
  fill_by = NULL,
  border_color = "black",
  border_size = 0.5,
  border_alpha = 1,
  palette = NULL,
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  fill_name = NULL,
  highlight = NULL,
  highlight_alpha = 1,
  highlight_size = 1,
  highlight_color = "black",
  highlight_stroke = 0.8,
  facet_scales = "fixed",
  facet_nrow = NULL,
  facet_ncol = NULL,
  facet_byrow = TRUE,
  return_layer = FALSE,
  theme = "theme_box",
  theme_args = list(),
  legend.position = ifelse(return_layer, "none", "right"),
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525
)

\method{SpatShapesPlot}{SpatVector}(
  data,
  x = NULL,
  y = NULL,
  group = NULL,
  ext = NULL,
  flip_y = TRUE,
  fill_by = NULL,
  border_color = "black",
  border_size = 0.5,
  border_alpha = 1,
  palette = NULL,
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  fill_name = NULL,
  highlight = NULL,
  highlight_alpha = 1,
  highlight_size = 1,
  highlight_color = "black",
  highlight_stroke = 0.8,
  facet_scales = "fixed",
  facet_nrow = NULL,
  facet_ncol = NULL,
  facet_byrow = TRUE,
  return_layer = FALSE,
  theme = "theme_box",
  theme_args = list(),
  legend.position = ifelse(return_layer, "none", "right"),
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525
)

\method{SpatShapesPlot}{data.frame}(
  data,
  x,
  y,
  group,
  ext = NULL,
  flip_y = TRUE,
  fill_by = "grey90",
  border_color = "black",
  border_size = 0.5,
  border_alpha = 1,
  palette = NULL,
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  fill_name = NULL,
  highlight = NULL,
  highlight_alpha = 1,
  highlight_size = 1,
  highlight_color = "black",
  highlight_stroke = 0.8,
  facet_scales = "fixed",
  facet_nrow = NULL,
  facet_ncol = NULL,
  facet_byrow = TRUE,
  return_layer = FALSE,
  theme = "theme_box",
  theme_args = list(),
  legend.position = ifelse(return_layer, "none", "right"),
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525
)

SpatPointsPlot(
  data,
  x = NULL,
  y = NULL,
  ext = NULL,
  flip_y = TRUE,
  color_by = NULL,
  size_by = NULL,
  size = NULL,
  fill_by = NULL,
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  palette = NULL,
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  color_name = NULL,
  size_name = NULL,
  shape = 16,
  border_color = "black",
  border_size = 0.5,
  border_alpha = 1,
  raster = NULL,
  raster_dpi = c(512, 512),
  hex = FALSE,
  hex_linewidth = 0.5,
  hex_count = FALSE,
  hex_bins = 50,
  hex_binwidth = NULL,
  label = FALSE,
  label_size = 4,
  label_fg = "white",
  label_bg = "black",
  label_bg_r = 0.1,
  label_repel = FALSE,
  label_repulsion = 20,
  label_pt_size = 1,
  label_pt_color = "black",
  label_segment_color = "black",
  label_insitu = FALSE,
  label_pos = c("median", "mean", "max", "min", "first", "last", "center", "random"),
  highlight = NULL,
  highlight_alpha = 1,
  highlight_size = 1,
  highlight_color = "black",
  highlight_stroke = 0.8,
  graph = NULL,
  graph_x = NULL,
  graph_y = NULL,
  graph_xend = NULL,
  graph_yend = NULL,
  graph_value = NULL,
  edge_size = c(0.05, 0.5),
  edge_alpha = 0.1,
  edge_color = "grey40",
  facet_scales = "fixed",
  facet_nrow = NULL,
  facet_ncol = NULL,
  facet_byrow = TRUE,
  return_layer = FALSE,
  theme = "theme_box",
  theme_args = list(),
  legend.position = ifelse(return_layer, "none", "right"),
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525
)
}
\arguments{
\item{data}{A \code{SpatRaster}, \code{SpatVector}, or \code{data.frame}
depending on the function. See individual function descriptions.}

\item{ext}{A numeric vector of length 4 \code{c(xmin, xmax, ymin, ymax)}
or a \code{terra::SpatExtent} object. Default is \code{NULL} (use full
extent).}

\item{raster}{Whether to rasterize for efficient rendering of large
datasets. Default is \code{NULL} (auto-detect: rasterize when
\code{ncell > 1e6} or \code{nrow > 1e6}). Uses
\code{scattermore::geom_scattermore()} for points.}

\item{raster_dpi}{A numeric vector of length 1 or 2 specifying the
raster output resolution in pixels. Default is \code{c(512, 512)}.}

\item{flip_y}{Whether to negate the y-coordinates so the axis labels can
be displayed with reversed sign. Default is \code{TRUE}.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{fill_name}{A character string for the fill legend title.}

\item{return_layer}{Whether to return a list of ggplot layers instead
of a complete plot. Default is \code{FALSE}. When \code{TRUE}, the
returned list has a \code{"scales"} attribute for layer conflict
detection, allowing multiple spatial layers in a single custom ggplot.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{seed}{The random seed to use. Default is 8525.}

\item{add_border}{Whether to add polygon borders around mask regions in
\code{SpatMasksPlot}. Default is \code{TRUE}.}

\item{border_color}{A character string for the border color. Default is
\code{"black"}. When \code{TRUE} in \code{SpatShapesPlot} or
\code{SpatPointsPlot}, the border maps to the same variable as fill.}

\item{border_size}{A numeric value for the border line width.
Default is \code{0.5}.}

\item{border_alpha}{A numeric value for the border transparency.
Default is \code{1}.}

\item{x}{A character string specifying the x coordinate column for
\code{SpatPointsPlot} and \code{SpatShapesPlot} (when \code{data} is a
data frame). Auto-detected from common column names when \code{NULL}.}

\item{y}{A character string specifying the y coordinate column for
\code{SpatPointsPlot} and \code{SpatShapesPlot} (when \code{data} is a
data frame). Auto-detected from common column names when \code{NULL}.}

\item{group}{A character string specifying the grouping column for
\code{SpatShapesPlot} when \code{data} is a data.frame. Each unique
value in this column defines a separate polygon.}

\item{fill_by}{A character string or vector specifying the column(s) to
map to fill color in \code{SpatShapesPlot}. When multiple columns are
provided (all must be numeric), the data is reshaped and faceted.
When a single string that does not match a column name, it is treated
as a fixed fill color.}

\item{highlight}{A character vector of row names to highlight, a filter
expression string (e.g., \code{'cat == "A"'}), or \code{TRUE}
(highlight all). Highlighted points are overlaid with larger markers.}

\item{highlight_alpha}{A numeric value for highlight transparency.
Default is \code{1}.}

\item{highlight_size}{A numeric value for the highlight marker size.
Default is \code{1}.}

\item{highlight_color}{A character string for the highlight marker
color. Default is \code{"black"}.}

\item{highlight_stroke}{A numeric value for the highlight stroke width
(added to \code{border_size}). Default is \code{0.8}.}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{color_by}{A character string or vector specifying the column(s) to
map to point color in \code{SpatPointsPlot}. Multiple numeric columns
trigger faceting.}

\item{size_by}{A character string specifying the column to map to point
size in \code{SpatPointsPlot}. Mutually exclusive with \code{size}.}

\item{size}{A numeric value for a fixed point size in
\code{SpatPointsPlot}. Alias for \code{size_by} when given a scalar.}

\item{lower_quantile, upper_quantile}{Lower and upper quantiles for the continuous color/fill scale.
The actual cutoffs are determined by these quantiles when \code{lower_cutoff} and
\code{upper_cutoff} are \code{NULL}. Defaults: \code{lower_quantile = 0}, \code{upper_quantile = 0.99}.}

\item{lower_cutoff, upper_cutoff}{Explicit lower and upper cutoffs for the continuous color/fill scale.
When \code{NULL} (the default), the cutoffs are determined by \code{lower_quantile} and
\code{upper_quantile} via \code{\link[stats]{quantile}}. Values outside the
\code{[lower_cutoff, upper_cutoff]} range are clamped (winsorized) to the nearest cutoff value.}

\item{color_name}{A character string for the color legend title.}

\item{size_name}{A character string for the size legend title.}

\item{shape}{A numeric value (21--25 for border shapes) or character
string specifying the point shape in \code{SpatPointsPlot}.
Default is \code{16} (filled circle).}

\item{hex}{Whether to use hexagonal binning in \code{SpatPointsPlot}.
Requires a numeric \code{color_by}. Default is \code{FALSE}.}

\item{hex_linewidth}{A numeric value for the hex bin border width.
Default is \code{0.5}.}

\item{hex_count}{Whether to show hex bin count as point opacity.
Default is \code{FALSE}.}

\item{hex_bins}{A numeric value for the number of hex bins.
Default is \code{50}.}

\item{hex_binwidth}{A numeric value for the hex bin width.
Default is \code{NULL}.}

\item{label}{Whether to show group labels in \code{SpatPointsPlot}.
Default is \code{FALSE}. Requires a categorical \code{color_by}.}

\item{label_size}{A numeric value for the label text size.
Default is \code{4}.}

\item{label_fg}{A character string for the label text color.
Default is \code{"white"}.}

\item{label_bg}{A character string for the label background color.
Default is \code{"black"}.}

\item{label_bg_r}{A numeric value for the label background corner
radius ratio. Default is \code{0.1}.}

\item{label_repel}{Whether to repel labels from each other and from
data points. Default is \code{FALSE}.}

\item{label_repulsion}{A numeric value for the repulsion force.
Default is \code{20}.}

\item{label_pt_size}{A numeric value for the label anchor point size.
Default is \code{1}.}

\item{label_pt_color}{A character string for the label anchor point
color. Default is \code{"black"}.}

\item{label_segment_color}{A character string for the label connector
line color. Default is \code{"black"}.}

\item{label_insitu}{Whether to place raw group names as labels
instead of numeric indices. Default is \code{FALSE}.}

\item{label_pos}{A character string or function specifying how label
positions are computed per group. Options: \code{"median"} (default),
\code{"mean"}, \code{"center"}, \code{"first"}, \code{"last"},
\code{"random"}, \code{"min"}, \code{"max"}, or a custom function
that takes a numeric vector and returns a single value.}

\item{graph}{Graph/network edge data for \code{SpatPointsPlot}. Can be
a square adjacency matrix with row/column names matching \code{data},
a data.frame with edge coordinates (see \code{graph_x} etc.), a column
name or index, or \code{"@graph"} (extracts from the \code{data}
attribute named \code{"graph"}). Default is \code{NULL}.}

\item{graph_x}{A character string for the x start coordinate column
in the graph data.}

\item{graph_y}{A character string for the y start coordinate column
in the graph data.}

\item{graph_xend}{A character string for the x end coordinate column
in the graph data.}

\item{graph_yend}{A character string for the y end coordinate column
in the graph data.}

\item{graph_value}{A character string for the edge weight column
in the graph data.}

\item{edge_size}{A numeric vector of length 2 specifying the line width
range for graph edges. Default is \code{c(0.05, 0.5)}.}

\item{edge_alpha}{A numeric value for graph edge transparency.
Default is \code{0.1}.}

\item{edge_color}{A character string for the graph edge color.
Default is \code{"grey40"}.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes when \code{return_layer = FALSE} (the default). When
\code{return_layer = TRUE}, a list of \code{ggplot} layers with a
\code{"scales"} attribute is returned. When faceted via multiple
\code{fill_by} or \code{color_by} columns, a faceted \code{ggplot}
is returned.
}
\description{
These functions provide publication-quality spatial visualizations built on
\pkg{ggplot2}, supporting raster images, categorical masks, vector shapes,
and point data from the \pkg{terra} package or standard data frames.
}
\details{
\describe{
\item{\code{SpatImagePlot}}{Render a \code{SpatRaster} as a raster image.
Supports single-layer continuous values (with gradient fill) and 3-channel
RGB data (with automatic color identity scaling).}
\item{\code{SpatMasksPlot}}{Render a \code{SpatRaster} as a categorical
mask overlay. Zero-valued cells are treated as transparent background;
optional polygon borders can be added around mask regions.}
\item{\code{SpatShapesPlot}}{Render spatial shapes (polygons) from a
\code{SpatVector} or a data frame of vertex coordinates. Supports
single and multi-column fill mapping with automatic faceting.}
\item{\code{SpatPointsPlot}}{Render spatial points from a data frame with
support for color/size/fill mapping, hex binning, rasterized rendering,
network/graph overlays, labels, and point highlighting.}
}
}
\section{Rendering Pipeline for SpatImagePlot}{

\enumerate{
\item \strong{Extent cropping} — if \code{ext} is provided, the
\code{SpatRaster} is cropped via \code{terra::crop()}. An error is
raised if no cells remain within the extent.
\item \strong{Auto-rasterization} — if the raster exceeds 1e6 cells (or
\code{raster = TRUE}), the raster is aggregated via
\code{terra::aggregate()} to a target resolution of \code{raster_dpi}.
\item \strong{Y-axis flipping} — when \code{flip_y = TRUE}, the raster
is flipped vertically via \code{\link{.flip_y}()} and its y-extent is
negated so that axis labels can be displayed with reversed sign.
\item \strong{RGB detection} — if the raster has exactly 3 layers, they
are treated as red/green/blue channels: each channel is rescaled to
0--255 and combined into a hex color via \code{rgb()}. A
\code{scale_fill_identity()} is used with no legend guide.
\item \strong{Single-layer rendering} — otherwise, the raster is
converted to an x/y/value data frame and rendered via
\code{geom_raster()} with \code{scale_fill_gradientn()}.
\item \strong{Layer return or full assembly} — if
\code{return_layer = TRUE}, the list of layers (with a \code{"scales"}
attribute set to \code{"fill"}) is returned. Otherwise,
\code{\link{.wrap_spatial_layers}()} assembles a complete ggplot with
\code{coord_sf(expand = 0)}, theme, labels, legend, and dimension
attributes.
}
}

\section{Rendering Pipeline for SpatMasksPlot}{

\enumerate{
\item \strong{Extent cropping} — if \code{ext} is provided, the
\code{SpatRaster} is cropped via \code{terra::crop()}.
\item \strong{Y-axis flipping} — when \code{flip_y = TRUE}, the raster
is flipped vertically and its y-extent is negated.
\item \strong{Background masking} — cells with value 0 are set to
\code{NA} so they render as transparent via
\code{na.value = "transparent"}.
\item \strong{Raster layer} — the mask is converted to an x/y/value data
frame and rendered via \code{geom_raster()} with a gradient fill scale.
\item \strong{Optional borders} — when \code{add_border = TRUE}, the
mask values are polygonized via \code{terra::as.polygons()}, converted
to sf, and overlaid as unfilled \code{geom_sf()} with the specified
\code{border_color}, \code{border_size}, and \code{border_alpha}.
\item \strong{Layer return or full assembly} — if
\code{return_layer = TRUE}, the layers are returned; otherwise,
\code{\link{.wrap_spatial_layers}()} creates the complete ggplot.
}
}

\section{Rendering Pipeline for SpatPointsPlot}{

\enumerate{
\item \strong{Column resolution} — \code{x} and \code{y} coordinates are
resolved from the data, auto-detecting common column names
(\code{"x"}, \code{"X"}, \code{"sdimx"}, etc. for x; \code{"y"},
\code{"Y"}, \code{"sdimy"}, etc. for y) when not explicitly provided.
\item \strong{Extent cropping} — if \code{ext} is provided, rows
outside the extent are filtered.
\item \strong{Y-axis flipping} — when \code{flip_y = TRUE}, the y
coordinate column is negated via \code{\link{.flip_y}()}.
\item \strong{Multi-column faceting} — when \code{color_by} has multiple
columns (all numeric), the data is reshaped to long format with a
\code{.facet_var} column and faceted via \code{facet_plot()}.
\item \strong{Cutoff winsorization} — for numeric \code{color_by},
values outside \code{[lower_cutoff, upper_cutoff]} (derived from
quantiles or explicit values) are clamped to the nearest cutoff.
\item \strong{Graph / network edges} — if \code{graph} is provided,
edges are resolved from an adjacency matrix, data.frame, or data
attribute, and rendered as \code{geom_segment()} segments with line
width proportional to edge weight.
\item \strong{Main point layer} — one of three rendering modes:
\describe{
\item{\emph{Regular}}{(\code{hex = FALSE}, \code{raster = FALSE}) —
\code{geom_point()} with shape, size, color, and fill aesthetic
mappings. Shapes 21--25 support separate fill and border colors.}
\item{\emph{Hex}}{(\code{hex = TRUE}) — \code{geom_hex()} or
\code{stat_summary_hex()} for binned aggregation of numeric
\code{color_by} values.}
\item{\emph{Raster}}{(\code{raster = TRUE}) —
\code{scattermore::geom_scattermore()} for efficient rendering of
large datasets (>1e6 rows).}
}
\item \strong{Highlight} — if \code{highlight} is specified,
highlighted points are overlaid as larger, colored markers using
\code{geom_point()} or \code{scattermore::geom_scattermore()}.
\item \strong{Labels} — if \code{label = TRUE} and \code{color_by} is a
categorical column, group centroid positions are computed via
\code{aggregate()} with the \code{label_pos} function, and labels are
rendered via \code{ggrepel::geom_text_repel()} with optional
background styling and repulsion.
\item \strong{Layer return or full assembly} — if
\code{return_layer = TRUE}, the layers are returned; otherwise,
\code{\link{.wrap_spatial_layers}()} creates the complete ggplot and
\code{\link{facet_plot}()} is applied when multi-column faceting
is active.
}
}

\examples{
\donttest{
set.seed(8525)
# --- SpatImagePlot ---
# Generate a sample SpatRaster
r <- terra::rast(
    nrows = 50, ncols = 40, vals = runif(2000),
    xmin = 0, xmax = 40, ymin = 0, ymax = 50,
    crs = ""
)
SpatImagePlot(r)
SpatImagePlot(r, raster = TRUE, raster_dpi = 20)
SpatImagePlot(r, alpha = 0.5, theme = "theme_blank",
    theme_args = list(add_coord = FALSE), fill_name = "value")
SpatImagePlot(r, ext = c(0, 10, 0, 10), flip_y = FALSE, palette = "viridis")

# --- SpatMasksPlot ---
m <- terra::rast(
   nrows = 50, ncols = 40,
   vals = sample(c(1:5, NA), 2000, replace = TRUE, prob = c(rep(0.04, 5), 0.8)),
   xmin = 0, xmax = 40, ymin = 0, ymax = 50,
   crs = ""
)
SpatMasksPlot(m, border_color = "red")
SpatMasksPlot(m, ext = c(0, 15, 0, 20), add_border = FALSE,
    palreverse = TRUE, fill_name = "value")

# --- SpatShapesPlot ---
polygons <- data.frame(
   id = paste0("poly_", 1:10),
   cat = sample(LETTERS[1:3], 10, replace = TRUE),
   feat1 = rnorm(10),
   feat2 = rnorm(10),
   geometry = c(
       'POLYGON((64.6 75.3,66.0 70.5,66.4 70.2,67.0 69.8,72.8 70.4,64.6 75.3))',
       'POLYGON((56.7 63.0,52.3 65.6,48.0 63.2,51.2 55.7,57.1 59.2,56.7 63.0))',
       'POLYGON((9.9 16.5,9.3 15.9,8.0 13.1,11.5 7.8,17.8 11.3,9.9 16.5))',
       'POLYGON((64.9 37.2,60.3 37.4,57.6 31.7,58.9 29.3,64.0 28.1,64.9 37.2))',
       'POLYGON((30.5 49.1,22.4 46.5,22.4 43.9,30.9 41.9,31.6 42.9,30.5 49.1))',
       'POLYGON((78.3 57.8,70.5 61.6,71.6 52.7,72.2 52.5,77.4 54.5,78.3 57.8))',
       'POLYGON((41.8 23.8,41.3 25.9,41.0 26.4,36.5 28.7,35.8 28.6,41.8 23.8))',
       'POLYGON((15.7 75.9,14.2 74.4,15.7 67.5,23.0 69.8,23.4 71.7,15.7 75.9))',
       'POLYGON((80.7 37.4,75.3 31.3,77.1 28.5,82.5 28.0,83.1 28.5,80.7 37.4))',
       'POLYGON((15.5 37.8,14.4 38.6,7.3 32.6,8.3 30.9,15.1 30.2,15.5 37.8))'
   )
)

polygons <- terra::vect(polygons, crs = "EPSG:4326", geom = "geometry")

SpatShapesPlot(polygons)
SpatShapesPlot(polygons, ext = c(0, 20, 0, 20))
SpatShapesPlot(polygons, highlight = 'cat == "A"', highlight_color = "red2")
SpatShapesPlot(polygons, border_color = "red", border_size = 2)
SpatShapesPlot(polygons, fill_by = "cat", fill_name = "category")
# Let border color be determined by fill
SpatShapesPlot(polygons, fill_by = "cat", alpha = 0.6, border_color = TRUE)
SpatShapesPlot(polygons, fill_by = "feat1")
SpatShapesPlot(polygons, fill_by = c("feat1", "feat2"), palette = "RdYlBu")

# --- SpatPointsPlot ---
# create some random points in the above polygons
points <- data.frame(
  id = paste0("point_", 1:30),
  gene = sample(LETTERS[1:3], 30, replace = TRUE),
  feat1 = runif(30, 0, 100),
  feat2 = runif(30, 0, 100),
  size = runif(30, 1, 5),
  x = c(
    61.6, 14.3, 12.7, 49.6, 74.9, 58.9, 13.9, 24.7, 16.9, 15.6,
    72.4, 60.1, 75.4, 14.9, 80.3, 78.8, 16.7, 27.6, 48.9, 52.5,
    12.9, 11.8, 50.4, 25.6, 10.4, 51.9, 73.4, 26.8, 50.4, 60.0
  ),
  y = c(
    32.1, 12.8, 33.2, 59.9, 57.8, 31.9, 10.1, 46.8, 75.3, 69.0,
    60.0, 29.4, 54.2, 34.2, 35.3, 33.1, 74.7, 48.0, 63.2, 59.2,
    9.2, 15.1, 64.5, 47.1, 11.4, 60.1, 54.1, 44.5, 61.9, 30.3
  )
)

SpatPointsPlot(points)
SpatPointsPlot(points, color_by = "gene", size_by = "size", shape = 22,
  border_size = 1)
SpatPointsPlot(points, raster = TRUE, raster_dpi = 30, color_by = "feat1")
SpatPointsPlot(points, color_by = c("feat1", "feat2"), size_by = "size")
SpatPointsPlot(points, color_by = "feat1", upper_cutoff = 50)
SpatPointsPlot(points, color_by = "feat1", hex = TRUE)
SpatPointsPlot(points, color_by = "gene", label = TRUE)
SpatPointsPlot(points, color_by = "gene", highlight = 1:20,
  highlight_color = "red2", highlight_stroke = 0.8)

# --- Graph/Network functionality ---
# Create a simple adjacency matrix for demonstration
set.seed(8525)
graph_mat <- matrix(0, nrow = 30, ncol = 30)
# Add some random connections with weights
for(i in 1:30) {
  neighbors <- sample(setdiff(1:30, i), size = sample(2:5, 1))
  graph_mat[i, neighbors] <- runif(length(neighbors), 0.1, 1)
}
rownames(graph_mat) <- colnames(graph_mat) <- rownames(points)
attr(points, "graph") <- graph_mat

SpatPointsPlot(points, color_by = "gene", graph = "@graph",
  edge_color = "grey60", edge_alpha = 0.3)
SpatPointsPlot(points, color_by = "feat1", graph = graph_mat,
  edge_size = c(0.1, 1), edge_alpha = 0.5)

# --- Use the `return_layer` argument to get the ggplot layers
ext = c(0, 40, 0, 50)
ggplot2::ggplot() +
  SpatImagePlot(r, return_layer = TRUE, alpha = 0.2, ext = ext) +
  SpatShapesPlot(polygons, return_layer = TRUE, ext = ext, fill_by = "white") +
  SpatPointsPlot(points, return_layer = TRUE, ext = ext, color_by = "feat1") +
  theme_box() +
  ggplot2::coord_sf(expand = 0) +
  ggplot2::scale_y_continuous(labels = function(x) -x)

}
}
\concept{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linkedheatmap.R
\name{LinkedHeatmapAtomic}
\alias{LinkedHeatmapAtomic}
\title{Atomic linked heatmap (internal)}
\usage{
LinkedHeatmapAtomic(
  data,
  left_values_by,
  right_values_by,
  left_rows_by,
  right_rows_by,
  left_columns_by,
  right_columns_by,
  left_columns_split_by = NULL,
  right_columns_split_by = NULL,
  left_pie_group_by = NULL,
  right_pie_group_by = NULL,
  rows_split_by = NULL,
  values_fill = NA,
  palette = "RdBu",
  palcolor = NULL,
  palreverse = FALSE,
  pie_size_name = "size",
  pie_size = NULL,
  pie_values = "length",
  pie_palette = "Spectral",
  pie_palcolor = NULL,
  bars_sample = 1,
  label = identity,
  label_size = 10,
  label_color = "black",
  label_name = "label",
  mark = identity,
  mark_color = "black",
  mark_size = 1,
  mark_name = "mark",
  violin_fill = NULL,
  boxplot_fill = NULL,
  dot_size = 8,
  dot_size_name = "size",
  legend_items = NULL,
  legend_discrete = FALSE,
  legend.position = "right",
  legend.direction = "vertical",
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  add_bg = FALSE,
  bg_alpha = 0.5,
  keep_na = FALSE,
  keep_empty = FALSE,
  add_reticle = FALSE,
  reticle_color = "grey",
  left_cluster_rows = NULL,
  right_cluster_rows = NULL,
  cluster_columns = NULL,
  show_row_names = NULL,
  show_column_names = NULL,
  border = TRUE,
  title = NULL,
  title_params = NULL,
  column_title = NULL,
  row_title = NULL,
  na_col = "grey85",
  column_names_side = "bottom",
  row_annotation = NULL,
  row_annotation_side = NULL,
  row_annotation_palette = NULL,
  row_annotation_palcolor = NULL,
  row_annotation_type = NULL,
  row_annotation_params = NULL,
  row_annotation_agg = NULL,
  column_annotation = NULL,
  column_annotation_side = NULL,
  column_annotation_palette = NULL,
  column_annotation_palcolor = NULL,
  column_annotation_type = NULL,
  column_annotation_params = NULL,
  column_annotation_agg = NULL,
  links_span = 0.5,
  link_width_by = NULL,
  link_width_scale = 5,
  link_color = "grey30",
  link_alpha = 0.8,
  alpha = 1,
  seed = 8525,
  padding = 15,
  base_size = 1,
  aspect.ratio = NULL,
  draw_opts = list(),
  layer_fun_callback = NULL,
  cell_type = c("tile", "bars", "label", "mark", "label+mark", "mark+label", "dot",
    "violin", "boxplot", "pie"),
  cell_agg = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame in long format. Each row represents one
observation; columns specify row/column membership for both left and
right heatmaps as well as the values to encode as color.}

\item{left_values_by, right_values_by}{Column name whose values determine
the fill color of cells in the left / right heatmap.  These are the
primary data columns visualized by the colour scale.}

\item{left_rows_by, right_rows_by}{Column name that defines the rows of
the left / right heatmap.  Each unique value becomes one row.}

\item{left_columns_by, right_columns_by}{Column name that defines the
columns of the left / right heatmap.  Each unique value becomes one
column.}

\item{left_columns_split_by, right_columns_split_by}{Optional column name
to split the columns of the left / right heatmap into groups (passed to
\code{ComplexHeatmap::Heatmap} as \code{column_split}).}

\item{left_pie_group_by, right_pie_group_by}{When \code{cell_type = "pie"},
column(s) to group values by within each pie cell for the left / right
heatmap.  Multiple columns are concatenated.}

\item{rows_split_by}{Optional column name to split the rows of \strong{both}
heatmaps into groups (passed as \code{row_split}).  When provided, row
names in the link table are prefixed with the split level to
disambiguate rows across splits.}

\item{values_fill}{A value used to fill missing cells in the matrix
(passed to \code{HeatmapAtomic}).  Default is \code{NA} (cells with no
data are left empty).}

\item{palette}{A character string naming a palette (see
\code{\link{show_palettes}}) or a character vector of colours for the
main heatmap colour scale.  Default \code{"RdBu"}.  Applied to both
heatmaps unless overridden per-side via \code{...}.}

\item{palcolor}{A custom colour vector that overrides \code{palette} for
the main heatmap colour scale.  Applied to both heatmaps unless
overridden per-side.}

\item{palreverse}{Logical; if \code{TRUE}, reverse the palette direction.}

\item{pie_size_name}{Legend title for the pie size when
\code{cell_type = "pie"}.}

\item{pie_size}{A numeric value or function returning the pie radius.
When a function, it receives the count of groups in the pie and should
return a radius.}

\item{pie_values}{A function or string (convertible via
\code{\link[base]{match.arg}}) to compute the value represented by each
pie slice.  Default \code{"length"} counts observations per group.}

\item{pie_palette, pie_palcolor}{Palette and custom colours for pie slice
fill colours.}

\item{bars_sample}{Fraction of each cell's observations (0 < x <= 1;
\code{1} uses all data) or a whole count > 1 sampled per cell when
\code{cell_type = "bars"}.  Column widths are proportional to the
number of bars drawn per column.  Default 1 (all data).}

\item{label}{A function to compute text labels when
\code{cell_type = "label"} (or \code{"label+mark"}).  Receives the
aggregated value for a cell and optionally row/column indices and names.
See \code{\link{HeatmapAtomic}} for the full dispatch contract.}

\item{label_size}{Default point size for label text (used as fallback
when the \code{label} function does not return a \code{size} field).}

\item{label_color}{Default colour for label text (used as fallback when
the \code{label} function does not return a \code{color} field).}

\item{label_name}{Legend title for the label colour scale.}

\item{mark}{A function to compute mark symbols when
\code{cell_type = "mark"} (or \code{"label+mark"}).  Same dispatch
contract as \code{label}.  See \code{\link{HeatmapAtomic}} for supported
mark types.}

\item{mark_color}{Default mark colour (fallback).}

\item{mark_size}{Default mark stroke width in pt (fallback).}

\item{mark_name}{Legend title for the mark colour scale.}

\item{violin_fill}{A character vector of colours to use as fill for
violin plots when \code{cell_type = "violin"}.  If \code{NULL}, the
annotation colour is used.}

\item{boxplot_fill}{A character vector of colours to use as fill for
boxplots when \code{cell_type = "boxplot"}.  If \code{NULL}, the
annotation colour is used.}

\item{dot_size}{Dot size when \code{cell_type = "dot"}.  Can be a
numeric value or a function.}

\item{dot_size_name}{Legend title for the dot size.}

\item{legend_items}{A named numeric vector specifying custom legend
entries for the main colour scale.  Names become the displayed labels.}

\item{legend_discrete}{Logical; if \code{TRUE}, treat the main colour
scale as discrete.}

\item{legend.position}{A character string specifying where to place the
combined legend: \code{"right"} (default), \code{"left"}, \code{"top"},
\code{"bottom"}, or \code{"none"}.}

\item{legend.direction}{Legend stacking direction:
\code{"vertical"} (default) or \code{"horizontal"}.}

\item{lower_quantile, upper_quantile}{Quantiles used for clipping the
colour scale when \code{lower_cutoff} / \code{upper_cutoff} are
\code{NULL}.  Defaults are 0 and 0.99 respectively.}

\item{lower_cutoff, upper_cutoff}{Explicit cutoffs for the colour scale.
Values outside the range are clamped (winsorized).  Override
\code{lower_quantile} / \code{upper_quantile} when set.}

\item{add_bg}{Logical; if \code{TRUE}, add a background fill behind
non-tile cell types.  Not used for \code{cell_type = "tile"} or
\code{"bars"}.}

\item{bg_alpha}{Numeric in \eqn{[0, 1]} for background transparency.}

\item{keep_na, keep_empty}{Passed through to \code{HeatmapAtomic}.
See \code{\link{common_args}} for details.}

\item{add_reticle}{Logical; if \code{TRUE}, draw a reticle (crosshair
pattern) over the heatmap.}

\item{reticle_color}{Colour for the reticle lines.}

\item{left_cluster_rows, right_cluster_rows}{Logical controlling whether
rows are clustered in the left / right heatmap.  \code{NULL} (default)
lets \code{HeatmapAtomic} decide based on
\code{rows_orderby}/\code{cluster_rows}.}

\item{cluster_columns}{Logical; cluster columns in both heatmaps.
\code{NULL} lets \code{HeatmapAtomic} decide.}

\item{show_row_names, show_column_names}{Logical; show row/column names.
May also be a character vector of display modes; see
\code{\link{Heatmap}}.}

\item{border}{Logical; draw a border around each heatmap.  Default
\code{TRUE}.}

\item{title}{A character string for the overall plot title.  A function
can be used to generate a dynamic title from the default.
Note that, \code{left_title} and \code{right_title} are used to set the title for each heatmap,
and \code{title} is used to set the overall title for the combined plot.}

\item{title_params}{A list of parameters passed to \code{grid::grid.text()} to control the title appearance.
Default is \code{list(gp = gpar(fontsize = 14, fontface = "bold"))}.}

\item{column_title, row_title}{Character title displayed above the columns
/ beside the rows of each heatmap.  May also be a character vector of
display modes; see \code{\link{Heatmap}}.}

\item{na_col}{Colour used for \code{NA} cells.  Default \code{"grey85"}.}

\item{column_names_side}{Side for column names.  Default
\code{"bottom"}.}

\item{row_annotation}{A structured list specifying row annotations.
See \code{\link{HeatmapAtomic}} for the full specification.}

\item{row_annotation_side}{\strong{Deprecated}: use
\code{row_annotation} with the \code{side} sub-key instead.}

\item{row_annotation_palette}{\strong{Deprecated}: use
\code{row_annotation} with the \code{palette} sub-key instead.}

\item{row_annotation_palcolor}{\strong{Deprecated}: use
\code{row_annotation} with the \code{palcolor} sub-key instead.}

\item{row_annotation_type}{\strong{Deprecated}: use
\code{row_annotation} with the \code{type} sub-key instead.}

\item{row_annotation_params}{\strong{Deprecated}: use
\code{row_annotation} with the \code{params} sub-key instead.}

\item{row_annotation_agg}{\strong{Deprecated}: use
\code{row_annotation} with the \code{agg} sub-key instead.}

\item{column_annotation}{A structured list specifying column annotations.
See \code{\link{HeatmapAtomic}} for the full specification.}

\item{column_annotation_side}{\strong{Deprecated}: use
\code{column_annotation} with the \code{side} sub-key instead.}

\item{column_annotation_palette}{\strong{Deprecated}: use
\code{column_annotation} with the \code{palette} sub-key instead.}

\item{column_annotation_palcolor}{\strong{Deprecated}: use
\code{column_annotation} with the \code{palcolor} sub-key instead.}

\item{column_annotation_type}{\strong{Deprecated}: use
\code{column_annotation} with the \code{type} sub-key instead.}

\item{column_annotation_params}{\strong{Deprecated}: use
\code{column_annotation} with the \code{params} sub-key instead.}

\item{column_annotation_agg}{\strong{Deprecated}: use
\code{column_annotation} with the \code{agg} sub-key instead.}

\item{links_span}{Width (in inches) of the gap column between the two
heatmaps where link curves are drawn.  Default 0.5.}

\item{link_width_by}{Optional column name in \code{data} whose values
determine the stroke width of each link line (e.g. interaction
strength).  Values are min-max scaled to \eqn{[0, 1]} and multiplied by
\code{link_width_scale}.
You can also pass a numeric value to use a constant width for all links.}

\item{link_width_scale}{Numeric scaling factor applied to the normalised
link intensity values to produce final line widths (\code{lwd}).
Default 5.}

\item{link_color}{Colour of the link spline curves.  Default
\code{"grey30"}.}

\item{link_alpha}{Alpha transparency of link curves in \eqn{[0, 1]}.
Default 0.8.}

\item{alpha}{Alpha transparency for heatmap cells in \eqn{[0, 1]}.}

\item{seed}{Random seed for reproducibility.  Default 8525.}

\item{padding}{Padding around the heatmap in CSS order (top, right,
bottom, left).  Supports 1–4 values.  Default 15 (mm).}

\item{base_size}{A positive numeric scalar used as a scaling factor for
the overall heatmap size.  Default 1 (no scaling).  Values > 1 enlarge
all cell dimensions proportionally.}

\item{aspect.ratio}{Height-to-width ratio of a single heatmap cell.
When \code{NULL} (default), sensible defaults are chosen per
\code{cell_type} (e.g. 1 for tiles, 0.5 for bars, 2 for violins).}

\item{draw_opts}{A named list of additional arguments passed to
\code{\link[ComplexHeatmap]{draw,HeatmapList-method}}.  Internally
managed arguments (\code{padding}, \code{show_heatmap_legend}, etc.)
take precedence.}

\item{layer_fun_callback}{A function to add custom graphical layers on
top of each heatmap cell.  Receives \code{j}, \code{i}, \code{x},
\code{y}, \code{w}, \code{h}, \code{fill}, \code{sr}, \code{sc}.
See \code{\link[ComplexHeatmap]{Heatmap}} for details.}

\item{cell_type}{The type of cell to render.  One of \code{"tile"}
(default), \code{"bars"}, \code{"label"}, \code{"mark"},
\code{"label+mark"} (or \code{"mark+label"}), \code{"dot"},
\code{"violin"}, \code{"boxplot"}, \code{"pie"}.  Different cell types
use different \code{cell_fun} / \code{layer_fun} implementations.}

\item{cell_agg}{A function to aggregate values within each cell when
\code{cell_type = "tile"} or \code{"label"}.  Default is
\code{\link[base]{mean}}.}

\item{...}{Additional arguments passed to
\code{\link[ComplexHeatmap]{Heatmap}}.  Arguments prefixed with
\code{left_} or \code{right_} are routed to the corresponding heatmap
(the prefix is stripped before passing).  Unprefixed arguments are
passed to both heatmaps but can be overridden by a prefixed version.
Notable examples:
\itemize{
\item \code{left_name}, \code{right_name} — legend titles for the
left / right colour scales.
\item \code{left_row_gap}, \code{right_row_gap} — per-side row gaps.
\item \code{left_column_gap}, \code{right_column_gap} — per-side
column gaps.
\item \code{row_names_max_width}, \code{column_names_max_height} —
passed through to \code{ComplexHeatmap::Heatmap}.
}}
}
\value{
A \code{patchwork}-wrapped grob (class \code{wrap_plots}) with
\code{height} and \code{width} attributes (in inches).  The original
\code{data} is stored in \code{p$data}.
}
\description{
Draws two heatmaps side-by-side with spline link lines connecting matching
rows across the left and right heatmaps.  This is the core implementation
layer — it takes a \strong{single} data frame and a full set of left/right
parameters, delegates to \code{\link{HeatmapAtomic}} (with
\code{return_ht = TRUE}) to obtain prepared \code{ComplexHeatmap::Heatmap}
objects, extracts exact dimension information, and then assembles
everything into a composite \code{grid} layout.
}
\section{Architecture}{

\enumerate{
\item \strong{Parameter resolution} — extra arguments passed via
\code{...} are split into left-specific and right-specific groups
by prefix (\code{left_} vs \code{right_}).  Each group is merged
with the shared defaults using \code{\%||\%} fallback chains,
giving the caller full per-side override capability.
\item \strong{Cell dimension pre-computation} — the internal helper
\code{.get_dim_pre()} calculates exact cell width and height
(in \code{"inches"} units) from \code{cell_type},
\code{aspect.ratio}, \code{base_size}, and the unique row/column
counts (including split groups).  These are passed as explicit
\code{width} / \code{height} arguments to
\code{ComplexHeatmap::Heatmap()} so that cells have a guaranteed
physical size rather than using \code{null} units.
\item \strong{Delegation to \code{HeatmapAtomic}} — both the left and
right heatmaps are created by calling
\code{do_call(HeatmapAtomic, ...)} with \code{return_ht = TRUE}.
This returns the prepared \code{Heatmap} object (with
\code{cell_w}/\code{cell_h} attributes) without drawing it.
\item \strong{Dimension extraction} — exact component heights and widths
are read from the prepared \code{Heatmap} objects via
\code{ComplexHeatmap:::component_height()} and
\code{ComplexHeatmap:::component_width()}.  These 9-element vectors
break down every part of the heatmap (title, dendrogram, names,
annotations, body) so the composite layout can be sized precisely.
\item \strong{Link table construction} — rows in the left and right
heatmaps are matched by grouping the input data on the row
identifier columns (plus optional \code{rows_split_by}).  Each
link's position is mapped to the ordered row index in its
respective heatmap (respecting clustering order).
\item \strong{Legend collection} — legends from both heatmaps are
collected, packed via \code{ComplexHeatmap::packLegend()}, and
placed according to \code{legend.position} (left, right, top, or
bottom).
\item \strong{Composite drawing} — a \code{grid.layout} is built
dynamically based on the legend position.  Each heatmap is drawn
inside a centered, exact-size viewport so
\code{ComplexHeatmap::draw()} sizes the body to the pre-computed
row/column counts.  Link lines are drawn as \code{grid.xspline()}
curves in a dedicated gap column between the two heatmaps.
\item \strong{Dimension attributes} — the final \code{patchwork}-wrapped
grob carries \code{height} and \code{width} attributes (in inches)
for consistent rendering in downstream layouts.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\name{Network}
\alias{Network}
\title{Network}
\usage{
Network(
  links,
  nodes = NULL,
  split_by = NULL,
  split_by_sep = "_",
  split_nodes = FALSE,
  from = NULL,
  from_sep = "_",
  to = NULL,
  to_sep = "_",
  node_by = NULL,
  node_by_sep = "_",
  link_weight_by = 2,
  link_weight_name = NULL,
  link_type_by = "solid",
  link_type_name = NULL,
  node_size_by = 15,
  node_size_name = NULL,
  node_color_by = "black",
  node_color_name = NULL,
  node_shape_by = 21,
  node_shape_name = NULL,
  node_fill_by = "grey20",
  node_fill_name = NULL,
  link_alpha = 1,
  node_alpha = 0.95,
  node_stroke = 1.5,
  cluster_scale = c("fill", "color", "shape"),
  node_size_range = c(5, 20),
  link_weight_range = c(0.5, 5),
  link_arrow_offset = 20,
  link_curvature = 0,
  link_color_by = "from",
  link_color_name = NULL,
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  link_palette = ifelse(link_color_by \%in\% c("from", "to"), palette, "Set1"),
  link_palcolor = if (link_color_by \%in\% c("from", "to")) palcolor else NULL,
  directed = TRUE,
  layout = "circle",
  cluster = "none",
  add_mark = FALSE,
  mark_expand = ggplot2::unit(10, "mm"),
  mark_type = c("hull", "ellipse", "rect", "circle"),
  mark_alpha = 0.1,
  mark_linetype = 1,
  add_label = TRUE,
  label_size = 3,
  label_fg = "white",
  label_bg = "black",
  label_bg_r = 0.1,
  arrow = ggplot2::arrow(type = "closed", length = ggplot2::unit(0.1, "inches")),
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  aspect.ratio = 1,
  theme = "theme_this",
  theme_args = list(),
  legend.position = "right",
  legend.direction = "vertical",
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{links}{A data frame containing the edge list. Must contain the
\code{from} and \code{to} columns specifying source and target node
identifiers. Additional columns can be referenced by other parameters
(e.g., \code{link_weight_by}, \code{link_type_by},
\code{link_color_by}).}

\item{nodes}{An optional data frame of node metadata. When provided,
columns such as \code{node_size_by}, \code{node_color_by},
\code{node_shape_by}, and \code{node_fill_by} can reference its
columns. When \code{NULL}, the node set is inferred from the unique
values in the \code{from} and \code{to} columns. If a single character
string starting with \code{"@"}, the nodes data frame is extracted
from the corresponding attribute of \code{links} (e.g.
\code{"@nodes"} extracts \code{attr(links, "nodes")}).}

\item{split_by}{The column(s) to split data by and plot separately.
When \code{combine = TRUE}, the combined plot's data (\code{p$data})
contains the rows of every split, tagged with the \code{split_by}
column. For ggraph-based plots (e.g. \code{\link{Network}},
\code{\link{ClustreePlot}}), the node layout of every split is
exposed with the split column, and the original link rows (also
tagged with the \code{split_by} column) are available as the
\code{"edges"} attribute of the data.}

\item{split_by_sep}{The separator for multiple split_by columns. See \code{split_by}}

\item{split_nodes}{A logical value. When \code{TRUE} and
\code{split_by} is provided, the \code{nodes} data frame is split by
the same \code{split_by} column in addition to the links. Both data
frames must have a column with the same name as \code{split_by}.
Default \code{FALSE}.}

\item{from}{A character string specifying the column name in
\code{links} for the source node identifiers. Defaults to
\code{"from"}, or the first column of \code{links} if that column
name does not exist. Multiple columns can be provided; they are
concatenated with \code{from_sep}.}

\item{from_sep}{A character string to join multiple \code{from}
columns. Default \code{"_"}. Ignored when \code{from} is a single
column.}

\item{to}{A character string specifying the column name in
\code{links} for the target node identifiers. Defaults to
\code{"to"}, or the second column of \code{links} if that column
name does not exist. Multiple columns can be provided; they are
concatenated with \code{to_sep}.}

\item{to_sep}{A character string to join multiple \code{to} columns.
Default \code{"_"}. Ignored when \code{to} is a single column.}

\item{node_by}{A character string specifying the column name in
\code{nodes} for the node identifiers. These must match the values
in the \code{from} / \code{to} columns of \code{links}. Defaults to
\code{"name"}, or the first column of \code{nodes} if that column
name does not exist. Multiple columns can be provided; they are
concatenated with \code{node_by_sep}.}

\item{node_by_sep}{A character string to join multiple
\code{node_by} columns. Default \code{"_"}. Ignored when
\code{node_by} is a single column.}

\item{link_weight_by}{A numeric value or a character string. If
numeric, all edges receive that constant line width. If a column
name, the edge line width is mapped to that column. Default
\code{2}.}

\item{link_weight_name}{A character string for the link weight legend
title. When \code{NULL} (default), the column name from
\code{link_weight_by} is used. Only relevant when
\code{link_weight_by} is a column name.}

\item{link_type_by}{A character string or a column name specifying
the edge linetype. Can be \code{"solid"}, \code{"dashed"},
\code{"dotted"}, etc. If a column name from \code{links} is
supplied, the linetype is mapped to that column (with a version
check for ggplot2 4.0.0, where mapping is unsupported and a
warning is issued). Default \code{"solid"}.}

\item{link_type_name}{A character string for the link linetype legend
title. When \code{NULL} (default), the column name from
\code{link_type_by} is used. Only relevant when
\code{link_type_by} is a column name.}

\item{node_size_by}{A numeric value or a character string. If
numeric, all nodes receive that constant point size. If a column
name, the size is mapped to that column. Default \code{15}.}

\item{node_size_name}{A character string for the node size legend
title. When \code{NULL} (default), the column name from
\code{node_size_by} is used. Only relevant when
\code{node_size_by} is a column name.}

\item{node_color_by}{A character string specifying the node colour.
If a colour name or hex code (e.g. \code{"black"}), all nodes
receive that constant colour. If a column name from \code{nodes} is
supplied, the colour is mapped to that column. Default
\code{"black"}.}

\item{node_color_name}{A character string for the node colour legend
title. When \code{NULL} (default), the column name from
\code{node_color_by} is used. Only relevant when
\code{node_color_by} is a column name.}

\item{node_shape_by}{A numeric value or a character string. If
numeric, all nodes receive that constant shape (see
\code{\link[ggplot2]{shape}}). If a column name, the shape is
mapped to that column (cast to factor). Default \code{21} (filled
circle with border).}

\item{node_shape_name}{A character string for the node shape legend
title. When \code{NULL} (default), the column name from
\code{node_shape_by} is used. Only relevant when
\code{node_shape_by} is a column name.}

\item{node_fill_by}{A character string specifying the node fill
colour. If a colour name or hex code (e.g. \code{"grey20"}), all
nodes receive that constant fill. If a column name from
\code{nodes} is supplied, the fill is mapped to that column.
Default \code{"grey20"}.}

\item{node_fill_name}{A character string for the node fill legend
title. When \code{NULL} (default), the column name from
\code{node_fill_by} is used. Only relevant when
\code{node_fill_by} is a column name.}

\item{link_alpha}{A numeric value specifying the transparency
(alpha) of the edge lines. Between \code{0} (invisible) and
\code{1} (opaque). Default \code{1}.}

\item{node_alpha}{A numeric value specifying the fill transparency
of the nodes. Only applies when \code{node_shape_by} is one of the
filled shapes (21--25). Default \code{0.95}.}

\item{node_stroke}{A numeric value specifying the border stroke
width of the node points. Default \code{1.5}.}

\item{cluster_scale}{A character string specifying which node
aesthetic is overridden by cluster membership. One of
\code{"fill"}, \code{"color"}, or \code{"shape"}. The value is
matched via \code{\link{match.arg}}; default is \code{"fill"}.}

\item{node_size_range}{A numeric vector of length 2 giving the
minimum and maximum node size (in ggplot2 point units) when
\code{node_size_by} is a column name. Default \code{c(5, 20)}.}

\item{link_weight_range}{A numeric vector of length 2 giving the
minimum and maximum edge line width (in mm) when
\code{link_weight_by} is a column name. Default
\code{c(0.5, 5)}.}

\item{link_arrow_offset}{A numeric value (in points) specifying the
offset distance for the arrow end cap from the target node.
Prevents arrow heads from overlapping the node points. Only
relevant when \code{directed = TRUE}. Default \code{20}.}

\item{link_curvature}{A numeric value controlling the curvature of
the edges. \code{0} (default) produces straight edges; positive
values curve them away from the direct path.}

\item{link_color_by}{A character string controlling how edge colour
is determined. Options:
\itemize{
\item \code{"from"} (default) -- colour follows the source node's
fill or colour aesthetic.
\item \code{"to"} -- colour follows the target node's fill or
colour.
\item A column name from \code{links} -- colour is mapped
directly to that column.
}}

\item{link_color_name}{A character string for the edge colour legend
title. Only used when \code{link_color_by} is a column name (not
\code{"from"} or \code{"to"}). When \code{NULL} (default), the
column name is used.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{link_palette}{A character string specifying the palette for
edge colours when they are mapped. When \code{link_color_by} is
\code{"from"} or \code{"to"}, defaults to the node
\code{palette}. Otherwise defaults to \code{"Set1"}.}

\item{link_palcolor}{A character vector specifying custom colours
for the edge palette. When \code{link_color_by} is \code{"from"}
or \code{"to"}, defaults to the node \code{palcolor}. Otherwise
defaults to \code{NULL}.}

\item{directed}{A logical value. When \code{TRUE}, edges are drawn
with arrow heads and an end-cap offset. Default \code{TRUE}.}

\item{layout}{A character string or an \code{igraph_layout_spec}
object specifying the node placement algorithm. Built-in shortcuts:
\code{"circle"} (circular layout), \code{"tree"} (hierarchical
tree), \code{"grid"} (grid layout). Any other string is prefixed
with \code{"layout_with_"} and called as an igraph function (e.g.
\code{"fr"} for Fruchterman--Reingold, \code{"kk"} for
Kamada--Kawai). Default \code{"circle"}.}

\item{cluster}{A character string specifying the community detection
algorithm. One of \code{"none"}, \code{"fast_greedy"},
\code{"walktrap"}, \code{"edge_betweenness"}, \code{"infomap"}, or
a custom clustering function from igraph. When not \code{"none"},
cluster membership overrides the aesthetic selected by
\code{cluster_scale}. Default \code{"none"}.}

\item{add_mark}{A logical value. When \code{TRUE} (and
\code{cluster != "none"}), an enclosure mark is drawn around each
cluster's nodes. Default \code{FALSE}.}

\item{mark_expand}{A \code{\link[grid]{unit}} object specifying the
extra space around points within a cluster mark. Default
\code{unit(10, "mm")}.}

\item{mark_type}{A character string specifying the mark geometry.
One of \code{"hull"}, \code{"ellipse"}, \code{"rect"}, or
\code{"circle"}, corresponding to ggforce's \code{geom_mark_hull},
\code{geom_mark_ellipse}, \code{geom_mark_rect}, and
\code{geom_mark_circle}. The value is matched via
\code{\link{match.arg}}; default is \code{"hull"}.}

\item{mark_alpha}{A numeric value for the fill transparency of
cluster marks. Default \code{0.1}.}

\item{mark_linetype}{A numeric or character value specifying the
border line type of the cluster marks. Default \code{1} (solid).}

\item{add_label}{A logical value. When \code{TRUE} (default), node
identifiers are drawn as repulsive text labels via
\code{\link[ggrepel]{geom_text_repel}}.}

\item{label_size}{A numeric value for the font size of node labels.
Scaled by the theme base size. Default \code{3}.}

\item{label_fg}{A character string specifying the text colour of
node labels. Default \code{"white"}.}

\item{label_bg}{A character string specifying the background colour
of node labels. Default \code{"black"}.}

\item{label_bg_r}{A numeric value specifying the background box
radius (as a fraction of label height). Passed to
\code{\link[ggrepel]{geom_text_repel}}'s \code{bg.r} argument.
Default \code{0.1}.}

\item{arrow}{A \code{\link[ggplot2]{arrow}} object for the link
arrow heads. Only used when \code{directed = TRUE}. Default is
\code{arrow(type = "closed", length = unit(0.1, "inches"))}.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{seed}{The random seed to use. Default is 8525.}

\item{combine}{Whether to combine the plots into one when facet is FALSE. Default is TRUE.}

\item{nrow}{A numeric value specifying the number of rows in the facet.}

\item{ncol}{A numeric value specifying the number of columns in the facet.}

\item{byrow}{A logical value indicating whether to fill the plots by row.}

\item{axes}{A string specifying how axes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axes in individual plots.
\item 'collect' will remove duplicated axes when placed in the same run of rows or columns of the layout.
\item 'collect_x' and 'collect_y' will remove duplicated x-axes in the columns or duplicated y-axes in the rows respectively.
}}

\item{axis_titles}{A string specifying how axis titltes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axis titles in individual plots.
\item 'collect' will remove duplicated titles in one direction and merge titles in the opposite direction.
\item 'collect_x' and 'collect_y' control this for x-axis titles and y-axis titles respectively.
}}

\item{guides}{A string specifying how guides should be treated in the layout. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'collect' will collect guides below to the given nesting level, removing duplicates.
\item 'keep' will stop collection at this level and let guides be placed alongside their plot.
\item 'auto' will allow guides to be collected if a upper level tries, but place them alongside the plot if not.
}}

\item{design}{Specification of the location of areas in the layout, passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE. When specified, \code{nrow}, \code{ncol}, and \code{byrow} are ignored.
See \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}} for more details.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object (no \code{split_by}), a
\code{patchwork} object (\code{combine = TRUE}), or a named list of
\code{ggplot} objects (\code{combine = FALSE}), each with
\code{height} and \code{width} attributes in inches.
}
\description{
Draws a network graph from a links (edge list) data frame and an optional
nodes (vertex metadata) data frame. The graph is constructed via
\code{\link[igraph]{igraph}}, laid out with igraph layout algorithms, and
rendered with \code{\link[ggraph]{ggraph}}. Supports directed or
undirected edges, variable link widths/linetypes/colours, node
sizes/shapes/colours/fills, community detection with enclosure marks,
automatic node labels, and a wide range of layout options.

When \code{links} (and optionally \code{nodes}) contain a
\code{split_by} column, separate sub-plots are generated for each split
level and combined via \code{\link[patchwork]{patchwork}}. Unlike most
other plot types, \code{Network} operates on two data frames; splitting
may affect both.
}
\section{split_by workflow}{

When \code{split_by} is provided:
\enumerate{
\item \strong{Column validation} -- The \code{split_by} column is
validated in \code{links} via \code{\link{check_columns}},
force-converted to a factor, and empty levels are dropped.
\item \strong{Node split} -- If \code{split_nodes = TRUE} and
\code{nodes} is provided, the same \code{split_by} column is
validated in \code{nodes}. It must be identical in name to the
links \code{split_by} or an error is raised. Empty levels are
also dropped.
\item \strong{Data splitting} -- The \code{links} data frame is
split by the \code{split_by} levels into a named list,
preserving factor level order.
\item \strong{Attach node splits} -- If \code{split_nodes = TRUE},
the \code{nodes} data frame is split identically. Each split's
node data is attached as the \code{"nodes"} attribute on the
corresponding links split.
\item \strong{Dispatch to atomic} -- \code{\link{NetworkAtomic}} is
called for each split. The \code{nodes} argument is passed as
\code{"@nodes"} when \code{split_nodes = TRUE} so that it is
extracted from the attribute. If \code{title} is a function, it
receives the split level name for dynamic title generation.
\item \strong{Combination} -- Results are combined via
\code{\link{combine_plots}()} (when \code{combine = TRUE}) or
returned as a named list of \code{ggplot} objects.
\item \strong{Combined data} -- When \code{combine = TRUE}, the
combined plot's \code{data} (\code{p$data}) exposes the node
layout of every split, each row tagged with the
\code{split_by} level. The \code{"graph"} attribute carries a
combined graph of all splits' nodes and links, and the
original link rows (tagged with the \code{split_by} column)
are available in the \code{"edges"} attribute.
}
}

\examples{
\donttest{
# Create example data
actors <- data.frame(
  name = c("Alice", "Bob", "Cecil", "David", "Esmeralda"),
  age = c(48, 33, 45, 34, 21),
  shape = c(21, 22, 21, 22, 23),
  gender = c("F", "M", "F", "M", "F")
)
relations <- data.frame(
  from = c("Bob", "Cecil", "Cecil", "David", "David", "Esmeralda", "Bob", "Alice",
     "Cecil", "David"),
  to = c("Alice", "Bob", "Alice", "Alice", "Bob", "Alice", "Bob", "Alice", "Cecil",
     "David"),
  friendship = c(4, 5, 5, 2, 1, 1, 2, 1, 3, 4),
  type = c(1, 1, 1, 1, 1, 2, 2, 2, 2, 2)
)

# Basic network
Network(relations, actors)

# Blank theme with no coordinate axes
Network(relations, actors, theme = "theme_blank",
        theme_args = list(add_coord = FALSE))

# Mapped aesthetics with custom layout
Network(relations, actors,
        link_weight_by = "friendship",
        node_size_by = "age",
        link_weight_name = "FRIENDSHIP",
        node_fill_by = "gender",
        link_color_by = "to",
        link_type_by = "type",
        node_color_by = "black",
        layout = "circle",
        link_curvature = 0.2)

# Tree layout with clustering and marks
Network(relations, actors, layout = "tree",
        directed = FALSE, cluster = "fast_greedy",
        add_mark = TRUE)

# Split by a column
Network(relations, actors, split_by = "type")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clustreeplot.R
\name{ClustreePlotAtomic}
\alias{ClustreePlotAtomic}
\title{Atomic clustree plot}
\usage{
ClustreePlotAtomic(
  data,
  prefix,
  flip = FALSE,
  alpha = 0.85,
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  edge_palette = "Spectral",
  edge_palcolor = NULL,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  expand = c(0.1, 0.1),
  theme = "theme_this",
  theme_args = list(),
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{prefix}{A character string specifying the common prefix of the
resolution columns in \code{data}. All columns whose names start with
this prefix are selected as resolution columns. The suffix after the
prefix is parsed as a numeric resolution value. Supports \code{"_"}
and \code{"."} as separators between the prefix and the resolution
value (e.g. \code{"res_0.5"} or \code{"p.0.5"}).}

\item{flip}{A logical value. If \code{TRUE}, the tree is flipped so that
resolutions are displayed on the x-axis (left to right) and cluster
assignments are shown as row labels on the y-axis.
Default: \code{FALSE}.}

\item{alpha}{A numeric value in \code{[0, 1]} specifying the transparency
of the nodes in the clustree plot. Only used when \code{node_alpha} is
not explicitly provided via \code{...}. Default: \code{0.85}.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{edge_palette}{A character string specifying the palette name for
the edge colour gradient. Edges are coloured by the number of
transitioning cells between clusters at adjacent resolutions, using
\code{ggraph::scale_edge_color_gradientn()}. Default: \code{"Spectral"}.}

\item{edge_palcolor}{A character vector of custom colours for the edge
colour gradient. When \code{NULL} (the default), colours are derived
from \code{edge_palette}.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{expand}{The values to expand the x and y axes. It is like CSS padding.
When a single value is provided, it is used for both axes on both sides.
When two values are provided, the first value is used for the top/bottom side and the second value is used for the left/right side.
When three values are provided, the first value is used for the top side, the second value is used for the left/right side, and the third value is used for the bottom side.
When four values are provided, the values are used for the top, right, bottom, and left sides, respectively.
You can also use a named vector to specify the values for each side.
When the axis is discrete, the values will be applied as 'add' to the 'expansion' function.
When the axis is continuous, the values will be applied as 'mult' to the 'expansion' function.
See also \url{https://ggplot2.tidyverse.org/reference/expansion.html}}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{...}{Additional arguments passed to \code{clustree::clustree()}.
Commonly used overrides include \code{node_size_range},
\code{node_text_size}, \code{layout} (default: \code{"sugiyama"}),
\code{show_axis}, and \code{node_text_colour}. Note that \code{x}
(the data) and \code{prefix} are set internally and cannot be
overridden here.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes.
}
\description{
Core implementation for clustree (clustering tree) plots. This internal
function renders a tree-based visualisation showing how cluster assignments
change across increasing clustering resolutions. It is called by the
exported \code{ClustreePlot()} function.
}
\details{
The function expects a data frame containing columns named with a common
\code{prefix} followed by numeric resolution values (e.g. \code{"res_0.1"},
\code{"res_0.3"}, \code{"res_0.5"}). Each column represents cluster
assignments at a given resolution, and the tree visualises how cells (rows)
transition between clusters as resolution increases.

The core layout is produced via \code{clustree::clustree()}, which computes
a \code{ggraph} layout (default: \code{"sugiyama"}) linking clusters across
adjacent resolutions. Nodes represent clusters and edges represent cell
transitions; edge width and colour encode the number of transitioning cells.
Cluster labels are overlaid on each node via \code{ggplot2::geom_text()}.

Key features:
\itemize{
\item \strong{Resolution-level node colouring} — each resolution level
receives a distinct colour from \code{palette} via
\code{ggplot2::scale_color_manual()}.
\item \strong{Edge gradient} — edges are coloured by transition count
using \code{ggraph::scale_edge_color_gradientn()} with the
\code{edge_palette} gradient.
\item \strong{Flip support} — \code{flip = TRUE} places resolutions on
the x-axis with cluster IDs as y-axis row labels for left-to-right
reading.
\item \strong{Automatic dimension calculation} — plot height and width
are sized based on the number of resolutions, number of clusters, and
legend configuration, with different formulae for flipped vs. non-flipped
layouts.
}
}
\section{Architecture}{


\strong{ClustreePlotAtomic} executes the following steps:
\enumerate{
\item \strong{Column selection} — selects all columns whose names start
with \code{prefix} via \code{dplyr::starts_with()}.
\item \strong{NA filtering} — removes rows with any missing values via
\code{stats::complete.cases()}.
\item \strong{Expansion normalisation} — \code{\link{norm_expansion}()}
parses the \code{expand} argument for continuous x and y axes.
\item \strong{Empty data guard} — stops with an informative error if no
columns match the prefix or no complete rows remain.
\item \strong{Resolution parsing} — extracts the suffix after
\code{prefix} from each column name and strips leading separators
(\code{"_"} or \code{"."}) if present. Values are converted to numeric
and sorted. Column names are rewritten to use the original prefix for
consistent \code{clustree} processing.
\item \strong{Cluster counting} — computes the number of unique clusters
at the highest resolution (\code{max_clusters}), used for dimension
calculation in the non-flipped layout.
\item \strong{Argument merging} — builds a call to
\code{clustree::clustree()} by merging user-supplied \code{...} with
sensible defaults for \code{node_alpha}, \code{edge_width},
\code{show_axis}, \code{layout} (\code{"sugiyama"}),
\code{node_size_range} (\code{c(6, 12)}), \code{node_text_size}
(\code{3}), and \code{node_text_colour} (\code{"black"}).
\item \strong{clustree execution} — calls
\code{gglogger::register(clustree::clustree)} with the merged arguments
to produce the base \code{ggplot} object.
\item \strong{Node labels} — overlays \code{ggplot2::geom_text()} with
the cluster label from the \code{clustree} output, using the configured
text size and colour.
\item \strong{Resolution colour scale} — applies
\code{ggplot2::scale_color_manual()} with one colour per resolution
level using \code{palette_this()}. The colour legend is suppressed (the
edge gradient legend serves as the primary guide).
\item \strong{Edge colour scale} — applies
\code{ggraph::scale_edge_color_gradientn()} with the
\code{edge_palette} palette, encoding transition counts via a
colour-bar guide with black frame and ticks.
\item \strong{Theme and labels} — applies the selected theme and sets
\code{title}, \code{subtitle}, x-axis label, and y-axis label (default
y-label is the \code{prefix} with trailing separator stripped).
\item \strong{Flip branch} (\code{flip = TRUE}):
\itemize{
\item Resolutions are placed on the x-axis (horizontal); clusters
become y-axis rows.
\item Width scales with \code{max(3, 0.5 + nres * 1.0)}; height
scales with \code{max(3, 0.5 + max_clusters * 0.5)} (or via
aspect.ratio).
\item \code{scale_y_reverse()} + \code{coord_flip()} reorients the
layout so resolutions read left-to-right.
\item Horizontal grid lines are dashed grey80; y-axis text and
ticks are suppressed.
}
\item \strong{Non-flip branch} (\code{flip = FALSE}, default):
\itemize{
\item Resolutions are on the y-axis (rows); clusters spread across
the x-axis (columns).
\item Width scales with \code{max(3, 0.5 + max_clusters * 0.5)};
height scales with \code{max(3, 0.5 + nres * 1.0)}
(or via aspect.ratio).
\item \code{coord_cartesian(clip = "off")} and
\code{scale_y_continuous()} show resolution labels on the y-axis.
\item Vertical grid lines are dashed grey80; x-axis text and ticks
are suppressed.
}
\item \strong{Dimension storage} — \code{\link{calculate_plot_dimensions}()}
computes \code{height} and \code{width} attributes, stored on the
\code{ggplot} object.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatialplot.R
\name{.prepare_extent}
\alias{.prepare_extent}
\title{Prepare the extent for spatial plots}
\usage{
.prepare_extent(ext)
}
\arguments{
\item{ext}{A numeric vector of length 4 specifying the extent as \code{c(xmin, xmax, ymin, ymax)},
or a \code{SpatExtent} object from the \code{terra} package.}
}
\value{
A \code{SpatExtent} object if \code{ext} is a numeric vector, or the original \code{SpatExtent} if it is already one.
NULL is returned if \code{ext} is NULL.
}
\description{
Prepare the extent for spatial plots
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dotplot.R
\name{DotPlot}
\alias{DotPlot}
\alias{LollipopPlot}
\title{Dot Plot, Scatter Plot, and Lollipop Plot}
\usage{
DotPlot(
  data,
  x,
  y,
  x_sep = "_",
  y_sep = "_",
  flip = FALSE,
  split_by = NULL,
  split_by_sep = "_",
  size_name = NULL,
  fill_name = NULL,
  fill_cutoff_name = NULL,
  add_bg = FALSE,
  bg_palette = "stripe",
  bg_palcolor = NULL,
  bg_alpha = 0.2,
  bg_direction = c("vertical", "horizontal", "v", "h"),
  size_by = NULL,
  fill_by = NULL,
  fill_cutoff = NULL,
  palreverse = FALSE,
  size_min = 1,
  size_max = 10,
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  alpha = 1,
  border_color = "black",
  border_size = 0.5,
  border_alpha = 1,
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  x_text_angle = 0,
  seed = 8525,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  keep_na = FALSE,
  keep_empty = FALSE,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)

LollipopPlot(
  data,
  x,
  y,
  y_sep = NULL,
  flip = FALSE,
  split_by = NULL,
  split_by_sep = "_",
  size_name = NULL,
  fill_name = NULL,
  fill_cutoff_name = NULL,
  size_by = NULL,
  fill_by = NULL,
  fill_cutoff = NULL,
  palreverse = FALSE,
  size_min = 1,
  size_max = 10,
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  alpha = 1,
  border_color = "black",
  border_size = 0.5,
  border_alpha = 1,
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  x_text_angle = 0,
  seed = 8525,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  keep_na = FALSE,
  keep_empty = FALSE,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string naming the column for the x-axis. Must be a
numeric column (bars extend from 0 to the data value).}

\item{y}{A character string naming the column for the y-axis. Must be a
factor or character column (each level gets a lollipop bar).}

\item{x_sep}{A character string used to join multiple \code{x} column values
into a single factor level. Only used when x is non-numeric and multiple
columns are provided. Default: \code{"_"}.}

\item{y_sep}{A character string used to join multiple \code{y} column values
into a single factor level. Only used when y is non-numeric and multiple
columns are provided. Default: \code{"_"}.}

\item{flip}{A logical value. If \code{TRUE}, the x and y axes are swapped via
\code{coord_flip()}. Dimension calculation accounts for the flip.
Default: \code{FALSE}.}

\item{split_by}{The column(s) to split data by and generate separate plots
for each level. The split column is processed for \code{keep_na} /
\code{keep_empty} before splitting.}

\item{split_by_sep}{A character string used to concatenate multiple
\code{split_by} column values. Default: \code{"_"}.}

\item{size_name}{A character string for the size legend title. When
\code{NULL} (the default), the \code{size_by} column name is used.}

\item{fill_name}{A character string for the fill colour-bar legend title.
When \code{NULL} (the default), the \code{fill_by} column name is used.}

\item{fill_cutoff_name}{A character string for the fill cutoff legend title
(shown when \code{fill_cutoff} is active). Defaults to
\code{"<fill_by> <fill_cutoff>"}, e.g. \code{"mpg < 18"}.}

\item{add_bg}{A logical value. If \code{TRUE}, alternating background
stripes are drawn behind the points via \code{\link{bg_layer}()}. The
striped axis is determined by \code{bg_direction}. Requires the striped
axis to be non-numeric. Default: \code{FALSE}.}

\item{bg_palette}{A character string specifying the palette for the
background stripe colours. Passed to \code{\link{bg_layer}()}.
Default: \code{"stripe"}.}

\item{bg_palcolor}{A character vector of colours for the background stripes.
Passed to \code{\link{bg_layer}()}. When \code{NULL} (default), colours
are derived from \code{bg_palette}.}

\item{bg_alpha}{A numeric value in \code{[0, 1]} for the transparency of
the background stripes. Default: \code{0.2}.}

\item{bg_direction}{A character string specifying which axis receives the
alternating background stripes. \code{"vertical"} (default) stripes by x
levels; \code{"horizontal"} stripes by y levels. Abbreviations \code{"v"}
and \code{"h"} are also accepted.}

\item{size_by}{A character string naming a numeric column whose values
control dot size. When \code{NULL} (the default), the per-combination
observation count is computed automatically (via \code{dplyr::summarise(n =
  n())}) and used as the size variable. If \code{fill_by} is also present,
the first value of \code{fill_by} per combination is retained with a
warning. A single numeric value is also accepted and sets a constant dot
size (used by \code{ScatterPlot}).}

\item{fill_by}{A character string naming a numeric column whose values
control the fill colour of the dots (and lollipop inner bars). A
continuous gradient from \code{palette} is applied via
\code{scale_fill_gradientn()}. When \code{NULL} (the default), all dots
are filled with a single constant colour from the middle of the palette.}

\item{fill_cutoff}{A string expression specifying which values of
\code{fill_by} to grey out. Format: an operator followed by a number,
e.g. \code{"< 18"}, \code{"<= 18"}, \code{"> 18"}, or \code{">= 18"}.
Values matching the condition are set to \code{NA} and rendered in grey
(\code{"grey80"}), while the rest are coloured by the fill gradient. The
operator determines which side of the threshold is greyed out,
independent of \code{palreverse}. A numeric value is also accepted as
shorthand for \code{"<"} (e.g. \code{18} is equivalent to
\code{"< 18"}). Requires \code{fill_by} to be set.}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{size_min}{A numeric value for the smallest dot size in the
\code{scale_size(range = c(size_min, size_max))} range.
Default: \code{1}.}

\item{size_max}{A numeric value for the largest dot size in the
\code{scale_size(range = c(size_min, size_max))} range.
Default: \code{10}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{border_color}{Controls the dot border colour and lollipop outer-shadow
appearance:
\itemize{
\item \code{TRUE} — dot borders and lollipop inner bars follow the
\code{fill_by} gradient via \code{scale_color_gradientn()}; lollipop
outer shadow is black.
\item \code{"black"} (default) — constant black borders on dots and
black outer shadow on lollipop bars.
\item A colour string (e.g. \code{"red"}, \code{"#FF0000"}) — constant
colour for both dot borders and lollipop outer shadows.
\item \code{FALSE} — no dot borders and no lollipop outer shadow (the
inner coloured bars remain visible in lollipop mode).
}}

\item{border_size}{A numeric value for the stroke width of dot borders and
the base linewidth of lollipop bars. In lollipop mode, the outer shadow
uses \code{border_size * 4} and the inner bar uses \code{border_size * 2}.
Default: \code{0.5}.}

\item{border_alpha}{A numeric value in \code{[0, 1]} controlling the
transparency of dot borders and lollipop bar segments.
Default: \code{1}.}

\item{lower_quantile, upper_quantile}{Lower and upper quantiles for the continuous color/fill scale.
The actual cutoffs are determined by these quantiles when \code{lower_cutoff} and
\code{upper_cutoff} are \code{NULL}. Defaults: \code{lower_quantile = 0}, \code{upper_quantile = 0.99}.}

\item{lower_cutoff, upper_cutoff}{Explicit lower and upper cutoffs for the continuous color/fill scale.
When \code{NULL} (the default), the cutoffs are determined by \code{lower_quantile} and
\code{upper_quantile} via \code{\link[stats]{quantile}}. Values outside the
\code{[lower_cutoff, upper_cutoff]} range are clamped (winsorized) to the nearest cutoff value.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{seed}{The random seed for reproducibility. Passed to
\code{validate_common_args()}. Default: \code{8525}.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{combine}{A logical value. If \code{TRUE} (the default), the list of
per-split plots is combined into a single \code{patchwork} object. If
\code{FALSE}, returns the raw list.}

\item{nrow, ncol, byrow}{Integers controlling the layout of combined plots via
\code{patchwork::wrap_plots()}. \code{byrow = TRUE} (default) fills the
layout row-wise.}

\item{axes, axis_titles}{Strings controlling how axes and axis titles are
handled across combined plots. Passed to \code{\link{combine_plots}()}.
See \code{?patchwork::wrap_plots} for options (\code{"keep"},
\code{"collect"}, \code{"collect_x"}, \code{"collect_y"}).}

\item{guides}{A string controlling guide collection across combined plots.
Passed to \code{\link{combine_plots}()}.}

\item{design}{A custom layout specification for combined plots. Passed to
\code{\link{combine_plots}()}. When specified, \code{nrow}, \code{ncol},
and \code{byrow} are ignored.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object (single plot), a \code{patchwork} object
(when \code{combine = TRUE} with \code{split_by}), or a list of
\code{ggplot} objects (when \code{combine = FALSE}).

A \code{ggplot} object (single plot), a \code{patchwork} object
(when \code{combine = TRUE} with \code{split_by}), or a list of
\code{ggplot} objects (when \code{combine = FALSE}).
}
\description{
\code{DotPlot()} renders a matrix of filled circles (dot plot) where dot
size encodes one numeric variable and fill colour encodes another. Either
axis can be numeric or factor, enabling four layout combinations:

\itemize{
\item \strong{Both axes factor} — a classic dot matrix (e.g. genes × cell
types), where each cell is a dot whose size reflects expression magnitude
and whose colour reflects a summary statistic.
\item \strong{Both axes numeric} — a scatter plot, with dots positioned by
x/y coordinates, sized by a third variable, and coloured by a fourth.
\item \strong{One numeric, one factor} — a strip plot or (with
\code{lollipop = TRUE}) a lollipop chart.
}

\code{LollipopPlot()} is a convenience wrapper that sets
\code{lollipop = TRUE}, producing horizontal bars from the y-axis to each
data point, capped by filled dots. It expects a numeric \code{x} and a
factor/character \code{y}.

Key features:
\itemize{
\item \strong{Auto-count:} when \code{size_by = NULL}, the per-combination
observation count is computed automatically.
\item \strong{fill_cutoff:} values in \code{fill_by} matching a threshold
expression (e.g. \code{"< 18"}) are greyed out with a dedicated legend
entry.
\item \strong{Background stripes:} \code{add_bg = TRUE} draws alternating
background bands along the discrete axis for visual grouping.
\item \strong{Border modes:} \code{border_color} can track the fill
gradient (\code{TRUE}), use a constant colour (\code{"black"}), or be
suppressed (\code{FALSE}).
\item \strong{Colour scale trimming:} \code{lower_quantile} /
\code{upper_quantile} (or explicit \code{lower_cutoff} /
\code{upper_cutoff}) trim the continuous fill scale extremes.
}

\code{LollipopPlot()} is a convenience wrapper around \code{DotPlot()}
that sets \code{lollipop = TRUE}. It renders a horizontal bar extending
from the y-axis (\code{x = 0}) to each data point, capped by a filled dot.
The bar has a two-layer construction: an outer shadow (black or custom
colour) and an inner coloured segment that follows the \code{fill_by}
gradient. Dot size scales by \code{size_by} (or the per-combination
observation count when \code{size_by = NULL}).

Expects \code{x} to be a numeric column and \code{y} to be a factor or
character column.
}
\section{split_by Workflow (DotPlot)}{


When \code{split_by} is provided, the following pipeline executes:
\enumerate{
\item \strong{Column validation} — \code{check_columns()} resolves
\code{split_by} (force_factor, allow_multi, concat_multi).
\item \strong{NA / empty pre-processing} — \code{process_keep_na_empty()}
handles \code{keep_na} / \code{keep_empty} for the split column before
splitting, then removes the split column from the per-split
\code{keep_na}/\code{keep_empty} lists.
\item \strong{Data splitting} — splits \code{data} by \code{split_by}
levels (preserving factor level order).
\item \strong{Per-split palette / colour} — \code{check_palette()} and
\code{check_palcolor()} resolve per-split palette and colour overrides.
\item \strong{Per-split legend} — \code{check_legend()} resolves
\code{legend.position} and \code{legend.direction} per split.
\item \strong{Per-split title} — when \code{title} is a function, it
receives the default title (the split level name) and can return a
custom string; otherwise \code{title \%||\% split_level} is used.
\item \strong{Dispatch} — each split subset is passed to
\code{\link{DotPlotAtomic}} (with \code{lollipop = FALSE}).
\item \strong{Combination} — \code{\link{combine_plots}()} assembles the
list of plots via \code{patchwork::wrap_plots}, honouring
\code{nrow}/\code{ncol}/\code{byrow}/\code{design}.
}
}

\section{split_by Workflow (LollipopPlot)}{


Same pipeline as \code{DotPlot} above, but dispatches to
\code{\link{DotPlotAtomic}} with \code{lollipop = TRUE}.
}

\examples{
\donttest{
mtcars <- datasets::mtcars
mtcars$carb <- factor(mtcars$carb)
mtcars$gear <- factor(mtcars$gear)

# --- Basic dot plot (factor × factor, size + fill) ---
DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
        fill_by = "mpg", fill_cutoff = "< 18")
DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
        fill_by = "mpg", fill_cutoff = "> 18")

# --- Background stripes ---
DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
        fill_by = "mpg", fill_cutoff = "< 18", add_bg = TRUE)
DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
        fill_by = "mpg", fill_cutoff = "< 18", add_bg = TRUE,
        bg_direction = "h")

# --- Faceting ---
DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
        fill_by = "mpg", fill_cutoff = "< 18", facet_by = "cyl")
DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
        fill_by = "mpg", fill_cutoff = "< 18", facet_by = "cyl",
        facet_scales = "free_x")

# --- split_by ---
DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
        fill_by = "mpg", fill_cutoff = "< 18", split_by = "cyl")
DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
        fill_by = "mpg", fill_cutoff = "< 18", split_by = "cyl",
        palette = list("4" = "Set1", "6" = "Paired", "8" = "Reds"))

# --- Scatter plot (both axes numeric) ---
DotPlot(mtcars, x = "qsec", y = "drat", size_by = "wt",
        fill_by = "mpg", fill_cutoff = "< 18",
        fill_cutoff_name = "Small mpgs")

# --- keep_na and keep_empty ---
mtcars$carb[mtcars$carb == "1"] <- NA
mtcars$gear[mtcars$gear == "3"] <- NA
DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
        fill_by = "mpg", fill_cutoff = "< 18",
        keep_na = TRUE, keep_empty = TRUE)

# --- Border customization ---
DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
        fill_by = "mpg", border_color = "red", border_size = 2)
DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
        fill_by = "mpg", border_color = TRUE, border_size = 1.5,
        border_alpha = 0.5)
DotPlot(mtcars, x = "carb", y = "gear",
        fill_by = "mpg", border_color = FALSE)

# --- Colour scale trimming ---
DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
        fill_by = "mpg", lower_quantile = 0.05, upper_quantile = 0.95)
DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
        fill_by = "mpg", lower_cutoff = 15, upper_cutoff = 25)
}
\donttest{
mtcars <- datasets::mtcars

# --- Basic lollipop ---
LollipopPlot(mtcars, x = "qsec", y = "drat", size_by = "wt",
             fill_by = "mpg")

# --- Faceting ---
LollipopPlot(mtcars, x = "qsec", y = "drat", size_by = "wt",
             fill_by = "mpg", fill_cutoff = "< 18", facet_by = "cyl",
             facet_scales = "free_y")

# --- split_by ---
LollipopPlot(mtcars, x = "qsec", y = "drat", size_by = "wt",
             split_by = "vs", palette = list("0" = "Reds", "1" = "Blues"))

# --- Border customization ---
LollipopPlot(mtcars, x = "qsec", y = "drat", size_by = "wt",
             fill_by = "mpg", border_color = "red", border_size = 2)
LollipopPlot(mtcars, x = "qsec", y = "drat", size_by = "wt",
             fill_by = "mpg", border_color = TRUE, border_size = 1.5,
             border_alpha = 0.5)
LollipopPlot(mtcars, x = "qsec", y = "drat",
             fill_by = "mpg", border_color = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/velocityplot.R
\name{.compute_velocity_on_grid}
\alias{.compute_velocity_on_grid}
\title{Compute velocity on a regular grid from sparse cell embeddings}
\usage{
.compute_velocity_on_grid(
  embedding,
  v_embedding,
  density = NULL,
  smooth = NULL,
  n_neighbors = NULL,
  min_mass = NULL,
  scale = 1,
  adjust_for_stream = FALSE,
  cutoff_perc = NULL
)
}
\arguments{
\item{embedding}{A numeric matrix of dimension n_obs x n_dim containing the low-dimensional embedding coordinates of each cell.}

\item{v_embedding}{A numeric matrix of dimension n_obs x n_dim containing the velocity vectors for each cell.}

\item{density}{A numeric value specifying the density of the grid points along each dimension. Higher values produce a finer grid. Default is 1.}

\item{smooth}{A numeric value specifying the standard deviation multiplier for the Gaussian kernel used to weight neighboring cells when averaging velocities onto grid points. Default is 0.5.}

\item{n_neighbors}{An integer value specifying the number of nearest neighbors to consider for each grid point when computing the weighted average velocity. Default is \code{ceiling(n_obs / 50)}.}

\item{min_mass}{A numeric value specifying the minimum mass threshold. Grid points with total weight below this threshold are filtered out. When \code{adjust_for_stream = TRUE}, this is interpreted on a logarithmic scale (\code{10^(min_mass - 6)}). Default is 1.}

\item{scale}{A numeric value specifying the scaling factor to apply to the resulting grid velocity vectors. Only used when \code{adjust_for_stream = FALSE}. Default is 1.}

\item{adjust_for_stream}{A logical value indicating whether to adjust the output for streamline rendering. When \code{TRUE}, returns a 2D array format with different filtering logic suitable for \code{metR::geom_streamline}. Default is \code{FALSE}.}

\item{cutoff_perc}{A numeric value specifying the percentile cutoff for removing low-density grid points. Only used when \code{adjust_for_stream = TRUE}. Default is 5.}
}
\value{
A list with components \code{x_grid} (grid point coordinates) and \code{v_grid} (velocity vectors at each grid point). When \code{adjust_for_stream = TRUE}, \code{x_grid} is a 2-row matrix of unique coordinates and \code{v_grid} is a 3D array.
}
\description{
Computes velocity vectors on a regular grid by averaging cell-level velocities
within a Gaussian-weighted neighborhood of each grid point. This function is
adapted from the scvelo Python implementation at
\url{https://github.com/theislab/scvelo/blob/master/scvelo/plotting/velocity_embedding_grid.py}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsea.R
\name{GSEASummaryPlot}
\alias{GSEASummaryPlot}
\alias{GSEAPlot}
\title{GSEA summary dot plot}
\usage{
GSEASummaryPlot(
  data,
  in_form = c("auto", "dose", "fgsea"),
  gene_ranks = "@gene_ranks",
  gene_sets = "@gene_sets",
  top_term = 10,
  metric = "p.adjust",
  cutoff = 0.05,
  character_width = 50,
  line_plot_size = 0.25,
  metric_name = metric,
  nonsig_name = "Insignificant",
  linewidth = 0.2,
  line_by = c("prerank", "running_score"),
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  alpha = 0.6,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  palreverse = FALSE,
  seed = 8525,
  ...
)

GSEAPlot(
  data,
  in_form = c("auto", "dose", "fgsea"),
  gene_ranks = "@gene_ranks",
  gene_sets = "@gene_sets",
  gs = NULL,
  sample_coregenes = FALSE,
  line_width = 1.5,
  line_alpha = 1,
  line_color = "#6BB82D",
  n_coregenes = 10,
  genes_label = NULL,
  label_fg = "black",
  label_bg = "white",
  label_bg_r = 0.1,
  label_size = 4,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  seed = 8525,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{in_form}{The format of the input data. See \code{\link{GSEASummaryPlot}}
for details.}

\item{gene_ranks}{A named numeric vector of gene-level rank statistics, with
gene identifiers as names. Used to construct the per-term line plots.
If a character string starting with \code{"@"}, the attribute of \code{data}
with that name (minus the \code{"@"}) is used as the gene ranks vector.}

\item{gene_sets}{A named list of gene sets. Each name must correspond to an
\code{ID} in \code{data}, and each element is a character vector of gene
identifiers. A GSEA ridge plot is generated for each gene set in the list.
If you only want to plot a subset of gene sets, subset the list before
passing it to this function. If a character string starting with
\code{"@"}, the attribute of \code{data} with that name (minus the
\code{"@"}) is used.}

\item{top_term}{Integer specifying the number of top terms to display, ranked
by \code{metric}. If \code{NULL}, all terms are shown.}

\item{metric}{Character string specifying the column name used to rank terms
and assess significance. Typically \code{"p.adjust"} or \code{"pvalue"}.
Terms are ranked by this column (ascending, lower is better) when
\code{top_term} is set. The same column is transformed to
\code{-log10(metric)} for the colour gradient.}

\item{cutoff}{Numeric threshold for the \code{metric} column. Terms with
values below this cutoff are coloured on a gradient; terms above are drawn
in grey (\code{"grey80"}) and labelled as insignificant via
\code{nonsig_name}. Default is \code{0.05}. If \code{NULL}, all terms are
treated as significant.}

\item{character_width}{Integer specifying the maximum character width for
wrapping term descriptions on the y-axis. Default is \code{50}.}

\item{line_plot_size}{Numeric controlling the size of the per-term miniature
enrichment plots embedded in each row. Expressed as a fraction of the plot
panel dimensions. Default is \code{0.25}.}

\item{metric_name}{Character string for the colour bar legend title.
Defaults to the value of \code{metric}.}

\item{nonsig_name}{Character string for the legend entry label used for
non-significant terms. Default is \code{"Insignificant"}.}

\item{linewidth}{Numeric specifying the line width within the per-term
miniature enrichment plots. Default is \code{0.2}.}

\item{line_by}{The method used to compute the per-term line plots:
\itemize{
\item{\code{"prerank"} (default): Use the gene ranks as the bar heights
(raw ranking metric).}
\item{\code{"running_score"}: Use the running enrichment score computed
by \code{gsea_running_score()}.}
}}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{seed}{A numeric seed for reproducibility. Passed to
\code{\link{validate_common_args}()}.}

\item{...}{Additional arguments.}

\item{gs}{Character vector of gene set \code{ID}s to plot. If \code{NULL}
(default), all gene sets in \code{gene_sets} that appear in \code{data$ID}
are plotted.}

\item{sample_coregenes}{Logical; if \code{TRUE}, core enrichment genes are
sampled randomly for labelling. If \code{FALSE} (default), the first
\code{n_coregenes} core enrichment genes are used.}

\item{line_width}{Numeric specifying the line width for the running
enrichment score curve. Default is \code{1.5}.}

\item{line_alpha}{Numeric alpha transparency for the running score line and
hit indicator bars. Default is \code{1}.}

\item{line_color}{Character string specifying the colour of the running
enrichment score line. Default is \code{"#6BB82D"}.}

\item{n_coregenes}{Integer specifying the number of core enrichment genes to
label on the running score plot. Default is \code{10}. Ignored when
\code{genes_label} is provided.}

\item{genes_label}{Character vector of specific gene names to label on the
running score plot. When provided, \code{n_coregenes} is ignored.}

\item{label_fg}{Character string specifying the text colour of gene labels.
Default is \code{"black"}.}

\item{label_bg}{Character string specifying the background colour of gene
labels. Default is \code{"white"}.}

\item{label_bg_r}{Numeric specifying the corner radius of the label
background. Default is \code{0.1}.}

\item{label_size}{Numeric specifying the font size of the label text.
Default is \code{4}.}

\item{combine}{Logical; when \code{TRUE} (default), returns a combined
\code{patchwork} object. When \code{FALSE}, returns a named list of
individual \code{patchwork} objects (one per gene set).}

\item{ncol, nrow}{Integer number of columns / rows for the combined layout
(passed to \code{\link{combine_plots}()}).}

\item{byrow}{Logical; fill the combined layout by row. Default \code{TRUE}
(passed to \code{\link{combine_plots}()}).}

\item{axes}{A character string specifying how axes should be treated across
the combined layout (passed to \code{\link{combine_plots}()}).}

\item{axis_titles}{A character string specifying how axis titles should be
treated across the combined layout. Defaults to \code{axes}.}

\item{guides}{A character string specifying how guides (legends) should be
collected across panels (passed to \code{\link{combine_plots}()}).}

\item{design}{A custom layout design for the combined plot (passed to
\code{\link{combine_plots}()}).}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.

A \code{patchwork} object when \code{combine = TRUE}, or a named
list of \code{patchwork} objects when \code{combine = FALSE}. Each
individual plot has \code{height} and \code{width} attributes in inches.
}
\description{
Produces a summary dot plot of GSEA (Gene Set Enrichment Analysis) results.
Each row represents a gene set (term), positioned along the x-axis by its
Normalized Enrichment Score (NES). Dot colour encodes the significance level
(typically \code{-log10(p.adjust)}) on a continuous gradient, and each row
includes a miniature line plot showing the gene ranks or running enrichment
score for that term's gene set.

The function supports both \code{DOSE} and \code{fgsea} package output
formats via the \code{in_form} parameter. Terms can be ranked and selected
by a significance metric (\code{top_term}, \code{metric}), with
non-significant terms rendered in grey. The per-term line plots can show
either the raw preranked gene statistics (\code{line_by = "prerank"}) or
the running enrichment score (\code{line_by = "running_score"}).
}
\examples{
\donttest{
data(gsea_example)

# Default summary dot plot with preranked gene statistics
GSEASummaryPlot(gsea_example)

# Use running enrichment score for per-term line plots
GSEASummaryPlot(gsea_example, line_by = "running_score")

# Raise the significance cutoff (all terms are coloured)
GSEASummaryPlot(gsea_example, cutoff = 0.01)
}
\donttest{
data(gsea_example)

# Single gene set
GSEAPlot(gsea_example, gene_sets = attr(gsea_example, "gene_sets")[1])

# Multiple gene sets arranged in a grid
GSEAPlot(gsea_example, gene_sets = attr(gsea_example, "gene_sets")[1:4])
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{words_excluded}
\alias{words_excluded}
\title{Excluded words in keyword enrichment analysis and extraction}
\description{
The variable "words_excluded" represents the words that are excluded during keyword enrichment analysis or keyword extraction process. These mainly include words that are excessively redundant or of little value.
}
\examples{
\dontrun{
if (interactive()) {
  words_excluded <- c(
    "the", "is", "and", "or", "a", "in", "on", "under", "between", "of", "through",
    "via", "along", "that", "for", "with", "within", "without", "cell", "cellular",
    "dna", "rna", "protein", "peptide", "amino", "acid", "development", "involved",
    "organization", "system", "regulation", "regulated", "positive", "negative",
    "response", "process", "processing", "small", "large", "change", "disease"
  )
}
}
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{do_call}
\alias{do_call}
\title{Call a function with a list of arguments}
\usage{
do_call(what, args, quote = FALSE, envir = parent.frame())
}
\arguments{
\item{what}{either a function or a non-empty character string naming the
    function to be called.}

\item{args}{a \emph{list} of arguments to the function call.  The
    \code{names} attribute of \code{args} gives the argument names.}

\item{quote}{a logical value indicating whether to quote the
    arguments.}

\item{envir}{an environment within which to evaluate the call.  This
    will be most useful if \code{what} is a character string and
    the arguments are symbols or quoted expressions.}
}
\value{
The result of the function call
}
\description{
A faster alternative to \code{\link[base]{do.call}}, especially when there are
large objects in the argument list. Named arguments are looked up by symbol
in the evaluation environment rather than being copied into the call, which
avoids the copying overhead of \code{base::do.call}. Unnamed arguments are
embedded in the call directly and do not benefit from this optimization.
Borrowed from \code{Gmisc::fastDoCall}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/densityplot.R
\name{DensityPlot}
\alias{DensityPlot}
\alias{Histogram}
\title{Density Plot / Histogram}
\usage{
DensityPlot(
  data,
  x,
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  xtrans = "identity",
  ytrans = "identity",
  split_by = NULL,
  split_by_sep = "_",
  flip = FALSE,
  position = "identity",
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 0.5,
  theme = "theme_this",
  theme_args = list(),
  add_bars = FALSE,
  bar_height = 0.025,
  bar_alpha = 1,
  bar_width = 0.1,
  keep_na = FALSE,
  keep_empty = FALSE,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  expand = c(bottom = 0, left = 0, right = 0),
  facet_by = NULL,
  facet_scales = "free_y",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  legend.position = ifelse(is.null(group_by), "none", "right"),
  legend.direction = "vertical",
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)

Histogram(
  data,
  x,
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  xtrans = "identity",
  ytrans = "identity",
  split_by = NULL,
  split_by_sep = "_",
  flip = FALSE,
  bins = NULL,
  binwidth = NULL,
  trend_skip_zero = FALSE,
  add_bars = FALSE,
  bar_height = 0.025,
  bar_alpha = 1,
  bar_width = 0.1,
  position = "identity",
  keep_na = FALSE,
  keep_empty = FALSE,
  use_trend = FALSE,
  add_trend = FALSE,
  trend_alpha = 1,
  trend_linewidth = 0.8,
  trend_pt_size = 1.5,
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 0.5,
  theme = "theme_this",
  theme_args = list(),
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  expand = c(bottom = 0, left = 0, right = 0),
  facet_by = NULL,
  facet_scales = "free_y",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  legend.position = ifelse(is.null(group_by), "none", "right"),
  legend.direction = "vertical",
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis.}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{group_name}{A character string used as the legend title for the
\code{group_by} aesthetic. When \code{NULL} (default), the (possibly concatenated)
\code{group_by} column name is used.}

\item{xtrans}{A character string specifying the transformation applied to
the x-axis. Passed to \code{ggplot2::scale_x_continuous(transform = ...)}.
Supported values include \code{"identity"} (default), \code{"log10"}, \code{"log2"},
\code{"sqrt"}, \code{"reverse"}, etc.}

\item{ytrans}{A character string specifying the transformation applied to
the y-axis. Passed to \code{ggplot2::scale_y_continuous(transform = ...)}.
Used by \code{trend_skip_zero} to correctly interpolate across zero bins on
a transformed scale. Default: \code{"identity"}.}

\item{split_by}{The column(s) to split data by and plot separately.
When \code{combine = TRUE}, the combined plot's data (\code{p$data})
contains the rows of every split, tagged with the \code{split_by}
column. For ggraph-based plots (e.g. \code{\link{Network}},
\code{\link{ClustreePlot}}), the node layout of every split is
exposed with the split column, and the original link rows (also
tagged with the \code{split_by} column) are available as the
\code{"edges"} attribute of the data.}

\item{split_by_sep}{The separator for multiple split_by columns. See \code{split_by}}

\item{flip}{A logical value. If \code{TRUE}, the x and y axes are swapped via
\code{coord_flip()}. Dimension calculation accounts for the flip.}

\item{position}{A character string specifying the position adjustment for
the bars or density curves. Default: \code{"identity"}, which shows the
actual count / density per group (unlike \code{ggplot2}'s default \code{"stack"}).
Other options: \code{"stack"}, \code{"dodge"}, \code{"fill"}.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{add_bars}{A logical value. If \code{TRUE}, a data-distribution rug is
drawn along the y = 0 axis using \code{geom_linerange()}. Each group's bars
are vertically offset to avoid overlap.}

\item{bar_height}{A numeric value specifying the height (in data units,
relative to the maximum y) of the rug bars added by \code{add_bars}.
The actual pixel height scales with \code{max_y}. Default: \code{0.025}.}

\item{bar_alpha}{A numeric value in \verb{[0, 1]} for the transparency of the
rug bars. Default: \code{1}.}

\item{bar_width}{A numeric value passed as the \code{linewidth} aesthetic of
\code{geom_linerange()}. Controls the thickness of each rug tick.
Default: \code{0.1}.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{expand}{The values to expand the x and y axes. It is like CSS padding.
When a single value is provided, it is used for both axes on both sides.
When two values are provided, the first value is used for the top/bottom side and the second value is used for the left/right side.
When three values are provided, the first value is used for the top side, the second value is used for the left/right side, and the third value is used for the bottom side.
When four values are provided, the values are used for the top, right, bottom, and left sides, respectively.
You can also use a named vector to specify the values for each side.
When the axis is discrete, the values will be applied as 'add' to the 'expansion' function.
When the axis is continuous, the values will be applied as 'mult' to the 'expansion' function.
See also \url{https://ggplot2.tidyverse.org/reference/expansion.html}}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{seed}{The random seed to use. Default is 8525.}

\item{combine}{Whether to combine the plots into one when facet is FALSE. Default is TRUE.}

\item{nrow}{A numeric value specifying the number of rows in the facet.}

\item{ncol}{A numeric value specifying the number of columns in the facet.}

\item{byrow}{A logical value indicating whether to fill the plots by row.}

\item{axes}{A string specifying how axes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axes in individual plots.
\item 'collect' will remove duplicated axes when placed in the same run of rows or columns of the layout.
\item 'collect_x' and 'collect_y' will remove duplicated x-axes in the columns or duplicated y-axes in the rows respectively.
}}

\item{axis_titles}{A string specifying how axis titltes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axis titles in individual plots.
\item 'collect' will remove duplicated titles in one direction and merge titles in the opposite direction.
\item 'collect_x' and 'collect_y' control this for x-axis titles and y-axis titles respectively.
}}

\item{guides}{A string specifying how guides should be treated in the layout. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'collect' will collect guides below to the given nesting level, removing duplicates.
\item 'keep' will stop collection at this level and let guides be placed alongside their plot.
\item 'auto' will allow guides to be collected if a upper level tries, but place them alongside the plot if not.
}}

\item{design}{Specification of the location of areas in the layout, passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE. When specified, \code{nrow}, \code{ncol}, and \code{byrow} are ignored.
See \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}} for more details.}

\item{...}{Additional arguments.}

\item{bins}{A numeric value specifying the number of bins for the histogram.
Ignored when \code{type = "density"}. Defaults to \code{30} when neither \code{bins} nor
\code{binwidth} is provided.}

\item{binwidth}{A numeric value specifying the width of individual bins for
the histogram. Ignored when \code{type = "density"}. Takes precedence over
\code{bins} when both are set.}

\item{trend_skip_zero}{A logical value. If \code{TRUE}, bins with zero count are
set to \code{NA} before the trend line is computed, and \code{zoo::na.approx()} is
used to interpolate across the gaps — producing a continuous curve even
when some bins are empty. Requires \code{ytrans} to be correctly specified.
Only applies when \code{type = "histogram"} and \code{use_trend} or \code{add_trend} is
active.}

\item{use_trend}{A logical value. If \code{TRUE}, the histogram bars are replaced
entirely by a trend line (points + connecting line). Only applies when
\code{type = "histogram"}.}

\item{add_trend}{A logical value. If \code{TRUE}, a trend line is overlaid on
top of the histogram bars. Only applies when \code{type = "histogram"}.}

\item{trend_alpha}{A numeric value in \verb{[0, 1]} controlling the transparency
of the trend points and line. Default: \code{1}.}

\item{trend_linewidth}{A numeric value for the thickness of the trend line.
Default: \code{0.8}.}

\item{trend_pt_size}{A numeric value for the size of the trend points.
Default: \code{1.5}.}
}
\value{
A \code{ggplot} object (single plot), a \code{patchwork} / \code{wrap_plots} object
(when \code{split_by} is provided and \code{combine = TRUE}), or a list of \code{ggplot}
objects (when \code{split_by} is provided and \code{combine = FALSE}).

A \code{ggplot} object (single plot), a \code{patchwork} / \code{wrap_plots} object
(when \code{split_by} is provided and \code{combine = TRUE}), or a list of \code{ggplot}
objects (when \code{split_by} is provided and \code{combine = FALSE}).
}
\description{
Density plot for visualising the distribution of a numeric variable. Uses
\code{ggplot2::geom_density()} to render smooth kernel density estimates, with
optional grouping, faceting, split-by splitting, and data-distribution rug
bars along the baseline.

This is the public entry point for density plots; the companion
\code{Histogram()} function provides binned-histogram rendering.
Both dispatch to the same internal engine (\code{DensityHistoPlotAtomic})
with \code{type = "density"} or \code{type = "histogram"} respectively.

Histogram for visualising the distribution of a numeric variable via binned
counts. Uses \code{ggplot2::geom_histogram()}, with optional trend-line overlays,
zero-skip interpolation, grouping, faceting, and split-by splitting.

This is the histogram companion to \code{DensityPlot()}. Both dispatch to the
same internal engine (\code{DensityHistoPlotAtomic}) with \code{type = "histogram"}
or \code{type = "density"} respectively.

When \code{use_trend = TRUE}, the histogram bars are replaced entirely by a
point-and-line trend; when \code{add_trend = TRUE}, the trend is overlaid on top
of the bars. The \code{trend_skip_zero} option uses \code{zoo::na.approx()} to
interpolate across empty bins for a continuous trend curve — particularly
useful with transformed y-axes.
}
\section{split_by Workflow}{


When \code{split_by} is specified, \code{DensityPlot()} executes the following pipeline:
\enumerate{
\item \strong{Argument validation} — \code{validate_common_args()} checks the seed and
facet-by consistency.
\item \strong{NA / empty normalisation} — \code{check_keep_na()} / \code{check_keep_empty()}
convert \code{keep_na} / \code{keep_empty} to per-column lists.
\item \strong{Theme resolution} — \code{process_theme()} resolves the theme string to a
theme function.
\item \strong{Split column resolution} — \code{check_columns()} validates \code{split_by}
(force_factor, concat_multi).
\item \strong{Pre-filtering} — \code{process_keep_na_empty()} removes NA / empty levels
from the split column, then \code{data} is split by \code{split_by} levels (order
preserved).
\item \strong{Per-split parameter resolution} — \code{check_palette()},
\code{check_palcolor()}, \code{check_legend()} resolve palette, palcolor,
legend.position, and legend.direction for each split.
\item \strong{Per-split dispatch} — each split is passed to
\code{DensityHistoPlotAtomic(type = "density", ...)} with its resolved
parameters. Title defaults to the split level name unless \code{title} is
a function.
\item \strong{Combination} — \code{combine_plots()} assembles the list of plots via
\code{patchwork::wrap_plots()}, applying \code{nrow}, \code{ncol}, \code{byrow}, \code{axes},
\code{axis_titles}, \code{guides}, and \code{design}.
}



When \code{split_by} is specified, \code{Histogram()} executes the following pipeline:
\enumerate{
\item \strong{Argument validation} — \code{validate_common_args()} checks the seed and
facet-by consistency.
\item \strong{NA / empty normalisation} — \code{check_keep_na()} / \code{check_keep_empty()}
convert \code{keep_na} / \code{keep_empty} to per-column lists.
\item \strong{Theme resolution} — \code{process_theme()} resolves the theme string to a
theme function.
\item \strong{Split column resolution} — \code{check_columns()} validates \code{split_by}
(force_factor, concat_multi).
\item \strong{Pre-filtering} — \code{process_keep_na_empty()} removes NA / empty levels
from the split column, then \code{data} is split by \code{split_by} levels (order
preserved).
\item \strong{Per-split parameter resolution} — \code{check_palette()},
\code{check_palcolor()}, \code{check_legend()} resolve palette, palcolor,
legend.position, and legend.direction for each split.
\item \strong{Per-split dispatch} — each split is passed to
\code{DensityHistoPlotAtomic(type = "histogram", ...)} with its resolved
parameters (including \code{bins}, \code{binwidth}, \code{use_trend}, \code{add_trend},
\code{trend_skip_zero}, \code{trend_alpha}, \code{trend_linewidth}, \code{trend_pt_size}).
Title defaults to the split level name unless \code{title} is a function.
\item \strong{Combination} — \code{combine_plots()} assembles the list of plots via
\code{patchwork::wrap_plots()}, applying \code{nrow}, \code{ncol}, \code{byrow}, \code{axes},
\code{axis_titles}, \code{guides}, and \code{design}.
}
}

\examples{
\donttest{
set.seed(8525)
data <- data.frame(
    x = c(rnorm(500, -1), rnorm(500, 1)),
    group = factor(rep(c("A", NA, "C", "D"), each = 250), levels = LETTERS[1:4]),
    facet = sample(c("F1", "F2"), 1000, replace = TRUE)
)

# basic density
DensityPlot(data, x = "x")
DensityPlot(data, x = "x", group_by = "group")

# NA / empty level handling
DensityPlot(data, x = "x", group_by = "group",
    keep_na = TRUE, keep_empty = TRUE)
DensityPlot(data, x = "x", group_by = "group",
    keep_na = TRUE, keep_empty = 'level')

# faceting and splitting
DensityPlot(data, x = "x", group_by = "group", facet_by = "facet")
DensityPlot(data, x = "x", split_by = "facet", add_bars = TRUE)
DensityPlot(data, x = "x", split_by = "facet", add_bars = TRUE,
    palette = c(F1 = "Set1", F2 = "Set2"))
}
set.seed(8525)
data <- data.frame(
    x = sample(setdiff(1:100, c(30:36, 50:55, 70:77)), 1000, replace = TRUE),
    group = factor(rep(c("A", "B", NA, "D"), each = 250), levels = LETTERS[1:4]),
    facet = sample(c("F1", "F2"), 1000, replace = TRUE)
)

# basic histogram
Histogram(data, x = "x")
Histogram(data, x = "x", group_by = "group")

# NA / empty level handling
Histogram(data, x = "x", group_by = "group", keep_na = TRUE, keep_empty = 'level')

# add_bars and trend overlays
Histogram(data, x = "x", split_by = "facet", add_bars = TRUE)
Histogram(data, x = "x", group_by = "group", add_trend = TRUE)
Histogram(data, x = "x", group_by = "group", add_trend = TRUE, trend_skip_zero = TRUE)

# use_trend replaces bars entirely
Histogram(data, x = "x", group_by = "group", split_by = "facet",
 use_trend = TRUE, trend_pt_size = 3)

# per-split palettes
Histogram(data, x = "x", group_by = "group", split_by = "facet",
 palette = c(F1 = "Paired", F2 = "Spectral"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/piechart.R
\name{PieChartAtomic}
\alias{PieChartAtomic}
\title{Atomic pie chart (internal)}
\usage{
PieChartAtomic(
  data,
  x,
  y = NULL,
  label = y,
  clockwise = TRUE,
  keep_na = FALSE,
  keep_empty = FALSE,
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  facet_by = NULL,
  facet_scales = "free_y",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name for the x-axis
(categories).  Must be character or factor.  Each unique value becomes a
pie slice.}

\item{y}{A character string specifying the numeric column for the y-axis.
When \code{NULL} (default), the count of observations in each
(\code{x}, \code{facet_by}) combination is used and stored as
\code{.y}.}

\item{label}{A character string specifying the column to use for slice
labels.  \code{NULL} (default) hides labels.  When \code{TRUE}, the
\code{y} values are used as labels.  When \code{y = NULL}, use
\code{".y"} to label with the computed counts.}

\item{clockwise}{A logical value.  When \code{TRUE} (default), the pie
slices are ordered clockwise starting from the top.  When \code{FALSE},
slices are ordered counter-clockwise.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a single pie chart.  This is the
workhorse behind the exported \code{\link{PieChart}} function — it takes a
\strong{single} data frame (no \code{split_by} support) and returns a
\code{ggplot} object.  The chart maps the proportion of each x-axis
category to the angle of a pie slice using
\code{\link[ggplot2]{coord_polar}()}, with optional faceting and
per-slice labels.
}
\section{Architecture}{

\enumerate{
\item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{Column resolution} — \code{x} is forced to factor;
\code{y} is validated (numeric, optional).
\item \strong{Count aggregation} — when \code{y = NULL}, the count of
observations per (\code{x}, \code{facet_by}) combination is
computed as a new \code{.y} column.  Factor levels are preserved
after aggregation.
\item \strong{Label resolution} — when \code{label = TRUE}, the label
column is set to \code{y}.  The \code{label} column is then
validated.
\item \strong{Facet column resolution} — \code{facet_by} columns are
validated as factors, allowing up to two columns.
\item \strong{NA / empty-level handling} —
\code{\link{process_keep_na_empty}()} applies \code{keep_na} and
\code{keep_empty} policies.  Per-column \code{keep_empty} settings
are extracted for \code{x} and \code{facet_by}.  When more than one
facet column is provided, their \code{keep_empty} values must be
identical.
\item \strong{Clockwise ordering} — when \code{clockwise = TRUE}
(default), the levels of \code{x} are reversed so that the first
slice starts from the top and proceeds clockwise.  The data is then
sorted by the (possibly reversed) \code{x} factor levels.
\item \strong{Position calculation} — if faceted, grouping by facet
variables; computes cumulative sums (\code{csum}) and label
midpoint positions (\code{pos}) for each slice.
\item \strong{Colour mapping} — \code{\link{palette_this}()} assigns
colours to all \code{x} levels, including \code{NA} (defaulting to
\code{"grey80"}).
\item \strong{Plot assembly} — the \code{ggplot} object is built with
\code{geom_col(width = 1)} and
\code{coord_polar(theta = "y")} to create the circular pie
layout.  The fill scale uses \code{scale_fill_manual()} with
per-slice colours; the \code{drop} argument is controlled by
\code{keep_empty_x}.
\item \strong{Labels} — when \code{label} is not \code{NULL},
\code{\link[ggrepel]{geom_label_repel}()} adds text labels at
the computed midpoint positions (\code{pos}).
\item \strong{Dimension calculation} —
\code{\link{calculate_plot_dimensions}()} computes plot height and
width from \code{aspect.ratio} and legend metrics.  The resulting
\code{height} / \code{width} attributes are stored on the
\code{ggplot} object.
\item \strong{Faceting} — \code{\link{facet_plot}()} wraps the plot
with \code{facet_wrap} / \code{facet_grid} if \code{facet_by} is
provided, respecting the \code{keep_empty} setting for facet
variables.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/velocityplot.R
\name{VelocityPlot}
\alias{VelocityPlot}
\title{Cell velocity plot}
\usage{
VelocityPlot(
  embedding,
  v_embedding,
  plot_type = c("raw", "grid", "stream"),
  split_by = NULL,
  group_by = NULL,
  group_name = "Group",
  group_palette = "Paired",
  group_palcolor = NULL,
  n_neighbors = NULL,
  density = 1,
  smooth = 0.5,
  scale = 1,
  min_mass = 1,
  cutoff_perc = 5,
  arrow_angle = 20,
  arrow_color = "black",
  arrow_alpha = 1,
  keep_na = FALSE,
  keep_empty = FALSE,
  streamline_l = 5,
  streamline_minl = 1,
  streamline_res = 1,
  streamline_n = 15,
  streamline_width = c(0, 0.8),
  streamline_alpha = 1,
  streamline_color = NULL,
  streamline_palette = "RdYlBu",
  streamline_palcolor = NULL,
  palreverse = FALSE,
  streamline_bg_color = "white",
  streamline_bg_stroke = 0.5,
  aspect.ratio = 1,
  title = "Cell velocity",
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  theme = "theme_this",
  theme_args = list(),
  return_layer = FALSE,
  seed = 8525
)
}
\arguments{
\item{embedding}{A matrix or data frame of dimension n_obs x n_dim specifying the low-dimensional embedding coordinates (e.g., UMAP, t-SNE) of the cells. The first two columns are used for the x and y axes.}

\item{v_embedding}{A matrix or data frame of dimension n_obs x n_dim specifying the velocity vectors for each cell. Must have the same dimensions as \code{embedding}.}

\item{plot_type}{A character string specifying the visualization method. \code{"raw"} plots arrows directly from each cell's embedding position. \code{"grid"} averages velocities onto a regular grid and plots arrows at grid points. \code{"stream"} computes smooth streamline paths from the gridded velocity field. Default is \code{"raw"}.}

\item{split_by}{Not supported for VelocityPlot. Setting this parameter will raise an error.}

\item{group_by}{An optional vector of the same length as the number of rows in \code{embedding} specifying a grouping variable for cells. When provided, arrows are colored by group using \code{group_palette}. Only applies to \code{plot_type = "raw"}; ignored with a warning for \code{"grid"} and \code{"stream"}. Default is \code{NULL}.}

\item{group_name}{A character string specifying the legend title for the grouping variable. Default is \code{"Group"}.}

\item{group_palette}{A character string specifying the color palette to use for the grouping variable. Passed to \code{\link{palette_this}}. Default is \code{"Paired"}.}

\item{group_palcolor}{An optional character vector of specific colors for the grouping variable. If \code{NULL}, colors are generated from \code{group_palette}. Default is \code{NULL}.}

\item{n_neighbors}{An integer value specifying the number of nearest neighbors for computing grid velocities. Only used when \code{plot_type} is \code{"grid"} or \code{"stream"}. Default is \code{ceiling(nrow(embedding) / 50)}.}

\item{density}{A numeric value specifying the grid density along each dimension. Only used when \code{plot_type} is \code{"grid"} or \code{"stream"}. For \code{plot_type = "raw"}, when \code{density} is between 0 and 1, it specifies the fraction of cells to randomly subsample. Default is 1.}

\item{smooth}{A numeric value specifying the standard deviation multiplier for the Gaussian kernel when averaging cell velocities onto grid points. Only used when \code{plot_type} is \code{"grid"} or \code{"stream"}. Default is 0.5.}

\item{scale}{A numeric value specifying the scaling factor for the velocity vectors. Applied to raw and grid arrows. For \code{plot_type = "stream"}, this is fixed to 1 internally. Default is 1.}

\item{min_mass}{A numeric value specifying the minimum mass threshold for retaining grid points. Only used when \code{plot_type} is \code{"grid"} or \code{"stream"}. Default is 1.}

\item{cutoff_perc}{A numeric value specifying the percentile cutoff for removing low-density grid points. Only used when \code{plot_type} is \code{"stream"}. Default is 5.}

\item{arrow_angle}{A numeric value specifying the angle of the arrowheads in degrees. Applied to \code{\link[grid]{arrow}} when \code{plot_type} is \code{"raw"} or \code{"grid"}. Default is 20.}

\item{arrow_color}{A character string specifying the color of the velocity arrows. For \code{plot_type = "stream"}, this sets only the arrowhead color. Default is \code{"black"}.}

\item{arrow_alpha}{A numeric value between 0 and 1 specifying the transparency of the velocity arrows. Only used when \code{plot_type = "raw"} or \code{"grid"}; for \code{plot_type = "stream"}, use \code{streamline_alpha} instead. Default is 1.}

\item{keep_na}{A logical or character value specifying how to handle NA values in \code{group_by}. Unlike other plot functions, VelocityPlot does not support named lists for per-column control. See \code{keep_na} in \code{common_args} for details of supported values. Default is \code{FALSE}.}

\item{keep_empty}{One of \code{FALSE}, \code{TRUE}, or \code{"level"} specifying how to handle empty factor levels in \code{group_by}. Unlike other plot functions, VelocityPlot does not support named lists for per-column control. See \code{keep_empty} in \code{common_args} for details. Default is \code{FALSE}.}

\item{streamline_l}{A numeric value specifying the integration length of the streamlines. Passed to \code{\link[metR]{geom_streamline}} as the \code{L} parameter. Default is 5.}

\item{streamline_minl}{A numeric value specifying the minimum streamline length. Shorter streamlines are not drawn. Passed to \code{\link[metR]{geom_streamline}} as the \code{min.L} parameter. Default is 1.}

\item{streamline_res}{A numeric value specifying the resolution of the streamline integration. Passed to \code{\link[metR]{geom_streamline}} as the \code{res} parameter. Default is 1.}

\item{streamline_n}{A numeric value specifying the number of streamlines to draw. Passed to \code{\link[metR]{geom_streamline}} as the \code{n} parameter. Default is 15.}

\item{streamline_width}{A numeric vector of length 2 specifying the range of line widths for streamlines. Passed to \code{scale_size(range = ...)}. Only used when \code{streamline_color} is \code{NULL}. Default is \code{c(0, 0.8)}.}

\item{streamline_alpha}{A numeric value between 0 and 1 specifying the transparency of the velocity streamlines. Default is 1.}

\item{streamline_color}{An optional character string specifying a fixed color for streamlines. When \code{NULL} (the default), streamlines are colored by velocity magnitude using \code{streamline_palette}.}

\item{streamline_palette}{A character string specifying the color palette for streamline velocity magnitude. Passed to \code{\link{palette_this}}. Only used when \code{streamline_color} is \code{NULL}. Default is \code{"RdYlBu"}.}

\item{streamline_palcolor}{An optional character vector of specific colors for the streamline velocity gradient. If \code{NULL}, colors are generated from \code{streamline_palette}. Default is \code{NULL}.}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{streamline_bg_color}{A character string specifying the background (outline) color applied to streamlines to create a stroke effect. Default is \code{"white"}.}

\item{streamline_bg_stroke}{A numeric value specifying the additional line width of the background stroke relative to the foreground streamline. Default is 0.5.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{return_layer}{A logical value indicating whether to return only the ggplot layers instead of the full assembled plot. When \code{TRUE}, returns a list of ggplot layers suitable for combining with other ggplot objects. Default is \code{FALSE}.}

\item{seed}{The random seed to use. Default is 8525.}
}
\value{
A ggplot object representing the cell velocity plot, with \code{height} and \code{width} attributes set for consistent rendering. If \code{return_layer = TRUE}, returns a list of ggplot layers instead.
}
\description{
Plots RNA velocity vectors on a low-dimensional embedding (e.g., UMAP, t-SNE)
to visualize the direction and magnitude of cellular state transitions. Supports
three visualization modes: raw arrows at each cell position, arrows on a regular
grid, and streamline paths. Optionally colors arrows by cell metadata groups.
}
\section{Rendering Pipeline}{

The \code{VelocityPlot} function proceeds through the following steps:
\enumerate{
\item{\strong{Input validation} --- Verifies that \code{embedding} and \code{v_embedding} are matrices or data frames of equal dimensions, that \code{group_by} matches the number of rows (if provided), and that \code{split_by} is \code{NULL} (unsupported and raises an error).}
\item{\strong{Axis label resolution} --- Uses the column names of \code{embedding} as axis labels, falling back to \code{"Reduction 1"} and \code{"Reduction 2"} when column names are \code{NULL}.}
\item{\strong{Grouping setup} --- Converts \code{group_by} to a factor and applies \code{keep_na} / \code{keep_empty} logic to filter or recode missing values and empty factor levels.}
\item{\strong{Plot-type dispatch} --- Branches on \code{plot_type}:}
\itemize{
\item{\strong{raw} --- Optionally subsamples cells when \code{density < 1}, scales velocity vectors by \code{scale}, computes arrow lengths proportional to the embedding range, and renders \code{\link[ggplot2]{geom_segment}} with arrowheads. When \code{group_by} is provided, arrows are colored by group using \code{group_palette}.}
\item{\strong{grid} --- Delegates to \code{\link{.compute_velocity_on_grid}} to interpolate the sparse cell velocities onto a regular grid, then renders \code{\link[ggplot2]{geom_segment}} with arrowheads at each grid point. \code{group_by} is ignored with a warning.}
\item{\strong{stream} --- Delegates to \code{\link{.compute_velocity_on_grid}} with \code{adjust_for_stream = TRUE}, then renders smooth streamline paths via \code{\link[metR]{geom_streamline}}. When \code{streamline_color} is provided, streamlines use a fixed color with a background stroke; when \code{NULL}, streamlines are colored by velocity magnitude using \code{streamline_palette}. \code{group_by} is ignored with a warning.}
}
\item{\strong{Layer return or plot assembly} --- If \code{return_layer = TRUE}, returns the list of ggplot layers. Otherwise, constructs a full \code{ggplot} object with labels, theme, aspect ratio, legend configuration, and \code{height} / \code{width} attributes via \code{\link{calculate_plot_dimensions}()}.}
}
}

\examples{
\donttest{
data(dim_example)
dim_example$clusters[dim_example$clusters == "Ductal"] <- NA

# Basic velocity plot with group coloring
VelocityPlot(dim_example[, 1:2], dim_example[, 3:4], group_by = dim_example$clusters)

# Handle NA groups with keep_na / keep_empty
VelocityPlot(dim_example[, 1:2], dim_example[, 3:4], group_by = dim_example$clusters,
    keep_na = TRUE, keep_empty = TRUE)
VelocityPlot(dim_example[, 1:2], dim_example[, 3:4], group_by = dim_example$clusters,
    keep_na = TRUE, keep_empty = 'level')
VelocityPlot(dim_example[, 1:2], dim_example[, 3:4], group_by = dim_example$clusters,
    keep_na = TRUE, keep_empty = FALSE)
}
}
\seealso{
\code{\link{DimPlot}} \code{\link{FeatureDimPlot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap-utils.R
\name{join_heatmap_meta}
\alias{join_heatmap_meta}
\title{Join the meta data to the main data frame for heatmap}
\usage{
join_heatmap_meta(data, meta_data, by, cr_split_by, split_by, which)
}
\arguments{
\item{data}{A data frame containing the main data for the heatmap.}

\item{meta_data}{A data frame containing the meta data to be joined.}

\item{by}{A character string specifying the column name in \code{meta_data} to join on.
Either \code{rows_by} or \code{columns_by} should be specified in \code{data}.}

\item{cr_split_by}{A character string specifying the column name in \code{data} to join on.
Either \code{rows_split_by} or \code{columns_split_by} should be specified in \code{data}.}

\item{split_by}{A character string specifying the column name in \code{data} to join on.
Used to split the data into multiple heatmaps.}

\item{which}{A character string specifying whether to join on rows or columns.
Can be either \code{"row"} or \code{"column"}.}
}
\value{
A data frame with the meta data joined to the main data.
}
\description{
Join the meta data to the main data frame for heatmap
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/radarplot.R
\name{RadarPlot}
\alias{RadarPlot}
\alias{SpiderPlot}
\title{Radar plot / Spider plot}
\usage{
RadarPlot(
  data,
  x,
  x_sep = "_",
  group_by = NULL,
  group_by_sep = "_",
  y = NULL,
  group_name = NULL,
  groups = NULL,
  scale_y = c("group", "global", "x", "none"),
  y_min = 0,
  y_max = NULL,
  y_nbreaks = 4,
  bg_color = "grey80",
  bg_alpha = 0.1,
  fill = TRUE,
  linewidth = 1,
  pt_size = 4,
  max_charwidth = 16,
  split_by = NULL,
  split_by_sep = "_",
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  alpha = 0.2,
  aspect.ratio = 1,
  legend.position = waiver(),
  legend.direction = "vertical",
  keep_na = FALSE,
  keep_empty = FALSE,
  title = NULL,
  subtitle = NULL,
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)

SpiderPlot(
  data,
  x,
  x_sep = "_",
  group_by = NULL,
  group_by_sep = "_",
  y = NULL,
  group_name = NULL,
  groups = NULL,
  scale_y = c("group", "global", "x", "none"),
  y_min = 0,
  y_max = NULL,
  y_nbreaks = 4,
  bg_color = "grey80",
  bg_alpha = 0.1,
  fill = TRUE,
  linewidth = 1,
  pt_size = 4,
  max_charwidth = 16,
  split_by = NULL,
  split_by_sep = "_",
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  alpha = 0.2,
  aspect.ratio = 1,
  legend.position = waiver(),
  legend.direction = "vertical",
  keep_na = FALSE,
  keep_empty = FALSE,
  title = NULL,
  subtitle = NULL,
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis.}

\item{x_sep}{A character string used to join multiple \code{x}
columns.  Default \code{"_"}.  Ignored when \code{x} is a single
column.}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{y}{A character string specifying the column name of the data frame to plot for the y-axis.}

\item{group_name}{A character string used as the colour/fill legend
title.  When \code{NULL}, the \code{group_by} column name is used.}

\item{groups}{A character vector of group values (in the
\code{group_by} column) to include in the plot.  When \code{NULL},
all groups are included.  This can control which groups appear and
their legend order.  Implies \code{keep_empty = FALSE} for the
\code{group_by} column: groups not present in the data are not
shown in the legend.}

\item{scale_y}{How should the radial axis be scaled?  Default is
\code{"group"}.  Options are \code{"group"}, \code{"global"},
\code{"x"}, and \code{"none"}.
\itemize{
\item{\code{"group"} — scaled to the fraction within each group.}
\item{\code{"global"} — scaled to the fraction of the total.}
\item{\code{"x"} — scaled to the fraction within each x-axis category.}
\item{\code{"none"} — raw counts or values, no scaling.}
}}

\item{y_min}{A numeric value setting the minimum of the radial
axis.  Default \code{0}.}

\item{y_max}{A numeric value setting the maximum of the radial
axis.  When \code{NULL}, the maximum data value is used.}

\item{y_nbreaks}{A numeric value for the number of breaks
(concentric grid lines) on the radial axis.  Default \code{4}.}

\item{bg_color}{A character string specifying the background fill
colour.  Default \code{"grey80"}.}

\item{bg_alpha}{A numeric value for the transparency of the
background fill.  Default \code{0.1}.}

\item{fill}{A logical value.  When \code{TRUE} (default), the data
polygons are filled with the group colour.  When \code{FALSE},
only outlines are drawn.}

\item{linewidth}{A numeric value for the width of the polygon
outline lines.  Default \code{1}.}

\item{pt_size}{A numeric value for the size of the data point
markers.  Default \code{4}.}

\item{max_charwidth}{A numeric value for the maximum character
width of x-axis labels before wrapping.  Default \code{16}.}

\item{split_by}{The column(s) to split the data by and produce
separate sub-plots.  Multiple columns are concatenated with
\code{split_by_sep}.}

\item{split_by_sep}{A character string to separate concatenated
\code{split_by} columns.  Default \code{"_"}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{seed}{A numeric seed for reproducibility.  Passed to
\code{\link{validate_common_args}()}.}

\item{combine}{Logical; when \code{TRUE} (default), returns a
combined \code{patchwork} object.  When \code{FALSE}, returns a
named list of individual \code{ggplot} objects.}

\item{ncol, nrow}{Integer number of columns / rows for the combined
layout (passed to \code{\link[patchwork]{wrap_plots}}).}

\item{byrow}{Logical; fill the combined layout by row.  Default
\code{TRUE} (passed to \code{\link[patchwork]{wrap_plots}}).}

\item{axes}{A character string specifying how axes should be
treated across the combined layout (passed to
\code{\link[patchwork]{wrap_plots}}).}

\item{axis_titles}{A character string specifying how axis titles
should be treated across the combined layout.  Defaults to
\code{axes}.}

\item{guides}{A character string specifying how guides (legends)
should be collected across panels (passed to
\code{\link{combine_plots}()}).}

\item{design}{A custom layout design for the combined plot (passed
to \code{\link{combine_plots}()}).}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object (when \code{combine = TRUE} and
\code{split_by} is \code{NULL}), a \code{patchwork} object (when
\code{combine = TRUE} and \code{split_by} is provided), or a named
list of \code{ggplot} objects (when \code{combine = FALSE}), each
with \code{height} and \code{width} attributes in inches.

A \code{ggplot} object (when \code{combine = TRUE} and
\code{split_by} is \code{NULL}), a \code{patchwork} object (when
\code{combine = TRUE} and \code{split_by} is provided), or a named
list of \code{ggplot} objects (when \code{combine = FALSE}), each
with \code{height} and \code{width} attributes in inches.
}
\description{
Draws a radar chart (concentric circular grid) or spider chart
(polygonal grid) displaying multivariate data in a two-dimensional
polar coordinate system.  Each x-axis category is placed at an
evenly spaced angular position around the chart, and numeric values
are plotted along the radial axis.

The function supports \strong{count aggregation} (omit \code{y} to
plot observation counts), \strong{proportion scaling} (via
\code{scale_y}), per-group colour control, faceting, and splitting
into separate sub-plots via \code{split_by}.

\code{\link{SpiderPlot}} is an alias that renders the same data with
polygonal grid lines (spider chart style) by using
\code{polygon = TRUE}.

\code{SpiderPlot} is a variant of \code{\link{RadarPlot}} that renders the chart with
straight polygonal grid lines (spider chart) instead of concentric
circles.  Internally, it calls \code{\link{RadarPlotAtomic}} with
\code{polygon = TRUE} but is otherwise identical to
\code{\link{RadarPlot}} in behaviour and parameters.
}
\section{split_by Workflow}{

When \code{split_by} is provided:
\enumerate{
\item \code{\link{check_keep_na}()} and
\code{\link{check_keep_empty}()} normalise the
\code{keep_na} / \code{keep_empty} arguments for all
relevant columns (\code{x}, \code{split_by},
\code{group_by}, \code{facet_by}).
\item The \code{split_by} column is validated and its NA / empty
levels are processed via
\code{\link{process_keep_na_empty}()}.  It is then removed
from the per-column \code{keep_na} / \code{keep_empty}
lists.
\item The data frame is split by \code{split_by} (preserving
level order).  If \code{split_by} is \code{NULL}, the data
is wrapped in a single-element list with name \code{"..."}.
\item Per-split \code{palette}, \code{palcolor},
\code{legend.position}, and \code{legend.direction} are
resolved via \code{\link{check_palette}()},
\code{\link{check_palcolor}()}, and
\code{\link{check_legend}()}.
\item \code{\link{RadarPlotAtomic}()} is called for each split
with \code{polygon = FALSE}.  If \code{title} is a
function, it receives the split level name and can generate
dynamic titles.
\item Results are combined via \code{\link{combine_plots}()}
(when \code{combine = TRUE}) or returned as a named list.
}
}

\examples{
\donttest{
set.seed(8525)

# --- Radar chart with observation counts ---
data <- data.frame(
    x = factor(
        c(rep("A", 20), rep("B", 30), rep(NA, 30), rep("D", 40), rep("E", 50)),
        levels = LETTERS[1:5]
    ),
    group = factor(
        sample(c("G1", NA, "G3", "G4"), 170, replace = TRUE),
        levels = c("G1", "G2", "G3", "G4")
    )
)

# Basic radar chart
RadarPlot(data, x = "x")

# Keep NA and empty factor levels
RadarPlot(data, x = "x", keep_na = TRUE, keep_empty = TRUE)

# Custom background colour
RadarPlot(data, x = "x", bg_color = "lightpink")

# Raw counts (no proportion scaling)
RadarPlot(data, x = "x", scale_y = "none")

# Grouped by a variable
RadarPlot(data, x = "x", group_by = "group", keep_na = TRUE)

# Faceted by a variable
RadarPlot(data, x = "x", facet_by = "group")

# Spider chart variant (polygonal grid)
SpiderPlot(data, x = "x")
SpiderPlot(data, x = "x", group_by = "group")

# --- Radar chart with explicit y values ---
data <- data.frame(
    x = rep(LETTERS[1:5], 2),
    y = c(1, 3, 6, 4, 2, 5, 7, 8, 9, 10),
    group = rep(c("G1", "G2"), each = 5)
)

# Grouped radar with raw values
RadarPlot(data, x = "x", y = "y", scale_y = "none", group_by = "group")

# Faceted radar
RadarPlot(data, x = "x", y = "y", facet_by = "group")

# Split into separate sub-plots
RadarPlot(data, x = "x", y = "y", split_by = "group")

# Per-split palettes
RadarPlot(data, x = "x", y = "y", split_by = "group",
          palette = c(G1 = "Set1", G2 = "Paired"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsea.R
\name{gsea_running_score}
\alias{gsea_running_score}
\title{Get the running enrichment score of a gene set}
\usage{
gsea_running_score(genes, gene_ranks, exponent = 1, hits_only = TRUE)
}
\arguments{
\item{genes}{A vector of genes}

\item{gene_ranks}{A numeric vector of gene ranks with names}

\item{exponent}{A numeric value to raise the gene ranks to}

\item{hits_only}{A logical value to return only the running enrichment score of the hits}
}
\value{
A numeric vector of the running enrichment score
}
\description{
Get the running enrichment score of a gene set
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrich.R
\name{adjust_network_layout}
\alias{adjust_network_layout}
\title{Adjust_network_layout}
\usage{
adjust_network_layout(
  graph,
  layout,
  width,
  height = 2,
  scale = 100,
  iter = 100
)
}
\description{
Adjust_network_layout
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsea.R
\name{prepare_fgsea_result}
\alias{prepare_fgsea_result}
\title{Prepare fgsea result for plotting}
\usage{
prepare_fgsea_result(data)
}
\arguments{
\item{data}{A data frame of fgsea results}
}
\value{
A data frame with the desired columns for plotting and the gene ranks and gene sets as attributes
}
\description{
Prepare fgsea result for plotting
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/upsetplot.R
\name{UpsetPlot}
\alias{UpsetPlot}
\title{UpSet Plot}
\usage{
UpsetPlot(
  data,
  in_form = c("auto", "long", "wide", "list", "upset"),
  split_by = NULL,
  split_by_sep = "_",
  group_by = NULL,
  group_by_sep = "_",
  id_by = NULL,
  label = TRUE,
  label_fg = "black",
  label_size = NULL,
  label_bg = "white",
  label_bg_r = 0.1,
  palette = "Blues",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  specific = TRUE,
  theme = "theme_this",
  theme_args = list(),
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  aspect.ratio = 0.6,
  legend.position = "right",
  legend.direction = "vertical",
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  seed = 8525,
  combmatrix_gap = 6,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{in_form}{A character string specifying the input format.  One of
\code{"auto"} (default; detect from \code{data} structure),
\code{"long"}, \code{"wide"}, \code{"list"}, or \code{"upset"}.}

\item{split_by}{The column(s) to split the data by and produce separate
sub-plots.  Only supported for \code{data.frame} input (list input raises
an error).  Multiple columns concatenated with \code{split_by_sep}.}

\item{split_by_sep}{A character string to separate concatenated
\code{split_by} columns.  Default: \code{"_"}.}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{id_by}{A character string specifying the column name for instance
identifiers.  Required for long format; optional for wide format (a
synthetic \code{.id} column is created if omitted).}

\item{label}{A logical value.  When \code{TRUE} (default), count labels are
displayed above each intersection bar via \code{\link[ggrepel]{geom_text_repel}()}.}

\item{label_fg}{A character string specifying the colour of the label text.
Default: \code{"black"}.}

\item{label_size}{A numeric value specifying the size of the label text.
Default: \code{NULL} (computed from \code{base_size / 12 * 3.5}).}

\item{label_bg}{A character string specifying the background fill colour of
the label.  Default: \code{"white"}.}

\item{label_bg_r}{A numeric value specifying the corner radius of the label
background, passed to \code{geom_text_repel(bg.r)}.  Default: \code{0.1}.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{specific}{A logical value.  When \code{TRUE} (default), only specific
intersections are returned (elements belonging exclusively to the shown
set combination).  When \code{FALSE}, all overlapping items are included.
See \url{https://github.com/gaospecial/ggVennDiagram/issues/64}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{combine}{Logical; when \code{TRUE} (default), returns a combined
\code{patchwork} object.  When \code{FALSE}, returns a named list of
individual \code{ggplot} objects.}

\item{ncol, nrow}{Integer number of columns / rows for the combined layout
(passed to \code{\link[patchwork]{wrap_plots}}).}

\item{byrow}{Logical; fill the combined layout by row.  Default \code{TRUE}
(passed to \code{\link[patchwork]{wrap_plots}}).}

\item{seed}{A numeric seed for reproducibility.  Passed to
\code{\link{validate_common_args}()}.  Default: \code{8525}.}

\item{combmatrix_gap}{A numeric value specifying the gap between rows of the
combination matrix, measured at \code{base_size = 12}.  The actual gap is
scaled by \code{text_size_scale = base_size / 12}.  Default: \code{6}.}

\item{axes}{A character string specifying how axes should be treated across
the combined layout (passed to \code{\link[patchwork]{wrap_plots}}).}

\item{axis_titles}{A character string specifying how axis titles should be
treated across the combined layout.  Defaults to \code{axes}.}

\item{guides}{A character string specifying how legends should be collected
across panels (passed to \code{\link{combine_plots}()}).}

\item{design}{A custom layout design for the combined plot (passed to
\code{\link{combine_plots}()}).}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object, a \code{patchwork} object, or a named list
of \code{ggplot} objects (when \code{combine = FALSE}), each with
\code{height} and \code{width} attributes in inches.
}
\description{
Draws an UpSet plot visualising set intersections and set sizes.  The plot
comprises:
\itemize{
\item A \strong{horizontal bar chart} showing the size of each
intersection, filled by the intersection count.
\item A \strong{combination matrix} (rows = sets, columns = intersections)
with membership dots and connecting lines.
\item A \strong{set-size bar chart} on the left of the matrix (added
automatically by \code{ggupset}).
}

The function accepts data in four formats:
\itemize{
\item \strong{List} --- a named list of element vectors (one per set).
\item \strong{Long} --- a data frame with one row per (set, element) pair.
\item \strong{Wide} --- a data frame where each row is an element and each
set has its own logical or 0/1 membership column.
\item \strong{UpsetPlotData} --- a pre-processed object from
\code{\link{prepare_upset_data}()}.
}

Supports splitting into sub-plots via \code{split_by}, per-split colour
palettes and legend control, and combining sub-plots via \code{patchwork}.
}
\section{split_by Workflow}{

When \code{split_by} is provided:
\enumerate{
\item \strong{Guard} --- \code{split_by} is only supported for
\code{data.frame} input.  If \code{data} is a list (or other
non-data.frame type) and \code{split_by} is non-\code{NULL}, an
error is raised.
\item \strong{Column validation} --- \code{\link{check_columns}()} resolves
the \code{split_by} column(s) with \code{force_factor = TRUE} and
\code{allow_multi = TRUE}.  Multiple columns are concatenated with
\code{split_by_sep}.
\item \strong{Data splitting} --- empty factor levels in \code{split_by}
are dropped via \code{\link[base]{droplevels}()}, then the data
frame is split by \code{split_by} (level order is preserved).  If
\code{split_by} is \code{NULL}, the data is wrapped in a
single-element list with name \code{"..."}.
\item \strong{Per-split resolution} --- \code{\link{check_palette}()},
\code{\link{check_palcolor}()}, and \code{\link{check_legend}()}
resolve per-split \code{palette}, \code{palcolor},
\code{legend.position}, and \code{legend.direction}.
\item \strong{Atomic dispatch} --- \code{UpsetPlotAtomic} is called for
each split.  If \code{title} is a function, it receives the split
level name and can generate dynamic titles.  When in wide mode
(\code{in_form} is \code{"auto"} or \code{"wide"}) and
\code{group_by} is \code{NULL}, the set columns are auto-detected
as all columns except \code{id_by} and \code{split_by}.
\item \strong{Combination} --- Results are combined via
\code{\link{combine_plots}()} (when \code{combine = TRUE}) or
returned as a named list.
}
}

\examples{
\donttest{
# ---- list input -------------------------------------------------------
data <- list(
    A = 1:5,
    B = 2:6,
    C = 3:7,
    D = 4:8
)
UpsetPlot(data)
UpsetPlot(data, label = FALSE)
UpsetPlot(data, palette = "Reds", specific = FALSE)

# ---- long-format data frame ------------------------------------------
data_long <- data.frame(
    group_by = factor(
         c(rep("A", 5), rep("B", 5), rep("C", 5), rep("D", 5)),
         levels = c("A", "B", "C", "D")
    ),
    id_by = c(1:5, 2:6, 3:7, 4:8)
)
UpsetPlot(data_long, in_form = "long", group_by = "group_by", id_by = "id_by")

# ---- wide-format data frame ------------------------------------------
data <- data.frame(
    id = LETTERS[1:10],
    B = c(1, 0, 1, 1, 0, 0, 1, 0, 1, 0),
    A = c(1, 1, 1, 0, 0, 1, 0, 0, 1, 0),
    D = c(1, 0, 0, 1, 1, 0, 0, 1, 0, 1),
    C = c(0, 1, 1, 0, 1, 0, 1, 0, 1, 0)
)
UpsetPlot(data, in_form = "wide", id_by = "id", n_intersections = 4)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{process_keep_na_empty}
\alias{process_keep_na_empty}
\title{process_keep_na_empty
Process keep_na and keep_empty to data}
\usage{
process_keep_na_empty(data, keep_na = NULL, keep_empty = NULL, col = NULL)
}
\arguments{
\item{data}{data frame}

\item{keep_na}{List of keep_na}

\item{keep_empty}{List of keep_empty}
}
\value{
processed data frame
}
\description{
process_keep_na_empty
Process keep_na and keep_empty to data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wordcloudplot.R
\name{WordCloudPlotAtomic}
\alias{WordCloudPlotAtomic}
\title{Word cloud plot (internal)}
\usage{
WordCloudPlotAtomic(
  data,
  word_by = NULL,
  sentence_by = NULL,
  count_by = NULL,
  score_by = NULL,
  count_name = NULL,
  score_name = NULL,
  words_excluded = plotthis::words_excluded,
  score_agg = mean,
  minchar = 2,
  word_size = c(2, 8),
  top_words = 100,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  alpha = 1,
  palreverse = FALSE,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  seed = 8525,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{word_by}{A character string specifying the column name containing
pre-tokenized words. A character column is expected. Use this when your
data already has one word per row (or a list of words per row). Mutually
exclusive with \code{sentence_by}.}

\item{sentence_by}{A character string specifying the column name containing
sentences or phrases to be split into individual words. A character column
is expected. The text is lowercased and punctuation is removed before
splitting on whitespace boundaries. Mutually exclusive with
\code{word_by}.}

\item{count_by}{A character string specifying the numeric column for the
count or frequency of each word. When \code{NULL} (the default), each
occurrence counts as 1. Must be \code{NULL} when \code{sentence_by} is
used, as counts are derived from the number of occurrences after splitting.}

\item{score_by}{A character string specifying the numeric column for the
score of each word, mapped to the text colour via a continuous gradient.
When \code{NULL} (the default), all words receive a score of 1 and are
coloured at the low end of the palette.}

\item{count_name}{A character string for the size legend title. When
\code{NULL} (the default), \code{"Count"} is used.}

\item{score_name}{A character string for the colour-bar legend title. When
\code{NULL} (the default), \code{"Score"} is used.}

\item{words_excluded}{A character vector of words to exclude from the word
cloud. Matching is case-insensitive. Defaults to
\code{plotthis::words_excluded}, a built-in set of common English stop
words.}

\item{score_agg}{A function to aggregate the scores when multiple
observations of the same word exist. Default is \code{mean}. Other
options include \code{sum}, \code{median}, or a custom function.}

\item{minchar}{A numeric value specifying the minimum number of characters
a word must have to be included. Words with fewer characters are filtered
out. Default: \code{2}.}

\item{word_size}{A numeric vector of length 2 specifying the range of font
sizes (in mm) for the words. Passed to \code{scale_size(range =
 word_size)}. Default: \code{c(2, 8)}.}

\item{top_words}{A numeric value specifying the maximum number of words to
display, selected by highest score. Default: \code{100}.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{seed}{The random seed to use. Default is 8525.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a word cloud plot. This is the workhorse
behind the exported \code{\link{WordCloudPlot}} function — it takes a
\strong{single} data frame (no \code{split_by} support) and returns a
\code{ggplot} object rendered via
\code{\link[ggwordcloud]{geom_text_wordcloud}}.

The function supports two input modes:
\itemize{
\item \strong{Pre-tokenized words} (\code{word_by}) — each row contains
a single word (or multiple words in a list column that is unnested).
\item \strong{Sentence splitting} (\code{sentence_by}) — each row
contains a sentence or phrase; the function splits the text into
individual words after removing punctuation and lowercasing.
}

Words are displayed with \strong{font size} proportional to a count
variable (\code{count_by}) and \strong{colour} based on a continuous
score variable (\code{score_by}). Text filtering options allow removal of
short words, common stop words, and bracketed patterns.
}
\section{Architecture}{

\enumerate{
\item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{Input validation} — ensures exactly one of \code{word_by}
or \code{sentence_by} is specified. Errors if both are provided
or neither is provided. When \code{sentence_by} is used,
\code{count_by} must be \code{NULL} (counts are derived from
occurrences after splitting).
\item \strong{Column resolution} — \code{facet_by}, \code{count_by},
and \code{score_by} are validated and transformed via
\code{\link{check_columns}}.
\item \strong{Default values} — when \code{score_by} is \code{NULL}, a
synthetic \code{.score} column with value \code{1} is created.
When \code{count_by} is \code{NULL}, a synthetic \code{.count}
column with value \code{1} is created.
\item \strong{Sentence splitting branch} (when \code{sentence_by} is set):
\itemize{
\item The \code{sentence_by} column is validated via
\code{\link{check_columns}}.
\item Text is lowercased, punctuation is removed, and the
string is split into individual words on whitespace
boundaries.
\item Words are unnested into separate rows via
\code{\link[tidyr]{unnest}}.
\item Data is grouped by word (and \code{facet_by} columns
when present) and aggregated: \code{sum(count_by)} and
\code{score_agg(score_by)}. Separate aggregation
templates handle 0, 1, or 2 \code{facet_by} columns.
}
\item \strong{Word branch} (when \code{word_by} is set):
\itemize{
\item The \code{word_by} column is validated via
\code{\link{check_columns}}.
\item Column values are unnested into separate rows (supports
list columns where a row may contain multiple words).
\item Data is grouped by word (and \code{facet_by} columns
when present) and aggregated similarly to the sentence
branch.
}
\item \strong{Text filtering pipeline} — the aggregated data is
processed sequentially:
\itemize{
\item Words matching the pattern \verb{[.*]} (bracketed
content) are removed.
\item Words with fewer than \code{minchar} characters are
removed.
\item Words listed in \code{words_excluded} are removed
(case-insensitive matching).
\item Duplicate rows are removed.
\item The top \code{top_words} words by score are retained
via \code{\link[dplyr]{slice_max}}.
}
\item \strong{Angle assignment} — each word is randomly assigned a
rotation angle of 0 (60\\% probability) or 90 degrees (40\\%
probability), creating the characteristic word cloud appearance.
\item \strong{Colour scale preparation} —
\code{\link{palette_this}()} with \code{type = "continuous"}
generates a smooth gradient for the \code{score} variable. A
\code{colors_value} sequence is created spanning
\code{min(score)} to \code{quantile(score, 0.99)} for colour
interpolation.
\item \strong{Plot assembly} — the \code{ggplot} object is built with:
\itemize{
\item \code{ggwordcloud::geom_text_wordcloud()} renders the
words with \code{rm_outside = TRUE}, square shape, and
eccentricity of 1.
\item \code{scale_color_gradientn()} maps the score to the
continuous colour gradient, with a colour-bar legend
(titled by \code{score_name} or \code{"Score"}).
\item \code{scale_size()} maps the count to font size within
the \code{word_size} range, with a size legend (titled
by \code{count_name} or \code{"Count"}).
\item \code{coord_flip()} swaps the axes (word cloud
convention).
\item \code{do_call(theme, theme_args)} applies the selected
theme; \code{aspect.ratio}, \code{legend.position}, and
\code{legend.direction} are set directly.
}
\item \strong{Dimension calculation} —
\code{\link{calculate_plot_dimensions}()} computes plot height
and width using \code{base_height = 4.5}, \code{aspect.ratio},
and legend metrics. The resulting \code{height} / \code{width}
attributes are stored on the \code{ggplot} object.
\item \strong{Faceting} — \code{\link{facet_plot}()} wraps the plot
with \code{facet_wrap} or \code{facet_grid} if \code{facet_by}
is provided, up to a maximum of 2 facet columns.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_palette}
\alias{check_palette}
\title{check_palette
Check if the palette can be properly used}
\usage{
check_palette(palette, datas_name)
}
\arguments{
\item{palette}{palette}

\item{datas_name}{names of the split data}
}
\value{
named list containing palette names
}
\description{
check_palette
Check if the palette can be properly used
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap-utils.R
\name{.setup_annos}
\alias{.setup_annos}
\title{Unified annotation builder for built-in (split/name) and user-defined annotations}
\usage{
.setup_annos(
  which,
  names_side,
  anno_title,
  show_names,
  annotation,
  annotation_type,
  annotation_side,
  annotation_palette,
  annotation_palcolor,
  annotation_agg,
  annotation_params,
  annotation_name,
  split_by,
  splits,
  by,
  by_labels,
  flip,
  legend.direction,
  legend.position,
  data
)
}
\arguments{
\item{which}{The annotation direction ("row" or "column")}

\item{names_side}{The side to place the row/column name annotation ("top", "bottom", "left", "right")}

\item{anno_title}{The title of the annotation (for split)}

\item{show_names}{A logical value indicating whether to show row/column names}

\item{annotation}{A list of user-defined annotations, where names are annotation names and values are annotation objects or parameters to build annotation objects}

\item{annotation_type}{A list of annotation types, where names are annotation names and values are annotation types ("simple", "label", "rownames", "colnames", "block", "ggcat", "ggseries", or "auto")}

\item{annotation_side}{A list of annotation sides, where names are annotation names and values are annotation sides ("top", "bottom", "left", "right")}

\item{annotation_palette}{A list of annotation palettes, where names are annotation names and values are palette names or color vectors}

\item{annotation_palcolor}{A list of annotation palette colors, where names are annotation names and values are color vectors to override the palette}

\item{annotation_agg}{A list of functions to aggregate the original data for each annotation, where names are annotation names and values are functions that take a vector of values in the cell and return an aggregated value}

\item{annotation_params}{A list of additional parameters for each annotation, where names are annotation names and values are lists of parameters to pass to the annotation constructor}

\item{annotation_name}{A list of display names for each annotation, where names are annotation names and values are character strings (aliasing the displayed name and legend title) or \code{FALSE} (hiding the displayed name)}

\item{split_by}{The name of the column used for split annotation}

\item{splits}{A factor vector of splits for the split annotation}

\item{by}{The name of the column used for name annotation}

\item{by_labels}{A factor vector of labels for the name annotation}

\item{flip}{A logical value indicating whether to flip the annotation (for ggseries annotations)}

\item{legend.direction}{The direction of the legend ("vertical" or "horizontal")}

\item{legend.position}{The position of the legend ("right", "left", "top", "bottom")}

\item{data}{A data frame used for ggcat and ggseries annotations, where each row corresponds to a cell in the heatmap and contains the original values before aggregation}
}
\value{
A list of annotations and legends to be passed to ComplexHeatmap.
}
\description{
Unified annotation builder for built-in (split/name) and user-defined annotations
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manhattanplot.R
\name{ManhattanPlotAtomic}
\alias{ManhattanPlotAtomic}
\title{Atomic Manhattan plot (internal)}
\usage{
ManhattanPlotAtomic(
  data,
  chr_by,
  pos_by,
  pval_by,
  label_by = NULL,
  chromosomes = NULL,
  pt_size = 0.75,
  pt_color = NULL,
  pt_alpha = alpha,
  pt_shape = 19,
  label_size = 3,
  label_fg = NULL,
  highlight = NULL,
  highlight_color = NULL,
  highlight_size = 1.5,
  highlight_alpha = 1,
  highlight_shape = 19,
  preserve_position = TRUE,
  chr_gap_scaling = 1,
  pval_transform = "-log10",
  signif = c(5e-08, 1e-05),
  signif_color = NULL,
  signif_rel_pos = 0.2,
  signif_label = TRUE,
  signif_label_size = 3.5,
  signif_label_pos = c("left", "right"),
  thin = NULL,
  thin_n = 1000,
  thin_bins = 200,
  rescale = TRUE,
  rescale_ratio_threshold = 5,
  palette = "Dark2",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  theme = "theme_this",
  theme_args = list(),
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = expression("-" * log[10](p)),
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{chr_by}{A character string specifying the column name for chromosome
identifiers.  Default: \code{"chr"}.}

\item{pos_by}{A character string specifying the column name for genomic
positions (integer or numeric).  Default: \code{"pos"}.}

\item{pval_by}{A character string specifying the column name for p-values
(numeric).  Default: \code{"pval"}.}

\item{label_by}{A character string specifying the column name for variant
labels.  Only variants with non-empty values in this column will be
labelled.  Default: \code{NULL} (no labels).}

\item{chromosomes}{A character or numeric vector specifying which
chromosomes to include and/or their display order.  When \code{NULL}
(the default), all chromosomes present in the data are plotted in
their natural factor order.  A single value filters to that
chromosome; a vector reorders and subsets.}

\item{pt_size}{A numeric value specifying the size of the points.
Default: \code{0.75}.}

\item{pt_color}{A character string specifying a single colour for all
background (non-highlighted) points.  When \code{NULL} (the default),
alternating chromosome colours from \code{palette} / \code{palcolor}
are used.  Typically set to \code{"grey80"} when \code{highlight} is
used with a distinct \code{highlight_color}.}

\item{pt_alpha}{A numeric value in \code{[0, 1]} specifying the
transparency of the points.  Default: \code{alpha} (aliased
parameter).}

\item{pt_shape}{A numeric value specifying the shape of the points.
Default: \code{19} (filled circle).}

\item{label_size}{A numeric value specifying the font size of the
variant labels.  Default: \code{3}.}

\item{label_fg}{A character string specifying the colour of the variant
labels.  When \code{NULL} (the default), each label inherits the
colour of its corresponding point.}

\item{highlight}{Either a numeric vector of row indices or a character
string containing an R expression (parsed via
\code{rlang::parse_expr()}) to select variants to highlight.
Default: \code{NULL} (no highlighting).}

\item{highlight_color}{A character string specifying the colour of
highlighted points.  When \code{NULL} (the default), highlighted
points inherit the chromosome colour from the underlying
\code{geom_point()} layer.}

\item{highlight_size}{A numeric value specifying the size of
highlighted points.  Default: \code{1.5}.}

\item{highlight_alpha}{A numeric value in \code{[0, 1]} specifying the
transparency of highlighted points.  Default: \code{1}.}

\item{highlight_shape}{A numeric value specifying the shape of
highlighted points.  Default: \code{19} (filled circle).}

\item{preserve_position}{A logical value.  When \code{TRUE} (the
default), the width of each chromosome segment reflects its number
of variants and variant positions are correctly scaled.  When
\code{FALSE}, all chromosomes have equal width and variants are
equally spaced.}

\item{chr_gap_scaling}{A numeric scaling factor for the gap between
chromosomes.  Larger values increase the gap.  Default: \code{1}.}

\item{pval_transform}{A function or character string that can be
evaluated to a function for transforming p-values.  Default:
\code{"-log10"}, which computes \eqn{-\log_{10}(p)}.  Other
examples: \code{"-log2"} or a custom
\code{function(x) -log10(x)}.}

\item{signif}{A numeric vector of significance thresholds to draw as
horizontal dashed lines.  Default: \code{c(5e-8, 1e-5)}.}

\item{signif_color}{A character vector of colours for the significance
threshold lines, of equal length as \code{signif}.  When \code{NULL}
(the default), the smallest threshold is coloured black and the rest
grey.}

\item{signif_rel_pos}{A numeric value between \code{0.1} and \code{0.9}
specifying the relative position of the y-axis jump when rescaling
is active.  Default: \code{0.2}.}

\item{signif_label}{A logical value.  When \code{TRUE} (the default),
significance threshold values are annotated on the plot.}

\item{signif_label_size}{A numeric value for the font size of the
significance threshold labels.  Default: \code{3.5}.}

\item{signif_label_pos}{A character string specifying where to place
the significance threshold labels: \code{"left"} (default) or
\code{"right"}.}

\item{thin}{A logical value indicating whether to thin dense data by
sampling points per horizontal partition.  Defaults to \code{TRUE}
when \code{chromosomes} selects fewer chromosomes than in the data,
and \code{FALSE} otherwise.}

\item{thin_n}{An integer specifying the maximum number of points per
horizontal partition after thinning.  Default: \code{1000}.}

\item{thin_bins}{An integer specifying the number of horizontal bins
for thinning.  Default: \code{200}.}

\item{rescale}{A logical value.  When \code{TRUE} (the default), the
y-axis is automatically rescaled (broken axis) if extreme
significance values would otherwise compress the main data cloud.}

\item{rescale_ratio_threshold}{A numeric threshold for triggering
y-axis rescaling.  The ratio is computed as
\code{ceiling(max(log10pval) / 5) * 5 / signif_jump}.
Default: \code{5}.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a GWAS-style Manhattan plot.  This is the
internal workhorse dispatched by the exported \code{\link{ManhattanPlot}}
function --- it takes a \strong{single} data frame (no \code{split_by} support)
and returns a \code{ggplot} object.  The plot displays genetic association
p-values across chromosomes, with \eqn{-\log_{10}(p)} on the y-axis and
genomic position on the x-axis.  Each chromosome is rendered in alternating
colours, and configurable horizontal dashed lines mark genome-wide
significance thresholds.

The function is adapted from \code{ggmanh::manhattan_plot()} with the
following enhancements:
\itemize{
\item Dot-separated argument names are converted to underscores
(e.g. \code{chr.colname} \eqn{\rightarrow} \code{chr_by}).
\item \code{chromosomes} merges the original \code{chromosome} and
\code{chr.order} arguments into a single parameter for subsetting
and reordering.
\item \code{highlight} accepts index vectors or R expressions (via a
character string) instead of a column name.
\item Dedicated \code{pt_*}, \code{label_*}, and \code{highlight_*}
parameter families give granular control over point appearance,
label styling, and highlight styling.
\item \code{pval_transform} accepts any function (or a character
string parsed as a function) rather than a fixed log-transform toggle.
}

Key features include per-chromosome alternating colours, configurable
significance threshold lines with labels, optional data thinning for
dense SNP sets, automatic y-axis rescaling (broken-axis) when a small
number of highly significant points would otherwise compress the
majority of the data, and support for highlighting and labeling
specific variants.
}
\section{Architecture}{


\strong{ManhattanPlotAtomic} executes the following steps:
\enumerate{
\item \strong{ggplot dispatch} --- selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{signif_label_pos normalisation} --- \code{match.arg()}
resolves \code{signif_label_pos} to \code{"left"} or \code{"right"}.
\item \strong{Data preprocessing} ---
\code{ggmanh::manhattan_data_preprocess()} performs chromosome
ordering, optional thinning (\code{thin_n} points per
\code{thin_bins} horizontal partitions), chromosome gap scaling
(\code{chr_gap_scaling}), position preservation
(\code{preserve_position}), and significance threshold colour
assignment.  When \code{chromosomes} is a single value, it is
passed as the \code{chromosome} filter; otherwise it is used for
ordering via \code{chr.order}.
\item \strong{label_by validation} --- \code{\link{check_columns}()}
validates and factors the \code{label_by} column if provided.
\item \strong{Chromosome colour assignment} --- if \code{highlight}
and \code{highlight_color} are both set, or if \code{pt_color} is
given, \code{pt_color} (defaulting to \code{"grey80"}) is used as
the base colour for all chromosomes; otherwise
\code{\link{palette_this}()} generates distinct colours per
chromosome from \code{palette} / \code{palcolor}.
\item \strong{pval_transform resolution} --- when a character string
starting with \code{"-"} (e.g. \code{"-log10"}), the minus sign is
stripped, the remainder is evaluated as a function, and the result
is negated.  Plain character strings are evaluated directly.  The
result is applied to the p-value column to produce \code{log10pval}.
\item \strong{Y-axis rescaling check} --- when \code{rescale = TRUE},
the ratio of the ceiling-scaled maximum to the significance-threshold
jump is checked against \code{rescale_ratio_threshold}.  If the
ratio exceeds the threshold, a broken y-axis is constructed via
\code{ggmanh::get_transform()} that compresses the empty space
between the main data and the extreme points.
\item \strong{Single-chromosome branch} --- when only one chromosome
is present, the original position column is used directly, x-axis
breaks and labels use \code{waiver()}, and the x-axis label is set
to the chromosome name (or \code{"Chromosome <name>"}).
\item \strong{Multi-chromosome branch} --- position coordinates are
recalculated across chromosome boundaries via
\code{ggmanh::calc_new_pos_()}.  Chromosome centre positions serve
as x-axis breaks, chromosome labels are displayed, and limits span
the full genomic range.
\item \strong{Base plot assembly} --- creates a \code{ggplot} object
with \code{geom_point()} (chromosome-coloured dots),
\code{scale_color_manual()}, \code{scale_y_continuous()} (with
optional broken-axis transform), \code{scale_x_continuous()},
\code{geom_hline()} for significance thresholds, the resolved theme
(with hidden grid lines and suppressed legend), and \code{labs()}.
\item \strong{Significance labels} --- when \code{signif_label = TRUE},
\code{geom_text()} annotates each significance threshold at the
left or right edge of the plot (controlled by
\code{signif_label_pos}), coloured by \code{signif_color} (smallest
threshold in black, others in grey by default).
\item \strong{Variant labels} --- when \code{label_by} is provided,
\code{ggrepel::geom_label_repel()} adds labels for variants that
have non-empty values in the \code{label_by} column.  If \code{label_fg}
is set, all labels receive that colour; otherwise each label inherits
the chromosome colour of its point.
\item \strong{Rescale tick marks} --- when y-axis rescaling is
active, the axis tick style is changed (via \code{axis.ticks.y})
and a double-equals annotation is placed at the jump point to
indicate the axis break, with \code{coord_cartesian(clip = "off")}.
\item \strong{Highlight overlay} --- when \code{highlight} is
numeric (row indices) or a character string (R expression), the
matching points are overlaid with a separate \code{geom_point()}
layer styled by \code{highlight_*} parameters, optionally in a
distinct colour when \code{highlight_color} is specified.
\item \strong{Dimension calculation} ---
\code{\link{calculate_plot_dimensions}()} computes height and width
from the number of chromosomes (\code{base_height = 4.5},
\code{x_scale_factor = 0.4}), and stores them as \code{height} /
\code{width} attributes on the plot.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/venndiagram.R
\name{VennDiagramAtomic}
\alias{VennDiagramAtomic}
\title{Atomic Venn / Euler diagram (internal)}
\usage{
VennDiagramAtomic(
  data,
  in_form = "auto",
  group_by = NULL,
  group_by_sep = "_",
  id_by = NULL,
  label = "count",
  label_fg = "black",
  label_size = NULL,
  label_bg = "white",
  label_bg_r = 0.1,
  fill_mode = "count",
  palreverse = FALSE,
  fill_name = NULL,
  aspect.ratio = 1,
  palette = ifelse(fill_mode == "set", "Paired", "Spectral"),
  palcolor = NULL,
  alpha = 1,
  theme = "theme_this",
  theme_args = list(),
  title = NULL,
  subtitle = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{in_form}{A character string specifying the input format.  One of
\code{"auto"} (default; detect automatically via
\code{\link{detect_venn_datatype}()}), \code{"long"}, \code{"wide"},
\code{"list"}, or \code{"venn"}.}

\item{group_by}{A character string (or vector) specifying the column name(s)
that define the set membership.  For \code{in_form = "long"}, this is the
grouping column; for \code{in_form = "wide"}, these are the set columns
(must be logical or 0/1); when \code{NULL} and \code{data} is a data
frame, all columns are treated as sets (wide format).}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{id_by}{A character string specifying the column name that identifies
individual elements.  Required for long-format data; ignored otherwise.}

\item{label}{A character string or function controlling the text shown in
each intersection region.  One of:
\itemize{
\item \code{"count"} (default) — the raw count of elements in that region.
\item \code{"percent"} — the percentage of the total element count.
\item \code{"both"} — count and percentage on separate lines.
\item \code{"none"} — no region labels are drawn.
\item A \strong{function} — receives a data frame with columns
\code{"id"}, \code{"X"}, \code{"Y"}, \code{"name"}, \code{"item"},
and \code{"count"}, and must return a character vector of labels.
}}

\item{label_fg}{A character string specifying the colour of the label text.}

\item{label_size}{A numeric value specifying the font size of the label
text.  When \code{NULL} (the default), auto-sized at 3.5 for region labels
and 4 for set labels, scaled by \code{base_size / 12}.}

\item{label_bg}{A character string specifying the background colour of the
label text (passed to \code{\link[ggrepel]{geom_text_repel}()} as
\code{bg.color}).  Default \code{"white"}.}

\item{label_bg_r}{A numeric value specifying the corner radius of the label
background rectangle (passed as \code{bg.r}).  Default \code{0.1}.}

\item{fill_mode}{A character string specifying how intersection regions are
coloured.  One of:
\itemize{
\item \code{"count"} — continuous gradient based on element count
(default palette: \code{"Spectral"}).
\item \code{"count_rev"} — continuous gradient with reversed count order.
\item \code{"set"} — discrete blended colours per set combination
(default palette: \code{"Paired"}).  No legend is drawn.
}}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{fill_name}{A character string for the colour bar legend title when
\code{fill_mode} is \code{"count"} or \code{"count_rev"}.  Ignored when
\code{fill_mode = "set"}.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a Venn or Euler diagram that visualises
the overlap relationships among multiple sets.  Supports four input formats
(long, wide, list, and pre-computed \code{VennPlotData}) which are
normalised by \code{\link{prepare_venn_data}()} into the internal
\code{VennPlotData} representation used by \pkg{ggVennDiagram}.

Intersection regions can be filled either by a continuous colour gradient
based on the element count (\code{fill_mode = "count"} or \code{"count_rev"})
or by blended set colours (\code{fill_mode = "set"}).  Region labels can
display the raw count, the percentage of total elements, both, nothing, or
a custom function result.  Set labels always show the set name and its
total element count.
}
\section{Architecture}{

\enumerate{
\item \strong{Data type detection} — \code{\link{detect_venn_datatype}()}
identifies whether \code{data} is in long, wide, list, or
\code{VennPlotData} format.
\item \strong{Data preparation} — \code{\link{prepare_venn_data}()}
converts the input into a \code{VennPlotData} object suitable for
rendering by \pkg{ggVennDiagram}.
\item \strong{Geometry extraction} — \pkg{ggVennDiagram} utilities
(\code{venn_regionedge()}, \code{venn_setedge()},
\code{venn_regionlabel()}, \code{venn_setlabel()}) compute polygon
vertices and label positions for all sets and their intersections.
\item \strong{Fill resolution} — When \code{fill_mode = "set"},
\code{\link{palette_this}()} assigns colours to each set; the
\code{blend_colors()} helper blends the colours of overlapping sets
for each intersection region.  When \code{fill_mode = "count"} or
\code{"count_rev"}, a continuous gradient is applied across regions
via \code{scale_fill_gradientn()} with a colour bar legend.
\item \strong{Label computation} — Region labels are formatted per the
\code{label} parameter: raw count, percentage, both, none, or a
custom function that receives a data frame with columns
\code{"id"}, \code{"X"}, \code{"Y"}, \code{"name"}, \code{"item"},
and \code{"count"}.  Set labels are formatted as
\code{"setName\\n(count)"}.
\item \strong{Plot assembly} — \code{geom_polygon()} draws filled
regions; \code{geom_path()} draws set outlines; two
\code{\link[ggrepel]{geom_text_repel}()} layers add region labels
and set labels.  For \code{fill_mode = "set"}, colours are mapped
directly (no aesthetic mapping, no legend); for count modes, a
\code{scale_fill_gradientn()} continuous scale is used.
\item \strong{Theme and coordinate system} — \code{coord_equal()}
enforces a square aspect ratio.  The theme removes all axis text,
ticks, titles, grid lines, and panel borders.  The x-axis
expansion is widened to accommodate set label text width.
\item \strong{Dimension calculation} —
\code{\link{calculate_plot_dimensions}()} computes \code{height}
and \code{width} attributes (in inches) from \code{base_height},
\code{aspect.ratio}, and legend metrics.  When
\code{fill_mode = "set"}, the legend position is forced to
\code{"none"}.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsea.R
\name{GSEAPlotAtomic}
\alias{GSEAPlotAtomic}
\title{Atomic GSEA plot (internal)}
\usage{
GSEAPlotAtomic(
  data,
  gene_ranks = "@gene_ranks",
  gs,
  genes,
  metric = "p.adjust",
  sample_coregenes = FALSE,
  line_width = 1.5,
  line_alpha = 1,
  line_color = "#6BB82D",
  n_coregenes = 10,
  genes_label = NULL,
  label_fg = "black",
  label_bg = "white",
  label_bg_r = 0.1,
  label_size = 4,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{gene_ranks}{A named numeric vector of gene-level rank statistics, with
gene identifiers as names. Used to calculate the running enrichment score
and the ranked list metric. If a character string starting with
\code{"@"}, the attribute of \code{data} with that name (minus the
\code{"@"}) is used.}

\item{gs}{Character string specifying the \code{ID} of the gene set to plot.
Must match a value in \code{data$ID}.}

\item{genes}{Character vector of gene identifiers belonging to the gene set.
These are intersected with \code{names(gene_ranks)} for the running score
calculation.}

\item{metric}{Character string specifying the column name in \code{data}
used to compute significance stars in the subtitle. Default is
\code{"p.adjust"}.}

\item{sample_coregenes}{Logical; if \code{TRUE}, core enrichment genes are
sampled randomly for labelling. If \code{FALSE} (default), the first
\code{n_coregenes} core enrichment genes are used.}

\item{line_width}{Numeric specifying the line width for the running
enrichment score curve. Default is \code{1.5}.}

\item{line_alpha}{Numeric alpha transparency for the running score line and
hit indicator bars. Default is \code{1}.}

\item{line_color}{Character string specifying the colour of the running
enrichment score line. Default is \code{"#6BB82D"}.}

\item{n_coregenes}{Integer specifying the number of core enrichment genes to
label on the running score plot. Default is \code{10}. Ignored when
\code{genes_label} is provided.}

\item{genes_label}{Character vector of specific gene names to label on the
running score plot. When provided, \code{n_coregenes} is ignored.}

\item{label_fg}{Character string specifying the text colour of gene labels.
Default is \code{"black"}.}

\item{label_bg}{Character string specifying the background colour of gene
labels. Default is \code{"white"}.}

\item{label_bg_r}{Numeric specifying the corner radius of the label
background. Default is \code{0.1}.}

\item{label_size}{Numeric specifying the font size of the label text.
Default is \code{4}.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{Character string for a right-side y-axis label (rotated 90
degrees). When provided, an extra text grob is added to the right of the
assembled plot, and the plot width increases from 7.5 to 8 inches.}

\item{...}{Additional arguments.}
}
\value{
A \code{patchwork} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a single GSEA (Gene Set Enrichment Analysis)
ridge plot. This is the workhorse behind the exported \code{\link{GSEAPlot}}
function — it takes a single gene set and produces a three-panel figure
showing the running enrichment score, hit positions, and the ranked list
metric.

The plot consists of three vertically stacked panels:
\enumerate{
\item \strong{Running score panel} — traces the running enrichment score
across the ranked gene list, with a peak annotation and optional gene
labels for core enrichment genes.
\item \strong{Hit indicator panel} — marks the positions of gene set
members with vertical lines, colour-coded by the rank metric value
(red for positive, blue for negative).
\item \strong{Gene ranking panel} — shows every gene's rank as a vertical
segment, with annotations indicating positively and negatively correlated
tails and the zero-crossing point.
}

The running enrichment score is computed via \code{\link{gsea_running_score}()}
using the \code{gene_ranks} and \code{genes} provided. Core enrichment genes
(from the \code{core_enrichment} column) can be labelled on the score panel
using \code{\link[ggrepel]{geom_text_repel}()}.
}
\section{Architecture}{

\enumerate{
\item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{Data subsetting} — if \code{data} contains multiple gene
sets (\code{nrow > 1}), it is filtered to the single row matching
\code{gs}. An error is raised if no rows match.
\item \strong{Gene rank resolution} — resolves \code{gene_ranks} from its
\code{"@"}-prefixed attribute reference, validates that it is a named
numeric vector, and sorts it in descending order.
\item \strong{Title and subtitle construction} — \code{title} defaults to
\code{data$Description}. \code{subtitle} is built from the NES value and
the \code{metric} column with significance stars (\code{ns/*/**/***/****}).
\item \strong{Plot data preparation} — builds a data frame with position
(1:N), gene names, running enrichment score (via
\code{gsea_running_score()} with \code{hits_only = FALSE}), raw ranks,
and a hit indicator column.
\item \strong{Running score panel} — renders the top panel with a red/blue
background (positive/negative score regions), a horizontal baseline at
\code{y = 0}, the running score line, and annotations for the peak
(dashed reference lines and an upward/downward triangle). Title and
subtitle are added via \code{\link[ggplot2]{ggtitle}()}.
\item \strong{Gene labeling} — when \code{n_coregenes > 1} or
\code{genes_label} is provided, core enrichment genes are extracted from
\code{data$core_enrichment} and labelled on the running score panel using
\code{geom_text_repel()}. Missing genes trigger a warning.
\item \strong{Hit indicator panel} — renders the middle panel: vertical
lines (\code{geom_linerange}) at each hit position, with coloured
rectangles beneath showing the rank metric gradient (red for positive,
blue for negative).
\item \strong{Gene ranking panel} — renders the bottom panel: vertical
segments (\code{geom_segment}) for every gene's rank, with annotations
for positively/negatively correlated tails. If both signs exist, the
zero-crossing point is marked with a dashed vertical line.
\item \strong{Panel assembly} — the three panels are stacked vertically
via \code{\link[patchwork]{wrap_plots}()} with spacer panels and
proportional heights (\code{3.5 / 1 / 1.5}). When \code{ylab} is provided,
a rotated \code{\link[grid]{textGrob}()} is added to the right side and
the plot width increases from 7.5 to 8 inches.
\item \strong{Dimensions} — height is fixed at 6.5 inches. Width is 7.5
inches (without \code{ylab}) or 8 inches (with \code{ylab}). Attributes
\code{height} and \code{width} are stored on the returned object.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theming.R
\name{palette_this}
\alias{palette_this}
\title{Color palettes collected in plotthis.}
\usage{
palette_this(
  x,
  n = 100,
  palette = "Paired",
  palcolor = NULL,
  type = "auto",
  keep_names = TRUE,
  alpha = 1,
  matched = FALSE,
  reverse = FALSE,
  NA_keep = FALSE,
  NA_color = "grey80",
  transparent = TRUE
)
}
\arguments{
\item{x}{A vector of character/factor or numeric values. If missing, numeric values 1:n will be used as x.}

\item{n}{The number of colors to return for numeric values.}

\item{palette}{Palette name. All available palette names can be queried with \code{show_palettes()}.}

\item{palcolor}{Custom colors used to create a color palette.}

\item{type}{Type of \code{x}. Can be one of "auto", "discrete" or "continuous". The default is "auto", which automatically detects if \code{x} is a numeric value.}

\item{keep_names}{Whether to keep the names of the color vector.}

\item{alpha}{The alpha value of the colors. Default is 1.}

\item{matched}{If \code{TRUE}, will return a color vector of the same length as \code{x}.}

\item{reverse}{Whether to invert the colors.}

\item{NA_keep}{Whether to keep the color assignment to NA in \code{x}.}

\item{NA_color}{Color assigned to NA if NA_keep is \code{TRUE}.}

\item{transparent}{Whether to make the colors transparent when alpha < 1.
When \code{TRUE}, \code{\link[ggplot2:reexports]{ggplot2::alpha()}} is used to make the colors transparent.
Otherwise, \code{adjcolors} is used to adjust the colors based on the alpha. The color will be not be actually transparent.
For example, \code{ggplot2::alpha("red", 0.5) == "#FF000080"}; while \code{adjcolors("red", 0.5) == "#FF8080"}.}
}
\value{
A vector of colors.
}
\description{
Color palettes collected in plotthis.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_legend}
\alias{check_legend}
\title{check_legend
Check if the legend.position and legend.direction are valid}
\usage{
check_legend(
  legend,
  datas_name,
  which = c("legend.position", "legend.direction")
)
}
\arguments{
\item{legend}{The value legend.position or legend.direction}

\item{datas_name}{names of the split data}
}
\value{
named list containing legend names
}
\description{
check_legend
Check if the legend.position and legend.direction are valid
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrich.R
\name{EnrichMap}
\alias{EnrichMap}
\alias{EnrichNetwork}
\title{Enrichment Map and Enrichment Network}
\usage{
EnrichMap(
  data,
  in_form = c("auto", "clusterProfiler", "clusterprofiler", "enrichr"),
  split_by = NULL,
  split_by_sep = "_",
  top_term = 10,
  metric = "p.adjust",
  layout = "fr",
  minchar = 2,
  cluster = "fast_greedy",
  show_keyword = FALSE,
  nlabel = 4,
  character_width = 50,
  mark = "ellipse",
  label = c("term", "feature"),
  labelsize = 5,
  expand = c(0.4, 0.4),
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)

EnrichNetwork(
  data,
  in_form = c("auto", "clusterProfiler", "clusterprofiler", "enrichr"),
  split_by = NULL,
  split_by_sep = "_",
  top_term = 10,
  metric = "p.adjust",
  character_width = 50,
  layout = "fr",
  layoutadjust = TRUE,
  adjscale = 60,
  adjiter = 100,
  blendmode = "blend",
  labelsize = 5,
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame containing enrichment results in clusterProfiler
format (see \code{\link{EnrichMap}} for the expected columns).
If you have enrichment results from multiple databases, you can combine
them into one data frame and add a column (e.g. \code{Database}) to
indicate the source.  Use \code{split_by = "Database"} to plot them
side by side.}

\item{in_form}{A character string specifying the input format.
When \code{"auto"} (default), the function infers the format from the
column names.  Other options are \code{"clusterProfiler"},
\code{"clusterprofiler"}, and \code{"enrichr"}.}

\item{split_by}{The column(s) to split data by and plot separately.
When \code{combine = TRUE}, the combined plot's data (\code{p$data})
contains the rows of every split, tagged with the \code{split_by}
column. For ggraph-based plots (e.g. \code{\link{Network}},
\code{\link{ClustreePlot}}), the node layout of every split is
exposed with the split column, and the original link rows (also
tagged with the \code{split_by} column) are available as the
\code{"edges"} attribute of the data.}

\item{split_by_sep}{The separator for multiple split_by columns. See \code{split_by}}

\item{top_term}{An integer specifying the maximum number of terms to
include.  Terms are ranked by \code{metric} (ascending).  Default
\code{100}.}

\item{metric}{A character string specifying the significance metric
used for top-term selection and node scoring: \code{"p.adjust"}
(default) or \code{"pvalue"}.  The value is transformed as
\eqn{-\log_{10}(metric)}.}

\item{layout}{A character string naming the igraph layout algorithm.
Built-in shortcuts: \code{"circle"}, \code{"tree"}, \code{"grid"}.
Otherwise, the suffix passed to \code{layout_with_<layout>} in igraph
(e.g. \code{"fr"} for Fruchterman-Reingold, \code{"kk"} for
Kamada-Kawai).  Default \code{"fr"}.}

\item{minchar}{An integer specifying the minimum character length for
words to be included as keywords when \code{show_keyword = TRUE}.
Default \code{2}.}

\item{cluster}{A character string naming the igraph community detection
algorithm.  The suffix passed to \code{cluster_<cluster>} in igraph
(e.g. \code{"fast_greedy"}, \code{"walktrap"},
\code{"edge_betweenness"}, \code{"infomap"}).  Default
\code{"fast_greedy"}.}

\item{show_keyword}{A logical value.  When \code{TRUE}, the
\code{Description} text is tokenized and the most significant words
per cluster are shown as keywords.  When \code{FALSE} (default), the
original term descriptions are used as labels.}

\item{nlabel}{An integer specifying the number of keywords or term
descriptions to show per cluster in the legend labels.  Default
\code{4}.}

\item{character_width}{An integer specifying the maximum width (in
characters) at which keyword labels are wrapped via
\code{strwrap(width = character_width)}.  Default \code{50}.}

\item{mark}{A character string naming the ggforce hull function.
One of \code{"ellipse"} (default), \code{"rect"}, \code{"circle"}, or
\code{"text"} -- passed as the suffix to \code{geom_mark_<mark>}.}

\item{label}{A character string specifying what information to display
in the legend labels.  Either \code{"term"} (default; shows top term
descriptions/keywords per cluster) or \code{"feature"} (shows top
gene symbols per cluster).}

\item{labelsize}{A numeric value specifying the font size of the
cluster labels drawn by the ggforce mark layer.  Default \code{5}.}

\item{expand}{The values to expand the x and y axes. It is like CSS padding.
When a single value is provided, it is used for both axes on both sides.
When two values are provided, the first value is used for the top/bottom side and the second value is used for the left/right side.
When three values are provided, the first value is used for the top side, the second value is used for the left/right side, and the third value is used for the bottom side.
When four values are provided, the values are used for the top, right, bottom, and left sides, respectively.
You can also use a named vector to specify the values for each side.
When the axis is discrete, the values will be applied as 'add' to the 'expansion' function.
When the axis is continuous, the values will be applied as 'mult' to the 'expansion' function.
See also \url{https://ggplot2.tidyverse.org/reference/expansion.html}}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{seed}{The random seed to use. Default is 8525.}

\item{combine}{Whether to combine the plots into one when facet is FALSE. Default is TRUE.}

\item{nrow}{A numeric value specifying the number of rows in the facet.}

\item{ncol}{A numeric value specifying the number of columns in the facet.}

\item{byrow}{A logical value indicating whether to fill the plots by row.}

\item{axes}{A string specifying how axes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axes in individual plots.
\item 'collect' will remove duplicated axes when placed in the same run of rows or columns of the layout.
\item 'collect_x' and 'collect_y' will remove duplicated x-axes in the columns or duplicated y-axes in the rows respectively.
}}

\item{axis_titles}{A string specifying how axis titltes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axis titles in individual plots.
\item 'collect' will remove duplicated titles in one direction and merge titles in the opposite direction.
\item 'collect_x' and 'collect_y' control this for x-axis titles and y-axis titles respectively.
}}

\item{guides}{A string specifying how guides should be treated in the layout. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'collect' will collect guides below to the given nesting level, removing duplicates.
\item 'keep' will stop collection at this level and let guides be placed alongside their plot.
\item 'auto' will allow guides to be collected if a upper level tries, but place them alongside the plot if not.
}}

\item{design}{Specification of the location of areas in the layout, passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE. When specified, \code{nrow}, \code{ncol}, and \code{byrow} are ignored.
See \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}} for more details.}

\item{...}{Additional arguments.}

\item{layoutadjust}{A logical value.  When \code{TRUE} (default),
applies \code{\link{adjust_network_layout}()} after the initial layout
to reduce node overlap based on label width and a repulsion simulation.}

\item{adjscale}{A numeric value controlling the scale of the layout
adjustment.  Passed as the \code{scale} argument to
\code{\link{adjust_network_layout}()}.  Default \code{60}.}

\item{adjiter}{A numeric value controlling the number of iterations for
the layout adjustment.  Passed as the \code{iter} argument to
\code{\link{adjust_network_layout}()}.  Default \code{100}.}

\item{blendmode}{A character string specifying how gene colours are
computed from the colours of the terms they belong to.  One of
\code{"blend"} (default), \code{"average"}, \code{"multiply"}, or
\code{"screen"}.  Passed to \code{blend_colors()}.}
}
\value{
A \code{ggplot} object (single plot), a \code{patchwork} /
\code{wrap_plots} object (when \code{split_by} is provided and
\code{combine = TRUE}), or a list of \code{ggplot} objects (when
\code{split_by} is provided and \code{combine = FALSE}).

A \code{ggplot} object (single plot), a \code{patchwork} /
\code{wrap_plots} object (when \code{split_by} is provided and
\code{combine = TRUE}), or a list of \code{ggplot} objects (when
\code{split_by} is provided and \code{combine = FALSE}).
}
\description{
\code{EnrichMap} draws an enrichment map -- a gene-set similarity network
where each node is an enriched term, node size encodes the number of
associated genes, node fill colour encodes cluster membership (detected
via igraph community detection), and edge thickness encodes the number
of overlapping genes between term pairs.  The plot uses a force-directed
layout to arrange terms, and ggforce hull annotations group terms into
clusters.  Keyword or term-description labels appear in the legend.

\code{EnrichNetwork} draws an enrichment network -- a term-gene bipartite
graph where term nodes are shown as numbered circles and gene nodes as
labelled rectangles.  Gene node colours are blended from the colours of
all terms they belong to.  A force-directed layout positions the nodes,
with optional overlap adjustment for better readability.

Both functions accept enrichment results from clusterProfiler or Enrichr
(the latter is auto-detected and preprocessed via
\code{\link{prepare_enrichr_result}()}).
}
\section{split_by Workflow (EnrichMap)}{


When \code{split_by} is provided, \code{EnrichMap()} executes the
following pipeline:
\enumerate{
\item \strong{Argument validation} --
\code{\link{validate_common_args}()} checks the seed.
\item \strong{Input format detection} -- \code{match.arg()} resolves
\code{in_form}; \code{"auto"} mode infers the format from
column names.
\item \strong{Enrichr preprocessing} -- when format is \code{"enrichr"},
calls \code{\link{prepare_enrichr_result}()} to rename columns
and infer GeneRatio/BgRatio.
\item \strong{Split column resolution} --
\code{\link{check_columns}()} validates \code{split_by}
(force_factor, allow_multi, concat_multi).
\item \strong{Data splitting} -- splits \code{data} by
\code{split_by} levels, preserving factor level order.
\item \strong{Per-split palette/colour} --
\code{\link{check_palette}()} and
\code{\link{check_palcolor}()} resolve per-split palette and
colour overrides.
\item \strong{Per-split legend} -- \code{\link{check_legend}()}
resolves \code{legend.position} and \code{legend.direction}
per split.
\item \strong{Per-split title} -- when \code{title} is a function,
it receives the default title (the split level name); otherwise
\code{title \%||\% split_level} is used.
\item \strong{Dispatch} -- each split subset is passed to
\code{\link{EnrichMapAtomic}} with its resolved parameters.
\item \strong{Combination} -- \code{\link{combine_plots}()} assembles
the list of plots via \code{patchwork::wrap_plots}, honouring
\code{nrow}/\code{ncol}/\code{byrow}/\code{axes}/
\code{axis_titles}/\code{guides}/\code{design}.
}
}

\section{split_by Workflow (EnrichNetwork)}{


When \code{split_by} is provided, \code{EnrichNetwork()} executes the
same pipeline as \code{EnrichMap()} above, but dispatches each split
subset to \code{\link{EnrichNetworkAtomic}}.
}

\examples{
\donttest{
data(enrich_example)
EnrichMap(enrich_example)
EnrichMap(enrich_example, label = "feature")
EnrichMap(enrich_example, show_keyword = TRUE, label = "term")
EnrichMap(enrich_example, show_keyword = TRUE, label = "feature")

data(enrich_multidb_example)
EnrichMap(enrich_multidb_example, split_by = "Database")
EnrichMap(enrich_multidb_example, split_by = "Database",
          palette = list(DB1 = "Paired", DB2 = "Set1"))
}
\donttest{
EnrichNetwork(enrich_example, top_term = 5)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lineplot.R
\name{LinePlot}
\alias{LinePlot}
\title{Line plot}
\usage{
LinePlot(
  data,
  x,
  y = NULL,
  group_by = NULL,
  group_by_sep = "_",
  split_by = NULL,
  split_by_sep = "_",
  fill_point_by_x_if_no_group = TRUE,
  color_line_by_x_if_no_group = TRUE,
  add_bg = FALSE,
  bg_palette = "stripe",
  bg_palcolor = NULL,
  bg_alpha = 0.2,
  add_errorbars = FALSE,
  errorbar_width = 0.1,
  errorbar_alpha = 1,
  errorbar_color = "grey30",
  errorbar_linewidth = 0.75,
  errorbar_min = NULL,
  errorbar_max = NULL,
  errorbar_sd = NULL,
  highlight = NULL,
  highlight_size = pt_size - 0.75,
  highlight_color = "red2",
  highlight_alpha = 0.8,
  pt_alpha = 1,
  pt_size = 5,
  keep_na = FALSE,
  keep_empty = FALSE,
  line_type = "solid",
  line_width = 1,
  line_alpha = 0.8,
  add_hline = FALSE,
  hline_type = "solid",
  hline_width = 0.5,
  hline_color = "black",
  hline_alpha = 1,
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  x_text_angle = 0,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  facet_by = NULL,
  facet_scales = "fixed",
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  facet_nrow = NULL,
  facet_ncol = NULL,
  facet_byrow = TRUE,
  facet_args = list(),
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis.}

\item{y}{A character string specifying the column name of the data frame to plot for the y-axis.}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{split_by}{A character vector of column names to split the data by.
Each split level produces a separate sub-plot. Multiple columns are
concatenated with \code{split_by_sep}.}

\item{split_by_sep}{A character string used to join multiple
\code{split_by} column values. Default \code{"_"}.}

\item{fill_point_by_x_if_no_group}{A logical value. When TRUE (default),
points are filled by the x-axis categories via the palette when
\code{group_by = NULL}. Passed to \code{LinePlotSingle} as
\code{fill_point_by_x}. Has no effect when \code{group_by} is set.}

\item{color_line_by_x_if_no_group}{A logical value. When TRUE (default),
lines are coloured by the x-axis categories via the palette when
\code{group_by = NULL}. Passed to \code{LinePlotSingle} as
\code{color_line_by_x}. Has no effect when \code{group_by} is set.}

\item{add_bg}{A logical value. When TRUE, alternating background stripes
are drawn via \code{\link{bg_layer}()}. Default FALSE.}

\item{bg_palette}{A character string specifying the palette for the
background stripe colours. Default \code{"stripe"}.}

\item{bg_palcolor}{A character vector of colours for the background
stripes. When NULL (default), colours are derived from
\code{bg_palette}.}

\item{bg_alpha}{A numeric value in \code{[0, 1]} for the transparency
of background stripes. Default 0.2.}

\item{add_errorbars}{A logical value. When TRUE, error bars are added via
\code{\link[ggplot2]{geom_errorbar}()}. Requires \code{errorbar_sd} or
\code{errorbar_min}/\code{errorbar_max}. Default FALSE.}

\item{errorbar_width}{A numeric value for the width of the error bar
caps. Default 0.1.}

\item{errorbar_alpha}{A numeric value in \code{[0, 1]} for the transparency of
error bars. Default 1.}

\item{errorbar_color}{A character string for the colour of the error
bars. When \code{"line"}, error bars are coloured the same as the lines
(by x when \code{color_line_by_x = TRUE}, or single colour otherwise).
Default \code{"grey30"}.}

\item{errorbar_linewidth}{A numeric value for the line width of error
bars. Default 0.75.}

\item{errorbar_min}{A character string naming the column with the lower
error bar bound. Ignored when \code{errorbar_sd} is provided.}

\item{errorbar_max}{A character string naming the column with the upper
error bar bound. Ignored when \code{errorbar_sd} is provided.}

\item{errorbar_sd}{A character string naming the column with the standard
deviation. When \code{errorbar_min} and \code{errorbar_max} are not
provided, error bars are computed as y +/- \code{errorbar_sd}.}

\item{highlight}{A vector of row indices, row names, a single string
expression (e.g. \code{"y > 10"}) filtering rows to highlight, or TRUE
to highlight all points. When NULL (default), no highlighting is
applied.}

\item{highlight_size}{A numeric value for the size of highlighted points.
Defaults to \code{pt_size - 0.75}.}

\item{highlight_color}{A character string for the colour of highlighted
points. Default \code{"red2"}.}

\item{highlight_alpha}{A numeric value in \code{[0, 1]} for the transparency of
highlighted points. Default 0.8.}

\item{pt_alpha}{A numeric value in \code{[0, 1]} for the transparency of points.
Default 1.}

\item{pt_size}{A numeric value for the point size. Default 5.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{line_type}{A character string specifying the line type. Default
\code{"solid"}.}

\item{line_width}{A numeric value for the line width (in mm). Default 1.}

\item{line_alpha}{A numeric value in \code{[0, 1]} for the transparency of the
line. Default 0.8.}

\item{add_hline}{A numeric value specifying the y-intercept of a
horizontal reference line. When FALSE (default), no line is drawn.}

\item{hline_type}{A character string specifying the line type of the
horizontal reference line. Default \code{"solid"}.}

\item{hline_width}{A numeric value for the width of the horizontal
reference line. Default 0.5.}

\item{hline_color}{A character string for the colour of the horizontal
reference line. Default \code{"black"}.}

\item{hline_alpha}{A numeric value in \code{[0, 1]} for the transparency of the
horizontal reference line. Default 1.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{combine}{Logical; when TRUE (default), per-split plots are combined
into a single \code{patchwork} object. When FALSE, a named list of
\code{ggplot} objects is returned.}

\item{nrow, ncol}{Integer number of rows / columns for the combined
layout (passed to \code{\link[patchwork]{wrap_plots}}).}

\item{byrow}{Logical; fill the combined layout by row. Default TRUE
(passed to \code{\link[patchwork]{wrap_plots}}).}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{facet_args}{A list of additional arguments passed to
\code{\link{facet_plot}()} for fine-grained control over faceting
(e.g. \code{scales}, \code{space}, \code{labeller}).}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{seed}{A numeric seed for reproducibility. Passed to
\code{\link{validate_common_args}()}. Default 8525.}

\item{axes}{A character string specifying how axes should be treated
across the combined layout (passed to
\code{\link[patchwork]{wrap_plots}}).}

\item{axis_titles}{A character string specifying how axis titles should
be treated across the combined layout. Defaults to \code{axes}.}

\item{guides}{A character string specifying how guides should be
collected across panels. Passed to \code{\link{combine_plots}()}.}

\item{design}{A custom layout specification for combined plots (passed
to \code{\link{combine_plots}()}). Overrides \code{nrow}/\code{ncol}
when specified.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object, a \code{patchwork} object (when
\code{combine = TRUE} with \code{split_by}), or a named list of
\code{ggplot} objects (when \code{combine = FALSE}), each with
\code{height} and \code{width} attributes in inches.
}
\description{
Draws a line plot showing the change of a numeric value across the
progression of a categorical x-axis variable.  Each x-axis category is
rendered as a point connected by a line, with support for multiple
grouped series, error bars, highlighted points, background stripes,
and a horizontal reference line.

Key features:
\itemize{
\item \strong{Colour modes:} lines and points can be coloured by x
category (single-series) or by a \code{group_by} variable
(multi-series), or use a single uniform colour.
\item \strong{Error bars:} additive error bars via
\code{errorbar_sd}, \code{errorbar_min}, or \code{errorbar_max}.
\item \strong{Highlighting:} specific points can be emphasised with a
different colour and size via indices, row names, or a filter
expression.
\item \strong{Background stripes:} \code{add_bg = TRUE} draws
alternating bands for visual grouping.
\item \strong{Count aggregation:} omit \code{y} to plot observation
counts per x category.
}
}
\section{split_by Workflow}{

When \code{split_by} is provided:
\enumerate{
\item \strong{Column validation} -- \code{\link{check_columns}()}
resolves \code{split_by} with multi-column concatenation.
\item \strong{NA / empty pre-processing} --
\code{\link{process_keep_na_empty}()} handles \code{keep_na} /
\code{keep_empty} for the split column before splitting, then
removes the split column from the per-split lists.
\item \strong{Data splitting} -- splits \code{data} by
\code{split_by} levels, preserving factor level order.  When
\code{split_by = NULL}, the data is wrapped in a single-element
list with name \code{"..."}.
\item \strong{Per-split palette / colour} --
\code{\link{check_palette}()} and \code{\link{check_palcolor}()}
resolve per-split palette and colour overrides.
\item \strong{Per-split legend} -- \code{\link{check_legend}()}
resolves \code{legend.position} and \code{legend.direction} per
split level.
\item \strong{Per-split title} -- when \code{title} is a function, it
receives the default title (the split level name) and can return
a custom string; otherwise \code{title \%||\% split_level} is
used.
\item \strong{Dispatch} -- each split subset is passed to
\code{\link{LinePlotAtomic}}.
\item \strong{Combination} -- \code{\link{combine_plots}()} assembles
the list of plots via \code{patchwork::wrap_plots}, honouring
\code{nrow}/\code{ncol}/\code{byrow}/\code{design}.
}
}

\examples{
\donttest{
data <- data.frame(
   x = factor(c("A", "B", "C", "D", "A", "B", "C", "D"), levels = LETTERS[1:6]),
   y = c(10, 8, 16, 4, 6, 12, 14, 2),
   group = c("G1", "G1", "G1", "G1", "G2", "G2", "G2", "G2"),
   facet = c("F1", "F1", "F2", "F2", "F3", "F3", "F4", "F4")
)

# --- Basic usage ---
LinePlot(data, x = "x", y = "y")
LinePlot(data, x = "x", y = "y", highlight = "group == 'G1'",
   fill_point_by_x_if_no_group = FALSE, color_line_by_x_if_no_group = FALSE)

# --- Grouped lines ---
LinePlot(data, x = "x", y = "y", group_by = "group")
LinePlot(data, x = "x", y = "y", group_by = "group",
   add_hline = 10, hline_color = "red")
LinePlot(data, x = "x", y = "y", group_by = "group", add_bg = TRUE,
   highlight = "y > 10")
LinePlot(data, x = "x", y = "y", group_by = "group", facet_by = "facet")
LinePlot(data, x = "x", y = "y", group_by = "group", split_by = "facet")

# --- Per-split styling ---
LinePlot(data, x = "x", y = "y", split_by = "group",
         palcolor = list(G1 = c("red", "blue"), G2 = c("green", "black")))

# --- keep_na and keep_empty ---
data <- data.frame(
   x = factor(c("A", "B", NA, "D", "A", "B", NA, "D"), levels = LETTERS[1:4]),
   y = c(10, 8, 16, 4, 6, 12, 14, 2),
   group = factor(c("G1", "G1", "G1", NA, NA, "G3", "G3", "G3"),
     levels = c("G1", "G2", "G3")),
   facet = c("F1", "F1", "F2", "F2", "F3", "F3", "F4", "F4")
)

LinePlot(data, x = "x", y = "y", keep_na = TRUE)
LinePlot(data, x = "x", y = "y", keep_empty = TRUE)
LinePlot(data, x = "x", y = "y", keep_empty = 'level')
LinePlot(data, x = "x", y = "y", group_by = "group", keep_na = TRUE)
LinePlot(data, x = "x", y = "y", group_by = "group", keep_empty = TRUE)
LinePlot(data, x = "x", y = "y", group_by = "group",
   keep_empty = list(x = TRUE, group = 'level'))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manhattanplot.R
\name{ManhattanPlot}
\alias{ManhattanPlot}
\title{Manhattan plot}
\usage{
ManhattanPlot(
  data,
  chr_by,
  pos_by,
  pval_by,
  split_by = NULL,
  split_by_sep = "_",
  label_by = NULL,
  chromosomes = NULL,
  pt_size = 0.75,
  pt_color = NULL,
  pt_alpha = alpha,
  pt_shape = 19,
  label_size = 3,
  label_fg = NULL,
  highlight = NULL,
  highlight_color = NULL,
  highlight_size = 1.5,
  highlight_alpha = 1,
  highlight_shape = 19,
  preserve_position = TRUE,
  chr_gap_scaling = 1,
  pval_transform = "-log10",
  signif = c(5e-08, 1e-05),
  signif_color = NULL,
  signif_rel_pos = 0.2,
  signif_label = TRUE,
  signif_label_size = 3.5,
  signif_label_pos = c("left", "right"),
  thin = NULL,
  thin_n = 1000,
  thin_bins = 200,
  rescale = TRUE,
  rescale_ratio_threshold = 5,
  palette = "Dark2",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  theme = "theme_this",
  theme_args = list(),
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = expression("-" * log[10](p)),
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  facet_by = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{chr_by}{A character string specifying the column name for chromosome
identifiers.  Default: \code{"chr"}.}

\item{pos_by}{A character string specifying the column name for genomic
positions (integer or numeric).  Default: \code{"pos"}.}

\item{pval_by}{A character string specifying the column name for p-values
(numeric).  Default: \code{"pval"}.}

\item{split_by}{The column(s) to split data by and produce separate
sub-plots.  Multiple columns are concatenated with
\code{split_by_sep}.}

\item{split_by_sep}{A character string used to concatenate multiple
\code{split_by} column values.  Default: \code{"_"}.}

\item{label_by}{A character string specifying the column name for variant
labels.  Only variants with non-empty values in this column will be
labelled.  Default: \code{NULL} (no labels).}

\item{chromosomes}{A character or numeric vector specifying which
chromosomes to include and/or their display order.  When \code{NULL}
(the default), all chromosomes present in the data are plotted in
their natural factor order.  A single value filters to that
chromosome; a vector reorders and subsets.}

\item{pt_size}{A numeric value specifying the size of the points.
Default: \code{0.75}.}

\item{pt_color}{A character string specifying a single colour for all
background (non-highlighted) points.  When \code{NULL} (the default),
alternating chromosome colours from \code{palette} / \code{palcolor}
are used.  Typically set to \code{"grey80"} when \code{highlight} is
used with a distinct \code{highlight_color}.}

\item{pt_alpha}{A numeric value in \code{[0, 1]} specifying the
transparency of the points.  Default: \code{alpha} (aliased
parameter).}

\item{pt_shape}{A numeric value specifying the shape of the points.
Default: \code{19} (filled circle).}

\item{label_size}{A numeric value specifying the font size of the
variant labels.  Default: \code{3}.}

\item{label_fg}{A character string specifying the colour of the variant
labels.  When \code{NULL} (the default), each label inherits the
colour of its corresponding point.}

\item{highlight}{Either a numeric vector of row indices or a character
string containing an R expression (parsed via
\code{rlang::parse_expr()}) to select variants to highlight.
Default: \code{NULL} (no highlighting).}

\item{highlight_color}{A character string specifying the colour of
highlighted points.  When \code{NULL} (the default), highlighted
points inherit the chromosome colour from the underlying
\code{geom_point()} layer.}

\item{highlight_size}{A numeric value specifying the size of
highlighted points.  Default: \code{1.5}.}

\item{highlight_alpha}{A numeric value in \code{[0, 1]} specifying the
transparency of highlighted points.  Default: \code{1}.}

\item{highlight_shape}{A numeric value specifying the shape of
highlighted points.  Default: \code{19} (filled circle).}

\item{preserve_position}{A logical value.  When \code{TRUE} (the
default), the width of each chromosome segment reflects its number
of variants and variant positions are correctly scaled.  When
\code{FALSE}, all chromosomes have equal width and variants are
equally spaced.}

\item{chr_gap_scaling}{A numeric scaling factor for the gap between
chromosomes.  Larger values increase the gap.  Default: \code{1}.}

\item{pval_transform}{A function or character string that can be
evaluated to a function for transforming p-values.  Default:
\code{"-log10"}, which computes \eqn{-\log_{10}(p)}.  Other
examples: \code{"-log2"} or a custom
\code{function(x) -log10(x)}.}

\item{signif}{A numeric vector of significance thresholds to draw as
horizontal dashed lines.  Default: \code{c(5e-8, 1e-5)}.}

\item{signif_color}{A character vector of colours for the significance
threshold lines, of equal length as \code{signif}.  When \code{NULL}
(the default), the smallest threshold is coloured black and the rest
grey.}

\item{signif_rel_pos}{A numeric value between \code{0.1} and \code{0.9}
specifying the relative position of the y-axis jump when rescaling
is active.  Default: \code{0.2}.}

\item{signif_label}{A logical value.  When \code{TRUE} (the default),
significance threshold values are annotated on the plot.}

\item{signif_label_size}{A numeric value for the font size of the
significance threshold labels.  Default: \code{3.5}.}

\item{signif_label_pos}{A character string specifying where to place
the significance threshold labels: \code{"left"} (default) or
\code{"right"}.}

\item{thin}{A logical value indicating whether to thin dense data by
sampling points per horizontal partition.  Defaults to \code{TRUE}
when \code{chromosomes} selects fewer chromosomes than in the data,
and \code{FALSE} otherwise.}

\item{thin_n}{An integer specifying the maximum number of points per
horizontal partition after thinning.  Default: \code{1000}.}

\item{thin_bins}{An integer specifying the number of horizontal bins
for thinning.  Default: \code{200}.}

\item{rescale}{A logical value.  When \code{TRUE} (the default), the
y-axis is automatically rescaled (broken axis) if extreme
significance values would otherwise compress the main data cloud.}

\item{rescale_ratio_threshold}{A numeric threshold for triggering
y-axis rescaling.  The ratio is computed as
\code{ceiling(max(log10pval) / 5) * 5 / signif_jump}.
Default: \code{5}.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{seed}{A numeric seed for reproducibility.  Passed to
\code{\link{validate_common_args}()}.  Default: \code{8525}.}

\item{combine}{A logical value.  When \code{TRUE} (the default), the
list of per-split plots is combined into a single \code{patchwork}
object.  When \code{FALSE}, returns the raw list.}

\item{nrow, ncol, byrow}{Integers controlling the layout of combined
plots via \code{patchwork::wrap_plots()}.  \code{byrow = TRUE}
fills the layout row-wise.}

\item{axes, axis_titles}{Strings controlling how axes and axis titles
are handled across combined plots.  Passed to
\code{\link{combine_plots}()}.  See \code{?patchwork::wrap_plots}
for options (\code{"keep"}, \code{"collect"}, \code{"collect_x"},
\code{"collect_y"}).}

\item{guides}{A string controlling guide collection across combined
plots.  Passed to \code{\link{combine_plots}()}.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{design}{A custom layout specification for combined plots.
Passed to \code{\link{combine_plots}()}.  When specified,
\code{nrow}, \code{ncol}, and \code{byrow} are ignored.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object (single plot, no \code{split_by}), a
\code{patchwork} object (when \code{combine = TRUE} with
\code{split_by}), or a named list of \code{ggplot} objects (when
\code{combine = FALSE}).  Each individual plot carries
\code{height} and \code{width} attributes.
}
\description{
Renders a publication-quality Manhattan plot for genetic association
results.  The y-axis displays \eqn{-\log_{10}(p)} (or a user-specified
transformation) of p-values, and the x-axis shows genomic positions
organised by chromosome.  Each chromosome is rendered in alternating
colours, and configurable horizontal dashed lines mark genome-wide
significance thresholds.

The function is adapted from \code{ggmanh::manhattan_plot()} with
extended control over point appearance, variant labels, highlighting,
data thinning, y-axis rescaling, and \code{split_by} support for
creating multi-panel layouts (e.g. faceted by cohort or phenotype).
}
\note{
\code{facet_by} is not supported by this plot type and triggers
a warning if provided.  Use \code{split_by} instead to produce
comparable multi-panel layouts.
}
\section{split_by Workflow}{


When \code{split_by} is provided:
\enumerate{
\item \strong{Column validation} --- \code{\link{check_columns}()}
resolves \code{split_by} with \code{force_factor = TRUE},
\code{allow_multi = TRUE}, and \code{concat_multi = TRUE}.  For
\code{GRanges} inputs, validation is performed on the
\code{@elementMetadata} slot.
\item \strong{GRanges support} --- \code{data} can be a
\code{data.frame} or a \code{GenomicRanges::GRanges} object.
When \code{GRanges} is used, \code{split_by} is read from the
metadata columns.
\item \strong{Data splitting} --- drops unused \code{split_by}
levels, splits \code{data} by \code{split_by} (preserving factor
level order), and wraps into a named list.  When \code{split_by}
is \code{NULL}, the data is wrapped as a single-element list with
name \code{"..."}.
\item \strong{Per-split palette / colour} ---
\code{\link{check_palette}()} and \code{\link{check_palcolor}()}
resolve per-split palette and colour overrides.
\item \strong{Per-split title} --- when \code{title} is a function,
it receives the default title (the split level name) and can return
a custom string; otherwise \code{title \%||\% split_level} is used.
\item \strong{Dispatch} --- each split subset is passed to
\code{\link{ManhattanPlotAtomic}}.
\item \strong{Combination} --- \code{\link{combine_plots}()}
assembles the list of plots via \code{patchwork::wrap_plots},
honouring \code{nrow} / \code{ncol} / \code{byrow} / \code{design}.
}
}

\examples{
\donttest{
set.seed(1000)

nsim <- 50000

# --- Data simulation ---
simdata <- data.frame(
  "chromosome" = sample(c(1:22,"X"), size = nsim, replace = TRUE),
  "position" = sample(1:100000000, size = nsim),
  "P.value" = rbeta(nsim, shape1 = 5, shape2 = 1)^7,
  "cohort" = sample(c("A", "B"), size = nsim, replace = TRUE)
)
simdata$chromosome <- factor(simdata$chromosome, c(1:22, "X"))
options(repr.plot.width=10, repr.plot.height=5)

# --- Basic Manhattan plot ---
if (requireNamespace("ggmanh", quietly = TRUE)) {
ManhattanPlot(
   simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
   title = "Simulated P.Values", ylab = "P")
}

# --- split_by ---
if (requireNamespace("ggmanh", quietly = TRUE)) {
ManhattanPlot(
   simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
   title = "Simulated P.Values", ylab = "P", split_by = "cohort", ncol = 1)
}

# --- Customized p-value transformation and significance threshold line colors ---
if (requireNamespace("ggmanh", quietly = TRUE)) {
ManhattanPlot(
   simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
   title = "Simulated -Log2 P.Values", ylab = "-log2(P)", pval_transform = "-log2",
   signif_color = c("red", "blue"))
}

# --- Different palette and no significance threshold labels ---
if (requireNamespace("ggmanh", quietly = TRUE)) {
ManhattanPlot(
   simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
   palette = "Set1", signif_label = FALSE)
}

# --- Reverse palette and label position on the right ---
if (requireNamespace("ggmanh", quietly = TRUE)) {
ManhattanPlot(
   simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
   palette = "Set1", palreverse = TRUE, signif_label_pos = "right")
}

# --- Single chromosome ---
if (requireNamespace("ggmanh", quietly = TRUE)) {
ManhattanPlot(
   simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
   title = "Simulated P.Values", chromosomes = 5)
}

# --- Chromosome subset and reorder ---
if (requireNamespace("ggmanh", quietly = TRUE)) {
ManhattanPlot(
   simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
   title = "Simulated P.Values", chromosomes = c(20, 4, 6))
}

tmpdata <- data.frame(
  "chromosome" = c(rep(5, 10), rep(21, 5)),
  "position" = c(sample(250000:250100, 10, replace = FALSE),
    sample(590000:600000, 5, replace = FALSE)),
  "P.value" = c(10^-(rnorm(10, 100, 3)), 10^-rnorm(5, 9, 1)),
  "cohort" = c(rep("A", 10), rep("B", 5))
)

simdata <- rbind(simdata, tmpdata)
simdata$chromosome <- factor(simdata$chromosome, c(1:22, "X"))

# --- Disable y-axis rescaling ---
if (requireNamespace("ggmanh", quietly = TRUE)) {
ManhattanPlot(
    simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
    title = "Simulated P.Values - Significant", rescale = FALSE)
}

# --- Y-axis rescaling with custom break position ---
if (requireNamespace("ggmanh", quietly = TRUE)) {
ManhattanPlot(
    simdata, pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
    title = "Simulated P.Values - Significant", rescale = TRUE, signif_rel_pos = 0.5)
}

sig <- simdata$P.value < 5e-07

simdata$label <- ""
simdata$label[sig] <- sprintf("Label: \%i", 1:sum(sig))
simdata$label2 <- ""
i <- (simdata$chromosome == 5) & (simdata$P.value < 5e-8)
simdata$label2[i] <- paste("Chromosome 5 label", 1:sum(i))

# --- Variant labels ---
if (requireNamespace("ggmanh", quietly = TRUE)) {
ManhattanPlot(simdata, label_by = "label", pval_by = "P.value", chr_by = "chromosome",
    pos_by = "position", title = "Simulated P.Values with labels", label_size = 4)
}

# --- Variant labels with custom color ---
if (requireNamespace("ggmanh", quietly = TRUE)) {
ManhattanPlot(simdata, label_by = "label2", pval_by = "P.value", chr_by = "chromosome",
    pos_by = "position", title = "Simulated P.Values with labels",
    label_size = 3, label_fg = "black")
}

simdata$color <- "Not Significant"
simdata$color[simdata$P.value <= 5e-8] <- "Significant"

# --- Highlight points with custom shape ---
if (requireNamespace("ggmanh", quietly = TRUE)) {
ManhattanPlot(simdata, title = "Highlight Points with shapes",
    pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
    highlight = "color == 'Significant'", highlight_color = NULL, highlight_shape = 6,
    highlight_size = 5, pt_alpha = 0.2, pt_size = 1)
}

# --- Highlight points with custom color ---
if (requireNamespace("ggmanh", quietly = TRUE)) {
ManhattanPlot(simdata, title = "Highlight Points",
    pval_by = "P.value", chr_by = "chromosome", pos_by = "position",
    highlight = "color == 'Significant'", highlight_color = "black",
    pt_color = "lightblue", pt_alpha = 0.2, pt_size = 0.1)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/densityplot.R
\name{RidgePlot}
\alias{RidgePlot}
\title{Ridge Plot}
\usage{
RidgePlot(
  data,
  x = NULL,
  in_form = c("long", "wide"),
  split_by = NULL,
  split_by_sep = "_",
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  scale = NULL,
  keep_na = FALSE,
  keep_empty = FALSE,
  add_vline = NULL,
  vline_type = "solid",
  vline_color = TRUE,
  vline_width = 0.5,
  vline_alpha = 1,
  flip = FALSE,
  alpha = 0.8,
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  x_text_angle = 90,
  x_min = NULL,
  x_max = NULL,
  reverse = FALSE,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  legend.position = "none",
  legend.direction = "vertical",
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  seed = 8525,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis.}

\item{in_form}{A character string specifying whether \code{data} is in
\code{"long"} (default) or \code{"wide"} format.}

\item{split_by}{The column(s) to split data by and plot separately.
When \code{combine = TRUE}, the combined plot's data (\code{p$data})
contains the rows of every split, tagged with the \code{split_by}
column. For ggraph-based plots (e.g. \code{\link{Network}},
\code{\link{ClustreePlot}}), the node layout of every split is
exposed with the split column, and the original link rows (also
tagged with the \code{split_by} column) are available as the
\code{"edges"} attribute of the data.}

\item{split_by_sep}{The separator for multiple split_by columns. See \code{split_by}}

\item{group_by}{Columns to group the data for plotting
For those plotting functions that do not support multiple groups,
They will be concatenated into one column, using \code{group_by_sep} as the separator}

\item{group_by_sep}{The separator for multiple group_by columns. See \code{group_by}}

\item{group_name}{A character string used as the legend title for the
\code{group_by} fill aesthetic. Defaults to the (concatenated) \code{group_by}
column name.}

\item{scale}{A numeric value controlling the vertical overlap of ridges.
Passed to \code{ggridges::geom_density_ridges(scale = ...)}. Smaller values
increase overlap. When \code{NULL}, ggridges auto-computes the scale.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{add_vline}{A specification for vertical reference lines:
\itemize{
\item \code{NULL} or \code{FALSE}: no lines.
\item \code{TRUE}: draw a line at the mean of each group.
\item A numeric vector: draw the same lines for all groups.
\item A named list of numeric vectors: per-group lines, where names should
match \code{group_by} levels.
}}

\item{vline_type}{A character string specifying the line type for the
vertical reference lines. Passed as \code{linetype} to \code{geom_vline()}.
Default: \code{"solid"}.}

\item{vline_color}{The colour of the vertical reference lines:
\itemize{
\item A literal colour value or vector (recycled): applied directly.
\item \code{TRUE} (default): each line is coloured with a darkened blend of
the corresponding ridge fill colour, computed via
\code{blend_colors(mode = "multiply")}.
}}

\item{vline_width}{A numeric value for the thickness of the vertical
reference lines. Passed as \code{linewidth} to \code{geom_vline()}.
Default: \code{0.5}.}

\item{vline_alpha}{A numeric value in \verb{[0, 1]} for the transparency of
the vertical reference lines. Default: \code{1}.}

\item{flip}{A logical value. If \code{TRUE}, the axes are swapped via
\code{coord_flip()}. X-axis text angle and grid-line placement are adjusted
accordingly.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{x_min, x_max}{Numeric limits for the x-axis. When \code{NULL} (default),
limits are determined from the data range. Passed to \code{scale_x_continuous(limits = ...)}.}

\item{reverse}{A logical value. If \code{TRUE}, the y-axis group order is
reversed. NA groups are renamed to the literal string \code{"NA"} and
placed at the end.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{combine}{Whether to combine the plots into one when facet is FALSE. Default is TRUE.}

\item{nrow}{A numeric value specifying the number of rows in the facet.}

\item{ncol}{A numeric value specifying the number of columns in the facet.}

\item{byrow}{A logical value indicating whether to fill the plots by row.}

\item{seed}{The random seed to use. Default is 8525.}

\item{axes}{A string specifying how axes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axes in individual plots.
\item 'collect' will remove duplicated axes when placed in the same run of rows or columns of the layout.
\item 'collect_x' and 'collect_y' will remove duplicated x-axes in the columns or duplicated y-axes in the rows respectively.
}}

\item{axis_titles}{A string specifying how axis titltes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axis titles in individual plots.
\item 'collect' will remove duplicated titles in one direction and merge titles in the opposite direction.
\item 'collect_x' and 'collect_y' control this for x-axis titles and y-axis titles respectively.
}}

\item{guides}{A string specifying how guides should be treated in the layout. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'collect' will collect guides below to the given nesting level, removing duplicates.
\item 'keep' will stop collection at this level and let guides be placed alongside their plot.
\item 'auto' will allow guides to be collected if a upper level tries, but place them alongside the plot if not.
}}

\item{design}{Specification of the location of areas in the layout, passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE. When specified, \code{nrow}, \code{ncol}, and \code{byrow} are ignored.
See \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}} for more details.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object (single plot), a \code{patchwork} / \code{wrap_plots} object
(when \code{split_by} is provided and \code{combine = TRUE}), or a list of \code{ggplot}
objects (when \code{split_by} is provided and \code{combine = FALSE}).
}
\description{
Ridge (joy) plot for visualising the distribution of a numeric variable across
multiple groups. Each group is rendered as a partially overlapping density
curve along the y-axis, making it easy to compare distribution shapes, central
tendency, and spread across categories.

The function supports both \strong{long} and \strong{wide} data formats:
\itemize{
\item \strong{Long form} (\code{in_form = "long"}, default) — a numeric column (\code{x}) plus a
factor column (\code{group_by}) whose levels become the y-axis ridges.
\item \strong{Wide form} (\code{in_form = "wide"}) — multiple numeric columns listed in
\code{group_by} are gathered internally into long form.
}

Optional vertical reference lines (\code{add_vline}) can mark group means,
specific values, or per-group thresholds. Supports faceting, split-by
splitting, and full palette customisation.
}
\section{split_by Workflow}{


When \code{split_by} is specified, \code{RidgePlot()} executes the following pipeline:
\enumerate{
\item \strong{Argument validation} — \code{validate_common_args()} checks the seed and
facet-by consistency.
\item \strong{NA / empty normalisation} — \code{check_keep_na()} / \code{check_keep_empty()}
convert \code{keep_na} / \code{keep_empty} to per-column lists.
\item \strong{Theme resolution} — \code{process_theme()} resolves the theme string to a
theme function.
\item \strong{Split column resolution} — \code{check_columns()} validates \code{split_by}
(force_factor, concat_multi).
\item \strong{Pre-filtering} — \code{process_keep_na_empty()} removes NA / empty levels
from the split column, then \code{data} is split by \code{split_by} levels (order
preserved).
\item \strong{Per-split parameter resolution} — \code{check_palette()},
\code{check_palcolor()}, \code{check_legend()} resolve palette, palcolor,
legend.position, and legend.direction for each split.
\item \strong{Per-split dispatch} — each split is passed to \code{RidgePlotAtomic()} with
its resolved parameters. Title defaults to the split level name unless
\code{title} is a function (in which case it is called with the default).
\item \strong{Combination} — \code{combine_plots()} assembles the list of plots via
\code{patchwork::wrap_plots()}, applying \code{nrow}, \code{ncol}, \code{byrow}, \code{axes},
\code{axis_titles}, \code{guides}, and \code{design}.
}
}

\examples{
\donttest{
set.seed(8525)
data <- data.frame(
   x = c(rnorm(250, -1), rnorm(250, 1)),
   group = factor(rep(c("A", NA, LETTERS[3:5]), each = 100), levels = LETTERS[1:6])
)

# basic usage
RidgePlot(data, x = "x")  # single ridge (no group_by)
RidgePlot(data, x = "x", add_vline = 0, vline_color = "black")

# grouped ridges
RidgePlot(data, x = "x", group_by = "group")
RidgePlot(data, x = "x", group_by = "group",
   keep_na = TRUE, keep_empty = TRUE)
RidgePlot(data, x = "x", group_by = "group", reverse = TRUE)
RidgePlot(data, x = "x", group_by = "group",
   add_vline = TRUE, vline_color = TRUE, alpha = 0.7)

# faceting
RidgePlot(data, x = "x", facet_by = "group",
   keep_na = TRUE, keep_empty = TRUE)

# wide form
data_wide <- data.frame(
   A = rnorm(100),
   B = rnorm(100),
   C = rnorm(100),
   D = rnorm(100),
   E = rnorm(100),
   group = sample(letters[1:4], 100, replace = TRUE)
)
RidgePlot(data_wide, group_by = LETTERS[1:5], in_form = "wide")
RidgePlot(data_wide, group_by = LETTERS[1:5], in_form = "wide", facet_by = "group")

# split_by with per-split palettes
RidgePlot(data_wide, group_by = LETTERS[1:5], in_form = "wide", split_by = "group",
   palette = list(a = "Reds", b = "Blues", c = "Greens", d = "Purples"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/volcanoplot.R
\name{VolcanoPlot}
\alias{VolcanoPlot}
\title{Volcano plot}
\usage{
VolcanoPlot(
  data,
  x,
  y,
  ytrans = "-log10",
  color_by = NULL,
  color_name = NULL,
  xlim = NULL,
  flip_negatives = FALSE,
  x_cutoff = NULL,
  y_cutoff = 0.05,
  split_by = NULL,
  split_by_sep = "_",
  label_by = NULL,
  x_cutoff_name = NULL,
  y_cutoff_name = NULL,
  x_cutoff_color = "red2",
  y_cutoff_color = "blue2",
  x_cutoff_linetype = "dashed",
  y_cutoff_linetype = "dashed",
  x_cutoff_linewidth = 0.5,
  y_cutoff_linewidth = 0.5,
  pt_size = 2,
  pt_alpha = 0.5,
  pt_shape = 21,
  pt_border_color = TRUE,
  pt_border_size = 0.5,
  nlabel = 5,
  labels = NULL,
  label_size = 3,
  label_fg = "black",
  label_bg = "white",
  label_bg_r = 0.1,
  highlight = NULL,
  highlight_color = "red",
  highlight_size = 2,
  highlight_alpha = 1,
  highlight_stroke = 0.5,
  raster = NULL,
  raster_dpi = c(512, 512),
  trim = c(0, 1),
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  palreverse = FALSE,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  seed = 8525,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name of the data frame to plot for the x-axis.}

\item{y}{A character string specifying the column name of the data frame to plot for the y-axis.}

\item{ytrans}{A function or a function name (as a string) to transform
the y-axis values before plotting. The transformed values are used for
both the y-axis and cutoff comparisons. Default: \code{"-log10"}
(converts p-values to a -log10 scale). Other named functions can be
passed as strings, e.g. \code{"sqrt"}.}

\item{color_by}{A character string specifying the column name to colour
the points by. When \code{NULL} (default), points are automatically
categorised as \code{"sig_pos_x"}, \code{"sig_neg_x"}, or
\code{"insig"} based on \code{x_cutoff} and \code{y_cutoff}, and the
colour legend is suppressed. When a column name is provided, the colour
mapping follows the column type — discrete (character/factor) uses
\code{scale_color_manual()} with the specified \code{palette};
numeric (continuous) uses \code{scale_color_gradientn()}.}

\item{color_name}{A character string for the colour legend title when
\code{color_by} is a numeric column. When \code{NULL} (default), the
\code{color_by} column name is used.}

\item{xlim}{A numeric vector of length 2 to set the x-axis limits.
Passed to \code{\link[ggplot2]{xlim}()}. When \code{NULL} (default),
limits are determined automatically from the data.}

\item{flip_negatives}{A logical value. When \code{TRUE}, y-values of
points with negative x-values are multiplied by -1, creating a mirrored
volcano plot where both up- and down-regulated features show their
significance on the same side of the y-axis. A horizontal line at
\code{y = 0} and absolute-value axis labels are added. Default:
\code{FALSE}.}

\item{x_cutoff}{A numeric value specifying the x-axis significance
cutoff. Both the negative and positive of this value are used as
vertical threshold lines. When \code{NULL} or \code{0}, no x-cutoff
line is drawn. Default: \code{NULL}.}

\item{y_cutoff}{A numeric value specifying the y-axis significance
cutoff in the \strong{original} (untransformed) scale. The value is
transformed by \code{ytrans} before plotting. When \code{NULL}, no
y-cutoff line is drawn and the category assignment uses only the
x-cutoff. Default: \code{0.05}.}

\item{split_by}{The column(s) to split the data by and produce separate
sub-plots. Multiple columns are concatenated with \code{split_by_sep}.}

\item{split_by_sep}{A character string to separate concatenated
\code{split_by} columns. Default \code{"_"}.}

\item{label_by}{A character string specifying the column whose values
are used as label text. When \code{NULL} (default), row names of the
data frame are used.}

\item{x_cutoff_name}{A character string for the x-cutoff legend entry.
When \code{"none"}, the legend for the x-cutoff line is suppressed
entirely (the line is still drawn). When \code{NULL} (default), a
label of the form \code{"<x> = +/-<value>"} is generated.}

\item{y_cutoff_name}{A character string for the y-cutoff legend entry.
When \code{"none"}, the legend for the y-cutoff line is suppressed
entirely (the line is still drawn). When \code{NULL} (default), a
label of the form \code{"<ylab> = <value>"} is generated.}

\item{x_cutoff_color}{A character string specifying the colour of the
x-axis cutoff line(s). Default: \code{"red2"}.}

\item{y_cutoff_color}{A character string specifying the colour of the
y-axis cutoff line(s). Default: \code{"blue2"}.}

\item{x_cutoff_linetype}{A character string specifying the linetype of
the x-axis cutoff line(s). Default: \code{"dashed"}.}

\item{y_cutoff_linetype}{A character string specifying the linetype of
the y-axis cutoff line(s). Default: \code{"dashed"}.}

\item{x_cutoff_linewidth}{A numeric value specifying the linewidth of
the x-axis cutoff line(s). Default: \code{0.5}.}

\item{y_cutoff_linewidth}{A numeric value specifying the linewidth of
the y-axis cutoff line(s). Default: \code{0.5}.}

\item{pt_size}{A numeric value specifying the point size for all data
points. Default: \code{2}.}

\item{pt_alpha}{A numeric value in \code{[0, 1]} specifying the
transparency of all data points. Default: \code{0.5}.}

\item{pt_shape}{A numeric value specifying the point shape. Default: \code{21}
(filled circle with border). Shapes 21--25 support separate fill and border
colour aesthetics; all other shapes use a single colour aesthetic. In
raster mode, all points are drawn as filled circles (shape is ignored).}

\item{pt_border_color}{Controls the point border colour. For shapes 21--25:
\itemize{
\item \code{TRUE} (default) -- border colour tracks the \code{color_by}
gradient / palette.
\item A colour string (e.g. \code{"black"}) -- constant colour border.
}
\code{FALSE} or \code{NULL} disables the border. For shapes without a fill
aesthetic (not 21--25), this parameter has no effect. In raster mode the
border is drawn as a slightly larger disc behind each point (the shape is
always a circle there), and \code{TRUE} falls back to a border disc in the
\code{color_by} colour.}

\item{pt_border_size}{A numeric value specifying the point border size
(stroke width, in mm). \code{0} disables the border. Default: \code{0.5}.}

\item{nlabel}{An integer specifying the number of top features to label
automatically. Points are ranked by Euclidean distance to the origin
within each \code{sign(x)} group (and per facet level if
\code{facet_by} is set). Only non-insignificant points receive labels.
Default: \code{5}.}

\item{labels}{A character vector of row names or integer indices
specifying which points to label. Overrides automatic \code{nlabel}
selection. When \code{NULL} (default), top \code{nlabel} points are
chosen automatically.}

\item{label_size}{A numeric value specifying the font size of the
labels. Default: \code{3}.}

\item{label_fg}{A character string specifying the text colour of the
labels. Default: \code{"black"}.}

\item{label_bg}{A character string specifying the background colour of
the label boxes (passed to \code{geom_text_repel(bg.color = ...)}).
Default: \code{"white"}.}

\item{label_bg_r}{A numeric value specifying the corner radius of the
label background boxes (passed to \code{geom_text_repel(bg.r = ...)}).
Default: \code{0.1}.}

\item{highlight}{A character vector of row names or integer indices
specifying which points to highlight with an overlaid point layer in
\code{highlight_color}. When \code{NULL} (default), no highlighting is
applied.}

\item{highlight_color}{A character string specifying the colour of the
highlight points. Default: \code{"red"}.}

\item{highlight_size}{A numeric value specifying the point size of the
highlight layer. Default: \code{2}.}

\item{highlight_alpha}{A numeric value in \code{[0, 1]} specifying the
transparency of the highlight points. Default: \code{1}.}

\item{highlight_stroke}{A numeric value specifying the stroke width of
the highlight point borders. Default: \code{0.5}.}

\item{raster}{A logical value. If \code{TRUE}, points are rendered via
\code{scattermore::geom_scattermore()} for efficient rasterised plotting.
Default is \code{NULL}, which auto-enables when \code{nrow(data) > 1e3}.}

\item{raster_dpi}{A numeric vector of length 2 \verb{[x_dpi, y_dpi]} specifying the raster
resolution in pixels. Passed to \code{scattermore::geom_scattermore(pixels = ...)}.
Default is \code{c(512, 512)}. If a single value is provided it is recycled to both dimensions.}

\item{trim}{A numeric vector of length 2 specifying quantile bounds for
winsorizing the x-axis values. Values below the first quantile are
clamped to that quantile; values above the second quantile are clamped
to that quantile. Both values must be in \code{[0, 1]}. When both bounds
are nonzero and of opposite sign, they are symmetrised to the smaller
absolute value. Default: \code{c(0, 1)} (no trimming).}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{seed}{The random seed to use. Default is 8525.}

\item{combine}{Logical; when \code{TRUE} (default), returns a combined
\code{patchwork} object. When \code{FALSE}, returns a named list of
individual \code{ggplot} objects.}

\item{ncol, nrow}{Integer number of columns / rows for the combined layout
(passed to \code{\link[patchwork]{wrap_plots}}).}

\item{byrow}{Logical; fill the combined layout by row. Default
\code{TRUE} (passed to \code{\link[patchwork]{wrap_plots}}).}

\item{axes}{A character string specifying how axes should be treated
across the combined layout (passed to
\code{\link[patchwork]{wrap_plots}}).}

\item{axis_titles}{A character string specifying how axis titles should
be treated across the combined layout. Defaults to \code{axes}.}

\item{guides}{A character string specifying how guides (legends) should
be collected across panels. Default \code{"collect"} (passed to
\code{\link{combine_plots}()}).}

\item{design}{A custom layout design for the combined plot (passed to
\code{\link{combine_plots}()}).}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object, a \code{patchwork} object, or a named
list of \code{ggplot} objects (when \code{combine = FALSE}), each with
\code{height} and \code{width} attributes in inches.
}
\description{
Produces a volcano plot — a scatter plot that displays statistical
significance (typically -log10 adjusted p-value) on the y-axis versus
magnitude of change (log2 fold change) on the x-axis. Points are coloured
automatically by significance category (\code{"sig_pos_x"},
\code{"sig_neg_x"}, \code{"insig"}) or by a user-supplied column. The
most significant features can be labelled automatically via
\code{\link[ggrepel]{geom_text_repel}()}, and specific points can be
highlighted.

The function supports \strong{automatic labelling} of top features (by
distance to origin), \strong{mirrored layout} via
\code{flip_negatives}, \strong{x-axis trimming} to reduce the influence
of extreme values, \strong{faceting}, and \strong{splitting} into
separate sub-plots via \code{split_by} with per-split colour palette and
legend control.
}
\section{split_by Workflow}{

When \code{split_by} is provided:
\enumerate{
\item The \code{split_by} column(s) are validated via
\code{\link{check_columns}()} with \code{force_factor = TRUE} and
\code{concat_multi = TRUE} (multiple columns are concatenated
with \code{split_by_sep}).
\item The data frame is split by \code{split_by} (preserving factor
level order). If \code{split_by} is \code{NULL}, the data is
wrapped in a single-element list with name \code{"..."}.
\item Per-split \code{palette}, \code{palcolor},
\code{legend.position}, and \code{legend.direction} are resolved
via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
and \code{\link{check_legend}()}.
\item \code{\link{VolcanoPlotAtomic}()} is called for each split. If
\code{title} is a function, it receives the split level name and
can generate dynamic titles.
\item Results are combined via \code{\link{combine_plots}()} (when
\code{combine = TRUE}) or returned as a named list.
}
}

\examples{
\donttest{
set.seed(8525)
n <- 200
n_de <- 150

## Non-DE genes
fc_null <- rnorm(n - n_de, 0, 0.35)
z_null  <- rnorm(n - n_de, 0, 1)

## DE genes
fc_de <- rnorm(
  n_de,
  mean = sample(c(-1, 1), n_de, replace = TRUE),
  sd = 0.45
)

## Make significance related to effect size,
## but with substantial variation
z_de <- fc_de * rnorm(n_de, 4.5, 0.8)

avg_log2FC <- c(fc_null, fc_de)
z <- c(z_null, z_de)

p_val <- 2 * pnorm(-abs(z))
p_val_adj <- p.adjust(p_val, method = "BH")

## Shuffle genes
i <- sample(n)

data <- data.frame(
  avg_log2FC = avg_log2FC[i],
  p_val_adj = p_val_adj[i],
  gene = paste0("gene", seq_len(n))[i],
  pct_diff = rnorm(n, 0, 1),
  group = sample(LETTERS[1:2], n, replace = TRUE)
)

# --- Basic usage ---
VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", color_by = "pct_diff",
   y_cutoff_name = "-log10(0.05)")
# --- With gene labels ---
VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", color_by = "pct_diff",
   y_cutoff_name = "-log10(0.05)", label_by = "gene")
# --- Mirrored layout ---
VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", y_cutoff_name = "none",
   flip_negatives = TRUE, label_by = "gene")
# --- With faceting ---
VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", y_cutoff_name = "none",
   flip_negatives = TRUE, facet_by = "group", label_by = "gene")
# --- With splitting ---
VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", y_cutoff_name = "none",
   flip_negatives = TRUE, split_by = "group", label_by = "gene")
# --- With highlighting ---
VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", y_cutoff_name = "none",
   highlight = c("gene196", "gene151"), label_by = "gene")
# --- Per-split palettes ---
VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", color_by = "pct_diff",
   y_cutoff_name = "-log10(0.05)", split_by = "group", label_by = "gene",
   palette = c(A = "Spectral", B = "PuOr"))
# Trim extreme x-values (winsorize to 40\% and 50\% quantiles), for demo purposes
VolcanoPlot(data, x = "avg_log2FC", y = "p_val_adj", color_by = "pct_diff",
   y_cutoff_name = "-log10(0.05)", trim = c(0.4, 0.5))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qqplot.R
\name{QQPlot}
\alias{QQPlot}
\title{QQ/PP plot}
\usage{
QQPlot(
  data,
  val,
  val_trans = NULL,
  type = c("qq", "pp"),
  split_by = NULL,
  split_by_sep = "_",
  band = NULL,
  line = list(),
  point = list(),
  fill_name = "Bands",
  band_alpha = 0.5,
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  palreverse = FALSE,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  legend.position = waiver(),
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlim = NULL,
  ylim = NULL,
  xlab = ifelse(type == "qq", "Theoretical Quantiles", "Probability Points"),
  ylab = ifelse(type == "qq", "Sample Quantiles", "Cumulative Probability"),
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  seed = 8525,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{val}{A character string naming the numeric column whose distribution
is compared against the theoretical distribution.}

\item{val_trans}{A transformation function applied to the \code{val}
column before plotting. For example, \code{log} or \code{sqrt}.
Default: \code{NULL} (no transformation).}

\item{type}{A character string specifying the plot type. Either
\code{"qq"} (quantile-quantile, the default) or \code{"pp"}
(probability-probability). Partial matching is supported.}

\item{split_by}{The column(s) to split data by and plot separately.
When \code{combine = TRUE}, the combined plot's data (\code{p$data})
contains the rows of every split, tagged with the \code{split_by}
column. For ggraph-based plots (e.g. \code{\link{Network}},
\code{\link{ClustreePlot}}), the node layout of every split is
exposed with the split column, and the original link rows (also
tagged with the \code{split_by} column) are available as the
\code{"edges"} attribute of the data.}

\item{split_by_sep}{The separator for multiple split_by columns. See \code{split_by}}

\item{band}{A list of arguments passed to \code{\link[qqplotr]{stat_qq_band}}
or \code{\link[qqplotr]{stat_pp_band}}, depending on \code{type}.
Set to \code{TRUE} or an empty list to use default arguments.
Set to \code{NULL} (the default) to suppress bands entirely.
To add multiple bands, provide a list of lists, each containing
arguments for one band (e.g. different \code{bandType} or
\code{distribution}). Each band can also include a custom
\code{mapping} aesthetic to control its fill colour legend entry.}

\item{line}{A list of arguments passed to \code{\link[qqplotr]{stat_qq_line}}
or \code{\link[qqplotr]{stat_pp_line}}, depending on \code{type}.
Default: \code{list()} (adds a reference line with default arguments).
Set to \code{NULL} to omit the line entirely.}

\item{point}{A list of arguments passed to \code{\link[qqplotr]{stat_qq_point}}
or \code{\link[qqplotr]{stat_pp_point}}, depending on \code{type}.
Default: \code{list()} (adds points with default arguments).
Set to \code{NULL} to omit points (not recommended).}

\item{fill_name}{A character string for the fill legend title used
when bands are present. Default: \code{"Bands"}.}

\item{band_alpha}{A numeric value in \code{[0, 1]} setting the
transparency of all bands. Individual bands can override this via
\code{alpha} inside the \code{band} argument list.
Default: \code{0.5}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlim}{A numeric vector of length 2 specifying the x-axis limits.
Default: \code{NULL} (use data range).}

\item{ylim}{A numeric vector of length 2 specifying the y-axis limits.
Default: \code{NULL} (use data range).}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{combine}{Whether to combine the plots into one when facet is FALSE. Default is TRUE.}

\item{nrow}{A numeric value specifying the number of rows in the facet.}

\item{ncol}{A numeric value specifying the number of columns in the facet.}

\item{byrow}{A logical value indicating whether to fill the plots by row.}

\item{seed}{The random seed to use. Default is 8525.}

\item{axes}{A string specifying how axes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axes in individual plots.
\item 'collect' will remove duplicated axes when placed in the same run of rows or columns of the layout.
\item 'collect_x' and 'collect_y' will remove duplicated x-axes in the columns or duplicated y-axes in the rows respectively.
}}

\item{axis_titles}{A string specifying how axis titltes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axis titles in individual plots.
\item 'collect' will remove duplicated titles in one direction and merge titles in the opposite direction.
\item 'collect_x' and 'collect_y' control this for x-axis titles and y-axis titles respectively.
}}

\item{guides}{A string specifying how guides should be treated in the layout. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'collect' will collect guides below to the given nesting level, removing duplicates.
\item 'keep' will stop collection at this level and let guides be placed alongside their plot.
\item 'auto' will allow guides to be collected if a upper level tries, but place them alongside the plot if not.
}}

\item{design}{Specification of the location of areas in the layout, passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE. When specified, \code{nrow}, \code{ncol}, and \code{byrow} are ignored.
See \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}} for more details.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object (single plot), a \code{patchwork} object
(combined split plots), or a named list of \code{ggplot} objects
(when \code{combine = FALSE}), each with \code{height} and
\code{width} attributes in inches.
}
\description{
Produces a quantile-quantile (QQ) plot or probability-probability (PP)
plot to compare the empirical distribution of a numeric variable against
a theoretical distribution (default: standard normal). The function
delegates to the \pkg{qqplotr} package for the underlying statistics
and rendering.

Key features:
\itemize{
\item \strong{QQ and PP modes} -- switch between quantile-quantile
and probability-probability displays via \code{type}.
\item \strong{Confidence bands} -- overlay one or more confidence bands
(pointwise, KS, Tukey simultaneous, or bootstrap) with custom fill
colours and alpha.
\item \strong{Reference line} -- a diagonal reference line (QQ) or
diagonal probability line (PP) for comparison.
\item \strong{Distribution fitting} -- compare against any
distribution supported by \pkg{qqplotr} (normal, exponential, uniform,
etc.) by passing \code{distribution} and \code{dparams} inside the
\code{band}, \code{line}, and \code{point} lists.
\item \strong{Detrending} -- enable \code{detrend = TRUE} inside the
argument lists to remove the reference line and visualise only
deviations (flat PP plot centred at zero).
\item \strong{Splitting} -- use \code{split_by} to produce separate
QQ/PP plots for different groups, combined into a single layout.
}
}
\section{split_by Workflow}{

When \code{split_by} is provided:
\enumerate{
\item \strong{Common arg validation} --
\code{\link{validate_common_args}()} checks the \code{seed} and
\code{facet_by} constraints.
\item \strong{Theme processing} -- \code{\link{process_theme}()}
resolves the \code{theme} string or function.
\item \strong{split_by column resolution} --
\code{\link{check_columns}()} validates the \code{split_by} column(s)
with \code{force_factor = TRUE}. Multiple columns are concatenated
with \code{split_by_sep}.
\item \strong{Data splitting} -- the data frame is split by
\code{split_by} levels (droplevels applied, level order preserved).
If \code{split_by} is \code{NULL}, the data is wrapped in a
single-element list with name \code{"..."}.
\item \strong{Per-split parameter resolution} --
\code{\link{check_palette}()}, \code{\link{check_palcolor}()}, and
\code{\link{check_legend}()} resolve per-split \code{palette},
\code{palcolor}, \code{legend.position}, and
\code{legend.direction}.
\item \strong{Dispatch per split} -- \code{\link{QQPlotAtomic}()} is
called for each split level. If \code{title} is a function, it
receives the split level name and generates a dynamic title;
otherwise the level name is used as the default title.
\item \strong{Combination} -- results are combined via
\code{\link{combine_plots}()} (when \code{combine = TRUE}) or
returned as a named list (when \code{combine = FALSE}).
}
}

\examples{
set.seed(8525)
data <- data.frame(norm = rnorm(100))

# Basic QQ plot with default confidence band
QQPlot(data, val = "norm", band = TRUE)

# Multiple confidence bands with custom fill labels
QQPlot(data, val = "norm", band = list(
    list(bandType = "ks", mapping = ggplot2::aes(fill = "KS"), alpha = 0.3),
    list(bandType = "ts", mapping = ggplot2::aes(fill = "TS")),
    list(bandType = "pointwise", mapping = ggplot2::aes(fill = "Normal")),
    list(bandType = "boot", mapping = ggplot2::aes(fill = "Bootstrap"))
), band_alpha = 0.6)

# Compare against exponential distribution
data(airquality, package = "datasets")
di <- "exp"
dp <- list(rate = 2)
QQPlot(airquality, val = "Ozone",
    band = list(distribution = di, dparams = dp),
    line = list(distribution = di, dparams = dp),
    point = list(distribution = di, dparams = dp)
)

# Detrended QQ plot: deviations from the reference line
de <- TRUE
QQPlot(airquality, val = "Ozone",
    band = list(distribution = di, dparams = dp, detrend = de),
    line = list(distribution = di, dparams = dp, detrend = de),
    point = list(distribution = di, dparams = dp, detrend = de)
)

# PP plot (probability-probability)
QQPlot(data, val = "norm", type = "pp", band = TRUE)

# PP plot with shifted/scaled normal distribution
dp <- list(mean = 2, sd = 2)
QQPlot(data, val = "norm", type = "pp",
    band = list(dparams = dp),
    point = list(dparams = dp))

# PP plot with custom intercept/slope line
QQPlot(data, val = "norm", type = "pp", band = TRUE,
    line = list(ab = c(.2, .5)))

# Detrended PP plot with axis limits
di <- "exp"
dp <- list(rate = .022)
de <- TRUE
QQPlot(airquality, val = "Ozone", type = "pp",
    band = list(distribution = di, detrend = de, dparams = dp),
    line = list(detrend = de),
    point = list(distribution = di, detrend = de, dparams = dp),
    ylim = c(-.5, .5)
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{facet_plot}
\alias{facet_plot}
\title{Facetting a plot}
\usage{
facet_plot(
  plot,
  facet_by,
  facet_scales,
  nrow,
  ncol,
  byrow,
  legend.position = "right",
  legend.direction = "vertical",
  recalc_size = TRUE,
  ...
)
}
\arguments{
\item{plot}{The plot to facet or a list list(plot, height, width) if guess_size is TRUE}

\item{facet_by}{The column(s) to split data by and plot separately or facet by
If NULL, no faceting will be done}

\item{facet_scales}{Whether to scale the axes of facets.}

\item{nrow}{The number of rows in facet_wrap}

\item{ncol}{The number of columns in facet_wrap}

\item{byrow}{Whether to fill the plots by row}

\item{legend.position}{The position of the legend}

\item{legend.direction}{The direction of the legend}

\item{recalc_size}{Whether to re-calculate the size of the plot}

\item{...}{Additional arguments to pass to facet_wrap or facet_grid}
}
\value{
The faceted plot. If guess_size is TRUE, attr(p, "height") and attr(p, "width") will be set
}
\description{
Facetting a plot
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{palette_list}
\alias{palette_list}
\title{A list of palettes for use in data visualization}
\description{
A list of palettes for use in data visualization
}
\examples{
\dontrun{
if (interactive()) {
  library(stringr)
  library(RColorBrewer)
  library(Redmonder)
  library(rcartocolor)
  library(nord)
  library(viridis)
  library(pals)
  library(dichromat)
  library(jcolors)
  library(scales)
  library(ggthemes)
  syspals <- utils::getFromNamespace("syspals", "pals")
  brewer.pal.info <- RColorBrewer::brewer.pal.info
  ggsci_db <- utils::getFromNamespace("ggsci_db", "ggsci")
  redmonder.pal.info <- Redmonder::redmonder.pal.info
  metacartocolors <- rcartocolor::metacartocolors
  rownames(metacartocolors) <- metacartocolors$Name
  nord_palettes <- nord::nord_palettes
  viridis_names <- c("magma", "inferno", "plasma", "viridis", "cividis", "rocket",
     "mako", "turbo")
  viridis_palettes <- lapply(stats::setNames(viridis_names, viridis_names),
     function(x) viridis::viridis(100, option = x))
  ocean_names <- names(syspals)[grep("ocean", names(syspals))]
  ocean_palettes <- syspals[ocean_names]
  dichromat_palettes <- dichromat::colorschemes
  jcolors_names <- paste0("jcolors-", c("default", "pal2", "pal3", "pal4", "pal5",
     "pal6", "pal7", "pal8", "pal9", "pal10", "pal11", "pal12", "rainbow"))
  custom_names <- c("jet", "simspec", "GdRd")
  custom_palettes <- list(
    oompaBase::jetColors(N = 100),
    c("#c22b86", "#f769a1", "#fcc5c1", "#253777", "#1d92c0", "#9ec9e1", "#015b33",
        "#42aa5e", "#d9f0a2", "#E66F00", "#f18c28", "#FFBB61"),
    c("gold", "red3")
  )
  names(custom_palettes) <- custom_names
  seurat_discrete_palettes <- list(
     alphabet = c(
       "#F0A0FF", "#0075DC", "#993F00", "#4C005C", "#191919", "#005C31",
       "#2BCE48", "#FFCC99", "#808080", "#94FFB5", "#8F7C00", "#9DCC00",
       "#C20088", "#003380", "#FFA405", "#FFA8BB", "#426600", "#FF0010",
       "#5EF1F2", "#00998F", "#E0FF66", "#740AFF", "#990000", "#FFFF80",
       "#FFE100", "#FF5005"
     ),
     alphabet2 = c(
       "#AA0DFE", "#3283FE", "#85660D", "#782AB6", "#565656", "#1C8356",
       "#16FF32", "#F7E1A0", "#E2E2E2", "#1CBE4F", "#C4451C", "#DEA0FD",
       "#FE00FA", "#325A9B", "#FEAF16", "#F8A19F", "#90AD1C", "#F6222E",
       "#1CFFCE", "#2ED9FF", "#B10DA1", "#C075A6", "#FC1CBF", "#B00068",
       "#FBE426", "#FA0087"
     ),
     glasbey = c(
       "#0000FF", "#FF0000", "#00FF00", "#000033", "#FF00B6", "#005300",
       "#FFD300", "#009FFF", "#9A4D42", "#00FFBE", "#783FC1", "#1F9698",
       "#FFACFD", "#B1CC71", "#F1085C", "#FE8F42", "#DD00FF", "#201A01",
       "#720055", "#766C95", "#02AD24", "#C8FF00", "#886C00", "#FFB79F",
       "#858567", "#A10300", "#14F9FF", "#00479E", "#DC5E93", "#93D4FF",
       "#004CFF", "#F2F318"
     ),
     polychrome = c(
       "#5A5156", "#E4E1E3", "#F6222E", "#FE00FA", "#16FF32", "#3283FE",
       "#FEAF16", "#B00068", "#1CFFCE", "#90AD1C", "#2ED9FF", "#DEA0FD",
       "#AA0DFE", "#F8A19F", "#325A9B", "#C4451C", "#1C8356", "#85660D",
       "#B10DA1", "#FBE426", "#1CBE4F", "#FA0087", "#FC1CBF", "#F7E1A0",
       "#C075A6", "#782AB6", "#AAF400", "#BDCDFF", "#822E1C", "#B5EFB5",
       "#7ED7D1", "#1C7F93", "#D85FF7", "#683B79", "#66B0FF", "#3B00FB"
     ),
     stepped = c(
       "#990F26", "#B33E52", "#CC7A88", "#E6B8BF", "#99600F", "#B3823E",
       "#CCAA7A", "#E6D2B8", "#54990F", "#78B33E", "#A3CC7A", "#CFE6B8",
       "#0F8299", "#3E9FB3", "#7ABECC", "#B8DEE6", "#3D0F99", "#653EB3",
       "#967ACC", "#C7B8E6", "#333333", "#666666", "#999999", "#CCCCCC"
     ),
     parade = c(
       '#ff6969', '#9b37ff', '#cd3737', '#69cdff', '#ffff69', '#69cdcd',
       '#9b379b', '#3737cd', '#ffff9b', '#cdff69', '#ff9b37', '#37ffff',
       '#9b69ff', '#37cd69', '#ff3769', '#ff3737', '#37ff9b', '#cdcd37',
       '#3769cd', '#37cdff', '#9b3737', '#ff699b', '#9b9bff', '#cd9b37',
       '#69ff37', '#cd3769', '#cd69cd', '#cd6937', '#3737ff', '#cdcd69',
       '#ff9b69', '#cd37cd', '#9bff37', '#cd379b', '#cd6969', '#69ff9b',
       '#ff379b', '#9bff9b', '#6937ff', '#69cd37', '#cdff37', '#9bff69',
       '#9b37cd', '#ff37ff', '#ff37cd', '#ffff37', '#37cd9b', '#379bff',
       '#ffcd37', '#379b37', '#ff9bff', '#379b9b', '#69ffcd', '#379bcd',
       '#ff69ff', '#ff9b9b', '#37ff69', '#ff6937', '#6969ff', '#699bff',
       '#ffcd69', '#69ffff', '#37ff37', '#6937cd', '#37cd37', '#3769ff',
       '#cd69ff', '#6969cd', '#9bcd37', '#69ff69', '#37cdcd', '#cd37ff',
       '#37379b', '#37ffcd', '#69cd69', '#ff69cd', '#9bffff', '#9b9b37'
     )
  )
  seurat_continuous_palettes <- list(
    seurat = hue_pal()(16),
    seurat.16 = hue_pal()(16),
    seurat.32 = hue_pal()(32),
    seurat.64 = hue_pal()(64)
  )
  stripe_palettes <- list(
    stripe = rep(c("white", "grey60"), 8),
    stripe.16 = rep(c("white", "grey60"), 8),
    stripe.32 = rep(c("white", "grey60"), 16),
    stripe.64 = rep(c("white", "grey60"), 32)
  )
  tableau_palettes <- list()
  orig_tableau_palettes <- ggthemes::ggthemes_data[["tableau"]][["color-palettes"]]
  for (g in names(orig_tableau_palettes)) {
    for (pal in names(orig_tableau_palettes[[g]])) {
      palcolors <- as.list(orig_tableau_palettes[[g]][[pal]])
      if (!is.null(palcolors$name)) {
        tableau_palettes[[pal]] <- stats::setNames(palcolors$value, palcolors$name)
      } else {
        tableau_palettes[[pal]] <- palcolors$value
      }
    }
  }

  palette_list <- list()
  all_colors <- c(
    rownames(brewer.pal.info), names(ggsci_db), rownames(redmonder.pal.info),
    rownames(metacartocolors), names(nord_palettes), names(viridis_palettes),
    ocean_names, names(dichromat_palettes), jcolors_names, names(seurat_discrete_palettes),
    names(seurat_continuous_palettes), custom_names, names(stripe_palettes),
    names(tableau_palettes)
  )
  for (pal in all_colors) {
    if (!pal \%in\% all_colors) {
      stop(paste0("Invalid pal Must be one of ", paste0(all_colors, collapse = ",")))
    }
    if (pal \%in\% rownames(brewer.pal.info)) {
      pal_n <- brewer.pal.info[pal, "maxcolors"]
      pal_category <- brewer.pal.info[pal, "category"]
      if (pal_category == "div") {
        palcolor <- rev(brewer.pal(name = pal, n = pal_n))
      } else {
        if (pal == "Paired") {
          palcolor <- brewer.pal(12, "Paired")[c(1:4, 7, 8, 5, 6, 9, 10, 11, 12)]
        } else {
          palcolor <- brewer.pal(name = pal, n = pal_n)
        }
      }
      if (pal_category == "qual") {
        attr(palcolor, "type") <- "discrete"
      } else {
        attr(palcolor, "type") <- "continuous"
      }
    } else if (pal \%in\% names(ggsci_db)) {
      if (pal \%in\% c("d3", "uchicago", "material")) {
        for (subpal in names(ggsci_db[[pal]])) {
          palcolor <- ggsci_db[[pal]][[subpal]]
          if (pal == "material") {
            attr(palcolor, "type") <- "continuous"
          } else {
            attr(palcolor, "type") <- "discrete"
          }
          palette_list[[paste0(pal, "-", subpal)]] <- palcolor
        }
        next
      } else {
        palcolor <- ggsci_db[[pal]][[1]]
        if (pal == "gsea") {
          attr(palcolor, "type") <- "continuous"
        } else {
          attr(palcolor, "type") <- "discrete"
        }
      }
    } else if (pal \%in\% rownames(redmonder.pal.info)) {
      pal_n <- redmonder.pal.info[pal, "maxcolors"]
      pal_category <- redmonder.pal.info[pal, "category"]
      if (pal_category == "div") {
        palcolor <- rev(redmonder.pal(name = pal, n = pal_n))
      } else {
        palcolor <- redmonder.pal(name = pal, n = pal_n)
      }
      if (pal_category == "qual") {
        attr(palcolor, "type") <- "discrete"
      } else {
        attr(palcolor, "type") <- "continuous"
      }
    } else if (pal \%in\% rownames(metacartocolors)) {
      pal_n <- metacartocolors[pal, "Max_n"]
      palcolor <- carto_pal(name = pal, n = pal_n)
      if (pal_category == "qualitative") {
        attr(palcolor, "type") <- "discrete"
      } else {
        attr(palcolor, "type") <- "continuous"
      }
    } else if (pal \%in\% names(nord_palettes)) {
      palcolor <- nord_palettes[[pal]]
      attr(palcolor, "type") <- "discrete"
    } else if (pal \%in\% names(viridis_palettes)) {
      palcolor <- viridis_palettes[[pal]]
      attr(palcolor, "type") <- "continuous"
    } else if (pal \%in\% names(ocean_palettes)) {
      palcolor <- ocean_palettes[[pal]]
      attr(palcolor, "type") <- "continuous"
    } else if (pal \%in\% names(dichromat_palettes)) {
      palcolor <- dichromat_palettes[[pal]]
      if (pal \%in\% c("Categorical.12", "SteppedSequential.5")) {
        attr(palcolor, "type") <- "discrete"
      } else {
        attr(palcolor, "type") <- "continuous"
      }
    } else if (pal \%in\% jcolors_names) {
      palcolor <- jcolors(palette = gsub("jcolors-", "", pal))
      if (pal \%in\% paste0("jcolors-", c("pal10", "pal11", "pal12", "rainbow"))) {
        attr(palcolor, "type") <- "continuous"
      } else {
        attr(palcolor, "type") <- "discrete"
      }
    } else if (pal \%in\% custom_names) {
      palcolor <- custom_palettes[[pal]]
      if (pal \%in\% c("jet")) {
        attr(palcolor, "type") <- "continuous"
      } else {
        attr(palcolor, "type") <- "discrete"
      }
    } else if (pal \%in\% names(seurat_discrete_palettes)) {
      palcolor <- seurat_discrete_palettes[[pal]]
      attr(palcolor, "type") <- "discrete"
    } else if (pal \%in\% names(seurat_continuous_palettes)) {
      palcolor <- seurat_continuous_palettes[[pal]]
      attr(palcolor, "type") <- "continuous"
    } else if (pal \%in\% names(stripe_palettes)) {
      palcolor <- stripe_palettes[[pal]]
      attr(palcolor, "type") <- "discrete"
    } else if (pal \%in\% names(tableau_palettes)) {
      palcolor <- tableau_palettes[[pal]]
      attr(palcolor, "type") <- "discrete"
    }
    palette_list[[pal]] <- palcolor
  }
}
}
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chordplot.R
\name{ChordPlot}
\alias{ChordPlot}
\alias{CircosPlot}
\title{Chord / Circos plot}
\usage{
ChordPlot(
  data,
  y = NULL,
  from = NULL,
  from_sep = "_",
  to = NULL,
  to_sep = "_",
  split_by = NULL,
  split_by_sep = "_",
  flip = FALSE,
  links_color = c("from", "to"),
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 0.5,
  labels_rot = FALSE,
  title = NULL,
  subtitle = NULL,
  seed = 8525,
  keep_na = FALSE,
  keep_empty = FALSE,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)

CircosPlot(
  data,
  y = NULL,
  from = NULL,
  from_sep = "_",
  to = NULL,
  to_sep = "_",
  split_by = NULL,
  split_by_sep = "_",
  flip = FALSE,
  links_color = c("from", "to"),
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 0.5,
  labels_rot = FALSE,
  title = NULL,
  subtitle = NULL,
  seed = 8525,
  keep_na = FALSE,
  keep_empty = FALSE,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{y}{A character string specifying the column name of the data frame to plot for the y-axis.}

\item{from}{A character string (or vector) specifying the column name(s)
for the source nodes.  Character/factor columns are expected.  Multiple
columns are concatenated with \code{from_sep}.}

\item{from_sep}{A character string to join multiple \code{from} columns.
Default \code{"_"}.}

\item{to}{A character string (or vector) specifying the column name(s)
for the target nodes.  Character/factor columns are expected.  Multiple
columns are concatenated with \code{to_sep}.}

\item{to_sep}{A character string to join multiple \code{to} columns.
Default \code{"_"}.}

\item{split_by}{The column(s) to split the data by for separate sub-diagrams.
Multiple columns are concatenated with \code{split_by_sep}.}

\item{split_by_sep}{A character string to separate concatenated
\code{split_by} columns.  Default \code{"_"}.}

\item{flip}{Logical; if \code{TRUE}, swap the source and target nodes,
reversing the link direction.}

\item{links_color}{A character string controlling which node's colour
each link ribbon takes: \code{"from"} (default) or \code{"to"}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{labels_rot}{Logical; if \code{TRUE}, rotate sector labels by 90
degrees (clockwise).  Default \code{FALSE} uses \code{niceFacing} for
automatic orientation.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{seed}{A numeric seed for reproducibility.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{combine}{Logical; when \code{TRUE} (default), returns a combined
\code{patchwork} object.  When \code{FALSE}, returns a named list of
individual wrapped elements.}

\item{ncol, nrow}{Integer number of columns / rows for the combined layout.}

\item{byrow}{Logical; fill the combined layout by row (default \code{TRUE}).}

\item{axes, axis_titles}{Character strings for axis handling in the
combined layout.}

\item{guides}{Character string for legend collection across panels.}

\item{design}{A custom layout design for the combined plot.}

\item{...}{Additional arguments.}
}
\value{
A \code{patchwork} object or a named list of wrapped elements
(when \code{combine = FALSE}), each with \code{height} and \code{width}
attributes in inches.
}
\description{
Draws a chord diagram (also known as a circos plot) to visualise
relationships between two categorical variables.  Categories are arranged
around a circle, and connecting ribbons (links) represent the flow or
association between source and target nodes.  The width of each link is
proportional to the associated numeric value or observation count.

The function supports \strong{count aggregation} (omit \code{y} to plot
observation counts per pair), \strong{link colouring} by source or target
node, \strong{label rotation} options, and splitting into separate
sub-diagrams via \code{split_by}.

\code{CircosPlot} is an alias of \code{ChordPlot}.

\code{CircosPlot} is an alias for \code{\link{ChordPlot}}.
}
\section{split_by workflow}{

When \code{split_by} is provided:
\enumerate{
\item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
normalise the \code{keep_na} / \code{keep_empty} arguments for all
columns (\code{split_by}, \code{from}, \code{to}).
\item The \code{split_by} column is validated and its NA / empty levels
are processed.  It is then removed from the per-column lists.
\item The data is split by \code{split_by} (preserving level order).  If
\code{split_by} is \code{NULL}, the data is wrapped in a
single-element list with name \code{"..."}.
\item Per-split \code{palette} and \code{palcolor} are resolved via
\code{\link{check_palette}()} and \code{\link{check_palcolor}()}.
\item \code{\link{ChordPlotAtomic}()} is called for each split.
When \code{title} is a function, it receives the split level name
for dynamic titles.
\item Results are combined via \code{\link{combine_plots}()}.
}
}

\examples{
\donttest{
set.seed(8525)
data <- data.frame(
    nodes1 = sample(c("Soure1", "Source2", "Source3"), 10, replace = TRUE),
    nodes2 = sample(letters[1:3], 10, replace = TRUE),
    y = sample(1:5, 10, replace = TRUE)
)

# Basic chord diagram (counts)
ChordPlot(data, from = "nodes1", to = "nodes2")

# Links coloured by target + rotated labels
ChordPlot(data, from = "nodes1", to = "nodes2",
          links_color = "to", labels_rot = TRUE)

# With explicit y values (link thickness)
ChordPlot(data, from = "nodes1", to = "nodes2", y = "y")

# Split by a column — one diagram per split level
ChordPlot(data, from = "nodes1", to = "nodes2", split_by = "y")

# Per-split palettes
ChordPlot(data, from = "nodes1", to = "nodes2", split_by = "y",
          palette = c("1" = "Reds", "2" = "Blues",
                      "3" = "Greens", "4" = "Purp"))

# Flip source/target direction
ChordPlot(data, from = "nodes1", to = "nodes2", flip = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{blend_colors}
\alias{blend_colors}
\title{Blend colors}
\usage{
blend_colors(colors, mode = c("blend", "average", "screen", "multiply"))
}
\arguments{
\item{colors}{Color vectors.}

\item{mode}{Blend mode. One of "blend", "average", "screen", or "multiply".}
}
\value{
The blended color.
}
\description{
This function blends a list of colors using the specified blend mode.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrich.R
\name{EnrichNetworkAtomic}
\alias{EnrichNetworkAtomic}
\title{Atomic enrichment network (internal)}
\usage{
EnrichNetworkAtomic(
  data,
  top_term = 6,
  metric = "p.adjust",
  character_width = 50,
  layout = "fr",
  layoutadjust = TRUE,
  adjscale = 60,
  adjiter = 100,
  blendmode = "blend",
  labelsize = 5,
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525,
  ...
)
}
\arguments{
\item{data}{A data frame containing enrichment results in clusterProfiler
format (see \code{\link{EnrichMapAtomic}} for the expected columns).}

\item{top_term}{An integer specifying the maximum number of terms to
include.  Terms are ranked by \code{metric} (ascending).  Default
\code{6}.}

\item{metric}{A character string specifying the significance metric for
top-term selection: \code{"p.adjust"} (default) or \code{"pvalue"}.}

\item{character_width}{An integer specifying the maximum width (in
characters) at which term descriptions are wrapped via
\code{strwrap(width = character_width)}.  Default \code{50}.}

\item{layout}{A character string naming the igraph layout algorithm.
Built-in shortcuts: \code{"circle"}, \code{"tree"}, \code{"grid"}.
Otherwise, the suffix passed to \code{layout_with_<layout>} in igraph.
Default \code{"fr"}.}

\item{layoutadjust}{A logical value.  When \code{TRUE} (default),
applies \code{\link{adjust_network_layout}()} after the initial layout
to reduce node overlap based on label width and a repulsion simulation.}

\item{adjscale}{A numeric value controlling the scale of the layout
adjustment.  Passed as the \code{scale} argument to
\code{\link{adjust_network_layout}()}.  Default \code{60}.}

\item{adjiter}{A numeric value controlling the number of iterations for
the layout adjustment.  Passed as the \code{iter} argument to
\code{\link{adjust_network_layout}()}.  Default \code{100}.}

\item{blendmode}{A character string specifying how gene colours are
computed from the colours of the terms they belong to.  One of
\code{"blend"} (default), \code{"average"}, \code{"multiply"}, or
\code{"screen"}.  Passed to \code{blend_colors()}.}

\item{labelsize}{A numeric value specifying the font size of the
numeric term labels displayed via \code{ggrepel::geom_text_repel()}.
Default \code{5}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{seed}{The random seed to use. Default is 8525.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a single enrichment network -- a
term-gene bipartite graph where enriched terms and their member genes
are shown as interconnected nodes.  Term nodes are displayed as numbered
filled circles with a colour-coded legend; gene nodes are displayed as
labelled rectangles coloured by a blend of the term colours they belong
to.  This is the workhorse behind the exported \code{\link{EnrichNetwork}}
function -- it takes a \strong{single} data frame (no \code{split_by} support)
and returns a \code{ggplot} object.

The function constructs a bipartite graph between terms and genes,
computes a force-directed layout, optionally adjusts node positions to
reduce overlap, blends term colours for shared genes, and renders the
result as a labelled network.
}
\section{Architecture}{

\enumerate{
\item \strong{ggplot dispatch} -- selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{Data format conversion} -- if \code{data} inherits from
\code{"enrichResult"}, converts via \code{as.data.frame()}.
\item \strong{Column validation} -- \code{\link{check_columns}()}
verifies that \code{Description}, \code{GeneRatio},
\code{pvalue}, \code{p.adjust}, and \code{geneID} are present.
\item \strong{Top-term selection} -- when \code{top_term} is not
\code{NULL}, selects the top N terms by \code{metric} via
\code{dplyr::slice_min()}.
\item \strong{Metric and description preparation} -- computes
\eqn{-\log_{10}(metric)} as the scoring variable.  Wraps
\code{Description} text to \code{character_width} and parses
\code{geneID} by splitting on \code{"/"}.
\item \strong{Gene-term unnesting} -- unnests the \code{geneID}
column to produce a gene-term mapping table.
\item \strong{Bipartite node construction} -- creates nodes for both
terms (\code{class = "term"}) and genes (\code{class = "gene"}),
carrying the \code{Database} attribute if present.
\item \strong{Bipartite edge construction} -- edges connect each term
to its member genes with uniform weight (\code{weight = 1}).
\item \strong{igraph graph construction} --
\code{igraph::graph_from_data_frame()} builds an undirected
bipartite graph.
\item \strong{Layout computation} -- dispatches to the chosen igraph
layout function (\code{layout_with_*}) or built-in shortcuts
(\code{"circle"}, \code{"tree"}, \code{"grid"}).
\item \strong{Layout adjustment} -- when \code{layoutadjust = TRUE}
(default), calls \code{\link{adjust_network_layout}()} to push
overlapping nodes apart based on label width and a repulsion
simulation.
\item \strong{Node coordinates} -- extracts vertex positions into
\code{dim1} and \code{dim2} columns.
\item \strong{Colour computation} -- palette colours are assigned to
term nodes.  Gene node colours are computed by blending the
colours of all connected terms via the specified
\code{blendmode} using \code{blend_colors()}.
\item \strong{Label colour} -- each node's text colour is chosen as
black or white based on the luminance of its fill colour (sum
of RGB channels > 510).
\item \strong{Numeric labels} -- term nodes receive sequential integer
labels; gene nodes display their gene symbol.
\item \strong{Custom legend key} -- \code{draw_key_cust()} renders
term legend entries as numbered circles via
\code{ggrepel::shadowtextGrob()}.
\item \strong{Edge rendering} -- \code{ggplot2::geom_segment()} draws
edges coloured by the source term (legend suppressed).
\item \strong{Gene node rendering} -- \code{ggplot2::geom_label()}
displays gene symbols with fill colour blended from connected
terms.
\item \strong{Term node rendering} -- two \code{geom_point()} layers
draw black-outlined circles filled with the term colour, with
\code{draw_key_cust} as the key glyph.
\item \strong{Term labels} -- \code{ggrepel::geom_text_repel()}
places the numeric term labels with white text on a black
background.
\item \strong{Scale configuration} -- \code{scale_color_identity()}
and \code{scale_fill_identity()} with a manual legend mapping
term colours to term descriptions.
\item \strong{Theme and dimensions} -- applies the resolved theme,
sets aspect ratio and legend position, then calls
\code{\link{calculate_plot_dimensions}()} to attach
\code{height}/\code{width} attributes.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roccurve.R
\name{ROCCurveAtomic}
\alias{ROCCurveAtomic}
\title{Atomic ROC curve (internal)}
\usage{
ROCCurveAtomic(
  data,
  truth_by,
  score_by,
  pos_label = NULL,
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  x_axis_reverse = FALSE,
  percent = FALSE,
  ci = NULL,
  n_cuts = 0,
  cutoffs_at = NULL,
  cutoffs_labels = NULL,
  cutoffs_accuracy = 0.01,
  cutoffs_pt_size = 5,
  cutoffs_pt_shape = 4,
  cutoffs_pt_stroke = 1,
  cutoffs_labal_fg = "black",
  cutoffs_label_size = 4,
  cutoffs_label_bg = "white",
  cutoffs_label_bg_r = 0.1,
  show_auc = c("auto", "none", "legend", "plot"),
  auc_accuracy = 0.01,
  auc_size = 4,
  increasing = TRUE,
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  aspect.ratio = 1,
  legend.position = waiver(),
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = ifelse(x_axis_reverse, "Specificity", "1 - Specificity"),
  ylab = "Sensitivity",
  ...
)
}
\arguments{
\item{data}{A data frame with the truth and score columns. See
\url{https://CRAN.R-project.org/package=plotROC}
for the expected format.}

\item{truth_by}{A character string naming the column that contains the true
class labels (binary outcome, 0/1 or TRUE/FALSE).}

\item{score_by}{A character vector of column names containing the predicted
scores (classifier output values). When multiple columns are provided, each
column becomes a separate ROC curve grouped by a \code{.group} identifier.
When multiple columns are used, \code{group_by} must be NULL.}

\item{pos_label}{A character string specifying the positive class label in
\code{truth_by}. When NULL (default), the labels are handled by the
\code{plotROC} package: if \code{truth_by} is a factor, the last level is
used; otherwise it is coerced to a factor with a warning.}

\item{group_by}{A character vector of column names to group the ROC curve by.
Each unique combination of group values renders a separate ROC curve.
When \code{score_by} contains multiple columns, \code{group_by} must be
NULL because the score columns themselves define the groups. Multiple
\code{group_by} columns are concatenated with \code{group_by_sep}.}

\item{group_by_sep}{A character string used to separate concatenated
\code{group_by} columns. Default: \code{"_"}.}

\item{group_name}{A character string to use as the legend title for the
ROC curve groups. When NULL (default), the \code{group_by} column name
is used.}

\item{x_axis_reverse}{A logical value. If TRUE, the x-axis is reversed (from
1 to 0), displaying specificity instead of 1 - specificity. The x-axis
label automatically changes to \code{"Specificity"}. Default: FALSE.}

\item{percent}{A logical value. If TRUE, the x and y axes are displayed as
percentages (0 to 100). Default: FALSE.}

\item{ci}{A list of arguments passed to \code{plotROC::geom_rocci()} to add
confidence intervals to the ROC curve. When NULL (default), no confidence
intervals are drawn. Example: \code{ci = list(sig.level = 0.05)}.}

\item{n_cuts}{An integer specifying the number of evenly-spaced quantile-based
cutoff points to annotate on the ROC curve. Quantiles are computed from the
\code{score_by} distribution. Default: \code{0} (no quantile cutoffs).
Ignored when \code{cutoffs_at} is non-NULL.}

\item{cutoffs_at}{A vector of user-supplied cutoff values to annotate as
points on the ROC curve. When non-NULL, overrides \code{n_cuts}. Accepts
raw numeric score thresholds and/or named method strings from the
\code{\link[OptimalCutpoints]{optimal.cutpoints}} package for automatic
optimal cutoff identification. Both \code{cutoffs_at} and
\code{cutoffs.labels} are passed to \code{plotROC::geom_roc()}.
Supported method values are:
\itemize{
\item \code{"CB"} (cost-benefit method);
\item \code{"MCT"} (minimises Misclassification Cost Term);
\item \code{"MinValueSp"} (a minimum value set for Specificity);
\item \code{"MinValueSe"} (a minimum value set for Sensitivity);
\item \code{"ValueSe"} (a value set for Sensitivity);
\item \code{"MinValueSpSe"} (a minimum value set for Specificity and Sensitivity);
\item \code{"MaxSp"} (maximises Specificity);
\item \code{"MaxSe"} (maximises Sensitivity);
\item \code{"MaxSpSe"} (maximises Sensitivity and Specificity simultaneously);
\item \code{"MaxProdSpSe"} (maximises the product of Sensitivity and Specificity);
\item \code{"ROC01"} (minimises distance between ROC plot and point (0,1));
\item \code{"SpEqualSe"} (Sensitivity = Specificity);
\item \code{"Youden"} (Youden Index);
\item \code{"MaxEfficiency"} (maximises Efficiency/Accuracy);
\item \code{"Minimax"} (minimises the most frequent error);
\item \code{"MaxDOR"} (maximises Diagnostic Odds Ratio);
\item \code{"MaxKappa"} (maximises Kappa Index);
\item \code{"MinValueNPV"} (a minimum value set for Negative Predictive Value);
\item \code{"MinValuePPV"} (a minimum value set for Positive Predictive Value);
\item \code{"ValueNPV"} (a value set for Negative Predictive Value);
\item \code{"ValuePPV"} (a value set for Positive Predictive Value);
\item \code{"MinValueNPVPPV"} (a minimum value set for Predictive Values);
\item \code{"PROC01"} (minimises distance between PROC plot and point (0,1));
\item \code{"NPVEqualPPV"} (Negative Predictive Value = Positive Predictive Value);
\item \code{"MaxNPVPPV"} (maximises Positive and Negative Predictive Values simultaneously);
\item \code{"MaxSumNPVPPV"} (maximises the sum of the Predictive Values);
\item \code{"MaxProdNPVPPV"} (maximises the product of Predictive Values);
\item \code{"ValueDLR.Negative"} (a value set for Negative Diagnostic Likelihood Ratio);
\item \code{"ValueDLR.Positive"} (a value set for Positive Diagnostic Likelihood Ratio);
\item \code{"MinPvalue"} (minimises p-value of the Chi-squared test);
\item \code{"ObservedPrev"} (closest value to observed prevalence);
\item \code{"MeanPrev"} (closest value to the mean of the test values);
\item \code{"PrevalenceMatching"} (predicted prevalence equals observed prevalence).
}}

\item{cutoffs_labels}{A character vector of user-supplied labels for the
cutoff points. Must be the same length as \code{cutoffs_at}. When NULL,
labels are generated automatically (score value or method name).}

\item{cutoffs_accuracy}{A numeric value controlling the rounding precision of
automatically generated cutoff labels. Default: \code{0.01}.}

\item{cutoffs_pt_size}{A numeric value specifying the size of the cutoff
point markers. Default: \code{5}.}

\item{cutoffs_pt_shape}{A numeric value specifying the shape of the cutoff
point markers. Default: \code{4} (cross).}

\item{cutoffs_pt_stroke}{A numeric value specifying the stroke width of the
cutoff point markers. Default: \code{1}.}

\item{cutoffs_labal_fg}{A character string specifying the text colour of
the cutoff labels. Default: \code{"black"}.}

\item{cutoffs_label_size}{A numeric value specifying the font size of the
cutoff labels. Default: \code{4}.}

\item{cutoffs_label_bg}{A character string specifying the background colour
of the cutoff labels. Default: \code{"white"}.}

\item{cutoffs_label_bg_r}{A numeric value specifying the background radius
of the cutoff labels (passed to \code{ggrepel::geom_text_repel()}).
Default: \code{0.1}.}

\item{show_auc}{A character string specifying the display mode for AUC values:
\itemize{
\item \code{"auto"} (default): Automatically determine the position. When
there is a single group or \code{facet_by} is provided, AUC is placed on
the plot; otherwise AUC is placed in the legend.
\item \code{"none"}: Do not display AUC values.
\item \code{"legend"}: Display AUC values in the legend labels.
\item \code{"plot"}: Display AUC values as text on the plot.
}}

\item{auc_accuracy}{A numeric value controlling the rounding precision of
AUC values in labels. Default: \code{0.01}.}

\item{auc_size}{A numeric value specifying the font size of AUC labels when
displayed on the plot. Default: \code{4}.}

\item{increasing}{A logical value. If TRUE (default), higher scores indicate
the positive class; if FALSE, lower scores indicate the positive class.
Controls the direction of comparison in the ROC analysis.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached, plus \code{attr(p, "auc")} and
\code{attr(p, "cutoffs")} data frames.
}
\description{
Core implementation for drawing a single Receiver Operating Characteristic
(ROC) curve. This is the internal workhorse behind the exported
\code{\link{ROCCurve}} function. It takes a \strong{single} data frame (no
\code{split_by} support) and returns a \code{ggplot} object.
}
\details{
The function produces an ROC curve using \code{plotROC::geom_roc()}, with
the following capabilities:
\itemize{
\item \strong{Multiple classifiers} — \code{score_by} accepts multiple
column names, automatically pivoting them into a grouped format so
several prediction scores can be compared on a single plot.
\item \strong{AUC calculation} — area under the curve is computed via
\code{plotROC::calc_auc()} and displayed either on the plot or in the
legend, controlled by \code{show_auc}.
\item \strong{Cutoff annotation} — user-specified cutoffs (numeric score
thresholds or named optimal-cutoff methods from the
\code{OptimalCutpoints} package) are rendered as markers with labels,
using \code{ggrepel::geom_text_repel()} for label placement.
\item \strong{Confidence intervals} — optional ROC confidence bands via
\code{plotROC::geom_rocci()}.
\item \strong{Axis flexibility} — supports reversed x-axis (displaying
specificity) and percent-scaled axes.
}
}
\section{Architecture}{

\enumerate{
\item \strong{show_auc resolution} — \code{match.arg()} resolves
\code{show_auc} to one of \code{"auto"}, \code{"none"}, \code{"legend"},
or \code{"plot"}.
\item \strong{Column validation} — \code{\link{check_columns}()} validates
\code{truth_by} (single column), \code{score_by} (multiple allowed), and
\code{group_by} (factor, multi-column concatenated). An error is raised if
\code{group_by} is provided alongside multiple \code{score_by} columns.
\item \strong{Positive label encoding} — Converts \code{truth_by} to binary
numeric (0/1) with three paths:
\itemize{
\item \code{pos_label} provided: re-factor with \code{pos_label} as the
last level, then convert.
\item \code{truth_by} is a factor: warn that the last level is treated
as positive, then convert.
\item Non-numeric, non-factor: coerce to factor, warn, then convert.
}
\item \strong{Multi-score_by expansion} — When \code{score_by} contains
multiple columns, \code{tidyr::pivot_longer()} reshapes into a single
\code{.score} column with a \code{.group} identifier, which becomes the
\code{group_by} variable.
\item \strong{ggplot dispatch} — Selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{Dummy group insertion} — When \code{group_by = NULL}, creates
a synthetic \code{..group} column (constant \code{""}) so the curve still
renders. The legend is suppressed (\code{"none"}).
\item \strong{Auto AUC placement} — When \code{show_auc = "auto"},
single-group or faceted plots place AUC on the plot; multi-group plots
place it in the legend.
\item \strong{Base ROC geometry} — \code{plotROC::geom_roc()} with
\code{aes(d = truth, m = score, color = group)}, controlling direction
via the \code{increasing} parameter.
\item \strong{AUC calculation} — Temporarily facets via
\code{\link{facet_plot}()}, then computes per-group (and per-facet) AUC
values via \code{plotROC::calc_auc()}.
\item \strong{Cutoff computation} — \code{get_cutoffs_data()} combines
\code{group_by} and \code{facet_by} columns into a \code{.cat} identifier
and computes per-category cutoff data, supporting both numeric thresholds
and named \code{OptimalCutpoints} methods.
\item \strong{Cutoff rendering} — When cutoff data is non-NULL, splits the
\code{.cat} column back into group/facet columns and adds
\code{geom_point()} (markers) and \code{geom_text_repel()} (labels) with
configurable size, shape, stroke, colour, and background styling.
\item \strong{Confidence intervals} — When \code{ci} is non-NULL,
\code{plotROC::geom_rocci()} is added with the provided arguments.
\item \strong{AUC display} — Three modes:
\itemize{
\item \code{"plot"}: \code{geom_text()} places AUC labels at a corner
position determined by \code{increasing} and \code{x_axis_reverse}.
\item \code{"legend"}: AUC values are appended to the
\code{scale_color_manual()} labels (with per-facet prefixes when
faceting is active).
\item \code{"none"}: group level names are used as-is.
}
\item \strong{Diagonal reference} — \code{geom_abline()} draws the
no-discrimination line (y = x, or y = -x when x-axis is reversed) as a
dashed grey line.
\item \strong{Color scale} — \code{scale_color_manual()} assigns
palette-derived colours via \code{\link{palette_this}()}, with
AUC-augmented labels when \code{show_auc = "legend"}.
\item \strong{Axis formatting} — Percent labels on y-axis (and x-axis)
when \code{percent = TRUE}. X-axis reversed (1 to 0) when
\code{x_axis_reverse = TRUE}, changing the axis label to
\code{"Specificity"}.
\item \strong{Labels and theme} — \code{labs()} sets title, subtitle, x,
and y labels. The theme, aspect ratio, legend position/direction, and
dashed grid lines are applied.
\item \strong{Dimension calculation} — \code{\link{calculate_plot_dimensions}()}
computes \code{height} and \code{width} attributes from
\code{base_height = 4.5}, aspect ratio, and legend metrics.
\item \strong{Attribute storage} — \code{auc} and \code{cutoffs} data
frames are stored as \code{attr(p, "auc")} and
\code{attr(p, "cutoffs")} for retrieval by the exported wrapper.
\item \strong{Faceting} — \code{\link{facet_plot}()} wraps the plot with
\code{facet_wrap} / \code{facet_grid} if \code{facet_by} is provided.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dotplot.R
\name{DotPlotAtomic}
\alias{DotPlotAtomic}
\title{Atomic dot/lollipop plot}
\usage{
DotPlotAtomic(
  data,
  x,
  y,
  x_sep = "_",
  y_sep = "_",
  flip = FALSE,
  lollipop = FALSE,
  size_by = NULL,
  fill_by = NULL,
  fill_cutoff = NULL,
  palreverse = FALSE,
  size_name = NULL,
  fill_name = NULL,
  fill_cutoff_name = NULL,
  size_min = 1,
  size_max = 10,
  theme = "theme_this",
  theme_args = list(),
  palette = "Spectral",
  palcolor = NULL,
  alpha = 1,
  border_color = "black",
  border_size = 0.5,
  border_alpha = 1,
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  x_text_angle = 0,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  add_bg = FALSE,
  bg_palette = "stripe",
  bg_palcolor = NULL,
  bg_alpha = 0.2,
  bg_direction = c("vertical", "horizontal", "v", "h"),
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  keep_na = FALSE,
  keep_empty = FALSE,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character vector specifying the column(s) to use for the x-axis.
Can be numeric (for scatter/lollipop mode) or factor/character (for dot
matrix mode). When multiple columns are provided for a non-numeric axis,
they are concatenated with \code{x_sep} as the separator.}

\item{y}{A character vector specifying the column(s) to use for the y-axis.
Can be numeric (for scatter mode) or factor/character (for dot matrix or
lollipop mode). When multiple columns are provided for a non-numeric axis,
they are concatenated with \code{y_sep} as the separator.}

\item{x_sep}{A character string used to join multiple \code{x} column values
into a single factor level. Only used when x is non-numeric and multiple
columns are provided. Default: \code{"_"}.}

\item{y_sep}{A character string used to join multiple \code{y} column values
into a single factor level. Only used when y is non-numeric and multiple
columns are provided. Default: \code{"_"}.}

\item{flip}{A logical value. If \code{TRUE}, the x and y axes are swapped via
\code{coord_flip()}. Dimension calculation accounts for the flip.
Default: \code{FALSE}.}

\item{lollipop}{A logical value. If \code{TRUE}, renders a lollipop plot
with bars extending from x = 0 to each data point, capped by filled dots.
Requires \code{x} to be numeric and \code{y} to be factor/character.
Default: \code{FALSE}.}

\item{size_by}{A character string naming a numeric column whose values
control dot size. When \code{NULL} (the default), the per-combination
observation count is computed automatically (via \code{dplyr::summarise(n =
  n())}) and used as the size variable. If \code{fill_by} is also present,
the first value of \code{fill_by} per combination is retained with a
warning. A single numeric value is also accepted and sets a constant dot
size (used by \code{ScatterPlot}).}

\item{fill_by}{A character string naming a numeric column whose values
control the fill colour of the dots (and lollipop inner bars). A
continuous gradient from \code{palette} is applied via
\code{scale_fill_gradientn()}. When \code{NULL} (the default), all dots
are filled with a single constant colour from the middle of the palette.}

\item{fill_cutoff}{A string expression specifying which values of
\code{fill_by} to grey out. Format: an operator followed by a number,
e.g. \code{"< 18"}, \code{"<= 18"}, \code{"> 18"}, or \code{">= 18"}.
Values matching the condition are set to \code{NA} and rendered in grey
(\code{"grey80"}), while the rest are coloured by the fill gradient. The
operator determines which side of the threshold is greyed out,
independent of \code{palreverse}. A numeric value is also accepted as
shorthand for \code{"<"} (e.g. \code{18} is equivalent to
\code{"< 18"}). Requires \code{fill_by} to be set.}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{size_name}{A character string for the size legend title. When
\code{NULL} (the default), the \code{size_by} column name is used.}

\item{fill_name}{A character string for the fill colour-bar legend title.
When \code{NULL} (the default), the \code{fill_by} column name is used.}

\item{fill_cutoff_name}{A character string for the fill cutoff legend title
(shown when \code{fill_cutoff} is active). Defaults to
\code{"<fill_by> <fill_cutoff>"}, e.g. \code{"mpg < 18"}.}

\item{size_min}{A numeric value for the smallest dot size in the
\code{scale_size(range = c(size_min, size_max))} range.
Default: \code{1}.}

\item{size_max}{A numeric value for the largest dot size in the
\code{scale_size(range = c(size_min, size_max))} range.
Default: \code{10}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{border_color}{Controls the dot border colour and lollipop outer-shadow
appearance:
\itemize{
\item \code{TRUE} — dot borders and lollipop inner bars follow the
\code{fill_by} gradient via \code{scale_color_gradientn()}; lollipop
outer shadow is black.
\item \code{"black"} (default) — constant black borders on dots and
black outer shadow on lollipop bars.
\item A colour string (e.g. \code{"red"}, \code{"#FF0000"}) — constant
colour for both dot borders and lollipop outer shadows.
\item \code{FALSE} — no dot borders and no lollipop outer shadow (the
inner coloured bars remain visible in lollipop mode).
}}

\item{border_size}{A numeric value for the stroke width of dot borders and
the base linewidth of lollipop bars. In lollipop mode, the outer shadow
uses \code{border_size * 4} and the inner bar uses \code{border_size * 2}.
Default: \code{0.5}.}

\item{border_alpha}{A numeric value in \code{[0, 1]} controlling the
transparency of dot borders and lollipop bar segments.
Default: \code{1}.}

\item{lower_quantile, upper_quantile}{Lower and upper quantiles for the continuous color/fill scale.
The actual cutoffs are determined by these quantiles when \code{lower_cutoff} and
\code{upper_cutoff} are \code{NULL}. Defaults: \code{lower_quantile = 0}, \code{upper_quantile = 0.99}.}

\item{lower_cutoff, upper_cutoff}{Explicit lower and upper cutoffs for the continuous color/fill scale.
When \code{NULL} (the default), the cutoffs are determined by \code{lower_quantile} and
\code{upper_quantile} via \code{\link[stats]{quantile}}. Values outside the
\code{[lower_cutoff, upper_cutoff]} range are clamped (winsorized) to the nearest cutoff value.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{add_bg}{A logical value. If \code{TRUE}, alternating background
stripes are drawn behind the points via \code{\link{bg_layer}()}. The
striped axis is determined by \code{bg_direction}. Requires the striped
axis to be non-numeric. Default: \code{FALSE}.}

\item{bg_palette}{A character string specifying the palette for the
background stripe colours. Passed to \code{\link{bg_layer}()}.
Default: \code{"stripe"}.}

\item{bg_palcolor}{A character vector of colours for the background stripes.
Passed to \code{\link{bg_layer}()}. When \code{NULL} (default), colours
are derived from \code{bg_palette}.}

\item{bg_alpha}{A numeric value in \code{[0, 1]} for the transparency of
the background stripes. Default: \code{0.2}.}

\item{bg_direction}{A character string specifying which axis receives the
alternating background stripes. \code{"vertical"} (default) stripes by x
levels; \code{"horizontal"} stripes by y levels. Abbreviations \code{"v"}
and \code{"h"} are also accepted.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes.
}
\description{
Core implementation for dot plots and lollipop plots. This is the internal
workhorse dispatched by both \code{DotPlot()} and \code{LollipopPlot()}. It
renders a matrix of points where dot size encodes one variable and dot fill
colour encodes another, with optional background stripes and a lollipop
(bar + dot) display mode.
}
\details{
The function supports two display modes:
\itemize{
\item \strong{Dot plot} (\code{lollipop = FALSE}, the default) — renders
\code{ggplot2::geom_point()} with \code{shape = 21} (filled circle). Either
or both axes can be numeric (producing a scatter plot) or factor (producing
a dot matrix). Dot size scales by \code{size_by} (or the per-combination
observation count when \code{size_by} is \code{NULL}) via
\code{scale_size(range = c(size_min, size_max))}. Dot fill follows the
\code{fill_by} column via \code{scale_fill_gradientn()}.
\item \strong{Lollipop plot} (\code{lollipop = TRUE}) — expects a numeric
\code{x} and a factor/character \code{y}. Renders a two-layer bar:
an outer shadow segment followed by an inner coloured segment, each capped
by a filled dot. The outer shadow is black (or a custom colour when
\code{border_color} is a string), and the inner segment + dot fill follow
\code{fill_by}.
}

When \code{add_bg = TRUE}, alternating background stripes are drawn via
\code{\link{bg_layer}()} along the discrete axis (vertical stripes for
factor \code{x}, horizontal stripes for factor \code{y}). The fill colour
scale is prepared via \code{\link{prepare_continuous_color_scale}()} and
can be trimmed with \code{lower_quantile}/\code{upper_quantile} or
\code{lower_cutoff}/\code{upper_cutoff}.

When \code{fill_cutoff} is set (e.g. \code{"< 18"}), values of
\code{fill_by} matching the condition are set to \code{NA} and rendered in
grey ("grey80"), with a separate legend entry documenting the cutoff.

\strong{Border colour modes:}
\itemize{
\item \code{border_color = TRUE} — dot borders and lollipop inner bar
colour track the \code{fill_by} gradient via
\code{scale_color_gradientn()}; lollipop outer shadow is black.
\item \code{border_color = "black"} (default) — constant black borders on
dots and a black outer shadow on lollipop bars.
\item \code{border_color = FALSE} — no dot borders, no lollipop outer
shadow (the inner coloured segment remains).
}
}
\section{Architecture}{


\strong{DotPlotAtomic} executes the following steps:
\enumerate{
\item \strong{ggplot dispatch} — selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{bg_direction normalisation} — \code{match.arg()} resolves
\code{"v"}/\code{"h"} abbreviations to \code{"vertical"}/\code{"horizontal"}.
\item \strong{Axis type detection} — determines whether \code{x} (resp.
\code{y}) is numeric by checking that the column is a single name and the
data column is neither character nor factor.
\item \strong{Column resolution} — non-numeric x and y are processed via
\code{check_columns()} with \code{force_factor = TRUE, allow_multi = TRUE,
  concat_multi = TRUE} using \code{x_sep}/\code{y_sep}. \code{fill_by} and
\code{facet_by} are also validated.
\item \strong{NA / empty handling} — \code{process_keep_na_empty()} filters
data and extracts \code{keep_empty} settings for x, y, and facet_by.
\item \strong{Multi-facet keep_empty guard} — when \code{facet_by} has more
than one column, the \code{keep_empty} values must be identical for all
facet columns (consistent drop behaviour).
\item \strong{fill_cutoff guard} — errors if \code{fill_cutoff} is set but
\code{fill_by} is \code{NULL}.
\item \strong{Size-by resolution} — when \code{size_by = NULL}:
\itemize{
\item Groups data by the unique combination of x, y, and facet_by
columns and counts rows per combination into \code{.size}.
\item If \code{fill_by} is present, \code{summarise()} also takes the
first value of \code{fill_by} per group (with a warning that only the
first value is used).
\item Factor levels of x, y, and facet_by are preserved post-summary.
\item Sets \code{size_by <- ".size"}.
}
\item \strong{fill_cutoff parsing} — if \code{fill_cutoff} is set:
\itemize{
\item Numeric shorthand (e.g. \code{18}) is converted to \code{"< 18"}.
\item The string is parsed with regex \code{^(<=?|>=?)\\\\s*(-?[0-9.]+)$}
to extract operator and numeric threshold.
\item A \code{switch()} on the operator NAs-out matching values in
\code{fill_by} (e.g. \code{"<"} sets values \emph{below} the threshold
to \code{NA}).
\item A label \code{"<fill_by> <fill_cutoff>"} is generated for the
legend.
}
\item \strong{Default fill_by} — when \code{fill_by = NULL}, a synthetic
\code{.fill_by} column (constant \code{1}) is created and the fill legend
is suppressed.
\item \strong{Continuous colour scale preparation} — when \code{fill_by}
is numeric, \code{\link{prepare_continuous_color_scale}()} computes
\code{feat_colors_value} (the range endpoints after optional quantile
trimming or cutoff clamping).
\item \strong{Base ggplot} — initialises \code{ggplot(data, aes(x, y))}.
\item \strong{Background layer} — if \code{add_bg = TRUE}:
\itemize{
\item Vertical stripes (\code{bg_direction = "vertical"}) require a
non-numeric x-axis; horizontal stripes require a non-numeric y-axis.
\item Calls \code{\link{bg_layer}()} with the relevant discrete column
and stripe styling.
}
\item \strong{Discrete axis scales} — \code{scale_x_discrete()} /
\code{scale_y_discrete()} with \code{drop = !isTRUE(keep_empty_*)} for
non-numeric axes.
\item \strong{Lollipop branch} (\code{lollipop = TRUE}):
\itemize{
\item Sets x-axis expansion to \code{c(0, 0, 0.05, 0)} (bars start at
x = 0 with a 5\\% right pad).
\item \strong{Outer shadow} — \code{geom_segment()} from x = 0 to the
data value, coloured black (or \code{border_color} when a string),
with linewidth \code{border_size * 4}. Skipped when
\code{border_color = FALSE}.
\item \strong{Inner coloured bar} — \code{geom_segment()} mapped to
the \code{fill_by} gradient via \code{scale_color_gradientn()}, with
linewidth \code{border_size * 2}.
\item \code{ggnewscale::new_scale_color()} resets the colour scale for
the subsequent point layer.
}
\item \strong{Point layer} — \code{geom_point(shape = 21)} (filled circle
with border) with four dispatch paths:
\itemize{
\item \emph{Numeric size + gradient border}: \code{aes(fill, color)}
both mapped to \code{fill_by}; \code{size = size_by} constant.
\item \emph{Numeric size + constant/no border}: \code{aes(fill,
      color = "")} with a constant colour string.
\item \emph{Column size + gradient border}: \code{aes(size, fill,
      color)} with \code{scale_size(range = c(size_min, size_max))} and a
size legend (title = \code{size_name}, order = 1).
\item \emph{Column size + constant/no border}: \code{aes(size, fill,
      color = "")} with the same size scale.
}
\item \strong{Fill scale} — \code{scale_fill_gradientn()} with
\code{palette_this()}, \code{feat_colors_value} for rescaled colour
mapping, \code{na.value = "grey80"}, and a colour-bar legend (title =
\code{fill_name}, order = 2) or \code{guide_none()} when no
\code{fill_by} was provided.
\item \strong{Labels} — \code{labs(title, subtitle, x, y)} with fallback
to column names via \code{\%||\%}.
\item \strong{Theme} — \code{do_call(theme, theme_args)} plus
\code{panel.grid.major} (grey80 dashed) and rotated x-axis text via
\code{calc_just(x_text_angle)}.
\item \strong{Colour (border) scale}:
\itemize{
\item \code{border_color = TRUE} — \code{scale_color_gradientn()}
following the \code{fill_by} gradient (with border_alpha), guide
suppressed (the fill colour-bar serves both).
\item \code{border_color = FALSE} — \code{scale_color_manual()} with
\code{"transparent"}, guide suppressed.
\item Constant string — \code{scale_color_manual()} with the
alpha-adjusted colour, guide suppressed.
}
\item \strong{fill_cutoff legend} — when \code{fill_cutoff} is active and
there are NA values in \code{fill_by}, a \code{guide_legend()} (order = 3)
is added showing the cutoff label with a grey fill and the border colour.
\item \strong{Flip} — optional \code{coord_flip()} swaps x and y axes.
\item \strong{Dimension calculation} — \code{calculate_plot_dimensions()}
with \code{base_height = 4.5}, \code{aspect.ratio = NULL}, and per-axis
scale factors (0.9 for the categorical axis driving width, 0.6 for the
categorical axis driving height). Legend width, y-axis label length, and
minimum dimensions (width \eqn{\ge} 5, height \eqn{\ge} 4) are factored in. A fallback
manual calculation is used when \code{calculate_plot_dimensions()} returns
\code{NULL}.
\item \strong{Faceting} — \code{facet_plot()} applies
\code{facet_grid}/\code{facet_wrap} with \code{drop = !isTRUE(keep_empty_facet)}.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatialplot.R
\name{.wrap_spatial_layers}
\alias{.wrap_spatial_layers}
\title{Wrap spatial plot if plotted independently}
\usage{
.wrap_spatial_layers(
  layers,
  ext = NULL,
  flip_y = TRUE,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  theme = "theme_box",
  theme_args = list()
)
}
\arguments{
\item{layers}{A list of ggplot layers to be wrapped.}

\item{ext}{A numeric vector of length 4 specifying the extent as \code{c(xmin, xmax, ymin, ymax)}. Default is NULL.}

\item{flip_y}{Whether to flip the y-axis direction. Default is TRUE.}

\item{legend.position}{The position of the legend. Default is "right".}

\item{legend.direction}{The direction of the legend. Default is "vertical".}

\item{title}{The title of the plot. Default is NULL.}

\item{subtitle}{The subtitle of the plot. Default is NULL.}

\item{xlab}{The x-axis label. Default is NULL.}

\item{ylab}{The y-axis label. Default is NULL.}

\item{theme}{The theme to be used for the plot. Default is "theme_box".}

\item{theme_args}{A list of arguments to be passed to the theme function. Default is an empty list.}
}
\value{
A ggplot object with the specified layers.
}
\description{
This function is used to wrap spatial plots if they are plotted independently
with return_layer = FALSE.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{HeatmapAtomic}
\alias{HeatmapAtomic}
\title{Atomic heatmap (internal)}
\usage{
HeatmapAtomic(
  data,
  values_by,
  values_fill = NA,
  rows_by = NULL,
  rows_split_by = NULL,
  columns_by = NULL,
  columns_split_by = NULL,
  palette = "RdBu",
  palcolor = NULL,
  palreverse = FALSE,
  pie_size_name = "size",
  pie_size = NULL,
  pie_values = "length",
  pie_group_by = NULL,
  pie_palette = "Spectral",
  pie_palcolor = NULL,
  bars_sample = 1,
  label = scales::label_number_auto(),
  label_size = 10,
  label_color = "black",
  label_name = "label",
  mark = identity,
  mark_color = "black",
  mark_size = 1,
  mark_name = "mark",
  violin_fill = NULL,
  boxplot_fill = NULL,
  dot_size = 8,
  dot_size_name = "size",
  legend_items = NULL,
  legend_discrete = FALSE,
  legend.position = "right",
  legend.direction = "vertical",
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  add_bg = FALSE,
  bg_alpha = 0.5,
  keep_na = FALSE,
  keep_empty = FALSE,
  add_reticle = FALSE,
  reticle_color = "grey",
  cluster_columns = TRUE,
  cluster_rows = TRUE,
  show_row_names = NULL,
  show_column_names = NULL,
  border = TRUE,
  title = NULL,
  column_title = NULL,
  row_title = NULL,
  na_col = "grey85",
  row_names_side = "right",
  column_names_side = "bottom",
  row_annotation = NULL,
  row_annotation_side = NULL,
  row_annotation_palette = NULL,
  row_annotation_palcolor = NULL,
  row_annotation_type = NULL,
  row_annotation_params = NULL,
  row_annotation_agg = NULL,
  column_annotation = NULL,
  column_annotation_side = NULL,
  column_annotation_palette = NULL,
  column_annotation_palcolor = NULL,
  column_annotation_type = NULL,
  column_annotation_params = NULL,
  column_annotation_agg = NULL,
  flip = FALSE,
  alpha = 1,
  seed = 8525,
  padding = 15,
  base_size = 1,
  aspect.ratio = NULL,
  draw_opts = list(),
  return_ht = FALSE,
  layer_fun_callback = NULL,
  cell_type = "tile",
  cell_agg = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame in long format. Each row represents one
observation; columns specify row/column membership and the value to
encode as colour.}

\item{values_by}{A character of column name in \code{data} that contains the values to be plotted.
This is required when \code{in_form} is \code{"long"}. For other formats, the values are pivoted into a column named by \code{values_by}.}

\item{values_fill}{A value used to fill missing cells in the matrix.
Default \code{NA}.  Missing values prevent clustering when not filled.}

\item{rows_by}{A vector of column names in \code{data} that contains the row information.
This is used to create the rows of the heatmap.
When \code{in_form} is \code{"long"} or \code{"wide-columns"}, this is requied, and multiple columns can be specified,
which will be concatenated by \code{rows_by_sep} into a single column.}

\item{rows_split_by}{A character of column name in \code{data} that contains the split information for rows.}

\item{columns_by}{A vector of column names in \code{data} that contains the column information.
This is used to create the columns of the heatmap.
When \code{in_form} is \code{"long"} or \code{"wide-rows"}, this is required, and multiple columns can be specified,
which will be concatenated by \code{columns_by_sep} into a single column.}

\item{columns_split_by}{A character of column name in \code{data} that contains the split information for columns.}

\item{palette}{A character string naming a palette (see
\code{\link{show_palettes}}) or a character vector of colours for the
main heatmap colour scale.  Default \code{"RdBu"}.}

\item{palcolor}{A custom colour vector overriding \code{palette}.}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{pie_size_name}{Legend title for the pie size.}

\item{pie_size}{A numeric value or function returning the pie radius.
When a function, it receives the count of groups in the pie.}

\item{pie_values}{A function or string (convertible via
\code{\link[base]{match.arg}}) to compute the value represented by
each pie slice.  Default \code{"length"} counts observations per
group.}

\item{pie_group_by}{A character of column name in \code{data} that contains the group information for pie charts.
This is used to create pie charts in the heatmap when \code{cell_type} is \code{"pie"}.}

\item{pie_palette, pie_palcolor}{Palette and custom colours for pie slice
fill colours.}

\item{bars_sample}{Fraction of each cell's observations (0 < x <= 1;
\code{1} uses all data) or a whole count > 1 sampled per cell when
\code{cell_type = "bars"}.  Column widths are proportional to the
number of bars drawn per column, so a numeric count gives equal
column widths and a fraction gives widths proportional to the
column data sizes.  Proportional widths are skipped when
\code{columns_split_by} is given, and column clustering is
disabled in proportional mode.  Default 1 (all data).}

\item{label}{A function to compute text labels when
\code{cell_type = "label"} (or \code{"label+mark"}).  Receives the
aggregated value for a cell and optionally row/column indices and
names.  See below for the full dispatch contract.}

\item{label_size}{Default point size for label text (used as fallback
when the \code{label} function does not return a \code{size} field).}

\item{label_color}{Default colour for label text (fallback).}

\item{label_name}{Legend title for the label colour scale.  The legend
is shown automatically when the \code{label} function returns a
\code{legend} field for at least one cell.}

\item{mark}{A function to compute mark symbols when
\code{cell_type = "mark"} (or \code{"label+mark"}).  Same dispatch
contract as \code{label}.}

\item{mark_color}{Default mark colour (fallback).}

\item{mark_size}{Default mark stroke width (\code{lwd}) in pt (fallback).}

\item{mark_name}{Legend title for the mark colour scale.}

\item{violin_fill}{A character vector of colours to use as fill for
violin plots when \code{cell_type = "violin"}.  If \code{NULL}, the
annotation colour is used.}

\item{boxplot_fill}{A character vector of colours to use as fill for
boxplots when \code{cell_type = "boxplot"}.  If \code{NULL}, the
annotation colour is used.}

\item{dot_size}{Dot size when \code{cell_type = "dot"}.  Can be a
numeric value or a function.}

\item{dot_size_name}{Legend title for the dot size.}

\item{legend_items}{A named numeric vector specifying custom legend
entries for the main colour scale.  Names become the displayed labels.}

\item{legend_discrete}{Logical; if \code{TRUE}, treat the main colour
scale as discrete.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{lower_quantile, upper_quantile, lower_cutoff, upper_cutoff}{Quantile or explicit cutoffs for clipping the colour scale.  Applied
to aggregated values for \code{tile} / \code{label} cell types; applied
to raw values for \code{bars} / \code{violin} / \code{boxplot} types.}

\item{add_bg}{Logical; if \code{TRUE}, add a background fill behind
non-tile cell types.  Not used for \code{cell_type = "tile"} or
\code{"bars"}.}

\item{bg_alpha}{Numeric in \eqn{[0, 1]} for background transparency.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{add_reticle}{Logical; if \code{TRUE}, draw a reticle (crosshair
pattern) over the heatmap.}

\item{reticle_color}{Colour for the reticle lines.}

\item{cluster_columns}{Logical; cluster the columns.  If \code{TRUE} and
\code{columns_split_by} is provided, clustering is applied within each
split group.}

\item{cluster_rows}{Logical; cluster the rows.  If \code{TRUE} and
\code{rows_split_by} is provided, clustering is applied within each
split group.}

\item{show_row_names}{Logical or character vector; show row names.  If
\code{TRUE}, the legend of the row group annotation is hidden.
\code{FALSE} only hides the in-place names.  Alternatively, a character
vector combining the display modes \code{"inplace"} (in-place names,
same as \code{TRUE}), \code{"legend"} (row group annotation with
legend), \code{"simple"} (row group annotation without legend),
\code{"anno"} (alias \code{"annotation"}; label annotation showing the
row names as text) and \code{"none"} (no names, annotation, or legend).
Modes are lower-priority defaults: explicit \code{row_annotation}
entries (e.g. \code{.row}) still win.}

\item{show_column_names}{Logical or character vector; show column names.
See \code{show_row_names} for the display modes.}

\item{border}{A logical value indicating whether to draw borders around
the heatmap.  If \code{TRUE}, slice borders are also drawn.  Default
\code{TRUE}.}

\item{title}{The global (column) title of the heatmap.}

\item{column_title}{Character string/vector used as the column group
annotation title.  A character vector consisting entirely of the display
modes \code{"inplace"} (per-slice split titles), \code{"legend"} (split
annotation with legend, no title), \code{"simple"} (split annotation,
no legend/title), \code{"anno"} (alias \code{"annotation"}; label block
annotation) and \code{"none"} (no title, annotation, or legend)
controls how the split titles are displayed.  A literal title identical
to a mode keyword is interpreted as a mode.}

\item{row_title}{Character string/vector used as the row group
annotation title.  See \code{column_title} for the display modes.}

\item{na_col}{Colour for \code{NA} cells.  Default \code{"grey85"}.}

\item{row_names_side}{Side for row names.  Default \code{"right"}.}

\item{column_names_side}{Side for column names.  Default \code{"bottom"}.}

\item{row_annotation}{A structured list specifying row annotations.
Same format as \code{column_annotation}.  Sides default to
\code{"left"}.  Aliases: \code{.row}/\code{.rows} for
\code{rows_by}, \code{.row.split}/\code{.rows.split} for
\code{rows_split_by}.}

\item{row_annotation_side}{\strong{Deprecated}: use
\code{row_annotation} with the \code{side} sub-key instead.}

\item{row_annotation_palette}{\strong{Deprecated}: use
\code{row_annotation} with the \code{palette} sub-key instead.}

\item{row_annotation_palcolor}{\strong{Deprecated}: use
\code{row_annotation} with the \code{palcolor} sub-key instead.}

\item{row_annotation_type}{\strong{Deprecated}: use
\code{row_annotation} with the \code{type} sub-key instead.}

\item{row_annotation_params}{\strong{Deprecated}: use
\code{row_annotation} with the \code{params} sub-key instead.}

\item{row_annotation_agg}{\strong{Deprecated}: use
\code{row_annotation} with the \code{agg} sub-key instead.}

\item{column_annotation}{A structured list specifying column annotations.
Each entry is a named list with sub-keys:
\describe{
\item{\code{col}}{Column name in \code{data} supplying the annotation
values.  If omitted, the entry name is used as the column name.}
\item{\code{side}}{\code{"top"} or \code{"bottom"}.}
\item{\code{palette}}{Palette name (see \code{\link{show_palettes}}).}
\item{\code{palcolor}}{Custom colour vector overriding \code{palette}.}
\item{\code{type}}{Annotation type: \code{"auto"}, \code{"simple"},
\code{"pie"}, \code{"ring"}, \code{"bar"}, \code{"violin"},
\code{"boxplot"}, \code{"density"}, \code{"label"},
\code{"rownames"}, \code{"points"}, \code{"lines"}.
For split annotations, \code{"rownames"} (row splits) /
\code{"colnames"} (column splits) is like \code{"label"} but labels
each split block with the concatenated row/column names;
\code{"names"} and \code{"dimnames"} are aliases.
\code{params$sep} (default \code{" "}) controls the separator and
\code{params$wrap_by} the number of names per line.
For name annotations (the built-in \code{rows_by}/\code{columns_by}
annotation), \code{"rownames"}/\code{"colnames"} renders the
row/column names as text labels (this is what
\code{show_row_names = "anno"} uses).}
\item{\code{params}}{A list of additional parameters passed to the
annotation constructor.  \code{FALSE} disables the annotation.
\code{$show_legend} controls legend visibility.  See
\code{\link[ComplexHeatmap]{HeatmapAnnotation}}.}
\item{\code{agg}}{A function to aggregate values for the annotation.}
\item{\code{name}}{Display name for the annotation, shown next to the
annotation bar and used as the legend title.  \code{FALSE} hides the
displayed name.  Defaults to the annotation key (column name).}
}
\strong{Shortcuts:}
\itemize{
\item \code{column_annotation = list(Score = "score")} is short for
\code{list(Score = list(col = "score"))}.
\item \code{column_annotation = TRUE} enables annotations with
defaults.  \code{FALSE} disables all column annotations.
}
\strong{Special keys:}
\itemize{
\item \code{.default} — default values inherited by all entries.
\code{params} is merged recursively; other keys are inherited
only when the entry does not already specify them.
\item \code{.col} / \code{.cols} / \code{.column} / \code{.columns} —
alias for \code{columns_by} (the built-in name annotation).
\item \code{.col.split} / \code{.cols.split} / \code{.column.split} /
\code{.columns.split} — alias for \code{columns_split_by}
(the built-in split annotation).
\item \code{.row} / \code{.rows} — alias for \code{rows_by}.
\item \code{.row.split} / \code{.rows.split} — alias for
\code{rows_split_by}.
}}

\item{column_annotation_side}{\strong{Deprecated}: use
\code{column_annotation} with the \code{side} sub-key instead.}

\item{column_annotation_palette}{\strong{Deprecated}: use
\code{column_annotation} with the \code{palette} sub-key instead.}

\item{column_annotation_palcolor}{\strong{Deprecated}: use
\code{column_annotation} with the \code{palcolor} sub-key instead.}

\item{column_annotation_type}{\strong{Deprecated}: use
\code{column_annotation} with the \code{type} sub-key instead.}

\item{column_annotation_params}{\strong{Deprecated}: use
\code{column_annotation} with the \code{params} sub-key instead.}

\item{column_annotation_agg}{\strong{Deprecated}: use
\code{column_annotation} with the \code{agg} sub-key instead.}

\item{flip}{Logical; if \code{TRUE}, swap rows and columns
transparently.  The caller does not need to swap row- and
column-related arguments manually.}

\item{alpha}{Alpha transparency for heatmap cells in \eqn{[0, 1]}.}

\item{seed}{The random seed to use. Default is 8525.}

\item{padding}{Padding around the heatmap in CSS order (top, right,
bottom, left).  Supports 1–4 values.  Default 15 (mm).  Note that
this is different from \code{ComplexHeatmap::draw()}'s \code{padding}
argument which uses bottom-left-top-right order.}

\item{base_size}{A positive numeric scalar used as a scaling factor for
the overall heatmap size.  Default 1 (no scaling).  Values > 1 enlarge
all cell dimensions proportionally.}

\item{aspect.ratio}{Height-to-width ratio of a single heatmap cell.
When \code{NULL} (default), sensible per-\code{cell_type} defaults are
used: 1 for \code{tile}/\code{label}/\code{dot}, 0.5 for \code{bars},
and 2 for \code{violin}/\code{boxplot}/\code{pie}.  The ratio is
constrained by the overall plot dimensions.}

\item{draw_opts}{A named list of additional arguments passed to
\code{\link[ComplexHeatmap]{draw,HeatmapList-method}}.  Internally
managed arguments take precedence.}

\item{return_ht}{Logical; if \code{TRUE}, return the prepared
\code{ComplexHeatmap::Heatmap} object without drawing.  Used
internally by \code{\link{LinkedHeatmapAtomic}}.}

\item{layer_fun_callback}{A function to add custom graphical layers on
top of each heatmap cell.  Receives \code{j}, \code{i}, \code{x},
\code{y}, \code{w}, \code{h}, \code{fill}, \code{sr}, \code{sc}.
See \code{\link[ComplexHeatmap]{Heatmap}} for details.}

\item{cell_type}{The type of cell to render.  One of \code{"tile"}
(default), \code{"bars"}, \code{"label"}, \code{"mark"},
\code{"label+mark"} (or \code{"mark+label"}), \code{"dot"},
\code{"violin"}, \code{"boxplot"}, \code{"pie"}.  See the
\strong{Cell types} section for details.}

\item{cell_agg}{A function to aggregate values within each cell when
\code{cell_type = "tile"} or \code{"label"}.  Default is
\code{\link[base]{mean}}.}

\item{...}{Additional arguments passed to
\code{\link[ComplexHeatmap]{Heatmap}}.  When
\code{row_names_max_width} is passed, a \code{unit} is expected but
numeric values (default unit \code{"inches"}) or strings like
\code{"5inches"} are also accepted.  Unmatched arguments produce a
warning.}
}
\value{
An object of heatmap that is wrapped by \code{patchwork::wrap_plots()}
}
\description{
Core implementation for drawing a single heatmap using
\code{ComplexHeatmap::Heatmap}.  This function takes pre-processed data
(already in long format with resolved row/column identifiers) and handles
colour mapping, cell rendering, annotation construction, and the final
\code{draw()} call.  It is the workhorse behind the exported
\code{\link{Heatmap}} function and is also reused by
\code{\link{LinkedHeatmapAtomic}} (via \code{return_ht = TRUE}).
}
\note{
Removed parameters: \code{rows_palette}, \code{rows_palcolor},
\code{columns_palette}, \code{columns_palcolor},
\code{columns_split_palette}, \code{columns_split_palcolor},
\code{rows_split_palette}, \code{rows_split_palcolor} — use
\code{row_annotation}/\code{column_annotation} with
sub-key \code{palette}/\code{palcolor} and alias
\code{.row} / \code{.row.split} / \code{.col} / \code{.col.split}.
Also removed:
\code{row_name_annotation}, \code{row_name_legend},
\code{column_name_annotation}, \code{column_name_legend} — set
\code{row_annotation = list(.row = list(params = FALSE))} /
\code{column_annotation = list(.col = list(params = FALSE))}.
}
\section{Architecture}{

\enumerate{
\item \strong{Annotation resolution} — row and column annotation
parameters (side, type, palette, params, aggregation) are
resolved with alias support (\code{.row}, \code{.col},
\code{.rows.split}, etc.) so that built-in name/split annotations
and user-defined annotations can be configured uniformly.
\item \strong{Keep-NA / keep-empty handling} — \code{keep_na} and
\code{keep_empty} are processed per column to control whether
\code{NA} values and empty factor levels are preserved in the
data before constructing the heatmap matrix.
\item \strong{Flip} — when \code{flip = TRUE}, row and column
parameters are swapped transparently so the caller does not need
to manually swap every argument.
\item \strong{Cell dimension calculation} — cell width and height are
pre-computed from \code{cell_type}, \code{aspect.ratio},
\code{base_size}, and the unique row/column counts (accounting
for split groups).  These are passed as explicit
\code{"inches"} units to \code{ComplexHeatmap::Heatmap()} so
cells have guaranteed physical sizes.
\item \strong{Colour mapping} — the main colour scale is built from
\code{palette} / \code{palcolor} with optional quantile-based
or explicit cutoff clamping.
\item \strong{Cell function dispatch} — depending on
\code{cell_type}, the appropriate \code{cell_fun} or
\code{layer_fun} is installed (tile, bars, label, mark,
label+mark, dot, violin, boxplot, pie).
\item \strong{Annotation construction} — row and column annotations
are built via \code{ComplexHeatmap::HeatmapAnnotation()} using
the resolved type and parameters.
\item \strong{Drawing} — the heatmap is drawn via
\code{ComplexHeatmap::draw()} with controlled padding and legend
collection.  When \code{return_ht = TRUE}, the prepared
\code{Heatmap} object is returned without drawing.
\item \strong{Dimension attributes} — the returned object carries
\code{height} / \code{width} attributes (in inches) and
\code{cell_w} / \code{cell_h} attributes for downstream layout
calculations.
}
}

\section{Label / mark dispatch contract}{

The \code{label} and \code{mark} functions can take 1, 3, or 5
arguments.  The first argument is always the aggregated value for a
single cell.  With 3 arguments, the second and third are the row and
column indices.  With 5 arguments, the fourth and fifth are the row
and column names.  The function should return one of:
\itemize{
\item \code{NA} — nothing is drawn for this cell.
\item A character scalar — used as the label text / mark type;
default size and colour are used.
\item A named list with fields \code{label}/\code{mark},
\code{size}, \code{color}, \code{legend}, and \code{order}
(all optional; smaller \code{order} appears first in the
legend).
}
For label cells with custom indices, use
\code{ComplexHeatmap::pindex()} to translate between data and heatmap
coordinates.
}

\section{Supported mark types}{

\itemize{
\item \strong{Primitives:} \code{-} (h-line), \code{|} (v-line),
\code{+} (cross), \code{/} (l-diag), \code{\\} (r-diag),
\code{x} (both diags), \code{o} (circle), \code{()}
(edge-touching circle), \code{<>} (diamond).
\item \strong{With rectangular border:} \code{[]}, \code{[-]},
\code{[|]}, \code{[+]}, \code{[/]}, \code{[\\]}, \code{[x]},
\code{[o]}, \code{[()]}, \code{[<>]}.
\item \strong{With full circle:} \code{(-)}, \code{(|)},
\code{(+)}, \code{(/)}, \code{(\\)}, \code{(x)}, \code{(o)},
\code{(<>)}.
\item \strong{With diamond:} \code{<->}, \code{<|>}, \code{<+>},
\code{</>}, \code{<\\>}, \code{<x>}, \code{<o>}.
\item \strong{Octagon:} \code{\{\}}, \code{\{-\}}, \code{\{|\}},
\code{\{+\}}, \code{\{/\}}, \code{\{\\\}}, \code{\{x\}},
\code{\{o\}}, \code{\{()\}}, \code{\{<>\}}.
\item \strong{Combinations:} e.g. \code{[(|)]}, \code{[(-)]},
\code{[(+)]}, \code{[(/)]}, \code{[(\\]}, \code{[(x)]},
\code{[(o)]}, \code{[(<>)]}.
}
}

\section{Annotations}{

Row and column annotations are configured via the
\code{row_annotation} / \code{column_annotation} structured list
arguments.  Built-in name annotations (for \code{rows_by} /
\code{columns_by}) and split annotations (for \code{rows_split_by} /
\code{columns_split_by}) are added automatically and can be configured
using the aliases \code{.row} / \code{.col} and \code{.row.split} /
\code{.col.split} as keys in the annotation list.  Setting an alias
entry's \code{params} to \code{FALSE} disables that annotation.
\code{.default} provides default values inherited by all entries
(with recursive merge for \code{params}).  Ordering within each side:
name annotations are closest to the body, split annotations farthest
away, user-defined annotations in between.
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/piechart.R
\name{PieChart}
\alias{PieChart}
\title{Pie chart}
\usage{
PieChart(
  data,
  x,
  y = NULL,
  label = y,
  split_by = NULL,
  split_by_sep = "_",
  clockwise = TRUE,
  facet_by = NULL,
  facet_scales = "free_y",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  aspect.ratio = 1,
  keep_na = FALSE,
  keep_empty = FALSE,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  seed = 8525,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name for the x-axis
(categories).  Must be character or factor.  Each unique value becomes a
pie slice.}

\item{y}{A character string specifying the numeric column for the y-axis.
When \code{NULL} (default), the count of observations in each
(\code{x}, \code{facet_by}) combination is used and stored as
\code{.y}.}

\item{label}{A character string specifying the column to use for slice
labels.  \code{NULL} (default) hides labels.  When \code{TRUE}, the
\code{y} values are used as labels.  When \code{y = NULL}, use
\code{".y"} to label with the computed counts.}

\item{split_by}{The column(s) to split the data by and produce separate
sub-plots.  Multiple columns are concatenated with \code{split_by_sep}.}

\item{split_by_sep}{A character string to separate concatenated
\code{split_by} columns.  Default \code{"_"}.}

\item{clockwise}{A logical value.  When \code{TRUE} (default), the pie
slices are ordered clockwise starting from the top.  When \code{FALSE},
slices are ordered counter-clockwise.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{combine}{Logical; when \code{TRUE} (default), returns a combined
\code{patchwork} object.  When \code{FALSE}, returns a named list of
individual \code{ggplot} objects.}

\item{ncol, nrow}{Integer number of columns / rows for the combined layout
(passed to \code{\link[patchwork]{wrap_plots}}).}

\item{byrow}{Logical; fill the combined layout by row.  Default \code{TRUE}
(passed to \code{\link[patchwork]{wrap_plots}}).}

\item{seed}{A numeric seed for reproducibility.  Passed to
\code{\link{validate_common_args}()}.}

\item{axes}{A character string specifying how axes should be treated across
the combined layout (passed to \code{\link[patchwork]{wrap_plots}}).}

\item{axis_titles}{A character string specifying how axis titles should be
treated across the combined layout.  Defaults to \code{axes}.}

\item{guides}{A character string specifying how guides (legends) should be
collected across panels.  Default \code{"collect"} (passed to
\code{\link{combine_plots}()}).}

\item{design}{A custom layout design for the combined plot (passed to
\code{\link{combine_plots}()}).}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object, a \code{patchwork} object, or a named list
of \code{ggplot} objects (when \code{combine = FALSE}), each with
\code{height} and \code{width} attributes in inches.
}
\description{
Draws a pie chart illustrating the numerical proportion of each group
relative to the whole.  Each slice corresponds to a level of the x-axis
variable and its angle is proportional to the y-axis value (or the
observation count when \code{y} is omitted).

The function supports \strong{count aggregation} (omit \code{y} to plot
observation counts per x-category), \strong{slice labels} via
\code{ggrepel::geom_label_repel()}, clockwise or counter-clockwise slice
ordering, faceting, and splitting into separate sub-plots via
\code{split_by}.
}
\section{split_by workflow}{

When \code{split_by} is provided:
\enumerate{
\item \code{\link{validate_common_args}()} validates the \code{seed}
and \code{facet_by} settings.
\item \code{\link{check_keep_na}()} and \code{\link{check_keep_empty}()}
normalise the \code{keep_na} / \code{keep_empty} arguments for all
columns (\code{x}, \code{split_by}, \code{facet_by}).
\item \code{\link{process_theme}()} resolves the theme function.
\item The \code{x} column is forced to factor; \code{y} is validated.
\item The \code{split_by} column is validated and its NA / empty levels
are processed via \code{\link{process_keep_na_empty}()}.  It is
then removed from the per-column \code{keep_na} / \code{keep_empty}
lists.
\item The data frame is split by \code{split_by} (preserving level
order).  If \code{split_by} is \code{NULL}, the data is wrapped in
a single-element list with name \code{"..."}.
\item Per-split \code{palette}, \code{palcolor},
\code{legend.position}, and \code{legend.direction} are resolved
via \code{\link{check_palette}()}, \code{\link{check_palcolor}()},
and \code{\link{check_legend}()}.
\item \code{\link{PieChartAtomic}()} is called for each split.  If
\code{title} is a function, it receives the split level name and
can generate dynamic titles.
\item Results are combined via \code{\link{combine_plots}()} (when
\code{combine = TRUE}) or returned as a named list.
}
}

\examples{
\donttest{
data <- data.frame(
   x = factor(c("A", "B", "C", NA, "E", "F", "G", "H"), levels = LETTERS[1:8]),
   y = c(10, 8, 16, 4, 6, 12, 14, 2),
   group = factor(c("G1", "G1", NA, NA, "G3", "G3", "G4", "G4"),
       levels = c("G1", "G2", "G3", "G4")),
   facet = factor(c("F1", NA, "F3", "F4", "F1", NA, "F3", "F4"),
       levels = c("F1", "F2", "F3", "F4"))
)

# Basic pie chart
PieChart(data, x = "x", y = "y")

# Keep NA and empty levels
PieChart(data, x = "x", y = "y", keep_na = TRUE, keep_empty = TRUE)

# Counter-clockwise ordering
PieChart(data, x = "x", y = "y", clockwise = FALSE)
PieChart(data, x = "x", y = "y", clockwise = FALSE,
         keep_na = TRUE, keep_empty = TRUE)

# With slice labels
PieChart(data, x = "x", y = "y", label = "group")

# Faceting
PieChart(data, x = "x", y = "y", facet_by = "facet")
PieChart(data, x = "x", y = "y", facet_by = c("facet", "group"),
    keep_empty = "level")
PieChart(data, x = "x", y = "y", facet_by = c("facet", "group"),
    keep_empty = TRUE)

# Split into sub-plots
PieChart(data, x = "x", y = "y", split_by = "group")

# Per-split palettes
PieChart(data, x = "x", y = "y", split_by = "group",
         palette = list(G1 = "Reds", G2 = "Blues", G3 = "Greens", G4 = "Purp"))

# Y from count
PieChart(data, x = "group")

# Y from count with label
PieChart(data, x = "group", label = ".y")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trendplot.R
\name{TrendPlotAtomic}
\alias{TrendPlotAtomic}
\title{Atomic trend plot (internal)}
\usage{
TrendPlotAtomic(
  data,
  x,
  y = NULL,
  x_sep = "_",
  group_by = NULL,
  group_by_sep = "_",
  group_name = NULL,
  scale_y = FALSE,
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  facet_by = NULL,
  facet_scales = "fixed",
  facet_ncol = NULL,
  facet_nrow = NULL,
  facet_byrow = TRUE,
  x_text_angle = 0,
  aspect.ratio = 1,
  legend.position = waiver(),
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  keep_empty = FALSE,
  keep_na = FALSE,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{x}{A character string specifying the column name to plot on the
x-axis. Must be character or factor. Multiple columns can be provided;
they are concatenated with \code{x_sep} as the separator.}

\item{y}{A character string specifying the numeric column for the y-axis.
When \code{NULL}, the count of observations in each (\code{x},
\code{group_by}, \code{facet_by}) combination is used.}

\item{x_sep}{A character string used to join multiple \code{x} columns.
Default \code{"_"}. Ignored when \code{x} is a single column.}

\item{group_by}{A character vector of column names to fill the bars and
areas by. Each unique combination becomes a separate stacked segment.
Multiple columns are concatenated with \code{group_by_sep}. When
\code{NULL}, a single filled bar/area is drawn and the legend is hidden.}

\item{group_by_sep}{A character string to separate concatenated
\code{group_by} columns. Default \code{"_"}.}

\item{group_name}{A character string used as the fill legend title.
When \code{NULL}, the \code{group_by} column name is used.}

\item{scale_y}{A logical value. When \code{TRUE}, y-values are scaled to
proportions within each (\code{x}, \code{facet_by}) group so that each
x position stacks to 1.0. The y-axis labels switch from numeric to
percent format automatically.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{facet_by}{A character string specifying the column name of the data frame to facet the plot.
Otherwise, the data will be split by \code{split_by} and generate multiple plots
and combine them into one using \code{patchwork::wrap_plots}}

\item{facet_scales}{Whether to scale the axes of facets. Default is "fixed"
Other options are "free", "free_x", "free_y". See \code{ggplot2::facet_wrap}}

\item{facet_ncol}{A numeric value specifying the number of columns in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_nrow}{A numeric value specifying the number of rows in the facet.
When facet_by is a single column and facet_wrap is used.}

\item{facet_byrow}{A logical value indicating whether to fill the plots by row. Default is TRUE.}

\item{x_text_angle}{A numeric value specifying the angle of the x-axis text.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a single trend plot. This is the workhorse
behind the exported \code{\link{TrendPlot}} function -- it takes a \strong{single}
data frame (no \code{split_by} support) and returns a \code{ggplot} object.
A trend plot combines stacked bars (\code{\link[ggplot2]{geom_col}()}) with
a semi-transparent area background (\code{\link[ggplot2]{geom_area}()}) to
show how one or more groups accumulate across a discrete x-axis. This hybrid
style sits between a bar plot and an area plot, preserving the discrete
category separation of bars while softening the visual with an area fill.

The function supports count aggregation (omit \code{y} to plot observation
counts per x-category), proportion scaling (\code{scale_y = TRUE} normalises
each x position to 100\\%), per-group colour control, and faceting.
}
\section{Architecture}{

\enumerate{
\item \strong{Column resolution} -- \code{x}, \code{y}, \code{group_by},
and \code{facet_by} are validated and transformed via
\code{\link{check_columns}}. Multi-column inputs for \code{x} and
\code{group_by} are concatenated into single columns using their
respective separators (\code{x_sep}, \code{group_by_sep}).
\item \strong{Count aggregation} -- when \code{y = NULL}, the count of
observations in each unique (\code{x}, \code{group_by},
\code{facet_by}) combination is computed as a new \code{.count}
column. Factor levels are preserved after aggregation.
\item \strong{Proportion scaling} -- when \code{scale_y = TRUE}, the
y-values are divided by the sum within each (\code{x},
\code{facet_by}) group, producing a proportion (0--1). Percent
labels are used automatically on the y-axis.
\item \strong{NA / empty-level handling} --
\code{\link{process_keep_na_empty}()} applies \code{keep_na} and
\code{keep_empty} policies. Per-column \code{keep_empty} settings
are extracted for \code{x}, \code{group_by}, and \code{facet_by}
independently. The facet columns must have identical
\code{keep_empty} values. Note that \code{keep_empty = TRUE} is
not supported globally because empty categories would break the
continuity of the x-axis.
\item \strong{Group fill setup} -- when \code{group_by = NULL}, a dummy
\code{.fill} factor is created so the single group still draws with
the first palette colour. The legend is suppressed
(\code{legend.position = "none"}).
\item \strong{Data completion} -- \code{\link[tidyr]{complete}()} pads
all \code{x} by \code{group_by} (by \code{facet_by})
combinations with \code{y = 0}. This prevents
\code{\link[ggplot2]{geom_area}()} from interpolating across missing
groups, which would otherwise cause stacked areas to exceed the
correct total.
\item \strong{Area layer construction} -- the completed data is expanded
by duplicating each row and offsetting the x-axis positions by
\eqn{\pm 0.2} on a numeric scale, creating discrete-width area
strips that align perfectly with the bars.
\item \strong{Colour mapping} -- \code{\link{palette_this}()} assigns
colours to all \code{group_by} levels, including \code{NA}
(defaulting to \code{"grey80"}).
\item \strong{Plot assembly} -- the \code{ggplot} object is built with
\code{geom_area()} (at half \code{alpha}, grey outline) as a
background fill, overlaid with \code{geom_col()} (full \code{alpha},
black outline, width 0.4) as the foreground bars. Both layers use
\code{position_stack(vjust = 0.5)}. The x-axis uses
\code{scale_x_discrete()} and the y-axis uses
\code{scale_y_continuous()} (percent labels when scaled). The
fill scale \code{drop} argument is controlled by
\code{keep_empty_group}.
\item \strong{Dimension calculation} --
\code{\link{calculate_plot_dimensions}()} computes plot height and
width from the x-axis category count, \code{aspect.ratio}, and
legend metrics. The base height increases for plots with 10 or more
x categories (from 4.5 to 6.5 inches). The resulting \code{height}
and \code{width} attributes are stored on the \code{ggplot} object.
\item \strong{Faceting} -- \code{\link{facet_plot}()} wraps the plot with
\code{facet_wrap} / \code{facet_grid} if \code{facet_by} is
provided, respecting the \code{keep_empty} setting for facet
variables.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chordplot.R
\name{ChordPlotAtomic}
\alias{ChordPlotAtomic}
\title{Atomic chord plot (internal)}
\usage{
ChordPlotAtomic(
  data,
  y = NULL,
  from = NULL,
  from_sep = "_",
  to = NULL,
  to_sep = "_",
  flip = FALSE,
  links_color = c("from", "to"),
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 0.5,
  labels_rot = FALSE,
  title = NULL,
  subtitle = NULL,
  keep_na = FALSE,
  keep_empty = FALSE,
  ...
)
}
\arguments{
\item{data}{A data frame.}

\item{y}{A character string specifying the numeric column whose values
determine link thickness.  When \code{NULL}, the count of observations
per (\code{from}, \code{to}) pair is used.}

\item{from}{A character string (or vector) specifying the column name(s)
for the source nodes.  Character/factor columns are expected.  Multiple
columns are concatenated with \code{from_sep}.}

\item{from_sep}{A character string to join multiple \code{from} columns.
Default \code{"_"}.}

\item{to}{A character string (or vector) specifying the column name(s)
for the target nodes.  Character/factor columns are expected.  Multiple
columns are concatenated with \code{to_sep}.}

\item{to_sep}{A character string to join multiple \code{to} columns.
Default \code{"_"}.}

\item{flip}{Logical; if \code{TRUE}, swap the source and target nodes,
reversing the link direction.}

\item{links_color}{A character string controlling which node's colour
each link ribbon takes: \code{"from"} (default) or \code{"to"}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{Numeric transparency for the link ribbons (0–1).
Default \code{0.5}.}

\item{labels_rot}{Logical; if \code{TRUE}, rotate sector labels by 90
degrees (clockwise).  Default \code{FALSE} uses \code{niceFacing} for
automatic orientation.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{keep_na}{A logical value or a character to replace the NA values in the data.
It can also take a named list to specify different behavior for different columns.
If TRUE or NA, NA values will be replaced with NA.
If FALSE, NA values will be removed from the data before plotting.
If a character string is provided, NA values will be replaced with the provided string.
If a named vector/list is provided, the names should be the column names to apply the behavior to,
and the values should be one of TRUE, FALSE, or a character string.
Without a named vector/list, the behavior applies to categorical/character columns used on the plot,
for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.}

\item{keep_empty}{One of FALSE, TRUE and "level". It can also take a named list to specify
different behavior for different columns. Without a named list, the behavior applies to the
categorical/character columns used on the plot, for example, the \code{x}, \code{group_by}, \code{fill_by}, etc.
\itemize{
\item{\code{FALSE} (default): Drop empty factor levels from the data before plotting.}
\item{\code{TRUE}: Keep empty factor levels and show them as a separate category in the plot.}
\item{\code{"level"}: Keep empty factor levels, but do not show them in the plot.
But they will be assigned colors from the palette to maintain consistency across multiple plots.
Alias: \code{levels}}
}}

\item{...}{Additional arguments.}
}
\value{
A \code{patchwork} wrapped element with \code{height} and
\code{width} attributes (in inches) attached.  The original data is
stored in the \code{p$data} field.
}
\description{
Core implementation for drawing a chord diagram using
\code{circlize::chordDiagram()}.  This is the workhorse behind the
exported \code{\link{ChordPlot}} function — it takes a \strong{single} data
frame (no \code{split_by} support) and returns a \code{patchwork}
wrapped element.

Chord diagrams visualise relationships (flows) between two sets of
categories arranged around a circle.  The width of each link is
proportional to the value (\code{y}) connecting its source and target
nodes.
}
\section{Architecture}{

\enumerate{
\item \strong{Column resolution} — \code{from}, \code{to}, and
\code{y} are validated and transformed via
\code{\link{check_columns}}.  Multi-column \code{from} and
\code{to} are concatenated with their respective separators
(\code{from_sep}, \code{to_sep}).
\item \strong{Count aggregation} — when \code{y = NULL}, the count
of observations per (\code{from}, \code{to}) pair is computed
as a new \code{.y} column.  Factor levels from the original
data are preserved.
\item \strong{NA / empty-level handling} —
\code{\link{process_keep_na_empty}()} applies \code{keep_na}
and \code{keep_empty} policies.
\item \strong{Flip} — when \code{flip = TRUE}, the \code{from} and
\code{to} columns are swapped, effectively reversing the link
direction.
\item \strong{Colour mapping} — \code{\link{palette_this}()} assigns
colours to all unique \code{from} and \code{to} values (the grid
sectors).  \code{NA} values are mapped to a literal \code{"NA"}
level and assigned a colour from the palette.
\item \strong{Link colour resolution} — \code{links_color} controls
whether each connecting ribbon takes its colour from the
\code{"from"} side (default) or the \code{"to"} side.
\item \strong{circlize rendering} — two rendering paths exist,
selected by \code{labels_rot}:
\itemize{
\item \strong{Horizontal labels} (\code{labels_rot = FALSE}):
track 1 has a fixed height of 1 mm and uses
\code{niceFacing = TRUE} for automatic label rotation.
Track 2 adds axis ticks on the outside for wide-enough
sectors.
\item \strong{Rotated labels} (\code{labels_rot = TRUE}):
track 1 height is computed from the maximum string width
of all node names, and labels are rendered with
\code{facing = "clockwise"}.  Track 2 adds axis ticks.
}
Links use arrow heads (\code{link.arr.type = "big.arrow"}) and
differentiated height (\code{direction.type = c("diffHeight",
        "arrows")}).  Both tracks set \code{bg.border = NA} to prevent
border lines from appearing between sectors.
\item \strong{Patchwork integration} — the circlize formula is wrapped
via \code{\link[patchwork]{wrap_elements}()} so it can be
composed with other ggplot objects.  Title and subtitle are added
via \code{\link[patchwork]{plot_annotation}()}.
\item \strong{Dimension attributes} — \code{height} and \code{width}
attributes (in inches) are set to a base size (7) scaled up by
2, 4, or 6 depending on the maximum label character width when
\code{labels_rot = TRUE}.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap-utils.R
\name{process_linkedheatmap_data}
\alias{process_linkedheatmap_data}
\title{Process data for LinkedHeatmap}
\usage{
process_linkedheatmap_data(
  data,
  split_by,
  split_by_sep,
  rows_split_by,
  rows_split_by_sep,
  rows_split_name,
  left_values_by,
  left_name,
  left_rows_by,
  left_rows_by_sep,
  left_rows_name,
  left_rows_orderby,
  left_columns_orderby,
  left_columns_by,
  left_columns_by_sep,
  left_columns_name,
  left_columns_split_by,
  left_columns_split_by_sep,
  left_columns_split_name,
  left_pie_group_by,
  left_pie_group_by_sep,
  left_pie_name,
  left_rows_data,
  left_columns_data,
  right_values_by,
  right_name,
  right_rows_by,
  right_rows_by_sep,
  right_rows_name,
  right_rows_orderby,
  right_columns_orderby,
  right_columns_by,
  right_columns_by_sep,
  right_columns_name,
  right_columns_split_by,
  right_columns_split_by_sep,
  right_columns_split_name,
  right_pie_group_by,
  right_pie_group_by_sep,
  right_pie_name,
  right_rows_data,
  right_columns_data,
  keep_na,
  keep_empty
)
}
\description{
Process data for LinkedHeatmap
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrich.R
\name{EnrichMapAtomic}
\alias{EnrichMapAtomic}
\title{Atomic enrichment map (internal)}
\usage{
EnrichMapAtomic(
  data,
  in_form = "clusterProfiler",
  top_term = 100,
  metric = "p.adjust",
  layout = "fr",
  minchar = 2,
  cluster = "fast_greedy",
  show_keyword = FALSE,
  nlabel = 4,
  character_width = 50,
  words_excluded = plotthis::words_excluded,
  mark = "ellipse",
  label = c("term", "feature"),
  labelsize = 5,
  expand = c(0.4, 0.4),
  theme = "theme_this",
  theme_args = list(),
  palette = "Paired",
  palcolor = NULL,
  palreverse = FALSE,
  alpha = 1,
  aspect.ratio = 1,
  legend.position = "right",
  legend.direction = "vertical",
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  seed = 8525,
  ...
)
}
\arguments{
\item{data}{A data frame containing enrichment results in clusterProfiler
format with at least the columns: \code{Description}, \code{GeneRatio},
\code{pvalue}, \code{p.adjust}, and \code{geneID}.
\itemize{
\item \code{ID}, \code{qvalue}, \code{BgRatio}, and \code{Count} are
optional.
\item \code{Description} is the term description, displayed as the
default keyword.
\item \code{GeneRatio} is the fraction of input genes annotated to
the term (e.g. \code{"10/500"}).
\item \code{BgRatio} is the fraction of background genes annotated
to the term (e.g. \code{"50/20000"}).
\item \code{Count}, if given, must equal the numerator of
\code{GeneRatio}.
\item \code{geneID} contains gene symbols separated by \code{"/"}.
}}

\item{in_form}{A character string specifying the input format.
When \code{"auto"} (default), the function infers the format from the
column names: clusterProfiler columns (\code{pvalue}, \code{p.adjust},
\code{qvalue}) or Enrichr columns (\code{P.value},
\code{Adjusted.P.value}).  Other options are \code{"clusterProfiler"}
and \code{"enrichr"}.}

\item{top_term}{An integer specifying the maximum number of terms to
include.  Terms are ranked by \code{metric} (ascending).  Default
\code{100}.}

\item{metric}{A character string specifying the significance metric
used for top-term selection and node scoring: \code{"p.adjust"}
(default) or \code{"pvalue"}.  The value is transformed as
\eqn{-\log_{10}(metric)}.}

\item{layout}{A character string naming the igraph layout algorithm.
Built-in shortcuts: \code{"circle"}, \code{"tree"}, \code{"grid"}.
Otherwise, the suffix passed to \code{layout_with_<layout>} in igraph
(e.g. \code{"fr"} for Fruchterman-Reingold, \code{"kk"} for
Kamada-Kawai).  Default \code{"fr"}.}

\item{minchar}{An integer specifying the minimum character length for
words to be included as keywords when \code{show_keyword = TRUE}.
Default \code{2}.}

\item{cluster}{A character string naming the igraph community detection
algorithm.  The suffix passed to \code{cluster_<cluster>} in igraph
(e.g. \code{"fast_greedy"}, \code{"walktrap"},
\code{"edge_betweenness"}, \code{"infomap"}).  Default
\code{"fast_greedy"}.}

\item{show_keyword}{A logical value.  When \code{TRUE}, the
\code{Description} text is tokenized and the most significant words
per cluster are shown as keywords.  When \code{FALSE} (default), the
original term descriptions are used as labels.}

\item{nlabel}{An integer specifying the number of keywords or term
descriptions to show per cluster in the legend labels.  Default
\code{4}.}

\item{character_width}{An integer specifying the maximum width (in
characters) at which keyword labels are wrapped via
\code{strwrap(width = character_width)}.  Default \code{50}.}

\item{words_excluded}{A character vector of words to exclude from
keyword extraction when \code{show_keyword = TRUE}.  Defaults to
\code{plotthis::words_excluded}.}

\item{mark}{A character string naming the ggforce hull function.
One of \code{"ellipse"} (default), \code{"rect"}, \code{"circle"}, or
\code{"text"} -- passed as the suffix to \code{geom_mark_<mark>}.}

\item{label}{A character string specifying what information to display
in the legend labels.  Either \code{"term"} (default; shows top term
descriptions/keywords per cluster) or \code{"feature"} (shows top
gene symbols per cluster).}

\item{labelsize}{A numeric value specifying the font size of the
cluster labels drawn by the ggforce mark layer.  Default \code{5}.}

\item{expand}{A numeric vector of length 2 (or 4) specifying the
expansion of the x and y axes, processed via
\code{\link{norm_expansion}()}.  Default \code{c(0.4, 0.4)}.}

\item{theme}{A character string or a theme class (i.e. ggplot2::theme_classic) specifying the theme to use.
Default is "theme_this".}

\item{theme_args}{A list of arguments to pass to the theme function.}

\item{palette}{A character string specifying the palette to use.
A named list or vector can be used to specify the palettes for different \code{split_by} values.}

\item{palcolor}{A character string specifying the color to use in the palette.
A named list can be used to specify the colors for different \code{split_by} values.
If some values are missing, the values from the palette will be used (palcolor will be NULL for those values).}

\item{palreverse}{A logical value indicating whether to reverse the palette. Default is FALSE.}

\item{alpha}{A numeric value specifying the transparency of the plot.}

\item{aspect.ratio}{A numeric value specifying the aspect ratio of the plot.}

\item{legend.position}{A character string specifying the position of the legend.
if \code{waiver()}, for single groups, the legend will be "none", otherwise "right".}

\item{legend.direction}{A character string specifying the direction of the legend.}

\item{title}{A character string specifying the title of the plot.
A function can be used to generate the title based on the default title.
This is useful when split_by is used and the title needs to be dynamic.}

\item{subtitle}{A character string specifying the subtitle of the plot.}

\item{xlab}{A character string specifying the x-axis label.}

\item{ylab}{A character string specifying the y-axis label.}

\item{seed}{The random seed to use. Default is 8525.}

\item{...}{Additional arguments.}
}
\value{
A \code{ggplot} object with \code{height} and \code{width}
attributes (in inches) attached.
}
\description{
Core implementation for drawing a single enrichment map -- a gene-set
similarity network where nodes represent enriched terms, node fill colour
encodes cluster membership, node size encodes the number of genes per
term, and edge thickness encodes the number of overlapping genes between
pairs of terms.  This is the workhorse behind the exported
\code{\link{EnrichMap}} function -- it takes a \strong{single} data frame (no
\code{split_by} support) and returns a \code{ggplot} object.

The function constructs an undirected graph from the term-term gene
overlap matrix, computes a force-directed layout, detects term clusters
via igraph community detection, and renders the result as a labelled
network plot with ggforce hull annotations around each cluster.
}
\section{Architecture}{

\enumerate{
\item \strong{ggplot dispatch} -- selects \code{gglogger::ggplot} or
\code{ggplot2::ggplot} based on
\code{getOption("plotthis.gglogger.enabled")}.
\item \strong{Data format conversion} -- if \code{data} inherits from
\code{"enrichResult"}, converts via \code{as.data.frame()}; if
\code{in_form == "enrichr"}, calls
\code{\link{prepare_enrichr_result}()} to rename columns and
infer GeneRatio/BgRatio.
\item \strong{Column validation} -- \code{\link{check_columns}()}
verifies that \code{Description}, \code{GeneRatio},
\code{pvalue}, \code{p.adjust}, and \code{geneID} are present.
\item \strong{Top-term selection} -- when \code{top_term} is not
\code{NULL}, selects the top N terms by the chosen \code{metric}
via \code{dplyr::slice_min()}.
\item \strong{Metric transformation} -- computes
\eqn{-\log_{10}(metric)} as the node scoring variable.  Splits
\code{geneID} on \code{"/"} to obtain per-term gene lists.
\item \strong{ID assignment} -- uses the \code{ID} column if present;
otherwise falls back to row names or generates synthetic IDs
(\code{"GS1"}, \code{"GS2"}, \dots).
\item \strong{Edge construction} -- creates all pairwise combinations
(\code{utils::combn()}) of term IDs and computes the overlap
count (intersection of geneID elements) as edge weight.  Edges
with weight 0 are dropped.
\item \strong{igraph graph construction} --
\code{igraph::graph_from_data_frame()} builds an undirected
graph from the edge list, with node attributes from the term
data.
\item \strong{Layout computation} -- dispatches to the chosen igraph
layout function (\code{layout_with_*}) or one of the built-in
shortcuts (\code{"circle"}, \code{"tree"}, \code{"grid"}).
\item \strong{Cluster detection} -- runs the chosen igraph clustering
algorithm (\code{cluster_*}) to group related terms into
modules.
\item \strong{Node coordinates} -- extracts vertex positions and
cluster assignments into a data frame with \code{dim1},
\code{dim2}, and \code{clusters} columns.
\item \strong{Keyword extraction} -- when \code{show_keyword = TRUE},
tokenizes \code{Description} text, filters words by
\code{minchar} and \code{words_excluded}, scores remaining
words by summed \code{metric} per cluster, and keeps the top
\code{nlabel} keywords.  Otherwise, the term \code{Description}
texts themselves are used as keywords.
\item \strong{Gene keyword extraction} -- always computes per-cluster
gene-level keywords (top \code{nlabel} genes by summed
\code{metric}) for the feature legend mode.
\item \strong{Mark layer} -- \code{ggforce::geom_mark_*()} (ellipse,
rect, circle, or text) draws cluster hulls with labels
containing the cluster ID and either term or feature keywords.
\item \strong{Edge rendering} -- \code{ggplot2::geom_segment()} draws
edges with line width proportional to overlap weight.
\item \strong{Node rendering} -- \code{ggplot2::geom_point(shape = 21)}
draws nodes sized by \code{Count} and filled by cluster
membership.
\item \strong{Scale configuration} -- \code{scale_size()} for node
size, \code{scale_linewidth()} for edge width,
\code{scale_fill_manual()} for cluster colours with custom
legend labels (term keywords or gene keywords depending on
\code{label}).
\item \strong{Theme and dimensions} -- applies the resolved theme,
sets aspect ratio and legend position, then calls
\code{\link{calculate_plot_dimensions}()} to attach
\code{height}/\code{width} attributes.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linkedheatmap.R
\name{LinkedHeatmap}
\alias{LinkedHeatmap}
\title{Linked Heatmap}
\usage{
LinkedHeatmap(
  data,
  values_by,
  values_fill = NA,
  name = NULL,
  split_by = NULL,
  split_by_sep = "_",
  rows_by = NULL,
  rows_by_sep = "_",
  rows_split_by = NULL,
  rows_split_by_sep = "_",
  columns_by = NULL,
  columns_by_sep = "_",
  columns_split_by = NULL,
  columns_split_by_sep = "_",
  rows_data = NULL,
  columns_data = NULL,
  keep_na = FALSE,
  keep_empty = FALSE,
  rows_orderby = NULL,
  columns_orderby = NULL,
  columns_name = NULL,
  columns_split_name = NULL,
  rows_name = NULL,
  rows_split_name = NULL,
  palette = "RdBu",
  palcolor = NULL,
  palreverse = FALSE,
  pie_size_name = "size",
  pie_size = NULL,
  pie_values = "length",
  pie_name = NULL,
  pie_group_by = NULL,
  pie_group_by_sep = "_",
  pie_palette = "Spectral",
  pie_palcolor = NULL,
  bars_sample = 1,
  label = identity,
  label_size = 10,
  label_color = "black",
  label_name = "label",
  mark = identity,
  mark_color = "black",
  mark_size = 1,
  mark_name = "mark",
  violin_fill = NULL,
  boxplot_fill = NULL,
  dot_size = 8,
  dot_size_name = "size",
  legend_items = NULL,
  legend_discrete = FALSE,
  legend.position = "right",
  legend.direction = "vertical",
  lower_quantile = 0,
  upper_quantile = 0.99,
  lower_cutoff = NULL,
  upper_cutoff = NULL,
  add_bg = FALSE,
  bg_alpha = 0.5,
  add_reticle = FALSE,
  reticle_color = "grey",
  cluster_columns = NULL,
  cluster_rows = NULL,
  show_row_names = NULL,
  show_column_names = NULL,
  border = TRUE,
  title = NULL,
  title_params = NULL,
  column_title = NULL,
  row_title = NULL,
  na_col = "grey85",
  row_names_side = "right",
  column_names_side = "bottom",
  row_annotation = NULL,
  row_annotation_side = NULL,
  row_annotation_palette = NULL,
  row_annotation_palcolor = NULL,
  row_annotation_type = NULL,
  row_annotation_params = NULL,
  row_annotation_agg = NULL,
  column_annotation = NULL,
  column_annotation_side = NULL,
  column_annotation_palette = NULL,
  column_annotation_palcolor = NULL,
  column_annotation_type = NULL,
  column_annotation_params = NULL,
  column_annotation_agg = NULL,
  link_width_by = NULL,
  link_width_scale = 5,
  link_color = "grey40",
  link_alpha = 0.6,
  flip = FALSE,
  alpha = 1,
  seed = 8525,
  padding = 15,
  base_size = 1,
  aspect.ratio = NULL,
  draw_opts = list(),
  layer_fun_callback = NULL,
  cell_type = c("tile", "bars", "label", "mark", "label+mark", "mark+label", "dot",
    "violin", "boxplot", "pie"),
  cell_agg = NULL,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  axes = NULL,
  axis_titles = axes,
  guides = NULL,
  design = NULL,
  ...
)
}
\arguments{
\item{data}{A data frame in long format. Each row represents one
observation; columns specify row/column membership for both left and
right heatmaps as well as the values to encode as color.}

\item{values_by}{Default column name for heatmap cell values.  Used as
fallback when \code{left_values_by} / \code{right_values_by} are not
explicitly provided via \code{...}.}

\item{values_fill}{A value used to fill missing cells in the matrix
(passed to \code{HeatmapAtomic}).  Default is \code{NA} (cells with no
data are left empty).}

\item{name}{Default legend title for the colour scale.  Used as fallback
when \code{left_name} / \code{right_name} are not provided via
\code{...}.  The suffixes \code{" (left)"} / \code{" (right)"} are
appended automatically.}

\item{split_by}{The column(s) to split data by and plot separately.
When \code{combine = TRUE}, the combined plot's data (\code{p$data})
contains the rows of every split, tagged with the \code{split_by}
column. For ggraph-based plots (e.g. \code{\link{Network}},
\code{\link{ClustreePlot}}), the node layout of every split is
exposed with the split column, and the original link rows (also
tagged with the \code{split_by} column) are available as the
\code{"edges"} attribute of the data.}

\item{split_by_sep}{The separator for multiple split_by columns. See \code{split_by}}

\item{rows_by}{Default column for rows in both heatmaps.  Used as
fallback for \code{left_rows_by} / \code{right_rows_by}.}

\item{rows_by_sep}{Separator for concatenated \code{rows_by} columns.}

\item{rows_split_by}{Optional column name to split the rows of \strong{both}
heatmaps into groups (passed as \code{row_split}).  When provided, row
names in the link table are prefixed with the split level to
disambiguate rows across splits.}

\item{rows_split_by_sep}{Separator for concatenated
\code{rows_split_by} columns.}

\item{columns_by}{Default column for columns in both heatmaps.  Used as
fallback for \code{left_columns_by} / \code{right_columns_by}.}

\item{columns_by_sep}{Separator for concatenated \code{columns_by}
columns.}

\item{columns_split_by}{Default column to split columns into groups.
Used as fallback for \code{left_columns_split_by} /
\code{right_columns_split_by}.}

\item{columns_split_by_sep}{Separator for concatenated
\code{columns_split_by} columns.}

\item{rows_data, columns_data}{Optional data frames providing additional
row / column metadata for annotations.  Passed through to
\code{HeatmapAtomic}.}

\item{keep_na, keep_empty}{Passed through to \code{HeatmapAtomic}.
See \code{\link{common_args}} for details.}

\item{rows_orderby, columns_orderby}{Column name to order rows / columns
by (disables clustering when set).}

\item{columns_name}{Display name for the column annotation.}

\item{columns_split_name}{Display name for the column split annotation.}

\item{rows_name}{Display name for the row annotation.}

\item{rows_split_name}{Display name for the row split annotation.}

\item{palette}{A character string naming a palette (see
\code{\link{show_palettes}}) or a character vector of colours for the
main heatmap colour scale.  Default \code{"RdBu"}.  Applied to both
heatmaps unless overridden per-side via \code{...}.}

\item{palcolor}{A custom colour vector that overrides \code{palette} for
the main heatmap colour scale.  Applied to both heatmaps unless
overridden per-side.}

\item{palreverse}{Logical; if \code{TRUE}, reverse the palette direction.}

\item{pie_size_name}{Legend title for the pie size when
\code{cell_type = "pie"}.}

\item{pie_size}{A numeric value or function returning the pie radius.
When a function, it receives the count of groups in the pie and should
return a radius.}

\item{pie_values}{A function or string (convertible via
\code{\link[base]{match.arg}}) to compute the value represented by each
pie slice.  Default \code{"length"} counts observations per group.}

\item{pie_name}{Default name for the pie legend.  Used as fallback for
\code{left_pie_name} / \code{right_pie_name}.}

\item{pie_group_by}{Default column(s) for pie grouping.  Used as fallback
for \code{left_pie_group_by} / \code{right_pie_group_by}.}

\item{pie_group_by_sep}{Separator for concatenated \code{pie_group_by}
columns.}

\item{pie_palette, pie_palcolor}{Palette and custom colours for pie slice
fill colours.}

\item{bars_sample}{Fraction of each cell's observations (0 < x <= 1;
\code{1} uses all data) or a whole count > 1 sampled per cell when
\code{cell_type = "bars"}.  Column widths are proportional to the
number of bars drawn per column.  Default 1 (all data).}

\item{label}{A function to compute text labels when
\code{cell_type = "label"} (or \code{"label+mark"}).  Receives the
aggregated value for a cell and optionally row/column indices and names.
See \code{\link{HeatmapAtomic}} for the full dispatch contract.}

\item{label_size}{Default point size for label text (used as fallback
when the \code{label} function does not return a \code{size} field).}

\item{label_color}{Default colour for label text (used as fallback when
the \code{label} function does not return a \code{color} field).}

\item{label_name}{Legend title for the label colour scale.}

\item{mark}{A function to compute mark symbols when
\code{cell_type = "mark"} (or \code{"label+mark"}).  Same dispatch
contract as \code{label}.  See \code{\link{HeatmapAtomic}} for supported
mark types.}

\item{mark_color}{Default mark colour (fallback).}

\item{mark_size}{Default mark stroke width in pt (fallback).}

\item{mark_name}{Legend title for the mark colour scale.}

\item{violin_fill}{A character vector of colours to use as fill for
violin plots when \code{cell_type = "violin"}.  If \code{NULL}, the
annotation colour is used.}

\item{boxplot_fill}{A character vector of colours to use as fill for
boxplots when \code{cell_type = "boxplot"}.  If \code{NULL}, the
annotation colour is used.}

\item{dot_size}{Dot size when \code{cell_type = "dot"}.  Can be a
numeric value or a function.}

\item{dot_size_name}{Legend title for the dot size.}

\item{legend_items}{A named numeric vector specifying custom legend
entries for the main colour scale.  Names become the displayed labels.}

\item{legend_discrete}{Logical; if \code{TRUE}, treat the main colour
scale as discrete.}

\item{legend.position}{A character string specifying where to place the
combined legend: \code{"right"} (default), \code{"left"}, \code{"top"},
\code{"bottom"}, or \code{"none"}.}

\item{legend.direction}{Legend stacking direction:
\code{"vertical"} (default) or \code{"horizontal"}.}

\item{lower_quantile, upper_quantile}{Quantiles used for clipping the
colour scale when \code{lower_cutoff} / \code{upper_cutoff} are
\code{NULL}.  Defaults are 0 and 0.99 respectively.}

\item{lower_cutoff, upper_cutoff}{Explicit cutoffs for the colour scale.
Values outside the range are clamped (winsorized).  Override
\code{lower_quantile} / \code{upper_quantile} when set.}

\item{add_bg}{Logical; if \code{TRUE}, add a background fill behind
non-tile cell types.  Not used for \code{cell_type = "tile"} or
\code{"bars"}.}

\item{bg_alpha}{Numeric in \eqn{[0, 1]} for background transparency.}

\item{add_reticle}{Logical; if \code{TRUE}, draw a reticle (crosshair
pattern) over the heatmap.}

\item{reticle_color}{Colour for the reticle lines.}

\item{cluster_columns}{Logical; cluster columns in both heatmaps.
\code{NULL} lets \code{HeatmapAtomic} decide.}

\item{cluster_rows}{Default clustering setting for rows.  Used as
fallback for \code{left_cluster_rows} / \code{right_cluster_rows}.}

\item{show_row_names, show_column_names}{Logical; show row/column names.
May also be a character vector of display modes; see
\code{\link{Heatmap}}.}

\item{border}{Logical; draw a border around each heatmap.  Default
\code{TRUE}.}

\item{title}{A character string for the overall plot title.  A function
can be used to generate a dynamic title from the default.
Note that, \code{left_title} and \code{right_title} are used to set the title for each heatmap,
and \code{title} is used to set the overall title for the combined plot.}

\item{title_params}{A list of parameters passed to \code{grid::grid.text()} to control the title appearance.
Default is \code{list(gp = gpar(fontsize = 14, fontface = "bold"))}.}

\item{column_title, row_title}{Character title displayed above the columns
/ beside the rows of each heatmap.  May also be a character vector of
display modes; see \code{\link{Heatmap}}.}

\item{na_col}{Colour used for \code{NA} cells.  Default \code{"grey85"}.}

\item{row_names_side}{Default side for row names.  Used as fallback for
\code{left_row_names_side} / \code{right_row_names_side}.  Default
\code{"right"}.}

\item{column_names_side}{Side for column names.  Default
\code{"bottom"}.}

\item{row_annotation}{A structured list specifying row annotations.
See \code{\link{HeatmapAtomic}} for the full specification.}

\item{row_annotation_side}{\strong{Deprecated}: use
\code{row_annotation} with the \code{side} sub-key instead.  Used as
fallback for \code{left_row_annotation_side} /
\code{right_row_annotation_side}.  Default \code{"left"}.}

\item{row_annotation_palette}{\strong{Deprecated}: use
\code{row_annotation} with the \code{palette} sub-key instead.}

\item{row_annotation_palcolor}{\strong{Deprecated}: use
\code{row_annotation} with the \code{palcolor} sub-key instead.}

\item{row_annotation_type}{\strong{Deprecated}: use
\code{row_annotation} with the \code{type} sub-key instead.}

\item{row_annotation_params}{\strong{Deprecated}: use
\code{row_annotation} with the \code{params} sub-key instead.}

\item{row_annotation_agg}{\strong{Deprecated}: use
\code{row_annotation} with the \code{agg} sub-key instead.}

\item{column_annotation}{A structured list specifying column annotations.
See \code{\link{HeatmapAtomic}} for the full specification.}

\item{column_annotation_side}{\strong{Deprecated}: use
\code{column_annotation} with the \code{side} sub-key instead.}

\item{column_annotation_palette}{\strong{Deprecated}: use
\code{column_annotation} with the \code{palette} sub-key instead.}

\item{column_annotation_palcolor}{\strong{Deprecated}: use
\code{column_annotation} with the \code{palcolor} sub-key instead.}

\item{column_annotation_type}{\strong{Deprecated}: use
\code{column_annotation} with the \code{type} sub-key instead.}

\item{column_annotation_params}{\strong{Deprecated}: use
\code{column_annotation} with the \code{params} sub-key instead.}

\item{column_annotation_agg}{\strong{Deprecated}: use
\code{column_annotation} with the \code{agg} sub-key instead.}

\item{link_width_by}{Optional column name in \code{data} whose values
determine the stroke width of each link line (e.g. interaction
strength).  Values are min-max scaled to \eqn{[0, 1]} and multiplied by
\code{link_width_scale}.
You can also pass a numeric value to use a constant width for all links.}

\item{link_width_scale}{Numeric scaling factor applied to the normalised
link intensity values to produce final line widths (\code{lwd}).
Default 5.}

\item{link_color}{Colour of the link spline curves.  Default
\code{"grey30"}.}

\item{link_alpha}{Alpha transparency of link curves in \eqn{[0, 1]}.
Default 0.8.}

\item{flip}{Logical; must be \code{FALSE} for linked heatmaps (flipping
is not supported).  Default \code{FALSE}.}

\item{alpha}{Alpha transparency for heatmap cells in \eqn{[0, 1]}.}

\item{seed}{Random seed for reproducibility.  Default 8525.}

\item{padding}{Padding around the heatmap in CSS order (top, right,
bottom, left).  Supports 1–4 values.  Default 15 (mm).}

\item{base_size}{A positive numeric scalar used as a scaling factor for
the overall heatmap size.  Default 1 (no scaling).  Values > 1 enlarge
all cell dimensions proportionally.}

\item{aspect.ratio}{Height-to-width ratio of a single heatmap cell.
When \code{NULL} (default), sensible defaults are chosen per
\code{cell_type} (e.g. 1 for tiles, 0.5 for bars, 2 for violins).}

\item{draw_opts}{A named list of additional arguments passed to
\code{\link[ComplexHeatmap]{draw,HeatmapList-method}}.  Internally
managed arguments (\code{padding}, \code{show_heatmap_legend}, etc.)
take precedence.}

\item{layer_fun_callback}{A function to add custom graphical layers on
top of each heatmap cell.  Receives \code{j}, \code{i}, \code{x},
\code{y}, \code{w}, \code{h}, \code{fill}, \code{sr}, \code{sc}.
See \code{\link[ComplexHeatmap]{Heatmap}} for details.}

\item{cell_type}{The type of cell to render.  One of \code{"tile"}
(default), \code{"bars"}, \code{"label"}, \code{"mark"},
\code{"label+mark"} (or \code{"mark+label"}), \code{"dot"},
\code{"violin"}, \code{"boxplot"}, \code{"pie"}.  Different cell types
use different \code{cell_fun} / \code{layer_fun} implementations.}

\item{cell_agg}{A function to aggregate values within each cell when
\code{cell_type = "tile"} or \code{"label"}.  Default is
\code{\link[base]{mean}}.}

\item{combine}{Whether to combine the plots into one when facet is FALSE. Default is TRUE.}

\item{nrow}{A numeric value specifying the number of rows in the facet.}

\item{ncol}{A numeric value specifying the number of columns in the facet.}

\item{byrow}{A logical value indicating whether to fill the plots by row.}

\item{axes}{A string specifying how axes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axes in individual plots.
\item 'collect' will remove duplicated axes when placed in the same run of rows or columns of the layout.
\item 'collect_x' and 'collect_y' will remove duplicated x-axes in the columns or duplicated y-axes in the rows respectively.
}}

\item{axis_titles}{A string specifying how axis titltes should be treated. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'keep' will retain all axis titles in individual plots.
\item 'collect' will remove duplicated titles in one direction and merge titles in the opposite direction.
\item 'collect_x' and 'collect_y' control this for x-axis titles and y-axis titles respectively.
}}

\item{guides}{A string specifying how guides should be treated in the layout. Passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE.
Options are:
\itemize{
\item 'collect' will collect guides below to the given nesting level, removing duplicates.
\item 'keep' will stop collection at this level and let guides be placed alongside their plot.
\item 'auto' will allow guides to be collected if a upper level tries, but place them alongside the plot if not.
}}

\item{design}{Specification of the location of areas in the layout, passed to \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}}.
Only relevant when \code{split_by} is used and \code{combine} is TRUE. When specified, \code{nrow}, \code{ncol}, and \code{byrow} are ignored.
See \code{\link[patchwork:wrap_plots]{patchwork::wrap_plots()}} for more details.}

\item{...}{Additional arguments passed to \code{\link{LinkedHeatmapAtomic}}.
All parameters listed above (and those inherited from
\code{LinkedHeatmapAtomic}) can be specified with \code{left_} or
\code{right_} prefixes for per-side control (e.g.
\code{left_palette = "Blues"}, \code{right_palette = "Reds"}).
Unprefixed arguments apply to both sides.  Also forwarded to
\code{\link[ComplexHeatmap]{Heatmap}} after prefix stripping.}
}
\value{
A \code{patchwork} object (class \code{wrap_plots}) with
\code{height} and \code{width} attributes (in inches).  When
\code{combine = FALSE}, a named list of such objects, one per
\code{split_by} level.
}
\description{
Draw two heatmaps side-by-side with spline link lines connecting matching
rows across the two heatmaps.  This is the public, exported interface for
creating linked-heatmap visualisations.

A typical use case is visualising ligand–receptor interactions: the
left heatmap shows ligand expression (rows = ligands, columns = cell
sources), the right heatmap shows receptor expression (rows = receptors,
columns = cell targets), and link curves connect each ligand to its
cognate receptor(s).
}
\section{Left / right specification}{

Parameters that differ between the two heatmaps are prefixed with
\code{left_} or \code{right_}.  Shared parameters (e.g. \code{palette},
\code{cell_type}, \code{cluster_columns}) apply to both sides but can be
overridden per-side via the \code{...} argument.  Every parameter listed
below can also be passed with a \code{left_} or \code{right_} prefix in
\code{...} for full per-side control.

The \code{...} argument is also forwarded to
\code{\link[ComplexHeatmap]{Heatmap}} after prefix-stripping, allowing
direct access to \code{ComplexHeatmap} parameters (e.g.
\code{left_row_names_gp}, \code{right_column_names_rot}).
}

\section{Split-by support}{

When \code{split_by} is provided, the data is partitioned into subsets
and an independent linked-heatmap pair is produced for each level.  The
results are combined via \code{\link[patchwork]{wrap_plots}} according to
\code{nrow}, \code{ncol}, \code{byrow}, and \code{design}.
Per-split \code{palette}, \code{palcolor}, and \code{legend.position} can
be specified as named lists keyed by split level.
}

\section{Dimension calculation}{

Cell dimensions are pre-computed from \code{cell_type},
\code{aspect.ratio}, \code{base_size}, and the unique row/column counts
in the data (accounting for split groups).  These exact dimensions are
passed to \code{ComplexHeatmap::Heatmap} so cells have guaranteed
physical sizes, ensuring that the two heatmaps' bodies align precisely
and that link line endpoints land on the correct rows.  The final
\code{height} / \code{width} attributes on the returned object include
legend space and are clamped to \eqn{[4, 64]} inches with aspect-ratio
correction.
}

\examples{
\donttest{
set.seed(8525)
# Define sparse ligand-receptor pairs
pairs_df <- data.frame(
    ligand = c("Ligand1", "Ligand2", "Ligand3", "Ligand4", "Ligand5",
               "Ligand1", "Ligand3", "Ligand5"),
    receptor = c("Receptor1", "Receptor2", "Receptor1", "Receptor3", "Receptor4",
                 "Receptor5", "Receptor2", "Receptor5"),
    stringsAsFactors = FALSE
)
sources <- paste0("Source", 1:4)
targets <- paste0("Target", 1:6)

# Expand pairs across all sources and targets
data <- merge(
    merge(pairs_df, data.frame(source = sources, stringsAsFactors = FALSE)),
    data.frame(target = targets, stringsAsFactors = FALSE)
)
data$split <- sample(c("A", "B"), nrow(data), replace = TRUE)
data$ligand_expr <- runif(nrow(data), 0, 10)
data$receptor_expr <- runif(nrow(data), 0, 10)
data$intensity <- runif(nrow(data), 0, 1)

if (requireNamespace("ComplexHeatmap", quietly = TRUE)) {
    LinkedHeatmap(
        data,
        show_column_names = TRUE,
        column_names_side = "bottom",
        show_row_names = FALSE,
        left_row_names_side = "right",
        left_rows_by = "ligand",
        left_columns_by = "source",
        left_values_by = "ligand_expr",
        left_name = "Ligand",
        left_row_annotation = list(
            .row = list(
                type = "label",
                params = list(labels_rot = 0),
                side = "right"
            )
        ),
        right_cluster_rows = FALSE,
        right_row_names_side = "left",
        right_row_annotation = list(
            .row = list(
                type = "label",
                params = list(labels_rot = 0),
                side = "left"
            )
        ),
        right_rows_by = "receptor",
        right_columns_by = "target",
        right_values_by = "receptor_expr",
        right_name = "Receptor",
        link_width_by = "intensity"
    )
}
}
}
\seealso{
\code{\link{Heatmap}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{blend_to_color}
\alias{blend_to_color}
\title{Blend two colors}
\usage{
blend_to_color(C1, C2, mode = "blend")
}
\description{
Blend two colors
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap-utils.R
\name{.anno_ggseries}
\alias{.anno_ggseries}
\alias{anno_pie}
\alias{anno_ring}
\alias{anno_bar}
\alias{anno_violin}
\alias{anno_boxplot}
\alias{anno_density}
\alias{anno_simple}
\alias{anno_points}
\alias{anno_lines}
\alias{anno_block}
\alias{anno_text}
\title{Heatmap annotation functions}
\usage{
.anno_ggseries(
  x,
  split_by = NULL,
  group_by,
  column,
  title,
  side = "left",
  which = "row",
  palette,
  palcolor = NULL,
  border = TRUE,
  legend.direction,
  show_legend = TRUE,
  .plotting,
  ...
)

anno_pie(
  x,
  split_by = NULL,
  group_by,
  column,
  title,
  which = "row",
  palette,
  side = "left",
  palcolor = NULL,
  border = TRUE,
  legend.direction,
  show_legend = TRUE,
  ...
)

anno_ring(
  x,
  split_by = NULL,
  group_by,
  column,
  title,
  which = "row",
  palette,
  side = "left",
  palcolor = NULL,
  border = TRUE,
  legend.direction,
  show_legend = TRUE,
  ...
)

anno_bar(
  x,
  split_by = NULL,
  group_by,
  column,
  title,
  which = "row",
  palette,
  side = "left",
  palcolor = NULL,
  border = TRUE,
  legend.direction,
  show_legend = TRUE,
  ...
)

anno_violin(
  x,
  split_by = NULL,
  group_by,
  column,
  title,
  which = "row",
  palette,
  side = "left",
  palcolor = NULL,
  border = TRUE,
  legend.direction,
  show_legend = TRUE,
  ...
)

anno_boxplot(
  x,
  split_by = NULL,
  group_by,
  column,
  title,
  which = "row",
  palette,
  side = "left",
  palcolor = NULL,
  border = TRUE,
  legend.direction,
  show_legend = TRUE,
  ...
)

anno_density(
  x,
  split_by = NULL,
  group_by,
  column,
  title,
  which = "row",
  palette,
  side = "left",
  palcolor = NULL,
  border = TRUE,
  legend.direction,
  show_legend = TRUE,
  ...
)

anno_simple(
  x,
  split_by = NULL,
  group_by = NULL,
  column = NULL,
  title,
  which = "row",
  side = "left",
  palette,
  palcolor = NULL,
  border = TRUE,
  legend.direction,
  show_legend = TRUE,
  alpha = 1,
  ...
)

anno_points(
  x,
  split_by = NULL,
  group_by,
  column,
  title,
  which = "row",
  palette,
  side = "left",
  palcolor = NULL,
  border = TRUE,
  legend.direction,
  show_legend = TRUE,
  alpha = 1,
  ...
)

anno_lines(
  x,
  split_by = NULL,
  group_by,
  column,
  title,
  which = "row",
  palette,
  side = "left",
  palcolor = NULL,
  border = TRUE,
  legend.direction,
  show_legend = TRUE,
  alpha = 1,
  add_points = TRUE,
  ...
)

anno_block(
  x,
  split_by = NULL,
  group_by = NULL,
  column = NULL,
  title,
  which = "row",
  side = "left",
  palette,
  palcolor = NULL,
  border = TRUE,
  legend.direction,
  show_legend = FALSE,
  alpha = 1,
  ...
)

anno_text(
  x,
  split_by = NULL,
  group_by,
  column = NULL,
  title,
  which = "row",
  palette,
  side = "left",
  palcolor = NULL,
  border = TRUE,
  legend.direction,
  show_legend = FALSE,
  alpha = 1,
  ...
)
}
\arguments{
\item{x}{A data frame}

\item{split_by}{A character string of the column name to split the data (heatmap)}

\item{group_by}{A character string of the column name to group the data (rows or columns of the heatmap)}

\item{column}{A character string of the column name of the data \code{x} to plot}

\item{title}{A character string to name the legend}

\item{side}{A character string specifying the side of the annotation. Default is "left".}

\item{which}{A character string specifying the direction of the annotation. Default is "row".
Other options are "column".}

\item{palette}{A character string specifying the palette of the annotation}

\item{palcolor}{A character vector of colors to override the palette}

\item{border}{A logical value indicating whether to draw the border of the annotation}

\item{legend.direction}{A character string specifying the direction of the legend. Default is "vertical".
Other options are "horizontal".}

\item{show_legend}{A logical value indicating whether to show the legend}

\item{.plotting}{A function to create the plot for each split and each group}

\item{...}{Other arguments passed to \code{ComplexHeatmap::AnnotationFunction}
The parameters passed to \code{row_annotation_params} and \code{column_annotation_params} will be passed here.}

\item{alpha}{A numeric value between 0 and 1 specifying the transparency of the annotation}

\item{add_points}{A logical value indicating whether to add points to the annotation}
}
\description{
Heatmap annotation functions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap-utils.R
\name{.anno_ggcat}
\alias{.anno_ggcat}
\title{Heatmap annotation function for categorical data}
\usage{
.anno_ggcat(
  x,
  split_by = NULL,
  group_by,
  column,
  title,
  side = "left",
  which = "row",
  palette,
  palcolor = NULL,
  border = TRUE,
  legend.direction,
  show_legend = TRUE,
  .plotting,
  ...
)
}
\arguments{
\item{x}{A data frame}

\item{split_by}{A character string of the column name to split the data}

\item{group_by}{A character string of the column name to group the data}

\item{column}{A character string of the column name to plot}

\item{title}{A character string to name the legend}

\item{side}{A character string showing where the annotation is.}

\item{which}{A character string specifying the direction of the annotation. Default is "row".
Other options are "column".}

\item{palette}{A character string specifying the palette of the annotation}

\item{palcolor}{A character vector of colors to override the palette}

\item{border}{A logical value indicating whether to draw the border of the annotation}

\item{legend.direction}{A character string specifying the direction of the legend. Default is "vertical".
Other options are "horizontal".}

\item{show_legend}{A logical value indicating whether to show the legend}

\item{.plotting}{A function to create the plot for each split and each group}

\item{...}{Other arguments passed to \code{ComplexHeatmap::AnnotationFunction}}
}
\description{
Heatmap annotation function for categorical data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap-utils.R
\name{.gggrob}
\alias{.gggrob}
\title{Get the grid.draw-able ggplot grob
The output from ggplotGrob can not be directly used in grid.draw, the position can not be set.
This function extracts the gTree from the ggplot grob.}
\usage{
.gggrob(p, void = TRUE, nolegend = TRUE)
}
\arguments{
\item{p}{A ggplot object}

\item{void}{If TRUE, the theme_void will be added to the ggplot object}

\item{nolegend}{If TRUE, the legend will be removed from the ggplot object}
}
\value{
A gTree object
}
\description{
Get the grid.draw-able ggplot grob
The output from ggplotGrob can not be directly used in grid.draw, the position can not be set.
This function extracts the gTree from the ggplot grob.
}
\keyword{internal}
